/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import <Cocoa/Cocoa.h>

#ifndef ARPI_h
#include "ARPI.h"
#endif //ARPI_h

#ifndef ARPIBlueWindowPreset_h
#include "ARPIBlueWindowPreset.h"
#endif

#import "ARPIPluginView.h"

class TARPIHost;
class VARPIPluginClass;
class TArmeWSObj;
@class ARPIBlueWindowSubview;

#define kBWLinkNone 0
#define	kBWLinkHorizontal 1
#define	kBWLinkVertical 2
#define	kBWLinkPressure 3

#define kMaxNbParams 32

#define MAX_LOG_SCALE .0001

@interface ARPIBlueWindow : ARPIPluginView
{
	// the list of NSViews (note that we duplicate the 
	// normal Cocoa hierarchy here).
	// NSPopUpButton*		fControllerList[kMaxNbParams];
	NSButton*				fBWCursorButton[kMaxNbParams];
	NSTextField* 			fMinValueTextBox[kMaxNbParams];
	NSSlider* 				fSlider[kMaxNbParams];
	NSTextField* 			fParamText[kMaxNbParams];
	NSTextField* 			fMaxValueTextBox[kMaxNbParams];
	NSTextField* 			fParamValueTextBox[kMaxNbParams];

//	ARPIBlueWindowPreset*	fPreset;
	//float					fMinValues[kMaxNbParams];
	//float					fMaxValues[kMaxNbParams];

	ARPIBlueWindowSubview*	fBlueWindow;
	ARPIParamInfo			fParamInfo[kMaxNbParams];
	UInt32					fBWLink[kMaxNbParams];
	NSImage*				fBWLinkImages[4];
	SInt32					fNbParams;
	
	NSButton*				fDebugReset;
	NSButton*				fByPassButton;
	NSPopUpButton*			fPresetButton;
	SInt32					fDummyPreset;
	
	NSNumberFormatter*		fNumberFormatter;
	
	int						fUserPresetIndex;
	bool					fUserPreset;
	MIObject*				fMIObject;
	const VARPIPluginClass*	fPluginClass;
}

-(id)initWithFrame:(NSRect)frame MIObject:(MIObject*)miobject;
-(id)initWithMIObject:(MIObject*)miobject;
-(TArmeWSObj*)getProcessor;
-(void)postInit;
-(const VARPIPluginClass*)audioPluginClass;

-(void)syncPresetFromBlueWindow;
-(void)syncPresetFromValueBox:(SInt32)param;
-(void)syncPresetFromMaxValueBox:(SInt32)param;
-(void)syncPresetFromMinValueBox:(SInt32)param;
-(void)syncPresetFromSlider:(SInt32)param;
-(void)syncPresetFromSourceBox:(SInt32)param;

-(void)syncInterfaceFromPreset;
-(void)syncBlueWindowFromPreset;
-(void)syncValueBoxFromPreset:(SInt32)param;
-(void)syncMaxValueBoxFromPreset:(SInt32)param;
-(void)syncMinValueBoxFromPreset:(SInt32)param;
-(void)syncSliderFromPreset:(SInt32)param;
-(void)syncSourceBoxFromPreset:(SInt32)param;

-(void)syncPluginParamFromPreset:(SInt32)param;
-(void)syncPluginFromPreset;


-(bool)isLinear:(SInt32)param;
-(float)getValueFromFraction:(float)fraction Param:(SInt32)param;
-(float)getFractionFromValue:(float)value Param:(SInt32)param;

/*
	actions from controls
*/

- (void)fBlueWindowAction;

// the control that should have the first key focus
- (id)firstFocusControl; 

- (IBAction)fBWCursorButtonAction:(id)sender;
- (IBAction)fMinValueTextBoxAction:(id)sender;
- (IBAction)fSliderAction:(id)sender;
- (IBAction)fMaxValueTextBoxAction:(id)sender;
- (IBAction)fParamValueTextBoxAction:(id)sender;
//- (void)textDidChange:(NSNotification *)aNotification;
//- (void)textDidEndEditing:(NSNotification *)aNotification;

- (IBAction)fSavePresetButtonAction:(id)sender;

- (void)dealloc;

@end
