/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#ifndef ZzTypes_h
#include "ZzTypes.h"
#endif //ZzTypes_h

#import "ARPIBlueWindow.h"
#import "ARPIBlueWindowSubview.h"
#import "ARPIBlueWindowController.h"
#import "MIMontageDocument.h"

#ifndef ARPI_h
#include "ARPI.h"
#endif //ARPI_h

#ifndef TArmeWSObj_h
#include "TArmeWSObj.h"
#endif //TArmeWSObj_h

#import "MIObject.h"

#define kBWLeftMargin 20.0
#define kBWRightMargin 20.0
#define kBWTopMargin 10.0
#define kBWBottomMargin 20.0

/*
#define kControllerListHPos 17.0
#define kControllerListVPos 0.0
#define kControllerListHSize 228.0
#define kControllerListVSize 26.0
*/
#define kBWCursorButtonHPos kBWLeftMargin
#define kBWCursorButtonVPos 0.0
#define kBWCursorButtonHSize 23.0
#define kBWCursorButtonVSize 23.0


#define kValueTextBox1HPos (kBWCursorButtonHPos + kBWCursorButtonHSize + 6.0)
#define kValueTextBoxesVPos 3.0
#define kValueTextBox1HSize 72.0
#define kValueTextBoxesVSize 17.0

#define kSliderHPos (kValueTextBox1HPos + kValueTextBox1HSize + 6.0)
#define kSliderVPos 2.0
#define kSliderHSize 127.0
#define kSliderVSize 15.0

#define kParamTextHPos (kValueTextBox1HPos + kValueTextBox1HSize + 5.0)
#define kParamTextVPos 18.0
#define kParamTextHSize 127.0
#define kParamTextVSize 14.0

#define kValueTextBox2HPos (kSliderHPos + kSliderHSize + 8.0)
#define kValueTextBox3HPos (kValueTextBox2HPos + kValueTextBox2HSize + 8.0)
#define kValueTextBox2HSize 72.0
#define kValueTextBox3HSize 72.0

// the vertical offset between control rows
#define kVDelta 37.0
#define kBWBetweenControlsSpace 10.0
#define kBWZoneDefaultHeight 300.0
#define kViewWidth (kValueTextBox3HPos + kValueTextBox3HSize + kBWRightMargin)

// room reserved for preset option box
#define kPresetHeight 23.0
#define kBWPresetButtonHPos kBWLeftMargin + kBWBypassButtonHSize + kBWBypassPresetSpace
#define kBWPresetButtonVPos 0.0
#define kBWPresetButtonHSize (kViewWidth - kBWLeftMargin - kBWRightMargin - kBWBypassButtonHSize - kBWBypassPresetSpace)
#define kBWPresetButtonVSize 23.0


// room reserved for bypass button
//#define kByPassHeight 23.0
#define kBWBypassPresetSpace 10.0
#define kBWBypassButtonHPos kBWLeftMargin 
#define kBWBypassButtonVPos 0.0
#define kBWBypassButtonHSize 80.0
#define kBWBypassButtonVSize 23.0


//#define BWDEBUG

#ifdef BWDEBUG
#define kDebugHeight 30.0f
#else
#define kDebugHeight 0.0f;
#endif

@implementation ARPIBlueWindow

- (void)dealloc
{
		// need to pass on the preset to the tarpihost here
  //    delete [self getPreset];
       [super dealloc];
}

- (ARPIBlueWindowPreset*)getPreset
{
	return [self getProcessor]->GetPluginPreset();
}

- (void)setPreset:(ARPIBlueWindowPreset*)preset
{
	return [self getProcessor]->SetPluginPreset(preset);
}


-(id)initWithFrame:(NSRect)frame MIObject:(MIObject*)miobject
{
	SInt32 param;
	
	fMIObject = miobject;
	
	// cache the plugin class
	fPluginClass = [self audioPluginClass];
	
	// store the parameter infos
	fNbParams = fPluginClass->GetNbParams();

	// get param info
	for(param = 0; param < fNbParams; param++)
	{
		fPluginClass->GetParamInfo(&(fParamInfo[param]),param);
	}

	// we override the view height
	frame.size.height = kBWZoneDefaultHeight + 2*kBWBottomMargin 
                        + fNbParams*kVDelta + kPresetHeight + kDebugHeight;
	
	id _self = [super initWithFrame:frame];
		
	// set the autoresizing mask
	[_self setAutoresizingMask:(NSViewWidthSizable|NSViewHeightSizable)];
		
	// create a blue window and set its initial size
	NSRect bwFrame;
	bwFrame = frame;
	bwFrame.origin.x = kBWLeftMargin;
	bwFrame.size.width = frame.size.width - kBWLeftMargin - kBWRightMargin;
	bwFrame.origin.y = kBWBottomMargin;
	bwFrame.size.height = kBWZoneDefaultHeight; //frame.size.width - kBWBottomMargin - kBWTopMargin;
							
	fBlueWindow = [[ARPIBlueWindowSubview alloc] initWithFrame:bwFrame];
	
	[fBlueWindow loadCursorImage];
	
	[fBlueWindow setAutoresizingMask:(NSViewWidthSizable|NSViewHeightSizable)];

	[_self addSubview:fBlueWindow];
	
	// start drawing the controls just above the blue window
	float controlsVPos = kBWZoneDefaultHeight + kBWBottomMargin + kBWBetweenControlsSpace;

	// load BW link button images
	NSBundle*	theBundle;
	theBundle = [NSBundle mainBundle];

	fBWLinkImages[kBWLinkNone] = [[NSImage alloc] initWithContentsOfFile:[theBundle pathForResource:@"BWNoneSmall" ofType:@"tiff"]];
	fBWLinkImages[kBWLinkHorizontal] = [[NSImage alloc] initWithContentsOfFile:[theBundle pathForResource:@"BWHorizontalSmall" ofType:@"tiff"]];
	fBWLinkImages[kBWLinkVertical] = [[NSImage alloc] initWithContentsOfFile:[theBundle pathForResource:@"BWVerticalSmall" ofType:@"tiff"]];
	fBWLinkImages[kBWLinkPressure] = [[NSImage alloc] initWithContentsOfFile:[theBundle pathForResource:@"BWPressureSmall" ofType:@"tiff"]];
	

	// setup formatter
	fNumberFormatter = [[[NSNumberFormatter alloc] init] autorelease];
	[fNumberFormatter setFormat:@"####0.00;0.00;-####0.00"];

	for( SInt32 param = fPluginClass->GetNbParams() - 1; param >= 0; param-- )
	{
		// create controls from left to right
		// 1) the controller chooser
		#if 0
		/*fControllerList[param] = [[NSPopUpButton alloc] initWithFrame:NSMakeRect(
			kControllerListHPos, controlsVPos + kControllerListVPos,
			kControllerListHSize, kControllerListVSize)];*/
		#endif
		fBWCursorButton[param] = [[NSButton alloc] initWithFrame:NSMakeRect(
			kBWCursorButtonHPos, controlsVPos + kBWCursorButtonVPos,
			kBWCursorButtonHSize, kBWCursorButtonVSize)];
		
		[fBWCursorButton[param] setAction:@selector(fBWCursorButtonAction:)];
		[fBWCursorButton[param] setTarget:self];
		[fBWCursorButton[param] setButtonType:NSMomentaryPushButton];
		[fBWCursorButton[param] setBezelStyle:NSRegularSquareBezelStyle];
		[fBWCursorButton[param] setStringValue:@""];
		[fBWCursorButton[param] setBezelStyle:NSRegularSquareBezelStyle];
		// this effect is, by default, not affected by blue window cursor
		fBWLink[param] = kBWLinkNone;
		[fBWCursorButton[param] setImage:fBWLinkImages[kBWLinkNone]];
		
		// 2) the first value text box
		fMinValueTextBox[param] = [[NSTextField alloc] initWithFrame:NSMakeRect(
			kValueTextBox1HPos, controlsVPos + kValueTextBoxesVPos,
			kValueTextBox1HSize, kValueTextBoxesVSize)];
		[fMinValueTextBox[param] setAction:@selector(fMinValueTextBoxAction:)];
		[fMinValueTextBox[param] setTarget:self];
		[[fMinValueTextBox[param] cell] setFont:[NSFont systemFontOfSize:[NSFont smallSystemFontSize]]];
		[fMinValueTextBox[param] setDelegate:self];
		[[fMinValueTextBox[param] cell] setSendsActionOnEndEditing:YES];
		[[fMinValueTextBox[param] cell] setFormatter:fNumberFormatter];
		
		// 3) the slider
		fSlider[param] = [[NSSlider alloc] initWithFrame:NSMakeRect(
			kSliderHPos, controlsVPos + kSliderVPos, kSliderHSize, kSliderVSize)];
			
		[[fSlider[param] cell] setControlSize:NSSmallControlSize];
			
		[fSlider[param] setAction:@selector(fSliderAction:)];
		[fSlider[param] setTarget:self];
			
		[fSlider[param] setMinValue:0.0];
		[fSlider[param] setMaxValue:1.0];
		
		[fSlider[param] setTitle:@"azeaze"];
		
		
		// 4) the parameter name
		fParamText[param] = [[NSTextField alloc] initWithFrame:NSMakeRect(
			kParamTextHPos, controlsVPos + kParamTextVPos,
			kParamTextHSize, kParamTextVSize)];
		[fParamText[param] setEditable:NO];
		[fParamText[param] setBordered:NO];
		[fParamText[param] setDrawsBackground:NO];
		[fParamText[param] setStringValue:[NSString stringWithCString:fParamInfo[param].name]];
		[fParamText[param] setAction:@selector(fParamTextAction:)];
		[[fParamText[param] cell] setFont:[NSFont systemFontOfSize:[NSFont smallSystemFontSize]]];
					
		// 5) the second value text box
		fMaxValueTextBox[param] = [[NSTextField alloc] initWithFrame:NSMakeRect(
			kValueTextBox2HPos, controlsVPos + kValueTextBoxesVPos,
			kValueTextBox2HSize, kValueTextBoxesVSize)];
		[fMaxValueTextBox[param] setAction:@selector(fMaxValueTextBoxAction:)];
		[fMaxValueTextBox[param] setTarget:self];
		[[fMaxValueTextBox[param] cell] setSendsActionOnEndEditing:YES];
				
		[[fMaxValueTextBox[param] cell] setFont:[NSFont systemFontOfSize:[NSFont smallSystemFontSize]]];
		[[fMaxValueTextBox[param] cell] setFormatter:fNumberFormatter];


		// 6) the third value text box
		fParamValueTextBox[param] = [[NSTextField alloc] initWithFrame:NSMakeRect(
			kValueTextBox3HPos, controlsVPos + kValueTextBoxesVPos,
			kValueTextBox3HSize, kValueTextBoxesVSize)];
		[fParamValueTextBox[param] setAction:@selector(fParamValueTextBoxAction:)];
		[fParamValueTextBox[param] setTarget:self];
		[[fParamValueTextBox[param] cell] setSendsActionOnEndEditing:YES];
		
		[[fParamValueTextBox[param] cell] setFont:[NSFont systemFontOfSize:[NSFont smallSystemFontSize]]];

		[[fParamValueTextBox[param] cell] setFormatter:fNumberFormatter];

		
		[_self addSubview:fBWCursorButton[param]];
		[_self addSubview:fMinValueTextBox[param]];
		[_self addSubview:fSlider[param]];
		[_self addSubview:fParamText[param]];
		[_self addSubview:fMaxValueTextBox[param]];
		[_self addSubview:fParamValueTextBox[param]];
		
		controlsVPos += kVDelta;
		
						
	}
	
	// preset popup button

        
	fPresetButton = [[NSPopUpButton alloc] initWithFrame:NSMakeRect(
			kBWPresetButtonHPos, controlsVPos + kBWPresetButtonVPos,
			kBWPresetButtonHSize, kBWPresetButtonVSize)];

	[fPresetButton setAction:@selector(fPresetButtonAction:)];
	[fPresetButton setTarget:self];
	[[fPresetButton cell] setFont:[NSFont systemFontOfSize:[NSFont smallSystemFontSize]]];
	
	SInt32 preset;
	for (preset = 0; preset < fPluginClass->GetNbFactoryPresets(); preset++)
		{
			[fPresetButton addItemWithTitle:[NSString 
				stringWithCString: fPluginClass->GetFactoryPresetName(preset)]];
		}
	fUserPresetIndex = preset;
	fDummyPreset = fPluginClass->GetNbFactoryPresets();
	[fPresetButton addItemWithTitle:@"User Preset"];
	[fPresetButton selectItemAtIndex:fUserPresetIndex];
	[_self addSubview:fPresetButton];
	fUserPreset = true;

	

	// controlsVPos += kVDelta;

	// bypass button
	

	fByPassButton = [[NSButton alloc] initWithFrame:NSMakeRect(
			kBWBypassButtonHPos, controlsVPos + kBWBypassButtonVPos,
			kBWBypassButtonHSize, kBWBypassButtonVSize)];
		
	[fByPassButton setAction:@selector(fByPassButtonAction:)];
	[fByPassButton setTarget:self];
	[fByPassButton setState:NSOffState];
	[fByPassButton setButtonType:NSToggleButton	];
	[fByPassButton setBezelStyle:NSRegularSquareBezelStyle];
	[fByPassButton setTitle:@"On"];
	[fByPassButton setAlternateTitle:@"Off"];
	[fByPassButton setBezelStyle:NSRegularSquareBezelStyle];
	
	[_self addSubview:fByPassButton];


#ifdef BWDEBUG

	// debug reset button
	controlsVPos += kVDelta;


	fDebugReset = [[NSButton alloc] initWithFrame:NSMakeRect(
			kBWCursorButtonHPos, controlsVPos + kBWCursorButtonVPos,
			kBWCursorButtonHSize, kBWCursorButtonVSize)];
		
	[fDebugReset setAction:@selector(fDebugResetAction:)];
	[fDebugReset setTarget:self];
	[fDebugReset setButtonType:NSMomentaryPushButton];
	[fDebugReset setBezelStyle:NSRegularSquareBezelStyle];
	[fDebugReset setTitle:@"R"];
	[fDebugReset setBezelStyle:NSRegularSquareBezelStyle];
	
	[_self addSubview:fDebugReset];

#endif
        
	return _self;
}

#ifdef BWDEBUG
- (IBAction)fDebugResetAction:(id)sender
{
	[self getProcessor]->ResetRecursive();
}
#endif

- (id)firstFocusControl
{
	return fBWCursorButton[0];
}

-(id)initWithMIObject:(MIObject*)miobject
{
	NSRect rect = NSMakeRect(0,0,kValueTextBox3HPos + kValueTextBox3HSize + kBWRightMargin,0);
	return [self initWithFrame:rect MIObject:miobject];
}

-(TArmeWSObj*)getProcessor
{
//	TArmeWSObj* wsobj = fARPIHost->GetProcessor(self);
	TArmeWSObj* wsobj = [fMIObject WSObject]; 
//	vernl(0,"ARPIBlueWindow getProcessor : self = " << (void*)self << "returning wsobj = " << (void*)wsobj);
	ZzAssert(wsobj);
	return wsobj;
}


-(void)postInit
{
	// now that the plugin and view are linked together, we can tell the view to update
	// itself according to plugin values
	//[self syncMinAndMaxValuesFromPlugin]; // fills the fMinValues and fMaxValues once
	//[self syncAllViewControlsFromPlugin]; // means syncs all the params
	
	// TODO this should be SyncPluginFromPreset
	

	[self syncInterfaceFromPreset];
	[self syncPluginFromPreset];

	// set the correct name of the preset
	// [myString isEqualTo:myOtherString]
	
	NSString *savedname = [NSString stringWithCString:[self getPreset]->GetName()];

	int preset;
	for (preset = 0; preset < fPluginClass->GetNbFactoryPresets(); preset++)
	{
		if ([savedname isEqualTo: [fPresetButton itemTitleAtIndex: preset]]) break;
	}
			
	fUserPreset = (fUserPresetIndex == preset);		
	[fPresetButton selectItemAtIndex:preset];
	
}


#pragma mark Scale conversion methods

-(bool)isLinear:(SInt32)param
{
    if (fParamInfo[param].unit == ARPIUnit_Hz) return false;
    else return true;
}

-(float)getValueFromFraction:(float)fraction Param:(SInt32)param
{
    float value;
    //float max = fMaxValues[param];
    //float min = fMinValues[param];
    float max = [self getPreset]->GetParamMaxValue(param);
    float min = [self getPreset]->GetParamMinValue(param);
    if([self isLinear:param])
        value = (fraction * max + (1.0f - fraction) * min);
    else
    {
        float max_scale = MAX_LOG_SCALE;
        if (fabs(min) < fabs(max) * max_scale) min = max * max_scale;
        value = min * pow(max/min, fraction);
    }
    return value;
}

-(float)getFractionFromValue:(float)value Param:(SInt32)param
{
    float fraction;
    //float max = fMaxValues[param];
    //float min = fMinValues[param];
    float max = [self getPreset]->GetParamMaxValue(param);
    float min = [self getPreset]->GetParamMinValue(param);
    if([self isLinear:param])
        fraction = (value - min) / (max - min);
    else
    {
        float max_scale = MAX_LOG_SCALE;
        if (fabs(min) < fabs(max) * max_scale) min = max * max_scale;
        fraction = log(value/min) / (log(max/min));
    }
    return fraction;
}


#pragma mark ----- SYNC PRESET FROM UI




-(void)syncPresetFromBlueWindow
{
	[self getPreset]->SetSourceValue(0, [fBlueWindow getX]);
	[self getPreset]->SetSourceValue(1, [fBlueWindow getY]);
	[self getPreset]->SetSourceValue(2, [fBlueWindow getZ]);
	
}

-(void)syncPresetFromValueBox:(SInt32)param
{
	float maxbox = [self getPreset]->GetParamMaxValue(param);
	float minbox = [self getPreset]->GetParamMinValue(param);
	float val = [fParamValueTextBox[param] floatValue];

	float min, max;
	
	if (minbox < maxbox){
		max = maxbox;
		min = minbox;
	}
	else{
		max = minbox;
		min = maxbox;
	}

	val = (val < max) ? val : max;
	val = (val > min) ? val : min;
	
	[self getPreset]->SetParamValue(param, val);
}

-(void)syncPresetFromMaxValueBox:(SInt32)param
{
	//it is allowed for maxvalue to be smaller than the value in the minvalue box
	//this makes inverted ranges possible

	float max = fParamInfo[param].absMaxVal;
	float min = fParamInfo[param].absMinVal;
	//float min = [self getPreset]->GetParamMinValue(param);

	float val = [fMaxValueTextBox[param] floatValue];


	val = (val < max) ? val : max;
	val = (val > min) ? val : min;
	
	[self getPreset]->SetParamMaxValue(param, val);
}

-(void)syncPresetFromMinValueBox:(SInt32)param
{
	//it is allowed for minvalue to be largerr than the value in the maxvalue box
	//this makes inverted ranges possible

	//float max = [self getPreset]->GetParamMaxValue(param);
	float minbox = fParamInfo[param].absMinVal;
	float maxbox = fParamInfo[param].absMaxVal;
	
	float min, max;
	
	if (minbox < maxbox){
		max = maxbox;
		min = minbox;
	}
	else{
		max = minbox;
		min = maxbox;
	}
	

	float val = [fMinValueTextBox[param] floatValue];

	val = (val < max) ? val : max;
	val = (val > min) ? val : min;
	
	[self getPreset]->SetParamMinValue(param, val);
}

-(void)syncPresetFromSlider:(SInt32)param
{

	float fraction = [fSlider[param] floatValue];
	float val = [self getValueFromFraction:fraction Param:param];
	[self getPreset]->SetParamValue(param, val);
}


-(void)syncPresetFromSourceBox:(SInt32)param
{
	[self getPreset]->SetParamSource(param, fBWLink[param]);
}


#pragma mark ----- SYNC UI FROM PRESET

-(void)syncInterfaceFromPreset
{
	for (int param=0; param<fNbParams; param++)
	{
		[self syncSliderFromPreset: param];
		[self syncValueBoxFromPreset: param];
		[self syncMinValueBoxFromPreset: param];
		[self syncMaxValueBoxFromPreset: param];
		[self syncSourceBoxFromPreset: param];
	}
	[self syncBlueWindowFromPreset];
}

-(void)syncBlueWindowFromPreset
{
	[fBlueWindow setX: [self getPreset]->GetSourceValue(0)];
	[fBlueWindow setY: [self getPreset]->GetSourceValue(1)];
	[fBlueWindow setZ: [self getPreset]->GetSourceValue(2)];

	// seems to be necessary ?? [ts]
	[fBlueWindow display];
}

-(void)syncValueBoxFromPreset:(SInt32)param
{
	[fParamValueTextBox[param] setFloatValue:[self getPreset]->GetParamValue(param)];
}

-(void)syncMaxValueBoxFromPreset:(SInt32)param
{
	[fMaxValueTextBox[param] setFloatValue:[self getPreset]->GetParamMaxValue(param)];
}

-(void)syncMinValueBoxFromPreset:(SInt32)param
{
	[fMinValueTextBox[param] setFloatValue:[self getPreset]->GetParamMinValue(param)];
}

-(void)syncSliderFromPreset:(SInt32)param
{
	ARPIBlueWindowPreset* theBWPreset = [self getPreset];
	ZzAssert(theBWPreset);
	float val = theBWPreset->GetParamValue(param);
	[fSlider[param] setFloatValue:
		[self getFractionFromValue:val Param:param]];
		
}

-(void)syncSourceBoxFromPreset:(SInt32)param
{
	fBWLink[param] = [self getPreset]->GetParamSource(param);
	[fBWCursorButton[param] setImage:fBWLinkImages[fBWLink[param]]];
	
}

#pragma mark ----- SYNC PLUGIN FROM PRESET

-(void)syncPluginParamFromPreset:(SInt32)param
{
	float value = [self getPreset]->GetParamValue(param);
	[self getProcessor]->SetAudioPluginParamValue(param,value);
}

-(void)syncPluginFromPreset
{
	for (int param=0; param < fNbParams; param++) [self syncPluginParamFromPreset:param];
}


/* change the user preset name */
- (void)AdjustUserPreset
{
	fUserPreset = true;
	[fPresetButton selectItemAtIndex:fUserPresetIndex];
	
}

#pragma mark ----- INTERFACE ACTION HANDLERS

- (void)fBlueWindowAction
{
	[fMIObject markModified];
	[[fMIObject montageDoc] updateDocumentChangeCount:MIChangeDone];
	if (!fUserPreset) [self AdjustUserPreset];
	[self syncPresetFromBlueWindow];
	for (int param = 0; param < fNbParams; param++)
	{
		SInt32 source = [self getPreset]->GetParamSource(param);
		if (source)
		{
			[fSlider[param] setFloatValue:[self getPreset]->GetSourceValue(source - 1)];

			[self syncPresetFromSlider: param];
			[self syncValueBoxFromPreset: param];
			[self syncSliderFromPreset: param];
			[self syncPluginParamFromPreset: param];
		}
	
	}
}

- (IBAction)fBWCursorButtonAction:(id)sender
{
	[fMIObject markModified];
	[[fMIObject montageDoc] updateDocumentChangeCount:MIChangeDone];
	if (!fUserPreset) [self AdjustUserPreset];
	for( SInt32 param = 0; param < fNbParams; param++ )
	{
		if( sender == fBWCursorButton[param] )
		{
			SInt32 source = [self getPreset]->GetParamSource(param);
			source++;
			if(source > 3) source = 0;
			[self getPreset]->SetParamSource(param, source);
			[fBWCursorButton[param] setImage:fBWLinkImages[source]];
			return;
		}
	}
}

- (IBAction)fMinValueTextBoxAction:(id)sender
{
	[fMIObject markModified];
	[[fMIObject montageDoc] updateDocumentChangeCount:MIChangeDone];
	if (!fUserPreset) [self AdjustUserPreset];
	for( SInt32 param = 0; param < fNbParams; param++ )
	{
		if( sender == fMinValueTextBox[param] )
		{
			[self syncPresetFromMinValueBox: param];
			[self syncPresetFromValueBox: param];
			[self syncMinValueBoxFromPreset: param];
			[self syncValueBoxFromPreset: param];
			[self syncSliderFromPreset: param];
			[self syncPluginParamFromPreset: param];
			return;
		}
	}
	ZzAssert(0);
}

- (IBAction)fSliderAction:(id)sender
{
	[fMIObject markModified];
	[[fMIObject montageDoc] updateDocumentChangeCount:MIChangeDone];
	if (!fUserPreset) [self AdjustUserPreset];
	for( SInt32 param = 0; param < fNbParams; param++ )
	{
		if( sender == fSlider[param] )
		{
			[self syncPresetFromSlider: param];
			[self syncValueBoxFromPreset: param];
			[self syncSliderFromPreset: param];
			[self syncPluginParamFromPreset: param];
			return;
		}
	}
	ZzAssert(0);
}

- (IBAction)fParamValueTextBoxAction:(id)sender
{
	[fMIObject markModified];
	[[fMIObject montageDoc] updateDocumentChangeCount:MIChangeDone];
	if (!fUserPreset) [self AdjustUserPreset];

	for( SInt32 param = 0; param < fNbParams; param++ )
	{
		if( sender == fParamValueTextBox[param] )
		{
			[self syncPresetFromValueBox: param];
			[self syncValueBoxFromPreset: param];
			[self syncSliderFromPreset: param];
			[self syncPluginParamFromPreset: param];
			return;
		}
	}
	ZzAssert(0);
}

- (IBAction)fMaxValueTextBoxAction:(id)sender
{
	[fMIObject markModified];
	[[fMIObject montageDoc] updateDocumentChangeCount:MIChangeDone];
	if (!fUserPreset) [self AdjustUserPreset];
	for( SInt32 param = 0; param < fNbParams; param++ )
	{
		if( sender == fMaxValueTextBox[param] )
		{
			[self syncPresetFromMaxValueBox: param];
			[self syncPresetFromValueBox: param];
			[self syncMaxValueBoxFromPreset: param];
			[self syncValueBoxFromPreset: param];
			[self syncSliderFromPreset: param];
			[self syncPluginParamFromPreset: param];
			return;
		}
	}
	ZzAssert(0);
}

- (IBAction)fSavePresetButtonAction:(id)sender
{

}

- (IBAction)fByPassButtonAction:(id)sender
{
	[fMIObject markModified];
	[[fMIObject montageDoc] updateDocumentChangeCount:MIChangeDone];
	[self getProcessor]->SetAudioPluginByPass([sender state] == NSOnState);
}

- (IBAction)fPresetButtonAction:(id)sender
{

	[fMIObject markModified];
	[[fMIObject montageDoc] updateDocumentChangeCount:MIChangeDone];

	int preset = [sender indexOfSelectedItem];
	if (preset != fDummyPreset)
	{
		[self setPreset: fPluginClass->CreatePresetFromFactoryPreset(preset)];
		[self syncInterfaceFromPreset];
		[self syncPluginFromPreset];
		fUserPreset = false;
	}

	
#ifdef DEBUG	
	
	else
	{
		char* sourcename[] = {"BW X-axis", "BW Y-axis", "BW Z-axis"};
		printf("// %s preset\n", fPluginClass->GetPluginName());
	
		printf("PRESET_NAME(\"%s\")\n",[self getPreset]->GetName());
		//printf("PRESET_NAME(\"Preset name\");\n");
		for (int param=0; param<fNbParams; param++)
		{
			float val = [self getPreset]->GetParamValue(param);
			float min = [self getPreset]->GetParamMinValue(param);
			float max = [self getPreset]->GetParamMaxValue(param);
			SInt32 source = [self getPreset]->GetParamSource(param);
			char *name = fParamInfo[param].name;
			printf("PRESET_PARAM(%f, %f, %f, %d); //%s\n", val, min, max, (int)source, name);
		}
		for (int control=0; control<3; control++)
		{
			float sourceval = [self getPreset]->GetSourceValue(control);
			printf("PRESET_SOURCE(%f); //%s\n", sourceval, sourcename[control]);
		}
		printf("PRESET_NEXT;\n");

	};

#endif
	[self getPreset]->SetName([[fPresetButton itemTitleAtIndex:preset] cString]);
}

-(const VARPIPluginClass*)audioPluginClass
{
	TArmeWSObj* obj = [fMIObject WSObject];
	ZzAssert(obj);
	return obj->GetAudioPluginClass();
}

@end
