/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
 
#ifdef WIN32
#include "stdafx.h"
#endif //WIN32

#include "ARPIBlueWindowPreset.h"

#ifndef WIN32
#ifndef ZzVerbose_h
#include "../../../Zz/ZzVerbose.h"
#endif //ZzVerbose_h
#endif //WIN32

#ifndef ZzRtAssert_h
#include "../../../Zz/ZzRtAssert.h"
#endif //ZzRtAssert_h

#ifndef ARBORETUM_RGPAU
#ifndef NO_ARME_ARCHIVE
#ifndef ArmeArchiveHandler_h
#include "ArmeArchiveHandler.h"
#endif //ArmeArchiveHandler_h
#endif //#ifndef NO_ARME_ARCHIVE
#endif //#ifndef ARBORETUM_RGPAU

// private methods

void ARPIBlueWindowPreset::UpdateNbParams(SInt32 nbParams)
{
	ZzRtAssert(nbParams <= ARPIBlueWindowPresetMaxNbParams);
	if (nbParams > fNbParams) fNbParams = nbParams;
}
void ARPIBlueWindowPreset::UpdateNbSources(SInt32 nbSources)
{
	ZzRtAssert(nbSources <= ARPIBlueWindowPresetMaxNbSources);
	if (nbSources > fNbSources) fNbSources = nbSources;
}


// init stuff

ARPIBlueWindowPreset::ARPIBlueWindowPreset()
{
	memset(&fParam, 0, ARPIBlueWindowPresetMaxNbParams*sizeof(ARPIBlueWindowParamState));
	memset(&fSource, 0, ARPIBlueWindowPresetMaxNbSources*sizeof(float));
	strncpy(fName, "Unnamed Preset", ARPIBlueWindowPresetNameLength);
	fNbParams = 0;
	fNbSources = 0;
	
}
ARPIBlueWindowPreset::~ARPIBlueWindowPreset(){}

void ARPIBlueWindowPreset::AddSource(float sourceVal)
{
	UpdateNbSources(fNbSources + 1);
	fSource[fNbSources-1] = sourceVal;
}
void ARPIBlueWindowPreset::AddSourceArray(float *sourceArray, SInt32 nbElements)
{
	for (int i=0; i<nbElements; i++) AddSource(sourceArray[i]);
}
void ARPIBlueWindowPreset::AddParam(ARPIBlueWindowParamState* paramState)
{
	UpdateNbParams(fNbParams + 1);
	memcpy(&fParam[fNbParams-1], paramState, sizeof(ARPIBlueWindowParamState));
}
void ARPIBlueWindowPreset::AddParamArray(ARPIBlueWindowParamState* paramStateArray, SInt32 nbElements)
{
	for (int i=0; i<nbElements; i++) AddParam(&paramStateArray[i]);
}
	
// init / re-init stuff

void ARPIBlueWindowPreset::SetNbParams(SInt32 nbParams)
{
	ZzRtAssert(nbParams <= ARPIBlueWindowPresetMaxNbParams);
	fNbParams = nbParams;
}

void ARPIBlueWindowPreset::SetNbSources(SInt32 nbSources)
{
	ZzRtAssert(nbSources <= ARPIBlueWindowPresetMaxNbSources);
	fNbSources = nbSources;
}


void ARPIBlueWindowPreset::SetName(const char* name)
{
	int maxIndex = (ARPIBlueWindowPresetNameLength - 1);
	strncpy(fName, name, maxIndex);
	fName[maxIndex] = 0;
}
void ARPIBlueWindowPreset::SetSourceValue(SInt32 source, float value)
{
	ZzRtAssert(source < fNbSources); 
	fSource[source] = value;
}
void ARPIBlueWindowPreset::SetParamSource(SInt32 param, SInt32 source)
{
	ZzRtAssert(param < fNbParams); 
	fParam[param].Source = source;

}
void ARPIBlueWindowPreset::SetParamValue(SInt32 param, float value)
{
	ZzRtAssert(param < fNbParams); 
	fParam[param].Value = value;

}
void ARPIBlueWindowPreset::SetParamMinValue(SInt32 param, float minValue)
{
	ZzRtAssert(param < fNbParams); 
	fParam[param].MinValue = minValue;
}
void ARPIBlueWindowPreset::SetParamMaxValue(SInt32 param, float maxValue)
{
	ZzRtAssert(param < fNbParams); 
	fParam[param].MaxValue = maxValue;
}

	
// query stuff

SInt32 ARPIBlueWindowPreset::GetNbSources(){return fNbSources;}
SInt32 ARPIBlueWindowPreset::GetNbParams(){return fNbParams;}

const char * ARPIBlueWindowPreset::GetName() const
{
	return fName;
}

float ARPIBlueWindowPreset::GetSourceValue(SInt32 source)
{
	ZzRtAssert(source < fNbSources);
	return fSource[source];
}

SInt32 ARPIBlueWindowPreset::GetParamSource(SInt32 param)
{
	ZzRtAssert(param < fNbParams);
	return fParam[param].Source;

}
float ARPIBlueWindowPreset::GetParamValue(SInt32 param)
{
	ZzRtAssert(param < fNbParams);
	return fParam[param].Value;
}

float ARPIBlueWindowPreset::GetParamMinValue(SInt32 param)
{
	ZzRtAssert(param < fNbParams);
	return fParam[param].MinValue;
}


float ARPIBlueWindowPreset::GetParamMaxValue(SInt32 param)
{
	ZzRtAssert(param < fNbParams);
	return fParam[param].MaxValue;
}

#ifndef ARBORETUM_RGPAU
#ifndef NO_ARME_ARCHIVE
void ARPIBlueWindowPreset::ReadFromArchive(ArmeArchiveHandler* unarchiver)
{
	// class ARPIBlueWindowPreset data :
	// char							fName[ARPIBlueWindowPresetNameLength];
	// ARPIBlueWindowParamState		fParam[ARPIBlueWindowPresetMaxNbParams];
	// float						fSource[ARPIBlueWindowPresetMaxNbSources];
	// SInt32 fNbParams;			
	// SInt32 fNbSources;
	SInt32 i;
	unarchiver->ReadRawData((void*)fName,ARPIBlueWindowPresetNameLength*sizeof(char));
	for(i = 0; i < ARPIBlueWindowPresetMaxNbParams; i++)
	{
		fParam[i].ReadFromArchive(unarchiver);
	}
	for(i = 0; i < ARPIBlueWindowPresetMaxNbSources; i++)
	{
		fSource[i] = unarchiver->ReadFloat();
	}
	fNbParams = unarchiver->ReadSInt32();
	fNbSources = unarchiver->ReadSInt32();
}

void ARPIBlueWindowPreset::WriteToArchive(ArmeArchiveHandler* archiver) const
{
	// class ARPIBlueWindowPreset data :
	// char							fName[ARPIBlueWindowPresetNameLength];
	// ARPIBlueWindowParamState		fParam[ARPIBlueWindowPresetMaxNbParams];
	// float						fSource[ARPIBlueWindowPresetMaxNbSources];
	// SInt32 fNbParams;			
	// SInt32 fNbSources;
	SInt32 i;
	archiver->WriteRawData((void*)fName,ARPIBlueWindowPresetNameLength*sizeof(char));
	for(i = 0; i < ARPIBlueWindowPresetMaxNbParams; i++)
	{
		fParam[i].WriteToArchive(archiver);
	}
	for(i = 0; i < ARPIBlueWindowPresetMaxNbSources; i++)
	{
		archiver->WriteFloat(fSource[i]);
	}
	archiver->WriteSInt32(fNbParams);
	archiver->WriteSInt32(fNbSources);
}

void ARPIBlueWindowParamState::ReadFromArchive(ArmeArchiveHandler* unarchiver)
{
	Value = unarchiver->ReadFloat();
	MinValue = unarchiver->ReadFloat();
	MaxValue = unarchiver->ReadFloat();
	Source = unarchiver->ReadSInt32();
}

void ARPIBlueWindowParamState::WriteToArchive(ArmeArchiveHandler* archiver) const
{
	archiver->WriteFloat(Value);
	archiver->WriteFloat(MinValue);
	archiver->WriteFloat(MaxValue);
	archiver->WriteSInt32(Source);
}
#endif //#ifndef NO_ARME_ARCHIVE
#endif //#ifndef ARBORETUM_RGPAU

//float Value;		// value of the slider, or float encodes int index of an optionbox
//float MinValue;		// minimum bound of the slider
//float MaxValue;		// maximum bound of the slider, or float encoded int nbOptions
//SInt32 Source;		// blue window source parameter for this slider 
