/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "ARPIBlueWindowSubview.h"
#import "ARPIBlueWindow.h"
#import "MIImageController.h"

NSPoint pt1;

@implementation ARPIBlueWindowSubview

NSPoint pt2;

-(void)drawRect:(NSRect)aRect
{
	// draw background
	[self clearWithBackgroundColor:aRect];

	// draw logo
	NSRect dstRect;
	dstRect = [self bounds];
	dstRect.size.height = [mainImageController->fBlueWindowLogo size].height;

	// border
	dstRect = NSInsetRect(dstRect, 5, 0);
	
	// aspect ratio
	//dstRect.size.height = NSWidth(dstRect) * ([mainImageController->fBlueWindowLogo size].height/[mainImageController->fBlueWindowLogo size].width);
	
	dstRect.size.height = NSWidth(dstRect) * ([mainImageController->fBlueWindowLogo size].height/[mainImageController->fBlueWindowLogo size].width);

	if ( NSIntersectsRect(aRect, dstRect) || NSContainsRect(aRect, dstRect) )
	{
		[mainImageController->fBlueWindowLogo 
			drawInRect:dstRect 
			fromRect:NSMakeRect(0,0, [mainImageController->fBlueWindowLogo size].width,
				[mainImageController->fBlueWindowLogo size].height) 
			operation:NSCompositeSourceAtop fraction:1]; // fraction == alpha multiplier
	}
	
	// draw cursor
	if(NSIntersectsRect([self getCursorRect], aRect))
	[self drawCursor];
}

- (void)loadCursorImage
{
	NSBundle*	theBundle;

    theBundle = [NSBundle mainBundle];

	fCursorImage = [[NSImage alloc] initWithContentsOfFile:[theBundle pathForResource:@"BWCursor" ofType:@"tiff"]];
}

- (BOOL)acceptsFirstMouse:(NSEvent*)theEvent
{
    // this tells that the view considers the first click as part of the process and not only a way to give
    // the view the focus
    return YES;
}

- (void)mouseDown:(NSEvent*)theEvent
{
	// the mouse button has been pressed
	NSPoint    		mouseLoc;
	unsigned int	modifiers;
	
	// retrieve modifers such as ALT/COMMAND/SHIFT/...
	modifiers = [theEvent modifierFlags];
	
	// convert the mouse location of the event to the local coordinate system
	mouseLoc = [self convertPoint:[theEvent locationInWindow] fromView:nil];
	float pressure = [theEvent pressure];

	// actually change the cursor pos and redraw
	[self mouseClickOrDrag:mouseLoc withPressure:pressure];
}

- (void)mouseClickOrDrag:(NSPoint)mouseLoc withPressure:(float)pressure
{
//	qrintf("mouseClickOrDrag mouseLoc : (%f,%f) pressure : %f\n",mouseLoc.x,mouseLoc.y,pressure);
    if(!NSPointInRect(mouseLoc, [self bounds]))
    {
        // the mouse has been dragged outside the boundaries of the view,
		if( mouseLoc.x < 0.0 )
			mouseLoc.x = 0.0;
		if( mouseLoc.x >= [self bounds].size.width )
			mouseLoc.x = [self bounds].size.width;
		if( mouseLoc.y < 0.0 )
			mouseLoc.y = 0.0;
		if( mouseLoc.y >= [self bounds].size.height )
			mouseLoc.y = [self bounds].size.height;
    }
	
	// let's compute the new cursor position
	// (put the center of the image 
	NSPoint newCursorPos;
	newCursorPos.x = mouseLoc.x - [fCursorImage size].width/2.0;
	newCursorPos.y = mouseLoc.y - [fCursorImage size].width/2.0;
	if(newCursorPos.x < 0)
		newCursorPos.x = 0;
	if(newCursorPos.y < 0)
		newCursorPos.y = 0;
	if(newCursorPos.x > [self bounds].size.width - [fCursorImage size].width )
		newCursorPos.x = [self bounds].size.width - [fCursorImage size].width;
	if(newCursorPos.y > [self bounds].size.height - [fCursorImage size].height )
		newCursorPos.y = [self bounds].size.height - [fCursorImage size].height;

	// compute the rectangle completely containing the new rectangle and
	// the old one so that we will be able to call displayRect on this
	// rectangle.
	NSRect updateRect = NSUnionRect([self getCursorRect],NSMakeRect(newCursorPos.x,newCursorPos.y,[fCursorImage size].width,[fCursorImage size].height));
	
	// tell the view we're going to draw
	[self lockFocus];
		
	// let's clear the previous rectangle
	[self clearWithBackgroundColor:[self getCursorRect]];
	
	// set the new cursor pos
	fCursorPos = newCursorPos;
	fPressure = pressure;
	
	// let's draw the cursor
	[self drawCursor];
	
	// we're finished drawing
	[self unlockFocus];
	
	// ask the view to redraw itself
	[self displayRect:updateRect];

	// warn the super view the cursor pos has changed
	[(ARPIBlueWindow*)[self superview] fBlueWindowAction];

}

-(float)getX
{
	float x = fCursorPos.x/([self bounds].size.width - [self getCursorRect].size.width);
	return x;
}

-(float)getY
{
	float y = fCursorPos.y/([self bounds].size.height - [self getCursorRect].size.height);
	return y;
}

-(float)getZ
{
	return fPressure;
}

-(void)setX:(float)x
{
	fCursorPos.x = x * ([self bounds].size.width - [self getCursorRect].size.width);
	[self drawCursor];
}

-(void)setY:(float)y
{
	fCursorPos.y = y * ([self bounds].size.height - [self getCursorRect].size.height);
	[self drawCursor];
}

-(void)setZ:(float)z
{
	fPressure = z;
}

- (void)mouseDragged:(NSEvent*)theEvent
{
    // the mouse button is still on and the mouse has moved
    NSPoint    		mouseLoc;
    unsigned int	modifiers;

    // retrieve modifers such as ALT/COMMAND/SHIFT/...
    modifiers = [theEvent modifierFlags];

    // convert the mouse location of the event to the local coordinate system
    mouseLoc = [self convertPoint:[theEvent locationInWindow] fromView:nil];
	float pressure = [theEvent pressure];

	// actually change the cursor pos and redraw
	[self mouseClickOrDrag:mouseLoc withPressure:pressure];
}

- (void)drawCursor
{
	[fCursorImage compositeToPoint:fCursorPos operation:NSCompositeCopy];
}

- (void)clearWithBackgroundColor:(NSRect)rect
{
	[[NSColor blueColor] set];
	[NSBezierPath fillRect:rect];
}

- (NSRect)getCursorRect
{
	NSRect rc;
	rc.origin = fCursorPos;
	rc.size = [fCursorImage size];
	return rc;
}


- (void)mouseUp:(NSEvent*)theEvent
{
    // the mouse button has been released

    NSPoint    		mouseLoc;
    unsigned int	modifiers;

    // retrieve modifers such as ALT/COMMAND/SHIFT/...
    modifiers = [theEvent modifierFlags];

    // convert the mouse location of the event to the local coordinate system
    mouseLoc = [self convertPoint:[theEvent locationInWindow] fromView:nil];

}

@end
