/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#define TARPIHost_h

#ifndef TARPIHost_h
#include "TARPIHost.h"
#endif //TARPIHost_h

#include <CoreMIDI/MIDIServices.h>

#ifndef ZzVerbose_h
//#include "ZzVerbose.h"
#endif //ZzVerbose_h

#ifndef ZzSlowMap_h
#include "ZzSlowMap.h"
#endif //ZzSlowMap_h

#ifndef VARPIPluginClass_h
#include "VARPIPluginClass.h"
#endif VARPIPluginClass_h

/*!
	@class TARPIHost This object is passed both to the plugin and to the
	plugin interface view at creation time. It represents the host application
	and provides services.
*/

class VARPIPluginSet;
class VARPIPluginClass;
class VARPIPlugin;
class TArmeWSObj;

class TARPIHost
{
public :
	/*! @function Constructor */
				TARPIHost();

	/*! @function Destructor */
	virtual		~TARPIHost();

	/*! @function SpawnMIDIThread Attaches a callback to the MIDI in messages */
	void		SpawnMIDIThread();
		
	/*! 
		@function SetValue This function must be called by the view (controller) when
		a processor parameter needs to be changed 
	*/
	void		SetValueFromView(void* view,SInt32 paramIndex,float val);
	
	/*!
		@function GetProcessor This function allows a view to retrieve its associated
		plugin. 
	*/
	TArmeWSObj*		GetProcessor(void* view);
	
	/*!
		@function DeleteView This sets the view value to NULL
	*/
	void				DeleteView(void* view);
		
	/*!
		@function GetNbPluginSets Retrieves the number of plugin sets that have
		been registered at initialization time.
	*/
	SInt32					GetNbPluginSets() { return fPluginSetArray.CountItems(); } 
	
	/*!
		@function GetPluginSet Retrieves the Nth plugin set.
	*/
	VARPIPluginSet*		GetPluginSet(SInt32 set) { return fPluginSetArray.ItemAt(set); }
	
	VARPIPluginClass*	GetAudioPluginClassFromSig(const VARPIPluginClassSig& sig);
	
	/*
		@function CreatePlugin This is the recommended way to create a plug-in which allows
		the host to track the view interface/plugin relationship $$$TODO ownership ?
	*/
	VARPIPlugin*			CreatePlugin(const VARPIPluginClass* pluginClass);

		
	void*					CreateProcessorView(void* miobject);
	
	/*!
		@function GetProcessorFromControllerNumber This function retrieves the processor of the
	    param associated with a particular MIDI controller ($$$TODO unspecialize from the
		4CONTROL)
	*/
	TArmeWSObj*				GetProcessorFromControllerNumber(SInt32 controller);

	/*!
		@function GetParamFromControllerNumber This function retrieves the index of the param 
		associated with a particular MIDI controller ($$$TODO unspecialize from the 4CONTROL)
	*/
	SInt32					GetParamNumberFromControllerNumber(SInt32 controller);
	
	/*
		@function CreateBlueWindowPluginView This function instantiates a new default 
		view for a plugin. $$$TODO ownership ?
		Please note you will be able create as many views for a plug as needed....
		BUT NOW ($$$TODO) you can only create one view per plugin
	*/
	void*					CreateBlueWindowPluginView(void* miobject);
	
	
	void					CloseWindowForProcessor(TArmeWSObj* processor);
	
	/*
		@function myMIDIReadProc This is called by CoreMIDI when MIDI packets arrive. Right
		now, it is hardwired to interpret data from the 4CONTROL device by Native Instruments
	*/
	void 					MyMIDIReadProc(const MIDIPacketList *pktlist, void *readProcRefCon, void *srcConnRefCon);
	/*!
		@function LoadPluginSets The base class version of this function should load all
	    the arpi plugin sets from the applicaion ARPI sub-folder of the application, from the
		/Library/Audio/Plug-Ins/ARPI/ folder and from the ~/Library/Audio/Plug-Ins/ARPI/ folder.
		You can override it, not to add search folders ($$$TODO this will be done using another
		function) but to add classes that are maybe defined in the application binary rather than
		in bundles. Simply add VARPIPluginSet* pointers to the fPluginSetList array. Don't forget
		to call the base version before or after (it doesn't matter, except perhaps for the order
		in which plugins appear in a list box but, then, they should be sorted alphabetically,
		both the plugin sets among themselves and both the pluginclasses in a given set. ($$$TODO ?)
	*/
	virtual void 			LoadPluginSets();

private :
	ZzArray<VARPIPluginSet*>			fPluginSetArray;
	ZzSlowMap							fProcessorToViewMap;
	ZzSlowMap							fViewToProcessorMap;
	MIDIEndpointRef 					fMIDIEndPointRefSrc;
	MIDIPortRef							fMIDIPortRef;
	MIDIClientRef						fMIDIClientRef;
	bool								fHasMidiSupport;
};




