/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
 
#ifdef WIN32
#include "stdafx.h"
#endif //WIN32

#ifndef VARPIPlugin_h
#include "VARPIPlugin.h"
#endif VARPIPlugin_h

#ifndef VARPIPluginClass_h
#include "VARPIPluginClass.h"
#endif VARPIPluginClass_h

#ifndef ZzRtAssert_h
#include "ZzRtAssert.h"
#endif //ZzRtAssert_h

//#import "CSLicensing.h"

// the length of the mute cycle for protection
#define VARPIPLUGIN_MUTECYCLE_TIMEUNITS 11

// this is hardcoded, but that's ok.
#define VARPIPLUGIN_MUTECYCLE_TIMEUNIT_LENGTH 44100 

// this is the value the clock should be initialized to
#define VARPIPLUGIN_MUTECYCLE_CLOCKINIT  VARPIPLUGIN_MUTECYCLE_TIMEUNIT_LENGTH * (VARPIPLUGIN_MUTECYCLE_TIMEUNITS - 1)


bool VARPIPlugin::IncrParamValue(SInt32 param, float val)
{
	return SetParamValue(param,GetParamValue(param)+val);
}

void VARPIPlugin::Reset() {}
	


VARPIPlugin::VARPIPlugin(const VARPIPluginClass* mother)
{	
	fClass = mother;
	fBufferNbFrames = -1;
	
	fByPass = false;
	fLastByPass = true;

	// start in an on state
	fClockState = VARPIPLUGIN_MUTECYCLE_CLOCKINIT;
	fSoundAmplitudeZero = false;
	
	// the clock is incremented with the nbFrames processed * this factor
	// so if it is 0, the clock never changes, and stays in the on state
	if(mother)
		fClockIncrementFactor = (SInt32)(mother->PluginIsDemo());
	else
		fClockIncrementFactor = 0;
	
	
	//fNbChannels = -1;
	//fFrameRate = -1.0; //only one value at -1 means uninitialized
}

bool VARPIPlugin::FormatChange() 
{ 
    return true;
}



void	VARPIPlugin::OutputBuffersClear(SInt32 nbFrames)
{
    SInt32 outputs = fClass->GetNbAudioOutputs();
    SInt32 byteSizePerChannel = sizeof(float) * nbFrames;
    for (SInt32 i=0; i<outputs; i++) 
        memset (GetOutputBuffer(i),0,GetOutputChannels(i) * byteSizePerChannel);
}

void	VARPIPlugin::OutputBuffersFade(SInt32 nbFrames, float initialGain)
{
    // the behaviour is only defined for initialGain == 0.0 or 1.0
    // initialGain == 0.0 => fadein, increment > 0
    // initialGain == 1.0 => fadeout, increment < 0
    float gainIncrement = (1.0f - 2.0f*initialGain) / (float)nbFrames;
    SInt32 outputs = fClass->GetNbAudioOutputs();
    for (SInt32 i=0; i<outputs; i++)
    {
        float gain = initialGain;
        SInt32 chans = GetOutputChannels(i);
        float* buffer = GetOutputBuffer(i);
        SInt32 j;
        switch(chans)
        {
        case 1:	
            for ( j=0; j<nbFrames; j++)
            {
                buffer[j] *= gain;
                gain += gainIncrement;
            }
            break;
        case 2:
            for ( j=0; j<2*nbFrames; j+=2)
            {
                buffer[j]   *= gain;
                buffer[j+1] *= gain;
                gain += gainIncrement;
            }
            break;
        default:
            for ( j=0; j<chans*nbFrames; j+=chans)
            {
                for (SInt32 k=0; k<chans; k++) buffer[j+k] *= gain;
                gain += gainIncrement;
            }
            break;
        }
    }
}

void	VARPIPlugin::ProcessFramesOverwrite(SInt32 nbFrames)
{
	assert(fInBufferPtr[0] != fOutBufferPtr[0]);
    // standard method for classes that don't implement this
    OutputBuffersClear(nbFrames);
    ProcessFramesAdd(nbFrames);
}

void 	VARPIPlugin::ProtectedProcessFramesAdd(SInt32 nbFrames)
{
    ProcessFramesAdd(nbFrames);
    OutputBuffersDemoMute(nbFrames);

    //This clears the entire summing bus instead of just the current plugin's output.
    //This is quite odd behaviour, but i dont see how to do it correctly without 
    //giving up too much efficiency or maintainability.
    //Since the point of all this is to annoy the user if he doesnt have a correct licence
    //i think its ok. [ts] probably this call will become depreciated
	
	// if you use ProcessFramesOverwrite it gives correct behaviour
	
	// also, this call DOES NOT SUPPORT bypass!!
	// use ProcessFramesOverwrite for bypass support
    
}


void	VARPIPlugin::CrossfadeInputOutput(SInt32 nbFrames, float init_source_amp, float source_amp_inc)
{
	int inputs = GetClass()->GetNbAudioInputs();
	int outputs = GetClass()->GetNbAudioInputs();
	
	int xfade_outputs = min (inputs, outputs);
	
  int o;
	for ( o=0; o < xfade_outputs; o++) //xfade the first inputs and outputs
	{
		// in theory, the nb of channels per input/output for a VARPI plugin is completely free
		// however in montage it is currently limited to all mono or all stereo
		// so we assume the nb of channels for input and output are equal.
		
		// if this constraint breaks it is time to think about another stratigy
		// this will boil down to mixing stereo to mono, convert to 5.1 sound etc... [ts]
		
		ZzRtAssert(fInBufferNbChannels[o] == fOutBufferNbChannels[o]);

		float* ibuf = fInBufferPtr[o];
		float* obuf = fOutBufferPtr[o];
		float amp = init_source_amp;
		float amp_inc = source_amp_inc;

		int nbChannels = fOutBufferNbChannels[o];

		for (int i=0; i<nbFrames*nbChannels; i+=nbChannels)
		{
			for (int k=0; k<nbChannels; k++) 
				obuf[k+i] = ibuf[k+i] * amp + obuf[k+i] * (1.0f - amp);
			amp += amp_inc;
		}		
		
	}
	for ( o=xfade_outputs; o<outputs; o++) // xfade the remaining outputs with 0 input
	{
		float* buf = fOutBufferPtr[o];
		float amp = (1.0f - init_source_amp);
		float amp_inc = -source_amp_inc;
		int nbChannels = fOutBufferNbChannels[o];

		for (int i=0; i<nbFrames*nbChannels; i+=nbChannels)
		{
			for (int k=0; k<nbChannels; k++) buf[k+i] *= amp;
			amp += amp_inc;
		}		
	
	}

}

void 	VARPIPlugin::ProtectedProcessFramesOverwrite(SInt32 nbFrames)
{
	// dont run plugin if bypass is on and was on
	if (!(fByPass && fLastByPass))     ProcessFramesOverwrite(nbFrames);

	// perform output mute for demo mode
    OutputBuffersDemoMute(nbFrames);

	// dont process output if bypass is off and was off
	if (! (fByPass || fLastByPass)) goto exit;
	
	
	float init_source_amp;
	float source_amp_inc;
	

	
	// bypass is being switched on: crossfade output (fade out) with input (fade in)
	if (fByPass && (!fLastByPass))
	{
		init_source_amp = 0.0f;
		source_amp_inc =  1.0f / (float)nbFrames;
	}

	// bypass is being switched off: crossfade input (fade out) with output (fade in)
	if((!fByPass) && fLastByPass)
	{
	
		init_source_amp = 1.0f;
		source_amp_inc = -1.0f / (float)nbFrames;
	}
	
	// bypass is on and was on last time, so just copy input to output without fade
	if (fByPass && fLastByPass)
	{
		init_source_amp = 1.0f;
		source_amp_inc = 0.0f;
	}

	// perform parametric crossfade
	CrossfadeInputOutput(nbFrames, init_source_amp, source_amp_inc);

exit:	
	// save bypass state
	fLastByPass = fByPass;
}

void	VARPIPlugin::SetByPass(bool bypass) 
{
	fByPass = bypass;
}


void 	VARPIPlugin::OutputBuffersDemoMute(SInt32 nbFrames)
{
//#ifdef VARPIDEMO
    //fclockstate counts the number of frames


    UInt32 divClock = VARPIPLUGIN_MUTECYCLE_TIMEUNIT_LENGTH; // the number of frames per time unit of protection cycle
    UInt32 modClock = VARPIPLUGIN_MUTECYCLE_TIMEUNITS;	// the number of time units the protection period (one will be silent)
    UInt32 modClockState = (fClockState / divClock) % modClock;
    
    if (modClockState==0){ // drop sound during this phase
        if (fSoundAmplitudeZero == false) 
        {
            // fucking how do i fucking fucking do this!!
            //challenge = createTimeChallenge ();
            //reply = [NSApp isHarmonyModeWithChallenge:challenge];
            //if (verifyChallenge (reply) == challenge)

            // fade out if we enter drop phase
            OutputBuffersFade(nbFrames, 1.0f);
            fSoundAmplitudeZero = true;
        }
        else OutputBuffersClear(nbFrames);
    }
    else {
        if (fSoundAmplitudeZero) 
        {
            // fade in if we enter play phase
            OutputBuffersFade(nbFrames, 0.0f);
            fSoundAmplitudeZero = false;
        }
    }
    fClockState += fClockIncrementFactor * nbFrames;
//#endif
}
