/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
 
#ifdef WIN32
  #include "stdafx.h"
#endif //WIN32

#ifndef ARBORETUM_RGPAU
#ifndef NO_ARME_ARCHIVE
  #ifndef ArmeArchiveHandler_h
  #include "ArmeArchiveHandler.h"
  #endif //ArmeArchiveHandler_h
#endif //NO_ARME_ARCHIVE
#endif //#ifndef ARBORETUM_RGPAU

#include "VARPIPluginClass.h"

// default preset implementation is a wrapper around ARPIParamInfo
// (which contains param meta data and one default)

SInt32					VARPIPluginClass::GetNbAudioInputs() const {return 1;}
SInt32					VARPIPluginClass::GetNbAudioOutputs() const {return 1;}
SInt32					VARPIPluginClass::GetNbFactoryPresets() const {return 1;}
const char*				VARPIPluginClass::GetFactoryPresetName(SInt32 preset) const {return "Default Preset";}

ARPIBlueWindowPreset*	VARPIPluginClass::CreatePresetFromFactoryPreset(SInt32 factoryPreset) const
{
	SInt32 nbParams = GetNbParams();
	ARPIBlueWindowPreset* preset = new ARPIBlueWindowPreset();
	ARPIParamInfo  paramInfo;
	ARPIBlueWindowParamState paramState; 
	
	preset->SetName("Default preset");
	preset->AddSource(0.5f); // X
	preset->AddSource(0.5f); // Y
	preset->AddSource(1.0f); // Z
	
	for (int param=0; param<nbParams; param++)
	{
		GetParamInfo(&paramInfo, param);
		paramState.Value = paramInfo.defVal;
		paramState.MaxValue = paramInfo.maxVal;
		paramState.MinValue = paramInfo.minVal;
		paramState.Source = 0;
		preset->AddParam(&paramState);
	}
	return preset;
}

VARPIPluginClassSig VARPIPluginClass::GetSig() const
{
	VARPIPluginClassSig sig;
	sig.type = VARPIPluginClassSig::kVARPI;
	sig.versionNumber = 0x00010000; //1.0
	strncpy(sig.manufacturer,GetManufacturerName(),32);
	sig.manufacturer[31] = 0;
	strncpy(sig.name,GetPluginName(),32);
	sig.name[31] = 0;
	memset(sig.uniqueData,0,32);
	return sig;
}

bool VARPIPluginClassSig::IsCompatibleWithArchivedPluginSig(const VARPIPluginClassSig& project)
{
	bool b1 = (type == project.type);
	bool b2 = (versionNumber >= project.versionNumber);
	bool b3 = (strcmp(manufacturer,project.manufacturer) == 0) ? true :  false;
	bool b4 = (strcmp(name,project.name) == 0) ? true : false;
	bool b5 = true;
	for(SInt32 i = 0; i < 32; i++)
	{
		if(uniqueData[i] != project.uniqueData[i])
		{
			b5 = false;
			break;
		}
	}
	return b1 && b2 && b3 && b4 && b5;
}

#ifndef ARBORETUM_RGPAU
#ifndef NO_ARME_ARCHIVE
void VARPIPluginClassSig::WriteToArchive(ArmeArchiveHandler* archiver) const
{
//	SInt32	type;
//	UInt32	versionNumber,
//	char	manufacturer[32];
//	char	name[32];
//	UInt8	uniqueData[32];
	archiver->WriteSInt32(type);
	archiver->WriteUInt32(versionNumber);
	archiver->WriteRawData(manufacturer,32);
	archiver->WriteRawData(name,32);
	archiver->WriteRawData(uniqueData,32);
}

void VARPIPluginClassSig::ReadFromArchive(ArmeArchiveHandler* unarchiver)
{
	type = unarchiver->ReadSInt32();
	versionNumber = unarchiver->ReadUInt32();
	unarchiver->ReadRawData(manufacturer,32);
	unarchiver->ReadRawData(name,32);
	unarchiver->ReadRawData(uniqueData,32);
}

#endif //#ifndef NO_ARME_ARCHIVE
#endif //#ifndef ARBORETUM_RGPAU

