/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
#define VARPIPluginClass_h
#ifndef VARPIPluginClass_h
#include "VARPIPluginClass.h"
#endif VARPIPluginClass_h

#ifndef VARPIPlugin_h
#include "VARPIPlugin.h"
#endif VARPIPlugin_h

#ifndef ARPIBlueWindowPreset_h
#include "ARPIBlueWindowPreset.h" 
#endif //ARPIBlueWindowPreset_h

#ifndef ZzRtAssert_h
#include "../../../Zz/ZzRtAssert.h"
#endif //ZzRtAssert_h

#ifndef WIN32
// not ready for WIN32
#ifndef ZzVerbose_h
#include "../../../Zz/ZzVerbose.h"
#endif //ZzVerbose_h
#endif //WIN32

#ifdef WIN32
#include <stdlib.h>
#endif //WIN32

enum ARPIUnit
{
	ARPIUnit_generic,
	ARPIUnit_scaleFactor,
	ARPIUnit_dB,
	ARPIUnit_ms,
	ARPIUnit_bpm,
	ARPIUnit_meter,
	ARPIUnit_second,
	ARPIUnit_percent,
	ARPIUnit_Hz,
	ARPIUnit_kHz,
	ARPIUnit_integer,
	ARPIUnit_QFactor,
    ARPIUnit_mps,							//meter per second
    ARPIUnit_semitone,
    ARPIUnit_degree,
    ARPIUnit_optionbox,						//this slider is an optionbox (an enum type)
    ARPIUnit_HzLinear,
    ARPIUnit_kHzLinear,
	ARPIUnit_force32bits = 0xffffffff
};

struct ARPIParamInfo
{
	char			name[32];
	ARPIUnit		unit;
          
    // absolute boundaries
    // values outside of this range don't make sense
    float			absMinVal;
    float			absMaxVal;
    
    
    // suggested boundaries for blue window
    // this is the "nominal operating range" of the parameter
    // values outside this range and inside the absolute boundaries
    // are "overstretched" parameters
 	float			minVal;
	float			maxVal;
    

    // the default value of the parameter
    float			defVal;
    
    // in case this is an option box (determined by ubit==ARPIUnit_optionbox)
    // the following will be defined, and all the float values are ignored
    SInt32 nbOptions;
    char **optionList;
};

class TARPIHost;

struct VARPIPluginClassSig
{
	enum Type
	{
		kVARPI,
		kVST,
		kAudioUnit
	};

	SInt32	type;
	UInt32	versionNumber;
	char	manufacturer[32];
	char	name[32];
	UInt8	uniqueData[32];
	
#ifndef ARBORETUM_RGPAU
#ifndef NO_ARME_ARCHIVE
	void	WriteToArchive(ArmeArchiveHandler* archiver) const;
	void	ReadFromArchive(ArmeArchiveHandler* unarchiver);
#endif //#ifndef NO_ARME_ARCHIVE
#endif //#ifndef ARBORETUM_RGPAU
	
	bool	IsCompatibleWithArchivedPluginSig(const VARPIPluginClassSig& project);
};

class VARPIPluginClass
{
public :
	void*								CreateBlueWindow();
	void								SetPluginIsDemo(bool isDemo){fIsDemo = isDemo;}
	bool								PluginIsDemo() const {return fIsDemo;}
	virtual								~VARPIPluginClass();
public :
	virtual const char*					GetManufacturerName() const = 0;
	virtual const char*					GetPluginName() const = 0;
	// by default, GetSig creates a sig with version 1 and filled with the plugin name in data.
	virtual VARPIPluginClassSig 		GetSig() const;
	
	virtual VARPIPlugin*				CreatePlugin() const = 0;
#ifndef ARBORETUM_RGPAU
	virtual void*						CreateView(void* miobject) const = 0;
#endif //ARBORETUM_RGPAU

	virtual SInt32						GetNbParams() const = 0;
	virtual void						GetParamInfo(ARPIParamInfo* paramInfo, SInt32 param) const = 0;

    virtual SInt32						GetNbAudioInputs() const;
    virtual SInt32						GetNbAudioOutputs() const;

	virtual SInt32						GetNbFactoryPresets() const;
	virtual const char*					GetFactoryPresetName(SInt32 preset) const;
	virtual ARPIBlueWindowPreset*		CreatePresetFromFactoryPreset(SInt32 factoryPreset) const;
/*

	virtual void			_reserved12();
	virtual void			_reserved13();
	virtual void			_reserved14();
	virtual void			_reserved15();
	virtual void			_reserved16();
	virtual void			_reserved17();
	virtual void			_reserved18();
	virtual void			_reserved19();
	virtual void			_reserved20();
	virtual void			_reserved21();
	virtual void			_reserved22();
	virtual void			_reserved23();
	virtual void			_reserved24();
	virtual void			_reserved25();
	virtual void			_reserved26();
	virtual void			_reserved27();
	virtual void			_reserved28();
	virtual void			_reserved29();
	virtual void			_reserved30();
	virtual void			_reserved31();
	virtual void			_reserved32();
*/

private:
	/* this is only to determine if the returned name should be
		a demofied name or not.  Plugin (Demo) or Plugin */
	bool fIsDemo;

};

inline VARPIPluginClass::~VARPIPluginClass()
{
#ifdef DEBUG
	printf("VARPIPluginClass::~VARPIPluginClass()\n");
#endif //#ifdef DEBUG
}

template <class DerivedType> class VARPIPluginClassBase : public VARPIPluginClass
{
public :
											VARPIPluginClassBase();
	virtual									~VARPIPluginClassBase();
//#ifndef AUDIO_UNITS_BUILD
	static DerivedType*						GetInstance();
private :
	static VARPIPluginClass*				fsInstance;		
//#endif //#ifndef AUDIO_UNITS_BUILD
};

//#ifndef AUDIO_UNITS_BUILD
template<class DerivedType> inline VARPIPluginClassBase<DerivedType>::VARPIPluginClassBase()
{
	//ZzRtAssert(fsInstance == NULL);
	fsInstance = this;
}

template<class DerivedType> inline VARPIPluginClassBase<DerivedType>::~VARPIPluginClassBase()
{
#ifdef DEBUG
	printf("VARPIPluginClassBase<DerivedType>::~VARPIPluginClassBase()\n");
#endif //#ifdef DEBUG
	fsInstance = NULL;
}

template<class DerivedType> inline DerivedType* VARPIPluginClassBase<DerivedType>::GetInstance()
{
	DerivedType* instance = dynamic_cast<DerivedType*>(fsInstance);
	ZzAssert(instance != NULL);
	return instance;
}

template <class DerivedType> VARPIPluginClass* VARPIPluginClassBase<DerivedType>::fsInstance = NULL;
//#endif //#ifndef AUDIO_UNITS_BUILD


