/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#define ArmeArchivable_h

#ifndef ArmeArchivable_h
#include "ArmeArchivable.h"
#endif //ArmeArchivable_h

#ifndef ArmeBasicTypes_h
#include "ArmeBasicTypes.h"
#endif //ArmeBasicTypes_h

#ifndef ArmeArchiveHandler_h
#include "ArmeArchiveHandler.h"
#endif //ArmeArchiveHandler_h

/*! 
	@class ArmeArchivable
	
	This class serves two purposes : 
	
	- it declares the interface the archivable objects must
	  present
	  
	- it implements the GetArchID method that assigns a DUID
	  to an object (Document Unique ID)
	  
	Archivable objects need to derive from this class.
	  
*/	

class ArmeArchiveHandler;

class ArmeArchivable
{
public :
	ArmeArchivable();
	virtual ~ArmeArchivable() {};
	
	/*! @function ArmeArchivable This ctor takes an ArmeArchiveHandler 
	    as parameter. It must be used when the object is unarchived. */
	ArmeArchivable(ArmeArchiveHandler* unarchiver);
	virtual bool		WriteToArchive(ArmeArchiveHandler* archiver) const;
	
#ifndef NO_VERBOSE_AT_ALL
	friend std::ostream& 	operator<<(std::ostream& s, ArmeArchivable& obj);
	virtual void		DumpToStream(std::ostream& s);
#endif //NO_VERBOSE_AT_ALL
};
