/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#define ArmeArchiveHandler_h

#ifndef ArmeArchiveHandler_h
#include "ArmeArchiveHandler.h"
#endif //ArmeArchiveHandler_h

#ifndef ArmeBasicTypes_h
#include "ArmeBasicTypes.h"
#endif //ArmeBasicTypes_h

#ifdef __OBJC__
#import <Cocoa/Cocoa.h>
//#import "MontageInterface.h"
#endif //__OBJC__

#include <Quicktime/Quicktime.h>


#define kMaxNbArchObjects 1048576

class ArmeArchivable;

enum
{
	kDebugArchTag_SInt32 = 1973,
	kDebugArchTag_UInt32,
	kDebugArchTag_RawData,
	kDebugArchTag_CPPObject,
	kDebugArchTag_ObjCObject,
	kDebugArchTag_SInt64,
	kDebugArchTag_bool,
	kDebugArchTag_BOOL,
	kDebugArchTag_ArmeTime,
	kDebugArchTag_Float,
	kDebugArchTag_NSRect,
	kDebugArchTag_NSPoint,
	kDebugArchTag_QTAtomContainer,
	kDebugArchTag_MISize,
	kDebugArchTag_MIPoint,
	kDebugArchTag_MIRect,
};

enum
{
	kArchiveHandlerMode_Writing,
	kArchiveHandlerMode_Reading,
	kArchiveHandlerMode_Idle,
};


class MEAudioClipData;

//#define DEBUG_MIOBJECTS_WITH_SAME_WSOBJ

struct MIRect;
struct MISize;
struct MIPoint;

class ArmeArchiveHandler
{
public :
			ArmeArchiveHandler();
	virtual	~ArmeArchiveHandler();

	char	fProjectDirectory[kMaxPathLength];
	const char* GetProjectDirectory() {return fProjectDirectory;};

	enum Error
	{
		NoErr = 0,
		FileOpeningError,
		FileWriteError,
		FileReadError,
		BadFileFormat,
		SomeError
	};
	
	Error 			WriteArchiveWithCPPHead(const char* archPath, ArmeArchivable* head);
	ArmeArchivable*		ReadArchiveWithCPPHead(const char* archPath);
#ifdef __OBJC__
	Error			WriteArchiveWithObjCHead(const char* archPath, id head, id objectNotToBeArchived);
	Error			WriteArchiveWithObjCHead(const char* archPath, id head);
	id			ReadArchiveWithObjCHead(const char* archFilePath, id objectNotArchived);
	id			ReadArchiveWithObjCHead(const char* archFilePath );
#endif //__OBJC__

	/*! @funtion GetOurSig Must return the sig of this project archive (for example,
	    the first four letters of the application) */
	virtual UInt32				GetOurSig() const = 0;

	/*! @funtion GetMaxSig Must return the maximum version number of archives this 
	    unarchiver can read */
	virtual UInt32				GetMaxVer() const = 0;

	/*! @funtion GetMinSig Must return the minimum version number of archives this 
	    unarchiver can read */
	virtual UInt32				GetMinVer() const = 0;
	
	/*! @funtion GetFileVer The version of the file we're reading. Only valid when
	    READING an archive */
	UInt32						GetFileVer() const;

	void 	SetCurrentlyBeingBuiltObjectInstanceID(SInt32 instanceID);
	SInt32	GetCurrentlyBeingBuiltObjectInstanceID();
	
	// those functions are public because objects use it in their constructors
	void	SetCPPObjForInstanceID(SInt32 instanceID, ArmeArchivable* obj);
#ifdef __OBJC__
	void	SetObjCObjForInstanceID(SInt32 instanceID, id obj);
#endif //__OBJC__

private :

	Error 	CheckArchiveHeader();
	Error	WriteArchiveHeader(const char* archFilePath);
	Error	_OpenWriteArchiveFile(const char* archFilePath);
	Error	_OpenReadArchiveFile(const char* archFilePath);
	bool 	_CloseArchiveFile();
	virtual bool	WriteInFile(const void* data, size_t size, size_t nb = 1);
	virtual bool	ReadFromFile(void* data, size_t size, size_t nb = 1);
	void 	AllocInitMappingTables();
	void 	DeleteMappingTables();
	
#ifdef __OBJC__
	id		GetObjCObjForInstanceID(SInt32 instanceID);
#endif
	ArmeArchivable* GetCPPObjForInstanceID(SInt32 instanceID);

	virtual const char*	GetNameFromClassID(UInt32 classID) = 0;
	
	SInt32 	GetCPPInstanceID(ArmeArchivable* obj);
	SInt32 	AllocateCPPInstanceID(ArmeArchivable* obj);
#ifdef __OBJC__
	SInt32 	GetObjCInstanceID(id obj);
	SInt32 	AllocateObjCInstanceID(id obj);
#endif //__OBJC__

protected :

// functions called by the concrete subclass

	virtual UInt32 	GetCPPClassID(ArmeArchivable* obj);
//#ifdef __OBJC__
//	virtual UInt32 	GetObjCClassID(id obj);
//#else
	virtual UInt32 	GetObjCClassID(void* obj); // dirty hack to avoid vtable trouble
//#endif //__OBJC__

	virtual ArmeArchivable*	CreateCPPObject(UInt32 classID);
#ifdef __OBJC__
	virtual id				CreateObjCObject(UInt32 classID);
#else
	virtual void*			CreateObjCObject(UInt32 classID);
#endif //__OBJC__

	static const char*	RemoveLeadingDigits(const char* src);
	static const char*	GetCPPObjectClassName(ArmeArchivable* obj);
#ifdef __OBJC__
	static const char*	GetObjCObjectClassName(id obj);
#endif //__OBJC__
		
public :
	bool	WriteRawData(const void* data, UInt32 nbBytes);
	bool	ReadRawData(void* data, UInt32 nbBytes); // needs to be pre-allocated

	bool 	WriteUInt32(const UInt32& data);
	UInt32 	ReadUInt32();
	
	bool 	WriteDebugTag(const UInt32& data);
	UInt32 	ReadDebugTag();

	bool 	WriteSInt32(const SInt32& data);
	SInt32	ReadSInt32();

	bool 	WriteFloat(const float& data);
	float	ReadFloat();

	bool	WriteQTAtomContainer(QTAtomContainer* atomContainer);
	bool	ReadQTAtomContainer(QTAtomContainer* atomContainer);
protected :
	bool	WriteQTAtom(QTAtomContainer* atomContainer, QTAtom parentAtom);
	bool 	ReadQTAtom(QTAtomContainer* atomContainer, QTAtom parentAtom);

public :
#ifdef __OBJC__
	bool	WriteNSRect(const NSRect& data);
	NSRect	ReadNSRect();
	
	bool	WriteNSPoint(const NSPoint& data);
	NSPoint	ReadNSPoint();

	bool	WriteMIRect(const MIRect& data);
	MIRect	ReadMIRect();
	
	bool	WriteMISize(const MISize& data);
	MISize	ReadMISize();
	
	bool	WriteMIPoint(const MIPoint& data);
	MIPoint ReadMIPoint();
	
#endif //__OBJC__	    

	bool 		WriteArmeTime(const ArmeTime& data);
	ArmeTime	ReadArmeTime();

	bool 		WriteSInt64(const SInt64& data);
	SInt64		ReadSInt64();

	bool 		WriteBool(const bool& data);
	bool		ReadBool();

#ifdef __OBJC__
	bool 		WriteBOOL(const BOOL& data);
	BOOL		ReadBOOL();
#endif //__OBJC__


//	bool 	WriteString(const char* string);
//	bool 	ReadString(char* string); // needs to be preallocated

#ifdef __OBJC__
	bool	WriteObjCObject(id obj);
	id		ReadObjCObject(const char* verifyClassName = NULL);
#endif //__OBJC__

	bool 			WriteCPPObject(ArmeArchivable* obj);
	ArmeArchivable*	ReadCPPObject();
		
private :

// $$$TODO remove this debug stuff
#ifdef DEBUG_MIOBJECTS_WITH_SAME_WSOBJ
#ifdef __OBJC__
	id							fMIObjectsArray;
#else
	void*						fMIObjectsArray;
#endif //__OBJC__
#endif //#ifdef DEBUG_MIOBJECTS_WITH_SAME_WSOBJ

	SInt32						fMode;
	FILE*						fArchFile;
	ArmeArchivable**		fCPPInstancesPtrs;
#ifdef __OBJC__
	id*							fObjCInstancesPtrs;
#else
	void**						fObjCInstancesPtrs;
#endif //__OBJC__
	SInt32						fNextCPPInstanceID;
	SInt32						fNextObjCInstanceID;
	
	SInt32						fCurrentlyBeingBuiltObjectInstanceID;
	
	UInt32						fThisFileVer;
};

