/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#ifndef ArmeArchiveHandler_h
#include "ArmeArchiveHandler.h"
#endif //ArmeArchiveHandler_h

#ifndef ArmeArchive_h
#include "ArmeArchive.h"
#endif //ArmeArchive_h

#ifndef ZzArchPtrArray_h
#include "ZzArchPtrArray.h"
#endif //ZzArchPtrArray_h

#ifndef ArmeArchivable_h
#include "ArmeArchivable.h"
#endif //ArmeArchivable_h

#ifndef ArmeObjCArchivable_h
#include "ArmeObjCArchivable.h"
#endif //ArmeObjCArchivable_h

#ifndef ZzVerbose_h
//#include "ZzVerbose.h"
#endif //ZzVerbose_h

#ifndef TArmeDoc_h
#include "TArmeDoc.h"
#endif //TArmeDoc_h

#ifndef MEAudioClipData_h
#include "MEAudioClipData.h"
#endif //MEAudioClipData_h

#ifndef MEAudioClip_h
#include "MEAudioClip.h"
#endif //MEAudioClip_h

#ifndef TArmeMasterMovie_h
#include "TArmeMasterMovie.h"
#endif //TArmeMasterMovie_h

#ifndef TVideoEffect_h
#include "TVideoEffect.h"
#endif //TVideoEffect.h

#ifndef TVideoClip_h
#include "TVideoClip.h"
#endif //TVideoClip_h

#ifndef TArmeTextClip_h
#include "TArmeTextClip.h"
#endif //TArmeTextClip_h

#ifndef ArmeDataFileRef_h
#include "ArmeDataFileRef.h"
#endif //ArmeDataFileRef_h

#ifndef MEMixingInfo_h
#include "MEMixingInfo.h"
#endif //MEMixingInfo_h

#include <typeinfo>

#import "MIObject.h"


#undef VER_DEF_TOPIC
#define VER_DEF_TOPIC archivingEngine

#include <iostream>

using namespace std;

ArmeArchiveHandler::ArmeArchiveHandler()
{
	vernl(5,"ArmeArchiveHandler::ArmeArchiveHandler()");
//	printf("ArmeArchiveHandler::ArmeArchiveHandler()\n");
	fMode = kArchiveHandlerMode_Idle;
	fThisFileVer = 0x00000000;
}

ArmeArchiveHandler::Error ArmeArchiveHandler::WriteArchiveHeader(const char* archFilePath)
{
	vernl(5,"ArmeArchiveHandler::WriteArchiveHeader(" << archFilePath << ")");
	
	Error 	err;
	bool	ok;
	
	err = _OpenWriteArchiveFile(archFilePath);
	if(err != NoErr)
		return err;
		
	// write the small header
	ok = WriteUInt32(kGeneralArmeArchiveSig);
	if(!ok)
		return FileWriteError;
	ok = WriteUInt32(kGeneralArmeArchiveVer);
	if(!ok)
		return FileWriteError;
	ok = WriteUInt32(GetOurSig());
	if(!ok)
		return FileWriteError;
	ok = WriteUInt32(GetMaxVer());
	if(!ok)
		return FileWriteError;
	return NoErr;
}

void ArmeArchiveHandler::AllocInitMappingTables()
{
	vernl(5,"ArmeArchiveHandler::AllocInitMappingTables()");
	fCPPInstancesPtrs = (ArmeArchivable**)malloc(kMaxNbArchObjects*sizeof(ArmeArchivable*));
	memset(fCPPInstancesPtrs,0,kMaxNbArchObjects*sizeof(ArmeArchivable*));
	fObjCInstancesPtrs = (id*)malloc(kMaxNbArchObjects*sizeof(id));
	memset(fObjCInstancesPtrs,0,kMaxNbArchObjects*sizeof(id));
	fNextCPPInstanceID = 0;
	fNextObjCInstanceID = 0;
}

void ArmeArchiveHandler::DeleteMappingTables()
{
	vernl(5,"ArmeArchiveHandler::DeleteMappingTables()");
	free(fCPPInstancesPtrs);
	free(fObjCInstancesPtrs);
}

ArmeArchiveHandler::Error ArmeArchiveHandler::WriteArchiveWithCPPHead(const char* archPath, ArmeArchivable* head)
{
        ZzAssert(fMode == kArchiveHandlerMode_Idle);
        fMode = kArchiveHandlerMode_Writing;
        vernl(5,"ArmeArchiveHandler::WriteArchiveWithCPPHead(" << archPath << ", " << head << ")");
        Error err;
        bool ok;
        AllocInitMappingTables();
        err = WriteArchiveHeader(archPath);
        if(err != NoErr)
            return err;
        ok = WriteCPPObject(head);
        fclose(fArchFile);
        DeleteMappingTables();
        fMode = kArchiveHandlerMode_Idle;
        if(!ok)
            return FileWriteError;
        else
            return NoErr;

}

ArmeArchiveHandler::Error ArmeArchiveHandler::WriteArchiveWithObjCHead(const char* archPath, id head )
{
    return this->WriteArchiveWithObjCHead( archPath, head, NULL );
}

ArmeArchiveHandler::Error ArmeArchiveHandler::WriteArchiveWithObjCHead(const char* archPath, id head, id objectNotToBeArchived )
{


	ZzAssert(fMode == kArchiveHandlerMode_Idle);
	fMode = kArchiveHandlerMode_Writing;
	
#ifdef DEBUG_MIOBJECTS_WITH_SAME_WSOBJ
	fMIObjectsArray = [[NSMutableArray alloc] init];
#endif //#ifdef DEBUG_MIOBJECTS_WITH_SAME_WSOBJ
	vernl(5,"ArmeArchiveHandler::WriteArchiveWithObjCHead(" << archPath << ", " << (void*)head << ")");
	Error err;
	bool ok;
	AllocInitMappingTables();
	err = WriteArchiveHeader(archPath);
	if(err != NoErr)
		return err;
        if ( objectNotToBeArchived ) AllocateObjCInstanceID(objectNotToBeArchived);
	ok = WriteObjCObject(head);
	fclose(fArchFile);
	DeleteMappingTables();
	fMode = kArchiveHandlerMode_Idle;

#ifdef DEBUG_MIOBJECTS_WITH_SAME_WSOBJ
	[fMIObjectsArray release];
#endif //#ifdef DEBUG_MIOBJECTS_WITH_SAME_WSOBJ

	if(!ok)
		return FileWriteError;
	else
		return NoErr;

}

bool ArmeArchiveHandler::WriteCPPObject(ArmeArchivable* obj)
{
	// for debug
	WriteDebugTag(kDebugArchTag_CPPObject);
	
	vernl(5,"ArmeArchiveHandler::WriteCPPObject(" << obj << ")");
	if(obj == NULL)
	{
		vernl(5,"obj == NULL -----> writing 0xfffffffe");
		WriteUInt32(0xfffffffe); // a classID of 0xfffffffe means a null CPP object
		return true;
	}
	
	// this function returns the instance ID 
	// of the object. This function returns 0 if
	// this object hasn't been saved in this CreateArchive()
	// call.
	SInt32 instanceID = GetCPPInstanceID(obj); 
	if(instanceID != -1)
	{
		// since the object has been saved before, we
		// only write a reference to it.
		// then we only write the instanceID in the archive.
		// otherwise, we write the full object.
		WriteUInt32(0); // a classID of 0 means an already existing object.
		WriteSInt32(instanceID);
	}
	else
	{
		// this function returns the enum for the real type of the object pointed
		// to by "archivable". It must be overridden in client archivers to take
		// client objects into account
		UInt32 classID = GetCPPClassID(obj);
		
		WriteUInt32(classID);
		instanceID = AllocateCPPInstanceID(obj);
		WriteSInt32(instanceID);
		obj->WriteToArchive(this);
	}
	return true;
}

bool ArmeArchiveHandler::WriteObjCObject(id obj)
{
	// for debug
	WriteDebugTag(kDebugArchTag_ObjCObject);

	vernl(5,"ArmeArchiveHandler::WriteObjCObject(" << obj << ")");
	
	if(obj == nil)
	{
		vernl(5,"obj == nil -----> writing 0xffffffff");
		WriteUInt32(0xffffffff); // a classID of 0xffffffff means a nil ObjC object
		return true;
	}

	// this function returns the instance ID
	// of the object. This function returns 0 if
	// this object hasn't been saved in this CreateArchive()
	// call.
	SInt32 instanceID = GetObjCInstanceID(obj);
	vvernl(5,instanceID);
	if(instanceID != -1)
	{
		// since the object has been saved before, we
		// only write a reference to it.
		// then we only write the instanceID in the archive.
		// otherwise, we write the full object.
		WriteUInt32(0); // a classID of 0 means an already existing object.
		WriteSInt32(instanceID);
	}
	else
	{
		// this function returns the enum for the real type of the object pointed
		// to by "archivable". It must be overridden in client archivers to take
		// client objects into account
		UInt32 classID = GetObjCClassID((void*)obj);

#ifdef DEBUG_MIOBJECTS_WITH_SAME_WSOBJ
		const char* className = [[obj archClassName] cString];
		// distinct miobjs ALWAYS have distinct wsobjs
		// this code helps tracking violations of this
		if(strcmp("MIObject",className) == 0)
		{
			ZzAssert(![fMIObjectsArray containsObject:obj])
			TArmeWSObj* wsobj = [obj WSObject];
			for(unsigned int i = 0; i < [fMIObjectsArray count]; i++)
			{
				MIObject* _obj = [fMIObjectsArray objectAtIndex:i];
				TArmeWSObj* _wsobj = [_obj WSObject];
				ZzAssert(wsobj != _wsobj);
			}
			[fMIObjectsArray addObject:obj];
		}
#endif //#ifdef DEBUG_MIOBJECTS_WITH_SAME_WSOBJ

		vvernl(5,classID);
		WriteUInt32(classID);
		instanceID = AllocateObjCInstanceID(obj);
		vvernl(5,instanceID);
		WriteSInt32(instanceID);
		[obj writeToArchive:this];
	}
	return true;
}

ArmeArchivable* ArmeArchiveHandler::ReadCPPObject()
{
	ZzAssert(fMode == kArchiveHandlerMode_Reading);
	// for debug
	UInt32 debugTag = ReadDebugTag();
	ZzAssert(debugTag == kDebugArchTag_CPPObject);
	
	ArmeArchivable* obj;
	vernl(5,"ArmeArchiveHandler::ReadCPPObject()");
	
	UInt32 classID = ReadUInt32();
	if(classID == 0xfffffffe) // a classID of 0xfffffffe means a NULL CPP object
	{
		vernl(5,"classID = 0xfffffffe -----> returning NULL CPP object");
		return NULL;
	}
	vernl(5,"classID = " << classID << "(" << GetNameFromClassID(classID) << ")");
	ZzAssert(classID != 0xffffffff); // this is for ObC objects only
		
	if(classID != 0)
	{
		// this means there is a real object in the archive
		SInt32 instanceID = ReadSInt32();
		vvernl(5,instanceID);
		SetCurrentlyBeingBuiltObjectInstanceID(instanceID);
		obj = CreateCPPObject(classID);
		vvernl(5,obj);
		
	}
	else
	{
		// it's only a reference to an instance...
		SInt32 instanceID = ReadSInt32();
		obj = GetCPPObjForInstanceID(instanceID);
		ZzAssert(obj != NULL);
	}
	return obj;
}

id ArmeArchiveHandler::ReadObjCObject(const char* verifyClassName)
{
	// for debug
	UInt32 debugTag = ReadDebugTag();
	ZzAssert(debugTag == kDebugArchTag_ObjCObject);

	id obj;
	vernl(5,"ArmeArchiveHandler::ReadObjCObject()");
	
	UInt32 classID = ReadUInt32();
	if(classID == 0xffffffff) // a classID of 0xffffffff means a nil ObjC object
	{
		vernl(5,"classID = 0xffffffff -----> returning NULL CPP object");
		return nil;
	}
	
	vernl(5,"classID = " << classID << "(" << GetNameFromClassID(classID) << ")");
	if(classID != 0)
	{
		// this means there is a real object in the archive
		SInt32 instanceID = ReadSInt32();
		vvernl(5,instanceID);
		SetCurrentlyBeingBuiltObjectInstanceID(instanceID);
		obj = CreateObjCObject(classID);
	}
	else
	{
		// it's only a reference to an instance...
		SInt32 instanceID = ReadSInt32();
		obj = GetObjCObjForInstanceID(instanceID);
		ZzAssert(obj != NULL);
	}

	// we check whether the ObjC object we read is of the same kind as the one we expect
	if(verifyClassName)
	{
		ZzAssert(strcmp(GetNameFromClassID(GetObjCClassID(obj)),verifyClassName) == 0);
	}

	return obj;
}

SInt32 ArmeArchiveHandler::GetCPPInstanceID(ArmeArchivable* obj)
{	
	for(SInt32 i = 0; i < fNextCPPInstanceID; i++)
	{
		if( fCPPInstancesPtrs[i] == obj)
			return i;
	}
	return -1;
}

SInt32 ArmeArchiveHandler::GetObjCInstanceID(id obj)
{
	for(SInt32 i = 0; i < fNextObjCInstanceID; i++)
	{
		if( fObjCInstancesPtrs[i] == obj)
			return i;
	}
	return -1;
}

SInt32 ArmeArchiveHandler::AllocateCPPInstanceID(ArmeArchivable* obj)
{
	fCPPInstancesPtrs[fNextCPPInstanceID] = obj;
	fNextCPPInstanceID++;
	ZzAssert(fNextCPPInstanceID < kMaxNbArchObjects);
	return fNextCPPInstanceID-1;
}

SInt32 ArmeArchiveHandler::AllocateObjCInstanceID(id obj)
{
	fObjCInstancesPtrs[fNextObjCInstanceID] = obj;
	fNextObjCInstanceID++;
	ZzAssert(fNextObjCInstanceID < kMaxNbArchObjects);
	return fNextObjCInstanceID-1;
}

UInt32 ArmeArchiveHandler::GetCPPClassID(ArmeArchivable* obj)
{
	verntl(archivingEngine,5,"ArmeArchiveHandler::GetCPPClassID");
	const char* className = GetCPPObjectClassName(obj);
	vverntl(archivingEngine,5,className);
	UInt32 type = 0;
#define ARCH_CLASS(a) if(strcmp(#a,className) == 0) type = kArchClass_##a;
#include "ArmeArchiveCPPClasses.h"
#undef ARCH_CLASS
	if(type == 0)
	{
		verntl(archivingEngine,5,"ArmeArchiveHandler::GetCPPClassID() for class " << className << " :  couldn't find type");
	}
	return type;
}

//UInt32 ArmeArchiveHandler::GetObjCClassID(id obj)
UInt32 ArmeArchiveHandler::GetObjCClassID(void* obj)
{
	const char* className = [[(id)obj archClassName] cString];
	UInt32 type = 0;
#define ARCH_CLASS(a) if(strcmp(#a,className) == 0) type = kArchClass_##a;
#include "ArmeArchiveObjCClasses.h"
#undef ARCH_CLASS
	if(type == 0)
	{
		verntl(archivingEngine,5,"ArmeArchiveHandler::GetObjCClassID() for class " << className << " :  couldn't find type");
	}
	return type;
}

ArmeArchiveHandler::Error ArmeArchiveHandler::_OpenWriteArchiveFile(const char* archFilePath)
{
	fArchFile = fopen(archFilePath,"w");
	if(!fArchFile)
	{
		verntl(archivingEngine,1,"warning : couldn't open the archive file");
		return FileOpeningError;
	}
	return NoErr;
}
	
bool ArmeArchiveHandler::WriteRawData(const void* data, UInt32 nbBytes)
{
	// for debug
	WriteDebugTag(kDebugArchTag_RawData);
	size_t nbObjects = WriteInFile(data,nbBytes,1);
	return(nbObjects == 1);
}

bool ArmeArchiveHandler::WriteUInt32(const UInt32& data)
{
	// for debug
	WriteDebugTag(kDebugArchTag_UInt32);
	size_t nbObjects = WriteInFile((void*)(&data),sizeof(UInt32),1);
	return(nbObjects == 1);
}

bool ArmeArchiveHandler::WriteSInt32(const SInt32& data)
{
	// for debug
	WriteDebugTag(kDebugArchTag_SInt32);
	size_t nbObjects = WriteInFile((void*)(&data),sizeof(SInt32),1);
	return(nbObjects == 1);
}

/*
bool ArmeArchiveHandler::WriteString(const char* string)
{
	UInt32 strLen = strlen(string);
	size_t nbObjects;
	bool ok = WriteUInt32(strLen);
	if(!ok)
		return false;
	nbObjects = WriteInFile((void*)string,strLen,1);
	return(nbObjects == 1);
}
*/

ArmeArchiveHandler::~ArmeArchiveHandler()
{
}

bool ArmeArchiveHandler::WriteInFile(const void* data, size_t size, size_t nb)
{
	ZzAssert(fMode == kArchiveHandlerMode_Writing);
	size_t nbWritten = fwrite(data,size,nb,fArchFile); //$$$DISKFULL
	return (nbWritten == nb);
}

ArmeArchiveHandler::Error ArmeArchiveHandler::CheckArchiveHeader()
{
	UInt32 generalArmeArchSig, generalArmeArchVer, thisFileSig;

	generalArmeArchSig = ReadUInt32();
	if(generalArmeArchSig != kGeneralArmeArchiveSig)
		return BadFileFormat;
	generalArmeArchVer = ReadUInt32();
	if(generalArmeArchVer != kGeneralArmeArchiveVer)
		return BadFileFormat;
	thisFileSig = ReadUInt32();
	if(thisFileSig != GetOurSig())
		return BadFileFormat;
	fThisFileVer = ReadUInt32();
	if(fThisFileVer > GetMaxVer() || fThisFileVer < GetMinVer())
		return BadFileFormat;
	return NoErr;
}

id ArmeArchiveHandler::ReadArchiveWithObjCHead(const char* archFilePath)
{
    return this->ReadArchiveWithObjCHead(archFilePath, NULL);
}

id ArmeArchiveHandler::ReadArchiveWithObjCHead(const char* archFilePath, id objectNotArchived)
{
	ZzAssert(fMode == kArchiveHandlerMode_Idle);
	fMode = kArchiveHandlerMode_Reading;
	AllocInitMappingTables();
	
	Error err =  _OpenReadArchiveFile(archFilePath);
	if(err != NoErr)
	{
		_CloseArchiveFile();
		fMode = kArchiveHandlerMode_Idle;
		return nil;
	}

	err = CheckArchiveHeader();
	if(err != NoErr)
	{
		_CloseArchiveFile();
		fMode = kArchiveHandlerMode_Idle;
		return nil;
	}
        if ( objectNotArchived ) SetObjCObjForInstanceID(0, objectNotArchived);
	id obj = ReadObjCObject();
	_CloseArchiveFile();
	fMode = kArchiveHandlerMode_Idle;
	return obj;
}

ArmeArchivable* ArmeArchiveHandler::ReadArchiveWithCPPHead(const char* archFilePath)
{
	ZzAssert(fMode == kArchiveHandlerMode_Idle);
	fMode = kArchiveHandlerMode_Reading;
	AllocInitMappingTables();

	Error err =  _OpenReadArchiveFile(archFilePath);
	if(err != NoErr)
	{
		_CloseArchiveFile();
		return NULL;
	}

	err = CheckArchiveHeader();
	if(err != NoErr)
	{
		_CloseArchiveFile();
		return NULL;
	}
	
	ArmeArchivable* obj = ReadCPPObject();
	_CloseArchiveFile();
	fMode = kArchiveHandlerMode_Idle;
	
	return obj;
}

bool ArmeArchiveHandler::ReadRawData(void* data, UInt32 bytes)
{
	// for debug
	UInt32 debugTag = ReadDebugTag();
	ZzAssert(debugTag == kDebugArchTag_RawData);
	bool ok = ReadFromFile(data,bytes,1);
	return ok;
}

UInt32 ArmeArchiveHandler::ReadUInt32()
{
	// for debug
	UInt32 debugTag = ReadDebugTag();
	ZzAssert(debugTag == kDebugArchTag_UInt32);
	UInt32 data;
	bool ok = ReadFromFile((void*)(&data),sizeof(UInt32));
	ZzAssert(ok);
	return data;
}

UInt32 ArmeArchiveHandler::ReadDebugTag()
{
	UInt32 data;
	bool ok = ReadFromFile((void*)(&data),sizeof(UInt32));
	ZzAssert(ok);
	return data;
}

SInt32 ArmeArchiveHandler::ReadSInt32()
{
	// for debug
	UInt32 debugTag = ReadDebugTag();
	ZzAssert(debugTag == kDebugArchTag_SInt32);
	SInt32 data;
	bool ok = ReadFromFile((void*)(&data),sizeof(SInt32));
	ZzAssert(ok);
	return data;
}

ArmeArchiveHandler::Error ArmeArchiveHandler::_OpenReadArchiveFile(const char* archFilePath)
{
	fArchFile = fopen(archFilePath,"r");
	if(!fArchFile)
	{
		verntl(archivingEngine,1,"warning : couldn't open the archive file");
		return FileOpeningError;
	}
	return NoErr;
}

bool ArmeArchiveHandler::ReadFromFile(void* data, size_t size, size_t nb)
{
	size_t nbRead = fread(data,size,nb,fArchFile);
	return (nbRead == nb);
}

bool ArmeArchiveHandler::_CloseArchiveFile()
{
	fclose(fArchFile);
	return true;
}

id ArmeArchiveHandler::CreateObjCObject(UInt32 classID)
{
	id obj = nil;
	switch(classID)
	{
#ifdef ARCH_CLASS
#error ("ARCH_CLASS already defined")
#endif //ARCH_CLASS
#define ARCH_CLASS(a) \
		case kArchClass_##a : \
			obj = [[[a alloc] initFromArchive:this] autorelease]; \
			break;
#include "ArmeArchiveObjCClasses.h"
#undef ARCH_CLASS
	}
	return obj;
}


ArmeArchivable* ArmeArchiveHandler::CreateCPPObject(UInt32 classID)
{
	ArmeArchivable* obj = NULL;
	switch(classID)
	{
#ifdef ARCH_CLASS
#error ("ARCH_CLASS already defined")
#endif //ARCH_CLASS
#define ARCH_CLASS(a) \
		case kArchClass_##a : \
			obj = new a(this); \
			break;
#include "ArmeArchiveCPPClasses.h"
#undef ARCH_CLASS
	}
	return obj;
}

void ArmeArchiveHandler::SetObjCObjForInstanceID(SInt32 instanceID, id obj)
{
	verntl(archivingEngine,5,"SetObjCObjForInstanceID("<<instanceID<<", "<<(void*)obj<<")");
	ZzAssert(instanceID < kMaxNbArchObjects);
	ZzAssert(fObjCInstancesPtrs[instanceID] == nil);
	fObjCInstancesPtrs[instanceID] = obj;
}

id ArmeArchiveHandler::GetObjCObjForInstanceID(SInt32 instanceID)
{
	ZzAssert(instanceID < kMaxNbArchObjects);
	ZzAssert(fObjCInstancesPtrs[instanceID] != nil);
	return fObjCInstancesPtrs[instanceID];
}

void ArmeArchiveHandler::SetCPPObjForInstanceID(SInt32 instanceID, ArmeArchivable* obj)
{
	verntl(archivingEngine,5,"SetCPPObjForInstanceID("<<instanceID<<", "<<obj<<")");
	ZzAssert(instanceID < kMaxNbArchObjects);
	ZzAssert(fCPPInstancesPtrs[instanceID] == NULL);
	fCPPInstancesPtrs[instanceID] = obj;
}

ArmeArchivable* ArmeArchiveHandler::GetCPPObjForInstanceID(SInt32 instanceID)
{
	ZzAssert(instanceID < kMaxNbArchObjects);
	ZzAssert(fCPPInstancesPtrs[instanceID] != NULL);
	return fCPPInstancesPtrs[instanceID];
}

const char*	ArmeArchiveHandler::RemoveLeadingDigits(const char* src)
{
	const char* dst = src;
	while(*dst >= '0' && *dst <= '9')
		dst++;
	return dst;
}

const char*	ArmeArchiveHandler::GetCPPObjectClassName(ArmeArchivable* obj)
{
	const std::type_info& typeInfo = typeid(*obj);
	const char* tmpClassName = typeInfo.name();
	vverntl(archivingEngine,5,tmpClassName);
	const char* className = RemoveLeadingDigits(tmpClassName);
	return className;
}

const char*	ArmeArchiveHandler::GetObjCObjectClassName(id obj)
{
	return [[obj archClassName] cString];
}

bool ArmeArchiveHandler::WriteDebugTag(const UInt32& data)
{
	size_t nbObjects = WriteInFile((void*)(&data),sizeof(UInt32),1);
	return(nbObjects == 1);
}
	
void ArmeArchiveHandler::SetCurrentlyBeingBuiltObjectInstanceID(SInt32 instanceID)
{
	fCurrentlyBeingBuiltObjectInstanceID = instanceID;
}

SInt32 ArmeArchiveHandler::GetCurrentlyBeingBuiltObjectInstanceID()
{
	return fCurrentlyBeingBuiltObjectInstanceID;
}

bool ArmeArchiveHandler::WriteArmeTime(const ArmeTime& data)
{
	WriteDebugTag(kDebugArchTag_ArmeTime);
	size_t nbObjects = WriteInFile((void*)(&data),sizeof(ArmeTime),1);
	return(nbObjects == 1);
}

ArmeTime ArmeArchiveHandler::ReadArmeTime()
{
	// for debug
	UInt32 debugTag = ReadDebugTag();
	ZzAssert(debugTag == kDebugArchTag_ArmeTime);
	ArmeTime data;
	bool ok = ReadFromFile((void*)(&data),sizeof(ArmeTime));
	ZzAssert(ok);
	return data;
}

bool ArmeArchiveHandler::WriteSInt64(const SInt64& data)
{
	WriteDebugTag(kDebugArchTag_SInt64);
	size_t nbObjects = WriteInFile((void*)(&data),sizeof(SInt64),1);
	return(nbObjects == 1);
}

SInt64 ArmeArchiveHandler::ReadSInt64()
{
	// for debug
	UInt32 debugTag = ReadDebugTag();
	ZzAssert(debugTag == kDebugArchTag_SInt64);
	SInt64 data;
	bool ok = ReadFromFile((void*)(&data),sizeof(SInt64));
	ZzAssert(ok);
	return data;
}

bool ArmeArchiveHandler::WriteBool(const bool& data)
{
	WriteDebugTag(kDebugArchTag_bool);
	size_t nbObjects = WriteInFile((void*)(&data),sizeof(bool),1);
	return(nbObjects == 1);
}

bool ArmeArchiveHandler::ReadBool()
{
	// for debug
	UInt32 debugTag = ReadDebugTag();
	ZzAssert(debugTag == kDebugArchTag_bool);
	bool data;
	bool ok = ReadFromFile((void*)(&data),sizeof(bool));
	ZzAssert(ok);
	return data;
}

bool ArmeArchiveHandler::WriteBOOL(const BOOL& data)
{
	WriteDebugTag(kDebugArchTag_BOOL);
	size_t nbObjects = WriteInFile((void*)(&data),sizeof(BOOL),1);
	return(nbObjects == 1);
}

BOOL ArmeArchiveHandler::ReadBOOL()
{
	// for debug
	UInt32 debugTag = ReadDebugTag();
	ZzAssert(debugTag == kDebugArchTag_BOOL);
	BOOL data;
	bool ok = ReadFromFile((void*)(&data),sizeof(BOOL));
	ZzAssert(ok);
	return data;
}

bool ArmeArchiveHandler::WriteFloat(const float& data)
{
	// for debug
	WriteDebugTag(kDebugArchTag_Float);
	size_t nbObjects = WriteInFile((void*)(&data),sizeof(float),1);
	return(nbObjects == 1);
}

float ArmeArchiveHandler::ReadFloat()
{
	// for debug
	UInt32 debugTag = ReadDebugTag();
	ZzAssert(debugTag == kDebugArchTag_Float);
	float data;
	bool ok = ReadFromFile((void*)(&data),sizeof(float));
	ZzAssert(ok);
	return data;
}

bool ArmeArchiveHandler::WriteNSRect(const NSRect& data)
{
    // for debug
    WriteDebugTag(kDebugArchTag_NSRect);
    size_t nbObjects = WriteInFile((void*)(&data),sizeof(NSRect),1);
    return(nbObjects == 1);
}

NSRect ArmeArchiveHandler::ReadNSRect()
{
    // for debug
    UInt32 debugTag = ReadDebugTag();
    ZzAssert(debugTag == kDebugArchTag_NSRect);
    NSRect data;
    bool ok = ReadFromFile((void*)(&data),sizeof(NSRect));
    ZzAssert(ok);
    return data;
}

bool ArmeArchiveHandler::WriteMIRect(const MIRect& data)
{
	// for debug
	WriteDebugTag(kDebugArchTag_MIRect);
	size_t nbObjects = WriteInFile((void*)(&data),sizeof(MIRect),1);
	return (nbObjects == 1);
}

MIRect ArmeArchiveHandler::ReadMIRect()
{
	// for debug
    UInt32 debugTag = ReadDebugTag();
    ZzAssert(debugTag == kDebugArchTag_MIRect);
    MIRect data;
    bool ok = ReadFromFile((void*)(&data),sizeof(MIRect));
    ZzAssert(ok);
    return data;
}
	
bool ArmeArchiveHandler::WriteMISize(const MISize& data)
{
	// for debug
	WriteDebugTag(kDebugArchTag_MISize);
	size_t nbObjects = WriteInFile((void*)(&data),sizeof(MISize),1);
	return (nbObjects == 1);
}

MISize ArmeArchiveHandler::ReadMISize()
{
	// for debug
    UInt32 debugTag = ReadDebugTag();
    ZzAssert(debugTag == kDebugArchTag_MISize);
    MISize data;
    bool ok = ReadFromFile((void*)(&data),sizeof(MISize));
    ZzAssert(ok);
    return data;
}
	
bool ArmeArchiveHandler::WriteMIPoint(const MIPoint& data)
{
	// for debug
	WriteDebugTag(kDebugArchTag_MIPoint);
	size_t nbObjects = WriteInFile((void*)(&data),sizeof(MIPoint),1);
	return (nbObjects == 1);
}

MIPoint ArmeArchiveHandler::ReadMIPoint()
{
	// for debug
    UInt32 debugTag = ReadDebugTag();
    ZzAssert(debugTag == kDebugArchTag_MIPoint);
    MIPoint data;
    bool ok = ReadFromFile((void*)(&data),sizeof(MIPoint));
    ZzAssert(ok);
    return data;
}

bool ArmeArchiveHandler::WriteNSPoint(const NSPoint& data)
{
    // for debug
    WriteDebugTag(kDebugArchTag_NSPoint);
    size_t nbObjects = WriteInFile((void*)(&data),sizeof(NSPoint),1);
    return(nbObjects == 1);
}

NSPoint ArmeArchiveHandler::ReadNSPoint()
{
    // for debug
    UInt32 debugTag = ReadDebugTag();
    ZzAssert(debugTag == kDebugArchTag_NSPoint);
    NSPoint data;
    bool ok = ReadFromFile((void*)(&data),sizeof(NSPoint));
    ZzAssert(ok);
    return data;
}

bool ArmeArchiveHandler::WriteQTAtomContainer(QTAtomContainer* atomContainer)
{
	QTLockContainer(*atomContainer);
    WriteQTAtom(atomContainer, kParentAtomIsContainer);
    WriteSInt32(0); //mark the end of atoms
	QTUnlockContainer(*atomContainer);

	return true;
}

bool ArmeArchiveHandler::WriteQTAtom(QTAtomContainer* atomContainer, QTAtom parentAtom)
{
 //   char tempBuffer[512];
	Ptr tempBuffer;
	long sizeWritten;
    
    QTAtomType atomType = 0;
    while (atomType = QTGetNextChildType(*atomContainer, parentAtom, atomType))
    {
		short atomCount = QTCountChildrenOfType(*atomContainer, parentAtom, atomType);
		for (short i=0; i < atomCount; ++i)
		{
			QTAtom atom = QTFindChildByIndex(*atomContainer, parentAtom, atomType, i+1, NULL);
			OSErr res;
//			res = QTCopyAtomDataToPtr(*atomContainer, atom, TRUE, 512, (void*)tempBuffer, &sizeWritten);
			res = QTGetAtomDataPtr(*atomContainer, atom, &sizeWritten, &tempBuffer);
			if (res == -2102) //the atom is not a leaf atom
			{
				WriteSInt32(-1);
				WriteSInt32(atomType);
				WriteSInt32(parentAtom);
				WriteQTAtom(atomContainer, atom);
				WriteSInt32(0);
			}
			else
			{
				WriteSInt32(atomType);
				WriteSInt32(parentAtom);
				WriteSInt32(sizeWritten);
				WriteRawData(tempBuffer, sizeWritten);
			}	
		}
    }

	return true;
}

bool ArmeArchiveHandler::ReadQTAtomContainer(QTAtomContainer* atomContainer)
{
//	QTLockContainer(*atomContainer);
    ReadQTAtom(atomContainer, kParentAtomIsContainer);
//	QTUnlockContainer(*atomContainer);

	return true;
}

bool ArmeArchiveHandler::ReadQTAtom(QTAtomContainer* atomContainer, QTAtom parentAtom)
{
    SInt32 atomType;
    SInt32 size;
	char* temp;
	QTAtom newAtom;
	OSErr res;		

    while (atomType = ReadSInt32())
    {
		if (atomType == -1) // this is not a leaf atom
		{
			atomType = ReadSInt32();
			ReadSInt32();  //read parent atom (unused)
			res = QTInsertChild(*atomContainer, parentAtom, atomType, 0, 0, 0, 0, &newAtom);
		    ThrowOnOSStatus(res);	
			ReadQTAtom(atomContainer, newAtom);
		}
		else
		{
			ReadSInt32();  //read parent atom (unused)
			size = ReadSInt32();
			temp = new char[size];
			ReadRawData(temp, size);
			res = QTInsertChild(*atomContainer, parentAtom, atomType, 0, 0, size, temp, NULL);
		    ThrowOnOSStatus(res);
			delete temp;
		}
    }
	
	return true;
}



/*
UInt32 ArmeArchiveHandler::ReadUInt32()
{
	// for debug
	UInt32 debugTag = ReadDebugTag();
	ZzAssert(debugTag == kDebugArchTag_UInt32);
	UInt32 data;
	bool ok = ReadFromFile((void*)(&data),sizeof(UInt32));
	ZzAssert(ok);
	return data;
}

bool ArmeArchiveHandler::WriteUInt32(const UInt32& data)
{
	// for debug
	WriteDebugTag(kDebugArchTag_UInt32);
	size_t nbObjects = WriteInFile((void*)(&data),sizeof(UInt32),1);
	return(nbObjects == 1);
}

*/

UInt32 ArmeArchiveHandler::GetFileVer() const
{
	return fThisFileVer;
}


