/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
 /*
	basic types definitions. beware of name clashes
	might provide config #defines for compatibility
*/

#define ArmeBasicTypes_h

#ifndef ArmeBasicTypes_h
#include "ArmeBasicTypes.h"
#endif //ArmeBasicTypes_h

#ifndef ZzTypes_h
#include "ZzTypes.h"
#endif //ZzTypes_h

#ifndef ZzVerbose_h
#include "ZzVerbose.h"
#endif //ZzVerbose_h

enum EArmeParamType
{
	kArmeParamTypeInt32 = 0,
	kArmeParamFloat,
	kArmeParamString
};

struct ArmeParamDesc
{
	EArmeParamType	fType;
	ZzChar			fName[32];
	UInt32			fArrayCount;
};


/*
	this macro generates struct wrappers around the C++ classes.

	example :

	DECLARE_PTR_WRAP_STRUCT(ArmeDoc)

	gets translated, in C++, to :

	typedef struct { IArmeDoc* _ptr; } ArmeDocRef

	and, in C, to :

	typedef struct { void* _ptr; } ArmeDocRef
*/

//#define SAMPLE_IS_INT
#define SAMPLE_IS_INT64

#ifdef SAMPLE_IS_INT
	typedef int sample_t;
	#ifdef SWIG
		%apply int { sample_t };
	#endif //SWIG
#elif defined SAMPLE_IS_INT64
	typedef int64 sample_t;
	#ifdef SWIG
		#error("better not to mess with int64 and SWIG")
	#endif //SWIG
#else //sample_t type
	#error("please define a type for sample_t")
#endif //sample_t type

#undef ARME_NO_WRAP_STRUCTS
	
#ifdef ARME_NO_WRAP_STRUCTS
	typedef sample_t ArmeTime;
	
	#ifdef SWIG
		%apply sample_t { ArmeTime };
	#endif //SWIG
	
	inline ArmeTime _ArmeTime(sample_t x)
	{
		return x;
	}
	
	inline ArmeTime	_ArmeTimeSub(ArmeTime a, ArmeTime b)
	{
		return a - b;
	}
#else //ARME_NO_WRAP_STRUCTS
	struct ArmeTime
	{
		sample_t	data;
		
		bool operator==(const ArmeTime& arg2) const {return data == arg2.data;};
		bool operator!=(const ArmeTime& arg2) const {return data != arg2.data;};
		bool operator<(const ArmeTime& arg2) const {return data < arg2.data;};
		bool operator>(const ArmeTime& arg2) const {return data > arg2.data;};
		bool operator<=(const ArmeTime& arg2) const {return data <= arg2.data;};
		bool operator>=(const ArmeTime& arg2) const {return data >= arg2.data;};
		ArmeTime& operator +=(const ArmeTime& arg) {data += arg.data; return *this;};
		ArmeTime& operator -=(const ArmeTime& arg) {data -= arg.data; return *this;};
		ArmeTime& operator =(const ArmeTime& arg) {data = arg.data; return *this;};

	};

	inline ArmeTime operator+(const ArmeTime& arg1, const ArmeTime& arg2) {ArmeTime res; res.data = arg1.data + arg2.data; return res;};
	inline ArmeTime operator-(const ArmeTime& arg1, const ArmeTime& arg2) {ArmeTime res; res.data = arg1.data - arg2.data; return res;};

	inline ArmeTime	_ArmeTime(sample_t x)
	{
		ArmeTime ret;
		ret.data = x;
		return ret;
	}
	
	inline ArmeTime	_ArmeTimeSub(ArmeTime a, ArmeTime b)
	{
		ArmeTime ret;
		ret.data = a.data - b.data;
		return ret;
	}
	
	#define GET_VAL(a) (a.data)
	#define ARME_TIME_ZERO	_ArmeTime(0)
	#define ARME_TIME_MAX 	_ArmeTime(9223372036854775807LL)
	#define ARME_TIME_MIN 	_ArmeTime(-9223372036854775806LL)
	#define ARME_TIME_UNDEF _ArmeTime(-9223372036854775807LL)
	sample_t		GetVal(ArmeTime time);
	
#endif //ARME_NO_WRAP_STRUCTS

inline ArmeTime ArmeTimeSub(const ArmeTime& time1, const ArmeTime& time2)
{
	return _ArmeTime(GET_VAL(time1)-GET_VAL(time2));
}

inline std::ostream& operator<<(std::ostream& s, const ArmeTime& armeTime)
{
	return s << (GET_VAL(armeTime));
}

#ifndef ArmeErrors_h
#include "ArmeErrors.h"
#endif //ArmeErrors_h

