/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#include "ArmeDataFileRef.h"

#ifndef ZzVerbose_h
//#include "ZzVerbose.h"
#endif //ZzVerbose_h

#ifndef ZzHelpers_h
#include "ZzHelpers.h"
#endif //ZzHelpers_h

#ifndef Zz_PathHelpers_h
#include "Zz_PathHelpers.h"
#endif //Zz_PathHelpers_h

#ifndef TArmeDoc_h
#include "TArmeDoc.h"
#endif //TArmeDoc_h

#ifndef ArmeArchiveHandler_h
#include "ArmeArchiveHandler.h"
#endif //ArmeArchiveHandler_h

#ifndef MovieHelper_h
#include "MovieHelper.h"
#endif //MovieHelper_h

#undef VER_DEF_TOPIC
#define VER_DEF_TOPIC dataFileRef

bool ArmeDataFileRef::WriteToArchive(ArmeArchiveHandler* archiver) const
{
	// don't save the nb of references (each of the object will do so)
	archiver->WriteUInt32(fFileIndex);
	
	//from version 0x0001000B
	if (fHighResSourceFile)
	{
		archiver->WriteUInt32(strlen(fHighResSourceFile)+1);
		archiver->WriteRawData(fHighResSourceFile, strlen(fHighResSourceFile)+1);
	}
	else
		archiver->WriteUInt32(0);
	archiver->WriteCPPObject(fHighResSourceDataFileRef);
	//from version 0x00010010
	archiver->WriteCPPObject(fDoc);
	
	//from version 0x00010030
	archiver->WriteBool(fHasBeenCreatedByMontage);
	if (fSourceFilePath)
	{
		archiver->WriteUInt32(strlen(fSourceFilePath)+1);
		archiver->WriteRawData(fSourceFilePath, strlen(fSourceFilePath)+1);
	}
	else
		archiver->WriteUInt32(0);
	//from version 0x00010033
	archiver->WriteBool(fIsReferenceToExternalFile);
	
	return true;
}

ArmeDataFileRef::ArmeDataFileRef(ArmeArchiveHandler* unarchiver) : ArmeArchivable(unarchiver)
{
	fFileIndex = unarchiver->ReadUInt32();
	fNbReferences = 0;

	UInt32 temp = unarchiver->ReadUInt32();
	if (temp)
	{
		fHighResSourceFile = new char[temp];
		unarchiver->ReadRawData(fHighResSourceFile, temp);
	}
	else
		fHighResSourceFile = NULL;
		
	fHighResSourceDataFileRef = dynamic_cast<ArmeDataFileRef*>(unarchiver->ReadCPPObject());
	if (fHighResSourceDataFileRef)
		fHighResSourceDataFileRef->AddRef();

	fDoc = dynamic_cast<TArmeDoc*>(unarchiver->ReadCPPObject());
	
	if (unarchiver->GetFileVer() >= 0x00010030)
	{
		fHasBeenCreatedByMontage = unarchiver->ReadBool();
		temp = unarchiver->ReadUInt32();
		if (temp)
		{
			fSourceFilePath = new char[temp];
			unarchiver->ReadRawData(fSourceFilePath, temp);
		}
		else
		{
			fSourceFilePath = NULL;
		}
		
		if (unarchiver->GetFileVer() >= 0x00010033)
		{
			fIsReferenceToExternalFile = unarchiver->ReadBool();
		}
		else
		{
			if (temp)
				fIsReferenceToExternalFile = true;
			else
				fIsReferenceToExternalFile = false;
		}
	}
	else
	{
		fHasBeenCreatedByMontage = true;  // in previous version, all the files were copied into project's folder and thus generated by HyperEngine-AV
		fSourceFilePath = NULL;
		fIsReferenceToExternalFile = false;
	}
}


void ArmeDataFileRef::Release()
{
	ZzAssert(fNbReferences > 0);
	
	fNbReferences--;
}

ArmeDataFileRef::ArmeDataFileRef(UInt32 fileIndex, TArmeDoc* doc, const char* sourceFile) : fFileIndex(fileIndex), fDoc(doc)
{
	Init();

	if (sourceFile)
	{
		SetSourceFilePath(sourceFile);
	}

	fHasBeenCreatedByMontage = true;
}

void ArmeDataFileRef::Init()
{
	fNbReferences = 0;
	fHighResSourceFile = NULL;
	fHighResSourceDataFileRef = NULL;
	fSourceFilePath = NULL;
	fHasBeenCreatedByMontage = false;
	fIsReferenceToExternalFile = false;
}

void ArmeDataFileRef::PreDestructor()
{
	ZzAssert(fNbReferences == 0);
	
	// if the file is not referenced anymore, delete it 
	if (fNbReferences == 0)
	{
		char* tempStringForDestructor = new char[kMaxPathLength];
		GetFilePath(tempStringForDestructor);

		//anyway, remove the references in the master.mov
		fDoc->fMasterMovie->RemoveReferenceTo(this);
		//vernl(4, "Removing references on " << tempStringForDestructor << " in WorkingMaster.mov");
		//done somewhere else

		if (fHasBeenCreatedByMontage)
		{
			//delete file
			verntl(dataFileRef, 4, "Deleting Data File " << tempStringForDestructor << " (not referenced anymore)");
			remove(tempStringForDestructor);
            FILE* fp = fopen(tempStringForDestructor, "a+b");
            fwrite(tempStringForDestructor, strlen(tempStringForDestructor), 1, fp);
            fclose(fp);
		}

		delete [] tempStringForDestructor;
	}
	
	delete fHighResSourceFile;
	if (fHighResSourceDataFileRef)
		fHighResSourceDataFileRef->Release();
	
	delete fSourceFilePath;
}

ArmeDataFileRef::~ArmeDataFileRef()
{
}

void ArmeDataFileRef::SetSourceFilePath(const char* filePath)
{
	fSourceFilePath = new char[strlen(filePath) + 1];
	strcpy(fSourceFilePath, filePath);
}

const char* ArmeDataFileRef::GetSourceFilePath()
{
	return fSourceFilePath;
}

void ArmeDataFileRef::SetReferenceToExternalFile()
{
	fIsReferenceToExternalFile = true;
	fHasBeenCreatedByMontage = false;
}

bool ArmeDataFileRef::IsReferenceToExternalFile() const
{
	if (fIsReferenceToExternalFile)
		return true;
	else
		return false;
}
void ArmeDataFileRef::UpdateForSelfContainedProject()
{
//	delete fFilePath;
//	fFilePath = NULL;
	fIsReferenceToExternalFile = false;
	fHasBeenCreatedByMontage = true;
}
	
void ArmeDataFileRef::AddRef()
{
	fNbReferences++;
};

UInt32 ArmeDataFileRef::GetRefCount() const 
{
	return fNbReferences;
};

void ArmeDataFileRef::GetFilePath(char* destString)
{
	if (IsReferenceToExternalFile())
	{
		strcpy(destString, fSourceFilePath);
	}
	else
	{
		GetFilePathInSelfContainedProject(destString);
	}
}

void ArmeDataFileRef::GetFilePathInSelfContainedProject(char* destString)
{
	const char* projectPath = fDoc->GetProjectFolderPath();
	const char* fileNameBase = GetFileNameBase();
	
	ZzAppendFileNameToPath(destString, kMaxPathLength, projectPath, fileNameBase);
	ZzAddIndexToFileName(destString, kMaxPathLength, fFileIndex);
}

void ArmeVideoDataFileRef::GetFilePathForIndex(const TArmeDoc* doc, char* destString, int index)
{
	const char* projectPath = doc->GetProjectFolderPath();
	
	ZzAppendFileNameToPath(destString, kMaxPathLength, projectPath, "MovieFile.mov");
	ZzAddIndexToFileName(destString, kMaxPathLength, index);
}


void ArmeDataFileRef::SetHighResSourceFile(const char* source)
{
	delete fHighResSourceFile;
	fHighResSourceFile = new char[strlen(source)+1];
	strcpy(fHighResSourceFile, source);
}

void ArmeDataFileRef::SetHighResSourceFile(ArmeDataFileRef*	dataFileRef)
{
	if (fHighResSourceDataFileRef)
		fHighResSourceDataFileRef->Release();
	fHighResSourceDataFileRef = dataFileRef;
	fHighResSourceDataFileRef->AddRef();
}

const char* ArmeDataFileRef::GetHighResSourceFile()
{
	static char temp[kMaxPathLength];

	if (fHighResSourceDataFileRef)
	{
		fHighResSourceDataFileRef->GetFilePath(temp);
		return temp;
	}
	else
		return fHighResSourceFile;
}

const char* ArmeVideoTakeDataFileRef::GetFileNameBase()
{
	return "Take.mov";
}


void ArmeVideoTakeDataFileRef::GetFilePath(char* destString)
{
	ZzAppendFileNameToPath(destString, kMaxPathLength, fDoc->GetProjectFolderPath(), GetFileNameBase());
	ZzAddIndexToFileName(destString, kMaxPathLength, fFileIndex);
}

ArmeVideoTakeDataFileRef::ArmeVideoTakeDataFileRef(UInt32 fileIndex, TArmeDoc* doc) : ArmeVideoDataFileRef(fileIndex, doc) {};

ArmeVideoTakeDataFileRef::ArmeVideoTakeDataFileRef(ArmeArchiveHandler* unarchiver) : ArmeVideoDataFileRef(unarchiver) {};

const char* ArmeVideoDataFileRef::GetFileNameBase()
{
	return "MovieFile.mov";
};

/*
const char* ArmeAudioDataFileRef::GetFileNameBase()
{
	return "AudioFile.MTGAudio";
};
*/

ArmeVideoDataFileRef::ArmeVideoDataFileRef(UInt32 fileIndex, TArmeDoc* doc, const char* sourceFile) : ArmeDataFileRef(fileIndex, doc, sourceFile) 
{
};
ArmeVideoDataFileRef::ArmeVideoDataFileRef(ArmeArchiveHandler* unarchiver) : ArmeDataFileRef(unarchiver) 
{
};

bool ArmeDataFileRef::IsVideoRef() const 
{
	return false;
};

bool ArmeVideoDataFileRef::IsVideoRef() const 
{
	return true;
};

/*
ArmeAudioDataFileRef::ArmeAudioDataFileRef(UInt32 fileIndex, TArmeDoc* doc) : ArmeDataFileRef(fileIndex, doc)
{
};
ArmeAudioDataFileRef::ArmeAudioDataFileRef(ArmeArchiveHandler* unarchiver) : ArmeDataFileRef(unarchiver) 
{
};
*/
