/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#include <Carbon/Carbon.h>

#ifndef ArmeErrors_h
#include "ArmeErrors.h"
#endif //ArmeErrors_h

#ifndef ZzException_h
#include "ZzException.h"
#endif //ZzException_h

#include <assert.h>

#undef ARME_ERROR_DEF
const char* g_aszErrorStrings[] =
{
#define ARME_ERROR_DEF(a) #a,
#include "ArmeErrorsInternal.h"
"kArmeErrLastString"
};

const char* ArmeGetStringFromError(ArmeErr err)
{
	if(err.type == kArmeQuickTimeError)
		return ArmeGetQTError((OSStatus)err.data);
	else
		return g_aszErrorStrings[err.type];
}

#ifdef SHOW_DEBUG_INFO_IN_ERROR_MSG
#define THROW_ZZEXCEPTION(ExceptionClass) throw new ExceptionClass(sourceFile, lineNumber);
#define THROW_ZZEXCEPTION_1ARG(ExceptionClass,arg) throw new ExceptionClass(arg,sourceFile, lineNumber);
#else // SHOW_DEBUG_INFO_IN_ERROR_MSG
#define THROW_ZZEXCEPTION(ExceptionClass) throw new ExceptionClass();
#define THROW_ZZEXCEPTION_1ARG(ExceptionClass,arg) throw new ExceptionClass(arg);
#endif // SHOW_DEBUG_INFO_IN_ERROR_MSG

bool ThrowOnArmeErrFunction(ArmeErr err, const char* sourceFile, int lineNumber)
{
	if(err.type == kArmeOK)
		return false;
	else
	{
		THROW_ZZEXCEPTION_1ARG(ZzExceptionArmeErr,err);
	}
}

//#define RANDOM_ERRORS
bool ThrowOnOSStatusFunction(OSStatus st, const char* sourceFile, int lineNumber)
{
#ifdef RANDOM_ERRORS
	if (float(rand())/float(RAND_MAX) < 0.01f)
	{
		THROW_ZZEXCEPTION(ZzExceptionTest);
	}
#endif

	if (st == noErr)
		return false;

	switch(st)
	{
		case -34 :
			THROW_ZZEXCEPTION(ZzExceptionDiskIsFull);
			break;
		case -33 :
		case -35 :
		case -37 :
		case -42 :
		case -44 :
		case -45 :
		case -46 :
		case -47 :
		case -48 :
		case -49 :
			THROW_ZZEXCEPTION(ZzExceptionFile);
			break;
/*		case -54 :
			THROW_ZZEXCEPTION_1ARG(ZzGenericException,"Can not edit project file.  Can not open file for writing");
			break;*/
		case -43 :
			THROW_ZZEXCEPTION(ZzExceptionFileNotFound);
			break;
		case -2019 :
			THROW_ZZEXCEPTION(ZzExceptionCancel);
			break;
		case -8969 :
			THROW_ZZEXCEPTION_1ARG(ZzGenericException,"Could not export the movie because the compressor found that the image data may be corrupted");
			break;
		default :
			char *temp = new char[256];
			sprintf(temp, "Unknown Exception : %d", (int)st);
			THROW_ZZEXCEPTION_1ARG(ZzGenericException, temp);
	};
	
	return true;
}
//note : look in MacErrors.h for all the error codes

const char* ArmeGetQTError(short err)
{
	if( err== -33 )
		return "File direcotry full					";
	if( err== -34 )
		return "Disk is full						";
	if( err== -35 )
		return "No such Volume						";
	if( err== -36 )
		return "I/O error							";
	if( err== -37 )
		return "Bad filename						";
	if( err== -38 )
		return "File Not Open						";
	if( err== -39 )
		return "Logical end-of-file reached			";
	if( err== -40 )
		return "Position mark before start of file	";
	if( err== -42 )
		return "Too many files open					";
	if( err== -43 )
		return "File not found						";
	if( err== -44 )
		return "HW Volume lock						";
	if( err== -45 )
		return "File is locked						";
	if( err== -46 )
		return "SW Volume lock						";
	if( err== -47 )
		return "File busy, directory not empty, or working directory control block open";
	if( err== -48 )
		return "Duplicate filename and version		";
	if( err== -49 )
		return "File already open for writing		";
	if( err== -50 )
		return "Invalid Parameter					";
	if( err== -51 )
		return "Bad reference number				";
	if( err== -54 )
		return "Attempt to open locked file for writing";
	if( err== -108 )
		return "Not Enough Memory					";
	if( err== -117 )
		return "Memory Block is locked";
	if( err== -120 )
		return "Directory not found or incomplete pathname";
	if( err== -192 )
		return "Resource Not Found					";
	if( err== -193 )
		return "Resource File Not Found				";
	if( err== -194 )
		return "AddResource procedure failed		";
	if( err== -196 )
		return "RemoveResource procedure failed		";
	if( err== -198 )
		return "Attribute inconsistent with oper.	";
	if( err== -199 )
		return "Map inconsistent with operation		";

	if( err == -2000 )
		return "couldNotResolveDataRef              ";
	if( err == -2001 )
		return "badImageDescription                 ";
	if( err == -2002 )
		return "badPublicMovieAtom                  ";
	if( err == -2003 )
		return "cantFindHandler                     ";
	if( err == -2004 )
		return "cantOpenHandler                     ";
	if( err == -2005 )
		return "badComponentType                    ";
	if( err == -2006 )
		return "noMediaHandler                      ";
	if( err == -2007 )
		return "noDataHandler                       ";
	if( err == -2008 )
		return "invalidMedia                        ";
	if( err == -2009 )
		return "invalidTrack                        ";
	if( err == -2010 )
		return "invalidMovie                        ";
	if( err == -2011 )
		return "invalidSampleTable                  ";
	if( err == -2012 )
		return "invalidDataRef                      ";
	if( err == -2013 )
		return "invalidHandler                      ";
	if( err == -2014 )
		return "invalidDuration                     ";
	if( err == -2015 )
		return "invalidTime                         ";
	if( err == -2016 )
		return "cantPutPublicMovieAtom              ";
	if( err == -2017 )
		return "badEditList                         ";
	if( err == -2018 )
		return "mediaTypesDontMatch                 ";
	if( err == -2019 )
		return "progressProcAborted                 ";
	if( err == -2020 )
		return "movieToolboxUninitialized           ";
	if( err == -2020 )
		return "noRecordOfApp                       ";
	if( err == -2021 )
		return "wfFileNotFound                      ";
	if( err == -2022 )
		return "cantCreateSingleForkFile            ";
	if( err == -2023 )
		return "invalidEditState                    ";
	if( err == -2024 )
		return "nonMatchingEditState                ";
	if( err == -2025 )
		return "staleEditState                      ";
	if( err == -2026 )
		return "userDataItemNotFound                ";
	if( err == -2027 )
		return "maxSizeToGrowTooSmall               ";
	if( err == -2028 )
		return "badTrackIndex                       ";
	if( err == -2029 )
		return "trackIDNotFound                     ";
	if( err == -2030 )
		return "trackNotInMovie                     ";
	if( err == -2031 )
		return "timeNotInTrack                      ";
	if( err == -2032 )
		return "timeNotInMedia                      ";
	if( err == -2033 )
		return "badEditIndex                        ";
	if( err == -2034 )
		return "internalQuickTimeError              ";
	if( err == -2035 )
		return "cantEnableTrack                     ";
	if( err == -2036 )
		return "invalidRect                         ";
	if( err == -2037 )
		return "invalidSampleNum                    ";
	if( err == -2038 )
		return "invalidChunkNum                     ";
	if( err == -2039 )
		return "invalidSampleDescIndex              ";
	if( err == -2040 )
		return "invalidChunkCache                   ";
	if( err == -2041 )
		return "invalidSampleDescription            ";
	if( err == -2042 )
		return "dataNotOpenForRead                  ";
	if( err == -2043 )
		return "dataNotOpenForWrite                 ";
	if( err == -2044 )
		return "dataAlreadyOpenForWrite             ";
	if( err == -2045 )
		return "dataAlreadyClosed                   ";
	if( err == -2046 )
		return "endOfDataReached                    ";
	if( err == -2047 )
		return "dataNoDataRef                       ";
	if( err == -2048 )
		return "noMovieFound                        ";
	if( err == -2049 )
		return "invalidDataRefContainer             ";
	if( err == -2050 )
		return "badDataRefIndex                     ";
	if( err == -2051 )
		return "noDefaultDataRef                    ";
	if( err == -2052 )
		return "couldNotUseAnExistingSample         ";
	if( err == -2053 )
		return "featureUnsupported                  ";
	if( err == -2057 )
		return "unsupportedAuxiliaryImportData      ";
	if( err == -2058 )
		return "auxiliaryExportDataUnavailable      ";
	if( err == -2059 )
		return "samplesAlreadyInMediaErr            ";
	if( err == -2060 )
		return "noSourceTreeFoundErr                ";
	if( err == -2061 )
		return "sourceNotFoundErr                   ";
	if( err == -2062 )
		return "movieTextNotFoundErr                ";
	if( err == -2063 )
		return "missingRequiredParameterErr         ";
	if( err == -2064 )
		return "invalidSpriteWorldPropertyErr       ";
	if( err == -2065 )
		return "invalidSpritePropertyErr            ";
	if( err == -2066 )
		return "gWorldsNotSameDepthAndSizeErr       ";
	if( err == -2067 )
		return "invalidSpriteIndexErr               ";
	if( err == -2068 )
		return "invalidImageIndexErr                ";
	if( err == -2069 )
		return "invalidSpriteIDErr                  ";
	if( err == -2070 )
		return "internalComponentErr                ";
	if( err == -2071 )
		return "notImplementedMusicOSErr            ";
	if( err == -2072 )
		return "cantSendToSynthesizerOSErr          ";
	if( err == -2073 )
		return "cantReceiveFromSynthesizerOSErr     ";
	if( err == -2074 )
		return "illegalVoiceAllocationOSErr         ";
	if( err == -2075 )
		return "illegalPartOSErr                    ";
	if( err == -2076 )
		return "illegalChannelOSErr                 ";
	if( err == -2077 )
		return "illegalKnobOSErr                    ";
	if( err == -2078 )
		return "illegalKnobValueOSErr               ";
	if( err == -2079 )
		return "illegalInstrumentOSErr              ";
	if( err == -2080 )
		return "illegalControllerOSErr              ";
	if( err == -2081 )
		return "midiManagerAbsentOSErr              ";
	if( err == -2082 )
		return "synthesizerNotRespondingOSErr       ";
	if( err == -2083 )
		return "synthesizerOSErr                    ";
	if( err == -2084 )
		return "illegalNoteChannelOSErr             ";
	if( err == -2085 )
		return "noteChannelNotAllocatedOSErr        ";
	if( err == -2086 )
		return "tunePlayerFullOSErr                 ";
	if( err == -2087 )
		return "tuneParseOSErr                      ";
	if( err == -2089 )
		return "noExportProcAvailableErr            ";
	if( err == -2090 )
		return "videoOutputInUseErr                 ";
	if( err == -2091 )
		return "componentDllLoadErr                 ";
	if( err == -2092 )
		return "componentDllEntryNotFoundErr        ";
	if( err == -2093 )
		return "qtmlDllLoadErr                      ";
	if( err == -2094 )
		return "qtmlDllEntryNotFoundErr             ";
	if( err == -2095 )
		return "qtmlUninitialized                   ";
	if( err == -2096 )
		return "unsupportedOSErr                    ";
	if( err == -2097 )
		return "unsupportedProcessorErr             ";
	if( err == -2054 )
		return "noVideoTrackInMovieErr              ";
	if( err == -2055 )
		return "noSoundTrackInMovieErr              ";
	if( err == -2056 )
		return "soundSupportNotAvailableErr         ";
	if( err == -2101 )
		return "cannotFindAtomErr                   ";
	if( err == -2102 )
		return "notLeafAtomErr                      ";
	if( err == -2103 )
		return "atomsNotOfSameTypeErr               ";
	if( err == -2104 )
		return "atomIndexInvalidErr                 ";
	if( err == -2105 )
		return "duplicateAtomTypeAndIDErr           ";
	if( err == -2106 )
		return "invalidAtomErr                      ";
	if( err == -2107 )
		return "invalidAtomContainerErr             ";
	if( err == -2108 )
		return "invalidAtomTypeErr                  ";
	if( err == -2109 )
		return "cannotBeLeafAtomErr                 ";
	if( err == -2110 )
		return "pathTooLongErr                      ";
	if( err == -2111 )
		return "emptyPathErr                        ";
	if( err == -2112 )
		return "noPathMappingErr                    ";
	if( err == -2113 )
		return "pathNotVerifiedErr                  ";
	if( err == -2114 )
		return "unknownFormatErr                    ";
	if( err == -2115 )
		return "wackBadFileErr                      ";
	if( err == -2116 )
		return "wackForkNotFoundErr                 ";
	if( err == -2117 )
		return "wackBadMetaDataErr                  ";
	if( err == -2118 )
		return "qfcbNotFoundErr                     ";
	if( err == -2119 )
		return "qfcbNotCreatedErr                   ";
	if( err == -2120 )
		return "AAPNotCreatedErr                    ";
	if( err == -2121 )
		return "AAPNotFoundErr                      ";
	if( err == -2122 )
		return "ASDBadHeaderErr                     ";
	if( err == -2123 )
		return "ASDBadForkErr                       ";
	if( err == -2124 )
		return "ASDEntryNotFoundErr                 ";
	if( err == -2125 )
		return "fileOffsetTooBigErr                 ";
	if( err == -2126 )
		return "notAllowedToSaveMovieErr            ";
	if( err == -2127 )
		return "qtNetworkAlreadyAllocatedErr        ";
	if( err == -2129 )
		return "urlDataHHTTPProtocolErr             ";
	if( err == -2130 )
		return "urlDataHHTTPNoNetDriverErr          ";
	if( err == -2131 )
		return "urlDataHHTTPURLErr                  ";
	if( err == -2132 )
		return "urlDataHHTTPRedirectErr             ";
	if( err == -2133 )
		return "urlDataHFTPProtocolErr              ";
	if( err == -2134 )
		return "urlDataHFTPShutdownErr              ";
	if( err == -2135 )
		return "urlDataHFTPBadUserErr               ";
	if( err == -2136 )
		return "urlDataHFTPBadPasswordErr           ";
	if( err == -2137 )
		return "urlDataHFTPServerErr                ";
	if( err == -2138 )
		return "urlDataHFTPDataConnectionErr        ";
	if( err == -2139 )
		return "urlDataHFTPNoDirectoryErr           ";
	if( err == -2140 )
		return "urlDataHFTPQuotaErr                 ";
	if( err == -2141 )
		return "urlDataHFTPPermissionsErr           ";
	if( err == -2142 )
		return "urlDataHFTPFilenameErr              ";
	if( err == -2143 )
		return "urlDataHFTPNoNetDriverErr           ";
	if( err == -2144 )
		return "urlDataHFTPBadNameListErr           ";
	if( err == -2145 )
		return "urlDataHFTPNeedPasswordErr          ";
	if( err == -2146 )
		return "urlDataHFTPNoPasswordErr            ";
	if( err == -2147 )
		return "urlDataHFTPServerDisconnectedErr    ";
	if( err == -2148 )
		return "urlDataHFTPURLErr                   ";
	if( err == -2149 )
		return "notEnoughDataErr                    ";
	if( err == -2157 )
		return "qtActionNotHandledErr               ";
	if( err == -2201 )
		return "digiUnimpErr                        ";
	if( err == -2202 )
		return "qtParamErr                          ";
	if( err == -2203 )
		return "matrixErr                           ";
	if( err == -2204 )
		return "notExactMatrixErr                   ";
	if( err == -2205 )
		return "noMoreKeyColorsErr                  ";
	if( err == -2206 )
		return "notExactSizeErr                     ";
	if( err == -2207 )
		return "badDepthErr                         ";
	if( err == -2208 )
		return "noDMAErr                            ";
	if( err == -2209 )
		return "badCallOrderErr                     ";
	if( err== -5000 )
		return "Access denied						";
	if( err== -5025 )
		return "object type err						";
	if( err == -8960 )
		return "codecErr                         	";
	if( err == -8961 )
		return "noCodecErr                          ";
	if( err == -8962 )
		return "codecUnimpErr                       ";
	if( err == -8963 )
		return "codecSizeErr                        ";
	if( err == -8964 )
		return "codecScreenBufErr                   ";
	if( err == -8965 )
		return "codecImageBufErr                    ";
	if( err == -8966 )
		return "codecSpoolErr                       ";
	if( err == -8967 )
		return "codecAbortErr                       ";
	if( err == -8968 )
		return "codecWouldOffscreenErr              ";
	if( err == -8969 )
		return "codecBadDataErr                     ";
	if( err == -8970 )
		return "codecDataVersErr                    ";
	if( err == -8971 )
		return "codecExtensionNotFoundErr           ";
	if( err == -8971 )
		return "scTypeNotFoundErr                   ";
	if( err == -8972 )
		return "codecConditionErr                   ";
	if( err == -8973 )
		return "codecOpenErr                        ";
	if( err == -8974 )
		return "codecCantWhenErr                    ";
	if( err == -8975 )
		return "codecCantQueueErr                   ";
	if( err == -8976 )
		return "codecNothingToBlitErr               ";
	if( err == -8977 )
		return "codecNoMemoryPleaseWaitErr          ";
	if( err == -8978 )
		return "codecDisabledErr                    ";
	if( err == -8979 )
		return "codecNeedToFlushChainErr            ";
	if( err == -8980 )
		return "lockPortBitsBadSurfaceErr           ";
	if( err == -8981 )
		return "lockPortBitsWindowMovedErr          ";
	if( err == -8982 )
		return "lockPortBitsWindowResizedErr        ";
	if( err == -8983 )
		return "lockPortBitsWindowClippedErr        ";
	if( err == -8984 )
		return "lockPortBitsBadPortErr              ";
	if( err == -8985 )
		return "lockPortBitsSurfaceLostErr       ";
	if( err == -8986 )
		return "codecParameterDialogConfirm         ";
	if( err == -8987 )
		return "codecNeedAccessKeyErr               ";
	if( err == -8988 )
		return "codecOffscreenFailedErr             ";
	if( err == -8989 )
		return "codecDroppedFrameErr                ";
	if( err == -8990 )
		return "directXObjectAlreadyExists          ";
	if( err == -8991 )
		return "lockPortBitsWrongGDeviceErr         ";
	if( err == -8992 )
		return "codecOffscreenFailedPleaseRetryErr  ";
	if( err == -30540 )
		return "notAQTVRMovieErr                    ";
	if( err == -30541 )
		return "constraintReachedErr                ";
	if( err == -30542 )
		return "callNotSupportedByNodeErr           ";
	if( err == -30543 )
		return "selectorNotSupportedByNodeErr       ";
	if( err == -30544 )
		return "invalidNodeIDErr                    ";
	if( err == -30545 )
		return "invalidViewStateErr                 ";
	if( err == -30546 )
		return "timeNotInViewErr                    ";
	if( err == -30547 )
		return "propertyNotSupportedByNodeErr       ";
	if( err == -30548 )
		return "settingNotSupportedByNodeErr        ";
	if( err == -30549 )
		return "limitReachedErr                     ";
	if( err == -30550 )
		return "invalidNodeFormatErr                ";
	if( err == -30551 )
		return "invalidHotSpotIDErr                 ";
	if( err == -30552 )
		return "noMemoryNodeFailedInitialize        ";
	if( err == -30553 )
		return "streamingNodeNotReadyErr            ";
	if( err == -30554 )
		return "qtvrLibraryLoadErr                  ";
	if( err == -30555  )
		return "qtvrUninitialized                   ";
	else
		return "unknown QT error                   ";
}

const char* ZzExceptionArmeErr::ErrorMessage()
{
	return ArmeGetStringFromError(fArmeErr);
}


		







