/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#ifndef ArmeObjCArchivable_h
#include "ArmeObjCArchivable.h"
#endif //ArmeObjCArchivable_h

#ifndef ArmeArchiveHandler_h
#include "ArmeArchiveHandler.h"
#endif //ArmeArchiveHandler_h

@implementation NSObject (ArmeObjCArchivable)

-(id)initFromArchive:(ArmeArchiveHandler*)archiver
{
	verntl(archivingEngine,05,"NSObject (ArmeObjCArchivable) :: initFromArchive");
	self = [self init];
	archiver->SetObjCObjForInstanceID(archiver->GetCurrentlyBeingBuiltObjectInstanceID(),self);
	return self;
}

- (BOOL)writeToArchive:(ArmeArchiveHandler*)archiver
{
	verntl(archivingEngine,5,"NSObject (ArmeObjCArchivable) :: writeToArchive");
	return YES;
}

-(NSString*)archClassName
{
	return NSStringFromClass([self class]);
}

@end

@implementation NSArray (ArmeObjCArchivable)

-(id)initFromArchive:(ArmeArchiveHandler*)archiver
{
	verntl(archivingEngine,05,"NSArray (ArmeObjCArchivable) :: initFromArchive SHOULD NEVER BE CALLED");
	ZzAssert(0);
	return nil;
}

- (BOOL)writeToArchive:(ArmeArchiveHandler*)archiver
{
	verntl(archivingEngine,5,"NSArray (ArmeObjCArchivable) :: writeToArchive");
	SInt32 count = [self count];
	archiver->WriteSInt32(count);
	for(SInt32 i = 0; i < count; i++)
	{	
		id obj = [self objectAtIndex:i];
		archiver->WriteObjCObject(obj);
	}
	return YES;
}

-(NSString*)archClassName
{
	return @"NSArray";
}

@end

@implementation NSMutableArray (ArmeObjCArchivable)

-(id)initFromArchive:(ArmeArchiveHandler*)archiver
{
	verntl(archivingEngine,05,"NSMutableArray (ArmeObjCArchivable) :: initFromArchive");
	self = [self init];
	archiver->SetObjCObjForInstanceID(archiver->GetCurrentlyBeingBuiltObjectInstanceID(),self);
	if(self)
	{
		SInt32 count = archiver->ReadSInt32();
		verntl(archivingEngine,06,"    count = " << count);
		for(SInt32 i = 0; i < count; i++)
		{
			id newObj = archiver->ReadObjCObject();
			[self addObject:newObj];
		}
	}
	return self;
}

-(NSString*)archClassName
{
	return @"NSMutableArray";
}

@end

@implementation NSString (ArmeObjCArchivable)

-(id)initFromArchive:(ArmeArchiveHandler*)archiver
{
	verntl(archivingEngine,05,"NSString (ArmeObjCArchivable) :: initFromArchive");
	SInt32 countChars = archiver->ReadSInt32();
	verntl(archivingEngine,06,"    countChars = " << countChars);
	char* buf = (char*)malloc(sizeof(char)*countChars + 1);
	archiver->ReadRawData((void*)buf,countChars);
	buf[countChars] = 0;
	self = [self initWithUTF8String:buf];
	archiver->SetObjCObjForInstanceID(archiver->GetCurrentlyBeingBuiltObjectInstanceID(),self);
	free(buf);
	return self;
}

-(BOOL)writeToArchive:(ArmeArchiveHandler*)archiver
{
	verntl(archivingEngine,05,"NSString (ArmeObjCArchivable) :: writeToArchive");
	const char* utf8string = [self UTF8String];
	SInt32 length = strlen(utf8string);
	verntl(archivingEngine,06,"    length = " << length);
	archiver->WriteSInt32(length);
	archiver->WriteRawData((void*)(utf8string),length);
	return YES;
}

-(NSString*)archClassName
{
	return @"NSString";
}

@end

@implementation NSColor (ArmeObjCArchivable)

-(id)initFromArchive:(ArmeArchiveHandler*)archiver
{
	verntl(archivingEngine,05,"NSColor (ArmeObjCArchivable) :: initFromArchive");
	SInt32 instanceID = archiver->GetCurrentlyBeingBuiltObjectInstanceID();
	id object = [NSUnarchiver unarchiveObjectWithData:archiver->ReadObjCObject()];
	archiver->SetObjCObjForInstanceID( instanceID, object );
	[self release];
	return [object retain];
}

-(BOOL)writeToArchive:(ArmeArchiveHandler*)archiver
{
	verntl(archivingEngine,05,"NSColor (ArmeObjCArchivable) :: writeToArchive");
	archiver->WriteObjCObject( [NSArchiver archivedDataWithRootObject:self] );
	return YES;
}

-(NSString*)archClassName
{
	return @"NSColor";
}

@end

@implementation NSData (ArmeObjCArchivable)

-(id)initFromArchive:(ArmeArchiveHandler*)archiver
{
	verntl(archivingEngine,05,"NSData (ArmeObjCArchivable) :: initFromArchive");
	SInt32 length = archiver->ReadSInt32();
	verntl(archivingEngine,06,"    length = " << length);
	void* buf = malloc( length );
	archiver->ReadRawData( buf, length );
	self = [self initWithBytesNoCopy:buf length:length];
	archiver->SetObjCObjForInstanceID( archiver->GetCurrentlyBeingBuiltObjectInstanceID(), self );
	return self;
}

-(BOOL)writeToArchive:(ArmeArchiveHandler*)archiver
{
	verntl(archivingEngine,05,"NSData (ArmeObjCArchivable) :: writeToArchive");
	SInt32 length = [self length];
	verntl(archivingEngine,06,"    length = " << length);
	archiver->WriteSInt32(length);
	archiver->WriteRawData([self bytes],length);
	return YES;
}

-(NSString*)archClassName
{
	return @"NSData";
}

@end

@implementation NSAttributedString (ArmeObjCArchivable )

-(id)initFromArchive:(ArmeArchiveHandler*)archiver
{
	verntl(archivingEngine,05,"NSAttributed (ArmeObjCArchivable) :: initFromArchive");
	SInt32 instanceID = archiver->GetCurrentlyBeingBuiltObjectInstanceID();
	id object = [NSUnarchiver unarchiveObjectWithData:archiver->ReadObjCObject()];
	archiver->SetObjCObjForInstanceID( instanceID, object );
	[self release];
	return [object retain];
}

-(BOOL)writeToArchive:(ArmeArchiveHandler*)archiver
{
	verntl(archivingEngine,05,"NSAttributedString (ArmeObjCArchivable) :: writeToArchive");
	archiver->WriteObjCObject( [NSArchiver archivedDataWithRootObject:self] );
	return YES;
}

-(NSString*)archClassName
{
	return @"NSAttributedString";
}

@end

@implementation NSDictionary (ArmeObjCArchivable )

-(id)initFromArchive:(ArmeArchiveHandler*)archiver
{
	verntl(archivingEngine,05,"NSDictionary (ArmeObjCArchivable) :: initFromArchive");
/*	SInt32 length = archiver->ReadSInt32();
	verntl(archivingEngine,06,"    length = " << length);
	void* buf = malloc( length );
	archiver->ReadRawData( buf, length );
	self = [self initWithBytesNoCopy:buf length:length];
	archiver->SetObjCObjForInstanceID( archiver->GetCurrentlyBeingBuiltObjectInstanceID(), self );*/
	ZzAssert(0);
	return self;
}

-(BOOL)writeToArchive:(ArmeArchiveHandler*)archiver
{
	ZzAssert(0);
	return NO;
#if 0
	verntl(archivingEngine,05,"NSDictionary (ArmeObjCArchivable) :: writeToArchive");
	SInt32 length = [self count];
	verntl(archivingEngine,06,"    length = " << length);
	archiver->WriteSInt32(length);
	
	archiver->WriteRawData([self bytes],length);
#endif
	return YES;
}

-(NSString*)archClassName
{
	return @"NSDictionary";
}

@end
