/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#define MovieHelper_h

#ifndef MovieHelper_h
#include "MovieHelper.h"
#endif //MovieHelper_h

#include <Quicktime/Quicktime.h>

#ifndef ArmeErrors_h
#include "ArmeErrors.h"
#endif //ArmeErrors_h

#ifndef TArmeDoc_h
#include "TArmeDoc.h"
#endif //TArmeDoc_h

#ifndef ArmeDataFileRef_h
#include "ArmeDataFileRef.h"
#endif //ArmeDataFileRef_h

#ifndef WorkingCodec_h
#include "WorkingCodec.h"
#endif //WorkingCodec_h

#ifndef ArmeChunk_h
#include "ArmeChunk.h"
#endif //ArmeChunk_h

class MIProgressBarDlgManager;

#define kSourceOneName					FOUR_CHAR_CODE('srcA')
#define kSourceTwoName					FOUR_CHAR_CODE('srcB')
#define kSourceThreeName	            FOUR_CHAR_CODE('srcC')


class MovieHelper
{
	QTAtomContainer		fEffectsList;

public :
	QTAtomContainer		GetEffectsList() const;

	MovieHelper();
	~MovieHelper();
};


/**************************************************************************************
Time management functions
**************************************************************************************/
//TimeValue 	TimeValueFromSec(const Movie movie, float sec);
//float 		SecondsFromTimeValue(const Movie movie, TimeValue time);
TimeValue 	TimeValueFromArmeTime2(const Movie movie, const TArmeDoc* doc, const ArmeTime time);
ArmeTime 	ArmeTimeFromTimeValue2(const Movie movie, const TArmeDoc* doc, const TimeValue time); 

float 		GetMovieDurationSec(const Movie movie);
void 		SetMovieArmeTime(Movie movie, const TArmeDoc* doc, const ArmeTime time);
void		PrerollMovieArmeTime(
				const Movie movie, 
				const TArmeDoc* doc, 
				const ArmeTime time);
				
ArmeTime 	GetMovieArmeTime(const Movie movie, const TArmeDoc* doc);
float 		GetMovieTimeSec(const Movie movie);
void 		AdjustMovieArmeTime(Movie movie, const TArmeDoc* doc, const ArmeTime time);


/**************************************************************************************
File management functions
**************************************************************************************/

FSSpec Path2FSspec(const char* szFilePath, bool deleteIfFileExists = false);
FSSpec FSSpecFromNonExistingFilePath(char* szFilePath, OSType creator, OSType type, OSErr& err);
Movie LoadMovie(const char* szFilePath, short* movieResFile, short* movieResId, ArmeErr* err, FSSpec* fsSpec = NULL, bool forEditing = false);
Movie LoadMovie(const FSSpec& movieFile, int trackIndex, short* movieResFile, ArmeErr* err, bool forEditing = false);
void SaveMovie(const Movie movie, char* szFilePath);
Movie CreatePreviewMovie(const Movie movie, char* szFilePath, int width=64, int height=48);
OSStatus FSpPathFromLocation(const  FSSpec *spec, char *fullPath);
// Benjamin's suggested temp file naming code, with a 37 character uuid.
// If anyone knows how to make an FSSpec with > 31 characters in the name, then we could use this for Carbon API calls
char* CreateTempMoviePath(char* projectPath);
// "ViaFSRef" documents that path's target must exist or this will fail
OSStatus Path2FSSpecViaFSRef(FSSpec& outSpec, const char *inPath, Boolean inIsDirectory);
// A guaranteed-unique-FSSpec maker, expecting template of "description <%ld>.extension" form
OSStatus MakeUniqueFSSpec(FSSpec& outSpec, const char* inProjectPath, const char* inNameTemplate);
int CountClippableTracksInFile(const char* path);
void SavePlaybackHintsToMovie(FSSpecPtr movieFile, bool highQuality, bool singleField);
void ReplaceMediaReferences(Movie movie, const char* oldFileName, const char* newFileName);


/**************************************************************************************
Track management functions
**************************************************************************************/

Track GetFirstVideoTrack(const Movie movie);

OSErr CopyMovieSegment(Movie destMovie, const Movie srcMovie, TimeValue destStartTime, TimeValue srcStartTime, TimeValue srcDuration);
OSErr CopyTrackSegment(Movie destMovie, Track destTrack, const Movie srcMovie, const Track srcTrack, TimeValue destStartTime, TimeValue srcStartTime, TimeValue srcDuration); 
Track GetCopyOfTrack(Movie movie, const Track srcTrack, int trackWidth, int trackHeight);
Track CreateNewTrackInMovie(Movie destMovie, TimeScale scale, int trackWidth, int trackHeight, OSType trackType = VideoMediaType);
OSErr CopyChunkListIntoTrack(Track destTrack, TArmeChunkList<TVideoChunk>* chunkList);
OSErr CopyEffectDescIntoTrack(TArmeDoc* doc, Track destTrack, QTAtomContainer effectDesc, ArmeTime startTime, ArmeTime endTime);
Track InsertMovieSegmentAsNewTrackInMovie(Movie destMovie, const Movie srcMovie, TimeValue destStartTime, TimeValue srcStartTime, TimeValue duration, int trackWidth, int trackHeight); 
Track InsertTrackSegmentAsNewTrackInMovie(Movie destMovie, const Track srcTrack, TimeValue destStartTime, TimeValue srcStartTime, TimeValue duration, int trackWidth, int trackHeight);
Track InsertTrackSegmentAsNewTrackInMovie(Movie destMovie, const Movie srcMovie, int trackIndex, TimeValue destStartTime, TimeValue srcStartTime, TimeValue duration, int trackWidth, int trackHeight);
// ioMovie is deleted and replaced with the new movie in the returned ArmeDataFileRef*
ArmeDataFileRef* RenderMovieIntoSingleTrackMovie(ArmeErr &err, Movie movie, TArmeDoc* inDoc, MIProgressBarDlgManager* prog = NULL, ArmeTime startTime = ARME_TIME_UNDEF, ArmeTime endTime = ARME_TIME_UNDEF, bool reverse = false, bool useCustomAlgo = false, bool highRes = false, bool stillClipToScale = false);
//int GetNbTracksInCommon(ZzArray<TVideoTrack*>& listOfTrackRepi, ZzArray<TVideoTrack*>& listOfTrackRepj, int startSearchIndexInListI, int& firstIndexInListJ);

/**************************************************************************************
Effect management functions
**************************************************************************************/

Track CreateTrackForEffect(Movie movieSource, Track trackA, Track trackB, QTAtomContainer& effectDesc);
Track CreateEffectOnMovie(Movie movieSource, Track trackA, Track trackB);
bool ConfigureEffect(QTAtomContainer& effectDesc, int nbTrackSource, PicHandle posterSrc1 = NULL, PicHandle posterSrc2 = NULL);
QTAtomContainer GetQTEffectsList(int nbSource);
void GetCopyOfTransitionEffect(QTAtomContainer& effectDest, QTAtomContainer& effectSrc, float from = 0.0f, float to = 0.0f);
void GetFromAndToParamOfTransition(QTAtomContainer& effectDesc, float& from, float& to);
bool ExploreQTAtom(QTAtomContainer* atomContainer, QTAtom parentAtom);
bool ExploreQTAtomContainer(QTAtomContainer* atomContainer);
void GetCopyOfEffectDesc(QTAtomContainer& effectDest, QTAtomContainer& effectSrc, float from, float to);
void ExtractEffectDesc(QTAtomContainer& effectDesc, QTAtom parentAtom, float from, float to);
void GetEffectName(QTAtomContainer& effectDesc, char* dest);

void	AddNewSplits(TArmeWSObj* fromParent, ArmeTime startTime = ARME_TIME_ZERO, ArmeTime endTime = ARME_TIME_MAX);
void	RemoveUselessSplits(TArmeWSObj* fromParent, ArmeTime startTime = ARME_TIME_ZERO, ArmeTime endTime = ARME_TIME_MAX);
void 	RemoveUselessXFadeEffect(TArmeWSObj* fromParent, ArmeTime startTime = ARME_TIME_ZERO, ArmeTime endTime = ARME_TIME_MAX);
void	CreateXFadeEffect(TArmeWSObj* fromParent, bool bConstantMixTransition, ArmeTime startTime, ArmeTime endTime, ZzArray<TVideoXFadeEffect*>* transitionsOuttaHierarchy, ZzArray<TVideoXFadeEffect*>* transitionsCreated, ZzArray<TVideoXFadeEffect*>* transitionsToDelete, ZzArray<TVideoXFadeEffect*>* transitionsToUpdate);

TVideoXFadeEffect* FindTransitionBetween(ZzArray<TVideoXFadeEffect*>* transitionsOuttaHierarchy, const TVideoObject* clip1, const TVideoObject* clip2);

void				ConvertMovie(const char* pathSrc, char* pathDest, WorkingCodecType type);
ComponentInstance	GetCodecInstance(WorkingCodecType type);
ComponentInstance	GetCodecInstance();
ComponentInstance	GetAiffCodecInstance(int sampleRate);
pascal OSErr QTMovieProgressProc (Movie theMovie, short message, short whatOperation, Fixed percentDone, long refcon);


/**************************************************************************************
Possibly generally useful utilities
**************************************************************************************/
Rect ScaleRectToRect(const Rect& inSourceRect, const Rect& inDestRect, bool inCenter);


/************************************************************************************************
INLINE INLINE INLINE INLINE INLINE INLINE INLINE INLINE INLINE INLINE INLINE INLINE INLINE INLINE
************************************************************************************************/
/*inline TimeValue TimeValueFromSec(const Movie movie, float sec) 
{
    return (TimeValue)(GetMovieTimeScale(movie)*sec+0.5f);
};

inline float SecondsFromTimeValue(const Movie movie, TimeValue time)
{
	return (float)time/(float)GetMovieTimeScale(movie);
};

inline TimeValue TimeValueFromArmeTime(const Movie movie, const TArmeDoc* doc, const ArmeTime time)
{
	return TimeValueFromSec(movie, doc->ArmeTimeToSeconds(time));
}

inline ArmeTime ArmeTimeFromTimeValue(const Movie movie, const TArmeDoc* doc, const TimeValue time)
{
	return doc->SecondsToArmeTime(SecondsFromTimeValue(movie, time));
}

inline float GetMovieDurationSec(const Movie movie) 
{
	return float(GetMovieDuration(movie))/float(GetMovieTimeScale(movie));
};
*/