/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#include "TArmeContainer.h"

#ifndef TVideoObject_h
#include "TVideoObject.h"
#endif //TVideoObject_h

#ifndef MovieHelper_h
#include "MovieHelper.h"
#endif //MovieHelper_h

#ifndef ArmeTimeRange_h
#include "ArmeTimeRange.h"
#endif //ArmeTimeRange_h

#ifndef TVideoEffect_h
#include "TVideoEffect.h"
#endif //TVideoEffect_h

#ifndef ZzArray_h
#include "ZzArray.h"
#endif //ZzArray_h

#ifndef VARPIPluginClass_h
#include "VARPIPluginClass.h"
#endif VARPIPluginClass_h

TArmeContainer::TArmeContainer(const char* path, TArmeDoc* doc) : 
	TVideoObject(path, doc)
{


}

TArmeContainer::~TArmeContainer()
{

}


TArmeContainer::TArmeContainer(const char* name) : TVideoObject(name)
{

}

TArmeContainer::TArmeContainer(ArmeArchiveHandler* handler) :
	TVideoObject(handler)
{

}

bool TArmeContainer::WriteToArchive(ArmeArchiveHandler* handler) const
{
	return TVideoObject::WriteToArchive(handler);
}

/*void TArmeContainer::_NotifyParentOuttaHierarchy()
{
	TArmeWSObj::_NotifyParentOuttaHierarchy();
}

void TArmeContainer::_NotifyParentBackInHierarchy()
{
	TArmeWSObj::_NotifyParentBackInHierarchy();
}*/

ArmeTime TArmeContainer::InsertSlice(ArmeTime insertTime, const TArmeWSObj* obj, MIProgressBarDlgManager* progressMngr)
{
	// the insert slice only insert a slice of an empty container

	ArmeTime length = obj->GetDuration();
	_SetLength(GetDuration() + length);
	
	// so we have to move all the obects starting after the insertion point
	for (int i=0; i < GetChildrenCount(); ++i)
	{
		if (GetChild(i)->GetAbsoluteStartTime() > insertTime)
		{
			TArmeWSObjPos pos = GetChild(i)->GetPos();
			pos.fStartTime += length;
			GetChild(i)->_SetPos(pos);
		}
	}

	fHasBeenEdited = true;

	return insertTime + length;
}	


TArmeWSObj* TArmeContainer::GetSliceCopy(ArmeTime startTime, ArmeTime endTime, const char* tempDirectoryName, MIProgressBarDlgManager* progressMngr) const
{
	ArmeTime copyStartTime;  //the relative pos of the copied object
	ArmeTime absoluteStartTime = GetAbsoluteStartTime();
	ArmeTime absoluteEndxTime = GetAbsoluteEndxTime();

	if ((startTime == ARME_TIME_UNDEF) || (startTime == ARME_TIME_MIN))
		copyStartTime = GetPos().fStartTime;
	else if (startTime < absoluteStartTime)
	{
		if (startTime < fPos.fParent->GetAbsoluteStartTime())
			copyStartTime = absoluteStartTime - fPos.fParent->GetAbsoluteStartTime();
		else
			copyStartTime = absoluteStartTime - startTime;
	}
	else
		copyStartTime = ARME_TIME_ZERO;
	
	if ((startTime == ARME_TIME_UNDEF) || (startTime < absoluteStartTime))
		startTime = absoluteStartTime;
	
	if ((endTime == ARME_TIME_UNDEF) || (endTime > absoluteEndxTime))
		endTime = absoluteEndxTime;

	if (startTime == endTime)
		return NULL;

	TArmeContainer* copy = new TArmeContainer(GetName());
	copy->SetMuted(GetMuted());
	copy->fPos.fStartTime = copyStartTime;
	copy->fPos.fParent = NULL;
	copy->fLength = endTime - startTime;
	copy->fCachedAbsoluteStartTime = startTime;
	copy->SetVAORouting(GetVAORouting());

	// copy plugin params if any.
	// make sure the offline process is null. The offline process should always be 
	// cleared after an offline operation such as render or merge
	ZzAssert(fOfflinePlugin == NULL);
	
	copy->CopyAudioPluginFrom(this);
	return copy;
}

bool TArmeContainer::DeleteSlice(ArmeTime startTime, ArmeTime endTime, MIProgressBarDlgManager* progressMngr)
{
	bool bDeleteWholeObject = true;
	ArmeTime absoluteStartTime = GetAbsoluteStartTime();
	ArmeTime absoluteEndxTime = GetAbsoluteEndxTime();

	ArmeTime cutStartTime = startTime;

	if ((startTime == ARME_TIME_UNDEF) || (startTime == ARME_TIME_MIN))
		startTime = absoluteStartTime;
	if ((endTime == ARME_TIME_UNDEF) || (endTime == ARME_TIME_MAX))
		endTime = absoluteEndxTime;

	if (cutStartTime <= absoluteStartTime)
		cutStartTime = absoluteStartTime;
	else
		bDeleteWholeObject = false;
	if (endTime >= absoluteEndxTime)
		endTime = absoluteEndxTime;
	else
		bDeleteWholeObject = false;
		
	if (bDeleteWholeObject)
	{
		return true;  // the object will be deleted outside this routine
	}
	else
	{
		// if we have deleted 'something' before the video clip, the start of the clip must be moved to the start of what has been deleted
		if (cutStartTime != startTime)
		{
			ZzAssert(fPos.fParent);

			if (endTime > absoluteStartTime)
			{
				if (startTime < fPos.fParent->GetAbsoluteStartTime())
					fPos.fStartTime = ARME_TIME_ZERO;
				else
				{
					ArmeTime newAbsoluteStartTime = startTime; //startTime - fPos.fParent->GetAbsoluteStartTime();
					fPos.fStartTime = newAbsoluteStartTime - fPos.fParent->GetAbsoluteStartTime();
				}
			}
			else
			{
				if (startTime < fPos.fParent->GetAbsoluteStartTime())
				{
					if (endTime > fPos.fParent->GetAbsoluteStartTime())
						fPos.fStartTime -= (endTime - fPos.fParent->GetAbsoluteStartTime());
				}
				else			
					fPos.fStartTime -= (endTime - startTime); //startTime - fPos.fParent->GetAbsoluteStartTime();
			}

			fPos.fParent->_NotifyChildPosHasChanged();
			InvalidateCachedTimeInfo();
		}
		if (endTime > absoluteStartTime)
		{
			_SetLength(GetDuration() - (endTime - cutStartTime));
		}

		return false;
	}
}

ArmeTime TArmeContainer::GetDuration() const
{
	return TArmeWSObj::GetDuration();
}

bool TArmeContainer::IsVideoObject() const
{
	// a container is a video object if it has at least one child with video
	return (GetVideoChildrenCount() != 0);
}

void TArmeContainer::SetMuted(bool bMuted)
{
	TArmeWSObj::SetMuted(bMuted);

	for (int i=0; i < GetChildrenCount(); ++i)
	{
		GetChild(i)->SetMuted(bMuted);
	}
}

bool TArmeContainer::ContainsVideoAtTime(ArmeTime t) const
{
	for (int i=0; i < GetChildrenCount(); ++i)
	{
		TVideoObject* obj = dynamic_cast<TVideoObject*>(GetChild(i));
		if (obj && (obj->GetType() != kVideoXFadeEffect) && obj->ContainsVideoAtTime(t))
			return true;
	}
	return false;
}

bool TArmeContainer::ContainsVideoDuring(ArmeTime startTime, ArmeTime endTime, bool bConsiderAlsoMutedObjects, bool bConsiderAlsoTransparentTextClips) const
{
	for (int i=0; i < GetChildrenCount(); ++i)
	{
		TArmeWSObj* obj = GetChild(i);
		if (obj && obj->ContainsVideoDuring(startTime, endTime, bConsiderAlsoMutedObjects, bConsiderAlsoTransparentTextClips))
			return true;
	}
	return false;
}

bool TArmeContainer::ContainsAudioDuring(ArmeTime startTime, ArmeTime endTime, bool bConsiderAlsoMutedObjects) const
{
	for (int i=0; i < GetChildrenCount(); ++i)
	{
		TArmeWSObj* obj = GetChild(i);
		if (obj && obj->ContainsAudioDuring(startTime, endTime, bConsiderAlsoMutedObjects))
			return true;
	}
	return false;
}

ArmeTimeRange* TArmeContainer::GetVideoCoverage() const
{
	ArmeTimeRange* range = NULL;

	for (int i=0; i < GetChildrenCount(); ++i)
	{
		TArmeWSObj* obj = GetChild(i);
		if (obj)
		{
			if (!range)
				range = obj->GetVideoCoverage();
			else
				range->Add(obj->GetVideoCoverage());
		}
	}
	
	return range;
}

ZzArray<TArmeWSObj*>* TArmeContainer::RenderVideo() const
{
	//create a temporary movie in which we copy all the tracks that needs to be rendered
	ZzAssert(0);
	
	return NULL;
	
}
	
MEAudioClip* TArmeContainer::RenderAudio() const
{
	ZzAssert(0);

	return NULL;
}

void TArmeContainer::SetPos(const TArmeWSObjPos& pos)
{
	//Modify the hierarchy, move the tracks in the master movie and delete useless VideoEffectTrack (not the XFade)
	TArmeWSObjPos copy = pos;
	if (ContainsVideo(true, true))
		fDoc->RoundArmeTimeToNearestVideoFrame(copy.fStartTime);
	_SetPos(copy);

//	UpdateAffectedHierarchy();
}

void TArmeContainer::PosWillChange()
{
	if (ContainsVideo(true, true))
		fMasterMovie->RemoveSubMovie(this);
}
void TArmeContainer::PosHasChanged()
{
	if (ContainsVideo(true, true))
		fMasterMovie->AddSubMovie(this);
}

void TArmeContainer::Move()
{
	ArmeTime newStartTime = GetAbsoluteStartTime();
	if (fOldStartTime != newStartTime)
	{
		for (int i=0; i < GetChildrenCount(); ++i)
		{
			TVideoObject* obj = dynamic_cast<TVideoObject*>(GetChild(i));
			if (obj && (obj->GetType() != kVideoXFadeEffect)) 
			{
				obj->Move();
			}
		}
		
		//keep in memory the old position and length of the object
		fOldStartTime = GetAbsoluteStartTime();
	}
}

void TArmeContainer::Dump(int indentLevel) const
{
#ifndef NO_VERBOSE_AT_ALL
    static char* whiteSpace = "           ";

    // Display all the tracks
    verntl(armeWSObjsBenjamin,5,&(whiteSpace[10-indentLevel]) << "Container " << this);
    verntl(armeWSObjsBenjamin,5,&(whiteSpace[10-indentLevel]) << "Clips in the container :");
    TVideoObject* clip;
    for (int i=0; i < GetChildrenCount(); ++i)
    {
		if (GetChild(i)->IsVideoObject())
		{
			clip = (TVideoObject*)GetChild(i);
			clip->Dump(indentLevel+1);
		}
	}
#endif //NO_VERBOSE_AT_ALL
}


SInt32 TArmeContainer::GetType() const
{
	if(fRealTimePlugin)
		return kAudioProcessor;
	else
		return kContainer;
}

