/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#define TArmeContainer_h

#ifndef TArmeContainer_h
#include "TArmeContainer.h"
#endif //TArmeContainer_h

#ifndef ArmeBasicTypes_h
#include "ArmeBasicTypes.h"
#endif //ArmeBasicTypes_h

#ifndef ZzArray_h
#include "ZzArray.h"
#endif //ZzArray_h

#ifndef ArmeArchiveHandler_h
#include "ArmeArchiveHandler.h"
#endif //ArmeArchiveHandler_h

#ifndef TVideoObject_h
#include "TVideoObject.h"
#endif //TVideoObject_h

class MEAudioClip;
class ArmeTimeRange;

class TArmeContainer : public TVideoObject
{
public :
	/*! @function GetDuration Get the total duration of all the tracks of the video object (in ArmeTime)*/
	virtual ArmeTime		GetDuration() const;

	/* @function IsVideoObject test if this object contains video data*/
	virtual bool			IsVideoObject() const;

	/*! @function GetType RTTI information for all TArmeWSObj derivatives. See TArmeWSObj */
	virtual SInt32			GetType() const;

	virtual void			Move();

	/*! @function ContainsAudioDuring*/
	virtual bool			ContainsAudioDuring(ArmeTime startTime, ArmeTime endTime, bool bConsiderAlsoMutedObjects) const;

	/*! @function ContainsVideoDuring*/
	virtual bool			ContainsVideoDuring(ArmeTime startTime, ArmeTime endTime, bool bConsiderAlsoMutedObjects=false, bool bConsiderAlsoTransparentTextClips = false) const;
	virtual ArmeTimeRange* 	GetVideoCoverage() const;
	
	/*! @function ContainsVideoAtTime*/
	bool					ContainsVideoAtTime(ArmeTime t) const;

	/*! @function RenderVideo Creates a single video clip containing the rendering of all the movie objects in the container*/
	ZzArray<TArmeWSObj*>*	RenderVideo() const;
	
	/*! @function RenderAudio Creates a single audio clip containing the rendering of all the audio objects in the container*/
	MEAudioClip*			RenderAudio() const;

	/*! @function RemoveUselessSplits remove all the splits that are not in the list*/
//	virtual void 			RemoveUselessSplitsInClip(ArmeTime startTime, ArmeTime endTime, ZzArray<ArmeTime> &listOfUsefulSplits);

	virtual void 			SetMuted(bool bMuted);

	virtual void 			Dump(int indentLevel) const;

public :
	/*! @function Constructor This constructor creates a stand-alone video object (mainly used to be put in the clipboard)*/
	TArmeContainer(const char* name);
    TArmeContainer(const char* path, TArmeDoc* doc);
    virtual			~TArmeContainer();
	
	// arch stuff
	TArmeContainer(ArmeArchiveHandler* handler);
	virtual bool WriteToArchive(ArmeArchiveHandler* handler) const;
	
	virtual TArmeWSObj* GetSliceCopy(ArmeTime startTime, ArmeTime endTime, const char* tempDirectoryName, MIProgressBarDlgManager* progressMngr) const;	
	virtual ArmeTime	InsertSlice(ArmeTime insertTime, const TArmeWSObj* obj, MIProgressBarDlgManager* progressMngr);
	virtual bool 		DeleteSlice(ArmeTime startTime, ArmeTime endTime, MIProgressBarDlgManager* progressMngr);

	virtual void		SetPos(const TArmeWSObjPos& pos);
protected :
/*	virtual void		_NotifyParentOuttaHierarchy();
	virtual void		_NotifyParentBackInHierarchy();*/
	
	virtual void			PosWillChange();
	virtual void			PosHasChanged();
};
