/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#ifndef TArmeDoc_h
#include "TArmeDoc.h"
#endif //TArmeDoc_h

#ifndef TArmeMediaEngine_h
#include "TArmeMediaEngine.h"
#endif //TArmeMediaEngine_h

#ifndef TArmeTLC_h
#include "TArmeTLC.h"
#endif //TArmeTLC_h

#ifndef ArmeNotifications_h
#include "ArmeNotifications.h"
#endif //ArmeNotifications_h

#ifndef TArmeMasterMovie_h
#include "TArmeMasterMovie.h"
#endif //TArmeMasterMovie_h

#ifndef MovieHelper_h
#include "MovieHelper.h"
#endif //MovieHelper_h

#ifndef ZzHelpers_h
#include "ZzHelpers.h"
#endif //ZzHelpers_h

#ifndef ArmeDataFileRef_h
#include "ArmeDataFileRef.h"
#endif //ArmeDataFileRef_h

#ifndef MEMixingInfo_h
#include "MEMixingInfo.h"
#endif //MEMixingInfo_h

#ifndef VArmeMediaFileHandler_h
#include "VArmeMediaFileHandler.h"
#endif //VArmeMediaFileHandler_h

#ifndef VARPIPluginClass_h
#include "VARPIPluginClass.h"
#endif VARPIPluginClass_h

#ifndef MECoreAudioPlayer_h
#include "MECoreAudioPlayer.h"
#endif MECoreAudioPlayer_h

#ifndef TVideoEffect_h
#include "TVideoEffect.h"
#endif //TVideoEffect_h

#ifndef MIProgressBarDlgManager_h
#include "MIProgressBarDlgManager.h"
#endif //MIProgressBarDlgManager_h

#ifndef ZzException_h
#include "ZzException.h"
#endif //ZzException_h

#undef VER_DEF_TOPIC
#define VER_DEF_TOPIC TArmeDoc

TArmeDoc::TArmeDoc(const char* path, bool bNewFile, WorkingCodecType workingCodec)
{
	TArmeMediaEngine::CheckForCoreAudioPlayer();
//	LockWrite(); // so that RT thread does not ask for info
	fWasDocDirtifiedDuringAudioChunkManagerUnarchiving = false; // useless in this ctor
	fDoesUpdateMovieArmeTime = true; // for slice sel tracker in video

	fVideoSoloObject = NULL;
	fAudioSoloObject = NULL;
	strcpy(fCurrentProjectFolderPath,path);

	fMEAudioChunkFileNextIndex = 0;
	fAudioChunkNextIndex = 0;
	fVideoChunkNextIndex = 0;
	fStillImageIndex = 0;
	
	fViewerIsVisible = false;
/*	fVideoFileRefCount.SetSize(20);
	fVideoFileNames.SetSize(20);*/

	//create the master movie
	fTopContainer = new TArmeTLC("TopLevelContainer",this);
	fMasterMovie = new TArmeMasterMovie(this, workingCodec);
	fMixingInfo = new MEMixingInfo(this);
	fPlayer = new MEDocPlayer(this);

//	fOuttaHierarchyTransitionsPool = new ZzArray<TVideoXFadeEffect*>(32);
	fOldTransitionsDesc = new ZzArray<TVideoXFadeEffectDesc*>(32);
	fNewlyCreatedTransitions = new ZzArray<TVideoXFadeEffect*>(8);
	fTransitionsToRemove = new ZzArray<TVideoXFadeEffect*>(8);
	
	fDataFileRefArray = new ZzArray<ArmeDataFileRef*>(32);

	TArmeMediaEngine::AddDoc(this);
	
	fAudioChunkFileManager = new MEAudioChunkFileManager(this);			
//	UnlockWrite();
}

TArmeDoc::~TArmeDoc()
{
	vernl(6,"TArmeDoc::~TArmeDoc()");
	TArmeMediaEngine::RemoveDoc(this);
	//$$$TODO insert a notification for inter doc clipboard copying
	delete fTopContainer; // [MIObject dealloc] does NOT delete the arme object.
	fTopContainer = NULL;
	// release the master movie stuffs
	delete fMasterMovie;
	fMasterMovie = NULL;
	delete fPlayer;
	fPlayer = NULL; // to track a nasty(?) bug
	delete fMixingInfo;
	fMixingInfo = NULL;
	
//	delete fOuttaHierarchyTransitionsPool;

	delete fOldTransitionsDesc;
	delete fNewlyCreatedTransitions;
	delete fTransitionsToRemove;
	delete fDataFileRefArray;
}

/*
TArmeWSObj* TArmeDoc::CreateWSObjFromFile( 
    const ZzString* 			cstrPath, 
    const TArmeWSObjPos&		pos,
	MIProgressBarDlgManager*	progressMngr,
	ArmeErr*					err)
{
	for( int32 i = 0; i < TArmeMediaEngine::GetMediaFileHandlerCount(); i++ )
	{
		VArmeMediaFileHandler* pMediaFileHandler = TArmeMediaEngine::GetMediaFileHandler(i);
		if( pMediaFileHandler->CanHandleFile(cstrPath) )
		{	
			return pMediaFileHandler->CreateWSObjFromFile(this, cstrPath, pos, progressMngr, err);
		}
	}
	return NULL;
}
*/

ArmeErr TArmeDoc::CreateWSObjsFromFile( 
	ZzArchPtrArray*				theObjects,
	const ZzString*				cstrPath,
	ArmeDataFileRef*			fileRef,
    const TArmeWSObjPos&		pos,
	MIProgressBarDlgManager*	progressMngr)
{
	char temp[1024];
	ZzString* theRealPath;
	ArmeErr err;
	bool bHasBeenHandledByExtension = false;

	if (fileRef) // if fileRef is not NULL, then it takes precedence over the ZzStringPath
	{
		fileRef->GetFilePath(temp);
		theRealPath = new ZzString(temp);
		//theRealPath = theRealPathCopy;
	}
	else
	{
		theRealPath = new ZzString(*cstrPath);
	}

	for( int32 i = 0; i < TArmeMediaEngine::GetMediaFileHandlerCount(); i++ )
	{
		VArmeMediaFileHandler* pMediaFileHandler = TArmeMediaEngine::GetMediaFileHandler(i);
		if( pMediaFileHandler->CanHandleFileByLookingAtExtension(theRealPath) )
		{	
			err =  pMediaFileHandler->CreateWSObjsFromFile(this, theObjects, theRealPath, fileRef, pos, progressMngr);
			
			//as soon as a handler has imported the file, we stop searching
			if (err.type == kArmeOK)
			{
				bHasBeenHandledByExtension = true;
				break;
			}
		}
		else
		{
			err.type = kArmeErrNoSuitableMediaHandlerFound;
		}
	}

	if (!bHasBeenHandledByExtension && (err.type == kArmeErrNoSuitableMediaHandlerFound))  // if the import failed for any other reason, don't even retry here
	{
		for( int32 i = 0; i < TArmeMediaEngine::GetMediaFileHandlerCount(); i++ )
		{
			VArmeMediaFileHandler* pMediaFileHandler = TArmeMediaEngine::GetMediaFileHandler(i);
			if( pMediaFileHandler->CanHandleFileByLookingAtType(theRealPath) )
			{	
				err =  pMediaFileHandler->CreateWSObjsFromFile(this, theObjects, theRealPath, fileRef, pos, progressMngr);
				//as soon as a handler has imported the file, we stop searching
				if (err.type == kArmeOK)
				{
					bHasBeenHandledByExtension = true;
					break;
				}
			}
			else
			{
				err.type = kArmeErrNoSuitableMediaHandlerFound;
			}
		}
	}
	
	delete theRealPath;
//	delete theRealPathCopy;
	return err;
}
		
TArmeWSObjPos TArmeDoc::GetZeroPos() const
{
	TArmeWSObjPos pos;
	pos.fParent = fTopContainer;
	pos.fInput = 0;
	pos.fStartTime = _ArmeTime(0);
	return pos;
}
		
ArmeErr TArmeDoc::InitWithFile(const char* path)
{
	ArmeErr err;
	err.type = kArmeErrNYI;
	return err;
}

void TArmeDoc::NotifyModelChange()
{
	Notify(kNotification_ModelHasChanged,NULL);
}

void TArmeDoc::NotifyAudioClipDataHasChanged(MEAudioClip* clip)
{
	Notify(kNotification_AudioClipDataHasChanged,(void*)clip);
}

ArmeTime TArmeDoc::GetDuration() const
{
	return fTopContainer->GetDuration();
}

const char* TArmeDoc::GetProjectFolderPath() const
{
	return fCurrentProjectFolderPath;
}

void TArmeDoc::SetProjectFolderPath(const char* folderPath)
{
	strcpy(fCurrentProjectFolderPath,folderPath);
}

TArmeDoc::TArmeDoc(ArmeArchiveHandler* unarchiver) :
	ArmeArchivable(unarchiver)
{
	fWasDocDirtifiedDuringAudioChunkManagerUnarchiving = false;
	fViewerIsVisible = false;
	fDoesUpdateMovieArmeTime = true; // for slice sel tracker in video

	TArmeMediaEngine::CheckForCoreAudioPlayer();

	ArmeArchivable* archObj;
	
	/**************************************************************************
	
	fAudioChunkFileManager
	
	**************************************************************************/
	fAudioChunkFileManager = new MEAudioChunkFileManager(this);
	
	if(unarchiver->GetFileVer() < 0x00010031)
	{
		SInt32 audioChunkNextIndex = unarchiver->ReadSInt32();
		fAudioChunkFileManager->SetOldStyleRefsUpToExc(audioChunkNextIndex);
	}
	else
	{
		// NORMAL CASE
		fAudioChunkFileManager->ReadFromArchive(unarchiver);
	}

	/**************************************************************************

		reading the fMasterMovie

	**************************************************************************/
	// first import master movie (it has to be unarchived before any video clip)
	archObj = unarchiver->ReadCPPObject();
	fMasterMovie = dynamic_cast<TArmeMasterMovie*>(archObj);
	if(archObj)
	{
		ZzAssert(fMasterMovie);
	}

	/**************************************************************************

		reading fPlayer (the audio player that has lloc, markers, playpos, etc...)

	**************************************************************************/
	archObj = unarchiver->ReadCPPObject();
	fPlayer = dynamic_cast<MEDocPlayer*>(archObj);
	ZzAssert(fPlayer);

	/**************************************************************************

		reading fMixingInfo

	**************************************************************************/
	archObj = unarchiver->ReadCPPObject();
	fMixingInfo = dynamic_cast<MEMixingInfo*>(archObj);
	ZzAssert(fMixingInfo);

	/**************************************************************************

		reading fSoloObject

	**************************************************************************/
	fVideoSoloObject = dynamic_cast<TArmeWSObj*>(unarchiver->ReadCPPObject());
	fAudioSoloObject = dynamic_cast<TArmeWSObj*>(unarchiver->ReadCPPObject());
	
	/**************************************************************************
	
	reading the TLC (fTopContainer)
	
	**************************************************************************/
	archObj = unarchiver->ReadCPPObject();
	fTopContainer = dynamic_cast<TArmeTLC*>(archObj);
	if(archObj)
	{
		ZzAssert(fTopContainer);
	}
	
	/**************************************************************************
	
	reading the old fAudioChunkNextIndex that isn't used anymore if version >= 0x00010031
	
	**************************************************************************/
	if(unarchiver->GetFileVer() < 0x00010031)
	{
		// thisVariableIsUnused was fAudioChunkNextIndex before
		/*SInt32 thisVariableIsUnused = */unarchiver->ReadUInt32();
	}
	else
	{
	}
		
	/**************************************************************************
	
	video and still image indices
	
	**************************************************************************/
	fVideoChunkNextIndex = unarchiver->ReadUInt32();
	fStillImageIndex = unarchiver->ReadUInt32();
	

	/**************************************************************************
	
	indices for the audio processor incremental names
	
	**************************************************************************/
	ReadPluginClassInstanceIndices(unarchiver);
	

	/**************************************************************************
	
	reading the xfade effects
	
	**************************************************************************/
	SInt32 transitionsCount = unarchiver->ReadSInt32();
//	fOuttaHierarchyTransitionsPool = new ZzArray<TVideoXFadeEffect*>(transitionsCount);
	for (int i =0; i < transitionsCount; ++i)
	{
		(TVideoXFadeEffect*)(unarchiver->ReadCPPObject());
//		fOuttaHierarchyTransitionsPool->AddItem(temp);
	}

	fNewlyCreatedTransitions = new ZzArray<TVideoXFadeEffect*>(8);
	fTransitionsToRemove = new ZzArray<TVideoXFadeEffect*>(8);

	/**************************************************************************
	
	reading data file refs if version >= 0x00010020
	
	**************************************************************************/
	if (unarchiver->GetFileVer() >= 0x00010020)
	{
		SInt32 count = unarchiver->ReadSInt32();
		fDataFileRefArray = new ZzArray<ArmeDataFileRef*>(count);
		for (int i=0; i < count; ++i)
		{
			fDataFileRefArray->AddItem(dynamic_cast<ArmeDataFileRef*>(unarchiver->ReadCPPObject()));
		}
	}
	else
	{
		fDataFileRefArray = new ZzArray<ArmeDataFileRef*>(32);
	}

	/**************************************************************************
	
	reading the transitions if version >= 0x0001002E
	
	**************************************************************************/
	if (unarchiver->GetFileVer() >= 0x0001002E)
	{
		SInt32 transitionsCount = unarchiver->ReadSInt32();
		fOldTransitionsDesc = new ZzArray<TVideoXFadeEffectDesc*>(transitionsCount);
		for (int i =0; i < transitionsCount; ++i)
		{
			TVideoXFadeEffectDesc* temp = (TVideoXFadeEffectDesc*)(unarchiver->ReadCPPObject());
			fOldTransitionsDesc->AddItem(temp);
		}
	}
	else
	{
		fOldTransitionsDesc = new ZzArray<TVideoXFadeEffectDesc*>(32);
	}

	/**************************************************************************
	
	registering the doc in the media engine
	
	**************************************************************************/
	TArmeMediaEngine::AddDoc(this);
}

bool TArmeDoc::WriteToArchive(ArmeArchiveHandler* archiver) const
{
	ArmeArchivable::WriteToArchive(archiver);

	// < 0x0010031 archiver->WriteSInt32(fMEAudioChunkFileNextIndex);
	// >= 0x0010031 
	fAudioChunkFileManager->WriteToArchive(archiver);
	// end of 0x0010031 test for audio indices

	archiver->WriteCPPObject(fMasterMovie);  // master movie is the first thing to import so we save it first
	archiver->WriteCPPObject(fPlayer);
	archiver->WriteCPPObject(fMixingInfo);
	archiver->WriteCPPObject(fVideoSoloObject);
	archiver->WriteCPPObject(fAudioSoloObject);
	archiver->WriteCPPObject(fTopContainer);
	// < 0x0010031 archiver->WriteUInt32(fAudioChunkNextIndex);
	// >= 0x0010031  don't do anything
	// end of 0x0010031 test for the old fAudioChunkNextIndex
	
	archiver->WriteUInt32(fVideoChunkNextIndex);
	archiver->WriteUInt32(fStillImageIndex);
	
	WritePluginClassInstanceIndices(archiver);

	archiver->WriteSInt32(0);  //obsolete
/*	Disabled for version 1.0
	SInt32 count = fOuttaHierarchyTransitionsPool->CountItems();
	archiver->WriteSInt32(count);
	for (int i =0; i < count; ++i)
	{
		TVideoXFadeEffect* xFade = fOuttaHierarchyTransitionsPool->ItemAt(i);
		archiver->WriteCPPObject(xFade);  
	}	*/


	//from version 0x00010020
	archiver->WriteSInt32(fDataFileRefArray->CountItems());
	for (int i = 0; i < fDataFileRefArray->CountItems(); ++i)
	{
		archiver->WriteCPPObject(fDataFileRefArray->ItemAt(i));
	}
				
	//from version 0x0001002E
	SInt32 count = fOldTransitionsDesc->CountItems();
	archiver->WriteSInt32(count);
	for (int i =0; i < count; ++i)
	{
		TVideoXFadeEffectDesc* xFade = fOldTransitionsDesc->ItemAt(i);
		archiver->WriteCPPObject(xFade);  
	}

	return true;
}

bool TArmeDoc::MakeProjectSelfContained(MIProgressBarDlgManager* progressMngr)
{
	try
	{
		verntl(dataFileRef, 3, "Making project self contained");
	
		//compute the number of sub-tasks 
		//*******************************
		int nbSubTasks = 0;
		
		//for video
		for (int i=0; i < fDataFileRefArray->CountItems(); ++i)
		{
			if (fDataFileRefArray->ItemAt(i) && fDataFileRefArray->ItemAt(i)->IsReferenceToExternalFile())
				nbSubTasks++;
		}
		
		//for audio
	
		// if it is already self-contained
		if (nbSubTasks == 0)
			return true;
	
		if (progressMngr) progressMngr->SetNbSubTasks(nbSubTasks);
	
		//VIDEO VIDEO VIDEO
		//*****************

		fMasterMovie->UpdateFile();
	
		//first, go through all the ArmeVideoDataFileRef, check all the external dependencies and copy the files into the project folder
		for (int i=0; i < fDataFileRefArray->CountItems(); ++i)
		{
			if (fDataFileRefArray->ItemAt(i) && fDataFileRefArray->ItemAt(i)->IsReferenceToExternalFile())
			{
				//then, the TArmeMasterMovie should update its references to update these files
				if (!fMasterMovie->UpdateForSelfContainedProject(fDataFileRefArray->ItemAt(i), progressMngr))
					return false;
			}
		}
		
		fMasterMovie->UpdateFile();
		
		//AUDIO AUDIO AUDIO
		//*****************
		
		//$$TODO
		
		
		return true;
	}
	catch(ZzException* e)
	{
		e->DefaultCatcher();
		delete e;
		return false;
	}
}

void TArmeDoc::CleanOldTransitionsDesc()
{
	int i =0;
	if (fOldTransitionsDesc)
	{
		while (i < fOldTransitionsDesc->CountItems())
		{
			TVideoXFadeEffectDesc* xFade = fOldTransitionsDesc->ItemAt(i);
			if ((xFade->fClip1->GetMIObjectRefCount() == 0)
			|| (xFade->fClip2->GetMIObjectRefCount() == 0))
			{
				fOldTransitionsDesc->RemoveItemByIndex(i);
				delete xFade;
				i--;
			}
			i++;
		}
	}
}

void TArmeDoc::RemoveReferencesOnUnusedFilesInMasterMovie()
{
	//delete all the transitions desc that references TVideoObjects that no longer exists
	CleanOldTransitionsDesc();

	int i=0;
	
	// remove all the dataFileRef that are not references anymore
	while (i < fDataFileRefArray->CountItems())
	{
		if (fDataFileRefArray->ItemAt(i) && fDataFileRefArray->ItemAt(i)->GetRefCount() == 0)
		{
			ArmeVideoDataFileRef* videoFile = dynamic_cast<ArmeVideoDataFileRef*>(fDataFileRefArray->ItemAt(i));
			if (videoFile)
				fMasterMovie->GetOriginalTrack()->RemoveFileFromOriginalTrack(videoFile);

			//remove the references in the master.mov
#ifndef NO_VERBOSE_AT_ALL
			char* tempString = new char[kMaxPathLength];
			fDataFileRefArray->ItemAt(i)->GetFilePath(tempString);
			verntl(dataFileRef, 4, "Removing references on " << tempString << " in WorkingMaster.mov");
#endif// NO_VERBOSE_AT_ALL
			fMasterMovie->RemoveReferenceTo(fDataFileRefArray->ItemAt(i));
		}
		i++;
	}
	
	fMasterMovie->UpdateFile();
}

void TArmeDoc::DeleteUnusedFiles()
{
	verntl(dataFileRef, 4, "TArmeDoc::DeleteUnusedFiles");

/*   v   v   iii  dddd     eeeeee    oooo
     v   v    i   d   dd   ee       o    o
     v   v    i   d   dd   eeee     o    o
      v v     i   d   dd   ee       o    o
       v     iii  dddd     eeeeee    oooo                  */

	int i=0;
	
	// remove all the dataFileRef that are not references anymore
	while (i < fDataFileRefArray->CountItems())
	{
		if (fDataFileRefArray->ItemAt(i) && fDataFileRefArray->ItemAt(i)->GetRefCount() == 0)
		{
//			ArmeVideoDataFileRef* videoFile = dynamic_cast<ArmeVideoDataFileRef*>(fDataFileRefArray->ItemAt(i));
/*			if (videoFile)
				fMasterMovie->GetOriginalTrack()->RemoveFileFromOriginalTrack(videoFile);*/

			fDataFileRefArray->ItemAt(i)->PreDestructor();
			delete fDataFileRefArray->ItemAt(i);
			fDataFileRefArray->ItemAt(i) = NULL;
		}
		i++;
	}
	
	// look for all the videoFiles that have been generated in a previous run but that are not referenced at all because the document has not been saved.
	// translation for benja : the files that have an index > maxIndex are bogus. Get rid of them
	i = fVideoChunkNextIndex;
	char temp[1024];
	FILE* fp = NULL;
	ArmeVideoDataFileRef::GetFilePathForIndex(this, temp, i);

	while (fp = fopen(temp, "rb"))
	{
		// if the file exist, remove it.
		fclose(fp);
		remove(temp);
        fp = fopen(temp, "a+b");
        fwrite(temp, strlen(temp), 1, fp);
        fclose(fp);
		verntl(dataFileRef, 4, "Deleting Data File " << temp << " (file with no ArmeDataFileRef)");
		ArmeVideoDataFileRef::GetFilePathForIndex(this, temp, ++i);
	}

	// audio : 
	AudioChunkFileManager()->DeleteUnusedFiles();
}

void TArmeDoc::RecreateNonArchivedCachedData(MIProgressBarDlgManager* progressMngr)
{
	UInt32 nbLengthyTasks = 0;
	// count nb of lengthy tasks 
	fTopContainer->CountPostUnarchivingLengthyTasks(&nbLengthyTasks);
	
	progressMngr->SetNbSubTasks(nbLengthyTasks);
	fTopContainer->RecreateNonArchivedCachedData(progressMngr);
	fPlayer->RecreateTrackMapping();

	//delete all the files that are not currently used
	//DeleteUnusedFiles(); done somewhereelse
}

void TArmeDoc::RemoveVideoDataFileRef(ArmeDataFileRef* fileRef)
{
	//you can only remove the last data file ref
	ZzAssert(fileRef->fFileIndex == fVideoChunkNextIndex-1);
	fDataFileRefArray->RemoveItemByValue(fileRef);
	fVideoChunkNextIndex--;
}

ArmeDataFileRef* TArmeDoc::GetNewVideoDataFileRef(const char* sourceFile, bool* isNewRef)
{
	ArmeDataFileRef* ref = NULL;
	bool newRefTemp = true;
	if (!isNewRef) isNewRef = &newRefTemp;
	
	if (sourceFile)
	{
		//first, try to find a VideoDataFileRef on this file
		for (int i=0; i < fDataFileRefArray->CountItems(); ++i)
		{
			ArmeVideoDataFileRef* vidRef = dynamic_cast<ArmeVideoDataFileRef*>(fDataFileRefArray->ItemAt(i));
			if (vidRef)
			{
				const char* vidRefSourceFilePath = vidRef->GetSourceFilePath();
				if (vidRefSourceFilePath && (strcmp(sourceFile, vidRefSourceFilePath) == 0))
				{
					*isNewRef = false;
					return vidRef;
				}
			}
		}
	}

	//otherwise, create a new ref
	ref = new ArmeVideoDataFileRef(fVideoChunkNextIndex++, this, sourceFile);
	
	fDataFileRefArray->AddItem(ref);
	return ref;
}

/* OBSOLETE */
/*
ArmeDataFileRef* TArmeDoc::GetNewAudioDataFileRef()
{
	ArmeDataFileRef* ret = new ArmeAudioDataFileRef(fAudioChunkNextIndex++, this);
	fDataFileRefArray->AddItem(ret);
	return ret;
}
*/	
void TArmeDoc::Seek(ArmeTime newTime)
{
	SetMovieArmeTime(fMasterMovie->GetQTMovie(), this, newTime);
	LockWrite();
	GetTLCPlayer()->Seek(newTime);
	UnlockWrite();
}

void TArmeDoc::SeekToStart()
{
	GoToBeginningOfMovie(fMasterMovie->GetQTMovie());
	LockWrite();
	GetTLCPlayer()->Seek(_ArmeTime(0));
	UnlockWrite();
}

void TArmeDoc::SeekToEnd()
{
	GoToEndOfMovie(fMasterMovie->GetQTMovie());
	LockWrite();
	GetTLCPlayer()->Seek(fTopContainer->GetDuration());
	UnlockWrite();
}


void TArmeDoc::SetVideoSoloObject(TArmeWSObj* obj)
{
	if (obj == NULL)
	{
		fVideoSoloObject = NULL;

		//unmute all video objects (note : we need to do it after we have changed the fSoloObject to be sure that the previous soloObject
		// will be set to the right visible status (i.e. if it was muted before being the solo object))
//		((TVideoObject*)GetTopContainer())->SetMutedForSolo(false);
		fMasterMovie->UpdateSubMovie(GetTopContainer());

		return;
	}
	else
	{
		fVideoSoloObject = obj;
		
		//unmute all video objects (note : we need to do it after we have changed the fSoloObject to be sure that the previous soloObject
		// will be set to the right visible status (i.e. if it was muted before being the solo object))
//		((TVideoObject*)GetTopContainer())->SetMutedForSolo(false);
		fMasterMovie->UpdateSubMovie(GetTopContainer());

		// if there is some video in the solo object,
/*		if (fVideoSoloObject->ContainsVideo(true))
		{
			//mute all video objects
			((TVideoObject*)GetTopContainer())->SetMutedForSolo(true);
		
			//un-mute the selection
			TVideoObject* fSolo = dynamic_cast<TVideoObject*>(fVideoSoloObject);
			if (fSolo)
			{
				fSolo->SetMutedForSolo(false);
			}
		}*/
	}

}

void TArmeDoc::SetAudioSoloObject(TArmeWSObj* obj)
{
	if (obj == NULL)
	{
		fAudioSoloObject = NULL;

		// put all the audio objects in their previous muted state
		GetTopContainer()->RecursiveSetAudioGainAccordingToMute();

		return;
	}
	else
	{
		fAudioSoloObject = obj;
		
		// we need to set the new audio gain of the objects according to the new solo state
		
		// "mute" everything
		GetTopContainer()->SetAudioGainInSelfAndDescendants(0.0);
		// "enable" the object and its descendants
		fAudioSoloObject->SetAudioGainInSelfAndDescendants(1.0);
		// "enable" the object and its ascendants
		fAudioSoloObject->SetAudioGainInSelfAndAscendants(1.0);
	}
}

/*
void TArmeDoc::SetSoloObject(TArmeWSObj* obj)
{
	if (fSoloObject == obj)	// if the solo command is called on the object that was already the solo object, there is nothing else to do 
	{
		fSoloObject = NULL;

		//unmute all video objects (note : we need to do it after we have changed the fSoloObject to be sure that the previous soloObject
		// will be set to the right visible status (i.e. if it was muted before being the solo object))
		((TVideoObject*)GetTopContainer())->SetMutedForSolo(false);

		return;
	}
	else
	{
		fSoloObject = obj;
		
		//unmute all video objects (note : we need to do it after we have changed the fSoloObject to be sure that the previous soloObject
		// will be set to the right visible status (i.e. if it was muted before being the solo object))
		((TVideoObject*)GetTopContainer())->SetMutedForSolo(false);

		if (!fSoloObject)
		{
			GetTopContainer()->RecursiveSetAudioGainAccordingToMute();
			return;
		}

		// if there is some video in the solo object,
		if (fSoloObject->ContainsVideo(true))
		{
			//mute all video objects
			((TVideoObject*)GetTopContainer())->SetMutedForSolo(true);
		
			//un-mute the selection
			TVideoObject* fVideoSoloObject = dynamic_cast<TVideoObject*>(fSoloObject);
			if (fVideoSoloObject)
			{
				fVideoSoloObject->SetMutedForSolo(false);
			}
		}
		
		// we need to set the new audio gain of the objects according to the new solo state
		
		// "mute" everything
		GetTopContainer()->SetAudioGainInSelfAndDescendants(0.0);
		// "enable" the object and its descendants
		fSoloObject->SetAudioGainInSelfAndDescendants(1.0);
		// "enable" the object and its ascendants
		fSoloObject->SetAudioGainInSelfAndAscendants(1.0);
	}
}
*/

UInt32 TArmeDoc::GetAndIncrNextIndexForAudioPluginClass(VARPIPluginClass* pluginClass)
{
	string pluginClassStringSig;
	pluginClassStringSig += pluginClass->GetManufacturerName();
	pluginClassStringSig += ":";
	pluginClassStringSig += pluginClass->GetPluginName();

	SInt32 nbFound = fPluginClassInstanceIndices.count(pluginClassStringSig);
	
	ZzAssert((nbFound==1||nbFound==0));
	
	if(nbFound == 0)
	{
		fPluginClassInstanceIndices[pluginClassStringSig] = 2; // must contain the next value to be returned
		return 1;
	}
	else if(nbFound == 1)
	{
		int currentCount = fPluginClassInstanceIndices[pluginClassStringSig];
		fPluginClassInstanceIndices[pluginClassStringSig] = currentCount + 1;
		return currentCount;
	}
	else
	{
		ZzAssert(0);
		return 0;
	}
}


#define kPluginClassMaxNameLength 128

void TArmeDoc::ReadPluginClassInstanceIndices(ArmeArchiveHandler* unarchiver)
{
//	std::map<std::string,int>	fPluginClassInstanceIndices;
	SInt32 nbIndices = unarchiver->ReadSInt32();
	char pluginClassName[kPluginClassMaxNameLength];
	for(SInt32 i = 0; i < nbIndices; i++)
	{
		unarchiver->ReadRawData((void*)pluginClassName,kPluginClassMaxNameLength);
		UInt32 value = unarchiver->ReadUInt32();
		fPluginClassInstanceIndices[pluginClassName] = value;
	}
}

void TArmeDoc::WritePluginClassInstanceIndices(ArmeArchiveHandler* archiver) const
{
//	std::map<std::string,int>	fPluginClassInstanceIndices;
	SInt32 count = 0;
	// first count all the entries
	for(	std::map<std::string,int>::const_iterator iter = fPluginClassInstanceIndices.begin(); 
			iter != fPluginClassInstanceIndices.end();
			++iter	)
	{
		count++;
	}
	archiver->WriteSInt32(count);
	char pluginClassName[kPluginClassMaxNameLength];
	for(	std::map<std::string,int>::const_iterator iter2 = fPluginClassInstanceIndices.begin(); 
			iter2 != fPluginClassInstanceIndices.end(); 
			++iter2	)
	{
		std::string key = iter2->first;
		int keyLen = key.length();
		const char* szKey = key.data();
		ZzAssert(keyLen < kPluginClassMaxNameLength);
		strcpy(pluginClassName,szKey);
		int value = iter2->second;
		archiver->WriteRawData((const void*)pluginClassName,kPluginClassMaxNameLength);
		archiver->WriteUInt32(value);
	}
}

TimeValue TArmeDoc::RoughTimeValueFromArmeTime(ArmeTime time) const
{
	TimeValue ret;
	ret = ((int64)time.data * (int64)GetVideoTimeScale() / (int64)GetAudioFrameRate());
	
	return ret;
}

TimeValue TArmeDoc::TimeValueFromArmeTime(ArmeTime time) const
{
	TimeValue ret;
	ret = ((int64)time.data * (int64)GetVideoTimeScale() / (int64)GetAudioFrameRate() + (int64)GetVideoFrameDuration()/2) / (int64)GetVideoFrameDuration();
	ret *= GetVideoFrameDuration();
	
	return ret;
}

ArmeTime TArmeDoc::ArmeTimeFromTimeValue(TimeValue time) const
{
	ArmeTime ret;
	
	ret.data = long((double)time * (double)GetAudioFrameRate() / (double)GetVideoTimeScale() + 0.5);
	
	return ret;
}

ArmeTime TArmeDoc::RoundArmeTimeToNearestVideoFrame(ArmeTime &time) const
{
	TimeValue temp = TimeValueFromArmeTime(time);
	time = ArmeTimeFromTimeValue(temp);
	
	return time;
}

void TArmeDoc::LockRead()
{
	fRwLocker.LockRead();
}

void TArmeDoc::UnlockRead()
{
	fRwLocker.UnlockRead();
}

void TArmeDoc::LockWrite()
{
#ifdef DEBUG
	fLockTimer.Reset();
#endif//DEBUG
	fRwLocker.LockWrite();
}

void TArmeDoc::UnlockWrite()
{
	fRwLocker.UnlockWrite();
#ifdef DEBUG
//	UInt64 lockWriteDuration = fLockTimer.GetUS();
//	vernl(6,"lockWrite body = " << (UInt32)lockWriteDuration);
#endif//DEBUG
}

void TArmeDoc::Preroll(ArmeTime pos)
{
	PrerollMovieArmeTime(fMasterMovie->GetQTMovie(), this, pos);
	// we don't need to lock the doc since prerolling just means sending a message the audio track players DRT
	GetTLCPlayer()->Preroll(GET_VAL(pos));
}

void TArmeDoc::Preroll(SInt64 pos)
{
	PrerollMovieArmeTime(fMasterMovie->GetQTMovie(), this, _ArmeTime(pos));
	// we don't need to lock the doc since prerolling just means sending a message the audio track players DRT
	GetTLCPlayer()->Preroll(pos);
}

void TArmeDoc::SetDoesUpdateMovieArmeTime(bool doesUpdate)
{
	if(doesUpdate)
	{
		if(IsPlaying())
		{
			StartMovie(fMasterMovie->GetQTMovie());
		}
	}
	else
	{
		StopMovie(fMasterMovie->GetQTMovie());
	}
	fDoesUpdateMovieArmeTime = doesUpdate; // for slice sel tracker in video
}

UInt32 TArmeDoc::GetAudioFrameRate() const
{
	return TArmeMediaEngine::CoreAudioPlayer()->GetAudioFrameRate(); 
}


void TArmeDoc::AddToOldTransitionsDesc(TVideoXFadeEffect* xFade)
{
	if (!xFade)
		return;
		
	//no need to keep track of transitions on transitions
	if ((xFade->fClip1->GetType() == TArmeWSObj::kVideoXFadeEffect) || (xFade->fClip2->GetType() == TArmeWSObj::kVideoXFadeEffect))
		return;
	
	if (FindEffectDescInOldTransitions(xFade->fClip1, xFade->fClip2))
		RemoveFromOldTransitionsDesc(xFade->fClip1, xFade->fClip2);
		
	fOldTransitionsDesc->AddItem(new TVideoXFadeEffectDesc(xFade->fClip1, xFade->fClip2, xFade->fEffectDesc, xFade->fTransitionType, xFade->fFixedParameters));
}

TVideoXFadeEffectDesc* TArmeDoc::FindEffectDescInOldTransitions(TVideoObject* clip1, TVideoObject* clip2)
{
	for (int i = 0; i < fOldTransitionsDesc->CountItems(); ++i)
	{
		TVideoXFadeEffectDesc* xFadeDesc = fOldTransitionsDesc->ItemAt(i);

		if ( ((xFadeDesc->fClip1 == clip1) && (xFadeDesc->fClip2 == clip2))
			|| ((xFadeDesc->fClip1 == clip2) && (xFadeDesc->fClip2 == clip1))) 
		{
			return xFadeDesc;
		}
	}
	return NULL;
}

void TArmeDoc::RemoveFromOldTransitionsDesc(TVideoObject* obj)
{
	int i = 0;

	while (i < fOldTransitionsDesc->CountItems())
	{
		TVideoXFadeEffectDesc* xFadeDesc = fOldTransitionsDesc->ItemAt(i);

		if ((xFadeDesc->fClip1 == obj) || (xFadeDesc->fClip2 == obj))  
		{
			fOldTransitionsDesc->RemoveItemByIndex(i);
			delete xFadeDesc;
			i--;
		}
		
		i++;
	}
}

void TArmeDoc::RemoveFromOldTransitionsDesc(TVideoObject* obj1, TVideoObject* obj2)
{
	int i = 0;

	while (i < fOldTransitionsDesc->CountItems())
	{
		TVideoXFadeEffectDesc* xFadeDesc = fOldTransitionsDesc->ItemAt(i);

		if ( ((xFadeDesc->fClip1 == obj1) && (xFadeDesc->fClip2 == obj2))
			|| ((xFadeDesc->fClip1 == obj2) && (xFadeDesc->fClip2 == obj1))) 
		{
			fOldTransitionsDesc->RemoveItemByIndex(i);
			delete xFadeDesc;
			i--;
		}
		
		i++;
	}
}

void TArmeDoc::SetAudioChunkFileManagerDidDirtifyDoc()
{
	fWasDocDirtifiedDuringAudioChunkManagerUnarchiving = true;
}

bool TArmeDoc::GetAudioChunkFileManagerDidDirtifyDoc() const
{
	return fWasDocDirtifiedDuringAudioChunkManagerUnarchiving;
}


