/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#ifndef TArmeMasterMovie_h
#include "TArmeMasterMovie.h"
#endif //TArmeMasterMovie_h

#ifndef MovieHelper_h
#include "MovieHelper.h"
#endif //MovieHelper_h

#ifndef TArmeDoc_h
#include "TArmeDoc.h"
#endif //TArmeDoc_h

#ifndef TVideoClip_h
#include "TVideoClip.h"
#endif //TVideoClip_h

#ifndef ZzArray_h
#include "ZzArray.h"
#endif //ZzArray_h

#ifndef TVideoEffect_h
#include "TVideoEffect.h"
#endif //TVideoEffect_h

#ifndef ZzHelpers_h
#include "ZzHelpers.h"
#endif //ZzHelpers_h

#ifndef Zz_PathHelpers_h
#include "Zz_PathHelpers.h"
#endif //Zz_PathHelpers_h

#ifndef TArmeMediaEngine_h
#include "TArmeMediaEngine.h"
#endif //TArmeMediaEngine_h

#ifndef TArmeTextClip_h
#include "TArmeTextClip.h"
#endif //TArmeTextClip_h

#ifndef MIProgressBarDlgManager_h
#include "MIProgressBarDlgManager.h"
#endif //MIProgressBarDlgManager_h

#ifndef MEApplicationServices_h
#include "MEApplicationServices.h"
#endif //MEApplicationServices_h

#undef MASTER_MOVIE_MAX_DURATION_SEC
#define MASTER_MOVIE_MAX_DURATION_SEC 36000


Track TArmeMasterMovie::GetMasterTrackForText()
{
	if (!fTrackIdForTextMedia)
	{
		Track qtTrack = NewMovieTrack(GetQTMovie(), GetWidth() << 16, GetHeight() << 16, kNoVolume);
		fTrackIdForTextMedia = GetTrackID(qtTrack);
		
		//create the media
		//----------------
		Media mediaEffect;
		//this track is a source track, it can be a textTrack or a videoTrack
		mediaEffect = NewTrackMedia(qtTrack, TextMediaType, GetVideoTimeScale(), NULL, 0); 
		
		SetTrackEnabled(qtTrack, false);
	}

	return GetMovieTrack(GetQTMovie(), fTrackIdForTextMedia);
}


TArmeMasterMovie::TArmeMasterMovie(TArmeDoc* doc, WorkingCodecType workingCodec)
{
	//we need to do this now because we will call some methods of the doc that requires to know the masterMovie
	doc->fMasterMovie = this;
	doc->GetTopContainer()->fMasterMovie = this;

	Init(doc, workingCodec, "WorkingMaster.Mov");
};

void TArmeMasterMovie::Init(TArmeDoc* doc, WorkingCodecType workingCodec, const char* fileName)
{
	char filePath[kMaxPathLength];
	OSErr err;
	//Boolean bHasChanged;
	fQTBackgroundTrack = NULL;
	
	fDoc = doc;
	
	fResourceId = 0;
	
	/*
	Now have two version of the Master.mov (2nd = WorkingMaster.mov)
	All the temporary work is done on WorkingMaster.mov.  When saving a doc, workingMaster.mov is copied in master.mov.
	When opening a doc, master.mov is copied in workingMaster.mov.
	By this way, a save as doesn't mess the current movie and you can safely qui without saving, you will efectively loose your changes.
	*/	
	ZzAppendFileNameToPath(filePath, kMaxPathLength, doc->GetProjectFolderPath(), fileName);
	
	fsSpec = FSSpecFromNonExistingFilePath(filePath, FOUR_CHAR_CODE('TVOD'), FOUR_CHAR_CODE('MooV'), err);
	ThrowOnOSStatus(err);
	err = CreateMovieFile(&fsSpec, FOUR_CHAR_CODE('TVOD'), smCurrentScript, 
			createMovieFileDontCreateResFile | newMovieActive, &fResourceRefNum, &fQTMovie);
	ThrowOnOSStatus(err);
	        
	fResourceId = movieInDataForkResID;
	err = AddMovieResource(fQTMovie, fResourceRefNum, &fResourceId, (unsigned char*)"MasterMovie");
	ThrowOnOSStatus(err);
	CloseMovieFile(fResourceRefNum);

	SetWorkingCodecType(workingCodec);
	CreateBackground();

//	SetMoviePlayHints(fQTMovie, hintsHighQuality, hintsHighQuality);
	fOriginalTrack = new TArmeMasterMovieOriginalTrack(this);

	fSplitsArray = new 	ZzArray<ArmeTime>(128);
	fChunksArray = new ZzArray<TArmeMasterMovieChunk*>(128);
	fHierarchiesArray = new ZzArray<TArmeMasterMovieHierarchyElement*>(32);
	fTextTrackArray = new ZzArray<TArmeMasterMovieTextTrack*>(2);

	fSplitsArray->AddItem(ARME_TIME_ZERO);
	fSplitsArray->AddItem(GetDoc()->SecondsToArmeTime(MASTER_MOVIE_MAX_DURATION_SEC));
	fChunksArray->AddItem(new TArmeMasterMovieChunk(this, ARME_TIME_ZERO, GetDoc()->SecondsToArmeTime(MASTER_MOVIE_MAX_DURATION_SEC)));
	fChunksArray->ItemAt(0)->fTopChunk = new TArmeMasterMovieChunkHierarchyElement(GetDoc()->GetTopContainer(), NULL, fChunksArray->ItemAt(0), NULL);

	fTrackIdForTextMedia = 0;

	verntl(videoAlain, 6, "TArmeMasterMovie::TArmeMasterMovie(), err = " << err);
}

TArmeMasterMovie::TArmeMasterMovie(TArmeDoc* doc, WorkingCodecType workingCodec, const char* fileName)
{
	Init(doc, workingCodec, fileName);
}


TArmeMasterMovie::TArmeMasterMovie(ArmeArchiveHandler* unarchiver) :
	ArmeArchivable(unarchiver)
{
	char temp[kMaxPathLength];
	ArmeErr err;
	verntl(archivingEngine, 6, "TArmeMasterMovie::TArmeMasterMovie(ArmeArchiveHandler*)");

	fDoc = dynamic_cast<TArmeDoc*>(unarchiver->ReadCPPObject());
	fDoc->fMasterMovie = this;
	fWorkingCodecType = (WorkingCodecType)(unarchiver->ReadSInt32());

	fPreviewCodecType = (WorkingCodecType)(unarchiver->ReadSInt32());

	ZzAppendFileNameToPath(temp, kMaxPathLength, unarchiver->GetProjectDirectory(), "WorkingMaster.mov");
	// import master movie
	fQTMovie = LoadMovie(temp, &fResourceRefNum, &fResourceId, &err, &fsSpec, true);
	if(!fQTMovie)
	{
		ZzAssertMsg(fQTMovie,"Cannot find Master Movie");
	}

	fOriginalTrack = dynamic_cast<TArmeMasterMovieOriginalTrack*>(unarchiver->ReadCPPObject());

	SInt32 splitsCount = unarchiver->ReadSInt32();
	fSplitsArray = new ZzArray<ArmeTime>(splitsCount);
	for (int i =0; i < splitsCount; ++i)
		fSplitsArray->AddItem(unarchiver->ReadArmeTime());

	SInt32 chunksCount = unarchiver->ReadSInt32();
	fChunksArray = new ZzArray<TArmeMasterMovieChunk*>(chunksCount);
	for (int i =0; i < chunksCount; ++i)
		fChunksArray->AddItem((TArmeMasterMovieChunk*)(unarchiver->ReadCPPObject()));

	SInt32 hiararchiesCount = unarchiver->ReadSInt32();
	fHierarchiesArray = new ZzArray<TArmeMasterMovieHierarchyElement*>(hiararchiesCount);
	for (int i =0; i < hiararchiesCount; ++i)
		fHierarchiesArray->AddItem((TArmeMasterMovieHierarchyElement*)(unarchiver->ReadCPPObject()));
	
	SInt32 trackID = unarchiver->ReadSInt32();
	fQTBackgroundTrack = GetMovieTrack(fQTMovie, trackID);

	if (unarchiver->GetFileVer() >= 0x00010021)
	{
		SInt32 textTrackCount = unarchiver->ReadSInt32();
		fTextTrackArray = new ZzArray<TArmeMasterMovieTextTrack*>(textTrackCount);
		for (int i =0; i < textTrackCount; ++i)
			fTextTrackArray->AddItem((TArmeMasterMovieTextTrack*)(unarchiver->ReadCPPObject()));
	}
	else
	{
		fTextTrackArray = new ZzArray<TArmeMasterMovieTextTrack*>(2);
	}
	
	if (unarchiver->GetFileVer() >= 0x0001002F)
	{
		fTrackIdForTextMedia = unarchiver->ReadSInt32();
	}
	else
	{
		fTrackIdForTextMedia = 0;
	}
		
	// initialize timescale, ...
	SetWorkingCodecType(fWorkingCodecType);
	
//	SetMoviePlayHints(fQTMovie, hintsHighQuality, hintsHighQuality);
}

bool TArmeMasterMovie::WriteToArchive(ArmeArchiveHandler* archiver) const
{
	verntl(archivingEngine, 6, "TArmeMasterMovie::WriteToArchive(ArmeArchiveHandler*)");
	archiver->WriteCPPObject(fDoc);

	archiver->WriteSInt32((SInt32)fWorkingCodecType);
	archiver->WriteSInt32((SInt32)fPreviewCodecType);
	archiver->WriteCPPObject(fOriginalTrack);
	
	archiver->WriteSInt32(fSplitsArray->CountItems());
	for (int i =0; i < fSplitsArray->CountItems(); ++i)
		archiver->WriteArmeTime(fSplitsArray->ItemAt(i));
	
	archiver->WriteSInt32(fChunksArray->CountItems());
	for (int i =0; i < fChunksArray->CountItems(); ++i)
		archiver->WriteCPPObject(fChunksArray->ItemAt(i));

	archiver->WriteSInt32(fHierarchiesArray->CountItems());
	for (int i =0; i < fHierarchiesArray->CountItems(); ++i)
		archiver->WriteCPPObject(fHierarchiesArray->ItemAt(i));
	
	archiver->WriteSInt32(GetTrackID(fQTBackgroundTrack));
	
	//from version 0x00010021
	archiver->WriteSInt32(fTextTrackArray->CountItems());
	for (int i =0; i < fTextTrackArray->CountItems(); ++i)
		archiver->WriteCPPObject(fTextTrackArray->ItemAt(i));
	
	//from version 0x0001002F
	archiver->WriteSInt32(fTrackIdForTextMedia);
	
	//update the file that contains the MasterMovie (workingmaster.mov)
	UpdateFile();  //$$TESTS$$ generate errors inside

	return true;
}


TArmeMasterMovie::~TArmeMasterMovie()
{
	delete fOriginalTrack;
	delete fSplitsArray;
	delete fChunksArray;
	delete fHierarchiesArray;
	delete fTextTrackArray;
	DisposeMovie(fQTMovie);
}; 

bool TArmeMasterMovie::UpdateForSelfContainedProject(ArmeDataFileRef* dataFileRef, MIProgressBarDlgManager* progressMngr)
{
	char oldFileName[1024];
	char newFileName[1024];
	dataFileRef->GetFilePath(oldFileName);
	dataFileRef->GetFilePathInSelfContainedProject(newFileName);

	// let's first copy the file in the project directory
	if (progressMngr) progressMngr->EnterSubTask();
	if (ZzCopyMacFile(oldFileName, newFileName, progressMngr))
	{
		//if the copy has succeeded, we can can update the ArmeDataFileRef
		dataFileRef->UpdateForSelfContainedProject();
	}
	else
	{
		return false;
	}
	if (progressMngr) progressMngr->LeaveSubTask();
	
	//then, replace the references in the QTMovie
	ReplaceMediaReferences(fQTMovie, oldFileName, newFileName);
	return true;
}

bool TArmeMasterMovie::RemoveReferenceTo(ArmeDataFileRef* dataFileRef)
{
	//actually, we replace the reference to the file by a reference to the background.mov
	char oldFileName[1024];
	char backgroundFileName[1024];
	backgroundFileName[0] = 0;
	
	dataFileRef->GetFilePath(oldFileName);
	ZzAppendFileNameToPath(backgroundFileName, 1024, GetDoc()->GetProjectFolderPath(), "background.mov");

	ReplaceMediaReferences(fQTMovie, oldFileName, backgroundFileName);
	
	return true;
}

void TArmeMasterMovie::CreateBackground()
{
	verntl(videoAlain, 2, "Creating MasterMovie Background ");
	if (fQTBackgroundTrack)
	{
		verntl(videoAlain, 4, "Dispose old background");
		DisposeMovieTrack(fQTBackgroundTrack);
	}

	//build a background track that contains only white frames.  It will be a source to copy white frames in any other tracks of the master movie
	//that's why it is not visible
	ArmeErr armeErr;
	short movieResFile;
	short movieResId;
	char* curPath = (char*)malloc(1024);
	getcwd(curPath,1024);
	//note : right now, background is always white
	ZzString bundlePath = ZzGetMainBundlePath();
	ZzString imagePath = bundlePath + "/Contents/Resources/white3232.jpg";
	Movie backgroundMovie = LoadMovie(imagePath.getUTF8String(), &movieResFile, &movieResId, &armeErr);
	verntl(videoAlain, 4, "Imported white3232.jpg" );

	// export white3232.jpg into "background.mov" in the project folder
	char backgroundInProject[1024];
	ZzAppendFileNameToPath(backgroundInProject, 1024, fDoc->GetProjectFolderPath(), "background.mov");
	SaveMovie(backgroundMovie, backgroundInProject);
	DisposeMovie(backgroundMovie);
	backgroundMovie = LoadMovie(backgroundInProject, &movieResFile, &movieResId, &armeErr);

	fQTBackgroundTrack = CreateNewTrackInMovie(GetQTMovie(), GetVideoTimeScale(), GetWidth(), GetHeight());
	ArmeTime srcBackgroundDuration = ArmeTimeFromTimeValue2(backgroundMovie, GetDoc(), GetMovieDuration(backgroundMovie));
	CopyTrackSegment(GetQTMovie(), fQTBackgroundTrack, backgroundMovie, GetFirstVideoTrack(backgroundMovie), 0, 0, GetMovieDuration(backgroundMovie));
	OSErr err = ScaleTrackSegment(fQTBackgroundTrack, 0, GetDoc()->TimeValueFromArmeTime(srcBackgroundDuration), GetDoc()->TimeValueFromArmeTime(GetDoc()->SecondsToArmeTime(MASTER_MOVIE_MAX_DURATION_SEC)));
	ThrowOnOSStatus(err);
	DisposeMovie(backgroundMovie);
	//put the background track behind
	SetTrackLayer(fQTBackgroundTrack, 1500);
	verntl(videoAlain, 2, "MasterMovie Background created");
}

int TArmeMasterMovie::GetWorkingWidth() const 
{
	switch (fWorkingCodecType)
	{
		case kDvPal720_576 :
		case kDvPal16_9:
			return 720+1;
		case kDvNtsc720_480 :
		case kDvNtsc16_9 :
			return 720+1;
		case kMjpeg320_240 :
			return 320+1;
		case kMjpeg160_120 :
			return 160+1;
		default :
			return 320;  
	}
} 
int	TArmeMasterMovie::GetWorkingHeight() const 
{
	switch (fWorkingCodecType)
	{
		case kDvPal720_576 :
		case kDvPal16_9 :
			return 576+1;
		case kDvNtsc720_480 :
		case kDvNtsc16_9 :
			return 480+1;
		case kMjpeg320_240 :
			return 240+1;
		case kMjpeg160_120 :
			return 120+1;
		default :
			return 240;  
	}
}
int	TArmeMasterMovie::GetViewerWidth() const
{
	switch (fWorkingCodecType)
	{
		case kDvPal720_576 :
			return 768;
		case kDvPal16_9 :
			return 1024;
		case kDvNtsc720_480 :
			return 720;
		case kDvNtsc16_9 :
			return 960;
		case kMjpeg320_240 :
			return 320;
		case kMjpeg160_120 :
			return 160;
		default :
			return 320;  
	}
}
int	TArmeMasterMovie::GetViewerHeight() const
{
	switch (fWorkingCodecType)
	{
		case kDvPal720_576 :
		case kDvPal16_9 :
			return 576;
		case kDvNtsc720_480 :
		case kDvNtsc16_9 :
			return 540;
		case kMjpeg320_240 :
			return 240;
		case kMjpeg160_120 :
			return 120;
		default :
			return 240;  
	}
}


int TArmeMasterMovie::GetWidth() const 
{
	return GetWorkingWidth();
/*	switch (fPreviewCodecType)
	{
		case kDvPal720_576 :
			return 720;
		case kDvNtsc720_480 :
			return 720;
		case kMjpeg320_240 :
			return 320;
		case kMjpeg160_120 :
			return 160;
		default :
			return 320;  
	}*/
} 
int	TArmeMasterMovie::GetHeight() const 
{
	return GetWorkingHeight();
/*	switch (fPreviewCodecType)
	{
		case kDvPal720_576 :
			return 576;
		case kDvNtsc720_480 :
			return 480;
		case kMjpeg320_240 :
			return 240;
		case kMjpeg160_120 :
			return 120;
		default :
			return 240;  
	}*/
}
float TArmeMasterMovie::GetFPS() const 
{
	switch (fWorkingCodecType)
	{
		case kDvPal720_576 :
		case kDvPal16_9 :
			return 25.0f;
		case kDvNtsc720_480 :
		case kDvNtsc16_9 :
			return 29.97f;  //$$CHECK
		case kMjpeg320_240 :
			return 30.0f; 
		case kMjpeg160_120 :
			return 24.0f;
		default :
			return 30.0f; 
	}
};

WorkingCodecType TArmeMasterMovie::GetWorkingCodecType() const {return fWorkingCodecType;};

WorkingCodecType TArmeMasterMovie::GetWorkingCodecTypeForPictures() const 
{
	switch (fWorkingCodecType)
	{
		case kDvPal720_576 :
		case kDvPal16_9 :
		{
			return kPictureHighQuality720_576;
		};break;
		case kDvNtsc720_480 :
		case kDvNtsc16_9 :
		{
			return kPictureHighQuality720_480;
		};break;
		default :
		return fWorkingCodecType;
	}
};


void TArmeMasterMovie::SetWorkingCodecType(WorkingCodecType codec) 
{	
	fWorkingCodecType = codec;
	switch (fWorkingCodecType)
	{
		case kDvPal720_576 :
		case kDvPal16_9 :
		{
			fVideoTimeScale = 600;
			fVideoFrameDuration = 24;
		};break;
		case kDvNtsc720_480 :
		case kDvNtsc16_9 :
		{
			fVideoTimeScale = 2997;
			fVideoFrameDuration = 100;
		};break;
		case kMjpeg320_240 :
		{
			fVideoTimeScale = 600;
			fVideoFrameDuration = 20;  
		};break;
		case kMjpeg160_120 :
		{
			fVideoTimeScale = 600;
			fVideoFrameDuration = 25;
		};break;
		default :
			ZzAssert(0);
	}
	SetMovieTimeScale(fQTMovie, fVideoTimeScale);

};

WorkingCodecType TArmeMasterMovie::GetPreviewCodecType() const {return fPreviewCodecType;};
void TArmeMasterMovie::SetPreviewCodecType(WorkingCodecType codec) 
{
	fPreviewCodecType = codec; 
};

/*
Adds a split to the array of split.
If there is already a split at this time, don't do anything.
If this is a new split, split the chunk that is below the splitTime.
*/
void TArmeMasterMovie::AddSplit(ArmeTime splitTime)
{
	for (int i = 0; i < fSplitsArray->CountItems()-1; ++i)
	{
		if (fSplitsArray->ItemAt(i) == splitTime)
			return;
		
		if ((fSplitsArray->ItemAt(i) < splitTime) && (fSplitsArray->ItemAt(i+1) > splitTime))
		{
			fSplitsArray->InsertItem(splitTime, i+1);
			
			//split the Chunk below the split
			GetChunkAtTime(splitTime)->Split(splitTime);
		
			return;
		}
	}
}

/*
Removes a split from the array of split.
Merge the two chunks that were before and after the split (only if they have the same hierarchy).
If the merge fails, it means that the split is still useful and it must not be removed.
*/
void TArmeMasterMovie::RemoveSplit(ArmeTime splitTime)
{
	// we start at 1 because split 0 is at ARME_TIME_ZERO and cannot be removed
	// the same for the last one that cannot be removed as well
	for (int i = 1; i < fSplitsArray->CountItems()-1; ++i)
	{
		if (fSplitsArray->ItemAt(i) == splitTime)
		{
			//try to merge the chunks that were on each side of the split
			if (GetChunk(i-1)->MergeWith(GetChunk(i)))
			{
				// if the merge has succeeded, the split is not useful anymore and can be removed
				fSplitsArray->RemoveItemByIndex(i);
			}
			return;
		}
	}
}

TArmeMasterMovieChunk* TArmeMasterMovie::GetChunk(int index)
{
	return fChunksArray->ItemAt(index);
}

TArmeMasterMovieChunk* TArmeMasterMovie::GetChunk(ArmeTime startTime, ArmeTime endTime)
{
	TArmeMasterMovieChunk* chunk = GetChunkStartingAtTime(startTime);

	if (chunk && chunk->fEndTime == endTime)
		return chunk;
	else
		return NULL;
}

TArmeMasterMovieChunk*	TArmeMasterMovie::GetChunkStartingAtTime(ArmeTime startTime)
{
	for (int i = 0; i < fChunksArray->CountItems(); ++i)
	{
		if (fChunksArray->ItemAt(i)->fStartTime == startTime)
			return fChunksArray->ItemAt(i);
	}
	
	return NULL;
}

TArmeMasterMovieChunk* TArmeMasterMovie::GetChunkAtTime(ArmeTime time)
{
	for (int i = 0; i < fChunksArray->CountItems(); ++i)
	{
		if ((fChunksArray->ItemAt(i)->fStartTime < time) && (fChunksArray->ItemAt(i)->fEndTime >= time))
			return fChunksArray->ItemAt(i);
	}
	
	return NULL;
}

/* Insert the chunk in the list of chunks*/
void TArmeMasterMovie::InsertChunkAfter(TArmeMasterMovieChunk* insertAfter, TArmeMasterMovieChunk* insertObject)
{
	for (int i = 0; i < fChunksArray->CountItems(); ++i)
	{
		if (fChunksArray->ItemAt(i) == insertAfter)
		{
			fChunksArray->InsertItem(insertObject, i+1);
			return;
		}
	}
}
/* removes the chunk from the list of chunks and cleans the hierarchy where the chunk was (or remove the hierarchy if is is not used anymore*/
void TArmeMasterMovie::RemoveChunk(TArmeMasterMovieChunk* chunkToRemove)
{
	// remove from the list
	fChunksArray->RemoveItemByValue(chunkToRemove);
	delete chunkToRemove; // delete the chunk
}

/* removes the hierarchy from the array of hierarchies and delete it*/
void TArmeMasterMovie::RemoveHierarchy(TArmeMasterMovieHierarchyElement* hierarchyToRemove)
{
	fHierarchiesArray->RemoveItemByValue(hierarchyToRemove);
	delete hierarchyToRemove;
}
/* add the hierarchy in the array of hierarchy*/
void TArmeMasterMovie::AddHierarchy(TArmeMasterMovieHierarchyElement* hierarchyToAdd)
{
	fHierarchiesArray->AddItem(hierarchyToAdd);
}
/*
Finds a hierarchy that can host the current chunk.
If no hierarchies has been found, create a new one
*/
TArmeMasterMovieHierarchyElement* TArmeMasterMovie::GetHierarchyForChunk(TArmeMasterMovieChunk* chunk)
{
	TArmeMasterMovieHierarchyElement* ret = NULL;
	
	ZzAssert(chunk);
	//try to fin a hierarchy that can host the current chunk
	for (int i=0; i < fHierarchiesArray->CountItems(); ++i)
	{
		if (fHierarchiesArray->ItemAt(i)->CanHostChunk(chunk->fTopChunk))
		{
			ret = fHierarchiesArray->ItemAt(i);
			ret->AddRef();  //the chunk will have a reference on this hierarchy
			chunk->fHierarchy = ret;
			return ret;
		}
	}

	verntl(videoAlain, 4, "GetHierarchy, create new hierarchy for chunk from " << chunk->fStartTime << " to " << chunk->fEndTime);

	//if no hierarchies has been found, create a new one
	ret = CreateNewHierarchyForChunk(chunk->fTopChunk);
	if (ret)  //it can be NULL if there is only the TopChunk (TLC)
	{
		fHierarchiesArray->AddItem(ret);
		ret->AddRef(); //the chunk will have a reference on this hierarchy
	}
	chunk->fHierarchy = ret;
	
	return ret;
}

/*
Creates a hierarchy that can host the given chunk.
*/
TArmeMasterMovieHierarchyElement* TArmeMasterMovie::CreateNewHierarchyForChunk(TArmeMasterMovieChunkHierarchyElement* chunkElement)
{
	if (chunkElement == NULL)
		return NULL;
		
	SInt32 objectType = chunkElement->fVideoObject->GetType();
		
	//if the chunkElement is a container (group), it is not represented on any track; so we don't create a hierarchyElement for it
	if((objectType == TArmeWSObj::kContainer) || (objectType == TArmeWSObj::kTLC)||(objectType==TArmeWSObj::kAudioProcessor))
	{
		//group always have only one children (if they are more than one childre, they are children of a transition that is the only child of the group)
		ZzAssert(chunkElement->fSourceB == NULL);
		return CreateNewHierarchyForChunk(chunkElement->fSourceA);
	}
	
	// create the hierarchyElement recursively
	return new TArmeMasterMovieHierarchyElement(this, CreateNewHierarchyForChunk(chunkElement->fSourceA), CreateNewHierarchyForChunk(chunkElement->fSourceB), chunkElement->GetType());
};

/*
Adds a reference on the object and its children
*/
void RecursiveAddRef(TVideoObject* obj)
{
	obj->AddRef();
	for (int i=0; i < obj->GetChildrenCount(); ++i)
	{
		TVideoObject* videoChild = dynamic_cast<TVideoObject*>(obj->GetChild(i));
		if (videoChild)
			RecursiveAddRef(videoChild);
	}
}

/*
Release a reference on the object and its children
*/
void RecursiveReleaseRef(TVideoObject* obj)
{
	obj->Release();
	for (int i=0; i < obj->GetChildrenCount(); ++i)
	{
		TVideoObject* videoChild = dynamic_cast<TVideoObject*>(obj->GetChild(i));
		if (videoChild)
			RecursiveReleaseRef(videoChild);
	}
}


/*
Adds a video object in the master movie.
For every chunk that are covered by this object :
- add it in the hierarchy (it will create the transitions where needed)
*/
void TArmeMasterMovie::AddSubMovie(TVideoObject* obj)
{
	if (!obj->ContainsVideo(false, true))  // actually, an audio processor is an audio object !! so this test is very useful
		return;

	verntl(videoAlain, 3, "TArmeMasterMovie::AddSubMovie(" << obj->GetName() << ") from " << obj->GetAbsoluteStartTimeOfVideoClip(false) << " to " << obj->GetAbsoluteEndxTimeOfVideoClip(false));

	// the object can not be in the hierarchy already (we check that it is not referenced by any chunk)
	ZzAssert(!obj->IsInMasterMovie());
	
	UpdateSubMovie(obj);
}

/*
Updated the representation of an object in the master movie. (i.e : a video effect that has been re-configured or a video transition).
*/
void TArmeMasterMovie::UpdateSubMovie(TVideoObject* obj)
{
	TArmeMasterMovieChunk* chunk = NULL;
	
	/*$$$TEMP*/
/*	ArmeTime splitTime;
	for (int i = 0; i < fSplitsArray->CountItems()-1; ++i)
	{
		splitTime = fSplitsArray->ItemAt(i);
	}*/
	
	
	verntl(videoAlain, 3, "TArmeMasterMovie::UpdateSubMovie(" << obj->GetName() << ") from " << obj->GetAbsoluteStartTimeOfVideoClip(false) << " to " << obj->GetAbsoluteEndxTimeOfVideoClip(false) << ")");

	// add the splits at the beginning and at the end of the object we insert in the mastermovie (here, we consider only the video objects that are visible)
	ZzArray<ArmeTime>	listOfSplit;
	obj->AddListOfStartAndEndTimeOfVideoClip(obj->GetAbsoluteStartTimeOfVideoClip(false), obj->GetAbsoluteEndxTimeOfVideoClip(false), listOfSplit, false);
	for (int i = 0; i < listOfSplit.CountItems(); ++i)
	{
		AddSplit(listOfSplit.ItemAt(i));
	}

	//determine the time range that is covered by the object and that must be updated
	ArmeTime startTime = ARME_TIME_UNDEF;
	ArmeTime endTime = ARME_TIME_UNDEF;
	
	// find the first chunk covered by the object
	if (obj->GetType() == TArmeWSObj::kVideoXFadeEffect)
	{
		startTime = obj->GetAbsoluteStartTime();
		endTime = obj->GetAbsoluteEndxTime();
		
		chunk = GetChunkStartingAtTime(startTime);
	}
	else
	{
		startTime = obj->GetAbsoluteStartTimeOfVideoClip(true);
		endTime = obj->GetAbsoluteEndxTimeOfVideoClip(true);
		chunk = GetChunkStartingAtTime(startTime);

		if ((obj->GetAbsoluteStartTimeOfVideoClip(false) < startTime) || !chunk)
		{
			startTime = obj->GetAbsoluteStartTimeOfVideoClip(false);
			if (GetChunkStartingAtTime(startTime))
				chunk = GetChunkStartingAtTime(startTime);
		}
		if (obj->GetAbsoluteEndxTimeOfVideoClip(false) > endTime)
			endTime = obj->GetAbsoluteEndxTimeOfVideoClip(false);
	}

	//if there is now video at all in the object, return directly
	if ((startTime < ARME_TIME_ZERO) || (startTime == ARME_TIME_UNDEF) || (startTime == ARME_TIME_MAX))
		return;

	ZzAssert(chunk);
	int chunkIndex = fChunksArray->FindIndexForItem(chunk);

	// go through all the chunks that are covered by the object
	while (chunk->fEndTime <= endTime)
	{
		// we are going to remove and add an object in the chunk hierarchy.  So, the chunk will use another hierarchy as the one it is using right now;
		// so we can clean the chunk hierarhy
		delete chunk->fTopChunk;
		chunk->RemoveAllTextClips();

		// and clean the track hierarchy that was used by the chunk
		if (chunk->fHierarchy)
		{
			verntl(videoAlain, 4, "TArmeMasterMovie::UpdateSubMovie ("<<obj->GetName()<<") : Cleaning Old Hierarchy from " << chunk->fStartTime << " to " << chunk->fEndTime);

			chunk->fHierarchy->Clean(chunk->fStartTime, chunk->fEndTime);
			chunk->fHierarchy->Release();
		}

		// the chunk hierarchy has been emptied.  Now, we need to add all the objects that contains video during the chunk.
		chunk->InsertPieceOfVideoObject(obj->GetDoc()->GetTopContainer(), NULL);
		
		//find a track structure in which we can insert this chunk
		chunk->fHierarchy = chunk->fMasterMovie->GetHierarchyForChunk(chunk);
		
		verntl(videoAlain, 4, "TArmeMasterMovie::UpdateSubMovie ("<<obj->GetName()<<") : Copy in new hierarchy from " << chunk->fStartTime << " to " << chunk->fEndTime);
		//copy the media into the tracks
		if (chunk->fHierarchy) 
			chunk->fHierarchy->CopyChunk(chunk->fTopChunk);
		
		//next chunk
		chunk = fChunksArray->ItemAt(++chunkIndex);
	}
	
	//check all the splits that are currently used by this object (including the muted children).  If they are not useful anymore, they will be deleted
	ZzArray<ArmeTime>	listOfSplitToRemove;
	obj->AddListOfStartAndEndTimeOfVideoClip(obj->GetAbsoluteStartTimeOfVideoClip(true), obj->GetAbsoluteEndxTimeOfVideoClip(true), listOfSplitToRemove, true);
	for (int i = 0; i < listOfSplitToRemove.CountItems(); ++i)
	{
		RemoveSplit(listOfSplitToRemove.ItemAt(i));
	}
}

void  TArmeMasterMovie::UpdateFile() const
{
	verntl(dataFileRef, 4, "Saving WorkingMaster.mov");

	//export master movie (the working movie) (including all disabled tracks)
	//update the movie 
	OSErr err = OpenMovieFile(&fsSpec, (short int*)&fResourceRefNum, fsRdWrPerm);
	ThrowOnOSStatus(err);
	verntl(dataFileRef, 5, "Updating Resources in WorkingMaster.mov");
    err = UpdateMovieResource(fQTMovie, fResourceRefNum, fResourceId, NULL);
	verntl(dataFileRef, 5, "Closing WorkingMaster.mov");
	ThrowOnOSStatus(err);
	CloseMovieFile(fResourceRefNum);
}

/*
Removes a video object from the master movie.
- removes if from all the chunks where it is, remove the transition it was involved in.
*/
void TArmeMasterMovie::RemoveSubMovie(TVideoObject* obj)
{
	if (!obj->ContainsVideo(false, true))  // actually, an audio processor is a video object !! so this test is very useful
		return;

	//if the object was not yet in the master movie, don't even try to remove it (it happens the first time you call _SetPos() on the object)
	if (!obj->IsInMasterMovie())
		return;

	// we put it out of hierarchy artificially so ContainsVideo method will return false
	TArmeWSObj* oldParent = obj->GetPos().fParent;
	int indexInParentChildren = 0;
	if (oldParent)
	{
		indexInParentChildren = oldParent->_GetIndexOfChild(obj);
		oldParent->_RemoveChild(obj);  
	}

	verntl(videoAlain, 3, "TArmeMasterMovie::RemoveSubMovie(" << obj->GetName() << ") from " << obj->GetAbsoluteStartTimeOfVideoClip(false) << " to " << obj->GetAbsoluteEndxTimeOfVideoClip(false) << ") object is referenced " << obj->GetRefCount() << " times");
	
	UpdateSubMovie(obj);

	// the object has been artificially removed from its parent at the beginning of the function.
	// we need to add it again.  Note : it will probably be removed soon in a nother function
	if (oldParent)
	{
		oldParent->_InsertChild(obj, indexInParentChildren);
	}
}

void TArmeMasterMovie::RebuildMasterMovie()
{
	return;

	for (int i=0; i < fChunksArray->CountItems(); ++i)
	{
		delete fChunksArray->ItemAt(i);
	}
	fChunksArray->RemoveAll();
	fSplitsArray->RemoveAll();
	
	fSplitsArray->AddItem(ARME_TIME_ZERO);
	fSplitsArray->AddItem(GetDoc()->SecondsToArmeTime(MASTER_MOVIE_MAX_DURATION_SEC));
	fChunksArray->AddItem(new TArmeMasterMovieChunk(this, ARME_TIME_ZERO, GetDoc()->SecondsToArmeTime(MASTER_MOVIE_MAX_DURATION_SEC)));
	fChunksArray->ItemAt(0)->fTopChunk = new TArmeMasterMovieChunkHierarchyElement(GetDoc()->GetTopContainer(), NULL, fChunksArray->ItemAt(0), NULL);

	UpdateSubMovie(GetDoc()->GetTopContainer());
}

PicHandle TArmeMasterMovie::GetPictAtTime(TVideoObject* obj, TVideoObject* inEffect, ArmeTime time)
{
	PicHandle pict = NULL;
	TArmeMasterMovieChunk* chunk = NULL;
	
	if (!obj->ContainsVideo(false, false))
		return NULL;
	
	if (!obj->IsInMasterMovie())
		return NULL;

/*	TArmeMasterMovie* tempMasterMovie = new TArmeMasterMovie(fDoc, GetWorkingCodecType(), "Temp.Mov");
//	tempMasterMovie->UpdateSubMovie(obj);
	
//	chunk = tempMasterMovie->GetChunkStartingAtTime(time);
//	pict = GetTrackPict(chunk->fHierarchy->GetMasterMovieTrack()->GetQTTrack(), fDoc->TimeValueFromArmeTime(time));
	//tempMasterMovie->RemoveSubMovie(obj);
	delete tempMasterMovie;
	
	return pict;*/

	chunk = GetChunkStartingAtTime(time);
	ZzAssert(chunk != 0);

//	TArmeMasterMovieChunkHierarchyElement* backupCopy = chunk->fTopChunk->GetCopyOfHierarchy(chunk);

	//REPLACE CURRENT CHUNK BY A CHUNK CONTAINING ONLY THE CLIP WE WANT TO SEE

	// we are going to remove and add an object in the chunk hierarchy.  So, the chunk will use another hierarchy as the one it is using right now;
	// so we can clean the chunk hierarhy
//	delete chunk->fTopChunk;

	// and clean the track hierarchy that was used by the chunk
	if (chunk->fHierarchy)
	{
		verntl(videoAlain, 4, "TArmeMasterMovie::GetPictAtTime ("<<obj->GetName()<<") : Cleaning Old Hierarchy from " << chunk->fStartTime << " to " << chunk->fEndTime);

		chunk->fHierarchy->Clean(chunk->fStartTime, chunk->fEndTime);
		chunk->fHierarchy->Release();
	}

	// the chunk hierarchy has been emptied.  Now, we need to add all the objects that contains video during the chunk.
//	chunk->InsertPieceOfVideoObject(obj->GetDoc()->GetTopContainer(), NULL);
	
	TArmeMasterMovieChunkHierarchyElement* objElement = chunk->fTopChunk->FindChunkHierarchyElementHosting(obj);
	TArmeMasterMovieChunkHierarchyElement* effectElement = chunk->fTopChunk->FindChunkHierarchyElementHosting(inEffect); 
	TArmeMasterMovieChunkHierarchyElement* tempObjElement = objElement->fParent;
	
	// find the child of effectElement that hosts objElement (there might be some transitions between objElement and effectElement)
	if (effectElement)
	{
		while (tempObjElement != effectElement)
		{
			if (objElement->fParent == NULL)
				break;
			objElement = objElement->fParent;
			tempObjElement = objElement->fParent;
		}
	}
	
	//make a copy of objElement and its children
	TArmeMasterMovieChunkHierarchyElement* objHierarchy = objElement->GetCopyOfHierarchy(chunk);
	
	// remove everything except the top chunk and add the objElement and its children right below the top chunk
	delete chunk->fTopChunk->fSourceA;
	chunk->fTopChunk->fSourceA = objHierarchy;
	
	//find a track structure in which we can insert this chunk
	chunk->fHierarchy = chunk->fMasterMovie->GetHierarchyForChunk(chunk);
	
	verntl(videoAlain, 4, "TArmeMasterMovie::GetPictAtTime ("<<obj->GetName()<<") : Copy in new hierarchy from " << chunk->fStartTime << " to " << chunk->fEndTime);
	//copy the media into the tracks
	if (chunk->fHierarchy) 
		chunk->fHierarchy->CopyChunk(chunk->fTopChunk);
		
	pict = GetTrackPict(chunk->fHierarchy->GetMasterMovieTrack()->GetQTTrack(), fDoc->TimeValueFromArmeTime(time));
	
	//PUT BACK THE PREVIOUS CHUNK 
	// we are going to remove and add an object in the chunk hierarchy.  So, the chunk will use another hierarchy as the one it is using right now;
	// so we can clean the chunk hierarhy
	delete chunk->fTopChunk;
	chunk->RemoveAllTextClips();

	// and clean the track hierarchy that was used by the chunk
	if (chunk->fHierarchy)
	{
		verntl(videoAlain, 4, "TArmeMasterMovie::GetPictAtTime ("<<obj->GetName()<<") : Cleaning Old Hierarchy from " << chunk->fStartTime << " to " << chunk->fEndTime);

		chunk->fHierarchy->Clean(chunk->fStartTime, chunk->fEndTime);
		chunk->fHierarchy->Release();
	}

	chunk->InsertPieceOfVideoObject(obj->GetDoc()->GetTopContainer(), NULL);
	
	//find a track structure in which we can insert this chunk
	chunk->fHierarchy = chunk->fMasterMovie->GetHierarchyForChunk(chunk);
	
	verntl(videoAlain, 4, "TArmeMasterMovie::GetPictAtTime ("<<obj->GetName()<<") : Copy in new hierarchy from " << chunk->fStartTime << " to " << chunk->fEndTime);
	//copy the media into the tracks
	if (chunk->fHierarchy) 
		chunk->fHierarchy->CopyChunk(chunk->fTopChunk);
	
	return pict;
}


/*
PicHandle TArmeMasterMovie::GetPictAtTime(TVideoObject* obj, ArmeTime time)
{
	if (!obj->ContainsVideo())
		return NULL;
	
	if (!obj->IsInMasterMovie())
		return NULL;
		
	TArmeMasterMovieChunk* chunk = GetChunkStartingAtTime(time);
	PicHandle pict = NULL;
	ZzAssert(chunk != 0);

	TArmeMasterMovieChunkHierarchyElement* backupCopy = chunk->fTopChunk->GetCopyOfHierarchy(chunk);

	//REPLACE CURRENT CHUNK BY A CHUNK CONTAINING ONLY THE CLIP WE WANT TO SEE

	// we are going to remove and add an object in the chunk hierarchy.  So, the chunk will use another hierarchy as the one it is using right now;
	// so we can clean the chunk hierarhy
	delete chunk->fTopChunk;

	// and clean the track hierarchy that was used by the chunk
	if (chunk->fHierarchy)
	{
		verntl(videoAlain, 4, "TArmeMasterMovie::GetPictAtTime ("<<obj->GetName()<<") : Cleaning Old Hierarchy from " << chunk->fStartTime << " to " << chunk->fEndTime);

		chunk->fHierarchy->Clean(chunk->fStartTime, chunk->fEndTime);
		chunk->fHierarchy->Release();
	}

	// the chunk hierarchy has been emptied.  Now, we need to add all the objects that contains video during the chunk.
	chunk->InsertPieceOfVideoObject(obj->GetPos().fParent, NULL);
	
	//find a track structure in which we can insert this chunk
	chunk->fHierarchy = chunk->fMasterMovie->GetHierarchyForChunk(chunk);
	
	verntl(videoAlain, 4, "TArmeMasterMovie::GetPictAtTime ("<<obj->GetName()<<") : Copy in new hierarchy from " << chunk->fStartTime << " to " << chunk->fEndTime);
	//copy the media into the tracks
	if (chunk->fHierarchy) 
		chunk->fHierarchy->CopyChunk(chunk->fTopChunk->fSourceA);
		
	pict = GetTrackPict(chunk->fHierarchy->GetMasterMovieTrack()->GetQTTrack(), fDoc->TimeValueFromArmeTime(time));
	
	//PUT BACK THE PREVIOUS CHUNK 
	// we are going to remove and add an object in the chunk hierarchy.  So, the chunk will use another hierarchy as the one it is using right now;
	// so we can clean the chunk hierarhy
	delete chunk->fTopChunk;

	// and clean the track hierarchy that was used by the chunk
	if (chunk->fHierarchy)
	{
		verntl(videoAlain, 4, "TArmeMasterMovie::GetPictAtTime ("<<obj->GetName()<<") : Cleaning Old Hierarchy from " << chunk->fStartTime << " to " << chunk->fEndTime);

		chunk->fHierarchy->Clean(chunk->fStartTime, chunk->fEndTime);
		chunk->fHierarchy->Release();
	}

	chunk->fTopChunk = backupCopy;
	
	//find a track structure in which we can insert this chunk
	chunk->fHierarchy = chunk->fMasterMovie->GetHierarchyForChunk(chunk);
	
	verntl(videoAlain, 4, "TArmeMasterMovie::GetPictAtTime ("<<obj->GetName()<<") : Copy in new hierarchy from " << chunk->fStartTime << " to " << chunk->fEndTime);
	//copy the media into the tracks
	if (chunk->fHierarchy) 
		chunk->fHierarchy->CopyChunk(chunk->fTopChunk);
	
	return pict;
}
*/

/*
Removes the content of the track during the given time range
*/
void TArmeMasterMovie::CopyBackgroundInTrack(Track destTrack, ArmeTime startTime, ArmeTime endTime)
{
	verntl(videoAlain, 5, "CopyBackgroundInTrack(" << destTrack << ", from " << startTime << " to " << endTime << ")");

	TimeValue duration = GetDoc()->TimeValueFromArmeTime(endTime - startTime);
	TimeValue startValue = GetDoc()->TimeValueFromArmeTime(startTime);
	if (duration > 0)
	{
		//first delete the track where the chunk will be copied
		OSErr res = DeleteTrackSegment(destTrack, startValue, duration);
		ThrowOnOSStatus(res);
	
		//insert the background at that place (an empty track)
		res = InsertEmptyTrackSegment(destTrack, startValue, duration);
		ThrowOnOSStatus(res);
	}
}


TArmeMasterMovieOriginalTrack::TArmeMasterMovieOriginalTrack(TArmeMasterMovie* masterMovie)
{
	Track qtTrack = CreateNewTrackInMovie(masterMovie->GetQTMovie(), masterMovie->GetVideoTimeScale(), masterMovie->GetWidth(), masterMovie->GetHeight(), VideoMediaType);
	fTrackID = GetTrackID(qtTrack);

	SetTrackEnabled(qtTrack, false);  // the original track is not visible
	SetTrackLayer(qtTrack, 2000);

	fFileStartTimeArray = new ZzArray<TimeValue>(32);
	fFileDurationArray = new ZzArray<TimeValue>(32);
	fFileCachedThumbs = new ZzArray<ZzArray<PicHandle>*>(32);
	fFileCachedThumbsTime = new ZzArray<ZzArray<TimeValue>*>(32);

	fIndexArray = new ZzArray<SInt32>(32);
	fCurrentLength = 0;
	fMasterMovie = masterMovie;

	// this track is used for the thumbnails, it is always low quality
	SetMediaPlayHints(GetTrackMedia(GetQTTrack()), 0, hintsHighQuality);
}

TArmeMasterMovieOriginalTrack::~TArmeMasterMovieOriginalTrack()
{
	delete fFileStartTimeArray;
	delete fFileDurationArray;
	delete fIndexArray;
	
	for (int i=0; i < fFileCachedThumbs->CountItems(); ++i)
	{
		for (int j=0; j < fFileCachedThumbs->ItemAt(i)->CountItems(); ++j)
		{
			KillPicture(fFileCachedThumbs->ItemAt(i)->ItemAt(j));
		}
		delete fFileCachedThumbs->ItemAt(i);
		delete fFileCachedThumbsTime->ItemAt(i);
	}
	DisposeMovieTrack(GetQTTrack());
}

TArmeMasterMovieOriginalTrack::TArmeMasterMovieOriginalTrack(ArmeArchiveHandler* unarchiver) : ArmeArchivable(unarchiver)
{
	fTrackID = unarchiver->ReadSInt32();

	SInt32 arrayCount = unarchiver->ReadSInt32();
	fFileStartTimeArray = new ZzArray<TimeValue>(arrayCount);
	fFileDurationArray = new ZzArray<TimeValue>(arrayCount);
	fFileCachedThumbs = new ZzArray<ZzArray<PicHandle>*>(arrayCount); // not serialized
	fFileCachedThumbsTime = new ZzArray<ZzArray<TimeValue>*>(arrayCount); // not serialized

	
	if (unarchiver->GetFileVer() >= 0x00010018)
	{
		for (int i=0; i < arrayCount; i++)
		{
			fFileStartTimeArray->AddItem(unarchiver->ReadSInt32());
			fFileDurationArray->AddItem(unarchiver->ReadSInt32());
			fFileCachedThumbs->AddItem(new ZzArray<PicHandle>(32));  //not serialized
			fFileCachedThumbsTime->AddItem(new ZzArray<TimeValue>(32)); // not serialized
		}
		
		fMasterMovie = dynamic_cast<TArmeMasterMovie*>(unarchiver->ReadCPPObject());
		fCurrentLength = unarchiver->ReadSInt32();
	}
	else
	{
		// in previous version, everythin was saved in ArmeTime instead of TimeValue !
		ZzArray<ArmeTime>* armeTimeFileStartTimeArray = new ZzArray<ArmeTime>(arrayCount);
		ZzArray<ArmeTime>* armeTimeFileDurationArray = new ZzArray<ArmeTime>(arrayCount);

		for (int i=0; i < arrayCount; i++)
		{
			armeTimeFileStartTimeArray->AddItem(unarchiver->ReadArmeTime());
			armeTimeFileDurationArray->AddItem(unarchiver->ReadArmeTime());
		}
		
		fMasterMovie = dynamic_cast<TArmeMasterMovie*>(unarchiver->ReadCPPObject());
		ArmeTime armeTimeCurrentLength = unarchiver->ReadArmeTime();

		for (int i=0; i < arrayCount; i++)
		{
			fFileStartTimeArray->AddItem(fMasterMovie->GetDoc()->TimeValueFromArmeTime(armeTimeFileStartTimeArray->ItemAt(i)));
			fFileDurationArray->AddItem(fMasterMovie->GetDoc()->TimeValueFromArmeTime(armeTimeFileDurationArray->ItemAt(i)));		
		}
		
		delete armeTimeFileStartTimeArray;
		delete armeTimeFileDurationArray;

		fCurrentLength = fMasterMovie->GetDoc()->TimeValueFromArmeTime(armeTimeCurrentLength);
	}

	if (unarchiver->GetFileVer() >= 0x00010019)
	{
		SInt32 count = unarchiver->ReadSInt32();
		fIndexArray = new ZzArray<SInt32>(count);
		for (int i=0; i < count; i++)
		{
			fIndexArray->AddItem(unarchiver->ReadSInt32());
		}
	}
	else
	{
		fIndexArray = new ZzArray<SInt32>(arrayCount);
		for (int i=0; i < arrayCount; i++)
		{
			fIndexArray->AddItem(i);
		}
	}

	//retrieve a pointer to the QTTrack
//	fQTTrack = GetMovieTrack(fMasterMovie->GetQTMovie(), trackID);

	// this track is used for the thumbnails, it is always low quality
	SetMediaPlayHints(GetTrackMedia(GetQTTrack()), 0, hintsHighQuality);
}

bool TArmeMasterMovieOriginalTrack::WriteToArchive(ArmeArchiveHandler* archiver) const
{
	archiver->WriteSInt32(fTrackID);

	archiver->WriteSInt32(fFileStartTimeArray->CountItems());
	for (int i=0; i < fFileStartTimeArray->CountItems(); i++)
	{
		archiver->WriteSInt32(fFileStartTimeArray->ItemAt(i));
		archiver->WriteSInt32(fFileDurationArray->ItemAt(i));
	}
	
	archiver->WriteCPPObject(fMasterMovie);
	archiver->WriteSInt32(fCurrentLength);

	//from version 0x00010019
	archiver->WriteSInt32(fIndexArray->CountItems());
	for (int i=0; i < fIndexArray->CountItems(); i++)
	{
		archiver->WriteSInt32(fIndexArray->ItemAt(i));
	}

	return true;
}

Track TArmeMasterMovieOriginalTrack::GetQTTrack()
{
	return GetMovieTrack(fMasterMovie->GetQTMovie(), fTrackID);
}
SInt32 TArmeMasterMovieOriginalTrack::GetIndex(const ArmeVideoDataFileRef* file) const
{
	ZzAssert(fIndexArray->ItemAt(file->fFileIndex) != -1);
	return fIndexArray->ItemAt(file->fFileIndex);
}

/*
When a new video file is imported in the document, it is added to the original track.  All the chunks of the TVideoClip will reference 
pieces of this track.
*/
void TArmeMasterMovieOriginalTrack::AddFileInOriginalTrack(ArmeVideoDataFileRef* file, Movie movie, int trackIndex)
{
	while (fIndexArray->CountItems() < SInt32(file->fFileIndex))
	{
		fIndexArray->AddItem(-1);
	}
	fIndexArray->AddItem(fFileStartTimeArray->CountItems());

	// store the information about the position of the media in the originalTrack
	fFileStartTimeArray->AddItem(fCurrentLength);

	// copy the media (by reference in the original track)
	CopyTrackSegment(fMasterMovie->GetQTMovie(), GetQTTrack(), movie, GetFirstVideoTrack(movie), fCurrentLength, 0, GetMovieDuration(movie));
	TimeValue movieDuration = GetTrackDuration(GetQTTrack()) - fCurrentLength;
	
	fFileDurationArray->AddItem(movieDuration);
	fCurrentLength += movieDuration+1;

	fFileCachedThumbs->AddItem(new ZzArray<PicHandle>(32));
	fFileCachedThumbsTime->AddItem(new ZzArray<TimeValue>(32));
}

/*
When a video file is not referenced anymore, it is removed from the original track.
*/
void TArmeMasterMovieOriginalTrack::RemoveFileFromOriginalTrack(ArmeVideoDataFileRef* file)
{
	//delete the reference on the media
	if (file->fFileIndex >= fFileStartTimeArray->count)  //we have seen some docs where the fFileIndex was invalid so, in case this happens, just ignore this file
		return;
	
	TimeValue startTimeOfFile = fFileStartTimeArray->ItemAt(GetIndex(file));
	TimeValue durationOfFile = fFileDurationArray->ItemAt(GetIndex(file));
	OSErr res = DeleteTrackSegment(GetQTTrack(), startTimeOfFile, durationOfFile);
	ThrowOnOSStatus(res);
	res = InsertEmptyTrackSegment(GetQTTrack(), startTimeOfFile, durationOfFile);
	ThrowOnOSStatus(res);

/*	short movieResId = 0;
	short movieResRefNum = 0;

	OSErr err = OpenMovieFile(&(fMasterMovie->fsSpec), (short int*)&(fMasterMovie->fResourceRefNum), fsRdWrPerm);
	res = RemoveMovieResource(fMasterMovie->fResourceRefNum, movieResId);
	CloseMovieFile(fMasterMovie->fResourceRefNum);*/
	
		
	//offset all remaining startTime after this clip
/*	for (int i = GetIndex(file)+1; i < fFileStartTimeArray->CountItems(); i++)
		fFileStartTimeArray->ItemAt(i) -= durationOfFile; 
	
	fCurrentLength -= durationOfFile;*/
	
	//write wrong values in the cell that is not used anymore
	fFileStartTimeArray->ItemAt(GetIndex(file)) = -1;
	fFileDurationArray->ItemAt(GetIndex(file)) = -1;
	fIndexArray->ItemAt(file->fFileIndex) = -1;
}

PicHandle TArmeMasterMovieOriginalTrack::GetPict(const ArmeVideoDataFileRef* file, ArmeTime localTime)
{
	PicHandle thumb = NULL;
	TimeValue thumbTime = GetStartTimeOfFile(file) + fMasterMovie->GetDoc()->RoughTimeValueFromArmeTime(localTime);
	SInt32 fileIndex = GetIndex(file);
	SInt32 thumbIndex = fFileCachedThumbsTime->ItemAt(fileIndex)->FindIndexForItem(thumbTime);
	
	if (thumbIndex == -1)
	{
		/*CGrafPtr port;
		GDHandle gdh;
		GWorldPtr offscreenGWorld;
		GetMovieGWorld ( fMasterMovie->GetQTMovie(), &port,  &gdh );
		Rect frame = {0, 0, fMasterMovie->GetWidth(), fMasterMovie->GetHeight()};
		fprintf( stderr, "fMasterMovie->GetWidth() = %d, fMasterMovie->GetHeight() = %d\n", fMasterMovie->GetWidth(), fMasterMovie->GetHeight() ) ;
		OSErr err = NewGWorld( &offscreenGWorld, k32ARGBPixelFormat, &frame, NULL, NULL, 0);
		if ( err ) {
			fprintf( stderr, "error = %d\n", err ) ;
		}
		SetMovieGWorld ( fMasterMovie->GetQTMovie(), offscreenGWorld,  NULL );*/
		thumb = GetTrackPict(GetQTTrack(), thumbTime);
		/*SetMovieGWorld ( fMasterMovie->GetQTMovie(), port,  gdh );
		DisposeGWorld( offscreenGWorld ) ;*/
		fFileCachedThumbs->ItemAt(fileIndex)->AddItem(thumb);
		fFileCachedThumbsTime->ItemAt(fileIndex)->AddItem(thumbTime);
	}
	else
	{
		thumb = fFileCachedThumbs->ItemAt(fileIndex)->ItemAt(thumbIndex);
	}

	return thumb;
}

TimeValue TArmeMasterMovieOriginalTrack::GetStartTimeOfFile(const ArmeVideoDataFileRef* file) const
{
	return fFileStartTimeArray->ItemAt(GetIndex(file));
}

TimeValue TArmeMasterMovieOriginalTrack::GetStartTimeOfNextFile(const ArmeVideoDataFileRef* file) const
{
	int indexOfNextFile = file->fFileIndex+1;
	
	
	//find next file
	while ((indexOfNextFile < fFileStartTimeArray->CountItems()) && (fIndexArray->ItemAt(indexOfNextFile) == -1))
	{
		indexOfNextFile++;
	}	
	
	// if this file is the last one in the original track, return its end time;
	if (indexOfNextFile >= fFileStartTimeArray->CountItems())
	{
		return fFileStartTimeArray->ItemAt(GetIndex(file)) + fFileDurationArray->ItemAt(GetIndex(file));
	}
	else
	{
		return fFileStartTimeArray->ItemAt(indexOfNextFile);
	}
}


TArmeMasterMovieTrack::TArmeMasterMovieTrack(TArmeMasterMovie* masterMovie, TArmeMasterMovieTrack* sourceA, TArmeMasterMovieTrack* sourceB, int trackLayer, UInt32 trackType)
{
	verntl(videoAlain, 4, "creating new TArmeMasterMovieTrack (" << this << ") type : " << trackType << " layer : " << trackLayer);

	OSErr res = 0;

	switch (trackType)
	{
		case kVideoMedia :
			fTrackType = kVideoMedia;
			trackType = VideoMediaType;
			break;
		case kTextMedia : 
			fTrackType = kTextMedia;
			trackType = TextMediaType;
			break;
		case kKeyedTextMedia : 
			fTrackType = kKeyedTextMedia;
			trackType = TextMediaType;
			break;
		default :
			ZzAssert(0);
	}


	fMasterMovie = masterMovie;
	fSourceA = sourceA;
	fSourceB = sourceB;
	
	//create the track itself
	//-----------------------
	Track qtTrack = NewMovieTrack(fMasterMovie->GetQTMovie(), fMasterMovie->GetWidth() << 16, fMasterMovie->GetHeight() << 16, kNoVolume);
	fTrackID = GetTrackID(qtTrack);

	//create the media
	//----------------
	Media mediaEffect;
	if (!sourceA && !sourceB)
	{
		//this track is a source track, it can be a textTrack or a videoTrack
		mediaEffect = NewTrackMedia(qtTrack, trackType, fMasterMovie->GetVideoTimeScale(), NULL, 0); 
	}
	// if there are some source tracks, this track is an effect track so we have to create the media for it
	else 
	{
		//this track is an effect track so it is a videoMediaType
		mediaEffect = NewTrackMedia(qtTrack, VideoMediaType, fMasterMovie->GetVideoTimeScale(), NULL, 0); 

		//create the input map
		//--------------------
		
		// create the input map and add references for the first effect track
		QTAtomContainer myInputMap;
		OSType myType;
		
		res = QTNewAtomContainer(&myInputMap);
		ThrowOnOSStatus(res);
		
		QTAtom				myInputAtom;
		long				myRefIndex;
		
		// first track
		res = AddTrackReference(qtTrack, fSourceA->GetQTTrack(), kTrackModifierReference, &myRefIndex);
		ThrowOnOSStatus(res);		
	
		// add a reference atom to the input map
		res = QTInsertChild(myInputMap, kParentAtomIsContainer, kTrackModifierInput, myRefIndex, 0, 0, NULL, &myInputAtom);
		ThrowOnOSStatus(res);		
	
		// add two child atoms to the parent reference atom
		myType = EndianU32_NtoB(kTrackModifierTypeImage);
		res = QTInsertChild(myInputMap, myInputAtom, kTrackModifierType, 1, 0, sizeof(myType), &myType, NULL);
		ThrowOnOSStatus(res);	
	
		myType = EndianU32_NtoB(kSourceOneName);
		res = QTInsertChild(myInputMap, myInputAtom, kEffectDataSourceType, 1, 0, sizeof(myType), &myType, NULL);
		ThrowOnOSStatus(res);	
	
		// SECOND TRACK
		if (fSourceB)
		{
			res = AddTrackReference(qtTrack, fSourceB->GetQTTrack(), kTrackModifierReference, &myRefIndex);
			ThrowOnOSStatus(res);			
			
			// add a reference atom to the input map
			res = QTInsertChild(myInputMap, kParentAtomIsContainer, kTrackModifierInput, myRefIndex, 0, 0, NULL, &myInputAtom);
			ThrowOnOSStatus(res);			
		
			// add two child atoms to the parent reference atom
			myType = EndianU32_NtoB(kTrackModifierTypeImage);
			res = QTInsertChild(myInputMap, myInputAtom, kTrackModifierType, 1, 0, sizeof(myType), &myType, NULL);
		
			myType = EndianU32_NtoB(kSourceTwoName);
			res = QTInsertChild(myInputMap, myInputAtom, kEffectDataSourceType, 1, 0, sizeof(myType), &myType, NULL);
			ThrowOnOSStatus(res);	
		}
		// add the input map to the effects track
		res = SetMediaInputMap(mediaEffect, myInputMap);
		ThrowOnOSStatus(res);
	}
	
	fMasterMovie->CopyBackgroundInTrack(qtTrack, ARME_TIME_ZERO, fMasterMovie->GetDoc()->SecondsToArmeTime(MASTER_MOVIE_MAX_DURATION_SEC));
	
	SetTrackLayer(qtTrack, trackLayer);
}


TArmeMasterMovieTrack::~TArmeMasterMovieTrack()
{
	verntl(videoAlain, 4, "deleting TArmeMasterMovieTrack (" << this << ")");

	DisposeMovieTrack(GetQTTrack());
}

TArmeMasterMovieTrack::TArmeMasterMovieTrack(ArmeArchiveHandler* unarchiver) : ArmeArchivable(unarchiver)
{
	fTrackID = unarchiver->ReadSInt32();
	fMasterMovie = dynamic_cast<TArmeMasterMovie*>(unarchiver->ReadCPPObject());
	fSourceA = dynamic_cast<TArmeMasterMovieTrack*>(unarchiver->ReadCPPObject());
	fSourceB = dynamic_cast<TArmeMasterMovieTrack*>(unarchiver->ReadCPPObject());
	fTrackType = unarchiver->ReadUInt32();

//	fQTTrack = GetMovieTrack(fMasterMovie->GetQTMovie(), trackID);
}
bool TArmeMasterMovieTrack::WriteToArchive(ArmeArchiveHandler* archiver) const
{
	archiver->WriteSInt32(fTrackID);
	archiver->WriteCPPObject(fMasterMovie);
	archiver->WriteCPPObject(fSourceA);
	archiver->WriteCPPObject(fSourceB);
	archiver->WriteUInt32(fTrackType);

	return true;
}

Track TArmeMasterMovieTrack::GetQTTrack() const
{
	return GetMovieTrack(fMasterMovie->GetQTMovie(), fTrackID);
}


TArmeMasterMovieChunk::TArmeMasterMovieChunk(TArmeMasterMovie* masterMovie, ArmeTime startTime, ArmeTime endTime)
{
	verntl(videoAlain, 3, "Creating new TArmeMasterMovieChunk(" << this << ") from " << startTime << " to " << endTime);

	fMasterMovie = masterMovie;
	fTopChunk = NULL;
	fStartTime = startTime;
	fEndTime = endTime;
	fHierarchy = NULL;
	fTextClipsArray = new ZzArray<TArmeTextClip*>(2);
	fTextClipOwnerArray = new ZzArray<TVideoObject*>(2);
}

TArmeMasterMovieChunk::~TArmeMasterMovieChunk()
{
	verntl(videoAlain, 3, "Deleting TArmeMasterMovieChunk(" << this << ") from " << fStartTime << " to " << fEndTime);
	delete fTopChunk;
	for (int i=0; i < fTextClipsArray->CountItems(); ++i)
	{
		fTextClipsArray->ItemAt(i)->Release();
	}

	delete fTextClipsArray;

	for (int i=0; i < fTextClipOwnerArray->CountItems(); ++i)
	{
		fTextClipOwnerArray->ItemAt(i)->Release();
	}

	delete fTextClipOwnerArray;
		
	if (fHierarchy && (fHierarchy->Release() != 0))
	{
		// if the release returned zero, it means that the hierarchy is not used anymore (it has been deleted in the Release)
		// else, we need to clean the hierarchy where the chunk was
		// fHierarchy->Clean(fStartTime, fEndTime);
	}
}

TArmeMasterMovieChunk::TArmeMasterMovieChunk(ArmeArchiveHandler* unarchiver) : ArmeArchivable(unarchiver)
{
	fMasterMovie = dynamic_cast<TArmeMasterMovie*>(unarchiver->ReadCPPObject());
	fTopChunk = dynamic_cast<TArmeMasterMovieChunkHierarchyElement*>(unarchiver->ReadCPPObject());
	fStartTime = unarchiver->ReadArmeTime();
	fEndTime = unarchiver->ReadArmeTime();
	fHierarchy = dynamic_cast<TArmeMasterMovieHierarchyElement*>(unarchiver->ReadCPPObject());

	if (unarchiver->GetFileVer() >= 0x00010025)
	{
		SInt32 count = unarchiver->ReadSInt32();
		fTextClipsArray = new ZzArray<TArmeTextClip*>(count);
		for (int i=0; i < count; ++i)
		{
			fTextClipsArray->AddItem((TArmeTextClip*)(unarchiver->ReadCPPObject()));
		}
	}
	else
	{
		fTextClipsArray = new ZzArray<TArmeTextClip*>(2);
	}

	if (unarchiver->GetFileVer() >= 0x00010028)
	{
		SInt32 count = unarchiver->ReadSInt32();
		fTextClipOwnerArray = new ZzArray<TVideoObject*>(count);
		for (int i=0; i < count; ++i)
		{
			fTextClipOwnerArray->AddItem((TVideoObject*)(unarchiver->ReadCPPObject()));
		}
	}
	else
	{
		fTextClipOwnerArray = new ZzArray<TVideoObject*>(2);
	}
}

bool TArmeMasterMovieChunk::WriteToArchive(ArmeArchiveHandler* archiver) const
{
	archiver->WriteCPPObject(fMasterMovie);
	archiver->WriteCPPObject(fTopChunk);
	archiver->WriteArmeTime(fStartTime);
	archiver->WriteArmeTime(fEndTime);
	archiver->WriteCPPObject(fHierarchy);

	//from 0x00010025
	archiver->WriteSInt32(fTextClipsArray->CountItems());
	for (int i=0; i < fTextClipsArray->CountItems(); ++i)
	{
		archiver->WriteCPPObject(fTextClipsArray->ItemAt(i));
	}

	//from 0x00010028
	archiver->WriteSInt32(fTextClipOwnerArray->CountItems());
	for (int i=0; i < fTextClipOwnerArray->CountItems(); ++i)
	{
		archiver->WriteCPPObject(fTextClipOwnerArray->ItemAt(i));
	}
	
	return true;
}


void TArmeMasterMovieChunk::InsertPieceOfVideoObject(TVideoObject* object, TVideoObject* parentObject)
{
	fTopChunk = new TArmeMasterMovieChunkHierarchyElement(object, parentObject, this, NULL);
	fTopChunk->InsertChildrenOf(object);
}

void TArmeMasterMovieChunk::AddTextClip(TArmeTextClip* textClip)
{
	fMasterMovie->GetTextTrack(fTextClipsArray->CountItems())->AddTextClip(textClip, fStartTime, fEndTime);

	fTextClipsArray->AddItem(textClip);
	textClip->AddRef();
}

void TArmeMasterMovieChunk::AddRefOnTextClipOwner(TVideoObject* container)
{
	fTextClipOwnerArray->AddItem(container);
	container->AddRef();
}

void TArmeMasterMovieChunk::RemoveAllTextClips()
{
	for (int i=0; i < fTextClipsArray->CountItems(); ++i)
	{
		fMasterMovie->GetTextTrack(i)->Clean(fStartTime, fEndTime);
		fTextClipsArray->ItemAt(i)->Release();
	}
	fTextClipsArray->RemoveAll();

	for (int i=0; i < fTextClipOwnerArray->CountItems(); ++i)
	{
		fTextClipOwnerArray->ItemAt(i)->Release();
	}
	fTextClipOwnerArray->RemoveAll();
}

TArmeMasterMovieChunkHierarchyElement* TArmeMasterMovieChunk::FindChunkHierarchyElementHosting(TVideoObject* object)
{
	ZzAssert(fTopChunk);
	return fTopChunk->FindChunkHierarchyElementHosting(object);
}

void TArmeMasterMovieChunkHierarchyElement::InsertChildrenOf(TVideoObject* object)
{
	ZzArray<TVideoObject*> children(3);
	for (int i=0; i < object->GetChildrenCount(); ++i)
	{
		TVideoObject* videoChild = (TVideoObject*)(object->GetChild(i));

		if (videoChild->ContainsVideoDuring(fMasterMovieChunk->fStartTime, fMasterMovieChunk->fEndTime, false, true))
		{
			// if child is a keyed text clip
			if ((videoChild->GetType() == TArmeWSObj::kTextClip) && (((TArmeTextClip*)videoChild)->IsKeyed()))
			{
				fMasterMovieChunk->AddTextClip((TArmeTextClip*)videoChild);
			}
			else if (videoChild->GetType() != TArmeWSObj::kVideoXFadeEffect)
			{
				// if the child contains real video (no keyed text clips)
				if (videoChild->ContainsVideoDuring(fMasterMovieChunk->fStartTime, fMasterMovieChunk->fEndTime, false, false))
				{
					children.AddItem((TVideoObject*)(object->GetChild(i)));
				}
				else // the child contains only keyed text clips
				{
					//go through all its hierarchy until we find the keyed text clip
					InsertChildrenOf(videoChild);
					fMasterMovieChunk->AddRefOnTextClipOwner(videoChild);
				}
			}
		}
	}
	
	//then, sort these children according to their vpos
	ZzArray<TVideoObject*> sortedChildren(3);
	int nbChildren = children.CountItems();
	for (int i=0; i < nbChildren; ++i)
	{
		TVideoObject* highestClip = NULL;
		SInt32 highestVPos = -9999;

		for (int j=0; j < children.CountItems(); ++j)
		{
			TVideoObject* child = children.ItemAt(j);
			if (child->GetVerticalPos() > highestVPos)
			{
				highestVPos = child->GetVerticalPos();
				highestClip = child;
			}
		}
		sortedChildren.AddItem(highestClip);
		children.RemoveItemByValue(highestClip);
	}
	
	//create a new sub-hierarchy with a transition on top and the children below ...
	fSourceA = CreateTransitions(&sortedChildren, object, fMasterMovieChunk);
}


void TArmeMasterMovieChunk::Split(ArmeTime splitTime)
{
	verntl(videoAlain, 4, "(" << this << ")->TArmeMasterMovieChunk::Split(" << splitTime << ") from " << fStartTime << " to " << fEndTime);

	//make sure the split time is not too close to one of the end of the chunk (less than one video frame)
	ZzAssert(splitTime - fStartTime > _ArmeTime(10));
	ZzAssert(fEndTime - splitTime > _ArmeTime(10));

	TArmeMasterMovieChunk* newChunk = new TArmeMasterMovieChunk(fMasterMovie, splitTime, fEndTime);
	fEndTime = splitTime;

	newChunk->fHierarchy = fHierarchy;
	if (fHierarchy)
		fHierarchy->AddRef();

	//copy the hierarchy
	ZzAssert(fTopChunk);
	newChunk->fTopChunk = fTopChunk->GetCopyOfHierarchy(newChunk);

	//copy the textClipsArray
	for (int i=0; i < fTextClipsArray->CountItems(); ++i)
	{
		newChunk->fTextClipsArray->AddItem(fTextClipsArray->ItemAt(i));
		fTextClipsArray->ItemAt(i)->AddRef();
	}

	fMasterMovie->InsertChunkAfter(this, newChunk);

	// when a chunk that contains text clips is split, we need to copy again the media (such that the transition between the two chunks is smooth)
	if ((fHierarchy && fHierarchy->ContainsTextClips()) || fTextClipsArray->CountItems())
	{
		if (fHierarchy)
		{
			fHierarchy->CopyChunk(fTopChunk);
			newChunk->fHierarchy->CopyChunk(newChunk->fTopChunk);
		}

		for (int i=0; i < fTextClipsArray->CountItems(); ++i)
		{
			fMasterMovie->GetTextTrack(i)->AddTextClip(fTextClipsArray->ItemAt(i), fStartTime, fEndTime);
			fMasterMovie->GetTextTrack(i)->AddTextClip(fTextClipsArray->ItemAt(i), newChunk->fStartTime, newChunk->fEndTime);
		}
	}
}

bool TArmeMasterMovieChunk::MergeWith(TArmeMasterMovieChunk* mergeWith)
{
	//if the hierarchies are the same, the two chunks can be merged
	if (TArmeMasterMovieChunkHierarchyElement::CompareHierarchy(fTopChunk, mergeWith->fTopChunk) && CompareTextClips(mergeWith))
	{
		verntl(videoAlain, 4, "(" << this << ")->TArmeMasterMovieChunk::MergeWith(" << mergeWith << ") from " << fStartTime << " - " << fEndTime << " to " << mergeWith->fEndTime);

		fEndTime = mergeWith->fEndTime;
	
		fMasterMovie->RemoveChunk(mergeWith);

		//ensure that all the data contained in both chunks are now copied into the tracks
/*		if (fHierarchy) //NULL if there is only the TLC
		{
			fHierarchy->CopyChunk(fTopChunk);
		}*/
		
		return true;
	}
	else
		return false;
}

bool TArmeMasterMovieChunk::CompareTextClips(TArmeMasterMovieChunk* chunk)
{
	int count1 = fTextClipsArray->CountItems();
	int count2 = chunk->fTextClipsArray->CountItems();

	if (count1 != count2)
		return false;
		
	for (int i=0; i < fTextClipsArray->CountItems(); ++i)
	{
		TArmeTextClip* text1 = fTextClipsArray->ItemAt(i);
		TArmeTextClip* text2 = chunk->fTextClipsArray->ItemAt(i);

		if (text1 != text2)
			return false;
	}

	return true;
}


TArmeMasterMovieChunkHierarchyElement::TArmeMasterMovieChunkHierarchyElement(TVideoObject* videoObject, TVideoObject* parentObject, TArmeMasterMovieChunk* masterMovieChunk, TArmeMasterMovieChunkHierarchyElement* parent)
{
	fVideoObject = videoObject;
	if (fVideoObject)
		fVideoObject->AddRef();
	fParentObject = parentObject;
	fParent = parent;
	fMasterMovieChunk = masterMovieChunk;
	fSourceA = NULL;
	fSourceB = NULL;
}

TArmeMasterMovieChunkHierarchyElement::~TArmeMasterMovieChunkHierarchyElement()
{
	//delete all childrens recursively
	delete fSourceA;
	delete fSourceB;
	
	// the parent has a reference on this element, remove it
	if (fParent && fParent->fSourceA == this)
		fParent->fSourceA = NULL;
	if (fParent && fParent->fSourceB == this)
		fParent->fSourceB = NULL;

	// if a chunk has a reference on this element, remove it
	if (fMasterMovieChunk->fTopChunk == this)
		fMasterMovieChunk->fTopChunk = NULL;

	if (fVideoObject)
		fVideoObject->Release();
}


UInt32 TArmeMasterMovieChunkHierarchyElement::GetType() const
{
	switch (fVideoObject->GetType())
	{
		case TArmeWSObj::kVideoClip :
		case TArmeWSObj::kVideoEffect :
		case TArmeWSObj::kVideoXFadeEffect :
			return kVideoMedia;
			break;
		case TArmeWSObj::kTextClip :
		{
			if (fVideoObject->IsVideoObject())
				return kTextMedia;
			else
				return kKeyedTextMedia;
		};break;
		default :
		{
			printf("ABOUT TO CRASH AT LINE 1115 IN TArmeMasterMovieChunkHierarchyElement::GetType()\n");
			ZzAssert(0);
		}
	};
	return 0xFFFF;
}


TArmeMasterMovieChunkHierarchyElement::TArmeMasterMovieChunkHierarchyElement(ArmeArchiveHandler* unarchiver) : ArmeArchivable(unarchiver)
{
	fVideoObject = dynamic_cast<TVideoObject*>(unarchiver->ReadCPPObject());
	fParentObject = dynamic_cast<TVideoObject*>(unarchiver->ReadCPPObject());
	fParent = dynamic_cast<TArmeMasterMovieChunkHierarchyElement*>(unarchiver->ReadCPPObject());
	fMasterMovieChunk = dynamic_cast<TArmeMasterMovieChunk*>(unarchiver->ReadCPPObject());
	fSourceA = dynamic_cast<TArmeMasterMovieChunkHierarchyElement*>(unarchiver->ReadCPPObject());
	fSourceB = dynamic_cast<TArmeMasterMovieChunkHierarchyElement*>(unarchiver->ReadCPPObject());
}

bool TArmeMasterMovieChunkHierarchyElement::WriteToArchive(ArmeArchiveHandler* archiver) const
{
	archiver->WriteCPPObject(fVideoObject);
	archiver->WriteCPPObject(fParentObject);
	archiver->WriteCPPObject(fParent);
	archiver->WriteCPPObject(fMasterMovieChunk);
	archiver->WriteCPPObject(fSourceA);
	archiver->WriteCPPObject(fSourceB);

	return true;
}


TArmeMasterMovieChunkHierarchyElement* TArmeMasterMovieChunkHierarchyElement::CreateTransitions(ZzArray<TVideoObject*>* sortedChildren, TVideoObject* parentObject, TArmeMasterMovieChunk* masterMovieChunk)
{
	if (sortedChildren->CountItems() == 0)
	{
		return NULL;
	}
	else if (sortedChildren->CountItems() == 1)
	{
		TArmeMasterMovieChunkHierarchyElement* ret = new TArmeMasterMovieChunkHierarchyElement(sortedChildren->ItemAt(0), parentObject, masterMovieChunk, this);
		ret->InsertChildrenOf(sortedChildren->ItemAt(0));
		return ret;
	}
	else if (sortedChildren->CountItems() == 2)
	{
		TVideoObject* transition = parentObject->GetTransitionFor(sortedChildren->ItemAt(0), sortedChildren->ItemAt(1));
		TArmeMasterMovieChunkHierarchyElement* transitionChunk = new TArmeMasterMovieChunkHierarchyElement(transition, parentObject, masterMovieChunk, this);
		TArmeMasterMovieChunkHierarchyElement* sourceA = new TArmeMasterMovieChunkHierarchyElement(sortedChildren->ItemAt(0), parentObject, masterMovieChunk, transitionChunk);
		TArmeMasterMovieChunkHierarchyElement* sourceB = new TArmeMasterMovieChunkHierarchyElement(sortedChildren->ItemAt(1), parentObject, masterMovieChunk, transitionChunk);
		sourceA->InsertChildrenOf(sortedChildren->ItemAt(0));
		sourceB->InsertChildrenOf(sortedChildren->ItemAt(1));
		
		transitionChunk->fSourceA = sourceA;
		transitionChunk->fSourceB = sourceB;
		
		return transitionChunk;
	}
	else
	{
		//divide the children in two half
		ZzArray<TVideoObject*> firstHalf(2);
		ZzArray<TVideoObject*> secondHalf(2);	
		
		for (int i=0; i < sortedChildren->CountItems()/2; ++i)
		{
			firstHalf.AddItem(sortedChildren->ItemAt(i));
		}
		for (int i=sortedChildren->CountItems()/2; i < sortedChildren->CountItems(); ++i)
		{
			secondHalf.AddItem(sortedChildren->ItemAt(i));
		}

		//build the transitions for the two half of the children
		TArmeMasterMovieChunkHierarchyElement* sourceA = CreateTransitions(&firstHalf, parentObject, masterMovieChunk);
		TArmeMasterMovieChunkHierarchyElement* sourceB = CreateTransitions(&secondHalf, parentObject, masterMovieChunk);

		//and build the transition on top of them
		TVideoObject* transition = parentObject->GetTransitionFor(sourceA->fVideoObject, sourceB->fVideoObject);
		TArmeMasterMovieChunkHierarchyElement* transitionChunk = new TArmeMasterMovieChunkHierarchyElement(transition, parentObject, masterMovieChunk, this);
		transitionChunk->fSourceA = sourceA;
		transitionChunk->fSourceB = sourceB;
		
		return transitionChunk;
	}
}

TArmeMasterMovieChunkHierarchyElement*	TArmeMasterMovieChunkHierarchyElement::GetCopyOfHierarchy(TArmeMasterMovieChunk* inMasterMovieChunk) const
{
	TArmeMasterMovieChunkHierarchyElement* sourceACopy = NULL;
	TArmeMasterMovieChunkHierarchyElement* sourceBCopy = NULL;
	TArmeMasterMovieChunkHierarchyElement* ret = NULL;
	
	if (fSourceA)
		sourceACopy = fSourceA->GetCopyOfHierarchy(inMasterMovieChunk);
	if (fSourceB)
		sourceBCopy = fSourceB->GetCopyOfHierarchy(inMasterMovieChunk);
		
	ret = new TArmeMasterMovieChunkHierarchyElement(fVideoObject, fParentObject, inMasterMovieChunk, /*fMasterMovieTrack,*/ NULL/*, false*/);
	
	if (sourceACopy)
	{
		sourceACopy->fParent = ret;
		ret->fSourceA = sourceACopy;
	}
	if (sourceBCopy)
	{
		sourceBCopy->fParent = ret;
		ret->fSourceB = sourceBCopy;
	}
	
	return ret;
}

bool TArmeMasterMovieChunkHierarchyElement::CompareHierarchy(const TArmeMasterMovieChunkHierarchyElement* arg1, const TArmeMasterMovieChunkHierarchyElement* arg2)
{
	bool bSourceAHierarchyIdentical = false;
	bool bSourceBHierarchyIdentical = false;
	
	//if both arguments are NULL, they have the same hierarchy
	if (!arg1 && !arg2)
		return true;
		
	//if one of them is NULL, we cannot compare them
	if (!arg1 || !arg2)
		return false;
		
	if (arg1->fSourceA && arg2->fSourceA)
		bSourceAHierarchyIdentical = CompareHierarchy(arg1->fSourceA, arg2->fSourceA);

	if (!arg1->fSourceA && !arg2->fSourceA)
		bSourceAHierarchyIdentical = true;
		
	if (arg1->fSourceB && arg2->fSourceB)
		bSourceBHierarchyIdentical = CompareHierarchy(arg1->fSourceB, arg2->fSourceB);

	if (!arg1->fSourceB && !arg2->fSourceB)
		bSourceBHierarchyIdentical = true;

	if (arg1->fVideoObject == arg2->fVideoObject)
	{
		return bSourceAHierarchyIdentical && bSourceBHierarchyIdentical;
	}
	
	return false;
}

TArmeMasterMovieChunkHierarchyElement* TArmeMasterMovieChunkHierarchyElement::FindChunkHierarchyElementHosting(TVideoObject* object)
{
	if (fVideoObject == object)
		return this;

	TArmeMasterMovieChunkHierarchyElement* searchResult = NULL;
	if (fSourceA)
		searchResult = fSourceA->FindChunkHierarchyElementHosting(object);
	if (searchResult != NULL)
		return searchResult;
	if (fSourceB)
		searchResult = fSourceB->FindChunkHierarchyElementHosting(object);

	return searchResult;
}

TArmeMasterMovieHierarchyElement::TArmeMasterMovieHierarchyElement(TArmeMasterMovie* masterMovie, TArmeMasterMovieHierarchyElement* sourceA, TArmeMasterMovieHierarchyElement* sourceB, UInt32 trackType)
{
	fSourceA = sourceA;
	fSourceB = sourceB;
	fMasterMovieTrack = new TArmeMasterMovieTrack(masterMovie, (fSourceA ? fSourceA->GetMasterMovieTrack() : NULL), (fSourceB ? fSourceB->GetMasterMovieTrack() : NULL), 1000, trackType);
	
	fRefCount = 0;
}
	
TArmeMasterMovieHierarchyElement::TArmeMasterMovieHierarchyElement(ArmeArchiveHandler* unarchiver) : ArmeArchivable(unarchiver)
{
	fSourceA = dynamic_cast<TArmeMasterMovieHierarchyElement*>(unarchiver->ReadCPPObject());
	fSourceB = dynamic_cast<TArmeMasterMovieHierarchyElement*>(unarchiver->ReadCPPObject());
	fMasterMovieTrack = dynamic_cast<TArmeMasterMovieTrack*>(unarchiver->ReadCPPObject());
	fRefCount = unarchiver->ReadSInt32();
}
bool TArmeMasterMovieHierarchyElement::WriteToArchive(ArmeArchiveHandler* archiver) const
{
	archiver->WriteCPPObject(fSourceA);
	archiver->WriteCPPObject(fSourceB);
	archiver->WriteCPPObject(fMasterMovieTrack);	
	archiver->WriteSInt32(fRefCount);

	return true;
}


bool TArmeMasterMovieHierarchyElement::ContainsTextClips() const
{
	bool bSourceAContainsTextClips = false;
	bool bSourceBContainsTextClips = false;
	
	if (fSourceA)
		bSourceAContainsTextClips = fSourceA->ContainsTextClips();
	if (fSourceB)
		bSourceBContainsTextClips = fSourceB->ContainsTextClips();
	
	return bSourceAContainsTextClips || bSourceBContainsTextClips || (GetType() == kTextMedia);
}

bool TArmeMasterMovieHierarchyElement::CanHostChunk(TArmeMasterMovieChunkHierarchyElement* chunkElement)
{
	if (chunkElement == NULL)
		return false;

	//if the chunkElement is a container (group), it is not represented on any track; so we don't need a hierarchyElement for it
	if ((chunkElement->fVideoObject->GetType() == TArmeWSObj::kContainer) ||
	(chunkElement->fVideoObject->GetType() == TArmeWSObj::kTLC) || (chunkElement->fVideoObject->GetType() == TArmeWSObj::kAudioProcessor))
	{
		//group always have only one children (if they are more than one childre, they are children of a transition that is the only child of the group)
		ZzAssert(chunkElement->fSourceB == NULL);
		return CanHostChunk(chunkElement->fSourceA);
	}
		
	return ((chunkElement->GetType() == GetType()) && 
	((!fSourceA && !chunkElement->fSourceA) || (fSourceA && fSourceA->CanHostChunk(chunkElement->fSourceA))) && 
	((!fSourceB && !chunkElement->fSourceB) || (fSourceB && fSourceB->CanHostChunk(chunkElement->fSourceB))));
}

void TArmeMasterMovieHierarchyElement::CopyChunk(TArmeMasterMovieChunkHierarchyElement* chunkElement)
{
	if (chunkElement == NULL)
		return;

	verntl(videoAlain, 4, "TArmeMasterMovieHierarchyElement::CopyChunk(" << chunkElement->fVideoObject->GetName() << ") in Track " << fMasterMovieTrack << " from " << chunkElement->fMasterMovieChunk->fStartTime << " to " << chunkElement->fMasterMovieChunk->fEndTime);

	//if the chunkElement is a container (group), it is not represented on any track; so we don't need a hierarchyElement for it
	if ((chunkElement->fVideoObject->GetType() == TArmeWSObj::kContainer) ||
	(chunkElement->fVideoObject->GetType() == TArmeWSObj::kTLC) || (chunkElement->fVideoObject->GetType() == TArmeWSObj::kAudioProcessor))
	{
		//group always have only one children (if they are more than one childre, they are children of a transition that is the only child of the group)
		ZzAssert(chunkElement->fSourceB == NULL);
		return CopyChunk(chunkElement->fSourceA);
	}
	
	if (fSourceA)
	{
		fSourceA->CopyChunk(chunkElement->fSourceA);
	}
	if (fSourceB)
	{
		fSourceB->CopyChunk(chunkElement->fSourceB);
	}

	//copy the media into the mastermovie
	switch (chunkElement->fVideoObject->GetType())
	{
		case TArmeWSObj::kTextClip :
		{
			TArmeDoc* doc = chunkElement->fMasterMovieChunk->fMasterMovie->GetDoc();
			OSErr res = DeleteTrackSegment(fMasterMovieTrack->GetQTTrack(), doc->TimeValueFromArmeTime(chunkElement->fMasterMovieChunk->fStartTime), doc->TimeValueFromArmeTime(chunkElement->fMasterMovieChunk->fEndTime - chunkElement->fMasterMovieChunk->fStartTime));
			ThrowOnOSStatus(res);	

			((TArmeTextClip*)(chunkElement->fVideoObject))->CopyMediaIntoTrack(	fMasterMovieTrack->GetQTTrack(),
																			chunkElement->fMasterMovieChunk->fStartTime,
																			chunkElement->fMasterMovieChunk->fEndTime);
		};break;
		case TArmeWSObj::kVideoClip :
		{
			//extract the chunkList that will be used in this MasterMovieChunk
			TArmeChunkList<TVideoChunk> chunkList = ((TVideoClip*)(chunkElement->fVideoObject))->GetChunkList()->ExtractSlice(chunkElement->fMasterMovieChunk->fStartTime, chunkElement->fMasterMovieChunk->fEndTime - chunkElement->fMasterMovieChunk->fStartTime);
		
			//copy it to the track
			CopyChunkListIntoTrack(fMasterMovieTrack->GetQTTrack(), &chunkList);
		};break;
		case TArmeWSObj::kVideoEffect :
		case TArmeWSObj::kVideoXFadeEffect :
		{
			if (!fSourceA && !fSourceB)
				return;
				
			// get the effect description for the given time range
			TVideoEffect* videoEffect = dynamic_cast<TVideoEffect*>(chunkElement->fVideoObject);
			QTAtomContainer effectDesc = videoEffect->GetEffectDescFor(chunkElement->fMasterMovieChunk->fStartTime, chunkElement->fMasterMovieChunk->fEndTime);
		
			// add the samples in the track
			CopyEffectDescIntoTrack(videoEffect->GetDoc(), fMasterMovieTrack->GetQTTrack(), effectDesc, chunkElement->fMasterMovieChunk->fStartTime, chunkElement->fMasterMovieChunk->fEndTime);
		};break;
		case TArmeWSObj::kContainer :
		case TArmeWSObj::kAudioProcessor :
		case TArmeWSObj::kTLC :
		{
			//the container does not have any 'media' so nothing must be copied in any masterMovieTrack.
		};break;
		default :
			ZzAssert(0);
	}
}


void TArmeMasterMovieHierarchyElement::Clean(ArmeTime startTime, ArmeTime endTime)
{
	//copy background in every master movie track
	if (fSourceA)
		fSourceA->Clean(startTime, endTime);
	if (fSourceB)
		fSourceB->Clean(startTime, endTime);
	
	fMasterMovieTrack->fMasterMovie->CopyBackgroundInTrack(fMasterMovieTrack->GetQTTrack(), startTime, endTime);
}

TArmeMasterMovieHierarchyElement::~TArmeMasterMovieHierarchyElement()
{
	ZzAssert(fRefCount == 0);

	delete fSourceA;
	delete fSourceB;
	
	delete fMasterMovieTrack;
}


SInt32 TArmeMasterMovieHierarchyElement::Release() 
{
	fRefCount--;
	if (fRefCount == 0)
	{
		verntl(videoAlain, 4, "TArmeMasterMovieHierarchyElement::Release(), deleting hierarchy");

		fMasterMovieTrack->fMasterMovie->RemoveHierarchy(this);
		return 0;
	}
	else
		return fRefCount;
};

TArmeMasterMovieTextTrack*	TArmeMasterMovie::CreateNewTextTrack()
{
	TArmeMasterMovieTextTrack* ret = new TArmeMasterMovieTextTrack(this);
	fTextTrackArray->AddItem(ret);
	
	return ret;
}

void TArmeMasterMovie::RemoveTextTrack(TArmeMasterMovieTextTrack* textTrack)
{
	fTextTrackArray->RemoveItemByValue(textTrack);
	delete textTrack;
}

TArmeMasterMovieTextTrack* TArmeMasterMovie::GetTextTrack(int i)
{
	if (i < fTextTrackArray->CountItems())
		return fTextTrackArray->ItemAt(i);
	else
		return CreateNewTextTrack();
}


TArmeMasterMovieTextTrack::TArmeMasterMovieTextTrack(TArmeMasterMovie* masterMovie) : TArmeMasterMovieTrack(masterMovie, NULL, NULL, -32000, kKeyedTextMedia)
{
	//only the text tracks are high quality
	SetMediaPlayHints(GetTrackMedia(GetQTTrack()), hintsHighQuality, hintsHighQuality);
}

TArmeMasterMovieTextTrack::~TArmeMasterMovieTextTrack()
{
}

TArmeMasterMovieTextTrack::TArmeMasterMovieTextTrack(ArmeArchiveHandler* unarchiver) : TArmeMasterMovieTrack(unarchiver)
{
	//only the text tracks are high quality
	SetMediaPlayHints(GetTrackMedia(GetQTTrack()), hintsHighQuality, hintsHighQuality);
}

bool TArmeMasterMovieTextTrack::WriteToArchive(ArmeArchiveHandler* archiver) const
{
	TArmeMasterMovieTrack::WriteToArchive(archiver);
	
	return true;
}
	
void TArmeMasterMovieTextTrack::AddTextClip(TArmeTextClip* textClip, ArmeTime startTime, ArmeTime endTime)
{
	OSErr res = 0;
	DeleteTrackSegment(GetQTTrack(), fMasterMovie->GetDoc()->TimeValueFromArmeTime(startTime), fMasterMovie->GetDoc()->TimeValueFromArmeTime(endTime - startTime));
	ThrowOnOSStatus(res);
	textClip->CopyMediaIntoTrack(GetQTTrack(), startTime, endTime);
}

void TArmeMasterMovieTextTrack::Clean(ArmeTime startTime, ArmeTime endTime)
{
	fMasterMovie->CopyBackgroundInTrack(GetQTTrack(), startTime, endTime);
}

void TArmeMasterMovie::SetPlaybackQuality(bool highQuality, bool singleField)
{
	for (int i = 0; i < fHierarchiesArray->CountItems(); ++i)
	{
		fHierarchiesArray->ItemAt(i)->SetPlaybackQuality(highQuality, singleField);
	}
}

void TArmeMasterMovieHierarchyElement::SetPlaybackQuality(bool highQuality, bool singleField)
{
	long flags = 0;
	if (highQuality)
		flags |= hintsHighQuality;

	if (singleField)
		flags |= hintsSingleField;
		
	SetMediaPlayHints(GetTrackMedia(fMasterMovieTrack->GetQTTrack()), flags, hintsSingleField | hintsHighQuality);

	if (fSourceA)
		fSourceA->SetPlaybackQuality(highQuality, singleField);

	if (fSourceB)
		fSourceB->SetPlaybackQuality(highQuality, singleField);
}


