/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#ifndef TArmeMediaEngine_h
#include "TArmeMediaEngine.h"
#endif //TArmeMediaEngine_h

#ifndef VArmeGraphManager_h
#include "VArmeGraphManager.h"
#endif //VArmeGraphManager_h

#ifndef TArmeDoc_h
#include "TArmeDoc.h"
#endif //TArmeDoc_h

#ifndef TArmeAudioFileHandler_h
#include "TArmeAudioFileHandler.h"
#endif //TArmeAudioFileHandler_h

#ifndef TArmeVideoFileHandler_h
#include "TArmeVideoFileHandler.h"
#endif //TArmeVideoFileHandler_h

#ifndef TArmeQTFileHandler_h
#include "TArmeQTFileHandler.h"
#endif //TArmeQTFileHandler_h

#ifndef TArmeTextFileHandler_h
#include "TArmeTextFileHandler.h"
#endif //TArmeTextFileHandler_h

#ifndef TARPIHost_h
#include "TARPIHost.h"
#endif //TARPIHost_h

#ifndef MECoreAudioPlayer_h
#include "MECoreAudioPlayer.h"
#endif MECoreAudioPlayer_h

#ifndef ZzSplitTask_h
#include "ZzSplitTask.h"
#endif //ZzSplitTask_h

#ifndef ArmeNotifications_h
#include "ArmeNotifications.h"
#endif //ArmeNotifications_h

#ifndef MEApplicationServices_h
#include "MEApplicationServices.h"
#endif //MEApplicationServices_h

#ifndef MEAudioMonitoring_h
#include "MEAudioMonitoring.h"
#endif //MEAudioMonitoring_h

#ifndef MEMTGAPlayer_h
#include "MEMTGAPlayer.h"
#endif //MEMTGAPlayer_h

//TArmeMediaEngine* TArmeMediaEngine::fgInstance = NULL;

ZzPtrArray<VArmeMediaFileHandler*,true>	TArmeMediaEngine::fsMediaFileHandlers;
TARPIHost* 								TArmeMediaEngine::fsARPIHost = NULL;
MECoreAudioPlayer* 						TArmeMediaEngine::fsCoreAudioPlayer = NULL;
ZzArray<TArmeDoc*> 						TArmeMediaEngine::fsDocArray;
ZzRwLocker 								TArmeMediaEngine::fsDocArrayRwLocker;
MEApplicationServices* 					TArmeMediaEngine::fsApplicationServices = NULL;
AudioDeviceID 							TArmeMediaEngine::fsCurrentOutputDeviceID = 0;
UInt32 									TArmeMediaEngine::fsOutDevSuggestedBufferNbFrames = 0;
pthread_t 								TArmeMediaEngine::fsAudioVideoSyncThreadID = 0;
bool 									TArmeMediaEngine::fsContinueAudioVideoSyncThread = true;
bool									TArmeMediaEngine::fsAlive = false;
ZzArray<MEAudioMonitoring*>				TArmeMediaEngine::fsAudioMonitors;
ZzRwLocker								TArmeMediaEngine::fsAudioMonitorsArrayRwLocker;
ZzArray<MEMTGAPlayer*>					TArmeMediaEngine::fsMTGAPlayers;
ZzRwLocker								TArmeMediaEngine::fsMTGAPlayerArrayRwLocker;

void TArmeMediaEngine::Initialize(MEApplicationServices* applicationServices, AudioDeviceID outDevID, UInt32* outDevSuggestedBufferNbFrames)
{
	verntl(armeWSObjsBenjamin,6,"TArmeMediaEngine::Initialize() build : " << __DATE__ << " " << __TIME__);
	RegisterMediaFileHandlers();
	fsARPIHost = new TARPIHost;
	fsARPIHost->LoadPluginSets();
	
	fsCurrentOutputDeviceID = outDevID;
	fsOutDevSuggestedBufferNbFrames = *outDevSuggestedBufferNbFrames;
	fsCoreAudioPlayer = new MECoreAudioPlayer(fsCurrentOutputDeviceID,&fsOutDevSuggestedBufferNbFrames);
	*outDevSuggestedBufferNbFrames = fsOutDevSuggestedBufferNbFrames;
//	usleep(500000); 
	delete fsCoreAudioPlayer; // we don't need it yet since there are no docs playing so stop it
	fsCoreAudioPlayer = NULL;
		
//	fsCoreAudioPlayer = new MECoreAudioPlayer(outDevID,outDevSuggestedBufferNbFrames); //deferred until doc creation
	fsApplicationServices = applicationServices;
	
	CreateAudioVideoSyncThread();
	//RegisterMediaNodes();
	//RegisterClipClasses();
	fsAlive = true;
}

void TArmeMediaEngine::ChangeOutputDevice(AudioDeviceID outDevID, UInt32* outDevSuggestedBufferNbFrames)
{
	fsCurrentOutputDeviceID = outDevID;
	delete fsCoreAudioPlayer;
//	usleep(500000); // wait for half a second $$$TODO
	fsCoreAudioPlayer = new MECoreAudioPlayer(fsCurrentOutputDeviceID,outDevSuggestedBufferNbFrames);
	fsOutDevSuggestedBufferNbFrames = *outDevSuggestedBufferNbFrames;
	WarnDocsCoreAudioHasChanged();
	fsCoreAudioPlayer->Start();
}

void TArmeMediaEngine::WarnDocsCoreAudioHasChanged()
{ 
	// loop on every doc
	LockReadDocArray();
	SInt32 nbDocs = GetNbDocs();
	for(SInt32 iDoc = 0; iDoc < nbDocs; iDoc++)
	{
		ZzAssert(GetNbDocs() == nbDocs);
		// loop on every doc player
		TArmeDoc* doc = TArmeMediaEngine::GetDoc(iDoc);
		ZzAssert(GetNbDocs() == nbDocs);
		doc->GetMixingInfo()->InitCoreAudioDependentInfo();
	}
	UnlockReadDocArray();
}

void TArmeMediaEngine::ForceRestartOutputDevice()
{
	ZzAssert(fsCoreAudioPlayer);
	UInt32 outDevSuggestedBufferNbFrames = fsOutDevSuggestedBufferNbFrames;
	ChangeOutputDevice(fsCurrentOutputDeviceID, &outDevSuggestedBufferNbFrames);
}

void TArmeMediaEngine::Deinitialize()
{
	fsAlive = false;
	DestroyAudioVideoSyncThread();
//	UnregisterClipClasses();
	delete fsCoreAudioPlayer;
	// we don't delete this one because the static members ARE destroyed anyway.... we don't really care
//	UnregisterMediaFileHandlers();
	delete fsARPIHost;
//	UnregisterMediaNodes();
}

int32 TArmeMediaEngine::GetMediaFileHandlerCount( )
{
	return fsMediaFileHandlers.GetCount();
}

VArmeMediaFileHandler* TArmeMediaEngine::GetMediaFileHandler( int32 index )
{
	ZzAssert(index < GetMediaFileHandlerCount());
	return fsMediaFileHandlers[index];
}

void TArmeMediaEngine::RegisterMediaFileHandlers()
{
	// should register both disk-based and internal media handlers
	// (which turn files into clip refs)
        
	fsMediaFileHandlers.AddItem( new TArmeAudioFileHandler() ); // audio files can also be handled by QT so we need to catch them first.
	fsMediaFileHandlers.AddItem( new TArmeTextFileHandler() );  // text files can also be handled by QT so we need to catch them first.
	fsMediaFileHandlers.AddItem( new TArmeQTFileHandler() );

	//fsMediaFileHandlers.AddItem( new TArmeAudioFileHandler() );
	//fsMediaFileHandlers.AddItem( new TArmeVideoFileHandler() );
}

void TArmeMediaEngine::UnregisterMediaFileHandlers()
{
	// should unregister both disk-based and internal media handlers
	// (which turn files into clip refs)
	for( int32 i = 0; i < fsMediaFileHandlers.GetCount(); i++ )
	{
		VArmeMediaFileHandler* pMediaHandler = fsMediaFileHandlers[i];
		delete pMediaHandler;
	}
	fsMediaFileHandlers.RemoveAll();
}

int32 TArmeMediaEngine::GetSupportedExtensionsCount()
{
	int32 count = 0;
	for( int32 i = 0; i < GetMediaFileHandlerCount(); i++ )
	{
		count += GetMediaFileHandler(i)->GetSupportedExtensionsCount();
	}
	return count;
}

char* TArmeMediaEngine::GetSupportedExtension(int32 i)
{
	// the start and end index of extensions in the current media file handler
	int32 startIndex = 0;
	int32 stopIndex = 0;
	
	for( int32 j = 0; j < GetMediaFileHandlerCount(); j++ )
	{
		startIndex = stopIndex;
		stopIndex += GetMediaFileHandler(j)->GetSupportedExtensionsCount();
		if( i >= startIndex && i < stopIndex )
			return GetMediaFileHandler(j)->GetSupportedExtension(i-startIndex);
	}
	return NULL;
}

TARPIHost* TArmeMediaEngine::ARPIHost()
{
	return fsARPIHost;
}

void TArmeMediaEngine::AddDoc(TArmeDoc* doc)
{
	TArmeMediaEngine::LockWriteDocArray();
	fsDocArray.AddItem(doc);
	TArmeMediaEngine::UnlockWriteDocArray();
}

void TArmeMediaEngine::CheckForCoreAudioPlayer()
{
	if(!fsCoreAudioPlayer)
	{
		fsCoreAudioPlayer = new MECoreAudioPlayer(fsCurrentOutputDeviceID,&fsOutDevSuggestedBufferNbFrames);
		WarnDocsCoreAudioHasChanged();
		fsCoreAudioPlayer->Start();
	}
}

TArmeDoc* TArmeMediaEngine::GetDoc(SInt32 i)
{
	return fsDocArray.ItemAt(i);
}

SInt32 TArmeMediaEngine::GetNbDocs()
{
	return fsDocArray.CountItems();
}

void TArmeMediaEngine::RemoveDoc(TArmeDoc* doc)
{
	SInt32 i = fsDocArray.FindIndexForItem(doc);
	ZzAssert(i >= 0);
	TArmeMediaEngine::LockWriteDocArray();
	fsDocArray.RemoveItemByValue(doc);
	TArmeMediaEngine::UnlockWriteDocArray();
//	if(fsDocArray.CountItems() == 0) // it's the first doc.... don't do this anymore. this 
// is handled by the MI side.
//	{
//		delete fsCoreAudioPlayer;
//		fsCoreAudioPlayer = NULL;
//	}
}

#ifdef HARD_LOCKS_HACK
void TArmeMediaEngine::LockReadDocArray()
{
	fsDocArrayLocker.Lock();
}

void TArmeMediaEngine::UnlockReadDocArray()
{
	fsDocArrayLocker.Unlock();
}

void TArmeMediaEngine::LockWriteDocArray()
{
	fsDocArrayLocker.Lock();
}

void TArmeMediaEngine::UnlockWriteDocArray()
{
	fsDocArrayLocker.Unlock();
}
#else
void TArmeMediaEngine::LockReadDocArray()
{
	fsDocArrayRwLocker.LockRead();
}

void TArmeMediaEngine::UnlockReadDocArray()
{
	fsDocArrayRwLocker.UnlockRead();
}

void TArmeMediaEngine::LockWriteDocArray()
{
	fsDocArrayRwLocker.LockWrite();
}

void TArmeMediaEngine::UnlockWriteDocArray()
{
	fsDocArrayRwLocker.UnlockWrite();
}
#endif


MECoreAudioPlayer* TArmeMediaEngine::CoreAudioPlayer()
{
	return fsCoreAudioPlayer;
}

MEApplicationServices* TArmeMediaEngine::ApplicationServices()
{
	return fsApplicationServices;
}

void TArmeMediaEngine::BlindlyExecuteAndDeleteSplitTask(ZzSplitTask* splitTask)
{
	ZzAssert(0);
	float progress = 0.0;
	while(progress < 1.0)
	{
		splitTask->_PerformNextBit();
		progress = splitTask->GetProgress();
	}
	delete splitTask;
}

void* _AudioVideoSyncThread(void* cookie)
{
	TArmeMediaEngine::AudioVideoSyncThread();
	return (void*)0;
}

void TArmeMediaEngine::AudioVideoSyncThread()
{
	while(fsContinueAudioVideoSyncThread)
	{
		ApplicationServices()->Notify(kNotification_AdjustMovieArmeTime,0);
		usleep(50000);
	}
}

void TArmeMediaEngine::CreateAudioVideoSyncThread()
{
	pthread_create( &fsAudioVideoSyncThreadID, NULL, _AudioVideoSyncThread, NULL);
}	

void TArmeMediaEngine::DestroyAudioVideoSyncThread()
{
	fsContinueAudioVideoSyncThread = 0;
	void* retCode;
	pthread_join(fsAudioVideoSyncThreadID,&retCode);
}

bool TArmeMediaEngine::IsAlive()
{
	return fsAlive;
}

MEAudioMonitoring* TArmeMediaEngine::CreateAudioMonitor(SInt32 log2NbFrames, SInt32 nbChannels)
{
	MEAudioMonitoring* monitor = new MEAudioMonitoring(log2NbFrames,nbChannels);
	fsAudioMonitorsArrayRwLocker.LockWrite();
	fsAudioMonitors.AddItem(monitor);
	fsAudioMonitorsArrayRwLocker.UnlockWrite();
	return monitor;
}

void TArmeMediaEngine::RemoveAudioMonitor(MEAudioMonitoring* monitor)
{
	fsAudioMonitorsArrayRwLocker.LockWrite();
	fsAudioMonitors.RemoveItemByValue(monitor);
	fsAudioMonitorsArrayRwLocker.UnlockWrite();
	delete monitor;
}

void TArmeMediaEngine::LockReadAudioMonitorArray()
{
	fsAudioMonitorsArrayRwLocker.LockRead();
}

MEAudioMonitoring* TArmeMediaEngine::GetAudioMonitor(SInt32 i)
{
	return fsAudioMonitors.ItemAt(i);
}

void TArmeMediaEngine::UnlockReadAudioMonitorArray()
{
	fsAudioMonitorsArrayRwLocker.UnlockRead();
}

SInt32 TArmeMediaEngine::AudioMonitorArrayCountItems()
{
	return fsAudioMonitors.CountItems();
}

ArmeErr TArmeMediaEngine::CreateMTGAPlayer(MEMTGAPlayer** pPlayer, TArmeDoc* doc, const MEAudioChunkFileIndex& fileIndex)
{
	MEMTGAPlayer* player = new MEMTGAPlayer();
	ArmeErr err = player->Initialize(doc,fileIndex);
	if(err.type != kArmeOK)
	{
		delete player;
		*pPlayer = NULL;
		return err;
	}
	*pPlayer = player;
	fsMTGAPlayerArrayRwLocker.LockWrite();
	fsMTGAPlayers.AddItem(player);
	fsMTGAPlayerArrayRwLocker.UnlockWrite();
	return err;
}

void TArmeMediaEngine::DeleteMTGAPlayer(MEMTGAPlayer* player)
{
	fsMTGAPlayerArrayRwLocker.LockWrite();
	fsMTGAPlayers.RemoveItemByValue(player);
	fsMTGAPlayerArrayRwLocker.UnlockWrite();
	delete player;
}

void TArmeMediaEngine::LockReadMTGAPlayerArray()
{
	fsMTGAPlayerArrayRwLocker.LockRead();
}

SInt32 TArmeMediaEngine::MTGAPlayerArrayCountItems()
{
	return fsMTGAPlayers.CountItems();
}

MEMTGAPlayer* TArmeMediaEngine::GetMTGAPlayer(SInt32 i)
{
	return fsMTGAPlayers.ItemAt(i);
}

void TArmeMediaEngine::UnlockReadMTGAPlayerArray()
{
	fsMTGAPlayerArrayRwLocker.UnlockRead();
}


