/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#define TArmeMediaEngine_h

#ifndef TArmeMediaEngine_h
#include "TArmeMediaEngine.h"
#endif //TArmeMediaEngine_h

#ifndef ZzPtrArray_h
#include "ZzPtrArray.h"
#endif //ZzPtrArray_h

#ifndef ZzArray_h
#include "ZzArray.h"
#endif //ZzArray_h

#ifndef ZzLocker_h
#include "ZzLocker.h"
#endif //ZzLocker_h

#ifndef TArmeDoc_h
#include "TArmeDoc.h"
#endif //TArmeDoc_h

#ifndef ME_AudioChunks_h
#include "ME_AudioChunks.h"
#endif //ME_AudioChunks_h

#include <CoreAudio/CoreAudio.h>

class TArmeAudioFileHandler;
class TArmeVideoFileHandler;
class TArmeQTFileHandler;
class VArmeMediaFileHandler;
class VArmeGraphManager;
class TCoreAudioGraphManager;
class TARPIHost;
class MECoreAudioPlayer;
class MEAudioRecordManager;
class TArmeDoc;
class MEApplicationServices;
class ZzSplitTask;
class MEAudioMonitoring;
class MEMTGAPlayer;

class TArmeMediaEngine
{
private : // can't be built
									TArmeMediaEngine();
									TArmeMediaEngine(TArmeMediaEngine&);
	virtual							~TArmeMediaEngine();

public : //STATIC
	// returns the only instance of this class OBSOLETE WE DO NOT NEED IT BECAUSE ALL METHODS ARE STATIC
	// static TArmeMediaEngine*		Instance() { return fgInstance; } 
	
	static void						Initialize(MEApplicationServices* applicationServices, AudioDeviceID outDevID, UInt32* outDevSuggestedBufferNbFrames);
	static void 					ChangeOutputDevice(AudioDeviceID outDevID, UInt32* outDevSuggestedBufferNbFrames);
	static void						CheckForCoreAudioPlayer();
	static void 					WarnDocsCoreAudioHasChanged();

	// this call is meant to solve a problem where a too long processing in the ioproc at doc 
	// creation time leads to a permanent sound drop out. it did not happen under 10.1.X where the
	// coreaudio code better handled overheads.
	static void 					ForceRestartOutputDevice();


	static void						Deinitialize();
	static bool						IsAlive();

	static MEAudioMonitoring* 		CreateAudioMonitor(SInt32 log2NbFrames, SInt32 nbChannels);
	static void						RemoveAudioMonitor(MEAudioMonitoring* monitor);

	static void						LockReadAudioMonitorArray();
	static SInt32					AudioMonitorArrayCountItems();
	static MEAudioMonitoring* 		GetAudioMonitor(SInt32 i);
	static void						UnlockReadAudioMonitorArray();

	static ArmeErr 					CreateMTGAPlayer(MEMTGAPlayer** pPlayer, TArmeDoc* doc, const MEAudioChunkFileIndex& fileIndex);
	static void						DeleteMTGAPlayer(MEMTGAPlayer* player);

	static void						LockReadMTGAPlayerArray();
	static SInt32					MTGAPlayerArrayCountItems();
	static MEMTGAPlayer*	 		GetMTGAPlayer(SInt32 i);
	static void						UnlockReadMTGAPlayerArray();
	
public : //STATIC
	static void 					CreateAudioVideoSyncThread();
	static void 					AudioVideoSyncThread();
	static void 					DestroyAudioVideoSyncThread();
private : //STATIC
	static pthread_t				fsAudioVideoSyncThreadID;
	static bool						fsContinueAudioVideoSyncThread;
	
	
public : //STATIC
	static void						AddDoc(TArmeDoc* doc);
	static TArmeDoc*				GetDoc(SInt32 i);
	static SInt32					GetNbDocs();
	static void 					RemoveDoc(TArmeDoc* doc);
	static void						LockReadDocArray();
	static void						UnlockReadDocArray();
	static void						LockWriteDocArray();
	static void						UnlockWriteDocArray();
	
//	static TCoreAudioGraphManager*	CreateCoreAudioGraph();
	static TARPIHost*				ARPIHost();
	
//	static UInt32					GetNodeClassCount( );
//	static IArmeNodeClass*			GetNodeClass( UInt32 index );
	static int32					GetMediaFileHandlerCount( );
	static VArmeMediaFileHandler*	GetMediaFileHandler( int32 index );
	
	static MECoreAudioPlayer*		CoreAudioPlayer();
		
	static int32					GetSupportedExtensionsCount();
	static char*					GetSupportedExtension(int32 i);
	static MEApplicationServices*	ApplicationServices(); // gui dependent services such as message boxes with or without progress bars...
	// returns the audio media handler (the audio clip factory)
	static void 					BlindlyExecuteAndDeleteSplitTask(ZzSplitTask* splitTask);
	
public :
	/*
		registers the media handlers (either from disk or built-in)
	*/
	static void	RegisterMediaFileHandlers();
	static void	UnregisterMediaFileHandlers();
	
private :
	static TARPIHost*								fsARPIHost;
	static ZzPtrArray<VArmeMediaFileHandler*,true>	fsMediaFileHandlers;
	static ZzArray<TArmeDoc*>						fsDocArray;
	static MECoreAudioPlayer*						fsCoreAudioPlayer;
	static ZzRwLocker								fsDocArrayRwLocker;
	static MEApplicationServices*					fsApplicationServices;
	static AudioDeviceID 							fsCurrentOutputDeviceID;
	static UInt32 									fsOutDevSuggestedBufferNbFrames;
	static bool										fsAlive;

	static ZzArray<MEAudioMonitoring*>				fsAudioMonitors;
	static ZzRwLocker								fsAudioMonitorsArrayRwLocker;

	static ZzArray<MEMTGAPlayer*>					fsMTGAPlayers;
	static ZzRwLocker								fsMTGAPlayerArrayRwLocker;


};


