/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#ifndef ZzRABI_h
#include "ZzRABI.h"
#endif //ZzRABI_h

#ifndef TArmeQTFileHandler_h
#include "TArmeQTFileHandler.h"
#endif //TArmeQTFileHandler_h

#ifndef TArmeDoc_h
#include "TArmeDoc.h"
#endif //TArmeDoc_h

#ifndef MEAudioClip_h
#include "MEAudioClip.h"
#endif //MEAudioClip_h

#ifndef TVideoClip_h
#include "TVideoClip.h"
#endif //TVideoClip_h

#ifndef Zz_PathHelpers_h
#include "Zz_PathHelpers.h"
#endif //Zz_PathHelpers_h

#ifndef TArmeMediaEngine_h
#include "TArmeMediaEngine.h"
#endif //TArmeMediaEngine_h

#ifndef TArmeDoc_h
#include "TArmeDoc.h"
#endif //TArmeDoc_h

#ifndef ZzHelpers_h
#include "ZzHelpers.h"
#endif //ZzHelpers_h

#ifndef MovieHelper_h
#include "MovieHelper.h"
#endif //MovieHelper_h

#ifndef ArmeDataFileRef_h
#include "ArmeDataFileRef.h"
#endif //ArmeDataFileRef_h

#ifndef ZzHeapWalker_h
#include "ZzHeapWalker.h"
#endif ZzHeapWalker_h

#ifndef MIProgressBarDlgManager_h
#include "MIProgressBarDlgManager.h"
#endif //MIProgressBarDlgManager_h

#ifndef ZzException_h
#include "ZzException.h"
#endif //ZzException_h

#ifndef MEApplicationServices_h
#include "MEApplicationServices.h"
#endif //MEApplicationServices_h

class ZzString;

// Note that this will not work for muxed MPEG-1 tracks, because Apple sucks.
ArmeDataFileRef* TranslateToAIFF(FSSpec& outFile, Movie inMovie, Track inTrack, TArmeDoc* inDoc, MIProgressBarDlgManager* progressMngr)
{
	OSErr err = noErr;

	ArmeDataFileRef* fileRef = inDoc->GetNewVideoDataFileRef();
	char path[kMaxPathLength + 1] = { 0 };
	fileRef->AddRef();
	fileRef->GetFilePath(path);
	
	try
	{
		FSSpec dstFile = FSSpecFromNonExistingFilePath(path, FOUR_CHAR_CODE('TVOD'), FOUR_CHAR_CODE('MooV'), err);
		ThrowOnOSStatus(err);
	
		// NB. Caller should use the returned fileRef and we can lose outFile, eventually
		outFile = dstFile;
		
		// use the default progress procedure, if any
	//    ::SetMovieProgressProc(inMovie, (MovieProgressUPP)-1L, 0);
		
		ComponentInstance aiffComponent = nil;
		// hey, we can make this REALLY simple; it's smart enough to do a 44.1/16/stereo/uncompressed
		// with musicif we don't give it ANY hints except 'AIFF' :)
		
		/*
		// try 32-bit fp as per benja's specs
		aiffComponent = OpenDefaultComponent(MovieExportType, kQTFileTypeAIFF);
		SoundDescriptionHandle myDesc = (SoundDescriptionHandle)::NewHandleClear(sizeof(SoundDescription));
		(**myDesc).descSize = sizeof(SoundDescription);
		(**myDesc).numChannels = 2;
		(**myDesc).sampleSize = 32;
		(**myDesc).sampleRate = rate44khz;
		(**myDesc).dataFormat = kFloat32Format;
		ComponentResult myErr = MovieExportSetSampleDescription(aiffComponent, (SampleDescriptionHandle)myDesc, SoundMediaType);
		*/
	
		// export the movie into a file
		SetMovieProgressProc(inMovie, QTMovieProgressProc, (long)progressMngr);
		err = ::ConvertMovieToFile(inMovie, inTrack, &dstFile, 'AIFF', 'TVOD', smSystemScript, nil,
				createMovieFileDeleteCurFile | movieToFileOnlyExport /*| showUserSettingsDialog*/,	
				aiffComponent);
		ThrowOnOSStatus(err);
	
		/*
		// export the movie into a handle
		OSStatus err = ::PutMovieIntoTypedHandle(inMovie, inTrack, &dstFile, 'AIFF', 'TVOD', smSystemScript, nil,
				createMovieFileDeleteCurFile | movieToFileOnlyExport,	
				aiffComponent);*/
	}
	catch (ZzException* e)
	{
		e->DefaultCatcher();
		delete e;
		fileRef->Release();
		fileRef = NULL;
	}

    return fileRef;
}

bool TArmeQTFileHandler::CanHandleFileByLookingAtExtension( const ZzString* zzPath )
{
	if (IsThisFileRejectedByItsExtension(zzPath))
		return false;

    FSSpec file;
    Boolean canGraphics, canMovie, preferGraphics;
	char temp[10] = "         ";
	OSType ext;
	
	//get the uppercase extension
	ZzGetUpperCaseFileExtensionFromPath(temp, 9, zzPath->getUTF8String());
	//pad it with 'spaces'
	temp[strlen(temp)] = ' ';
	
	*((unsigned long*)&ext) = *((unsigned long*)temp);

	if( ZzPathToFSSpec(zzPath->getCFString(),&file) != noErr )	
        return false;

    OSErr err = ::CanQuickTimeOpenFile(0, 0, (OSType)ext, &canGraphics, &canMovie, &preferGraphics,
            kQTAllowOpeningStillImagesAsMovies // pictures can be video clips for now
            | kQTAllowImportersThatWouldCreateNewFile // sure, we're making work codecs anyway
            | kQTAllowAggressiveImporters // a bit iffier, but why not, assume user knows what they're doing
            );

	if (!err and canMovie)
		return true;
	else
	{
		// this nice function from QT does not recognize 'DV' and 'AIFF' as interesting files !!!
	
		for (int i=0; i < GetSupportedExtensionsCount(); ++i)
			if (!strcmp(temp, GetSupportedExtension(i)))
				return true;
	}
	
	return false;
}

bool TArmeQTFileHandler::IsThisFileRejectedByItsExtension( const ZzString* zzPath)
{
	char temp[10] = "         ";

	//get the uppercase extension
	ZzGetUpperCaseFileExtensionFromPath(temp, 9, zzPath->getUTF8String());

	for (int i=0; i < GetUnsupportedExtensionsCount(); ++i)
		if (!strcmp(temp, GetUnsupportedExtension(i)))
			return true;
			
	return false;
}


bool TArmeQTFileHandler::CanHandleFileByLookingAtType( const ZzString* zzPath )
{
    FSSpec file;
    Boolean canGraphics, canMovie, preferGraphics;
	
//    if (noErr != Path2FSSpecViaFSRef(file, path, false))
	if( ZzPathToFSSpec(zzPath->getCFString(),&file) != noErr )	
        return false;

    OSErr err = ::CanQuickTimeOpenFile(&file, 0, 0, &canGraphics, &canMovie, &preferGraphics,
            kQTAllowOpeningStillImagesAsMovies // pictures can be video clips for now
            | kQTAllowImportersThatWouldCreateNewFile // sure, we're making work codecs anyway
            | kQTAllowAggressiveImporters // a bit iffier, but why not, assume user knows what they're doing
            );

    return !err and canMovie;
}

bool IsTrackAudible(const Movie inMovie, const Track inTrack)
{
    Track checkTrack;
    long trackIndex = 1;
     while ((checkTrack = ::GetMovieIndTrackType(inMovie, trackIndex, AudioMediaCharacteristic, movieTrackCharacteristic)) != nil)
     {
        if (checkTrack == inTrack)
            return true;
        trackIndex++;
     }
   
    return false;
}

bool IsTrackVisible(const Track inTrack)
{
	//$$What's that test ??????? we should test that the track is enabled

    Fixed width = 0, height = 0;
    ::GetTrackDimensions(inTrack, &width, &height);
    return (width + height) > 0;
}

// return number of tracks HyperEngine-AV can make clips out of
int CountClippableTracks(const Movie inMovie)
{
    long trackIndex = 1;
    Track srcTrack;
    OSType srcType;
    long clipCount = 0;
    while ((srcTrack = ::GetMovieIndTrack(inMovie, trackIndex)) != nil)
    {
        trackIndex++;
        if (!::GetTrackEnabled(srcTrack))
            continue;
        
		//if there are any effects in the movie, reject the whole movie
		if (GetTrackReferenceCount(srcTrack, kTrackModifierReference) != 0)
			return 0;
				
        Media theMedia = ::GetTrackMedia(srcTrack);
        ::GetMediaHandlerDescription(theMedia, &srcType, nil, nil);

        switch (srcType)
        {
            case SoundMediaType:
            case VideoMediaType:
            case FlashMediaType:
            case ThreeDeeMediaType:
            case TimeCodeMediaType:
            case SpriteMediaType:
            case MusicMediaType:
               clipCount++;
		break;
            case TextMediaType:
		break;
            case MPEGMediaType:
                /* really no way to do this in QT 5 it seems
                if (IsTrackAudible(inMovie, srcTrack))
                    clipCount++;
                */
                /*if (IsTrackVisible(srcTrack))
                    clipCount++; // one for video too*/
					//Finally no ! it's a different kind of media
                break;
            case MovieMediaType:
            case TweenMediaType:
            default:
                // not clippable yet
                break;
        } // switch srcType
            
    } // while srcTrack

    return clipCount;
}

// return number of tracks HyperEngine-AV can make clips out of
int CountClippableVideoTracks(const Movie inMovie)
{
    long trackIndex = 1;
    Track srcTrack;
    OSType srcType;
    long clipCount = 0;
    while ((srcTrack = ::GetMovieIndTrack(inMovie, trackIndex)) != nil)
    {
        trackIndex++;
        if (!::GetTrackEnabled(srcTrack))
            continue;
        
		//if there are any effects in the movie, reject the whole movie
		if (GetTrackReferenceCount(srcTrack, kTrackModifierReference) != 0)
			return 0;
				
        Media theMedia = ::GetTrackMedia(srcTrack);
        ::GetMediaHandlerDescription(theMedia, &srcType, nil, nil);

        switch (srcType)
        {
            case VideoMediaType:
            case FlashMediaType:
            case ThreeDeeMediaType:
            case SpriteMediaType:
               clipCount++;
            default:
                // not clippable yet
                break;
        } // switch srcType
            
    } // while srcTrack

    return clipCount;
}

ArmeErr TArmeQTFileHandler::CreateWSObjsFromFile( 
	TArmeDoc* 					pDoc, 
	ZzArchPtrArray*				theObjects,
	const ZzString*				zzPath,
	ArmeDataFileRef*			fileRef,
	const TArmeWSObjPos&		pos, 
	MIProgressBarDlgManager*	progressMngr)
{
	ArmeErr err;
	OSErr sysErr = noErr;
	TArmeWSObjPos newArmePos(pos);
	char temp[1024];
	ZzString zzRealPath;

	// this RABI object will clean the array in its dtor UNLESS we 
	// tell it not to do so
	ZzRABI_DeleteWSObjectsInArchPtrArray archObjsDeleter(theObjects);

	//all dropped files should start on video frames boundaries
	
	// if a fileRef was given, there should be no path given.  But, since the path is used, fill it.
	if (fileRef)
	{
		fileRef->GetFilePath(temp);
		zzRealPath = ZzString(temp);
	}
	else
		zzRealPath = *zzPath;

	ZzString zzFileName = ZzGetFileNameFromPath(zzRealPath);

	// find and open the movie
	FSSpec movieFile;
//	if (noErr != Path2FSSpecViaFSRef(movieFile, path, false))
	if(ZzPathToFSSpec(zzRealPath.getCFString(),&movieFile) != noErr)
	{
		err.type = kArmeCannotCreateFSSpecFromPath;
		ThrowOnArmeErr(err); // this throws an exception
	}
   
	short movieRef;
	short itsID = DoTheRightThing;
	Movie movie = nil;
	OSErr qtErr = ::OpenMovieFile(&movieFile, &movieRef, fsRdPerm);
	// the file was opened correctly
	if (!qtErr) 
	{
		qtErr = ::NewMovieFromFile(&movie, movieRef, &itsID, nil, 0 , nil);
		::CloseMovieFile(movieRef);
	}
    // something went wrong
	if (qtErr || !movie)
	{
		err.type = kArmeQuickTimeError;
		err.data = (void*)(int(qtErr));
		ThrowOnArmeErr(err); // this throws an exception
	}
    
	// first RABI guard which will dispose the movie in its dtor
	ZzRABI_DisposeMovie movieDisposer(movie);
		
	///////////////////////////////////////////////////////////
	//	count the number of tracks we can extract clips from
	///////////////////////////////////////////////////////////
	//compute the number of sub tasks.
	//Note : when referencing external files, a video import does not need a progress bar
	SInt32 countClips = CountClippableTracks(movie);
	if (TArmeMediaEngine::ApplicationServices()->IsReferencingExternalFiles()) 
	{
		if (countClips - CountClippableVideoTracks(movie) > 0)
			progressMngr->SetNbSubTasks(countClips - CountClippableVideoTracks(movie));
	}
	else
		progressMngr->SetNbSubTasks(countClips);
		
    if(countClips == 0)
    {
		err.type = kArmeErrNoSuitableMediaHandlerFound;
		ThrowOnArmeErr(err); // this throws an exception
	}

	bool moreThanOneClippableTrack = (countClips >= 2);
	
    // loop through the tracks looking for useful ones
	OSType mediaOrder[9] = 
	{
		SoundMediaType, 
		MusicMediaType, 
		VideoMediaType, 
		FlashMediaType, 
		TextMediaType, 
		ThreeDeeMediaType, 
		TimeCodeMediaType, 
		SpriteMediaType, 
		MPEGMediaType
	};

	// if the movie contains one video track, round the drop position to the nearest video frame
	if (GetFirstVideoTrack(movie))
	{
		pDoc->RoundArmeTimeToNearestVideoFrame(newArmePos.fStartTime);
	}

	TArmeWSObj* newClip = nil;
	long trackIndex;
	
	for (int i=0; i < 9; i++) // loop on all the track types
	{
		trackIndex = 1;
		Track srcTrack;
		OSType srcType;
		FSSpec exportFile;

		while ((srcTrack = ::GetMovieIndTrack(movie, trackIndex)) != nil)
		{
			if (!::GetTrackEnabled(srcTrack))
			{
				trackIndex++;
				continue;
			}
	
			Media theMedia = ::GetTrackMedia(srcTrack);
			::GetMediaHandlerDescription(theMedia, &srcType, nil, nil);
			
			err.type = kArmeOK;

			if (srcType == mediaOrder[i])
			{
				switch (srcType)
				{
					case SoundMediaType:
					{
						//progressMngr->EnterSubTask();
						ZzRABI_SubTaskScope subTaskScope(progressMngr);
						
						if(moreThanOneClippableTrack)  //we consider it's the audio track of a movie
						{
							progressMngr->SetNbSubTasks(2);
							//since we had many problems with these audio tracks (in movies coming from capture or DV), we will first export
							//the audio track into aiff, and import it as a separate file
	#define BENJA_USE_A_TEMP_AIFF_FILE
//	#define BENJA_READ_MTGA_DIRECTLY_FROM_MOVIE
//	#define BENJA_USE_A_REALTIME_AIFF
	
	#ifdef BENJA_USE_A_TEMP_AIFF_FILE
							// create a file that's named "geefmijeenkus.aiff" in the doc project folder
							UInt32 docFolderLength = strlen(pDoc->GetProjectFolderPath());
							char* pathName = new char[docFolderLength+50];
							
							// this rabi object deletes the array whatever happens
							ZzRABI_ArrayDeleter<char> pathNameDeleter(&pathName);
														
							sprintf(pathName,"%s/geefmijeenkus.aiff",pDoc->GetProjectFolderPath());
													
							// create an FSSpec for the soon-to-be-exported AIFF movie file
							FSSpec destAIFFFSSpec = FSSpecFromNonExistingFilePath(pathName, FOUR_CHAR_CODE('TVOD'), FOUR_CHAR_CODE('MooV'), sysErr);
							
							if(sysErr != noErr)
								ThrowOnOSStatus(sysErr); // THIS THROWS !!!!!!!
							
							progressMngr->EnterSubTask();
							// convert the movie to an aiff file (with the track audio sample rate?) and display a progress bar.
							SetMovieProgressProc(movie, QTMovieProgressProc, (long)progressMngr);
							short resID;
							
							// ConvertMovieToFile doesn't throw so no need for resource
							// freeing for ::CloseMovieFile(resID);
							qtErr = ::ConvertMovieToFile(movie, GetMovieTrack(movie, trackIndex), &destAIFFFSSpec, 0, 0, -1, 0, 0, GetCodecInstance(kAiff)); 
							::CloseMovieFile(resID);
							progressMngr->LeaveSubTask();
							
							// this RABI will delete the temp AIFF file when going out of scope
							ZzRABI_FSpDelete deleteTmpAIFFFile(&destAIFFFSSpec);
	
							if(qtErr != noErr)
								ThrowOnOSStatus(qtErr);
			
							// import it
							// if ctor throws, object is freed, no problem
							progressMngr->EnterSubTask();
							newClip = new MEAudioClip(zzFileName,destAIFFFSSpec, 1, pDoc, progressMngr); // 1 means we want the FIRST track (since it's an AIFF file it's ok)
							std::auto_ptr<TArmeWSObj> audioClipDeleter(newClip);
							progressMngr->LeaveSubTask();
																
							newClip->SetNameCopy(zzFileName.getUTF8String()); // UTF-8 encoded name!!
							// this will maybe throw only BEFORE adding the object 
							theObjects->AddItem(newClip); // add the object to the dest array 
							audioClipDeleter.release(); // transfer of ownership to the array
	#elif defined BENJA_READ_MTGA_DIRECTLY_FROM_MOVIE
			// there were problems with DV stuff... let's check if it's any better now
			ZzAssert(0);
	#elif defined BENJA_USE_A_REALTIME_AIFF
		// waiting for alain's code to export to aiff with the doc sample rate 
		#ifdef EXPORT_TO_16BITS
			ZzAssert(0);
		#elif defined EXPORT_TO_FLOAT32
			ZzAssert(0);
		#endif //aiff format
	#endif //way of dealing with movie audio track import
						}
						else
						{
							newClip = new MEAudioClip(zzFileName,movieFile, trackIndex, pDoc, progressMngr); // zzPath is given for the name
							std::auto_ptr<TArmeWSObj> audioClipDeleter(newClip);
							theObjects->AddItem(newClip); // add the object to the dest array
							audioClipDeleter.release(); // transfer of ownership to the array
 						}
//						newClip->SetPos(newArmePos);
					}
					break;
				case TextMediaType:
				break;
				case VideoMediaType:
				case FlashMediaType:
				case ThreeDeeMediaType:
				case TimeCodeMediaType:
				case SpriteMediaType:
				{
					if (!TArmeMediaEngine::ApplicationServices()->IsReferencingExternalFiles())
						progressMngr->EnterSubTask();

					if (fileRef)
						newClip = new TVideoClip(fileRef, pDoc, progressMngr, &err, newArmePos, true);
					else
						newClip = new TVideoClip(zzFileName,movieFile, trackIndex, pDoc, progressMngr, &err, newArmePos);
					std::auto_ptr<TArmeWSObj> videoClipDeleter(newClip);
					if(err.type != kArmeOK)
						ThrowOnArmeErr(err);
					theObjects->AddItem(newClip); // add the object to the dest array
					videoClipDeleter.release(); // transfer of ownership to the array

					if (!TArmeMediaEngine::ApplicationServices()->IsReferencingExternalFiles())
						progressMngr->LeaveSubTask();
				}
				break;
						
				case MPEGMediaType:
#if 0
/*
				 really no way to do this in QT 5 it seems
					if (IsTrackAudible(movie, srcTrack))
					{
					if (TranslateToAIFF(exportFile, movie, srcTrack, pDoc->GetProjectFolderPath()))
					{
					newClip = new MEAudioClip(exportFile, 1, pDoc, bopTask, err);
							if (err->type == kArmeOK)
								newClip->SetPos(newArmePos);
							}
						}
*/
#endif
					if (IsTrackVisible(srcTrack))
					{
						ZzRABI_SubTaskScope subTaskScope(progressMngr);
						newClip = new TVideoClip(zzFileName,movieFile, trackIndex, pDoc, progressMngr, &err, newArmePos);
						std::auto_ptr<TArmeWSObj> videoClipDeleter(newClip);
						if(err.type != kArmeOK)
							ThrowOnArmeErr(err);
						theObjects->AddItem(newClip); // add the object to the dest array
						videoClipDeleter.release(); // release the resource ownership
					}
					break;
				case MusicMediaType:
					{
						ZzRABI_SubTaskScope subTaskScope(progressMngr);
						progressMngr->SetNbSubTasks(2);  //export aiff + create audio clip
						progressMngr->EnterSubTask();
						ArmeDataFileRef* tempRef = TranslateToAIFF(exportFile, movie, srcTrack, pDoc, progressMngr);
						// TranslateToAIFF returns a ref with ref 1 or NULL
						if(tempRef == NULL)
						{
							ZZ_THROW(ZzException);
						}
						
						ZzRABI_ArmeDataFileRef_Release fileRefReleaser(tempRef);
							
						progressMngr->LeaveSubTask();
						progressMngr->EnterSubTask();
	
						char refPath[PATH_MAX];
						tempRef->GetFilePath(refPath);

						newClip = new MEAudioClip(zzFileName,exportFile,0,pDoc, progressMngr);
						std::auto_ptr<TArmeWSObj> audioClipDeleter(newClip);
						if(err.type != kArmeOK)
							ThrowOnArmeErr(err);
						theObjects->AddItem(newClip); // add the object to the dest array
						audioClipDeleter.release(); // prevent the clip from being deleted!
						//newClip->SetPos(newArmePos); postponed
						progressMngr->LeaveSubTask();
					}
					break;
		
				case MovieMediaType:
				case TweenMediaType:
				default:
					err.type = kArmeErrNoSuitableMediaHandlerFound;
					break;
				} // switch srcType
			}
		trackIndex++;
		} // while srcTrack
	}
	archObjsDeleter.SaveResource();
	TArmeWSObj* obj;
	for(SInt32 i = 0; i < theObjects->CountItems(); i++)
	{
		ArmeArchivable* archObj = theObjects->ItemAt(i);
		obj = dynamic_cast<TArmeWSObj*>(archObj);
		ZzRtAssert(obj);
		obj->SetPos(newArmePos);
	}
		
    // figure out what to return
    return err;
}


int32 TArmeQTFileHandler::GetSupportedExtensionsCount() {
	return 9;
}

char* TArmeQTFileHandler::GetSupportedExtension(int32 i) {
	//extension must be 4 chars long !

	switch (i) {
		case 0:
			return "DV  ";
		case 1:
			return "AIFF";
		case 2:
			return "MOV ";
		case 3:
			return "WAV ";
		case 4:
			return "MP3 ";
		case 5:
			return "MP4 ";
		case 6:
			return "JPG ";
		case 7:
			return "PICT";
		case 8:
			return "PNG ";
	}
	return "???";
}

int32 TArmeQTFileHandler::GetUnsupportedExtensionsCount() {
	return 2;
}

char* TArmeQTFileHandler::GetUnsupportedExtension(int32 i) {
	switch (i) {
		case 0:
			return "MPEG";
		case 1:
			return "MPG ";
		case 2:
			return "???";
		case 3:
			return "???";
		case 4:
			return "???";
		case 5:
			return "???";
		case 6:
			return "???";
		case 7:
			return "???";
		case 8:
			return "???";
	}
	return "???";
}



/*
TArmeWSObj* TArmeQTFileHandler::CreateWSObjFromFile( 
    TArmeDoc* 					pDoc, 
    const ZzString*				zzPath, 
    const TArmeWSObjPos&		pos,
	MIProgressBarDlgManager*	progressMngr,
	ArmeErr*					err)
{
    err->type = kArmeOK;
    TArmeWSObjPos newArmePos(pos);

	// extract file Name
	char fileName[FILENAME_MAX];
    ZzGetFileNameFromPath(fileName, FILENAME_MAX, zzPath->getUTF8String());

    //
	// convert path to fsspec
    //
	FSSpec movieFile;
	if (noErr != ZzPathToFSSpec(zzPath->getCFString(),&movieFile))
	{
		err->type = kArmeCannotCreateFSSpecFromPath;
		return nil;
	}
   
    //
	// open movie file 
    //
	short movieRef;
    short itsID = DoTheRightThing;
    Movie movie = nil;
    OSErr qtErr = ::OpenMovieFile(&movieFile, &movieRef, fsRdPerm);
    
	if (!qtErr)
        qtErr = ::NewMovieFromFile(&movie, movieRef, &itsID, nil, 0 , nil);
    ::CloseMovieFile(movieRef);
    
	if (qtErr or !movie)
	{
		err->type = kArmeQuickTimeError;
		return nil;
	}
    
	//
	// create group if more than 1 clippable track (clippables = video, audio, text)
	TArmeContainer* group = nil;
	SInt32 countClips = CountClippableTracks(movie);
	progressMngr->SetNbSubTasks(countClips);
    switch (countClips)
    {
        case 0:
            ::DisposeMovie(movie);
            err->type = kArmeErrNoSuitableMediaHandlerFound;
            return nil;
        case 1:
            // return single clip
            break;
        default:
            { 
                heapwalker(group = new TArmeContainer(fileName, pDoc));
                newArmePos.fParent = group;
            }
            break;
    }
    
	//
	// loop through the tracks looking for useful ones
	//
	OSType mediaOrder[9] = {SoundMediaType, MusicMediaType, VideoMediaType, FlashMediaType, TextMediaType, ThreeDeeMediaType, TimeCodeMediaType, SpriteMediaType, MPEGMediaType};

	TArmeWSObj* newClip = nil;
	long trackIndex;
	
	//
	// there are indeed 9 track types worth examining
	//
	for (int i=0; i < 9; i++)
	{
		trackIndex = 1;
		Track srcTrack;
		OSType srcType;
		FSSpec exportFile;

		while ((srcTrack = ::GetMovieIndTrack(movie, trackIndex)) != nil)
		{
			if (!::GetTrackEnabled(srcTrack))
			{
				trackIndex++;
				continue;
			}
	
			Media theMedia = ::GetTrackMedia(srcTrack);
			::GetMediaHandlerDescription(theMedia, &srcType, nil, nil);
			
			err->type = kArmeOK;

			if (srcType == mediaOrder[i])
			{
				switch (srcType)
				{
				case SoundMediaType:
					progressMngr->EnterSubTask();
					
					if (group)  //we consider it's the audio track of a movie
					{
						progressMngr->SetNbSubTasks(2);  //export aiff + create audio clip
		
						//since we had many problems with these audio tracks (in movies coming from capture or DV), we will first export
						//the audio track into aiff, and import it as a separate file
					
						//export to aiff (note, we keep the original extension)
						char pathName[kMaxPathLength];
						sprintf(pathName,"%s/%s.aiff",pDoc->GetProjectFolderPath(),fileName);
						
						//ZzAppendFileNameToPath(pathName, kMaxPathLength, pDoc->GetProjectFolderPath(), aiffFileName);
						
						FSSpec dest = FSSpecFromNonExistingFilePath(pathName, FOUR_CHAR_CODE('TVOD'), FOUR_CHAR_CODE('MooV'));
						SetMovieProgressProc(movie, QTMovieProgressProc, (long)progressMngr);
						progressMngr->EnterSubTask();
						::ConvertMovieToFile(movie, GetMovieTrack(movie, trackIndex), &dest, 0, 0, -1, 0, 0, GetCodecInstance(kAiff)); 
						progressMngr->LeaveSubTask();
		
						//import it
						progressMngr->EnterSubTask();
						heapwalker(newClip = new MEAudioClip(pathName, pDoc, progressMngr, err));
						progressMngr->LeaveSubTask();
		
						//remove the aiff file
						remove(pathName);
					}
					else
						heapwalker(newClip = new MEAudioClip(ZzString(fileName),movieFile, trackIndex, pDoc, progressMngr, err));
					progressMngr->LeaveSubTask();
					if (err->type == kArmeOK)
						newClip->SetPos(newArmePos);
					break;
		
					case VideoMediaType:
					case FlashMediaType:
					case TextMediaType:
					case ThreeDeeMediaType:
				case TimeCodeMediaType:
				case SpriteMediaType:
					progressMngr->EnterSubTask();
					heapwalker(newClip = new TVideoClip(ZzString(fileName),movieFile, trackIndex, pDoc, progressMngr, err, newArmePos));
					progressMngr->LeaveSubTask();
				break;
						
				case MPEGMediaType:
				#if 0
				  really no way to do this in QT 5 it seems
					if (IsTrackAudible(movie, srcTrack))
					{
					if (TranslateToAIFF(exportFile, movie, srcTrack, pDoc->GetProjectFolderPath()))
					{
					newClip = new MEAudioClip(exportFile, 1, pDoc, bopTask, err);
							if (err->type == kArmeOK)
								newClip->SetPos(newArmePos);
							}
						}
				#endif
						
					if (IsTrackVisible(srcTrack))
					{
						progressMngr->EnterSubTask();
						heapwalker(newClip = new TVideoClip(ZzString(fileName),movieFile, trackIndex, pDoc, progressMngr, err, newArmePos));
						progressMngr->LeaveSubTask();
					}
					break;
				case MusicMediaType:
					{
						progressMngr->EnterSubTask();
						ArmeDataFileRef* tempRef = TranslateToAIFF(exportFile, movie, srcTrack, pDoc, progressMngr);
						progressMngr->LeaveSubTask();
						//tempRef->AddRef();
						
						// NB: Audio clip constructors should take the ArmeDataFileRef* I imagine,
						// Benjamin wants to change those himself
						delete tempRef;
						
						if (tempRef)
						{
							progressMngr->EnterSubTask();
							heapwalker(newClip = new MEAudioClip(ZzString(fileName),exportFile, 1, pDoc, progressMngr,err));
							progressMngr->LeaveSubTask();
							if (err->type == kArmeOK)
								newClip->SetPos(newArmePos);
						}
					}
					break;
		
				case MovieMediaType:
				case TweenMediaType:
				default:
					err->type = kArmeErrNoSuitableMediaHandlerFound;
					break;
				} // switch srcType
			}
	//
	//		if (newClip and (err->type == kArmeOK))
	//		{
	#if 0
	saibot: it's basically only a matter of doing a SetPos with the relevant TArmeWSObjPos
	saibot: not even calling addchild or anything
	saibot: the code I have (on my side) to put an object in another object
	saibot: if the object already exists is:
	saibot: TArmeWSObjPos   newArmePos;
	saibot: 	newArmePos.fParent = [fParentObject WSObject]; // this is the new parent TArmeWSObj
	saibot: 	newArmePos.fInput = 0; // support for multiple entries will come, although not right now
	saibot: 	newArmePos.fStartTime = newPos; // in arme time
	saibot:     fWSObject->SetPos(newArmePos);
	saibot: and that's all
	* /
				if (group)
					group->_AddChild(newClip);
				else
					break;
			}
			else
			{
				delete newClip;
				newClip = nil;
			}
	#endif           
		trackIndex++;
		} // while srcTrack
	}
    // clean up
    ::DisposeMovie(movie);
        
    // figure out what to return
    return group ? group : newClip;
}
*/