/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#define TArmeTextClip_h

#ifndef ZzTypes_h
#include "ZzTypes.h"
#endif //ZzTypes_h

#ifndef TArmeWSObj_h
#include "TArmeWSObj.h"
#endif //TArmeWSObj_h

#ifndef TArmeTextClip_h
#include "TArmeTextClip.h"
#endif //TArmeTextClip_h

#ifndef TVideoObject_h
#include "TVideoObject.h"
#endif //TVideoObject_h

#ifdef __OBJC__
#import <Cocoa/Cocoa.h>
#endif //#ifdef __OBJC__

class TArmeDoc;
/*class ZzSplitTask;
class TArmeMasterMovie; 
class TVideoClipMemento;
class ArmeChunkList;*/
struct TextParams;

typedef enum _TCScrollType
{
	noScroll = 0,			// No scrolling.
	scrollVerticalReverse,	// Up to down.
	scrollVertical,			// Down to up.
	scrollHorizontalReverse,// Left to right.
	scrollHorizontal		// Right to left.
} TCScrollType;

class TArmeTextClip : public TVideoObject
{
public :

	TArmeTextClip(
		const TArmeTextClip *		obj,
		TArmeDoc * 					doc, 
		TArmeWSObjPos &				pos);
		
	TArmeTextClip(
		const char * 			name, 
		TArmeDoc * 				doc);

	TArmeTextClip	(const char *			name,
					TArmeDoc * 				doc,
					const TArmeWSObjPos &	pos,
					TextParams &			textParams);

#ifdef __OBJC__	
	TArmeTextClip	(NSString *				text,
					const char *			name,
					TArmeDoc * 				doc,
					const TArmeWSObjPos &	pos);
	TArmeTextClip	(NSAttributedString *	text,
					const char *			name,
					TArmeDoc * 				doc,
					const TArmeWSObjPos &	pos);
#endif //#ifdef __OBJC__	

	~TArmeTextClip();

	TArmeTextClip(ArmeArchiveHandler* unarchiver);
	virtual bool 	WriteToArchive(ArmeArchiveHandler* archiver) const;

#ifdef __OBJC__	
	void			SetTextSpecs(
						int						inScrollType,
						bool					inTextKeyed,
						NSColor *				inTrackBackground,
						ArmeTime				inClipDuration,
						NSData *				inRichText,
						NSString *				inPlainText,
						NSAttributedString *	inStyledText,
						short					inJustify,
						short					inVerticalAlignment
						);
#endif //#ifdef __OBJC__	
		
	virtual	SInt32	GetType() const;

	// keyed text isn't, backgrounded text is
	virtual bool	IsVideoObject() const;
	
	bool			IsKeyed() const;

	// called when object rep double-clicked
	bool			EditText();
	
	virtual TArmeWSObj* GetSliceCopy(ArmeTime startTime, ArmeTime endTime, const char* tempDirectoryName, MIProgressBarDlgManager* progressMngr) const;	
	virtual ArmeTime	InsertSlice(ArmeTime insertTime, const TArmeWSObj* obj, MIProgressBarDlgManager* progressMngr);
	virtual bool 		DeleteSlice(ArmeTime startTime, ArmeTime endTime, MIProgressBarDlgManager* progressMngr);
	virtual void		TimeStretch(ArmeTime startTime, ArmeTime endTime, ArmeTime newEndTime);
	virtual void 		SetEndTrim(ArmeTime endTrim);
	virtual void 		SetStartTrim(ArmeTime startTrim);
	
	virtual void		RoundTimingInfo();

	
	void				CopyMediaIntoTrack(Track destTrack, ArmeTime startTime, ArmeTime endTime);
	void				InsertMediaIntoMasterMovieMediaTrack();
	TimeValue			fMediaStartTime;

	virtual void		PosWillChange();
	virtual void		PosHasChanged();

	virtual ArmeTime	GetAbsoluteStartTimeOfVideoClip(bool bConsiderAlsoMutedObjects = false) const;
	virtual ArmeTime	GetAbsoluteEndxTimeOfVideoClip(bool bConsiderAlsoMutedObjects = false) const;
	virtual void		AddListOfStartAndEndTimeOfVideoClip(ArmeTime startTime, ArmeTime endTime, ZzArray<ArmeTime> &list, bool bConsiderAlsoMutedObjects) const;

    virtual ArmeTime 		GetDuration() const;

	// GP
	virtual void		Move();
	virtual bool		ContainsVideoAtTime(ArmeTime t) const;
	virtual bool		ContainsVideoDuring(ArmeTime startTime, ArmeTime endTime, bool bConsiderAlsoMutedObjects = false, bool bConsiderAlsoTransparentTextClips = false) const;
	// GP

	virtual void 					SetMemento(TArmeWSObjMemento* memento);
	virtual TArmeWSObjMemento*	 	GetMemento() const;
	
protected :
	virtual void					_FillMemento(TArmeWSObjMemento* memento) const;

public :

	// accessors for MITextController when editing
	int				GetScrollType() { return fScrollType; }
	bool			GetTextKeyed() { return fTextKeyed; }
    ArmeTime		GetTrackDuration() { return fWholeTextClipDuration; }
  
#ifdef __OBJC__	
	NSColor *		GetTrackBackground() { return [fTrackBackground copy]; }
    NSData * 		GetRichText() { return [fRichText copy]; }
    NSAttributedString *	GetStyledText();
    NSString *		GetPlainText() { return [fPlainText copy]; }
#else  //#ifdef __OBJC__	
	void *			GetTrackBackground() { return nil; }
    void *			GetRichText() { return nil; }
    void *			GetStyledText() { return nil; }
    void *			GetPlainText() { return nil; }
#endif //#ifdef __OBJC__	
	
	short			GetTextJustify() { return fTextJustify; }
	short			GetVerticalAlignment() { return fVerticalAlignment; }

protected:

	int				fScrollType;
    bool			fTextKeyed;
#ifdef __OBJC__	
    NSColor *		fTrackBackground;
#else  //#ifdef __OBJC__	
    void *			fTrackBackground;
#endif //#ifdef __OBJC__	
    ArmeTime		fWholeTextClipDuration;
	short			fTextJustify;
	short			fVerticalAlignment;

#ifdef __OBJC__	
	// GP : do we need this as the text style is in fStyledText (see bellow).
	IBOutlet id 	fFontPopupButton;
	IBOutlet id 	fSizePopupButton;
	int				fCustomSizeTextField;
	IBOutlet id 	fStylePopupButton;
	NSColor*		fTextColorWell;
#else  //#ifdef __OBJC__	
	void * 			fFontPopupButton;
	void * 			fSizePopupButton;
	int				fCustomSizeTextField;
	void *			fStylePopupButton;
	void *			fTextColorWell;
#endif //#ifdef __OBJC__	

#ifdef __OBJC__	
	NSData *		fRichText;
	NSString *		fPlainText;
	NSAttributedString *		fStyledText;
#else  //#ifdef __OBJC__	
	void *			fRichText;
	void *			fPlainText;
	void *			fStyledText;
#endif //#ifdef __OBJC__	
};

class TArmeTextClipMemento : public TVideoObjectMemento
{
public :

	int				fScrollType;
    bool			fTextKeyed;
#ifdef __OBJC__	
    NSColor *		fTrackBackground;
#else  //#ifdef __OBJC__	
    void *			fTrackBackground;
#endif //#ifdef __OBJC__	
    ArmeTime		fWholeTextClipDuration;
	short			fTextJustify;
	short			fVerticalAlignment;

#ifdef __OBJC__	
	NSData *		fRichText;
	NSString *		fPlainText;
	NSAttributedString *		fStyledText;
#else  //#ifdef __OBJC__	
	void *			fRichText;
	void *			fPlainText;
	void *			fStyledText;
#endif //#ifdef __OBJC__	

	TimeValue		fMediaStartTime;

	TArmeTextClipMemento();
	TArmeTextClipMemento(ArmeArchiveHandler* unarchiver);
	virtual bool WriteToArchive(ArmeArchiveHandler* archiver) const;
	virtual ~TArmeTextClipMemento();
};