/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "TArmeTextClip.h"
#import "MITextController.h"
#import "MovieHelper.h"
#import "ArmeChunk.h"

#import "MIBackupPasteboard.h"

#ifndef ZzHelpers_h
#include "ZzHelpers.h"
#endif //ZzHelpers_h

#ifndef MIProgressBarDlgManager_h
#include "MIProgressBarDlgManager.h"
#endif //MIProgressBarDlgManager_h

#import "MIViewerController.h"

#import "MIPreferences.h"

#ifndef MovieHelper_h
#include "MovieHelper.h"
#endif //MovieHelper_h



// constructor used in GetSliceCopy
TArmeTextClip::TArmeTextClip(
		const char* 			name, 
		TArmeDoc* 				doc): 
	TVideoObject(name, doc),
	fTrackBackground(NULL),
	fRichText(NULL),
	fPlainText(NULL),
	fStyledText(NULL)
{
	fMediaStartTime = 0;

}


TArmeTextClip::TArmeTextClip(
		const TArmeTextClip*		obj,
		TArmeDoc* 					doc, 
		TArmeWSObjPos&		pos) :
	TVideoObject(obj->GetName(), doc),
	fTrackBackground(NULL),
	fRichText(NULL),
	fPlainText(NULL),
	fStyledText(NULL)
{
	SetTextSpecs(	obj->fScrollType,
					obj->fTextKeyed,
					obj->fTrackBackground,
					obj->fWholeTextClipDuration,
					obj->fRichText,
					obj->fPlainText,
					obj->fStyledText,
					obj->fTextJustify,
					obj->fVerticalAlignment);

	fStartTrim = obj->fStartTrim;
	fEndTrim = obj->fEndTrim;

	_SetPos(pos);
}

// GP : new
TArmeTextClip::TArmeTextClip	(const char *			name,
								TArmeDoc * 				doc,
								const TArmeWSObjPos &	pos,
								TextParams &			textParams)
:
	TVideoObject(name, doc),
	fTrackBackground(NULL),
	fRichText(NULL),
	fPlainText(NULL),
	fStyledText(NULL)
{
	SetTextSpecs(	textParams.scrollType,
					textParams.textKeyed,
					textParams.trackBackground,
					textParams.clipDuration,
					textParams.richText,
					textParams.plainText,
					textParams.styledText,
					textParams.textJustify,
					textParams.verticalAlignment);

	_SetPos(pos);
}
// GP

// +--------------------------------+
// | For Drag And Drop : text file. |
// +--------------------------------+
TArmeTextClip::TArmeTextClip	(NSString *				text,
								const char *			name,
								TArmeDoc * 				doc,
								const TArmeWSObjPos &	pos)
:
	TVideoObject(name, doc),
	fTrackBackground(NULL),
	fRichText(NULL),
	fPlainText(NULL),
	fStyledText(NULL)
{
	float	duration = [[MIPreferences sharedPrefs] textClipDefaultDuration]; 

	NSColor *	theWhiteColor = [NSColor colorWithDeviceRed:1.0 green:1.0 blue:1.0 alpha:1.0];

	SetTextSpecs(	noScroll,
					true,//textKeyed
					theWhiteColor,//textParams.trackBackground
					doc->SecondsToArmeTime(duration),//trackDuration,
					nil,//richText
					text,//plainText
					nil,//styledText
					teCenter,//textJustify
					MIBottomTextAlignment);//verticalAlignment

	_SetPos(pos);
}

// +-------------------------------+
// | For Drag And Drop : RTF file. |
// +-------------------------------+
TArmeTextClip::TArmeTextClip	(NSAttributedString *	styledText,
								const char *			name,
								TArmeDoc * 				doc,
								const TArmeWSObjPos &	pos)
:
	TVideoObject(name, doc),
	fTrackBackground(NULL),
	fRichText(NULL),
	fPlainText(NULL),
	fStyledText(NULL)
{
	float duration = [[MIPreferences sharedPrefs] textClipDefaultDuration]; 

	//NSColor *	theWhiteColor = [NSColor whiteColor];
	NSColor *	theWhiteColor = [NSColor colorWithDeviceRed:1.0 green:1.0 blue:1.0 alpha:1.0];

	SetTextSpecs(	noScroll,
					true,//textKeyed
					theWhiteColor,//textParams.trackBackground
					doc->SecondsToArmeTime(duration),//trackDuration,
					nil,//richText
					nil,//plainText
					styledText,//styledText
					teCenter,//textJustify
					MIBottomTextAlignment);//verticalAlignment

	_SetPos(pos);
}

TArmeTextClip::~TArmeTextClip()
{
	[fTrackBackground release];
	[fRichText release];
	[fStyledText release];
	[fPlainText release];
}

TArmeTextClip::TArmeTextClip(ArmeArchiveHandler* unarchiver) : TVideoObject(unarchiver)
{
	fScrollType = unarchiver->ReadSInt32();
	fTextKeyed = unarchiver->ReadBool();
	fTrackBackground = [unarchiver->ReadObjCObject() retain];
	fWholeTextClipDuration = unarchiver->ReadArmeTime();
	fTextJustify = unarchiver->ReadSInt32();
	
	fRichText = [unarchiver->ReadObjCObject() retain];
	fPlainText = [unarchiver->ReadObjCObject() retain];
	fStyledText = [unarchiver->ReadObjCObject() retain];
	
	if (unarchiver->GetFileVer() >= 0x00010027)
		fVerticalAlignment = unarchiver->ReadSInt32();
	else
		fVerticalAlignment = MITopTextAlignment;

	if (unarchiver->GetFileVer() >= 0x0001002F)
		fMediaStartTime = unarchiver->ReadSInt32();
	else
		fMediaStartTime = 0;
}

bool TArmeTextClip::WriteToArchive(ArmeArchiveHandler* archiver) const
{
	TVideoObject::WriteToArchive(archiver);

	archiver->WriteSInt32(fScrollType);
	archiver->WriteBool(fTextKeyed);
    archiver->WriteObjCObject(fTrackBackground);
	archiver->WriteArmeTime(fWholeTextClipDuration);
	archiver->WriteSInt32(fTextJustify);
	
	archiver->WriteObjCObject(fRichText);
	archiver->WriteObjCObject(fPlainText);
	archiver->WriteObjCObject(fStyledText);

	//from version 0x00010027
	archiver->WriteSInt32(fVerticalAlignment);

	//from version 0x0001002F
	archiver->WriteSInt32(fMediaStartTime);
	return true;
}

extern const short kKeyedTextLayer;
extern const short kUnkeyedTextLayer;

TArmeWSObj* TArmeTextClip::GetSliceCopy(ArmeTime startTime, ArmeTime endTime, const char* tempDirectoryName, MIProgressBarDlgManager* progressMngr) const
{
	if (!ContainsVideoDuring(startTime, endTime, true, true))
		return NULL;

	ArmeTime copyStartTime;  //the relative pos of the copied object
	ArmeTime absoluteStartTime = GetAbsoluteStartTimeOfVideoClip(true);
	ArmeTime absoluteEndxTime = GetAbsoluteEndxTimeOfVideoClip(true);

	if ((startTime == ARME_TIME_UNDEF) || (startTime == ARME_TIME_MIN))
		copyStartTime = GetPos().fStartTime;
	else if (startTime < absoluteStartTime)
	{
		if (startTime < fPos.fParent->GetAbsoluteStartTime())
			copyStartTime = absoluteStartTime - fPos.fParent->GetAbsoluteStartTime();
		else
			copyStartTime = absoluteStartTime - startTime;
	}
	else
		copyStartTime = ARME_TIME_ZERO;

	if ((startTime == ARME_TIME_UNDEF) || (startTime == ARME_TIME_MIN))
		startTime = absoluteStartTime;
	if ((endTime == ARME_TIME_UNDEF) || (endTime == ARME_TIME_MAX))
		endTime = absoluteEndxTime;

	if (startTime < absoluteStartTime)
		startTime = absoluteStartTime;
	if (endTime > absoluteEndxTime)
		endTime = absoluteEndxTime;

	GetDoc()->RoundArmeTimeToNearestVideoFrame(startTime);
	GetDoc()->RoundArmeTimeToNearestVideoFrame(endTime);

	bool bCopyCompleteObject = (startTime == absoluteStartTime) && (endTime == absoluteEndxTime);


	//create a new clip with the same parameters as this one
	TArmeTextClip* copy = new TArmeTextClip(GetName(), GetDoc());  
	copy->SetTextSpecs(	fScrollType,
						fTextKeyed,
						fTrackBackground,
						fWholeTextClipDuration,
						fRichText,
						fPlainText,
						fStyledText,
						fTextJustify,
						fVerticalAlignment);

	copy->SetMuted(GetMuted());
	copy->fPos.fStartTime = copyStartTime; 
	copy->fPos.fParent = NULL;
	copy->fCachedAbsoluteStartTime = startTime;
	copy->fMasterMovie = fMasterMovie;

	if (bCopyCompleteObject)
	{
		copy->fStartTrim = fStartTrim;
		copy->fEndTrim = fEndTrim;
	}
	else
	{
		copy->fStartTrim = ARME_TIME_ZERO;
		copy->fEndTrim = ARME_TIME_ZERO;
		copy->fWholeTextClipDuration = endTime - startTime;
	}
	
	return copy;
}

ArmeTime	TArmeTextClip::InsertSlice(ArmeTime insertTime, const TArmeWSObj* obj, MIProgressBarDlgManager* progressMngr)
{
	// don't really know what it shall do
	return _ArmeTime(0);
}

bool TArmeTextClip::DeleteSlice(ArmeTime startTime, ArmeTime endTime, MIProgressBarDlgManager* progressMngr)
{
	bool bDeleteWholeObject = true;
	bool bDeleteStartTrim = false;
	bool bDeleteEndTrim = false;
	ArmeTime absoluteStartTime = GetAbsoluteStartTimeOfVideoClip(true);
	ArmeTime absoluteEndxTime = GetAbsoluteEndxTimeOfVideoClip(true);

	if ((startTime == ARME_TIME_UNDEF) || (startTime == ARME_TIME_MIN))
		startTime = absoluteStartTime;
	if ((endTime == ARME_TIME_UNDEF) || (endTime == ARME_TIME_MAX))
		endTime = absoluteEndxTime;

	ArmeTime cutStartTime = startTime;
	GetDoc()->RoundArmeTimeToNearestVideoFrame(cutStartTime);
	GetDoc()->RoundArmeTimeToNearestVideoFrame(endTime);

	if (cutStartTime <= absoluteStartTime)
		cutStartTime = absoluteStartTime;
	else
		bDeleteWholeObject = false;

	if (endTime >= absoluteEndxTime)
		endTime = absoluteEndxTime;
	else
		bDeleteWholeObject = false;

	if ((cutStartTime < absoluteEndxTime) && (endTime >= absoluteEndxTime))
		bDeleteEndTrim = true;
	if ((cutStartTime <= absoluteStartTime) && (endTime > absoluteStartTime))
		bDeleteStartTrim = true;

	fHasBeenEdited = true;
		
	if (bDeleteWholeObject)
	{
		return true;  // the object will be deleted outside this routine (in this case, we must not modify the startTrim and the endTrim)
	}
	else
	{
		//remove representation of the clip in the master movie
		fMasterMovie->RemoveSubMovie(this);

		// delete the trim if the slice was reaching one of the end of the clip
		if (bDeleteEndTrim)
		{
			fWholeTextClipDuration -= fEndTrim;
			fEndTrim = ARME_TIME_ZERO;
		}
		if (bDeleteStartTrim)
		{
			fWholeTextClipDuration -= fStartTrim;
			fStartTrim = ARME_TIME_ZERO;
		}
	
		if ((endTime > absoluteStartTime) && (cutStartTime < absoluteEndxTime))
		{
			fWholeTextClipDuration -= endTime - cutStartTime;
		}

		InvalidateCachedTimeInfo();

		// if we have deleted 'something' before the video clip, the start of the clip must be moved to the start of what has been deleted
		if (cutStartTime != startTime)
		{
			ZzAssert(fPos.fParent);

			if (endTime > absoluteStartTime)
			{
				if (startTime < fPos.fParent->GetAbsoluteStartTime())
					fPos.fStartTime = ARME_TIME_ZERO;
				else
				{
					ArmeTime newAbsoluteStartTime = startTime; //startTime - fPos.fParent->GetAbsoluteStartTime();
					fPos.fStartTime = newAbsoluteStartTime - fPos.fParent->GetAbsoluteStartTime();
				}
			}
			else
			{
				if (startTime < fPos.fParent->GetAbsoluteStartTime())
				{
					if (endTime > fPos.fParent->GetAbsoluteStartTime())
						fPos.fStartTime -= (endTime - fPos.fParent->GetAbsoluteStartTime());
				}
				else			
					fPos.fStartTime -= (endTime - startTime); //startTime - fPos.fParent->GetAbsoluteStartTime();
			}

			fPos.fParent->_NotifyChildPosHasChanged();
		}

		RoundTimingInfo();

		// insert the new representation of the clip in the master movie
		fMasterMovie->AddSubMovie(this);

		return false;
	}
}

void TArmeTextClip::PosWillChange()
{
	fMasterMovie->RemoveSubMovie(this);
}
void TArmeTextClip::PosHasChanged()
{
	fMasterMovie->AddSubMovie(this);
}

ArmeTime TArmeTextClip::GetAbsoluteStartTimeOfVideoClip(bool bConsiderAlsoMutedObjects) const 
{
	if (bConsiderAlsoMutedObjects || IsVisible())
	{
		return GetAbsoluteStartTime();  //$$NOTE : we will need to take the trim into account here !
	}
	else
		return ARME_TIME_UNDEF;
}

ArmeTime TArmeTextClip::GetAbsoluteEndxTimeOfVideoClip(bool bConsiderAlsoMutedObjects) const 
{
	if (bConsiderAlsoMutedObjects || IsVisible())
		return GetAbsoluteStartTime() + GetDuration(); //$$NOTE : we will need to take the trim into account here !
	else
		return ARME_TIME_UNDEF;
};

void TArmeTextClip::AddListOfStartAndEndTimeOfVideoClip(ArmeTime startTime, ArmeTime endTime, ZzArray<ArmeTime> &list, bool bConsiderAlsoMutedObjects) const
{
	//NOTE, there is the same implementation in TVideoClip

	ArmeTime clipStartTime = GetAbsoluteStartTimeOfVideoClip(bConsiderAlsoMutedObjects);
	ArmeTime clipEndTime = GetAbsoluteEndxTimeOfVideoClip(bConsiderAlsoMutedObjects);
	
	int i = 0;
	
	if ((clipStartTime >= startTime) && (clipStartTime <= endTime))
	{
	    if (list.FindIndexForItem(clipStartTime) == -1) // the trackrep is not yet in the list
		{
			if (list.CountItems() == 0)
				list.AddItem(clipStartTime);
			else
			{
				while ((i < list.CountItems()) && (clipStartTime > list[i]))
					i++;
				list.InsertItem(clipStartTime, i);
			}
		}
	}
	if ((clipEndTime >= startTime) && (clipEndTime <= endTime))
	{
	    if (list.FindIndexForItem(clipEndTime) == -1) // the trackrep is not yet in the list
		{
			if (list.CountItems() == 0)
				list.AddItem(clipEndTime);
			else
			{
				while ((i < list.CountItems()) && (clipEndTime > list[i]))
					i++;
				list.InsertItem(clipEndTime, i);
			}
		}
	}
}

void TArmeTextClip::SetTextSpecs(
		int						inScrollType,
		bool					inTextKeyed,
		NSColor *				inTrackBackground,
		ArmeTime				inClipDuration,
		NSData *				inRichText,
		NSString *				inPlainText,
		NSAttributedString *	inStyledText,
		short					inJustify,
		short					inVerticalAlignment
		)
{
    if (fTrackBackground != nil)
		[fTrackBackground release];
	if (fRichText != nil)
		[fRichText release];
	if (fPlainText != nil)
		[fPlainText release];
	if (fStyledText != nil)
		[fStyledText release];
	
	fScrollType = inScrollType;
	fTextKeyed = inTextKeyed;

	fTrackBackground = inTrackBackground;

	ArmeTime newDuration = GetDoc()->RoundArmeTimeToNearestVideoFrame(inClipDuration);
	
	if (fWholeTextClipDuration != newDuration)
	{
		fWholeTextClipDuration = newDuration;
		fStartTrim = ARME_TIME_ZERO;
		fEndTrim = ARME_TIME_ZERO;
	}

	//Make sure the absoluteEndTime of the text clip will be on a frame boundary
	ArmeTime endTime = GetAbsoluteEndxTimeOfVideoClip();
	if (endTime != ARME_TIME_UNDEF)
	{
		fDoc->RoundArmeTimeToNearestVideoFrame(endTime);
		fWholeTextClipDuration = endTime - GetAbsoluteStartTimeOfVideoClip();
	}

	RoundTimingInfo();	

	fRichText = inRichText;
	fPlainText = inPlainText;
	fTextJustify = inJustify;
	fStyledText = inStyledText;
	fVerticalAlignment = inVerticalAlignment;

	[fTrackBackground retain];
	[fRichText retain];
	[fPlainText retain];
	if ((fStyledText != nil) && ([fStyledText length] == 0))
		[fStyledText release];
	if (fStyledText == nil)
		fStyledText = [[NSAttributedString alloc] initWithRTF:fRichText documentAttributes:NULL];
	[fStyledText retain];
    
    if (fTextKeyed)
    {
        [fTrackBackground release];

        // and now comes one of the ugliest work-around of this software:  There is a bug in QT (really?) such that when you draw text on a transparent background,
        // it anti-alias with the color of the background (but at this stage, it does not realize it is transparent !) so usually, you have black text anti-aliased
        // with blue and this is super-imposed over the video.  This is very nice !.
        // So, to avoid this, I have decided to set the background color to the one of the first letter of the text so, anti-aliasing is done over the same color as the text
        // and this is beautiful.  He he !  But there comes another bug (or feature) of QT, since it realizes that I'm drawing text over a background with the same color, it does not
        // draw it at all (what a niece optimisation ! but QT forgets that my background is transparent and that I'm then expecting the text to be drawn)
        // HACK : set the background color very close to the text color :-)
        NSColor* textColor = [fStyledText attribute:NSForegroundColorAttributeName atIndex:0 effectiveRange:nil];
        if (textColor == nil) textColor=[NSColor blackColor];

        NSColor* textColor2 = [textColor colorUsingColorSpaceName:NSCalibratedRGBColorSpace];
        if ([textColor2 redComponent] < 0.99f)
            fTrackBackground = [NSColor colorWithDeviceRed:([textColor2 redComponent] + 0.01f) green:[textColor2 greenComponent] blue:[textColor2 blueComponent] alpha:[textColor2 alphaComponent]];
        else
            fTrackBackground = [NSColor colorWithDeviceRed:([textColor2 redComponent] - 0.01f) green:[textColor2 greenComponent] blue:[textColor2 blueComponent] alpha:[textColor2 alphaComponent]];

        [fTrackBackground retain];
    }

	if (strlen(GetName()) == 0)	// The title of the clip is empty : fill it.
		{
		// Set the title of the text clip.
		const unsigned short	textClipTitleMaxLength = 60;
		char					clipName[textClipTitleMaxLength + 1];
		unsigned short			startIndex, endIndex;
		const char *			textClipString;
		unsigned short			textClipLength/*, textClipLength2*/;
		unsigned short			textClipTitleLength;
		BOOL	overLengthText = false;
	
		if ((fPlainText != nil) && ([fPlainText length] != 0))
			{
			textClipString = [fPlainText UTF8String];
			//textClipLength2 = [fPlainText length];
			textClipLength = strlen(textClipString); // number of 8bit char, not number of unicode chars
			}
		else if ((fStyledText != nil) && ([fStyledText length] != 0))
			{
			textClipString = [[fStyledText string] UTF8String];
			//textClipLength2 = [fStyledText length];
			textClipLength = strlen(textClipString); // number of 8bit char, not number of unicode chars
			}

		/*if (textClipLength2 == 0)
			NSBeep();*/
		// Look for returns before any text.
		startIndex = 0;
		while (textClipString[startIndex] == '\n')
			startIndex++;
		// Look for returns after the first return.
		endIndex = startIndex;
		while ((textClipString[endIndex + 1] != '\n') && (endIndex + 1 < textClipLength))
			endIndex++;
	
		// Look if the text is not too long.
		if ((endIndex - startIndex) >= textClipTitleMaxLength)
			{
			endIndex = startIndex + textClipTitleMaxLength - 1;
			overLengthText = true;
			}
	
		textClipTitleLength = endIndex - startIndex + 1;
	
		// Copy the entire text.
		strncpy(clipName, textClipString + startIndex, textClipTitleLength);
		clipName[textClipTitleLength] = 0;
		// Look if it's too long : put '...' at the end.
		if (overLengthText)
			strncpy(clipName + (textClipTitleMaxLength - 3), "...", 3);
	
		SetNameCopy(clipName);
		}
		
	InsertMediaIntoMasterMovieMediaTrack();	
}

SInt32 TArmeTextClip::GetType() const
{
	return kTextClip;
}

bool TArmeTextClip::IsVideoObject() const
{
	return !fTextKeyed;
}

bool TArmeTextClip::IsKeyed() const
{
	return fTextKeyed;
}

void TArmeTextClip::TimeStretch(ArmeTime startTime, ArmeTime endTime, ArmeTime newEndTime)
{
	//Note : the text clips can only be stretched totally (not a slice selection) =>
	startTime = GetAbsoluteStartTimeOfVideoClip(true);
	endTime = GetAbsoluteEndxTimeOfVideoClip(true);

	GetDoc()->RoundArmeTimeToNearestVideoFrame(newEndTime);

	if (startTime == newEndTime)
		return;

	//remove representation of the clip in the master movie
	fMasterMovie->RemoveSubMovie(this);

	//Note : time-stretch removes the trim (on text clip only since we can not stretch only a slice)
	fStartTrim = ARME_TIME_ZERO;
	

	/* GP : TimeStretch exists for a TVideoClip but not for a TVideoObject.
	TVideoObject::TimeStretch(startTime, endTime, newEndTime);
	GP */
	fWholeTextClipDuration = newEndTime - startTime;
	TVideoObject::SetStartTrim(ARME_TIME_ZERO);
	TVideoObject::SetEndTrim(ARME_TIME_ZERO);

	RoundTimingInfo();
	
	InsertMediaIntoMasterMovieMediaTrack();
	
	// insert the new representation of the clip in the master movie
	fMasterMovie->AddSubMovie(this);
}

void TArmeTextClip::SetStartTrim(ArmeTime startTrim)
{
	//remove representation of the clip in the master movie
	fMasterMovie->RemoveSubMovie(this);

	TVideoObject::SetStartTrim(startTrim);

	RoundTimingInfo();
	
	// insert the new representation of the clip in the master movie
	fMasterMovie->AddSubMovie(this);
}

void TArmeTextClip::SetEndTrim(ArmeTime endTrim)
{
	//remove representation of the clip in the master movie
	fMasterMovie->RemoveSubMovie(this);

	TVideoObject::SetEndTrim(endTrim);

	RoundTimingInfo();
	
	// insert the new representation of the clip in the master movie
	fMasterMovie->AddSubMovie(this);
}

void TArmeTextClip::RoundTimingInfo()
{
	//first, make sure the object starts on a frame boundary :
	ArmeTime absoluteStartTime = GetAbsoluteStartTimeOfVideoClip();
	ArmeTime roundedAbsoluteStartTime = absoluteStartTime;
	GetDoc()->RoundArmeTimeToNearestVideoFrame(roundedAbsoluteStartTime);

	if (absoluteStartTime == ARME_TIME_UNDEF)
		return;

	if (absoluteStartTime != roundedAbsoluteStartTime)
	{
		fPos.fStartTime += roundedAbsoluteStartTime - absoluteStartTime;
	}

	//then, make sure the objects ends on a frame boundary :
	ArmeTime duration = fWholeTextClipDuration;
	ArmeTime absoluteEndTime = GetAbsoluteEndxTimeOfVideoClip();

	ArmeTime roundedAbsoluteEndTime = absoluteEndTime;

	GetDoc()->RoundArmeTimeToNearestVideoFrame(roundedAbsoluteEndTime);

	if (absoluteEndTime != roundedAbsoluteEndTime)
	{
		fWholeTextClipDuration = roundedAbsoluteEndTime - roundedAbsoluteStartTime + fEndTrim + fStartTrim;
		duration = fWholeTextClipDuration;
	}

	InvalidateCachedTimeInfo();
}


// Called by a double-click on clip rep as per George's instructions
bool TArmeTextClip::EditText()
{
	//ArmeDataFileRef* newFileRef = [[MITextController sharedTextController] editTextClip:this];
	/* GP : old
	ArmeDataFileRef* newFileRef = [[[MITextController alloc] init] editTextClip:this];
	if (newFileRef)
	{
		UpdateFileRef(newFileRef);
		// GP : according to the new movie architecture, do this :
		fMasterMovie->UpdateSubMovie(this);
//		SetMuted(IsMuted();
		return true;
	}
	return false;*/
	MITextController *	theMITextController = [[MITextController alloc] init];
	bool	result = [theMITextController editTextClip:this];
	[theMITextController release];

	return result;
}

/* GP : old
void TArmeTextClip::UpdateFileRef(ArmeDataFileRef* inRef)
{
	fDataFileRef->Release();
	//delete fDataFileRef; where exactly are refs added?

	fDataFileRef = inRef;
	char path[kMaxPathLength + 1] = { 0 };
	fDataFileRef->GetFilePath(path);

	ArmeErr err;
	Movie movie = ImportFile(path, GetDoc(), &err);
	float movieLength = GetMovieDurationSec(movie);
	ArmeTime start = GetAbsoluteStartTime();
	ArmeTime length = GetDoc()->SecondsToArmeTime(movieLength);
	ArmeTime end = start + length;

	//replace the current track
//	delete fTrackList;
//	fTrackList = new TVideoTrack(this, start, end);

	delete fVisibleChunkList;
	fVisibleChunkList = new TArmeChunkList<TVideoChunk>(new TVideoChunk(fDataFileRef, start, length), 1, start);

	//$$TODO : add chunkList to submovie ?
//	fTrackList->fQTTrack = fMasterMovie->AddSubMovie(this, movie, 1, start);

	//_SetLength(length);
	SetMuted(IsMuted());
}
GP */


const short kKeyedTextLayer = -32000;
const short kUnkeyedTextLayer = -31000;


void TArmeTextClip::InsertMediaIntoMasterMovieMediaTrack()
{
	TimeValue wholeDuration = fDoc->TimeValueFromArmeTime(fWholeTextClipDuration);
	if (wholeDuration == 0)
		return;

	// The height and the width of the the movie.
	/*short		theMovieHeight = fMasterMovie->GetViewerHeight();//;fMasterMovie->GetHeight();
	short		theMovieWidth = fMasterMovie->GetViewerWidth();//fMasterMovie->GetWidth();*/
	short		theMovieHeight = fMasterMovie->GetHeight();
	short		theMovieWidth = fMasterMovie->GetWidth();

    Rect		textRect = {0, 0, theMovieHeight, theMovieWidth};
    // Unused : TimeScale	trackTimeScale = GetMovieTimeScale(fMasterMovie->GetQTMovie());

	Track destTrack = fMasterMovie->GetMasterTrackForText();
	Media textMedia = GetTrackMedia(destTrack);
	
    MediaHandler textHandler = ::GetMediaHandler(textMedia);
    
    // get background color
	RGBColor background;
	background.red = (short)(0xFFFF * [fTrackBackground redComponent]);
    background.green = (short)(0xFFFF * [fTrackBackground greenComponent]);
    background.blue = (short)(0xFFFF * [fTrackBackground blueComponent]);

   // get flags
    long flags = 0; // dfShrinkTextBoxToFit?

    if (fTextKeyed)
        flags += dfKeyedText;
                
	/*flags += dfClipToTextBox;*/
	flags += dfShrinkTextBoxToFit;
	/*flags += dfDontAutoScale;*/

	MIBackupPasteboard	* theBackupPasteboard = [MIBackupPasteboard backupPasteboard];
	[theBackupPasteboard backup];
	if (fRichText != nil)
            [theBackupPasteboard setData:fRichText forType:NSRTFPboardType];
	else if (fPlainText != nil)
            [theBackupPasteboard setString:fPlainText forType:NSStringPboardType];
        else if (fStyledText != nil)
        {
            NSRange	theRange = NSMakeRange(0, [fStyledText length]);
            NSData * theRTFData = [fStyledText RTFFromRange:theRange documentAttributes:nil];
            [theBackupPasteboard setData:theRTFData forType:NSRTFPboardType];
        }

	Rect	destRect = textRect;
	if ((fScrollType == scrollHorizontal) || (fScrollType == scrollHorizontalReverse))
            destRect.right = 32767;

	//[[MIViewerController sharedViewerController] setMovie:nil];

	// Create a new GWorld where we'll do our text things...
	GrafPtr		savePort;
	GWorldPtr	offscreenGWorld = nil;
	QDErr		theQDErr = ::NewGWorld (&offscreenGWorld, 0, &textRect, NULL, NULL, 0);
	if (theQDErr == noErr)
        {
            ::GetPort(&savePort);
            ::SetGWorld(offscreenGWorld, NULL);
        }

	// Create the TEHandle.
	TEHandle theTEHandle = ::TEStyleNew(&destRect, &textRect);
	// Don't wrap the text in case of horizontal scrolling.
	if ((fScrollType == scrollHorizontal) || (fScrollType == scrollHorizontalReverse))
            (**theTEHandle).crOnly = -1;
	// Paste the text into the TEHandle.
	::TEStylePaste(theTEHandle);

	[theBackupPasteboard retrieve];
	[theBackupPasteboard release];

	// Get the Height of the text.
	long theTextHeight = ::TEGetHeight(32767, 0, theTEHandle);

	// Find the longuest line length of the text.
	int	theTextWidth = 0;
	for (int i = 0 ; i < (**theTEHandle).nLines ; i++)
        {
            ::TESetSelect((**theTEHandle).lineStarts[i], (**theTEHandle).lineStarts[i + 1], theTEHandle);
            if (((**theTEHandle).selRect.right) - ((**theTEHandle).selRect.left) > theTextWidth)
                theTextWidth = (**theTEHandle).selRect.right - (**theTEHandle).selRect.left;
        }

	if ((fScrollType == scrollHorizontal) || (fScrollType == scrollHorizontalReverse) || fScrollType == noScroll)
	{
            switch (fVerticalAlignment)
            {
                case MITopTextAlignment :
                    break;
                case MICenterTextAlignment :
                    textRect.top = theMovieHeight/2 - theTextHeight/2;
                    textRect.bottom = textRect.top + theTextHeight;
                    break;
                case MIBottomTextAlignment :
                    textRect.bottom = theMovieHeight;
                    textRect.top = textRect.bottom - theTextHeight;
                    break;
            }
        }
        
	short	textJustify = fTextJustify;
	if ((fScrollType == scrollHorizontal) || (fScrollType == scrollHorizontalReverse))
            textJustify = teFlushLeft;
	
    // add text sample
    TimeValue sampleTime = 0;
/*	
	$$NOTE : this code works fine but I had to remove it due to a QT bug :
	If a text clip (or a still image) is involved in a cross fade and if there is only
	one media sample at the beginning of the clip, on export in DV, QT will only compute the
	transition each time it sees a sample (which means only once => no transition at all)
	
	if (fScrollType == noScroll)
		{
		::BeginMediaEdits(textMedia);

		::TextMediaAddTESample(	textHandler,
								theTEHandle,
								&background,
								textJustify,
								&textRect,
								flags,        
								0,//scrollDelay,
								0,
								0,
								&blackRGBColor,
								wholeDuration,
								&fMediaStartTime);
		
		OSErr res1 = GetMoviesError();

		// finish movie up
		::EndMediaEdits(textMedia);
		OSErr res2 = GetMoviesError();
		ThrowOnOSStatus(res1);
		ThrowOnOSStatus(res2);

		::InsertMediaIntoTrack(	destTrack,
								fMediaStartTime,
								fMediaStartTime, // the time where the new media has been added
								wholeDuration, // the duration of the new media added
								fixed1);
		res2 = GetMoviesError();
		ThrowOnOSStatus(res2);
		}
	else*/
		{
		// Frame duration.
		TimeValue	videoFrameDuration = fDoc->GetVideoFrameDuration();
	
		// Absolute times.
/*		ArmeTime	theAbsoluteStart = GetAbsoluteStartTimeOfVideoClip() - fStartTrim;
		ArmeTime	theAbsoluteEnd = GetAbsoluteEndxTimeOfVideoClip() + fEndTrim;
		ArmeTime	theAbsoluteLength = theAbsoluteEnd - theAbsoluteStart;*/
	
		// Project Frames per second.
		float		projectFPS = fDoc->fMasterMovie->GetFPS();
	
		// Number of frames.
		long		theNumberOfFrames = (long)ceil((fDoc->ArmeTimeToSeconds(fWholeTextClipDuration) * projectFPS) - 0.5);
	
		// Percentage of this part from the total time of the text.
		float		thePercentage = 1.0f; //float(endTime.data - startTime.data) / float(theAbsoluteLength.data);
	
		// The current height and width;
		short		theRectHeight = short((theMovieHeight + theTextHeight) * thePercentage);
		short		theRectWidth = short((theMovieWidth + theTextWidth) * thePercentage);
		
		// Increment of each frame in pixels.
		float		theVPixelsIncrement = float(theRectHeight) / float(theNumberOfFrames);
		float		theHPixelsIncrement = float(theRectWidth) / float(theNumberOfFrames);
	
		/*if (theVPixelsIncrement < 1.0)
			NSBeep();
		if (theHPixelsIncrement < 1.0)
			NSBeep();*/
	
		// The starting text box rect.
		float		theStartFrame = 0.0f;
		if ((fScrollType == scrollVertical) || (fScrollType == scrollHorizontal))
                    theStartFrame = 0.0f;
		else if ((fScrollType == scrollVerticalReverse) || (fScrollType == scrollHorizontalReverse))
                    theStartFrame = ceil((fDoc->ArmeTimeToSeconds(fWholeTextClipDuration) * projectFPS) - 0.5);
	
		short		startTop = short(theStartFrame * theVPixelsIncrement);
		short		startLeft = short(theStartFrame * theHPixelsIncrement);
		
		MIProgressBarDlgManager* progDlgManager = NULL; 

		if ((fScrollType != noScroll) && (wholeDuration > GetDoc()->TimeValueFromArmeTime(_ArmeTime(300000))))
                    progDlgManager = new MIProgressBarDlgManager("Creating Text Clip...");

		bool bFirstSampleHasBeenAdded = false;


		::BeginMediaEdits(textMedia);
		for (int i = theNumberOfFrames ; i >= 0 ; i--)  // invert the loop because everything is inverted since I introduced the optimisation
//		for (int i = 0 ; i <= theNumberOfFrames ; i++)
		{
                    if (progDlgManager)
                        progDlgManager->Update(float(theNumberOfFrames-i) / theNumberOfFrames);

                    ::SetGWorld(offscreenGWorld, NULL);
                    if (fScrollType == scrollVertical)
                    {
                        textRect.top = (startTop + short(i * theVPixelsIncrement))  - theTextHeight;
                        textRect.bottom = textRect.top + theTextHeight;
                    }
                    else if (fScrollType == scrollVerticalReverse)
                    {
                        textRect.top = (startTop - short(i * theVPixelsIncrement)) - theTextHeight;
                        textRect.bottom = textRect.top + theTextHeight;
                    }
                    else if (fScrollType == scrollHorizontal)
                    {
                        textRect.left = (startLeft + short(i * theHPixelsIncrement)) - theTextWidth;
                        textRect.right = textRect.left + theTextWidth;
                    }
                    else if (fScrollType == scrollHorizontalReverse)
                    {
                        textRect.left = (startLeft - short(i * theHPixelsIncrement)) - theTextWidth;
                        textRect.right = textRect.left + theTextWidth;
                    }
                    
                    ::TextMediaAddTESample( textHandler,
                                            theTEHandle,
                                            &background,
                                            textJustify,
                                            &textRect,
                                            flags,        
                                            0,
                                            0,
                                            0,
                                            0, //&blackRGBColor,
                                            videoFrameDuration,
                                            &sampleTime);
                    
                    if (!bFirstSampleHasBeenAdded)
                    {
                        fMediaStartTime = sampleTime;
                        bFirstSampleHasBeenAdded = true;
                    }
			
                    OSErr res1 = GetMoviesError();
                    ThrowOnOSStatus(res1);
                    }

                    // finish movie up
                    ::EndMediaEdits(textMedia);
                    OSErr res2 = GetMoviesError();

                    
                    ThrowOnOSStatus(res2);
                    ::InsertMediaIntoTrack( destTrack,
                                            fMediaStartTime,
                                            fMediaStartTime, // the time where the new media has been added
                                            wholeDuration, // the duration of the new media added
                                            fixed1);

                    res2 = GetMoviesError();
                    
                    ThrowOnOSStatus(res2);

                    if (progDlgManager)
                    {
                        progDlgManager->Delete();
                        delete progDlgManager;
                    }
		}
	
                ::TEDispose(theTEHandle);

                if (theQDErr == noErr)
		{
                    ::SetPort(savePort);
                    ::DisposeGWorld(offscreenGWorld);	
		}
}


void TArmeTextClip::CopyMediaIntoTrack(Track destTrack, ArmeTime startTime, ArmeTime endTime)
{
	OSErr res = 0;
	TimeValue duration = fDoc->TimeValueFromArmeTime(endTime - startTime);
	if (duration > 0)
	{
		TimeValue insertPoint = fDoc->TimeValueFromArmeTime(startTime);
		TimeValue sourceStartTime = fMediaStartTime + fDoc->TimeValueFromArmeTime(fStartTrim + startTime - GetAbsoluteStartTimeOfVideoClip());
		res = InsertTrackSegment(fMasterMovie->GetMasterTrackForText(), destTrack, sourceStartTime, duration, insertPoint);
		ThrowOnOSStatus(res);
	}

#if 0
	TimeValue duration = fDoc->TimeValueFromArmeTime(endTime - startTime);
	if (duration == 0)
		return;

	// The height and the width of the the movie.
	/*short		theMovieHeight = fMasterMovie->GetViewerHeight();//;fMasterMovie->GetHeight();
	short		theMovieWidth = fMasterMovie->GetViewerWidth();//fMasterMovie->GetWidth();*/
	short		theMovieHeight = fMasterMovie->GetHeight();
	short		theMovieWidth = fMasterMovie->GetWidth();

    Rect		textRect = {0, 0, theMovieHeight, theMovieWidth};
    // Unused : TimeScale	trackTimeScale = GetMovieTimeScale(fMasterMovie->GetQTMovie());

    Media textMedia = GetTrackMedia(destTrack);
    MediaHandler textHandler = ::GetMediaHandler(textMedia);
    
    // get background color
	RGBColor blackRGBColor = {0, 0, 0};
	RGBColor background;
	background.red = (short)(0xFFFF * [fTrackBackground redComponent]);
    background.green = (short)(0xFFFF * [fTrackBackground greenComponent]);
    background.blue = (short)(0xFFFF * [fTrackBackground blueComponent]);

   // get flags
    long flags = 0; // dfShrinkTextBoxToFit?

    if (fTextKeyed)
        flags += dfKeyedText;
		
	/*flags += dfClipToTextBox;*/
	flags += dfShrinkTextBoxToFit;
	/*flags += dfDontAutoScale;*/

	MIBackupPasteboard	* theBackupPasteboard = [MIBackupPasteboard backupPasteboard];
	[theBackupPasteboard backup];
	if (fRichText != nil)
            [theBackupPasteboard setData:fRichText forType:NSRTFPboardType];
	else if (fPlainText != nil)
            [theBackupPasteboard setString:fPlainText forType:NSStringPboardType];
        else if (fStyledText != nil)
        {
            NSRange	theRange = NSMakeRange(0, [fStyledText length]);
            NSData * theRTFData = [fStyledText RTFFromRange:theRange documentAttributes:nil];
            [theBackupPasteboard setData:theRTFData forType:NSRTFPboardType];
        }

	Rect	destRect = textRect;
	if ((fScrollType == scrollHorizontal) || (fScrollType == scrollHorizontalReverse))
		destRect.right = 32767;

	//[[MIViewerController sharedViewerController] setMovie:nil];

	// Create a new GWorld where we'll do our text things...
	GrafPtr		savePort;
	GWorldPtr	offscreenGWorld = nil;
	QDErr		theQDErr = ::NewGWorld (&offscreenGWorld, 0, &textRect, NULL, NULL, 0);
	if (theQDErr == noErr)
        {
            ::GetPort(&savePort);
            ::SetGWorld(offscreenGWorld, NULL);
        }

	// Create the TEHandle.
	TEHandle theTEHandle = ::TEStyleNew(&destRect, &textRect);
	// Don't wrap the text in case of horizontal scrolling.
	if ((fScrollType == scrollHorizontal) || (fScrollType == scrollHorizontalReverse))
            (**theTEHandle).crOnly = -1;
	// Paste the text into the TEHandle.
	::TEStylePaste(theTEHandle);

	[theBackupPasteboard retrieve];
	[theBackupPasteboard release];

	// Get the Height of the text.
	long theTextHeight = ::TEGetHeight(32767, 0, theTEHandle);

	// Find the longuest line length of the text.
	int	theTextWidth = 0;
	for (int i = 0 ; i < (**theTEHandle).nLines ; i++)
		{
		::TESetSelect((**theTEHandle).lineStarts[i], (**theTEHandle).lineStarts[i + 1], theTEHandle);
		if (((**theTEHandle).selRect.right) - ((**theTEHandle).selRect.left) > theTextWidth)
			theTextWidth = (**theTEHandle).selRect.right - (**theTEHandle).selRect.left;
		}

	if ((fScrollType == scrollHorizontal) || (fScrollType == scrollHorizontalReverse) || fScrollType == noScroll)
		switch (fVerticalAlignment)
			{
			case MITopTextAlignment :
				break;
			case MICenterTextAlignment :
				textRect.top = theMovieHeight/2 - theTextHeight/2;
				textRect.bottom = textRect.top + theTextHeight;
				break;
			case MIBottomTextAlignment :
				textRect.bottom = theMovieHeight;
				textRect.top = textRect.bottom - theTextHeight;
				break;
			}

	short	textJustify = fTextJustify;
	if ((fScrollType == scrollHorizontal) || (fScrollType == scrollHorizontalReverse))
		textJustify = teFlushLeft;
	
    // add text sample
    TimeValue sampleTime = 0;

	if (fScrollType == noScroll)
		{
		::BeginMediaEdits(textMedia);

		::TextMediaAddTESample(	textHandler,
								theTEHandle,
								&background,
								textJustify,
								&textRect,
								flags,        
								0,/*scrollDelay,*/
								0,
								0,
								&blackRGBColor,
								duration,
								&sampleTime);
		
		OSErr res1 = GetMoviesError();

		// finish movie up
		::EndMediaEdits(textMedia);
		OSErr res2 = GetMoviesError();
		ThrowOnOSStatus(res1);
		ThrowOnOSStatus(res2);
		::InsertMediaIntoTrack(	destTrack,
								fDoc->TimeValueFromArmeTime(startTime),
								sampleTime, // the time where the new media has been added
								duration, // the duration of the new media added
								fixed1);
		res2 = GetMoviesError();
		ThrowOnOSStatus(res2);
		}
	else
		{
		// Frame duration.
		TimeValue	videoFrameDuration = fDoc->GetVideoFrameDuration();
	
		// Absolute times.
		ArmeTime	theAbsoluteStart = GetAbsoluteStartTimeOfVideoClip() - fStartTrim;
		ArmeTime	theAbsoluteEnd = GetAbsoluteEndxTimeOfVideoClip() + fEndTrim;
		ArmeTime	theAbsoluteLength = theAbsoluteEnd - theAbsoluteStart;
	
		// Project Frames per second.
		float		projectFPS = fDoc->fMasterMovie->GetFPS();
	
		// Number of frames.
		long		theNumberOfFrames = (long)ceil((fDoc->ArmeTimeToSeconds(endTime - startTime) * projectFPS) - 0.5);
	
		// Percentage of this part from the total time of the text.
		float		thePercentage = float(endTime.data - startTime.data) / float(theAbsoluteLength.data);
	
		// The current height and width;
		short		theRectHeight = short((theMovieHeight + theTextHeight) * thePercentage);
		short		theRectWidth = short((theMovieWidth + theTextWidth) * thePercentage);
		
		// Increment of each frame in pixels.
		float		theVPixelsIncrement = float(theRectHeight) / float(theNumberOfFrames);
		float		theHPixelsIncrement = float(theRectWidth) / float(theNumberOfFrames);
	
		/*if (theVPixelsIncrement < 1.0)
			NSBeep();
		if (theHPixelsIncrement < 1.0)
			NSBeep();*/
	
		// The starting text box rect.
		float		theStartFrame;
		if ((fScrollType == scrollVertical) || (fScrollType == scrollHorizontal))
			theStartFrame = ceil((fDoc->ArmeTimeToSeconds(theAbsoluteEnd - endTime) * projectFPS) - 0.5);
		else
			if ((fScrollType == scrollVerticalReverse) || (fScrollType == scrollHorizontalReverse))
				theStartFrame = ceil((fDoc->ArmeTimeToSeconds(endTime - theAbsoluteStart) * projectFPS) - 0.5);
	
		short		startTop = short(theStartFrame * theVPixelsIncrement);
		short		startLeft = short(theStartFrame * theHPixelsIncrement);
		
		MIProgressBarDlgManager* progDlgManager = NULL; 

		if ((fScrollType != noScroll) && (duration > GetDoc()->TimeValueFromArmeTime(_ArmeTime(300000))))
			progDlgManager = new MIProgressBarDlgManager("Creating Text Clip...");

		for (int i = theNumberOfFrames ; i >= 0 ; i++)  // invert the loop because everything is inverted since I introduced the optimisation
//		for (int i = 0 ; i <= theNumberOfFrames ; i++)
			{
			if (progDlgManager)
				progDlgManager->Update(float(i-theNumberOfFrames) / theNumberOfFrames);  

			::SetGWorld(offscreenGWorld, NULL);
			if (fScrollType == scrollVertical)
				{
				textRect.top = (startTop + short(i * theVPixelsIncrement))  - theTextHeight;
				textRect.bottom = textRect.top + theTextHeight;
				}
			else
				if (fScrollType == scrollVerticalReverse)
					{
					textRect.top = (startTop - short(i * theVPixelsIncrement)) - theTextHeight;
					textRect.bottom = textRect.top + theTextHeight;
					}
				else
					if (fScrollType == scrollHorizontal)
						{
						textRect.left = (startLeft + short(i * theHPixelsIncrement)) - theTextWidth;
						textRect.right = textRect.left + theTextWidth;
						}
					else
						if (fScrollType == scrollHorizontalReverse)
							{
							textRect.left = (startLeft - short(i * theHPixelsIncrement)) - theTextWidth;
							textRect.right = textRect.left + theTextWidth;
							}
	
			::BeginMediaEdits(textMedia);

			::TextMediaAddTESample(	textHandler,
									theTEHandle,
									&background,
									textJustify,
									&textRect,
									flags,        
									0,/*scrollDelay,*/
									0,
									0,
									&blackRGBColor,
									videoFrameDuration,
									&sampleTime);
			
			OSErr res1 = GetMoviesError();
	
			// finish movie up
			::EndMediaEdits(textMedia);

			OSErr res2 = GetMoviesError();
			ThrowOnOSStatus(res1);
			ThrowOnOSStatus(res2);
			::InsertMediaIntoTrack(	destTrack,
									fDoc->TimeValueFromArmeTime(startTime),
									sampleTime, // the time where the new media has been added
									videoFrameDuration, // the duration of the new media added
									fixed1);
			OSErr res2 = GetMoviesError();
			ThrowOnOSStatus(res1);
			}
			if (progDlgManager)
			{
				progDlgManager->Delete();
				delete progDlgManager;
			}
		}
	
	::TEDispose(theTEHandle);

	if (theQDErr == noErr)
		{
		::SetPort(savePort);
		::DisposeGWorld(offscreenGWorld);	
		}

	//[[MIViewerController sharedViewerController] show:nil];
	#endif //0
}

// GP
void	TArmeTextClip::Move()
{
	//Make sure the absoluteEndTime of the text clip will be on a frame boundary
/*	ArmeTime endTime = GetAbsoluteEndxTimeOfVideoClip();
	fDoc->RoundArmeTimeToNearestVideoFrame(endTime);
	fWholeTextClipDuration = endTime - GetAbsoluteStartTimeOfVideoClip() + fEndTrim + fStartTrim;*/
	
	RoundTimingInfo();
}

bool	TArmeTextClip::ContainsVideoAtTime(ArmeTime t) const
{
	ArmeTime	start, end;
	start = GetAbsoluteStartTimeOfVideoClip();
	end = GetAbsoluteEndxTimeOfVideoClip();
	
	if (IsVisible())
		if (t != ARME_TIME_UNDEF)
			return (start <= t) && (end >= t);

	return false;
	// GP : TO DO!
	//return true;
}

bool	TArmeTextClip::ContainsVideoDuring(ArmeTime startTime, ArmeTime endTime, bool bConsiderAlsoMutedObjects, bool bConsiderAlsoTransparentTextClips) const
{
	//it it is outtaHierarchy
	if (GetPos().fParent == NULL)
		return false;
	
	if (!bConsiderAlsoTransparentTextClips && IsKeyed())
		return false;
	
	if (bConsiderAlsoMutedObjects || IsVisible())
		{
		if ((startTime != ARME_TIME_UNDEF) && (endTime != ARME_TIME_UNDEF))
			{
			if (GetAbsoluteStartTimeOfVideoClip(bConsiderAlsoTransparentTextClips) >= startTime) 
				{
				if (GetAbsoluteStartTimeOfVideoClip(bConsiderAlsoTransparentTextClips) < endTime)
					return true;
				}
			else
				if (GetAbsoluteEndxTimeOfVideoClip(bConsiderAlsoTransparentTextClips) > startTime)
					return true;
			}
		else
			if ((startTime == ARME_TIME_UNDEF) && (endTime != ARME_TIME_UNDEF))
				{
				if (GetAbsoluteStartTimeOfVideoClip(bConsiderAlsoTransparentTextClips) < endTime)
					return true;
				}
			else
				if ((startTime != ARME_TIME_UNDEF) && (endTime == ARME_TIME_UNDEF))
					{
					if (GetAbsoluteEndxTimeOfVideoClip(bConsiderAlsoTransparentTextClips) > startTime)
						return true;
					}
				else
					return true; //both startTime and endTime are undef
		}

	return false;
}
// GP

ArmeTime TArmeTextClip::GetDuration() const
{
	return fWholeTextClipDuration - fStartTrim - fEndTrim;
}

NSAttributedString *	TArmeTextClip::GetStyledText()
{
	if ((fStyledText != nil) && ([fStyledText length] != 0))
		return [fStyledText copy];
	else
		if ((fPlainText != nil) && ([fPlainText length] != 0))
			return [[NSAttributedString alloc] initWithString:fPlainText];
	return nil; // GP : BUG?
}

void TArmeTextClip::SetMemento(TArmeWSObjMemento* memento)
{
	TArmeTextClipMemento* textMemento = dynamic_cast<TArmeTextClipMemento*>(memento);
	ZzAssert(textMemento);

	//remove representation of the clip in the master movie
	fMasterMovie->RemoveSubMovie(this);

	//copy the parameters in the current clip
	fScrollType = textMemento->fScrollType;
	fTextKeyed = textMemento->fTextKeyed;
	[fTrackBackground release];
	fTrackBackground = [textMemento->fTrackBackground copy];
	fWholeTextClipDuration = textMemento->fWholeTextClipDuration;
	fTextJustify = textMemento->fTextJustify;
	
	[fRichText release];
	fRichText = [textMemento->fRichText copy];
	[fPlainText release];
	fPlainText = [textMemento->fPlainText copy];
	[fStyledText release];
	fStyledText = [textMemento->fStyledText copy];
	fVerticalAlignment = textMemento->fVerticalAlignment;
	fMediaStartTime = textMemento->fMediaStartTime;

	// insert the new representation of the clip in the master movie  (note, there is _SetPos in there so, the clip will be added to MasterMovie)
	TVideoObject::SetMemento(memento);
}

TArmeWSObjMemento* TArmeTextClip::GetMemento() const
{
	TArmeWSObjMemento* memento = new TArmeTextClipMemento();
	_FillMemento(memento);
	return memento;
}

void TArmeTextClip::_FillMemento(TArmeWSObjMemento* memento) const
{
	TArmeTextClipMemento* textMemento = dynamic_cast<TArmeTextClipMemento*>(memento);
	ZzAssert(textMemento);

	TVideoObject::_FillMemento(memento);
	
	//copy the parameters of the current clip
	textMemento->fScrollType = fScrollType;
	textMemento->fTextKeyed = fTextKeyed;
	textMemento->fTrackBackground = [fTrackBackground copy];
	textMemento->fWholeTextClipDuration = fWholeTextClipDuration;
	textMemento->fTextJustify = fTextJustify;
	
	textMemento->fRichText = [fRichText copy];
	textMemento->fPlainText = [fPlainText copy];
	textMemento->fStyledText = [fStyledText copy];
	textMemento->fVerticalAlignment = fVerticalAlignment;
	textMemento->fMediaStartTime = fMediaStartTime;
}



TArmeTextClipMemento::TArmeTextClipMemento() 
	: TVideoObjectMemento(),
	fTrackBackground(NULL),
	fRichText(NULL),
	fPlainText(NULL),
	fStyledText(NULL)
{
}

TArmeTextClipMemento::TArmeTextClipMemento(ArmeArchiveHandler* unarchiver) : TVideoObjectMemento(unarchiver)
{
	fScrollType = unarchiver->ReadSInt32();
	fTextKeyed = unarchiver->ReadBool();
	fTrackBackground = [unarchiver->ReadObjCObject() retain];
	fWholeTextClipDuration = unarchiver->ReadArmeTime();
	fTextJustify = unarchiver->ReadSInt32();
	
	fRichText = [unarchiver->ReadObjCObject() retain];
	fPlainText = [unarchiver->ReadObjCObject() retain];
	fStyledText = [unarchiver->ReadObjCObject() retain];
	
	fVerticalAlignment = unarchiver->ReadSInt32();
	
	if (unarchiver->GetFileVer() >= 0x0001002F)
		fMediaStartTime = unarchiver->ReadSInt32();
	else
		fMediaStartTime = 0;

}

bool TArmeTextClipMemento::WriteToArchive(ArmeArchiveHandler* archiver) const
{
	TVideoObjectMemento::WriteToArchive(archiver);

	archiver->WriteSInt32(fScrollType);
	archiver->WriteBool(fTextKeyed);
    archiver->WriteObjCObject(fTrackBackground);
	archiver->WriteArmeTime(fWholeTextClipDuration);
	archiver->WriteSInt32(fTextJustify);
	
	archiver->WriteObjCObject(fRichText);
	archiver->WriteObjCObject(fPlainText);
	archiver->WriteObjCObject(fStyledText);

	archiver->WriteSInt32(fVerticalAlignment);

	//from version 0x0001002F
	archiver->WriteSInt32(fMediaStartTime);

	return true;
}

TArmeTextClipMemento::~TArmeTextClipMemento()
{
	[fTrackBackground release];
	[fRichText release];
	[fStyledText release];
	[fPlainText release];
}
