/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#pragma once

#define TArmeQTFileHandler_h

#ifndef VArmeMediaFileHandler_h
#include "VArmeMediaFileHandler.h"
#endif //VArmeMediaFileHandler_h

/*!
	@class TArmeTextFileHandler
	@abstract This class is a subclass of VArmeMediaFileHandler that handles text (TXT and RTF) files
	@discussion See the VArmeMediaFileHandler class for a description of the functions.
*/
class ZzSplitTask;
class MIProgressBarDlgManager;

class TArmeTextFileHandler : public VArmeMediaFileHandler
{
public :
	virtual ~TArmeTextFileHandler() {};

        // call CanHandleFile instead, which asks QuickTime directly
	virtual int32		GetSupportedExtensionsCount();
	virtual char*		GetSupportedExtension(int32 i);
	virtual int32		GetSupportedTypesCount();
	virtual char*		GetSupportedType(int32 i);
    
        
	virtual bool		CanHandleFileByLookingAtExtension( const ZzString* path );
	virtual bool		CanHandleFileByLookingAtType( const ZzString* path );
	virtual bool		CanHandleRTFFileByLookingAtExtension( const ZzString* zzPath );
	virtual bool		CanHandleRTFFileByLookingAtType( const ZzString* zzPath );
	virtual bool		CanHandleTextFileByLookingAtExtension( const ZzString* zzPath );
	virtual bool		CanHandleTextFileByLookingAtType( const ZzString* zzPath );
       
	bool				CanHandleRTFFile( const ZzString* path );
	bool				CanHandleTextFile( const ZzString* path );
		  
	virtual ArmeErr		CreateWSObjsFromFile( 
							TArmeDoc* 					pDoc,
							ZzArchPtrArray*				theObjects,
							const ZzString*				cstrPath,
							ArmeDataFileRef*			fileRef,
							const TArmeWSObjPos&		pos,
							MIProgressBarDlgManager*	progressMngr);

/*
	virtual TArmeWSObj*	CreateWSObjFromFile( 
							TArmeDoc* 					pDoc, 
						//	const char*					path, 
							const ZzString*				cstrPath,
							const TArmeWSObjPos&		pos,
							MIProgressBarDlgManager*	progressMngr,
							ArmeErr*					err);
*/
							
};


