/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#ifndef TArmeWSObj_h
#include "TArmeWSObj.h"
#endif //TArmeWSObj_h

#ifndef TArmeMediaEngine_h
#include "TArmeMediaEngine.h"
#endif //TArmeMediaEngine_h

#ifndef TArmeDoc_h
#include "TArmeDoc.h"
#endif //TArmeDoc_h

#ifndef TARPIHost_h
#include "TARPIHost.h"
#endif //TARPIHost_h

#ifndef VARPIPlugin_h
#include "VARPIPlugin.h"
#endif VARPIPlugin_h

#ifndef TArmeMasterMovie_h
#include "TArmeMasterMovie.h"
#endif //TArmeMasterMovie_h

#ifndef MovieHelper_h
#include "MovieHelper.h"
#endif //MovieHelper_h

#ifndef TVideoClip_h
#include "TVideoClip.h"
#endif //TVideoClip_h

#ifndef MECoreAudioPlayer_h
#include "MECoreAudioPlayer.h"
#endif MECoreAudioPlayer_h

#ifndef TVideoEffect_h
#include "TVideoEffect.h"
#endif //TVideoEffect_h

#ifndef ZzSplitTask_h
#include "ZzSplitTask.h"
#endif //ZzSplitTask_h

#ifndef ArmeNotifications_h
#include "ArmeNotifications.h"
#endif //ArmeNotifications_h

#ifndef ZzHelpers_h
#include "ZzHelpers.h"
#endif //ZzHelpers_h

#ifndef ZzDisableVerbose_h
#include "ZzDisableVerbose.h"
#endif //ZzDisableVerbose_h

#ifndef TArmeMediaEngine_h
#include "TArmeMediaEngine.h"
#endif //TArmeMediaEngine_h

#ifndef ARPI_h
#include "ARPI.h"
#endif //ARPI_h

#ifndef MEApplicationServices_h
#include "MEApplicationServices.h"
#endif //MEApplicationServices_h

#ifndef MIProgressBarDlgManager_h
#include "MIProgressBarDlgManager.h"
#endif //MIProgressBarDlgManager_h

#ifndef Zz_PathHelpers_h
#include "Zz_PathHelpers.h"
#endif //Zz_PathHelpers_h

TArmeWSObj::TArmeWSObj(const char* name, TArmeDoc* doc, VARPIPluginClass* audioPluginClass) :
	fDoc(doc)
{
	SetNameCopy(name);
	
	fMuted = false;
	fAudioGain = 1.0;
	fOldAudioGain = 0.0;
	fMIObjectRefCount = 0;
			
	fChildren = new ZzArchPtrArray;
	// $$$TEST : roughly 10 seconds
	fLength = _ArmeTime(400000);
	// the object doesn't have a real parent yet
	fPos.fParent = NULL;
	fPos.fInput = 0;
	fPos.fStartTime = _ArmeTime(0);	
	fPos.fVerticalPos = 0;
	
	fStillImageCounter = 0;
	fCopyCounter = 0;
	
	InvalidateCachedTimeInfo();
	
	// initialize plugins
	fRealTimePlugin = NULL;
	fOfflinePlugin = NULL;
	fPluginPreset = NULL;

#ifdef AUTOMATION_DATA
	fAudioAutomationData = NULL;
#endif	
	// we only init the first stereo plugin. the other ones will be created on demand
	if( audioPluginClass != NULL )
	{
		SetAudioPlugin(TArmeMediaEngine::ARPIHost()->CreatePlugin(audioPluginClass));
		SetPresetAsFirstFromFactory();
		SyncPluginFromPreset();
	}
		
	// default routing
	fVAORouting.index = 0;
	fVAORouting.bStereo = true;
	NotifyVAORoutingChange();
}

TArmeWSObj::TArmeWSObj(const char* name)
{
	SetNameCopy(name);
	fDoc = NULL;
	fLength = _ArmeTime(400000);
	
	fMuted = false;
	fAudioGain = 1.0;
	fOldAudioGain = 0.0;
	fMIObjectRefCount = 0;

	// initialize plugins
	fRealTimePlugin = NULL;
	fOfflinePlugin = NULL;
		
#ifdef AUTOMATION_DATA
	fAudioAutomationData = NULL;
#endif
	// the object doesn't have a real parent yet
	fPos.fParent = NULL;
	fPos.fInput = 0;
	fPos.fStartTime = _ArmeTime(0);	
	fPos.fVerticalPos = 0;

	fStillImageCounter = 0;
	fCopyCounter = 0;
	
	fChildren = new ZzArchPtrArray;
	
	fPluginPreset = NULL;
	
	InvalidateCachedTimeInfo();
	
	// default routing
	fVAORouting.index = 0;
	fVAORouting.bStereo = true;
	NotifyVAORoutingChange();
}

TArmeWSObjMemento* TArmeWSObj::GetMemento() const
{
	TArmeWSObjMemento* memento = new TArmeWSObjMemento();
	_FillMemento(memento);
	return memento;
}

void TArmeWSObj::_FillMemento(TArmeWSObjMemento* memento) const
{
	memento->fPos = fPos;
	memento->fLength = fLength;
	memento->fMuted = IsMuted();
	memento->fStillImageCounter = fStillImageCounter;
	memento->fCopyCounter = fCopyCounter;
}

bool TArmeWSObj::WriteToArchive(ArmeArchiveHandler* archiver) const
{
	ArmeArchivable::WriteToArchive(archiver);
	archiver->WriteRawData(fName,64);
	archiver->WriteBool(fMuted);
	archiver->WriteCPPObject(fChildren);
	archiver->WriteCPPObject(fPos.fParent);
	archiver->WriteSInt32(fPos.fInput);
	archiver->WriteArmeTime(fPos.fStartTime);
	archiver->WriteSInt32(fPos.fVerticalPos);
	archiver->WriteCPPObject(fDoc);
	archiver->WriteArmeTime(fLength);
#if 1 
	if(fPluginPreset)
	{
		archiver->WriteSInt32(1); // means there is an instance of fPluginPreset in the arch
		fRealTimePlugin->GetClass()->GetSig().WriteToArchive(archiver);
		fPluginPreset->WriteToArchive(archiver);
	}
	else
	{
		archiver->WriteSInt32(0);
	}
#else
/*

	VARPIPlugin* firstPlugin = fStereoAudioPlugins[0];
	// if there us a plugin, we first write the nb of params, then 
	// we write the plugin class sig, then we write the actual parameters

	if(firstPlugin)
	{
		SInt32 nbParams = fStereoAudioPlugins[0]->GetClass()->GetNbParams();
		archiver->WriteSInt32(nbParams);
		fStereoAudioPlugins[0]->GetClass()->GetSig().WriteToArchive(archiver);
		for(SInt32 i = 0; i < nbParams; i++)
		{
			archiver->WriteFloat(fStereoAudioPlugins[0]->GetParamValue(i));
		}
	}
	// otherwise we simply write -1
	else
	{
		archiver->WriteSInt32(-1);
	}
*/
#endif //#if 1

#ifdef AUTOMATION_DATA
	fAudioAutomationData->WriteToArchive( archiver );
#endif

	archiver->WriteUInt32(fStillImageCounter);
	archiver->WriteUInt32(fCopyCounter);
	
	archiver->WriteSInt32(fVAORouting.index);
	archiver->WriteSInt32(fVAORouting.bStereo);

	return true;
}

TArmeWSObj::TArmeWSObj(ArmeArchiveHandler* unarchiver) :
	ArmeArchivable(unarchiver)
{
	fMIObjectRefCount = 0;

	ArmeArchivable* archObj;
	unarchiver->ReadRawData(fName,64);
    fName[63] = 0;
	fMuted = unarchiver->ReadBool();
	fAudioGain = fMuted ? 0.0 : 1.0;
	fOldAudioGain = 0.0;
	archObj = unarchiver->ReadCPPObject();
	fChildren = dynamic_cast<ZzArchPtrArray*>(archObj);
	if(archObj)
	{
		ZzAssert(fChildren);
	}
		
	archObj = unarchiver->ReadCPPObject();
	fPos.fParent = dynamic_cast<TArmeWSObj*>(archObj);
	if(archObj)
	{
		ZzAssert(fPos.fParent);
	}
	
	fPos.fInput = unarchiver->ReadSInt32();
	fPos.fStartTime = unarchiver->ReadArmeTime();
	fPos.fVerticalPos = unarchiver->ReadSInt32();
	archObj = unarchiver->ReadCPPObject();
	fDoc = dynamic_cast<TArmeDoc*>(archObj);
	if(archObj)
	{
		ZzAssert(fDoc);
	}
	
	fLength = unarchiver->ReadArmeTime();
	
	fRealTimePlugin = NULL;
	fOfflinePlugin = NULL;
	fPluginPreset = NULL;
	
	SInt32 pluginPresetAvailable = unarchiver->ReadSInt32();
	if(pluginPresetAvailable == 1)
	{
		fPluginPreset = new ARPIBlueWindowPreset;
		
		VARPIPluginClassSig audioPluginClassSig;

		// read the class sig (see the VARPIPluginClassSig class in VARPIPluginClass.h)
		audioPluginClassSig.ReadFromArchive(unarchiver);
		
		// read the preset
		fPluginPreset->ReadFromArchive(unarchiver);
		
		// try to create the class from the sig
		VARPIPluginClass* pluginClass = TArmeMediaEngine::ARPIHost()->GetAudioPluginClassFromSig(audioPluginClassSig);
		// if the class we found, create a plugin and set it
		if(pluginClass)
		{
			VARPIPlugin* plugin = pluginClass->CreatePlugin();
			SetAudioPlugin(plugin); // format changes are handled by this call
			SyncPluginFromPreset();  
		}
		// otherwise, display a message box to complain about the missing plugin
		else
		{
			TArmeMediaEngine::ApplicationServices()->ComplainAboutMissingAudioPlugin(audioPluginClassSig);
		}
	}
	else if(pluginPresetAvailable == 0)
	{
	}
	else
	{
		ZzAssert(0);
	}

#ifdef AUTOMATION_DATA
	fAudioAutomationData = new MEAudioPluginAutomationData( GetAudioPluginClass()->GetNbParams() );
	fAudioAutomationData->ReadFromArchive( unarchiver );
#endif

	fStillImageCounter = unarchiver->ReadUInt32();
	fCopyCounter = unarchiver->ReadUInt32();
	
	fVAORouting.index = unarchiver->ReadSInt32();
	fVAORouting.bStereo = unarchiver->ReadSInt32();
	
	InvalidateCachedTimeInfo();
	
	NotifyVAORoutingChange();
}

void TArmeWSObj::SetMemento(TArmeWSObjMemento* memento)
{
	fLength = memento->fLength;

	InvalidateCachedTimeInfo();
	_SetPos(memento->fPos);
	SetMuted(memento->fMuted);
	fStillImageCounter = memento->fStillImageCounter;
	fCopyCounter = memento->fCopyCounter;
}

TArmeWSObj* TArmeWSObj::GetCopy(const char* tempDirectoryName, MIProgressBarDlgManager* progressMngr) const
{
	return GetSliceCopy(ARME_TIME_MIN, ARME_TIME_MAX, tempDirectoryName, progressMngr);
}

TArmeWSObj* TArmeWSObj::CutSlice(ArmeTime startTime, ArmeTime endTime, const char* tempDirectoryName, MIProgressBarDlgManager* progressMngr)
{
	TArmeWSObj* slice = GetSliceCopy(startTime, endTime, tempDirectoryName, progressMngr);
	DeleteSlice(startTime, endTime, progressMngr);
	return slice;
}

TArmeWSObj* TArmeWSObj::Cut(const char* tempDirectoryName, MIProgressBarDlgManager* progressMngr)
{
	return CutSlice(ARME_TIME_UNDEF, ARME_TIME_UNDEF, tempDirectoryName, progressMngr);
}

TArmeWSObj* TArmeWSObj::GetSliceCopy(ArmeTime startTime, ArmeTime endTime, const char* tempDirectoryName, MIProgressBarDlgManager* progressMngr) const
{
	ZzAssert(0);
	return NULL;
}

ArmeTime TArmeWSObj::InsertSlice(ArmeTime insertTime, const TArmeWSObj* obj, MIProgressBarDlgManager* progressMngr)
{
	ZzAssert(0);
	
	return ARME_TIME_UNDEF;
}

bool TArmeWSObj::DeleteSlice(ArmeTime startTime, ArmeTime endTime, MIProgressBarDlgManager* progressMngr)
{
	ZzAssert(0);

	//returns true if the whole object has been deleted
	return false;
}


void TArmeWSObj::_AddChild( TArmeWSObj* child )
{
	ZzAssert(child);
	ArmeArchivable* archChild = dynamic_cast<ArmeArchivable*>(child);
	ZzAssert(archChild);
	fChildren->AddItem(archChild);
}

void TArmeWSObj::_InsertChild( TArmeWSObj* child, int index)
{
	ZzAssert(child);
	ArmeArchivable* archChild = dynamic_cast<ArmeArchivable*>(child);
	ZzAssert(archChild);
	fChildren->InsertItem(archChild, index);
}

int TArmeWSObj::_GetIndexOfChild( TArmeWSObj* child )
{
	return fChildren->FindIndexForItem(child);
}

void TArmeWSObj::_RemoveChild( TArmeWSObj* child )
{
	ZzAssert(child);
	ArmeArchivable* archChild = dynamic_cast<ArmeArchivable*>(child);
	ZzAssert(archChild);
	fChildren->RemoveItemByValue(archChild);
}

TArmeWSObj* TArmeWSObj::GetChild(int32 i) const
{
	ArmeArchivable* archChild = fChildren->ItemAt(i);
	ZzAssert(archChild);
	TArmeWSObj* child = dynamic_cast<TArmeWSObj*>(archChild);
	ZzAssert(child);
	return child;
}

SInt32 TArmeWSObj::GetChildrenCount() const
{
	return fChildren->CountItems();
}

TArmeWSObj* TArmeWSObj::FindChildRecursiveByName(const char* name)
{
	SInt32 i;
	TArmeWSObj* grandChild;
//	TArmeWSObj* child;
	
	if( strcmp(GetName(), name) == 0 )
		return this;
	
	for(i = 0; i < GetChildrenCount(); i++)
	{
		grandChild = GetChild(i)->FindChildRecursiveByName(name);
		if( grandChild )
			return grandChild;
	}
	return NULL;
}

void TArmeWSObj::ResetRecursive()
{
	for (SInt32 i=0; i<GetChildrenCount(); i++) 
		GetChild(i)->ResetRecursive();
	if(fRealTimePlugin)
		fRealTimePlugin->Reset();
}


ArmeTime TArmeWSObj::GetDuration() const
{
	return fLength;
}


bool TArmeWSObj::IsTheVideoSolo() const
{
	return (this == GetDoc()->GetVideoSoloObject());
}

bool TArmeWSObj::IsTheAudioSolo() const
{
	return (this == GetDoc()->GetAudioSoloObject());
}

bool TArmeWSObj::IsInSolo() const
{
	if ((!GetDoc()->GetVideoSoloObject()) && (!GetDoc()->GetAudioSoloObject()))
		return false;
	else
	{
		if (GetDoc()->GetVideoSoloObject())
		{
			if (IsDescendantOf(GetDoc()->GetVideoSoloObject()))
				return true;
		}
		if (GetDoc()->GetAudioSoloObject())
		{
			if (IsDescendantOf(GetDoc()->GetAudioSoloObject()))
				return true;
		}
	}
	return false;
}

bool TArmeWSObj::IsDescendantOf(const TArmeWSObj* grandParent) const
{
	if (grandParent == this)
		return true;
	else
	{
		if (fPos.fParent)
			return fPos.fParent->IsDescendantOf(grandParent);
		else
			return false;
	}
}

			
bool TArmeWSObj::IsVideoObject() const
{
	switch (GetType())
	{
		case kVideoClip :
		case kVideoEffect : 
		case kContainer : 
			return true;
		case kTextClip :
		default :
			return false;
	}
}

int TArmeWSObj::GetVideoChildrenCount() const
{
	int count = 0;
	
	for (int i=0; i < GetChildrenCount(); ++i)
	{
		if (GetChild(i)->IsVideoObject())
			count++;
	}

	return count;
}

TArmeWSObj::~TArmeWSObj()
{
	if (fPos.fParent)
	{
		fPos.fParent->_RemoveChild(this);
	}

	for(SInt32 i = 0; i < GetChildrenCount(); i++)
	{
		delete GetChild(i);
	}
	if(fRealTimePlugin)
		delete fRealTimePlugin;
	ZzAssert(fOfflinePlugin == NULL);
	delete fChildren;
}

ArmeTime TArmeWSObj::GetAbsoluteStartTime() const
{
	if (fCachedAbsoluteStartTime != ARME_TIME_UNDEF)
		return fCachedAbsoluteStartTime;

	if( GetType() == TArmeWSObj::kTLC )
		return _ArmeTime(0);
	
	if (GetPos().fParent == NULL)
		return GetPos().fStartTime;

	return fCachedAbsoluteStartTime = _ArmeTime( GET_VAL(GetPos().fStartTime) + GET_VAL(GetPos().fParent->GetAbsoluteStartTime()));
}
	
void TArmeWSObj::_SetLength( ArmeTime len )
{
	verntl(armeWSObjsBenjamin,6,"TArmeWSObj::_SetLengh(" << (int)GET_VAL(len) << ")");
	ZzAssert(GET_VAL(len) > 0);
	ZzAssert(GetDoc());

//	GetDoc()->LockWrite();  //problem : this freeze the app because setLength is called on the transitions (and it is called inside a _SetPos that has already locked the resource)
	fLength = len;
//	GetDoc()->UnlockWrite();
}

void TArmeWSObj::PosWillChange()
{

}

void TArmeWSObj::PosHasChanged()
{

}

void TArmeWSObj::_NotifyChildPosHasChanged()
{
	InvalidateCachedTimeInfo();
	
	if (fPos.fParent)
		fPos.fParent->_NotifyChildPosHasChanged();
}

void TArmeWSObj::_NotifyParentPosHasChanged()
{
	InvalidateCachedTimeInfo();

	for( int i = 0; i < fChildren->CountItems(); i++ )
	{
		GetChild(i)->_NotifyParentPosHasChanged();
	}
}

#ifdef OLD_VAO_RAO
void TArmeWSObj::SetAudioPlugin(VARPIPlugin* plugin)
{
	ZzAssert(fStereoAudioPlugins[0] == NULL);
	fStereoAudioPlugins[0] = plugin;

#ifdef AUTOMATION_DATA
	fAudioAutomationData = new MEAudioPluginAutomationData( GetAudioPluginClass()->GetNbParams() );
#endif
	
	ZzAssert(fPluginPreset == NULL);
	
	fPluginPreset = GetAudioPluginClass()->CreatePresetFromFactoryPreset(0);

	// we got the preset from the class, now set the plugin params according to this preset
	for(int param = 0; param < GetAudioPluginClass()->GetNbParams(); param++)
	{
		float value = fPluginPreset->GetParamValue(param);
		SetAudioPluginParamValue(param,value);
	}
	
	if(plugin)
	{
		fStereoAudioPlugins[0]->SetFrameRate(TArmeMediaEngine::CoreAudioPlayer()->GetAudioFrameRate());
		fStereoAudioPlugins[0]->SetBufferNbFrames(TArmeMediaEngine::CoreAudioPlayer()->OutDeviceBufferNbFrames());
		// send notification
		fStereoAudioPlugins[0]->FormatChange();
	}
}
#else //#ifdef OLD_VAO_RAO
void TArmeWSObj::SetAudioPlugin(VARPIPlugin* plugin)
{
	ZzAssert(fRealTimePlugin == NULL);
	fRealTimePlugin = plugin;

#ifdef AUTOMATION_DATA
	fAudioAutomationData = new MEAudioPluginAutomationData( GetAudioPluginClass()->GetNbParams() );
#endif
	
	if(plugin)
	{
		fRealTimePlugin->SetFrameRate(TArmeMediaEngine::CoreAudioPlayer()->GetAudioFrameRate());
		fRealTimePlugin->SetBufferNbFrames(TArmeMediaEngine::CoreAudioPlayer()->OutDeviceBufferNbFrames());
		// send notification
		fRealTimePlugin->FormatChange();
	}
}
#endif //#ifdef OLD_VAO_RAO

void TArmeWSObj::SetPresetAsFirstFromFactory()
{
	if(fPluginPreset)
		delete fPluginPreset;
	fPluginPreset = GetAudioPluginClass()->CreatePresetFromFactoryPreset(0);
}

void TArmeWSObj::SyncPluginFromPreset()
{
	for(int param = 0; param < GetAudioPluginClass()->GetNbParams(); param++)
	{
		float value = fPluginPreset->GetParamValue(param);
		SetAudioPluginParamValue(param,value);
	}
}		

void TArmeWSObj::SetVerticalPos(SInt32 pos)
{
	fPos.fVerticalPos = pos;
}

void TArmeWSObj::_SetPos( const TArmeWSObjPos& pos )
{
	TArmeWSObj* oldParent = NULL;
	
	PosWillChange();

	ZzAssert(GetDoc());
	GetDoc()->LockWrite();
	
	// if we change our parent and if the old parent is not null,
	// let's tell the old parent we leave him
	if( (pos.fParent != fPos.fParent) && (fPos.fParent != NULL) )
	{
		fPos.fParent->_RemoveChild(this);
		oldParent = fPos.fParent;
	}

	// if we change our parent and if the new parent is not null,
	// let's tell the new parent we join him
	if( ( pos.fParent != fPos.fParent ) && (pos.fParent != NULL))
	{
		pos.fParent->_AddChild(this);
		//_NotifyAddedToParent();
	}

	// let's copy the new pos
	fPos = pos;
	InvalidateCachedTimeInfo();

	GetDoc()->UnlockWrite();

	// tell to all the children that this position has changed (this will invalidate the cached absolute start time)
	for( int i = 0; i < fChildren->CountItems(); i++ )
	{
		GetChild(i)->_NotifyParentPosHasChanged();
	}
	
	if (oldParent)
		oldParent->_NotifyChildPosHasChanged();
	if (pos.fParent)
		pos.fParent->_NotifyChildPosHasChanged();

	Move(); // some objects, like video objects, need to perform some operation when moved

	PosHasChanged();
}

void TArmeWSObj::Move()
{

}

void TArmeWSObj::__SetPos( const TArmeWSObjPos& pos )
{
	fPos = pos;
}
void TArmeWSObj::SetPos(const TArmeWSObjPos& pos)
{
	// for every object except video and TLC, SetPos is equal to _SetPos
	_SetPos(pos);
}

ArmeTime TArmeWSObj::GranulateTime(ArmeTime t) const
{ 
	return t;
}

void TArmeWSObj::Enable(bool onOff)
{
	for (int i=0; i < GetChildrenCount(); ++i)
	{
		GetChild(i)->Enable(onOff);
	}
	
}

const float TArmeWSObj::GetStartTimeSec() const
{
	return fDoc->ArmeTimeToSeconds(GetPos().fStartTime);
}

float TArmeWSObj::GetAbsoluteStartTimeSec() const
{
	return fDoc->ArmeTimeToSeconds(GetAbsoluteStartTime());
}

float TArmeWSObj::GetDurationSec() const
{
	return fDoc->ArmeTimeToSeconds(GetDuration());
}

//SInt32 TArmeWSObj::GetPrerollNbFrames()
//{
//	ZzAssert(0); //$$$TODO supress this call
//	return 0;
//}

void TArmeWSObj::SetNameCopy( const char* name )
{
	size_t srcLen = strlen(name);
	if( srcLen < kArmeWSObjUTF8NameNbBytes	)
	{
		memcpy(fName,name,srcLen);
		fName[srcLen] = 0;
	}
	else
	{
		memcpy(fName,name,kArmeWSObjUTF8NameNbBytes);
		fName[kArmeWSObjUTF8NameNbBytes-1] = 0;
	}
}

const char* TArmeWSObj::GetName() const
{
    if (fName)
        return fName;
    else
        return "Untitled";
}


void TArmeWSObj::GetNameForRenderedClip(char* target, int maxChar) 
{
	ZzAssert((int)strlen(GetName()) < maxChar-15);
	
	strcpy(target, GetName());
	
	ZzAppendExtension(target, maxChar, "rendered");
}

void TArmeWSObj::GetNameForStillImage(char* target, int maxChar)
{
	ZzAssert((int)strlen(GetName()) < maxChar-15);
	
	strcpy(target, GetName());
	
	char temp[12];
	sprintf(temp, ".still (%d)", (int)(++fStillImageCounter));
	ZzAppendExtension(target, maxChar, temp);
}

void TArmeWSObj::GetNameForCopy(char* target, int maxChar)
{
	ZzAssert((int)strlen(GetName()) < maxChar-15);
	
	strcpy(target, GetName());

	char temp[12];
	sprintf(temp, ".copy (%d)", (int)(++fCopyCounter));
	ZzAppendExtension(target, maxChar, temp);
}

void TArmeWSObj::RecreateNonArchivedCachedData(MIProgressBarDlgManager* progressMngr)
{
	for(SInt32 i = 0; i < GetChildrenCount(); i++)
	{
		GetChild(i)->RecreateNonArchivedCachedData(progressMngr);
	}
}

void TArmeWSObj::CountPostUnarchivingLengthyTasks(UInt32* nbOfTasks)
{
	for(SInt32 i = 0; i < GetChildrenCount(); i++)
	{
		GetChild(i)->CountPostUnarchivingLengthyTasks(nbOfTasks);
	}
}

void TArmeWSObj::SetOuttaHierarchy()
{
	verntl(armeWSObjsBenjamin,5,"TArmeWSObj::SetOuttaHierarchy() on object " << this->GetName());

	ZzAssert(GetDoc());
	GetDoc()->LockWrite();

	// we can't set an object outta the hierarchy 2 times
	ZzAssert(fPos.fParent != NULL);
	
	fPos.fParent->_RemoveChild(this);
	fPos.fParent = NULL;

	_NotifyParentOuttaHierarchy();
	
	GetDoc()->UnlockWrite();
}

void TArmeWSObj::SetBackInHierarchy(TArmeWSObj* parent)
{
	verntl(armeWSObjsBenjamin,5,"TArmeWSObj::SetBackInHierarchy(" << parent->GetName() << ") on object " << this->GetName());
	ZzAssert(fPos.fParent == NULL);
	ZzAssert(parent != NULL);
	
	ZzAssert(GetDoc());
	GetDoc()->LockWrite();

	fPos.fParent = parent;
	// let's tell the new parent we join him
	fPos.fParent->_AddChild(this);

	// let's copy the new pos
	InvalidateCachedTimeInfo();
	
	// tell to all the children that this position has changed (this will invalidate the cached absolute start time)
	for( int i = 0; i < fChildren->CountItems(); i++ )
	{
		GetChild(i)->_NotifyParentPosHasChanged();
	}
	
	if (fPos.fParent)
		fPos.fParent->_NotifyChildPosHasChanged();

	GetDoc()->UnlockWrite();
	
	_NotifyParentBackInHierarchy();

	Move(); // some objects, like video objects, need to perform some operation when moved
	
	CheckAudioDataNeedsPreviewComputingRecursive();
}

void TArmeWSObj::CheckAudioDataNeedsPreviewComputingRecursive()
{
	for (int i=0; i < GetChildrenCount(); ++i)
		GetChild(i)->CheckAudioDataNeedsPreviewComputingRecursive();
}

void TArmeWSObj::_NotifyParentOuttaHierarchy()
{
	// we need to close the editor window
	TArmeMediaEngine::ARPIHost()->CloseWindowForProcessor(this);
	for (int i=0; i < GetChildrenCount(); ++i)
		GetChild(i)->_NotifyParentOuttaHierarchy();
}

void TArmeWSObj::_NotifyParentBackInHierarchy()
{
	for (int i=0; i < GetChildrenCount(); ++i)
		GetChild(i)->_NotifyParentBackInHierarchy();
}


void TArmeWSObj::SetMuted(bool muted)
{
	fMuted = muted;
}


bool TArmeWSObj::OneOfTheChildrenIsNotMuted() const
{
	if (GetChildrenCount())
	{
		for (int i=0; i < GetChildrenCount(); ++i)
		{
			if (GetChild(i)->OneOfTheChildrenIsNotMuted())
				return true;
		}
		return false;
	}
	else
		return !IsMuted();
}
bool TArmeWSObj::IsMuted() const
{
	if(fDoc)
	{
		// if this is in the solo, it is not muted
		if (IsInSolo())
			return false;

		if (GetChildrenCount() > 0)
		{
			// if one of the children is not muted, it is not muted
			if (OneOfTheChildrenIsNotMuted())
				return false;
		}			
		if (fMuted) 
			return true;
			
		// if there is a solo object and if it is not this object, this is muted
		if (ContainsVideo(true, true) && GetDoc()->GetVideoSoloObject() && !IsInSolo())
			return true;
		if (ContainsAudio(true) && GetDoc()->GetAudioSoloObject() && !IsInSolo())
			return true;

		return false;
	}
	else
		return fMuted;
}


bool TArmeWSObj::ContainsVideo(bool bConsiderAlsoMutedObjects, bool bConsiderAlsoTransparentTextClips) const 
{
	return ContainsVideoDuring(ARME_TIME_UNDEF, ARME_TIME_UNDEF, bConsiderAlsoMutedObjects, bConsiderAlsoTransparentTextClips);
};

bool TArmeWSObj::ContainsAudio(bool bConsiderAlsoMutedObjects) const 
{
	return ContainsAudioDuring(ARME_TIME_UNDEF, ARME_TIME_UNDEF, bConsiderAlsoMutedObjects);
};


TArmeWSObjMemento::TArmeWSObjMemento()
{
	fPos.fParent = NULL;
	fPos.fStartTime = ARME_TIME_UNDEF;
	fPos.fInput = -1;
	fLength = ARME_TIME_UNDEF;
	fMuted = false;
}

TArmeWSObjMemento::TArmeWSObjMemento(ArmeArchiveHandler* unarchiver) : ArmeArchivable(unarchiver)
{
	ArmeArchivable* archObj;

	archObj = unarchiver->ReadCPPObject();
	fPos.fParent = dynamic_cast<TArmeWSObj*>(archObj);
	if(archObj)
	{
		ZzAssert(fPos.fParent);
	}
	
	fPos.fInput = unarchiver->ReadSInt32();
	fPos.fStartTime = unarchiver->ReadArmeTime();
	fLength = unarchiver->ReadArmeTime();
	fMuted = unarchiver->ReadBool();
	fStillImageCounter = unarchiver->ReadUInt32();
	fCopyCounter = unarchiver->ReadUInt32();
}

bool TArmeWSObjMemento::WriteToArchive(ArmeArchiveHandler* archiver) const
{
	archiver->WriteCPPObject(fPos.fParent);
	archiver->WriteSInt32(fPos.fInput);
	archiver->WriteArmeTime(fPos.fStartTime);
	archiver->WriteArmeTime(fLength);
	archiver->WriteBool(fMuted);
	archiver->WriteUInt32(fStillImageCounter);
	archiver->WriteUInt32(fCopyCounter);
	return true;
}

int TArmeWSObj::GetFamilyDepth() const
{
	if( this == GetDoc()->GetTopContainer() )
		return 0;
	return GetPos().fParent->GetFamilyDepth() +1 ;
}

SInt32 TArmeWSObj::GetAudioTrackForPlayer(UInt32 playerIndex) const
{
	return -1;
}

bool TArmeWSObj::ContainsVideoDuring(ArmeTime startTime, ArmeTime endTime, bool bConsiderAlsoMutedObjects, bool bConsiderAlsoTransparentTextClips) const
{
	return false;
}

bool TArmeWSObj::ContainsAudioDuring(ArmeTime startTime, ArmeTime endTime, bool bConsiderAlsoMutedObjects) const
{
	return false;
}

TArmeWSObj* TArmeWSObj::FindVideoObjectAtTime(ArmeTime time) const
{
	for (int i=0; i < GetChildrenCount(); ++i)
	{
		if (GetChild(i)->ContainsVideoDuring(time, time + _ArmeTime(1), false))
			return GetChild(i);
	}
	return NULL;
}

#ifdef OLD_VAO_RAO
bool TArmeWSObj::ContainsAudioObjectsRoutedTo(bool bStereo, SInt32 vaoIndex) const
{
	for (int i=0; i < GetChildrenCount(); ++i)
	{
		if(GetChild(i)->ContainsAudioObjectsRoutedTo(bStereo, vaoIndex))
			return true;
	}
	return false;
}
#endif //#ifdef OLD_VAO_RAO

#ifdef OLD_VAO_RAO
void TArmeWSObj::GebruikenVAOs(ZzArray<SInt32>* monoVAOsUsed, ZzArray<SInt32>* stereoVAOsUsed) const
{
	for (int i=0; i < GetChildrenCount(); ++i)
	{
		GetChild(i)->GebruikenVAOs(monoVAOsUsed, stereoVAOsUsed);
	}
}
#endif //#ifdef OLD_VAO_RAO

/*
MEVAORoutingDistributionDesc TArmeWSObj::GetVAORoutingDistributionForObjSet(MEVAORouting* vaoRouting,ZzArchPtrArray* objects)
{
	MEVAORouting firstVAORouting;
	MEVAORouting curChildVAORouting;
	bool bFirstRoutingRead = false;
	MEVAORoutingDistributionDesc thisDesc = kMEVAORoutingDistributionDesc_NoVAORouting; 
	
	for (int i=0; i < objects->CountItems(); ++i)
	{
		TArmeWSObj* object = dynamic_cast<TArmeWSObj*>(objects->ItemAt(i));
		ZzAssert(object);
	
		MEVAORoutingDistributionDesc curChildDesc = object->GetVAORoutingDistribution(&curChildVAORouting);
		switch(curChildDesc)
		{
		case kMEVAORoutingDistributionDesc_NoVAORouting : // this child is not routed to any vao
			{
				// don't do anything. our return values are already configured for this
			}
			break;
		case kMEVAORoutingDistributionDesc_SingleVAORouting : // this child is routed to a single vao
			{
				if(!bFirstRoutingRead) // this is the first single routing we see in a child. let's change our return values to single routing and store this routing for later comparison
				{
					firstVAORouting = curChildVAORouting;
					bFirstRoutingRead = true;
					thisDesc = kMEVAORoutingDistributionDesc_SingleVAORouting;
				}
				else // we already had a child with a single vao routing. if they are different, then we need to return multi
				{
					if(firstVAORouting != curChildVAORouting)
						return kMEVAORoutingDistributionDesc_MultiVAORouting;
					// otherwise they are equal, don't do anything
				}
			}
			break;
		case kMEVAORoutingDistributionDesc_MultiVAORouting :
			return kMEVAORoutingDistributionDesc_MultiVAORouting;
		default :
			ZzAssert(0);
			return kMEVAORoutingDistributionDesc_MultiVAORouting; // just to avoid compiler warning
		};
	}
	// now at the end of the routing, either no vao routing at all has been detected among the children, or a single vao routing.
	// otherwise, in case of different routing, the function would already have returned
	if(thisDesc == kMEVAORoutingDistributionDesc_SingleVAORouting)
	{
		*vaoRouting = firstVAORouting;
	}
	return thisDesc;
}

MEVAORoutingDistributionDesc TArmeWSObj::GetVAORoutingDistribution(MEVAORouting* vaoRouting) const
{
	return GetVAORoutingDistributionForObjSet(vaoRouting, fChildren);
}

*/




#ifdef OLD_RAO_VAO
VARPIPlugin* TArmeWSObj::GetAudioPlugin(bool bStereo, SInt32 vaoIndex)
{
	if(fStereoAudioPlugins[0] == NULL)
		return NULL;
	else
	{
		if(bStereo)
		{
			if(fStereoAudioPlugins[vaoIndex] != NULL)
				return fStereoAudioPlugins[vaoIndex];
			else
			{
				fStereoAudioPlugins[vaoIndex] = TArmeMediaEngine::ARPIHost()->CreatePlugin(fStereoAudioPlugins[0]->GetClass());
				fStereoAudioPlugins[vaoIndex]->SetFrameRate(TArmeMediaEngine::CoreAudioPlayer()->GetAudioFrameRate());
				fStereoAudioPlugins[vaoIndex]->SetBufferNbFrames(TArmeMediaEngine::CoreAudioPlayer()->OutDeviceBufferNbFrames());
				// send notification
				fStereoAudioPlugins[vaoIndex]->FormatChange();
				// copy the params
				for(SInt32 param = 0; param < fStereoAudioPlugins[0]->GetClass()->GetNbParams(); param++)
				{
					fStereoAudioPlugins[vaoIndex]->SetParamValue(param,fStereoAudioPlugins[0]->GetParamValue(param));
				}
				return fStereoAudioPlugins[vaoIndex];
			}
		}
		else
		{
			if(fMonoAudioPlugins[vaoIndex] != NULL)
				return fMonoAudioPlugins[vaoIndex];
			else
			{
				fMonoAudioPlugins[vaoIndex] = TArmeMediaEngine::ARPIHost()->CreatePlugin(fStereoAudioPlugins[0]->GetClass());
				fMonoAudioPlugins[vaoIndex]->SetFrameRate(TArmeMediaEngine::CoreAudioPlayer()->GetAudioFrameRate());
				fMonoAudioPlugins[vaoIndex]->SetBufferNbFrames(TArmeMediaEngine::CoreAudioPlayer()->OutDeviceBufferNbFrames());
				// send notification
				fMonoAudioPlugins[vaoIndex]->FormatChange();

				// copy the params
				for(SInt32 param = 0; param < fStereoAudioPlugins[0]->GetClass()->GetNbParams(); param++)
				{
					fMonoAudioPlugins[vaoIndex]->SetParamValue(param,fStereoAudioPlugins[0]->GetParamValue(param));
				}
				return fMonoAudioPlugins[vaoIndex];
			}
		}
	}
}
#endif //#ifdef OLD_RAO_VAO

#ifdef AUTOMATION_DATA
MEAudioPluginAutomationData* TArmeWSObj::GetAudioAutomationData()
{
	return fAudioAutomationData;
}
#endif

void TArmeWSObj::SetAudioGainInSelfAndDescendants(float audioGain)
{	
	fAudioGain = audioGain;
	for(SInt32 i = 0 ; i < GetChildrenCount(); i++)
	{
		GetChild(i)->fAudioGain = audioGain;
	}
}

void TArmeWSObj::SetAudioGainInSelfAndAscendants(float audioGain)
{
	fAudioGain = audioGain;
	if(fPos.fParent)
		fPos.fParent->SetAudioGainInSelfAndAscendants(audioGain);
}

void TArmeWSObj::RecursiveSetAudioGainAccordingToMute()
{
	if(fMuted)
		fAudioGain = 0.0;
	else
		fAudioGain = 1.0;
	for(SInt32 i = 0 ; i < GetChildrenCount(); i++)
	{	
		GetChild(i)->RecursiveSetAudioGainAccordingToMute();
	}
}

void TArmeWSObj::SetStartTrim(ArmeTime startTrim)
{
	vernl(1,"please implement SetStartTrim() in your TArmeWSObj subclass");
}

void TArmeWSObj::SetEndTrim(ArmeTime endTrim)
{
	vernl(1,"please implement SetEndTrim() in your TArmeWSObj subclass");
}

ArmeTime TArmeWSObj::GetStartTrim() const
{
	return _ArmeTime(0);
}

ArmeTime TArmeWSObj::GetEndTrim() const
{
	return _ArmeTime(0);
}


void TArmeWSObj::DeleteOfflinePlugin()
{
	delete fOfflinePlugin;
	fOfflinePlugin = NULL;
}

#ifdef OLD_RAO_VAO
void TArmeWSObj::CreateOfflineAudioPlugin(SInt32 bufferNbFrames)
{
	if(fStereoAudioPlugins[0] == NULL)
	{
		delete fOfflinePlugin;
		fOfflinePlugin = NULL;
	}
	else
	{
		fOfflinePlugin = TArmeMediaEngine::ARPIHost()->CreatePlugin(fStereoAudioPlugins[0]->GetClass());
		fOfflinePlugin->SetFrameRate(TArmeMediaEngine::CoreAudioPlayer()->GetAudioFrameRate());
		fOfflinePlugin->SetBufferNbFrames(bufferNbFrames);
		// send notification
		fOfflinePlugin->FormatChange();
		// copy the params
		for(SInt32 param = 0; param < fStereoAudioPlugins[0]->GetClass()->GetNbParams(); param++)
		{
			fOfflinePlugin->SetParamValue(param,fStereoAudioPlugins[0]->GetParamValue(param));
		}
	}
}
#else //#ifdef OLD_RAO_VAO
void TArmeWSObj::CreateOfflineAudioPlugin(SInt32 bufferNbFrames)
{
	if(fRealTimePlugin == NULL)
	{
		delete fOfflinePlugin;
		fOfflinePlugin = NULL;
	}
	else
	{
		fOfflinePlugin = TArmeMediaEngine::ARPIHost()->CreatePlugin(fRealTimePlugin->GetClass());
		fOfflinePlugin->SetFrameRate(TArmeMediaEngine::CoreAudioPlayer()->GetAudioFrameRate());
		fOfflinePlugin->SetBufferNbFrames(bufferNbFrames);
		// send notification
		fOfflinePlugin->FormatChange();
		// copy the params
		for(SInt32 param = 0; param < fRealTimePlugin->GetClass()->GetNbParams(); param++)
		{
			fOfflinePlugin->SetParamValue(param,fRealTimePlugin->GetParamValue(param));
		}
	}
}
#endif //#ifdef OLD_RAO_VAO

ARPIBlueWindowPreset* TArmeWSObj::GetPluginPreset()
{
	return fPluginPreset;
}

const ARPIBlueWindowPreset* TArmeWSObj::GetPluginPreset() const
{
	return fPluginPreset;
}

void TArmeWSObj::SetPluginPreset(ARPIBlueWindowPreset* preset)
{
	if(fPluginPreset)
		delete fPluginPreset;
	fPluginPreset = preset;
}

/************************************************************************************************************************
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
************************************************************************************************************************/
MEVAORouting TArmeWSObj::GetVAORouting() const
{
	return fVAORouting;
}

void TArmeWSObj::SetVAORouting(const MEVAORouting& vaoRouting)
{
	if(GetDoc())
	{
		GetDoc()->LockWrite();
	}
	fVAORouting = vaoRouting;
	if(GetDoc())
	{
		GetDoc()->UnlockWrite();
		NotifyVAORoutingChange();
	}
}

void TArmeWSObj::NotifyVAORoutingChange()
{
	if(GetDoc()) // it is possible that GetDoc() returns null for clipboard clips
	{
		if(fVAORouting.bStereo)
			GetDoc()->Notify(kNotification_VAOChangeToStereoVAO,(void*)(fVAORouting.index));
		else
			GetDoc()->Notify(kNotification_VAOChangeToMonoVAO,(void*)(fVAORouting.index));
	}
}

void TArmeWSObj::CopyAudioPluginFrom(const TArmeWSObj* other)
{
	const VARPIPlugin* otherObjPlugin = other->GetAudioPlugin();
	if(otherObjPlugin)
	{
		const VARPIPluginClass* thePluginClass = otherObjPlugin->GetClass();
		VARPIPlugin* plugin = thePluginClass->CreatePlugin();
		for(SInt32 param = 0; param < thePluginClass->GetNbParams(); param++)
		{
			plugin->SetParamValue(param,otherObjPlugin->GetParamValue(param));
		}
		SetAudioPlugin(plugin);
		ARPIBlueWindowPreset* newPreset = new ARPIBlueWindowPreset;
		ZzAssert(other->GetPluginPreset() != NULL);
		*newPreset = *(other->GetPluginPreset());
		SetPluginPreset(newPreset);
	}
}

MEVAORouting TArmeWSObj::GetTopParentVAORouting()
{
	ZzAssert(GetDoc()); // calling this on a docless obj doesn't make no sense
	ZzAssert(this != GetDoc()->GetTopContainer()); // calling this on the TLC doesn't make no sense
	if(GetParent() == GetDoc()->GetTopContainer())
		return GetVAORouting();
	else
		return GetParent()->GetVAORouting();
}

void TArmeWSObj::AddMIObjectRef()
{
	ZzAssert(fMIObjectRefCount == 0);
	fMIObjectRefCount++;
}

void TArmeWSObj::ReleaseMIObjectRef()
{
	ZzAssert(fMIObjectRefCount == 1);
	fMIObjectRefCount--;
}

int	 TArmeWSObj::GetMIObjectRefCount()
{
	return fMIObjectRefCount;
}

TArmeWSObj*	TArmeWSObj::GetTopmostParent()
{
	if(GetDoc() == NULL)
		return NULL;
	TArmeWSObj* tlc = GetDoc()->GetTopContainer();
	if(this == tlc)
		return NULL;
	if(GetParent() == tlc)
		return this;
	else
		return GetParent()->GetTopmostParent();
}
		

