/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
#define TArmeWSObj_h
#ifndef TArmeWSObj_h
#include "TArmeWSObj.h"
#endif //TArmeWSObj_h

#ifndef ArmeBasicTypes_h
#include "ArmeBasicTypes.h"
#endif //ArmeBasicTypes_h

#ifndef ZzArchPtrArray_h
#include "ZzArchPtrArray.h"
#endif //ZzArchPtrArray_h

#ifndef ZzArray_h
#include "ZzArray.h"
#endif //ZzArray_h

#ifndef ArmeArchivable_h
#include "ArmeArchivable.h"
#endif //ArmeArchivable_h

#ifndef MEMixingInfo_h
#include "MEMixingInfo.h"
#endif //MEMixingInfo_h

#ifndef VARPIPlugin_h
#include "VARPIPlugin.h"
#endif VARPIPlugin_h

#ifndef MEAudioPluginAutomationData_h
#include "MEAudioPluginAutomationData.h"
#endif //MEAudioPluginAutomationData_h

//#define AUTOMATION_DATA

/*! 
	@class TArmeWSObjPos
	This class encapsulates the parameters that define the position of a TArmeWSObj :
	- its parent
	- the parent input container index
	- the time relative to the parent start time
*/

class TArmeWSObj;
class VARPIPluginClass;
class VARPIPlugin;
class TArmeDoc;
class ArmeTimeRange;
class ARPIBlueWindowPreset;

class TArmeWSObjPos
{
public :
	TArmeWSObj*	fParent;
	SInt32		fInput;
	ArmeTime	fStartTime;		//relative to parent
	SInt32		fVerticalPos;  	//relative to parent
};

/*! 
	@class TArmeWSObj
	This class represents a workspace object, which _can_ always contain object of the same kind. It
	is up to the implementor to ensure some objects are leaf objects and do not contain children (example :
	clips)
*/

class TArmeWSObj;
class VARPIPluginClass;
class TArmeWSObjMemento;
class ZzSplitTask;
class MIProgressBarDlgManager;

//enum MEVAORoutingDistributionDesc
//{
//	kMEVAORoutingDistributionDesc_NoVAORouting = 1973,
//	kMEVAORoutingDistributionDesc_SingleVAORouting,
//	kMEVAORoutingDistributionDesc_MultiVAORouting
//};
	
#define kArmeWSObjUTF8NameNbBytes 256
	
class TArmeWSObj : public ArmeArchivable
{
public :
	enum
	{
		kArmeWSObj,
		kContainer,
		kAudioProcessor,
		kTLC,
		kAudioClip,
		kVideoClip,
		kVideoEffect,
		kVideoXFadeEffect,
		kTextClip,
		kMIDIClip
	};
	
public :
	/*! 
		@function Constructor This is the regular constructor*/

		TArmeWSObj(const char* name, TArmeDoc* doc, VARPIPluginClass* audioPluginClass = NULL);
							
	// arch stuff
							TArmeWSObj(ArmeArchiveHandler* unarchiver);
	virtual bool			WriteToArchive(ArmeArchiveHandler* archiver) const;


private :
	/*! &function Constructor This is a disabled (private) copy constructor */
	TArmeWSObj(const TArmeWSObj& );

public :
	/*! &function Constructor Create a TArmeWSObj that is not attached to any document (for clipboard)*/
	TArmeWSObj(const char* name);

protected :
	int		fMIObjectRefCount;
public :
	void	AddMIObjectRef();
	void	ReleaseMIObjectRef();
	int		GetMIObjectRefCount();
	
	
	//Cut/Copy/Paste related functions.  Normally, you just need to overwrite GetSliceCopy, DeleteSlice and InsertSlice to be able to support cut/copy/paste.
	//Note : all paramters are absolute time
	virtual TArmeWSObj* GetCopy(const char* tempDirectoryName, MIProgressBarDlgManager* progressMngr) const;
	virtual TArmeWSObj* CutSlice(ArmeTime startTime, ArmeTime endTime, const char* tempDirectoryName, MIProgressBarDlgManager* progressMngr);
	virtual TArmeWSObj* Cut(const char* tempDirectoryName, MIProgressBarDlgManager* progressMngr);
	virtual TArmeWSObj* GetSliceCopy(ArmeTime startTime, ArmeTime endTime, const char* tempDirectoryName, MIProgressBarDlgManager* progressMngr) const;
	virtual ArmeTime	InsertSlice(ArmeTime insertTime, const TArmeWSObj* obj, MIProgressBarDlgManager* progressMngr);
	/*
		@function DeleteSlice Override to delete a slice. It should return true if the whole object has
		                      to be deleted (it will be done by another routine)
	*/
	virtual bool		DeleteSlice(ArmeTime startTime, ArmeTime endTime, MIProgressBarDlgManager* progressMngr);  
	virtual TArmeWSObjMemento* 	GetMemento() const;
	virtual void				SetMemento(TArmeWSObjMemento* memento);
protected :
	virtual void				_FillMemento(TArmeWSObjMemento* memento) const;

public :
	virtual void 		RecreateNonArchivedCachedData(MIProgressBarDlgManager* progressMngr);
	
	// needs to return the number of lengthy tasks at unarchiving time. All tasks will
	// be level-1 tasks.
	virtual void 		CountPostUnarchivingLengthyTasks(UInt32* nbOfTasks);
	
	virtual void		SetOuttaHierarchy();
	virtual void		SetBackInHierarchy(TArmeWSObj* parent);

	// needs to be called  to compute the audio clip data preview
	// when someone is put back in hierarch, it's possible that itself or 
	// children don't have any preview computed becaause their 
	// RecreateNonArchivedCachedData( ) method hasn't been called
	// since there are orphan thus we need this hack. $$$TODO : make them child of
	// a 'outta hierarch mother node' that can be browsed at archive load time
	virtual void 		CheckAudioDataNeedsPreviewComputingRecursive();

protected :
	virtual void		_NotifyParentOuttaHierarchy();
	virtual void		_NotifyParentBackInHierarchy();

public :
	/*! @function Destructor The virtual destructor */
	virtual				~TArmeWSObj();

	/*! @function GetTopmostParent Returns the topmost parent which is a child of the tlcThe virtual destructor */
	TArmeWSObj*			GetTopmostParent();
	
	/*! @function GetChild Returns the i-th child. Don't cache this because 
	order may change. Use for enumerations only */
	TArmeWSObj*		GetChild(int32 i) const;

	/*! @function GetParent Returns the parent */
	TArmeWSObj*		GetParent() const;
	
	/*! @function GetChildrenCount Returns the number of children. Don't cache it between 
	doc locks because this number may change. Use for enumerations only */
	int32			GetChildrenCount() const;
	
//	SInt32			GetChildrenCountWithVAO(SInt32 vao);
//	TArmeWSObj*		GetChildWithVAO(SInt32 vao, SInt32 i);
	
	/*! @function GetType RTTI information for all TArmeWSObj derivatives. Values can be :
		TArmeWSObj::kArmeWSObj,
		TArmeWSOBj::kAudioClip */
	virtual	SInt32		GetType() const = 0;

	/* @function IsVideoObject test if this object contains video data*/
	virtual bool			IsVideoObject() const;
	
	/* @function GetAudioTrackForPlayer only overridden in MEAudioClip at the moment */
	virtual SInt32			GetAudioTrackForPlayer(UInt32 playerIndex) const;
	
	/* @function GetVideoChildrenCount get the number of children with video properties*/
	int						GetVideoChildrenCount() const;

	virtual bool			ContainsVideoDuring(ArmeTime startTime, ArmeTime endTime, bool bConsiderAlsoMutedObjects = false, bool bConsiderAlsoTransparentTextClips = false) const;
	virtual bool			ContainsAudioDuring(ArmeTime startTime, ArmeTime endTime, bool bConsiderAlsoMutedObjects = false) const;
	bool					ContainsVideo(bool bConsiderAlsoMutedObjects = false, bool bConsiderAlsoTransparentTextClips = false) const;
	bool					ContainsAudio(bool bConsiderAlsoMutedObjects = false) const;

	// this function is a helper for the previous one, so that it can be called externally to check how a set of objects are 
	// routed even if they are not the children of a common parent.
	//static MEVAORoutingDistributionDesc	GetVAORoutingDistributionForObjSet(MEVAORouting* vaoRouting,ZzArchPtrArray* objects);  

	virtual ArmeTimeRange* 	GetVideoCoverage() const {return NULL;};

	/*! @function GebruikenVAOs This function must add in the arrays the vaos the objects (or its (grand)children) use. same indices are not
		added twice */
#ifdef OLD_VAO_RAO
	virtual void			GebruikenVAOs(ZzArray<SInt32>* monoVAOsUsed, ZzArray<SInt32>* stereoVAOsUsed) const;
#endif OLD_VAO_RAO

	/*! @function GetDuration Returns the length as an ArmeTime. If length value is -1 then it 
	means this is an infinite length object	(example : live feed). Derived classes will probably
	overridde this. */
	virtual ArmeTime		GetDuration() const;

	/*! @function GranulateTime Returns the closest time corresponding to the position of a sample (mainly used by selection functions*/
	virtual ArmeTime		GranulateTime(ArmeTime t) const; 
	
	virtual void			Enable(bool onOff);

	virtual void			SetStartTrim(ArmeTime startTrim);
	virtual void			SetEndTrim(ArmeTime endTrim);
	virtual ArmeTime		GetStartTrim() const;
	virtual ArmeTime		GetEndTrim() const;
	
	/*! @function _SetLength Brute-force setter for the length. Not designed for client
	(reserved to the model layer) use. To be used by command classes ON CONTAINERS only */
	virtual void			_SetLength( ArmeTime len );
	
	// whether this object contains audio objects routed to the bStereo,vaoIndex bus
#ifdef OLD_VAO_RAO
	virtual bool			ContainsAudioObjectsRoutedTo(bool bStereo, SInt32 vaoIndex) const;
#endif //#ifdef OLD_VAO_RAO
	
public :
	
	bool					IsTheVideoSolo() const;
	bool					IsTheAudioSolo() const;
	bool					IsInSolo() const;
	bool 					IsMuted() const;  //this function returns true if the clip is muted or if it is not visible for any other reason (there is a solo object)
	bool					GetMuted() const {return fMuted;};
	bool					OneOfTheChildrenIsNotMuted() const;
	virtual void			SetMuted(bool muted);
	void					RecursiveSetAudioGainAccordingToMute();
	void 					SetAudioGainInSelfAndAscendants(float audioGain);
	void 					SetAudioGainInSelfAndDescendants(float audioGain);
	MEVAORouting			GetVAORouting() const;
	MEVAORouting			GetTopParentVAORouting();
	void					SetVAORouting(const MEVAORouting& vaoRouting);
	void 					NotifyVAORoutingChange();

			
	bool IsDescendantOf(const TArmeWSObj* grandParent) const;
			
	/*! @function GetPos Returns the position of the WS object */
	const TArmeWSObjPos&	GetPos() const;
	
	/*! @function GetStartTime Returns the start time of the WS object (stored in the position object) */
	const ArmeTime			GetStartTime() const { return GetPos().fStartTime; }
	
// now this functionality is built into the PLAYERS (doc player -> wsobj player -> track player)
//	virtual SInt32			GetPrerollNbFrames();
//	virtual void 			CancelPreroll();
//	virtual void			Preroll(SInt64 pos);
	
	/*! @function GetStartTimeSec Returns the start time of the WS object in seconds (stored in the position object) */
	const float				GetStartTimeSec() const;
		
	/*! @function GetDoc() Returns the owning document of the WS object. Please note all the nodes 
	of a given hierarchy (starting with the WSObj which has a NULL parent) **MUST** have the same document */
	TArmeDoc*				GetDoc() const { return fDoc; };
	/*! @function FindChildRecursiveByName This function searches recursively in the children of this wsobj
	(_taking_ 'this' into account) for the FIRST one (sorted by parent/child link time) whose name exactly 
	matches 'name' and returns it. Don't forget to lock the doc before if you are running the doc player */
	TArmeWSObj*				FindChildRecursiveByName(const char* name);

	TArmeWSObj*				FindVideoObjectAtTime(ArmeTime time) const;
	
	bool					IntersectsTimeRange(ArmeTime start, ArmeTime length) const;
	
public :
	/*! @function GetFamilyDepth (WARNING : EXPENSIVE CALL) Returns the depth in the hierarchy :
		) "deadlock" (which is, alas, not a return value) means nil object or dissident hierarchy,
		) 0 means TLC, 
		) 1 means the TLC child,
		) 2 means the TLC grandchild, etc... 
	*/
	int						GetFamilyDepth() const;

protected :
	mutable ArmeTime		fCachedAbsoluteStartTime; 
	virtual void			InvalidateCachedTimeInfo() {fCachedAbsoluteStartTime = ARME_TIME_UNDEF;};

public :
	/*! @function GetAbsoluteStartTime (WARNING : EXPENSIVE CALL) Returns the start time relative to the TLC */
	ArmeTime				GetAbsoluteStartTime() const;
	float					GetAbsoluteStartTimeSec() const;

	ArmeTime				GetAbsoluteEndxTime() const {return GetAbsoluteStartTime() + GetDuration();};
	float					GetAbsoluteEndxTimeSec() const {return GetAbsoluteStartTimeSec() + GetDurationSec();};
	
	SInt32					GetVerticalPos() const {return fPos.fVerticalPos;};
	void					SetVerticalPos(SInt32 pos);
	
	/*! @function _SetPos Brute-force setter for the position. Not designed for client
	(reserved to the model layer) use. To be used by command classes only */
	virtual void			_SetPos( const TArmeWSObjPos& pos );
	void					__SetPos( const TArmeWSObjPos& pos );
	
	/*! @function SetPos Brute-force setter for the position. to be used called by the interface */
	virtual void			SetPos( const TArmeWSObjPos& pos );
	
	virtual void			Move();
				
	/*! @function _NotifyParentPosHasChanged Each time a parent is moved, all its children will receive this message */
	virtual void			_NotifyParentPosHasChanged();

	/*! @function _NotifyChildPosHasChanged Each time a child is moved, all its parents will receive this message */
	virtual void			_NotifyChildPosHasChanged();
	
	/*! @function _AddChild Brute-force setter for the child array. Not designed for client
	(reserved to the model layer) use. To be used by command classes only */
	virtual void			_AddChild( TArmeWSObj* child );
	virtual void			_InsertChild( TArmeWSObj* child, int index);
	int						_GetIndexOfChild( TArmeWSObj* child );
	/*! @function _RemoveChild Brute-force setter for the children array. Not designed for client
	(reserved to the model layer) use. To be used by command classes only */
	virtual void			_RemoveChild( TArmeWSObj* child );

	virtual void			PosWillChange();
	virtual void			PosHasChanged();
	
	void						SetAudioPlugin(VARPIPlugin* plugin);
	const VARPIPluginClass*		GetAudioPluginClass() const;
	float						GetAudioPluginParamValue(SInt32 iParam) const;
	void						SetAudioPluginParamValue(SInt32 iParam, float val);
	void						SetAudioPluginByPass(bool bypass);
	void 						IncrAudioPluginParamValue(SInt32 iParam, float incr);
	ARPIBlueWindowPreset*		GetPluginPreset();
	const ARPIBlueWindowPreset*	GetPluginPreset() const;
	void						SetPluginPreset(ARPIBlueWindowPreset* preset);
	VARPIPlugin*				GetAudioPlugin();
	const VARPIPlugin*			GetAudioPlugin() const;

	void						CopyAudioPluginFrom(const TArmeWSObj* other);

		
	// offline plugin stuff
	void					DeleteOfflinePlugin();
	void					CreateOfflineAudioPlugin(SInt32 bufferNbFrames); // this function create a plugin similar to fStereoPlugins[0], or NULL if there is no plugin...
	VARPIPlugin*			GetOfflineRenderingPlugin();
	
	
	void 					SetPresetAsFirstFromFactory();
	void 					SyncPluginFromPreset();

	
#ifdef AUTOMATION_DATA
	MEAudioPluginAutomationData*	GetAudioAutomationData();
#endif						
	/*! @function SetNameCopy Sets the name of the object by COPYING the passed argument */
	void		SetNameCopy( const char* name );
	/*! @function GetName Returns the name as a const char* */
	const char*	GetName() const;
	
	void	GetNameForRenderedClip(char* target, int maxChar);
	void	GetNameForStillImage(char* target, int maxChar);
	void	GetNameForCopy(char* target, int maxChar);
protected : 
	UInt32	fStillImageCounter;  //only used for stil image name generation
	UInt32 	fCopyCounter;		//only used for copy name generation
	
public :

	/*! @function GetDurationSec Returns the length in seconds. This is just a conversion from 
	GetDuration()*/
	float					GetDurationSec() const;
		
protected :
	/*! @var fPluginPreset This holds the full state of blue window/plugin */
	ARPIBlueWindowPreset*	fPluginPreset;

	/*! @var fName the object name */
	char				fName[kArmeWSObjUTF8NameNbBytes];

	/*! @var fMuted Whether this object and children are muted or not */
	bool				fMuted;
	
	/*! @var fChildren This is the array of children... this is a non-owning array. */
	ZzArchPtrArray*		fChildren;	

	/*! @var fPos The position. */
	TArmeWSObjPos		fPos;

	/*! @var fDoc The cached doc pointer. */
	TArmeDoc*			fDoc;

	/*! @var fLength The length. Probably not meaningful for derived classes. */
	ArmeTime			fLength;

#ifdef OLD_RAO_VAO
	/*! @var fMonoAudioPlugins The plugins that processes the mono VAO busses */
	VARPIPlugin*		fMonoAudioPlugins[kMEMixingInfo_NbMaxMonoVAO];

	/*! @var fMonoAudioPlugins The plugins that processes the stereo VAO busses */
	VARPIPlugin*		fStereoAudioPlugins[kMEMixingInfo_NbMaxStereoVAO];
#else //#ifdef OLD_RAO_VAO
	VARPIPlugin*		fRealTimePlugin;
#endif //#ifdef OLD_RAO_VAO
	
	/*! @var fOfflinePlugin The plugin used when doing offline rendering */
	VARPIPlugin*		fOfflinePlugin;
	
#ifdef AUTOMATION_DATA
	MEAudioPluginAutomationData	*fAudioAutomationData;
#endif

public :
	void 				ResetRecursive();
	
public :
	ZzArchPtrArray*		GetChildrenDangerousFunction() {return fChildren;};

	float				fAudioGain;
	float				fOldAudioGain; // for dezipping purposes. a clip is only read once per buffer.
	
	/*! @var fVAORouting Where this object is routed to. ONLY VALID FOR AUDIO CLIPS */
	MEVAORouting		fVAORouting;

};

inline const TArmeWSObjPos& TArmeWSObj::GetPos() const
{
	return fPos;
}

class TArmeWSObjMemento : public ArmeArchivable
{
public :
	TArmeWSObjPos			fPos;
	ArmeTime				fLength;
	bool					fMuted;
	// audioPlugin ?

	UInt32					fStillImageCounter;
	UInt32					fCopyCounter;

							TArmeWSObjMemento();
							TArmeWSObjMemento(ArmeArchiveHandler* unarchiver);
	virtual bool			WriteToArchive(ArmeArchiveHandler* archiver) const;
	virtual ~TArmeWSObjMemento() {};
};

inline bool TArmeWSObj::IntersectsTimeRange(ArmeTime start, ArmeTime length) const
{
	bool rangeTotallyBefore = (GET_VAL(start)+GET_VAL(length)) <= GET_VAL(GetAbsoluteStartTime());
	bool rangeTotallyAfter = GET_VAL(start) >= GET_VAL(GetAbsoluteStartTime()) + GET_VAL(GetDuration());
	return (!rangeTotallyBefore) && (!rangeTotallyAfter);
}

#ifdef OLD_RAO_VAO
inline float TArmeWSObj::GetAudioPluginParamValue(SInt32 iParam) const 
{
	return fStereoAudioPlugins[0]->GetParamValue(iParam);
}

inline void TArmeWSObj::IncrAudioPluginParamValue(SInt32 iParam, float incr)
{
	for(SInt32 i = 0; i < kMEMixingInfo_NbMaxMonoVAO; i++)
	{
		if(fMonoAudioPlugins[i])
		{
			fMonoAudioPlugins[i]->IncrParamValue(iParam,incr);
		}
	}
	for(SInt32 i = 0; i < kMEMixingInfo_NbMaxStereoVAO; i++)
	{
		if(fStereoAudioPlugins[i])
		{
			fStereoAudioPlugins[i]->IncrParamValue(iParam,incr);
		}
	}
}

inline void TArmeWSObj::SetAudioPluginParamValue(SInt32 iParam, float val)
{
	for(SInt32 i = 0; i < kMEMixingInfo_NbMaxMonoVAO; i++)
	{
		if(fMonoAudioPlugins[i])
		{
			fMonoAudioPlugins[i]->SetParamValue(iParam,val);
		}
	}
	for(SInt32 i = 0; i < kMEMixingInfo_NbMaxStereoVAO; i++)
	{
		if(fStereoAudioPlugins[i])
		{
			fStereoAudioPlugins[i]->SetParamValue(iParam,val);
		}
	}
}


inline void TArmeWSObj::SetAudioPluginByPass(bool bypass)
{
	for(SInt32 i = 0; i < kMEMixingInfo_NbMaxMonoVAO; i++)
	{
		if(fMonoAudioPlugins[i])
		{
			fMonoAudioPlugins[i]->SetByPass(bypass);
		}
	}
	for(SInt32 i = 0; i < kMEMixingInfo_NbMaxStereoVAO; i++)
	{
		if(fStereoAudioPlugins[i])
		{
			fStereoAudioPlugins[i]->SetByPass(bypass);
		}
	}
}

inline VARPIPluginClass* TArmeWSObj::GetAudioPluginClass()
{
	return fStereoAudioPlugins[0]->GetClass();
}

#else //#ifdef OLD_RAO_VAO
inline float TArmeWSObj::GetAudioPluginParamValue(SInt32 iParam) const 
{
	return fRealTimePlugin->GetParamValue(iParam);
}

inline void TArmeWSObj::IncrAudioPluginParamValue(SInt32 iParam, float incr)
{
	fRealTimePlugin->IncrParamValue(iParam,incr);
}

inline void TArmeWSObj::SetAudioPluginParamValue(SInt32 iParam, float val)
{
	fRealTimePlugin->SetParamValue(iParam,val);
}

inline void TArmeWSObj::SetAudioPluginByPass(bool bypass)
{
	fRealTimePlugin->SetByPass(bypass);
}

inline const VARPIPluginClass* TArmeWSObj::GetAudioPluginClass() const
{
	return fRealTimePlugin->GetClass();
}


#endif //#ifdef OLD_RAO_VAO


inline VARPIPlugin* TArmeWSObj::GetOfflineRenderingPlugin()
{
	return fOfflinePlugin;
}

inline TArmeWSObj* TArmeWSObj::GetParent() const
{
	return GetPos().fParent;
}

inline VARPIPlugin* TArmeWSObj::GetAudioPlugin()
{
	return fRealTimePlugin;
}

inline const VARPIPlugin* TArmeWSObj::GetAudioPlugin() const
{
	return fRealTimePlugin;
}


