/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#ifndef TVideoClip_h
#include "TVideoClip.h"
#endif //TVideoClip_h

#ifndef ZzHelpers_h
#include "ZzHelpers.h"
#endif //ZzHelpers_h

#ifndef Zz_PathHelpers_h
#include "Zz_PathHelpers.h"
#endif //Zz_PathHelpers_h

#ifndef MovieHelper_h
#include "MovieHelper.h"
#endif //MovieHelper_h

#ifndef TArmeMasterMovie_h
#include "TArmeMasterMovie.h"
#endif //TArmeMasterMovie_h

#ifndef MEQTFloatSampleReader_h
#include "MEQTFloatSampleReader.h"
#endif //MEQTFloatSampleReader_h

#include "MESplitTaskImportMovieDataInVideoClip.h"

#ifndef ArmeChunk_h
#include "ArmeChunk.h"
#endif //ArmeChunk_h

#ifndef MIProgressBarDlgManager_h
#include "MIProgressBarDlgManager.h"
#endif //MIProgressBarDlgManager_h

#ifndef ArmeDataFileRef_h
#include "ArmeDataFileRef.h"
#endif //ArmeDataFileRef_h

#ifndef MEApplicationServices_h
#include "MEApplicationServices.h"
#endif //MEApplicationServices_h

#ifndef TArmeMediaEngine_h
#include "TArmeMediaEngine.h"
#endif //TArmeMediaEngine_h

#ifndef ZzException_h
#include "ZzException.h"
#endif //ZzException_h

/*TVideoClip::TVideoClip(ArmeDataFileRef* dataFileRef, TArmeDoc* doc, ZzSplitTask** splitTask, ArmeErr* err, const TArmeWSObjPos& pos) :
	TVideoObject("", doc, NULL, NULL)
{
	Init();
	
	char clipName[64];
	ZzGetFileNameFromPath(clipName,63,path);
	SetNameCopy(clipName);

//	fMasterMovie = doc->fMasterMovie;
	// load the movie
	Movie movie = ImportFile(path, doc, err);

	// add it in the master movie
	ArmeTime end;
	end = pos.fStartTime + doc->SecondsToArmeTime(GetMovieDurationSec(movie));
	fTrackList = new TVideoTrack(this, pos.fStartTime, end);
	// insert the clip into the document
	fTrackList->fQTTrack = fMasterMovie->AddSubMovie(this, movie, pos.fStartTime);

	ArmeChunk chunk(dataFileRef, ARME_TIME_ZERO, end - pos.fStartTime);
	fVisibleChunkList = new ArmeChunkList(chunk);

	_SetPos(pos);

	ZzSplitTask* mySplitTask = new MESplitTaskImportMovieDataInVideoClip(path, this, err);
	delete mySplitTask;
	//$$$TODO_ALEX
	// instead of deleting mySplitTask, assign it to *splitTask and move the actual code
	// from the ctor to PerformNextBit.
	//$$$TODO_ALEX handle the err return code, and delete the task if needed.
	*splitTask = NULL;
};*/

//initialize all member variables to default value (is called by every constructor)
void TVideoClip::Init()
{

	//fOriginalTrack = NULL;
	fCompleteChunkList = NULL;
	fVisibleChunkList = NULL;
}

TVideoClip::TVideoClip(ArmeDataFileRef* inRef, TArmeDoc* doc, MIProgressBarDlgManager* progressMngr, ArmeErr* err, const TArmeWSObjPos& pos, bool stretchSmallClips) :
	TVideoObject("", doc)
{
	ArmeDataFileRef* highResFileRef = NULL;

	try 
	{
		Init();
	
		// this may change if tracks don't start at beginning!
		TArmeWSObjPos localPos(pos);
	
		// make a movie out of it
		int trackIndex = 1;
		char path[kMaxPathLength + 1] = { 0 };
		ArmeDataFileRef* dataFileRef = inRef;
		dataFileRef->GetFilePath(path);
		Movie movie = ImportFile(path, doc, err);
//		fMasterMovie->GetOriginalTrack()->AddFileInOriginalTrack((ArmeVideoDataFileRef*)dataFileRef , movie, trackIndex);
	
		bool bScaleStill = false;

		// check for imported picture
		TimeValue duration = ::GetMovieDuration(movie);
		TimeScale scale = ::GetMovieTimeScale(movie);
		float importMovieDurationInSec = float(duration)/float(scale);
		float movieFrameDurationInSec = float(doc->GetVideoFrameDuration())/float(doc->GetVideoTimeScale());

		if ((importMovieDurationInSec < (2.5f * movieFrameDurationInSec)) && stretchSmallClips)
		//check if it is a single frame
//		if ((duration < long(3*fMasterMovie->GetVideoFrameDuration())) && stretchSmallClips)
		{
			bScaleStill = true;
			/*	$$NOTE : this code works fine but I had to remove it due to a QT bug :
				If a text clip (or a still image) is involved in a cross fade and if there is only
				one media sample at the beginning of the clip, on export in DV, QT will only compute the
				transition each time it sees a sample (which means only once => no transition at all)*/
/*			::ScaleMovieSegment(movie, 0, duration, TimeValue(TArmeMediaEngine::ApplicationServices()->GetStillClipDuration() * float(scale)));*/
			TimeValue oneFrameDuration = GetMovieDuration(movie); 
			TimeValue currentDuration = oneFrameDuration;
			while (currentDuration < TimeValue(TArmeMediaEngine::ApplicationServices()->GetStillClipDuration() * float(GetMovieTimeScale(movie))))
			{
				CopyTrackSegment(movie, GetFirstVideoTrack(movie), movie, GetFirstVideoTrack(movie), currentDuration, 0, oneFrameDuration);
				currentDuration += oneFrameDuration;
			}
	
			//and save the 5 seconds movie on disk
/*			FSSpec dstFile;
			OSErr res;
			char sourceStillFile[kMaxPathLength];
			Rect stillImageBox;
	
			GetMovieBox(movie, &stillImageBox);
			
			//Export the still image to a high res file
			highResFileRef = doc->GetNewVideoDataFileRef();
			dataFileRef = highResFileRef;
			highResFileRef->GetFilePath(sourceStillFile);
			dstFile = FSSpecFromNonExistingFilePath(sourceStillFile, FOUR_CHAR_CODE('TVOD'), FOUR_CHAR_CODE('MooV'), res);
			ThrowOnOSStatus(res);
	
			SetMovieProgressProc(movie, QTMovieProgressProc, (long)progressMngr);
			res = ConvertMovieToFile(movie, GetMovieTrack(movie, 1), &dstFile, 0, 0, -1, 0, 0, GetCodecInstance(fMasterMovie->GetWorkingCodecType())); 
			ThrowOnOSStatus(res);
			
	#ifdef WORK_WITH_PREVIEW
			//Export the still image to a preview file
			char previewStillFile[kMaxPathLength];
			dataFileRef = doc->GetNewVideoDataFileRef();
			dataFileRef->GetFilePath(previewStillFile);
			dataFileRef->SetHighResSourceFile(highResFileRef);
			dstFile = FSSpecFromNonExistingFilePath(previewStillFile, FOUR_CHAR_CODE('TVOD'), FOUR_CHAR_CODE('MooV'), res);
			ThrowOnOSStatus(res);
	
			SetMovieProgressProc(movie, QTMovieProgressProc, (long)progressMngr);
			res = ConvertMovieToFile(movie, GetMovieTrack(movie, 1), &dstFile, 0, 0, -1, 0, 0, GetCodecInstance(fMasterMovie->GetPreviewCodecType())); 
			ThrowOnOSStatus(res);
	#endif //WORK_WITH_PREVIEW
			
			trackIndex = 1;
			DisposeMovie(movie);
			movie = ImportFile(dstFile, trackIndex, doc, err);
			fMasterMovie->GetOriginalTrack()->AddFileInOriginalTrack((ArmeVideoDataFileRef*)dataFileRef , movie, trackIndex);*/
		}

		fMasterMovie->GetOriginalTrack()->AddFileInOriginalTrack((ArmeVideoDataFileRef*)dataFileRef , movie, trackIndex);
			
		char clipName[64];
		ZzGetFileNameFromPath(clipName,63,path);
		SetNameCopy(clipName);
	
		// add it in the master movie
		ArmeTime movieLength = ArmeTimeFromTimeValue2(movie, doc, GetMovieDuration(movie));
		doc->RoundArmeTimeToNearestVideoFrame(localPos.fStartTime);
		TVideoChunk chunk(dataFileRef, ARME_TIME_ZERO, movieLength);
		// let's ensure that the clip will have an integer number of frames
		ArmeTime end = localPos.fStartTime + movieLength;
	/*	if (bScaleStill)
			end = localPos.fStartTime + doc->SecondsToArmeTime(kPictureMovieLength);*/
		doc->RoundArmeTimeToNearestVideoFrame(end);
		fVisibleChunkList = new TArmeChunkList<TVideoChunk>(&chunk, 1, localPos.fStartTime);
		//time stretch the movie to an integer number of frames
		fVisibleChunkList->TimeStretch(localPos.fStartTime, localPos.fStartTime + movieLength, end); 
		
		//the original track contains the whole track (including the part that is trimmed)
		fCompleteChunkList = fVisibleChunkList->GetCopy();
	
		_SetPos(localPos);
	
		// if there is a solo object, this one must be muted
		if (GetDoc()->GetVideoSoloObject())
			SetMutedForSolo(true);
			
		DisposeMovie(movie);
	}
	catch (ZzException* e)
	{
		if (fVisibleChunkList) delete fVisibleChunkList;
		if (fCompleteChunkList) delete fCompleteChunkList;
		if (highResFileRef)
		{			

			doc->RemoveVideoDataFileRef(highResFileRef);
			delete highResFileRef;
		}

		throw;
		err->type = kArmeErrUnknown;
	}
	
}

TVideoClip::TVideoClip(const ZzString& name, const FSSpec& movieFile, int trackIndex, TArmeDoc* doc, MIProgressBarDlgManager*	progressMngr, ArmeErr* err, const TArmeWSObjPos& pos) :
	TVideoObject("", doc)
{
	ArmeDataFileRef* dataFileRef = NULL;
	char moviePath[1024];
	ArmeErr armeErr;
	
	try
	{
		Movie movie;
		bool isNewDataRef = true;
	
		Init();

		FSpPathFromLocation(&movieFile, moviePath);
		
		// this may change if tracks don't start at beginning!
		TArmeWSObjPos localPos(pos);
			
	//	char clipName[64];
	//	p2cstrcpy(clipName, movieFile.name);
		SetNameCopy(name.getUTF8String());
	
		// load the movie
		Movie source = ImportFile(movieFile, trackIndex, doc, err);
		if (!source)
		{
			vernl(0, "TVideoClip(): Movie could not be opened!\n");
		}
	
		// check for imported picture
		TimeValue duration = ::GetMovieDuration(source);
		TimeScale scale = ::GetMovieTimeScale(source);
		float importMovieDurationInSec = float(duration)/float(scale);
		float movieFrameDurationInSec = float(doc->GetVideoFrameDuration())/float(doc->GetVideoTimeScale());
		if (importMovieDurationInSec < (2.5f * movieFrameDurationInSec))
		{
		//	::ScaleMovieSegment(source, 0, duration, GetDoc()->GetVideoFrameDuration() /*TimeValue(TArmeMediaEngine::ApplicationServices()->GetStillClipDuration() * float(scale))*/);
	
			char sourceStillFile[kMaxPathLength];
			FSSpec dstFile;
			OSErr res;
			Rect stillImageBox;
			
			GetMovieBox(source, &stillImageBox);
	
			//Export the still image to a high res file
			dataFileRef = doc->GetNewVideoDataFileRef(moviePath, &isNewDataRef);
			dataFileRef->GetFilePath(sourceStillFile);

			if (!isNewDataRef)
			{
				dstFile = Path2FSspec(sourceStillFile, false);
			}
			else
			{
				dstFile = FSSpecFromNonExistingFilePath(sourceStillFile, FOUR_CHAR_CODE('TVOD'), FOUR_CHAR_CODE('MooV'), res);
				ThrowOnOSStatus(res);
		
				//SetMovieProgressProc(source, QTMovieProgressProc, (long)progressMngr);
				//res = ConvertMovieToFile(source, GetMovieTrack(source, 1), &dstFile, 0, 0, -1, 0, 0, GetCodecInstance(fMasterMovie->GetWorkingCodecType())); 
				
				short movieResFile;
				short movieResId;
				ZzString bundlePath = ZzGetMainBundlePath();
				ZzString imagePath = bundlePath + "/Contents/Resources/black3232.jpg";
				Movie backgroundMovie = LoadMovie(imagePath.getUTF8String(), &movieResFile, &movieResId, &armeErr);
				verntl(videoAlain, 4, "Imported black3232.jpg" );
	//			ScaleMovieSegment(backgroundMovie, 0, GetMovieDuration(backgroundMovie), GetDoc()->GetVideoFrameDuration() /*TimeValue(TArmeMediaEngine::ApplicationServices()->GetStillClipDuration() * float(scale))*/);
			
				Movie scaledImage = NewMovie(newMovieActive);
				
				Track QTBackgroundTrack = CreateNewTrackInMovie(scaledImage, fMasterMovie->GetVideoTimeScale(), fMasterMovie->GetWidth(), fMasterMovie->GetHeight());
				ArmeTime srcBackgroundDuration = ArmeTimeFromTimeValue2(backgroundMovie, GetDoc(), GetMovieDuration(backgroundMovie));
				CopyTrackSegment(scaledImage, QTBackgroundTrack, backgroundMovie, GetFirstVideoTrack(backgroundMovie), 0, 0, GetMovieDuration(backgroundMovie));
				OSErr err = ScaleTrackSegment(QTBackgroundTrack, 0, GetDoc()->TimeValueFromArmeTime(srcBackgroundDuration), GetDoc()->GetVideoFrameDuration() /*GetDoc()->TimeValueFromArmeTime(GetDoc()->SecondsToArmeTime(TimeValue(TArmeMediaEngine::ApplicationServices()->GetStillClipDuration())))*/);
				ThrowOnOSStatus(err);
				DisposeMovie(backgroundMovie);
		
				Track track = CreateNewTrackInMovie(scaledImage, fMasterMovie->GetVideoTimeScale(), doc->fMasterMovie->GetWidth(), doc->fMasterMovie->GetHeight(), VideoMediaType);
				ArmeTime srcDuration = ArmeTimeFromTimeValue2(source, GetDoc(), GetMovieDuration(source));
				CopyTrackSegment(scaledImage, track, source, GetFirstVideoTrack(source), 0, 0, GetMovieDuration(source) /*TimeValue(TArmeMediaEngine::ApplicationServices()->GetStillClipDuration() * float(scale))*/);
		
				ScaleTrackSegment(track, 0, GetDoc()->TimeValueFromArmeTime(srcDuration), GetDoc()->GetVideoFrameDuration());
		
				//put the background track behind
				SetTrackLayer(QTBackgroundTrack, 1500);
				SetTrackLayer(track, 1000);
		
				MatrixRecord aMatrix;
				Rect masterMovieBox;
				Rect target;
				Rect temp;
		
		//		GetMovieBox(GetDoc()->fMasterMovie->GetQTMovie(), &masterMovieBox);
				masterMovieBox.top = 0;
				masterMovieBox.bottom = doc->fMasterMovie->GetHeight();
				masterMovieBox.left = 0;
				masterMovieBox.right = doc->fMasterMovie->GetWidth();
				
				target = masterMovieBox;		
				GetMovieBox(scaledImage, &temp);
				
				
				float targetRatio = float(target.right)/float(target.bottom);
				float imageRatio;
				if (fMasterMovie->GetWorkingCodecType() == kDvPal720_576)
				{
					imageRatio = float(stillImageBox.right/768.0f*720.0f)/float(stillImageBox.bottom);  //the 768/720 takes care of non-squared pixels
				}
				else if (fMasterMovie->GetWorkingCodecType() == kDvNtsc720_480)
				{
					imageRatio = float(stillImageBox.right/768.0f*864.0f)/float(stillImageBox.bottom); //the 768/864 takes care of non-squared pixels
				}
				else if (fMasterMovie->GetWorkingCodecType() == kDvPal16_9)
				{
					imageRatio = float(stillImageBox.right/1024.0f*720.0f)/float(stillImageBox.bottom);  //the 768/720 takes care of non-squared pixels
				}
				else if (fMasterMovie->GetWorkingCodecType() == kDvNtsc16_9)
				{
					imageRatio = float(stillImageBox.right/1024.0f*864.0f)/float(stillImageBox.bottom); //the 768/864 takes care of non-squared pixels
				}
				else
				{
					imageRatio = float(stillImageBox.right)/float(stillImageBox.bottom);
				}
		
				if (targetRatio < imageRatio)
				{
					int newHeight = int(float(target.bottom) * targetRatio/imageRatio);
					int offset = (target.bottom - newHeight)/2;
					target.top = offset;
					target.bottom = offset + newHeight;
				}
				else
				{
					int newWidth = int(float(target.right) * imageRatio/targetRatio);
					int offset = (target.right - newWidth)/2;
					target.left = offset;
					target.right = offset + newWidth;		
				}
				RectMatrix(&aMatrix, &masterMovieBox, &target);
				ThrowOnOSStatus(res);
				res = GetMoviesError();	
				SetTrackMatrix(track, &aMatrix);
				res = GetMoviesError();	
				ThrowOnOSStatus(res);
				
				GetMovieBox(scaledImage, &temp);
						
				SetMovieProgressProc(scaledImage, QTMovieProgressProc, (long)progressMngr);
				res = ConvertMovieToFile(scaledImage, 0, &dstFile, 0, 0, -1, 0, 0, GetCodecInstance(fMasterMovie->GetWorkingCodecTypeForPictures())); 
				//SaveMovie(scaledImage, sourceStillFile);
				DisposeMovie(scaledImage);
				ThrowOnOSStatus(res);
			}
			movie = ImportFile(dstFile, trackIndex, doc, &armeErr);
			/*	$$NOTE : this code works fine but I had to remove it due to a QT bug :
				If a text clip (or a still image) is involved in a cross fade and if there is only
				one media sample at the beginning of the clip, on export in DV, QT will only compute the
				transition each time it sees a sample (which means only once => no transition at all)*/
/*			ScaleTrackSegment(GetFirstVideoTrack(movie), 0, GetMovieDuration(movie), TimeValue(TArmeMediaEngine::ApplicationServices()->GetStillClipDuration() * float(GetMovieTimeScale(movie))));*/
			TimeValue oneFrameDuration = GetMovieDuration(movie); 
			TimeValue currentDuration = oneFrameDuration;
			while (currentDuration < TimeValue(TArmeMediaEngine::ApplicationServices()->GetStillClipDuration() * float(GetMovieTimeScale(movie))))
			{
				CopyTrackSegment(movie, GetFirstVideoTrack(movie), movie, GetFirstVideoTrack(movie), currentDuration, 0, oneFrameDuration);
				currentDuration += oneFrameDuration;
			}

		}
		else
		{
			// try importing to work codec if we're extracting a track (trackIndex > 0)
			FSSpec dstFile;
			OSErr res;
			char fileInProjectFolder[kMaxPathLength];
			dataFileRef = doc->GetNewVideoDataFileRef(moviePath, &isNewDataRef);
			dataFileRef->GetFilePath(fileInProjectFolder);
		
			if (!isNewDataRef)
			{
				movie = ImportFile(fileInProjectFolder, doc, err);
						if ( err->type != kArmeOK ) {
							DisposeMovie(source);
							return;
						}
			}
			else
			{
				bool convertToWorkingCodec = false; 
				
				if (convertToWorkingCodec)
				{
					//convert the movie to working codec
					dstFile = FSSpecFromNonExistingFilePath(fileInProjectFolder, FOUR_CHAR_CODE('TVOD'), FOUR_CHAR_CODE('MooV'), res);
					ThrowOnOSStatus(res);
					SetMovieProgressProc(source, QTMovieProgressProc, (long)progressMngr);
					res = ConvertMovieToFile(source, GetMovieTrack(source, trackIndex), &dstFile, 0, 0, -1, 0, 0, GetCodecInstance(fMasterMovie->GetWorkingCodecType())); 
					ThrowOnOSStatus(res);
					trackIndex = 1;
					movie = ImportFile(dstFile, trackIndex, doc, err);
				}
				else
				{
					char sourceFile[1024];
					FSpPathFromLocation(&movieFile, sourceFile);
	
					// if not referencing external files, copy into project's folder
					if (!TArmeMediaEngine::ApplicationServices()->IsReferencingExternalFiles())
					{
						//copy in project folder
						bool ok;
						ok = ZzCopyMacFile(sourceFile,fileInProjectFolder,progressMngr);
						if ( !ok ) {
							err->type = kArmeErrErrorWritingInFile;
							DisposeMovie(source);
							return;
						}
						movie = ImportFile(fileInProjectFolder, doc, err);
						if ( err->type != kArmeOK ) {
							remove(fileInProjectFolder);
							DisposeMovie(source);
							return;
						}
					}
					else //don't copy in project folder, simply keep a reference on the file
					{
						dataFileRef->SetReferenceToExternalFile();
						movie = ImportFile(sourceFile, doc, err);
						if ( err->type != kArmeOK ) {
							DisposeMovie(source);
							return;
						}
					}
				}
			}
		}
		
		
		// add it in the master movie
		ArmeTime movieLength = ArmeTimeFromTimeValue2(movie, doc, GetMovieDuration(movie));
		doc->RoundArmeTimeToNearestVideoFrame(localPos.fStartTime);
		TVideoChunk chunk(dataFileRef, ARME_TIME_ZERO, movieLength);
	
		// let's ensure that the clip will have an integer number of frames
		ArmeTime end = localPos.fStartTime + movieLength;
		doc->RoundArmeTimeToNearestVideoFrame(end);
		fVisibleChunkList = new TArmeChunkList<TVideoChunk>(&chunk, 1, localPos.fStartTime);
	
		//time stretch the movie to an integer number of frames
		fVisibleChunkList->TimeStretch(localPos.fStartTime, localPos.fStartTime + movieLength, end); 
		
		if (isNewDataRef)
		{
			fMasterMovie->GetOriginalTrack()->AddFileInOriginalTrack((ArmeVideoDataFileRef*)dataFileRef , movie, trackIndex);
		}
	
		//the original track contains the whole track (including the part that is trimmed)
		fCompleteChunkList = fVisibleChunkList->GetCopy();
	
		_SetPos(localPos);
	
		// if there is a solo object, this one must be muted
		if (GetDoc()->GetVideoSoloObject())
			SetMutedForSolo(true);
	
		DisposeMovie(source);
		DisposeMovie(movie);
	}
	catch (ZzException* e)
	{
		if (fVisibleChunkList) delete fVisibleChunkList;
		if (fCompleteChunkList) delete fCompleteChunkList;
		if (dataFileRef)
		{
			doc->RemoveVideoDataFileRef(dataFileRef);
			delete dataFileRef;
		}

		throw;
		err->type = kArmeErrUnknown;
	}
};

/*TVideoClip::TVideoClip(
	const char*				name, 
	TArmeDoc* 				doc, 
	const TArmeWSObjPos& 	pos) : 
	TVideoObject(name, doc)
{
	Init();
	
	//the original track contains the whole track (including the part that is trimmed)
//	fCompleteChunkList ???

	ZzAssert(0); //is this constructor still used ?

	// insert the clip into the document
	TArmeWSObjPos copyPos = pos;
	doc->RoundArmeTimeToNearestVideoFrame(copyPos.fStartTime);
	_SetPos(copyPos);
};*/

TVideoClip::~TVideoClip()
{
//	fMasterMovie->RemoveSubMovie(this);  object is already out of hierarchy

	delete fVisibleChunkList;
	delete fCompleteChunkList;
};

TVideoClip::TVideoClip(TVideoClip&) : TVideoObject("", NULL) {ZzAssert(0);};
TVideoClip::TVideoClip(const TVideoClip&) : TVideoObject("", NULL) {ZzAssert(0);};

// constructor used in GetSliceCopy
TVideoClip::TVideoClip(const char* name, TArmeDoc* doc) : TVideoObject(name)
{
	Init();
	fDoc = doc;
}

TVideoClip::TVideoClip(ArmeArchiveHandler* unarchiver) :
	TVideoObject(unarchiver)
{
	Init();
	
	fVisibleChunkList = new TArmeChunkList<TVideoChunk>();
	fVisibleChunkList->ReadFromArchive(unarchiver);
	fCompleteChunkList = new TArmeChunkList<TVideoChunk>();
	fCompleteChunkList->ReadFromArchive(unarchiver);

	if (unarchiver->GetFileVer() < 0x00010024)
	{
		ArmeTime dummyRead = unarchiver->ReadArmeTime();
		dummyRead = unarchiver->ReadArmeTime();
	}
}


bool TVideoClip::WriteToArchive(ArmeArchiveHandler* archiver) const
{
	TVideoObject::WriteToArchive(archiver);
	
	fVisibleChunkList->WriteToArchive(archiver);
	fCompleteChunkList->WriteToArchive(archiver);
	
/*	// from version 0x00010001  removed in 0x00010024
	archiver->WriteArmeTime(fStartTrim);
	archiver->WriteArmeTime(fEndTrim);*/
	
	// from version 0x00010003
//	archiver->WriteCPPObject(fOriginalTrack);
	
	return true;
}

void TVideoClip::SetStartTrim(ArmeTime startTrim)
{
	//remove representation of the clip in the master movie
	fMasterMovie->RemoveSubMovie(this);

	do 
	{
		ArmeTime startTime = fCompleteChunkList->GetStartTime() + startTrim;
		fDoc->RoundArmeTimeToNearestVideoFrame(startTime);
		fStartTrim = startTime - fCompleteChunkList->GetStartTime();
	
		// replace the chunkList by the new visible chunk list
		delete fVisibleChunkList;
		fVisibleChunkList = fCompleteChunkList->GetSliceCopy(fCompleteChunkList->GetStartTime() + fStartTrim, fCompleteChunkList->GetStartTime() + fCompleteChunkList->GetDuration() - fEndTrim);
		// if the trim is such that all the clip is hidden, reduce the startTrim until there is one frame left in the clip
		if (fVisibleChunkList->GetDuration() <= ARME_TIME_ZERO)
			startTrim -= GetDoc()->ArmeTimeFromTimeValue(fMasterMovie->GetVideoFrameDuration());
			
	} while (fVisibleChunkList->GetDuration() <= ARME_TIME_ZERO);

	// insert the new representation of the clip in the master movie
	fMasterMovie->AddSubMovie(this);
}

void TVideoClip::SetEndTrim(ArmeTime endTrim)
{
	//remove representation of the clip in the master movie
	fMasterMovie->RemoveSubMovie(this);

	do 
	{
		ArmeTime endTime = fCompleteChunkList->GetStartTime() + fCompleteChunkList->GetDuration() - endTrim;
		fDoc->RoundArmeTimeToNearestVideoFrame(endTime);
		fEndTrim = fCompleteChunkList->GetStartTime() + fCompleteChunkList->GetDuration() - endTime;

		// replace the chunkList by the new visible chunk list
		delete fVisibleChunkList;
		fVisibleChunkList = fCompleteChunkList->GetSliceCopy(fCompleteChunkList->GetStartTime() + fStartTrim, fCompleteChunkList->GetStartTime() + fCompleteChunkList->GetDuration() - fEndTrim);

		// if the trim is such that all the clip is hidden, increase the endTrim until there is one frame left in the clip
		if (fVisibleChunkList->GetDuration() <= ARME_TIME_ZERO)
			endTrim -= GetDoc()->ArmeTimeFromTimeValue(fMasterMovie->GetVideoFrameDuration());
			
	} while (fVisibleChunkList->GetDuration() <= ARME_TIME_ZERO);

	// insert the new representation of the clip in the master movie
	fMasterMovie->AddSubMovie(this);
}



void TVideoClip::SetMemento(TArmeWSObjMemento* memento)
{
	TVideoClipMemento* videoMemento = dynamic_cast<TVideoClipMemento*>(memento);
	ZzAssert(videoMemento);

	//remove representation of the clip in the master movie
	fMasterMovie->RemoveSubMovie(this);

	// replace visible chunkList by the one from the memento
	TArmeChunkList<TVideoChunk>* old = fVisibleChunkList;
	fVisibleChunkList = videoMemento->fVisibleChunkList->GetCopy();
	delete old;  // we delete it after we have made the copy to prevent the ref counters to reach 0 and then to delete the files

	// replace complete chunkList by the one from the memento
	old = fCompleteChunkList;
	fCompleteChunkList = videoMemento->fCompleteChunkList->GetCopy();
	delete old;  // we delete it after we have made the copy to prevent the ref counters to reach 0 and then to delete the files

/*	fStartTrim = videoMemento->fStartTrim;
	fEndTrim = videoMemento->fEndTrim;*/
	
	// insert the new representation of the clip in the master movie   (note, there is _SetPos in there so, the clip will be added to MasterMovie)
	TVideoObject::SetMemento(memento);
}

TArmeWSObjMemento* TVideoClip::GetMemento() const
{
	TArmeWSObjMemento* memento = new TVideoClipMemento();
	_FillMemento(memento);
	return memento;
}

void TVideoClip::_FillMemento(TArmeWSObjMemento* memento) const
{
	TVideoClipMemento* videoMemento = dynamic_cast<TVideoClipMemento*>(memento);
	ZzAssert(videoMemento);

	TVideoObject::_FillMemento(memento);
	
	//copy the parameters of the current clip
	videoMemento->fVisibleChunkList = fVisibleChunkList->GetCopy();
	videoMemento->fCompleteChunkList = fCompleteChunkList->GetCopy();
/*	videoMemento->fStartTrim = fStartTrim;
	videoMemento->fEndTrim = fEndTrim;*/
}

TArmeWSObj* TVideoClip::GetSliceCopy(ArmeTime startTime, ArmeTime endTime, const char* tempDirectoryName, MIProgressBarDlgManager* progressMngr) const
{
	if (!ContainsVideoDuring(startTime, endTime, true))
		return NULL;

	ArmeTime copyStartTime;  //the relative pos of the copied object
	ArmeTime absoluteStartTime = GetAbsoluteStartTimeOfVideoClip(true);
	ArmeTime absoluteEndxTime = GetAbsoluteEndxTimeOfVideoClip(true);
	
	if ((startTime == ARME_TIME_UNDEF) || (startTime == ARME_TIME_MIN))
		copyStartTime = GetPos().fStartTime;
	else if (startTime < absoluteStartTime)
	{
		if (startTime < fPos.fParent->GetAbsoluteStartTime())
			copyStartTime = absoluteStartTime - fPos.fParent->GetAbsoluteStartTime();
		else
			copyStartTime = absoluteStartTime - startTime;
	}
	else
		copyStartTime = ARME_TIME_ZERO;

	if ((startTime == ARME_TIME_UNDEF) || (startTime < absoluteStartTime))
		startTime = absoluteStartTime;
	
	if ((endTime == ARME_TIME_UNDEF) || (endTime > absoluteEndxTime))
		endTime = absoluteEndxTime;

	GetDoc()->RoundArmeTimeToNearestVideoFrame(startTime);
	GetDoc()->RoundArmeTimeToNearestVideoFrame(endTime);

	bool bCopyCompleteObject = (startTime == absoluteStartTime) && (endTime == absoluteEndxTime);

	if (startTime == endTime)
		return NULL;

	// first, export the piece of clip to a file (for inter-doc copy/paste)
/*	char exportFileName[256];
	ZzAppendFileNameToPath(exportFileName, 256, tempDirectoryName, GetName());
	ZzAppendExtension(exportFileName, 255, "mov");*/
	//$$$TEMP : this is to optimize single-doc copy operation but it breaks inter-doc copy operation
//	ExportToFile(exportFileName, startTime, endTime);
	
	//then, create a new clip with a reference on this file
	TVideoClip* copy = new TVideoClip(GetName(), GetDoc());  
//	strcpy(copy->fSourceFile, exportFileName);
	copy->fPos.fStartTime = copyStartTime; 
	copy->fPos.fParent = NULL;
	copy->SetMuted(GetMuted());
	copy->fMasterMovie = fMasterMovie;
	// copy chunk list
	copy->fVisibleChunkList = fVisibleChunkList->GetSliceCopy(startTime, endTime);
	
	if (bCopyCompleteObject)
	{
		copy->fCompleteChunkList = fCompleteChunkList->GetCopy();
		copy->fStartTrim = fStartTrim;
		copy->fEndTrim = fEndTrim;
		copy->fCachedAbsoluteStartTime = fCachedAbsoluteStartTime;
	}
	else
	{
		copy->fCompleteChunkList = fCompleteChunkList->GetSliceCopy(startTime, endTime);
		copy->fStartTrim = ARME_TIME_ZERO;
		copy->fEndTrim = ARME_TIME_ZERO;
		copy->fCachedAbsoluteStartTime = startTime;
	}
	
	return copy;
}

TVideoClip::TVideoClip(const TVideoClip* clip, TArmeDoc* doc, TArmeWSObjPos& pos) : TVideoObject(clip->GetName(), doc)
{
	Init();

	fVisibleChunkList = clip->fVisibleChunkList->GetCopy();

	fCompleteChunkList = clip->fCompleteChunkList->GetCopy();
	
	fStartTrim = clip->fStartTrim;
	fEndTrim = clip->fEndTrim;

	TArmeWSObjPos copyPos = pos;
	GetDoc()->RoundArmeTimeToNearestVideoFrame(copyPos.fStartTime);
	_SetPos(copyPos);
	
	// if there is a solo object, this one must be muted
	if (GetDoc()->GetVideoSoloObject())
		SetMutedForSolo(true);
}

void TVideoClip::TimeStretch(ArmeTime startTime, ArmeTime endTime, ArmeTime newEndTime)
{
	if ((startTime == ARME_TIME_UNDEF) || (startTime < GetAbsoluteStartTimeOfVideoClip(true)))
		startTime = GetAbsoluteStartTimeOfVideoClip(true);
	if ((endTime == ARME_TIME_UNDEF) || (endTime > GetAbsoluteEndxTimeOfVideoClip(true)))
		endTime = GetAbsoluteEndxTimeOfVideoClip(true);
	if ((newEndTime == ARME_TIME_UNDEF) || (newEndTime < GetAbsoluteStartTimeOfVideoClip(true)))
		return;

	GetDoc()->RoundArmeTimeToNearestVideoFrame(startTime);
	GetDoc()->RoundArmeTimeToNearestVideoFrame(endTime);
	GetDoc()->RoundArmeTimeToNearestVideoFrame(newEndTime);

	if (startTime == endTime)
		return;

	if (startTime == newEndTime)
		return;

	//remove representation of the clip in the master movie
	fMasterMovie->RemoveSubMovie(this);

	//time stretch the base chunkList and the visible chunk list
	fCompleteChunkList->TimeStretch(startTime, endTime, newEndTime);
	fVisibleChunkList->TimeStretch(startTime, endTime, newEndTime);

	// insert the new representation of the clip in the master movie
	fMasterMovie->AddSubMovie(this);
}

ArmeTime TVideoClip::InsertSlice(ArmeTime insertTime, const TArmeWSObj* obj, MIProgressBarDlgManager* progressMngr)
{
	if (!ContainsVideoAtTime(insertTime))
	{
		// when fading out, i.e, we insert something at the very end of the clip (ContainsVideoAtTime returns false then)
		if (!ContainsVideoAtTime(insertTime - _ArmeTime(1)))
			return ARME_TIME_UNDEF;
	}

	const TVideoClip* videoClip = dynamic_cast<const TVideoClip*>(obj);
	
	GetDoc()->RoundArmeTimeToNearestVideoFrame(insertTime);

	// if the clip we have to insert is coming from another document, we first need to import it (such that there is a movieFile in this
	// doc directory) and then insert a copy of it in this object and then, delete it
/*	Inter-doc copy is not supported now $$TODO*/

	//remove representation of the clip in the master movie
	fMasterMovie->RemoveSubMovie(this);

	// delete the trim if the insert point is at one of the end of the clip
	if (insertTime == GetAbsoluteStartTimeOfVideoClip())
	{
		fCompleteChunkList->DeleteSlice(fCompleteChunkList->GetStartTime(), insertTime - fCompleteChunkList->GetStartTime());
		fStartTrim = ARME_TIME_ZERO;
	}
	else if (insertTime == GetAbsoluteEndxTimeOfVideoClip())
	{
		fCompleteChunkList->DeleteSlice(insertTime, fEndTrim);
		fEndTrim = ARME_TIME_ZERO;
	}

	fCompleteChunkList->InsertChunkList(*(videoClip->fVisibleChunkList), insertTime);
	fVisibleChunkList->InsertChunkList(*(videoClip->fVisibleChunkList), insertTime);
	
	InvalidateCachedTimeInfo();

	// insert the new representation of the clip in the master movie
	fMasterMovie->AddSubMovie(this);

	fHasBeenEdited = true;

	return insertTime + videoClip->fVisibleChunkList->GetDuration();
}

bool TVideoClip::DeleteSlice(ArmeTime startTime, ArmeTime endTime, MIProgressBarDlgManager* progressMngr)
{
	bool bDeleteWholeObject = true;
	bool bDeleteStartTrim = false;
	bool bDeleteEndTrim = false;
	ArmeTime absoluteStartTime = GetAbsoluteStartTimeOfVideoClip(true);
	ArmeTime absoluteEndxTime = GetAbsoluteEndxTimeOfVideoClip(true);

	if ((startTime == ARME_TIME_UNDEF) || (startTime == ARME_TIME_MIN))
		startTime = absoluteStartTime;
	if ((endTime == ARME_TIME_UNDEF) || (endTime == ARME_TIME_MAX))
		endTime = absoluteEndxTime;

	ArmeTime cutStartTime = startTime;
	GetDoc()->RoundArmeTimeToNearestVideoFrame(cutStartTime);
	GetDoc()->RoundArmeTimeToNearestVideoFrame(endTime);

	if (cutStartTime <= absoluteStartTime)
		cutStartTime = absoluteStartTime;
	else
		bDeleteWholeObject = false;

	if (endTime >= absoluteEndxTime)
		endTime = absoluteEndxTime;
	else
		bDeleteWholeObject = false;

	if ((cutStartTime < absoluteEndxTime) && (endTime >= absoluteEndxTime))
		bDeleteEndTrim = true;
	if ((cutStartTime <= absoluteStartTime) && (endTime > absoluteStartTime))
		bDeleteStartTrim = true;

	fHasBeenEdited = true;
		
	if (bDeleteWholeObject)
	{
		return true;  // the object will be deleted outside this routine (in this case, we must not modify the startTrim and the endTrim)
	}
	else //delete a slice only
	{
		//remove representation of the clip in the master movie
		fMasterMovie->RemoveSubMovie(this);

		fVisibleChunkList->DeleteSlice(cutStartTime, endTime - cutStartTime);

		// delete the trim if the slice was reaching one of the end of the clip
		if (bDeleteEndTrim)
		{
			fCompleteChunkList->DeleteSlice(cutStartTime, endTime - cutStartTime + fEndTrim);
			fEndTrim = ARME_TIME_ZERO;
		}
		else if (bDeleteStartTrim)
		{
			fCompleteChunkList->DeleteSlice(cutStartTime - fStartTrim, endTime - cutStartTime + fStartTrim);
			fStartTrim = ARME_TIME_ZERO;
		}
		else
		{
			fCompleteChunkList->DeleteSlice(cutStartTime, endTime - cutStartTime);
		}
	
		InvalidateCachedTimeInfo();

		// if we have deleted 'something' before the video clip, the start of the clip must be moved to the start of what has been deleted
		if (cutStartTime != startTime)
		{
			ZzAssert(fPos.fParent);

			if (endTime > absoluteStartTime)
			{
				if (startTime < fPos.fParent->GetAbsoluteStartTime())
					fPos.fStartTime = ARME_TIME_ZERO;
				else
				{
					ArmeTime newAbsoluteStartTime = startTime; //startTime - fPos.fParent->GetAbsoluteStartTime();
					fPos.fStartTime = newAbsoluteStartTime - fPos.fParent->GetAbsoluteStartTime();
				}
			}
			else
			{
				if (startTime < fPos.fParent->GetAbsoluteStartTime())
				{
					if (endTime > fPos.fParent->GetAbsoluteStartTime())
						fPos.fStartTime -= (endTime - fPos.fParent->GetAbsoluteStartTime());
				}
				else			
					fPos.fStartTime -= (endTime - startTime); //startTime - fPos.fParent->GetAbsoluteStartTime();
			}
			fPos.fParent->_NotifyChildPosHasChanged();

			fVisibleChunkList->Move(startTime);
			fCompleteChunkList->Move(startTime - fStartTrim);
		}

		// insert the new representation of the clip in the master movie
		fMasterMovie->AddSubMovie(this);

		return false;
	}
}
	
void TVideoClip::Dump(int indentLevel) const
{
    static char* whiteSpace = "           ";

    // Display all the tracks
    verntl(armeWSObjsBenjamin,5,&(whiteSpace[10-indentLevel]) << "Clip : " << this);
    TVideoObject::Dump(indentLevel);
}

void TVideoClip::AddListOfStartAndEndTimeOfVideoClip(ArmeTime startTime, ArmeTime endTime, ZzArray<ArmeTime> &list, bool bConsiderAlsoMutedObjects) const
{
	//NOTE, there is the same implementation in TArmeTextClip

	ArmeTime clipStartTime = GetAbsoluteStartTimeOfVideoClip(bConsiderAlsoMutedObjects);
	ArmeTime clipEndTime = GetAbsoluteEndxTimeOfVideoClip(bConsiderAlsoMutedObjects);
	
	int i = 0;
	
	if ((clipStartTime >= startTime) && (clipStartTime <= endTime))
	{
	    if (list.FindIndexForItem(clipStartTime) == -1) // the trackrep is not yet in the list
		{
			if (list.CountItems() == 0)
				list.AddItem(clipStartTime);
			else
			{
				while ((i < list.CountItems()) && (clipStartTime > list[i]))
					i++;
				list.InsertItem(clipStartTime, i);
			}
		}
	}
	if ((clipEndTime >= startTime) && (clipEndTime <= endTime))
	{
	    if (list.FindIndexForItem(clipEndTime) == -1) // the trackrep is not yet in the list
		{
			if (list.CountItems() == 0)
				list.AddItem(clipEndTime);
			else
			{
				while ((i < list.CountItems()) && (clipEndTime > list[i]))
					i++;
				list.InsertItem(clipEndTime, i);
			}
		}
	}
}

ArmeTime TVideoClip::GetAbsoluteStartTimeOfVideoClip(bool bConsiderAlsoMutedObjects) const 
{
	if (bConsiderAlsoMutedObjects || IsVisible())
		return fVisibleChunkList->GetStartTime();
	else
		return ARME_TIME_UNDEF;
}

ArmeTime TVideoClip::GetAbsoluteEndxTimeOfVideoClip(bool bConsiderAlsoMutedObjects) const 
{
	if (bConsiderAlsoMutedObjects || IsVisible())
		return fVisibleChunkList->GetStartTime() + fVisibleChunkList->GetDuration();
	else
		return ARME_TIME_UNDEF;
};

ArmeTime TVideoClip::GranulateTime(ArmeTime t) const
{
	if (t >= GetAbsoluteEndxTimeOfVideoClip(true))
		return GetAbsoluteEndxTimeOfVideoClip(true);
	else if (t <= GetAbsoluteStartTimeOfVideoClip(true))
		return GetAbsoluteStartTimeOfVideoClip(true);

	return GetDoc()->RoundArmeTimeToNearestVideoFrame(t);
}


/*void TVideoClip::_SetPos( const TArmeWSObjPos& pos )
{
	// move the object in the hierarchy (it only updates position relative to the parent)
	TArmeWSObj::_SetPos(pos);
	
	// the track in the master movie must be moved to its new absolute position
	Move(GetAbsoluteStartTime());
}

void TVideoClip::_NotifyParentPosHasChanged()
{
	// recursive call to all children
	TArmeWSObj::_NotifyParentPosHasChanged();
	
	// the track in the master movie must be moved to its new absolute position
	Move(GetAbsoluteStartTime());
}*/


void TVideoClip::PosWillChange()
{
	fMasterMovie->RemoveSubMovie(this);
}
void TVideoClip::PosHasChanged()
{
	fMasterMovie->AddSubMovie(this);
}

void TVideoClip::Move()
{
	fVisibleChunkList->Move(GetAbsoluteStartTime());
	fCompleteChunkList->Move(GetAbsoluteStartTime() - GetStartTrim());
}

Movie TVideoClip::ImportFile(const char* path, TArmeDoc* doc, ArmeErr* err)
{
	short 		movieResFile = 0;
	short		movieResId = 0;

	Movie movie = LoadMovie(path, &movieResFile, &movieResId, err, nil, false);
	
/*	movieResId = 0;
	OSErr res = OpenMovieFile(&(doc->GetMasterMovie()->fsSpec), (short int*)&(doc->GetMasterMovie()->fResourceRefNum), fsRdWrPerm);
	res = AddMovieResource(movie, doc->GetMasterMovie()->fResourceRefNum, &movieResId, NULL);
	CloseMovieFile(doc->GetMasterMovie()->fResourceRefNum);*/
	return movie;
}

Movie TVideoClip::ImportFile(const FSSpec& movieFile, int trackIndex, TArmeDoc* doc, ArmeErr* err)
{
	short 		movieResFile = 0;

	return LoadMovie(movieFile, trackIndex, &movieResFile, err, false);
}

PicHandle TVideoClip::GetPict(ArmeTime time) const
{
    ArmeTime absoluteTime;
    absoluteTime = time + fVisibleChunkList->GetStartTime();
	absoluteTime = GetDoc()->RoundArmeTimeToNearestVideoFrame(absoluteTime);

	const TVideoChunk* chunk = dynamic_cast<const TVideoChunk*>(fVisibleChunkList->GetChunkAtTime(absoluteTime));
	if (chunk)
		return chunk->GetPict(absoluteTime - fVisibleChunkList->GetStartTimeOfChunk(chunk));
	else
		return NULL;
}

ArmeTime TVideoClip::GetDuration() const
{
	if (!fVisibleChunkList)
		return _ArmeTime(0);
	return fVisibleChunkList->GetDuration();
}


bool TVideoClip::ContainsVideoAtTime(ArmeTime t) const 
{
	ZzAssert(fVisibleChunkList);
	
	return (fVisibleChunkList->GetChunkAtTime(t) != NULL);
};

bool TVideoClip::ContainsVideoDuring(ArmeTime startTime, ArmeTime endTime, bool bConsiderAlsoMutedObjects, bool bConsiderAlsoTransparentTextClips) const
{
	if (!fVisibleChunkList)
		return false;

	//if it is currently not in the master movie, it is not visible (it is the case between a (RemoveSubMovie and AddSubMovie))
/*	if (!bConsiderAlsoMutedObjects && !IsInMasterMovie())
		return false;*/
		
	//it it is outtaHierarchy
	if (GetPos().fParent == NULL)
		return false;
	
	if (bConsiderAlsoMutedObjects || IsVisible())
	{
		if ((startTime != ARME_TIME_UNDEF) && (endTime != ARME_TIME_UNDEF))
		{
			if (fVisibleChunkList->GetStartTime() >= startTime) 
			{
				if (fVisibleChunkList->GetStartTime() < endTime)
					return true;
			}
			else if (fVisibleChunkList->GetEndxTime() > startTime)
			{
				return true;
			}
		}
		else if ((startTime == ARME_TIME_UNDEF) && (endTime != ARME_TIME_UNDEF))
		{
			if (fVisibleChunkList->GetStartTime() < endTime)
				return true;
		}
		else if ((startTime != ARME_TIME_UNDEF) && (endTime == ARME_TIME_UNDEF))
		{
			if (fVisibleChunkList->GetEndxTime() > startTime)
				return true;
		}
		else
			return true; //both startTime and endTime are undef
	}

	return false;
}


TVideoClipMemento::~TVideoClipMemento() 
{
	delete fVisibleChunkList;
	delete fCompleteChunkList;
}

TVideoClipMemento::TVideoClipMemento()
{
	fVisibleChunkList = NULL;
	fCompleteChunkList = NULL;
};

TVideoClipMemento::TVideoClipMemento(ArmeArchiveHandler* unarchiver) : TVideoObjectMemento(unarchiver)
{
	fVisibleChunkList = new TArmeChunkList<TVideoChunk>();
	fVisibleChunkList->ReadFromArchive(unarchiver);
	fCompleteChunkList = new TArmeChunkList<TVideoChunk>();
	fCompleteChunkList->ReadFromArchive(unarchiver);

	if (unarchiver->GetFileVer() < 0x00010024)
	{
		ArmeTime dummyRead = unarchiver->ReadArmeTime();
		dummyRead = unarchiver->ReadArmeTime();
	}
}

bool TVideoClipMemento::WriteToArchive(ArmeArchiveHandler* archiver) const
{
	TVideoObjectMemento::WriteToArchive(archiver);

	fVisibleChunkList->WriteToArchive(archiver);
	fCompleteChunkList->WriteToArchive(archiver);
	
/*	archiver->WriteArmeTime(fStartTrim);
	archiver->WriteArmeTime(fEndTrim);*/
		
	return true;
}
