/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
 
#define TVideoClip_h
#ifndef TVideoClip_h
#include "TVideoClip.h"
#endif //TVideoClip_h

#ifndef TVideoObject_h
#include "TVideoObject.h"
#endif //TVideoObject_h

#ifndef ArmeChunk_h
#include "ArmeChunk.h"
#endif //ArmeChunk_h
 
#include <Quicktime/Quicktime.h>


class ZzSplitTask;
class MIProgressBarDlgManager;
class TArmeMasterMovie; 
class TVideoClipMemento;
class ArmeDataFileRef;

// #define 1 to use StdCompression user interface for work codec compression
#ifndef USE_STDCOMPRESSION
#define USE_STDCOMPRESSION 0
#endif USE_STDCOMPRESSION

class TVideoClip : public TVideoObject
{
public :
/*	TVideoClip(
		ArmeDataFileRef* 		dataFileRef, 
		TArmeDoc*				doc, 
		ZzSplitTask**			splitTask, 
		ArmeErr* 				err, 
		const TArmeWSObjPos&	pos);*/

	TVideoClip(
		const ZzString&				name,
		const FSSpec&				movieFile, 
		int							trackIndex, 
		TArmeDoc*					doc, 
		MIProgressBarDlgManager*	progressMngr, 
		ArmeErr* 					err, 
		const TArmeWSObjPos&		pos);

	TVideoClip(
		ArmeDataFileRef* 			inRef,
		TArmeDoc* 					doc, 
		MIProgressBarDlgManager*	progressMngr, 
		ArmeErr* 					err, 
		const TArmeWSObjPos&		pos,
		bool 						stretchSmallClips);

/*	TVideoClip(
		const char*				name, 
		TArmeDoc* 				doc, 
		const TArmeWSObjPos& 	pos);*/
		
	TVideoClip(const TVideoClip* clip, TArmeDoc* doc, TArmeWSObjPos& pos);
	TVideoClip(const char* name, TArmeDoc* doc);
	TVideoClip(ArmeArchiveHandler* unarchiver);
	virtual bool 	WriteToArchive(ArmeArchiveHandler* archiver) const;
	virtual void	Init();

	virtual			~TVideoClip();
	
	virtual void			SetStartTrim(ArmeTime startTrim);
	virtual void			SetEndTrim(ArmeTime endTrim);
	
    /*! @function GetDuration Get the total duration of all the tracks of the video object (in ArmeTime)*/
    virtual ArmeTime 		GetDuration() const;

	/*! @function ContainsVideoAtTime*/
	virtual bool			ContainsVideoAtTime(ArmeTime t) const;

	/*! @function ContainsVideoDuring*/
	virtual bool			ContainsVideoDuring(ArmeTime startTime, ArmeTime endTime, bool bConsiderAlsoMutedObjects=false, bool bConsiderAlsoTransparentTextClips = false) const;
	
	virtual TArmeWSObj* GetSliceCopy(ArmeTime startTime, ArmeTime endTime, const char* tempDirectoryName, MIProgressBarDlgManager* progressMngr) const;	
	virtual ArmeTime	InsertSlice(ArmeTime insertTime, const TArmeWSObj* obj, MIProgressBarDlgManager* progressMngr);
	virtual bool 		DeleteSlice(ArmeTime startTime, ArmeTime endTime, MIProgressBarDlgManager* progressMngr);
	virtual void		TimeStretch(ArmeTime startTime, ArmeTime endTime, ArmeTime newEndTime);

	virtual void 					SetMemento(TArmeWSObjMemento* memento);
	virtual TArmeWSObjMemento*	 	GetMemento() const;
	
	const TArmeChunkList<TVideoChunk>*		GetChunkList() {return fVisibleChunkList;};
protected :
	virtual void					_FillMemento(TArmeWSObjMemento* memento) const;

	TArmeChunkList<TVideoChunk>*		fVisibleChunkList;
	TArmeChunkList<TVideoChunk>*		fCompleteChunkList;

	Movie		ImportFile(const char* path, TArmeDoc* doc, ArmeErr* err);
	Movie		ImportFile(const FSSpec& movieFile, int trackIndex, TArmeDoc* doc, ArmeErr* err);

private :
	TVideoClip(TVideoClip&);
	TVideoClip(const TVideoClip&);

public :

	virtual void			PosWillChange();
	virtual void			PosHasChanged();

	/*! @function GetType RTTI information for all TArmeWSObj derivatives. See TArmeWSObj */
	virtual SInt32 		GetType() const {return kVideoClip;};

	virtual void		Move();

	virtual PicHandle 		GetPict(ArmeTime time) const;
	

	virtual ArmeTime	GetAbsoluteStartTimeOfVideoClip(bool bConsiderAlsoMutedObjects = false) const;
	virtual ArmeTime	GetAbsoluteEndxTimeOfVideoClip(bool bConsiderAlsoMutedObjects = false) const;

	/*! @function AddListOfStartAndEndTimeOfVideoClip add to the lists all the times at which a video child starts or ends*/
	virtual void 			AddListOfStartAndEndTimeOfVideoClip(ArmeTime startTime, ArmeTime endTime, ZzArray<ArmeTime> &list, bool bConsiderAlsoMutedObjects) const;

	virtual ArmeTime		GranulateTime(ArmeTime t) const;
public :

	/*! @function Dump Display debug information about the object*/
	virtual void			Dump(int indentLevel = 0) const;
};



class TVideoClipMemento : public TVideoObjectMemento
{
public :
	TArmeChunkList<TVideoChunk>*		fVisibleChunkList;
	TArmeChunkList<TVideoChunk>*		fCompleteChunkList;
	ArmeTime							fStartTrim;
	ArmeTime							fEndTrim;

	TVideoClipMemento();
	TVideoClipMemento(ArmeArchiveHandler* unarchiver);
	virtual bool			WriteToArchive(ArmeArchiveHandler* archiver) const;
	virtual ~TVideoClipMemento();
};
