/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
#ifndef TVideoEffect_h
#include "TVideoEffect.h"
#endif //TVideoEffect_h

#ifndef ZzHelpers_h
#include "ZzHelpers.h"
#endif //ZzHelpers_h

#ifndef MovieHelper_h
#include "MovieHelper.h"
#endif //MovieHelper_h

#ifndef TArmeMasterMovie_h
#include "TArmeMasterMovie.h"
#endif //TArmeMasterMovie_h

#ifndef ZzVerbose_h
//#include "ZzVerbose.h"
#endif //ZzVerbose_h

#ifndef TArmeMediaEngine_h
#include "TArmeMediaEngine.h"
#endif //TArmeMediaEngine_h

#ifndef MEApplicationServices_h
#include "MEApplicationServices.h"
#endif //MEApplicationServices_h

#undef VER_DEF_TOPIC
#define VER_DEF_TOPIC videoAlain

extern bool ExploreQTAtomContainer(QTAtomContainer* atomContainer);

TVideoEffect::TVideoEffect(TArmeDoc* doc, ArmeErr* err, QTAtomContainer* effectDesc) : TArmeContainer("Video Effect", doc)
{
    OSErr res;
    Init();
    
    if (effectDesc)
    {
	bEffectDescDefined = true;
    	res = QTCopyAtom(*effectDesc, kParentAtomIsContainer, &fEffectDesc);
	res = GetMoviesError();
	ThrowOnOSStatus(res);
    }
    
    UpdateName();
};

TVideoEffect::TVideoEffect(const char* name) : TArmeContainer(name)
{
    Init();
    
    UpdateName();
};

void TVideoEffect::UpdateName()
{
    char* procName = new char[1024];
    GetEffectName(fEffectDesc, procName);
    SetNameCopy(procName);
    delete [] procName;				
}

TVideoEffect::TVideoEffect(ArmeArchiveHandler* handler) :
	TArmeContainer(handler)
{
    Init();
    
    handler->ReadQTAtomContainer(&fEffectDesc);
    bEffectDescDefined = handler->ReadBool();
    
    UpdateName();
}

bool TVideoEffect::WriteToArchive(ArmeArchiveHandler* handler) const
{
    TVideoObject::WriteToArchive(handler);

    handler->WriteQTAtomContainer((char***)&fEffectDesc);
    handler->WriteBool(bEffectDescDefined);

    return true;
}


TVideoEffect::~TVideoEffect()
{
    QTDisposeAtomContainer(fEffectDesc);
};

TVideoEffect::TVideoEffect(TVideoEffect&) : TArmeContainer("", NULL) {ZzAssert(0);};
TVideoEffect::TVideoEffect(const TVideoEffect&) : TArmeContainer("", NULL) {ZzAssert(0);};

void TVideoEffect::SetMemento(TArmeWSObjMemento* memento)
{
    TVideoEffectMemento* videoMemento = dynamic_cast<TVideoEffectMemento*>(memento);
    ZzAssert(videoMemento);
    
    QTCopyAtom(videoMemento->fEffectDesc, kParentAtomIsContainer, &fEffectDesc);
    bEffectDescDefined = true;

    TArmeContainer::SetMemento(memento);
    
    UpdateName();
}

TArmeWSObjMemento* TVideoEffect::GetMemento() const
{
    TArmeWSObjMemento* memento = new TVideoEffectMemento();
    _FillMemento(memento);
    return memento;
}

void TVideoEffect::_FillMemento(TArmeWSObjMemento* memento) const
{
    TVideoEffectMemento* videoMemento = dynamic_cast<TVideoEffectMemento*>(memento);
    ZzAssert(videoMemento);


    QTNewAtomContainer(&(videoMemento->fEffectDesc));
    QTCopyAtom(fEffectDesc, kParentAtomIsContainer, &(videoMemento->fEffectDesc));

    TArmeContainer::_FillMemento(memento);
}

void TVideoEffect::Init()
{
    QTNewAtomContainer(&fEffectDesc);
    bEffectDescDefined = false;
}

void TVideoEffect::CreateFadeEffectDescription(int from, int to)
{
    OSType myType;
    OSErr res=0;
    QTAtomID newAtom;
    Fixed	percentage[2];
    
    // to fade in or out, we use the RGB balance filter.
        
    // create the effect ID atom: the atom type is kParameterWhatName, and the atom ID is kParameterWhatID
    myType = EndianU32_NtoB(kRGBColorBalanceImageFilterType);
    res = QTInsertChild(fEffectDesc, kParentAtomIsContainer, kParameterWhatName, kParameterWhatID, 0, sizeof(myType), &myType, NULL);
    ThrowOnOSStatus(res);
    
    /* add the R Balance ('rmul') atom for the effect */
    myType = FOUR_CHAR_CODE('rmul');
    res = QTInsertChild(fEffectDesc, kParentAtomIsContainer, myType, 1, 0, 0, 0, &newAtom);
    ThrowOnOSStatus(res);

    /* this atom specifies the format of the tween data */
    myType = EndianU32_NtoB(kParameterTypeDataFixed);
    res = QTInsertChild(fEffectDesc, newAtom, kTweenType, 1 , 0, sizeof(OSType), &myType, NULL);
    ThrowOnOSStatus(res);

    percentage[0] =  EndianS32_NtoB( from );	/* multiplier of red at the end of the effect */
    percentage[1] =  EndianS32_NtoB( to );	/* multiplier of red at the start of the effect */

    /* here's the data for the tween  */
    res = QTInsertChild(fEffectDesc, newAtom, kTweenData, 1 , 0, sizeof(long) * 2, percentage, NULL);
    ThrowOnOSStatus(res);

    /* add the G Balance ('gmul') atom for the effect */
    myType = FOUR_CHAR_CODE('gmul');
    res = QTInsertChild(fEffectDesc, kParentAtomIsContainer, myType, 1, 0, 0, 0, &newAtom);
    ThrowOnOSStatus(res);

    /* this atom specifies the format of the tween data */
    myType = EndianU32_NtoB(kParameterTypeDataFixed);
    res = QTInsertChild(fEffectDesc, newAtom, kTweenType, 1 , 0, sizeof(OSType), &myType, NULL);
    ThrowOnOSStatus(res);

    percentage[0] =  EndianS32_NtoB( from );	/* multiplier of green at the end of the effect */
    percentage[1] =  EndianS32_NtoB( to );	/* multiplier of green at the start of the effect */

    /* here's the data for the tween  */
    res = QTInsertChild(fEffectDesc, newAtom, kTweenData, 1 , 0, sizeof(long) * 2, percentage, NULL);
    ThrowOnOSStatus(res);

    /* add the B Balance ('bmul') atom for the effect */
    myType = FOUR_CHAR_CODE('bmul');
    res = QTInsertChild(fEffectDesc, kParentAtomIsContainer, myType, 1, 0, 0, 0, &newAtom);
    ThrowOnOSStatus(res);

    /* this atom specifies the format of the tween data */
    myType = EndianU32_NtoB(kParameterTypeDataFixed);
    res = QTInsertChild(fEffectDesc, newAtom, kTweenType, 1 , 0, sizeof(OSType), &myType, NULL);
    ThrowOnOSStatus(res);

    percentage[0] =  EndianS32_NtoB( from );	/* multiplier of blue at the end of the effect */
    percentage[1] =  EndianS32_NtoB( to );	/* multiplier of blue at the start of the effect */

    /* here's the data for the tween  */
    res = QTInsertChild(fEffectDesc, newAtom, kTweenData, 1 , 0, sizeof(long) * 2, percentage, NULL);
    ThrowOnOSStatus(res);

    bEffectDescDefined = true;
}


TVideoEffectMemento::TVideoEffectMemento() : TVideoObjectMemento()
{
    QTNewAtomContainer(&fEffectDesc); 
}

TVideoEffectMemento::TVideoEffectMemento(ArmeArchiveHandler* unarchiver) : TVideoObjectMemento(unarchiver)
{
    QTNewAtomContainer(&fEffectDesc); 
    unarchiver->ReadQTAtomContainer(&fEffectDesc);
}

bool TVideoEffectMemento::WriteToArchive(ArmeArchiveHandler* archiver) const
{
    TVideoObjectMemento::WriteToArchive(archiver);
    archiver->WriteQTAtomContainer((char***)(&fEffectDesc));

    return true;
}

TVideoEffectMemento::~TVideoEffectMemento()
{
    QTDisposeAtomContainer(fEffectDesc);
}


TArmeWSObj* TVideoEffect::GetSliceCopy(ArmeTime startTime, ArmeTime endTime, const char* tempDirectoryName, MIProgressBarDlgManager* progressMngr) const
{
    ArmeTime copyStartTime;
    
    if ((startTime == ARME_TIME_UNDEF) || (startTime == ARME_TIME_MIN))
	copyStartTime = GetPos().fStartTime;
    else if (startTime < GetAbsoluteStartTime())
	copyStartTime = GetAbsoluteStartTime() - startTime;
    else
	copyStartTime = ARME_TIME_ZERO;
    
    if ((startTime == ARME_TIME_UNDEF) || (startTime < GetAbsoluteStartTime()))
	startTime = GetAbsoluteStartTime();
    
    if ((endTime == ARME_TIME_UNDEF) || (endTime > GetAbsoluteEndxTime()))
	endTime = GetAbsoluteEndxTime();

    if (startTime == endTime)
	return NULL;

    TVideoEffect* copy = new TVideoEffect(GetName());
    copy->SetMuted(GetMuted());
    copy->fPos.fStartTime = copyStartTime; 
    copy->fPos.fParent = NULL;
    copy->fLength = endTime - startTime;
    copy->fCachedAbsoluteStartTime = startTime;
    
    //copy the effectDescription
    copy->CopyEffectDescFrom(this);
 
    // since we don't have a common "base class" GetSliceCopy, i need to duplicate the following
    // code on audio clips and potential containers
    copy->SetVAORouting(GetVAORouting());
    return copy;
}


OSErr TVideoEffect::CopyEffectDescFrom(const TVideoEffect* obj)
{
    bEffectDescDefined = true;
    OSErr err = QTCopyAtom(obj->fEffectDesc, kParentAtomIsContainer, &fEffectDesc);
    ThrowOnOSStatus(err);
    UpdateName();
    return err;
}



QTAtomContainer TVideoEffect::GetEffectDescFor(ArmeTime startTime, ArmeTime endTime)
{
    QTAtomContainer effectDescCopy;
    QTNewAtomContainer(&effectDescCopy);
    
    // if the effectDesc has not yet been defined, ask the user to define it
    ZzAssert(bEffectDescDefined);
    if (!bEffectDescDefined)
	Configure();
    
    //make a copy of the effect description
//    QTCopyAtom(fEffectDesc, kParentAtomIsContainer, &effectDescCopy);
    float from = float(startTime.data - GetAbsoluteStartTimeOfVideoClip().data)/float(GetDuration().data);
    float to = float(endTime.data - GetAbsoluteStartTimeOfVideoClip().data)/float(GetDuration().data);

    GetCopyOfEffectDesc(effectDescCopy, fEffectDesc, from, to);

    OSType myType = EndianU32_NtoB(kSourceOneName);
    OSErr res = QTInsertChild(effectDescCopy, kParentAtomIsContainer, kEffectSourceName, 1, 0, sizeof(myType), &myType, NULL);
    ThrowOnOSStatus(res);

    if (GetType() == kVideoXFadeEffect)
    {
	OSType myType = EndianU32_NtoB(kSourceTwoName);
	OSErr res = QTInsertChild(effectDescCopy, kParentAtomIsContainer, kEffectSourceName, 2, 0, sizeof(myType), &myType, NULL);
	ThrowOnOSStatus(res);
    }

    //$$TODO,
    //if there are some tween tracks in the effect, we need to redefine the 'from' and 'to' parameters of these tween tracks
//    ExploreQTAtomContainer(&fEffectDesc);


    return effectDescCopy;
}

void TVideoEffect::Dump(int indentLevel) const
{
#ifndef NO_VERBOSE_AT_ALL
    static char* whiteSpace = "           ";

    // Display all the tracks
    verntl(armeWSObjsBenjamin,5,&(whiteSpace[10-indentLevel]) << "Effect : " << this);
    TVideoObject::Dump(indentLevel);
    verntl(armeWSObjsBenjamin,5,&(whiteSpace[10-indentLevel]) << "Clips in the effect : ");
    TVideoObject* clip;
    for (int i=0; i < GetChildrenCount(); ++i)
    {
	if (GetChild(i)->IsVideoObject())
	{
	    clip = (TVideoObject*)GetChild(i);
	    clip->Dump(indentLevel+1);
	}
    }
#endif // NO_VERBOSE_AT_ALL
}

void TVideoEffect::Configure()
{
    ZzAssert(0);
    if (GetVideoChildrenCount() == 0)
	return;

    PicHandle posterSrc1 = NULL;

    for (int i=0; i < GetChildrenCount(); i++)
    {
	if (GetChild(i)->IsVideoObject() && (GetChild(i)->GetType() != kVideoXFadeEffect))  //actually, this also includes xFade effect since the trackRep of the clips will be the track of the xFade
	{
	    TVideoObject* video = (TVideoObject*)GetChild(i);
	    if (!posterSrc1)
		posterSrc1 = video->fMasterMovie->GetPictAtTime(video, this, video->GetAbsoluteStartTimeOfVideoClip());
	}
    }

    if (ConfigureEffect(fEffectDesc, 1, posterSrc1, NULL))
    {
	bEffectDescDefined = true;
   }
    return;
}


void TVideoEffect::Move()
{
   TArmeContainer::Move();
}
				
/*! @function _NotifyChildPosHasChanged Each time a child is moved, all its parents will receive this message */
void TVideoEffect::_NotifyChildPosHasChanged()
{
    //invalidate all the cached timing information
    TVideoObject::_NotifyChildPosHasChanged();
}


TVideoXFadeEffect::TVideoXFadeEffect(TArmeDoc* doc, int transitionType, float pcnt0, float pcnt1) : 
    TVideoEffect(doc, NULL, NULL)
{
    fClip1 = NULL;
    fClip2 = NULL;
    fFrom = pcnt0;
    fTo = pcnt1;

    CreateEffectDesc(transitionType, pcnt0, pcnt1);
    bEffectDescDefined = true;
}

TVideoXFadeEffect::TVideoXFadeEffect(TArmeDoc* doc, int transitionType, QTAtomContainer& effectDesc, float from, float to, bool fixedParameters) : TVideoEffect(doc, NULL, NULL)
{
    fFrom = from;
    fTo = to;
    fClip1 = NULL;
    fClip2 = NULL;
    fTransitionType = transitionType;
    fFixedParameters = fixedParameters;

    //copy effectDesc,
    // if from and to are -1, use the from and to that are already in effectDesc
    if (fixedParameters)
    {
	GetFromAndToParamOfTransition(effectDesc, fFrom, fTo);
	GetCopyOfTransitionEffect(fEffectDesc, effectDesc, fFrom, fTo);
    }
    else 
//    if (!fixedParameters || ((from >= 0.0f) && (to >= 0.0f)))
    {
	GetCopyOfTransitionEffect(fEffectDesc, effectDesc, from, to);
    }
/*    else
    {
	GetFromAndToParamOfTransition(effectDesc, fFrom, fTo);
	GetCopyOfTransitionEffect(fEffectDesc, effectDesc, fFrom, fTo);
	//res = QTCopyAtom(effectDesc, kParentAtomIsContainer, &fEffectDesc);
    }*/
    bEffectDescDefined = true;
}

TVideoXFadeEffect::TVideoXFadeEffect(ArmeArchiveHandler* unarchiver) : TVideoEffect(unarchiver)
{
    fClip1 = dynamic_cast<TVideoObject*>(unarchiver->ReadCPPObject());
    fClip2 = dynamic_cast<TVideoObject*>(unarchiver->ReadCPPObject());
    
    if (unarchiver->GetFileVer() >= 0x00010023)
    {
	fTransitionType = unarchiver->ReadSInt32();
    }
    else
    {
	fTransitionType = CLIP1_CLIP2_TOGETHER;
    }

    if (unarchiver->GetFileVer() >= 0x00010032)
	fFixedParameters = unarchiver->ReadBool();
    else
	fFixedParameters = false;
    

    GetFromAndToParamOfTransition(fEffectDesc, fFrom, fTo);    
}

bool TVideoXFadeEffect::WriteToArchive(ArmeArchiveHandler* handler) const
{
    TVideoEffect::WriteToArchive(handler);

    handler->WriteCPPObject(fClip1);
    handler->WriteCPPObject(fClip2);
    //from version 0x00010023
    handler->WriteSInt32(fTransitionType);
    //from version 0x00010032
    handler->WriteBool(fFixedParameters);
    return true;
}


TVideoXFadeEffect::~TVideoXFadeEffect() 
{
    //remove all references in the doc
/*    for (int i = 0; i < fDoc->fOuttaHierarchyTransitionsPool->CountItems(); ++i)
    {
	if (fDoc->fOuttaHierarchyTransitionsPool->ItemAt(i) == this)
	{
	    fDoc->fOuttaHierarchyTransitionsPool->RemoveItemByIndex(i);
	    break;
	}
    }*/

}

void TVideoXFadeEffect::AddRef()
{
    if (fRefCount == 0)
    {
	GetDoc()->fNewlyCreatedTransitions->AddItem(this);
    }

    fRefCount++;
    verntl(videoAlain, 6, "Adding ref to (" << this << ", " << fName << " (" << fClip1->GetName() << ", " << fClip2->GetName() << ")) new ref count = " << fRefCount << ")");
}
void TVideoXFadeEffect::Release()
{
    fRefCount--;
    verntl(videoAlain, 6, "Releasing ref on (" << this << ", " << fName << " (" << fClip1->GetName() << ", " << fClip2->GetName() << ")) new ref count = " << fRefCount << ")");
    if (fRefCount == 0)
    {
	GetDoc()->fTransitionsToRemove->AddItem(this);
	fPos.fParent->_RemoveChild(this);

	//update the description of desc
	GetDoc()->AddToOldTransitionsDesc(this); 
	//GetDoc()->fOuttaHierarchyTransitionsPool->AddItem(this);
    }
}

void TVideoXFadeEffect::CreateEffectDesc(int type, float pcnt0, float pcnt1)
{
    QTAtomContainer defaultEffectDesc = TArmeMediaEngine::ApplicationServices()->GetDefaultTransition();

    if (TArmeMediaEngine::ApplicationServices()->KeepDefaultTransitionParameters())
    {
	QTCopyAtom(defaultEffectDesc, kParentAtomIsContainer, &fEffectDesc);
	fFixedParameters = true;
    }
    else
    {
	GetCopyOfEffectDesc(fEffectDesc, defaultEffectDesc, pcnt0, pcnt1);
	fFixedParameters = false;
    }

    return;
}

void TVideoXFadeEffect::SetOuttaHierarchy()
{
    // you can put a XFade outtaHierrachy but then it has to stay there forever, (the putBackHierarchy has no effect (see MIObject)
    if (fPos.fParent != NULL)
	TVideoObject::SetOuttaHierarchy();
    //GetPos().fParent->_RemoveChild

}

bool TVideoXFadeEffect::IsVideoObject() const
{
    return true;
}


bool TVideoXFadeEffect::IsStillUseful() const
{
    return IsInMasterMovie();

    ArmeTime clip1Start = fClip1->GetAbsoluteStartTimeOfVideoClip(false);
    ArmeTime clip2Start = fClip2->GetAbsoluteStartTimeOfVideoClip(false);
    ArmeTime clip1End = fClip1->GetAbsoluteEndxTimeOfVideoClip(false);
    ArmeTime clip2End = fClip2->GetAbsoluteEndxTimeOfVideoClip(false);

    // if they don't have the same parent, it is not useful anymore
    if (fClip1->GetPos().fParent != fClip2->GetPos().fParent)
	return false;
    
    // if a clip is not visible, it is not useful anymore
    if (!fClip1->IsVisible() || !fClip2->IsVisible())
	return false;

    // if one of the clips does not contain video anymore, it is not useful anymore
    if (!fClip1->ContainsVideo() || !fClip2->ContainsVideo())
	return false;

    // if they still have some common part, it is useful
    if ( ((clip1Start <= clip2Start) && (clip1End >= clip2End))
	|| ((clip2Start <= clip1Start) && (clip2End >= clip1End)) )
	return true;
    if ( (clip1Start < clip2End) && (clip1End > clip2End))
	return true;
    if ( (clip2Start < clip1End) && (clip2End > clip1End))
	return true;
    
    return false;
}

ArmeTime TVideoXFadeEffect::GetDuration() const
{
    return TVideoObject::GetDuration();
}

ArmeTime TVideoXFadeEffect::GetAbsoluteEndxTimeOfVideoClip(bool bConsiderAlsoMutedObjects) const
{
    if (bConsiderAlsoMutedObjects || IsVisible())
	return GetAbsoluteEndxTime();
    else
	return ARME_TIME_UNDEF;
}

ArmeTime TVideoXFadeEffect::GetAbsoluteStartTimeOfVideoClip(bool bConsiderAlsoMutedObjects) const
{
    if (bConsiderAlsoMutedObjects || IsVisible())
	return GetAbsoluteStartTime();
    else
	return ARME_TIME_UNDEF;
}

TArmeWSObj* TVideoXFadeEffect::GetSliceCopy(ArmeTime startTime, ArmeTime endTime, const char* tempDirectoryName, MIProgressBarDlgManager* progressMngr) const
{
    return NULL;
}

void TVideoXFadeEffect::Dump(int indentLevel) const
{
#ifndef NO_VERBOSE_AT_ALL
    static char* whiteSpace = "           ";

    // Display all the tracks
    verntl(armeWSObjsBenjamin, 5, &(whiteSpace[10-indentLevel]) << "XFade Effect (" << this <<")");
    TVideoObject::Dump(indentLevel);
    verntl(armeWSObjsBenjamin, 5, &(whiteSpace[10-indentLevel]) << "Clips in the effect :");
    TVideoObject* clip;
    for (int i=0; i < GetChildrenCount(); ++i)
    {
	if (GetChild(i)->IsVideoObject())
	{
	    clip = (TVideoObject*)GetChild(i);
	    clip->Dump(indentLevel+1);
	}
    }
#endif //NO_VERBOSE_AT_ALL
}

TVideoXFadeEffectDesc::TVideoXFadeEffectDesc(TVideoObject* clip1, TVideoObject* clip2, QTAtomContainer& effectDesc, int transitionType, bool fixedParameters)
{
    fClip1 = clip1;
    fClip2 = clip2;
    fFixedParameters = fixedParameters;

    QTNewAtomContainer(&fEffectDesc);
    QTCopyAtom(effectDesc, kParentAtomIsContainer, &fEffectDesc);
    fTransitionType = transitionType;
}

TVideoXFadeEffectDesc::~TVideoXFadeEffectDesc()
{
    QTDisposeAtomContainer(fEffectDesc);
}

TVideoXFadeEffectDesc::TVideoXFadeEffectDesc(ArmeArchiveHandler* unarchiver) : ArmeArchivable(unarchiver)
{
    fClip1 = dynamic_cast<TVideoObject*>(unarchiver->ReadCPPObject());
    fClip2 = dynamic_cast<TVideoObject*>(unarchiver->ReadCPPObject());
    QTNewAtomContainer(&fEffectDesc);
    unarchiver->ReadQTAtomContainer(&fEffectDesc);
    fTransitionType = unarchiver->ReadSInt32();

    if (unarchiver->GetFileVer() >= 0x00010032)
	fFixedParameters = unarchiver->ReadBool();
    else
	fFixedParameters = false;
}

bool TVideoXFadeEffectDesc::WriteToArchive(ArmeArchiveHandler* archiver) const
{
    archiver->WriteCPPObject(fClip1);
    archiver->WriteCPPObject(fClip2);

    archiver->WriteQTAtomContainer((char***)&fEffectDesc);
    archiver->WriteSInt32(fTransitionType);
    //from version 0x00010032
    archiver->WriteBool(fFixedParameters);
 
    return true;
}
