/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#define TVideoEffect_h

#ifndef TVideoEffect_h
#include "TVideoEffect.h"
#endif //TVideoEffect_h
 
#ifndef TArmeContainer_h
#include "TArmeContainer.h"
#endif //TArmeContainer_h

#include <Quicktime/Quicktime.h>

class TArmeMasterMovie; 

#define MAX_NB_CLIPS 3

class TVideoEffect : public TArmeContainer
{
public :
	virtual void 					SetMemento(TArmeWSObjMemento* memento);
	virtual TArmeWSObjMemento*	 	GetMemento() const;
protected :
	virtual void					_FillMemento(TArmeWSObjMemento* memento) const;
	void							Init();
public :
	TVideoEffect(TArmeDoc* doc, ArmeErr* err, QTAtomContainer* effectDesc);
	virtual			~TVideoEffect();

	// arch stuff
	TVideoEffect(ArmeArchiveHandler* handler);
	virtual bool	WriteToArchive(ArmeArchiveHandler* handler) const;

	virtual TArmeWSObj* GetSliceCopy(ArmeTime startTime, ArmeTime endTime, const char* tempDirectoryName, MIProgressBarDlgManager* progressMngr) const;
	OSErr CopyEffectDescFrom(const TVideoEffect* obj);
	void	CreateFadeEffectDescription(int from, int to); // parameters range : [0, 100]

	QTAtomContainer		GetEffectDescFor(ArmeTime startTime, ArmeTime endTime);

/*	virtual void		_NotifyParentOuttaHierarchy();
	virtual void		_NotifyParentBackInHierarchy();*/
	void					UpdateName();
	
	virtual void			Move();
				
	/*! @function _NotifyChildPosHasChanged Each time a child is moved, all its parents will receive this message */
	virtual void			_NotifyChildPosHasChanged();

protected :
	TVideoEffect(const char* name);
	TVideoEffect(TVideoEffect&);
	TVideoEffect(const TVideoEffect&);

	bool					bEffectDescDefined;
public :
	QTAtomContainer			fEffectDesc;

	/*! @function Dump Display debug information about the object*/
	virtual void			Dump(int indentLevel = 0) const;
	
	void			Configure();

	/*! @function GetType RTTI information for all TArmeWSObj derivatives. See TArmeWSObj */
	virtual SInt32 	GetType() const {return kVideoEffect;}; 
};

class TVideoXFadeEffect : public TVideoEffect
{
public :
	TVideoObject*						fClip1;
	TVideoObject*						fClip2;
	bool								fFixedParameters;
	float					fFrom;
	float					fTo;

	virtual void			AddRef();
	virtual void			Release();

	enum {
		INCREASE,
		CONSTANT,
		};

	enum {
		CLIP1_BEFORE_CLIP2,
		CLIP1_AFTER_CLIP2,
		CLIP1_CLIP2_TOGETHER,
		};

	int						fTransitionType;


	TVideoXFadeEffect(TArmeDoc* doc, int transitionType, float pcnt0, float pcnt1 = 1.0f);
	TVideoXFadeEffect(TArmeDoc* doc, int transitionType, QTAtomContainer& effectDesc, float _from = -1.0f, float _to = -1.0f, bool fixedParameters = false);
	virtual			~TVideoXFadeEffect();

	TVideoXFadeEffect(ArmeArchiveHandler* handler);
	virtual bool 		WriteToArchive(ArmeArchiveHandler* handler) const;
	virtual TArmeWSObj* GetSliceCopy(ArmeTime startTime, ArmeTime endTime, const char* tempDirectoryName, MIProgressBarDlgManager* progressMngr) const;

	bool ContainsVideoDuring(ArmeTime startTime, ArmeTime endTime, bool bConsiderAlsoMutedObjects=false, bool bConsiderAlsoTransparentTextClips = false) const {return false;};  //it does not contain any video media

	bool			IsStillUseful() const;
	virtual bool 	IsVideoObject() const;

	/*! @function GetType RTTI information for all TArmeWSObj derivatives. See TArmeWSObj */
	virtual SInt32 	GetType() const {return kVideoXFadeEffect;}; 

    /*! @function GetDuration Get the total duration of all the tracks of the video object (in ArmeTime)*/
    virtual ArmeTime 		GetDuration() const;
	virtual ArmeTime		GetAbsoluteEndxTimeOfVideoClip(bool bConsiderAlsoMutedObjects = false) const;
	virtual ArmeTime		GetAbsoluteStartTimeOfVideoClip(bool bConsiderAlsoMutedObjects = false) const;
	
	void 			CreateEffectDesc(int type, float pcnt0, float pcnt1);

	virtual void		SetOuttaHierarchy();

	/*! @function Dump Display debug information about the object*/
	virtual void			Dump(int indentLevel = 0) const;
};

class TVideoEffectMemento : public TVideoObjectMemento
{
public :
	QTAtomContainer			fEffectDesc;

	TVideoEffectMemento();
	TVideoEffectMemento(ArmeArchiveHandler* unarchiver);
	virtual bool			WriteToArchive(ArmeArchiveHandler* archiver) const;

	virtual ~TVideoEffectMemento();
};

class TVideoXFadeEffectDesc : public ArmeArchivable
{
public :
	TVideoObject* 		fClip1;
	TVideoObject* 		fClip2;
	QTAtomContainer		fEffectDesc;
	int					fTransitionType;
	bool				fFixedParameters;


	TVideoXFadeEffectDesc(TVideoObject* clip1, TVideoObject* clip2, QTAtomContainer& effectDesc, int transitionType, bool fFixedParameters);
	virtual ~TVideoXFadeEffectDesc();

	TVideoXFadeEffectDesc(ArmeArchiveHandler* unarchiver);
	virtual bool			WriteToArchive(ArmeArchiveHandler* archiver) const;

};
