/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#ifndef TVideoObject_h
#include "TVideoObject.h"
#endif //TVideoObject_h

#ifndef TVideoEffect_h
#include "TVideoEffect.h"
#endif //TVideoEffect_h

#ifndef MovieHelper_h
#include "MovieHelper.h"
#endif //MovieHelper_h

#ifndef ZzVerbose_h
//#include "ZzVerbose.h"
#endif //ZzVerbose_h

#ifndef ArmeTimeRange_h
#include "ArmeTimeRange.h"
#endif //ArmeTimeRange_h


#undef VER_DEF_TOPIC
#define VER_DEF_TOPIC videoAlain

/********************************************************************************************
TVideoObject TVideoObject TVideoObject TVideoObject TVideoObject TVideoObject TVideoObject
********************************************************************************************/

 
TVideoObject::TVideoObject(const char* path, TArmeDoc* doc) :
	TArmeWSObj(path, doc)
{
	Init();

    fMasterMovie = doc->fMasterMovie;

	InvalidateCachedTimeInfo();
}

TVideoObject::~TVideoObject()
{
	if (fDoc)
	{
		//remove all references in the doc
		fDoc->RemoveFromOldTransitionsDesc(this);
/*		for (int i = 0; i < fDoc->fOuttaHierarchyTransitionsPool->CountItems(); ++i)
		{
			TVideoXFadeEffect* xFade = fDoc->fOuttaHierarchyTransitionsPool->ItemAt(i);
			if((xFade->fClip1 == this) || (xFade->fClip2 == this))
			{
				fDoc->fOuttaHierarchyTransitionsPool->RemoveItemByIndex(i);
				delete xFade;
				i--;
			}
		}*/
	}
}

TVideoObject::TVideoObject(const char* name) : TArmeWSObj(name)
{
	Init();
}

void TVideoObject::Init()
{
	fMasterMovie = NULL;
	fHasBeenEdited = false;
	InvalidateCachedTimeInfo();
	fIsMutedForSolo = false;
	fRefCount = 0;
		
	fBeginOffset = ARME_TIME_ZERO;
    fEndOffset = ARME_TIME_ZERO;
    fEffectiveDuration = ARME_TIME_ZERO;
	
	fStartTrim = ARME_TIME_ZERO;
	fEndTrim = ARME_TIME_ZERO;
}

void TVideoObject::SetMemento(TArmeWSObjMemento* memento)
{
	TVideoObjectMemento* videoMemento = dynamic_cast<TVideoObjectMemento*>(memento);
	ZzAssert(videoMemento);

	fMasterMovie = videoMemento->fMasterMovie;
	fStartTrim = videoMemento->fStartTrim;
	fEndTrim = videoMemento->fEndTrim;
	
	TArmeWSObj::SetMemento(memento);

	fHasBeenEdited = true;
	SetMutedForSolo(videoMemento->fIsMutedForSolo);
}

TVideoObject::TVideoObject(ArmeArchiveHandler* handler) :
	TArmeWSObj(handler)
{
	ArmeArchivable* archObj;

	Init();
	vverntl(archivingEngine,6,"TVideoObject::TVideoObject(ArmeArchiveHandler* handler)");

	archObj = handler->ReadCPPObject();
	fMasterMovie = dynamic_cast<TArmeMasterMovie*>(archObj);
	if(archObj)
	{
		ZzAssert(fMasterMovie);
	}
	
	fIsMutedForSolo = handler->ReadBool();
	
	fRefCount = handler->ReadSInt32();
	if (handler->GetFileVer() >= 0x00010024)
	{
		fStartTrim = handler->ReadArmeTime();
		fEndTrim = handler->ReadArmeTime();
	}
	else
	{
		fStartTrim = ARME_TIME_ZERO;
		fEndTrim = ARME_TIME_ZERO;
	}
}

bool TVideoObject::WriteToArchive(ArmeArchiveHandler* handler) const
{
	vverntl(archivingEngine,6,"TVideoObject::WriteToArchive(ArmeArchiveHandler* handler) const");
	TArmeWSObj::WriteToArchive(handler);
	handler->WriteCPPObject(fMasterMovie);
	
	handler->WriteBool(fIsMutedForSolo);
	
	handler->WriteSInt32(fRefCount);

	//from 0x00010024
	handler->WriteArmeTime(fStartTrim);
	handler->WriteArmeTime(fEndTrim);

	return true;
}


/*void TVideoObject::_NotifyParentOuttaHierarchy()
{
	TArmeWSObj::_NotifyParentOuttaHierarchy();

	fMasterMovie->RemoveSubMovie(this);
}

void TVideoObject::_NotifyParentBackInHierarchy()
{
	fMasterMovie->AddSubMovie(this, (TVideoObject*)(GetPos().fParent), GetAbsoluteStartTimeOfVideoClip());

	TArmeWSObj::_NotifyParentBackInHierarchy();
}*/

void TVideoObject::SetOuttaHierarchy()
{
	if (ContainsVideo(false, true))
		fMasterMovie->RemoveSubMovie(this);

	TArmeWSObj::SetOuttaHierarchy();
}
void TVideoObject::SetBackInHierarchy(TArmeWSObj* parent)
{
	TArmeWSObj::SetBackInHierarchy(parent);
	
	if (ContainsVideo(false, true))
		fMasterMovie->AddSubMovie(this);
}


TArmeWSObjMemento* TVideoObject::GetMemento() const
{
	TArmeWSObjMemento* memento = new TVideoObjectMemento();
	_FillMemento(memento);
	return memento;
}

void TVideoObject::_FillMemento(TArmeWSObjMemento* memento) const
{
	TVideoObjectMemento* videoMemento = dynamic_cast<TVideoObjectMemento*>(memento);
	ZzAssert(videoMemento);

	TArmeWSObj::_FillMemento(memento);

	videoMemento->fMasterMovie = fMasterMovie;
	videoMemento->fIsMutedForSolo = fIsMutedForSolo;
	videoMemento->fStartTrim = fStartTrim;
	videoMemento->fEndTrim = fEndTrim;
}


void TVideoObject::InvalidateCachedTimeInfo() 
{
	fCachedAbsoluteStartTimeOfVideoClip = ARME_TIME_UNDEF; 
	fCachedAbsoluteEndTimeOfVideoClip = ARME_TIME_UNDEF;
	TArmeWSObj::InvalidateCachedTimeInfo();
};

void TVideoObject::Dump(int indentLevel) const
{
#ifndef NO_VERBOSE_AT_ALL

#endif //NO_VERBOSE_AT_ALL
}

ArmeTime TVideoObject::GetAbsoluteStartTimeOfVideoClip(bool bConsiderAlsoMutedObjects) const
{
    if (GetChildrenCount() <= 0)
        return _ArmeTime(0);

    ArmeTime startTime = ARME_TIME_MAX;

    for (int i=0; i < GetChildrenCount(); ++i)
    {
		TVideoObject* obj = dynamic_cast<TVideoObject*>(GetChild(i));
		if (obj && (obj->GetType() != kVideoXFadeEffect) && (bConsiderAlsoMutedObjects || obj->IsVisible()))
		{
			if (obj->GetAbsoluteStartTimeOfVideoClip(bConsiderAlsoMutedObjects) < startTime)
				startTime = obj->GetAbsoluteStartTimeOfVideoClip(bConsiderAlsoMutedObjects);
		}
    }

    return startTime;
}

ArmeTime TVideoObject::GetAbsoluteEndxTimeOfVideoClip(bool bConsiderAlsoMutedObjects) const
{
    if (GetChildrenCount() <= 0)
        return _ArmeTime(0);

    ArmeTime endTime = ARME_TIME_MIN;

    for (int i=0; i < GetChildrenCount(); ++i)
    {
		TVideoObject* obj = dynamic_cast<TVideoObject*>(GetChild(i));
		if (obj && (obj->GetType() != kVideoXFadeEffect) && (bConsiderAlsoMutedObjects || obj->IsVisible()))
		{
			if (obj->GetAbsoluteEndxTimeOfVideoClip(bConsiderAlsoMutedObjects) > endTime)
				endTime = obj->GetAbsoluteEndxTimeOfVideoClip(bConsiderAlsoMutedObjects);
		}
    }

    return endTime;
}

ArmeTime TVideoObject::GranulateTime(ArmeTime t) const
{
	if (t >= GetAbsoluteEndxTime())
		return GetAbsoluteEndxTime();
	else if (t <= GetAbsoluteStartTime())
		return GetAbsoluteStartTime();

    for (int i=0; i < GetChildrenCount(); ++i)
    {
		TVideoObject* obj = dynamic_cast<TVideoObject*>(GetChild(i));
		if (obj && (obj->GetType() != kVideoXFadeEffect))
		{
			if (obj->ContainsVideoAtTime(t))
			{
				return obj->GranulateTime(t);
			}
		}
    }

	return t;
}


void TVideoObject::AddListOfStartAndEndTimeOfVideoClip(ArmeTime startTime, ArmeTime endTime, ZzArray<ArmeTime> &list, bool bConsiderAlsoMutedObjects) const
{
    for (int i=0; i < GetChildrenCount(); ++i)
    {
		TVideoObject* obj = dynamic_cast<TVideoObject*>(GetChild(i));
		if (obj && (obj->GetType() != kVideoXFadeEffect))
		{
			if (obj->ContainsVideoDuring(startTime, endTime, bConsiderAlsoMutedObjects, true) && (bConsiderAlsoMutedObjects || obj->IsVisible()))
				obj->AddListOfStartAndEndTimeOfVideoClip(startTime, endTime, list, bConsiderAlsoMutedObjects);
		}
    }
}


void TVideoObject::SetPos(const TArmeWSObjPos& pos)
{
//	ArmeTime oldStartTime = GetAbsoluteStartTimeOfVideoClip();
//	ArmeTime oldEndTime = GetAbsoluteEndxTimeOfVideoClip();

	//Modify the hierarchy, move the tracks in the master movie and delete useless VideoEffectTrack (not the XFade)
	TArmeWSObjPos copy = pos;
	fDoc->RoundArmeTimeToNearestVideoFrame(copy.fStartTime);
	_SetPos(copy);

//	UpdateAffectedHierarchy();
}

void TVideoObject::SetMuted(bool muted)
{
	verntl(videoAlain, 4, "SetMuted (" << this << ", " << fName << ") new muted status = " << muted << ")");
	fMasterMovie->RemoveSubMovie(this);
	TArmeWSObj::SetMuted(muted);
	if (!muted)
		fMasterMovie->AddSubMovie(this);
	

/*	if (muted)
	{
		fMasterMovie->RemoveSubMovie(this);
		
		TArmeWSObj::SetMuted(muted);
	}
	else
	{
		TArmeWSObj::SetMuted(muted);
		if (!IsInMasterMovie())
			fMasterMovie->AddSubMovie(this, GetPos().fParent, GetAbsoluteStartTimeOfVideoClip());
	}*/
//	TArmeWSObj::SetMuted(muted);
}

void TVideoObject::_SetMuted(bool muted)
{
	if (muted)
		fMasterMovie->RemoveSubMovie(this);
	else if (!IsInMasterMovie())
		fMasterMovie->AddSubMovie(this);
}

void TVideoObject::SetMutedForSolo(bool muted)
{
/*	if (muted)
		fMasterMovie->RemoveSubMovie(this);
	else if (!IsInMasterMovie())
		fMasterMovie->AddSubMovie(this, GetPos().fParent, GetAbsoluteStartTimeOfVideoClip());


	fIsMutedForSolo = muted;

	// was _SetMuted(!IsVisible());
	_SetMuted(IsMuted());
	
    for (int i=0; i < GetChildrenCount(); ++i)
    {
		TVideoObject* obj = dynamic_cast<TVideoObject*>(GetChild(i));
		if (obj)
		{
			obj->SetMutedForSolo(muted);
		}
    }*/
}

bool TVideoObject::IsVisible() const
{
	//if it is currently not in the master movie, it is not visible (it is the case between a (RemoveSubMovie and AddSubMovie))
/*	if (!IsInMasterMovie())
		return false;*/

	//if it is outtaHierarchy, it is not visible
	if (GetPos().fParent == NULL)
		return false;
	return ((IsInSolo()) || (!IsMuted() && !fIsMutedForSolo));
}


// functions for UI-based stretching and cropping
void TVideoObject::SetEffectiveDuration (ArmeTime inEffectiveDuration)\
{
    fEffectiveDuration = inEffectiveDuration;
}

void TVideoObject::SetBeginOffset (ArmeTime inBeginOffset)
{
    fBeginOffset = inBeginOffset;
}

void TVideoObject::SetEndOffset (ArmeTime inEndOffset)
{
    fEndOffset = inEndOffset;
}


ArmeTime TVideoObject::GetStartTrim() const
{
	return fStartTrim;
}
ArmeTime TVideoObject::GetEndTrim() const
{
	return fEndTrim;
}

void TVideoObject::SetStartTrim(ArmeTime startTrim)
{
	do 
	{
		ArmeTime startTime = GetAbsoluteStartTimeOfVideoClip() - fStartTrim + startTrim;
		fDoc->RoundArmeTimeToNearestVideoFrame(startTime);
		fStartTrim = startTime - (GetAbsoluteStartTimeOfVideoClip() - fStartTrim);
	
		// if the trim is such that all the clip is hidden, reduce the startTrim until there is one frame left in the clip
		if (GetDuration() <= ARME_TIME_ZERO)
			startTrim -= GetDoc()->ArmeTimeFromTimeValue(fMasterMovie->GetVideoFrameDuration());
			
	} while (GetDuration() <= ARME_TIME_ZERO);
}

void TVideoObject::SetEndTrim(ArmeTime endTrim)
{
	do 
	{
		ArmeTime endTime = GetAbsoluteEndxTimeOfVideoClip() + fEndTrim - endTrim;
		fDoc->RoundArmeTimeToNearestVideoFrame(endTime);
		fEndTrim = GetAbsoluteEndxTimeOfVideoClip() + fEndTrim - endTime;

		// if the trim is such that all the clip is hidden, increase the endTrim until there is one frame left in the clip
		if (GetDuration() <= ARME_TIME_ZERO)
			endTrim -= GetDoc()->ArmeTimeFromTimeValue(fMasterMovie->GetVideoFrameDuration());
			
	} while (GetDuration() <= ARME_TIME_ZERO);
}

void TVideoObject::RoundTimingInfo()
{
	//first, make sure the object starts on a frame boundary :
	ArmeTime absoluteStartTime = GetAbsoluteStartTimeOfVideoClip();

}


void TVideoObject::TimeStretch(ArmeTime startTime, ArmeTime endTime, ArmeTime newEndTime)
{
	ZzAssert(0);
}

TVideoXFadeEffect* TVideoObject::FindTransitionBetweenChildren(const TVideoObject* clip1, const TVideoObject* clip2)
{
	for (int i=0; i < GetChildrenCount(); ++i)
	{
		TVideoXFadeEffect* xFade = dynamic_cast<TVideoXFadeEffect*>(GetChild(i));
		if (xFade)
		{
			if ( ((xFade->fClip1 == clip1) && (xFade->fClip2 == clip2))
				|| ((xFade->fClip1 == clip2) && (xFade->fClip2 == clip1))) 
			{
				return xFade;
			}
		}
	}

	return NULL;
}

TVideoObject* TVideoObject::GetTransitionFor(TVideoObject* obj1, TVideoObject* obj2)
{
	TVideoXFadeEffect* newTransition = NULL;
	TVideoXFadeEffect* oldTransition = NULL;

	TVideoXFadeEffectDesc* oldTransitionEffectDesc = NULL;

	//first, look if there is already a transition currently active for these objects :
	//look in the children of this object
	if (oldTransition = FindTransitionBetweenChildren(obj1, obj2))
		return oldTransition;

	SInt32 transitionType = 0;
	float from;
	float to;
	ArmeTime absoluteStartTime;
	ArmeTime absoluteEndTime;

	//obj1 starts before obj2
	if (obj1->GetAbsoluteStartTimeOfVideoClip(false) < obj2->GetAbsoluteStartTimeOfVideoClip(false))
	{
		if (obj1->GetAbsoluteEndxTimeOfVideoClip(false) > obj2->GetAbsoluteEndxTimeOfVideoClip(false))
		{
			transitionType = TVideoXFadeEffect::CLIP1_CLIP2_TOGETHER;
			from = 0.5f;
			to = 0.5f;			
			absoluteStartTime = obj2->GetAbsoluteStartTimeOfVideoClip(false);
			absoluteEndTime = obj2->GetAbsoluteEndxTimeOfVideoClip(false);
		}
		else
		{
			transitionType = TVideoXFadeEffect::CLIP1_BEFORE_CLIP2;
			from = 0.0f;
			to = 1.0f;			
			absoluteStartTime = obj2->GetAbsoluteStartTimeOfVideoClip(false);
			absoluteEndTime = obj1->GetAbsoluteEndxTimeOfVideoClip(false);
		}
	}
	else //obj1 starts at the same time or after obj2
	{
		if (obj1->GetAbsoluteEndxTimeOfVideoClip() > obj2->GetAbsoluteEndxTimeOfVideoClip())
		{
			transitionType = TVideoXFadeEffect::CLIP1_AFTER_CLIP2;
			from = 1.0f;
			to = 0.0f;			
			absoluteStartTime = obj1->GetAbsoluteStartTimeOfVideoClip(false);
			absoluteEndTime = obj2->GetAbsoluteEndxTimeOfVideoClip(false);
		}
		else
		{
			transitionType = TVideoXFadeEffect::CLIP1_CLIP2_TOGETHER;
			from = 0.5f;
			to = 0.5f;			
			absoluteStartTime = obj1->GetAbsoluteStartTimeOfVideoClip(false);
			absoluteEndTime = obj1->GetAbsoluteEndxTimeOfVideoClip(false);
		}
	}

	//TODO
	//then, look if there has already been a transition between these two objects somewhere in the doc but that is not currently active
	if (oldTransitionEffectDesc = GetDoc()->FindEffectDescInOldTransitions(obj1, obj2)) //    FindTransitionBetween(GetDoc()->fOuttaHierarchyTransitionsPool, obj1, obj2))
	{
		//create a new effect with the same properties as the one that was outta hierarchy
		//Note : I'm not reusing the effect outta hierarchy because the clips may have moved
		// and if I putBackInHierarchy the effect, it might not be at the correct position
		float oldFrom, oldTo;
		GetFromAndToParamOfTransition(oldTransitionEffectDesc->fEffectDesc, oldFrom, oldTo);
		
		// if the oldTransition was created with FixedParameters or if the old transition was the same type of the one we are building,
		// we keep the same parameters.  
		if (oldTransitionEffectDesc->fFixedParameters)
		{
			//keep the same param as the old transition
			newTransition = new TVideoXFadeEffect(GetDoc(), transitionType, oldTransitionEffectDesc->fEffectDesc, -1.0f, -1.0f, true); 
		}
		else
		{
			if (oldTransitionEffectDesc->fTransitionType == transitionType)
			{
				//keep the same param as the old transition
				newTransition = new TVideoXFadeEffect(GetDoc(), transitionType, oldTransitionEffectDesc->fEffectDesc, oldFrom, oldTo, false);
			}
			else
			{
				// adapt the param to the current config
				newTransition = new TVideoXFadeEffect(GetDoc(), transitionType, oldTransitionEffectDesc->fEffectDesc, from, to, false); 
			}
		}
		
//		GetDoc()->fNewlyCreatedTransitions->AddItem(newTransition);  // a MICommand will create the representation for these objects
	}
	//if not, create the transition
	else
	{
		newTransition = new TVideoXFadeEffect(GetDoc(), transitionType, from, to); 
		
		//remove old effect
//		GetDoc()->fNewlyCreatedTransitions->AddItem(newTransition);  // a MICommand will create the representation for these objects
	}

	newTransition->fClip1 = obj1;
	newTransition->fClip2 = obj2;

	// insert the effect in the hierarchy
	TArmeWSObjPos pos;
	pos.fParent = this;
	pos.fStartTime = absoluteStartTime - GetAbsoluteStartTime();
	newTransition->__SetPos(pos);
	newTransition->_SetLength(absoluteEndTime - absoluteStartTime);
	_AddChild(newTransition);

	//update the description of desc
//	GetDoc()->AddToOldTransitionsDesc(newTransition); 

	return newTransition;
}

void TVideoObject::RemoveUnusedTransitions()
{
	//go through all the transitions, check if they are still used.
	for (int i=0; i < GetChildrenCount(); ++i)
	{
		TVideoXFadeEffect* xFade = dynamic_cast<TVideoXFadeEffect*>(GetChild(i));
		if (xFade)
		{
			if (!xFade->IsStillUseful())
			{
				//if they are not used anymore, move them to the pool of transitions.
/*				GetDoc()->fOuttaHierarchyTransitionsPool->AddItem(xFade);
				GetDoc()->fTransitionsToRemove->AddItem(xFade);  // a MICommand will remove the representation for these objects
				xFade->GetPos().fParent->_RemoveChild(xFade);*/
			}
		}
		else
		{
			TVideoObject* video = dynamic_cast<TVideoObject*>(GetChild(i));
			if (video)
			{
				video->RemoveUnusedTransitions();
			}
		}
	}
}

void TVideoObject::AddRef()
{
	fRefCount++;
	verntl(videoAlain, 6, "Adding ref to (" << this << ", " << fName << ") new ref count = " << fRefCount << ")");
}
void TVideoObject::Release()
{
	fRefCount--;
	verntl(videoAlain, 6, "Releasing ref on (" << this << ", " << fName << ") new ref count = " << fRefCount << ")");
	ZzAssert(fRefCount >= 0);
}
bool TVideoObject::IsInMasterMovie() const
{
	return fRefCount != 0;  //it is in the master movie if at least one chunks has a reference on it
}


TVideoObjectMemento::TVideoObjectMemento()
{
	
}

TVideoObjectMemento::TVideoObjectMemento(ArmeArchiveHandler* unarchiver) : TArmeWSObjMemento(unarchiver)
{
	ArmeArchivable* archObj;
	archObj = unarchiver->ReadCPPObject();
	fMasterMovie = dynamic_cast<TArmeMasterMovie*>(archObj);
	if (archObj)
	{
		ZzAssert(fMasterMovie);
	}
	fIsMutedForSolo = unarchiver->ReadBool();
	
	if (unarchiver->GetFileVer() >= 0x00010024)
	{
		fStartTrim = unarchiver->ReadArmeTime();
		fEndTrim = unarchiver->ReadArmeTime();
	}
	else
	{
		fStartTrim = ARME_TIME_ZERO;
		fEndTrim = ARME_TIME_ZERO;
	}
}

bool TVideoObjectMemento::WriteToArchive(ArmeArchiveHandler* archiver) const
{
	TArmeWSObjMemento::WriteToArchive(archiver);

	archiver->WriteCPPObject(fMasterMovie);
	archiver->WriteBool(fIsMutedForSolo);
	
	//from 0x00010024
	archiver->WriteArmeTime(fStartTrim);
	archiver->WriteArmeTime(fEndTrim);
	return true;
}


