/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#define VArmeGraphManager_h

#ifndef VArmeGraphManager_h
#include "VArmeGraphManager.h"
#endif //VArmeGraphManager_h

#ifndef ArmeBasicTypes_h
#include "ArmeBasicTypes.h"
#endif //ArmeBasicTypes_h

/*!
	@class VArmeGraphManager
	@abstract Abstract base for classes that can manage exchanging data between TArmeDoc elements
	(processors,...)
	
	@discussion This class is the base class for all the graph managers that can play a TArmeDoc. 
	The way they manage to play this document is unknown. The purpose of this object is to translate
	the document hierarchical structure into a set of nodes that exchange data.

*/

class TArmeDoc;

class VArmeGraphManager
{
public :
	virtual ~VArmeGraphManager() {};
	/*! @function Constructor. The parent doc needs to be known */
	VArmeGraphManager(TArmeDoc* doc);

	/*! @function Start this function starts the graph. Please note that this doesn't mean the document
	playback starts, but that the thread is started, etc... */
	virtual void			StartGraph() = 0;
	
	/*! @function Stop this function stops the graph and release resources used by this graph such as 
	sound device, QuickTime resources and so on...*/
	virtual void			StopGraph() = 0;
	
	/*! @function Play switches from paused mode */
	virtual void			Play() = 0;
	
	/*! @function Stop switches to paused mode */
	virtual void			Stop() = 0;
		
	/*! @function SetDoc This function allows to connect the graph manager to a given document */
	void					SetDoc( TArmeDoc* doc );
	
	/*! @function GetAudioFrameRate This function returns the number of frames per second. Not for public use */
	virtual UInt32			GetAudioFrameRate() const;
	
	/*! @function IsPlaying Whether the graph manager is playing the document data or is paused (stopped ?) */
	virtual bool		 	IsPlaying() const = 0;

	/*! @function GetPlayPos The "current" playback position (minus buffering latency) */
	virtual SInt64			GetPlayPos() const = 0;
	
	/*! @function GetPlayPos Sets the current playback position (warning : there is buffering latency) */
	virtual void			SetPlayPos(SInt64 pos) = 0;

	/*! @function OutDeviceBufferNbFrames */
	virtual SInt32			OutDeviceBufferNbFrames() = 0;
	
	/*! @function InDeviceBufferNbFrames  */
	//virtual SInt32			InDeviceBufferNbFrames() = 0;
	
protected :
	/*! @var fDoc is the document connected to this graph manager */
	TArmeDoc*				fDoc;
};

