/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#ifndef MEApplicationServices_h
#include "MEApplicationServices.h"
#endif //MEApplicationServices_h

#ifndef ZzHelpers_h
#include "ZzHelpers.h"
#endif //ZzHelpers_h

#ifndef MovieHelper_h
#include "MovieHelper.h"
#endif //MovieHelper_h

#include "Zz_PathHelpers.h"



#import "MIApplicationController.h"
#import "MIPreferences.h"
#include "NDAlias.h"
#import "MIDocumentController.h"
#import "MIMontageDocument.h"

MEApplicationServices::MEApplicationServices(void* appController)
{
	fAppController = appController;
}

void MEApplicationServices::ComplainAboutMissingAudioPlugin(VARPIPluginClassSig sig)
{
	char szMessage[1024];
	
	sprintf(szMessage,"The following audio plugin cannot be loaded : '%s' by '%s' (version %d.%d). All instances of this plugin will be turned into pass-through containers.", sig.name, sig.manufacturer, (int)(sig.versionNumber << 16),(int)((sig.versionNumber & (1 << 16))));
	NSString* message = [NSString stringWithCString:szMessage];
		
	NSRunAlertPanel(@"Error", message, @"OK", nil, nil);
}

void* MEApplicationServices::CreateProgressBarDlg(const char* title)
{
	ZzAssert(0);
	return NULL;
}

void MEApplicationServices::SetProgressBarFraction(void* cookie, float fraction)
{
	ZzAssert(0);
}

void MEApplicationServices::EndProgressBar(void* cookie)
{
	ZzAssert(0);
}

void MEApplicationServices::WarningMessageBox(const char* msg)
{
	ZzWarningMessageBox(msg);
}

float MEApplicationServices::GetStillClipDuration()
{
	return [[MIPreferences sharedPrefs] stillClipDefaultDuration];
}

bool MEApplicationServices::IsReferencingExternalFiles()
{
	return [[MIPreferences sharedPrefs] referenceExternalFiles];
}	

QTAtomContainer	MEApplicationServices::GetDefaultTransition()
{
	return [[MIPreferences sharedPrefs] getDefaultTransitionEffectDesc];
}

bool MEApplicationServices::KeepDefaultTransitionParameters()
{
	return (![[MIPreferences sharedPrefs] automaticTransitionParameters]);
}

#define FINDER_STRING @"tell application \"Finder\"\nset f to \"%@\"\nset al to make new alias file at \"%@\" to f\nset the name of al to \"%@\"\nend tell"

NSString * hfsStylePathFromPosixPath(CFStringRef s,  BOOL isDirectory)
{
	OSErr myErr;
	CFStringRef hfsStyle;
	CFURLRef url = CFURLCreateWithFileSystemPath(kCFAllocatorDefault, s, kCFURLPOSIXPathStyle, isDirectory);
	
	if (url == NULL) {
	printf("Can't get URL.\n");
	return(nil);
	}

	if (hfsStyle = CFURLCopyFileSystemPath(url, kCFURLHFSPathStyle)) 
	{
		CFRelease(url);
		return [hfsStyle autorelease];
	}
	CFRelease(url);
	return nil;
}

@interface FileErrorHandling : NSObject
{

}
-(BOOL)fileManager:(NSFileManager *)manager 
        shouldProceedAfterError:(NSDictionary *)errorDict;
@end
@implementation FileErrorHandling
- (BOOL)fileManager:(NSFileManager *)manager 
        shouldProceedAfterError:(NSDictionary *)errorDict
{
    int result;
    result = NSRunAlertPanel(@"Gumby App", @"File operation error: %@ with file: %@", @"Proceed", @"Stop", NULL, 
            [errorDict objectForKey:@"Error"], 
            [errorDict objectForKey:@"Path"]);

    if (result == NSAlertDefaultReturn)
        return YES;
    else
        return NO;
}
@end

void createAliasFrom(CFStringRef  source, CFStringRef targetDirectory, NSString*  aliasName)
{
	NSString *command = [NSString stringWithFormat:FINDER_STRING, hfsStylePathFromPosixPath(source, NO), hfsStylePathFromPosixPath(targetDirectory, YES), aliasName];
	NSDictionary *errorInfo;
	NSAppleScript *as = [[[NSClassFromString(@"NSAppleScript") alloc] initWithSource:command] autorelease];
	NSAppleEventDescriptor *aedesc = [as executeAndReturnError:&errorInfo];
	if (aedesc == nil) NSLog([errorInfo description]);
}

void MEApplicationServices::CreateAliasFileInProjectPackage(ZzString* theAliasPathName, const ZzString* theFileToReference)
{
	MIMontageDocument *montageDocument = [[MIDocumentController sharedDocumentController] currentDocument];
	//Let's consider an example:
	//theLastAliasPathName = nothing now
	//theFileToReference = "/Users/alainm/Music/1beat.aiff"

	//find a suitable name for the alias
	ZzString theAliasFileName = ZzLastPathComponent(*theFileToReference);
	
	char theAliasCompletePathName[1024];
	ZzAppendFileNameToPath( theAliasCompletePathName, 1024, [montageDocument armeDocument]->GetProjectFolderPath(), theAliasFileName.getUTF8String());
	//theAliasCompletePathName = "/Users/alainm/MontageProjects/testAlias.HAV/1beat.aiff"
	//check if the name already exists
	while ([[NSFileManager defaultManager] fileExistsAtPath:[NSString stringWithCString:theAliasCompletePathName]])
	{
		ZzGenerateAlternativePath(theAliasCompletePathName, 1024);
		//theAliasCompletePathName = "/Users/alainm/MontageProjects/testAlias.HAV/1beat_2.aiff"
	}

	theAliasFileName = ZzLastPathComponent(theAliasCompletePathName);  // theAliasFileName = "1beat.aiff"

	//we can not create the alias directly in the document package so, we must create it in the parent directory
	ZzString theAliasTempPathName = ZzDeleteLastPathComponent(theAliasCompletePathName);
	theAliasTempPathName = ZzDeleteLastPathComponent(theAliasTempPathName);	//theAliasTempPathName = "/Users/alainm/MontageProjects"

	char theAliasCurrentPathName[1024];  //="/Users/alainm/MontageProjects/1beat.aiff"
	ZzAppendFileNameToPath(theAliasCurrentPathName, 1024, theAliasTempPathName.getUTF8String(), theAliasFileName.getUTF8String());

	//check if the alias temp name already exists
	while ([[NSFileManager defaultManager] fileExistsAtPath:[NSString stringWithCString:theAliasCurrentPathName]])
	{
		ZzGenerateAlternativePath(theAliasCurrentPathName, 1024);
		//theAliasTempPathName = "/Users/alainm/MontageProjects/1beat_2.aiff"
	}
	
	theAliasFileName = ZzLastPathComponent(theAliasCurrentPathName);  // theAliasFileName = "1beat.aiff" or "1beat_2.aiff"

	//this creates the alias in "/Users/alainm/MontageProjects/1beat.aiff"
	createAliasFrom(theFileToReference->getCFString(), theAliasTempPathName.getCFString(), theAliasFileName.getNSString());
	
	//this copies the alias in "/Users/alainm/MontageProjects/testAlias.HAV/1beat.aiff"
	FileErrorHandling* errorHandler = [[FileErrorHandling alloc] init];
	[[NSFileManager defaultManager] movePath:[NSString stringWithCString:theAliasCurrentPathName] toPath:[NSString stringWithCString:theAliasCompletePathName] handler:errorHandler];

	*theAliasPathName = theAliasCompletePathName;
	[errorHandler release]; 
}

const char* MEApplicationServices::ResolveAlias(const char* aliasPath)
{
	NDAlias* theNewAlias = [NDAlias aliasWithContentsOfFile:[NSString stringWithCString:aliasPath]];
	NSString* path = [theNewAlias path];
    return [path cString];
}