/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#ifndef MEAudioChunkList_h
#include "MEAudioChunkList.h"
#endif //MEAudioChunkList_h

#ifndef ArmeArchiveHandler_h
#include "ArmeArchiveHandler.h"
#endif //ArmeArchiveHandler_h

#ifndef ZzHeapWalker_h
#include "ZzHeapWalker.h"
#endif ZzHeapWalker_h

#undef VER_DEF_TOPIC
#define VER_DEF_TOPIC MEAudioChunkList

MEAudioChunkList::MEAudioChunkList() :
	fChunks(NULL),
	fNbChunks(0),
	fCachedDuration(0)
{
}

bool MEAudioChunkList::IsEmpty() const
{
	if(fChunks == NULL && fNbChunks == 0 && fCachedDuration == 0)
		return true;
	else
		return false;
}

MEAudioChunkList::MEAudioChunkList(const MEAudioChunk& chunk)
{
	//printf("MEAudioChunkList(%p)::MEAudioChunkList(const MEAudioChunk& chunk)\n",this);
	fChunks = new MEAudioChunk[1];
	*fChunks = chunk;
	chunk.doc->AudioChunkFileManager()->RetainFile(chunk); // ADD REF
	fNbChunks = 1;
	UpdateCachedDuration();
}

MEAudioChunkList::MEAudioChunkList(const MEAudioChunk* chunks, SInt32 nbChunks)
{
	//printf("MEAudioChunkList(%p)::MEAudioChunkList(const MEAudioChunk* chunks, SInt32 nbChunks)\n",this);
	fNbChunks = nbChunks;
	fChunks = new MEAudioChunk[fNbChunks];
	for(SInt32 i = 0; i < fNbChunks; i++)
	{
		fChunks[i] = chunks[i];
		fChunks[i].doc->AudioChunkFileManager()->RetainFile(fChunks[i]);  // ADD REF
	}
	UpdateCachedDuration();
}

MEAudioChunkList::~MEAudioChunkList()
{
	//printf("MEAudioChunkList(%p)::~MEAudioChunkList()\n",this);
	Clear();  // DEC REF
	fCachedDuration = -1973;
}

void MEAudioChunkList::InsertChunk(const MEAudioChunk& chunk, SInt64 pos)
{
	//printf("MEAudioChunkList(%p)::InsertChunk(const MEAudioChunk& chunk, SInt64 pos)\n",this);
	MEAudioChunkList insertedList(chunk); // ADD REF
	InsertChunkList(insertedList,pos); // DEC REF ADD REF
}

void MEAudioChunkList::InsertChunkList(const MEAudioChunkList& chunkList, SInt64 pos)
{
	//printf("MEAudioChunkList(%p)::InsertChunkList(const MEAudioChunkList& chunkList, SInt64 pos)\n",this);
//	vernl(6,"MEAudioChunkList::InsertChunkList(chunkList = " << chunkList << ", pos = " << (SInt32)pos);
	// the slice needs to be totally contained in the list
	// this should be handled by the parent object
	ZzAssert(pos >= 0);
	ZzAssert(pos <= fCachedDuration);
	
	// treat the special case where the current list is empty
	if(fNbChunks == 0)
	{
		*this = chunkList;
		UpdateCachedDuration();
	}
	else
	{
		// the resulting new chunk list cannot be longer than the sum of 
		// both lists + 2, in case the new list is inserted in the middle of
		// a chunk
		SInt32 maxNewNbChunks = fNbChunks+chunkList.GetNbChunks()+2; 
		MEAudioChunk* newChunks = new MEAudioChunk[maxNewNbChunks];
		SInt32 newNbChunks = 0;
		
		SInt64 curChunkStart = 0;
		SInt64 curChunkEndx = 0;
		bool bListInserted = false;
		for(SInt32 i = 0; i < fNbChunks; i++)
		{
			const MEAudioChunk& curChunk = fChunks[i];
			curChunkEndx += curChunk.duration;
			
			// if the insertion pos is at least 1 time after this chunk OR,
			// if the insertion point is before the current chunk OR if the
			// new list has already been added, then we simply add the current 
			// chunk to the list.
			if(pos > curChunkEndx || pos < curChunkStart || bListInserted)
			{
				newChunks[newNbChunks] = curChunk;
				newNbChunks++;
			}
			
			// if the insertion point is contained in this chunk OR immediatly next to this chunk
			// AND if the chunklist hasn't been inserted yet, we first insert the first splitted part
			// of the current chunk (if its length is > 0), then we add all the new chunks, then the 
			// the second splitted part of the current chunk (if its length is > 0). WE ALSO NEED TO
			// SET the bListHasBeenInserted flag, because, for example,
			// if we have the list :
			// chunk 0 : duration 100
			// chunk 1 : duration 100
			// and if we insert a new list at 100, it will get inserted twice,
			// one at the end of chunk 0, and one at the start of chunk 1 
			if(pos >= curChunkStart && pos <= curChunkEndx && !bListInserted)
			{
				bListInserted = true;
				SInt64 remainingOfCurChunkBeforeInsertionPos = pos - curChunkStart;
				if(remainingOfCurChunkBeforeInsertionPos > 0)
				{
					newChunks[newNbChunks] = curChunk;
					newChunks[newNbChunks].duration = remainingOfCurChunkBeforeInsertionPos;
					newNbChunks++;
				}
				for( SInt32 j = 0; j < chunkList.GetNbChunks(); j++)
				{
					newChunks[newNbChunks] = chunkList.GetChunk(j);
					newNbChunks++;
				}
				SInt64 remainingOfCurChunkAfterInsertionPos = curChunkEndx - pos;
				if(remainingOfCurChunkAfterInsertionPos > 0)
				{
					newChunks[newNbChunks] = curChunk;
					newChunks[newNbChunks].startInFile += remainingOfCurChunkBeforeInsertionPos;
					newChunks[newNbChunks].duration -= remainingOfCurChunkBeforeInsertionPos;
					newNbChunks++;
				}
			}
			curChunkStart += curChunk.duration;
		}
		ZzAssert(newNbChunks <= maxNewNbChunks);
		Clear(); // DEC REF
		fNbChunks = newNbChunks;
		fChunks = new MEAudioChunk[fNbChunks];
		for(SInt32 i = 0; i < fNbChunks; i++)
		{
			fChunks[i] = newChunks[i];
			fChunks[i].doc->AudioChunkFileManager()->RetainFile(fChunks[i]);// ADD REF
		}
		UpdateCachedDuration();
		delete [] newChunks;
	}
}

void MEAudioChunkList::Clear()
{
	//printf("MEAudioChunkList(%p)::Clear()\n",this);
	for(SInt32 i = 0; i < GetNbChunks(); i++)
	{
		if(GetChunk(i).GetFileIndex().data >= 0)
		{
			ZzAssert(GetChunk(i).doc);
			GetChunk(i).doc->AudioChunkFileManager()->ReleaseFile(GetChunk(i)); // DEC REF
		}
	}
	delete [] fChunks;
	fChunks = NULL;
	fNbChunks = 0;
	UpdateCachedDuration();
}

void MEAudioChunkList::ReplaceRangeWithChunk(const MEAudioChunk& chunk, SInt64 pos, SInt64 len)
{
//	vernl(6,"MEAudioChunkList::ReplaceRangeWithChunk(chunk = " << chunk << ", pos = " << (SInt32)pos << ", len = " << (SInt32)len << ")");
//	vernl(6,"MEAudioChunkList::ReplaceRangeWithChunk() BEFORE *this = " << *this);
	DeleteSlice(pos,len);
	InsertChunk(chunk,pos);
//	vernl(6,"MEAudioChunkList::ReplaceRangeWithChunk() AFTER *this = " << *this);
}

bool MEAudioChunkList::DeleteSlice(SInt64 sliceStart, SInt64 length)
{
	//printf("MEAudioChunkList(%p)::DeleteSlice(SInt64 sliceStart, SInt64 length)\n",this);
//	vernl(6,"MEAudioChunkList::DeleteSlice(sliceStart = " << (SInt32)sliceStart << ", length = " << (SInt32)length);
	// the slice needs to be totally contained in the list
	// this should be handled by the parent object
	ZzAssert(sliceStart >= 0);
	ZzAssert(sliceStart + length <= fCachedDuration);
	
	// delete slice can add max 1 chunk to the list, thus the new 
	// chunk can have, @ max, fNbChunks+1 items.
	SInt32 maxNewNbChunks = fNbChunks+1;
	MEAudioChunk* newChunks = new MEAudioChunk[maxNewNbChunks];
	SInt32 newNbChunks = 0;
	
	SInt64 sliceEndx = sliceStart + length;
	SInt64 curChunkStart = 0;
	SInt64 curChunkEndx = 0;
	
	for(SInt32 i = 0; i < fNbChunks; i++)
	{
		const MEAudioChunk& curChunk = fChunks[i];
		curChunkEndx += curChunk.duration;
		
		// case 1 : the ranges do NOT intersect
		
		bool cond1 = (sliceStart >= curChunkEndx); 
		bool cond2 = (sliceEndx <= curChunkStart);
		bool cond = cond1 || cond2;
		
		if(cond) // ranges do not intersect
		{
			newChunks[newNbChunks] = curChunk;
			newNbChunks++;
		}
		else
		{
			// compute unmodified part of the chunk before the slice
			SInt64 remainingLenBeforeSliceStart = sliceStart - curChunkStart;
			if(remainingLenBeforeSliceStart > 0)
			{
				newChunks[newNbChunks] = curChunk;
				newChunks[newNbChunks].duration = remainingLenBeforeSliceStart;
				newNbChunks++;
			}
			// compute unmodified part of the chunk after the slice
			SInt64 remainingLenAfterSliceEnd = curChunkEndx - sliceEndx;
			if(remainingLenAfterSliceEnd > 0)
			{
				newChunks[newNbChunks] = curChunk;
				// there is an offset in this chunk 
				newChunks[newNbChunks].startInFile += (sliceEndx - curChunkStart);
				newChunks[newNbChunks].duration = remainingLenAfterSliceEnd;
				newNbChunks++;
			}
		}
		curChunkStart = curChunkEndx;
	}
	ZzAssert(newNbChunks <= maxNewNbChunks);
	Clear(); // this takes care of decrementing the references
	fNbChunks = newNbChunks;
	if(newNbChunks == 0)
	{
		fChunks = nil;
		fCachedDuration = 0;
	}
	else
	{
		fChunks = new MEAudioChunk[fNbChunks];
		for(SInt32 i = 0; i < fNbChunks; i++)
		{
			fChunks[i] = newChunks[i];
			fChunks[i].doc->AudioChunkFileManager()->RetainFile(fChunks[i]);
		}
		UpdateCachedDuration();
	}
	ZzAssert(fCachedDuration >= 0);
	delete [] newChunks;
	return (fCachedDuration == 0);
}

void MEAudioChunkList::WriteToArchive(ArmeArchiveHandler* archiver) const
{
	archiver->WriteSInt32(fNbChunks);
	for(SInt32 i = 0; i < fNbChunks; i++)
	{
		fChunks[i].WriteToArchive(archiver);
	}
}

void MEAudioChunkList::ReadFromArchive(ArmeArchiveHandler* unarchiver)
{
	fNbChunks = unarchiver->ReadSInt32();
	ZzAssert(fChunks == NULL);
	fChunks = new MEAudioChunk[fNbChunks];
	for(SInt32 i = 0; i < fNbChunks; i++)
	{
		fChunks[i].ReadFromArchive(unarchiver);
		fChunks[i].doc->AudioChunkFileManager()->RetainFile(fChunks[i]);
	}
	UpdateCachedDuration();
}

void MEAudioChunkList::UpdateCachedDuration()
{
	fCachedDuration = 0;
	for(SInt32 i = 0; i < fNbChunks; i++)
	{
		fCachedDuration += fChunks[i].duration;
	}
}

MEAudioChunkList MEAudioChunkList::ExtractSlice(SInt64 sliceStart, SInt64 len) const
{
	// the slice needs to be totally contained in the list
	// this should be handled by the parent object
	ZzAssert(sliceStart >= 0);
	ZzAssert(sliceStart + len <= fCachedDuration);
	
	// the extracted slice cannot have more chunks than the list, obviously
	SInt32 maxNewNbChunks = fNbChunks;
	
//	heapwalker(MEAudioChunk* newChunks = new MEAudioChunk[maxNewNbChunks]);
	MEAudioChunk* newChunks = new MEAudioChunk[maxNewNbChunks];
	
	SInt32 newNbChunks = 0;
	
	SInt64 sliceEndx = sliceStart + len;
	SInt64 curChunkStart = 0;
	SInt64 curChunkEndx = 0;
	
	for(SInt32 i = 0; i < fNbChunks; i++)
	{
		const MEAudioChunk& curChunk = fChunks[i];
		curChunkEndx += curChunk.duration;
		
		// case 1 : the current chunk is totally outside of the requested slice
		bool cond1 = (sliceStart >= curChunkEndx); 
		bool cond2 = (sliceEndx <= curChunkStart);
		bool cond = cond1 || cond2;
		if(cond)
		{
			// then, we don't do anything... 
		}
		else
		{
			// compute part of the chunk before the slice
			SInt64 lenToRemoveBeforeSliceStart = sliceStart - curChunkStart;
			// if the slice starts before the chunk, we don't remove anything
			if(lenToRemoveBeforeSliceStart < 0)
				lenToRemoveBeforeSliceStart = 0;
			// compute part of the chunk adfter the slice
			SInt64 lenToRemoveAfterSliceEnd = curChunkEndx - sliceEndx;
			// if the slice starts before the chunk, we don't remove anything
			if(lenToRemoveAfterSliceEnd < 0)
				lenToRemoveAfterSliceEnd = 0;
			
			// let's add the chunk and remove the parts of the chunk not in the slice
			newChunks[newNbChunks] = curChunk;
			// there is an offset in this chunk 
			newChunks[newNbChunks].startInFile += lenToRemoveBeforeSliceStart;
			newChunks[newNbChunks].duration -= (lenToRemoveBeforeSliceStart + lenToRemoveAfterSliceEnd);
			newNbChunks++;
		}
		curChunkStart += curChunk.duration;
	}
	ZzAssert(newNbChunks <= maxNewNbChunks);
	MEAudioChunkList newChunkList(newChunks,newNbChunks);
	delete [] newChunks;
	return newChunkList;
}

MEAudioChunkList& MEAudioChunkList::operator=(const MEAudioChunkList& other)
{
	//printf("MEAudioChunkList(%p)::operator=(const MEAudioChunkList& other (%p) )\n",(void*)this,(void*)(&other));
	Clear(); // decrementing the ref counts
	fNbChunks = other.GetNbChunks();
	fChunks = new MEAudioChunk[fNbChunks];
	for(SInt32 i = 0; i < fNbChunks; i++)
	{
		fChunks[i] = other.GetChunk(i);
		fChunks[i].doc->AudioChunkFileManager()->RetainFile(fChunks[i]);
	}
	UpdateCachedDuration();
	return *this;
}

MEAudioChunkList::MEAudioChunkList(MEAudioChunkList& other)
{
	//printf("MEAudioChunkList(%p)::MEAudioChunkList(const MEAudioChunkList& other (%p) )\n",(void*)this,(void*)(&other));
	fNbChunks = other.GetNbChunks();
	if(fNbChunks == 0)
		fChunks = NULL;
	else
		fChunks = new MEAudioChunk[fNbChunks];
	for(SInt32 i = 0; i < fNbChunks; i++)
	{
		fChunks[i] = other.GetChunk(i);
		fChunks[i].doc->AudioChunkFileManager()->RetainFile(fChunks[i]);
	}
	UpdateCachedDuration();
}	

MEAudioChunk::~MEAudioChunk()
{
//	//printf("MEAudioChunk(%p)::~MEAudioChunk()\n",(void*)this);
//	vernl(6,"MEAudioChunk("<<(void*)this<<")::~MEAudioChunk()");
	if(doc)
	{
		doc->AudioChunkFileManager()->ReleaseFile(chunkFileIndex);
	}
	else
	{
		ZzAssert(chunkFileIndex.data == -1);
		ZzAssert(startInFile == -1);
		ZzAssert(duration == -1);
	}
}

MEAudioChunk::MEAudioChunk(TArmeDoc* _doc)
{
//	//printf("MEAudioChunk(%p)::MEAudioChunk(TArmeDoc* _doc)\n",(void*)this);
//	vernl(6,"MEAudioChunk("<<(void*)this<<")::MEAudioChunk( doc = " << (void*)_doc << ")");
	duration = 0;
	startInFile = 0;
	chunkFileIndex = kMEUninitializedChunkFileIndex;
	doc = _doc;
}

MEAudioChunkFileIndex MEAudioChunk::GetFileIndex() const
{
	return chunkFileIndex;
}

std::ostream& operator<<(std::ostream& s, const MEAudioChunk& audioChunk)
{
	return s << "(idx =" << audioChunk.chunkFileIndex.data << " st=" << (SInt32)audioChunk.startInFile << " dur=" << (SInt32)audioChunk.duration <<")";
}

bool MEAudioChunk::IsSilentChunk() const
{
	ZzAssert( ( (chunkFileIndex.data == -1) && (startInFile == -1) && (duration > 0) ) ||
				(chunkFileIndex.data >= 0) && (startInFile >= 0));
	return (chunkFileIndex.data == -1);
}

MEAudioChunk MEAudioChunk::CreateSilentChunk(TArmeDoc* _doc, SInt64 duration)
{
	MEAudioChunk chunk(_doc);
	chunk.chunkFileIndex.data = -1;
	chunk.startInFile = -1;
	chunk.duration = duration;
	return chunk;
}

MEAudioChunk& MEAudioChunk::operator=(const MEAudioChunk& other)
{
//	vernl(6,"MEAudioChunk("<<(void*)this<<")::operator=( other = " << (void*)(&other) << ")");
	// it is important to retain other before we release ours in
	// case both incices are the same
	other.doc->AudioChunkFileManager()->RetainFile(other);
	if(doc)
	{
		doc->AudioChunkFileManager()->ReleaseFile(chunkFileIndex);
	}
	else
	{
		ZzAssert(chunkFileIndex == kMEUninitializedChunkFileIndex);
	}
	chunkFileIndex = other.GetFileIndex();
	startInFile = other.startInFile;
	duration = other.duration;
	doc = other.doc;
	return *this;
}
	
MEAudioChunk::MEAudioChunk(const MEAudioChunk& other)
{
//	//printf("MEAudioChunk(%p)::MEAudioChunk(const MEAudioChunk& other (%p) )\n",(void*)this,(void*)(&other));
//	vernl(6,"MEAudioChunk("<<(void*)this<<")::MEAudioChunk(other = " << (void*)(&other) << ")");
	chunkFileIndex = other.GetFileIndex();
	doc = other.doc;
	doc->AudioChunkFileManager()->RetainFile(chunkFileIndex);
	startInFile = other.startInFile;
	duration = other.duration;
	doc = other.doc;
}

bool operator==(const MEAudioChunk& c1, const MEAudioChunk& c2)
{
	return (	(c1.GetFileIndex() == c2.GetFileIndex()) &&
				(c1.startInFile == c2.startInFile) &&
				(c1.duration == c2.duration) &&
				(c1.doc == c2.doc) );
}

bool operator!=(const MEAudioChunk& c1, const MEAudioChunk& c2)
{
	return !(c1 == c2);
}

void MEAudioChunk::SetFileIndex(MEAudioChunkFileIndex _chunkFileIndex)
{
	// it is important to retain other before we release ours in
	// case both incices are the same
	doc->AudioChunkFileManager()->RetainFile(_chunkFileIndex);
	doc->AudioChunkFileManager()->ReleaseFile(chunkFileIndex);
	ZzAssert(chunkFileIndex.data == kMEUninitializedChunkFileIndex.data);
	ZzAssert(duration == 0);
	ZzAssert(startInFile == 0);
	chunkFileIndex = _chunkFileIndex;
}

MEAudioChunk::MEAudioChunk()
{
//	//printf("MEAudioChunk(%p)::MEAudioChunk()\n",(void*)this);
//	//printf("MEAudioChunk(%p)::MEAudioChunk()\n",(void*)this);
//	vernl(6,"MEAudioChunk("<<(void*)this<<")::MEAudioChunk()");
	duration = -1;
	startInFile = -1;
	chunkFileIndex = kMEUninitializedChunkFileIndex;
//	usleep(300000);
	doc = NULL;
}

