/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#define MEAudioClip_h
 
#ifndef MEAudioClip_h
#include "MEAudioClip.h"
#endif //MEAudioClip_h

#ifndef ZzTypes_h
#include "ZzTypes.h"
#endif //ZzTypes_h

#ifndef TArmeDoc_h
#include "TArmeDoc.h"
#endif //TArmeDoc_h

#ifndef MEAudioClipData_h
#include "MEAudioClipData.h"
#endif //MEAudioClipData_h

#ifndef MEDocPlayer_h
#include "MEDocPlayer.h"
#endif //MEDocPlayer_h

#ifndef TArmeWSObj_h
#include "TArmeWSObj.h"
#endif //TArmeWSObj_h
 
class MEAudioClipData;
class ZzSplitTask;

class MEAudioClip : public TArmeWSObj
{
public :
					MEAudioClip(
						const ZzString&				name,
						const FSSpec&				movieFile, 
						int 						trackIndex, 
						TArmeDoc* 					doc, 
						MIProgressBarDlgManager* 	progressMngr);

/*
					MEAudioClip(
						const char*					path, 
						TArmeDoc* 					doc, 
						MIProgressBarDlgManager* 	progressMngr);
*/
						
					// the ctor that's used when initializing a clip with a new chunk.
					MEAudioClip(
						const MEAudioChunk&			chunk, 
						SInt32 						nbChannels, 
						MIProgressBarDlgManager* 	progressMngr);

					// the ctor that's used when rendering a set of objects into a single clip
					// all the objects need to belong to the same doc (and be siblings?)
					MEAudioClip(
						TArmeDoc* 					doc,
						ZzArchPtrArray*				objSet, 
						MIProgressBarDlgManager* 	progressMngr);
			
					// this ctor is used for clipboard copy. the data fDoc can be non-NULL if
					// its original doc still lives. The clip doc will be set to NUL.
					MEAudioClip(TArmeDoc* doc, MEAudioClipData* data, bool bCopyOrAlias);

					// when creating clips with an existing MEAudioClipData
					MEAudioClip(TArmeDoc* doc, MEAudioClipData* data);

	//
	// arch stuff
	//
							MEAudioClip(ArmeArchiveHandler* unarchiver);
	virtual bool			WriteToArchive(ArmeArchiveHandler* archiver) const;
	virtual void			RecreateNonArchivedCachedData(MIProgressBarDlgManager* progressMngr);
	virtual void 			CountPostUnarchivingLengthyTasks(UInt32* nbOfTasks);

	virtual					~MEAudioClip();
	MEAudioClipData*		GetData() { return fAudioClipData; }
	const MEAudioClipData*	GetData() const { return fAudioClipData; }
	virtual TArmeWSObj* 	GetSliceCopy(ArmeTime startTime, ArmeTime endTime, const char* tempDirectoryName, MIProgressBarDlgManager* progressMngr) const;
	virtual ArmeTime		InsertSlice(ArmeTime insertTime, const TArmeWSObj* obj, MIProgressBarDlgManager* progressMngr);
	virtual bool			DeleteSlice(ArmeTime startTime, ArmeTime endTime, MIProgressBarDlgManager* progressMngr);  
	virtual void			Move();

	virtual TArmeWSObjMemento* 	GetMemento() const;
	virtual void 				_FillMemento(TArmeWSObjMemento* memento) const;
	virtual void 				SetMemento(TArmeWSObjMemento* memento);


#ifdef OLD_VAO_RAO
	virtual bool			ContainsAudioObjectsRoutedTo(bool bStereo, SInt32 vaoIndex) const;
	virtual void			GebruikenVAOs(ZzArray<SInt32>* monoVAOsUsed, ZzArray<SInt32>* stereoVAOsUsed) const;// see TArmeWSObj
#endif //#ifdef OLD_VAO_RAO
	virtual void			_NotifyParentPosHasChanged();
	virtual void			SetMuted(bool muted);
	virtual void			SetStartTrim(ArmeTime startTrim);
	virtual void			SetEndTrim(ArmeTime endTrim);
	virtual ArmeTime		GetStartTrim() const;
	virtual ArmeTime		GetEndTrim() const;
	// we need to override SetOuttaHierarchy() in order to invalidate track index
	// otherwise, when undoing a delete, for example, an objet might suddenly come in the game
	// with a given track index that's totally bogus since it was removed from the hierarchy
	// weeks ago...
	virtual void			SetOuttaHierarchy();
	
	virtual void 			CheckAudioDataNeedsPreviewComputingRecursive();

private :
						MEAudioClip(MEAudioClip&);
						MEAudioClip(const MEAudioClip&);
public :
	void				InvalidateTrackIndices();
	virtual SInt32		GetAudioTrackForPlayer(UInt32 playerIndex) const;
	void				SetAudioTrackForPlayer(UInt32 playerIndex, SInt32 audioTrack);
	bool				IsStereo();
	SInt32				GetNbChannels() const;
	// we keep a local copy of the chunk list (with trim value). this function must be called when
	// this copy should be updated. it can be called by the clip data or by the clip itself.
//	void				InvalidateChunkList();

public :
//	const MEAudioChunkList&	GetChunkListNoLock(); // this returns the real chunk list, after trimming, without locking the doc
	virtual ArmeTime		GetDuration() const;
	virtual void			_SetLengh( ArmeTime len );
	virtual SInt32 			GetType() const;
	virtual bool			ContainsAudioDuring(ArmeTime startTime, ArmeTime endTime, bool bConsiderAlsoMutedObjects) const;
	
	// this is a tmp flag used for the track assignment. we don't need to initialize it or save it
	bool				fTrackAssignedFlag;
private :
	MEAudioClipData*	fAudioClipData;
	SInt32				fTrackIndex[kMEDocPlayer_MaxNbAudioPlayers];
};

inline void MEAudioClip::SetAudioTrackForPlayer(UInt32 playerIndex, SInt32 trackIndex)
{
	ZzAssert(playerIndex < kMEDocPlayer_MaxNbAudioPlayers);
	fTrackIndex[playerIndex] = trackIndex;
}

inline bool MEAudioClip::IsStereo()
{
	return fAudioClipData->IsStereo();
}

class MEAudioClipMemento : public TArmeWSObjMemento
{
	friend class MEAudioClip;
public : 
	MEAudioClipMemento();
	virtual ~MEAudioClipMemento();

public :  // ArmeArchivable stuff
	MEAudioClipMemento(ArmeArchiveHandler* handler);
	virtual bool WriteToArchive(ArmeArchiveHandler* handler) const;
private :
	MEAudioClipDataMemento*	fAudioClipDataMemento;
};

inline SInt32 MEAudioClip::GetNbChannels() const
{
	return fAudioClipData->GetNbChannels();
}




