/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#ifndef MEAudioClipData_h
#include "MEAudioClipData.h"
#endif //MEAudioClipData_h

#ifndef TArmeMediaEngine_h
#include "TArmeMediaEngine.h"
#endif //TArmeMediaEngine_h

#ifndef MEAudioClip_h
#include "MEAudioClip.h"
#endif //MEAudioClip_h

#ifndef MECompoundAudioFile_h
#include "MECompoundAudioFile.h"
#endif //MECompoundAudioFile_h

#ifndef TArmeDoc_h
#include "TArmeDoc.h"
#endif //TArmeDoc_h

#ifndef ZzHelpers_h
#include "ZzHelpers.h"
#endif //ZzHelpers_h

#ifndef ZzHeapWalker_h
#include "ZzHeapWalker.h"
#endif ZzHeapWalker_h

#ifndef ZzException_h
#include "ZzException.h"
#endif //ZzException_h

#ifndef ZzRABI_h
#include "ZzRABI.h"
#endif //ZzRABI_h

/*
#ifndef MESplitTaskImportMovieDataInAudioClip_h
#include "MESplitTaskImportMovieDataInAudioClip.h"
#endif //MESplitTaskImportMovieDataInAudioClip_h

#ifndef MESplitTaskImportAndResampleAudioTrackInAudioClip_h
#include "MESplitTaskImportAndResampleAudioTrackInAudioClip.h"
#endif //MESplitTaskImportAndResampleAudioTrackInAudioClip_h
*/

#ifndef ME_Audio_File_Helpers_h
#include "ME_Audio_File_Helpers.h"
#endif //ME_Audio_File_Helpers_h

#ifndef MESplitTaskComputeAudioClipDataPreview_h
#include "MESplitTaskComputeAudioClipDataPreview.h"
#endif //MESplitTaskComputeAudioClipDataPreview_h

#ifndef MIProgressBarDlgManager_h
#include "MIProgressBarDlgManager.h"
#endif //MIProgressBarDlgManager_h

#define kChunkArraySizeIncr 32

#undef VER_DEF_TOPIC
#define VER_DEF_TOPIC MEAudioClip

#define CLIP(min,x,max) (((x)>(max)) ? (max) : (((x)<(min)) ? (min) : (x)))

class ZzSplitTask;

MEAudioClipData::MEAudioClipData(TArmeDoc* doc) :
	fDoc(doc),
	fRefCnt(0),
	fFakeDuration(0)
{
	fNbChannels = 0;
	SetNameCopy("untitled empty MEAudioClipData");
	
	// the preview is NULL right now.
	fMainPreview = NULL;
	fMainPreviewNbFrames = 0;
	
	// let's initialize the owning clips
	fOwningClips = new ZzArchPtrArray; //this can throw
	Retain(NULL); // a null owner means the inital self ownership
}

void MEAudioClipData::AdoptNewChunkList(const MEAudioChunkList& chunkList)
{
	fDoc->LockWrite();
	fPreTrimChunkList = chunkList;
	fStartTrim = _ArmeTime(0);
	fEndTrim = _ArmeTime(0);
	RecomputeTrimmedChunkList();
	InvalidateMainPreview();
	fDoc->UnlockWrite();
}

MEAudioClipData::MEAudioClipData(MEAudioClipData* data) :
	fRefCnt(0),
	fFakeDuration(0)
{
	fDoc = data->GetDoc();
	fDoc->LockRead();
	SetNameCopy(data->GetName());
	SetNbChannels(data->GetNbChannels());
	// let's initialize the owning clips
	fOwningClips = new ZzArchPtrArray;
	ZzRABI_ObjectDeleter<ZzArchPtrArray> rabiOwningClips(&fOwningClips);
	
	Retain(NULL); // a null owner means the inital self ownership
	
	// let's copy the untrimmed playlist
	fPreTrimChunkList = data->GetPreTrimChunkList();

	fStartTrim = data->GetStartTrim();
	fEndTrim = data->GetEndTrim();
	
	fDoc->UnlockRead();
	
	RecomputeTrimmedChunkList();
						
	// we COPY the main preview instead of recomputing it. !!!! fNbChannels and the trimmed chunk list need to be inited for this
	// to succeed !!!!!!
	
	// We CAN modify fMainPreview without locking since it's only used in the AppKit thread
	SInt32 nbPreviewPoints = GetNbChannels()*GetPreviewNbBins();
	
	fMainPreview = new MEAudioClipDataPreviewPoint[nbPreviewPoints];
	
	ZzRABI_ArrayDeleter<MEAudioClipDataPreviewPoint> previewDeleter(&fMainPreview);
	
	fMainPreviewNbFrames = GetPreviewNbBins();
	
	SInt32 bytesToCopy = nbPreviewPoints*sizeof(MEAudioClipDataPreviewPoint);
	
	if(!data->GetMainPreview())
	{
		vernl(1,"ALERT : the audio clip data we want to copy doesn't have a valid preview !!! --> Computing one");
		data->ComputeMainPreview();
	}
	
	if(data->GetMainPreview())
	{
		memcpy(fMainPreview,data->GetMainPreview(),bytesToCopy);
	}
	else
	{
		// WE REALLY SHOULD NOT BE HERE. NO ASSERTION THOUGH BECAUSE I'M SUPPRESSING PREVIEW GEN TO DEBUG
		// SOME WEIRD HEAP CORRUPTION ISSUE. 2002-11-06
	}
	previewDeleter.SaveResource();// ctor succeeded. let's not delete the fPreview, please !
	rabiOwningClips.SaveResource(); // do NOT delete the owning clips array please !
}

bool MEAudioClipData::IsEmpty() const
{
	fDoc->LockRead();
	bool retVal = fPreTrimChunkList.IsEmpty();
	fDoc->UnlockRead();
	return retVal;
}

MEAudioClipData::MEAudioClipData(
	TArmeDoc* 					doc, 
	const MEAudioChunkList& 	list, 
	SInt32 						nbChannels, 
	MIProgressBarDlgManager* 	progressDlg) :
	
	fDoc(doc),
	fRefCnt(0),
	fFakeDuration(0)
{
	fNbChannels = nbChannels;
	SetNameCopy("untitled MEAudioClipData from chunk list");
	// the preview is NULL right now.

	fMainPreview = NULL;
	fMainPreviewNbFrames = 0;
	// let's initialize the owning clips
	fOwningClips = new ZzArchPtrArray;
	ZzRABI_ObjectDeleter<ZzArchPtrArray> rabiOwningClips(&fOwningClips);
	
	Retain(NULL); // a null owner means the inital self ownership
	AdoptNewChunkList(list);
	ComputeMainPreview(progressDlg);
	rabiOwningClips.SaveResource();
}

/*
MEAudioClipData::MEAudioClipData(
	TArmeDoc* 					doc, 
	const char* 				path, 
	MIProgressBarDlgManager*	progressDlg) :
	
	fDoc(doc),
	fRefCnt(0),
	fFakeDuration(0)
{
	fNbChannels = 0;
	SetNameCopy("untitled MEAudioClipData from file");
	// the preview is NULL right now.
	fMainPreview = NULL;
	fMainPreviewNbFrames = 0;
	heapwalker(fOwningClips = new ZzArchPtrArray);
	Retain(NULL); // a null owner means the inital self ownership

	// create a chunk that will represent the whole file
	MEAudioChunk onlyChunk;
	
	// $$$$$TODO implement according to ctor below
	ZzRtAssert(0); // when is it used ?
//	ComputeMainPreview(progressDlg);
}
*/

MEAudioClipData::MEAudioClipData(
	TArmeDoc* 					doc, 
	const FSSpec&				movieFile, 
	int 						trackIndex, 
	MIProgressBarDlgManager*	progressDlgManager) :

	fDoc(doc),
	fRefCnt(0),
	fFakeDuration(0)
{
	ZzRABI_SubTaskScope subTaskScope(progressDlgManager);
	progressDlgManager->SetNbSubTasks(2);
	
	SetNameCopy("untitled MEAudioClipData from movieFile and track index");
	fNbChannels = 0;

	// the preview is NULL right now.
	fMainPreview = NULL;
	fMainPreviewNbFrames = 0;
	heapwalker(fOwningClips = new ZzArchPtrArray);
	ZzRABI_ObjectDeleter<ZzArchPtrArray> rabiOwningClips(&fOwningClips);
	Retain(NULL); // a null owner means the inital self ownership

	MEAudioChunk newChunk = doc->AudioChunkFileManager()->AllocateNewZeroRefMTGAChunk();
	char newChunkPath[1024];
	newChunk.GetPath(newChunkPath,1024);
	
	progressDlgManager->EnterSubTask();
	ME_ConvertQTAudioTrackToMTGAFile(movieFile,trackIndex,ZzString(newChunkPath),doc->GetAudioFrameRate(),progressDlgManager);
	progressDlgManager->LeaveSubTask();
					
	// now the newChunkPath file does exist.
	MEMTGAAudioFile::GetFileInfo(newChunkPath,&fNbChannels,&(newChunk.duration));
	ZzRABI_RemoveFile mtgaFileRemover(newChunkPath); // we'll delete it when exiting scope unless everything succeeds
	
	//let's create a chunklist with the chunk
	MEAudioChunkList mainList(newChunk);
		
	// ok. this (re)sets the trim values and delete the old preview.
	AdoptNewChunkList(mainList);
	
	// this computes the preview.
	progressDlgManager->EnterSubTask();
	ComputeMainPreview(progressDlgManager);
	progressDlgManager->LeaveSubTask();

	rabiOwningClips.SaveResource(); // do NOT delete fOwningClips ! 
	mtgaFileRemover.SaveResource(); // do NOT delete the MTGA file !
}


MEAudioClipData::~MEAudioClipData()
{
	//printf("MEAudioClipData::~MEAudioClipData() deleting with : \n");
	//printf("fPreTrimChunkList.GetChunk(0).GetFileIndex().data = %d\n",fPreTrimChunkList.GetChunk(0).GetFileIndex().data);
	//printf("fTrimmedChunkList.GetChunk(0).GetFileIndex().data = %d\n",fTrimmedChunkList.GetChunk(0).GetFileIndex().data);
	ZzAssert(fRefCnt == 0);
	delete fOwningClips;
	delete [] fMainPreview;
}

void MEAudioClipData::AddStereoAudioData(float* buffer, ArmeTime start, ArmeTime length)
{
	ZzAssert(0);
}

void MEAudioClipData::ComputeMainPreview(MIProgressBarDlgManager* progressDlgManager)
{
	ZzSplitTask* splitTask;
	bool bDeleteProgressDlgManager = false;
	
	ComputeMainPreviewSplitTask(&splitTask);
	
	if(!progressDlgManager)
	{
		progressDlgManager = new MIProgressBarDlgManager("Computing preview...");
		// no rabi guard here since the ZzSplitTask::ExecuteInProgressManagerContext
		// will take ownership of the MIProgressBarDlgManager and will delete it anyway
		bDeleteProgressDlgManager = true;
	}
	
	try
	{
		ArmeErr err = ZzSplitTask::ExecuteInProgressManagerContext(splitTask,progressDlgManager);
		if(err.type != kArmeOK)
			ThrowOnArmeErr(err);
	}
	catch(...)
	{
		// now get rid of the progress manager if we own it
		if(bDeleteProgressDlgManager)
			delete progressDlgManager;
		throw;
	}
	
	if(bDeleteProgressDlgManager)
		delete progressDlgManager;
}

void MEAudioClipData::ComputeMainPreviewSplitTask(ZzSplitTask** splitTask)
{
//	*splitTask = NULL;
//	return; ////$$$DEBUG___TEST___DEBUG___TEST remove this line !!!!!!!!!!!!!
	ArmeErr err;
	heapwalker((*splitTask) = new MESplitTaskComputeAudioClipDataPreview(this,&err));
//	InvalidateMainPreview();
	ZzAssert(err.type == kArmeOK);
}

/*
MEAudioClipDataPreviewPoint* MEAudioClipData::ComputeMainPreview(
	const MEAudioChunkList&	chunkList, 
	ProgressTracker*	progressTracker)
{
	// we need as many sample points as there are samples / kMainPreviewBinSize
	// and this is stereo only
	SInt64 chunksSize = 0;
	SInt32 i;
	for(i = 0; i < nbChunks; i++)
	{
		chunksSize += chunks[i].length;
	}
				
	// NRDIV in frames (negative remainder division)
	SInt32 previewSize = (chunksSize-1)/kMainPreviewBinSize + 1; 
							
	// 2* because stereo
	MEAudioClipDataPreviewPoint* preview = new MEAudioClipDataPreviewPoint[2*previewSize]; 
	verntl(audioClipsBenjamin,5,"NEW : preview = " << preview);
	// the tmp buffer where we are going to read from the chunk set
	float* tmpBuffer = new float[kMainPreviewBinSize*2];
	verntl(audioClipsBenjamin,5,"NEW : tmpBuffer = " << tmpBuffer);
	
	// indices
	SInt64 curFrame = 0;
	SInt64 previewPoint = 0;

	while(curFrame < chunksSize)
	{
		if(progressTracker)
		{
			if(curFrame == 0)
				progressTracker->SetProgress(0.0);
			else
				progressTracker->SetProgress(1.0/(float)(chunksSize/curFrame));
		}

		ReadFloatFramesFromAChunkListToFlatBuffer(
			tmpBuffer, curFrame, kMainPreviewBinSize, chunks, nbChunks);
		curFrame += kMainPreviewBinSize;
		
		float Lmax = 0.0;
		float Rmax = 0.0;
		for(SInt32 i = 0; i < kMainPreviewBinSize; i++)
		{
			float Lval = tmpBuffer[(i << 1)];
			float Rval = tmpBuffer[(i << 1)+1];
			
			if(Lval > Lmax)
				Lmax = Lval;
			if(Rval > Rmax)
				Rmax = Rval;
		}
		preview[previewPoint].max = Lmax;
		previewPoint++;
		preview[previewPoint].max = Rmax;
		previewPoint++;
	}
	delete [] tmpBuffer;
	return preview;
};
*/

ArmeErr MEAudioClipData::ReadFloatFramesToFlatBuffer(
	float*					buffer, 
	SInt64 					start, 
	SInt64 					length,
	MECallingThreadType		callingThread)
{
	return ReadFloatFramesFromAChunkListToFlatBuffer(buffer, start, length, fTrimmedChunkList, callingThread);
}

ArmeErr MEAudioClipData::ReadFloatFramesFromAChunkListToFlatBuffer(
	float*					buffer, 
	SInt64 					start, 
	SInt64 					length, 
	const MEAudioChunkList&	chunkList,
	MECallingThreadType		callingThread)
{
	ZzAssert(callingThread == kCallingThread_AppKit || callingThread == kCallingThread_OfflineAudioSplitTaskThread || callingThread == kCallingThread_OfflineRenderer);
	ArmeErr err;
	// now we need to split this request among
	// the chunks that make up the clip.
	// and send zeroes outside of the clip
	SInt32 i;
	SInt32 nbChannels = fNbChannels;
	
	// if the range is totally before the clip, let's put zeroes
	if(start < 0 && start+length < 0)
	{
		memset(buffer,0,length*sizeof(float)*nbChannels);
		return err;
	}
	// if part of the range is before the clip, let's also put zero in there 
	if(start < 0 )
	{
		ZzAssert(-start <= length);
		memset(buffer,0,(-start)*sizeof(float)*nbChannels);
		buffer += (-start)*nbChannels;
		length += start;
		start = 0;
	}
	// maybe we are already finished ?
	if(length <= 0)
		return err;
	
	SInt64 currentChunkStartInClip = 0;
	SInt64 currentChunkEndInClip = 0;
	
	for( i = 0; i < chunkList.GetNbChunks(); i++ )
	{
		const MEAudioChunk& chunk = chunkList.GetChunk(i);
		currentChunkEndInClip += chunk.duration;
		SInt64 startInChunkSpace = start - currentChunkStartInClip;
		if( startInChunkSpace >= 0 && (start+length) < currentChunkEndInClip)
		{
			// the range is totally contained in this chunk->..
			// very easy. don't forget that we read in file space
			
			// check if the tmp file for this thread is opened for the right chunk
			err = OpenTmpFileForThreadForChunk(callingThread,chunk);
			if(err.type != kArmeOK)
				return err;
			
			if(length > 0)
				err = fTmpReadFiles[callingThread].ReadFloat32ContentToFlatBuffer(buffer, chunk.startInFile + startInChunkSpace, length);
			return err;
		}
		else if((startInChunkSpace >= 0) && (startInChunkSpace < chunk.duration)  && ((start+length) >= currentChunkEndInClip))
		{
			// the range starts _in this chunk_ but ends in another chunk later
			
			// the length of the range that intersects this chunk
			// which is the total length of the chunk - the space
			// in the chunk before the start of the range
			SInt64 lengthToCopy = chunk.duration - startInChunkSpace;
			
			// we read from the start of the range to the end of the chunk
			// but we work in file space
			// thus, start of the range in file space is the start of
			// the range in chunk space + the start of the chunk in file space
			// = chunk->startInFile+startInChunkSpace
			
			// check if the tmp file for this thread is opened for the right chunk
			err = OpenTmpFileForThreadForChunk(callingThread,chunk);
			if(err.type != kArmeOK)
				return err;
			if(lengthToCopy > 0)
				err = fTmpReadFiles[callingThread].ReadFloat32ContentToFlatBuffer(buffer, chunk.startInFile + startInChunkSpace,lengthToCopy);
			if(err.type != kArmeOK)
			{
				fTmpReadFiles[callingThread].Close();
				return err;
			}
			// now we shorten the given range by making its new start the
			// end of current chunk (or start of next chunk which is the same,
			// because we always work with ends exclusive)
			start = currentChunkEndInClip; // which is the startInClip of next chunk
			
			// we of course need to shorten the length accordingly
			length -= lengthToCopy;
			
			// and incr the write pos in the buffer
			buffer += lengthToCopy*nbChannels;
		}
		// next chunk start is equal to this chunk start + its length
		currentChunkStartInClip += chunk.duration;
	}
	if(length > 0)
	{
		// this means we are still requested some length after the end of the chunks... let's put 0 in there
		verntl(audioClipsBenjamin,5,"length = " << (SInt32)length << "after end. putting zero at " << buffer);
		memset(buffer,0,length*sizeof(float)*nbChannels);
		buffer += length*nbChannels;
		length = 0;
	}	
	return err;
}

void MEAudioClipData::NotifyDataHasChanged()
{
//	GetDoc()->LockRead();
	for(SInt32 i = 0; i < GetNbOwningClips(); i++)
	{
		MEAudioClip* clip = GetOwningClip(i);
		if(clip)
		{
			GetDoc()->NotifyAudioClipDataHasChanged(clip);
		}
	}
//	GetDoc()->UnlockRead();
}

void MEAudioClipData::FillPreviewPointArray(
	MEAudioClipDataPreviewPoint*	previewPoints,
	SInt32							bufMaxSize,
	SInt32							finalRatio,
	SInt32							nbBins,
	SInt32							reqStartTimeBinIndex,
	SInt32							reqEndTimeBinIndex,
	MECallingThreadType				callingThread)
{
	ZzAssert(fMainPreview);
	if(fMainPreviewNbFrames != GetPreviewNbBins())
	{
		//printf("recomputing preview because number of bins do not match\n");
		ComputeMainPreview();
	}
	ZzAssert(fMainPreviewNbFrames == GetPreviewNbBins())

//	ZzAssert(fNbChannels == 1 ||fNbChannels == 2);
	// finalRatio is always a power of two (2^k) or a sum of 2 successive
    // powers of two (2^k + 2^(k-1))
//	memset(previewPoints
	
	if(finalRatio >= 2 * kMainPreviewBinSize)
	{
        //USE MAIN PREVIEW
    
		// the number of user bins 
		SInt32 previewBinsPerMainBin = finalRatio/kMainPreviewBinSize;

        // if this one gives problems, just comment it out and let me know
        // it really shouldnt happen, but its not fatal. i still suspect there is a roundoff bug in there [ts] 
		if((finalRatio % kMainPreviewBinSize) != 0)
		{
			//printf("INFO : finalRatio modulo kMainPreviewBinSize != 0\n");
		//	vernl(0,"****** WARNING !!!! ******* finalRatio % kMainPreviewBinSize) != 0 in file " << __FILE__ << " at line " << __LINE__);
		}
    
		for(SInt32 reqBin = reqStartTimeBinIndex; reqBin <= reqEndTimeBinIndex; reqBin++)
		{
			SInt32 firstMainBin = reqBin*previewBinsPerMainBin;
			SInt32 lastMainBin = firstMainBin+previewBinsPerMainBin-1;
			if(lastMainBin >= GetPreviewNbBins())
				lastMainBin = GetPreviewNbBins()-1;
			ZzAssert((lastMainBin-firstMainBin+1) <= previewBinsPerMainBin);
			ZzAssert(fNbChannels >= 1 && fNbChannels <= 2);
			//ZzAssert(firstMainBin < GetPreviewNbBins());
			if(firstMainBin >= GetPreviewNbBins())
				firstMainBin = GetPreviewNbBins()-1;
			MEAudioClipDataPreviewPoint* mainPreviewBins = &(fMainPreview[fNbChannels*firstMainBin]);
            
            float clipmax = 0.99999;
			float clipmin = -0.99999;
			
			if(IsStereo())
            {
                float maxL = mainPreviewBins->max;
                float minL = mainPreviewBins->min;
                mainPreviewBins++;
				float maxR = mainPreviewBins->max;
				float minR = mainPreviewBins->min;
				mainPreviewBins++;


                for(SInt32 mainBin = firstMainBin+1; mainBin <= lastMainBin; mainBin++)
                {
                    if(mainPreviewBins->max > maxL)	maxL = mainPreviewBins->max;
                    if(mainPreviewBins->min < minL)	minL = mainPreviewBins->min;
                    mainPreviewBins++;

					if(mainPreviewBins->max > maxR)	maxR = mainPreviewBins->max;
					if(mainPreviewBins->max < minR)	minR = mainPreviewBins->min;
					mainPreviewBins++;
                }
                
                maxL = CLIP(clipmin, maxL, clipmax);
                minL = CLIP(clipmin, minL, clipmax);
                maxR = CLIP(clipmin, maxR, clipmax);
                minR = CLIP(clipmin, minR, clipmax);
                
                // the array starts at bin reqStartTimeBinIndex
                SInt32 indexL = 2*(reqBin-reqStartTimeBinIndex);
                ZzAssert(indexL < bufMaxSize);
                previewPoints[indexL].max = maxL;
                previewPoints[indexL].min = minL;

				SInt32 indexR = indexL+1;
				ZzAssert(indexR < bufMaxSize);
				previewPoints[indexR].max = maxR;
				previewPoints[indexR].min = minR;
   			}
			else //mono
            {
                float max = mainPreviewBins->max;
                float min = mainPreviewBins->min;
                mainPreviewBins++;


                for(SInt32 mainBin = firstMainBin+1; mainBin <= lastMainBin; mainBin++)
                {
                    if(mainPreviewBins->max > max)	max = mainPreviewBins->max;
                    if(mainPreviewBins->min < min)	min = mainPreviewBins->min;
                    mainPreviewBins++;

                }
                
                max = CLIP(clipmin, max, clipmax);
                min = CLIP(clipmin, min, clipmax);
                
                // the array starts at bin reqStartTimeBinIndex
                SInt32 index = (reqBin-reqStartTimeBinIndex);
                ZzAssert(index < bufMaxSize);
                previewPoints[index].max = max;
                previewPoints[index].min = min;

   			}

		}
	}
	else //DONT USE MAINPREVIEW
	{
		SInt64 curBinStartTime = reqStartTimeBinIndex*finalRatio;
		float* tmpBuffer;
		heapwalker(tmpBuffer = new float[finalRatio*2]);
		SInt32 previewPoint = 0;

		for(SInt32 reqBin = reqStartTimeBinIndex; reqBin <= reqEndTimeBinIndex; reqBin++)
		{
			SInt64 curBinEndxTime = curBinStartTime + finalRatio;
			// check if this last bin is maybe shorter than finalRatio frames
			if(curBinEndxTime>=GetDuration())
			{
				// make sure it's the last bin
				//ZzAssert(reqBin == reqEndTimeBinIndex); //$$$
				curBinEndxTime = GetDuration();
			}
			// BEWARE ! this function is called in the main event loop
			// we call LockRead on the document because we read the chunk list
			// for this clip and we don't want it to be modified while
			// we build the image. 
			ReadFloatFramesFromAChunkListToFlatBuffer(
				tmpBuffer, curBinStartTime, curBinEndxTime-curBinStartTime, fTrimmedChunkList, callingThread);
	
            float clipmax = 0.99999;
            float clipmin = -0.99999;
    
 			if(IsStereo())
			{
                float Lmin, Rmin, Lmax, Rmax;
                Lmin = Lmax = tmpBuffer[0];
				Rmin = Rmax = tmpBuffer[1];
                for(SInt32 i = 1; i < (curBinEndxTime-curBinStartTime); i++)
                {
                    float Lval = tmpBuffer[(i << 1)];
                    float Rval = tmpBuffer[(i << 1)+1];
                    if(Lval > Lmax) Lmax = Lval;
                    if(Lval < Lmin) Lmin = Lval;
                    if(Rval > Rmax)	Rmax = Rval;
                    if(Rval < Rmin)	Rmin = Rval;
                }
                Lmin = CLIP(clipmin, Lmin, clipmax);
                Lmax = CLIP(clipmin, Lmax, clipmax);
                Rmin = CLIP(clipmin, Rmin, clipmax);
                Rmax = CLIP(clipmin, Rmax, clipmax);

                previewPoints[previewPoint].max = Lmax;
                previewPoints[previewPoint].min = Lmin;
                previewPoint++;
				previewPoints[previewPoint].max = Rmax;
				previewPoints[previewPoint].min = Rmin;
				previewPoint++;
                
			}

            else //mono
            {
                float min, max;
                min = max = tmpBuffer[0];
                for(SInt32 i = 1; i < (curBinEndxTime-curBinStartTime); i++)
                {
                    float val = tmpBuffer[i];
                    if(val > max) max = val;
                    if(val < min) min = val;
                }
                min = CLIP(clipmin, min, clipmax);
                max = CLIP(clipmin, max, clipmax);

                previewPoints[previewPoint].max = max;
                previewPoints[previewPoint].min = min;
                previewPoint++;

            }
    
			curBinStartTime += finalRatio;
		}
		ZzAssert(previewPoint == nbBins*fNbChannels);
		delete [] tmpBuffer;
	}
}
				
SInt32 MEAudioClipData::GetPreviewNbBins() const
{
	// that's the negative remainder division of the total number of
	// frames by the preview bin size
	return (GetDuration()-1)/kMainPreviewBinSize + 1;
}

MEAudioClipDataMemento* MEAudioClipData::CreateMemento()
{
	MEAudioClipDataMemento* ptr;
	fDoc->LockRead();
	heapwalker(ptr = new MEAudioClipDataMemento(this));
	fDoc->UnlockRead();
	return ptr;
}

void MEAudioClipData::SetMemento(MEAudioClipDataMemento* memento, ZzSplitTask** splitTask)
{
	fDoc->LockWrite();
	fPreTrimChunkList = memento->fPreTrimChunkList;
	fStartTrim = memento->fStartTrim;
	fEndTrim = memento->fEndTrim;
	RecomputeTrimmedChunkList();
	fDoc->UnlockWrite();
	ComputeMainPreviewSplitTask(splitTask);
}

void MEAudioClipData::SetMementoWithoutPreviewComputing(MEAudioClipDataMemento* memento)
{
	fDoc->LockWrite();
	fPreTrimChunkList = memento->fPreTrimChunkList;
	fStartTrim = memento->fStartTrim;
	fEndTrim = memento->fEndTrim;
	RecomputeTrimmedChunkList();
	fDoc->UnlockWrite();
}

/*
void MEAudioClipData::ReplaceTimeRangeWithChunk(
	const char* 		chunkLocalName,
	SInt64				startTimeInClip,
	SInt64				oldSelDuration,
	SInt64				newChunkDuration,
	ProgressTracker*	progressTracker)
{
	ZzAssert(0);
}
*/

MEAudioClipDataMemento::MEAudioClipDataMemento(ArmeArchiveHandler* unarchiver) :
	ArmeArchivable(unarchiver)
{
	//printf("MEAudioClipDataMemento::MEAudioClipDataMemento()\n");
	fPreTrimChunkList.ReadFromArchive(unarchiver);
	fStartTrim  = unarchiver->ReadArmeTime();
	fEndTrim  = unarchiver->ReadArmeTime();
}

bool MEAudioClipDataMemento::WriteToArchive(ArmeArchiveHandler* archiver) const
{
	ArmeArchivable::WriteToArchive(archiver);
	fPreTrimChunkList.WriteToArchive(archiver);
	archiver->WriteArmeTime(fStartTrim);
	archiver->WriteArmeTime(fEndTrim);
	return true;
}

MEAudioClipDataMemento::MEAudioClipDataMemento(MEAudioClipData* originator)
{
	//printf("MEAudioClipDataMemento::MEAudioClipDataMemento()\n");
	fPreTrimChunkList = originator->fPreTrimChunkList;
	fStartTrim = originator->fStartTrim;
	fEndTrim = originator->fEndTrim;
}

MEAudioClipDataMemento::~MEAudioClipDataMemento()
{
	//printf("MEAudioClipDataMemento::~MEAudioClipDataMemento() deleting with fPreTrimChunkList.GetChunk(0).GetFileIndex().data = %d\n",fPreTrimChunkList.GetChunk(0).GetFileIndex().data);
}

MEAudioClipData::MEAudioClipData(ArmeArchiveHandler* unarchiver) :
	ArmeArchivable(unarchiver),
	fRefCnt(0)
{
	fOwningClips = new ZzArchPtrArray;
	Retain(NULL); // a null owner means the inital self ownership
	fNbChannels = unarchiver->ReadSInt32();
	unarchiver->ReadRawData(fName,64);
	ArmeArchivable* archObj;
	fPreTrimChunkList.ReadFromArchive(unarchiver);
	fStartTrim = unarchiver->ReadArmeTime();
	fEndTrim = unarchiver->ReadArmeTime();
	RecomputeTrimmedChunkList();
		
	archObj = unarchiver->ReadCPPObject();
	fDoc = dynamic_cast<TArmeDoc*>(archObj);
	if(archObj)
	{
		ZzAssert(fDoc);
	}
	fFakeDuration = unarchiver->ReadSInt64();
	fMainPreview = NULL; // let's compute the main preview later
	fMainPreviewNbFrames = 0;
}

bool MEAudioClipData::WriteToArchive(ArmeArchiveHandler* archiver) const 
{
	ArmeArchivable::WriteToArchive(archiver);
	archiver->WriteSInt32(fNbChannels);
	archiver->WriteRawData(fName,64);
	fPreTrimChunkList.WriteToArchive(archiver);
	archiver->WriteArmeTime(fStartTrim);
	archiver->WriteArmeTime(fEndTrim);
	archiver->WriteCPPObject(fDoc);
	archiver->WriteSInt64(fFakeDuration);
	return true;
}

MEAudioClip* MEAudioClipData::GetOwningClip(SInt32 i)
{
	return dynamic_cast<MEAudioClip*>(fOwningClips->ItemAt(i));
}

void MEAudioClipData::Retain(MEAudioClip* owner)
{
	fOwningClips->AddItem(dynamic_cast<ArmeArchivable*>(owner));
	fRefCnt++;
}

void MEAudioClipData::Release(MEAudioClip* owner)
{
	UInt32 count1 = GetNbOwningClips();
	fOwningClips->RemoveItemByValue(dynamic_cast<ArmeArchivable*>(owner));
	UInt32 count2 = GetNbOwningClips();
	ZzAssert(count2 == (count1-1));
	if(fRefCnt == 0)
	{
		verntl(audioClipsBenjamin,0,"CRASH : fRefCnt == 0 in MEAudioClipData::Release() which means an object has been created without Retain() being called");
		ZzAssert(0);
	}
	else
	{
		fRefCnt--;
	}
//	ZzAssert(count2 == fRefCnt);
	if(fRefCnt == 0)
		delete this;
}

void MEAudioClipData::InsertChunk(const MEAudioChunk& chunk, SInt64 pos, ZzSplitTask** splitTask)
{
	fDoc->LockWrite();
	fPreTrimChunkList.InsertChunk(chunk,pos+GET_VAL(fStartTrim));
	RecomputeTrimmedChunkList();
	fDoc->UnlockWrite();
	ComputeMainPreviewSplitTask(splitTask);
}

void MEAudioClipData::SetStartTrim(ArmeTime startTrim, bool recomputePreview)
{
	vernl(6,"BEFORE SetStartTrim(" << startTrim << ") fPreTrimChunkList = " << fPreTrimChunkList << " and fTrimmedChunkList = " << fTrimmedChunkList);
	fDoc->LockWrite();
	fStartTrim = startTrim;
	RecomputeTrimmedChunkList();
	fDoc->UnlockWrite();
	if(recomputePreview)
		ComputeMainPreview();
	vernl(6,"AFTER SetStartTrim(" << startTrim << ") fPreTrimChunkList = " << fPreTrimChunkList << " and fTrimmedChunkList = " << fTrimmedChunkList);
	ZzAssert(GET_VAL(fStartTrim) + GET_VAL(fEndTrim) < fPreTrimChunkList.GetDuration());
}

void MEAudioClipData::SetEndTrim(ArmeTime endTrim, bool recomputePreview)
{
	vernl(6,"BEFORE SetEndTrim(" << endTrim << ") fPreTrimChunkList = " << fPreTrimChunkList << " and fTrimmedChunkList = " << fTrimmedChunkList);
	fDoc->LockWrite();
	fEndTrim = endTrim;
	RecomputeTrimmedChunkList();
	fDoc->UnlockWrite();
	if(recomputePreview)
		ComputeMainPreview();
	vernl(6,"AFTER SetEndTrim(" << endTrim << ") fPreTrimChunkList = " << fPreTrimChunkList << " and fTrimmedChunkList = " << fTrimmedChunkList);
	ZzAssert(GET_VAL(fStartTrim) + GET_VAL(fEndTrim) < fPreTrimChunkList.GetDuration());
}

void MEAudioClipData::InsertChunkList(const MEAudioChunkList& chunkList, SInt64 pos, ZzSplitTask** splitTask)
{
	fDoc->LockWrite();
	fPreTrimChunkList.InsertChunkList(chunkList, pos+GET_VAL(fStartTrim));
	RecomputeTrimmedChunkList();
	fDoc->UnlockWrite();
	ComputeMainPreviewSplitTask(splitTask);
}

MEAudioChunkList MEAudioClipData::ExtractSlice(SInt64 sliceStart, SInt64 len) const
{
	fDoc->LockRead();
	MEAudioChunkList extractedChunkList = fTrimmedChunkList.ExtractSlice(sliceStart,len);
	fDoc->UnlockRead();
	return extractedChunkList;
}

void MEAudioClipData::ReplaceRangeWithChunk(const MEAudioChunk& chunk, SInt64 pos, SInt64 len, ZzSplitTask** splitTask)
{
	fDoc->LockWrite();
	fPreTrimChunkList.ReplaceRangeWithChunk(chunk, pos+GET_VAL(fStartTrim), len);
	RecomputeTrimmedChunkList();
	fDoc->UnlockWrite();
	ComputeMainPreviewSplitTask(splitTask);
}

bool MEAudioClipData::DeleteSlice(SInt64 sliceStart, SInt64 length, ZzSplitTask** splitTask)
{
	// special behaviour here
	bool ret;
	
	ZzAssert(length >= 0);
	
	fDoc->LockWrite();
	fPreTrimChunkList.DeleteSlice(sliceStart+GET_VAL(fStartTrim), length);
	// if the whole TRIMMED DURATION has been deleted, then we decide
	// to return "true", which means the whole stuff has been deleted
	if(sliceStart == 0 && length == fTrimmedChunkList.GetDuration())
	{
		fPreTrimChunkList.Clear();
		fTrimmedChunkList.Clear();
		ret = true;
	}
	else
	{
		RecomputeTrimmedChunkList();
		ret = false;
	}
	fDoc->UnlockWrite();
	ComputeMainPreviewSplitTask(splitTask);
	return ret;
}

void MEAudioClipData::AdoptMainPreview(MEAudioClipDataPreviewPoint* mainPreview, SInt32 mainPreviewNbFrames)
{
	if(fMainPreview)
	{
		delete [] fMainPreview;
	}
	fMainPreview = mainPreview;
	fMainPreviewNbFrames = mainPreviewNbFrames;
}

void MEAudioClipData::InvalidateMainPreview()
{
	if(fMainPreview)
	{
		vernl(6,"WARNING : MEAudioClipData::InvalidateMainPreview() has been called while fMainPreview != NULL");
		delete [] fMainPreview;
		fMainPreview = NULL;
		fMainPreviewNbFrames = 0;
	}
}

void MEAudioClipData::SetNameCopy(const char* name)
{
	strncpy(fName,name,63);
	fName[63] = 0;
}

const char* MEAudioClipData::GetName()
{
	return fName;
}

ArmeErr MEAudioClipData::OpenTmpFileForThreadForChunk(MECallingThreadType callingThread, const MEAudioChunk& chunk)
{
	bool b1  = fTmpReadFiles[callingThread].GetOpenMode() == MECompoundAudioFile::kNotOpen;
	bool b2 = fTmpReadFiles[callingThread].GetOpenMode() == MECompoundAudioFile::kOpenReadChunk;
	ZzAssert((b1 || b2));
	if(fTmpReadFiles[callingThread].GetOpenMode() == MECompoundAudioFile::kNotOpen || fTmpReadFiles[callingThread].GetOpenChunk() != chunk)
	{
		fTmpReadFiles[callingThread].Close();
		ArmeErr err = fTmpReadFiles[callingThread].OpenReadChunk(GetDoc(),chunk);
		if(err.type != kArmeOK)
		{
			fTmpReadFiles[callingThread].Close();
			return err;
		}
	}
	return ArmeErr(kArmeOK);
}

void MEAudioClipData::CloseTmpFile(MECallingThreadType callingThread)
{
	fTmpReadFiles[callingThread].Close();
}

