/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#define MEAudioClipData_h

#ifndef MEAudioClipData_h
#include "MEAudioClipData.h"
#endif //MEAudioClipData_h

#ifndef MEQTFloatSampleReader_h
#include "MEQTFloatSampleReader.h"
#endif //MEQTFloatSampleReader_h

#ifndef ArmeArchivable_h
#include "ArmeArchivable.h"
#endif //ArmeArchivable_h

#ifndef ZzArchPtrArray_h
#include "ZzArchPtrArray.h"
#endif //ZzArchPtrArray_h

#ifndef MEAudioChunkList_h
#include "MEAudioChunkList.h"
#endif //MEAudioChunkList_h

#ifndef MECompoundAudioFile_h
#include "MECompoundAudioFile.h"
#endif //MECompoundAudioFile_h

//class MECompoundAudioFile;
class ProgressTracker;

// the number of frames in the preview bin
#define kMainPreviewBinSize 256

class TArmeDoc;
class MEAudioClip;
class TArmeWSObj;

struct MEAudioClipDataPreviewPoint
{
	float max;
    float	min;
};

class MEAudioClipDataMemento;
class ZzSplitTask;
struct MEAudioChunk;

class MEAudioClipData : public ArmeArchivable
{
	friend class MEAudioClipDataMemento;
public :
	/*! function MEAudioClipData (1st ctor) This ctor is meant to be used when a new MEAudioClipData 
		instance needs to be created from an existing file. An import ZzSplitTask will be created and the
		doc will be asked to perform this task. */
					MEAudioClipData(
						TArmeDoc* 					doc, 
						const char*					path, 
						MIProgressBarDlgManager* 	progressDlgManager);

	/*! function MEAudioClipData (2nd ctor) This ctor is meant to be used when a new MEAudioClipData 
		instance needs to be created from a QT track from an existing movie file. An import ZzSplitTask 
		will be created and the doc will be asked to perform the task. */
					MEAudioClipData(
						TArmeDoc* 					doc, 
						const FSSpec&				movieFile, 
						int 						trackIndex, 
						MIProgressBarDlgManager* 	progressDlgManager);
        
					MEAudioClipData(ArmeArchiveHandler* unarchiver);
					
	/*! @function MEAudioClipData This constructor creates an empty clip data. You need to fill the chunks on your own */
					MEAudioClipData(TArmeDoc* doc);

	/*! @function MEAudioClipData This constructor takes a chunk list as params and copies it */
					MEAudioClipData(
						TArmeDoc* 					doc, 
						const MEAudioChunkList& 	list, 
						SInt32 						nbChannels, 
						MIProgressBarDlgManager* 	progressDlgManager);

	/*! @function MEAudioClipData This constructor copies an existing data into this one  */
					MEAudioClipData(MEAudioClipData* data);
					
	/*! @function SetNameCopy Sets the name of the audio data by copying at most 63 chars from the arg */
	void			SetNameCopy(const char* name);
	
	/*! @function GetName */
	const char*		GetName();

	/*! @function Retain incs the ref count */
	void					Retain(MEAudioClip* owner);

	/*! @function Release decs the ref count and deletes this if needed */
	void					Release(MEAudioClip* owner);

	virtual					~MEAudioClipData();
	
	/*! @function AdoptNewChunkList This function adopts an existing chunk list. The current one is totally
	              discarded */
	void 					AdoptNewChunkList(const MEAudioChunkList& chunkList);
	
	void					InsertChunk(const MEAudioChunk& chunk, SInt64 pos, ZzSplitTask** splitTask);
	
	bool 					IsEmpty() const;

	/*!
		@function InsertChunk Inserts a chunk list at the given position in the chunk list.
		it's up to the CALLER to recompute the preview and tell the GUI to 
	
	  */
	void					InsertChunkList(const MEAudioChunkList& chunkList, SInt64 pos, ZzSplitTask** splitTask);
	
	/*! @function ExtractSlide Returns a chunk list made with a subrange of this list*/
	MEAudioChunkList		ExtractSlice(SInt64 sliceStart, SInt64 len) const;

	/*! @function ReplaceRangeWithChunk Replaces a range of the list with another chunk.*/
	void					ReplaceRangeWithChunk(const MEAudioChunk& chunk, SInt64 pos, SInt64 len, ZzSplitTask** splitTask);
																																
	/*! @function DeleteSlice Removes a slice outta the chunk list. If the slice is greater than- or
	              equal to the whole list, then the function returns true */
	bool					DeleteSlice(SInt64 sliceStart, SInt64 length, ZzSplitTask** splitTask);
	
	const MEAudioChunkList&	GetChunkList() const;
	
	/*! @function SetNewDocForClipboardPaste This function sets the new doc for the data and copies chunk files in the new doc if needed. Both doc 
		          need to be open, of course. $$$find a way to cancel old doc if it has been deleted */
	void			SetNewDocForClipboardPaste(TArmeDoc* doc);
						
	virtual bool	WriteToArchive(ArmeArchiveHandler* archiver) const;

	/*! function GetMemento Retrieves a memento for the current state of the object (see the memento pattern in [Gamma84])*/
	MEAudioClipDataMemento*		CreateMemento();

	/*! function SetMemento Sets a memento to restore object data (used for undoing user actions) */
	void						SetMemento(MEAudioClipDataMemento* memento, ZzSplitTask** splitTask);

	/*! function SetMementoWithoutPreviewComputing Useful when an exception occured after having modified an object
	    but BEFORE having done anything else */

	void 						SetMementoWithoutPreviewComputing(MEAudioClipDataMemento* memento);
	
	void						ReplaceTimeRangeWithFile(
									const char* 		chunkLocalName,
									SInt64				startTimeInClip,
									SInt64				oldSelDuration,
									SInt64				newChunkDuration);
			
	/*! @function NotifyDataHasChanged Warns the owning clip the data and preview have changed. DO NOT call this function
		before the preview is computed. */
	void 			NotifyDataHasChanged();

	/*! @function GetNbOwningClips Returns the number of owning clips */
	SInt32			GetNbOwningClips() { return fOwningClips->CountItems(); }

	/*! @function GetOwningClip Returns the ith clip that owns this audio data */
	MEAudioClip*	GetOwningClip(SInt32 i);
	
	/*! @function AddStereoAudioData Mixes stereo data read from this object to an existing stereo buffer */ 
	void			AddStereoAudioData(float* buffer, ArmeTime start, ArmeTime length);
	
	/*! 
		@function ReadFloatFramesFromAChunkListToFlatBuffer
		
		This function takes as input a list of chunks, a write-enabled buffer, a start time 
		and a length, and fills the given buffer with the data contained in this chunk list
		from start to start+length.
		
		This function doesn't operate on the real clip data (it is not static because it must
		use the GetDoc() function to retrieve the doc project path).
		
		We use it in the preview computation : instead of locking the doc for a long time to
		compute the preview from the chunk list, we simply copy the chunk list and create another
		preview in a separate thread. Then, when we're done, we simply lock the doc, switch the
		pointers to the new preview, delete the old one, and then unlock the doc
		
		See the end of the BOPImportAudioFileInClipData::Perform()
		
		********************** This function is NOT used for playback (AND SHOULD NEVER BE !) *****************************
		********************** This function is NOT used for playback (AND SHOULD NEVER BE !) *****************************
		********************** This function is NOT used for playback (AND SHOULD NEVER BE !) *****************************
		********************** This function is NOT used for playback (AND SHOULD NEVER BE !) *****************************
		********************** This function is NOT used for playback (AND SHOULD NEVER BE !) *****************************
		********************** This function is NOT used for playback (AND SHOULD NEVER BE !) *****************************
		********************** This function is NOT used for playback (AND SHOULD NEVER BE !) *****************************
		********************** This function is NOT used for playback (AND SHOULD NEVER BE !) *****************************
		********************** This function is NOT used for playback (AND SHOULD NEVER BE !) *****************************
		********************** This function is NOT used for playback (AND SHOULD NEVER BE !) *****************************
		********************** This function is NOT used for playback (AND SHOULD NEVER BE !) *****************************
		********************** This function is NOT used for playback (AND SHOULD NEVER BE !) *****************************
	*/
	ArmeErr							ReadFloatFramesFromAChunkListToFlatBuffer(
										float*					buffer, 
										SInt64 					start, 
										SInt64 					length, 
										const MEAudioChunkList&	chunkList,
										MECallingThreadType		callingThread);

	ArmeErr							ReadFloatFramesToFlatBuffer(
										float*					buffer, 
										SInt64 					start, 
										SInt64 					length,
										MECallingThreadType		callingThread);

	// useful when opening lots of file for preview computing in RecreateNonArchivedData , for example
	void							CloseTmpFile(MECallingThreadType callingThread);

	SInt64							GetDuration() const;
	SInt32							GetPreviewNbBins() const;
	TArmeDoc*						GetDoc() { return fDoc; };
	const TArmeDoc*					GetDoc() const { return fDoc; };
	
	/*! @function ComputeMainPreviewSplitTask this function "returns" a split task that will 
		compute the audio data preview */
	void						 	ComputeMainPreviewSplitTask(ZzSplitTask** splitTask);
	
	/*! @function ComputeMainPreview This function computes the preview, either by creating its own 
		progress dlg manager (if the supplied param is null) or by using the one supplied */
	void 							ComputeMainPreview(MIProgressBarDlgManager* progressDlgManager = NULL);
	
	MEAudioClipDataPreviewPoint* 	ComputeMainPreview(
										MEAudioChunk* chunks, 
										SInt32 nbChunks, 
										ZzSplitTask** splitTask);
									
	// this function closes the tmp files used for reading in this audio data.
	// you need to call this after a set of operations.
	void							CloseTmpReadFiles(MECallingThreadType callingThread);

	void 							SetFakeDuration(SInt64 fakeDuration);
		
	/*! @function FillPreviewPointArray This function uses the in-memory preview OR the chunk files
	              to compute an averaged preview buffer */
	void 							FillPreviewPointArray(
										MEAudioClipDataPreviewPoint*	previewPoints,
										SInt32							bufMaxSize,
										SInt32							finalRatio,
										SInt32							nbBins,
										SInt32							reqStartTimeBinIndex,
										SInt32							reqEndTimeBinIndex,
										MECallingThreadType				callingThread);

	/*! @function GetMainPreview Returns the main in-memory preview */
	MEAudioClipDataPreviewPoint*	GetMainPreview() { return fMainPreview; }
	void							AdoptMainPreview(MEAudioClipDataPreviewPoint* mainPreview, SInt32 mainPreviewNbFrames);
	void							InvalidateMainPreview();
	bool							IsStereo() const;
	void							SetNbChannels(SInt32 nbChannels);
	SInt32							GetNbChannels() const;
	void							SetStartTrim(ArmeTime startTrim, bool recomputePreview = true);
	void							SetEndTrim(ArmeTime endTrim, bool recomputePreview = true);
	ArmeTime 						GetEndTrim() const;
	ArmeTime 						GetStartTrim() const;
	void 							RecomputeTrimmedChunkList(); // this recomputes the trimmed list from the original list
	const MEAudioChunkList& 		GetPreTrimChunkList();
private :
	ArmeErr							OpenTmpFileForThreadForChunk(
										MECallingThreadType callingThread, 
										const MEAudioChunk& chunk);

private :
	ZzArchPtrArray*					fOwningClips;
	MECompoundAudioFile				fTmpReadFiles[kCallingThread_Count];
	SInt32							fNbChannels;
	char							fName[64];
	TArmeDoc*						fDoc;
	UInt32							fRefCnt;
	// list of chunks
	MEAudioChunkList				fTrimmedChunkList;
	MEAudioChunkList				fPreTrimChunkList;
	ArmeTime						fStartTrim;
	ArmeTime						fEndTrim;
		
	/*! 
		@var fMainPreview 
		This array contains NRDIV(GetDuration(),kMainPreviewBinSize) preview sample points.
		Those points are computed using the min, max and sum of squares values over a kMainPreviewBinSize-frames
		range $$$TODO define synchronisation.
	*/
	MEAudioClipDataPreviewPoint*	fMainPreview;
	
	/*! @var fMainPreviewNbFrames DEBUG DEBUG should always be equal to this->GetPreviewNbBins() */
	SInt32							fMainPreviewNbFrames;

	/*! @var fFakeDuration When the audio data is created, but before the audio file has been imported, the audio
	data reports fFakeDuration as a value. it is up to the importer to set this to zero when setting the real value */
	SInt64							fFakeDuration;
};

/*!
	@class MEAudioClipDataMemento Memento (see [Gamma84]) for MEAudioClipData
*/

class MEAudioClipDataMemento : public ArmeArchivable
{
	friend class MEAudioClipData;
public :
							MEAudioClipDataMemento(MEAudioClipData* originator);
							MEAudioClipDataMemento(ArmeArchiveHandler* unarchiver);
	virtual bool			WriteToArchive(ArmeArchiveHandler* archiver) const;
	virtual					~MEAudioClipDataMemento();
//	const MEAudioChunkList&	GetChunkList() const;
public :
	MEAudioChunkList		fPreTrimChunkList;
	ArmeTime				fStartTrim;
	ArmeTime				fEndTrim;
};

inline SInt64 MEAudioClipData::GetDuration() const
{
	if(fFakeDuration)
		return fFakeDuration;
	return fTrimmedChunkList.GetDuration();
}

inline void MEAudioClipData::SetFakeDuration(SInt64 fakeDuration)
{
	fFakeDuration = fakeDuration;
}

//inline const MEAudioChunkList& MEAudioClipDataMemento::GetChunkList() const
//{
//	return fChunkList;
//}
	
inline const MEAudioChunkList& MEAudioClipData::GetChunkList() const
{
	return fTrimmedChunkList;
}

inline bool MEAudioClipData::IsStereo() const
{
	return (fNbChannels == 2);
}

inline void MEAudioClipData::SetNbChannels(SInt32 nbChannels)
{ 
	ZzAssert(nbChannels > 0 && nbChannels <= 2);
	fNbChannels = nbChannels;
}

inline SInt32 MEAudioClipData::GetNbChannels() const
{
	return fNbChannels;
}

inline void MEAudioClipData::RecomputeTrimmedChunkList()
{
	fTrimmedChunkList = fPreTrimChunkList.ExtractSlice(GET_VAL(fStartTrim),fPreTrimChunkList.GetDuration()-GET_VAL(fStartTrim)-GET_VAL(fEndTrim));
}

inline ArmeTime MEAudioClipData::GetEndTrim() const
{
	return fEndTrim;
}

inline ArmeTime MEAudioClipData::GetStartTrim() const
{
	return fStartTrim;
}

inline const MEAudioChunkList& MEAudioClipData::GetPreTrimChunkList()
{
	return fPreTrimChunkList;
}

/*!
	@class ZzRABI_AudioClipDataRelease
	
	This class calls, in its dtor, "audioClipData->Release(someAudioClip)" with audioClipData and 
	someAudioClip supplied in its ctor. Please note someAudioClip can be NULL for the initial audioclipdata
	ownership deletion (deletion of an orphan audio clip data)
*/

class ZzRABI_AudioClipDataRelease
{
public :
	ZzRABI_AudioClipDataRelease(MEAudioClipData* audioClipData, MEAudioClip* audioClip) { fAudioClipData = audioClipData; fAudioClip = audioClip; }
	~ZzRABI_AudioClipDataRelease() { fAudioClipData->Release(fAudioClip); }
private :
	MEAudioClipData*	fAudioClipData;
	MEAudioClip*		fAudioClip;
};

