/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#ifndef ZzVerbose_h
#include "ZzVerbose.h"
#endif //ZzVerbose_h

#ifndef ArmeErrors_h
#include "ArmeErrors.h"
#endif //ArmeErrors_h

#ifndef MEAudioDeviceDesc_h
#include "MEAudioDeviceDesc.h"
#endif //MEAudioDeviceDesc_h

#ifndef chkerrca_h
#include "chkerrca.h"
#endif //chkerrca_h

#ifndef ZzAudioUtilities_h
#include "ZzAudioUtilities.h"
#endif //ZzAudioUtilities_h

AudioDeviceID MEAudioDeviceDesc::GetAudioDeviceIDFromStringCode(CFStringRef stringCode)
{
	AudioDeviceID 	devID;
	UInt32 			count;
	ArmeErr			err;
	if( CFStringCompare(stringCode,GetDefaultInputStringCode(),0) == kCFCompareEqualTo )
	{
		count = sizeof(devID);
		chkerrca(&err,AudioHardwareGetProperty(kAudioHardwarePropertyDefaultInputDevice,  &count, (void *) &(devID)));
	
		if(devID == kAudioDeviceUnknown)
		{
			vernl(1,"NO RECORDING DEVICE FOUND");
			return 0x00000000;
		}
		if(err.type != kArmeOK)
		{	
			vernl(1,"ERROR WHILE RETRIEVING kAudioHardwarePropertyDefaultInputDevice");
			return 0x00000000;
		}
		return devID;
	}
	if( CFStringCompare(stringCode,GetDefaultOutputStringCode(),0) == kCFCompareEqualTo )
	{
		count = sizeof(devID);
		chkerrca(&err,AudioHardwareGetProperty(kAudioHardwarePropertyDefaultOutputDevice,  &count, (void *) &(devID)));
	
		if(devID == kAudioDeviceUnknown)
		{
			vernl(0,"NO OUTPUT DEVICE FOUND");
			return 0x00000000;
		}
		if(err.type != kArmeOK)
		{	
			vernl(0,"ERROR WHILE RETRIEVING kAudioHardwarePropertyDefaultOutputDevice");
			return 0x00000000;
		}
		return devID;
	}
	
	/*struct AudioValueTranslation
	{
		void*	mInputData;
		UInt32	mInputDataSize;
		void*	mOutputData;
		UInt32	mOutputDataSize;
	};*/
	
	/*    OSStatus				err = noErr;
    AudioValueTranslation	AVTrans;
    char					cStr[256];
    UInt32					theSize;
    
    AVTrans.mInputData = (void *) type;
    AVTrans.mInputDataSize = sizeof(UInt32);
    AVTrans.mOutputData = cStr;
    AVTrans.mOutputDataSize = 256;

    theSize = sizeof(AudioValueTranslation);
    err = AudioDeviceGetProperty(deviceInfo->ID, 0, isInput, kAudioDevicePropertyDataSourceNameForID, &theSize, &AVTrans);
    if (err == noErr)
    {
        sourceName[0] = AVTrans.mOutputDataSize - 1;
        BlockMove(cStr, &sourceName[1], sourceName[0]);
    }
    return (err);
	*/
	AudioValueTranslation avTrans;
	avTrans.mInputData = (void*)&(stringCode);
	avTrans.mInputDataSize = sizeof(CFStringRef);
	avTrans.mOutputData = (void*)&(devID);
	avTrans.mOutputDataSize = sizeof(AudioDeviceID);
	
	count = sizeof(AudioValueTranslation);
	
	chkerrca(&err,AudioHardwareGetProperty(kAudioHardwarePropertyDeviceForUID, &count,&avTrans));
	
	return devID;
}

CFStringRef MEAudioDeviceDesc::GetDefaultOutputStringCode()
{
	return CFSTR("BenjaminWazHere_DefOutput");
}

CFStringRef MEAudioDeviceDesc::GetDefaultInputStringCode()
{
	return CFSTR("BenjaminWazHere_DefInput");
}

UInt32 MEAudioDeviceDesc::GetDefaultOutputNbFrames()
{
	return 0;
}

MEAudioDeviceDescEnumerator::MEAudioDeviceDescEnumerator()
{
	UInt32 			size;
	UInt32			i;
	UInt32 			nbDevs;
	AudioDeviceID*	devList;
	OSStatus 		st;
	ArmeErr			err;
	
	// retrieve the list of AudioDeviceID's
	st = AudioHardwareGetPropertyInfo(kAudioHardwarePropertyDevices, &size, NULL);
	nbDevs = size / sizeof(AudioDeviceID);
	
	devList = (AudioDeviceID*)malloc(nbDevs*sizeof(AudioDeviceID));
	st = AudioHardwareGetProperty(kAudioHardwarePropertyDevices,&size,devList);

	CFStringRef* inputNames = new CFStringRef[nbDevs];
	CFStringRef* inputStringCodes = new CFStringRef[nbDevs];
	CFStringRef* inputManufacturerNames = new CFStringRef[nbDevs];
	
	CFStringRef* outputNames = new CFStringRef[nbDevs];
	CFStringRef* outputStringCodes = new CFStringRef[nbDevs];
	CFStringRef* outputManufacturerNames = new CFStringRef[nbDevs];

	SInt32	currentInputDevice = 0;
	SInt32	currentOutputDevice = 0;
	SInt32	nbIns,nbOuts;

	for(i = 0; i < nbDevs; i++)
	{
		nbIns = ZzAudioCAGetNbAvailableInputStreamsInDevice(devList[i]);
		if(nbIns > 0)
		{
			size = sizeof(CFStringRef);
			st = AudioDeviceGetProperty(devList[i],0,TRUE,kAudioDevicePropertyDeviceManufacturerCFString,&size,(void*)(&(inputManufacturerNames[currentInputDevice])));
			chkerrca(&err,st);
			ZzAssert(st == noErr);
			size = sizeof(CFStringRef);
			st = AudioDeviceGetProperty(devList[i],0,TRUE,kAudioDevicePropertyDeviceNameCFString,&size,(void*)(&(inputNames[currentInputDevice])));
			chkerrca(&err,st);
			ZzAssert(st == noErr);
			size = sizeof(CFStringRef);
			st = AudioDeviceGetProperty(devList[i],0,TRUE,kAudioDevicePropertyDeviceUID,&size,(void*)(&(inputStringCodes[currentInputDevice])));
			chkerrca(&err,st);
			ZzAssert(st == noErr);
			currentInputDevice++;
		}
		nbOuts = ZzAudioCAGetNbAvailableOutputStreamsInDevice(devList[i]);
		if(nbOuts > 0)
		{
			size = sizeof(CFStringRef);
			st = AudioDeviceGetProperty(devList[i],0,FALSE,kAudioDevicePropertyDeviceManufacturerCFString,&size,(void*)(&(outputManufacturerNames[currentOutputDevice])));
			chkerrca(&err,st);
			ZzAssert(st == noErr);
			size = sizeof(CFStringRef);
			st = AudioDeviceGetProperty(devList[i],0,FALSE,kAudioDevicePropertyDeviceNameCFString,&size,(void*)(&(outputNames[currentOutputDevice])));
			chkerrca(&err,st);
			ZzAssert(st == noErr);
			size = sizeof(CFStringRef);
			st = AudioDeviceGetProperty(devList[i],0,FALSE,kAudioDevicePropertyDeviceUID,&size,(void*)(&(outputStringCodes[currentOutputDevice])));
			chkerrca(&err,st);
			ZzAssert(st == noErr);
			currentOutputDevice++;
		}
	}
	free(devList);
	
	fNbInputDevices = currentInputDevice;
	if(fNbInputDevices >= 0)
	{
		fInputManufacturerNames = new CFStringRef[fNbInputDevices];
		memcpy(fInputManufacturerNames,inputManufacturerNames,fNbInputDevices*sizeof(CFStringRef));
		delete [] inputManufacturerNames;

		fInputNames = new CFStringRef[fNbInputDevices];
		memcpy(fInputNames,inputNames,fNbInputDevices*sizeof(CFStringRef));
		delete [] inputNames;

		fInputStringCodes = new CFStringRef[fNbInputDevices];
		memcpy(fInputStringCodes,inputStringCodes,fNbInputDevices*sizeof(CFStringRef));
		delete [] inputStringCodes;
	}	
	else
	{
		fInputManufacturerNames = NULL;
	}
	
	fNbOutputDevices = currentOutputDevice;
	if(fNbOutputDevices >= 0)
	{
		fOutputManufacturerNames = new CFStringRef[fNbOutputDevices];
		memcpy(fOutputManufacturerNames,outputManufacturerNames,fNbOutputDevices*sizeof(CFStringRef));
		delete [] outputManufacturerNames;

		fOutputNames = new CFStringRef[fNbOutputDevices];
		memcpy(fOutputNames,outputNames,fNbOutputDevices*sizeof(CFStringRef));
		delete [] outputNames;

		fOutputStringCodes = new CFStringRef[fNbOutputDevices];
		memcpy(fOutputStringCodes,outputStringCodes,fNbOutputDevices*sizeof(CFStringRef));
		delete [] outputStringCodes;
	}	
	else
	{
		fOutputManufacturerNames = NULL;
	}
}

MEAudioDeviceDescEnumerator::~MEAudioDeviceDescEnumerator()
{
	SInt32 i;
	for(i = 0; i < fNbInputDevices; i++)
	{
		CFRelease(fInputNames[i]);
		CFRelease(fInputStringCodes[i]);
		CFRelease(fInputManufacturerNames[i]);
	}
	for(i = 0; i < fNbOutputDevices; i++)
	{
		CFRelease(fOutputNames[i]);
		CFRelease(fOutputStringCodes[i]);
		CFRelease(fOutputManufacturerNames[i]);
	}
	delete [] fInputNames;
	delete [] fInputStringCodes;
	delete [] fInputManufacturerNames;
	delete [] fOutputNames;
	delete [] fOutputStringCodes;
	delete [] fOutputManufacturerNames;
}

CFStringRef MEAudioDeviceDescEnumerator::GetInputDeviceName(SInt32 i)
{
	ZzAssert(i < fNbInputDevices);
	return fInputNames[i];
}

CFStringRef MEAudioDeviceDescEnumerator::GetOutputDeviceName(SInt32 i)
{
	ZzAssert(i < fNbOutputDevices);
	return fOutputNames[i];
}

CFStringRef MEAudioDeviceDescEnumerator::GetInputDeviceStringCode(SInt32 i)
{
	ZzAssert(i < fNbInputDevices);
	return fInputStringCodes[i];
}

CFStringRef MEAudioDeviceDescEnumerator::GetOutputDeviceStringCode(SInt32 i)
{
	ZzAssert(i < fNbOutputDevices);
	return fOutputStringCodes[i];
}

SInt32 MEAudioDeviceDescEnumerator::GetNbInputDevices()
{
	return fNbInputDevices;
}

SInt32 MEAudioDeviceDescEnumerator::GetNbOutputDevices()
{
	return fNbOutputDevices;
}



