/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#ifndef MEAudioMonitoring_h
#include "MEAudioMonitoring.h"
#endif //MEAudioMonitoring_h

#ifndef ZzRingBuffer_h
#include "ZzRingBuffer.h"
#endif //ZzRingBuffer_h

#ifndef ZzAudioUtilities_h
#include "ZzAudioUtilities.h"
#endif //ZzAudioUtilities_h

MEAudioMonitoring::MEAudioMonitoring(UInt32 log2RingBufferNbFrames,UInt32 nbChannels)
{
	ZzAssert(nbChannels == 2);
	fMonitoringRingBuffer = new ZzRingBuffer(nbChannels*sizeof(float),log2RingBufferNbFrames,0); // initial delta is ZERO
	fSecondaryMuteGain = 1.0;
	fMainMuteGain = 1.0;
}

/************************************************************************************************************************
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
************************************************************************************************************************/
void MEAudioMonitoring::MixCurrentDataIntoBuffer(float* raoDest, SInt32 raoNbChannels, float* tmpStereoBuf,SInt32 nbFrames)
{
//	if(fState == kIdle)
//		return;
		
	// the rule : the write thread ALWAYS increments its write pointer after having pushed new data
	// the read thread (this one) should 
	
	// the inputs are always mixed into the first rao output BY THE WRITING THREAD, WITH their correct
	// fader level applied and mono expanded to stereo etc.. etc.. etc..
	fMonitoringRingBuffer->ReadToFlatBuffer((UInt8*)tmpStereoBuf,nbFrames);
	
//	printf("monitoring raoDest[0] = %f\n",raoDest[0]);
	
//	UInt8* writeAddress = fMonitoringRingBuffer->DebugGetCurrentWriteAddress();
//	UInt8* readAddress = fMonitoringRingBuffer->DebugGetCurrentReadAddress();
//	vernl(0,"**fMonitoringRingBuffer  readAddress = " << (void*)readAddress << " writeAddress = " << (void*)writeAddress << " tmpStereoBuf[0] = " << tmpStereoBuf[0]);
	
	SInt64 writePos, readPos;
	
	fMonitoringRingBufferPosLocker.Lock(); // that isn't a problem because both threads are real-time
	writePos = fMonitoringRingBuffer->WritePos(); // writepos is the first empty one
	fMonitoringRingBufferPosLocker.Unlock(); // that isn't a problem because both threads are real-time
	readPos = fMonitoringRingBuffer->ReadPos(); // readpos is the one we've just read

	if ( readPos >= writePos ) return ;
	
	// if we've just read an empty one, then do NOT increase the pos
	// only increase if we've read a written one
	
	// BUT if the writepos is tooo big, then it means we don't read fast enough
	// we'll say if we're 256 frames too late then we increase 
	
	if(readPos >= writePos)
		return;
											
	if(readPos < writePos)
	{
		fMonitoringRingBuffer->IncrReadPos(nbFrames);
	}
	if(readPos < (writePos - 32768) ) // if we're REALLY late
	{
		fMonitoringRingBuffer->IncrReadPos(32678);
	}
	
	// let's apply the mute factors
	float* inPlaceOperationBuffer = tmpStereoBuf;
	float gain = fSecondaryMuteGain*fMainMuteGain;

	for(SInt32 i = 0; i < 2*nbFrames; i++) // stereo monitoring hardcoded
	{
		*inPlaceOperationBuffer = *inPlaceOperationBuffer*gain;
		inPlaceOperationBuffer++;
	}

	
	ZzAudioMixStereoWithStrides(
		raoDest,  // dest  buffer
		0, // index of LEFT channel in the dest buffer 
		raoNbChannels, // nb channels in dest buffer
		tmpStereoBuf, // source stereo buffer
		0, // index of LEFT channel in the source buffer
		2, // nb of source channels
		nbFrames );
}

void MEAudioMonitoring::WriteFrames(float* frames, SInt32 nbFrames)
{
//	printf("monitoring WriteFrames nbFrames = %d frames[0] = %f\n",(int)nbFrames,frames[0]);
	fMonitoringRingBuffer->WriteFromFlatBuffer((UInt8*)frames,nbFrames);
	fMonitoringRingBufferPosLocker.Lock();
	fMonitoringRingBuffer->IncrWritePos(nbFrames);
	fMonitoringRingBufferPosLocker.Unlock();
}

void MEAudioMonitoring::SetSecondaryMute(bool muted)
{
	if(muted)
		fSecondaryMuteGain = 0.0;
	else
		fSecondaryMuteGain = 1.0;
}

void MEAudioMonitoring::SetMute(bool muted)
{
	if(muted)
		fMainMuteGain = 0.0;
	else
		fMainMuteGain = 1.0;
}





