/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#ifndef MEAudioRecordManager_h
#include "MEAudioRecordManager.h"
#endif //MEAudioRecordManager_h

#ifndef MECoreAudioRecorder_h
#include "MECoreAudioRecorder.h"
#endif MECoreAudioRecorder_h

#ifndef ZzMath_h
#include "ZzMath.h"
#endif //ZzMath_h

#ifndef ZzRingBuffer_h
#include "ZzRingBuffer.h"
#endif //ZzRingBuffer_h

#ifndef MEMTGAAudioFile_h
#include "MEMTGAAudioFile.h"
#endif //MEMTGAAudioFile_h

#ifndef ZzQueue_h
#include "ZzQueue.h"
#endif //ZzQueue_h

#ifndef ZzAudioUtilities_h
#include "ZzAudioUtilities.h"
#endif //ZzAudioUtilities_h

#ifndef MEAudioMonitoring_h
#include "MEAudioMonitoring.h"
#endif //MEAudioMonitoring_h

#ifndef TArmeMediaEngine_h
#include "TArmeMediaEngine.h"
#endif //TArmeMediaEngine_h

#ifndef MEMTGAPlayer_h
#include "MEMTGAPlayer.h"
#endif //MEMTGAPlayer_h

#undef VER_DEF_TOPIC
#define VER_DEF_TOPIC MEAudioRecordManager

const SInt32 kLog2RecordRingBufferNbFrames = 16;

MEAudioRecordManager* MEAudioRecordManager::fsSingletonInstance = NULL;

MEAudioRecordManager* MEAudioRecordManager::SharedAudioRecordManager()
{
	if(!fsSingletonInstance)
	{
		fsSingletonInstance = new MEAudioRecordManager();
	}
	return fsSingletonInstance;
}

void* RecordToDiskThreadEntryPoint(void* cookie)
{
	MEAudioRecordFileResources* rsrc = (MEAudioRecordFileResources*)cookie;
//	OSStatus st;
	SInt32 param1,param2,param3,param4;
	
	while(true)
	{
		rsrc->queue->Wait((void**)&param1,(void**)&param2,(void**)&param3,(void**)&param4);
		//if(st == kMPDeletedErr) // if the queue was deleted when this thread was IN the MPWaitOnQueue call
		//if(st == kMPInvalidIDErr) // if the queue was deleted BEFORE this thread was IN the MPWaitOnQueue call
//		vernl(6,"RecordToDiskThreadEntryPoint received msg : (" << param1 << "," << param2 << "," << param3 << "," << param4 << ")");
		
		// three NULL params means "thread must terminate"
		if( param1 == 0 && param2 == 0 && param3 == 0 )
			return (void*)0x00001973;

		// otherwise it means we gotta read in the ring buffer and write this 
		// into the file
		rsrc->ringBuffer->ReadToFlatBuffer((UInt8*)(rsrc->tmpBuffer),rsrc->tmpBufferNbFrames);
		rsrc->ringBuffer->IncrReadPos(rsrc->tmpBufferNbFrames);
		rsrc->file->WriteFloat32Frames(rsrc->tmpBuffer,rsrc->tmpBufferNbFrames);
	}
}

void MEAudioRecordManager::PrepareRecordingForDoc(TArmeDoc* doc, AudioDeviceID inDevID)
{
	vernl(6,"MEAudioRecordManager::PrepareRecordingForDoc()");
	ArmeErr err;
	ZzAssert(fState == kIdle);
	ZzAssert(fHardwareRecorder == NULL);
	
	// very important : fSendBuffersToRecordThread must be inited before startAudioInThread.
	// but anyway this call is useless since it's inited to false in the ctor and reset to false
	// at dlg end time.
	fSendBuffersToRecordThread = false;
	
	fHardwareRecorder = new MECoreAudioRecorder(this);
	err = fHardwareRecorder->PrepareStartingAudioInThread(inDevID); // this retrieve the number of inputs etc....
	ZzAssert(err.type == kArmeOK);

//	EnableMonitoring();
	
	// the tmp buffer where a mixdown of all the input channels will be copied
	fMonitoringStereoBuffer = new float[fHardwareRecorder->InDeviceBufferNbFrames()*2];
			
	ZzAssert(fInputConfig == NULL);
	SInt32 nbInputs = fHardwareRecorder->GetNbInputs();
	fInputConfig = new MEAudioRecordInputConfig[nbInputs];
	ZzAssert(nbInputs >= 2);
	
	// only enable the first TWO inputs.... bug #929
	fInputConfig[0].enabled = true;
	fInputConfig[1].enabled = true;
	
	ZzAssert(fCurrentDoc == NULL);
	fCurrentDoc = doc;
	
	ZzAssert(fRecordResources == NULL);
	ZzAssert(fNbRecordResources == 0);
	fNbRecordResources = 0;
	
	GET_VAL(fRecordStartTime) = -1;
	ZzAssert(fRecordResources == NULL);
	fState = kReadyToRecord;
	CreateRecordResources();
	
	double buffersPerSecond = (double)fHardwareRecorder->GetAudioFrameRate()/(double)fHardwareRecorder->InDeviceBufferNbFrames();
	// 40dB/sec
	fDBPerBufferAttenuation = 40.0/buffersPerSecond;
	
	fNbOfBuffersForSaturationReset = 2*(int)buffersPerSecond;
	if(fNbOfBuffersForSaturationReset == 0) // buffer is greater than a second
		fNbOfBuffersForSaturationReset = 1;
	// ONLY DO THIS AT THE END OTHERWISE YOU'LL GET PROBLEMS (ACCESSING THIS OBJECT WHILE ARRAYS NOT CREATED)

	fHardwareRecorder->StartAudioInThread(inDevID);
}

void MEAudioRecordManager::PutInIdleState()
{
	vernl(6,"MEAudioRecordManager::PutInIdleState()");
	for(SInt32 i = 0; i < fTakeInfos.CountItems(); i++)
	{
		delete fTakeInfos[i];
	}
	fTakeInfos.RemoveAll();
	
	DestroyRecordResources();
	delete [] fInputConfig;
	fInputConfig = NULL;
	delete [] fRecordResources;
	fRecordResources = NULL;
	fNbRecordResources = 0;
	fState = kIdle;
	
	delete fHardwareRecorder;
	fHardwareRecorder = NULL;
	fCurrentDoc = NULL;
	GET_VAL(fRecordStartTime) = -1;

	DisableMonitoring();
	
	// the tmp buffer where a mixdown of all the input channels will be copied
	delete [] fMonitoringStereoBuffer;
	fMonitoringStereoBuffer = NULL;
}

SInt32 MEAudioRecordManager::GetNbInputs() const
{
	return fHardwareRecorder->GetNbInputs();
}

float MEAudioRecordManager::GetInputDBFSValue(SInt32 input) const
{
//	return -6.0;
#ifdef DEBUG
	ZzAssert(input < fHardwareRecorder->GetNbInputs());
#else
	if(input >= fHardwareRecorder->GetNbInputs())
		return 0.0f;
#endif //DEBUG
	return fInputConfig[input].analyzeDBFS;
}

void MEAudioRecordManager::SetInputEnabled(SInt32 iInput, bool enabled)
{
	vernl(6,"MEAudioRecordManager::SetInputEnabled( )");
	ZzAssert(fState == kReadyToRecord);
	DestroyRecordResources();
	ZzAssert(iInput < GetNbInputs());
	fInputConfig[iInput].enabled = enabled;
	CreateRecordResources();
	fRealTimeThreadMutex.Lock();
	fState = kReadyToRecord;
	fRealTimeThreadMutex.Unlock();
}

void MEAudioRecordManager::DestroyRecordResources()
{
	vernl(6,"MEAudioRecordManager::DestroyRecordResources( )");
	DestroyDynamicRecordResources();
	DestroyStaticRecordResources();
}

bool MEAudioRecordManager::IsInputEnabled(SInt32 iInput)
{
//vernl(6,"MEAudioRecordManager::IsInputEnabled( )");
	bool ret;
	fRealTimeThreadMutex.Lock();
	if(fState == kIdle)
		ret = false;
	else
		ret = fInputConfig[iInput].enabled;
	fRealTimeThreadMutex.Unlock();
	return ret;
}	

void MEAudioRecordManager::SetInputJoined(SInt32 iFirstInputOfThePair, bool joined)
{
	vernl(6,"MEAudioRecordManager::SetInputJoined( )");
	ZzAssert(fState == kReadyToRecord);
	DestroyRecordResources();
	ZzAssert(iFirstInputOfThePair < GetNbInputs()-1);
	ZzAssert(iFirstInputOfThePair % 2 == 0);
	fInputConfig[iFirstInputOfThePair].joined = joined;
	fInputConfig[iFirstInputOfThePair+1].joined = joined;
	CreateRecordResources();
	fRealTimeThreadMutex.Lock();
	fState = kReadyToRecord;
	fRealTimeThreadMutex.Unlock();
}

bool MEAudioRecordManager::IsInputJoined(SInt32 iFirstInputOfThePair) const
{
	ZzAssert(fState == kReadyToRecord);
	ZzAssert(iFirstInputOfThePair < GetNbInputs());
	return fInputConfig[iFirstInputOfThePair].joined;
}

void MEAudioRecordManager::SetInputAttenuationDB(SInt32 iInput, float dB)
{
//	vernl(6,"MEAudioRecordManager::SetInputAttenuationDB( )");
	ZzAssert(fState != kIdle);
	ZzAssert(iInput < GetNbInputs());
	ZzAssert(dB <= 7.0); // 6.0 is max value
	fInputConfig[iInput].targetLinearGain = MEMixingInfo::dBToLinearGain(dB);
}

float MEAudioRecordManager::GetInputAttenuationDB(SInt32 iInput) const
{
	ZzAssert(fState != kIdle);
	ZzAssert(iInput < GetNbInputs());
	float dB = MEMixingInfo::linearGainTodB(fInputConfig[iInput].targetLinearGain);
	ZzAssert(dB <= 7.0); // 6.0 is max value
	return dB;
}

void MEAudioRecordManager::StartRecording()
{
	vernl(6,"MEAudioRecordManager::StartRecording()");
	ZzAssert(fState == kReadyToRecord);
	fState = kRecording;
	fSendBuffersToRecordThread = true;
	fRecordStartTime = fCurrentDoc->GetPlayPos();
}

void MEAudioRecordManager::DestroyDynamicRecordResources()
{
	vernl(6,"MEAudioRecordManager::DestroyDynamicRecordResources()");
	// before posting the messages that tell the drt to end, we need to make sure no other
	// write to disk messages will be posted. thsu, we need to take a mutex that's around
	// the ProcessIncomingFrames() call...
	fMonitoringObject->SetSecondaryMute(true);
	fRealTimeThreadMutex.Lock();

	// now this means the CoreAudio thread will not send
	// messages to the recording thread anymore.
	fSendBuffersToRecordThread = false;
	
	// since the in ioproc thread also performs monitoring when not recording, we need
	// to put it into idle state	
	fState = kIdle;

	SInt32 i;
	
	// post a 'quit thread message' to all the DRT
	for(i = 0; i < fNbRecordResources; i++)
	{
		fRecordResources[i].queue->Notify(0,0,0,0);
	}
	fRealTimeThreadMutex.Unlock(); // Notify(0,0,0,0) needs to be done synchronously 
	fMonitoringObject->SetSecondaryMute(false);

	
	// now we need to wait for the threads to finish
	for(i = 0; i < fNbRecordResources; i++)
	{
		void* retCode;
		int err = pthread_join(fRecordResources[i].threadID,&retCode);
		if(err == 0)
		{
			ZzAssert(retCode == (void*)0x00001973);
		}
		else
		{
			ZzAssert(err == ESRCH);
		}
		fRecordResources[i].threadID = 0;
	}
	
	// now we can destroy the actual queues
	for(i = 0; i < fNbRecordResources; i++)
	{
		delete fRecordResources[i].queue;
		fRecordResources[i].queue = NULL;
	}
			
	// now close the files for each thread,
	// open the file again to check its duration and 
	// make sure this duration is the same for all the files.
//	SInt64 recordedFilesDuration = -1;
	for(i = 0; i < fNbRecordResources; i++)
	{
		fRecordResources[i].file->Close();
		delete fRecordResources[i].file;
		fRecordResources[i].file = NULL;
	}
	
	// now the ring buffers and tmp buffers can safely be deleted
	for(i = 0; i < fNbRecordResources; i++)
	{
		delete fRecordResources[i].ringBuffer;
		fRecordResources[i].ringBuffer = NULL;
		delete [] fRecordResources[i].tmpBuffer;
		fRecordResources[i].tmpBuffer = NULL;
		fRecordResources[i].tmpBufferNbFrames = 0;
		delete [] fRecordResources[i].tmpBufferRT;
		fRecordResources[i].tmpBufferRT = NULL;
		fRecordResources[i].tmpBufferRTNbFrames = 0;
	}	
			
}

SInt32 MEAudioRecordManager::StopRecording()
{
	vernl(6,"MEAudioRecordManager::StopRecording()");
	SInt32 i;

	ZzAssert(fState == kRecording);
	
	// stop the recording threads , etc...
	DestroyDynamicRecordResources();
	
	// now we need to close the files for each thread,
	// open the file again to check its duration and 
	// make sure this duration is the same for all the files.
	SInt64 recordedFilesDuration = -1;
	for(i = 0; i < fNbRecordResources; i++)
	{
		SInt64 currentFileDuration;

		// create a chunk structure to open this file.
		MEAudioChunk chunk;
		MEMTGAAudioFile file;
		chunk.startInFile = 0;
		chunk.duration = 0;
		chunk.doc = fCurrentDoc;
		chunk.SetFileIndex(fRecordResources[i].fileIndex);
		file.OpenReadChunk(fCurrentDoc,chunk);
		currentFileDuration = file.GetNbFrames();
		file.Close();
		// if it's the first file, we store its duration
		if(recordedFilesDuration == -1)
		{
			recordedFilesDuration = currentFileDuration;
		}
		// other wise, it needs to be the same as the first duration
		else
		{
			//ZzAssert(recordedFilesDuration == currentFileDuration);
			if(recordedFilesDuration != currentFileDuration)
			{
#ifdef DEBUG
				printf("mtrack recording warning : recordedFilesDuration (%d) != currentFileDuration (%d)\n",(int)recordedFilesDuration,(int)currentFileDuration);
#endif //DEBUG
			}
		}
	}
	
	// create a new take and copy the settings to the take info
	MEAudioRecordManagerTakeInfo* takeInfo = new MEAudioRecordManagerTakeInfo;
	takeInfo->startTime = fRecordStartTime;
	takeInfo->duration = _ArmeTime(recordedFilesDuration);
	takeInfo->playerPosAtRest = 0; // the current position for the monitoring
	
	// now for each file in the file resources, we store the file index
	// and the number of channels
	for(i = 0; i < fNbRecordResources; i++)
	{
		takeInfo->fileIndices.AddItem(fRecordResources[i].fileIndex);
		takeInfo->fileNbChannels.AddItem(fRecordResources[i].nbChannels);
	}
	
	// now let's add this take to the list of takes
	fTakeInfos.AddItem(takeInfo);
	
	// now we can safely delete the file resources that have been used by this recording
	// session and create new ones
	DestroyStaticRecordResources();

	CreateRecordResources();
	fRealTimeThreadMutex.Lock();
	fState = kReadyToRecord;
	fRealTimeThreadMutex.Unlock();
	ZzAssert(fTakeInfos.CountItems() >= 1);
	return fTakeInfos.CountItems() - 1;
}

void MEAudioRecordManager::DestroyStaticRecordResources()
{
	vernl(6,"MEAudioRecordManager::DestroyStaticRecordResources()");
	// don't have anything side the file resources to delete anymore
	// (see struct MEAudioRecordFileResources)
	delete [] fRecordResources;
	fRecordResources = NULL;
	fNbRecordResources = 0;
}

SInt32 MEAudioRecordManager::GetNbTakes() const
{
	ZzAssert(fState == kReadyToRecord);
	SInt32 nbTakes = 0;
	for(SInt32 i = 0; i < fTakeInfos.CountItems(); i++)
	{
		if(fTakeInfos[i] != NULL)
			nbTakes++;
	}
	return nbTakes;
}

ArmeTime MEAudioRecordManager::GetTakeStartTime(SInt32 take) const
{
	ZzAssert(fState == kReadyToRecord);
	return fTakeInfos[take]->startTime;
}

ArmeTime MEAudioRecordManager::GetTakeDuration(SInt32 take) const
{
	ZzAssert(fState == kReadyToRecord);
	return fTakeInfos[take]->duration;
}

SInt32 MEAudioRecordManager::GetTakeFileNbChannels(SInt32 take, SInt32 file) const
{
	ZzAssert(fState == kReadyToRecord);
	ZzAssert(take < GetNbTakes());
	ZzAssert(file < GetTakeNbAudioFiles(take));
	return fTakeInfos[take]->fileNbChannels[file];
}

ArmeTime MEAudioRecordManager::GetTakeEndxTime(SInt32 take) const
{
	ZzAssert(fState == kReadyToRecord);
	ArmeTime s = fTakeInfos[take]->startTime;
	ArmeTime d = fTakeInfos[take]->duration;
	return _ArmeTime(GET_VAL(s)+GET_VAL(d));
}

SInt32 MEAudioRecordManager::GetTakeNbAudioFiles(SInt32 take) const
{
	ZzAssert(fState == kReadyToRecord);
	return fTakeInfos[take]->fileIndices.CountItems();
}

void MEAudioRecordManager::DiscardTake(SInt32 take)
{
	ZzAssert(fCurrentlyPlayedTake != take); // you should stop a take before playing a new one.
	delete fTakeInfos[take];
	fTakeInfos[take] = NULL;
}

SInt64 MEAudioRecordManager::GetTakePlayerPosAtRest(SInt32 take) const
{
	ZzAssert(fState == kReadyToRecord);
	ZzAssert(take < GetNbTakes());
	return fTakeInfos.ItemAt(take)->playerPosAtRest;
}

void MEAudioRecordManager::SetTakePlayerPosAtRest(SInt32 take, SInt64 pos)
{
	ZzAssert(fState == kReadyToRecord);
	ZzAssert(take < GetNbTakes());
	if(pos < 0)
	{
#ifdef DEBUG
		printf("WARNING : setting pos < 0 in SetTakePlayerPosAtRest\n");
#endif //#ifdef DEBUG
		pos = 0;
	}
	if(pos >= GET_VAL(fTakeInfos.ItemAt(take)->duration))
	{
#ifdef DEBUG
		printf("WARNING : setting pos >= fTakeInfos.ItemAt(take)->duration in SetTakePlayerPosAtRest\n");
#endif //#ifdef DEBUG
		pos = 0;
	}
	fTakeInfos.ItemAt(take)->playerPosAtRest = pos;
}

MEAudioChunkFileIndex MEAudioRecordManager::GetTakeFile(SInt32 take, SInt32 file) const
{
	ZzAssert(fState == kReadyToRecord);
	ZzAssert(take < GetNbTakes());
	ZzAssert(file < GetTakeNbAudioFiles(take));
	return fTakeInfos[take]->fileIndices[file];
}

void MEAudioRecordManager::StartTakeMonitoring(SInt32 take)
{
	ZzAssert(!fMTGAPlayers); // you should stop a take before playing a new one.
	ZzAssert(fCurrentlyPlayedTake == -1);
	if(fMonitoringObject)
	{
		fMonitoringObject->SetSecondaryMute(true);
	}

	fCurrentlyPlayedTake = take;
	
	ZzAssert(take < fTakeInfos.CountItems());
	
	fMTGAPlayers = new MEMTGAPlayer*[GetTakeNbAudioFiles(take)];
	
	for(SInt32 iFile = 0; iFile < GetTakeNbAudioFiles(take); iFile++)
	{
		ArmeErr err;
		err = TArmeMediaEngine::CreateMTGAPlayer(&(fMTGAPlayers[iFile]),fCurrentDoc,GetTakeFile(take,iFile));
		fMTGAPlayers[iFile]->Seek(GetTakePlayerPosAtRest(take));

		if(err.type != kArmeOK)
		{
			fMTGAPlayers[iFile] = NULL;
		}
	}
//	ZzAssert(0); // sorry this is not implemented yet
}

ArmeTime MEAudioRecordManager::GetTakeMonitoringPosition(SInt32 take)
{
//	ZzAssert(0); // sorry this is not implemented yet
	if(!fMTGAPlayers)
		return _ArmeTime(GetTakePlayerPosAtRest(take));
	ZzAssert(fMTGAPlayers);
	ZzAssert(fCurrentlyPlayedTake == take); // we can't report position on another non playing take
	return _ArmeTime(fMTGAPlayers[0]->GetPlayPos());
}

void MEAudioRecordManager::StopTakeMonitoring(SInt32 take)
{
	if(fMTGAPlayers == NULL)
		return; // this means there were no monitoring
	ZzAssert(fCurrentlyPlayedTake == take); // cannot stop a take that is not started if another one plays!

	for(SInt32 iFile = 0; iFile < GetTakeNbAudioFiles(take); iFile++)
	{
		SetTakePlayerPosAtRest(take,fMTGAPlayers[iFile]->GetPlayPos());
		fMTGAPlayers[iFile]->Stop();
		TArmeMediaEngine::DeleteMTGAPlayer(fMTGAPlayers[iFile]);
	}

	delete [] fMTGAPlayers;
	fMTGAPlayers = NULL;
	fCurrentlyPlayedTake = -1;
	
	if(fMonitoringObject)
	{
		fMonitoringObject->SetSecondaryMute(false); // sets the 2nd monitoring gain to 1.0
	}
}

void MEAudioRecordManager::SeekTakeMonitoringPosition(SInt32 take, ArmeTime seekp)
{
	if(fMTGAPlayers == NULL)
	{
		// this means there were no monitoring
		SetTakePlayerPosAtRest(take,GET_VAL(seekp));
	}
	else
	{
		ZzAssert(fCurrentlyPlayedTake == take); // cannot stop a take that is not started if another one plays!
		for(SInt32 iFile = 0; iFile < GetTakeNbAudioFiles(take); iFile++)
		{
			fMTGAPlayers[iFile]->Seek(GET_VAL(seekp));
		}
	}
}

void MEAudioRecordManager::CreateRecordResources()
{
	vernl(6,"MEAudioRecordManager::CreateRecordResources()");
	ZzAssert(fRecordResources == NULL);
	ZzAssert(fState != kRecording);
	SInt32 iInput = 0;
	fNbRecordResources = 0;
	// first pass : count the number of files needed
	while(iInput < GetNbInputs())
	{
		if(fInputConfig[iInput].enabled)
		{
			fNbRecordResources++;
			if(fInputConfig[iInput].joined)
			{
				//$$$TODO tASK 1116 FIX IT
				iInput += 2;
			}
			else
				iInput++;
		}
		else
		{
			iInput++;
		}
	}
		
	// second pass : create the array of resources and create them
	fRecordResources = new MEAudioRecordFileResources[fNbRecordResources];
	
	SInt32 recordResourceIndex = 0;
	
	iInput = 0;
	while(iInput < GetNbInputs())
	{
		if(fInputConfig[iInput].enabled)
		{
			fRecordResources[recordResourceIndex].input = iInput;
			if(fInputConfig[iInput].joined)
			{
				fRecordResources[recordResourceIndex].nbChannels = 2;
				iInput += 2;
			}
			else
			{
				fRecordResources[recordResourceIndex].nbChannels = 1;
				iInput++;
			}

			// create the queue the record-to-disk thread will constantly poll to check
			// for write messages
//			OSStatus st = MPCreateQueue(&(fRecordResources[recordResourceIndex].queueID));
//			ZzAssert(st == noErr);
			fRecordResources[recordResourceIndex].queue = new ZzQueue(1024);
			
			// create the ring buffer that's used for data passing between
			fRecordResources[recordResourceIndex].ringBuffer = new ZzRingBuffer(
				fRecordResources[recordResourceIndex].nbChannels*sizeof(float), // 1 frame
				kLog2RecordRingBufferNbFrames, // 2^kLog2RecordRingBufferNbFrames frames
				0);  //read and write pos initially at the same position
			
			// this flag is used to be able to delete empty files
			fRecordResources[recordResourceIndex].somethingHasBeenWritten = false;
		
			// create a new chunk file index for the soon-to-be-recorded file
			fRecordResources[recordResourceIndex].fileIndex = fCurrentDoc->AudioChunkFileManager()->AllocateNewZeroRefMTGAChunkIndex();
			
			// create and open the file that will be written in
			fRecordResources[recordResourceIndex].file = new MEMTGAAudioFile;
			MEAudioChunk chunk;
			chunk.startInFile = 0;
			chunk.duration = 0;
			chunk.doc = fCurrentDoc;
			chunk.SetFileIndex(fRecordResources[recordResourceIndex].fileIndex);
			
			// create a tmp buffer for ringBuffer -> file copies
			// with the same size as the buffer sent from the recorder.
			fRecordResources[recordResourceIndex].tmpBufferNbFrames = fHardwareRecorder->InDeviceBufferNbFrames();

			fRecordResources[recordResourceIndex].tmpBuffer = 
				new float[fRecordResources[recordResourceIndex].tmpBufferNbFrames*fRecordResources[recordResourceIndex].nbChannels];

			// create a tmp buffer for real time thread -> ring buffer copies
			// with the same size as the buffer sent from the recorder.
			fRecordResources[recordResourceIndex].tmpBufferRTNbFrames = fHardwareRecorder->InDeviceBufferNbFrames();

			fRecordResources[recordResourceIndex].tmpBufferRT = 
				new float[fRecordResources[recordResourceIndex].tmpBufferRTNbFrames*fRecordResources[recordResourceIndex].nbChannels];

			
			fRecordResources[recordResourceIndex].file->OpenSaveNewChunkFile(
				fCurrentDoc,chunk,fRecordResources[recordResourceIndex].nbChannels);
									
			// create the thread and pass the structure with all the info needed to write
			// in the file
			pthread_create(
				&(fRecordResources[recordResourceIndex].threadID),
				NULL,
				RecordToDiskThreadEntryPoint,
				(void*)&(fRecordResources[recordResourceIndex]));
			
			recordResourceIndex++;
		}
		else
		{
			iInput++;
		}
	}
}

/************************************************************************************************************************
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
************************************************************************************************************************/
void MEAudioRecordManager::ProcessIncomingFrames(float* src, SInt32 nbFrames)
{
//	printf("ProcessIncomingFrames src[0] = %f\n",src[0]);

	fRealTimeThreadMutex.Lock();
	if(fState != kIdle)
	{
		AttenuateInputBuffer(src,nbFrames);
		AnalyzeInputBuffer(src);
		SInt32 nbInputSamplesPerFrame = fHardwareRecorder->GetNbInputs();
		memset(fMonitoringStereoBuffer,0,fHardwareRecorder->InDeviceBufferNbFrames()*2*sizeof(float));
		
		SInt32 recordedFileIndex = 0;
		float* currentSample = src;
		float value;
		ZzRtAssert(fRecordResources);
		for(SInt32 i = 0; i < nbFrames; i++)
		{
			recordedFileIndex = 0;
			for(SInt32 sample = 0; sample < nbInputSamplesPerFrame; )
			{
				if(recordedFileIndex < fNbRecordResources && fRecordResources[recordedFileIndex].input == sample)
				{
					bool doItInStereo = false;
					
					if(
						fInputConfig[fRecordResources[recordedFileIndex].input].joined && // this channel is joined
						fRecordResources[recordedFileIndex].input < (nbInputSamplesPerFrame-1)  && // it's not the last one
						fInputConfig[fRecordResources[recordedFileIndex].input+1].enabled
						)
					{
						doItInStereo = true;
						ZzRtAssert(fRecordResources[recordedFileIndex].nbChannels == 2);
					}
									
					if(doItInStereo)
					{
						ZzRtAssert(fRecordResources[recordedFileIndex].nbChannels == 2);
						value = *currentSample;
						currentSample++;
						fRecordResources[recordedFileIndex].tmpBufferRT[2*i] = value;
						fMonitoringStereoBuffer[2*i] += value;
						
						value = *currentSample;
						currentSample++;
						fRecordResources[recordedFileIndex].tmpBufferRT[2*i+1] = value;
						fMonitoringStereoBuffer[2*i+1] += value;
						sample += 2;
					}
					else
					{
						value = *currentSample;
						currentSample++;
						if(fRecordResources[recordedFileIndex].nbChannels == 1)
						{
							fRecordResources[recordedFileIndex].tmpBufferRT[i] = value;
						}
						else
						{
							fRecordResources[recordedFileIndex].tmpBufferRT[2*i] = value;
							fRecordResources[recordedFileIndex].tmpBufferRT[2*i+1] = value;
						}
						value *= 0.5; // monitoring gain advice by doelie and dominic
						fMonitoringStereoBuffer[2*i] += value;
						fMonitoringStereoBuffer[2*i+1] += value;
						sample ++;
					}
					recordedFileIndex++;
				}
				else
				{
					sample++;
					currentSample++;
				}
			}
		}
	
		if(fSendBuffersToRecordThread)
		{
			// now let's write the actual data to the ring buffer and post a message
			for(SInt32 iFile = 0; iFile < fNbRecordResources; iFile++)
			{
				fRecordResources[iFile].ringBuffer->WriteFromFlatBuffer((UInt8*)(fRecordResources[iFile].tmpBufferRT),nbFrames);
				fRecordResources[iFile].ringBuffer->IncrWritePos(nbFrames);
				fRecordResources[iFile].queue->Notify((void*)nbFrames,0,0,0);
			}
		}
		if(fMonitoringObject) //fMonitoringObject == NULL means the monitoring is disabled for some reason...
			fMonitoringObject->WriteFrames(fMonitoringStereoBuffer,nbFrames);
	}
	fRealTimeThreadMutex.Unlock();
}

/************************************************************************************************************************
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
************************************************************************************************************************/
void MEAudioRecordManager::AttenuateInputBuffer(float* src, SInt32 nbFrames)
{
	float* currentSample = src;
	SInt32 recordedFileIndex = 0;
	SInt32 nbInputs = fHardwareRecorder->GetNbInputs();
	SInt32 nbInputSamplesPerFrame = nbInputs; // just an alias

	for(SInt32 i = 0; i < nbInputs; i++)
	{
		fInputConfig[i].dezipGainIncrPerFrame = (fInputConfig[i].targetLinearGain - fInputConfig[i].currentLinearGain)/(float)nbFrames;
	}

	for(SInt32 i = 0; i < nbFrames; i++)
	{
		recordedFileIndex = 0;
		for(SInt32 sample = 0; sample < nbInputSamplesPerFrame; )
		{
			if(recordedFileIndex < fNbRecordResources && fRecordResources[recordedFileIndex].input == sample)
			{
				ZzRtAssert((currentSample - src) < nbFrames*nbInputSamplesPerFrame);
				bool doItInStereo = false;

				if(
					fInputConfig[fRecordResources[recordedFileIndex].input].joined && // this channel is joined
					fRecordResources[recordedFileIndex].input < (nbInputs-1)  && // it's not the last one
					fInputConfig[fRecordResources[recordedFileIndex].input+1].enabled
					)
				{
					doItInStereo = true;
					ZzRtAssert(fRecordResources[recordedFileIndex].nbChannels == 2);
				}
									
				if(doItInStereo)
				{
					fInputConfig[fRecordResources[recordedFileIndex].input].currentLinearGain += fInputConfig[fRecordResources[recordedFileIndex].input].dezipGainIncrPerFrame;
					fInputConfig[fRecordResources[recordedFileIndex].input+1].currentLinearGain += fInputConfig[fRecordResources[recordedFileIndex].input+1].dezipGainIncrPerFrame;
					
					ZzRtAssert(fRecordResources[recordedFileIndex].nbChannels == 2);
					*currentSample = *currentSample * fInputConfig[fRecordResources[recordedFileIndex].input].currentLinearGain;
					currentSample++;
					*currentSample = *currentSample * fInputConfig[fRecordResources[recordedFileIndex].input + 1].currentLinearGain;
					currentSample++;
					sample += 2;
				}
				else
				{
					fInputConfig[fRecordResources[recordedFileIndex].input].currentLinearGain += fInputConfig[fRecordResources[recordedFileIndex].input].dezipGainIncrPerFrame;
					//ZzRtAssert(fRecordResources[recordedFileIndex].nbChannels == 1); // due to task 1116 hack (see $$$TODO above), we can't check this.
					*currentSample = *currentSample * fInputConfig[fRecordResources[recordedFileIndex].input].currentLinearGain;
					currentSample++;
					sample ++;
				}
				recordedFileIndex++;
			}
			else
			{
				sample++;
				currentSample++;
			}
		}
	}
}

MEAudioRecordManager::MEAudioRecordManager()
{
	fInputConfig = NULL;
	fRecordResources = NULL;
	fNbRecordResources = 0;
	fState = kIdle;
	fHardwareRecorder = NULL;
	fCurrentDoc = NULL;
	fSendBuffersToRecordThread = false;
	fDBPerBufferAttenuation = 0.0; // don't care...... we init it in PrepareRecordingForDoc()
	fMonitoringObject = NULL;
	fMTGAPlayers = NULL;
	fCurrentlyPlayedTake = -1;
}

/************************************************************************************************************************
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
************************************************************************************************************************/
void MEAudioRecordManager::AnalyzeInputBuffer(float* srcBuffer)
{
//	vernl(6,"MEAudioRecordManager::AnalyzeInputBuffer()");
	SInt32 i;
	SInt32 nbInputs = fHardwareRecorder->GetNbInputs();
		
	for(i = 0; i < nbInputs; i++)
	{
		fInputConfig[i].analyzeMaxSqAmpl = 0.0;
	}
	
	float sqAmpl;

	SInt32  nbFrames = fHardwareRecorder->InDeviceBufferNbFrames();
	float* src = srcBuffer;
	
	for(SInt32 frame = 0; frame < nbFrames; frame++)
	{
		for(i = 0; i < nbInputs; i++)
		{
			sqAmpl = *src * *src;
			if(sqAmpl > fInputConfig[i].analyzeMaxSqAmpl)
				fInputConfig[i].analyzeMaxSqAmpl = sqAmpl;
			src++;
		}
	}
	for(i = 0; i < nbInputs; i++)
	{
		float dBFS = 20*log10(sqrt(fInputConfig[i].analyzeMaxSqAmpl));
		
		if(dBFS > 0.0)
		{
			fInputConfig[i].analyzeSatur = true;
			fInputConfig[i].analyzeSaturResetCounter = fNbOfBuffersForSaturationReset;
		}
		
		if(fInputConfig[i].analyzeSatur)
		{
			fInputConfig[i].analyzeSaturResetCounter = fInputConfig[i].analyzeSaturResetCounter -1;
			if( fInputConfig[i].analyzeSaturResetCounter == 0)
				fInputConfig[i].analyzeSatur = false;
		}
					
		// here we attenuate the previous values
		if(fInputConfig[i].analyzeDBFS > -1000.0)
			fInputConfig[i].analyzeDBFS -= fDBPerBufferAttenuation;

		// and we immediatly assign a new value
		if(dBFS > fInputConfig[i].analyzeDBFS)
			fInputConfig[i].analyzeDBFS = dBFS;
	}
}

bool MEAudioRecordManager::IsInputSaturated(SInt32 input) const
{
	ZzAssert(input < fHardwareRecorder->GetNbInputs());
	return fInputConfig[input].analyzeSatur;
}
	
void MEAudioRecordManager::SetInputNotSaturated(SInt32 input)
{
	ZzAssert(input < fHardwareRecorder->GetNbInputs());
	fInputConfig[input].analyzeSatur = false;
}

void MEAudioRecordManager::DisableMonitoring()
{
	TArmeMediaEngine::RemoveAudioMonitor(fMonitoringObject);
	fMonitoringObject = NULL;
}

void MEAudioRecordManager::EnableMonitoring()
{
	ZzAssert(fHardwareRecorder != NULL);
	if(fMonitoringObject == NULL)
	{
		// we store 8 full buffers in the ring buffer
		UInt32 ringBufferNbFramesNbBits = ZzMathLog2Cap(8*fHardwareRecorder->InDeviceBufferNbFrames());
		ZzAssert(fMonitoringObject == NULL);
		fMonitoringObject = TArmeMediaEngine::CreateAudioMonitor(ringBufferNbFramesNbBits,2); // "2" means stereo.
		// now the monitoring object will be directly called
	}
}

UInt32 MEAudioRecordManager::GetAudioFrameRate() const
{
	ZzAssert(fHardwareRecorder != NULL);
	return fHardwareRecorder->GetAudioFrameRate();
}



