/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#ifndef MEAudioTrackPlayer_h
#include "MEAudioTrackPlayer.h"
#endif //MEAudioTrackPlayer_h

#ifndef TArmeDoc_h
#include "TArmeDoc.h"
#endif //TArmeDoc_h

#ifndef MEMTGAAudioFile_h
#include "MEMTGAAudioFile.h"
#endif //MEMTGAAudioFile_h

// for the thread priority stuff
#ifndef ZzHelpers_h
#include "ZzHelpers.h"
#endif //ZzHelpers_h

#ifndef ZzDisableVerbose_h
#include "ZzDisableVerbose.h"
#endif //ZzDisableVerbose_h

#undef VER_DEF_TOPIC
#define VER_DEF_TOPIC MEAudioTrackPlayer

MEAudioTrackPlayer::MEAudioTrackPlayer(MEWSObjAudioPlayer* parentPlayer)
{
	vernl(6,"MEAudioTrackPlayer::MEAudioTrackPlayer()");
	fNbChannels = 0;

	fParentPlayer = parentPlayer;

	fRingBufferNbBits = kRingBufferNbBits;
	fReadAheadNbFrames = 8192; // min 42ms of buffering (at 192KHz)
	
	fRingBuffer = new float[RingBufferNbFrames()*sizeof(float)*2]; // stereo hardcoded. normally not a problem for mono tracks
	
	// setting both the following to zero 
	// will lead to severe trouble when requested
	// play something if we don't do anything
	// meanwhile
	fReadPosInRingBuf = 0;
	//fWritePosInRingBuf = 65536;
	fWritePosInRingBuf = fReadAheadNbFrames;
	
	// the chunk stack
	fCurReadRequestBlockStartIndex = 0;
	fCurReadRequestIndex = 0;
	fCurReadRequestBlockSize = 0;
	
#ifdef MEAudioTrackPlayer_STATIC_CHUNK_ARRAY
	fNextChunkIndex = 0;
#endif //MEAudioTrackPlayer_STATIC_CHUNK_ARRAY
	
//	fChannelsPerFrame = 2;
//	fBytesPerFrame = 8;
	
	StartFeederThread();
}

/*
void MEAudioTrackPlayer::CopyFromRingBuffer( float* dst, SInt64 srcTime, SInt32 nbFrames )
{
	ZzAssert(nbFrames < RingBufferNbFrames());
	UInt32 frameIndexInRingBuffer = srcTime & ((1 << fRingBufferNbBits) - 1); //okokok
	SInt32 sampleIndexInRingBuffer = frameIndexInRingBuffer * fChannelsPerFrame;
	const float* srcPtr = fRingBuffer + sampleIndexInRingBuffer;
	// part 1 length in frames
	SInt32 part2NbFrames = frameIndexInRingBuffer + nbFrames - (1 << fRingBufferNbBits);
	if(part2NbFrames <= 0)
	{
		verntl(audioClipsBenjamin,5,"RB->BUF@"<< srcPtr << "-" << nbFrames*fBytesPerFrame);
		memcpy(dst,srcPtr,nbFrames*fBytesPerFrame);
	}
	else
	{
		SInt32 part1NbFrames = nbFrames - part2NbFrames;
		verntl(audioClipsBenjamin,5,"RB->BUF@"<< srcPtr << "-" << nbFrames*fBytesPerFrame << "&" << fRingBuffer << "-" << part2NbFrames*fBytesPerFrame);
		memcpy(dst,srcPtr,part1NbFrames*fBytesPerFrame);
		memcpy(dst + part1NbFrames*fChannelsPerFrame, fRingBuffer, part2NbFrames*fBytesPerFrame);
	}
}
*/

void MEAudioTrackPlayer::MixFromRingBuffer(float startGain, float endGain, float* dst, SInt64 srcTime, SInt32 nbFrames )
{
	vernl(6,"MEAudioTrackPlayer::MixFromRingBuffer() mixing from ring buf read pos = " << (SInt32)srcTime << " for " << nbFrames << " frames");
	SInt32 sample;
	SInt32 nbSamples;
	nbSamples = nbFrames *fNbChannels;
	ZzAssert(nbFrames < RingBufferNbFrames());
	UInt32 frameIndexInRingBuffer = srcTime & ((1 << fRingBufferNbBits) - 1); //okokok
	SInt32 sampleIndexInRingBuffer = frameIndexInRingBuffer * fNbChannels;
	const float* srcPtr = fRingBuffer + sampleIndexInRingBuffer;
	// part 1 length in frames
	float curGain = startGain;
	float incrGain = (endGain-startGain)/(float)nbSamples;
	SInt32 part2NbFrames = frameIndexInRingBuffer + nbFrames - (1 << fRingBufferNbBits);
	if(part2NbFrames <= 0)
	{
		vernl(6,"MEAudioTrackPlayer::MixFromRingBuffer() RB->mix->BUF@" << srcPtr << "-" << nbFrames << " frames");
		for(sample=0;sample<nbSamples;sample++)
		{
			dst[sample] += curGain*srcPtr[sample];
			curGain += incrGain;
		}
		vernl(6,"MEAudioTrackPlayer::MixFromRingBuffer() dst[0] after = " << dst[0]);
	}
	else
	{
		SInt32 part1NbFrames = nbFrames - part2NbFrames;
		SInt32 part1NbSamples = fNbChannels*part1NbFrames;
		SInt32 part2NbSamples = fNbChannels*part2NbFrames;
		vernl(6,"RB->mix->BUF@" << srcPtr << "-" << part1NbFrames*sizeof(float)*fNbChannels << "&" << fRingBuffer << "-" << part2NbFrames*sizeof(float)*fNbChannels);
		for(sample=0;sample<part1NbSamples;sample++)
		{
			dst[sample] += curGain*srcPtr[sample];
			curGain += incrGain;
		}
		vernl(6,"MEAudioTrackPlayer::MixFromRingBuffer() dst[0] after = " << dst[0]);
		float* dst2 = dst + part1NbFrames*fNbChannels;
		for(sample=0;sample<part2NbSamples;sample++)
		{
			dst2[sample] += curGain*fRingBuffer[sample];
			curGain += incrGain;
		}
	}
}


/************************************************************************************************************************
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
************************************************************************************************************************/
void MEAudioTrackPlayer::CreateFutureChunkList(SInt64 currentStart,SInt32 length,SInt32 offset)
{
//	vernl(6,"MEAudioTrackPlayer::CreateFutureChunkList(currentStart = " << (SInt32)currentStart << ",length = " << length << ",offset = " << offset);
	// this function will convert the absolute range from start to start+length (excl)
	// to a (set of) continuous doc range(s)
	// - to do so, we must take the locators into account and whether the loop is active
	//   or not
	// - we will write the chunks into fChunkDescRing, starting at fCurChunkIndex
	//   and we we will increment fCurChunkIndex and fCurChunkBlockSize accordingly
	// - we must mentally define the function y = f(x) where x is absolute time and 
	//   y is doc time. This is a function that looks like f(x) until rloc and then
	//   goes on like a saw teeth vertically between lloc and rloc, like that :
	//     /| /| /| /| /
	//    / |/ |/ |/ |/
	//   /
	SInt64 lloc = GET_VAL(GetDoc()->GetLLocator());
	SInt64 rloc = GET_VAL(GetDoc()->GetRLocator());
	SInt64 loopLen = rloc - lloc;
	
	// if the doc is not looping, abs range == doc range
	// if we are already past the loop end, then we do not loop anymore
	if( !GetDoc()->IsLooping() || (currentStart >= rloc) || (rloc <= lloc))
	{
		FillNextReadRequest(currentStart+offset,length);
		return;
	}
	
	// now we are going to compute
	// NBLS = number of full loops before start
	// LBS = last bit between lloc and start
	// NBLE = number of full loops before end
	// LBE = last bit between lloc and end
	
	SInt64 absStart = currentStart+offset;
	SInt64 absEnd = currentStart+offset+length;
	
	// if there is a bit before the left locator, create
	// a chunk and forget about it.

	// maybe everything is before the lloc
	if( absStart < lloc && absEnd <= lloc )
	{
		FillNextReadRequest(absStart,length);
		return;
	}

	// maybe a part only is before the lloc
	if( absStart < lloc && absEnd > lloc )
	{
		FillNextReadRequest(absStart,lloc-absStart);
		absStart = lloc;
	}
	
	// let's make them relative to the left locator
	SInt64 absStartRel = absStart - lloc;
	SInt64 absEndRel = absEnd - lloc;
	SInt64 NBLS = absStartRel / loopLen;
	SInt64 LBS = absStartRel % loopLen; // between 0 and loopLen
	SInt64 NBLE = absEndRel / loopLen;
	SInt64 LBE = absEndRel % loopLen; // between 0 and loopLen
	
	SInt64 nbFullLoops = NBLE - NBLS - 1;
	
	if( nbFullLoops == -1 ) // which means the start and end are in the same loop.
	{
		ZzRtAssert(LBE > LBS);
		// we queue from start to end
		FillNextReadRequest(lloc+LBS,LBE-LBS);
		absStart = lloc;
		return;
	}
	else // the loop end is at least crossed once
	{
		// we queue from start to rloc
		FillNextReadRequest(lloc+LBS,rloc-LBS-lloc);
		// we queue N times from lloc to rloc
		for(SInt32 i = 0; i < nbFullLoops; i++)
		{
			FillNextReadRequest(lloc,loopLen);
		}
		// we queue from lloc to end
		FillNextReadRequest(lloc,LBE);
	}
}

/************************************************************************************************************************
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
************************************************************************************************************************/
void MEAudioTrackPlayer::MixFromRingBuffer(float startGain,float endGain, float* dst, SInt32 nbFrames, SInt32 blankNbFramesBefore, SInt32 blankNbFramesAfter)
{
///	vernl(6,"MEAudioTrackPlayer::MixFromRingBuffer() dst = " << dst << " nbFrames = " << (SInt32)nbFrames << " blankNbFramesBefore = " << blankNbFramesBefore << " blankNbFramesAfter = " << blankNbFramesAfter);
	MixFromRingBuffer(startGain, endGain, dst+blankNbFramesBefore*fNbChannels,fReadPosInRingBuf+blankNbFramesBefore,nbFrames-blankNbFramesBefore-blankNbFramesAfter);
}

/************************************************************************************************************************
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
************************************************************************************************************************/
void MEAudioTrackPlayer::QueueNextBit(SInt64 pos, SInt32 nbFrames)
{
//	vernl(6,"MEAudioTrackPlayer::QueueNextBit(pos = "<<(SInt32)pos << ",nbFrames = " << nbFrames << ")");
	fReadPosInRingBuf += nbFrames;
	
	// now we need to tell the DRT to queue the frames
	// that will need to be played in fReadAheadNbFrames
	// from now... 
	CreateFutureChunkList(pos,nbFrames,fReadAheadNbFrames);
	
	// 	if we have reached enough queued data, let's read it.
	// 	This Tells the DRT to queue audio from file. Info is stored in the chunk desc
	//	ring, starting at index fCurChunkBlockStartIndex and ending (exclusive) at 
	// 	index fCurChunkIndex.
	//	The data should be put at fWritePosInRingBuf in the ring 
	if(fCurReadRequestBlockSize >= kMinimumFramesForDiskQueuing)
	{
#ifdef AUDIOTRACKPLAYER_USE_ZZQUEUE
		fFeederZzQueue->Notify((void*)fCurReadRequestBlockStartIndex,(void*)fCurReadRequestIndex,0,0);
#else //#ifdef AUDIOTRACKPLAYER_USE_ZZQUEUE
		OSStatus st = MPNotifyQueue(fFeederThreadQueueID,(void*)fCurReadRequestBlockStartIndex,(void*)fCurReadRequestIndex,0);
		// ZzAssert(st == kMPInvalidIDErr || st == noErr);
		// it IS possible that the queue has been deleted because of a track rearrangement.... 
		// PerhapsDeleteUnusedTracks is called outside of the LockWrite() because the DRT does a LockRead in itself so it would 
		// block trying to die.... $$$TODO think about this. this shouldn't be a problem.
		// reset the block OBSOLETE
		// NOW BECAUSE OF fTrackCount we should NEVER REACH THE POINT WHERE WE TALK to a being-deleted track
		// furthermore, the above explanation was wrong since the object destructor could have destroyed all our data, not only invalidate
		// the fFeederThreadQueueID. "this" could have been pointing to freed memory, for instance ;-)
		ZzAssert(st == noErr);
#endif //#ifdef AUDIOTRACKPLAYER_USE_ZZQUEUE
		fCurReadRequestBlockStartIndex = fCurReadRequestIndex;
		fCurReadRequestBlockSize = 0;
	}
}	
	
void* MEAudioTrackPlayer::FeederThread()
{
	while(true)
	{
		SInt32 start,endX,dummy,dummy2;
#ifdef AUDIOTRACKPLAYER_USE_ZZQUEUE
		fFeederZzQueue->Wait((void**)&start,(void**)&endX,(void**)&dummy,(void**)&dummy2);
#else //#ifdef AUDIOTRACKPLAYER_USE_ZZQUEUE
		st = MPWaitOnQueue(fFeederThreadQueueID,(void**)&start,(void**)&endX,(void**)&dummy,kDurationForever);
		// if queue has been deleted, this means we need to exit from this thread
		if(st == kMPDeletedErr) // if the queue was deleted when this thread was IN the MPWaitOnQueue call
			return NULL;
		if(st == kMPInvalidIDErr) // if the queue was deleted BEFORE this thread was IN the MPWaitOnQueue call
			return NULL;
		else
		{
			ZzAssert(st == noErr);
		}
#endif / AUDIOTRACKPLAYER_USE_ZZQUEUE
		vernl(6,"MEAudioTrackPlayer::DRTProc() received msg : (" << start << "," << endX << "," << dummy << ")");
		
		// three NULL params means "thread must terminate". normally, it shouldn't be used since we kill the queue first
		if( start == 0 && endX == 0 && dummy == 0 )
			return NULL;

		// dummy == 1 params means "preroll to pos : start"
		if( dummy == 1 )
		{	
			SInt64 pos; // the pos is binary coded into start and endX (which lose their normal meaning)
 			SInt32* posInSInt32 = (SInt32*)(&pos);
			posInSInt32[0] = start;
			posInSInt32[1] = endX;
			SInt32 ringBufferNbFrames = 1 << fRingBufferNbBits;
			//qrintf("PREROLL request for clip %s at position %d\n",fClip->GetName(),(SInt32)pos);
			//fWritePosInRingBuf = 0;
			//fReadPosInRingBuf = 0;
//			SInt64 curWritePos = fReadPosInRingBuf; ///$$$TODO mthread bug
			GetDoc()->LockWrite(); // we need to lock WRITE because we're modifying the "request packet"

			// doing this will cancel any pending packeted request.
			fCurReadRequestBlockStartIndex = fCurReadRequestIndex;
			fCurReadRequestBlockSize = 0;
			
			// this resets the read and write pointers
			fReadPosInRingBuf = 0;
			fWritePosInRingBuf = fReadAheadNbFrames;
			
			ReadFloat32ContentToRingBuffer(fRingBuffer,fRingBufferNbBits,fReadPosInRingBuf,pos,ringBufferNbFrames);
			GetDoc()->UnlockWrite(); 
			//fWritePosInRingBuf = fReadPosInRingBuf + ringBufferNbFrames;
			//qrintf("PREROLL : after read, fReadPosInRingBuf = %d and fWritePosInRingBuf = %d\n",
			//	(SInt32)fReadPosInRingBuf, (SInt32)fWritePosInRingBuf);
		}
		else
		{
			SInt32 i;
			for( i = start; i < endX; i++ )
			{
				// we need to lockread the doc because, otherwise, the thread files and chunks might change
				// during a call to ReadFloat32ContentToRingBuffer(), which would be a very bad thing
				GetDoc()->LockRead(); 
				MEAudioTrackReadRequest* readreq = GetReadRequest(i);
				// make sure it's a NEW request
				ZzAssert(readreq->start != -1);
//				verntl(audioClipsBenjamin,5,"chunk request start=" << readreq->start << " duration =" << readreq->duration);
				ReadFloat32ContentToRingBuffer(
					fRingBuffer,fRingBufferNbBits,fWritePosInRingBuf,readreq->start,readreq->duration);
				fWritePosInRingBuf += readreq->duration;
				// put start and length to -1 to tell 'DRT was here'
				readreq->start = -1;
				readreq->duration = -1;
				GetDoc()->UnlockRead(); 
			}
		}
	}
	return NULL;
}

void* MEAudioTrackPlayer_FeederThread(void* cookie)
{
	MEAudioTrackPlayer* This = (MEAudioTrackPlayer*)cookie;
	return This->FeederThread();
}

void MEAudioTrackPlayer::StartFeederThread()
{
	// create the queue where the file in thread reads its message
#ifdef AUDIOTRACKPLAYER_USE_ZZQUEUE
	fFeederZzQueue = new ZzQueue(10); // we store room for 1<<10 = 1024 elements
#else //#ifdef AUDIOTRACKPLAYER_USE_ZZQUEUE
	st = MPCreateQueue(&fFeederThreadQueueID);
	ZzAssert(st == noErr);
#endif //#ifdef AUDIOTRACKPLAYER_USE_ZZQUEUE
	// launch the thread
	pthread_create(&fFeederThread, NULL, MEAudioTrackPlayer_FeederThread, (void*)this);
	ZzSetDRTThreadPriority(fFeederThread);
}

void MEAudioTrackPlayer::ReadFloat32ContentToRingBuffer(
	float*			ringBuffer, 
	SInt32			ringBufferNbBits,
	SInt64			writePosInRingBuf,
	SInt64			start, 
	SInt64			duration)
{
//	vernl(6,"MEAudioTrackPlayer::ReadFloat32ContentToRingBuffer(writePosInRingBuf = " << (SInt32)writePosInRingBuf << ", start = " << start <<  ", duration = " << duration);
	// now we need to split this request among
	// the chunks that make up the track.
	SInt32 i;

	// if the range is totally before the track, let's put zeroes
	if(start < 0 && start+duration < 0)
	{
		ZzRtAssert(0); // impossible since tracks have an infinite time span
		PutSilenceInRingBuffer(ringBuffer,ringBufferNbBits,writePosInRingBuf,duration,fNbChannels);
		writePosInRingBuf += duration;
		duration = 0;
		return;
	}
	// if part of the range is before the track beginning, let's also put zero in there 
	if(start < 0 )
	{
		ZzRtAssert(0); // impossible since tracks have an infinite time span
		ZzRtAssert(-start <= duration);
		PutSilenceInRingBuffer(ringBuffer,ringBufferNbBits,writePosInRingBuf,-start,fNbChannels);
		writePosInRingBuf -= start;
		duration += start;
		start = 0;
	}
	// maybe we are already finished ?
	if(duration <= 0)
		return;
	
	SInt64 currentChunkStartInTrack = 0;
	SInt64 currentChunkEndInTrack = 0;
	//$$$TODO find a better algorithm for the time->chunk detection instead
	// of browsing all the chunk list.
	for( i = 0; i < GetNbChunks(); i++ )
	{
		const MEAudioChunk& chunk = GetChunk(i);
		currentChunkEndInTrack += chunk.duration;
		SInt64 startInChunkSpace = start - currentChunkStartInTrack;
		if( startInChunkSpace >= 0 && (start+duration) < currentChunkEndInTrack)
		{
			// if the range is contained INSIDE the chunk, then that's ok. we simply perform
			// a one-pass read
//			vernl(6,"MEAudioTrackPlayer::ReadFloat32ContentToRingBuffer() -> ReadFromChunk( chunk i = " << i << ", writePosInRingBuf = " << (SInt32)writePosInRingBuf << ", startInChunkSpace = " << startInChunkSpace <<  ", duration = " << duration);
			ReadFromChunkToRingBuffer(chunk,writePosInRingBuf,startInChunkSpace,duration);
			return;
		}
		else if((startInChunkSpace >= 0) && (startInChunkSpace < chunk.duration)  && ((start+duration) >= currentChunkEndInTrack))
		{
			// the range starts _in this chunk_ but ends in another chunk later
			
			// the length of the range that intersects this chunk
			// which is the total length of the chunk - the space
			// in the chunk before the start of the range
			SInt64 lengthToCopy = chunk.duration - startInChunkSpace;
			
			// we read from the start of the range to the end of the chunk
//			vernl(6,"MEAudioTrackPlayer::ReadFloat32ContentToRingBuffer() -> ReadFromChunk( chunk i = " << i << ", writePosInRingBuf = " << (SInt32)writePosInRingBuf << ", startInChunkSpace = " << startInChunkSpace <<  ", lengthToCopy = " << lengthToCopy);
			ReadFromChunkToRingBuffer(chunk,writePosInRingBuf,startInChunkSpace,lengthToCopy);
		
			// now we shorten the given range by making its new start the
			// end of current chunk (or start of next chunk which is the same,
			// because we always work with ends exclusive)
			start = currentChunkEndInTrack; // which is the startInTrack next chunk
			
			// we of course need to shorten the length accordingly
			duration -= lengthToCopy;
			
			// and incr the write pos in the ring buffer
			writePosInRingBuf += lengthToCopy;
		}
		// next chunk start is equal to this chunk start + its length
		currentChunkStartInTrack += chunk.duration;
	}
	if(duration > 0)
	{
		// this means we are still requested some length after the end of the chunks... let's put 0 in there
//		vernl(6,"duration = " << duration << "after end. putting zero at writePosInRingBuf = " << writePosInRingBuf);
		PutSilenceInRingBuffer(ringBuffer,ringBufferNbBits,writePosInRingBuf,duration,fNbChannels);
		writePosInRingBuf += duration;
		duration = 0;
	}	
}

void MEAudioTrackPlayer::ReadFromChunkToRingBuffer(const MEAudioChunk& chunk, SInt64 writePosInRingBuf, SInt64 startInChunk, SInt32 length)
{
//	float* RBFirstAddr = MEGetRingBufferAddress(fRingBuffer, fRingBufferNbBits, fNbChannels, writePosInRingBuf);
//	vernl(6,"MEAudioTrackPlayer::ReadFromChunkToRingBuffer() : *RBFirstAddr before write = " << *RBFirstAddr);
	
	if(chunk.IsSilentChunk())
	{
		PutSilenceInRingBuffer(fRingBuffer, fRingBufferNbBits, writePosInRingBuf, length,fNbChannels);
	}
	else
	{
		bool b1  = fAudioFile.GetOpenMode() == MECompoundAudioFile::kNotOpen;
		bool b2 = fAudioFile.GetOpenMode() == MECompoundAudioFile::kOpenReadChunk;
		ZzAssert((b1 || b2));
		if(fAudioFile.GetOpenMode() == MECompoundAudioFile::kNotOpen || fAudioFile.GetOpenChunk() != chunk)
		{
			fAudioFile.Close();
			ArmeErr err = fAudioFile.OpenReadChunk(GetDoc(),chunk);
			if(err.type != kArmeOK)
			{
				fAudioFile.Close();
				return; // Ae Ae... a craint !
			}
		}
		fAudioFile.ReadFloat32ContentToRingBuffer(
			fRingBuffer, fRingBufferNbBits, writePosInRingBuf, chunk.startInFile+startInChunk, length);
	}
//	vernl(6,"MEAudioTrackPlayer::ReadFromChunkToRingBuffer() : *RBFirstAddr after write = " << *RBFirstAddr);
}		

void MEAudioTrackPlayer::PutSilenceInRingBuffer(
	float*	ringBuffer, 
	SInt32	ringBufferNbBits,
	SInt64	writePosInRingBuf,
	SInt64	length,
	SInt32	nbChannels)
{
	// we are going to fill some ringbuffer space with zeroes
	// hopefully, 0x00000000 is the binary rep for float 0.0
	
	SInt32 ringBufferNbFrames = 1 << ringBufferNbBits;
		
	ZzAssert(length <= ringBufferNbFrames);
	UInt32 frameIndexInRingBuffer = writePosInRingBuf & ((1 << ringBufferNbBits) - 1);
	SInt32 sampleIndexInRingBuffer = frameIndexInRingBuffer * nbChannels;
	float* dstPtr = ringBuffer + sampleIndexInRingBuffer;
	// part 1 length inframes
	SInt32 part2NbFrames = frameIndexInRingBuffer + length - ( 1 << ringBufferNbBits );
	if(part2NbFrames <= 0)
	{
		verntl(audioClipsBenjamin,5,"MTGA->RB to:" << dstPtr << " length:" << length << " frames");
		SInt64 actualNbFrames = length; //$$$TODO it won't work with more than 32 bit signed lengths
		memset(dstPtr, 0, actualNbFrames*sizeof(float)*nbChannels);
	}
	else
	{
		SInt32 part1NbFrames = length - part2NbFrames;
		verntl(audioClipsBenjamin,5,"MTGA->RB to:" << dstPtr << " length:" << part1NbFrames << " frames");
		verntl(audioClipsBenjamin,5,"MTGA->RB to:" << dstPtr << " length:" << part2NbFrames << " frames");
		SInt64 actualNbFrames = part1NbFrames;
		memset(dstPtr, 0, actualNbFrames*sizeof(float)*nbChannels);
		actualNbFrames = part2NbFrames;
		memset(ringBuffer, 0, actualNbFrames*sizeof(float)*nbChannels);
	}
}

//void MEAudioTrackPlayer::CancelPreroll()
//{
//	PutSilenceInRingBuffer(fRingBuffer,fRingBufferNbBits,0,(1 << fRingBufferNbBits),fNbChannels);
//}

void MEAudioTrackPlayer::Preroll(SInt64 pos)
{
	SInt32* posInts = (SInt32*)(&pos);
	// this means preroll at pos
#ifdef AUDIOTRACKPLAYER_USE_ZZQUEUE
	fFeederZzQueue->Notify((void*)(posInts[0]),(void*)(posInts[1]),(void*)1,(void*)0);
#else //#ifdef AUDIOTRACKPLAYER_USE_ZZQUEUE
	OSStatus st = MPNotifyQueue(fFeederThreadQueueID,(void*)(posInts[0]),(void*)(posInts[1]),(void*)1);
	ZzAssert(st == noErr);
#endif //#ifdef AUDIOTRACKPLAYER_USE_ZZQUEUE
}

#ifdef MEAudioTrackPlayer_STATIC_CHUNK_ARRAY

void MEAudioTrackPlayer::AddSilentChunk(SInt64 duration)
{
	MEAudioChunk silentChunk = MEAudioChunk::CreateSilentChunk(GetDoc(),duration);
	if(fNextChunkIndex < kMEAudioTrackPlayer_MaxNbChunksInTrack)
	{
		fStaticChunks[fNextChunkIndex] = silentChunk;
		fNextChunkIndex++;
	}
}

void MEAudioTrackPlayer::AddChunk(const MEAudioChunk& chunk)
{
	if(fNextChunkIndex < kMEAudioTrackPlayer_MaxNbChunksInTrack)
	{
		fStaticChunks[fNextChunkIndex] = chunk;
		fNextChunkIndex++;
	}
}

SInt32 MEAudioTrackPlayer::GetNbChunks() const
{
	return fNextChunkIndex;
}

const MEAudioChunk& MEAudioTrackPlayer::GetChunk(SInt32 i) const
{
	ZzAssert(i < kMEAudioTrackPlayer_MaxNbChunksInTrack);
	return fStaticChunks[i];
}

#else  // ! MEAudioTrackPlayer_STATIC_CHUNK_ARRAY

void MEAudioTrackPlayer::AddSilentChunk(SInt64 duration)
{
	MEAudioChunk silentChunk = MEAudioChunk::CreateSilentChunk(GetDoc(),duration);
	fChunks.AddItem(silentChunk);
}

void MEAudioTrackPlayer::AddChunk(const MEAudioChunk& chunk)
{
	fChunks.AddItem(chunk);
}

SInt32 MEAudioTrackPlayer::GetNbChunks() const
{
	return fChunks.CountItems();
}

const MEAudioChunk& MEAudioTrackPlayer::GetChunk(SInt32 i) const
{
	const MEAudioChunk& tmp = fChunks.ItemAt(i);
	return tmp;
}
#endif //MEAudioTrackPlayer_STATIC_CHUNK_ARRAY

void MEAudioTrackPlayer::SetStereo(bool stereo)
{
	if(stereo)
	{
		// either the nb of channels hasn't been assigned yet
		// or the track is already stero.
		bool b1 = (fNbChannels == 2);
		bool b2 = (fNbChannels == 0);
		ZzAssert(b1||b2);
		fNbChannels = 2;
	}
	else
	{
		// either the nb of channels hasn't been assigned yet
		// or the track is already mono.
		bool b1 = (fNbChannels == 1);
		bool b2 = (fNbChannels == 0);
		ZzAssert(b1||b2);
		fNbChannels = 1;
	}
}

// RT CALLED
bool MEAudioTrackPlayer::IsStereo() const
{
	ZzRtAssert(fNbChannels != 0);
	return (fNbChannels == 2);
}

void MEAudioTrackPlayer::EmptyChunkList()
{
	fNbChannels = 0;
#ifndef MEAudioTrackPlayer_STATIC_CHUNK_ARRAY
	fChunks.RemoveAll();
#endif // ! MEAudioTrackPlayer_STATIC_CHUNK_ARRAY
}

// destructor : kills the queue and the thread
MEAudioTrackPlayer::~MEAudioTrackPlayer()
{
	vernl(6,"MEAudioTrackPlayer::~MEAudioTrackPlayer()");
	// post a message telling the thread to exit

#ifdef AUDIOTRACKPLAYER_USE_ZZQUEUE
	fFeederZzQueue->Notify((void*)0,(void*)0,(void*)0,(void*)0);
#else //#ifdef AUDIOTRACKPLAYER_USE_ZZQUEUE
	OSStatus st = MPNotifyQueue(fFeederThreadQueueID,(void*)0,(void*)0,0);
	ZzAssert(st == noErr);
#endif //#ifdef AUDIOTRACKPLAYER_USE_ZZQUEUE
	void* returnCode;

	// wait for the thread to finish
	pthread_join(fFeederThread,&returnCode);

#ifdef AUDIOTRACKPLAYER_USE_ZZQUEUE
	delete fFeederZzQueue;
	fFeederZzQueue = NULL;
#else //#ifdef AUDIOTRACKPLAYER_USE_ZZQUEUE
	// kill the queue DIRECTLY
	MPDeleteQueue(fFeederThreadQueueID);
#endif //#ifdef AUDIOTRACKPLAYER_USE_ZZQUEUE
	
	// delete the ring buffer
	delete [] fRingBuffer;
}

/************************************************************************************************************************
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
************************************************************************************************************************/
void MEAudioTrackPlayer::FillNextReadRequest(SInt64 start, SInt64 duration)
{
	ZzRtAssert(duration < RingBufferNbFrames());
	SInt32 realIndex = fCurReadRequestIndex % kNbReadRequestsInRing;
//	vernl(6,"MEAudioTrackPlayer::FillNextReadRequest() request = " << fCurReadRequestIndex << " realIndex = " << realIndex << " start = " << start << " duration = " << duration);
	fRingOfReadRequests[realIndex].start = start;
	fRingOfReadRequests[realIndex].duration = duration;
	fCurReadRequestIndex++;
	fCurReadRequestBlockSize += duration;
}
