/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#define MEBufferMachine_h

#ifndef MEBufferMachine_h
#include "MEBufferMachine.h"
#endif //MEBufferMachine_h

#ifndef ZzTypes_h
#include "ZzTypes.h"
#endif //ZzTypes_h

// feeder machine : allows to soften floating point samples
// throughput variations by introducing a latency

// by default the delay is half the size of the requested
// number of buffered samples (1 << bufferSamplesNbBits)

// each time samples a requested through ReadSamples,
// a counter is incremented. when the counter reaches
// the value returned by the MinimumNbSamplesRead virtual call,
// a notification is sent to the disk (or network or whatever
// slow "device" you can picture) reading thread and the 
// "buffer packet" counter is reset to zero

class ZzRingBuffer;
class ZzQueue;

class MEBufferMachine
{
public :
	MEBufferMachine();
	virtual ~MEBufferMachine() {};
	
	void			Initialize(SInt32 nbChannels, SInt32 log2LatencyNbFrames);

	// client code read frames through this interface
	virtual void	ReadFrames(float* dst, SInt32 nbFrames);
	SInt32			GetNbChannels() const;
	SInt32 			LatencyNbFrames() const;
	void*			FeederThread();

private :
	void			StartFeederThread();

	// to be overridden
	virtual void	FeederFunction(float* dst, SInt32 nbSamples) = 0;

protected :	
	SInt32			fNbChannels;
	SInt32			fLatencyNbFrames;
	ZzRingBuffer*	fRingBuffer;
	ZzQueue*		fNotifyQueue;
	pthread_t		fFeederThread;
};


