/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
#define MECompoundAudioFile_h
#ifndef MECompoundAudioFile_h
#include "MECompoundAudioFile.h"
#endif //#ifndef MECompoundAudioFile_h

#ifndef MEMTGAAudioFile_h
#include "MEMTGAAudioFile.h"
#endif //MEMTGAAudioFile_h

#ifndef MESFPAudioFile_h
#include "MESFPAudioFile.h"
#endif //MESFPAudioFile_h

/*!
	@class MECompoundAudioFile

	This class presents an unified interface for both proprietary MTGA and regular PCM
	files. Please note the PCM needs to be real-time accessible, which basically means it's 
	a format where the packet size (granularity of seeks) is EXACTLY one frame. Real-time can
	be misleading here because it really means 'full seek flexibility' (MTGA are always perfectly
	seekable).
*/

class MECompoundAudioFile
{
public :
	enum
	{
		kNotOpen, kOpenRead, kOpenReadChunk, kOpenWrite, kOpenWriteChunk
	};

	enum
	{
		kPCMOpen, kMTGAOpen
	};
	
	MECompoundAudioFile();
	~MECompoundAudioFile();
	
	
	ArmeErr					OpenReadChunk(TArmeDoc* doc, const MEAudioChunk& chunk);
	ArmeErr					OpenSaveNewChunkFile(
								TArmeDoc* 		doc, 
								MEAudioChunk&	chunk,
								SInt32			channels);
	ArmeErr					ReadFloat32Frames(float* samples, SInt64* nbFrames);
	ArmeErr					ReadFloat32ContentToFlatBuffer(float* buffer, SInt64 start, SInt64 length );
	ArmeErr					ReadFloat32ContentToRingBuffer(
								float*	ringBuffer, 
								SInt32	ringBufferNbBits,
								SInt64	writePosInRingBuf,
								SInt64	start, 
								SInt64	length); // to be deleted and moved in the parent code
	ArmeErr					WriteFloat32FramesFromFlatBuffer(const float* samples, SInt64 chunkStartTime, SInt64 dur); // it's only a seek THEN a WriteFloat32Frames
	ArmeErr					WriteFloat32Frames(const float* samples, SInt64 nbFrames);
	ArmeErr					Close();
	SInt64					GetNbFrames();
	UInt32					GetNbChannels();
	ArmeErr					Seek(SInt64 offset);
	UInt32					GetOpenMode();
	const MEAudioChunk&		GetOpenChunk();
	ArmeErr					OpenReadMTGA(const char* path);

private :
	bool				fUsesMTGA; // true if using MTGA. only valid if fState != kNotOpen
	UInt32 				fState; //mirrors the actual file object state
	MEAudioChunk		fChunk;
	MEMTGAAudioFile		fMTGAAudioFile;
	MESFPAudioFile		fSFPAudioFile; // SFP means single frame packet
};

