/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#ifndef MECoreAudioPlayer_h
#include "MECoreAudioPlayer.h"
#endif MECoreAudioPlayer_h

#ifndef chkerrca_h
#include "chkerrca.h"
#endif //chkerrca_h

#ifndef TArmeDoc_h
#include "TArmeDoc.h"
#endif //TArmeDoc_h

#ifndef ArmeBasicTypes_h
#include "ArmeBasicTypes.h"
#endif //ArmeBasicTypes_h

#ifndef TArmeMasterMovie_h
#include "TArmeMasterMovie.h"
#endif //TArmeMasterMovie_h

#ifndef MovieHelper_h
#include "MovieHelper.h"
#endif //MovieHelper_h

#ifndef ArmeNotifications_h
#include "ArmeNotifications.h"
#endif //ArmeNotifications_h

#ifndef TArmeMediaEngine_h
#include "TArmeMediaEngine.h"
#endif //TArmeMediaEngine_h

#ifndef MEMixingInfo_h
#include "MEMixingInfo.h"
#endif //MEMixingInfo_h

#ifndef MEAudioMonitoring_h
#include "MEAudioMonitoring.h"
#endif //MEAudioMonitoring_h

#ifndef MEMTGAPlayer_h
#include "MEMTGAPlayer.h"
#endif //MEMTGAPlayer_h

#undef VER_DEF_TOPIC
#define VER_DEF_TOPIC MECoreAudioPlayer

static OSStatus audioOutProc(
	AudioDeviceID 			inDevice, 
	const AudioTimeStamp*	inNow, 
	const AudioBufferList*	inInputData, 
	const AudioTimeStamp*	inInputTime, 
	AudioBufferList*  		outOutputData, 
	const AudioTimeStamp* 	inOutputTime, 
	void* 					defptr )
{
	MECoreAudioPlayer* This = (MECoreAudioPlayer*)defptr;
	This->AudioOutProc(inDevice,inNow,inInputData,inInputTime,outOutputData,inOutputTime);
	return kAudioHardwareNoError;
}

void MECoreAudioPlayer::AudioOutProc(
	AudioDeviceID 			inDevice, 
	const AudioTimeStamp*	inNow, 
	const AudioBufferList*	inInputData, 
	const AudioTimeStamp*	inInputTime, 
	AudioBufferList*  		outOutputData, 
	const AudioTimeStamp* 	inOutputTime )
{
//    fAudioOutProcTimeUsageTimer.Reset();
	
	float* dst = (float*)(outOutputData->mBuffers[0].mData);
	
	SInt32 theoActualFrames = OutDeviceBufferNbFrames();
	
	SInt32 actualFrames = outOutputData->mBuffers[0].mDataByteSize/(sizeof(float)*fOutDeviceFormat.mChannelsPerFrame);
	if(theoActualFrames != actualFrames)
	{
		static int theoActualFramesDiffActualFrames = 0;
		if(theoActualFramesDiffActualFrames < 100)
		{
			ZzRtAssert(0);
			//vernl(1,"WARNING ! theoActualFrames (" << theoActualFrames << ") != actualFrames (" << actualFrames << ")");
			theoActualFramesDiffActualFrames++;
		}
	}

//#define SIN_TEST_MECoreAudioPlayer_AudioOutProc

#ifdef SIN_TEST_MECoreAudioPlayer_AudioOutProc
	FillWithStereoFloatSin128(dst,(SInt32)actualFrames,0);
#else //SIN_TEST_MECoreAudioPlayer_AudioOutProc
	
	// zero the dst buffer
	memset(dst,0,outOutputData->mBuffers[0].mDataByteSize);
	

	// loop on every doc
	TArmeMediaEngine::LockReadDocArray();
	SInt32 nbDocs = TArmeMediaEngine::GetNbDocs();
	for(SInt32 iDoc = 0; iDoc < nbDocs; iDoc++)
	{
		// loop on every doc player
		TArmeDoc* doc = TArmeMediaEngine::GetDoc(iDoc);
		
		// fRAOTmpBuffer is exactly a buffer of N full frames. we
		// use it as a tmp storage for each doc output.
		ZzRtAssert(fOutDeviceBufferNbBytes == outOutputData->mBuffers[0].mDataByteSize);
		ZzRtAssert(fRAOTmpBufferNbSamples*sizeof(float) == fOutDeviceBufferNbBytes);
		memset(fRAOTmpBuffer,0,fOutDeviceBufferNbBytes);
				
		// fLocalBuffers is a set of kCoreAudioPlayer_MaxHierarchyDepth stereo buffers
		doc->GetPlayer()->PutDataIntoRAO(fRAOTmpBuffer,fLocalBuffers,actualFrames);
		
		// and now we mix fRAOTmpBuffer with the CoreAudio buffer
		SInt32 nbSamples = actualFrames*2*GetNbOutputStereoPairs();
		float* src = fRAOTmpBuffer;
		float* dest = dst;
		for(SInt32 sample = 0; sample < nbSamples; sample++)
		{	
			*dest += *src;
			src++;
			dest++;
		}
	}
	TArmeMediaEngine::UnlockReadDocArray();
	
	TArmeMediaEngine::LockReadAudioMonitorArray();
	for(SInt32 i = 0; i < TArmeMediaEngine::AudioMonitorArrayCountItems(); i++)
	{
		MEAudioMonitoring* monitor = TArmeMediaEngine::GetAudioMonitor(i);
		monitor->MixCurrentDataIntoBuffer(dst,GetNbOutputStereoPairs()*2,fLocalBuffers[0],actualFrames);
	}
	TArmeMediaEngine::UnlockReadAudioMonitorArray();

	TArmeMediaEngine::LockReadMTGAPlayerArray();
	for(SInt32 i = 0; i < TArmeMediaEngine::MTGAPlayerArrayCountItems(); i++)
	{
		MEMTGAPlayer* player = TArmeMediaEngine::GetMTGAPlayer(i);
		player->MixCurrentDataIntoBuffer(dst,GetNbOutputStereoPairs()*2,fLocalBuffers[0],actualFrames);
	}
	TArmeMediaEngine::UnlockReadMTGAPlayerArray();
		
/*
	if( fPlayingDoc )
	{
		SInt64 oldPlayPos = fPlayPos;
		fDoc->ReadFloatFrames(dst, &fPlayPos, &actualFrames );
		
		// we are going to perform audio/video resync
		// 1 time out of 200,
		// $$$TODO make it dependent from audio buffer size
		// furthermore, if there is a loop, we force the resync
		static UInt32 audioVideoResyncCounter = 0;
		audioVideoResyncCounter++;
				
		// if there was a loop, let's retrigger the
		if(fPlayPos != oldPlayPos + actualFrames)
			audioVideoResyncCounter = 0;
		
		if((audioVideoResyncCounter % 200) == 0)
		{
			fDoc->Notify(kNotification_AdjustMovieArmeTime,NULL);
			audioVideoResyncCounter = 0;
		}
	}
	else
	{
		memset(dst,0,fOutDeviceBufferNbBytes);
	}
	ZzAssert(actualFrames == OutDeviceBufferNbFrames());
*/
#endif //SIN_TEST_MECoreAudioPlayer_AudioOutProc

//	float lengthMS = fAudioOutProcTimeUsageTimer.GetMSFloat();
	
//	fAudioOutProcTimeUsageFraction =  lengthMS/fWholeAudioOutProcBufferMS;
	fAudioOutProcTimeUsageFraction = 0.10; //$$$TODO BENJA
//	if(fAudioOutProcTimeUsageFraction > 1.0)
//		usleep(500000);
}

MECoreAudioPlayer::MECoreAudioPlayer(AudioDeviceID devID, UInt32* suggestedBufferNbFrames)
{
	NullifyBuffers();
	SetupForDeviceAndBufferSize(devID,suggestedBufferNbFrames);
	fAudioOutProcTimeUsageFraction = 1.0; // let's start with max usage during startup
}

MECoreAudioPlayer::~MECoreAudioPlayer()
{
	StopAudioOutThread();
	DeleteBuffers();
}

ZzString MECoreAudioPlayer::GetDeviceShortName() const
{
	CFStringRef theName;
	OSStatus 	st;
	UInt32		size;
	ArmeErr 	err;
	ZzString	ret;
	
	size = sizeof(CFStringRef);
	st = AudioDeviceGetProperty(fOutDevice,0,FALSE,kAudioDevicePropertyDeviceNameCFString,&size,(void*)(&theName));
	chkerrca(&err,st);
	if(err.type != kArmeOK)
	{
		ret = ZzString("<UNKNOWN NAME>");
	}
	else
	{
		ret = ZzString(theName);
		CFRelease(theName);
	}
	return ret;
}

void MECoreAudioPlayer::StopAudioOutThread()
{
	// $$$TODO check if this function waits for the thread 
	// to die or not.
	ArmeErr err;
	chkerrca(&err,AudioDeviceStop(fOutDevice,audioOutProc));
		
	// remove the procedure from the list of procedures 
	// associated to the device
	chkerrca(&err,AudioDeviceRemoveIOProc(fOutDevice,audioOutProc));
	
	// we need to wait for the audio device to stop running....
	UInt32 isRunning;
	UInt32 count = sizeof(isRunning);	// it is required to pass the size of the data to be returned
	do
	{ 
		chkerrca(&err,AudioDeviceGetProperty(fOutDevice, 0, FALSE, kAudioDevicePropertyDeviceIsRunning, &count, &isRunning));
	} while (isRunning != 0);
}

UInt32 MECoreAudioPlayer::GetAudioFrameRate() const
{
	return (UInt32)(fOutDeviceFormat.mSampleRate);
}

ArmeErr MECoreAudioPlayer::SetupForDeviceAndBufferSize(AudioDeviceID devID, UInt32* suggestedBufferNbFrames)
{
	ArmeErr err;
	fOutDevice = devID;
	
	UInt32		count;

	// get a description of the data format used by the default output device
	count = sizeof(fOutDeviceFormat);	// it is required to pass the size of the data to be returned
	
	// retrieve stream format for device
	chkerrca(&err,AudioDeviceGetProperty(fOutDevice, 0, FALSE, kAudioDevicePropertyStreamFormat, &count, (void*)(&fOutDeviceFormat)));
	if(err.type != kArmeOK)
	{
		return err;
	}
	
	// try to set the buffersize specified in the ctor
	fOutDeviceBufferNbBytes =  (*suggestedBufferNbFrames)*fOutDeviceFormat.mBytesPerFrame;
	count = sizeof(fOutDeviceBufferNbBytes);	// it is required to pass the size of the data to be returned
	chkerrca(&err,AudioDeviceSetProperty(fOutDevice, NULL, 0, FALSE, kAudioDevicePropertyBufferSize, count, &(fOutDeviceBufferNbBytes)));
	if(err.type != kArmeOK)
	{
		err.type = kArmeOK;
	//	return err; NAON !!!!!!!!
	}
			
	// get the buffersize that the default output device uses
	count = sizeof(fOutDeviceBufferNbBytes);	// it is required to pass the size of the data to be returned
	chkerrca(&err,AudioDeviceGetProperty(fOutDevice, 0, FALSE, kAudioDevicePropertyBufferSize, &count, &(fOutDeviceBufferNbBytes)));
	if(err.type != kArmeOK)
	{
		err.type = kArmeOK;
		// return err; a ne marche pas bien a??????? no comprendo
	}
	
	*suggestedBufferNbFrames = fOutDeviceBufferNbBytes / fOutDeviceFormat.mBytesPerFrame;
	ZzAssert(fOutDeviceBufferNbBytes % fOutDeviceFormat.mBytesPerFrame == 0);
	
	// check device uses linear PCM
	if((fOutDeviceFormat).mFormatID != kAudioFormatLinearPCM)
	{
		err.type = kCoreAudioDeviceNotUsingLinearPCMError;
		return err;
	}
	
	// check device uses float
	if(((fOutDeviceFormat).mFormatFlags & kLinearPCMFormatFlagIsFloat) == 0)
	{
		err.type = kCoreAudioDeviceNotUsingLinearPCMError;
		return err;
	}
	
	ReallocBuffers();

	return kArmeOK;
}

ArmeErr MECoreAudioPlayer::Start()
{
	ArmeErr err;
	// setup our in device with an IO proc
	chkerrca(&err,AudioDeviceAddIOProc(fOutDevice, audioOutProc, (void*)(this)));	
	if(err.type != kArmeOK)
	{
		return err;
	}
	
	// start rec sound through the device
	chkerrca(&err,AudioDeviceStart(fOutDevice, audioOutProc));				
	if(err.type != kArmeOK)
	{
		return err;
	}
	
	return kArmeOK;
}

void MECoreAudioPlayer::ReallocBuffers()
{
	for(UInt32 i = 0; i < kCoreAudioPlayer_MaxHierarchyDepth; i++)
	{
		delete [] fLocalBuffers[i]; // delete even works if the ptr is null
		fLocalBuffers[i] = new float[2*OutDeviceBufferNbFrames()]; // local buffers are always stereo
	}
	
	fRAOTmpBufferNbSamples = 2*GetNbOutputStereoPairs()*OutDeviceBufferNbFrames();
	fRAOTmpBuffer = new float[fRAOTmpBufferNbSamples];
	
	// compute the new duration of a buffer
	fWholeAudioOutProcBufferMS = (float)((double)OutDeviceBufferNbFrames()/(double)fOutDeviceFormat.mSampleRate)*1000.0;
}

void MECoreAudioPlayer::NullifyBuffers()
{
	// zero the local tmp buffers
	for(UInt32 i = 0; i < kCoreAudioPlayer_MaxHierarchyDepth; i++)
	{
		fLocalBuffers[i] = NULL;
	}
	fRAOTmpBuffer = NULL;
	fRAOTmpBufferNbSamples = 0;
}

void MECoreAudioPlayer::DeleteBuffers()
{
	for(UInt32 i = 0; i < kCoreAudioPlayer_MaxHierarchyDepth; i++)
	{
		delete [] fLocalBuffers[i]; // delete even works if the ptr is null
		fLocalBuffers[i] = NULL;
	}
	delete [] fRAOTmpBuffer;
	fRAOTmpBuffer = NULL;
	fRAOTmpBufferNbSamples = 0;
}

