/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#define MECoreAudioPlayer_h

#ifndef MECoreAudioPlayer_h
#include "MECoreAudioPlayer.h"
#endif MECoreAudioPlayer_h

#ifndef ArmeBasicTypes_h
#include "ArmeBasicTypes.h"
#endif //ArmeBasicTypes_h

#include <CoreAudio/CoreAudio.h>
#include <pthread.h>

#ifndef ArmeErrors_h
#include "ArmeErrors.h"
#endif //ArmeErrors_h

#ifndef ZzTimer_h
#include "ZzTimer.h"
#endif ZzTimer_h

#ifndef MEMixingInfo_h
#include "MEMixingInfo.h"
#endif //MEMixingInfo_h


/*!

	IMPORTANT INFO :
	
	after the ctor, every getter is valid but the thread hasn't started yet
	
	to actually start the io proc, you need to call Start()
*/
		

class MECoreAudioPlayer
{
public :
					MECoreAudioPlayer(AudioDeviceID devID, UInt32* suggestedBufferNbFrames);
					~MECoreAudioPlayer();
public :
	UInt32			GetAudioFrameRate() const;
	SInt32			OutDeviceBufferNbFrames();
	SInt32 			OutDeviceBufferNbBytes();
	ArmeErr 		SetupForDeviceAndBufferSize(AudioDeviceID devID, UInt32* suggestedBufferNbFrames);
	void			StopAudioOutThread();
	SInt32 			GetNbOutputStereoPairs();
	ArmeErr			Start();
	ZzString 		GetDeviceShortName() const;
	
public :
	
	void	 	AudioOutProc(
					AudioDeviceID 			inDevice, 
					const AudioTimeStamp*	inNow, 
					const AudioBufferList*	inInputData, 
					const AudioTimeStamp*	inInputTime, 
					AudioBufferList*  		outOutputData, 
					const AudioTimeStamp* 	inOutputTime );

	float			AudioOutProcTimeUsageFraction() const;

private :
	
	/*! 
		@function ReallocBuffers NullifyBuffers DeleteBuffers
		These function manage the local buffers that are passed to the doc players for hierarchy node temp
		storage and RAO temp.
	*/
	void			ReallocBuffers();
	void 			NullifyBuffers();
	void 			DeleteBuffers();
	
	
private :
	/*! @var fOutDevice The ID of the output device */
	AudioDeviceID				fOutDevice;
	
	/*! @var fOutDeviceBufferNbBytes Bytes per buffer for the audio output device */
	UInt32						fOutDeviceBufferNbBytes;

	/*! @var fOutDeviceFormat Stream format of the audio output device */
	AudioStreamBasicDescription	fOutDeviceFormat;

	float*						fLocalBuffers[kCoreAudioPlayer_MaxHierarchyDepth];
	
	// the tmp buffer for rao computation
	float*						fRAOTmpBuffer;
	
	float						fAudioOutProcTimeUsageFraction;
	ZzTimer						fAudioOutProcTimeUsageTimer;
	float						fWholeAudioOutProcBufferMS;
	UInt32						fRAOTmpBufferNbSamples;
};

inline SInt32 MECoreAudioPlayer::OutDeviceBufferNbFrames()
{
	return fOutDeviceBufferNbBytes / fOutDeviceFormat.mBytesPerFrame;
}

inline SInt32 MECoreAudioPlayer::OutDeviceBufferNbBytes()
{
	return fOutDeviceBufferNbBytes;
}

inline float MECoreAudioPlayer::AudioOutProcTimeUsageFraction() const
{
	return fAudioOutProcTimeUsageFraction;
}

inline SInt32 MECoreAudioPlayer::GetNbOutputStereoPairs()
{
//	ZzAssert(fOutDeviceFormat.mChannelsPerFrame % 2 == 0); // WE DO NOT SUPPORT ODD NB OF CHANNELS
	return fOutDeviceFormat.mChannelsPerFrame / 2;
}

