/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
#ifndef ArmeBasicTypes_h
#include "ArmeBasicTypes.h"
#endif //ArmeBasicTypes_h

#ifndef MECoreAudioRecorder_h
#include "MECoreAudioRecorder.h"
#endif MECoreAudioRecorder_h

#ifndef chkerrca_h
#include "chkerrca.h"
#endif //chkerrca_h

#ifndef MEAudioRecordManager_h
#include "MEAudioRecordManager.h"
#endif //MEAudioRecordManager_h

#undef VER_DEF_TOPIC
#define VER_DEF_TOPIC MECoreAudioRecorder

/*
MERecordRoutingInfo::MERecordRoutingInfo()
{
	fActive = false;
	fStereoGrouping = false;
	for(UInt32 i = 0; i < kMaxNbInputs; i++)
	{
		fTargetFiles[i] = NULL;
	}
}

struct MERecordRoutingInfo
{
	bool			fActive;
	bool			fStereoGrouping;
	MEMTGAAudioFile*	fTargetFiles[64];
	MERecordRoutingInfo();
};
*/

static OSStatus audioInProc(
	AudioDeviceID 			inDevice, 
	const AudioTimeStamp*	inNow, 
	const AudioBufferList*	inInputData, 
	const AudioTimeStamp*	inInputTime, 
	AudioBufferList*  		outOutputData, 
	const AudioTimeStamp* 	inOutputTime, 
	void* 					defptr )
{
	MECoreAudioRecorder* This = (MECoreAudioRecorder*)defptr;
	This->AudioInProc(inDevice,inNow,inInputData,inInputTime,outOutputData,inOutputTime);
	return kAudioHardwareNoError;
}

void MECoreAudioRecorder::AudioInProc(
	AudioDeviceID 			inDevice, 
	const AudioTimeStamp*	inNow, 
	const AudioBufferList*	inInputData, 
	const AudioTimeStamp*	inInputTime, 
	AudioBufferList*  		outOutputData, 
	const AudioTimeStamp* 	inOutputTime )
{
    float* src = (float*)(inInputData->mBuffers[0].mData);
	SInt32 actualFrames = InDeviceBufferNbFrames();
#if 0
	static UInt32 debugCounter = 0;
	debugCounter++;	
	if((debugCounter % 88) == 0) // roughly one debug report per second for 4096 bytes buffer stereo float
	{
		debugCounter = 0;
	
		double averageSumOfSquaresL = 0.0;
		double averageSumOfSquaresR = 0.0;
        
        double averageDBL = 0.0;
        double averageDBR = 0.0;
		
		float* tmpSrc = src;
		for(SInt32 frame = 0; frame < actualFrames; frame++) //$$$STEREO STEREO DEBUG HARDCODE
		{
			averageSumOfSquaresL += (*tmpSrc)*(*tmpSrc);
			tmpSrc++;
			averageSumOfSquaresR += (*tmpSrc)*(*tmpSrc);
			tmpSrc++;
		}
		averageSumOfSquaresL /= (double)actualFrames;
		averageSumOfSquaresR /= (double)actualFrames;

        averageDBL = 10.0 * log10(averageSumOfSquaresL);
        averageDBR = 10.0 * log10(averageSumOfSquaresR);

		//vernl(6,"current averageSumOfSquaresL = " << averageSumOfSquaresL << " averageSumOfSquaresR = " << averageSumOfSquaresR);
		vernl(6,"current averageDB 1st channel = " << averageDBL << " averageDB 2nd channel = " << averageDBR);
		
//		fWriteRingBuffer->WriteFromFlatBuffer((UInt8*)dst,nbItems);
//		fWriteRingBuffer->	
	}
#endif
	if(fAudioRecordManager)
		fAudioRecordManager->ProcessIncomingFrames(src,actualFrames);
}

MECoreAudioRecorder::MECoreAudioRecorder(MEAudioRecordManager* audioRecordManager)
{
	fAudioRecordManager = audioRecordManager;
	////$$$todo : check why anything higher than 65536 bytes causes problems (clicks and pops) on the UA-30
	fInDeviceSuggestedBufferNbBytes = 65536;
	
	fHardwareFound = false;
}

MECoreAudioRecorder::~MECoreAudioRecorder()
{
	StopAudioInThread();
}

void MECoreAudioRecorder::StopAudioInThread()
{
	if(!fHardwareFound)
		return;
	ArmeErr err;
	// $$$TODO check if this function waits for the thread 
	// to die or not.
	chkerrca(&err,AudioDeviceStop(fInDevice,audioInProc));
	
	// remove the procedure from the list of procedures 
	// associated to the device
	chkerrca(&err,AudioDeviceRemoveIOProc(fInDevice,audioInProc));
	
/*	UInt32 isRunning;
	UInt32 count = sizeof(isRunning);	// it is required to pass the size of the data to be returned
	do
	{ 
		chkerrca(AudioDeviceGetProperty(fInDevice, 0, TRUE, kAudioDevicePropertyDeviceIsRunning, &count, &isRunning));
	} while (isRunning != 0);*/
}

UInt32 MECoreAudioRecorder::GetAudioFrameRate() const
{
	if(!fHardwareFound)
		return 0;
	//ZzAssert(0);
	return (UInt32)(fInDeviceFormat.mSampleRate);
}

ArmeErr MECoreAudioRecorder::PrepareStartingAudioInThread(AudioDeviceID devID)
{
	// if this is called,
	// this means there IS a device available
	fHardwareFound = true;
	
	ArmeErr err;
	UInt32		count;
	
	
/*
	AudioDeviceID theDefaultDevice;
	count = sizeof(fInDevice);
	chkerrca(AudioHardwareGetProperty(kAudioHardwarePropertyDefaultInputDevice,  &count, (void *) &(fInDevice)));
	
	if(fInDevice == kAudioDeviceUnknown)
	{
		vernl(1,"NO RECORDING DEVICE FOUND");
		return ArmeErr(kArmeErrNoRecordingDevice);
	}
*/
	fInDevice = devID;
		
/*
	// try to set the buffersize specified in the ctor
	fInDeviceBufferNbBytes = fInDeviceSuggestedBufferNbBytes;
	count = sizeof(fInDeviceBufferNbBytes);	// it is required to pass the size of the data to be returned
	chkerrca(AudioDeviceSetProperty(fInDevice, NULL, 0, TRUE, kAudioDevicePropertyBufferSize, count, &(fInDeviceBufferNbBytes)));
*/	
	// get the buffersize that the default input device uses
	count = sizeof(fInDeviceBufferNbBytes);	// it is required to pass the size of the data to be returned
	chkerrca(&err,AudioDeviceGetProperty(fInDevice, 0, TRUE, kAudioDevicePropertyBufferSize, &count, &(fInDeviceBufferNbBytes)));
//	vernl(4,"AFTER NEGOTIATING, fInDeviceBufferNbBytes = " << fInDeviceBufferNbBytes);
	if(err.type != kArmeOK)
	{
		err.type = kArmeOK;
		// it seems to happen $$$TODO check why !
	}
	
	// get a description of the data format used by the default input device
	count = sizeof(fInDeviceFormat);	// it is required to pass the size of the data to be returned
	
	// retrieve stream format for device
	chkerrca(&err, AudioDeviceGetProperty(fInDevice, 0, TRUE, kAudioDevicePropertyStreamFormat, &count, (void*)(&fInDeviceFormat)));
	if(err.type != kArmeOK)
		return err;
	
	// retrieve the name of the device
	CFStringRef inDeviceNameRef;
	count = sizeof(inDeviceNameRef);

	chkerrca(&err,AudioDeviceGetProperty(fInDevice, 0, TRUE, kAudioDevicePropertyDeviceNameCFString, &count, (void*)(&inDeviceNameRef)));
	if(err.type != kArmeOK)
		return err;

	char inDeviceName[1024];
	Boolean ok = CFStringGetCString(inDeviceNameRef,inDeviceName,1024,kCFStringEncodingASCII);
	ZzAssert(ok);
	vernl(4,"the default input device is : " << inDeviceName);
		
	// check device uses linear PCM
	if((fInDeviceFormat).mFormatID != kAudioFormatLinearPCM)
	{
		vernl(1,"NO PCM RECORDING DEVICE FOUND");
		return ArmeErr(kArmeErrNoPCMRecordingDevice);
	}
	
	// check device uses float
	if(!((fInDeviceFormat).mFormatFlags & kLinearPCMFormatFlagIsFloat))
	{
		vernl(1,"NO FLOAT PCM RECORDING DEVICE FOUND");
		return ArmeErr(kArmeErrNoPCMFloatingPointRecordingDevice);
	}
	return err;
}

ArmeErr MECoreAudioRecorder::StartAudioInThread(AudioDeviceID devID)
{
	if(!fHardwareFound)
		return ArmeErr(kCoreAudioNoInputDeviceFound);
	ArmeErr err;
	// setup our in device with an IO proc
	chkerrca(&err,AudioDeviceAddIOProc(fInDevice, audioInProc, (void*)(this)));	
	if(err.type != kArmeOK)
		return err;
	
	// start rec sound through the device
	chkerrca(&err,AudioDeviceStart(fInDevice, audioInProc));				
	if(err.type != kArmeOK)
		return err;
	
	vernl(6,"OK : FLOAT PCM RECORDING DEVICE FOUND");
	return kArmeOK;
}










