/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#define MECoreAudioRecorder_h

#ifndef MECoreAudioRecorder_h
#include "MECoreAudioRecorder.h"
#endif MECoreAudioRecorder_h

#include <CoreAudio/CoreAudio.h>

/*
	!!!!!!!!!!!!
	RIGHT NOW IT ONLY USES THE DEFAULT DEVICE
	!!!!!!!!!!!!
*/
class MEAudioRecordManager;

class MECoreAudioRecorder
{
public :
					MECoreAudioRecorder(MEAudioRecordManager* audioRecordManager);
					~MECoreAudioRecorder();
public :
	UInt32			GetAudioFrameRate() const;
	SInt32			InDeviceBufferNbFrames();
	SInt32 			InDeviceBufferNbBytes();
	ArmeErr 		PrepareStartingAudioInThread(AudioDeviceID devID);
	ArmeErr 		StartAudioInThread(AudioDeviceID devID);
	void			StopAudioInThread();
	SInt32 			GetNbInputs();
	
public :
	
	void	 	AudioInProc(
					AudioDeviceID 			inDevice, 
					const AudioTimeStamp*	inNow, 
					const AudioBufferList*	inInputData, 
					const AudioTimeStamp*	inInputTime, 
					AudioBufferList*  		outOutputData, 
					const AudioTimeStamp* 	inOutputTime );

private :
	/*! @var fInDevice The ID of the input device */
	AudioDeviceID				fInDevice;
	
	/*! 
		@var fInDeviceSuggestedBufferNbBytes 
		What we would like the audio input device buffer size to be. It can be refused by 
		CoreAudio and set to something different, however.
		This var is only used when starting the audio input device 
	*/
	UInt32						fInDeviceSuggestedBufferNbBytes;

	/*! @var fInDeviceBufferNbBytes Bytes per buffer for the audio output device */
	UInt32						fInDeviceBufferNbBytes;

	/*! @var fInDeviceFormat Stream format of the audio output device */
	AudioStreamBasicDescription	fInDeviceFormat;
	
	/*! @var fAudioRecordManager The audio record manager that handles incoming data */
	MEAudioRecordManager* 		fAudioRecordManager;
	
	bool						fHardwareFound;
};

inline SInt32 MECoreAudioRecorder::InDeviceBufferNbFrames()
{
	if(!fHardwareFound)
		return 0;
	return fInDeviceBufferNbBytes / fInDeviceFormat.mBytesPerFrame;
}

inline SInt32 MECoreAudioRecorder::InDeviceBufferNbBytes()
{
	if(!fHardwareFound)
		return 0;
	return fInDeviceBufferNbBytes;
}

inline SInt32 MECoreAudioRecorder::GetNbInputs()
{
	if(!fHardwareFound)
		return 0;
	return fInDeviceFormat.mChannelsPerFrame;
}

