/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#ifndef MEDocPlayer_h
#include "MEDocPlayer.h"
#endif //MEDocPlayer_h

#ifndef TArmeDoc_h
#include "TArmeDoc.h"
#endif //TArmeDoc_h

#ifndef MEMixingInfo_h
#include "MEMixingInfo.h"
#endif //MEMixingInfo_h

#ifndef ZzTimer_h
#include "ZzTimer.h"
#endif ZzTimer_h

#ifndef MECoreAudioPlayer_h
#include "MECoreAudioPlayer.h"
#endif MECoreAudioPlayer_h

#ifndef ArmeNotifications_h
#include "ArmeNotifications.h"
#endif //ArmeNotifications_h

#ifndef MEAudioMonitoring_h
#include "MEAudioMonitoring.h"
#endif //MEAudioMonitoring_h

#ifndef MEAudioRecordManager_h
#include "MEAudioRecordManager.h"
#endif //MEAudioRecordManager_h

#ifndef ZzAudioUtilities_h
#include "ZzAudioUtilities.h"
#endif //ZzAudioUtilities_h

#undef VER_DEF_TOPIC
#define VER_DEF_TOPIC MEDocPlayer

#ifndef TArmeMediaEngine_h
#include "TArmeMediaEngine.h"
#endif //TArmeMediaEngine_h

MEDocPlayer::MEDocPlayer(TArmeDoc* doc)
{
	fDoc = doc;
	fWSObjPlayers[0] = new MEWSObjAudioPlayer(fDoc->GetTopContainer(),0);
	fNbWSObjPlayers = 1;
}

MEDocPlayer::~MEDocPlayer()
{
	vernl(6,"MEDocPlayer::~MEDocPlayer()");
	for(SInt32 audioPlayerIndex = 0; audioPlayerIndex < GetNbWSObjAudioPlayers(); audioPlayerIndex++)
	{
		MEWSObjAudioPlayer* audioPlayer = GetWSObjAudioPlayer(audioPlayerIndex);
		delete audioPlayer;
	}
}

MEDocPlayer::MEDocPlayer(ArmeArchiveHandler* unarchiver) :
	ArmeArchivable(unarchiver)
{
	ArmeArchivable* archObj;
	
	archObj = unarchiver->ReadCPPObject();
	fDoc = dynamic_cast<TArmeDoc*>(archObj);
	ZzAssert(fDoc);

	fNbWSObjPlayers = unarchiver->ReadSInt32();
	
	for(SInt32 i =0; i < fNbWSObjPlayers; i++)
	{
		archObj = unarchiver->ReadCPPObject();
		fWSObjPlayers[i] = dynamic_cast<MEWSObjAudioPlayer*>(archObj);
		ZzAssert(fWSObjPlayers[i]);
	}
}

bool MEDocPlayer::WriteToArchive(ArmeArchiveHandler* archiver) const
{
	bool ok = ArmeArchivable::WriteToArchive(archiver);
	ZzAssert(ok);
	archiver->WriteCPPObject(fDoc);
	archiver->WriteSInt32(fNbWSObjPlayers);
	for(SInt32 i =0; i < fNbWSObjPlayers; i++)
	{
		ok = archiver->WriteCPPObject(fWSObjPlayers[i]);
		ZzAssert(ok);
	}
	return true;
}

#define TIME_MEDocPlayer_RecreateTrackMapping

#ifdef TIME_MEDocPlayer_RecreateTrackMapping
static ZzTimer gTimer1;
#endif TIME_MEDocPlayer_RecreateTrackMapping

void MEDocPlayer::RecreateTrackMapping()
{
	gTimer1.Reset();
	for(SInt32 audioPlayerIndex = 0; audioPlayerIndex < GetNbWSObjAudioPlayers(); audioPlayerIndex++)
	{
		MEWSObjAudioPlayer* audioPlayer = GetWSObjAudioPlayer(audioPlayerIndex);
		audioPlayer->RecreateTrackMapping();
	}
#ifdef TIME_MEDocPlayer_RecreateTrackMapping
	UInt32 ms = gTimer1.GetMS();
	vernl(6,"MEDocPlayer::RecreateTrackMapping() took " << ms << " ms");
#endif //TIME_MEDocPlayer_RecreateTrackMapping
}	

/************************************************************************************************************************
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
************************************************************************************************************************/
void MEDocPlayer::PutDataIntoRAO(float* raoTmpDst, float** localBuffers, UInt32 nbFrames)
{
	// raoTmpDst is already zeroed
#if 1
	fDoc->LockRead();
	fDoc->GetMixingInfo()->ResetVAOMirrorBuffers(); // this zeroes the mirror buffers used for level computation
	// raoTmpDst is already zeroed !
	for(SInt32 audioPlayerIndex = 0; audioPlayerIndex < GetNbWSObjAudioPlayers(); audioPlayerIndex++)
	{
		MEWSObjAudioPlayer* audioPlayer = GetWSObjAudioPlayer(audioPlayerIndex);
		audioPlayer->MixCurrentDataInRAO(raoTmpDst,localBuffers,nbFrames);
	}
	// following statement needs to be inside the lock because the number of allocated buffers can change through the gui and the vector
	// of pointers to the vao mirror buffers may indeed change
	fDoc->GetMixingInfo()->AnalyzeRAOBuffer(raoTmpDst); // this analyzes the rao buffer and computes the levels to be displayed by the mixer
	fDoc->GetMixingInfo()->AnalyzeVAOMirrorBuffers(); // this analyzes the vao buffers and computes the levels to be displayed by the mixer
	
	// now let's check if document end was reached or not. PS : we're using the main player (0) 
	// for this
	SInt64 playPos = GET_VAL(GetWSObjAudioPlayer(0)->GetPlayPos());
	if(playPos >= GET_VAL(fDoc->GetDuration()) && GetWSObjAudioPlayer(0)->IsPlaying())
	{
		if(GetWSObjAudioPlayer(0)->IsLooping())
		{
			// WE LOOP !
			GetWSObjAudioPlayer(0)->Seek(_ArmeTime(0));
		}
		else
		{
			// WE STOP !
			GetWSObjAudioPlayer(0)->Stop(); // so that sounds stops immediatly
			fDoc->Notify(kNotification_DocAutoStop,NULL); // so that button states are reset, etc...
		}
	}
	fDoc->UnlockRead();
	
	
#else
/*
	UInt32 i,j,k;
	
	// lockread the document
	fDoc->LockRead();
	
	// add all the mono VAO to the RAO buffer
	for(i = 0; i < fDoc->GetMixingInfo()->GetMaxMixerDisplayMonoVAOCount(); i++)
	{
		// only zero the first nbFrames mono frames
		memset((void*)(localBuffers[0]),0,nbFrames*sizeof(float));
		
		for(j = 0; j < GetNbWSObjAudioPlayers(); j++)
		{
			MEWSObjAudioPlayer* audioPlayer = GetWSObjAudioPlayer(j);
			audioPlayer->MixCurrentDataForVAO(false, i,localBuffers,nbFrames);
		}
		MixMapMonoVAOIntoRAOBuffer(raoDst,i,localBuffers[0],nbFrames);
	}
	
	// add all the stereo VAO to the RAO buffer
	for(i = 0; i < fDoc->GetMixingInfo()->GetMaxMixerDisplayStereoVAOCount(); i++)
	{
		// zero the nbFrames stereo frames
		memset((void*)(localBuffers[0]),0,2*nbFrames*sizeof(float));
		
		for(j = 0; j < GetNbWSObjAudioPlayers(); j++)
		{
			MEWSObjAudioPlayer* audioPlayer = GetWSObjAudioPlayer(j);
			audioPlayer->MixCurrentDataForVAO(true, i,localBuffers,nbFrames);
		}
		MixMapStereoVAOIntoRAOBuffer(raoDst,i,localBuffers[0],nbFrames);
	}
	fDoc->UnlockRead();
*/
#endif
}
/*
void MEDocPlayer::ReadFloatFrames(float* dst, SInt64* posInDoc)
{
	ZzAssert( (*actualFrames % 32) == 0);
	
	LockRead();
	// assume stereo and erase dst buffer (please note memory content for float 0.0 is 0x00000000 so that's ok)
	
	// the top buffer is the one where the current RAO's VAOs are mixed into
	fHierarchyBuffers[0] = dst;
	
	// we are going to split the playback in several chunks according to the loop and
	// current position in a similar fashion to the TAudioClip2Player.
	SInt64 lloc = GET_VAL(GetLLocator());
	SInt64 rloc = GET_VAL(GetRLocator());
	SInt64 loopLen = rloc - lloc;
	
	// if the doc is not looping, abs range == doc range
	// if we are already past the loop end, then we do not loop anymore
	if( !IsLooping() || (*posInDoc >= rloc) || (rloc <= lloc) )
	{
		ReadFlatFloatFrames(*posInDoc, *actualFrames);
		*posInDoc += *actualFrames;
	}
	else
	{
		// now we are going to compute
		// NBLS = number of full loops before start
		// LBS = last bit between lloc and start
		// NBLE = number of full loops before end
		// LBE = last bit between lloc and end
		
		SInt64 absStart = *posInDoc;
		SInt64 absEnd = *posInDoc + *actualFrames;
		SInt64 debugLength = 0;
	
		// if there is a bit before the left locator, create
		// a chunk and forget about it.
		if( absStart < lloc && absEnd <= rloc)
		{
			ReadFlatFloatFrames(absStart, *actualFrames);
			*posInDoc += *actualFrames;
		}
		else
		{
			// if there is a bit before the left locator, create
			// a chunk and forget about it.
			if( absStart < lloc && absEnd >= lloc)
			{
				ReadFlatFloatFrames(absStart, lloc-absStart);
				fHierarchyBuffers[0] += 2*(lloc-absStart); // stereo, thus frames = 2*samples (float*)
				debugLength += (lloc-absStart);
				absStart = lloc;
				*posInDoc = lloc;
			}
		
			// let's make them relative to the left locator
			SInt64 absStartRel = absStart - lloc;
			SInt64 absEndRel = absEnd - lloc;
			SInt64 NBLS = absStartRel / loopLen;
			SInt64 LBS = absStartRel % loopLen; // between 0 and loopLen
			SInt64 NBLE = absEndRel / loopLen;
			SInt64 LBE = absEndRel % loopLen; // between 0 and loopLen
			
			SInt64 nbFullLoops = NBLE - NBLS - 1;
			
			if( nbFullLoops == -1 ) // which means the start and end are in the same loop.
			{
				ZzAssert(LBE > LBS);
				// we queue from start to end
				ReadFlatFloatFrames(lloc+LBS, LBE-LBS);
				fHierarchyBuffers[0] += 2*(LBE-LBS); // stereo, thus frames = 2*samples (float*)
				debugLength += LBE-LBS;
				ZzAssert(debugLength == *actualFrames);
			}
			else // the loop end is at least crossed once
			{
				// we queue from start to rloc
				if(loopLen-LBS > 0) 
					ReadFlatFloatFrames(lloc+LBS, loopLen-LBS);
				fHierarchyBuffers[0] += 2*(loopLen-LBS); // stereo, thus frames = 2*samples (float*)
				debugLength += loopLen-LBS;
				// we queue N times from lloc to rloc
				for(SInt32 i = 0; i < nbFullLoops; i++)
				{
					ReadFlatFloatFrames(lloc, loopLen);
					fHierarchyBuffers[0] += 2*loopLen; // stereo, thus frames = 2*samples (float*)
					debugLength += loopLen;
				}
				// we queue from lloc to end
				if(LBE > 0)
					ReadFlatFloatFrames(lloc, LBE);
				fHierarchyBuffers[0] += 2*LBE; // stereo, thus frames = 2*samples (float*)
				debugLength += LBE;
				ZzAssert(debugLength == *actualFrames);
			}
			*posInDoc = lloc+LBE;
		}
	}
	float leftRMS = 0.0;
	float rightRMS = 0.0;
	for( SInt32 frame = 0; frame < *actualFrames; ++frame )
	{
		*dst = fRAOLeftFader[0]**dst;
		float leftSrc = *dst++;
		leftRMS += leftSrc*leftSrc;
		*dst = fRAORightFader[0]**dst;
		float rightSrc = *dst++;
		rightRMS += rightSrc*rightSrc;
	}
	leftRMS /= (float)(*actualFrames);
	leftRMS = sqrt(leftRMS);
	rightRMS /= (float)(*actualFrames);
	rightRMS = sqrt(rightRMS);
	UpdateRAOLeftLevelWithValue(0,leftRMS);
	UpdateRAORightLevelWithValue(0,rightRMS);
	UnlockRead();
}
*/			
