/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#define MEDocPlayer_h

#ifndef MEDocPlayer_h
#include "MEDocPlayer.h"
#endif //MEDocPlayer_h

#ifndef ArmeArchivable_h
#include "ArmeArchivable.h"
#endif //ArmeArchivable_h

#ifndef ZzRtAssert_h
#include "ZzRtAssert.h"
#endif //ZzRtAssert_h

class TArmeDoc;
class MEWSObjAudioPlayer;
#define kMEDocPlayer_MaxNbAudioPlayers 256

class MEDocPlayer : public ArmeArchivable
{
public :
						MEDocPlayer(TArmeDoc* doc);
	virtual				~MEDocPlayer();
	
	// (un)archiving methods
						MEDocPlayer(ArmeArchiveHandler* unarchiver);
	virtual bool		WriteToArchive(ArmeArchiveHandler* archiver) const;
		
	// two playback methods.
	// these functions must add or mix the data to the supplied RAO buffer
	// WARNING : CONTEXT THREAD IS REALTIME !!! 
	void				PutDataIntoRAO(float* raoTmpDst, float** localBuffers, UInt32 nbFrames);
	
	MEWSObjAudioPlayer*	GetWSObjAudioPlayer(SInt32 i);
	
	void				RecreateTrackMapping();
	
	SInt32				GetNbWSObjAudioPlayers() const;

private : // the array of wsobj players. the first one is always the TLC player
	TArmeDoc*			fDoc;
	SInt32				fNbWSObjPlayers;
	MEWSObjAudioPlayer*	fWSObjPlayers[kMEDocPlayer_MaxNbAudioPlayers];
};

// RT CALLED
inline MEWSObjAudioPlayer* MEDocPlayer::GetWSObjAudioPlayer(SInt32 i)
{
	ZzRtAssert(i < fNbWSObjPlayers);
	return fWSObjPlayers[i];
}

inline SInt32 MEDocPlayer::GetNbWSObjAudioPlayers() const
{
	return fNbWSObjPlayers;
}

