/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
 
#ifndef MEMTGAAudioFile_h
#include "MEMTGAAudioFile.h"
#endif //MEMTGAAudioFile_h

#ifndef ZzVerbose_h
//#include "ZzVerbose.h"
#endif //ZzVerbose_h

//#ifndef ZzDisableVerbose_h
//#include "ZzDisableVerbose.h"
//#endif //ZzDisableVerbose_h

#ifndef ZzException_h
#include "ZzException.h"
#endif //ZzException_h

#define kMEMTGAAudioFileHeaderMagicNumber 'MTGA'

//#define VERBOSE_MEMTGAAudioFile_ReadContentToRingBuffer

#undef VER_DEF_TOPIC
#define VER_DEF_TOPIC MEMTGAAudioFile

MEMTGAAudioFile::MEMTGAAudioFile()
{
	fOpenMode = kNotOpen;
	fFile = NULL;
}

MEMTGAAudioFile::~MEMTGAAudioFile()
{
	ZzAssert(!IsOpenedForWriting()); // we do NOT allow the dtor to close the file because that means no cleanup has been done !
	if(fOpenMode != kNotOpen)
		Close();
}

ArmeErr MEMTGAAudioFile::OpenReadMTGA(const char* path)
{
	ZzAssert(GetOpenMode() == kNotOpen);
	ArmeErr err;
	fFile = fopen(path,"r");
	if(fFile == NULL)
	{
		err.type = kArmeErrCannotOpenFileForReading;
		return err;
	}
	size_t nbReads = fread(&fPartialHeader,sizeof(MEMTGAAudioFileHeader),1,fFile);
	ZzRtAssert(nbReads == 1);
	ZzRtAssert(fPartialHeader.magic == kMEMTGAAudioFileHeaderMagicNumber);
	ZzRtAssert(fPartialHeader.version == 0x00010000);
	fOpenMode = kOpenRead;
	return err;
}

void MEMTGAAudioFile::GetFileInfo(const char* mtgaFilePath,SInt32* nbChannels,SInt64* duration)
{
	MEMTGAAudioFileHeader mtgaFileHeader;
	*nbChannels = 0;
	*duration = 0;
	FILE* file = fopen(mtgaFilePath,"r");
	if(file == NULL)
	{
		ZZ_THROW_1ARG(ZzAudioClipImportException,"Sorry, cannot open internal audio file");
	}
	size_t nbReads = fread(&mtgaFileHeader,sizeof(MEMTGAAudioFileHeader),1,file);
	if(nbReads != 1)
	{
		fclose(file);
		ZZ_THROW_1ARG(ZzAudioClipImportException,"Sorry, cannot read internal audio file header"); 
	}
	if(mtgaFileHeader.magic != kMEMTGAAudioFileHeaderMagicNumber)
	{
		fclose(file);
		ZZ_THROW_1ARG(ZzAudioClipImportException,"Sorry, wrong internal audio file format"); 
	}
	if(mtgaFileHeader.version != 0x00010000)
	{
		fclose(file);
		ZZ_THROW_1ARG(ZzAudioClipImportException,"Sorry, wrong internal audio file version"); 
	}
	*nbChannels = mtgaFileHeader.nbChannels;
	*duration = mtgaFileHeader.nbFrames;
	fclose(file);
}

ArmeErr MEMTGAAudioFile::OpenReadChunk(TArmeDoc* doc, const MEAudioChunk& chunk)
{
	ZzAssert(GetOpenMode() == kNotOpen);
	ArmeErr err;
	char	path[kMaxPathLength];
	chunk.GetPath(path,kMaxPathLength);
	fChunk = chunk;
	fFile = fopen(path,"r");
	if(fFile == NULL)
	{
		err.type = kArmeErrCannotOpenFileForReading;
		fOpenMode = kNotOpen;
		return err;
	}
	size_t nbReads = fread(&fPartialHeader,sizeof(MEMTGAAudioFileHeader),1,fFile);
	ZzRtAssert(nbReads == 1);
	ZzRtAssert(fPartialHeader.magic == kMEMTGAAudioFileHeaderMagicNumber);
	ZzRtAssert(fPartialHeader.version == 0x00010000);
	fOpenMode = kOpenReadChunk;
	return err;
}

const MEAudioChunk& MEMTGAAudioFile::GetOpenChunk()
{
	ZzAssert(fOpenMode == kOpenReadChunk || fOpenMode == kOpenWriteChunk);
	return fChunk;
}

ArmeErr MEMTGAAudioFile::OpenSaveNewChunkFile(
	TArmeDoc* 			doc, 
	const MEAudioChunk&	chunk,
	SInt32				nbChannels)
{
	ZzAssert(GetOpenMode() == kNotOpen);
	fChunk = chunk;
	ArmeErr err;
	char	path[kMaxPathLength];
	chunk.GetPath(path,kMaxPathLength);
	if(chunk.startInFile != 0)
	{
		verntl(MEMTGAAudioFile,0,"CRASH : MEMTGAAudioFile::OpenSaveStereoChunk() ----> chunk.startInFile != 0");
		ZzAssert(0);
	}
	fFile = fopen(path,"w");
	if(fFile == NULL)
	{
		err.type = kArmeErrCannotOpenFileForWriting;
		fOpenMode = kNotOpen;
		return err;
	}
	fPartialHeader.magic = kMEMTGAAudioFileHeaderMagicNumber;
	fPartialHeader.version = 0x00010000;
	fPartialHeader.frameRate = doc->GetAudioFrameRate();
	fPartialHeader.nbChannels = nbChannels;
	fPartialHeader.fmt = kMEMTGAAudioFileFmt_Float32;
	fPartialHeader.nbFrames = 0;
	size_t nbWrites = fwrite(&fPartialHeader,sizeof(MEMTGAAudioFileHeader),1,fFile);
	ZzAssert(nbWrites == 1); //$$$DISKFULL
//	int err = fseek(fFile,sizeof(MEMTGAAudioFileHeader),SEEK_SET);
//	ZzAssert(err==0);
	fOpenMode = kOpenWriteChunk;
	return err;
}

ArmeErr MEMTGAAudioFile::OpenAppend(const char* path)
{
	ZzAssert(GetOpenMode() == kNotOpen);
	ArmeErr err;
	err.type = kArmeErrNYI;
	return err;
}

ArmeErr MEMTGAAudioFile::OpenSaveMTGA(const char* path, UInt32 frameRate, UInt32 nbChannels, MEMTGAAudioFileFmt fmt)
{
	ZzAssert(GetOpenMode() == kNotOpen);
	ArmeErr err;
	fFile = fopen(path,"w");
	if(fFile == NULL)
	{
		err.type = kArmeErrCannotOpenFileForWriting;
		return err;
	}
	fPartialHeader.magic = kMEMTGAAudioFileHeaderMagicNumber;
	fPartialHeader.version = 0x00010000;
	fPartialHeader.frameRate = frameRate;
	fPartialHeader.nbChannels = nbChannels;
	fPartialHeader.fmt = fmt;
	fPartialHeader.nbFrames = 0;
	size_t nbWrites = fwrite(&fPartialHeader,sizeof(MEMTGAAudioFileHeader),1,fFile);
	ZzAssert(nbWrites == 1); //$$$DISKFULL
//	int err = fseek(fFile,sizeof(MEMTGAAudioFileHeader),SEEK_SET);
//	ZzAssert(err==0);
	fOpenMode = kOpenWrite;
	return err;
}

MEMTGAAudioFileFmt MEMTGAAudioFile::GetFmt()
{
	ZzAssert(fOpenMode != kNotOpen);
	return fPartialHeader.fmt;
}

UInt32 MEMTGAAudioFile::GetNbChannels()
{
	ZzAssert(fOpenMode != kNotOpen);
	return fPartialHeader.nbChannels;
}

UInt32 MEMTGAAudioFile::GetFrameRate()
{
	ZzAssert(fOpenMode != kNotOpen);
	return fPartialHeader.frameRate;
}

ArmeErr MEMTGAAudioFile::ReadSInt16Frames(short* samples, SInt64* nbFrames)
{
	ArmeErr err;
	ZzAssert(fOpenMode == kOpenRead);
	ZzAssert(fPartialHeader.fmt == kMEMTGAAudioFileFmt_Short16);
//#define SIN_TEST_MEMTGAAudioFile_ReadSInt16Frames
#ifdef SIN_TEST_MEMTGAAudioFile_ReadSInt16Frames
	FillWithStereoSInt16Sin128(samples,(SInt32)*nbFrames,0);
#else //SIN_TEST_MEMTGAAudioFile_ReadSInt16Frames
	*nbFrames = fread((void*)samples,2*fPartialHeader.nbChannels,*nbFrames,fFile); // 2 == sizeof(short)
#endif //SIN_TEST_MEMTGAAudioFile_ReadSInt16Frames 
	return err;
}

SInt64 MEMTGAAudioFile::GetNbFrames()
{
	return fPartialHeader.nbFrames;
}

ArmeErr MEMTGAAudioFile::ReadFloat32ContentToFlatBuffer(float* buffer, SInt64 start, SInt64 length )
{
	vernl(6,"ReadFloat32ContentToFlatBuffer( start = " << _ArmeTime(start) << " length = " << _ArmeTime(length) << ")");
	ArmeErr err;
	SInt64 nbFrames = GetNbFrames();
	ZzAssert(start >= 0 && start <= nbFrames);
	ZzAssert(length >= 0 && (start+length)<=nbFrames);
	err = Seek(start);
	if(err.type != kArmeOK)
		return err;
	return ReadFloat32Frames(buffer,&length);
}

ArmeErr MEMTGAAudioFile::WriteSInt16Frames(short* samples, SInt64 nbFrames)
{
//	ArmeErr err;
	ZzAssert(fOpenMode == kOpenWrite);
	ZzAssert(fPartialHeader.fmt == kMEMTGAAudioFileFmt_Short16);
//	fwrite((void*)samples,2,nbFrames*fPartialHeader.nbChannels,fFile);
	return ArmeErr(kArmeErrNYI);
}

ArmeErr MEMTGAAudioFile::WriteFloat32Frames(const float* samples, SInt64 nbFrames)
{
	ArmeErr err;
//	ZzAssert(nbFrames <= fTmpBufferNbFrames);
	ZzAssert(fPartialHeader.fmt == kMEMTGAAudioFileFmt_Float32);
	size_t nbWrites = fwrite((void*)samples,4,nbFrames*fPartialHeader.nbChannels,fFile); // 4 is sizeof(float)
	if(nbWrites != nbFrames*fPartialHeader.nbChannels)
		err =  ArmeErr(kArmeErrErrorWritingInFile); //$$$DISKFULL
	return err;
}

ArmeErr MEMTGAAudioFile::ReadSInt24Frames(void* samples, SInt64* nbFrames)
{
	return ArmeErr(kArmeErrNYI);
}

ArmeErr MEMTGAAudioFile::ReadSInt24PaddedFrames(void* samples, SInt64* nbFrames)
{
	return ArmeErr(kArmeErrNYI);
}

ArmeErr MEMTGAAudioFile::ReadFloat32Frames(float* samples, SInt64* nbFrames)
{
	ArmeErr err;
	ZzAssert(IsOpenedForReading());
	ZzAssert(fPartialHeader.fmt == kMEMTGAAudioFileFmt_Float32);
//#define SIN_TEST_MEMTGAAudioFile_ReadSInt16Frames
#ifdef SIN_TEST_MEMTGAAudioFile_ReadSInt16Frames
	FillWithStereoFloatSin128(samples,(SInt32)*nbFrames,0);
#else //SIN_TEST_MEMTGAAudioFile_ReadSInt16Frames
	*nbFrames = fread((void*)samples,4*fPartialHeader.nbChannels,*nbFrames,fFile); // 2 == sizeof(short)
#endif //SIN_TEST_MEMTGAAudioFile_ReadSInt16Frames 
	return err;
}

ArmeErr  MEMTGAAudioFile::WriteFloat32FramesFromFlatBuffer(
	const float*	samples, 
	SInt64 			chunkStartTime, 
	SInt64 			dur)
{
	ArmeErr err = Seek(chunkStartTime);
	if(err.type != kArmeOK)
		return err;
	return WriteFloat32Frames(samples,dur);
}
	

ArmeErr MEMTGAAudioFile::Seek(SInt64 offset)
{
	ArmeErr err;
	off_t _offset = sizeof(MEMTGAAudioFileHeader) + offset*SizeOfFrame(); // $$$TODO stereo SInt16
	int _err = fseeko(fFile,_offset,SEEK_SET);
	if(_err != 0)
		return ArmeErr(kArmeErrErrorSeekingInFile);
	return err;
}


SInt32 MEMTGAAudioFile::SizeOfSample() const
{
	switch(fPartialHeader.fmt)
	{
	case kMEMTGAAudioFileFmt_Short16 : 
		return 2;
	case kMEMTGAAudioFileFmt_Float32 :
		return 4;
	default :
		ZzAssert(0);
		return -1;
	}
}


SInt32 MEMTGAAudioFile::SizeOfFrame() const
{
	return SizeOfSample()*fPartialHeader.nbChannels;
}


ArmeErr MEMTGAAudioFile::Close()
{
	ArmeErr err;
	if(fOpenMode == kNotOpen)
		return err;
	if(IsOpenedForWriting())
	{
		// we are going to measure how many frames have been written fseek 
		// so far
		int _err = fseek(fFile,0,SEEK_END);
		if(_err != 0)
			return ArmeErr(kArmeErrErrorSeekingInFile);
		off_t fileSize = ftello(fFile);
		fileSize -= sizeof(MEMTGAAudioFileHeader);
		// ZzAssert((fileSize % SizeOfFrame()) == 0); ***marcp*** commented, as i need to quietly close the file, when an error occurs
		fPartialHeader.nbFrames =  fileSize / SizeOfFrame();
		_err = fseek(fFile,0,SEEK_SET);
		if(_err != 0)
			return ArmeErr(kArmeErrErrorSeekingInFile);
		size_t nbWrites = fwrite(&fPartialHeader,sizeof(MEMTGAAudioFileHeader),1,fFile);
		if(nbWrites != 1)
			return ArmeErr(kArmeErrErrorWritingInFile); //$$$DISKFULL
	}
//	if(fTmpBuffer)
//		delete [] fTmpBuffer;
	fclose(fFile);
	fOpenMode = kNotOpen;
	return err;
}

/*
#define kInv32768 0.000030515578125

void MEMTGAAudioFile::SInt16ToFloat32InPlace(void* buffer, SInt32 nbSamples)
{
	float* floatBuffer = (float*)buffer;
	SInt16* SInt16Buffer = (SInt16*)buffer;
	for( SInt32 i = nbSamples-1; i >= 0; --i)
	{
		floatBuffer[i] = (float)((double)SInt16Buffer[i]*(double)kInv32768);
	}
}
*/

ArmeErr MEMTGAAudioFile::ReadFloat32ContentToRingBuffer(
	float*	ringBuffer, 
	SInt32	ringBufferNbBits,
	SInt64	writePosInRingBuf,
	SInt64	start, 
	SInt64	length)
{
	ArmeErr err;
//	verntl(MEMTGAAudioFile,5,"MEMTGAAudioFile::ReadContentToRB rB: " << ringBuffer << " rBNbBits:" << ringBufferNbBits << " writePosInRB:" << writePosInRingBuf << " start:" << start << " length:" << length);
	SInt32 ringBufferNbFrames = 1 << ringBufferNbBits;
		
	ZzAssert(fOpenMode==kOpenRead || fOpenMode==kOpenReadChunk);
	
	// seek to the start of the range in the file
	err = Seek(start);
	if(err.type != kArmeOK)
		return err;
	
	switch(fPartialHeader.fmt)
	{
	case kMEMTGAAudioFileFmt_Float32 :
		{
			ZzAssert(length <= ringBufferNbFrames);
			UInt32 frameIndexInRingBuffer = writePosInRingBuf & ((1 << ringBufferNbBits) - 1);
			SInt32 sampleIndexInRingBuffer = frameIndexInRingBuffer * fPartialHeader.nbChannels;
			float* dstPtr = ringBuffer + sampleIndexInRingBuffer;
			// part 1 length inframes
			SInt32 part2NbFrames = frameIndexInRingBuffer + length - ( 1 << ringBufferNbBits );
			if(part2NbFrames <= 0)
			{
				SInt64 actualNbFrames = length;
	#ifdef SIN_TEST_MEMTGAAudioFile_ReadContentToRingBuffer
				FillWithStereoFloatSin128(dstPtr,(SInt32)actualNbFrames,0);
	#else //SIN_TEST_MEMTGAAudioFile_ReadContentToRingBuffer
				err = ReadFloat32Frames(dstPtr, &actualNbFrames);
				if(err.type != kArmeOK)
					return err;
	#endif //SIN_TEST_MEMTGAAudioFile_ReadContentToRingBuffer 
			}
			else
			{
				SInt32 part1NbFrames = length - part2NbFrames;
				SInt64 actualNbFrames = part1NbFrames;
#ifdef SIN_TEST_MEMTGAAudioFile_ReadContentToRingBuffer
				FillWithStereoFloatSin128(dstPtr,(SInt32)actualNbFrames,0);
#else //SIN_TEST_MEMTGAAudioFile_ReadContentToRingBuffer
				err = ReadFloat32Frames(dstPtr, &actualNbFrames);
				if(err.type != kArmeOK)
					return err;
#endif //SIN_TEST_MEMTGAAudioFile_ReadContentToRingBuffer 
				actualNbFrames = part2NbFrames;
#ifdef SIN_TEST_MEMTGAAudioFile_ReadContentToRingBuffer
				FillWithStereoFloatSin128(dstPtr,(SInt32)actualNbFrames,part1NbFrames);
#else //SIN_TEST_MEMTGAAudioFile_ReadContentToRingBuffer
				err = ReadFloat32Frames(dstPtr, &actualNbFrames);
				if(err.type != kArmeOK)
					return err;
#endif //SIN_TEST_MEMTGAAudioFile_ReadContentToRingBuffer 
			}
		}
		break;
	default :
		ZzAssert(0);
		break;
	}
	return err;
}

double MEMTGAAudioFile::GetDurationSec()
{
	ZzAssert(fOpenMode == MEMTGAAudioFile::kOpenRead);
	return (double)fPartialHeader.nbFrames/(double)fPartialHeader.frameRate;
}

