/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
 
#define MEMTGAAudioFile_h
#ifndef MEMTGAAudioFile_h
#include "MEMTGAAudioFile.h"
#endif //MEMTGAAudioFile_h

#ifndef MEAudioChunkList_h
#include "MEAudioChunkList.h"
#endif //MEAudioChunkList_h

#ifndef ZzRABI_h
#include "ZzRABI.h"
#endif //ZzRABI_h

/*! 	
	@struct MEMTGAAudioFileHeader 
*/
enum MEMTGAAudioFileFmt
{
	kMEMTGAAudioFileFmt_Short16 = 0,
	kMEMTGAAudioFileFmt_Float32
};

struct MEMTGAAudioFileHeader
{
	UInt32			magic;
	UInt32			version;
	UInt32			frameRate;
	UInt32			nbChannels;
	MEMTGAAudioFileFmt	fmt;
	SInt64			nbFrames;
};

/*!
	@class MEMTGAAudioFile
	
	This class allows to open and save MTGA audio files (proprietaty formtat 
*/
class TArmeDoc;

class MEMTGAAudioFile// : public MArmeNamedRefCount
{
public :
	// enum { kFloat32Format, kInt16Format, kInt24Format, kInt32Format, kFloat64Format };
	// enum { kSeekFromStart, kSeekFromCur, kSeekFromEnd };
	// some errors are handled, others are ZzAssert(0) so beware. no exceptions here.
	enum{ kNotOpen, kOpenRead, kOpenReadChunk, kOpenWrite, kOpenWriteChunk };

public :
	MEMTGAAudioFile();
	~MEMTGAAudioFile();
	
	
	ArmeErr					OpenReadChunk(TArmeDoc* doc, const MEAudioChunk& chunk);
	ArmeErr					OpenSaveNewChunkFile(
								TArmeDoc* 			doc, 
								const MEAudioChunk&	chunk,
								SInt32				channels);
	ArmeErr					ReadFloat32Frames(float* samples, SInt64* nbFrames);
	ArmeErr					ReadFloat32ContentToFlatBuffer(float* buffer, SInt64 start, SInt64 length );
	ArmeErr					ReadFloat32ContentToRingBuffer(
								float*	ringBuffer, 
								SInt32	ringBufferNbBits,
								SInt64	writePosInRingBuf,
								SInt64	start, 
								SInt64	length); // to be deleted and moved in the parent code
	ArmeErr					WriteFloat32FramesFromFlatBuffer(const float* samples, SInt64 chunkStartTime, SInt64 dur); // it's only a seek THEN a WriteFloat32Frames
	ArmeErr					WriteFloat32Frames(const float* samples, SInt64 nbFrames);
	ArmeErr					Close();
	SInt64					GetNbFrames();
	UInt32					GetNbChannels();
	ArmeErr					Seek(SInt64 offset);
	UInt32					GetOpenMode();
	const MEAudioChunk&		GetOpenChunk();
	ArmeErr					OpenReadMTGA(const char* path);
	ArmeErr					OpenSaveMTGA(
								const char*		path, 
								UInt32			frameRate, 
								UInt32			nbChannels, 
								MEMTGAAudioFileFmt	fmt);
	bool					IsOpenedForWriting() const;
	
	
	// static function allow to retrieve info on a fully created MTGA file
	static void 			GetFileInfo(const char* mtgaFilePath,SInt32* nbChannels,SInt64* duration);

private :
	ArmeErr	OpenAppend(const char* path);

//	ArmeErr	OpenAppend(TArmeDoc* doc, MEAudioChunk& chunk);
	
	MEMTGAAudioFileFmt	GetFmt();
	UInt32				GetFrameRate();
	double				GetDurationSec();
		
	SInt32 	SizeOfSample() const;
	SInt32 	SizeOfFrame() const;
	ArmeErr	ReadSInt16Frames(short* samples, SInt64* nbFrames);
	ArmeErr	ReadSInt24Frames(void* samples, SInt64* nbFrames);
	ArmeErr	ReadSInt24PaddedFrames(void* samples, SInt64* nbFrames);



	/*! @function SetTmpBufferNbFrames After calling OpenSave, you need to call this function
	    and tell the audiofile object the max number of frames you'll pass in a WriteXXXFrames
	    call when the sample type (float,SInt32) is not the same as the file format 
		(for example, when calling WriteFloat32Frames on a standard SInt16 file */
	void 				SetTmpBufferNbFrames(SInt32 nbFrames);
	
	ArmeErr				WriteSInt16Frames(short* samples, SInt64 nbFrames);
	

	/*! @function SInt16ToFloat32InPlace Converts a buffer of SInt16 from -32768 to 32767 to floats
	    between -1 and 1 */
//	static void	SInt16ToFloat32InPlace(void* buffer, SInt32 nbSamples);
	
	
	bool				IsOpenedForReading() const;
	

private :
	MEAudioChunk		fChunk; // this member is valid only if the file was opened for reading or writing a chunk
	UInt32				fOpenMode;
	FILE*				fFile;
	MEMTGAAudioFileHeader	fPartialHeader;

	/*! @var fTmpBuffer Temporary buffer used for write buffer float->SInt16 conversion 
	    $$$TODO allow for other conversion formats */
//	SInt16*				fTmpBuffer;

	/*! @var fTmpBufferNbFrames Number of frames in fTmpBuffer */
//	SInt32				fTmpBufferNbFrames;
};

inline UInt32 MEMTGAAudioFile::GetOpenMode()
{
	return fOpenMode;
}

inline bool MEMTGAAudioFile::IsOpenedForWriting() const
{
	// writing this in one line fools cpp_precomp for some reason with gcc 2.95.2 on MacOS X (Dec 2002 Dev Tools)
	bool b1 = (fOpenMode == kOpenWrite);
	bool b2 = (fOpenMode == kOpenWriteChunk);
	return b1 || b2;
}

inline bool MEMTGAAudioFile::IsOpenedForReading() const
{
	// writing this in one line fools cpp_precomp for some reason with gcc 2.95.2 on MacOS X (Dec 2002 Dev Tools)
	bool b1 = (fOpenMode == kOpenRead);
	bool b2 = (fOpenMode == kOpenReadChunk);
	return b1 || b2;
}

class ZzRABI_CloseMEMTGAAudioFile : public ZzRABI
{
public :
	ZzRABI_CloseMEMTGAAudioFile(MEMTGAAudioFile* audioFile) { fAudioFile = audioFile; }
	~ZzRABI_CloseMEMTGAAudioFile() { if(fDeleteResource) { fAudioFile->Close(); } };
public :
	MEMTGAAudioFile* fAudioFile;
};

