/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#ifndef MEMTGAPlayer_h
#include "MEMTGAPlayer.h"
#endif //MEMTGAPlayer_h

#ifndef MEMTGAAudioFile_h
#include "MEMTGAAudioFile.h"
#endif //MEMTGAAudioFile_h

#ifndef ZzAudioUtilities_h
#include "ZzAudioUtilities.h"
#endif //ZzAudioUtilities_h

MEMTGAPlayer::MEMTGAPlayer()
{
	fAudioFile = NULL;
}

ArmeErr	MEMTGAPlayer::Initialize(TArmeDoc* doc, const MEAudioChunkFileIndex& fileIndex)
{
	char	path[kMaxPathLength];
	ArmeErr	err;

	// retrieve the file path
	doc->AudioChunkFileManager()->GetPath(path,kMaxPathLength,fileIndex);

	// build the audio file object
	fAudioFile = new MEMTGAAudioFile();
	
	err = fAudioFile->OpenReadMTGA(path);

	//void Initialize(SInt32 nbChannels, SInt32 log2LatencyNbFrames);

	MEBufferMachine::Initialize(fAudioFile->GetNbChannels(),13); // 1 << 13 frames of latency is ok (8192 frames)
	
	return err;
}

void MEMTGAPlayer::MixCurrentDataIntoBuffer(float* raoDest, SInt32 raoNbChannels, float* tmpStereoBuf,SInt32 nbFrames)
{
	ReadFrames(tmpStereoBuf,nbFrames);
//	printf("MEMTGAPlayer::MixCurrentDataIntoBuffer     tmpStereoBuf[0] = %f\n",tmpStereoBuf[0]);
//	printf("MEMTGAPlayer::MixCurrentDataIntoBuffer writing %p to %p for %d frames  tmpStereoBuf[0] = %f tmpStereoBuf[256] = %f tmpStereoBuf[511] = %f\n",
//		tmpStereoBuf,raoDest,nbFrames,tmpStereoBuf[0],tmpStereoBuf[256],tmpStereoBuf[511]);
	ZzAudioMixStereoWithStrides(
		raoDest,  // dest  buffer
		0, // index of LEFT channel in the dest buffer 
		raoNbChannels, // nb channels in dest buffer
		tmpStereoBuf, // source stereo buffer
		0, // index of LEFT channel in the source buffer
		2, // nb of source channels
		nbFrames );
	
}
void MEMTGAPlayer::FeederFunction(float* dst, SInt64 startPos, SInt32 nbFrames)
{
	fAudioFile->ReadFloat32ContentToFlatBuffer(dst,startPos,nbFrames);
//	printf("MEMTGAPlayer::FeederFunction startPos = %d nbFrames = %d --> dst[0] = %f\n",(int)startPos,(int)nbFrames,dst[0]);
}

SInt64 MEMTGAPlayer::GetMediaNbFrames() const
{
	ZzRtAssert(fAudioFile);
	return fAudioFile->GetNbFrames();
}

MEMTGAPlayer::~MEMTGAPlayer()
{
}

