/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#ifndef MEMediaBufferMachine_h
#include "MEMediaBufferMachine.h"
#endif //MEMediaBufferMachine_h

#ifndef ZzVerbose_h
#include "ZzVerbose.h"
#endif //ZzVerbose_h

void MEMediaBufferMachine::Preroll(SInt64 pos)
{
}

void MEMediaBufferMachine::Seek(SInt64 pos)
{
	fCurFeederPos = pos+LatencyNbFrames();
}

SInt64 MEMediaBufferMachine::GetPlayPos() const
{
	return fCurFeederPos - LatencyNbFrames();
}

MEMediaBufferMachine::MEMediaBufferMachine()
{
	fLooped = false;
	fIsPlaying = true;
	fCurFeederPos = 0;
	fFeederProc = NULL;
	fGetMediaNbFramesProc = NULL;
	fCookie = NULL;
}

void MEMediaBufferMachine::SetLoop(bool isLooped)
{
	fLooped = isLooped;
}

bool MEMediaBufferMachine::IsLooped() const
{
	return fLooped;
}

void MEMediaBufferMachine::Play()
{
	fIsPlaying = true;
}

void MEMediaBufferMachine::Stop()
{
	fIsPlaying = false;
}

void MEMediaBufferMachine::FeederFunction(float* dst, SInt32 nbFrames)
{
//	printf("MEMediaBufferMachine::FeederFunction writing from %p to %p (%d float)\n",dst,dst+nbFrames*fNbChannels,nbFrames*fNbChannels);
	if(!fIsPlaying)
	{
		memset(dst,0,nbFrames*GetNbChannels());
	}
	else
	{
		if(fCurFeederPos >= GetMediaNbFrames())
		{
			memset(dst,0,nbFrames*GetNbChannels());
			fIsPlaying = false;
		}
		else
		{
				
			if(fCurFeederPos + nbFrames > GetMediaNbFrames()) //$$$TODO loop nice
			{
				SInt32 nbFramesToPlay = GetMediaNbFrames() - fCurFeederPos;
				FeederFunction(dst, fCurFeederPos, nbFramesToPlay);
				memset(dst+nbFramesToPlay*GetNbChannels(), 0, (nbFrames-nbFramesToPlay)*GetNbChannels()*sizeof(float));
				fIsPlaying = false;
				fCurFeederPos = 0;
			}
			else
			{
				FeederFunction(dst, fCurFeederPos, nbFrames);
				fCurFeederPos += nbFrames;
			}
		}
	}
}

void MEMediaBufferMachine::ReadFrames(float* dst, SInt32 nbFrames)
{
//	printf("MEBufferMachine::ReadFrames into %p (%d frames)\n",dst,(int)nbFrames);
	if(!fIsPlaying)
		memset(dst,0,GetNbChannels()*nbFrames*sizeof(float));
	else
		MEBufferMachine::ReadFrames(dst, nbFrames);
}

void MEMediaBufferMachine::FeederFunction(float* dst, SInt64 startPos, SInt32 nbFrames)
{
	ZzAssert(fFeederProc);
	return fFeederProc(fCookie,dst,startPos,nbFrames);
}

SInt64 MEMediaBufferMachine::GetMediaNbFrames() const
{
//	ZzAssert(fReadFramesProc);
	return fGetMediaNbFramesProc(fCookie);
}

void MEMediaBufferMachine::SetCallbacks(FeederFunctionProcPtr feederProc, GetMediaNbFramesProcPtr getMediaNbFramesProc, void* cookie)
{
	fFeederProc = feederProc;
	fGetMediaNbFramesProc = getMediaNbFramesProc;
	fCookie = cookie;
}



