/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#define MEMediaBufferMachine_h

#ifndef MEMediaBufferMachine_h
#include "MEMediaBufferMachine.h"
#endif //MEMediaBufferMachine_h

#ifndef MEBufferMachine_h
#include "MEBufferMachine.h"
#endif //MEBufferMachine_h

/*
		@class MEMediaBufferMachine
			
		To use this class, either subclass it and override the FeederFunction() and GetMediaNbFrames()
		or $$$TODO use the callbacks
*/

typedef void (*FeederFunctionProcPtr)(void* cookie, float* dst, SInt64 startPos, SInt32 nbSamples);
typedef SInt64 (*GetMediaNbFramesProcPtr)(void* cookie);

class MEMediaBufferMachine : public MEBufferMachine
{
public :
					MEMediaBufferMachine();
	virtual ~MEMediaBufferMachine() {};
					
	void			SetLoop(bool isLooped);
	bool			IsLooped(bool isLooped) const;

	SInt64			GetPlayPos() const;
	void			Play();
	void			Stop(); // rewinds to 0 ?
	bool 			IsLooped() const;

	void			Seek(SInt64 pos);
	void			Preroll(SInt64 pos);
	
public : //MEBufferMachine override
	virtual void	FeederFunction(float* dst, SInt32 nbSamples);
	virtual void 	ReadFrames(float* dst, SInt32 nbFrames);
	void			SetCallbacks(FeederFunctionProcPtr feederProc, GetMediaNbFramesProcPtr getMediaNbFramesProc, void* cookie = NULL);

public :
	virtual void	FeederFunction(float* dst, SInt64 startPos, SInt32 nbFrames);
	virtual SInt64	GetMediaNbFrames() const;	
	
	bool					fLooped;
	bool					fIsPlaying;
	SInt64					fCurFeederPos;
	FeederFunctionProcPtr	fFeederProc;
	GetMediaNbFramesProcPtr	fGetMediaNbFramesProc;
	void*					fCookie;
};
