/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#ifndef MEMixingInfo_h
#include "MEMixingInfo.h"
#endif //MEMixingInfo_h

#ifndef ArmeNotifications_h
#include "ArmeNotifications.h"
#endif //ArmeNotifications_h

#ifndef TArmeTLC_h
#include "TArmeTLC.h"
#endif //TArmeTLC_h

#ifndef TArmeDoc_h
#include "TArmeDoc.h"
#endif //TArmeDoc_h

#ifndef TArmeMediaEngine_h
#include "TArmeMediaEngine.h"
#endif //TArmeMediaEngine_h

// we need to ask the core audio player for the number of RAOs
#ifndef MECoreAudioPlayer_h
#include "MECoreAudioPlayer.h"
#endif MECoreAudioPlayer_h

#ifndef ZzHelpers_h
#include "ZzHelpers.h"
#endif //ZzHelpers_h

#ifndef DSP_Globals_h
#include "DSP_Globals.h"
#endif // DSP_Globals_h

#undef VER_DEF_TOPIC
#define VER_DEF_TOPIC MEMixingInfo

bool MEMixingInfo::WriteToArchive(ArmeArchiveHandler* archiver) const
{
	ArmeArchivable::WriteToArchive(archiver);
	archiver->WriteCPPObject(fDoc);
	SInt32 i;
	for(i = 0; i < kMEMixingInfo_NbMaxMonoVAO; i++)
	{
		fMonoVAOInfos[i].WriteToArchive(archiver);
	}
	for(i = 0; i < kMEMixingInfo_NbMaxStereoVAO; i++)
	{
		fStereoVAOInfos[i].WriteToArchive(archiver);
	}
	for(i = 0; i < kMEMixingInfo_NbMaxRAO; i++)
	{
		fStereoRAOInfos[i].WriteToArchive(archiver);
	}
	archiver->WriteSInt32(fSoloedMonoVAO);
	archiver->WriteSInt32(fSoloedStereoVAO);
	archiver->WriteSInt32(fSoloedStereoRAO);
	archiver->WriteSInt32(fMaxStereoVAOCount);
	archiver->WriteSInt32(fMaxMonoVAOCount);

	return true;
}

MEMixingInfo::~MEMixingInfo()
{
	DeallocateVAOMirrorBuffers();
}

void MEVAOInfo::WriteToArchive(ArmeArchiveHandler* archiver) const
{
	archiver->WriteBool(bStereo);
	archiver->WriteSInt32(fRAOIndex);
	archiver->WriteFloat(fLeftOrMonoGain);
	archiver->WriteFloat(fRightGain);
	archiver->WriteFloat(fPan);
	archiver->WriteFloat(fMuted);
	archiver->WriteRawData((void*)fName,kBusNameLen);
	archiver->WriteBool(fStereoLock);		// whether stereo sliders are locked
}

void MEVAOInfo::ReadFromArchive(ArmeArchiveHandler* unarchiver)
{
	bStereo = unarchiver->ReadBool();
	fRAOIndex = unarchiver->ReadSInt32();
	fLeftOrMonoGain = unarchiver->ReadFloat();
	fOldLeftOrMonoGain = fLeftOrMonoGain; //DEZIP
	fRightGain = unarchiver->ReadFloat();
	fOldRightGain = fRightGain; //DEZIP
	fOldRAOLeftGain = 0.0;
	fOldRAORightGain = 0.0;
	fPan = unarchiver->ReadFloat();
	fOldPan = fPan; //DEZIP
	fMuted = unarchiver->ReadFloat();
	fOldMuted = fMuted; //DEZIP
	unarchiver->ReadRawData((void*)fName,kBusNameLen);
	fStereoLock = unarchiver->ReadBool();		// whether stereo sliders are locked
}

void MERAOInfo::WriteToArchive(ArmeArchiveHandler* archiver) const
{
	archiver->WriteFloat(fLeftGain);
	archiver->WriteFloat(fRightGain);
	archiver->WriteFloat(fMuted);
	archiver->WriteRawData((void*)fName,kBusNameLen);
}

void MERAOInfo::ReadFromArchive(ArmeArchiveHandler* unarchiver)
{
	fLeftGain = unarchiver->ReadFloat();
	fRightGain = unarchiver->ReadFloat();
	fMuted = unarchiver->ReadFloat();
	fOldMuted = fMuted; //DEZIP
	unarchiver->ReadRawData((void*)fName,kBusNameLen);
}

MEMixingInfo::MEMixingInfo(ArmeArchiveHandler* unarchiver) :
	ArmeArchivable(unarchiver)
{
	ArmeArchivable* archObj;
	archObj = unarchiver->ReadCPPObject();
	fDoc = dynamic_cast<TArmeDoc*>(archObj);
	fDoc->Subscribe(this); // listen to document notifications
	SInt32 i;
	
	for(i = 0; i < kMEMixingInfo_NbMaxMonoVAO; i++)
	{
		fMonoVAOInfos[i].ReadFromArchive(unarchiver);
	}
	for(i = 0; i < kMEMixingInfo_NbMaxStereoVAO; i++)
	{
		fStereoVAOInfos[i].ReadFromArchive(unarchiver);
	}
	for(i = 0; i < kMEMixingInfo_NbMaxRAO; i++)
	{
		fStereoRAOInfos[i].ReadFromArchive(unarchiver);
	}
	fSoloedMonoVAO = unarchiver->ReadSInt32();
	fSoloedStereoVAO = unarchiver->ReadSInt32();
	fSoloedStereoRAO = unarchiver->ReadSInt32();
	
	for(i = 0; i < kMEMixingInfo_NbMaxMonoVAO; i++)
	{
		fMonoVAOSatur[i] = false;
		fMonoVAODBFS[i] = -1000.0;
	}
	for(i = 0; i < kMEMixingInfo_NbMaxStereoVAO; i++)
	{
		fStereoVAOLeftSatur[i] = false;
		fStereoVAORightSatur[i] = false;
		fStereoVAOLeftDBFS[i] = -1000.0;
		fStereoVAORightDBFS[i] = -1000.0;
	}
	for(i = 0; i < kMEMixingInfo_NbMaxRAO; i++)
	{
		fStereoRAOLeftSatur[i] = false;
		fStereoRAORightSatur[i] = false;
		fRAOLeftDBFS[i] = -1000.0;
		fRAORightDBFS[i] = -1000.0;
	}

	fMaxStereoVAOCount = unarchiver->ReadSInt32();
	fMaxMonoVAOCount = unarchiver->ReadSInt32();

/*	OLD CODE InitCore... calls AllocateVAOMirrorBuffers -> crash because fNbAllocatedMonoVAO... is not initialized
	InitCoreAudioDependentInfo();

	fNbAllocatedMonoVAOMirrorBuffers = 0;
	fNbAllocatedStereoVAOMirrorBuffers = 0;
	fMonoVAOMirrorBuffers = NULL;
	fStereoVAOMirrorBuffers = NULL;
	AllocateVAOMirrorBuffers();*/

	fNbAllocatedMonoVAOMirrorBuffers = 0;
	fNbAllocatedStereoVAOMirrorBuffers = 0;
	fMonoVAOMirrorBuffers = NULL;
	fStereoVAOMirrorBuffers = NULL;
	InitCoreAudioDependentInfo();
}

MEMixingInfo::MEMixingInfo(TArmeDoc* doc)
{
	fMaxAssignedStereoRAOCount = 0;
	fMaxStereoVAOCount = 0;
	fMaxMonoVAOCount = 0;
	fNbAllocatedMonoVAOMirrorBuffers = 0;
	fNbAllocatedStereoVAOMirrorBuffers = 0;
	fMonoVAOMirrorBuffers = NULL;
	fStereoVAOMirrorBuffers = NULL;
	
	fDoc = doc;
	fDoc->Subscribe(this); // listen to document notifications
	InitCoreAudioDependentInfo();
	SInt32 i;
	char name[kBusNameLen];
	
	for(i = 0; i < kMEMixingInfo_NbMaxMonoVAO; i++)
	{
		fMonoVAOInfos[i].bStereo = true;
		fMonoVAOInfos[i].fRAOIndex = 0; // route to first output.... maybe mofify this for mtrack device
		fMonoVAOInfos[i].fLeftOrMonoGain = 1.0;
		fMonoVAOInfos[i].fOldLeftOrMonoGain = 1.0;
		fMonoVAOInfos[i].fRightGain = 1.0; //useless
		fMonoVAOInfos[i].fOldRightGain = 1.0; //useless
		fMonoVAOInfos[i].fOldRAOLeftGain = 0.0;
		fMonoVAOInfos[i].fOldRAORightGain = 0.0;
		fMonoVAOInfos[i].fPan = 0.0;
		fMonoVAOInfos[i].fOldPan = 0.0;
		fMonoVAOInfos[i].fMuted = 1.0;
		fMonoVAOInfos[i].fOldMuted = 1.0;
		sprintf(name,"Mono %ld",i+1);
		_SetMonoVAOName(i,name);
		
		fMonoVAOSatur[i] = false;
		fMonoVAODBFS[i] = -1000.0;
	}
	for(i = 0; i < kMEMixingInfo_NbMaxStereoVAO; i++)
	{
		fStereoVAOInfos[i].bStereo = true;
		fStereoVAOInfos[i].fRAOIndex = 0; // route to first output.... maybe mofify this for mtrack device
		fStereoVAOInfos[i].fLeftOrMonoGain = 1.0;
		fStereoVAOInfos[i].fOldLeftOrMonoGain = 1.0;
		fStereoVAOInfos[i].fRightGain = 1.0;
		fStereoVAOInfos[i].fOldRightGain = 1.0;
		fStereoVAOInfos[i].fOldRAOLeftGain = 0.0;
		fStereoVAOInfos[i].fOldRAORightGain = 0.0;
		fStereoVAOInfos[i].fPan = 0.0;
		fStereoVAOInfos[i].fOldPan = 0.0;
		fStereoVAOInfos[i].fMuted = 1.0;
		fStereoVAOInfos[i].fOldMuted = 1.0;
		fStereoVAOInfos[i].fStereoLock = true;
		if (i < 4)
			sprintf(name,"Stereo %ld-%ld",2*i+1,2*i+2);
		else
			sprintf(name,"Ster. %ld-%ld",2*i+1,2*i+2);
		
		_SetStereoVAOName(i,name);

		fStereoVAOLeftSatur[i] = false;
		fStereoVAORightSatur[i] = false;
		fStereoVAOLeftDBFS[i] = -1000.0;
		fStereoVAORightDBFS[i] = -1000.0;
	}
	for(i = 0; i < kMEMixingInfo_NbMaxRAO; i++)
	{
		fStereoRAOInfos[i].fLeftGain = 1.0f;
		fStereoRAOInfos[i].fRightGain = 1.0f;
		fStereoRAOInfos[i].fMuted = 1.0;
		fStereoRAOInfos[i].fOldMuted = 1.0;
		fStereoRAOInfos[i].fStereoLock = true;
		sprintf(name,"<UNKNOWN> %ld-%ld",2*i+1,2*i+2); // this will be set in InitCoreAudioDependentInfo()
		_SetStereoRAOName(i,name);

		fStereoRAOLeftSatur[i] = false;
		fStereoRAORightSatur[i] = false;
		fRAOLeftDBFS[i] = -1000.0;
		fRAORightDBFS[i] = -1000.0;
	}
	
	fSoloedMonoVAO = -1;
	fSoloedStereoVAO = -1;
	fSoloedStereoRAO = -1;
}

SInt32 MEMixingInfo::GetMaxStereoVAOCount() const
{
	return fMaxStereoVAOCount;
}

SInt32 MEMixingInfo::GetMaxMonoVAOCount() const
{
	return fMaxMonoVAOCount;
}

// RT CALLED
SInt32 MEMixingInfo::GetMaxMixerDisplayStereoVAOCount() const
{
	if(fMaxStereoVAOCount > 4)
		return fMaxStereoVAOCount;
	else
		return 4;
}

SInt32 MEMixingInfo::GetMaxMixerDisplayMonoVAOCount() const
{
	if(fMaxMonoVAOCount > 4)
		return fMaxMonoVAOCount;
	else
		return 4;
}

SInt32 MEMixingInfo::GetMaxAssignedStereoRAOCount() const
{
//	return fMaxAssignedStereoRAOCount; $$$TODO TODO TODO
	return fMaxStereoRAOCount; //depends upon the hardware
}

SInt32 MEMixingInfo::GetMaxStereoRAOCount() const
{
	return fMaxStereoRAOCount; //depends upon the hardware
}

float MEMixingInfo::GetDBFSLevelForMonoVAO(SInt32 vaoIndex) const
{
	ZzAssert(vaoIndex < GetMaxMixerDisplayMonoVAOCount());
	// now it's time to convert from amplitude value to dB and from dB to slider
	// value.
	return fMonoVAODBFS[vaoIndex];
}

float MEMixingInfo::GetDBFSLevelForStereoVAO(SInt32 vaoIndex, bool bLeft) const
{
//	vernl(6,"MEMixingInfo::GetDBFSLevelForStereoVAO( vaoIndex = " << vaoIndex << ", bool bLeft = " << bLeft << ")");
	// now it's time to convert from amplitude value to dB and from dB to slider
	// value.
	ZzAssert(vaoIndex < GetMaxMixerDisplayStereoVAOCount());
	if(bLeft)
	{
//		vernl(6,"MEMixingInfo::GetDBFSLevelForStereoVAO() : returning fStereoVAOLeftDBFS[vaoIndex] = " << fStereoVAOLeftDBFS[vaoIndex]);
		return fStereoVAOLeftDBFS[vaoIndex];
	}
	else
	{
//		vernl(6,"MEMixingInfo::GetDBFSLevelForStereoVAO() : returning fStereoVAORightDBFS[vaoIndex] = " << fStereoVAORightDBFS[vaoIndex]);
		return fStereoVAORightDBFS[vaoIndex];
	}
}

float MEMixingInfo::GetDBFSLevelForStereoRAO(SInt32 raoIndex, bool bLeft) const
{
	// now it's time to convert from amplitude value to dB and from dB to slider
	// value.
	ZzAssert(raoIndex < GetMaxStereoRAOCount());
	if(bLeft)
		return fRAOLeftDBFS[raoIndex];
	else
		return fRAORightDBFS[raoIndex];
}

float MEMixingInfo::GetStereoVAOLinearGain(SInt32 vaoIndex, bool bLeft) const
{
	ZzAssert(vaoIndex < GetMaxMixerDisplayStereoVAOCount());
	if(bLeft)
		return fStereoVAOInfos[vaoIndex].fLeftOrMonoGain;
	else
		return fStereoVAOInfos[vaoIndex].fRightGain;
}

float MEMixingInfo::GetMonoVAOLinearGain(SInt32 vaoIndex) const
{
	ZzAssert(vaoIndex < GetMaxMixerDisplayMonoVAOCount());
	return fMonoVAOInfos[vaoIndex].fLeftOrMonoGain;
}

float MEMixingInfo::GetStereoRAOLinearGain(SInt32 raoIndex, bool bLeft) const
{
	ZzAssert(raoIndex < fMaxStereoRAOCount);
	if(bLeft)
		return fStereoRAOInfos[raoIndex].fLeftGain;
	else
		return fStereoRAOInfos[raoIndex].fRightGain;
}

float MEMixingInfo::GetStereoVAOFaderDB(SInt32 vaoIndex, bool bLeft) const
{
	ZzAssert(vaoIndex < GetMaxMixerDisplayStereoVAOCount());
	if(bLeft)
		return linearGainTodB(fStereoVAOInfos[vaoIndex].fLeftOrMonoGain);
	else
		return linearGainTodB(fStereoVAOInfos[vaoIndex].fRightGain);
}

float MEMixingInfo::GetMonoVAOFaderDB(SInt32 vaoIndex) const
{
	ZzAssert(vaoIndex < GetMaxMixerDisplayMonoVAOCount());
	return linearGainTodB(fMonoVAOInfos[vaoIndex].fLeftOrMonoGain);
}

float MEMixingInfo::GetStereoRAOFaderDB(SInt32 raoIndex, bool bLeft) const
{
	ZzAssert(raoIndex < fMaxStereoRAOCount);
	if(bLeft)
		return linearGainTodB(fStereoRAOInfos[raoIndex].fLeftGain);
	else
		return linearGainTodB(fStereoRAOInfos[raoIndex].fRightGain);
}

SInt32 MEMixingInfo::GetStereoRAOForStereoVAO(SInt32 vaoIndex) const
{
	ZzRtAssert(vaoIndex < GetMaxMixerDisplayStereoVAOCount());
	ZzRtAssert(fStereoVAOInfos[vaoIndex].bStereo);
	if(fStereoVAOInfos[vaoIndex].fRAOIndex < fMaxStereoRAOCount)
	{
		return fStereoVAOInfos[vaoIndex].fRAOIndex;
	}
	else
	{
		return 0;
	}
}

bool MEMixingInfo::IsMonoVAORoutedToStereoRAO(SInt32 vaoIndex) const
{
	ZzAssert(vaoIndex < GetMaxMixerDisplayMonoVAOCount());
	return fMonoVAOInfos[vaoIndex].bStereo;
}

float MEMixingInfo::GetMonoVAOPanning(SInt32 vaoIndex) const
{
	ZzAssert(vaoIndex < GetMaxMixerDisplayMonoVAOCount());
	return fMonoVAOInfos[vaoIndex].fPan;
}

float MEMixingInfo::GetStereoVAOPanning(SInt32 stereoVAOIndex) const
{
	ZzAssert(stereoVAOIndex < GetMaxMixerDisplayStereoVAOCount());
	return fStereoVAOInfos[stereoVAOIndex].fPan;
}

//RT
SInt32 MEMixingInfo::GetStereoRAOForMonoVAO(SInt32 vaoIndex) const
{
	ZzRtAssert(vaoIndex < GetMaxMixerDisplayMonoVAOCount());
	if(fMonoVAOInfos[vaoIndex].fRAOIndex < fMaxStereoRAOCount)
	{
		return fMonoVAOInfos[vaoIndex].fRAOIndex;
	}
	else
	{
		return 0;
	}
}

void MEMixingInfo::OnVAOChange(bool stereo, SInt32 vaoIndex)
{
//	bool notifyMixer = false; 
	if(stereo && vaoIndex > fMaxStereoVAOCount-1)
	{
		fMaxStereoVAOCount = vaoIndex+1;
		fDoc->Notify(kNotification_Mixer,0);
	}
	if(!stereo && vaoIndex > fMaxMonoVAOCount-1)
	{
		fMaxMonoVAOCount = vaoIndex+1;
		fDoc->Notify(kNotification_Mixer,0);
	}
}	

void MEMixingInfo::NotificationReceived( ZzBroadcaster* source, UInt32 code, void* param)
{
	switch(code)
	{
	case kNotification_VAOChangeToMonoVAO :
		OnVAOChange(false,(SInt32)param);
		AllocateVAOMirrorBuffers(); // this is not expensive if the number of vao hasnt change so lets fire it anyway
		break;
	case kNotification_VAOChangeToStereoVAO :
		OnVAOChange(true,(SInt32)param);
		AllocateVAOMirrorBuffers(); // this is not expensive if the number of vao hasnt change so lets fire it anyway
		break;
	case kNotification_CoreAudioOutputChange :
		InitCoreAudioDependentInfo();
		break;
	};
}

void MEMixingInfo::InitCoreAudioDependentInfo()
{
	double buffersPerSecond = (double)TArmeMediaEngine::CoreAudioPlayer()->GetAudioFrameRate()/(double)TArmeMediaEngine::CoreAudioPlayer()->OutDeviceBufferNbFrames();
	// 40dB/sec
	fDBPerBufferAttenuation = 40.0/buffersPerSecond;

	fNbOfBuffersForSaturationReset = 2*(int)buffersPerSecond;
	if(fNbOfBuffersForSaturationReset == 0) // buffer is greater than a second
		fNbOfBuffersForSaturationReset = 1;
	
	fMaxStereoRAOCount = TArmeMediaEngine::CoreAudioPlayer()->GetNbOutputStereoPairs();
		
	char raoName[256];
	for(int i = 0; i < fMaxStereoRAOCount; i++)
	{
		ZzString devName = TArmeMediaEngine::CoreAudioPlayer()->GetDeviceShortName();
		sprintf(raoName,"%s (%d-%d)",devName.getUTF8String(),2*i+1,2*i+2);
		_SetStereoRAOName(i,raoName);
	}
		
	AllocateVAOMirrorBuffers();
//	fDoc->Notify(kNotification_Mixer,0);
}

// RT CALLED
bool MEMixingInfo::IsMonoVAOMuted(SInt32 monoVAOIndex) const
{
	ZzRtAssert(monoVAOIndex < GetMaxMixerDisplayMonoVAOCount());
	return (fMonoVAOInfos[monoVAOIndex].fMuted == 0.0);
}

// RT CALLED
bool MEMixingInfo::IsStereoVAOMuted(SInt32 stereoVAOIndex) const
{
	ZzRtAssert(stereoVAOIndex < GetMaxMixerDisplayStereoVAOCount());
	return (fStereoVAOInfos[stereoVAOIndex].fMuted == 0.0);
}

// RT CALLED
bool MEMixingInfo::IsStereoRAOMuted(SInt32 stereoRAOIndex) const
{
	ZzRtAssert(stereoRAOIndex < GetMaxStereoRAOCount());
	return (fStereoRAOInfos[stereoRAOIndex].fMuted == 0.0);
}

// RT CALLED
bool MEMixingInfo::IsMonoVAOMutedThroughMutedRAO(SInt32 monoVAOIndex) const
{
	ZzRtAssert(monoVAOIndex < GetMaxMixerDisplayMonoVAOCount());
	// if this VAO is routed to a stereo RAO, then we check whether the RAO is muted
	if(fMonoVAOInfos[monoVAOIndex].bStereo)
		return IsStereoRAOMuted(GetStereoRAOForMonoVAO(monoVAOIndex));
	else
	// if this VAO is routed to a mono RAO, then we check whether the RAO contaning this mono RAO is muted
		return IsStereoRAOMuted((GetStereoRAOForMonoVAO(monoVAOIndex))/2);
}

// RT CALLED
bool MEMixingInfo::IsStereoVAOMutedThroughMutedRAO(SInt32 stereoVAOIndex) const
{
	ZzRtAssert(stereoVAOIndex < GetMaxMixerDisplayStereoVAOCount());
	ZzRtAssert(fStereoVAOInfos[stereoVAOIndex].bStereo);
	return IsStereoRAOMuted(GetStereoRAOForStereoVAO(stereoVAOIndex));
}

void MEMixingInfo::_SetMonoVAOMute(SInt32 monoVAOIndex, bool bMuted)
{
	ZzAssert(monoVAOIndex < GetMaxMixerDisplayMonoVAOCount());
	if(bMuted)
		fMonoVAOInfos[monoVAOIndex].fMuted = 0.0;
	else
		fMonoVAOInfos[monoVAOIndex].fMuted = 1.0;
}

void MEMixingInfo::_SetStereoVAOMute(SInt32 stereoVAOIndex, bool bMuted)
{
	ZzAssert(stereoVAOIndex < GetMaxMixerDisplayStereoVAOCount());
	if(bMuted)
		fStereoVAOInfos[stereoVAOIndex].fMuted = 0.0;
	else
		fStereoVAOInfos[stereoVAOIndex].fMuted = 1.0;
}

void MEMixingInfo::_SetStereoRAOMute(SInt32 stereoRAOIndex, bool bMuted)
{
	ZzAssert(stereoRAOIndex < GetMaxStereoRAOCount());
	if(bMuted)
		fStereoRAOInfos[stereoRAOIndex].fMuted = 0.0;
	else
		fStereoRAOInfos[stereoRAOIndex].fMuted = 1.0;
}

float MEMixingInfo::dBToFaderValue(float dB)
{
	return dBToVULevel(dB);
}

float MEMixingInfo::FaderValueTodB(float fader)
{
	return VULevelTodB(fader);
}

float MEMixingInfo::dBToVULevel(float dB)
{
	if(dB < -192.0)
		return 0.0;
	if(dB < -96.0)
		return ZzGetYFromXAnd2Points(dB,-192.0,0.0000,-96,11.1111);
	if(dB < -48.0)
		return ZzGetYFromXAnd2Points(dB,-96.0,11.1111,-48,22.2222);
	if(dB < -24.0)
		return ZzGetYFromXAnd2Points(dB,-48.0,22.2222,-24,33.3333);
	if(dB < -12.0)
		return ZzGetYFromXAnd2Points(dB,-24,33.3333,-12,44.4444);
	if(dB < -6.0)
		return ZzGetYFromXAnd2Points(dB,-12,44.4444,-6,55.5555);
	if(dB < -3.0)
		return ZzGetYFromXAnd2Points(dB,-6,55.5555,-3,66.6666);
	if(dB < 0.0)
		return ZzGetYFromXAnd2Points(dB,-3,66.6666,0.0,77.7777);
	if(dB < 3.0)
		return ZzGetYFromXAnd2Points(dB,0.0,77.7777,3.0,88.8888);
	if(dB <= 6.0)
		return ZzGetYFromXAnd2Points(dB,3.0,88.8888,6.0,100.0);
//	vernl(1,"WARNING : VALUE > 6dB given to dBToVULevel");
	return 100.0;
}

float MEMixingInfo::VULevelTodB(float vuLevel)
{
	ZzAssert(vuLevel >= 0.0 && vuLevel <= 100.0);
	if(vuLevel < 11.1111)
		return ZzGetYFromXAnd2Points(vuLevel,0.0000,-192.0,11.1111,-96);
	if(vuLevel < 22.2222)
		return ZzGetYFromXAnd2Points(vuLevel,11.1111,-96.0,22.2222,-48.0);
	if(vuLevel < 33.3333)
		return ZzGetYFromXAnd2Points(vuLevel,22.2222,-48.0,33.3333,-24.0);
	if(vuLevel < 44.4444)
		return ZzGetYFromXAnd2Points(vuLevel,33.3333,-24.0,44.4444,-12.0);
	if(vuLevel < 55.5555)
		return ZzGetYFromXAnd2Points(vuLevel,44.4444,-12.0,55.5555,-6.0);
	if(vuLevel < 66.6666)
		return ZzGetYFromXAnd2Points(vuLevel,55.5555,-6.0,66.6666,-3.0);
	if(vuLevel < 77.7777)
		return ZzGetYFromXAnd2Points(vuLevel,66.6666,-3.0,77.7777,0.0);
	if(vuLevel < 88.8888)
		return ZzGetYFromXAnd2Points(vuLevel,77.7777,0.0,88.8888,3.0);
	if(vuLevel < 100.1)
		return ZzGetYFromXAnd2Points(vuLevel,88.8888,3.0,100.1,6.0);
	ZzAssert(0);
	return 6.0;
}

float MEMixingInfo::dBToLinearGain(float dB)
{
	return pow(10.0,dB/20.0);
}

float MEMixingInfo::linearGainTodB(float gain)
{
	return 20.0*log10(gain);
}
	
void MEMixingInfo::_SetMonoVAOPanning(SInt32 monoVAOIndex, float panning)
{
	ZzAssert(monoVAOIndex < GetMaxMixerDisplayMonoVAOCount());
	ZzAssert(panning >= -1.0 && panning <= 1.0);
	fMonoVAOInfos[monoVAOIndex].fPan = panning;
}

void MEMixingInfo::_SetStereoVAOPanning(SInt32 stereoVAOIndex, float panning)
{
	ZzAssert(stereoVAOIndex < GetMaxMixerDisplayStereoVAOCount());
	ZzAssert(panning >= -1.0 && panning <= 1.0);
	fStereoVAOInfos[stereoVAOIndex].fPan = panning;
}

/*
void MEMixingInfo::_SetStereoRAOPanning(SInt32 stereoRAOIndex, float panning)
{
	ZzAssert(stereoRAOIndex < GetMaxStereoRAOCount());
	ZzAssert(panning >= -1.0 && panning <= 1.0);
	fStereoRAOInfos[stereoRAOIndex].fPan = panning;
}
*/

void MEMixingInfo::_SetStereoRAOForStereoVAO(SInt32 stereoVAOIndex, SInt32 stereoRAOIndex)
{
	ZzAssert(stereoVAOIndex < GetMaxMixerDisplayStereoVAOCount());
	ZzAssert(stereoRAOIndex < GetMaxStereoRAOCount());
	fStereoVAOInfos[stereoVAOIndex].bStereo = true;
	fStereoVAOInfos[stereoVAOIndex].fRAOIndex = stereoRAOIndex;
}

void MEMixingInfo::_SetStereoRAOForMonoVAO(SInt32 monoVAOIndex, SInt32 stereoRAOIndex)
{
	ZzAssert(monoVAOIndex < GetMaxMixerDisplayMonoVAOCount());
	ZzAssert(stereoRAOIndex < GetMaxStereoRAOCount());
	fMonoVAOInfos[monoVAOIndex].bStereo = true;
	fMonoVAOInfos[monoVAOIndex].fRAOIndex = stereoRAOIndex;
}

void MEMixingInfo::_SetMonoRAOForMonoVAO(SInt32 monoVAOIndex, SInt32 monoRAOIndex)
{
	ZzAssert(monoVAOIndex < GetMaxMixerDisplayMonoVAOCount());
	ZzAssert(monoRAOIndex < GetMaxStereoRAOCount()*2);
	fMonoVAOInfos[monoVAOIndex].bStereo = false;
	fMonoVAOInfos[monoVAOIndex].fRAOIndex = monoRAOIndex;
}

const char* MEMixingInfo::GetMonoVAOName(SInt32 monoVAOIndex) const
{
	ZzAssert(monoVAOIndex >= 0);
	ZzAssert(monoVAOIndex < GetMaxMixerDisplayMonoVAOCount());
	return fMonoVAOInfos[monoVAOIndex].fName;
}

const char* MEMixingInfo::GetStereoVAOName(SInt32 stereoVAOIndex) const
{
	ZzAssert(stereoVAOIndex >=  0);
	ZzAssert(stereoVAOIndex < GetMaxMixerDisplayStereoVAOCount());
	return fStereoVAOInfos[stereoVAOIndex].fName;
}

const char* MEMixingInfo::GetStereoRAOName(SInt32 stereoRAOIndex) const
{
	ZzAssert(stereoRAOIndex >= 0);
	ZzAssert(stereoRAOIndex < GetMaxStereoRAOCount());
	return fStereoRAOInfos[stereoRAOIndex].fName;
}

void MEMixingInfo::_SetMonoVAOName(SInt32 monoVAOIndex, const char* name)
{
	ZzAssert(monoVAOIndex >= 0);
//	ZzAssert(monoVAOIndex < GetMaxMixerDisplayMonoVAOCount());
	strncpy(fMonoVAOInfos[monoVAOIndex].fName,name,kBusNameLen-1);
	(fMonoVAOInfos[monoVAOIndex].fName)[kBusNameLen-1] = 0;
}

void MEMixingInfo::_SetStereoVAOName(SInt32 stereoVAOIndex, const char* name)
{
	ZzAssert(stereoVAOIndex >=  0);
//	ZzAssert(stereoVAOIndex < GetMaxMixerDisplayStereoVAOCount());
	strncpy(fStereoVAOInfos[stereoVAOIndex].fName,name,kBusNameLen-1);
	(fStereoVAOInfos[stereoVAOIndex].fName)[kBusNameLen-1] = 0;
}

void MEMixingInfo::_SetStereoRAOName(SInt32 stereoRAOIndex, const char* name)
{
	ZzAssert(stereoRAOIndex >= 0);
//	ZzAssert(stereoRAOIndex < GetMaxStereoRAOCount());
	strncpy(fStereoRAOInfos[stereoRAOIndex].fName,name,kBusNameLen-1);
	(fStereoRAOInfos[stereoRAOIndex].fName)[kBusNameLen-1] = 0;
}

bool MEMixingInfo::GetStereoVAOLock(SInt32 stereoVAOIndex) const
{
	ZzAssert(stereoVAOIndex >=  0);
	ZzAssert(stereoVAOIndex < GetMaxMixerDisplayStereoVAOCount());
	return fStereoVAOInfos[stereoVAOIndex].fStereoLock;
}

bool MEMixingInfo::GetStereoRAOLock(SInt32 stereoRAOIndex) const
{
	ZzAssert(stereoRAOIndex >= 0);
	ZzAssert(stereoRAOIndex < GetMaxStereoRAOCount());
	return fStereoRAOInfos[stereoRAOIndex].fStereoLock;
}

void MEMixingInfo::_SetStereoVAOLock(SInt32 stereoVAOIndex, bool locked)
{
	ZzAssert(stereoVAOIndex >=  0);
	ZzAssert(stereoVAOIndex < GetMaxMixerDisplayStereoVAOCount());
	fStereoVAOInfos[stereoVAOIndex].fStereoLock = locked;
}

void MEMixingInfo::_SetStereoRAOLock(SInt32 stereoRAOIndex, bool locked)
{
	ZzAssert(stereoRAOIndex >= 0);
	ZzAssert(stereoRAOIndex < GetMaxStereoRAOCount());
	fStereoRAOInfos[stereoRAOIndex].fStereoLock = locked;
}

bool MEMixingInfo::IsMonoVAOSaturated(SInt32 monoVAOIndex) const
{
	ZzAssert(monoVAOIndex >= 0);
	ZzAssert(monoVAOIndex < GetMaxMixerDisplayMonoVAOCount());
	return fMonoVAOSatur[monoVAOIndex];
}

bool MEMixingInfo::IsStereoVAOSaturated(SInt32 stereoVAOIndex, bool bLeft) const
{
	ZzAssert(stereoVAOIndex >=  0);
	ZzAssert(stereoVAOIndex < GetMaxMixerDisplayStereoVAOCount());
	if(bLeft)
		return fStereoVAOLeftSatur[stereoVAOIndex];
	else
		return fStereoVAORightSatur[stereoVAOIndex];
}

bool MEMixingInfo::IsStereoRAOSaturated(SInt32 stereoRAOIndex, bool bLeft) const
{
	ZzAssert(stereoRAOIndex >= 0);
	ZzAssert(stereoRAOIndex < GetMaxStereoRAOCount());
	if(bLeft)
		return fStereoRAOLeftSatur[stereoRAOIndex];
	else
		return fStereoRAORightSatur[stereoRAOIndex];
}

void MEMixingInfo::ResetMonoVAOSaturation(SInt32 monoVAOIndex)
{
	ZzAssert(monoVAOIndex >= 0);
	ZzAssert(monoVAOIndex < GetMaxMixerDisplayMonoVAOCount());
//	ZzAssert(fMonoVAOSatur[monoVAOIndex]);
	fMonoVAOSatur[monoVAOIndex] = false;
}

void MEMixingInfo::ResetStereoVAOSaturation(SInt32 stereoVAOIndex, bool bLeft)
{
	ZzAssert(stereoVAOIndex >=  0);
	ZzAssert(stereoVAOIndex < GetMaxMixerDisplayStereoVAOCount());
	if(bLeft)
	{
//		ZzAssert(fStereoVAOLeftSatur[stereoVAOIndex]);
		fStereoVAOLeftSatur[stereoVAOIndex] = false;
	}	
	else
	{
//		ZzAssert(fStereoVAORightSatur[stereoVAOIndex]);
		fStereoVAORightSatur[stereoVAOIndex] = false;
	}	
}

void MEMixingInfo::ResetStereoRAOSaturation(SInt32 stereoRAOIndex, bool bLeft)
{
	ZzAssert(stereoRAOIndex >= 0);
	ZzAssert(stereoRAOIndex < GetMaxStereoRAOCount());
	if(bLeft)
	{
//		ZzAssert(fStereoRAOLeftSatur[stereoRAOIndex]);
		fStereoRAOLeftSatur[stereoRAOIndex] = false;
	}	
	else
	{
//		ZzAssert(fStereoRAORightSatur[stereoRAOIndex]);
		fStereoRAORightSatur[stereoRAOIndex] = false;
	}	
}

void MEMixingInfo::_SetMonoVAOFaderValue(SInt32 monoVAOIndex,float value)
{
	ZzAssert(monoVAOIndex < GetMaxMixerDisplayMonoVAOCount());
	float gain = dBToLinearGain(VULevelTodB(value));
	fMonoVAOInfos[monoVAOIndex].fLeftOrMonoGain = gain;
}

void MEMixingInfo::_SetStereoVAOFaderValue(SInt32 stereoVAOIndex,float value, bool bLeft)
{
	ZzAssert(stereoVAOIndex < GetMaxMixerDisplayStereoVAOCount());
	float gain = dBToLinearGain(VULevelTodB(value));
	if(bLeft)
	{
		fStereoVAOInfos[stereoVAOIndex].fLeftOrMonoGain = gain;
		if(fStereoVAOInfos[stereoVAOIndex].fStereoLock)
		{
				fStereoVAOInfos[stereoVAOIndex].fRightGain = gain;
				fDoc->Notify(kNotification_Mixer_RightVAOFader,(void*)stereoVAOIndex);
		}
	}
	else
	{
		fStereoVAOInfos[stereoVAOIndex].fRightGain = gain;
		if(fStereoVAOInfos[stereoVAOIndex].fStereoLock)
		{
				//vernl(6,"    MEMixingInfo::_SetStereoVAOFaderValue() locked --->  fLeftOrMonoGain = " << gain);
				fStereoVAOInfos[stereoVAOIndex].fLeftOrMonoGain = gain;
				fDoc->Notify(kNotification_Mixer_LeftVAOFader,(void*)stereoVAOIndex);
		}
	}
}

void MEMixingInfo::_SetStereoRAOFaderValue(SInt32 stereoRAOIndex,float value, bool bLeft)
{
	ZzAssert(stereoRAOIndex < GetMaxStereoRAOCount());
	float gain = dBToLinearGain(VULevelTodB(value));
	//vernl(6,"MEMixingInfo::_SetStereoRAOFaderValue( stereoRAOIndex = " << stereoRAOIndex << " gain = " << gain << " bLeft = " << bLeft);
	if(bLeft)
	{
		fStereoRAOInfos[stereoRAOIndex].fLeftGain = gain;
		if(fStereoRAOInfos[stereoRAOIndex].fStereoLock)
		{
				fStereoRAOInfos[stereoRAOIndex].fRightGain = gain;
				fDoc->Notify(kNotification_Mixer_RightRAOFader,(void*)stereoRAOIndex);
		}
	}
	else
	{
		fStereoRAOInfos[stereoRAOIndex].fRightGain = gain;
		if(fStereoRAOInfos[stereoRAOIndex].fStereoLock)
		{
				fStereoRAOInfos[stereoRAOIndex].fLeftGain = gain;
				fDoc->Notify(kNotification_Mixer_LeftRAOFader,(void*)stereoRAOIndex);
		}
	}
}

MEVAOInfo& MEMixingInfo::RefMonoVAOMapping(SInt32 monoVAOIndex)
{
	ZzAssert(monoVAOIndex < GetMaxMixerDisplayMonoVAOCount());
	return fMonoVAOInfos[monoVAOIndex];
}

MEVAOInfo& MEMixingInfo::RefStereoVAOMapping(SInt32 stereoVAOIndex)
{
	ZzAssert(stereoVAOIndex < GetMaxMixerDisplayStereoVAOCount());
	return fStereoVAOInfos[stereoVAOIndex];
}

MERAOInfo& MEMixingInfo::RefStereoRAOMapping(SInt32 stereoRAOIndex)
{
	ZzAssert(stereoRAOIndex < GetMaxStereoRAOCount());
	return fStereoRAOInfos[stereoRAOIndex];
}

SInt32 MEMixingInfo::GetSoloedMonoVAO() const
{
	return fSoloedMonoVAO;
}

SInt32 MEMixingInfo::GetSoloedStereoVAO() const
{
	return fSoloedStereoVAO;
}

SInt32 MEMixingInfo::GetSoloedStereoRAO() const
{
	return fSoloedStereoRAO;
}

void MEMixingInfo::_SetSoloedMonoVAO(SInt32 monoVAOIndex)
{
	ZzAssert(monoVAOIndex < GetMaxMixerDisplayMonoVAOCount());
	fSoloedMonoVAO = monoVAOIndex;
	fDoc->Notify(kNotification_Mixer_SoloedMonoVAOChanged,(void*)monoVAOIndex);
}

void MEMixingInfo::_SetSoloedStereoVAO(SInt32 stereoVAOIndex)
{
	ZzAssert(stereoVAOIndex < GetMaxMixerDisplayStereoVAOCount());
	fSoloedStereoVAO = stereoVAOIndex;
	fDoc->Notify(kNotification_Mixer_SoloedStereoVAOChanged,(void*)stereoVAOIndex);
}

void MEMixingInfo::_SetSoloedStereoRAO(SInt32 stereoRAOIndex)
{
	ZzAssert(stereoRAOIndex < GetMaxStereoRAOCount());
	fSoloedStereoRAO = stereoRAOIndex;
	fDoc->Notify(kNotification_Mixer_SoloedStereoRAOChanged,(void*)stereoRAOIndex);
}

void MEMixingInfo::DeallocateVAOMirrorBuffers()
{
	SInt32 i;
	for(i = 0; i < fNbAllocatedMonoVAOMirrorBuffers; i++)
	{
		delete [] fMonoVAOMirrorBuffers[i];
	}
	free(fMonoVAOMirrorBuffers);
	
	for(i = 0; i < fNbAllocatedStereoVAOMirrorBuffers; i++)
	{
		delete [] fStereoVAOMirrorBuffers[i];
	}
	free(fStereoVAOMirrorBuffers);
}

void MEMixingInfo::AllocateVAOMirrorBuffers()
{
	fDoc->LockWrite(); // beware......the mirror buffers are accessed by the coreaudio output thread during playback
	SInt32 i;
	// are there new mono vao buffers to allocate ?
	if(fMaxMonoVAOCount > fNbAllocatedMonoVAOMirrorBuffers)
	{
		// first allocate a vector of pointers and 
		float** monoVAOMirrorBuffers = (float**)malloc(fMaxMonoVAOCount*sizeof(float*));
		
		// copy the first 'fNbAllocatedMonoVAOMirrorBuffers' pointers in it
		// (0 items to copy if it's a new allocation)
		for(i = 0; i < fNbAllocatedMonoVAOMirrorBuffers; i++)
		{
			monoVAOMirrorBuffers[i] = fMonoVAOMirrorBuffers[i];
		}
	
		// now allocate the new buffers
		for(i = fNbAllocatedMonoVAOMirrorBuffers; i < fMaxMonoVAOCount; i++)
		{
			SInt32 nbFrames = TArmeMediaEngine::CoreAudioPlayer()->OutDeviceBufferNbFrames();
			monoVAOMirrorBuffers[i] = new float[nbFrames];
		}
	
		fNbAllocatedMonoVAOMirrorBuffers = fMaxMonoVAOCount;
	
		// delete the original vector of pointers
		free(fMonoVAOMirrorBuffers);
		fMonoVAOMirrorBuffers = monoVAOMirrorBuffers;
	}
	
	if(fMaxStereoVAOCount > fNbAllocatedStereoVAOMirrorBuffers)
	{
		// first allocate a vector of pointers and 
		float** stereoVAOMirrorBuffers = (float**)malloc(fMaxStereoVAOCount*sizeof(float**));
	
		// copy the first 'fNbAllocatedStereoVAOMirrorBuffers' pointers in it
		// (0 items to copy if it's a new allocation)
		for(i = 0; i < fNbAllocatedStereoVAOMirrorBuffers; i++)
		{
			stereoVAOMirrorBuffers[i] = fStereoVAOMirrorBuffers[i];
		}

		// now allocate the new buffers
		for( i = fNbAllocatedStereoVAOMirrorBuffers; i < fMaxStereoVAOCount; i++)
		{
			stereoVAOMirrorBuffers[i] = new float[TArmeMediaEngine::CoreAudioPlayer()->OutDeviceBufferNbFrames()*2];
		}
		fNbAllocatedStereoVAOMirrorBuffers = fMaxStereoVAOCount;
	
		// delete the original vector of pointers
		free(fStereoVAOMirrorBuffers);
		fStereoVAOMirrorBuffers = stereoVAOMirrorBuffers;
	}
	fDoc->UnlockWrite(); // beware......the mirror buffers are accessed by the coreaudio output thread during playback
}
	
/************************************************************************************************************************
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
************************************************************************************************************************/
// this function adds the buffer to the local VAO mirror buffers
// when the current coreaudio buffer processing is done, this function analyzes the content 
// of the vao buffers and upates the vao level meters accordingly. Please note that the rao requests
// are built in slices according the the locator position. Thus, some buffers are only PART of the current 
// vao buffer. this is why the frameOffsetInCurrentBuffer and nbFrames parameters are needed.
void MEMixingInfo::AddBufferToVAOMirrorBuffer(bool bStereo, SInt32 vaoIndex,float* buf,SInt32 frameOffsetInCurrentBuffer,SInt32 nbFrames)
{
	float* src = buf;
	SInt32 nbSamples;
	float* dst;
	
	if(bStereo)
	{
		ZzRtAssert(vaoIndex < fNbAllocatedStereoVAOMirrorBuffers);
		dst = fStereoVAOMirrorBuffers[vaoIndex] + 2*frameOffsetInCurrentBuffer;
		nbSamples = nbFrames*2;
	}
	else
	{
		ZzRtAssert(vaoIndex < fNbAllocatedMonoVAOMirrorBuffers);
		dst = fMonoVAOMirrorBuffers[vaoIndex] + frameOffsetInCurrentBuffer;
		nbSamples = nbFrames;
	}
	for(SInt32 i = 0 ; i < nbSamples; i++)
	{
		*dst += *src;
		dst++;
		src++;
	}
}

/************************************************************************************************************************
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
************************************************************************************************************************/
void MEMixingInfo::ResetVAOMirrorBuffers()
{
	SInt32 i;
	for(i = 0; i < fNbAllocatedMonoVAOMirrorBuffers; i++)
	{
		memset(fMonoVAOMirrorBuffers[i],0,TArmeMediaEngine::CoreAudioPlayer()->OutDeviceBufferNbFrames()*sizeof(float));
	}
	for(i = 0; i < fNbAllocatedStereoVAOMirrorBuffers; i++)
	{
		memset(fStereoVAOMirrorBuffers[i],0,TArmeMediaEngine::CoreAudioPlayer()->OutDeviceBufferNbFrames()*sizeof(float)*2);
	}
}

/************************************************************************************************************************
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
************************************************************************************************************************/
void MEMixingInfo::AnalyzeRAOBuffer(float* raoBuffer)
{
	SInt32 i;
	
	for(i = 0; i < fMaxStereoRAOCount; i++)
	{
		fRAOAnalyzeMaxSqAmplL[i] = 0.0;
		fRAOAnalyzeMaxSqAmplR[i] = 0.0;
	}
	
	float sqAmplL,sqAmplR;

	SInt32  nbFrames = TArmeMediaEngine::CoreAudioPlayer()->OutDeviceBufferNbFrames();
	float* src = raoBuffer;
	for(SInt32 frame = 0; frame < nbFrames; frame++)
	{
		for(i = 0; i < fMaxStereoRAOCount; i++)
		{
			sqAmplL = *src * *src;
			if(sqAmplL > fRAOAnalyzeMaxSqAmplL[i])
				fRAOAnalyzeMaxSqAmplL[i] = sqAmplL;
			src++;
			sqAmplR = *src * *src;
			if(sqAmplR > fRAOAnalyzeMaxSqAmplR[i])
				fRAOAnalyzeMaxSqAmplR[i] = sqAmplR;
			src++;
		}
	}
	for(i = 0; i < fMaxStereoRAOCount; i++)
	{
		float dBFSL = 20*log10(sqrt(fRAOAnalyzeMaxSqAmplL[i]));
		float dBFSR = 20*log10(sqrt(fRAOAnalyzeMaxSqAmplR[i]));
		
		if(fStereoRAOLeftSatur[i])
		{
			fStereoRAOLeftSaturResetCounter[i] = fStereoRAOLeftSaturResetCounter[i] - 1;
			if(fStereoRAOLeftSaturResetCounter[i] == 0)
				fStereoRAOLeftSatur[i] = false;
		}
		if(fStereoRAORightSatur[i])
		{
			fStereoRAORightSaturResetCounter[i] = fStereoRAORightSaturResetCounter[i] - 1;
			if(fStereoRAORightSaturResetCounter[i] == 0)
				fStereoRAORightSatur[i] = false;
		}
		if(dBFSL > 0.0)
		{
			fStereoRAOLeftSatur[i] = true;
			fStereoRAOLeftSaturResetCounter[i] = fNbOfBuffersForSaturationReset;
		}
		if(dBFSR > 0.0)
		{
			fStereoRAORightSatur[i] = true;
			fStereoRAORightSaturResetCounter[i] = fNbOfBuffersForSaturationReset;
		}
					
		// here we attenuate the previous values
		if(fRAOLeftDBFS[i] > -1000.0)
			fRAOLeftDBFS[i] -= fDBPerBufferAttenuation;
		if(fRAORightDBFS[i] > -1000.0)
			fRAORightDBFS[i] -= fDBPerBufferAttenuation;
		// and we immediatly assign a new value
		if(dBFSL > fRAOLeftDBFS[i])
			fRAOLeftDBFS[i] = dBFSL;
		if(dBFSR > fRAORightDBFS[i])
			fRAORightDBFS[i] = dBFSR;
	}
}
	
/************************************************************************************************************************
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
************************************************************************************************************************/
void MEMixingInfo::AnalyzeVAOMirrorBuffers()
{
	SInt32 i;
	SInt32 nbFrames;
	for(i = 0; i < fNbAllocatedMonoVAOMirrorBuffers; i++)
	{
		float maxSqAmpl = 0.0;
		float maxAmpl;
		float dBFS;
		nbFrames = TArmeMediaEngine::CoreAudioPlayer()->OutDeviceBufferNbFrames();
		float* src = fMonoVAOMirrorBuffers[i];
		float sqAmpl;
		for(SInt32 frame = 0; frame < nbFrames; frame++)
		{
			sqAmpl = *src * *src;
			if(sqAmpl > maxSqAmpl)
				maxSqAmpl = sqAmpl;
			src++;
		}
		maxAmpl = sqrt(maxSqAmpl);
		dBFS = 20*log10(maxAmpl);

		// check for saturation reset
		if(fMonoVAOSatur[i])
		{
			fMonoVAOSaturResetCounter[i] = fMonoVAOSaturResetCounter[i] - 1;
			if(fMonoVAOSaturResetCounter[i] == 0)
				fMonoVAOSatur[i] = false;
		}
			
		if(dBFS > 0.0)
		{
			fMonoVAOSatur[i] = true;
			fMonoVAOSaturResetCounter[i] = fNbOfBuffersForSaturationReset;
		}

		static int staticCnt2 = 0;
		staticCnt2++;
		if((staticCnt2 % 25) == 0)
		{
			staticCnt2 = 0;
			//vernl(6,"mono vao buffer number " << i << " : dBFS = " << dBFS);
		}
				
		// here we attenuate the previous value
		if(fMonoVAODBFS[i] > -1000.0)
			fMonoVAODBFS[i] -= fDBPerBufferAttenuation;
		// and we immediatly assign a new value
		if(dBFS > fMonoVAODBFS[i])
			fMonoVAODBFS[i] = dBFS;
	}
	for(i = 0; i < fNbAllocatedStereoVAOMirrorBuffers; i++)
	{
		float maxSqAmplL = 0.0;
		float maxSqAmplR = 0.0;
		float maxAmplL;
		float maxAmplR;
		float dBFSL;
		float dBFSR;
		nbFrames = TArmeMediaEngine::CoreAudioPlayer()->OutDeviceBufferNbFrames();
		float* src = fStereoVAOMirrorBuffers[i];
		float sqAmplL,sqAmplR;
		for(SInt32 frame = 0; frame < nbFrames; frame++)
		{
			sqAmplL = *src * *src;
			if(sqAmplL > maxSqAmplL)
				maxSqAmplL = sqAmplL;
			src++;
			sqAmplR = *src * *src;
			if(sqAmplR > maxSqAmplR)
				maxSqAmplR = sqAmplR;
			src++;
		}
		maxAmplL = sqrt(maxSqAmplL);
		maxAmplR = sqrt(maxSqAmplR);
		dBFSL = 20*log10(maxAmplL);
		dBFSR = 20*log10(maxAmplR);


		// check for saturation reset
		if(fStereoVAOLeftSatur[i])
		{
			fStereoVAOLeftSaturResetCounter[i] = fStereoVAOLeftSaturResetCounter[i] - 1;
			if(fStereoVAOLeftSaturResetCounter[i] == 0)
				fStereoVAOLeftSatur[i] = false;
		}

		if(fStereoVAORightSatur[i])
		{
			fStereoVAORightSaturResetCounter[i] = fStereoVAORightSaturResetCounter[i] - 1;
			if(fStereoVAORightSaturResetCounter[i] == 0)
				fStereoVAORightSatur[i] = false;
		}
			

		if(dBFSL > 0.0)
		{
			fStereoVAOLeftSatur[i] = true;
			fStereoVAOLeftSaturResetCounter[i] = fNbOfBuffersForSaturationReset;
		}
		if(dBFSR > 0.0)
		{		
			fStereoVAORightSatur[i] = true;
			fStereoVAORightSaturResetCounter[i] = fNbOfBuffersForSaturationReset;
		}
					
		static int staticCnt = 0;
		staticCnt++;
		if((staticCnt % 25) == 0)
		{
			staticCnt = 0;
			//vernl(6,"stereo vao buffer number " << i << " : dBFSL = " << dBFSL << " : dBFSR = " << dBFSR);
		}
		// here we attenuate the previous values
		if(fStereoVAOLeftDBFS[i] > -1000.0)
			fStereoVAOLeftDBFS[i] -= fDBPerBufferAttenuation;
		if(fStereoVAORightDBFS[i] > -1000.0)
			fStereoVAORightDBFS[i] -= fDBPerBufferAttenuation;
		// and we immediatly assign a new value
		if(dBFSL > fStereoVAOLeftDBFS[i])
			fStereoVAOLeftDBFS[i] = dBFSL;
		if(dBFSR > fStereoVAORightDBFS[i])
			fStereoVAORightDBFS[i] = dBFSR;
	}
}

/************************************************************************************************************************
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
************************************************************************************************************************/
void MEMixingInfo::AttenuateVAOBuffer(bool bStereo, SInt32 vaoIndex,float* samples,SInt32 nbFrames)
{
	float invNbFrames = 1.0/(float)nbFrames;
	if(bStereo)
	{
		float curLeftGain = fStereoVAOInfos[vaoIndex].fOldLeftOrMonoGain;
		float curRightGain = fStereoVAOInfos[vaoIndex].fOldRightGain;
		float tgtLeftGain,tgtRightGain;
		tgtLeftGain = fStereoVAOInfos[vaoIndex].fLeftOrMonoGain;
		tgtRightGain = fStereoVAOInfos[vaoIndex].fRightGain;
		
		if(fSoloedStereoVAO != -1 && fSoloedStereoVAO != vaoIndex)
		{
			tgtLeftGain = tgtRightGain = 0.0;
		}
				
		float incrLeftGain = (tgtLeftGain-curLeftGain)*invNbFrames;
		float incrRightGain = (tgtRightGain-curRightGain)*invNbFrames;
		
		float* src = samples;
		for(SInt32 i = 0; i < nbFrames; i++)
		{
			*src = curLeftGain*(*src);
			src++;
			*src = curRightGain*(*src);
			src++;
			curLeftGain += incrLeftGain;
			curRightGain += incrRightGain;
		}
		fStereoVAOInfos[vaoIndex].fOldLeftOrMonoGain = tgtLeftGain;
		fStereoVAOInfos[vaoIndex].fOldRightGain = tgtRightGain;
	}
	else
	{
		float curGain = fMonoVAOInfos[vaoIndex].fOldLeftOrMonoGain;
		float tgtGain = fMonoVAOInfos[vaoIndex].fLeftOrMonoGain;

		if(fSoloedStereoVAO != -1 && fSoloedStereoVAO != vaoIndex)
		{
			tgtGain = 0.0;
		}

		float incrGain = (tgtGain-curGain)*invNbFrames;

		float* src = samples;
		for(SInt32 i = 0; i < nbFrames; i++)
		{
			*src = curGain*(*src);
			curGain += incrGain;
			src++;
		}
		fMonoVAOInfos[vaoIndex].fOldLeftOrMonoGain = tgtGain;
	}
}


#if 1

void MEMixingInfo::MixMapPanStereoVAOIntoRAOBuffer(float* raoDst, UInt32 vaoIndex, float* vaoBuffer, UInt32 nbFrames)
{
	// the VAO had already been faded yet in vaoBuffer
	// we need to fade the RAO here
	SInt32 raoIndex	= GetStereoRAOForStereoVAO(vaoIndex);
	if( raoIndex >= GetMaxStereoRAOCount())
	{
		//vernl(1,"raoIndex outta range (" << raoIndex << " > " << GetMaxStereoRAOCount() << ") in MixMapStereoVAOIntoRAOBuffer()");
		return;
	}
	
	float invFrames = 1.0/(float)nbFrames;
	
	// raoIncrement = how many sizeof(float) we need to add the a rao address pointing to a 
	// right sample of a pair to make it point to the left sample of the same channel in the
	// next frame
	UInt32 raoIncrement = (GetMaxStereoRAOCount() << 1) - 1;
	
	float curRaoGainL = fStereoVAOInfos[vaoIndex].fOldRAOLeftGain;
	float curRaoGainR = fStereoVAOInfos[vaoIndex].fOldRAORightGain;
	float targetRaoGainL = fStereoRAOInfos[raoIndex].fLeftGain;
	float targetRaoGainR = fStereoRAOInfos[raoIndex].fRightGain;

	float curA,curB,curC,curD;
	float targetA,targetB,targetC,targetD;
	float incrA,incrB,incrC,incrD;
	
	// panning -> dalpha
	// -1 -> 0
	// 0 -> /2
	// 1 -> 
	float curPanning = fStereoVAOInfos[vaoIndex].fOldPan;
	float targetPanning = fStereoVAOInfos[vaoIndex].fPan;
	double curDalpha = (curPanning+1.0)*kHalfPi;
	double targetDalpha = (targetPanning+1.0)*kHalfPi;
	
	// if panning to left (panning between -1 and 0)
	// dalpha between 0 and /2
	// LO = LI + cos(dalpha)*RI
	// RO = sin(dalpha)*RI
	// thus hard left means LO = LI + RI and RO = 0
	// and center means LO = LI and RO = RI

	// if panning to right (panning between 0 and 1)
	// dalpha between /2 and 
	// LO = sin(dalpha)*LI
	// RO = -cos(dalpha)*LI + RI
	// thus center means LO = LI and RO = RI
	// and hard right means LO = 0 and RO = LI + RI
	
	float curCosdalpha = (float)cos(curDalpha);
	float curSindalpha = (float)sin(curDalpha);
	float targetCosdalpha = (float)cos(targetDalpha);
	float targetSindalpha = (float)sin(targetDalpha);
	
	// we have
	// LO = A*LI + B*RI
	// RO = C*LI + D*RI
	// let's compute A, B, C and D
	
	if(curPanning <= 0.0)
	{
		curA = curRaoGainL;
		curB = curCosdalpha*curRaoGainL;
		curC = 0;
		curD = curRaoGainR*curSindalpha;
	}
	else
	{
		curA = curRaoGainL*curSindalpha;
		curB = 0;
		curC = -curCosdalpha*curRaoGainR;
		curD = 1;
	}

	if(targetPanning <= 0.0)
	{
		targetA = targetRaoGainL;
		targetB = targetCosdalpha*targetRaoGainL;
		targetC = 0;
		targetD = targetRaoGainR*targetSindalpha;
	}
	else
	{
		targetA = targetRaoGainL*targetSindalpha;
		targetB = 0;
		targetC = -targetCosdalpha*targetRaoGainR;
		targetD = 1;
	}
	
	incrA = (targetA-curA)*invFrames;
	incrB = (targetB-curB)*invFrames;
	incrC = (targetC-curC)*invFrames;
	incrD = (targetD-curD)*invFrames;
		
	float* sourceStereoFrame = vaoBuffer;

	// dstStereoFrame points to the first sample of the first frame of the rao channel this vao is routed to
	float* dstStereoFrame = raoDst + (raoIndex << 1);

	if(curRaoGainL != targetRaoGainL)
	{
/*
		vernl(6,"MEMixingInfo::MixMapPanStereoVAOIntoRAOBuffer()");
		vvernl(6,curRaoGainL);
		vvernl(6,curRaoGainR);
		vvernl(6,targetRaoGainL);
		vvernl(6,targetRaoGainR);
		vvernl(6,curPanning);
		vvernl(6,targetPanning);
		vernl(6,"curDalpha = " << curDalpha/kPi << " * pi");
		vernl(6,"targetDalpha = " << curDalpha/kPi << " * pi");
		vvernl(6,curCosdalpha);
		vvernl(6,curSindalpha);
		vvernl(6,targetCosdalpha);
		vvernl(6,targetSindalpha);
		vvernl(6,curA);
		vvernl(6,curB);
		vvernl(6,curC);
		vvernl(6,curD);
		vvernl(6,targetA);
		vvernl(6,targetB);
		vvernl(6,targetC);
		vvernl(6,targetD);
		vvernl(6,incrA);
		vvernl(6,incrB);
		vvernl(6,incrC);
		vvernl(6,incrD);
*/
	}
	
	UInt32 frame = 0;
	float inL,inR;
	while(frame < nbFrames)
	{
		inL = *sourceStereoFrame;
		sourceStereoFrame++;
		inR = *sourceStereoFrame;
		sourceStereoFrame++;
			
		*dstStereoFrame += curA*inL + curB*inR;
		dstStereoFrame++;
		
		*dstStereoFrame += curC*inL + curD*inR;
		dstStereoFrame += raoIncrement;
		frame++;
		curA += incrA;
		curB += incrB;
		curC += incrC;
		curD += incrD;
	}
	if(curRaoGainL != targetRaoGainL)
	{
//		vernl(6,"final curA = " << curA << " final curB = " << curB << " final curC = " << curC << " final curD = " << curD);
	}
		
	fStereoVAOInfos[vaoIndex].fOldRAOLeftGain = fStereoRAOInfos[raoIndex].fLeftGain;
	fStereoVAOInfos[vaoIndex].fOldRAORightGain = fStereoRAOInfos[raoIndex].fRightGain;
	fStereoVAOInfos[vaoIndex].fOldPan = fStereoVAOInfos[vaoIndex].fPan;
}

#else

void MEMixingInfo::MixMapPanStereoVAOIntoRAOBuffer(float* raoDst, UInt32 vaoIndex, float* vaoBuffer, UInt32 nbFrames)
{
	// the VAO had already been faded yet in vaoBuffer
	// we need to fade the RAO here
	UInt32 raoIndex	= GetStereoRAOForStereoVAO(vaoIndex);
	if( raoIndex >= GetMaxStereoRAOCount())
	{
//		vernl(1,"raoIndex outta range (" << raoIndex << " > " << GetMaxStereoRAOCount() << ") in MixMapStereoVAOIntoRAOBuffer()");
		return;
	}
	
	float invFrames = 1.0/(float)nbFrames;
	
	// raoIncrement = how many sizeof(float) we need to add the a rao address pointing to a 
	// right sample of a pair to make it point to the left sample of the same channel in the
	// next frame
	UInt32 raoIncrement = (GetMaxStereoRAOCount() << 1) - 1;
	
	float curRaoGainL = fStereoRAOInfos[raoIndex].fOldLeftGain;
	float curRaoGainR = fStereoRAOInfos[raoIndex].fOldRightGain;
	float targetRaoGainL = fStereoRAOInfos[raoIndex].fLeftGain;
	float targetRaoGainR = fStereoRAOInfos[raoIndex].fRightGain;
	float incrRaoGainL = (targetRaoGainL - curRaoGainL)*invFrames;
	float incrRaoGainR = (targetRaoGainR - curRaoGainR)*invFrames;
	
	float* sourceStereoFrame = vaoBuffer;

	// dstStereoFrame points to the first sample of the first frame of the rao channel this vao is routed to
	float* dstStereoFrame = raoDst + (raoIndex << 1);
	
	UInt32 frame = 0;
	float inL,inR;
	while(frame < nbFrames)
	{
		inL = *sourceStereoFrame;
		sourceStereoFrame++;
		inR = *sourceStereoFrame;
		sourceStereoFrame++;
			
		*dstStereoFrame += curRaoGainL*inL;
		dstStereoFrame++;
		
		*dstStereoFrame += curRaoGainR*inR;
		dstStereoFrame += raoIncrement;
		frame++;
		curRaoGainL += incrRaoGainL;
		curRaoGainR += incrRaoGainR;
	}
	
	fStereoRAOInfos[raoIndex].fOldLeftGain = fStereoRAOInfos[raoIndex].fLeftGain;
	fStereoRAOInfos[raoIndex].fOldRightGain = fStereoRAOInfos[raoIndex].fRightGain;
	fStereoVAOInfos[vaoIndex].fOldPan = fStereoVAOInfos[vaoIndex].fPan;
}

#endif

void MEMixingInfo::MixMapPanMonoVAOIntoStereoBuffer_Offline(float* dst, UInt32 vaoIndex, float* vaoBuffer, UInt32 nbFrames)
{
	// the VAO data HAS ALREADY BEEN faded
	SInt32 raoIndex	= GetStereoRAOForMonoVAO(vaoIndex);
//	float invNbFrames = 1.0/(float)nbFrames;
	ZzAssert( raoIndex < GetMaxStereoRAOCount() );
	
	float tgtPan = fMonoVAOInfos[vaoIndex].fPan;
	
	// alpha is 0 for hard left, /4 for middle and /2 for hard right
	double tgtalpha = (tgtPan+1.0)*kQuarterPi;
	
	double tgtcosalpha = cos(tgtalpha);
	double tgtsinalpha = sin(tgtalpha);
	
	float tgtLeftGain = fStereoRAOInfos[raoIndex].fLeftGain;
	float tgtRightGain = fStereoRAOInfos[raoIndex].fRightGain;
	float tgtTotalLeftFading = tgtcosalpha*tgtLeftGain;
	float tgtTotalRightFading = tgtsinalpha*tgtRightGain;
	
	float* sourceStereoFrame = vaoBuffer;
	
	// we don't render to a real rao... we render to OUR buffer
	float* dstStereoFrame = dst;
	UInt32 frame = 0;
	while(frame < nbFrames)
	{
		*dstStereoFrame += tgtTotalLeftFading*(*sourceStereoFrame);
		dstStereoFrame++;
		*dstStereoFrame += tgtTotalRightFading*(*sourceStereoFrame);
		dstStereoFrame++;
		sourceStereoFrame++;
		frame++;
	}
}

void MEMixingInfo::MixMapPanMonoVAOIntoRAOBuffer(float* raoDst, UInt32 vaoIndex, float* vaoBuffer, UInt32 nbFrames)
{
	// the VAO HAS ALREADY BEEN faded
	SInt32 raoIndex	= GetStereoRAOForMonoVAO(vaoIndex);
	float invNbFrames = 1.0/(float)nbFrames;
	if( raoIndex >= GetMaxStereoRAOCount())
	{
//		vernl(1,"raoIndex outta range (" << raoIndex << " > " << GetMaxStereoRAOCount() << ") in MixMapStereoVAOIntoRAOBuffer()");
		return;
	}
	// raoIncrement = how many sizeof(float) we need to add the a rao address pointing to a 
	// right sample of a pair to make it point to the left sample of the same channel in the
	// next frame
	UInt32 raoIncrement = (GetMaxStereoRAOCount() << 1) - 1;

	float curPan = fMonoVAOInfos[vaoIndex].fOldPan;
	float tgtPan = fMonoVAOInfos[vaoIndex].fPan;
	
	// alpha is 0 for hard left, /4 for middle and /2 for hard right
	double curalpha = (curPan+1.0)*kQuarterPi;
	double tgtalpha = (tgtPan+1.0)*kQuarterPi;
	
	double curcosalpha = cos(curalpha);
	double cursinalpha = sin(curalpha);
	double tgtcosalpha = cos(tgtalpha);
	double tgtsinalpha = sin(tgtalpha);
	
	float curLeftGain = fMonoVAOInfos[vaoIndex].fOldRAOLeftGain;
	float curRightGain = fMonoVAOInfos[vaoIndex].fOldRAORightGain;
	float curTotalLeftFading = curcosalpha*curLeftGain;
	float curTotalRightFading = cursinalpha*curRightGain;

	float tgtLeftGain = fStereoRAOInfos[raoIndex].fLeftGain;
	float tgtRightGain = fStereoRAOInfos[raoIndex].fRightGain;
	float tgtTotalLeftFading = tgtcosalpha*tgtLeftGain;
	float tgtTotalRightFading = tgtsinalpha*tgtRightGain;
	
	float incrTotalLeftFading = (tgtTotalLeftFading - curTotalLeftFading)*invNbFrames;
	float incrTotalRightFading = (tgtTotalRightFading - curTotalRightFading)*invNbFrames;

	float* sourceStereoFrame = vaoBuffer;
	// dstStereoFrame points to the first sample of the first frame of the rao channel this vao is routed to
	float* dstStereoFrame = raoDst + (raoIndex << 1);
	UInt32 frame = 0;
	while(frame < nbFrames)
	{
		*dstStereoFrame += curTotalLeftFading*(*sourceStereoFrame);
		dstStereoFrame++;
		*dstStereoFrame += curTotalRightFading*(*sourceStereoFrame);
		dstStereoFrame += raoIncrement;
		sourceStereoFrame++;
		frame++;
		curTotalLeftFading += incrTotalLeftFading;
		curTotalRightFading += incrTotalRightFading;
	}
	fMonoVAOInfos[vaoIndex].fOldRAOLeftGain = fStereoRAOInfos[raoIndex].fLeftGain;
	fMonoVAOInfos[vaoIndex].fOldRAORightGain = fStereoRAOInfos[raoIndex].fRightGain;
	fMonoVAOInfos[vaoIndex].fOldPan = fMonoVAOInfos[vaoIndex].fPan;
}
		
void MEMixingInfo::MixMapPanStereoVAOIntoStereoBuffer_Offline(float* dst, UInt32 vaoIndex, float* vaoBuffer, UInt32 nbFrames)
{
	// the VAO had already been faded yet in vaoBuffer
	// we need to fade the RAO here
	SInt32 raoIndex	= GetStereoRAOForStereoVAO(vaoIndex);
	ZzAssert( raoIndex < GetMaxStereoRAOCount());
	
//	float invFrames = 1.0/(float)nbFrames;
	
	float targetRaoGainL = fStereoRAOInfos[raoIndex].fLeftGain;
	float targetRaoGainR = fStereoRAOInfos[raoIndex].fRightGain;

	float targetA,targetB,targetC,targetD;
	
	// panning -> dalpha
	// -1 -> 0
	// 0 -> /2
	// 1 -> 
	float targetPanning = fStereoVAOInfos[vaoIndex].fPan;
	double targetDalpha = (targetPanning+1.0)*kHalfPi;
	
	// if panning to left (panning between -1 and 0)
	// dalpha between 0 and /2
	// LO = LI + cos(dalpha)*RI
	// RO = sin(dalpha)*RI
	// thus hard left means LO = LI + RI and RO = 0
	// and center means LO = LI and RO = RI

	// if panning to right (panning between 0 and 1)
	// dalpha between /2 and 
	// LO = sin(dalpha)*LI
	// RO = -cos(dalpha)*LI + RI
	// thus center means LO = LI and RO = RI
	// and hard right means LO = 0 and RO = LI + RI
	
	float targetCosdalpha = (float)cos(targetDalpha);
	float targetSindalpha = (float)sin(targetDalpha);
	
	// we have
	// LO = A*LI + B*RI
	// RO = C*LI + D*RI
	// let's compute A, B, C and D
	
	if(targetPanning <= 0.0)
	{
		targetA = targetRaoGainL;
		targetB = targetCosdalpha*targetRaoGainL;
		targetC = 0;
		targetD = targetRaoGainR*targetSindalpha;
	}
	else
	{
		targetA = targetRaoGainL*targetSindalpha;
		targetB = 0;
		targetC = -targetCosdalpha*targetRaoGainR;
		targetD = 1;
	}
	
	float* sourceStereoFrame = vaoBuffer;
	float* dstStereoFrame = dst;

	UInt32 frame = 0;
	float inL,inR;
	while(frame < nbFrames)
	{
		inL = *sourceStereoFrame;
		sourceStereoFrame++;
		inR = *sourceStereoFrame;
		sourceStereoFrame++;
			
		*dstStereoFrame += targetA*inL + targetB*inR;
		dstStereoFrame++;
		
		*dstStereoFrame += targetC*inL + targetD*inR;
		dstStereoFrame++;
		frame++;
	}
}

void MEMixingInfo::AttenuateVAOBuffer_Offline(bool bStereo, SInt32 vaoIndex,float* samples,SInt32 nbFrames)
{
//	float invNbFrames = 1.0/(float)nbFrames;
	if(bStereo)
	{
		float tgtLeftGain,tgtRightGain;
		tgtLeftGain = fStereoVAOInfos[vaoIndex].fLeftOrMonoGain;
		tgtRightGain = fStereoVAOInfos[vaoIndex].fRightGain;
		
		if(fSoloedStereoVAO != -1 && fSoloedStereoVAO != vaoIndex)
		{
			tgtLeftGain = tgtRightGain = 0.0;
		}
				
		float* src = samples;
		for(SInt32 i = 0; i < nbFrames; i++)
		{
			*src = tgtLeftGain*(*src);
			src++;
			*src = tgtRightGain*(*src);
			src++;
		}
	}
	else
	{
		float tgtGain = fMonoVAOInfos[vaoIndex].fLeftOrMonoGain;

		if(fSoloedStereoVAO != -1 && fSoloedStereoVAO != vaoIndex)
		{
			tgtGain = 0.0;
		}

		float* src = samples;
		for(SInt32 i = 0; i < nbFrames; i++)
		{
			*src = tgtGain*(*src);
			src++;
		}
	}
}

void MEVAORouting::ReadFromArchive(ArmeArchiveHandler* handler)
{
	index = handler->ReadSInt32();
	bStereo = handler->ReadBool();
}

void MEVAORouting::WriteToArchive(ArmeArchiveHandler* handler) const
{
	handler->WriteSInt32(index);
	handler->WriteBool(bStereo);
}

ZzString MEMixingInfo::GetCurrentOutputDeviceShortName() const
{
	return TArmeMediaEngine::CoreAudioPlayer()->GetDeviceShortName();
}

	
