/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#define MEMixingInfo_h

#ifndef MEMixingInfo_h
#include "MEMixingInfo.h"
#endif //MEMixingInfo_h

#ifndef ArmeBasicTypes_h
#include "ArmeBasicTypes.h"
#endif //ArmeBasicTypes_h


#ifndef ArmeArchiveHandler_h
#include "ArmeArchiveHandler.h"
#endif //ArmeArchiveHandler_h

class TArmeDoc;

#ifndef ZzBroadcaster_h
#include "ZzBroadcaster.h"
#endif //ZzBroadcaster_h

#ifndef ZzListener_h
#include "ZzListener.h"
#endif //ZzListener_h

#ifndef ZzString_h
#include "ZzString.h"
#endif //ZzString_h

#ifndef ArmeArchiveHandler_h
#include "ArmeArchiveHandler.h"
#endif //ArmeArchiveHandler_h

/*
	the routing from an object to a VAO. Please note this is used by audio clips ONLY
*/
class ArmeArchiveHandler;

struct MEVAORouting
{
	SInt32 	index;
	bool	bStereo;
	
	void	ReadFromArchive(ArmeArchiveHandler* handler);
	void	WriteToArchive(ArmeArchiveHandler* handler) const;
};

inline bool operator==(const MEVAORouting& routing1, const MEVAORouting& routing2)
{
	return ( (routing1.index == routing1.index) && (routing1.bStereo == routing2.bStereo) );
}

inline bool operator!=(const MEVAORouting& routing1, const MEVAORouting& routing2)
{
	return !(routing1 == routing2);
}

#define kCoreAudioPlayer_MaxHierarchyDepth 64
#define kBusNameLen 32

struct MEVAOInfo
{
	bool	bStereo; 			// is this VAO routed to a stereo RAO ?
	SInt32	fRAOIndex; 			// the rao index (pair or discrete channel)
	float	fLeftOrMonoGain;	// the gain from VAO -> RAO
	float	fOldLeftOrMonoGain;	// for dezipping
	float	fRightGain;			// the gain from VAO -> RAO
	float	fOldRightGain;		// for dezipping
	float	fOldRAOLeftGain;	// for dezipping
	float	fOldRAORightGain;	// for dezipping
	float	fPan;				// the panning (-1.0 ---> 1.0)
	float	fOldPan;			// for dezipping
	float	fMuted;				// 1.0 or 0.0
	float	fOldMuted;			// for dezipping
	char	fName[kBusNameLen];	// name
	bool	fStereoLock;		// whether stereo sliders are locked
	
	void	WriteToArchive(ArmeArchiveHandler* archiver) const;
	void	ReadFromArchive(ArmeArchiveHandler* unarchiver);
};

struct MERAOInfo
{
	float	fLeftGain; 			// the last gain applied to this RAO channel after VAO mixing
	float	fRightGain;			// the last gain applied to this RAO channel after VAO mixing
	float	fMuted;				// 1.0 or 0.0
	float	fOldMuted;			// for dezipping
	char	fName[kBusNameLen];	// name
	bool	fStereoLock;		// whether stereo sliders are locked

	void	WriteToArchive(ArmeArchiveHandler* archiver) const;
	void	ReadFromArchive(ArmeArchiveHandler* unarchiver);
};

const SInt32 kMEMixingInfo_NbMaxMonoVAO = 64;
const SInt32 kMEMixingInfo_NbMaxStereoVAO = 64; //$$$TODO replace with dynamic stuff
const SInt32 kMEMixingInfo_NbMaxRAO = 64;

class ArmeArchivable;

/*
#ifdef ArmeArchivable_h
#error "something strange is happening"
#endif //ArmeArchivable_g
*/

#ifndef ArmeArchivable_h
#include "ArmeArchivable.h"
#endif //ArmeArchivable_h

class ZzListener; // i know its strange... don't worry

class MEMixingInfo : public ArmeArchivable, public ZzListener
{
public :
          MEMixingInfo(TArmeDoc* doc);
	virtual ~MEMixingInfo();
	
	/*
		archive stuff
	*/
          MEMixingInfo(ArmeArchiveHandler* unarchiver);
	bool    WriteToArchive(ArmeArchiveHandler* archiver) const;

	/*
		archive stuff
	*/
	virtual void	NotificationReceived( ZzBroadcaster* source, UInt32 code, void* param); 

	// initializes what needs to be updated when coreaudio changes (buffer size, nb of outputs or sample rate)
	void InitCoreAudioDependentInfo();

	/*
		number of AO
	*/
	
	// return the number of mono and stereo vao which have audio clips assigned to
	SInt32	GetMaxStereoVAOCount() const;
	SInt32	GetMaxMonoVAOCount() const;
	
	ZzString	GetCurrentOutputDeviceShortName() const;
	
	// same as above, except they never return < 4, so that the mixer still looks like a real mixer
	SInt32	GetMaxMixerDisplayStereoVAOCount() const; 
	SInt32	GetMaxMixerDisplayMonoVAOCount() const;

	SInt32	GetMaxAssignedStereoRAOCount() const;
	SInt32	GetMaxStereoRAOCount() const; // depends upon the active audio interface.
	
	/*
		Level computation
	*/
	
	// this function allocates as many buffers as needed to store a full vao vector for a coreaudio iteration
	void	AllocateVAOMirrorBuffers();
	void 	DeallocateVAOMirrorBuffers();
		
	void 	MixMapPanVAOIntoRAOBuffer(float* raoDst, const MEVAORouting& routing, float* vaoBuffer, UInt32 nbFrames);
	void 	MixMapPanStereoVAOIntoRAOBuffer(float* raoDst, UInt32 vaoIndex, float* vaoBuffer, UInt32 nbFrames);
	void 	MixMapPanMonoVAOIntoRAOBuffer(float* raoDst, UInt32 vaoIndex, float* vaoBuffer, UInt32 nbFrames);
		
	// this function adds the buffer to the local VAO mirror buffers
	// when the current coreaudio buffer processing is done, this function analyzes the content 
	// of the vao buffers and upates the vao level meters accordingly. Please note that the rao requests
	// are built in slices according the the locator position. Thus, some buffers are only PART of the current 
	// vao buffer. this is why the frameOffsetInCurrentBuffer and nbFrames parameters are needed.
	void	AddBufferToVAOMirrorBuffer(bool bStereo, SInt32 vaoIndex,float* buf,SInt32 frameOffsetInCurrentBuffer,SInt32 nbFrames);
	
	// this zeroes the mirror buffers used to level computation
	void	ResetVAOMirrorBuffers(); 
	// this analyzes the vao buffers and computes the levels to be displayed by the mixer. GUARANTEED TO BE CALLED ONCE PER OUTPUT COREAUDIO BUFFER
	void	AnalyzeVAOMirrorBuffers(); 
	
	void	AnalyzeRAOBuffer(float* raoBuffer); 
	
	// attenuates the VAO buffer according to left (and right) levels DO NOT TAKE CARE OF PANNING NOW. WE'LL SEE LATER ON
	void	AttenuateVAOBuffer(bool bStereo, SInt32 vaoIndex,float* samples,SInt32 nbFrames);
	

	// OFFLINE RENDERING FUNCTIONS
	void 	AttenuateVAOBuffer_Offline(bool bStereo, SInt32 vaoIndex,float* samples,SInt32 nbFrames);
	void 	MixMapPanStereoVAOIntoStereoBuffer_Offline(float* dst, UInt32 vaoIndex, float* vaoBuffer, UInt32 nbFrames);
	void 	MixMapPanMonoVAOIntoStereoBuffer_Offline(float* dst, UInt32 vaoIndex, float* vaoBuffer, UInt32 nbFrames);
	
	/*
		actual mixing
	
	void MEMixingInfo::MixMapPanMonoVAOIntoRAOBuffer(float* raoDst, UInt32 vaoIndex, float* vaoBuffer, UInt32 nbFrames)
    void MEMixingInfo::MixMapStereoVAOIntoRAOBuffer(float* raoDst, UInt32 vaoIndex, float* vaoBuffer, UInt32 nbFrames)
    */

	/*
		mute/solo state
	*/
	
	// GETTERS
	
	// whether a VAO or RAO is really muted
	bool	IsMonoVAOMuted(SInt32 monoVAOIndex) const;
	bool	IsStereoVAOMuted(SInt32 stereoVAOIndex) const;
	bool	IsStereoRAOMuted(SInt32 stereoRAOIndex) const;
	
	// which *AO is soloed (returns -1 if no soloed object)
	SInt32	GetSoloedMonoVAO() const;
	SInt32 	GetSoloedStereoVAO() const;
	SInt32	GetSoloedStereoRAO() const;
	
	bool	IsVAOMutedThroughMutedRAO(const MEVAORouting& vaoRouting) const;

	// whether a VAO will sound or not depending on its RAO mute state
	bool	IsMonoVAOMutedThroughMutedRAO(SInt32 monoVAOIndex) const;
	bool	IsStereoVAOMutedThroughMutedRAO(SInt32 stereoVAOIndex) const;
	
	// SETTERS
	// *************************
	// PRIVATE USE : USE COMMANDS INSTEAD !
	// *************************
	void	_SetMonoVAOMute(SInt32 monoVAOIndex, bool bMuted);
	void	_SetStereoVAOMute(SInt32 stereoVAOIndex, bool bMuted);
	void	_SetStereoRAOMute(SInt32 stereorAOIndex, bool bMuted);
	// *************************

	void	_SetSoloedMonoVAO(SInt32 monoVAOIndex);
	void	_SetSoloedStereoVAO(SInt32 stereoVAOIndex);
	void	_SetSoloedStereoRAO(SInt32 stereoRAOIndex);
		
	/*
		VU level functions : must return a value between 0.0 and 100.0
	*/
	float	GetDBFSLevelForMonoVAO(SInt32 vaoIndex) const; 
	float	GetDBFSLevelForStereoVAO(SInt32 vaoIndex, bool left) const;
	float	GetDBFSLevelForStereoRAO(SInt32 raoIndex, bool left) const;
		
	
	float	GetStereoVAOLinearGain(SInt32 vaoIndex, bool bLeft) const;
	float	GetMonoVAOLinearGain(SInt32 vaoIndex) const;
	float	GetStereoRAOLinearGain(SInt32 raoIndex, bool bLeft) const;

	float	GetStereoVAOFaderDB(SInt32 vaoIndex, bool bLeft) const;
	float	GetMonoVAOFaderDB(SInt32 vaoIndex) const;
	float	GetStereoRAOFaderDB(SInt32 raoIndex, bool bLeft) const;
		
	float	GetMonoVAOPanning(SInt32 vaoIndex) const;
	float	GetStereoVAOPanning(SInt32 stereoVAOIndex) const;
	
	// SETTERS
	void	_SetMonoVAOPanning(SInt32 monoVAOIndex, float panning);
	void	_SetStereoVAOPanning(SInt32 stereoVAOIndex, float panning);
//	void	_SetStereoRAOPanning(SInt32 stereoRAOIndex, float panning);
																				
	/*
		actual routing information
	*/
	
	//GETTERS
	SInt32	GetStereoRAOForStereoVAO(SInt32 vaoIndex) const;
	SInt32	GetStereoRAOForMonoVAO(SInt32 vaoIndex) const;
	bool 	IsMonoVAORoutedToStereoRAO(SInt32 vaoIndex) const;
	SInt32	GetMonoRAOForMonoVAO(SInt32 vaoIndex) const;
	
	//SETTERS
	void	_SetStereoRAOForStereoVAO(SInt32 stereoVAOIndex, SInt32 stereoRAOIndex);
	void	_SetStereoRAOForMonoVAO(SInt32 monoVAOIndex, SInt32 stereoRAOIndex);
	void	_SetMonoRAOForMonoVAO(SInt32 monoVAOIndex, SInt32 monoRAOIndex);
		
	void	_SetMonoVAOFaderValue(SInt32 monoVAOIndex,float value);
	void	_SetStereoVAOFaderValue(SInt32 stereoVAOIndex,float value, bool bLeft);
	void	_SetStereoRAOFaderValue(SInt32 stereoRAOIndex,float value, bool bLeft);
		
		
	/*
		names
	*/
	
	// getters
	const char*	GetMonoVAOName(SInt32 monoVAOIndex) const;
	const char*	GetStereoVAOName(SInt32 stereoVAOIndex) const;
	const char*	GetStereoRAOName(SInt32 stereoRAOIndex) const;
						
	// setters (USE COMMANDS !!!)
	void	_SetMonoVAOName(SInt32 monoVAOIndex, const char* name);
	void	_SetStereoVAOName(SInt32 stereoVAOIndex, const char* name);
	void	_SetStereoRAOName(SInt32 stereoRAOIndex, const char* name);
	
	/*
		locks
	*/
	
	// getters
	bool	GetStereoVAOLock(SInt32 stereoVAOIndex) const;
	bool	GetStereoRAOLock(SInt32 stereoRAOIndex) const;
		
	// setters (USE COMMANDS !!!)
	void	_SetStereoVAOLock(SInt32 stereoVAOIndex, bool locked);
	void	_SetStereoRAOLock(SInt32 stereoRAOIndex, bool locked);
	
	/*
		staturation	
	*/
	// getters
	bool	IsMonoVAOSaturated(SInt32 monoVAOIndex) const;	
	bool	IsStereoVAOSaturated(SInt32 stereoVAOIndex, bool bLeft) const;	
	bool	IsStereoRAOSaturated(SInt32 stereoRAOIndex, bool bLeft) const;	
	
	//resetters
	void	ResetMonoVAOSaturation(SInt32 monoVAOIndex);	
	void	ResetStereoVAOSaturation(SInt32 stereoVAOIndex, bool bLeft);	
	void	ResetStereoRAOSaturation(SInt32 stereoRAOIndex, bool bLeft);	
																		
	/*
		packed structure reffer for VAO->RAO mapping
	*/
	MEVAOInfo&	RefMonoVAOMapping(SInt32 monoVAOIndex);
	MEVAOInfo&	RefStereoVAOMapping(SInt32 stereoVAOIndex);
	MERAOInfo&	RefStereoRAOMapping(SInt32 stereoRAOIndex);
	
	void 		OnVAOChange(bool stereo, SInt32 vaoIndex);
	
	// the levels to display (in DBFS value : 0.0dB == full scale) 
	float		fMonoVAODBFS[kMEMixingInfo_NbMaxMonoVAO];
	float		fStereoVAOLeftDBFS[kMEMixingInfo_NbMaxStereoVAO];
	float		fStereoVAORightDBFS[kMEMixingInfo_NbMaxStereoVAO];
	float		fRAOLeftDBFS[kMEMixingInfo_NbMaxRAO];
	float		fRAORightDBFS[kMEMixingInfo_NbMaxRAO];
	
	// whether a DBFS has been greater than 0.0 in the past
	bool		fMonoVAOSatur[kMEMixingInfo_NbMaxMonoVAO];
	bool		fStereoVAOLeftSatur[kMEMixingInfo_NbMaxStereoVAO];
	bool		fStereoVAORightSatur[kMEMixingInfo_NbMaxStereoVAO];
	bool		fStereoRAOLeftSatur[kMEMixingInfo_NbMaxRAO];
	bool		fStereoRAORightSatur[kMEMixingInfo_NbMaxRAO];

	// this counter is set to fNbOfBuffersForSaturationReset at clipping
	// and decremented at each analyzed buffer. when it reaches zero, then
	// the clipping indicator is reset.
	SInt32		fMonoVAOSaturResetCounter[kMEMixingInfo_NbMaxMonoVAO];
	SInt32		fStereoVAOLeftSaturResetCounter[kMEMixingInfo_NbMaxStereoVAO];
	SInt32		fStereoVAORightSaturResetCounter[kMEMixingInfo_NbMaxStereoVAO];
	SInt32		fStereoRAOLeftSaturResetCounter[kMEMixingInfo_NbMaxRAO];
	SInt32		fStereoRAORightSaturResetCounter[kMEMixingInfo_NbMaxRAO];
	
	// dB (-inf to +6) <---> VU Level (0.0 to 100.0) conversion functions
	static float	VULevelTodB(float vuLevel);
	static float 	dBToVULevel(float dB);
	
	static float	dBToFaderValue(float dB);
	static float	FaderValueTodB(float fader);
		
	// dB (anything) <---> gain (anything) conversion functions
	static float 	dBToLinearGain(float dB);
	static float 	linearGainTodB(float gain);

private :
	// the Arme document
	TArmeDoc*	fDoc;

	// the actual mixer info
	MEVAOInfo	fMonoVAOInfos[kMEMixingInfo_NbMaxMonoVAO];
	MEVAOInfo	fStereoVAOInfos[kMEMixingInfo_NbMaxStereoVAO];
	MERAOInfo	fStereoRAOInfos[kMEMixingInfo_NbMaxRAO];
	
	// tmp values for RAO analysis
	// we don't need to initialize them...
	float		fRAOAnalyzeMaxSqAmplL[kMEMixingInfo_NbMaxRAO];
	float		fRAOAnalyzeMaxSqAmplR[kMEMixingInfo_NbMaxRAO];

	// max busses counts
	SInt32		fMaxAssignedStereoRAOCount;
	SInt32		fMaxStereoRAOCount;
	SInt32		fMaxStereoVAOCount;
	SInt32		fMaxMonoVAOCount;
	
	// indices of the soloed *AO busses
	SInt32		fSoloedMonoVAO;
	SInt32		fSoloedStereoVAO;
	SInt32		fSoloedStereoRAO;
	
	// the current number of allocated VAO mirror buffers (these are used for vu meter analysis)
	SInt32		fNbAllocatedMonoVAOMirrorBuffers;
	SInt32		fNbAllocatedStereoVAOMirrorBuffers;
	
	// the vector of pointers to the mirror buffers
	float**		fMonoVAOMirrorBuffers;
	float**		fStereoVAOMirrorBuffers;
	
	// positive value for the dbfs decrease speed, in decibels per coreaudio output buffer
	// it needs to change when the buffer size or sample rate changes
	float		fDBPerBufferAttenuation;
	
	/*! @var fNbOfBuffersForSaturationReset The number of buffers it takes for a clipping red light
		to be turned off from the moment the clipping ends */
	SInt32 		fNbOfBuffersForSaturationReset;
};

inline bool MEMixingInfo::IsVAOMutedThroughMutedRAO(const MEVAORouting& vaoRouting) const
{
	if(vaoRouting.bStereo)
		return IsStereoVAOMutedThroughMutedRAO(vaoRouting.index);
	else
		return IsMonoVAOMutedThroughMutedRAO(vaoRouting.index);
}

inline void MEMixingInfo::MixMapPanVAOIntoRAOBuffer(float* raoDst, const MEVAORouting& vaoRouting, float* vaoBuffer, UInt32 nbFrames)
{
	if(vaoRouting.bStereo)
		MixMapPanStereoVAOIntoRAOBuffer(raoDst, vaoRouting.index, vaoBuffer, nbFrames);
	else
		MixMapPanMonoVAOIntoRAOBuffer(raoDst, vaoRouting.index, vaoBuffer, nbFrames);
}


