/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#ifndef MEQTFloatSampleReader_h
#include "MEQTFloatSampleReader.h"
#endif //MEQTFloatSampleReader_h

#ifndef ZzVerbose_h
#include "ZzVerbose.h"
#endif //ZzVerbose_h

#include "Zz_PathHelpers.h"

#undef VER_DEF_TOPIC
#define VER_DEF_TOPIC MEQTFloatSampleReader

#ifndef ZzString_h
#include "ZzString.h"
#endif //ZzString_h

#ifndef ZzHelpers_h
#include "ZzHelpers.h"
#endif //ZzHelpers_h

// $$$TODO FLOAT $$$TODO FLOAT
// $$$todo warning very unefficient conversion
const float kTWOSToShortConversionFactor = 1.0/32768.0;
inline float TWOSToShort(short r)
{
	return (float)r*kTWOSToShortConversionFactor;
}

//#define USE_AIFF_EXPORT

MEQTFloatSampleReader::MEQTFloatSampleReader(const char* filePath, ArmeErr* err) :
	fSampleRate(0),
	fMovie(0),
	fResRefNum(0),
	fSoundComp(0),
	fSoundTrack(0),
	fNbFrames(0)
{
	fNbChannels = 0;

	OSErr			ok;
	OSStatus 		st;
	Boolean			bok;
	FSSpec			fileSpec;

	// hopefully, everything will be ok
	err->type = kArmeOK;
	
	// init stuff to safe values so that the dtor can test if they were inited or not
	fMovie 		= nil;
	fSoundComp	= 0;
			
	vernl(5,"MEQTFloatSampleReader::MEQTFloatSampleReader() (filePath = " << filePath);
		
    // init QT (maybe it has already been done)
	EnterMovies();
	
	// create a FSSpec for our file
//	bok = FSSpecFromString(&fileSpec, filePath);
	st = ZzPathToFSSpec(ZzString::CreateFromFSRepresentation(filePath).getCFString(), &fileSpec);
	if(st != noErr)
	{
		err->type = kArmeCannotCreateFSSpecFromPath;
		return;
	}
	
	// open the movie file
	ok = OpenMovieFile(&fileSpec, &fResRefNum, fsRdPerm);
	if(ok != noErr)
	{
		err->type = kArmeQuickTimeError;
		err->data = (void*)ok;
		return;
	}
		
	// create the "Movie" instance from the resource number
	ok = NewMovieFromFile(&fMovie,fResRefNum,NULL,NULL,0,NULL);
	if(ok != noErr)
	{
		err->type = kArmeQuickTimeError;
		err->data = (void*)ok;
		return;
	}
	
	// create sound track for the movie
	fSoundTrack = GetMovieIndTrackType(fMovie, 1, SoundMediaType, movieTrackMediaType | movieTrackEnabledOnly);
	
	Initialize();
}

MEQTFloatSampleReader::MEQTFloatSampleReader(const FSSpec& movieFile, int trackIndex, ArmeErr* err) :
	fSampleRate(0),
	fMovie(0),
	fResRefNum(0),
	fSoundComp(0),
	fSoundTrack(0),
	fNbFrames(0)
{
	OSErr			ok;

	fNbChannels = 0;
	
	// hopefully, everything will be ok
	err->type = kArmeOK;
	fMovie 		= nil;
	fSoundComp	= 0;

	vernl(5,"MEQTFloatSampleReader::MEQTFloatSampleReader() (movie = " << movieFile.name << " track = " << trackIndex);

    // init QT (maybe it has already been done)
	EnterMovies();

	// open the movie file
	ok = OpenMovieFile(&movieFile, &fResRefNum, fsRdPerm);
	if(ok != noErr)
	{
		err->type = kArmeQuickTimeError;
		err->data = (void*)ok;
		return;
	}

	// create the "Movie" instance from the resource number
	ok = NewMovieFromFile(&fMovie,fResRefNum,NULL,NULL,0,NULL);
        ::CloseMovieFile(fResRefNum);
	if(ok != noErr)
	{
		err->type = kArmeQuickTimeError;
		err->data = (void*)ok;
		return;
	}
        
	// create sound track for the movie
	fSoundTrack = GetMovieIndTrack(fMovie, trackIndex);

	Initialize();
}

ArmeErr MEQTFloatSampleReader::Initialize()
{
	TimeValue		movieDuration;
	TimeValue		offset;
	TimeScale		timeScale;
	double			durationSec;
	ComponentResult	result;
	ArmeErr			err;

	// at this point, fSoundTrack is already initialized

	// get some debug info
	fTrackDuration = GetTrackDuration(fSoundTrack);
	vernl(5,"the sound track duration is " << fTrackDuration);
	
	timeScale = GetMovieTimeScale(fMovie);
	vernl(5,"the movie time scale is " << timeScale);
	
	offset = GetTrackOffset(fSoundTrack);
	vernl(5,"the sound track offset is " << offset);

	durationSec = (double)fTrackDuration/(double)timeScale;
	vernl(5,"the sound track duration is " << durationSec << "sec");

	fNbFrames = GetMediaSampleCount(GetTrackMedia(fSoundTrack));
	vernl(5,"***** the track media sample count is " << fNbFrames << " audio frames ****");
	
	// let's create an export component that we will use for the read of the
	// MEQTFloatSampleReader instance lifetime
#ifdef USE_AIFF_EXPORT
	fSoundComp = OpenAIFFMovieExportComponent();
#else //USE_AIFF_EXPORT
	fSoundComp = OpenDefaultComponent('spit', 'snd ');
#endif //USE_AIFF_EXPORT
	
	SoundDescriptionHandle	inDesc = (SoundDescriptionHandle)NewHandle(0);
	SoundDescriptionHandle	outDesc = nil;
		
	// 1 means the first sample description in the movie
	GetMediaSampleDescription(GetTrackMedia(fSoundTrack),1,(SampleDescriptionHandle)inDesc);
		
	// now I will try to set the time scale to the sampleRate
	// inDesc[0]->sampleRate is Fixed while TimeScale is long so we need to convert.
	// hopefully, no floating point sample rate exists
	// 
	fSampleRate = inDesc[0]->sampleRate >> 16;
	
	SetMovieTimeScale(fMovie,fSampleRate*128); // so that the duration hints given to ReadFloatFrames are measured (roughly) in audio frames

	fTrackDuration = GetTrackDuration(fSoundTrack);
	vernl(5,"the sound track duration AFTER MOVIE SCALE CHANGE is " << fTrackDuration);
		
	fNbChannels = inDesc[0]->numChannels;
	vernl(5,"the movie number of channels is " << inDesc[0]->numChannels);

	if(fNbChannels > 2)
		return ArmeErr(kArmeErrCantSupportMultichannelMovies);
		
	outDesc = (SoundDescriptionHandle) NewHandleClear(sizeof(SoundDescription));
		
#ifdef USE_AIFF_EXPORT
	outDesc[0]->descSize = sizeof(SoundDescription);
	outDesc[0]->dataFormat = kFloat32Format;
	outDesc[0]->numChannels = inDesc[0]->numChannels;
	outDesc[0]->sampleSize = 32;
	outDesc[0]->sampleRate = inDesc[0]->sampleRate;
#else //USE_AIFF_EXPORT
	outDesc[0]->descSize = sizeof(SoundDescription);
	outDesc[0]->dataFormat = k16BitBigEndianFormat;
	outDesc[0]->numChannels = inDesc[0]->numChannels;
	outDesc[0]->sampleSize = 16;
	outDesc[0]->sampleRate = inDesc[0]->sampleRate;
#endif //USE_AIFF_EXPORT

	// attach the export desc (outDesc) to the compressor component we've just created
	result = MovieExportSetSampleDescription(fSoundComp,(SampleDescriptionHandle)outDesc, 'soun');
	if(result != noErr)
	{
		err.type = kArmeQuickTimeError;
		err.data = (void*)result;
	}
		
	DisposeHandle((Handle)inDesc);
	DisposeHandle((Handle)outDesc);
	return err;
}

MEQTFloatSampleReader::~MEQTFloatSampleReader()
{
	OSErr	err;
	if (fSoundComp)
		CloseComponent(fSoundComp);
	err = CloseMovieFile(fResRefNum);
	ZzAssert(err == noErr);
	DisposeMovie(fMovie); // yes, I know it's strange the calls are in this order
}
	

/*	
Boolean MEQTFloatSampleReader::FSSpecFromString(FSSpec *fsSpec, const char *path)
{
    FSRef _fsRef, *fsRef = &_fsRef;

    if (!path || !fsSpec) 
	{
        return FALSE;
    }

    if (FSPathMakeRef((const UInt8*)path, fsRef, NULL)) 
	{
        return FALSE;
    }

    if (FSGetCatalogInfo(fsRef, kFSCatInfoNone, NULL, NULL, fsSpec, NULL)) 
	{
        return FALSE;
    }

    return TRUE;
}
*/

/*
	the following code uses PutMovieIntoTypedHandle to extract samples off a sound file

	GetMediaSample gives you compressed data in the native format. PutMovieIntoTypedHandle decodes it for 
	you into a format you want.

	Here's some code to do roughly what you want.

	As I said to Ron, if you want to do this iteratively, keep the export component open and make sure that 
	consecutive calls have the new starttime = to the old starttime+duration, and you will get all the
	samples, even though they will be rounded off to frame-sized chunks.

*/

typedef union {
   SoundHeader         s;         // Plain sound header
   CmpSoundHeader      c;         // Compressed sound header
   ExtSoundHeader      e;         // Extended sound header
} CommonSoundHeader, *CommonSoundHeaderPtr;

SInt64 MEQTFloatSampleReader::GetNbFrames() const
{
	return fNbFrames;
}

Boolean	MEQTFloatSampleReader::IsValidMovie()
{
	return (fMovie != nil);
}

// for linker debugging
ArmeErr MEQTFloatSampleReader::ReadFloatFrames(float* buf, TimeValue* startTime, TimeValue durationHint, SInt32* nbFrames)
{
	return ReadFloatFrames(buf, startTime, durationHint, nbFrames, true);
}

ArmeErr MEQTFloatSampleReader::ReadFloatFrames(float* buf, TimeValue* startTime, TimeValue durationHint, SInt32* nbFrames, bool bReplaceOrMix)
{
	vernl(0,"MEQTFloatSampleReader::ReadFloatFrames("<<(void*)buf<<","<<*startTime<<","<<durationHint<<","<<*nbFrames<<")");

	OSErr					result = noErr;
	long					i, guessedSampleBytes, sampleBytes;
	Handle					soundData = NewHandle(0);
	CommonSoundHeaderPtr	sh;
	short					compressionID;
	
	SInt32					shOffset;
	int						dataOffset;
	int						nbSamplesInBuffer;
	float*					floatData = nil;
	short*					shortData = nil; //$$$TODO FLOAT $$$TODO FLOAT
		
	ArmeErr err;
		
	// make sure we don't try to read past the movie end
	TimeValue remainingTime = fTrackDuration - *startTime;
	if( remainingTime < durationHint )
	{
		durationHint = remainingTime;
		vernl(6,"CORRECTED durationHint =" << durationHint);
		if(durationHint == 0)
		{
			// if there are no more tracks to read, then we just return
			*nbFrames = 0;
			DisposeHandle(soundData);
			return err;
		}
	}
		
	if(fSoundTrack ==nil)	
	{
		DisposeHandle(soundData);
		return ArmeErr(kArmeErrNilSoundTrack);
	}

#ifdef USE_AIFF_EXPORT
	result = PutMovieIntoTypedHandle(fMovie, fSoundTrack, 'AIFC', soundData, *startTime, durationHint, 0, fSoundComp);
#else //USE_AIFF_EXPORT
	result = PutMovieIntoTypedHandle(fMovie, fSoundTrack, 'snd ', soundData, *startTime, durationHint, 0, fSoundComp);
#endif //USE_AIFF_EXPORT

	if(result != noErr)
	{
		vernl(0,"result != noErr after PutMovieIntoTypedHandle");
		ZzAssert(0);
		DisposeHandle(soundData);
		return ArmeErr(kArmeErrPutMovieIntoTypedHandle);
	}
	*startTime += durationHint;

	// let's find the sound header address
	result = GetSoundHeaderOffset((SndListResource**)soundData,&shOffset);
	if(result != noErr)
	{
		DisposeHandle(soundData);
		return ArmeErr(kArmeErrGetSoundHeaderOffset);
	}
	
	// let's compute the SoundHeader (or extended) address
	sh = (CommonSoundHeaderPtr) (*soundData + shOffset);

	// we are going to compute the start of the header and 
	// the start of the sound data
	dataOffset = shOffset;
	
	// Extract the sound information based on encode type.
	switch (sh->s.encode) 
	{
	case stdSH:      // Standard sound header
		dataOffset += offsetof(SoundHeader, sampleArea);
		compressionID = notCompressed;
		sampleBytes = sh->s.length;
		break;
	case extSH:    // Extended sound header
		dataOffset += offsetof(ExtSoundHeader, sampleArea);
		compressionID = notCompressed;
		sampleBytes = sh->e.numFrames * sizeof(float) * fNbChannels;
		break;
	case cmpSH: 
		{     
			dataOffset += offsetof(CmpSoundHeader, sampleArea);
			compressionID = sh->c.compressionID;
			UInt32 float32Format = kFloat32Format; // to be able to compare in the debugger
			UInt32 shortFormat = k16BitBigEndianFormat; // to be able to compare in the debugger
			//$$$TODO FLOAT $$$TODO FLOAT
			sampleBytes = sh->c.numFrames * sizeof(short) * fNbChannels; // we have always two channels at the output of the codec
		}
		break;
	default :
		DisposeHandle(soundData);
		return ArmeErr(kArmeErrUnknownSoundHeaderType);
		break;
	}
	
	*nbFrames = sampleBytes / (sizeof(short) * fNbChannels); 
	vvernl(6,(*nbFrames));
	
	HLock(soundData);
	
	// this gives the size of the sound data, in bytes
	guessedSampleBytes = GetHandleSize(soundData)-dataOffset;
	
	// make sure the header and the data are coherent
	if( guessedSampleBytes != sampleBytes )
	{
		DisposeHandle(soundData);
		return ArmeErr(kArmeErrWrongSoundHeaderParsing);
	}
	
	nbSamplesInBuffer = guessedSampleBytes / sizeof(short);
	// $$$TODO FLOAT $$$TODO FLOAT
	//floatData = (float*)(*soundData + dataOffset);
	shortData = (short*)(*soundData + dataOffset);
			
	if( bReplaceOrMix )
	{
		for(i = 0; i < nbSamplesInBuffer; i++)
		{
			buf[i] = TWOSToShort(shortData[i]);
		}
//		memcpy(buf,floatData,guessedSampleBytes);
	}
	else
	{
		for(i = 0; i < nbSamplesInBuffer; i++)
		{
			buf[i] += TWOSToShort(shortData[i]);
		}
//		for(i = 0; i < nbSamplesInBuffer; i++)
//		{
//			buf[i] += floatData[i];
//		}
	}
	HUnlock(soundData);
	DisposeHandle(soundData);
	return err;
}

ComponentInstance MEQTFloatSampleReader::OpenAIFFMovieExportComponent()
{
    ComponentDescription cd;
    cd.componentType = MovieExportType;
    cd.componentSubType = kQTFileTypeAIFF;
    cd.componentManufacturer = SoundMediaType;
    cd.componentFlags = 0;
    cd.componentFlagsMask = 0;

    Component c = ::FindNextComponent( c, &cd );

    return c ? ::OpenComponent(c) : nil;
}

