/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#define MEQTFloatSampleReader_h

#ifndef ZzTypes_h
#include "ZzTypes.h"
#endif //ZzTypes_h

#ifndef MEQTFloatSampleReader_h
#include "MEQTFloatSampleReader.h"
#endif //MEQTFloatSampleReader_h

#ifndef ArmeErrors_h
#include "ArmeErrors.h"
#endif //ArmeErrors_h

class MEQTFloatSampleReader
{
public :
	/*!
		@function Constructor builds an object with a path to an audio file as argument
	*/
				MEQTFloatSampleReader(const char* filePath, ArmeErr* err);
				MEQTFloatSampleReader(const FSSpec& movieFile, int trackIndex, ArmeErr* err);

private :
				MEQTFloatSampleReader(const MEQTFloatSampleReader&);
	ArmeErr 	Initialize();

private :
	static ComponentInstance	OpenAIFFMovieExportComponent();

public :	
	/*!
		@function Destructor frees resources associated with this instance
	*/
	~MEQTFloatSampleReader();
	
	/*!
		@function GetNbFrames returns the number of frames
	*/
	SInt64	GetNbFrames() const;
	
	/*!
		@function GetSampleRate returns the native audio track saml
	*/
	UInt32	GetSampleRate() const;
	
	/*!
		@function IsValidMovie returns TRUE if the movie referenced by the
		sample reader is a valid one.
	*/
	Boolean	IsValidMovie();
	
	SInt32	GetNbChannels() const;

	/*!
		@function ReadFrames reads a set of frames. The duration is passed as
		a pointer because the function replaces it with the actual duration 
		that may be different due to the "frame size" used in some compressors. 
		So, buf needs to be potentially larger than (endFrame - startFrame)*frameSize.
		Right now, the rule of thumb is to allocate 2048 bytes more than the size
		you want to read but we will try to publish a clean way to know this offset 
		later.
	*/
	ArmeErr ReadFloatFrames(float* buf, TimeValue* startTime, TimeValue durationHint, SInt32* nbFrames);
	ArmeErr ReadFloatFrames(float* buf, TimeValue* startTime, TimeValue durationHint, SInt32* nbFrames, bool bReplaceOrMix);
	
public :
//	static Boolean	FSSpecFromString(FSSpec *fsSpec, const char *path);

private :
	UInt32				fSampleRate;
	Movie				fMovie;
	short				fResRefNum; // movie file resource number
	ComponentInstance	fSoundComp;
	Track				fSoundTrack;
	SInt32				fNbChannels;
	SInt64				fNbFrames;
	TimeValue			fTrackDuration;
};

inline UInt32 MEQTFloatSampleReader::GetSampleRate() const
{
	return fSampleRate;
}

inline SInt32 MEQTFloatSampleReader::GetNbChannels() const
{
	return fNbChannels;
}
