/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#ifndef MEQTSCAudioReader_h
#include "MEQTSCAudioReader.h"
#endif //MEQTSCAudioReader_h

#ifndef ZzString_h
#include "ZzString.h"
#endif //ZzString_h

#ifndef ZzHelpers_h
#include "ZzHelpers.h"
#endif //ZzHelpers_h

#ifndef Zz_PathHelpers_h
#include "Zz_PathHelpers.h"
#endif //Zz_PathHelpers_h

#define FILE_SAMPLE_BUFFER_SIZE (64 * 1024)
// Size (in bytes) of chunks of sample data to read from the file at one time. This is uncompressed, unconverted data.
// This is the amount we pass to GetMediaSample(), but that doesn't mean that the actual filesystem reads will necessarily be this size.

// the global C function that feeds the SoundConverter
pascal Boolean fillSoundConverterBuffer(SoundComponentDataPtr *data, void *refCon)
{
    return ((MEQTSCAudioReader*)refCon)->FillSoundConverterBuffer(data);
}

/*
Boolean MEQTSCAudioReader::FSSpecFromString(FSSpec *fsSpec, const char *path)
{
    FSRef _fsRef, *fsRef = &_fsRef;

    if (!path || !fsSpec) 
	{
        return FALSE;
    }

    if (FSPathMakeRef((const UInt8*)path, fsRef, NULL)) 
	{
        return FALSE;
    }

    if (FSGetCatalogInfo(fsRef, kFSCatInfoNone, NULL, NULL, fsSpec, NULL)) 
	{
        return FALSE;
    }

    return TRUE;
}
*/

MEQTSCAudioReader::MEQTSCAudioReader(
	const char* filePath, 
	UInt32 outputSampleRate,
	UInt32 maxFramesToWrite,
	ArmeErr* err)
{
	fNbChannels = 0;
	fOutputSampleRate = outputSampleRate; // if it is 0, then we use the source sample rate
	fMaxFramesToWrite = maxFramesToWrite;

	OSStatus		st;
	FSSpec			fileSpec;
	short			resRefNum;

	// hopefully, everything will be ok
	err->type = kArmeOK;
	
	// init stuff to safe values so that the dtor can test if they were inited or not
	fMovie 		= nil;
			
    // init QT (maybe it has already been done)
	EnterMovies();
	
	// create a FSSpec for our file
	st = ZzPathToFSSpec(ZzString::CreateFromFSRepresentation(filePath).getCFString(), &fileSpec);
	if(st != noErr)
	{
		err->type = kArmeCannotCreateFSSpecFromPath;
		return;
	}

	// open the movie file
	st = OpenMovieFile(&fileSpec, &resRefNum, fsRdPerm);
	if(st != noErr)
	{
		err->type = kArmeQuickTimeError;
		err->data = (void*)st; //never mind this warning........ ok is SInt16 and this is why it complains.
		return;
	}
		
	// create the "Movie" instance from the resource number
	st = NewMovieFromFile(&fMovie,resRefNum,NULL,NULL,0,NULL);
	::CloseMovieFile(resRefNum);
	if(st != noErr)
	{
		err->type = kArmeQuickTimeError;
		err->data = (void*)st;
		return;
	}
	
	// create sound track for the movie
	fSoundTrack = GetMovieIndTrackType(fMovie, 1, SoundMediaType, movieTrackMediaType | movieTrackEnabledOnly);
	
	Initialize();
}

MEQTSCAudioReader::MEQTSCAudioReader(
	const FSSpec&	movieFile, 
	int 			trackIndex, 
	UInt32 			outputSampleRate,
	UInt32 			maxFramesToWrite,
	ArmeErr* 		err)
{
	OSStatus		ok;
	short			resRefNum; // movie file resource number

	fNbChannels = 0;
	fOutputSampleRate = outputSampleRate; // if it is 0, then we use the source sample rate
	fMaxFramesToWrite = maxFramesToWrite;
	
	// hopefully, everything will be ok
	err->type = kArmeOK;
	fMovie 		= nil;

    // init QT (maybe it has already been done)
	EnterMovies();

	// open the movie file
	ok = OpenMovieFile(&movieFile, &resRefNum, fsRdPerm);
	if(ok != noErr)
	{
		err->type = kArmeQuickTimeError;
		err->data = (void*)ok;
		return;
	}
	// create the "Movie" instance from the resource number
	ok = NewMovieFromFile(&fMovie,resRefNum,NULL,NULL,0,NULL);
	::CloseMovieFile(resRefNum);
	
	if(ok != noErr)
	{
		err->type = kArmeQuickTimeError;
		err->data = (void*)ok;
		return;
	}
        
	// create sound track for the movie
	fSoundTrack = GetMovieIndTrack(fMovie, trackIndex);

	*err = Initialize();
}

ArmeErr MEQTSCAudioReader::Initialize()
{
	ArmeErr err;
	bool ok;

	// Get the sound track's media
	fMedia = GetTrackMedia(fSoundTrack);
	if (!fMedia)
		return kArmeQuickTimeErrorNoMedia;
	
    fSoundConverterBufferFillerUPP = NewSoundConverterFillBufferDataUPP(fillSoundConverterBuffer);
    fConverterSourceSampleDataHandle = NewHandle(FILE_SAMPLE_BUFFER_SIZE);
	
	ok = StartSoundConverter();
	if(!ok)
	{	
		ZzAssert(0);
		return ArmeErr(kArmeErrCantStartSoundConverter);
	}
	return err;
}

bool MEQTSCAudioReader::GetMovieSoundFormat( SoundComponentData* outSoundFormat, AudioFormatAtomPtr* outAtom)
{
	OSErr err;
	SoundDescriptionV1Handle sourceSoundDescription;
	Handle extension;

	// Get the description of the sample data
	sourceSoundDescription = (SoundDescriptionV1Handle)NewHandle(0);
	GetMediaSampleDescription(fMedia, 1, (SampleDescriptionHandle)sourceSoundDescription);
	err = GetMoviesError();
	if (ThrowOnOSStatus(err)) 
	{
		DisposeHandle((Handle)sourceSoundDescription);
		return false;
	}

	// Get the "magic" decompression atom.
	// This extension to the SoundDescription information stores data specific to a given audio decompressor.
	// Some audio decompression algorithms require a set of out-of-stream values to configure the decompressor.
	extension = NewHandle(0);
	
	err = GetSoundDescriptionExtension(
		(SoundDescriptionHandle)sourceSoundDescription, 
		&extension, 
		siDecompressionParams);
		
	if (err == noErr) 
	{
		// Copy the atom
		Size size;

		size = GetHandleSize(extension);
		HLock(extension);
		*outAtom = (AudioFormatAtomPtr)malloc(size);
		memcpy(*outAtom, *extension, size);
		HUnlock(extension);
	} 
	else
	{
		// If it doesn't have an atom, that's OK
		*outAtom = NULL;
	}

	// Remember the format of the audio in the movie
	// (converting from a SoundDescription to a SoundComponentData)
	outSoundFormat->flags = 0;
	outSoundFormat->format = (*sourceSoundDescription)->desc.dataFormat;
	outSoundFormat->numChannels = (*sourceSoundDescription)->desc.numChannels;
	outSoundFormat->sampleSize = (*sourceSoundDescription)->desc.sampleSize;
	outSoundFormat->sampleRate = (*sourceSoundDescription)->desc.sampleRate;
	outSoundFormat->sampleCount = 0;
	outSoundFormat->buffer = 0;
	outSoundFormat->reserved = 0;

	DisposeHandle(extension);
	DisposeHandle((Handle)sourceSoundDescription);

	return true;
}

UnsignedFixed ConvertUInt32ToUnsignedFixed(UInt32 uint32Value)
{
    UnsignedFixed fixedValue;

    // High 2 bytes is the integer part of the value
    // Low 2 bytes is the floating point part of the value which is 0
    fixedValue = (uint32Value << 16);

    return fixedValue;
}

bool MEQTSCAudioReader::GetOutputSoundConverterFormat(SoundComponentData* outputSoundConverterFormat)
{
	// create an output format (as SoundComponentData) for the sound converter.
	outputSoundConverterFormat->flags = 0;

	outputSoundConverterFormat->format = kFloat32Format;
	outputSoundConverterFormat->numChannels = 0; // use the source input channels
	outputSoundConverterFormat->sampleSize = 32;
	outputSoundConverterFormat->sampleRate = ConvertUInt32ToUnsignedFixed(fOutputSampleRate);
	outputSoundConverterFormat->sampleCount = 0;
	outputSoundConverterFormat->buffer = NULL;
	outputSoundConverterFormat->reserved = 0;

	return true;
}

bool MEQTSCAudioReader::StartSoundConverter()
{
	OSErr err;
	SoundComponentData outputSoundConverterFormat;
	SoundComponentData sourceSoundConverterFormat;
	AudioFormatAtomPtr audioFormatAtom;

	// Get the audio output's format, in a structure that the SoundConverter understands
	if (!GetOutputSoundConverterFormat(&outputSoundConverterFormat))
		goto errorReturn;

	// Get the format of the input file, along with any additional decompression parameters
	if (!GetMovieSoundFormat(&sourceSoundConverterFormat,&audioFormatAtom))
		goto errorReturn;
		
	// a specified sample rate of 0 means we're using the source file sample rate
	if(outputSoundConverterFormat.sampleRate == 0)
		outputSoundConverterFormat.sampleRate = sourceSoundConverterFormat.sampleRate;
		
	// whatever it is, we store the output sample rate in fFrameRate
	fFrameRate = outputSoundConverterFormat.sampleRate >> 16;
	
	// we're using the source movie number of input channels
	outputSoundConverterFormat.numChannels = sourceSoundConverterFormat.numChannels;
	fNbChannels = outputSoundConverterFormat.numChannels;
		
	// Create the sound converter
	err = SoundConverterOpen(&sourceSoundConverterFormat, &outputSoundConverterFormat, &fSoundConverter);
	if (ThrowOnOSStatus(err) || fSoundConverter == NULL)
		goto errorReturn;

    // If we have a decompression atom, give it to the SoundConverter
	if (audioFormatAtom) 
	{
		err = SoundConverterSetInfo(fSoundConverter, siDecompressionParams, audioFormatAtom);
#define BIG_TEST_HACK
#ifndef BIG_TEST_HACK
		if (ThrowOnOSStatus(err))
			goto errorReturn;
#endif //#ifdef BIG_TEST_HACK
	}        

    // Start reading the movie from the beginning, and remember how long it is
	fCurrentMediaTime = 0;
	fMediaDuration = GetMediaDuration(fMedia);

	// Fill in converterSourceSoundComponentData so as little of it needs to change as possible later on
	fConverterSourceSoundComponentData.desc.flags = kExtendedSoundData;
	fConverterSourceSoundComponentData.desc.format = sourceSoundConverterFormat.format;
	fConverterSourceSoundComponentData.desc.numChannels = sourceSoundConverterFormat.numChannels;
	fConverterSourceSoundComponentData.desc.sampleSize = sourceSoundConverterFormat.sampleSize;
	fConverterSourceSoundComponentData.desc.sampleRate = sourceSoundConverterFormat.sampleRate;
	fConverterSourceSoundComponentData.desc.flags = kExtendedSoundData;
	fConverterSourceSoundComponentData.recordSize = sizeof(ExtendedSoundComponentData);
	fConverterSourceSoundComponentData.extendedFlags = kExtendedSoundSampleCountNotValid | kExtendedSoundBufferSizeValid;
        
	// Finally, begin the conversion
	err = SoundConverterBeginConversion(fSoundConverter);
	if (ThrowOnOSStatus(err))
		goto errorReturn;

	return true;

errorReturn:
	if (fSoundConverter) 
	{
		SoundConverterClose(fSoundConverter);
		fSoundConverter = NULL;
    }

	if (audioFormatAtom)
		free(audioFormatAtom);

	return false;
}

void MEQTSCAudioReader::StopSoundConverter()
{
    if (fSoundConverter)
        SoundConverterClose(fSoundConverter);
    fSoundConverter = NULL;
}


MEQTSCAudioReader::~MEQTSCAudioReader()
{
	StopSoundConverter();

    if (fConverterSourceSampleDataHandle)
        DisposeHandle(fConverterSourceSampleDataHandle);
		
	::DisposeMovieTrack(fSoundTrack);
	::DisposeMovie(fMovie);
}

Boolean MEQTSCAudioReader::FillSoundConverterBuffer(SoundComponentDataPtr* data)
{
	long sourceBytesReturned;
	long numberOfSamples;
	TimeValue sourceReturnedTime, durationPerSample;
	OSErr err;

	if (fCurrentMediaTime >= fMediaDuration)
		return false;

	HUnlock(fConverterSourceSampleDataHandle);

	err = GetMediaSample(
		fMedia,
		fConverterSourceSampleDataHandle,	// sample data is returned into this handle
		0,	// maximum number of bytes of sample data to be returned >>>NO LIMIT<<<
		&sourceBytesReturned,	// the number of bytes of sample data returned
		fCurrentMediaTime,		// starting time of the sample to be retrieved
		&sourceReturnedTime,	// indicates the actual time of the returned sample data
		&durationPerSample,		// duration of each sample in the media
		NULL,				// sample description corresponding to the returned sample data (NULL to ignore)
		NULL,				// index value to the sample description that corresponds to the returned sample data (NULL to ignore)
		0,					// maximum number of samples to be returned (0 to use a value that is appropriate for the media)
		&numberOfSamples,		// number of samples it actually returned
		NULL);				// flags that describe the sample (NULL to ignore)

	HLock(fConverterSourceSampleDataHandle);

	if (ThrowOnOSStatus(err) || sourceBytesReturned == 0)
		return false;

	fCurrentMediaTime = sourceReturnedTime + (durationPerSample * numberOfSamples);

	fConverterSourceSoundComponentData.bufferSize = sourceBytesReturned;
	fConverterSourceSoundComponentData.desc.buffer = (Byte *)*fConverterSourceSampleDataHandle;
	*data = (SoundComponentDataPtr)&fConverterSourceSoundComponentData;

	return true;
}

void MEQTSCAudioReader::Reset()
{
	fCurrentMediaTime = 0;
}

void MEQTSCAudioReader::Seek(SInt64 pos)
{
	fCurrentMediaTime = (TimeValue)pos; //$$$TODO DANGER DANGER DANGER QUICK AWFUL HACK
}

ArmeErr MEQTSCAudioReader::ReadFloatFrames(bool* finished, float* buf, SInt32* nbFrames, float* progress)
{
	ArmeErr armeErr;
	UInt32 bytesToWrite;

	*finished = false;

	bytesToWrite = fMaxFramesToWrite*fNbChannels*sizeof(float);
	
	OSErr err;
	UInt32 bytesWritten;
	UInt32 framesWritten;
	UInt32 outputFlags;

	// progress thus goes from 0 -> 1 BUT NEVER ACTUALLY REACHES 1 if you use the finished flag
	*progress = (double)fCurrentMediaTime/(double)fMediaDuration;


	// Request the sound converter to convert one buffer of data.
	err = SoundConverterFillBuffer(
		fSoundConverter, 
		fSoundConverterBufferFillerUPP, 
		this, buf, bytesToWrite, &bytesWritten, &framesWritten, &outputFlags);
		
	*nbFrames = framesWritten;
	

	if (ThrowOnOSStatus(err)) 
	{
		ZzAssert(0);
		// Act like there's no more data
		bytesWritten = 0;
		outputFlags = kSoundConverterDidntFillBuffer;
	}

	if (outputFlags & kSoundConverterDidntFillBuffer) 
	{
		*finished = true;
	}
	return armeErr;
}

SInt32 MEQTSCAudioReader::GetNbChannels() const
{
	ZzAssert(fNbChannels >= 1 && fNbChannels <= 2);
	return fNbChannels;
}







