/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
 
#ifndef MESFPAudioFile_h
#include "MESFPAudioFile.h"
#endif //MESFPAudioFile_h

#ifndef ZzVerbose_h
#include "ZzVerbose.h"
#endif //ZzVerbose_h

#ifndef ZzHelpers_h
#include "ZzHelpers.h"
#endif //ZzHelpers_h

#ifndef Zz_PathHelpers_h
#include "Zz_PathHelpers.h"
#endif //Zz_PathHelpers_h

#ifndef ZzDebugPrintFunctions_h
#include "ZzDebugPrintFunctions.h"
#endif //ZzDebugPrintFunctions_h

#include "sndlib.h"

#undef VER_DEF_TOPIC
#define VER_DEF_TOPIC MESFPAudioFile

const UInt32 MESFPAudioFile::kTmpBufferNbFrames = 4096;

/*
OSStatus MESFPAudioFile_ACInputProc(
	AudioConverterRef	inAudioConverter,
	UInt32*				outDataSize,
	void**				outData,
	void*				inUserData)
{
	MESFPAudioFile* This = (MESFPAudioFile*)inUserData;
	return This->ACInputProc(inAudioConverter,outDataSize,outData);
   // UInt32   nBytes;
    // you determine how many frames you provide at once
    // (in this case it is FRAME_SIZE2 frames per block)
//    nBytes = FRAME_SIZE2*SAMPLESIZE*NOUTCHAN2;
	// function call to put the samples into the global
	// buffer myaudiobuf
//    getaudiodata(myaudiobuf);
	// pass data to AudioConverter
//    *outData     = myaudiobuf;
//    *outDataSize = nBytes;
}

OSStatus MESFPAudioFile::ACInputProc(
	AudioConverterRef	inAudioConverter,
	UInt32*				outDataSize,
	void**				outData)
{
	OSStatus	st;
	UInt32 bytesToRead = kTmpBufferNbFrames*GetBytesPerFrame();	
	SInt64 inStartingByte = fCurrentPos*GetBytesPerFrame();	
	st = AudioFileReadBytes(fAudioFileID, false, inStartingByte, bytesToRead, fTmpBuffer);
	*outData = (void*)fTmpBuffer;
	return st;
}
*/

MESFPAudioFile::MESFPAudioFile()
{
	fAudioFileID = 0;
	memset(&fOutputStreamDesc,0,sizeof(fOutputStreamDesc));
	fNbFrames = 0;
	fOpenMode = kNotOpen;
	fTheConverter = NULL;
	fCurrentPos = 0;
	fTmpBuffer = new float[kTmpBufferNbFrames*2];
}

MESFPAudioFile::~MESFPAudioFile()
{
	ZzAssert(!IsOpenedForWriting()); // we don't want an exception that deletes a file opened for writing to be allowed. someone must catch the exception, close the file, probably delete the file and then maybe re-throw the exception.
	if(fOpenMode != kNotOpen)
		Close();
	delete [] fTmpBuffer;
}

UInt32 MESFPAudioFile::GetNbChannels() const
{
	ZzAssert(fOpenMode != kNotOpen);
	return fOutputStreamDesc.mChannelsPerFrame;
}

UInt32 MESFPAudioFile::GetFrameRate() const
{
	ZzAssert(fOpenMode != kNotOpen);
	return UInt32(fOutputStreamDesc.mSampleRate); // set by the user of this class
}

SInt64 MESFPAudioFile::GetNbFrames() const
{
	return (SInt64)(fNbFrames);
}

ArmeErr MESFPAudioFile::ReadFloat32ContentToFlatBuffer(float* buffer, SInt64 start, SInt64 length )
{
	ArmeErr err;
	SInt64 nbFrames = GetNbFrames();
	ZzAssert(start >= 0 && start <= nbFrames);
	ZzAssert(length >= 0 && (start+length)<=nbFrames);
	err = Seek(start);
	if(err.type != kArmeOK)
		return err;
	return ReadFloat32Frames(buffer,&length);
}

ArmeErr MESFPAudioFile::WriteFloat32Frames(const float* samples, SInt64 nbFrames)
{
	ArmeErr err;
	ZzAssert(0);
	return err;
}

ArmeErr MESFPAudioFile::ReadFloat32Frames(float* samples, SInt64* nbFrames)
{
	ArmeErr err;
	ZzRtAssert(IsOpenedForReading()); // warning mthread issues
	////////////////////////
	// if the requested size is greater than the tmp buffer size, we
	// split in pieces
	////////////////////////
	if(*nbFrames > kTmpBufferNbFrames)
	{
		SInt64 remainingFramesToRead = *nbFrames;
		while(remainingFramesToRead > 0)
		{
			SInt64 framesToRead;
			if(remainingFramesToRead >= kTmpBufferNbFrames)
				framesToRead = kTmpBufferNbFrames;
			else
				framesToRead = remainingFramesToRead;
			ArmeErr err = ReadFloat32Frames(samples,&framesToRead);
			if(err.type != kArmeOK)
				return err;
			samples += framesToRead*GetNbChannels();			
			remainingFramesToRead -= framesToRead;
		}
		return err; 
	}
	
	OSStatus	st;

//#define READ_USING_PACKETS
#define READ_USING_BYTES

#ifdef READ_USING_PACKETS
	UInt32 numBytesRead = 0;
	UInt32 numPackets = *nbFrames;
	st = AudioFileReadPackets(fAudioFileID, true, &numBytesRead, NULL, fCurrentPos, &numPackets, (void*)fTmpBuffer);
	if(st != noErr)
	{
		err.type = kArmeErrorReadingUsingAudioFileAPI;
		err.data = (void*)st;
		return err;
	}
#elif defined READ_USING_BYTES
	UInt32 numBytesRead = *nbFrames * GetSourceBytesPerFrame();
	SInt64 startingByte = fCurrentPos * GetSourceBytesPerFrame();
	st = AudioFileReadBytes(fAudioFileID, true, startingByte, &numBytesRead, (void*)fTmpBuffer);
	if(st != noErr)
	{
		err.type = kArmeErrorReadingUsingAudioFileAPI;
		err.data = (void*)st;
		return err;
	}
#else // READ_USING_PACKETS and READ_USING_BYTES
#error("1 of READ_USING_BYTES and READ_USING_PACKETS must be defined")
#endif // READ_USING_PACKETS and READ_USING_BYTES
	UInt32 bytesConverted = *nbFrames * GetTargetBytesPerFrame();
	st = AudioConverterConvertBuffer(fTheConverter,numBytesRead,(void*)fTmpBuffer,&bytesConverted,(void*)samples);
	if(st != noErr)
	{
		err.type = kArmeAudioFileAudioConverterError;
		err.data = (void*)st;
	}
	fCurrentPos += *nbFrames;
	return err;
}

ArmeErr  MESFPAudioFile::WriteFloat32FramesFromFlatBuffer(
	const float*	samples, 
	SInt64 			chunkStartTime, 
	SInt64 			dur)
{
	ArmeErr err = Seek(chunkStartTime);
	if(err.type != kArmeOK)
		return err;
	return WriteFloat32Frames(samples,dur);
}
	
ArmeErr MESFPAudioFile::Seek(SInt64 offset)
{
	ZzRtAssert(IsOpenedForReading());
	if(offset < 0 || (UInt64)offset >= fNbFrames)
		return ArmeErr(kArmeErrErrorSeekingInFile);
	fCurrentPos = offset;
	return ArmeErr(kArmeOK);
}

SInt32 MESFPAudioFile::SizeOfSample() const
{
	return 4; //sizeof(float)
}

SInt32 MESFPAudioFile::SizeOfFrame() const
{
	return SizeOfSample()*GetNbChannels();
}

ArmeErr MESFPAudioFile::Close()
{
	ArmeErr 	err;
	OSStatus	st;
	if(fOpenMode == kNotOpen)
		return err;

	AudioFileID audioFileID = fAudioFileID;
	fAudioFileID = 0;
	st = AudioFileClose(audioFileID); // this improves mthread bugs detection
	if(st != noErr)
	{
		err.type = kArmeErrorClosingSFPAudioFile;
		err.data = (void*)st;
		return err;
	}
	
	// we don't need the converter anymore
	AudioConverterDispose(fTheConverter);
	fTheConverter = NULL;
	
	fOpenMode = kNotOpen;
	return err;
}

/*
#define kInv32768 0.000030515578125

void MESFPAudioFile::SInt16ToFloat32InPlace(void* buffer, SInt32 nbSamples)
{
	float* floatBuffer = (float*)buffer;
	SInt16* SInt16Buffer = (SInt16*)buffer;
	for( SInt32 i = nbSamples-1; i >= 0; --i)
	{
		floatBuffer[i] = (float)((double)SInt16Buffer[i]*(double)kInv32768);
	}
}
*/

ArmeErr MESFPAudioFile::ReadFloat32ContentToRingBuffer(
	float*	ringBuffer, 
	SInt32	ringBufferNbBits,
	SInt64	writePosInRingBuf,
	SInt64	start, 
	SInt64	length)
{
	ArmeErr err;

	SInt32 ringBufferNbFrames = 1 << ringBufferNbBits;
		
	ZzRtAssert(fOpenMode==kOpenRead);
	
	// seek to the start of the range in the file
	err = Seek(start);
	if(err.type != kArmeOK)
		return err;
	
	ZzAssert(length <= ringBufferNbFrames);
	UInt32 frameIndexInRingBuffer = writePosInRingBuf & ((1 << ringBufferNbBits) - 1);
	SInt32 sampleIndexInRingBuffer = frameIndexInRingBuffer * GetNbChannels();
	float* dstPtr = ringBuffer + sampleIndexInRingBuffer;
	// part 1 length inframes
	SInt32 part2NbFrames = frameIndexInRingBuffer + length - ( 1 << ringBufferNbBits );
	if(part2NbFrames <= 0)
	{
		SInt64 actualNbFrames = length;
#ifdef SIN_TEST_MESFPAudioFile_ReadContentToRingBuffer
		FillWithStereoFloatSin128(dstPtr,(SInt32)actualNbFrames,0);
#else //SIN_TEST_MESFPAudioFile_ReadContentToRingBuffer
		err = ReadFloat32Frames(dstPtr, &actualNbFrames);
		if(err.type != kArmeOK)
			return err;
#endif //SIN_TEST_MESFPAudioFile_ReadContentToRingBuffer 
	}
	else
	{
		SInt32 part1NbFrames = length - part2NbFrames;
		SInt64 actualNbFrames = part1NbFrames;
#ifdef SIN_TEST_MESFPAudioFile_ReadContentToRingBuffer
		FillWithStereoFloatSin128(dstPtr,(SInt32)actualNbFrames,0);
#else //SIN_TEST_MESFPAudioFile_ReadContentToRingBuffer
		err = ReadFloat32Frames(dstPtr, &actualNbFrames);
		if(err.type != kArmeOK)
			return err;
#endif //SIN_TEST_MESFPAudioFile_ReadContentToRingBuffer 
		actualNbFrames = part2NbFrames;
#ifdef SIN_TEST_MESFPAudioFile_ReadContentToRingBuffer
		FillWithStereoFloatSin128(dstPtr,(SInt32)actualNbFrames,part1NbFrames);
#else //SIN_TEST_MESFPAudioFile_ReadContentToRingBuffer
		err = ReadFloat32Frames(dstPtr, &actualNbFrames);
		if(err.type != kArmeOK)
			return err;
#endif //SIN_TEST_MESFPAudioFile_ReadContentToRingBuffer 
	}
	return err;
}

double MESFPAudioFile::GetDurationSec() const
{
	ZzAssert(fOpenMode == MESFPAudioFile::kOpenRead);
	return (double)GetNbFrames()/(double)GetFrameRate();
}

UInt64 MESFPAudioFile::GetFileNbFrames(const ZzString& zzpath)
{
	AudioFileID	audioFileID;
	ArmeErr		err;
	FSRef		fsref;
	
	OSStatus st = ZzPathToFSRef(zzpath.getCFString(),&fsref);
	if(st != noErr)
	{
		ThrowOnArmeErr(ArmeErr(kArmeAudioFileCannotOpen));
	}
	
	err = WrapAudioFileOpen(&fsref,&audioFileID);
	if(err.type != kArmeOK)
	{
		ThrowOnArmeErr(err);
	}
	
	AudioStreamBasicDescription streamBasicDesc;
	err = WrapGetAudioStreamBasicDesc(audioFileID, &streamBasicDesc);
	if(err.type != kArmeOK)
	{
		AudioFileClose(audioFileID);
		ThrowOnArmeErr(err);
	}
		
	if(streamBasicDesc.mBytesPerPacket != streamBasicDesc.mBytesPerFrame)
	{
		err.type = kArmeAudioFileNotSuitableForRealTime;
		err.data = (void*)0;
		AudioFileClose(audioFileID);
		ThrowOnArmeErr(err);
	}
	
	UInt64 nbFrames;
		
	err = MESFPAudioFile::WrapGetPacketCount(audioFileID,&nbFrames);
	if(err.type != kArmeOK)
	{
		AudioFileClose(audioFileID);
		ThrowOnArmeErr(err);
	}
	
	AudioFileClose(audioFileID);
	return nbFrames;
}

UInt32 MESFPAudioFile::GetFileNbChannels(const ZzString& zzpath)
{
	AudioFileID	audioFileID;
	ArmeErr		err;
	FSRef		fsref;
	
	OSStatus st = ZzPathToFSRef(zzpath.getCFString(),&fsref);
	if(st != noErr)
	{
		ThrowOnArmeErr(ArmeErr(kArmeAudioFileCannotOpen));
	}
	
	err = WrapAudioFileOpen(&fsref,&audioFileID);
	if(err.type != kArmeOK)
	{
		ThrowOnArmeErr(err);
	}
	
	AudioStreamBasicDescription streamBasicDesc;
	err = WrapGetAudioStreamBasicDesc(audioFileID, &streamBasicDesc);
	if(err.type != kArmeOK)
	{
		AudioFileClose(audioFileID);
		ThrowOnArmeErr(err);
	}
		
	if(streamBasicDesc.mBytesPerPacket != streamBasicDesc.mBytesPerFrame)
	{
		err.type = kArmeAudioFileNotSuitableForRealTime;
		err.data = (void*)0;
		AudioFileClose(audioFileID);
		ThrowOnArmeErr(err);
	}
	
	AudioFileClose(audioFileID);
	return streamBasicDesc.mChannelsPerFrame;
}

ArmeErr MESFPAudioFile::IsFileSuitableForRealTime(const ZzString& zzpath)
{
#define MESFPAudioFile_IsFileSuitableForRealTime_DEBUG 0

#if MESFPAudioFile_IsFileSuitableForRealTime_DEBUG
	printf("------------------------------------------------\n");
	printf("MESFPAudioFile::IsFileSuitableForRealTime(%s)\n",zzpath.getUTF8String());
#endif //MESFPAudioFile_IsFileSuitableForRealTime_DEBUG
	AudioFileID	audioFileID;
	ArmeErr		err;
	FSRef		fsref;
	
	printf("MESFPAudioFile::IsFileSuitableForRealTime(%s)\n",zzpath.getUTF8String());
	OSStatus st = ZzPathToFSRef(zzpath.getCFString(),&fsref);
	if(st != noErr)
		return ArmeErr(kArmeAudioFileCannotOpen);

	err = WrapAudioFileOpen(&fsref,&audioFileID);
	if(err.type != kArmeOK)
	{
#if MESFPAudioFile_IsFileSuitableForRealTime_DEBUG
		printf("WrapAudioFileOpen failed\n");
		printf("------------------------------------------------\n");
#endif //MESFPAudioFile_IsFileSuitableForRealTime_DEBUG
		return err;
	}
#if MESFPAudioFile_IsFileSuitableForRealTime_DEBUG
	printf("WrapAudioFileOpen succeeded\n");
#endif //MESFPAudioFile_IsFileSuitableForRealTime_DEBUG
	
	AudioStreamBasicDescription streamBasicDesc;
	err = WrapGetAudioStreamBasicDesc(audioFileID, &streamBasicDesc);
	if(err.type != kArmeOK)
	{
		AudioFileClose(audioFileID);
#if MESFPAudioFile_IsFileSuitableForRealTime_DEBUG
		printf("WrapGetAudioStreamBasicDesc failed\n");
		printf("------------------------------------------------\n");
#endif //MESFPAudioFile_IsFileSuitableForRealTime_DEBUG
		return err;
	}
#if MESFPAudioFile_IsFileSuitableForRealTime_DEBUG
	printf("WrapGetAudioStreamBasicDesc succeeded\n");
	ZzDebugPrint_AudioStreamBasicDescription(&streamBasicDesc);	
	printf("------------------------------------------------\n");
#endif //MESFPAudioFile_IsFileSuitableForRealTime_DEBUG
	
/* DEBUG 
	UInt32 maxPacketSize;
	err = WrapGetMaximumPacketSize(audioFileID, &maxPacketSize);
	if(err.type != kArmeOK)
	{
		AudioFileClose(audioFileID);
		return err;
	}
	ZzRtAssert(streamBasicDesc.mBytesPerPacket == maxPacketSize); 
 GUDEB */
	
	if(streamBasicDesc.mBytesPerPacket != streamBasicDesc.mBytesPerFrame)
	{
		AudioFileClose(audioFileID);
		return ArmeErr(kArmeAudioFileNotSuitableForRealTime);
	}
		
	AudioFileClose(audioFileID);
	return ArmeErr(kArmeOK);
}

ArmeErr MESFPAudioFile::OpenRead(const ZzString& zzpath)
{
	ZzAssert(fOpenMode == MESFPAudioFile::kNotOpen);
	FSRef 		fsref;
	OSStatus 	st;
	ArmeErr 	err;
	st = ZzPathToFSRef(zzpath.getCFString(),&fsref);
	if(st != noErr)
	{
		err.type = kArmeAudioFileCannotOpen;
		err.data = (void*)st;
		return err;
	}

/*
		now it's done with TryToCheckIfLittleEndianFile(FSRef* file);
	// HACK FOR AUDIOFILE API BUG THAT DOESN'T DETECT LITTLE ENDIAN FILES
	// now check if it's AIFC or CDDA...
	bool bHackForceLittleEndianIfSignedInteger = false;
	char extension[1024];
	ZzGetUpperCaseFileExtensionFromPath(extension,1024,zzpath.getUTF8String());
	if( (strncmp(extension,"AIFC",4) == 0) && (extension[4] == 0) )
		bHackForceLittleEndianIfSignedInteger = true;
	if( (strncmp(extension,"CDDA",4) == 0) && (extension[4] == 0) )
		bHackForceLittleEndianIfSignedInteger = true;
*/
	bool littleEndian = TryToCheckIfLittleEndianFile(zzpath.getUTF8String());
	err = OpenRead(&fsref,littleEndian);		
	if(err.type == kArmeOK)
		return err;
	else
		return err; //for bps
}

ArmeErr MESFPAudioFile::OpenRead(const char* path)
{
	ZzString zzpath(path);
	ArmeErr err = OpenRead(zzpath);
	if(err.type == kArmeOK)
		return err;
	else
		return err; //for bps
}


ArmeErr MESFPAudioFile::WrapAudioFileOpen(FSRef* fsref, AudioFileID* pAudioFileID)
{
	ArmeErr err;
	OSStatus st = AudioFileOpen(fsref,fsRdPerm,0,pAudioFileID);
	if(st != noErr)
	{
		err.type = kArmeAudioFileCannotOpen;
		err.data = (void*)st;
		return err;
	}
	return err;
}

ArmeErr MESFPAudioFile::WrapGetAudioStreamBasicDesc(AudioFileID audioFileID, AudioStreamBasicDescription* _streamBasicDesc)
{
	UInt32 						propertySize = 0;
	UInt32 						isWritable = 0;
	ArmeErr						err;
	AudioStreamBasicDescription	streamBasicDesc;
	
	OSStatus st = AudioFileGetPropertyInfo(audioFileID,kAudioFilePropertyDataFormat,&propertySize,&isWritable);
	if(st != noErr)
	{
		err.type = kArmeAudioFileCannotGetInfo;
		err.data = (void*)st;
		return err;
	}

	ZzRtAssert(propertySize == sizeof(streamBasicDesc));

	st = AudioFileGetProperty(
		audioFileID,
		kAudioFilePropertyDataFormat,
		&propertySize,
		((void*)(&streamBasicDesc))
		);
		
	if(st != noErr)
	{
		err.type = kArmeAudioFileCannotGetInfo;
		err.data = (void*)st;
		return err;
	}
	
	// WORKAROUND FOR FILES CONFUSING THE AUDIOFILE API 
	// (such as 11_QDesign.mov)
	if(
		streamBasicDesc.mSampleRate == 0 || 
		streamBasicDesc.mBytesPerPacket == 0 || 
		streamBasicDesc.mFramesPerPacket == 0 ||
		streamBasicDesc.mBytesPerFrame ==  0 || 
		streamBasicDesc.mChannelsPerFrame == 0 || 
		streamBasicDesc.mBitsPerChannel == 0)
	{
		err.type = kArmeAudioFileCannotGetInfo;
		err.data = (void*)st;
		return err;
	}
	
	*_streamBasicDesc = streamBasicDesc;
	return err;
}

ArmeErr MESFPAudioFile::WrapGetMaximumPacketSize(AudioFileID audioFileID, UInt32* _maximumPacketSize)
{
	UInt32 	propertySize = 0;
	UInt32 	isWritable = 0;
	UInt32 	maximumPacketSize;
	ArmeErr	err;

	OSStatus st = AudioFileGetPropertyInfo(audioFileID,kAudioFilePropertyMaximumPacketSize,&propertySize,&isWritable);
	if(st != noErr)
	{
		err.type = kArmeAudioFileCannotGetInfo;
		err.data = (void*)st;
		return err;
	}

	ZzRtAssert(propertySize = sizeof(maximumPacketSize));
	st = AudioFileGetProperty(audioFileID,kAudioFilePropertyMaximumPacketSize,&propertySize,((void*)(&maximumPacketSize)));
	if(st != noErr)
	{
		err.type = kArmeAudioFileCannotGetInfo;
		err.data = (void*)st;
		return err;
	}
	*_maximumPacketSize = maximumPacketSize;
	return err;
}


ArmeErr MESFPAudioFile::WrapGetPacketCount(AudioFileID audioFileID, UInt64* _packetCount)
{
	UInt64	packetCount;
	UInt32 	propertySize = sizeof(UInt64);
	ArmeErr	err;
	
	OSStatus st = AudioFileGetProperty(audioFileID,kAudioFilePropertyAudioDataPacketCount,&propertySize,((void*)(&packetCount)));
	if(st != noErr)
	{
		err.type = kArmeAudioFileCannotGetInfo;
		err.data = (void*)st;
		return err;
	}
	
	if(packetCount > LONG_LONG_MAX)
	{
		// we use SInt64 internally but UInt64 can be used
		err.type = kArmeAudioFileTooBigAFile;
		err.data = (void*)st;
		return err;
	}
	*_packetCount = packetCount;
	return err;
}

ArmeErr MESFPAudioFile::OpenRead(FSRef* fsref, bool isLittleEndian)
{
	ArmeErr err;
	OSStatus st;

	err = WrapAudioFileOpen(fsref,&fAudioFileID);
	if(err.type != kArmeOK)
		return err;		
	
	//////////////////////////////////////////////////////////////////////////////////////////
	// now let's check if the file is suitable for real-time reading
	// (ie the packet size must be equal to one frame)
	//////////////////////////////////////////////////////////////////////////////////////////
	err = WrapGetAudioStreamBasicDesc(fAudioFileID, &fSourceStreamDesc);
	if(err.type != kArmeOK)
	{	
		AudioFileClose(fAudioFileID);
		fAudioFileID = 0;
		return err;
	}
		
	// DEBUG
//	printf("streamBasicDesc = ");
//	ZzDebugPrint_AudioStreamBasicDescription(&fSourceStreamDesc);
	
	//$$$$$$ HACK HACK : workaround for AudioFile API bug
	bool isNotFloat = ( (fSourceStreamDesc.mFormatFlags & kAudioFormatFlagIsFloat) == 0);
	bool isSignedInteger = (fSourceStreamDesc.mFormatFlags & kAudioFormatFlagIsSignedInteger);
	if(isLittleEndian)
	{
		if( isNotFloat && isSignedInteger)
			fSourceStreamDesc.mFormatFlags &= ~kAudioFormatFlagIsBigEndian;
	}
		
	// GUBED
	
	//////////////////////////////////////////////////////////////////////////////////////////
	// now check if the bytes per packet is equal to the bytes per frame, which means frame-perfect seeks and reads,
	// which is the sine qua non condition for "real time" reading
	//////////////////////////////////////////////////////////////////////////////////////////
/*
	UInt32 maximumPacketSize;
	err = WrapGetMaximumPacketSize(fAudioFileID, &maximumPacketSize);
	if(err.type != kArmeOK)
	{	
		AudioFileClose(fAudioFileID);
		fAudioFileID = 0;
		return err;
	}

	// DEBUG
	printf("maximumPacketSize = %lu\n",maximumPacketSize);
	printf("\n");
	// GUBED

	ZzRtAssert(streamBasicDesc.mBytesPerPacket == maximumPacketSize);
*/		
	if(fSourceStreamDesc.mBytesPerPacket != fSourceStreamDesc.mBytesPerFrame)
	{
		err.type = kArmeAudioFileNotSuitableForRealTime;
		err.data = (void*)0;
		AudioFileClose(fAudioFileID);
		fAudioFileID = 0;
		return err;
	}

	//////////////////////////////////////////////////////////////////////////////////////////
	// retrieve file size in packets (packets == frames)
	//////////////////////////////////////////////////////////////////////////////////////////
	// in the .h : UInt64 fNbFrames; // frame == packet because (SFP means Single Frame Packet)
	err = WrapGetPacketCount(fAudioFileID,&fNbFrames);
	if(err.type != kArmeOK)
	{	
		AudioFileClose(fAudioFileID);
		fAudioFileID = 0;
		return err;
	}
	
	//////////////////////////////////////////////////////////////////////////////////////////
	// now create an audio converter that spits data in the same format except it's always
	// float32 data
	//////////////////////////////////////////////////////////////////////////////////////////
	fOutputStreamDesc = fSourceStreamDesc;
	fOutputStreamDesc.mFormatFlags = kAudioFormatFlagsNativeFloatPacked;
	fOutputStreamDesc.mBytesPerPacket = fSourceStreamDesc.mChannelsPerFrame*sizeof(float);
	fOutputStreamDesc.mBytesPerFrame = fSourceStreamDesc.mChannelsPerFrame*sizeof(float);
	fOutputStreamDesc.mBitsPerChannel = 8*sizeof(float);
	
	st = AudioConverterNew(&fSourceStreamDesc, &fOutputStreamDesc, &fTheConverter);
	if(st != noErr)
	{
		err.type = kArmeErrorCreatingSFPAudioFileConverter;
		err.data = (void*)st;
		AudioFileClose(fAudioFileID);
		fAudioFileID = 0;
		fTheConverter = 0;
		return err;
	}
	
	fOpenMode = kOpenRead;
	return err;
}

bool MESFPAudioFile::TryToCheckIfLittleEndianFile(const ZzString& zzpath)
{
	int format = mus_sound_data_format(zzpath.getUTF8String());
	if(format == MUS_ERROR)
		return false;
	if(format == MUS_LSHORT)
		return true;
	else
		return false;
}


/*

	QUICKTIME STINKS

	Movie 				theMovie;
	OSStatus			st;
	FSSpec				fsSpec;
	short				fsRefNum;
	SoundComponentData	sndInfo;
	unsigned long		numFrames;
	unsigned long		dataOffset;
	
	
	st = ::FSGetCatalogInfo(fsRef, kFSCatInfoNone, NULL, NULL, &fsSpec, NULL);
	if(st != noErr)
		return false;
		
	st = ::OpenMovieFile(&fsSpec,&fsRefNum,fsRdPerm);
	if(st != noErr)
		return false;
	
	st = ::ParseAIFFHeader(fsRefNum,&sndInfo,&numFrames,&dataOffset);
	if(st != noErr)
	{
		::CloseMovieFile(fsRefNum);
		return false;
	}
	
	printf("coucou\n");		
	::CloseMovieFile(fsRefNum);
	return false;
*/




