/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
 
#include "MESplitTaskAudioOfflineProc.h"

#ifndef OfflineAudioProcessing_h
#include "OfflineAudioProcessing.h"
#endif //OfflineAudioProcessing_h

#ifndef ZzTimer_h
#include "ZzTimer.h"
#endif ZzTimer_h

#undef VER_DEF_TOPIC
#define VER_DEF_TOPIC MESplitTaskAudioOfflineProc

MESplitTaskAudioOfflineProc::MESplitTaskAudioOfflineProc(OfflineAudioProcessor* oap, OfflineAudioProcessorParams* oapParams) :
	fOAP(oap),
	fParams(oapParams)
{
	vernl(6,"MESplitTaskAudioOfflineProc::MESplitTaskAudioOfflineProc()");
	fPhase = kNotYetStarted;
}

float MESplitTaskAudioOfflineProc::GetProgress() const
{
	vernl(6,"MESplitTaskAudioOfflineProc("<< (void*)this << ")::GetProgress() returning " << fProgress);
	return fProgress;
}
		
#define TIME_MESplitTaskAudioOfflineProc

#ifdef TIME_MESplitTaskAudioOfflineProc
static ZzTimer gTimer1;
#endif TIME_MESplitTaskAudioOfflineProc

ArmeErr MESplitTaskAudioOfflineProc::PerformNextBit()
{
	vernl(6,"MESplitTaskAudioOfflineProc::PerformNextBit()");
	ArmeErr err;
	if(fPhase == kNotYetStarted)
	{
		fPhase = kStarted;
#ifdef TIME_MESplitTaskAudioOfflineProc
		gTimer1.Reset();
#endif //TIME_MESplitTaskAudioOfflineProc
		StartProcessingThread();
	}
	float progress = fParams->GetProgress();
	SetProgress(progress);
#ifdef TIME_MESplitTaskAudioOfflineProc
	if(progress >= 1.0)
	{
		void* retCode;
		//int err = pthread_join(fProcessingThread,&retCode);
		pthread_join(fProcessingThread,&retCode);
		ZzAssert(retCode == (void*)kArmeOK);
		SInt32 ms = gTimer1.GetMS();
		vernl(6,"MESplitTaskAudioOfflineProc duration = " << ms << "ms");
	}
#endif //TIME_MESplitTaskAudioOfflineProc
//	usleep(20000); // let the other thread perform its job..... busy loops are no good
	return err;
}

MESplitTaskAudioOfflineProc::~MESplitTaskAudioOfflineProc()
{
	vernl(6,"MESplitTaskAudioOfflineProc::~MESplitTaskAudioOfflineProc()");
	// make sure we're not trying to delete the processor in the middle 
	// of the processing.
	if(fPhase == kStarted)
	{
		ZzAssert(fParams->GetProgress() >= 0);
	}
}

const char* MESplitTaskAudioOfflineProc::GetDesc() const
{
	return "Applying audio processing"; //$$$TODO provide a description for every audio processor
}

void* MESplitTaskAudioOfflineProc::ProcessingThread()
{
	vernl(6,"MESplitTaskAudioOfflineProc::ProcessingThread()");
	ArmeErr err = fOAP->PerformAudioProcessing(fParams);
	vernl(6,"****************************** MESplitTaskAudioOfflineProc::ProcessingThread() ENDING **************************************");
	return (void*)err.type;
}

void* MESplitTaskAudioOfflineProc_ProcessingThread(void* cookie)
{
	MESplitTaskAudioOfflineProc* This = (MESplitTaskAudioOfflineProc*)cookie;
	return This->ProcessingThread();
}

void MESplitTaskAudioOfflineProc::StartProcessingThread()
{
	pthread_create(&fProcessingThread, NULL, MESplitTaskAudioOfflineProc_ProcessingThread, (void*)this);
}

void MESplitTaskAudioOfflineProc::Cancel()
{
	vernl(6,"MESplitTaskAudioOfflineProc::Cancel()");
	// if the thread hasn't started yet, then no problem. we don't do anything...
	if(fPhase == kNotYetStarted)
		return;
	// otherwise, we call Cancel() on the params so that the next processing loop iteration
	// will notice the IsCancelled() function from the params returns true and the processing
	// needs to stop. Then, the processing function returns err.type = kArmeAudioProcCancelled
	// and this is returned by ProcessingThread(), which, in turns, returns this value in
	// the pthread_join 2nd parameter
	fParams->Cancel(); // only sets ifCancelled to that the thread exits
	void* retCode;
	vernl(6,"MESplitTaskAudioOfflineProc::Cancel() : starting to wait for thread to finish");
	int err = pthread_join(fProcessingThread,&retCode);
	vernl(6,"MESplitTaskAudioOfflineProc::Cancel() : pthread_join() return code " << err);
	if(err == 0) // success
	{
		// we CANNOT call ZzAssert(retCode == (void*)kArmeAudioProcCancelled);
		// because it might happen that the thread ends successfully between the call
		// to Cancel( ) and here, if it was doing its last iteration and thus didn't notice
		// that IsCancelled() returns true. in this case, we still do the same : delete the 
		// new mtga chunk file. this is not a problem at all.
	}
	// if err != 0, it means perhaps the thread has quit so fast that it was already finished
	// by the time of the pthread_join() call. same remark as above.
}

