/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#ifndef MEWSObjAudioPlayer_h
#include "MEWSObjAudioPlayer.h"
#endif //MEWSObjAudioPlayer_h

#ifndef TArmeWSObj_h
#include "TArmeWSObj.h"
#endif //TArmeWSObj_h

#ifndef MEAudioClip_h
#include "MEAudioClip.h"
#endif //MEAudioClip_h

#ifndef ArmeNotifications_h
#include "ArmeNotifications.h"
#endif //ArmeNotifications_h

#ifndef VARPIPlugin_h
#include "VARPIPlugin.h"
#endif VARPIPlugin_h

#ifndef MEAudioTrackPlayer_h
#include "MEAudioTrackPlayer.h"
#endif //MEAudioTrackPlayer_h

#ifndef TArmeMediaEngine_h
#include "TArmeMediaEngine.h"
#endif //TArmeMediaEngine_h

#ifndef MECoreAudioPlayer_h
#include "MECoreAudioPlayer.h"
#endif MECoreAudioPlayer_h

#ifndef MEMixingInfo_h
#include "MEMixingInfo.h"
#endif //MEMixingInfo_h

//#ifndef ZzDisableVerbose_h
//#include "ZzDisableVerbose.h"
//#endif //ZzDisableVerbose_h

#ifndef ZzVerbose_h
//#include "ZzVerbose.h"
#endif //ZzVerbose_h

#ifndef ZzRtAssert_h
#include "ZzRtAssert.h"
#endif //ZzRtAssert_h

#undef VER_DEF_TOPIC
#define VER_DEF_TOPIC MEWSObjAudioPlayer

/*

	update tracks (should be called with the modified clip(s) 
	
	create a track per thread. store absolute clip positions 
	
	each clip references a track
	each clip references an abs time
	
	when asked to supply data with this t-uple (clip,start,len),
	we simply send the data that currently in the clip->track->buffer
	at clip->track->readPos position and we tell the drt thread
	[for that track] to read from start+DISK_BUF_SIZE during len.
	
	The DRT is free to accumulate read requests and merge them.
	
	we have constants :
	
	1) the initial read and write delta pos in the ring buffer. read pos 
	   is used by rt thread while write pos is used by drt thread, let's
	   call it kRBufReadWriteDelta. Write pos must be further.
	   
	2) the amount to accumulate before actually reading = kMinimumFramesForDiskQueuing
	
*/

static int CompareAudioClips(const void* obj1, const void* obj2)
{
	MEAudioClip* clip1 = (MEAudioClip*)(*((void**)obj1));
	MEAudioClip* clip2 = (MEAudioClip*)(*((void**)obj2));
	SInt64 time1 = GET_VAL(clip1->GetAbsoluteStartTime());
	SInt64 time2 = GET_VAL(clip2->GetAbsoluteStartTime());
	if(time1 > time2)
		return 1;
	else if(time1 < time2)
		return -1;
	else
		return 0;
}

static void GrabAudioClipsVisitorFunction(TArmeWSObj* obj, ZzArray<MEAudioClip*>* clipArray)
{
	if(obj->GetType() == TArmeWSObj::kAudioClip)
	{
		MEAudioClip* audioClip = dynamic_cast<MEAudioClip*>(obj);
		ZzAssert(audioClip);
		clipArray->AddItem(audioClip);
		ZzAssert(obj->GetChildrenCount() == 0);
	}
	
	for(SInt32 i = 0; i < obj->GetChildrenCount(); i++)
	{
		GrabAudioClipsVisitorFunction(obj->GetChild(i),clipArray);
	}
}

bool MEWSObjAudioPlayer::WriteToArchive(ArmeArchiveHandler* archiver) const
{
	ArmeArchivable::WriteToArchive(archiver);
	archiver->WriteCPPObject(fTopObj);
	archiver->WriteSInt32(fIndexInDocPlayer);
	archiver->WriteArmeTime(fPosInDoc);
	archiver->WriteArmeTime(fLLocator);
	archiver->WriteArmeTime(fRLocator);
	archiver->WriteBool(fLooping);
	return true;
}

MEWSObjAudioPlayer::MEWSObjAudioPlayer(ArmeArchiveHandler* unarchiver) :
	ArmeArchivable(unarchiver)
{
	ArmeArchivable* archObj = unarchiver->ReadCPPObject();
	fTopObj = dynamic_cast<TArmeWSObj*>(archObj);
	fTopObj->GetDoc()->Subscribe(this);
	ZzAssert(fTopObj);
	fIndexInDocPlayer = unarchiver->ReadSInt32();
	fPosInDoc = unarchiver->ReadArmeTime();
	fLLocator = unarchiver->ReadArmeTime();
	fRLocator = unarchiver->ReadArmeTime();
	fLooping = unarchiver->ReadBool();
	fPlaying = false;
	fPosInDocIncrementation = true;
	fTrackCount = 0; // this will be computed in RecreateTrackMapping()
	
	fPlayHasAlreadyBeenCalled = false;
}

MEWSObjAudioPlayer::MEWSObjAudioPlayer(TArmeWSObj* topObj, SInt32 indexInDocPlayer)
{
	fPosInDocIncrementation = true;
	fTopObj = topObj;
	fTopObj->GetDoc()->Subscribe(this);
	fIndexInDocPlayer = indexInDocPlayer;
	RecreateTrackMapping();
	fPlaying = false;
	fLooping = false;
	fPosInDoc = _ArmeTime(-1);
	fLLocator = _ArmeTime(-1);
	fRLocator = _ArmeTime(-1);
	fTrackCount = 0; // this will be computed in RecreateTrackMapping()

	fPlayHasAlreadyBeenCalled = false;
}

void MEWSObjAudioPlayer::RecreateTrackMapping()
{
	SInt32 i;
	// let's put all the audio clips into a flat array
	ZzArray<MEAudioClip*> clipArray;
	
	GrabAudioClipsVisitorFunction(fTopObj,&clipArray);
	
	for(i = 0; i < clipArray.CountItems(); i++)
	{
		clipArray[i]->fTrackAssignedFlag = false;
		vernl(6,"non sorted array clip number " << i << " = " << clipArray[i]->GetName());
		vernl(6,"       start time = " << (SInt32)(GET_VAL(clipArray[i]->GetAbsoluteStartTime())));
		vernl(6,"       endx time = " << (SInt32)(GET_VAL(clipArray[i]->GetAbsoluteEndxTime())));
	}
	
	// then, let's sort this array according to hpos
	qsort((void*)(clipArray.data),clipArray.CountItems(),sizeof(MEAudioClip*),CompareAudioClips);
	
#if 1
	for(i = 0; i < clipArray.CountItems(); i++)
	{
		vernl(6,"sorted array clip number " << i << " = " << clipArray[i]->GetName());
		vernl(6,"       start time = " << (SInt32)(GET_VAL(clipArray[i]->GetAbsoluteStartTime())));
		vernl(6,"       endx time = " << (SInt32)(GET_VAL(clipArray[i]->GetAbsoluteEndxTime())));
	}
#endif

	//now assign tracks to these clips
	GetDoc()->LockWrite(); 
	
	SInt32 	clipsInThisTrack = 0;
	SInt32 	trackCount = 0;
	bool	bContinueTrackAssignment = true;
	i = 0;
	
	MEAudioClip* lastInsertedClipInThisTrack = nil;
	while(bContinueTrackAssignment)
	{
		// search the next non-unassigned clip in the array which 
		while(i < clipArray.CountItems())
		{
			if(clipArray[i]->fTrackAssignedFlag)
			{
				vernl(6,"clip n"<<i<<" ("<<clipArray[i]->GetName()<<") is already assigned...");
			}
			else
				break;
			i++;
		};
		
		if(i == clipArray.CountItems())
		{
			if(clipsInThisTrack == 0)
			{
				// if the end of the array is reached and there are NO clips in the current
				// track, it means track assignment is over and this track musn't be created
				vernl(6,"i == clipArray.CountItems() and clipsInThisTrack == 0 --> END OF TRACK ASSIGNMENT");
				bContinueTrackAssignment = false;
			}
			else
			{
				// otherwise, it means we need to start a new track
				vernl(6,"i == clipArray.CountItems() and clipsInThisTrack != 0 --> starting a new track");
				i = 0;
				trackCount++;
				clipsInThisTrack = 0;
				lastInsertedClipInThisTrack = NULL;
			}
		}
		else
		{
			// if this is the first clip in this new track, then we may
			// add it regardless of any other condition
			// if there is already at least clip in this track,
			// then we only add this one if it does NOT overlap the first one and
			// if it's of the same type
			MEAudioClip* clip = clipArray[i];
			SInt64 startOfThisClip = GET_VAL(clip->GetAbsoluteStartTime());
			SInt64 endxOfLastInsertedClip;
			if(lastInsertedClipInThisTrack)
				endxOfLastInsertedClip = GET_VAL(lastInsertedClipInThisTrack->GetAbsoluteEndxTime());
			else
				endxOfLastInsertedClip = -1;
			vvernl(6,clip->GetName());
			vvernl(6,startOfThisClip);
			vvernl(6,endxOfLastInsertedClip);
			vvernl(6,clip->IsStereo());
			if(	(lastInsertedClipInThisTrack == NULL) || 
				((startOfThisClip >= endxOfLastInsertedClip) &&
				 (clip->IsStereo() == lastInsertedClipInThisTrack->IsStereo())))
			{
				vernl(6,"clip n"<<i<<" ("<<clip->GetName()<<") can thus be inserted in track " << trackCount);
				clip->SetAudioTrackForPlayer(fIndexInDocPlayer,trackCount);
				clipsInThisTrack++;
				clip->fTrackAssignedFlag = true; // we don't need to visit this clip anymore
				lastInsertedClipInThisTrack = clip;
			}
			else
			{
				vernl(6,"clip n"<<i<<" ("<<clipArray[i]->GetName()<<") CANNOT be inserted ");
			}
			i++;
		}
	}
	
	// add new tracks or delete unused ones according to
	// the new layout
	PerhapsCreateNewTracks(trackCount);

	// since we start messing with the track objects now, it's better not to let the DRT read from the
	// tracks during this.

	// remove all references from the audio tracks
	EmptyAllAudioTracks();
	
#if 1
	for(i = 0; i < clipArray.CountItems(); i++)
	{
		vernl(6,"(2nd array) clip number " << i << " = " << clipArray[i]->GetName());
		vernl(6,"       start time = " << clipArray[i]->GetAbsoluteStartTimeSec() << "sec");
		vernl(6,"       endx time = " << clipArray[i]->GetAbsoluteEndxTimeSec() << "sec");
		vernl(6,"       track assignment = " << clipArray[i]->GetAudioTrackForPlayer(fIndexInDocPlayer));
	}
#endif

	// we reset the 'tail' value of each track (we're using this value to
	// insert silence between clips
	for(i = 0; i < trackCount; i++)
	{
		fTracks[i]->fTmpEndxTimeOfLastInsertedClipInThisTrack = 0;
	}
	
	for(i = 0; i < clipArray.CountItems(); i++)
	{
		MEAudioClip* clip = clipArray.ItemAt(i);
		//$$$
		SInt32 thisTrackIndex = clip->GetAudioTrackForPlayer(fIndexInDocPlayer);
		ZzAssert(thisTrackIndex < trackCount);
		const MEAudioChunkList& list = clip->GetData()->GetChunkList();
		fTracks[thisTrackIndex]->SetStereo(clip->IsStereo());
		SInt64 startOfCurrentClip = GET_VAL(clip->GetAbsoluteStartTime());
		// if the following assertion fails, it (probably) means you've been moving an audio clip
		// so that its absolute start time is now < 0
		ZzAssert(startOfCurrentClip >= fTracks[thisTrackIndex]->fTmpEndxTimeOfLastInsertedClipInThisTrack);
		if(startOfCurrentClip > fTracks[thisTrackIndex]->fTmpEndxTimeOfLastInsertedClipInThisTrack)
			fTracks[thisTrackIndex]->AddSilentChunk(startOfCurrentClip-fTracks[thisTrackIndex]->fTmpEndxTimeOfLastInsertedClipInThisTrack);
		fTracks[thisTrackIndex]->fTmpEndxTimeOfLastInsertedClipInThisTrack = GET_VAL(clip->GetAbsoluteEndxTime());
		for( SInt32 j = 0; j < list.GetNbChunks(); j++)
		{
			fTracks[thisTrackIndex]->AddChunk(list.GetChunk(j));
		}
	}
	fTrackCount = trackCount;
	GetDoc()->UnlockWrite(); 
	// it's very important that we don't lock the doc when deleting unused tracks because
	// the DRT that needs to quit (we'll call pthread_join from here) might be about to call
	// doc->LockRead() when told to be deleted thus leading to a deadlock
	PerhapsDeleteUnusedTracks(trackCount);
}

MEWSObjAudioPlayer::~MEWSObjAudioPlayer()
{
	vernl(6,"MEWSObjAudioPlayer::~MEWSObjAudioPlayer()");
	EmptyAllAudioTracks();
	for(SInt32 i = 0; i < fTracks.CountItems(); i++)
	{
		delete fTracks.ItemAt(i);
	}
}

void MEWSObjAudioPlayer::PerhapsDeleteUnusedTracks(SInt32 newNbTracks)
{
	// now let's adjust the thread pool size according to the new number of tracks
	// if there are NEW tracks, let's add them
	SInt32 i;
	SInt32 deltaNumberOfTracks = newNbTracks - fTracks.CountItems();
		
	// #2 tracks to get rid of
	if(deltaNumberOfTracks < 0)
	{
		for(i = newNbTracks; i < fTracks.CountItems(); i++)
		{
			delete fTracks.ItemAt(i);
		}
		fTracks.SetSize(newNbTracks);
	}
}

void MEWSObjAudioPlayer::PerhapsCreateNewTracks(SInt32 newNbTracks)
{
	// now let's adjust the thread pool size according to the new number of tracks
	// if there are NEW tracks, let's add them
	SInt32 i;
	SInt32 deltaNumberOfTracks = newNbTracks - fTracks.CountItems();
		
	// #1 : new tracks to create
	if(deltaNumberOfTracks > 0)
	{	
		for(i = 0; i < deltaNumberOfTracks; i++)
		{
			fTracks.AddItem(new MEAudioTrackPlayer(this));
		}
	}
}

void MEWSObjAudioPlayer::EmptyAllAudioTracks()
{
	for(SInt32 i = 0; i < fTracks.CountItems(); i++)
	{
		fTracks.ItemAt(i)->EmptyChunkList();
	}
}

//	FillWithStereoFloatSin128(buffers[level], length, start);

void MEWSObjAudioPlayer::NotificationReceived( ZzBroadcaster* source, UInt32 code, void* param)
{
	vernl(6,"MEWSObjAudioPlayer("<<(void*)this<<")::NotificationReceived(" << (void*)source << ", " << code << ", " << (void*)param << ")");
	switch(code)
	{
//	case kNotification_AudioClipMoved : we do this through the hasUpdatedAudioHierarchy system
//		RecreateTrackMapping();
//		break;
//	case kNotification_AudioClipDataHasChanged :
//		RecreateTrackMapping();
//		break;
	}
}

/************************************************************************************************************************
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
************************************************************************************************************************/
void MEWSObjAudioPlayer::MixCurrentDataInRAO(float* raoDst, float** localBuffers, UInt32 nbFrames)
{
	if(!fPlaying)
		return;
		
	float* baseRAODst = raoDst;
		
	SInt32 nbChannels = TArmeMediaEngine::CoreAudioPlayer()->GetNbOutputStereoPairs()*2;

	// the address of the SInt64 pos we are about to modify :
	SInt64 posInDoc = GET_VAL(fPosInDoc);
	
	// store the old value for loop detection
	//SInt64 oldPlayPos = posInDoc;

	// we are going to split the playback in several chunks according to the loop and
	// current position in a similar fashion to the TAudioClip2Player.
	SInt64 lloc = GET_VAL(GetLLocator());
	SInt64 rloc = GET_VAL(GetRLocator());
	SInt64 loopLen = rloc - lloc;
	
	// if the doc is not looping, abs range == doc range
	// if we are already past the loop end, then we do not loop anymore
	if( !IsLooping() || (posInDoc >= rloc) || (rloc <= lloc) )
	{
		MixDataInRAO(raoDst, 0, localBuffers, posInDoc, nbFrames);
		posInDoc += nbFrames;
	}
	else
	{
		// now we are going to compute
		// NBLS = number of full loops before start
		// LBS = last bit between lloc and start
		// NBLE = number of full loops before end
		// LBE = last bit between lloc and end
		
		SInt64 absStart = posInDoc;
		SInt64 absEnd = posInDoc + nbFrames;
		SInt64 debugLength = 0;
	
		if( absStart < lloc && absEnd <= rloc)
		{
			MixDataInRAO(raoDst, 0, localBuffers, absStart, nbFrames);
			posInDoc += nbFrames;
		}
		else
		{
			// if there is a bit before the left locator, create
			// a chunk and forget about it.
			if( absStart < lloc && absEnd >= lloc)
			{
				MixDataInRAO(raoDst, 0, localBuffers, absStart, lloc-absStart);
				raoDst += nbChannels*(lloc-absStart); // stereo, thus frames = 2*samples (float*)
				debugLength += (lloc-absStart);
				absStart = lloc;
				posInDoc = lloc;
			}
		
			// let's make them relative to the left locator
			SInt64 absStartRel = absStart - lloc;
			SInt64 absEndRel = absEnd - lloc;
			SInt64 NBLS = absStartRel / loopLen;
			SInt64 LBS = absStartRel % loopLen; // between 0 and loopLen
			SInt64 NBLE = absEndRel / loopLen;
			SInt64 LBE = absEndRel % loopLen; // between 0 and loopLen
			
			SInt64 nbFullLoops = NBLE - NBLS - 1;
			
			if( nbFullLoops == -1 ) // which means the start and end are in the same loop.
			{
				ZzRtAssert(LBE > LBS);
				// we queue from start to end
				MixDataInRAO(raoDst, (raoDst - baseRAODst)/nbChannels, localBuffers, lloc+LBS, LBE-LBS);
				raoDst += nbChannels*(LBE-LBS); // stereo, thus frames = 2*samples (float*)
				debugLength += LBE-LBS;
				ZzRtAssert(debugLength == nbFrames);
			}
			else // the loop end is at least crossed once
			{
				// we queue from start to rloc
				if(loopLen-LBS > 0) 
					MixDataInRAO(raoDst, (raoDst - baseRAODst)/nbChannels, localBuffers, lloc+LBS, loopLen-LBS);
				raoDst += nbChannels*(loopLen-LBS); // stereo, thus frames = 2*samples (float*)
				debugLength += loopLen-LBS;
				// we queue N times from lloc to rloc
				for(SInt32 i = 0; i < nbFullLoops; i++)
				{
					MixDataInRAO(raoDst, (raoDst - baseRAODst)/nbChannels, localBuffers, lloc, loopLen);
					raoDst += nbChannels*loopLen; // stereo, thus frames = 2*samples (float*)
					debugLength += loopLen;
				}
				// we queue from lloc to end
				if(LBE > 0)
					MixDataInRAO(raoDst, (raoDst - baseRAODst)/nbChannels, localBuffers, lloc, LBE);
				raoDst += nbChannels*LBE; // stereo, thus frames = 2*samples (float*)
				debugLength += LBE;
				ZzRtAssert(debugLength == nbFrames);
			}
			posInDoc = lloc+LBE;
		}
	}
	
	
	// let's detect if someone tried to seek
	
	
	
	// if we need to increment the pos, we copy the new pos to the old one.
	// we don't increment if, for example, the user is dragging the playbar.`
	// NO !!!!!! THIS DOESN'T GOOD AT ALL
	if(fPosInDocIncrementation)
		fPosInDoc = _ArmeTime(posInDoc);
	
/*
	// the movie is synced to the TLC audio player
	if(fTopObj == fTopObj->GetDoc()->GetTopContainer())
	{
		static UInt32 audioVideoResyncCounter = 1;
		//audioVideoResyncCounter++;
				
		// if there was a loop, let's retrigger the resync notification counter
		if(GET_VAL(fPosInDoc) != oldPlayPos + nbFrames)
			audioVideoResyncCounter = 0;
		
		// $$$$TODO $$$todo check why this produces hiccups in the DV playback
		if((audioVideoResyncCounter % 200) == 0)
		{
			//fTopObj->GetDoc()->Notify(kNotification_AdjustMovieArmeTime,NULL);
			audioVideoResyncCounter = 0;
		}
	}
*/
}

/*
void MEWSObjAudioPlayer::MixCurrentDataForVAO(bool bStereo, SInt32 vao, float** localBuffers, SInt32 actualFrames)
{
	if(!fPlaying)
		return;

	float* originalLocalBuffers0 = localBuffers[0];
	
	SInt32 nbChannels = bStereo ? 2 : 1;
	
	// the address of the SInt64 pos we are about to modify :
	SInt64* posInDoc = &fPosInDoc;
	
	// we are going to split the playback in several chunks according to the loop and
	// current position in a similar fashion to the TAudioClip2Player.
	SInt64 lloc = GET_VAL(GetLLocator());
	SInt64 rloc = GET_VAL(GetRLocator());
	SInt64 loopLen = rloc - lloc;
	
	// if the doc is not looping, abs range == doc range
	// if we are already past the loop end, then we do not loop anymore
	if( !IsLooping() || (*posInDoc >= rloc) || (rloc <= lloc) )
	{
		MixDataForVAO(bStereo, vao, localBuffers, *posInDoc, actualFrames);
		*posInDoc += actualFrames;
	}
	else
	{
		// now we are going to compute
		// NBLS = number of full loops before start
		// LBS = last bit between lloc and start
		// NBLE = number of full loops before end
		// LBE = last bit between lloc and end
		
		SInt64 absStart = *posInDoc;
		SInt64 absEnd = *posInDoc + actualFrames;
		SInt64 debugLength = 0;
	
		if( absStart < lloc && absEnd <= rloc)
		{
			MixDataForVAO(bStereo, vao, localBuffers, absStart, actualFrames);
			*posInDoc += actualFrames;
		}
		else
		{
			// if there is a bit before the left locator, create
			// a chunk and forget about it.
			if( absStart < lloc && absEnd >= lloc)
			{
				MixDataForVAO(bStereo, vao, localBuffers, absStart, lloc-absStart);
				localBuffers[0] += nbChannels*(lloc-absStart); // stereo, thus frames = 2*samples (float*)
				debugLength += (lloc-absStart);
				absStart = lloc;
				*posInDoc = lloc;
			}
		
			// let's make them relative to the left locator
			SInt64 absStartRel = absStart - lloc;
			SInt64 absEndRel = absEnd - lloc;
			SInt64 NBLS = absStartRel / loopLen;
			SInt64 LBS = absStartRel % loopLen; // between 0 and loopLen
			SInt64 NBLE = absEndRel / loopLen;
			SInt64 LBE = absEndRel % loopLen; // between 0 and loopLen
			
			SInt64 nbFullLoops = NBLE - NBLS - 1;
			
			if( nbFullLoops == -1 ) // which means the start and end are in the same loop.
			{
				ZzAssert(LBE > LBS);
				// we queue from start to end
				MixDataForVAO(bStereo, vao, localBuffers, lloc+LBS, LBE-LBS);
				localBuffers[0] += nbChannels*(LBE-LBS); // stereo, thus frames = 2*samples (float*)
				debugLength += LBE-LBS;
				ZzAssert(debugLength == actualFrames);
			}
			else // the loop end is at least crossed once
			{
				// we queue from start to rloc
				if(loopLen-LBS > 0) 
					MixDataForVAO(bStereo, vao, localBuffers, lloc+LBS, loopLen-LBS);
				localBuffers[0] += nbChannels*(loopLen-LBS); // stereo, thus frames = 2*samples (float*)
				debugLength += loopLen-LBS;
				// we queue N times from lloc to rloc
				for(SInt32 i = 0; i < nbFullLoops; i++)
				{
					MixDataForVAO(bStereo, vao, localBuffers, lloc, loopLen);
					localBuffers[0] += nbChannels*loopLen; // stereo, thus frames = 2*samples (float*)
					debugLength += loopLen;
				}
				// we queue from lloc to end
				if(LBE > 0)
					MixDataForVAO(bStereo, vao, localBuffers, lloc, LBE);
				localBuffers[0] += nbChannels*LBE; // stereo, thus frames = 2*samples (float*)
				debugLength += LBE;
				ZzAssert(debugLength == actualFrames);
			}
			*posInDoc = lloc+LBE;
		}
	}
	// restore the value of localBuffers[0] that has been modified for the continuous subrequests
	localBuffers[0] = originalLocalBuffers0;
}
*/


#ifdef OLD_VAO_RAO
/************************************************************************************************************************
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
************************************************************************************************************************/
void MEWSObjAudioPlayer::MixDataInRAO(float* raoDst, SInt32 frameOffsetInBaseRAODst, float** localBuffers, SInt64 pos, SInt32 nbFrames)
{
//	vernl(6,"MixDataInRAO() pos = " << (SInt32)pos << " nbFrames = " << (SInt32)nbFrames);
//	TArmeWSObj* tlc = dynamic_cast<TArmeWSObj*>(fTopObj->GetDoc()->GetTopContainer());
	
	for(SInt32 monoVAOIndex = 0; monoVAOIndex < GetDoc()->GetMixingInfo()->GetMaxMonoVAOCount(); monoVAOIndex++)
	{
		if( !GetDoc()->GetMixingInfo()->IsMonoVAOMuted(monoVAOIndex) )
		{
			// only zero the first nbFrames mono frames
			memset((void*)(localBuffers[0]),0,nbFrames*sizeof(float));
			MixObjectDataForVAO(1.0,1.0,false, monoVAOIndex, fTopObj, localBuffers, 0, pos, nbFrames);
			GetDoc()->GetMixingInfo()->AttenuateVAOBuffer(false, monoVAOIndex,localBuffers[0],nbFrames);
			GetDoc()->GetMixingInfo()->AddBufferToVAOMirrorBuffer(false, monoVAOIndex,localBuffers[0],frameOffsetInBaseRAODst,nbFrames);
			////////////////////////////////////////////////////////////////////////////////////////////////////
			
			if(!GetDoc()->GetMixingInfo()->IsMonoVAOMutedThroughMutedRAO(monoVAOIndex))
			{
				// if the RAO is muted, then we don't mixmap the vao in it
				GetDoc()->GetMixingInfo()->MixMapPanMonoVAOIntoRAOBuffer(raoDst,monoVAOIndex,localBuffers[0],nbFrames);
			}
		}
	}
	for(SInt32 stereoVAOIndex = 0; stereoVAOIndex < GetDoc()->GetMixingInfo()->GetMaxStereoVAOCount(); stereoVAOIndex++)
	{
		if( !GetDoc()->GetMixingInfo()->IsStereoVAOMuted(stereoVAOIndex) )
		{
			// only zero the first nbFrames stereo frames
			memset((void*)(localBuffers[0]),0,2*nbFrames*sizeof(float));
			MixObjectDataForVAO(1.0,1.0,true, stereoVAOIndex, fTopObj, localBuffers, 0, pos, nbFrames);
			GetDoc()->GetMixingInfo()->AttenuateVAOBuffer(true, stereoVAOIndex,localBuffers[0],nbFrames);
			GetDoc()->GetMixingInfo()->AddBufferToVAOMirrorBuffer(true, stereoVAOIndex,localBuffers[0],frameOffsetInBaseRAODst,nbFrames);
			//MixDataForVAO(true, stereoVAOIndex, localBuffers, pos, nbFrames);
			if(!GetDoc()->GetMixingInfo()->IsStereoVAOMutedThroughMutedRAO(stereoVAOIndex))
			{
				// if the RAO is muted, then we don't mixmap the vao in it
				GetDoc()->GetMixingInfo()->MixMapPanStereoVAOIntoRAOBuffer(raoDst,stereoVAOIndex,localBuffers[0],nbFrames);
			}
		}
	}
	// tell all audio tracks to queue the next bit of audio. we're using fTrackCount to make sure we're not dealing with
	// tracks being deleted in PerhapsRemovedUnusedTracks()
	for(SInt32 audioTrackIndex = 0; audioTrackIndex < fTrackCount; audioTrackIndex++)
	{
		fTracks[audioTrackIndex]->QueueNextBit(pos,nbFrames); //this means queue nbFrames if current pos is "pos" and we'll reach them in readAhead frames
	}
}

#else //OLD_VAO_RAO
/************************************************************************************************************************
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
************************************************************************************************************************/
void MEWSObjAudioPlayer::MixDataInRAO(float* raoDst, SInt32 frameOffsetInBaseRAODst, float** localBuffers, SInt64 pos, SInt32 nbFrames)
{
	// fTopObj can either be the TLC or a direct children of the TLC, in a future version
	ZzAssert(fTopObj == fTopObj->GetDoc()->GetTopContainer());
	
	for(int iChild = 0; iChild < fTopObj->GetChildrenCount(); iChild++)
	{
		TArmeWSObj* child = fTopObj->GetChild(iChild);
		
		if(!child->IntersectsTimeRange(_ArmeTime(pos),_ArmeTime(nbFrames)))
			continue;
		
		MEVAORouting childRouting = child->GetVAORouting();
		bool vaoMuted = false;
		SInt32 nbChannels = 0;
		if(childRouting.bStereo)
		{
			nbChannels = 2;
			if(GetDoc()->GetMixingInfo()->IsStereoVAOMuted(childRouting.index))
				vaoMuted = true;
		}
		else
		{
			nbChannels = 1;
			if(GetDoc()->GetMixingInfo()->IsMonoVAOMuted(childRouting.index))
				vaoMuted = true;
		}
		
		if(!vaoMuted)
		{
			memset((void*)(localBuffers[0]),0,nbChannels*nbFrames*sizeof(float));
			MixObjectData(1.0,1.0,childRouting.bStereo, child, localBuffers, 0, pos, nbFrames);
			GetDoc()->GetMixingInfo()->AttenuateVAOBuffer(childRouting.bStereo, childRouting.index,localBuffers[0],nbFrames);
			GetDoc()->GetMixingInfo()->AddBufferToVAOMirrorBuffer(childRouting.bStereo, childRouting.index,localBuffers[0],frameOffsetInBaseRAODst,nbFrames);
			if(!GetDoc()->GetMixingInfo()->IsVAOMutedThroughMutedRAO(childRouting))
			{
				// if the RAO is NOT muted, then we mixmap the VAO into it
				GetDoc()->GetMixingInfo()->MixMapPanVAOIntoRAOBuffer(raoDst,childRouting,localBuffers[0],nbFrames);
			}
		}
	}
			
	// tell all audio tracks to queue the next bit of audio. we're using fTrackCount to make sure we're not dealing with
	// tracks being deleted in PerhapsRemovedUnusedTracks()
	for(SInt32 audioTrackIndex = 0; audioTrackIndex < fTrackCount; audioTrackIndex++)
	{
		fTracks[audioTrackIndex]->QueueNextBit(pos,nbFrames); //this means queue nbFrames if current pos is "pos" and we'll reach them in readAhead frames
	}
}
#endif //OLD_VAO_RAO


void MEWSObjAudioPlayer::MixDataForVAO(bool bStereo, SInt32 vaoIndex, float** localBuffers, SInt64 pos, SInt32 nbFrames)
{
	



}

#ifdef OLD_VAO_RAO
/************************************************************************************************************************
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
************************************************************************************************************************/
void MEWSObjAudioPlayer::MixObjectDataForVAO(float startAudioGain, float endAudioGain, bool bStereo, SInt32 vaoIndex, TArmeWSObj* obj, float** localBuffers, SInt32 level, SInt64 pos, SInt32 nbFrames)
{
	if(startAudioGain != endAudioGain)
	{
		//vernl(6,"MEWSObjAudioPlayer::MixObjectDataForVAO() : startAudioGain != endAudioGain for obj = " << obj->GetName());
	}
	
	if((level + 2) > kCoreAudioPlayer_MaxHierarchyDepth-1)
		return; // to prevent crashes when hierarch gets too deep

	SInt32 trackIndex = obj->GetAudioTrackForPlayer(fIndexInDocPlayer);
	
	if(!obj->IntersectsTimeRange(_ArmeTime(pos),_ArmeTime(nbFrames)))
		return;
		
	if(!obj->ContainsAudioObjectsRoutedTo(bStereo,vaoIndex))
		return;
	
	if(trackIndex != -1) // it's really an audio clip
	{
		MEAudioClip* audioClip = dynamic_cast<MEAudioClip*>(obj);
		ZzRtAssert(audioClip);
		if(audioClip->GetVAORouting().bStereo == bStereo && audioClip->GetVAORouting().index == vaoIndex)
		{
			SInt64 postPadding = pos+nbFrames-GET_VAL(obj->GetAbsoluteEndxTime()); // the part after the obj doesn't need to be read
			if(postPadding < 0)
				postPadding = 0;
			SInt64 prePadding = GET_VAL(obj->GetAbsoluteStartTime()) - pos;
			if(prePadding < 0)
				prePadding = 0;
			// let's render the audio file data according to the dezip factor. dezip length will be smaller when looping. i guess that's ok.
			if(bStereo)
				MixFromTrackToStereoBuffer(startAudioGain*obj->fOldAudioGain,endAudioGain*obj->fAudioGain,trackIndex,localBuffers,level,nbFrames,(SInt32)prePadding,(SInt32)postPadding);
			else
				MixFromTrackToMonoBuffer(startAudioGain*obj->fOldAudioGain,endAudioGain*obj->fAudioGain, trackIndex,localBuffers,level,nbFrames,(SInt32)prePadding,(SInt32)postPadding);
			obj->fOldAudioGain = obj->fAudioGain;
			return;
		}
	}
	else
	{
		UInt32 iChild;
		// case 1 : it is only a container.
		if(obj->GetType() != TArmeWSObj::kAudioProcessor)
		{
			for(iChild = 0; iChild < obj->GetChildrenCount(); iChild++)
			{
				TArmeWSObj* child = obj->GetChild(iChild);
				// let's render it, in the same nb of channels. pos is absolute..... we don't use it, anyway
				MixObjectDataForVAO(startAudioGain*obj->fOldAudioGain, endAudioGain*obj->fAudioGain, bStereo, vaoIndex, child, localBuffers, level, pos, nbFrames);
			}
			obj->fOldAudioGain = obj->fAudioGain;
		}
		else
		{
		// case 2 it's a processor render all the data to the level+1 buffer and process it 
			SInt32 nbChannels = 2;
			if(!bStereo)
				nbChannels = 1;
			memset(localBuffers[level+2],0,nbFrames*sizeof(float)*nbChannels);
			for(iChild = 0; iChild < obj->GetChildrenCount(); iChild++)
			{
				TArmeWSObj* child = obj->GetChild(iChild);
				// let's render it, in the same nb of channels. pos is absolute..... we don't use it, anyway
				MixObjectDataForVAO(startAudioGain,endAudioGain,bStereo, vaoIndex, child, localBuffers, level+2, pos, nbFrames);
			}
	
            //REMARK[TS]
            //now the input and output can be either mono or stereo
            //also the plugin api is changed to accept multiple inputs/ouputs
            //however behaviour is default 1 input 1 output
            //so for now, this is a hack, but later the pluginclass needs to be queried
            //for the number of inputs
            
			if(bStereo)
			{
                // process that level+1 buffer
                VARPIPlugin* plugin = obj->GetAudioPlugin(bStereo,vaoIndex);
        
                if(plugin)
                {
                    plugin->SetInputChannels(0, 2);
                    plugin->SetOutputChannels(0, 2);
                    plugin->SetInputBuffer(0, localBuffers[level+2]);
                    plugin->SetOutputBuffer(0, localBuffers[level+1]);
                    plugin->ProtectedProcessFramesOverwrite((SInt32)nbFrames);
                    //call ProtectedProcessFramesOverwrite if the output should be overwritten
                    //call ProtectedProcessFramesAdd if the output should be mixed
                }
                                
				float* src = localBuffers[level+1];
				float* dst = localBuffers[level];
				
				float curGain = obj->fOldAudioGain;
				float incrGain = (obj->fAudioGain - obj->fOldAudioGain)/(float)nbFrames;
							
				for( SInt32 frame = 0; frame < nbFrames; ++frame )
				{
					*dst++ += curGain*(*src++);
					*dst++ += curGain*(*src++);
					curGain += incrGain;
				}
				obj->fOldAudioGain = obj->fAudioGain;
			}
			else
			{
                // process that level+1 buffer
                VARPIPlugin* plugin = obj->GetAudioPlugin(bStereo,vaoIndex);
        
                if(plugin)
                {
                    plugin->SetInputChannels(0, 1);
                    plugin->SetOutputChannels(0, 1);
                    plugin->SetInputBuffer(0, localBuffers[level+2]);
                    plugin->SetOutputBuffer(0, localBuffers[level+1]);
                    plugin->ProtectedProcessFramesOverwrite((SInt32)nbFrames);
                }

				float* src = localBuffers[level+1];
				float* dst = localBuffers[level];
				
				float curGain = obj->fOldAudioGain;
				float incrGain = (obj->fAudioGain - obj->fOldAudioGain)/(float)nbFrames;
							
				for( SInt32 frame = 0; frame < nbFrames; ++frame )
				{
					*dst++ += curGain*(*src++);
					curGain += incrGain;
				}
				obj->fOldAudioGain = obj->fAudioGain;
				//ZzAssert(0); //$$$TODO talk with Tom
			}
		}
	}
}
#endif //#ifdef //OLD_VAO_RAO


#ifndef OLD_VAO_RAO
/************************************************************************************************************************
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
************************************************************************************************************************/
void MEWSObjAudioPlayer::MixObjectData(float startAudioGain, float endAudioGain, bool bStereo, TArmeWSObj* obj, float** localBuffers, SInt32 level, SInt64 pos, SInt32 nbFrames)
{
	if(startAudioGain != endAudioGain)
	{
		//vernl(6,"MEWSObjAudioPlayer::MixObjectDataForVAO() : startAudioGain != endAudioGain for obj = " << obj->GetName());
	}
	
	if((level + 2) > kCoreAudioPlayer_MaxHierarchyDepth-1)
		return; // to prevent crashes when hierarch gets too deep

	SInt32 trackIndex = obj->GetAudioTrackForPlayer(fIndexInDocPlayer);
	
	if(!obj->IntersectsTimeRange(_ArmeTime(pos),_ArmeTime(nbFrames)))
		return;
		
//	if(!obj->ContainsAudioObjectsRoutedTo(bStereo,vaoIndex))
//		return;
	
	if(trackIndex != -1) // it's really an audio clip
	{
		MEAudioClip* audioClip = dynamic_cast<MEAudioClip*>(obj);
		ZzRtAssert(audioClip);

		SInt64 postPadding = pos+nbFrames-GET_VAL(obj->GetAbsoluteEndxTime()); // the part after the obj doesn't need to be read
		if(postPadding < 0)
			postPadding = 0;
		SInt64 prePadding = GET_VAL(obj->GetAbsoluteStartTime()) - pos;
		if(prePadding < 0)
			prePadding = 0;
		// let's render the audio file data according to the dezip factor. dezip length will be smaller when looping. i guess that's ok.
		if(bStereo)
			MixFromTrackToStereoBuffer(startAudioGain*obj->fOldAudioGain,endAudioGain*obj->fAudioGain,trackIndex,localBuffers,level,nbFrames,(SInt32)prePadding,(SInt32)postPadding);
		else
			MixFromTrackToMonoBuffer(startAudioGain*obj->fOldAudioGain,endAudioGain*obj->fAudioGain, trackIndex,localBuffers,level,nbFrames,(SInt32)prePadding,(SInt32)postPadding);
		obj->fOldAudioGain = obj->fAudioGain;
	}
	else
	{
		SInt32 iChild;
		// case 1 : it is only a container.
		if(obj->GetType() != TArmeWSObj::kAudioProcessor)
		{
			for(iChild = 0; iChild < obj->GetChildrenCount(); iChild++)
			{
				TArmeWSObj* child = obj->GetChild(iChild);
				// let's render it, in the same nb of channels. pos is absolute..... we don't use it, anyway
				MixObjectData(startAudioGain*obj->fOldAudioGain, endAudioGain*obj->fAudioGain, bStereo, child, localBuffers, level, pos, nbFrames);
			}
			obj->fOldAudioGain = obj->fAudioGain;
		}
		else
		{
		// case 2 it's a processor render all the data to the level+1 buffer and process it 
			SInt32 nbChannels = 2;
			if(!bStereo)
				nbChannels = 1;
			memset(localBuffers[level+2],0,nbFrames*sizeof(float)*nbChannels);
			for(iChild = 0; iChild < obj->GetChildrenCount(); iChild++)
			{
				TArmeWSObj* child = obj->GetChild(iChild);
				// let's render it, in the same nb of channels. pos is absolute..... we don't use it, anyway
				MixObjectData(startAudioGain,endAudioGain,bStereo, child, localBuffers, level+2, pos, nbFrames);
			}
	
            //REMARK[TS]
            //now the input and output can be either mono or stereo
            //also the plugin api is changed to accept multiple inputs/ouputs
            //however behaviour is default 1 input 1 output
            //so for now, this is a hack, but later the pluginclass needs to be queried
            //for the number of inputs
            
			if(bStereo)
			{
                // process that level+1 buffer
				VARPIPlugin* plugin = obj->GetAudioPlugin();
        
                if(plugin)
                {
                    plugin->SetInputChannels(0, 2);
                    plugin->SetOutputChannels(0, 2);
                    plugin->SetInputBuffer(0, localBuffers[level+2]);
                    plugin->SetOutputBuffer(0, localBuffers[level+1]);
                    plugin->ProtectedProcessFramesOverwrite((SInt32)nbFrames);
                    //call ProtectedProcessFramesOverwrite if the output should be overwritten
                    //call ProtectedProcessFramesAdd if the output should be mixed
                }
                                
				float* src = localBuffers[level+1];
				float* dst = localBuffers[level];
				
				float curGain = obj->fOldAudioGain;
				float incrGain = (obj->fAudioGain - obj->fOldAudioGain)/(float)nbFrames;
							
				for( SInt32 frame = 0; frame < nbFrames; ++frame )
				{
					*dst++ += curGain*(*src++);
					*dst++ += curGain*(*src++);
					curGain += incrGain;
				}
				obj->fOldAudioGain = obj->fAudioGain;
			}
			else
			{
                // process that level+1 buffer
                VARPIPlugin* plugin = obj->GetAudioPlugin();
        
                if(plugin)
                {
                    plugin->SetInputChannels(0, 1);
                    plugin->SetOutputChannels(0, 1);
                    plugin->SetInputBuffer(0, localBuffers[level+2]);
                    plugin->SetOutputBuffer(0, localBuffers[level+1]);
                    plugin->ProtectedProcessFramesOverwrite((SInt32)nbFrames);
                }

				float* src = localBuffers[level+1];
				float* dst = localBuffers[level];
				
				float curGain = obj->fOldAudioGain;
				float incrGain = (obj->fAudioGain - obj->fOldAudioGain)/(float)nbFrames;
							
				for( SInt32 frame = 0; frame < nbFrames; ++frame )
				{
					*dst++ += curGain*(*src++);
					curGain += incrGain;
				}
				obj->fOldAudioGain = obj->fAudioGain;
				//ZzAssert(0); //$$$TODO talk with Tom
			}
		}
	}
}
#endif //#ifndef //OLD_VAO_RAO


void MEWSObjAudioPlayer::Preroll(SInt64 pos)
{
	for(SInt32 iTrack = 0; iTrack < fTracks.CountItems(); iTrack++)
	{
		MEAudioTrackPlayer* trackPlayer = fTracks.ItemAt(iTrack);
		trackPlayer->Preroll(pos);
	}
}

void MEWSObjAudioPlayer::Seek(ArmeTime newTime)
{
	fPosInDoc = newTime;
}

void MEWSObjAudioPlayer::SetLLocator(ArmeTime leftLocator)
{
	fLLocator = leftLocator;
}

void MEWSObjAudioPlayer::SetRLocator(ArmeTime rightLocator)
{
	fRLocator = rightLocator;
}

void MEWSObjAudioPlayer::Play()
{
	if(!fPlayHasAlreadyBeenCalled)
	{
		TArmeMediaEngine::ForceRestartOutputDevice();
		fPlayHasAlreadyBeenCalled = true;
	}
	fPlaying = true;
}

void MEWSObjAudioPlayer::Stop()
{
	fPlaying = false;
	fTopObj->ResetRecursive();
}

/************************************************************************************************************************
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
************************************************************************************************************************/
TArmeDoc* MEWSObjAudioPlayer::GetDoc()
{
	return fTopObj->GetDoc();
}

/************************************************************************************************************************
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
************************************************************************************************************************/
ArmeTime MEWSObjAudioPlayer::GetLLocator() const
{
	return fLLocator;
}

/************************************************************************************************************************
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
************************************************************************************************************************/
ArmeTime MEWSObjAudioPlayer::GetRLocator() const
{
	return fRLocator;
}

ArmeTime MEWSObjAudioPlayer::GetPlayPos() const
{
	return fPosInDoc;
}

bool MEWSObjAudioPlayer::IsLooping() const
{
	return fLooping;
}

bool MEWSObjAudioPlayer::IsPlaying() const
{
	return fPlaying;
}

bool MEWSObjAudioPlayer::ToggleLoop()
{
	fLooping = !fLooping;
	return fLooping;
}

/************************************************************************************************************************
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
************************************************************************************************************************/
// if the track is stereo, we can directly render into the localBuffers[level]
// otherwise, we render mono into localBuffers[level+1] and then we copy each source sample to both samples
// in the target buffer.
void MEWSObjAudioPlayer::MixFromTrackToStereoBuffer(float startGain, float endGain, SInt32 trackIndex,float** localBuffers, SInt32 level, SInt32 nbFrames, SInt32 prePadding, SInt32 postPadding)
{
	MEAudioTrackPlayer* trackPlayer = fTracks.ItemAt(trackIndex);
	if(trackPlayer->IsStereo())
	{
		trackPlayer->MixFromRingBuffer(startGain, endGain, localBuffers[level],nbFrames,prePadding,postPadding);
	}
	else
	{
		memset(localBuffers[level+1],0,nbFrames*sizeof(float)*1); // it's mono
		trackPlayer->MixFromRingBuffer(startGain, endGain, localBuffers[level+1],nbFrames,prePadding,postPadding);
		float* srcFrame = localBuffers[level+1] + prePadding; // track is mono
		float* dstFrame = localBuffers[level] + 2*prePadding; // buffer is stereo
		for(SInt32 frame = 0; frame < nbFrames-(prePadding+postPadding); frame++) // we don't need to mix zeroes !!!! (pre and post padding are zeroes)
		{
			*dstFrame += *srcFrame;
			dstFrame++;
			*dstFrame += *srcFrame;
			dstFrame++;
			srcFrame++;
		}
	}
}

/************************************************************************************************************************
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME REALTIME 
************************************************************************************************************************/
void MEWSObjAudioPlayer::MixFromTrackToMonoBuffer(float startGain, float endGain, SInt32 trackIndex,float** localBuffers, SInt32 level, SInt32 nbFrames, SInt32 prePadding, SInt32 postPadding)
{
	MEAudioTrackPlayer* trackPlayer = fTracks.ItemAt(trackIndex);
	if(trackPlayer->IsStereo())
	{
		memset(localBuffers[level+1],0,nbFrames*sizeof(float)*2); // it's stereo
		trackPlayer->MixFromRingBuffer(startGain,endGain,localBuffers[level+1],nbFrames,prePadding,postPadding);
		float* srcFrame = localBuffers[level+1] + 2*prePadding; // stereo track
		float* dstFrame = localBuffers[level] + prePadding; // mono buffer
		for(SInt32 frame = 0; frame < nbFrames-(prePadding+postPadding); frame++)
		{
			*dstFrame += *srcFrame;
			srcFrame++;
			*dstFrame += *srcFrame;
			srcFrame++;
			dstFrame++;
		}
	}
	else
	{
		trackPlayer->MixFromRingBuffer(startGain,endGain,localBuffers[level],nbFrames,prePadding,postPadding);
	}
}

void MEWSObjAudioPlayer::SetPosInDocIncrementation(bool incr)
{
	fPosInDocIncrementation = incr;
}

