/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#define MEWSObjAudioPlayer_h

#ifndef MEWSObjAudioPlayer_h
#include "MEWSObjAudioPlayer.h"
#endif //MEWSObjAudioPlayer_h

#ifndef ZzTypes_h
#include "ZzTypes.h"
#endif //ZzTypes_h

#ifndef ZzArray_h
#include "ZzArray.h"
#endif //ZzArray_h

#ifndef ZzLocker_h
#include "ZzLocker.h"
#endif //ZzLocker_h

#ifndef ArmeBasicTypes_h
#include "ArmeBasicTypes.h"
#endif //ArmeBasicTypes_h

#ifndef ArmeArchivable_h
#include "ArmeArchivable.h"
#endif //ArmeArchivable_h

#ifndef ZzBroadcaster_h
#include "ZzBroadcaster.h"
#endif //ZzBroadcaster_h

#ifndef TArmeWSObj_h
#include "TArmeWSObj.h"
#endif //TArmeWSObj_h

class MEAudioClip;
class TArmeDoc;
class MEAudioTrackPlayer;

class MEWSObjAudioPlayer : public ArmeArchivable, public ZzListener
{
private :
			MEWSObjAudioPlayer(const MEWSObjAudioPlayer&) {};
public :
			MEWSObjAudioPlayer(TArmeWSObj* topObj, SInt32 indexInDocPlayer);
	virtual	~MEWSObjAudioPlayer();

public :
	virtual void	NotificationReceived( ZzBroadcaster* source, UInt32 code, void* param); 

public :
	virtual  bool	WriteToArchive(ArmeArchiveHandler* archiver) const;
					MEWSObjAudioPlayer(ArmeArchiveHandler* unarchiver);

	
	// whether the doc read position is incremented at each call to MixCurrentDataInRAO()
	void	SetPosInDocIncrementation(bool incr);

	void 	MixCurrentDataInRAO(float* raoDst, float** localBuffers, UInt32 nbFrames);
	void 	MixDataInRAO(float* raoDst, SInt32 frameOffsetInBaseRAODst, float** localBuffers, SInt64 pos, SInt32 nbFrames);
	
	void 	MixFromTrackToStereoBuffer(float startGain, float endGain, SInt32 trackIndex,float** localBuffers, SInt32 level, SInt32 nbFrames, SInt32 prePadding, SInt32 postPadding);
	void 	MixFromTrackToMonoBuffer(float startGain, float endGain, SInt32 trackIndex,float** localBuffers, SInt32 level, SInt32 nbFrames, SInt32 prePadding, SInt32 postPadding);

	/*! 
		@function MixObjectDataForVAO
		This function must put the data for "obj" on VAO bus, at absolute position pos, in buffer localBuffers[level], and
		must put it into bStereo mode. The request spans nbFrames frames.
		
		OBSOLETE
	*/

#ifdef OLD_VAO_RAO
	void MixObjectDataForVAO(float startGain, float endGain, bool bStereo, SInt32 vaoIndex, TArmeWSObj* obj, float** localBuffers, SInt32 level, SInt64 pos, SInt32 nbFrames);
#else //OLD_VAO_RAO
	/*! 
		@function MixObjectData
		This function must put the data for "obj", at absolute position pos, in buffer localBuffers[level], and
		must put it into bStereo mode. The request spans nbFrames frames.
	*/
	void MixObjectData(float startGain, float endGain, bool bStereo, TArmeWSObj* obj, float** localBuffers, SInt32 level, SInt64 pos, SInt32 nbFrames);
#endif //OLD_VAO_RAO

	
	/*
		@function MixCurrentDataForVAO
		this function must plat the current output for the vao track 
	*/
//	void  	MixCurrentDataForVAO(bool bStereo, SInt32 vao, float** localBuffers, SInt32 actualFrames);

	/*! 
		@function MixObjectData
		This function must put the data for the given bStereo vao for the top object, at absolute position pos, in buffer 
		localBuffers[0]. The request spans nbFrames frames
	*/
	void 	MixDataForVAO(bool bStereo, SInt32 vaoIndex, float** localBuffers, SInt64 pos, SInt32 nbFrames);

	void	RecreateTrackMapping();
	void	EmptyAllAudioTracks();
	void 	PerhapsCreateNewTracks(SInt32 newNbTracks);
	void 	PerhapsDeleteUnusedTracks(SInt32 newNbTracks);
	
	TArmeDoc*						GetDoc();

	
	bool							fPosInDocIncrementation;
	
	/*! @function Play This method tells player to start playing the document */
	void							Play();
	
	/*! @function Stop stops playback */
	void							Stop();
	
	/*! @function Seek moves the play position (result is not immediate due to buffering). Not thread-safe ! */
	void							Seek(ArmeTime newTime);

	/*! @function SetLLocator moves the left locator. Not thread-safe ! */
	void							SetLLocator(ArmeTime leftLocator);

	/*! @function SeRLLocator moves the right locator. Not thread-safe !  */
	void							SetRLocator(ArmeTime rightLocator);
	
	/*! @function GetLLocator Returns the left locator pos. Not thread-safe ! */
	ArmeTime 						GetLLocator() const;

	/*! @function GetRLocator Returns the right locator pos. Not thread-safe ! */
	ArmeTime 						GetRLocator() const;
	
	/*! @function IsPlaying Returns whether the player is playing or not */
	bool							IsPlaying() const;
	
	/*! @function ToggleLoop Switches the player state to/from looped */
	bool							ToggleLoop();

	/*! @function IsLooping Returns whether the wsobj player is in the "looping between locators" mode */
	bool							IsLooping() const;
	
	/*! @function GetPlayPos Returns the current play position in the player (meaningful even if paused or stopped) */
	ArmeTime						GetPlayPos() const;
	
	/*! @function Reload the buffers so that playback from position "pos" is instantaneous */
	void							Preroll(SInt64 pos);
	
	/*! @function CancelPreroll clears the buffer in case of seeking to avoid sticky buffers */
//	void							CancelPreroll();
	
private :
	// array
	ZzArray<MEAudioTrackPlayer*>	fTracks;
	// this variable is very important. 99.99% of the time it's equal to fTracks.CountItems(), except 
	// when the track is being delete OUTSIDE OF A LOCK WRITE/READ. It's deleted outside of the lock because 1) lock shouldn't be held for a long 
	// time and 2) because the track deletion means pthread_joining the DRT, which does a lockRead inside. Calling LockRead inside of a LockWrite 
	// bloack would lead to a deadlock
	SInt32							fTrackCount; 
	TArmeWSObj*						fTopObj;
	SInt32							fIndexInDocPlayer;
	
	/*! @var fPlayParamsLocker The mutex object that protects the play params (play position, locators, etc...) */
//	ZzLocker						fPlayParamsLocker;
	
	/*! @var fPosInDoc The current REAL playback position */
	ArmeTime						fPosInDoc;
	
	/*! @var fLLocator The left locator position */
	ArmeTime						fLLocator;
	
	/*! @var fRLocator The right locator position */
	ArmeTime						fRLocator;
	
	/*! @var fPlaying Is the player being played or not ? */
	bool							fPlaying;
	
	/*! @var fLooping Is the player being played in loop or or not ? */
	bool							fLooping;
	
	/* @var fPlayHasAlreadyBeenCalled DEBUG : SEE TArmeMediaEngine::ForceRestartOutputDevice(); */
	bool 							fPlayHasAlreadyBeenCalled;
};

/*
inline TArmeDoc* MEWSObjAudioPlayer::GetDoc()
{
	return fTopObj->GetDoc();
}

inline ArmeTime MEWSObjAudioPlayer::GetLLocator() const
{
	return fLLocator;
}

inline ArmeTime MEWSObjAudioPlayer::GetRLocator() const
{
	return fRLocator;
}

inline ArmeTime MEWSObjAudioPlayer::GetPlayPos() const
{
	return _ArmeTime(fPosInDoc);
}

inline bool MEWSObjAudioPlayer::IsLooping() const
{
	return fLooping;
}

inline bool MEWSObjAudioPlayer::IsPlaying() const
{
	return fPlaying;
}
*/







