/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#include "ME_AudioChunks.h"

#ifndef TArmeDoc_h
#include "TArmeDoc.h"
#endif //TArmeDoc_h

#ifndef MESFPAudioFile_h
#include "MESFPAudioFile.h"
#endif //MESFPAudioFile_h

#ifndef Zz_PathHelpers_h
#include "Zz_PathHelpers.h"
#endif //Zz_PathHelpers_h

#ifndef ZzHelpers_h
#include "ZzHelpers.h"
#endif //ZzHelpers_h

#include "TArmeMediaEngine.h"

#ifndef MEApplicationServices_h
#include "MEApplicationServices.h"
#endif //MEApplicationServices_h

MEAudioChunkFileIndex kMEUninitializedChunkFileIndex(-1);

void MEAudioChunkFileIndex::WriteToArchive(ArmeArchiveHandler* archiver) const
{
	archiver->WriteSInt32(data);
}

void MEAudioChunkFileIndex::ReadFromArchive(ArmeArchiveHandler* unarchiver)
{
	data = unarchiver->ReadSInt32();
}

void MEAudioChunk::WriteToArchive(ArmeArchiveHandler* archiver) const
{
	chunkFileIndex.WriteToArchive(archiver);
	archiver->WriteSInt64(startInFile);
	archiver->WriteSInt64(duration);
	archiver->WriteCPPObject(doc);
}

void MEAudioChunk::ReadFromArchive(ArmeArchiveHandler* unarchiver)
{
	chunkFileIndex.ReadFromArchive(unarchiver);
	startInFile = unarchiver->ReadSInt64();
	duration = unarchiver->ReadSInt64();
	ArmeArchivable* archDoc;
	archDoc = unarchiver->ReadCPPObject();
	doc = dynamic_cast<TArmeDoc*>(archDoc);
	doc->AudioChunkFileManager()->RetainFile(chunkFileIndex);
	ZzAssert(doc);
}

const char* MEAudioChunk::GetSFPPath() const
{
	const MEChunkFileInfo* info = doc->AudioChunkFileManager()->GetChunkFileInfo(chunkFileIndex);
	ZzRtAssert(!info->bMTGA);
	ZzRtAssert(info->fPath);
	
	//from version 1.4 all the source files are referenced by an alias in the project directory
	if (info->bFileIsAlias)
	{
		const char* ret = TArmeMediaEngine::ApplicationServices()->ResolveAlias(info->fPath);
		return ret;
	}
	else
		return info->fPath;
}
	
void MEAudioChunk::GetPath(char* path, SInt32 bufferSize) const
{
	MEAudioChunkFileManager::GetPath(doc,path,bufferSize,chunkFileIndex);
}

MEChunkFileInfo::MEChunkFileInfo()
{
	bMTGA = true;
	fPath = NULL;
	fRefCount = 0;
	fOwnedByMontage = true;
}

MEChunkFileInfo::~MEChunkFileInfo()
{ 
	if(fPath)
	{
		delete [] fPath;
	}
	fPath = (char*)0xDEADBEEF;
}

void MEAudioChunkFileManager::GetPath(const TArmeDoc* doc, char* path, UInt32 bufferSize, MEAudioChunkFileIndex index)
{
#ifdef DEBUG
	ZzRtAssert(doc);
#endif //DEBUG
	const MEAudioChunkFileManager* audioChunkFileManager = doc->AudioChunkFileManager();
	const MEChunkFileInfo* info = audioChunkFileManager->GetChunkFileInfo(index);
	if(info->fPath == NULL)
	{
		ZzRtAssert(info->bMTGA);
		GetMTGAPath(doc, path, bufferSize, index);
	}
	else
	{
		ZzRtAssert(bufferSize >= strlen(info->fPath) + 1);
		strcpy(path,info->fPath);
	}
}

void MEAudioChunkFileManager::GetMTGAPath(const TArmeDoc* doc, char* path, UInt32 bufferSize, MEAudioChunkFileIndex index)
{
	const MEAudioChunkFileManager* audioChunkFileManager = doc->AudioChunkFileManager();
	if(index.data < audioChunkFileManager->GetNbChunkFileInfos() )
	{
		// if the index is IN the correct range, make sure we're not asking the
		// MTGA path of a file that's an aiff, for example !
		const MEChunkFileInfo* info = audioChunkFileManager->GetChunkFileInfo(index);
		ZzRtAssert(info->bMTGA);
		ZzRtAssert(info->fOwnedByMontage);
	}
	// otherwise we don't really care
	UInt32 len1 = strlen(doc->GetProjectFolderPath());
	UInt32 len2 = /*strlen("/mtgaudiochunk00000000.mtga");*/ 27;
	ZzRtAssert(bufferSize >= len1 + len2 +1); // it needs to be at least len1+len2+1			
	sprintf(path,"%s/mtgaudiochunk%p.mtga",doc->GetProjectFolderPath(),*((void**)(&(index.data))));
}

void MEAudioChunkFileManager::GetMTGAPath(char* path, UInt32 bufferSize, MEAudioChunkFileIndex index) const
{
	GetMTGAPath(fDoc, path, bufferSize, index);
}

void MEAudioChunkFileManager::GetPath(char* path, UInt32 bufferSize, MEAudioChunkFileIndex index) const
{
	MEAudioChunkFileManager::GetPath(fDoc, path, bufferSize, index);
}

void MEAudioChunkFileManager::GetPath(char* path, UInt32 bufferSize, const MEAudioChunk& chunk) const
{
	ZzAssert(chunk.doc == fDoc);
	MEAudioChunkFileManager::GetPath(fDoc, path, bufferSize, chunk.GetFileIndex());
}

MEAudioChunkFileManager::MEAudioChunkFileManager(TArmeDoc* doc)
{
	ZzRtAssert(doc);
	fDoc = doc;
}

MEAudioChunkFileIndex MEAudioChunkFileManager::GetNewIndex()
{
	MEChunkFileInfo* info = new MEChunkFileInfo();
	MEAudioChunkFileIndex ret;
	ret.data = fAudioChunkFileInfos.CountItems();
	fAudioChunkFileInfos.AddItem(info);
	return ret;
}

void MEAudioChunkFileManager::DeleteUnusedFiles()
{
	char 					audioFileFullPath[1024];
	MEAudioChunkFileIndex 	curIndex;
	SInt32					i = 0;
	
	while (i < fAudioChunkFileInfos.CountItems())
	{
		if(fAudioChunkFileInfos[i]->fRefCount == 0 && fAudioChunkFileInfos[i]->fOwnedByMontage)
		{
			curIndex.data = i;
			GetPath(audioFileFullPath,1024,curIndex);
			remove(audioFileFullPath);
			// $$$TODO remove also the chunk preview file (that doesn't exist yet)
		}
		i++;
	}
	
	// look for all the audio chunk files that have been generated in a previous run but that are not referenced at all because the document has not been saved.
	// translation for benja : the files that have an index > maxIndex are bogus. Get rid of them
	curIndex.data = i;
	GetMTGAPath(audioFileFullPath,1024,curIndex);

	FILE* fp = NULL; // already declared above
	
	while (fp = fopen(audioFileFullPath, "rb"))
	{
		fclose(fp);
		remove(audioFileFullPath);
		i++;
		curIndex.data = i;
		GetMTGAPath(audioFileFullPath,1024,curIndex);
	}
}

void MEAudioChunkFileManager::ReleaseFile(const MEAudioChunk& chunk)
{
	ReleaseFile(chunk.GetFileIndex());
}

void MEAudioChunkFileManager::RetainFile(const MEAudioChunk& chunk)
{
	RetainFile(chunk.GetFileIndex());
}

void MEAudioChunkFileManager::ReleaseFile(MEAudioChunkFileIndex index)
{
	ZzAssert(index.data < fAudioChunkFileInfos.CountItems());
	if(index.data >= 0) // we don't care about silent chunks (with index -1) since they're not real files
	{
		MEChunkFileInfo* fileInfo = fAudioChunkFileInfos[index.data];
		ZzAssert(fileInfo->fRefCount >= 1); // big mistake lurking somewhere !
		fileInfo->fRefCount = fileInfo->fRefCount - 1;
		if(fAudioChunkFileInfos[index.data]->fRefCount == 0)
		{
			// we could print some debug infos in here
		}
	}
	else
	{
		// it is was a silent chunk
	}
}

void MEAudioChunkFileManager::RetainFile(MEAudioChunkFileIndex index)
{
	ZzAssert(index.data < fAudioChunkFileInfos.CountItems());
	if(index.data >= 0) // we don't care about silent chunks with index -1 since they're not real files
	{
		MEChunkFileInfo* fileInfo = fAudioChunkFileInfos[index.data];
		fileInfo->fRefCount = fileInfo->fRefCount + 1;
	}
	else
	{
		// it is was a silent chunk
	}
}

MEAudioChunk MEAudioChunkFileManager::AllocateNewZeroRefMTGAChunk()
{
	MEAudioChunkFileIndex newIndex(fAudioChunkFileInfos.CountItems());
	fAudioChunkFileInfos.AddItem(new MEChunkFileInfo); // default is mtga; 0 refs
	
	MEAudioChunk chunk;
	chunk.startInFile = 0;
	chunk.duration = 0;
	chunk.doc = fDoc;
	chunk.SetFileIndex(newIndex);
	return chunk;
}

MEAudioChunkFileIndex MEAudioChunkFileManager::AllocateNewZeroRefMTGAChunkIndex()
{
	MEAudioChunkFileIndex newIndex(fAudioChunkFileInfos.CountItems());
	fAudioChunkFileInfos.AddItem(new MEChunkFileInfo); // default is mtga; 0 refs
	return newIndex;
}

void MEAudioChunkFileManager::SetChunkFileExternalSFP(MEAudioChunkFileIndex index, const char* path)
{
	ZzAssert(fAudioChunkFileInfos[index.data]->fRefCount == 0); // can only do this before someones owns the file
	ZzAssert(MESFPAudioFile::IsFileSuitableForRealTime(path).type == kArmeOK); // triple check is good
	fAudioChunkFileInfos[index.data]->bMTGA = false; // it's an SFP file (AIFF, WAV, SD2, ...)
	fAudioChunkFileInfos[index.data]->fPath = new char[strlen(path)+1];
	fAudioChunkFileInfos[index.data]->bFileIsAlias = true; //from version 1.4 the path is a path to an alias file
	strcpy(fAudioChunkFileInfos[index.data]->fPath,path);
	// VERRRRRY IMPORTANT 	
	// VERRRRRY IMPORTANT 	
	// VERRRRRY IMPORTANT 	
	// VERRRRRY IMPORTANT !!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	fAudioChunkFileInfos[index.data]->fOwnedByMontage = false; 
};

MEAudioChunkFileManager::~MEAudioChunkFileManager()
{
	for(SInt32 i = 0; i < fAudioChunkFileInfos.CountItems(); i++)
	{
		MEChunkFileInfo* info = fAudioChunkFileInfos[i];
		delete info;
		fAudioChunkFileInfos[i] = NULL;
	}
	fAudioChunkFileInfos.RemoveAll();
}

const MEChunkFileInfo* MEAudioChunkFileManager::GetChunkFileInfo(MEAudioChunkFileIndex index) const
{
	return fAudioChunkFileInfos[index.data];
}

void MEAudioChunkFileManager::SetOldStyleRefsUpToExc(UInt32 audioChunkNextIndex)
{
	ZzRtAssert(fAudioChunkFileInfos.CountItems() == 0);
	for(UInt32 i = 0; i < audioChunkNextIndex; i++)
	{
		AllocateNewZeroRefMTGAChunkIndex();
	}
}
	
void MEAudioChunkFileManager::ReadFromArchive(ArmeArchiveHandler* unarchiver)
{
	//read the total nb of infos
	SInt32 nbOfChunkFileInfos = unarchiver->ReadSInt32();
	for(SInt32 i = 0; i < nbOfChunkFileInfos; i++)
	{
		// then write each of these. the array OWNS the infos
		MEChunkFileInfo* info = new MEChunkFileInfo;
		info->ReadFromArchive(unarchiver);
		
		// let's check if the file exists
		if(!info->bMTGA)
		{
            const char* filePath = info->fPath;
            if (info->bFileIsAlias)
                filePath = TArmeMediaEngine::ApplicationServices()->ResolveAlias(info->fPath);
        
			FILE* testFileHandle = fopen(filePath,"rb");
			if(testFileHandle)
			{
				fclose(testFileHandle);
			}
			else
			{
				bool isOK = false;
				fDoc->SetAudioChunkFileManagerDidDirtifyDoc();
                
                //first try to locate the file in the project package (this might be necessary if the project has moved)
                ZzString fileName = ZzGetFileNameFromPath(info->fPath);
                const char* projectDirectory = unarchiver->GetProjectDirectory();
                char temp[1024];
                
                ZzAppendFileNameToPath(temp, 1024, projectDirectory, fileName.getUTF8String());
                const char* temp2 = TArmeMediaEngine::ApplicationServices()->ResolveAlias(temp);
				if (temp2 != NULL)
                {
                    ArmeErr err = MESFPAudioFile::IsFileSuitableForRealTime(temp2);
                    if(err.type == kArmeOK)
                    {
                        delete [] info->fPath;
                        info->fPath = new char[strlen(temp)+1];
                        info->bFileIsAlias = true;
                        strcpy(info->fPath,temp);
                        isOK = true;
                    }
                }

                //otherwise, ask the user to find it
				while(!isOK)
				{
					ZzString warningMessage("Sorry, but the file\n");
					warningMessage += ZzString(info->fPath);
					warningMessage += ZzString("\ncannot be opened. Please locate a suitable file.");
					ZzWarningMessageBox(warningMessage.getUTF8String());
					
					ZzString fullMissingPath(info->fPath);
					ZzString missingFileName = ZzLastPathComponent(fullMissingPath);
															
					ZzString newPath;
					bool isCancelled = false;
					try
					{
						newPath = ZzGUILocateFile(missingFileName.getUTF8String()); // this can throw if user cancels
					}
					catch(ZzExceptionCancel* exc)
					{
						ZzString warningMessage("Sorry, but this document can't be opened... Please try to re-open it and locate missing files.");
						ZzWarningMessageBox(warningMessage.getUTF8String());
						//$$$HACK2003-03-17
						exit(0);
						throw;
					}
					catch(...)
					{
						throw;
					}
					if(!isCancelled)
					{
						ArmeErr err = MESFPAudioFile::IsFileSuitableForRealTime(newPath);
						if(err.type == kArmeOK)
						{
							const char* utf8NewPath = newPath.getUTF8String();
							delete [] info->fPath;
							info->fPath = new char[strlen(utf8NewPath)+1];
                            if (TArmeMediaEngine::ApplicationServices()->ResolveAlias(temp) != NULL)
                                info->bFileIsAlias = true;
                            else
                                info->bFileIsAlias = false;
							strcpy(info->fPath,utf8NewPath);
							isOK = true;
						}
						else
						{
							ZzWarningMessageBox("Sorry, this file cannot be opened or is not a supported uncompressed audio file. Please choose another one.");
						}
					}
				}
			}
		}
		fAudioChunkFileInfos.AddItem(info);
	}
}

void MEAudioChunkFileManager::WriteToArchive(ArmeArchiveHandler* archiver)
{
	// simply write the nb of infos
	archiver->WriteSInt32(fAudioChunkFileInfos.CountItems());
	for(SInt32 i = 0; i < fAudioChunkFileInfos.CountItems(); i++)
	{
		// then write each info
		MEChunkFileInfo* info = fAudioChunkFileInfos[i];
		info->WriteToArchive(archiver);
	}
}

void MEChunkFileInfo::ReadFromArchive(ArmeArchiveHandler* unarchiver)
{
	ZzAssert(fPath == NULL);
	ZzAssert(fRefCount == 0);
	bMTGA = unarchiver->ReadSInt32();
	if(unarchiver->GetFileVer() < 0x00010035)
	{
		bFileIsAlias = false;
	}
	else
	{
		bFileIsAlias = unarchiver->ReadSInt32();
	}
	SInt32 pathNbCharsWithNULL = unarchiver->ReadSInt32();
	if(pathNbCharsWithNULL == 0)
	{
		fPath = NULL;
	}
	else
	{
		ZzAssert(pathNbCharsWithNULL >= 2); // 1 char at least !!!!!
		fPath = new char[pathNbCharsWithNULL];
		unarchiver->ReadRawData(fPath,pathNbCharsWithNULL);
	}
	fRefCount = 0;
	fOwnedByMontage = unarchiver->ReadSInt32();
}

void MEChunkFileInfo::WriteToArchive(ArmeArchiveHandler* archiver) const
{
	archiver->WriteSInt32(bMTGA);
	archiver->WriteSInt32(bFileIsAlias);	//introduced in 0x00010035 (version 1.4)
	if(fPath == NULL)
	{
		archiver->WriteSInt32(0); // a zero length for path means NULL
	}
	else
	{
		SInt32 pathNbCharsWithNULL = strlen(fPath)+1;
		ZzAssert(pathNbCharsWithNULL >= 2);
		archiver->WriteSInt32(pathNbCharsWithNULL);
		archiver->WriteRawData(fPath,pathNbCharsWithNULL);
	}
	archiver->WriteSInt32(fOwnedByMontage);
}

bool MEAudioChunkFileManager::IsChunkFileMTGA(const MEAudioChunk& chunk) const
{
	return IsChunkFileMTGA(chunk.GetFileIndex());
}
	
bool MEAudioChunkFileManager::IsChunkFileMTGA(MEAudioChunkFileIndex index) const
{
	ZzRtAssert(index.data < fAudioChunkFileInfos.CountItems());
	return fAudioChunkFileInfos[index.data]->bMTGA;
}

SInt32 MEAudioChunkFileManager::GetNbChunkFileInfos() const
{
	return fAudioChunkFileInfos.CountItems();
}

