/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#ifndef ZzException_h
#include "ZzException.h"
#endif //ZzException_h

#ifndef ME_AudioExportFunctions_h
#include "ME_AudioExportFunctions.h"
#endif //ME_AudioExportFunctions_h

#ifndef MIProgressBarDlgManager_h
#include "MIProgressBarDlgManager.h"
#endif //MIProgressBarDlgManager_h

#ifndef ZzRABI_h
#include "ZzRABI.h"
#endif //ZzRABI_h

#ifndef MEAudioObjRenderer_h
#include "MEAudioObjRenderer.h"
#endif //MEAudioObjRenderer_h

#ifndef MEMTGAAudioFile_h
#include "MEMTGAAudioFile.h"
#endif //MEMTGAAudioFile_h

#ifndef Zz_PathHelpers_h
#include "Zz_PathHelpers.h"
#endif //Zz_PathHelpers_h

#ifndef ZzHelpers_h
#include "ZzHelpers.h"
#endif //ZzHelpers_h

#include <AudioToolbox/AudioFile.h>
#include <AudioToolbox/AudioConverter.h>

// +------------------------------------------+
// |                                          |                          
// |        MEExportToExternalFile            |                          
// |                                          |                          
// +------------------------------------------+

UInt32 MIExportToExternalFile_GetAudioFileAPITypeConstant(UInt32 sampleType, UInt32 fileType)
{
	switch(fileType)
	{
	case kMEExportAIFF :
		switch(sampleType)
		{
		case kMEExport16Bits :
			return kAudioFileAIFFType;
			break;
		case kMEExport8Bits :
		case kMEExport24Bits :
		case kMEExport32BitsFloat :
			return kAudioFileAIFCType;
			break;
		default :
			ZZ_THROW_1ARG(ZzGenericException,"wrong sampleType in MIExportToExternalFile_GetAudioFileAPITypeConstant()");
			break;
		}
		break;
	case kMEExportWAVE :
		return kAudioFileWAVEType;
		break;
	default :
		ZZ_THROW_1ARG(ZzGenericException,"wrong fileType in MIExportToExternalFile_GetAudioFileAPITypeConstant()");
		break;
	};
}

const char* MIExportToExternalFile_GetExtensionForTypeAndFormat(UInt32 sampleType, UInt32 fileType)
{
	// returns the extension WITHOUT the leading dot
	UInt32 audioFileAPIFileTypeConstant = MIExportToExternalFile_GetAudioFileAPITypeConstant(sampleType,fileType);
	switch(audioFileAPIFileTypeConstant)
	{
	case kAudioFileAIFCType :
		return "aifc";
		break;
	case kAudioFileAIFFType :
		return "aiff";
		break;
	case kAudioFileWAVEType :
		return "wav";
		break;
	default :
		ZZ_THROW_1ARG(ZzGenericException,"wrong audioFileAPIFileTypeConstant in MIExportToExternalFile_GetExtensionForTypeAndFormat()");
		break;
	}
}

struct MIExportToExternalFile_ACInputDataProcCookie
{
	MEAudioObjRenderer**	pRenderer;
	UInt32					srcBytesPerFrame;
	float* 					tmpBuffer;
};

OSStatus MIExportToExternalFile_ACInputDataProc(
	AudioConverterRef	ioAudioConverter,
	UInt32*				ioDataSize, // the number of bytes the SRC would like to receive
	void**				outData,
	void*				inUserData)
{
	// we need : 
	// - the nb of bytes per frame in the source format
	// - the tmp buffer where we can write into
	// - the converter
	MIExportToExternalFile_ACInputDataProcCookie* cookieStruct = 
		(MIExportToExternalFile_ACInputDataProcCookie*)inUserData;
	
	MEAudioObjRenderer* renderer = *(cookieStruct->pRenderer);
	UInt32 srcBytesPerFrame = cookieStruct->srcBytesPerFrame;
	float* tmpBuffer = cookieStruct->tmpBuffer;
	
	UInt32 framesToRead = *ioDataSize / srcBytesPerFrame;
	if(framesToRead > 65536)
		framesToRead = 65536; // $$$TO MAKE AUDIO CONVERTER HAPPY. THE AC WILL SIMPLY CALL US SEVERAL TIMES
	// read the float frames in the audio obj renderere
	renderer->ReadNextFloatFrames(tmpBuffer,framesToRead); // throws
	*outData = (void*)tmpBuffer;
	*ioDataSize = framesToRead * srcBytesPerFrame;
	return noErr;
}

void MIExportToExternalFile_FillTargetStreamDesc(
	AudioStreamBasicDescription*	tgtDesc, 	
	UInt32							targetAudioFrameRate,
	UInt32							fileType,
	UInt32							sampleType, 
	UInt32							nbChannels)
{
	memset(tgtDesc,0,sizeof(AudioStreamBasicDescription));
	tgtDesc->mSampleRate = Float64(targetAudioFrameRate);
	tgtDesc->mFormatID = kAudioFormatLinearPCM;
	switch(sampleType)
	{
	case kMEExport8Bits :
		tgtDesc->mFormatFlags = kAudioFormatFlagIsPacked;
		if(fileType == kMEExportAIFF) // yeah this is weird but that's what the audio file api wants or so it seems
				tgtDesc->mFormatFlags |= kAudioFormatFlagIsSignedInteger | kAudioFormatFlagIsBigEndian;
		tgtDesc->mBytesPerPacket = nbChannels * 1;
		tgtDesc->mFramesPerPacket = 1;
		tgtDesc->mBytesPerFrame = nbChannels * 1;
		tgtDesc->mChannelsPerFrame = nbChannels;
		tgtDesc->mBitsPerChannel = 8*1;
		tgtDesc->mReserved = 0;
		break;
	case kMEExport16Bits :
		tgtDesc->mFormatFlags = kAudioFormatFlagIsSignedInteger | kAudioFormatFlagIsPacked;
		if(fileType == kMEExportAIFF)
				tgtDesc->mFormatFlags |= kAudioFormatFlagIsBigEndian;
		tgtDesc->mBytesPerPacket = nbChannels * 2;
		tgtDesc->mFramesPerPacket = 1;
		tgtDesc->mBytesPerFrame = nbChannels * 2;
		tgtDesc->mChannelsPerFrame = nbChannels;
		tgtDesc->mBitsPerChannel = 8*2;
		tgtDesc->mReserved = 0;
		break;
	case kMEExport24Bits :
		tgtDesc->mFormatFlags = kAudioFormatFlagIsSignedInteger | kAudioFormatFlagIsPacked;
		if(fileType == kMEExportAIFF)
				tgtDesc->mFormatFlags |= kAudioFormatFlagIsBigEndian;
		tgtDesc->mBytesPerPacket = nbChannels * 3;
		tgtDesc->mFramesPerPacket = 1;
		tgtDesc->mBytesPerFrame = nbChannels * 3;
		tgtDesc->mChannelsPerFrame = nbChannels;
		tgtDesc->mBitsPerChannel = 8*3;
		tgtDesc->mReserved = 0;
		break;
	case kMEExport32BitsFloat : 
		tgtDesc->mFormatFlags = kAudioFormatFlagIsFloat | kAudioFormatFlagIsPacked;
		if(fileType == kMEExportAIFF)
				tgtDesc->mFormatFlags |= kAudioFormatFlagIsBigEndian;
		tgtDesc->mBytesPerPacket = nbChannels * 4;
		tgtDesc->mFramesPerPacket = 1;
		tgtDesc->mBytesPerFrame = nbChannels * 4;
		tgtDesc->mChannelsPerFrame = nbChannels;
		tgtDesc->mBitsPerChannel = 8*4;
		tgtDesc->mReserved = 0;
		break;
	default :
		ZZ_THROW_1ARG(ZzException,"Wrong sampleType in MIExportToExternalFile_CreateAudioConverter");
		break;
	}
}

AudioConverterRef MIExportToExternalFile_CreateAudioConverter(
	UInt32	targetAudioFrameRate, 
	UInt32	fileType, 
	UInt32	sampleType, 
	UInt32	nbChannels, 
	UInt32	srcAudioFrameRate)
{
	AudioStreamBasicDescription	tgtDesc;
	MIExportToExternalFile_FillTargetStreamDesc(&tgtDesc,targetAudioFrameRate,fileType,sampleType,nbChannels);

	// source is assumed to be float32,nbChannels,srcAudioFrameRate
	AudioStreamBasicDescription	srcDesc;
	memset(&srcDesc,0,sizeof(AudioStreamBasicDescription));
	
	srcDesc.mSampleRate = Float64(srcAudioFrameRate);
	srcDesc.mFormatID = kAudioFormatLinearPCM;
	
	srcDesc.mFormatFlags = kAudioFormatFlagIsFloat | kAudioFormatFlagIsBigEndian | kAudioFormatFlagIsPacked;
	srcDesc.mBytesPerPacket = nbChannels * sizeof(float);
	srcDesc.mFramesPerPacket = 1;
	srcDesc.mBytesPerFrame = nbChannels * sizeof(float);
	srcDesc.mChannelsPerFrame = nbChannels;
	srcDesc.mBitsPerChannel = 8*sizeof(float);
	srcDesc.mReserved = 0;
	
	AudioConverterRef audioConverter;
	OSStatus st = AudioConverterNew(&srcDesc,&tgtDesc,&audioConverter);
	ThrowOnOSStatus(st);
	
	if(srcDesc.mSampleRate != tgtDesc.mSampleRate) //otherwise there's an error.....
	{
		UInt32 propertyValue; 
		UInt32 propertySize;
		Boolean isPropWritable;
	
		st = AudioConverterGetPropertyInfo(audioConverter,kAudioConverterSampleRateConverterQuality,&propertySize,&isPropWritable);
		ThrowOnOSStatus(st);
		propertyValue = kAudioConverterQuality_Max;
		propertySize = sizeof(UInt32);
		st = AudioConverterSetProperty(audioConverter,kAudioConverterSampleRateConverterQuality,propertySize,(void*)(&propertyValue));
		ThrowOnOSStatus(st);
	}
	return audioConverter;
}

#ifndef ME_AudioExportFunctions_h
#include "ME_AudioExportFunctions.h"
#endif //ME_AudioExportFunctions_h

void MEExportToExternalFile(
	ZzArchPtrArray*				theObjects,
	ZzString					basePathName,
	bool						bSliceSel,
	SInt64						startGlobalTime,
	SInt64						endxGlobalTime,
	bool						bAsMultipleFiles,
	bool						bIncludeAllFX,
	bool						bUsesOriginalNames,
	UInt32						nbChannels, 		// 0 means use object routing
	UInt32						audioFrameRate,		// 22050, 44100, 48000, 88200, 96000 or 192000
	UInt32						fileType, 			// kMEExportAIFF, kMEExportWAVE
	UInt32						sampleType, 		// kMEExportFloatBE, kMEExportFloatLE, kMEExportShortLE, kMEExportShortBE 
	MIProgressBarDlgManager* 	dlgManager)
{
	int 				nbOfFiles;
	ZzArchPtrArray**	theSetsOfObjects;
	SInt64*				startTimes;
	SInt64*				endxTimes;
	
	if(theObjects->CountItems() < 1)
	{
		ZZ_THROW_1ARG(ZzGenericException,"The exported selection cannot be empty");
	}
	
	if(bAsMultipleFiles)
		nbOfFiles = theObjects->CountItems();
	else
		nbOfFiles = 1;
	
	// +----------------------------------+
	// |  allocate the arrays to render   |
	// +----------------------------------+
	try
	{ 
		theSetsOfObjects = new ZzArchPtrArray*[nbOfFiles]; 
	}
	catch(...)
	{
		throw;
	}
	
	for(SInt32 i = 0; i < nbOfFiles; i++)
	{
		theSetsOfObjects[i] = new ZzArchPtrArray;
	}
	// no RABI for this one.... we'll delete it on our own
		
	// +----------------------------------+
	// |  allocate the time ranges        |
	// +----------------------------------+
	startTimes = new SInt64[nbOfFiles];
	ZzRABI_ArrayDeleter<SInt64> rabiDeleteStartTimes(&startTimes);
	endxTimes = new SInt64[nbOfFiles];
	ZzRABI_ArrayDeleter<SInt64> rabiDeleteEndTimes(&endxTimes);
	
	// +----------------------------------------------+
	// |  transfer the objects in the discrete arrays |
	// +----------------------------------------------+
	if(bAsMultipleFiles)
	{
		for(SInt32 i = 0; i < nbOfFiles; i++)
		{
			//ZzArchPtrArray* tmpArray = new ZzArchPtrArray;
			//theSetsOfObjects[i]->AddItem(tmpArray);
			
			// now let's either add the object itself OR its topmost parent,
			// according to bIncludeAllFX
			TArmeWSObj* wsobj = dynamic_cast<TArmeWSObj*>(theObjects->ItemAt(i));

			if(!bIncludeAllFX)
			{
				theSetsOfObjects[i]->AddItem(wsobj);
				
				// then only use a slice if it has been defined as such
				if(bSliceSel)
				{
					startTimes[i] = startGlobalTime;
					endxTimes[i] = endxGlobalTime;
				}
				else
				{
					startTimes[i] = GET_VAL(wsobj->GetAbsoluteStartTime());
					endxTimes[i] = GET_VAL(wsobj->GetAbsoluteEndxTime());
				}
			}
			else
			{
				TArmeWSObj* topmostParent = wsobj->GetTopmostParent();
				theSetsOfObjects[i]->AddItem(topmostParent);
				bSliceSel = true; // IMPORTANT : if we use the topmost parent, then it's only a slice of that parent !
				startTimes[i] = GET_VAL(wsobj->GetAbsoluteStartTime());
				endxTimes[i] = GET_VAL(wsobj->GetAbsoluteEndxTime());
			}
		}
	}
	else
	{
		// now let's either add the object itself OR its topmost parent,
		// according to bIncludeAllFX
		
		// NOTE : if the first object is a children of  the TLC,
		// then it means that bIncludeAllFX needs to be set to FALSE.
		// because otherwise the topmost parents of the objects are
		// NOT the same since they are top level objects.....
		TArmeWSObj* wsobj = dynamic_cast<TArmeWSObj*>(theObjects->ItemAt(0));
		if(wsobj->GetParent() == wsobj->GetDoc()->GetTopContainer())
			bIncludeAllFX = false;

		if(!bIncludeAllFX)
		{
			for(SInt32 i = 0; i < theObjects->CountItems(); i++)
			{
				TArmeWSObj* wsobj = dynamic_cast<TArmeWSObj*>(theObjects->ItemAt(i));
				theSetsOfObjects[0]->AddItem(wsobj);
			}
			if(bSliceSel)
			{
				startTimes[0] = startGlobalTime;
				endxTimes[0] = endxGlobalTime;
			}
			else
			{
				// otherwise the time span is the extent of all the files....
				MEAudioObjRenderer::GetObjectSetTimeSpan(theObjects,&(startTimes[0]),&(endxTimes[0]));
			}
		}
		else
		{
			// the situation is as follows :
			// we have a set of objects, and we want to render them
			// into a common file, but we want to include the fx, too
			// a required condition for this is that the objects need
			// to be siblings. that is, they must have a common direct 
			// parent which is NOT the tlc (this has been handled before)
			// these constraints are enforced by the calling function
			// we double check the "siblings" constraint here with a 
			// ZzRtAssert for debug.
			
			TArmeWSObj* firstWSObj = dynamic_cast<TArmeWSObj*>(theObjects->ItemAt(0));
			TArmeWSObj* firstTopmostParent = firstWSObj->GetTopmostParent();
			theSetsOfObjects[0]->AddItem(firstTopmostParent);
			
			for(SInt32 i = 1; i < theObjects->CountItems(); i++)
			{
				TArmeWSObj* wsobj = dynamic_cast<TArmeWSObj*>(theObjects->ItemAt(i));
				TArmeWSObj* topmostParent = wsobj->GetTopmostParent();
				ZzRtAssert(topmostParent == firstTopmostParent);
				//theSetsOfObjects[0]->AddItem(topmostParent);
			}
			bSliceSel = true;
			MEAudioObjRenderer::GetObjectSetTimeSpan(theObjects,&(startTimes[0]),&(endxTimes[0]));
		}
	}
	
	MIProgressBarDlgManager* dlgManagerCreatedHere = NULL;

	if(!dlgManager)
	{
		dlgManagerCreatedHere = new MIProgressBarDlgManager("Preparing audio export");
		dlgManager = dlgManagerCreatedHere;
	}
	ZzRABI_ObjectDeleter<MIProgressBarDlgManager> rabiDeleteDlgManagerCreatedHere(&dlgManagerCreatedHere);

	dlgManager->SetNbSubTasks(nbOfFiles);
	
	const UInt32 kBufferNbFrames = 65536;
	
	// let's allocate the tmp buffer
	float* tmpSrcBuffer = new float[kBufferNbFrames*2]; // we hardcode stereo to make sure
	
	// retrieve the doc ptr
	ArmeArchivable* archObj;
	TArmeWSObj* wsObj;
	archObj = theObjects->ItemAt(0);
	wsObj = dynamic_cast<TArmeWSObj*>(archObj);
	TArmeDoc* doc = wsObj->GetDoc();
				
	// let's compute the resampling ratio
	double factor = (double)audioFrameRate/(double)(doc->GetAudioFrameRate());
	
	// $$$ src are known to sometimes ask more than they should....
	UInt32 tgtBufferNbSamples = ((UInt32)((factor*1.1)*kBufferNbFrames) + 512)*2;// we hardcode stereo to make sure (nb of channels is NOT known at this time)

	float* tmpTgtBuffer;
	try
	{
		tmpTgtBuffer = new float[tgtBufferNbSamples];
	}
	catch(...)
	{
		throw;
	}
		
	// a RABI that kills the array at the end of this scope
	ZzRABI_ArrayDeleter<float> rabiTmpBufferDeleter(&tmpTgtBuffer);
	
	ZzString 	fileFolderPath;
	ZzString 	fileName;
	OSStatus	st;
	char		actualFileName[1024];
	bool 		bUserWantsToAlwaysOverwrite = false;

	fileFolderPath = ZzDeleteLastPathComponent(basePathName);
	fileName = ZzLastPathComponent(basePathName); // shouldn't have an extension
	MEAudioObjRenderer* renderer;
	
	// this struct is the cookie for the AC fill proc
	MIExportToExternalFile_ACInputDataProcCookie cookieStruct;
	cookieStruct.pRenderer = &renderer;
	cookieStruct.srcBytesPerFrame = 0;
	cookieStruct.tmpBuffer = tmpSrcBuffer;
	
	SInt32 actualNbChannels;
	
	try
	{
		for(SInt32 i = 0; i < nbOfFiles; i++)
		{
			dlgManager->EnterSubTask();
			// let's compute the number of channels in the current file
			actualNbChannels = nbChannels;
			
			// if nbChannels == 0 that means we use stereo unless we are able
			// to use mono (possible only if all the objects are routed to mono vaos)
			if(actualNbChannels == 0)
			{
				actualNbChannels = MEAudioObjRenderer::GetMaxNbChannels(theSetsOfObjects[i]);
			}

			// create the audio converter from the document format to the 
			// export format
			AudioConverterRef audioConverter = MIExportToExternalFile_CreateAudioConverter(audioFrameRate, fileType, sampleType, actualNbChannels, doc->GetAudioFrameRate());

			cookieStruct.srcBytesPerFrame = actualNbChannels*sizeof(float);
		
			st = AudioConverterReset(audioConverter);
			ThrowOnOSStatus(st);
			
			// create the file
			const char* extension = MIExportToExternalFile_GetExtensionForTypeAndFormat(sampleType, fileType);
			
			if(!bUsesOriginalNames)
			{
				char szFileName[1024]; // the path, without an extension
				strcpy(szFileName,fileName.getUTF8String());
				ZzRemoveBadFileNameCharacters(szFileName); // removes * and / and turns them into other chars (see the ZzRemoveBadFileNameCharacters declaration in Zz_PathHelpers.h)
				ZzRemoveFileExtensionFromPath(szFileName,1024);
				if(nbOfFiles > 1)
					sprintf(actualFileName,"%s_%2d.%s",szFileName,(int)(i+1),extension);
				else
					sprintf(actualFileName,"%s.%s",szFileName,extension);
			}
			else
			{
				if(theSetsOfObjects[i]->CountItems() > 1)
				{
					ZzWarningMessageBox("The exported file will use the original name of the first object");
				}
				char objName[1024];
				ArmeArchivable* archWSObj = theSetsOfObjects[i]->ItemAt(0);
				TArmeWSObj* wsObj = dynamic_cast<TArmeWSObj*>(archWSObj);
				sprintf(objName,"%s",wsObj->GetName());
				ZzRemoveBadFileNameCharacters(objName); // removes * and / and turns them into _ and -
				ZzRemoveFileExtensionFromPath(objName,1024); // removes the trailing extension
				sprintf(actualFileName,"%s.%s",objName,extension);
			}
			
			ZzString filePath = fileFolderPath + ZzString("/") + ZzString(actualFileName);
				
			ZzString zzActualFileName(actualFileName);
						
			// convert file folder to FSRef
			FSRef fsFolder;
			st = ZzPathToFSRef(fileFolderPath.getCFString(),&fsFolder);
			ThrowOnOSStatus(st);
			
			// +------------------------------------------------+
			// | compute the file basic stream description      |
			// +------------------------------------------------+
			AudioStreamBasicDescription	tgtDesc;
			MIExportToExternalFile_FillTargetStreamDesc(&tgtDesc,audioFrameRate,fileType,sampleType,actualNbChannels);
					
			SInt32 bytesPerFrameInFile = tgtDesc.mBytesPerFrame;
					
			FSRef fsRefAudioFile;
			AudioFileID audioFileID;

			bool bKeepOnTrying = true;
			bool bSkipThisFile = false;
			
			while(bKeepOnTrying)
			{
				st = AudioFileCreate(
					&fsFolder, zzActualFileName.getCFString(),
					MIExportToExternalFile_GetAudioFileAPITypeConstant(sampleType, fileType),
					&tgtDesc, 0, &fsRefAudioFile, &audioFileID);
				if(st == noErr)
				{
					bKeepOnTrying = false;
				}
				else if(st == dupFNErr)
				{
					if(!bUserWantsToAlwaysOverwrite)
					{
						ZzString message("Do you want to overwrite ");
						message += ZzString(actualFileName);
						message += ZzString(" ?");
						ZzMessageBoxReply reply = ZzMessageBox("Warning",message,"No","Yes to All","Yes");
						if(reply == ZZ_MESSAGEBOXREPLY_DEFAULT_BUTTON)
						{
							bKeepOnTrying = false;
							bSkipThisFile = true;
						}
						else if(reply == ZZ_MESSAGEBOXREPLY_ALT_BUTTON)
						{
							bUserWantsToAlwaysOverwrite = true;
							remove(filePath.getUTF8String());
						}
						else if(reply == ZZ_MESSAGEBOXREPLY_OTHER_BUTTON)
						{
							remove(filePath.getUTF8String());
						}
					}
					else
					{
						remove(filePath.getUTF8String());
					}
				}
				else
				{
					bKeepOnTrying = false;
					if(st == 'fmt?')
						ZZ_THROW_1ARG(ZzGenericException,"Format error ('fmt?')");
					else
						ThrowOnOSStatus(st);
				}
			}
			
			
			if(!bSkipThisFile)
			{
				////////// reset the audio converter for the next file /////////
				///////// convert total number of frames ////// (multiply//////
				////// install callback with correct cookie ..........//////////
				try
				{
					// kBufferNbFrames means we'll never request more than kBufferNbFrames frames at once
					// "true" means we WANT the vaorao gain/pan to be applied to the results
					// bStereo means we want the nb of channels specified in the function params
					// ZzRABI_ObjectDeleter is a RABI that kills the object at the end of this scope
					try
					{
						renderer = new MEAudioObjRenderer(theSetsOfObjects[i],bSliceSel,startTimes[i],endxTimes[i],kBufferNbFrames,true,actualNbChannels);
					}
					catch(...)
					{
						throw;
					}
					ZzRABI_ObjectDeleter<MEAudioObjRenderer> rabiRendererDeleter(&renderer);
				
					// now start the pull loop
					SInt64 curTime = 0;
					SInt64 duration = endxTimes[i] - startTimes[i];
					
					SInt64 fileDuration = (SInt64)(factor*duration);
					
					while(curTime < fileDuration)
					{
						UInt32 framesToRead = fileDuration - curTime;
						if(framesToRead > kBufferNbFrames)
							framesToRead = kBufferNbFrames;
		
						// so that it starts on 0.0xxx and ends on 1.0
						dlgManager->Update((double)(curTime + framesToRead)/ (double)fileDuration);
						
						UInt32 bytesFromConverter = framesToRead*bytesPerFrameInFile;
						
						st = AudioConverterFillBuffer(
								audioConverter, 
								MIExportToExternalFile_ACInputDataProc, (void*)(&cookieStruct), 
								&bytesFromConverter, (void*)tmpTgtBuffer );
						ThrowOnOSStatus(st);
						
						UInt32 bytesToWriteInFile = bytesFromConverter;
						st = AudioFileWriteBytes(audioFileID,true,curTime*bytesPerFrameInFile,&bytesToWriteInFile,(void*)tmpTgtBuffer);
						ThrowOnOSStatus(st);
						curTime += framesToRead;
					}
				}
				catch(...)
				{
					st = AudioFileClose(audioFileID);
					remove(filePath.getUTF8String()); // deletes the file we were writing in.
					throw;
				}
				st = AudioFileClose(audioFileID);
			}
			dlgManager->LeaveSubTask();
			st = AudioConverterDispose(audioConverter);
		}
	}
	catch(...)
	{
		for(SInt32 i = 0; i < nbOfFiles; i++)
		{
			delete theSetsOfObjects[i];
			theSetsOfObjects[i] = NULL;
		}
		delete [] theSetsOfObjects;
		throw;
	}
	for(SInt32 i = 0; i < nbOfFiles; i++)
	{
		delete theSetsOfObjects[i];
		theSetsOfObjects[i] = NULL;
	}
	delete [] theSetsOfObjects;
}

// +------------------------------------------+
// |                                          |                          
// |        MEAudioExportInQTTrack            |                          
// |                                          |                          
// +------------------------------------------+

/*
myErr = AddMediaSample(	audioMedia,
	(Handle)&tmpBuffer,
	0,
	framesInThisShot*outputChannels*sizeof(float),
	1,
	(SampleDescriptionHandle)audioSampleDesc,
	framesInThisShot,
	0,
	NULL);
*/

void MEAudioExportInQTTrack(
	ZzArchPtrArray*				theObjects,
	bool						bSliceSel,
	SInt64						startGlobalTime,
	SInt64						endxGlobalTime,
	UInt32						nbChannels, // can only be 1 or 2
//	UInt32						audioFrameRate, // should use project audio frame rate
	Media* 						pAudioMedia, 
	SampleDescriptionHandle		audioSampleDesc,
	MIProgressBarDlgManager*	dlgManager)
{
	MIProgressBarDlgManager* dlgManagerCreatedHere = NULL;

	if(!dlgManager)
	{
		dlgManagerCreatedHere = new MIProgressBarDlgManager("Preparing audio export");
		dlgManager = dlgManagerCreatedHere;
	}
	ZzRABI_ObjectDeleter<MIProgressBarDlgManager> rabiDeleteDlgManagerCreatedHere(&dlgManagerCreatedHere);

//	ArmeErr 			err;
//	ArmeArchivable* 	archObj;
//	TArmeWSObj* 		wsObj;
	
	// compute the time span of the whole set of objects
	SInt64 absObjSetStartTime = -1;
	SInt64 absObjSetEndxTime = -1;
	SInt64 objSetDuration = -1;
	
	// nb of channels will be the same as the main vao routing
	ZzRtAssert(nbChannels == 1 || nbChannels == 2);
	
	MEAudioObjRenderer::GetObjectSetTimeSpan(theObjects,&absObjSetStartTime,&absObjSetEndxTime);
	objSetDuration = absObjSetEndxTime- absObjSetStartTime;
	if(bSliceSel)
		objSetDuration = endxGlobalTime - startGlobalTime;
		
	// create an MEAudioObjRenderer. It needs to know the max nb of frames
	// that will be asked at each ReadNextFloatFrames( ) call...
	const UInt32 kBufferNbFrames = 65536;
	
	// kBufferNbFrames means we'll never request more than kBufferNbFrames frames at once
	// "true" means we WANT the vaorao gain/pan to be applied to the results
	// bStereo means we want the nb of channels specified in the function params
	// ZzRABI_ObjectDeleter is a RABI that kills the object at the end of this scope
	MEAudioObjRenderer* renderer = new MEAudioObjRenderer(theObjects,bSliceSel,startGlobalTime,endxGlobalTime,kBufferNbFrames,true,nbChannels);
	ZzRABI_ObjectDeleter<MEAudioObjRenderer> rabiRendererDeleter(&renderer);
		
	// let's allocate the tmp buffer
	float* tmpBuffer = new float[kBufferNbFrames*nbChannels];
	
	// a RABI that kills the array at the end of this scope
	ZzRABI_ArrayDeleter<float> rabiTmpBufferDeleter(&tmpBuffer);
	
	// now start the pull loop
	SInt64 curTime = 0;
	
	while(curTime < objSetDuration)
	{
		UInt32 framesToRead = objSetDuration - curTime;
		if(framesToRead > kBufferNbFrames)
			framesToRead = kBufferNbFrames;

		// so that it starts on 0.0xxx and ends on 1.0
		dlgManager->Update((double)(curTime + framesToRead)/ (double)(objSetDuration));
		
		renderer->ReadNextFloatFrames(tmpBuffer,framesToRead); // throws
		
		OSStatus st = AddMediaSample( *pAudioMedia, (Handle)&tmpBuffer, 0, framesToRead*nbChannels*sizeof(float), 1, audioSampleDesc, framesToRead, 0, NULL);
		ThrowOnOSStatus(st);
		curTime += framesToRead;
	}
}

// +------------------------------------------+
// |                                          |                          
// |        MEAudioExportInMTGAFile           |                          
// |                                          |                          
// +------------------------------------------+

void MEAudioExportInMTGAFile(
	const char* 				mtgaPath,
	ZzArchPtrArray* 			objSet,
	bool						bSliceSel,
	SInt64						startGlobalTime,
	SInt64						endxGlobalTime,
	MEVAORouting* 				newRouting,
	MIProgressBarDlgManager*	dlgManager)
{
	MIProgressBarDlgManager* dlgManagerCreatedHere = NULL;

	if(!dlgManager)
	{
		dlgManagerCreatedHere = new MIProgressBarDlgManager("Preparing audio export");
		dlgManager = dlgManagerCreatedHere;
	}
	ZzRABI_ObjectDeleter<MIProgressBarDlgManager> rabiDeleteDlgManagerCreatedHere(&dlgManagerCreatedHere);

	ArmeErr 			err;
	ArmeArchivable* 	archObj;
	TArmeWSObj* 		wsObj;
	TArmeDoc*			doc;

	// the number of channels in the file
	// is the same as the objSet VAO
	
	// first make sure all objects are routed to the 
	// same VAO
	
	archObj = objSet->ItemAt(0);
	wsObj = dynamic_cast<TArmeWSObj*>(archObj);
	ZzAssert(wsObj);
	doc = wsObj->GetDoc();
	ZzAssert(doc);
	
	*newRouting = wsObj->GetTopParentVAORouting();
	
	for(SInt32 i = 1; i < objSet->CountItems(); i++)
	{
		archObj = objSet->ItemAt(0);
		wsObj = dynamic_cast<TArmeWSObj*>(archObj);
		ZzRtAssert(wsObj);
		if(*newRouting != wsObj->GetTopParentVAORouting())
		{
			ZZ_THROW_1ARG(ZzGenericException,"You can only export, merge or render objects routed to the same Virtual Output.");
		}
	}
	
	// compute the time span of the whole set of objects
	SInt64 absObjSetStartTime = -1;
	SInt64 objSetDuration = -1;
	SInt64 absObjSetEndxTime = -1;
	
	// nb of channels will be the same as the main vao routing
	SInt32 nbChannels = 1;
	if(newRouting->bStereo)
		nbChannels = 2;
	
	MEAudioObjRenderer::GetObjectSetTimeSpan(objSet,&absObjSetStartTime,&absObjSetEndxTime);
	objSetDuration = absObjSetEndxTime - absObjSetStartTime;
	if(bSliceSel)
		objSetDuration = endxGlobalTime - startGlobalTime;
			
	// open the destination file
	MEMTGAAudioFile dstFile;
	err = dstFile.OpenSaveMTGA(mtgaPath,doc->GetAudioFrameRate(),nbChannels,kMEMTGAAudioFileFmt_Float32);
	if(err.type != kArmeOK)
		ThrowOnArmeErr(err);
	ZzRABI_RemoveFile rabiRemoveDstFile(mtgaPath);
	ZzRABI_CloseMEMTGAAudioFile rabiCloseDstFile(&dstFile);
	
	// create an MEAudioObjRenderer. It needs to know the max nb of frames
	// that will be asked at each ReadNextFloatFrames( ) call...
	const UInt32 kBufferNbFrames = 65536;
	
	// kBufferNbFrames means we'll never request more than kBufferNbFrames frames at once
	// "false" means we don't want any vaorao gain/pan to be applied to the results
	// newRouting.bStereo means we want the same nb of files in the rendered data as 
	// the vao routing of the objects.
	// ZzRABI_ObjectDeleter is a RABI that kills the object at the end of this scope
	MEAudioObjRenderer* renderer = new MEAudioObjRenderer(objSet,bSliceSel,startGlobalTime,endxGlobalTime,kBufferNbFrames,false,nbChannels);
	ZzRABI_ObjectDeleter<MEAudioObjRenderer> rabiRendererDeleter(&renderer);
		
	// let's allocate the tmp buffer
	float* tmpBuffer = new float[kBufferNbFrames*nbChannels];
	
	// a RABI that kills the array at the end of this scope
	ZzRABI_ArrayDeleter<float> rabiTmpBufferDeleter(&tmpBuffer);
	
	// now start the pull loop
	SInt64 curTime = 0;
	
	while(curTime < objSetDuration)
	{
		UInt32 framesToRead = objSetDuration - curTime;
		if(framesToRead > kBufferNbFrames)
			framesToRead = kBufferNbFrames;

		// so that it starts on 0.0xxx and ends on 1.0
		dlgManager->Update((double)(curTime + framesToRead)/ (double)(objSetDuration));
		
		renderer->ReadNextFloatFrames(tmpBuffer,framesToRead); // throws
		err = dstFile.WriteFloat32Frames(tmpBuffer,framesToRead); // ArmeErr
		ThrowOnArmeErr(err);
		curTime += framesToRead;
	}
	// everything went well. let's prevent the MTGA file from being deleted
	rabiRemoveDstFile.SaveResource();
}

