/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
 
#define ME_AudioExportFunctions_h
#ifndef ME_AudioExportFunctions_h
#include "ME_AudioExportFunctions.h"
#endif //ME_AudioExportFunctions_h

#ifndef ZzTypes_h
#include "ZzTypes.h"
#endif //ZzTypes_h

#ifndef ZzString_h
#include "ZzString.h"
#endif //ZzString_h

#ifndef MIProgressBarDlgManager_h
#include "MIProgressBarDlgManager.h"
#endif //MIProgressBarDlgManager_h

class ZzArchPtrArray;

/*! @function MEExportToExternalFile

	- This function allows to export a set of objects (must be siblings) to an AIFF or WAVE file.

	- If bAsMultipleFiles == true, then a different file is created for each top level object by
	appending a number to the file title

	- if the file name contains an extension, then it is removed WARNING ONLY REMOVE .wav or .aif or .aifc or .aiff extensions !
	otherwise,  this will fail "this.is.my.file" which becomes "this.is.my.AIFF" !!!!!!!
	$$$TODO maybe display a dlg box when if the new extension doesn't match the one specified

	- Every object in "theObjects" has a ((grand)parent) VAO mapping (because TLC can't be 
	  specified). This mapping is either mono (with maybe a panning) or stereo. 

		- If nbChannels == 2, then the export is just like sampling the real RAO outputs and 
		  summing every L channels in the file(s) L channels and every R channels in the 
		  file(s) R channels

		- If nbChannels == 1, then the same is done, but both channels are added (with a gain
		  reduction?)

		- If nbChannels == 0, then 

			- if multiple files is NOT selected, then the exported file will be 
			  mono only if ALL TOP VAO ROUTINGS are mono for every object in "theObjects"
			- if multiple files is selected, then the exported file will be mono for 
			  objects that have a mono top vao routing, and stereo for the others
			  
	- Every top level object will be applied its RAOVAO gain and panning
	
	- the export can be restricted to a time slice, expressed in TLC time coords. 
			
*/

enum
{
	kMEExport8Bits = 1789,
	kMEExport16Bits,
	kMEExport24Bits,
	kMEExport32BitsFloat
};

enum
{
	kMEExportAIFF, kMEExportWAVE
};

const char* MIExportToExternalFile_GetExtensionForTypeAndFormat(UInt32 sampleType, UInt32 fileType);

void MEExportToExternalFile(
	ZzArchPtrArray*				theObjects,
	ZzString					basePathName,
	bool						bSliceSel,
	SInt64						startGlobalTime,
	SInt64						endxGlobalTime,
	bool						bAsMultipleFiles,
	bool						bIncludeAllFX,
	bool						bUsesOriginalNames,
	UInt32						nbChannels, 		// 0 means use object routing
	UInt32						audioFrameRate,		// 22050, 44100, 48000, 88200, 96000 or 192000
	UInt32						fileType, 			// kMEExportAIFF, kMEExportWAVE
	UInt32						sampleType, 		// kMEExportFloat32BE, kMEExportFloat32LE, kMEExportShort16LE, kMEExportShort16BE
	MIProgressBarDlgManager* 	dlgManager = NULL);

/*!
	@function MEAudioExportInQTTrack
	
	same as MEExportToExternalFile, except that :
	
	- it renders in an (already created and prepared) opened QT track
	- there is no "multiple files" setting
	- the number of channels cannot be "automatic" (0)
*/
	
void MEAudioExportInQTTrack(
	ZzArchPtrArray*				theObjects,
	bool						bSliceSel,
	SInt64						startGlobalTime,
	SInt64						endxGlobalTime,
	UInt32						nbChannels, // can only be 1 or 2
//	UInt32						audioFrameRate, // should use project audio frame rate
	Media* 						pAudioMedia, 
	SampleDescriptionHandle		audioSampleDesc,
	MIProgressBarDlgManager*	dlgManager = NULL);

/*! 
	@function MEAudioExportInMTGAFile
	
	- takes a set of objects
	- mix them into a single MTGA file
	- the selection can ONLY contain objects routed to the SAME vao
	- no gain/pan changes are applied
	- on return, newRouting will contain the VAO the new clip should
	  be routed to AND it will also tell the number of channels in the
	  rendered file (to adjust an audioclipdata fNbChannels var 
	  accordingly, for example).
	  	
	$$$TODO FUTURE :
	- the VAO returned is the FIRST STEREO VAO, if the set contains
	  both stereo and mono vao routed files, and the FIRST VAO if the 
	  set only contains mono vao
	- each vao has a gain and panning applied to correct for the vao change
	  we should also correct the panning
*/	
class ZzArchPtrArray;
class MEVAORouting;
class MIProgressBarDlgManager;

void MEAudioExportInMTGAFile(
	const char* 				mtgaPath,
	ZzArchPtrArray* 			objSet,
	bool						bSliceSel,
	SInt64						startGlobalTime,
	SInt64						endxGlobalTime,
	MEVAORouting* 				newRouting,
	MIProgressBarDlgManager*	dlgManager = NULL);

