/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#ifndef OfflineAudioProcessorDCAdjust_h
#include "OfflineAudioProcessorDCAdjust.h"
#endif //OfflineAudioProcessorDCAdjust_h


// the progress in this function is sent to the OfflineAudioProcessorParams
// instance, not to the boptask
ArmeErr OfflineAudioProcessorDCAdjust::PerformAudioProcessing(
	OfflineAudioProcessorParams* offlineParams)
{
    ArmeErr err;
        
    //$$$TODO progress!    
//	const SInt32 kTmpBufferNbFrames = 32768;
	
	// we need to call SetProgress from 0 to 1 in this function.
	// but we are going to call SetProgress range for our sub tasks
	float baseProgress,scaleProgress;
	offlineParams->GetProgressRange(&baseProgress,&scaleProgress);
	
	//curBaseProgress = baseProgress+scaleProgress/2.0;
	//curScaleProgress = scaleProgress/2.0;
	//progressStepPerClip = curScaleProgress / offlineParams->GetNbClips();

	for(SInt32 clip = 0; clip < offlineParams->GetNbClips(); clip++)
    {
        OfflineAudioReader* reader = offlineParams->GetReader(clip);
        OfflineAudioWriter* writer = offlineParams->GetWriter(clip);
        float *dcLevelArray = new float[reader->GetNbChannels()];
        getDCLevelArray(offlineParams,writer,reader, dcLevelArray);
        subDCLevelArray(offlineParams,writer,reader, dcLevelArray);
        delete [] dcLevelArray;
	
	}
	offlineParams->SetProgress(1.0);
    return err;
}

void OfflineAudioProcessorDCAdjust::subDCLevelArray(
	ZzProgressTracker*	progressTracker, 
	OfflineAudioWriter*	writer, 
	OfflineAudioReader*	reader, 
	float* 				dcLevelArray)
{
	float baseProgress = 0.0; //$$$TODO fix these
	float scaleProgress = 1.0;
	const SInt32 kTmpBufferNbFrames = 32768;
	SInt32 nbChannels = reader->GetNbChannels();
	SInt64 duration = reader->GetDuration();
	writer->SetDuration(duration);
	float* tmpBuffer = new float[nbChannels*kTmpBufferNbFrames];
	SInt64 pos = 0;

	while(duration - pos > 0)
	{
		// when pos = duration (minus something) and clip = getNbClip - 1, 
		// SetProgress(baseProgress + scaleProgress - epsilon)
		// = SetProgress((NBCLIPS-1)/NBCLIPS + 1/NBCLIPS - epsilon) = SetProgress(1-epsilon)
		progressTracker->SetProgress(baseProgress + scaleProgress*(float)pos/(float)duration);
		SInt32 readNbFrames;
		if( (duration-pos) >= kTmpBufferNbFrames )
			readNbFrames = kTmpBufferNbFrames;
		else
			readNbFrames = duration-pos;
		reader->ReadFrames(tmpBuffer,pos,readNbFrames);
					
		
		float* curFrame = tmpBuffer;
		for(SInt32 frame = 0; frame < readNbFrames; frame++)
		{
			for(SInt32 sampleInFrame = 0; sampleInFrame < nbChannels; sampleInFrame++ )
			{
				*curFrame = *curFrame - dcLevelArray[sampleInFrame];
				curFrame++;
			}
		}
		
		writer->WriteFrames(tmpBuffer,pos,readNbFrames);
		pos += readNbFrames;
	}
	delete [] tmpBuffer;
}

void OfflineAudioProcessorDCAdjust::getDCLevelArray(
	ZzProgressTracker*	progressTracker, 
	OfflineAudioWriter*	writer, 
	OfflineAudioReader*	reader,
    float*				dcLevelArray)
{
	float baseProgress = 0.0; //$$$TODO fix these
	float scaleProgress = 1.0;
	const SInt32 kTmpBufferNbFrames = 32768;
	SInt32 nbChannels = reader->GetNbChannels();
	SInt64 duration = reader->GetDuration();
	writer->SetDuration(duration);
	float* tmpBuffer = new float[nbChannels*kTmpBufferNbFrames];
	SInt64 pos = 0;
    for(SInt32 i = 0; i < nbChannels; i++ ) dcLevelArray[i] = 0.0f;
	while(duration - pos > 0)
	{
		// when pos = duration (minus something) and clip = getNbClip - 1, 
		// SetProgress(baseProgress + scaleProgress - epsilon)
		// = SetProgress((NBCLIPS-1)/NBCLIPS + 1/NBCLIPS - epsilon) = SetProgress(1-epsilon)
		progressTracker->SetProgress(baseProgress + scaleProgress*(float)pos/(float)duration);
		SInt32 readNbFrames;
		if( (duration-pos) >= kTmpBufferNbFrames )
			readNbFrames = kTmpBufferNbFrames;
		else
			readNbFrames = duration-pos;
		reader->ReadFrames(tmpBuffer,pos,readNbFrames);
					
		
		float* curFrame = tmpBuffer;
		for(SInt32 frame = 0; frame < readNbFrames; frame++)
		{
			for(SInt32 sampleInFrame = 0; sampleInFrame < nbChannels; sampleInFrame++ )
			{
				dcLevelArray[sampleInFrame] += *curFrame;
				curFrame++;
			}
		}
		
		writer->WriteFrames(tmpBuffer,pos,readNbFrames);
		pos += readNbFrames;
	}
	delete [] tmpBuffer;
    
    float inv_duration = 1.0f/(float)(duration);
    for(SInt32 i = 0; i < nbChannels; i++ ) dcLevelArray[i] *= inv_duration;
}


		
bool OfflineAudioProcessorDCAdjust::IsLengthy(OfflineAudioProcessorParams* offlineParams)
{	
	// we consider a long operation when the total number of samples frames to reverse is 
	// greater than 5,000,000
	SInt64 totalNbSamples = 0;
	for( SInt32 clip = 0; clip < offlineParams->GetNbClips(); clip++)
	{
		totalNbSamples += offlineParams->GetReader(clip)->GetNbChannels() *
			offlineParams->GetReader(clip)->GetDuration();
	}
	return(totalNbSamples > 5000000);
}
