/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#ifndef OfflineAudioProcessorFadeOut_h
#include "OfflineAudioProcessorFadeOut.h"
#endif //OfflineAudioProcessorFadeOut_h

#import "MIOfflineAudioProcessingFadeInDlgController.h"

#ifndef ABOTcomplex_h
#include "ABOTcomplex.h"
#endif

OfflineAudioProcessorFadeOutParams::OfflineAudioProcessorFadeOutParams()
{
	fAttenuationDBValue = 60.0;
	fFadeType = 0;
}

bool OfflineAudioProcessorFadeOut::DisplayAudioProcessorConfiguration(
												OfflineAudioProcessorParams* offlineParams)
{

	NSModalSession session;
	NSWindow* 	win;
    bool proceed;

    // $$$TODO rename *FadeInDlg* to *FadeInDlg* since fadeOut command
    // uses dialog and controller from fadeincommand
    // i didn't change this now because it is not clear at this moment
    // if dialogs will look different in the future.. [TS]

	id dlgController = [MIOfflineAudioProcessingFadeInDlgController alloc];
	[dlgController initWithWindowNibName:@"OfflineAudioProcessingFadeInDlg" owner:dlgController];
	win = [dlgController window];
    [win setTitle:@"Fade Out"];
	
	session = [NSApp beginModalSessionForWindow:win];
	while([NSApp runModalSession:session] == NSRunContinuesResponse)
	{
	/*progress = bopTask->GetProgress();
		[fProgressDlgController setProgress:progress];
		if(progress >= 1.0)
		{	
			[NSApp stopModal];
		}
		usleep(200000);*/
	}
	[NSApp endModalSession:session];					
	[win orderOut:0];

	OfflineAudioProcessorFadeOutParams* params = (OfflineAudioProcessorFadeOutParams*)offlineParams;
    params->setAttenuationDBValue([dlgController getAttenuationDBValue]);
    params->setFadeType([dlgController getFadeType]);
    proceed = [dlgController getProceed];
	[dlgController release];
	return proceed;
}



OfflineAudioProcessorParams* OfflineAudioProcessorFadeOut::CreateDefaultParams()
{
	return new OfflineAudioProcessorFadeOutParams;
}

// the progress in this function is sent to the OfflineAudioProcessorParams
// instance, not to the boptask
ArmeErr OfflineAudioProcessorFadeOut::PerformAudioProcessing(
	OfflineAudioProcessorParams* offlineParams)
{
    OfflineAudioProcessorFadeOutParams* params = (OfflineAudioProcessorFadeOutParams*)offlineParams;
	ArmeErr err;
    int type = params->getFadeType();
//    float att = params->getAttenuationDBValue();
//    printf("FadeOut: type:%d, att:%f\n", type, att);
    
    switch (type)
    {
        default:
        case 0:
            err = PerformFadeOutLinear(offlineParams);
            break;

        case 1:
            err = PerformFadeOutLogarithmic(offlineParams);
            break;

    }
	offlineParams->SetProgress(1.0);
	return err;
}


ArmeErr OfflineAudioProcessorFadeOut::PerformFadeOutLogarithmic(
	OfflineAudioProcessorParams* offlineParams)
{	

	const SInt32 kTmpBufferNbFrames = 32768;
    OfflineAudioProcessorFadeOutParams* params = (OfflineAudioProcessorFadeOutParams*)offlineParams;
    float log10_attenuation = params->getAttenuationDBValue()/20.0f;


	for( SInt32 clip = 0; clip < offlineParams->GetNbClips(); clip++)
	{
		float baseProgress = ((float)clip/(float)offlineParams->GetNbClips());
		float scaleProgress = 1.0f/(float)offlineParams->GetNbClips();
		OfflineAudioReader* reader = offlineParams->GetReader(clip);
		OfflineAudioWriter* writer = offlineParams->GetWriter(clip);
		SInt32 nbChannels = reader->GetNbChannels();
		SInt64 duration = reader->GetDuration();
		writer->SetDuration(duration);

        
        float *tmpBuffer = new float[nbChannels*kTmpBufferNbFrames];

        // use a fine grain and coarse grain incrementor and amplitude accumulator
        float amplifyPerFrame = pow(10, -log10_attenuation/(float)duration);
        float amplifyPerBuffer = pow(10, -(float)kTmpBufferNbFrames * log10_attenuation/(float)duration);
        float amplitudeFrame = 1.0f;
        float amplitudeBuffer = amplitudeFrame;

        SInt64 pos = 0;


        // perform a fade in
		while(duration - pos > 0)
		{
            // this is to reduce the effect of roundoff errors
            amplitudeFrame = amplitudeBuffer;
            
			offlineParams->SetProgress(baseProgress + scaleProgress*(float)pos/(float)duration);
			SInt32 nbFrames;
			if( (duration-pos) >= kTmpBufferNbFrames )
				nbFrames = kTmpBufferNbFrames;
			else
				nbFrames = duration-pos;
            reader->ReadFrames(tmpBuffer,pos,nbFrames);
            for (int i=0; i < nbFrames*nbChannels; i+=nbChannels)
            {
                for(int j=0; j < nbChannels; j++)
                {
                    tmpBuffer[i+j] *= amplitudeFrame;
                }
                amplitudeFrame *= amplifyPerFrame;
            }
			writer->WriteFrames(tmpBuffer,pos,nbFrames);
			pos += nbFrames;
            
            amplitudeBuffer *= amplifyPerBuffer;
		}
        delete [] tmpBuffer;
	}
	return kArmeOK;
}

ArmeErr OfflineAudioProcessorFadeOut::PerformFadeOutLinear(
	OfflineAudioProcessorParams* offlineParams)
{	

	const SInt32 kTmpBufferNbFrames = 32768;
    OfflineAudioProcessorFadeOutParams* params = (OfflineAudioProcessorFadeOutParams*)offlineParams;
    float min_gain = pow(10.0f, -params->getAttenuationDBValue()/20.0f);


	for( SInt32 clip = 0; clip < offlineParams->GetNbClips(); clip++)
	{
		float baseProgress = ((float)clip/(float)offlineParams->GetNbClips());
		float scaleProgress = 1.0f/(float)offlineParams->GetNbClips();
		OfflineAudioReader* reader = offlineParams->GetReader(clip);
		OfflineAudioWriter* writer = offlineParams->GetWriter(clip);
		SInt32 nbChannels = reader->GetNbChannels();
		SInt64 duration = reader->GetDuration();
		writer->SetDuration(duration);

        
        float *tmpBuffer = new float[nbChannels*kTmpBufferNbFrames];

        // use a fine grain and coarse grain incrementor and amplitude accumulator
        float incrementPerFrame = (min_gain - 1.0f) / (float)duration;
        float incrementPerBuffer = (float)kTmpBufferNbFrames * incrementPerFrame;
        float amplitudeFrame = 1.0f;
        float amplitudeBuffer = 1.0f;

        SInt64 pos = 0;


        // perform a fade in
		while(duration - pos > 0)
		{
            // this is to reduce the effect of roundoff errors
            amplitudeFrame = amplitudeBuffer;
            
			offlineParams->SetProgress(baseProgress + scaleProgress*(float)pos/(float)duration);
			SInt32 nbFrames;
			if( (duration-pos) >= kTmpBufferNbFrames )
				nbFrames = kTmpBufferNbFrames;
			else
				nbFrames = duration-pos;
            reader->ReadFrames(tmpBuffer,pos,nbFrames);
            for (int i=0; i < nbFrames*nbChannels; i+=nbChannels)
            {
                for(int j=0; j < nbChannels; j++)
                {
                    tmpBuffer[i+j] *= amplitudeFrame;
                }
                amplitudeFrame += incrementPerFrame;
            }
			writer->WriteFrames(tmpBuffer,pos,nbFrames);
			pos += nbFrames;
            
            amplitudeBuffer += incrementPerBuffer;
		}
        delete [] tmpBuffer;
	}
	return kArmeOK;
}
	
bool OfflineAudioProcessorFadeOut::IsLengthy(OfflineAudioProcessorParams* offlineParams)
{	
	return(true);
}
	
