/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
#ifndef OfflineAudioProcessorGain_h
#include "OfflineAudioProcessorGain.h"
#endif //OfflineAudioProcessorGain_h

#import "MIOfflineAudioProcessingGainDlgController.h"

bool OfflineAudioProcessorGain::DisplayAudioProcessorConfiguration(
												OfflineAudioProcessorParams* offlineParams)
{

	NSModalSession session;
	NSWindow* 	win;
    bool proceed;

	id dlgController = [MIOfflineAudioProcessingGainDlgController alloc];
	[dlgController initWithWindowNibName:@"OfflineAudioProcessingGainDlg" owner:dlgController];
	win = [dlgController window];
	
	session = [NSApp beginModalSessionForWindow:win];
	while([NSApp runModalSession:session] == NSRunContinuesResponse)
	{
	/*progress = bopTask->GetProgress();
		[fProgressDlgController setProgress:progress];
		if(progress >= 1.0)
		{	
			[NSApp stopModal];
		}
		usleep(200000);*/
	}
	[NSApp endModalSession:session];					
	[win orderOut:0];
	OfflineAudioProcessorGainParams* params = (OfflineAudioProcessorGainParams*)offlineParams;
    params->setGainDBValue([dlgController getGainDBValue]);
    proceed = [dlgController getProceed];
	[dlgController release];
	return proceed;
}


void OfflineAudioProcessorGain::Multiply(
	ZzProgressTracker*	progressTracker, 
	OfflineAudioWriter*	writer, 
	OfflineAudioReader*	reader, 
	float 				factor,
	float				baseProgress,
	float				scaleProgress)
{
	const SInt32 kTmpBufferNbFrames = 32768;
	SInt32 nbChannels = reader->GetNbChannels();
	SInt64 duration = reader->GetDuration();
	writer->SetDuration(duration);
	float* tmpBuffer = new float[nbChannels*kTmpBufferNbFrames];
	SInt64 pos = 0;
	while(duration - pos > 0)
	{
		// when pos = duration (minus something) and clip = getNbClip - 1, 
		// SetProgress(baseProgress + scaleProgress - epsilon)
		// = SetProgress((NBCLIPS-1)/NBCLIPS + 1/NBCLIPS - epsilon) = SetProgress(1-epsilon)
		float progress = baseProgress + scaleProgress*(float)pos/(float)duration;
		vernl(6,"progressTracker->SetProgress("<<progress<<")");
		progressTracker->SetProgress(progress);
		SInt32 readNbFrames;
		if( (duration-pos) >= kTmpBufferNbFrames )
			readNbFrames = kTmpBufferNbFrames;
		else
			readNbFrames = duration-pos;
		reader->ReadFrames(tmpBuffer,pos,readNbFrames);
					
		if(nbChannels == 1)
		{
			float* curFrame = tmpBuffer;
			for(SInt32 frame = 0; frame < readNbFrames; frame++)
			{
				*curFrame = *curFrame * factor;
				curFrame++;
			}
		}
		else if(nbChannels == 2)
		{
			float* curFrame = tmpBuffer;
			for(SInt32 frame = 0; frame < readNbFrames; frame++)
			{
				*curFrame = *curFrame * factor;
				curFrame++;
				*curFrame = *curFrame * factor;
				curFrame++;
			}
		}
		else
		{
			float* curFrame = tmpBuffer;
			for(SInt32 frame = 0; frame < readNbFrames; frame++)
			{
				for(SInt32 sampleInFrame = 0; sampleInFrame < nbChannels; sampleInFrame++ )
				{
					*curFrame = *curFrame * factor;
					curFrame++;
				}
			}
		}
		writer->WriteFrames(tmpBuffer,pos,readNbFrames);
		pos += readNbFrames;
	}
	delete [] tmpBuffer;
}

OfflineAudioProcessorParams* OfflineAudioProcessorGain::CreateDefaultParams()
{
	return new OfflineAudioProcessorGainParams;
}

// the progress in this function is sent to the OfflineAudioProcessorParams
// instance, not to the boptask
ArmeErr OfflineAudioProcessorGain::PerformAudioProcessing(
	OfflineAudioProcessorParams* offlineParams)
{
	ArmeErr err;
	OfflineAudioProcessorGainParams* params = (OfflineAudioProcessorGainParams*)offlineParams;
		
//	const SInt32 kTmpBufferNbFrames = 32768;
    
    float gainFactor = pow(10.0f, params->getGainDBValue()/20.0f);

	SInt32 clip;

	for( clip = 0; clip < offlineParams->GetNbClips(); clip++)
	{
		float scaleProgress = 1.0/(float)offlineParams->GetNbClips();
		float baseProgress =  (float)clip/(float)offlineParams->GetNbClips();
		OfflineAudioReader* reader = offlineParams->GetReader(clip);
		OfflineAudioWriter* writer = offlineParams->GetWriter(clip);
		Multiply(offlineParams,writer,reader,gainFactor,baseProgress,scaleProgress);
	}
	offlineParams->SetProgress(1.0);
	return err;
}
		
bool OfflineAudioProcessorGain::IsLengthy(OfflineAudioProcessorParams* offlineParams)
{		
	return true;
}
	
