/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#ifndef OfflineAudioProcessorLoopsmooth_h
#include "OfflineAudioProcessorLoopsmooth.h"
#endif //OfflineAudioProcessorLoopsmooth_h

#ifndef ABOTcomplex_h
#include "ABOTcomplex.h"
#endif

//#include <math.h>

// the progress in this function is sent to the OfflineAudioProcessorParams
// instance, not to the boptask
ArmeErr OfflineAudioProcessorLoopsmooth::PerformAudioProcessing(
	OfflineAudioProcessorParams* offlineParams)
{	
	ArmeErr err;
    // $$$TODO make the SmoothNbFrames customizable
    const SInt32 kSmoothNbFrames = 256;
	const SInt32 kTmpBufferNbFrames = 32768;

	for( SInt32 clip = 0; clip < offlineParams->GetNbClips(); clip++)
	{
		float baseProgress = ((float)clip/(float)offlineParams->GetNbClips());
		float scaleProgress = 1.0/(float)offlineParams->GetNbClips();
		OfflineAudioReader* reader = offlineParams->GetReader(clip);
		OfflineAudioWriter* writer = offlineParams->GetWriter(clip);
		SInt32 nbChannels = reader->GetNbChannels();
		SInt64 duration = reader->GetDuration();
        SInt64 restEndx, pos;
		writer->SetDuration(duration);

        
        float *tmpBuffer = new float[nbChannels*kTmpBufferNbFrames];

        // if a clip is long enough, smooth it, otherwize just copy it
        if (duration >= 2*kSmoothNbFrames){
            // smooth
            // setup cosine generator
            ABOTcomplex phase(0.5f,0.0f);
            ABOTcomplex freq(M_PI / (float)kSmoothNbFrames);

            // read beginning of file
            reader->ReadFrames(tmpBuffer, 0, kSmoothNbFrames);
            // read end of file
            reader->ReadFrames(tmpBuffer + kSmoothNbFrames*nbChannels , duration - kSmoothNbFrames, kSmoothNbFrames);

            // smooth begin and end with the 2 half periods of (1-cos)/2
            for (int i=0; i < 2 * kSmoothNbFrames * nbChannels; i+=nbChannels){
                for (int j=0; j < nbChannels; j++){
                    tmpBuffer[i+j] *= (0.5f - phase._r);
                }
                phase *= freq;
            }

            // write result to beginning of file
            writer->WriteFrames(tmpBuffer, 0, kSmoothNbFrames);
            // write result to end of file
            writer->WriteFrames(tmpBuffer +  kSmoothNbFrames*nbChannels, duration - kSmoothNbFrames, kSmoothNbFrames);

            
            // prepare to copy middle part
            restEndx = duration - kSmoothNbFrames;
            pos = kSmoothNbFrames;
        }
        else {
            // prepare to copy everything
            restEndx = duration;
            pos = 0;
        }

        // copy the rest of the file
		while(restEndx - pos > 0)
		{
			offlineParams->SetProgress(baseProgress + scaleProgress*(float)pos/(float)restEndx);
			SInt32 nbFrames;
			if( (restEndx-pos) >= kTmpBufferNbFrames )
				nbFrames = kTmpBufferNbFrames;
			else
				nbFrames = restEndx-pos;
            reader->ReadFrames(tmpBuffer,pos,nbFrames);
			writer->WriteFrames(tmpBuffer,pos,nbFrames);
			pos += nbFrames;
		}
        delete [] tmpBuffer;
	}
	offlineParams->SetProgress(1.0);
	return err;
}
	
bool OfflineAudioProcessorLoopsmooth::IsLengthy(OfflineAudioProcessorParams* offlineParams)
{	
	return(true);
}
	
