/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#ifndef OfflineAudioProcessorNormalize_h
#include "OfflineAudioProcessorNormalize.h"
#endif //OfflineAudioProcessorNormalize_h

#import "MIOfflineAudioProcessingNormalizeDlgController.h"

bool OfflineAudioProcessorNormalize::DisplayAudioProcessorConfiguration(
												OfflineAudioProcessorParams* offlineParams)
{
	NSModalSession session;
	NSWindow* 	win;
    BOOL returnVal;

	id dlgController = [MIOfflineAudioProcessingNormalizeDlgController alloc];
	[dlgController initWithWindowNibName:@"OfflineAudioProcessingNormalizeDlg" owner:dlgController];
	win = [dlgController window];
	
	session = [NSApp beginModalSessionForWindow:win];
	while([NSApp runModalSession:session] == NSRunContinuesResponse)
	{
	/*progress = bopTask->GetProgress();
		[fProgressDlgController setProgress:progress];
		if(progress >= 1.0)
		{	
			[NSApp stopModal];
		}
		usleep(200000);*/
	}
	[NSApp endModalSession:session];					
	[win orderOut:0];
	OfflineAudioProcessorNormalizeParams* params = (OfflineAudioProcessorNormalizeParams*)offlineParams;
    
    // store params in object
	params->SetAcrossAllClips([dlgController isAcrossAllClips]);
	params->SetDBVal([dlgController getDBVal]);
    
    // return false if cacel was pressed
    returnVal = [dlgController proceed];
    
	[dlgController release];
	return returnVal; 
}

void OfflineAudioProcessorNormalize::Multiply(
	ZzProgressTracker*	progressTracker, 
	OfflineAudioWriter*	writer, 
	OfflineAudioReader*	reader, 
	float 				factor)
{
	float baseProgress = 0.0; //$$$TODO fix these
	float scaleProgress = 1.0;
	const SInt32 kTmpBufferNbFrames = 32768;
	SInt32 nbChannels = reader->GetNbChannels();
	SInt64 duration = reader->GetDuration();
	writer->SetDuration(duration);
	float* tmpBuffer = new float[nbChannels*kTmpBufferNbFrames];
	SInt64 pos = 0;
	while(duration - pos > 0)
	{
		// when pos = duration (minus something) and clip = getNbClip - 1, 
		// SetProgress(baseProgress + scaleProgress - epsilon)
		// = SetProgress((NBCLIPS-1)/NBCLIPS + 1/NBCLIPS - epsilon) = SetProgress(1-epsilon)
		progressTracker->SetProgress(baseProgress + scaleProgress*(float)pos/(float)duration);
		SInt32 readNbFrames;
		if( (duration-pos) >= kTmpBufferNbFrames )
			readNbFrames = kTmpBufferNbFrames;
		else
			readNbFrames = duration-pos;
		reader->ReadFrames(tmpBuffer,pos,readNbFrames);
					
		if(nbChannels == 1)
		{
			float* curFrame = tmpBuffer;
			for(SInt32 frame = 0; frame < readNbFrames; frame++)
			{
				*curFrame = *curFrame * factor;
				curFrame++;
			}
		}
		else if(nbChannels == 2)
		{
			float* curFrame = tmpBuffer;
			for(SInt32 frame = 0; frame < readNbFrames; frame++)
			{
				*curFrame = *curFrame * factor;
				curFrame++;
				*curFrame = *curFrame * factor;
				curFrame++;
			}
		}
		else
		{
			float* curFrame = tmpBuffer;
			for(SInt32 frame = 0; frame < readNbFrames; frame++)
			{
				for(SInt32 sampleInFrame = 0; sampleInFrame < nbChannels; sampleInFrame++ )
				{
					*curFrame = *curFrame * factor;
					curFrame++;
				}
			}
		}
		writer->WriteFrames(tmpBuffer,pos,readNbFrames);
		pos += readNbFrames;
	}
	delete [] tmpBuffer;
}

OfflineAudioProcessorParams* OfflineAudioProcessorNormalize::CreateDefaultParams()
{
	return new OfflineAudioProcessorNormalizeParams;
}

// the progress in this function is sent to the OfflineAudioProcessorParams
// instance, not to the boptask
ArmeErr OfflineAudioProcessorNormalize::PerformAudioProcessing(
	OfflineAudioProcessorParams* offlineParams)
{
	ArmeErr err;
	OfflineAudioProcessorNormalizeParams* params = (OfflineAudioProcessorNormalizeParams*)offlineParams;
		
	const SInt32 kTmpBufferNbFrames = 32768;
	
	// we need to call SetProgress from 0 to 1 in this function.
	// but we are going to call SetProgress range for our sub tasks
	float baseProgress,scaleProgress;
	offlineParams->GetProgressRange(&baseProgress,&scaleProgress);
	
    // set the scaling factor
    float scaleFactor = pow(10.0f, params->GetDBVal()/20.0f);
    
	if(params->IsAcrossAllClips())
	{
		float maxAbsValue = 0.0;
		SInt32 clip;
		float curBaseProgress = baseProgress;
		// the scanning for max value spans half the progress range
		float curScaleProgress = scaleProgress/2.0;
		// this is the progress scale for each clip
		float progressStepPerClip = curScaleProgress / offlineParams->GetNbClips();
		for( clip = 0; clip < offlineParams->GetNbClips(); clip++)
		{
			// the scanning for this clip spans a part of the range
			// we need to have a linear progress from curBaseProgress
			// to curBaseProgress in offlineParams->GetNbClips() steps.
			// so :
			offlineParams->SetProgressRange(curBaseProgress + clip*progressStepPerClip, curBaseProgress + (clip+1)*progressStepPerClip);
			OfflineAudioReader* reader = offlineParams->GetReader(clip);
			SInt32 nbChannels = reader->GetNbChannels();
			float* tmpBuffer = new float[nbChannels*kTmpBufferNbFrames];
			SInt64 duration = reader->GetDuration();
			SInt64 pos = 0;
			while(duration - pos > 0)
			{
				//offlineParams->SetProgress(baseProgress + scaleProgress*(float)pos/(float)duration);
				SInt32 readNbFrames;
				if( (duration-pos) >= kTmpBufferNbFrames )
					readNbFrames = kTmpBufferNbFrames;
				else
					readNbFrames = duration-pos;
				reader->ReadFrames(tmpBuffer,pos,readNbFrames);
				for( SInt32 sample = 0; sample < readNbFrames*reader->GetNbChannels(); sample++ )
				{
					float val = fabs(tmpBuffer[sample]);
					if( val > maxAbsValue)
						maxAbsValue = val;
				}
				if(pos == 0)
					offlineParams->SetProgress(0.0);
				else
					offlineParams->SetProgress(1.0/(float)(duration/pos));
				pos += readNbFrames;
			}
			delete [] tmpBuffer;
		}
		//
		//  part 2 : normalizing factor is known, now we apply it
		//
		curBaseProgress = baseProgress+scaleProgress/2.0;
		// the actual normalization for max value spans the second half of 
		// the progress range
		curScaleProgress = scaleProgress/2.0;
		progressStepPerClip = curScaleProgress / offlineParams->GetNbClips();
		
		float factor;
		if(maxAbsValue < 0.0000001)
			factor = 0.0;
		else
			factor = scaleFactor/maxAbsValue;
		for( clip = 0; clip < offlineParams->GetNbClips(); clip++)
		{
			offlineParams->SetProgressRange(curBaseProgress + clip*progressStepPerClip, curBaseProgress + (clip+1)*progressStepPerClip);
			OfflineAudioReader* reader = offlineParams->GetReader(clip);
			OfflineAudioWriter* writer = offlineParams->GetWriter(clip);
			Multiply(offlineParams,writer,reader,factor);
		}
	}
	else
	{
		float curBaseProgress = baseProgress;
		// the scanning for max value spans half the progress range
		float curScaleProgress = scaleProgress/2.0;
		// this is the progress scale for each clip
		float progressStepPerClip = curScaleProgress / offlineParams->GetNbClips();
		SInt32 clip;
		
		for( clip = 0; clip < offlineParams->GetNbClips(); clip++)
		{
			float maxAbsValue = 0.0;
			
			// the scanning for this clip spans a part of the range
			// we need to have a linear progress from curBaseProgress
			// to curBaseProgress in offlineParams->GetNbClips() steps.
			// so :
			offlineParams->SetProgressRange(curBaseProgress + clip*progressStepPerClip, curBaseProgress + (clip+1)*progressStepPerClip);
			OfflineAudioReader* reader = offlineParams->GetReader(clip);
			SInt32 nbChannels = reader->GetNbChannels();
			float* tmpBuffer = new float[nbChannels*kTmpBufferNbFrames];
			SInt64 duration = reader->GetDuration();
			SInt64 pos = 0;
			while(duration - pos > 0)
			{
				offlineParams->SetProgress((float)pos/(float)duration);
				SInt32 readNbFrames;
				if( (duration-pos) >= kTmpBufferNbFrames )
					readNbFrames = kTmpBufferNbFrames;
				else
					readNbFrames = duration-pos;
				reader->ReadFrames(tmpBuffer,pos,readNbFrames);
				for( SInt32 sample = 0; sample < readNbFrames*reader->GetNbChannels(); sample++ )
				{
					float val = fabs(tmpBuffer[sample]);
					if( val > maxAbsValue)
						maxAbsValue = val;
				}
				if(pos == 0)
					offlineParams->SetProgress(0.0);
				else
					offlineParams->SetProgress(1.0/(float)(duration/pos));
				pos += readNbFrames;
			}
			delete [] tmpBuffer;

			//
			//  part 2 : normalizing factor is known, now we apply it
			//
			float factor = scaleFactor/maxAbsValue;
			
			reader = offlineParams->GetReader(clip);
			OfflineAudioWriter* writer = offlineParams->GetWriter(clip);
			Multiply(offlineParams,writer,reader,factor);
		}
	}
	offlineParams->SetProgress(1.0);
	return err;
}
		
bool OfflineAudioProcessorNormalize::IsLengthy(OfflineAudioProcessorParams* offlineParams)
{	
	// we consider a long operation when the total number of samples frames to reverse is 
	// greater than 5,000,000
	SInt64 totalNbSamples = 0;
	for( SInt32 clip = 0; clip < offlineParams->GetNbClips(); clip++)
	{
		totalNbSamples += offlineParams->GetReader(clip)->GetNbChannels() *
			offlineParams->GetReader(clip)->GetDuration();
	}
	return(totalNbSamples > 5000000);
}
	
