/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
#ifndef OfflineAudioProcessorReverse_h
#include "OfflineAudioProcessorReverse.h"
#endif //OfflineAudioProcessorReverse_h


// the progress in this function is sent to the OfflineAudioProcessorParams
// instance, not to the boptask
ArmeErr OfflineAudioProcessorReverse::PerformAudioProcessing(
	OfflineAudioProcessorParams* offlineParams)
{	
	ArmeErr err;
//	float tmpBuffer;
	const SInt32 kTmpBufferNbFrames = 32768;
	for( SInt32 clip = 0; clip < offlineParams->GetNbClips(); clip++)
	{
		float baseProgress = ((float)clip/(float)offlineParams->GetNbClips());
		float scaleProgress = 1.0/(float)offlineParams->GetNbClips();
		OfflineAudioReader* reader = offlineParams->GetReader(clip);
		OfflineAudioWriter* writer = offlineParams->GetWriter(clip);
		SInt32 nbChannels = reader->GetNbChannels();
		SInt64 duration = reader->GetDuration();
		writer->SetDuration(duration);
		float* tmpBuffer = new float[nbChannels*kTmpBufferNbFrames];
		float* tmpFrame;
		float tmpFrameLOrMono,tmpFrameR;
		if(nbChannels > 2)
			tmpFrame = new float[nbChannels];
		SInt64 pos = 0;
		while(duration - pos > 0)
		{
			// when pos = duration (minus something) and clip = getNbClip - 1, SetProgress(baseProgress + scaleProgress - epsilon)
			// = SetProgress((NBCLIPS-1)/NBCLIPS + 1/NBCLIPS - epsilon) = SetProgress(1-epsilon)
			offlineParams->SetProgress(baseProgress + scaleProgress*(float)pos/(float)duration);
			SInt32 readNbFrames;
			if( (duration-pos) >= kTmpBufferNbFrames )
				readNbFrames = kTmpBufferNbFrames;
			else
				readNbFrames = duration-pos;
			reader->ReadFrames(tmpBuffer,pos,readNbFrames);
			SInt64 writePos = duration-pos-readNbFrames;
						
			if(nbChannels == 1)
			{
				for(SInt32 frame = 0; frame < readNbFrames/2; frame++)
				{
					// the frame that's opposite to this one
					SInt32 oppFrame = readNbFrames-frame-1;
					// put 'frame' in 'tmp'
					tmpFrameLOrMono = *(tmpBuffer+frame);
					// put 'opp frame' in 'frame'
					*(tmpBuffer+frame) = *(tmpBuffer+oppFrame);
					// put 'tmp' in 'opp frame'
					*(tmpBuffer+oppFrame) = tmpFrameLOrMono;
				}
			}
			else if(nbChannels == 2)
			{
				for(SInt32 frame = 0; frame < readNbFrames/2; frame++)
				{
					// the frame that's opposite to this one
					SInt32 oppFrame = readNbFrames-frame-1;
					// put 'frame' in 'tmp'
					tmpFrameLOrMono = *(tmpBuffer+2*frame);
					tmpFrameR = *(tmpBuffer+2*frame+1);
					// put 'opp frame' in 'frame'
					*(tmpBuffer+2*frame) = *(tmpBuffer+2*oppFrame);
					*(tmpBuffer+2*frame+1) = *(tmpBuffer+2*oppFrame+1);
					// put 'tmp' in 'opp frame'
					*(tmpBuffer+2*oppFrame) = tmpFrameLOrMono;
					*(tmpBuffer+2*oppFrame+1) = tmpFrameR;
				}
			}
			else
			{
				for(SInt32 frame = 0; frame < readNbFrames/2; frame++)
				{
					SInt32 oppFrame = readNbFrames-frame-1;
					// put 'frame' in 'tmp'
					memcpy(tmpFrame, tmpBuffer+frame*nbChannels, nbChannels*sizeof(float));
					// put 'opp frame' in 'frame'
					memcpy(tmpBuffer+frame*nbChannels, tmpBuffer+oppFrame*nbChannels,
						nbChannels*sizeof(float));
					// put 'tmp' in 'opp frame'
					memcpy(tmpBuffer+oppFrame*nbChannels, tmpFrame, nbChannels*sizeof(float));
				}
			}
			writer->WriteFrames(tmpBuffer,writePos,readNbFrames);
			pos += readNbFrames;
		}
		if(nbChannels > 2)
			delete [] tmpFrame;
		delete [] tmpBuffer;
	}
        offlineParams->SetProgress(1.01);
	return err;
}
	
bool OfflineAudioProcessorReverse::IsLengthy(OfflineAudioProcessorParams* offlineParams)
{	
	// we consider a long operation when the total number of samples frames to reverse is 
	// greater than 5,000,000
	SInt64 totalNbSamples = 0;
	for( SInt32 clip = 0; clip < offlineParams->GetNbClips(); clip++)
	{
		totalNbSamples += offlineParams->GetReader(clip)->GetNbChannels() *
			offlineParams->GetReader(clip)->GetDuration();
	}
	return(totalNbSamples > 5000000);
}
	
