/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#ifndef OfflineAudioProcessorSwapChannels_h
#include "OfflineAudioProcessorSwapChannels.h"
#endif //OfflineAudioProcessorSwapChannels_h


ArmeErr OfflineAudioProcessorSwapChannels::PerformAudioProcessing(
	OfflineAudioProcessorParams* offlineParams)
{
    SInt32 tmpBufferNbFrames = 1 << 15; //32k

    ArmeErr err;
		
	SInt32 clip;

	for( clip = 0; clip < offlineParams->GetNbClips(); clip++)
	{
        
		OfflineAudioReader* reader = offlineParams->GetReader(clip);
		OfflineAudioWriter* writer = offlineParams->GetWriter(clip);
		SInt32 nbChannels = reader->GetNbChannels();
		SInt64 duration = reader->GetDuration();
		writer->SetDuration(duration);

        if (nbChannels != 2) {err = ArmeErr(kArmeAudioProcFailed);} 
        else
        {
            float *tmpBuffer = new float[2 * tmpBufferNbFrames];
            float tmpFloatL, tmpFloatR;

            SInt32 pos = 0;
            while(pos <= (duration - tmpBufferNbFrames))
            {
                reader->ReadFrames(tmpBuffer,pos,tmpBufferNbFrames);
                for(SInt32 i=0; i<tmpBufferNbFrames*2; i+=2)
                {
                    tmpFloatL = tmpBuffer[i];
                    tmpFloatR = tmpBuffer[i+1];
                    tmpBuffer[i] = tmpFloatR;
                    tmpBuffer[i+1] = tmpFloatL;
                }
                writer->WriteFrames(tmpBuffer,pos,tmpBufferNbFrames);
                pos += tmpBufferNbFrames;
            }
            SInt32 leftoverNbFrames = duration - pos;
            if (leftoverNbFrames)
            {
                reader->ReadFrames(tmpBuffer,pos,leftoverNbFrames);
                for(SInt32 i=0; i<leftoverNbFrames*2; i+=2)
                {
                    tmpFloatL = tmpBuffer[i];
                    tmpFloatR = tmpBuffer[i+1];
                    tmpBuffer[i] = tmpFloatR;
                    tmpBuffer[i+1] = tmpFloatL;
                }
                writer->WriteFrames(tmpBuffer,pos,leftoverNbFrames);
            }
            delete [] tmpBuffer;
        }
        

	}
    offlineParams->SetProgress(1);
	return err;
	
}
		
bool OfflineAudioProcessorSwapChannels::IsLengthy(OfflineAudioProcessorParams* offlineParams)
{		return(true);
}
	
