/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/


#include <CoreMIDI/MIDIServices.h>
#include <unistd.h>

inline void chkerrcm(OSStatus err)
{
	const char* strErr;
	switch(err)
	{
	case noErr : 
		strErr = "noErr";
		break;
	case kMIDIInvalidClient :
		strErr = "kMIDIInvalidClient";
		break;
	case kMIDIInvalidPort :
		strErr = "kMIDIInvalidPort";
		break;
	case kMIDIWrongEndpointType :// want source, got destination, or vice versa
		strErr = "kMIDIWrongEndpointType";
		break;
	case kMIDINoConnection :            // attempt to close a non-existant connection
		strErr = "kMIDINoConnection";
		break;
	case kMIDIUnknownEndpoint :          
		strErr = "kMIDIUnknownEndpoint";
		break;
	case kMIDIUnknownProperty :          
		strErr = "kMIDIUnknownProperty";
		break;
	case kMIDIWrongPropertyType :        
		strErr = "kMIDIWrongPropertyType";
		break;
	case kMIDINoCurrentSetup :           // there is no current setup, or it contains no devices
		strErr = "kMIDINoCurrentSetup";
		break;
	case kMIDIMessageSendErr :           // communication with server failed
		strErr = "kMIDIMessageSendErr";
		break;
	case kMIDIServerStartErr :           // couldn't start the server
		strErr = "kMIDIServerStartErr";
		break;
	case kMIDISetupFormatErr :          // unparseable saved state
		strErr = "kMIDISetupFormatErr";
		break;
	default :
		strErr = "benjamin <UNKWOWN>";
		break;
	}
	if( err != noErr )
		ZzAssert(0);
};

inline void debugPrintMIDIObjectNames(MIDIObjectRef obj)
{
	CFStringRef strName,strManufacturer,strModel;
	const char *szName,*szManufacturer,*szModel;
	chkerrcm(MIDIObjectGetStringProperty(obj,kMIDIPropertyName,&strName));
	chkerrcm(MIDIObjectGetStringProperty(obj,kMIDIPropertyManufacturer,&strManufacturer));
	chkerrcm(MIDIObjectGetStringProperty(obj,kMIDIPropertyModel,&strModel));
	szName = CFStringGetCStringPtr(strName,0);
	szManufacturer = CFStringGetCStringPtr(strManufacturer,0);
	szModel = CFStringGetCStringPtr(strModel,0);
	verntl(midiBenjamin,3,"(" << szManufacturer << "," << szName << "," << szModel <<")");
}	

inline void debugMIDIPrintSystemInfo()
{
	ItemCount ndev = MIDIGetNumberOfDevices();
	
	for (ItemCount idev = 0; idev < ndev; ++idev) 
	{
		MIDIDeviceRef device = MIDIGetDevice(idev);
		verntl(midiBenjamin,3,"device " << idev);
		debugPrintMIDIObjectNames(device);
		ItemCount nent = MIDIDeviceGetNumberOfEntities(device);
		for (ItemCount ient = 0; ient < nent; ++ient) 
		{
			MIDIEntityRef entity = MIDIDeviceGetEntity(device, ient);
			verntl(midiBenjamin,3,"    entity " << ient);
			debugPrintMIDIObjectNames(entity);
			
			// destinations
			ItemCount ndsts = MIDIEntityGetNumberOfDestinations(entity);
			for(ItemCount idst = 0; idst < ndsts; ++idst)
			{
				MIDIEndpointRef endpt = MIDIEntityGetDestination(entity,idst);
				verntl(midiBenjamin,3,"        destination " << idst);
				debugPrintMIDIObjectNames(endpt);
			}
		
			// sources
			ItemCount nsrcs = MIDIEntityGetNumberOfSources(entity);
			for(ItemCount isrc = 0; isrc < nsrcs; ++isrc)
			{
				MIDIEndpointRef endpt = MIDIEntityGetSource(entity,isrc);
				verntl(midiBenjamin,3,"        source " << isrc);
				debugPrintMIDIObjectNames(endpt);
			}

		} 
	}
}


