/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

/*
 * ABOTwavesync: A class for correlation based waveform synchronization
 *
 * instantiation arguments: 
 *
 * nbFrames = max nb of frames that will be processed
 * the nbFrames can be changed later but it won't realloc buffers
 *
 * nbChannels = max nb of channels that will be processed
 * the input will be summed to a mono block and then correlated
 *
 * usage: offset = GetSyncOffset(float* ideal, float* region);
 * ideal will be the block you are looking for (length == NbFrames)
 * region will be the block in which is searched (length == NbFrames * 2)
 * the method returns the offset of the maximum correlation offset
 * so region[offset*NbChannels] will be the start of NbFrames size block that 
 * is most similar to the NbFrames size block starting at ideal[0]
 */


#ifndef ABOTwavesync_h
#define ABOTwavesync_h

// this is the interface common to all platform
// dependent implementations of ABOTwavesync

#define DECLARE_INTERFACE___ABOTwavesync\
   	inline ABOTwavesync(SInt32 nbFrames, SInt32 nbChannels);\
    inline ~ABOTwavesync();\
    inline SInt32 GetSyncOffset(float* ideal, float* region);\
    inline SInt32 GetSyncOffsetCircular(float* ideal_buf, float* region_buf,\
		SInt32 ideal_index, SInt32 region_index, SInt32 ideal_mask, SInt32 region_mask);\
	inline void SetNbFrames(SInt32 i);\
	inline void SetNbChannels(SInt32 i);\
	inline void SetWeightCenter(float w);\
	inline void SetWeightTail(float w);\
	inline void SetBiasOffset(SInt32 o);
 

#ifdef __APPLE__
#include "ABOTwavesync_veclib.h"
#endif

#endif