/////////////////////////////////////////////////////////
// Float to Int conversion routines                    //
// Optimized in assembly language for x86 CPUs         //
//                                                     //
// C. Townsend 6/98                                    //
/////////////////////////////////////////////////////////
#ifndef FLOAT2INT_H
#define FLOAT2INT_H

#ifndef DSP_Globals_h
#include "DSP_Globals.h"
#endif //DSP_Globals_h

// Convert float to int and truncate.
inline Int32 f2i_t(float fval)
{
	int result;

#ifdef _X86_
	float OneHalfOffset=0.5;
	//add 1/2 so truncation occurs
	//Remember the FPU is usually set for convergent rounding so truncation will not always be exactly right
	_asm
	{
		fld	  fval
		fsub  OneHalfOffset
		fistp result
	}
#else
	result=(Int32)fval;
#endif

	return result;
}

// Convert float to int and truncate.
inline Int32 f2i_t(double dval)
{
	int result;

#ifdef _X86_
	float OneHalfOffset=0.5;
	//add 1/2 so truncation occurs
	//Remember the FPU is usually set for convergent rounding so truncation will not always be exactly right
	_asm
	{
		fld	  dval
		fsub  OneHalfOffset
		fistp result
	}
#else
	result=(Int32)dval;
#endif

	return result;
}


// Convert float to int and round.
inline Int32 f2i_r(float fval)
{
	Int32 result;

#ifdef _X86_
	_asm
	{
		fld	  fval
		fistp result
	}
#else
	//add 1/2 so rounding occurs
	//Remember if FPU truncation is set towards zero then rounding won't be right for negative numbers
	result=(Int32)(fval+0.5f);
#endif

	return result;
}


// Convert float to int and round.
inline Int32 f2i_r(double dval)
{
	Int32 result;

#ifdef _X86_
	_asm
	{
		fld	  dval
		fistp result
	}
#else
	//add 1/2 so rounding occurs
	//Remember if FPU truncation is set towards zero then rounding won't be right for negative numbers
	result=(Int32)(dval+0.5);
#endif

	return result;
}


#endif  //end FLOAT2INT_H