/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MIActiveCursor.h"


@implementation MIActiveCursor

- (id)initWithPlainCursor:(NSCursor*)cursor forActiveRect:(NSRect)rect {
	if (( self = [super init] )) {
		[self setPlainCursor:cursor];
		[self setActiveRect:rect];
	}
	return self;
}

- (void)dealloc {
	[fPlainCursor release];
	[fAlternateCursor release];
	[fControlCursor release];
	[fCommandCursor release];
	[fShiftCursor release];
	[super dealloc];
}

- (NSRect)activeRect {
	return fActiveRect;
}

- (void)setActiveRect:(NSRect)rect {
	fActiveRect = rect;
}

- (NSTrackingRectTag)tag {
	return fTag;
}

- (void)setTag:(NSTrackingRectTag)tag {
	fTag = tag;
}

- (NSCursor*)plainCursor {
	return fPlainCursor;
}

- (void)setPlainCursor:(NSCursor*)cursor {
	[cursor retain];
	[fPlainCursor release];
	fPlainCursor = cursor;
}

- (NSCursor*)alternateCursor {
	if ( fAlternateCursor )
		return fAlternateCursor;
	else return fPlainCursor;
}

- (void)setAlternateCursor:(NSCursor*)cursor {
	[cursor retain];
	[fAlternateCursor release];
	fAlternateCursor = cursor;
}

- (NSCursor*)controlCursor {
	if ( fControlCursor )
		return fControlCursor;
	else return fPlainCursor;
}

- (void)setControlCursor:(NSCursor*)cursor {
	[cursor retain];
	[fControlCursor release];
	fControlCursor = cursor;
}

- (NSCursor*)commandCursor {
	if ( fCommandCursor )
		return fCommandCursor;
	else return fPlainCursor;
}

- (void)setCommandCursor:(NSCursor*)cursor {
	[cursor retain];
	[fCommandCursor release];
	fCommandCursor = cursor;
}

- (NSCursor*)shiftCursor {
	if ( fShiftCursor )
		return fShiftCursor;
	else return fPlainCursor;
}

- (void)setShiftCursor:(NSCursor*)cursor {
	[cursor retain];
	[fShiftCursor release];
	fShiftCursor = cursor;
}



@end
