/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MIApplication.h"
#import "MIStartingPoint.h"
#import "MIErrorPanel.h"
#import "MIMainViewController.h"

#ifndef TArmeMediaEngine_h
#include "TArmeMediaEngine.h"
#endif //TArmeMediaEngine_h

#import <MIDocumentController.h>

#ifndef ZzString_h
#include "ZzString.h"
#endif //ZzString_h

#ifndef ZzHelpers_h
#include "ZzHelpers.h"
#endif //ZzHelpers_h


#undef VER_DEF_TOPIC
#define VER_DEF_TOPIC MIApplication

@implementation MIApplication

- (void)performKeyCommand:(NSString*)key withModifiers:(unsigned int)modifiers {
	NSAssert(NO, @"MIApplication - performKeyCommand:withModifiers: should never be called");
}

- (id)init {
	if (( self = [super init] )) {
		fKeyCommands = [[NSMutableArray alloc] initWithCapacity:10];
		fSignature = [[self methodSignatureForSelector:@selector(performKeyCommand:withModifiers:)] retain];
	}
	return self;
}


- (void)dealloc {
	[fKeyCommands release];
	[fSignature release];
	[super dealloc];
}

- (void)terminate:(id)sender {
	[[MIStartingPoint startingPoint] setShouldDisplayStartingPoint:NO];
	[super terminate:sender];
	[[MIStartingPoint startingPoint] setShouldDisplayStartingPoint:YES];
}

- (MIMainViewController*)mainViewController {
	id mainWindow = [NSApp mainWindow];
	if ( mainWindow ) {
		id controller = [mainWindow windowController];
		if ( controller ) {
			if ( [controller isMemberOfClass:[MIMainViewController class]] ) {
				fMainViewController = controller;
			}
		}
	} else {
		if ( ![[NSDocumentController sharedDocumentController] currentDocument] )
			fMainViewController = nil;
	}

	return (MIMainViewController*)fMainViewController;
}

- (void)resetMainViewController {
	fMainViewController = nil;
}

- (NSMenuItem*)searchForMenuItemWithTag:(int)tag {
	NSMenu *mainMenu = [NSApp mainMenu];
	NSEnumerator *enumerator;
	NSMenuItem* item;

	item = [mainMenu itemWithTag:tag];
	if ( item ) return item;
	
	enumerator = [[mainMenu itemArray] objectEnumerator];
	while (( item = [enumerator nextObject] )) {
		NSMenu* menu = [item submenu];
		if ( menu ) {
			NSMenuItem* menuItem = [menu itemWithTag:tag];
			if ( menuItem ) {
				return menuItem;
			} else {
				NSEnumerator *subenumerator = [[menu itemArray] objectEnumerator];
				while (( menuItem = [subenumerator nextObject] )) {
					if ( [menuItem hasSubmenu] ) {
						NSMenu* submenu = [menuItem submenu];
						if ( submenu ) {
							NSMenuItem* submenuItem = [submenu itemWithTag:tag];
							if ( submenuItem ) {
								return submenuItem;
							}
						}
					}
				}
			}
		}
	}
	return nil;
}

- (BOOL)doKeyCommand:(NSEvent *)anEvent
{
	unsigned x ;
	MIKeyCommand *keyCommand;
	NSString *key;
	unsigned int modifiers;
    NSRange range;
	NSString *str = [anEvent charactersIgnoringModifiers];
	
	for( x = 0 ; x < [fKeyCommands count] ; x++ )
	{
		keyCommand = [fKeyCommands objectAtIndex:x];
		key = [keyCommand key];
		range = [str rangeOfString:key];
		if ( range.location != NSNotFound ) 
		{
			modifiers = [anEvent modifierFlags] & (NSHelpKeyMask| NSControlKeyMask| NSShiftKeyMask| NSCommandKeyMask| NSAlternateKeyMask);
			if ( [keyCommand modifiers] == modifiers )
			{
				NSInvocation *invocation = [NSInvocation invocationWithMethodSignature:fSignature];
				[invocation setTarget:[keyCommand target]];
				[invocation setSelector:[keyCommand selector]];
				[invocation setArgument:&key atIndex:2];
				[invocation setArgument:&modifiers atIndex:3];
				[invocation invoke];
				return YES;
			}
		}
	}
	return NO;
}

- (void)sendEvent:(NSEvent *)anEvent
{
	if ( [anEvent type] == NSKeyDown )
	{
		unsigned int mod =[anEvent modifierFlags] & (NSHelpKeyMask| NSControlKeyMask| NSShiftKeyMask| NSCommandKeyMask| NSAlternateKeyMask); 
		if ( mod != 0 )
		{
			if ( ![self doKeyCommand:anEvent] )
				[super sendEvent:anEvent];
		}
		else
		{
			id keyWindow = [self keyWindow];
			if ( [[anEvent characters] isEqualToString:@"\r"] )
			{
				id defaultButtonCell = [keyWindow defaultButtonCell];
				if ( defaultButtonCell )
				{
					[super sendEvent:anEvent];
					return;
				}
			}

			id firstResponder = [keyWindow firstResponder];

			if ( firstResponder )
			{
				if ( ( [firstResponder isKindOfClass:[NSTextField class]] ) || ( [firstResponder isKindOfClass:[NSTextView class]] ) )
					[super sendEvent:anEvent];
				else if ( ![self doKeyCommand:anEvent] )
					[super sendEvent:anEvent];
			}
			else if ( ![self doKeyCommand:anEvent] )
				[super sendEvent:anEvent];
		}
	}
	else
		[super sendEvent:anEvent];
}

- (void)registerKeyCommand:(NSString*)key modifiers:(unsigned int)modifiers forTarget:(id)target selector:(SEL)selector
{
	MIKeyCommand *keyCommand = [[[MIKeyCommand alloc] initWithKey:key modifiers:modifiers forTarget:target selector:selector] autorelease];
	[fKeyCommands addObject: keyCommand];
}

- (void)unregisterKeyCommandsForTarget:(id)target
{
	signed x ;
	MIKeyCommand *keyCommand;

	for( x = [fKeyCommands count] - 1 ; x >= 0  ; x-- ) {
		keyCommand = [fKeyCommands objectAtIndex:x];
		if ( [keyCommand target] == target ) {
			[fKeyCommands removeObjectAtIndex:x];
		}
	}
}

- (void)unregisterKeyCommand:(NSString*)key modifiers:(unsigned int)modifiers forTarget:(id)target
{
	signed x ;
	MIKeyCommand *keyCommand;

	for( x = [fKeyCommands count] - 1 ; x >= 0  ; x-- ) {
		keyCommand = [fKeyCommands objectAtIndex:x];
		if ( [keyCommand target] == target ) {
			if ( [[keyCommand key] isEqualToString:key] ) {
				if ( [keyCommand modifiers] == modifiers ) {
					[fKeyCommands removeObjectAtIndex:x];
				}
			}
		}
	}
}

// The two following methods are here to work around a Cocoa weirdness : when a document doesn't use the NSUndoManager architecture, it doesn't
// receive the undo: and redo: messages, as it is not in the responder chain. (NSWindow is the class that forwards the undo: and redo: messages
// when the document makes use of the NSUndoManager)

// to enable undo: and redo: for the document
- (id)targetForAction:(SEL)anAction to:(id)aTarget from:(id)sender {
	id target = [super targetForAction:anAction to:aTarget from:sender];
	if ( ( anAction == @selector(undo:) ) || ( anAction == @selector(redo:) ) ) {
		BOOL flag;
		if ( [[[NSApp keyWindow] windowController] respondsToSelector:@selector(ignoresUndoRedoActions)] ) {
			id controller = [[NSApp keyWindow] windowController];
			NSMethodSignature *signature = [controller methodSignatureForSelector:@selector(ignoresUndoRedoActions)];
			NSInvocation *invocation = [NSInvocation invocationWithMethodSignature:signature];
			[invocation setSelector:@selector(ignoresUndoRedoActions)];
			[invocation setTarget:controller];
			[invocation invoke];
			[invocation getReturnValue:&flag];
		}
		if ( ( target == nil ) || ( flag ) || ( ![target respondsToSelector:anAction] ) || ( ( target == [NSApp mainWindow] ) && ( [[[NSApp mainWindow] windowController] isKindOfClass:NSClassFromString(@"MIMainViewController")] ) ) ) {
			id document = [[NSDocumentController sharedDocumentController] currentDocument];
			if ( document ) return document;
		}
	}
	return target;
}

// for the document to perform undo: and redo: 
- (BOOL)sendAction:(SEL)anAction to:(id)aTarget from:(id)sender {
	if ( ( anAction == @selector(undo:) ) || ( anAction == @selector(redo:) ) ) {
		if ( aTarget == nil ) {
			id target = [self targetForAction:anAction to:aTarget from:sender];
			if ( target == [[NSDocumentController sharedDocumentController] currentDocument] ) {
				return [super sendAction:anAction to:target from:sender];
			}
		}
	}
	return [super sendAction:anAction to:aTarget from:sender];
}


/*- (void)logMenus {
	[self logMenu:[NSApp mainMenu]];
}

- (void)logMenu:(NSMenu*)menu {
	NSEnumerator *enumerator = [[menu itemArray] objectEnumerator];
	id menuItem;

	while (( menuItem = [enumerator nextObject] )) {
		NSLog( @"%@ - %@", [menuItem title], [menuItem target] );
		if ( [menuItem hasSubmenu] ) [self logMenu:[menuItem submenu]];
	}
}*/

@end

@implementation MIKeyCommand

- (id)initWithKey:(NSString*)key modifiers:(unsigned int)modifiers forTarget:(id)target selector:(SEL)selector {
	if (( self = [super init] )) {
		fKeyCommand = [key copy];
		fModifiers = modifiers;
		fTarget = target;
		fSelector = selector;
	}
	return self;
}

- (void)dealloc {
	[fKeyCommand release];
	[super dealloc];
}

- (NSString*)key {
	return fKeyCommand;
}

- (unsigned int)modifiers {
	return fModifiers;
}

- (id)target {
	return fTarget;
}

- (SEL)selector {
	return fSelector;
}

@end

@implementation NSEvent (MTGCarbonModifiers)

+ (unsigned int)modifiers {
	UInt32 modifiers = GetCurrentKeyModifiers();
	unsigned int cocoaModifiers = 0;
	if ( modifiers & cmdKey ) cocoaModifiers |= NSCommandKeyMask;
	if ( modifiers & shiftKey ) cocoaModifiers |= NSShiftKeyMask;
	if ( modifiers & optionKey ) cocoaModifiers |= NSAlternateKeyMask;
	if ( modifiers & controlKey ) cocoaModifiers |= NSControlKeyMask;
	if ( modifiers & alphaLock ) cocoaModifiers |= NSAlphaShiftKeyMask;
	return cocoaModifiers;
}

@end

@implementation NSScreen (MIScreenAdditions)

+ (NSScreen*)menubarScreen {
	NSEnumerator* enumerator = [[self screens] objectEnumerator];
	NSScreen *screen;

	while (( screen = [enumerator nextObject] )) {
		if ( !NSEqualRects( [screen frame], [screen visibleFrame] ) ) {
			return screen;
		}
	}
	return nil;
}

@end

@implementation NSView (MIViewAdditions)

- (NSView*)viewOfClass:(Class)classToFind {
	NSEnumerator *enumerator = [[self subviews] objectEnumerator];
	NSView *view;

	while (( view = [enumerator nextObject] )) {
		if ( [view isKindOfClass:classToFind] ) return view;
	}

	enumerator = [[self subviews] objectEnumerator];
	while (( view = [enumerator nextObject] )) {
		view = [view viewOfClass:classToFind];
		if ( view ) return view;
	}

	return nil;
}
@end

@implementation NSWindow (MIWindowAdditions)

- (void)centerOnMenubarScreen {
	NSRect screen = [[NSScreen menubarScreen] frame];
	NSRect frame = [self frame];
	NSRect newFrame;

	newFrame = NSMakeRect( NSMidX(screen) - ( NSWidth(frame) / 2 ), NSMidY(screen) - ( NSHeight(frame) / 2 ), NSWidth(frame), NSHeight(frame) );
	[self setFrame:newFrame display:[self isVisible]];
}
@end




@implementation NSString (BaseConversionAddition)

- (int)intValueFromStringExpressedInBase:(short)base {
	SInt64 value;
	ZzString zzString(self);

	if ( ZzConvertNumberStringToDecimalNumber( &value, zzString, base ) ) return value;
	else return 0;
}

+ (NSString*)stringExpressedInBase:(short)base FromInt:(int)integer {
	ZzString zzString = ZzConvertDecimalNumberToNumberString(integer, base);
	return zzString.getNSString();
}

@end

@implementation NSString (PathAddition)

- (NSString*)mountPoint {
	NSArray *pathComponents = [self pathComponents];
	if ( ( [pathComponents count] > 2 ) && ( [[pathComponents objectAtIndex:1] isEqualToString:@"Volumes"] ) ) {
		return [NSString pathWithComponents:[NSArray arrayWithObjects:[pathComponents objectAtIndex:0], [pathComponents objectAtIndex:1], [pathComponents objectAtIndex:2], nil]];
	} else return @"/";
}

@end

@implementation NSColorPanel (MIAdditions)

- (void)setFloatAboveModalWindows:(BOOL)flag {
	if ( flag ) [self setLevel:NSModalPanelWindowLevel + 1];
	else [self setLevel:NSFloatingWindowLevel];
}

@end

/*
@implementation MIDelayedInvocation

+ (void)invoqueSelectorWithDelay:(SEL)selector target:(id)target object:(id)object {
	[[[self alloc] initWithSelector:selector target:target object:object] autorelease];
}

- (id)initWithSelector:(SEL)selector target:(id)target object:(id)object {
	if (( self = [super init] )) {
		fSelector = selector;
		fTarget = [target retain];
		fObject = [object retain];
	}
	return self;
}

- (void)dealloc {
	[NSApp sendAction:fSelector to:fTarget from:fObject];
	[fTarget release];
	[fObject release];
	[super dealloc];
}

@end
*/

void MIRunErrorPanel( NSString *title, NSString *nonTechnicalReason, NSString *errorNumber, NSString *technicalReason ) {
	MIErrorPanel* panel = [[MIErrorPanel alloc] init];
	[panel setTitle:title];
	[panel setNonTechnicalReason:nonTechnicalReason];
	[panel setErrorNumber:errorNumber];
	[panel setTechnicalReason:technicalReason];
	[panel runModal];
	[panel release];
}

