/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
/*
	The MIApplicationController is centralization of all the windows in the program.
	
	It holds an array of available windowControllers (which control windows)
	it holds pointers to the global instances of palettes such as the transport and
	the overview (although for these two there can be instances in the windows themselves
	
	it is also the prefered target for the menus.
	
	there is a global static instance of the facadecontroller reached through the
	mainController method.
*/

#import "MontageInterface.h"

@class MIMainViewController;
@class MITransportController;
@class MIOverviewController;
@class MIPlayStopPauseController;
//@class MIToolBoxController;
@class MIObject;
@class MIProgressDlgController;
@class MIChooseStackingDlgController;
@class MIMontageDocument;
@class MIVideoWindow;
@class MIFullScreenVideoWindow;
@class MITimePaletteController;
@class MIMixerController;
@class MIPickScreenController;
//@class MIRecordPreviewController;

class ZzSplitTask;
class MIObjCProxyListener;
// when TWO_THREADS_NOTIFICATIONS is defined, we don't use an extra thread $$$$$$obsolete
// and a notification queue.

//#define TWO_THREADS_NOTIFICATIONS

//#ifdef TWO_THREADS_NOTIFICATIONS
@protocol MINotificationListener
- (void)notificationReceived:(UInt32)code param:(void*)param;
@end
//#endif //#ifdef TWO_THREADS_NOTIFICATIONS

//@protocol MIApplicationControllerListener
/*!
	@function notificationReceived
	A notification has been sent by someone. This function
	will then forward the notification to the appropriate
	receiver (for example, a BOP thread may ask a MIDocumentController
	to refresh all views for a document because an audio edit command
	is done. Please note the param is a polymorphic argument. depending
	on the notification code, it may need to be
	- freed as a malloc'd zone (as in "free(zone)")
	- deleted as a cpp object (as in "delete cppObject")
	- deleted as an array of cpp objs (as in "delete [] cppArray")
	- released as an objc object (as in [obj release])
	- left alone (if it is passed as a 32 bit value such as a float, an SInt32, a pointer to a clip)
	
	This method is part of a protocol because this speeds up message passing.
*/
//#ifndef TWO_THREADS_NOTIFICATIONS
//- (void)notificationReceived:(NSArray*)packedParams;
//#else //#ifndef TWO_THREADS_NOTIFICATIONS
//- (void)forwardNotificationTo:(id<MINotificationListener>)object code:(int)code param:(unsigned int)param;
//#endif //#ifndef TWO_THREADS_NOTIFICATIONS
//@end

extern NSString *MTGMovieStoppedPlayingNotification;
extern NSString *MTGMovieStartedPlayingNotification;

extern GWorldPtr gGWorld;

@interface MIApplicationController : NSObject /*<MIApplicationControllerListener>*/
{   
@public         
	// recording window
	//MIRecordPreviewController		*fRecordPreviewController;
	
    // the menu item that has to be replenished with the data from the media engine
    IBOutlet	id					fProcessorMenuItem;

	IBOutlet	id					fVideoProcessorMenuItem;
	
	// the menu that is used when right-clicking on an item
	IBOutlet	id					fLocalEditMenu;

	IBOutlet	id					fFontMenu;

	IBOutlet	id					fFontSizeMenu;
	
	IBOutlet	id					fVirtualOutputMenu1;
	IBOutlet	id					fVirtualOutputMenu2;
	    
    // the timer to update the play bars
    NSTimer							*fPlayBarTimer;
	
	/*! @var fProgressDlgController The progress bar dialog controller. */
	MIProgressDlgController			*fProgressDlgController;

	/*! @var fChooseStackingDlgController The stacking method chooser dialog controller. */
	MIChooseStackingDlgController*	fChooseStackingDlgController;
	
//#ifndef TWO_THREADS_NOTIFICATIONS
//	/*! @var fNotificationThread the one and only thread that's used to forward NSNotification msgs to
//		the main app kit msg loop */
//	pthread_t						fNotificationThread;
//	
//	/*! @var fNotificationQueueID The queue used by the notif thread */
//	MPQueueID						fNotificationQueueID;
//#else //#ifndef TWO_THREADS_NOTIFICATIONS
//	/*! @var The notification proxy that represents the application controller singleton */
//	id 								fProxySelf;
//#endif //#ifndef TWO_THREADS_NOTIFICATIONS
	
	/*! @var fMainViewRedrawDisabled Debug variable that allows to bypass view redraw for DSP CPU timing */
	BOOL							fMainViewRedrawDisabled;

	/*! @var fClipBoard */
	MIObject*						fClipBoard;

	/*! @var fPasteboardTypes cache variable */
	NSArray							*fPasteboardTypes;

	/*! @var fFileTypes cache variable */
	NSArray							*fFileTypes;

	/*! @var fFileExtensions cache variable */
	NSArray							*fFileExtensions;

	/*! @var fQuickTimeFileTypes cache variable */
	NSArray							*fQuickTimeFileTypes;

	/*! @var fQuickTimeFileExtensions cache variable */
	NSArray							*fQuickTimeFileExtensions;

	/*! @var fFileTypesAndExtensions cache variable */
	NSArray							*fFileTypesAndExtensions;        

	BOOL							fFollowPlayBar;

	/*! @var fProxyListener The listener that subscribes to notifications from the MEApplicationServices singleton */
	MIObjCProxyListener*			fProxyListener;
}

// returns the main (and supposedly only) controller 
+ (MIApplicationController*)mainController;
+ (MIApplicationController*)sharedApplicationController;

+ (void)addDefaultValuesToDictionary:(NSMutableDictionary*)defaultValues;

- (MIObject*) clipBoard;
- (NSString*) tempDirectory;

// add and remove main view controllers from the array
- (void)addMainViewController:(MIMainViewController*)newMainViewController;
- (void)removeMainViewController:(MIMainViewController*)oldMainViewController;
// returns the array of main view controllers, one main view controller per active window
- (NSArray*)windowControllers;

- (void)clearAllSelections;

- (MIPlayStopPauseController*)playStopPauseController;
//- (MIToolBoxController*)toolBoxController;
- (MITimePaletteController*)timePaletteController;

- (void)registerPanelToLayoutManagers;

- (void)updatePlayMenuItem;

// sets the current main view controller
- (void)setCurrentMainViewController:(MIMainViewController*)theMainViewController;

// retrieves the current main view controller
- (MIMainViewController*)currentMainViewController;

- (NSMenu*)localEditMenu;

// recreate the processor menu with the data from the media engine
- (void)recreateProcessorMenu;
- (void)recreateVirtualOutputMenus;

- (void)addStuffToMenus;

// create the timer to update the play bars, should be created only once.
- (NSTimer*)createPlayBarTimer;

- (void)createProcessor:(id)sender;

// function called by the timer to update the play bars in the opened views.
- (void)playBarTimerTicked:(NSTimer*)theTimer;

// displays a sheet that is made globally modal and executes the split task until either
// it is finished or cancel is pressed. $$$TODO make it modal only wrt the controller's document
// window, not all the app windows.
- (BOOL)executeAndDeleteSplitTask:(ZzSplitTask*)splitTask forMainViewController:(MIMainViewController*)controller;

// same as above but without dialog box. not possible to cancel the split task then
- (BOOL)blindlyExecuteAndDeleteSplitTask:(ZzSplitTask*)splitTask;

/*! @function displayChooseStackingMethodForMainViewController: Displays the sheet that allows the user
	to choose how object will stack when opening several at once */
- (SInt32)displayChooseStackingMethodForMainViewController:(MIMainViewController*)controller;

// create a clip from a file name, if NSZeroPoint is passed as an argument, it creates them
// in the lower left corner of the visible window
// also, since creating clips usually involves long operations, the resulting ZzSplitTask is 
// stored in the splitTask pointer address
//- (MIObject*)createClipFromFileName:(NSString*)fileName atPos:(NSPoint)aPoint splitTask:(ZzSplitTask**)splitTask;

/*! @function displayBOPsProgress Displays a set of progress dlg boxes for an array of BOPs then returns when they're all done */
//- (void)displayBOPsProgress:(NSMutableArray*)bopTaskArray;

/*! 
	@function displayBOPProgress Displays a progress dlg box for a BOP then returns when the BOP's done
*/
//- (void)displayBOPProgress:(BOPTask*)bopTask;

/*! @function deleteBOPs deletes the cpp bop objects contained in the array */
//- (void)deleteBOPs:(NSMutableArray*)bopTaskArray;

/*! @function modelChangedForDocument description forthcoming ;-) */
- (void)modelChangedForDocument:(MIMontageDocument*)doc;

/*!
	@function createNotificationResources
	
	Creates the resources needed to be able to receive  notification messages from various threads and forward
	them to the appropriate object, in the AppKit thread context
*/
//- (void)createNotificationResources;

/*!
	@function sendNotification
	This function is used to send a notification to an object. Its
	notificationReceived message will be called in the main AppKit thread
	context, just as if it were a mouse or timer message
*/
- (void)sendNotification:(UInt32)code toObject:(id<MINotificationListener>)obj withParam:(void*)param;

- (void)notificationReceived:(UInt32)code param:(void*)param;

//#ifndef TWO_THREADS_NOTIFICATIONS
///*! @function notificationQueueID getter */
//- (MPQueueID)notificationQueueID;
//
///*! @function filterQueue Removes the queue notifications that have "code" and "param". The object doesn't
//matter (it's always the MIMontageDocument right now $$$TODO warning (sure ?)) */
//- (void)filterQueue:(UInt32)code param:(void*)param;
//
//#else //TWO_THREADS_NOTIFICATIONS
//
//#endif //TWO_THREADS_NOTIFICATIONS

///*! @function filterQueue Removes the queue notifications that are adressed to "object" .*/
//- (void)purgeQueueMessagesFor:(id)object;


/*! @function emptyMethod Empty methods detached in a new thread to make sure
    various multithreaded flags are set for the application. See awakeFromNib impl */
-(void)emptyMethod:(id)object;

-(void)redrawAllMainWindows;
- (void)deferAllRedraws;
- (void)flushAllDeferedRedrawRects;


///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////

// menu actions, also called by the buttons

///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////

- (IBAction)showAboutPanel:(id)sender;

// file menu
- (IBAction)showInformation:(id)sender;
- (IBAction)exportClip:(id)sender;
- (IBAction)exportRawAudio:(id)sender;
- (IBAction)pageSetup:(id)sender;
- (IBAction)print:(id)sender;
- (IBAction)makeDocumentSelfContained:(id)sender;


// Edit menu
- (IBAction)delete:(id)sender;
- (IBAction)deleteContainer:(id)sender;
- (IBAction)delBenja:(id)sender;
- (IBAction)cut:(id)sender;
- (IBAction)copy:(id)sender;
- (IBAction)paste:(id)sender;
- (IBAction)duplicate:(id)sender;
- (IBAction)rename:(id)sender;
- (IBAction)duplicateMany:(id)sender;
- (IBAction)trim:(id)sender;
- (IBAction)crop:(id)sender;
- (IBAction)reverse:(id)sender;
- (IBAction)dirtyResampler:(id)sender;
- (IBAction)resample:(id)sender;
- (IBAction)silence:(id)sender;
- (IBAction)split:(id)sender;
- (IBAction)changeVirtualAudioOutput:(id)sender;
- (IBAction)find:(id)sender;
- (IBAction)findAgain:(id)sender;

- (IBAction)swapChannels:(id)sender;
- (IBAction)adjustDCOffset:(id)sender;
- (IBAction)invertPhase:(id)sender;
- (IBAction)insertSilence:(id)sender;

- (IBAction)advancedFadeIn:(id)sender;
- (IBAction)advancedFadeOut:(id)sender;

- (IBAction)merge:(id)sender;
- (IBAction)clear:(id)sender;
- (IBAction)selectAll:(id)sender;



 // Record/Playback
- (IBAction)moveToDocumentStart:(id)sender;
- (IBAction)moveToDocumentEnd:(id)sender;
- (IBAction)startPlaying:(id)sender;
- (IBAction)pausePlaying:(id)sender;
- (IBAction)stopPlaying:(id)sender;
- (IBAction)startRecord:(id)sender;
- (IBAction)automaticSceneDetection:(id)sender;
- (IBAction)followPlaybar:(id)sender;
- (IBAction)toggleLoop:(id)sender;
- (IBAction)solo:(id)sender;
- (IBAction)mute:(id)sender;
- (IBAction)allOn:(id)sender;

- (IBAction)showDVCapture:(id)sender;
- (IBAction)showAudioRecording:(id)sender;


- (IBAction)gotoDocEnd:(id)sender;
- (IBAction)gotoDocStart:(id)sender;
- (IBAction)goToLeftLocator:(id)sender;
- (IBAction)goToRightLocator:(id)sender;



	// create menu
- (IBAction)renderClip:(id)sender;
- (IBAction)pictFromMovieView:(id)sender;
- (IBAction)createNewContainer:(id)sender;
- (IBAction)processorCustomize:(id)sender;
- (IBAction)normalize:(id)sender;
- (IBAction)changeGain:(id)sender;
- (IBAction)timeStretch:(id)sender;

- (IBAction)createAudioClip:(id)sender;
- (IBAction)createVideoClip:(id)sender;
- (IBAction)createTextClip:(id)sender;
- (IBAction)createMidiClip:(id)sender;
- (IBAction)createMarker:(id)sender;
- (IBAction)createAutoLoopPoints:(id)sender;


// convert submenu of the create menu
- (IBAction)convertSampleRate:(id)sender;
- (IBAction)convertSampleWidth:(id)sender;
- (IBAction)slitToInterleave:(id)sender;
- (IBAction)interleaveToSlit:(id)sender;



// view menu
- (IBAction)showPreferences:(id)sender;
- (IBAction)showGrid:(id)sender;
- (IBAction)stickToGrid:(id)sender;
- (IBAction)setGrid:(id)sender;
- (IBAction)viewControls:(id)sender;

// zoom in (or out) vertically and horizontally
- (IBAction)vZoomIn:(id)sender;
- (IBAction)vZoomOut:(id)sender;
- (IBAction)hZoomIn:(id)sender;
- (IBAction)hZoomOut:(id)sender;

// create or show panels
- (IBAction)showGeneralGridVerticalLines:(id)sender;
- (IBAction)showGeneralGridHorizontalLines:(id)sender;
- (IBAction)showObjectGridVerticalLines:(id)sender;
- (IBAction)showObjectGridHorizontalLines:(id)sender;
- (IBAction)snapToGeneralGrid:(id)sender;
- (IBAction)snapToObjectsGrid:(id)sender;

// arrange menu
- (IBAction)shrinkToFit:(id)sender;
- (IBAction)collapse:(id)sender;
- (IBAction)expand:(id)sender;
- (IBAction)chain:(id)sender;
- (IBAction)stack:(id)sender;
- (IBAction)paint:(id)sender;
- (IBAction)alignLeft:(id)sender;
- (IBAction)alignRight:(id)sender;
- (IBAction)lock:(id)sender;
- (IBAction)nudgeLeft:(id)sender;
- (IBAction)nudgeRight:(id)sender;
- (IBAction)unlock:(id)sender;

// processor menu
- (IBAction)fadeIn:(id)sender;
- (IBAction)fadeOut:(id)sender;
- (IBAction)customFade:(id)sender;
- (IBAction)createVideoEffect:(id)sender;

// debug actions
- (IBAction)debugAudioRecorder:(id)sender;
- (IBAction)toggleCursorRectsUpdate:(id)sender;
- (IBAction)toggleUseTitles:(id)sender;
- (IBAction)toggleRulerDrawing:(id)sender;
- (IBAction)toggleLazyOverviewUpdate:(id)sender;
- (IBAction)audioClipInspector:(id)sender;
- (IBAction)disableMainViewRedraw:(id)sender;
- (IBAction)showRedrawRects:(id)sender;
- (IBAction)displayVerboseConfigDlg:(id)sender;
- (IBAction)benjaDebugActionOne:(id)sender;
- (IBAction)testGenSineWave:(id)sender;
- (IBAction)configureTransition:(id)sender;
- (IBAction)benjaDebugActionTwo:(id)sender;
- (IBAction)benjaDebugActionThree:(id)sender;
- (IBAction)toggleRedrawRects:(id)sender;
- (IBAction)forceAudioTrackUpdate:(id)sender;
- (IBAction)recreateProcessorMenuAction:(id)sender;
- (IBAction)djSwitch:(id)sender;
- (IBAction)setLocatorsFromSelection:(id)sender;
- (IBAction)freezeSelection:(id)sender;
- (IBAction)unfreezeSelection:(id)sender;
- (IBAction)loopsmooth:(id)sender;

// debug stuff misc

/*! @function saveVerboseSettings Saves the verbose settings (when the user overrides them with the provided dlg box) */
-(void)saveVerboseSettings;

/*! @function addVerboseDefaultsToDictionary Saves the default verbose settings (used when the user does not override them */
+ (void)addVerboseDefaultsToDictionary:(NSMutableDictionary*)appDefaults;

/*! @function restoreVerboseSettings Restores the verbose settings from the persistent preferences database */
-(void)restoreVerboseSettings;

- (BOOL)mainViewRedrawDisabled;

// returns an array of NSStrings of file types QuickTime can import
- (NSArray*)quickTimeFileTypes;
// returns an array of NSStrings of file extensions QuickTime can import
- (NSArray*)quickTimeFileExtensions;

// returns an array of NSStrings of file extensions HyperEngine-AV can import (including QT file ext.)
- (NSArray*)fileExtensions;
// returns an array of NSStrings of file types HyperEngine-AV can import (including QT file types)
- (NSArray*)fileTypes;
// returns an array of NSStrings of file types and Extensions HyperEngine-AV can import (including QT file types and Extentions)
- (NSArray*)fileTypesAndExtensions;

// returns an array of NSStrings of pasteboard types HyperEngine-AV can import (including QT types)
- (NSArray*)pasteboardTypes;

- (void)beep;
- (void)conditionallyEnableOrDisableDemoMenus;

- (MIObjectRep*)highestObjectRepFromArray:(NSArray*)array;

- (void)setShouldFollowPlaybar:(BOOL)flag;
- (BOOL)shouldFollowPlaybar;
- (BOOL)isLooping;

- (IBAction)close:(id)sender;
- (IBAction)showHelp:(id)sender;

@end

@interface MIRectObject : NSObject
{
	MIRect fRect;
}
+ (id)rectObjectWithRect:(MIRect)rect;
- (id)initWithRect:(MIRect)rect;
- (MIRect)rect;
@end

@interface MIQuickDrawView : NSQuickDrawView {
	GWorldPtr fGWorld;
	NSTimer *fTimer;
	NSTimer *fModalTimer;
}

- (void)startUpdating;
- (void)stopUpdating;
	
@end
