/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
#ifndef LibraryToggle_h
#include "LibraryToggle.h"
#endif //LibraryToggle_h

#import "MIApplication.h"
#import "MIApplicationController.h"
#import "MIMainViewController.h"
#import "MIMontageDocument.h"
#import "MIPaletteController.h"
#import "MITime.h"
#import "MIObject.h"
#import "MIObjectRep.h"
#import "MITopLevelContainerRep.h"
#import "MIImageController.h"
#import "MIPlayStopPauseController.h"
#import "MITimePaletteController.h"
#import "MIOverviewController.h"
#import "MIRuler.h"
#import "MIProgressDlgController.h"
#import "MIPaletteLayoutManager.h"
#import "MIWSSelection.h"
#import "MIMixerController.h"
#import "MIVideoClipRep.h"
#import "MIVerboseConfigDlgController.h"
#import "MICreateAudioProcessorCommand.h"
#import "MICreateWSObjsFromFilesCommand.h"
#import "MICreateContainerCommand.h"
#import "MIPlayStopPauseController.h"
#import "MIViewerController.h"
#import "MITextController.h"
#import "MIPreferencesController.h"
#import "MIPreferences.h"
#import "MIInspectorController.h"
#import "MIStartingPoint.h"
#import "MIChooseStackingDlgController.h"
#import "MIExportSettingsController.h"
#import "MIFindDlgController.h"
#import "MIMoveCommand.h"
#import "MIChangeVAOCommand.h"
#import "MEApplicationServices.h"
#import "MISplash.h"
#import "MenuTags.h"
#import "MontageVerbose.h"
#import "MIExportRAWAudioController.h"
#import "MIExportPanel.h"
#import "MICamera.h"

#ifndef MEMTGAAudioFile_h
#include "MEMTGAAudioFile.h"
#endif //MEMTGAAudioFile_h

#ifndef CVideoOutput_h
#include "CVideoOutput.h"
#endif //CVideoOutput_h

#import "MILibraryElement.h"

#if USE_LIBRARY
#import "MILibraryController.h"
#endif //USE_LIBRARY

#import "MIDVCaptureController.h"
#import "MIAudioRecController.h"

#ifndef Zz_PathHelpers_h
#include "Zz_PathHelpers.h"
#endif //Zz_PathHelpers_h

#include <sys/syslimits.h>

#ifndef MIProgressBarDlgManager_h
#include "MIProgressBarDlgManager.h"
#endif //MIProgressBarDlgManager_h

#ifndef MEWSObjAudioPlayer_h
#include "MEWSObjAudioPlayer.h"
#endif //MEWSObjAudioPlayer_h

#ifndef ME_AudioExportFunctions_h
#include "ME_AudioExportFunctions.h"
#endif //ME_AudioExportFunctions_h

#ifdef MTG_USE_LICENSER_KIT
#if defined( MTG_LIGHT_VERSION ) || defined( MTG_FULL_VERSION )
#import "CSLicensing.h"
#import "MainApp.h"
#endif
#endif //#ifdef MTG_USE_LICENSER_KIT

#ifndef MontageArchiveHandler_h
#include "MontageArchiveHandler.h"
#endif //MontageArchiveHandler_h

#ifndef TARPIHost_h
#include "TARPIHost.h"
#endif //TARPIHost_h

#ifndef ARPI_h
#include "ARPI.h"
#endif //ARPI_h

#ifndef ZzSplitTask_h
#include "ZzSplitTask.h"
#endif //ZzSplitTask_h

#ifndef TArmeMasterMovie_h
#include "TArmeMasterMovie.h"
#endif //TArmeMasterMovie_h

#ifndef TVideoEffect_h
#include "TVideoEffect.h"
#endif //TVideoEffect_h

#ifndef TArmeTextClip_h
#include "TArmeTextClip.h"
#endif //TArmeTextClip_h

#ifndef TArmeContainer_h
#include "TArmeContainer.h"
#endif //TArmeContainer_h

#ifndef MovieHelper_h
#include "MovieHelper.h"
#endif //MovieHelper_h

#ifndef TArmeMediaEngine_h
#include "TArmeMediaEngine.h"
#endif //TArmeMediaEngine_h

#ifndef VArmeMediaFileHandler_h
#include "VArmeMediaFileHandler.h"
#endif //VArmeMediaFileHandler_h

#ifndef ZzHelpers_h
#include "ZzHelpers.h"
#endif //ZzHelpers_h

#ifndef ZzVerbose_h
//#include "ZzVerbose.h"
#endif //ZzVerbose_h

#import "MIAudioClipRep.h"

#ifndef MEAudioDeviceDesc_h
#include "MEAudioDeviceDesc.h"
#endif //MEAudioDeviceDesc_h

#ifndef ArmeNotifications_h
#include "ArmeNotifications.h"
#endif //ArmeNotifications_h

#ifndef ZzException_h
#include "ZzException.h"
#endif //ZzException_h

// for AudioDeviceID
#include <CoreAudio/CoreAudio.h>

// for mach_thread_self()
#include <mach/mach_init.h>
#include <mach/task_policy.h>
#include <mach/thread_act.h>
#include <mach/thread_policy.h>

#import <QuickTime/Quicktime.h>

#ifndef MIObjCProxyListener_h
#include "MIObjCProxyListener.h"
#endif //MIObjCProxyListener_h

#ifndef MEApplicationServices_h
#include "MEApplicationServices.h"
#endif //MEApplicationServices_h

#import <Foundation/NSDebug.h>

#undef VER_DEF_TOPIC
#define VER_DEF_TOPIC MIApplicationController

//performSelectorOnMainThread

MIApplicationController 				*_mainApplicationController;

extern BOOL gDeferRedrawsUntilCommandEnd;

GWorldPtr gGWorld;

NSString *MTGMovieStoppedPlayingNotification = @"Movie Stopped Playing Notification";
NSString *MTGMovieStartedPlayingNotification = @"Movie Started Playing Notification";


@implementation MIRectObject

+ (id)rectObjectWithRect:(MIRect)rect
{
	return [[[self alloc] initWithRect:rect] autorelease];
}

- (id)initWithRect:(MIRect)rect
{
	if (( self = [super init] ))
	{
		fRect = rect;
	}
	return self;
}

- (MIRect)rect
{
	return fRect;
}

@end

@implementation MIApplicationController

+ (void)initialize
{
    NSMutableDictionary *defaultValues = [NSMutableDictionary dictionary];
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    
    // Every Class that has defaults to register should be called here
    [self addDefaultValuesToDictionary:defaultValues];
    [MIOverviewController addDefaultValuesToDictionary:defaultValues];
    [MIPlayStopPauseController addDefaultValuesToDictionary:defaultValues];
    [MIViewerController addDefaultValuesToDictionary:defaultValues];
    //[MIToolBoxController addDefaultValuesToDictionary:defaultValues];
    [MITimePaletteController addDefaultValuesToDictionary:defaultValues];
    [MIMixerController addDefaultValuesToDictionary:defaultValues];
    [MIPreferences addDefaultValuesToDictionary:defaultValues];
    [MIInspectorController addDefaultValuesToDictionary:defaultValues];

    [defaults registerDefaults:defaultValues];
    [defaults synchronize];
}

+ (void)addDefaultValuesToDictionary:(NSMutableDictionary*)defaultValues
{
    [self addVerboseDefaultsToDictionary:defaultValues];
}

-(void)awakeFromNib
{

#ifdef DEBUG
	[NSAutoreleasePool enableFreedObjectCheck:YES];
	NSZombieEnabled = YES;
	NSDeallocateZombies = NO;
#endif //DEBUG

	// init debug stuff
	fMainViewRedrawDisabled = NO;
	fFollowPlayBar = YES;

	if ( ![self respondsToSelector:@selector(performSelectorOnMainThread:withObject:waitUntilDone:)] ) {
		NSRunAlertPanel(@"The system software version is older than 10.2", @"Sorry, HyperEngine-AV requires MacOS X version 10.2 (Jaguar) or later.", @"Quit", nil, nil);
		exit(0);
	}

	// now restore user-provided defaults settings
	[self restoreVerboseSettings];
	[[MIPreferences sharedPrefs] restoreFromUserSettings];
    
    // now check if HAV has already run once wih these preferences
    // if not, let's display a message reminding the user to enter his/her serial number
#ifndef MTG_DEMO_VERSION
/*    if(![[MIPreferences sharedPrefs] isSerialized]) {
		NSRunAlertPanel(@"Information", @"Thank you for choosing HyperEngine-AV.\r\nTo register this application, enter your serial numbers in the Licenses tab of the Preferences panel.", @"OK", nil, nil);
    }*/
#endif //#ifndef MTG_DEMO_VERSION
    		
	ZzVerbose_SetOutputFilePath("/tmp/MontageLogFile.txt");

	// initialize QuickTime.
	EnterMovies();
	
	fChooseStackingDlgController = nil;

    // set the main application controller, global var reachable through
    // [MIApplicationController sharedApplicationController]
	if ( !_mainApplicationController )
		_mainApplicationController = self;
		
	// this is a dummy thread creation (the selector returns immediatly) whose sole
	// putpose is to send NSWillBecomeMultiThreadedNotification to the
	// default notification center, which sets some flags for the app (it's said
	// that doing this allows for multithreaded posix calls)
	[NSThread detachNewThreadSelector:@selector(emptyMethod:) toTarget:self withObject:nil];
	
	// see header file 
//	[self createNotificationResources];
	
	fPlayBarTimer = nil;
	fProgressDlgController = nil;
	
    // create the image controller
	mainImageController = [[MIImageController alloc] init];
    
    // load the (default) images
    [mainImageController loadImages];
	
#if 0
	// This code has been moved to the method applicationWillFinishLaunching: because plug-ins should be loaded after the licenser Kit
	
    // create an MEApplicationServices instance linking Arme to us
	MEApplicationServices* applicationServices = new MEApplicationServices(self);
	
	//- (NSString*)audioOutputDeviceStringCode;
	//- (NSString*)audioInputDeviceStringCode;
	//- (UInt32)audioOutputDeviceBufferNbFrames;

	// step 1 : we retrieve the output device string code and the number of output frames 
	// as defined in the prerefences
	NSString* outDevStringCode = [[MIPreferences sharedPrefs] audioOutputDeviceStringCode];
	UInt32 outNbFrames = [[MIPreferences sharedPrefs] audioOutputDeviceBufferNbFrames];
	
	// step 2 : translate the string code into a CoreAudio AudioDeviceID
	AudioDeviceID outDevID = MEAudioDeviceDesc::GetAudioDeviceIDFromStringCode((CFStringRef)outDevStringCode);
	bool problemOpeningChosenDevice = false;
	if(outDevID == 0x00000000) // this means the device hasn't been found
	{	
		problemOpeningChosenDevice = true;
		NSRunAlertPanel(@"Warning", @"The Audio Output Device chosen in the preferences cannot be opened. Please open the preferences panel to choose another device. HyperEngine-AV now uses the default output device.", @"OK", nil, nil);
		NSString* defaultOutDevStringCode = [NSString stringWithString:(NSString*)MEAudioDeviceDesc::GetDefaultOutputStringCode()]; // returns a code that is 'always use default device'
		[[MIPreferences sharedPrefs] setAudioOutputDeviceStringCode:defaultOutDevStringCode];
		outDevID = MEAudioDeviceDesc::GetAudioDeviceIDFromStringCode((CFStringRef)defaultOutDevStringCode);
		ZzAssert(outDevID != 0x00000000);
	}
	UInt32 realOutNbFrames = outNbFrames;
	TArmeMediaEngine::Initialize(applicationServices,outDevID,&realOutNbFrames);

	// now this instance of the application controller needs to listen to notifications
	// posted by the MEApplicationServices instance
	fProxyListener = new MIObjCProxyListener(self);
	TArmeMediaEngine::ApplicationServices()->Subscribe(fProxyListener);	
		
	// if the chosen device was ok but if the frames set up couldn't have been set, let's display a message and
	// change the default value in the preferences
	if((realOutNbFrames != outNbFrames) && (!problemOpeningChosenDevice))
	{
		if(outNbFrames != 0)
		{
			NSRunAlertPanel(@"Warning", @"The number of audio output samples chosen in the preferences couldn't be used. Please open the preferences panel to choose another value. This setting has been reverted to a suitable value.", @"OK", nil, nil);
		}
		[[MIPreferences sharedPrefs] setAudioOutputDeviceBufferNbFrames:realOutNbFrames];
	}
			
	//create the clipboard
	fClipBoard = [[MIObject alloc] initWithArmeWSObj:(new TArmeContainer("clipboard"))];
	[self addStuffToMenus];
#endif	
	// $$$ TODO FIX THIS NAMM DEMO HACK
/*	
	[[fPlayStopPauseController window] setFrameOrigin:NSMakePoint(NSMaxX([[fCurrentMainViewController window] frame])+25, NSMinY([[fPlayStopPauseController window] frame]))];
	[[fOverviewController window] setFrameOrigin:NSMakePoint(NSMaxX([[fCurrentMainViewController window] frame])+25, (NSMinY([[fPlayStopPauseController window] frame]) - NSHeight([[fOverviewController window] frame]))-5)];
	[[fTimePaletteController window] setFrameOrigin:NSMakePoint(NSMaxX([[fCurrentMainViewController window] frame])+25, (NSMinY([[fOverviewController window] frame]) - NSHeight([[fTimePaletteController window] frame]))-5)];
	[fVideoWindow setFrameOrigin:NSMakePoint(NSMaxX([[fCurrentMainViewController window] frame]), (NSMinY([[fTimePaletteController window] frame]) - NSHeight([fVideoWindow frame]))-5)];
*/

    // initialize the timer and start it
    fPlayBarTimer = [self createPlayBarTimer];
 //   [fPlayBarTimer fire]; //optional, but better be sure.
	

	NSMenuItem *item = [NSApp searchForMenuItemWithTag:MTGDeleteMenuTag];
	[item setKeyEquivalent:[NSString stringWithFormat:@"%c",8]];
	[item setKeyEquivalentModifierMask:0];

	item = [NSApp searchForMenuItemWithTag:MTGDeleteContainerMenuTag];
	[item setKeyEquivalent:[NSString stringWithFormat:@"%c",8]];
	[item setKeyEquivalentModifierMask:NSCommandKeyMask];


#ifndef DEBUG
	item = [NSApp searchForMenuItemWithTag:MTGDebugMenuTag];
	[[NSApp mainMenu] removeItem:item];
#endif
	

#if !defined(__MWERKS__)
	[NSApp registerKeyCommand:@" " modifiers:0 forTarget:self selector:@selector(spaceKeyCommand:withModifiers:)];
	[NSApp registerKeyCommand:@"\r" modifiers:0 forTarget:self selector:@selector(returnKeyCommand:withModifiers:)];
	[NSApp registerKeyCommand:@"\177" modifiers:0 forTarget:self selector:@selector(backspaceKeyCommand:withModifiers:)];
	
	
///	NSLeftArrowFunctionKey @"\\uf702"
	// $$$HACK HACK HACK
	// Note by AM on 02-12-2003 : there was a 10485760| in front of all the modifiers.  I don't know why !! but nudge of selection was not possible then
	[NSApp registerKeyCommand:[NSString stringWithUTF8String:"\357\234\202"] modifiers:NSAlternateKeyMask forTarget:self selector:@selector(nudgeSelectionLeftSmall:withModifiers:)];
	[NSApp registerKeyCommand:[NSString stringWithUTF8String:"\357\234\203"] modifiers:NSAlternateKeyMask forTarget:self selector:@selector(nudgeSelectionRightSmall:withModifiers:)];
	[NSApp registerKeyCommand:[NSString stringWithUTF8String:"\357\234\202"] modifiers:NSAlternateKeyMask|NSShiftKeyMask forTarget:self selector:@selector(nudgeSelectionLeftMedium:withModifiers:)];
	[NSApp registerKeyCommand:[NSString stringWithUTF8String:"\357\234\203"] modifiers:NSAlternateKeyMask|NSShiftKeyMask forTarget:self selector:@selector(nudgeSelectionRightMedium:withModifiers:)];
	[NSApp registerKeyCommand:[NSString stringWithUTF8String:"\357\234\202"] modifiers:NSAlternateKeyMask|NSControlKeyMask forTarget:self selector:@selector(nudgeSelectionLeftLarge:withModifiers:)];
	[NSApp registerKeyCommand:[NSString stringWithUTF8String:"\357\234\203"] modifiers:NSAlternateKeyMask|NSControlKeyMask forTarget:self selector:@selector(nudgeSelectionRightLarge:withModifiers:)];

#endif //#if !defined(__MWERKS__)
    [self conditionallyEnableOrDisableDemoMenus];
}

- (void)conditionallyEnableOrDisableDemoMenus
{

}

- (void)dealloc
{
	// release Quicktime
	ExitMovies();
	[NSApp unregisterKeyCommandsForTarget:self];
	[fClipBoard release];
	[fPasteboardTypes release];
	[fFileExtensions release];
	[fFileTypes release];
	[fFileTypesAndExtensions release];
	[fQuickTimeFileExtensions release];
	[fQuickTimeFileTypes release];
	
	[super dealloc];
}

+(MIApplicationController*)mainController
{
        NSLog(@"MIApplicationController - mainController  is deprecated. Please use sharedApplicationController instead.");
    [[MIApplicationController sharedApplicationController] beep];
    return _mainApplicationController;
}

+ (MIApplicationController*)sharedApplicationController
{
    return _mainApplicationController;
}



- (void)setCurrentMainViewController:(MIMainViewController*)theMainViewController
{
    NSLog(@"MIApplicationController - setCurrentMainViewController:  is deprecated.");
    [[MIApplicationController sharedApplicationController] beep];
}

- (MIMainViewController*)currentMainViewController
{
        NSLog(@"MIApplicationController - currentMainViewController  is deprecated. Please use  [NSApp mainViewController] instead.");
    [[MIApplicationController sharedApplicationController] beep];
    return [NSApp mainViewController];
}

- (NSArray*)windowControllers
{
    NSLog(@"MIApplicationController - windowControllers  is deprecated. Please use  [[[NSDocumentController sharedDocumentController] currentDocument] windowControllers] instead.");
   [[MIApplicationController sharedApplicationController] beep];

    return [[[NSDocumentController sharedDocumentController] currentDocument] windowControllers];
}

- (void)clearAllSelections
{
	NSEnumerator *enumerator = [[[[NSDocumentController sharedDocumentController] currentDocument] windowControllers] objectEnumerator];
	MIMainViewController *ctrl;
	
	while ( ctrl = [enumerator nextObject] )
		[ctrl clearSelection];
}
- (NSMenu*)localEditMenu
{
	return fLocalEditMenu;
}

- (void)addStuffToMenus
{
	// stuff to add:
	// create the processorMenu
	// modify the keyboard shorcuts for space stuff
	// add all the virtual outputs

	[self recreateProcessorMenu];
	[self recreateVirtualOutputMenus];
}

- (void)recreateVirtualOutputMenus
{
    // creates the submenu for the 'processor' menu
    int			i;
	int			maxInternalStereo = 64;
	int			maxInternalMono = 64;
	//int			maxExternal = 2; WTF ???????
    NSMenuItem	*aMenuItem;
    NSString	*theMenuName;
    NSString	*theKeyEquivalent = @"";
    
/*
	
	WTF ?????
	
	for ( i = maxExternal-1; i > 0; i-=2 )
	{
		theMenuName = [NSString stringWithFormat:@"External %d-%d", i, i+1];
		aMenuItem = [fVirtualOutputMenu1 insertItemWithTitle:theMenuName action:@selector(selectRAO:) keyEquivalent:theKeyEquivalent atIndex:2];
		[aMenuItem setTag:i];
		aMenuItem = [fVirtualOutputMenu2 insertItemWithTitle:theMenuName action:@selector(selectRAO:) keyEquivalent:theKeyEquivalent atIndex:2];
		[aMenuItem setTag:i];
	}
	[fVirtualOutputMenu1 insertItemWithTitle:@"" action:nil keyEquivalent:@"" atIndex:1+maxExternal];
	[fVirtualOutputMenu2 insertItemWithTitle:@"" action:nil keyEquivalent:@"" atIndex:1+maxExternal];
*/

//#if defined( MTG_LIGHT_VERSION ) || defined( MTG_FULL_VERSION ) || defined( MTG_DEMO_VERSION )
#if defined( MTG_LIGHT_VERSION ) || defined( MTG_DEMO_VERSION )
	[fVirtualOutputMenu1 setAutoenablesItems:NO];
	[fVirtualOutputMenu2 setAutoenablesItems:NO];
#endif
	
    for ( i = maxInternalMono; i > 0 ; i-=1 )
    {
		theMenuName = [NSString stringWithFormat:@"Internal %d", i];
		aMenuItem = [fVirtualOutputMenu1 insertItemWithTitle:theMenuName action:@selector(selectVAO:) keyEquivalent:theKeyEquivalent atIndex:0];
		[aMenuItem setTag:1000+i];
//#if defined( MTG_LIGHT_VERSION ) || defined( MTG_FULL_VERSION ) || defined( MTG_DEMO_VERSION )
#if defined( MTG_LIGHT_VERSION ) || defined( MTG_DEMO_VERSION )
		[aMenuItem setEnabled:NO];
#endif
		aMenuItem = [fVirtualOutputMenu2 insertItemWithTitle:theMenuName action:@selector(selectVAO:) keyEquivalent:theKeyEquivalent atIndex:0];
		[aMenuItem setTag:1000+i];
//#if defined( MTG_LIGHT_VERSION ) || defined( MTG_FULL_VERSION ) || defined( MTG_DEMO_VERSION )
#if defined( MTG_LIGHT_VERSION ) || defined( MTG_DEMO_VERSION )
		[aMenuItem setEnabled:NO];
#endif
    }
    for ( i = maxInternalStereo-1; i > 0 ; i-=2 )
    {
		theMenuName = [NSString stringWithFormat:@"Internal %d-%d", i, i+1];
		aMenuItem = [fVirtualOutputMenu1 insertItemWithTitle:theMenuName action:@selector(selectVAO:) keyEquivalent:theKeyEquivalent atIndex:0];
		[aMenuItem setTag:i];
//#if defined( MTG_LIGHT_VERSION ) || defined( MTG_FULL_VERSION ) || defined( MTG_DEMO_VERSION )
#if defined( MTG_LIGHT_VERSION ) || defined( MTG_DEMO_VERSION )
		[aMenuItem setEnabled:NO];
#endif
		aMenuItem = [fVirtualOutputMenu2 insertItemWithTitle:theMenuName action:@selector(selectVAO:) keyEquivalent:theKeyEquivalent atIndex:0];
		[aMenuItem setTag:i];
//#if defined( MTG_LIGHT_VERSION ) || defined( MTG_FULL_VERSION ) || defined( MTG_DEMO_VERSION )
#if defined( MTG_LIGHT_VERSION ) || defined( MTG_DEMO_VERSION )
		[aMenuItem setEnabled:NO];
#endif
    }

}

- (void)recreateProcessorMenu
{
    // creates the submenu for the 'processor' menu
    NSMenu		*theMenu;
    int			i;
    int			j;
    int			numPluginSets;
    int			numPluginClasses;
    NSMenuItem	*aMenuItem;
    NSString	*theProcessorName;
    NSString	*theKeyEquivalent;
    
	
    theMenu = fProcessorMenuItem;

    numPluginSets = TArmeMediaEngine::ARPIHost()->GetNbPluginSets(); 
    
    for ( i = 0; i < numPluginSets ; i++ )
    {
		VARPIPluginSet* pluginSet = TArmeMediaEngine::ARPIHost()->GetPluginSet(i);
        numPluginClasses = pluginSet->GetNbPluginClasses();
        
        if ( !numPluginClasses || numPluginClasses == 1 )
        {
            // this is an individual processor
            
			theProcessorName = @"TheProcessorName";
            theKeyEquivalent = @"";
            
            aMenuItem = [theMenu addItemWithTitle:theProcessorName action:@selector(createProcessor:) keyEquivalent:@""];
            [aMenuItem setTag:(i*100+numPluginClasses)];
        }
        else
        {
            for ( j = 0; j < numPluginClasses; j++ )
            {
				VARPIPluginClass* pluginClass = pluginSet->GetPluginClass(j);
				if (pluginClass->PluginIsDemo())
				{ 	
					theProcessorName = [NSString stringWithCString:(pluginClass->GetPluginName())];
					theProcessorName = [NSString stringWithFormat:@"%@ (Demo)",theProcessorName];
				}
				else
				{
					theProcessorName = [NSString stringWithCString:(pluginClass->GetPluginName())];
				}
                theKeyEquivalent = @"";
                
                aMenuItem = [theMenu addItemWithTitle:theProcessorName action:@selector(createProcessor:)keyEquivalent:@""];
                [aMenuItem setTag:(i*100+j)];
            }
        }
    }
}

- (void)addMainViewController:(MIMainViewController*)newMainViewController
{
    NSLog(@"MIApplicationController - addMainViewController:  is deprecated. Please use  NSDocument's addWindowController: instead.");
    [[MIApplicationController sharedApplicationController] beep];
}

- (void)removeMainViewController:(MIMainViewController*)oldMainViewController
{
        NSLog(@"MIApplicationController - removeMainViewController:  is deprecated. Please use  NSDocument's removeWindowController: instead.");
    [[MIApplicationController sharedApplicationController] beep];
}

- (NSTimer*)createPlayBarTimer
{
    NSTimer		*aTimer;
    
    /*	Note from the documentation:
        Because of the various input sources a typical run loop manages,
        the effective resolution of the time interval for an NSTimer
        is limited to on the order of 50-100 milliseconds.
    */
    
	aTimer = 	[NSTimer	scheduledTimerWithTimeInterval:0.05 // interval in seconds
							target:self
							selector:@selector(playBarTimerTicked:)
							userInfo:nil					// if extra information is wanted, put an object here
							repeats:YES];					// does the timer repeat or is once only

        return aTimer;
}

- (void)playBarTimerTicked:(NSTimer*)theTimer
{
    NSEnumerator			*mainViewEnum;
    MIMainViewController	*aController;
    TArmeDoc				*aTArmeDoc;
	MIMontageDocument		*theDoc;
	
	if ( ![NSApp isActive] || [NSApp isHidden] )
		return; // don't need to update if we're not active (aka minimized, ...)

	theDoc = [[NSDocumentController sharedDocumentController] currentDocument];
	if ( !theDoc || ![theDoc windowControllers])
		return;
		
    mainViewEnum = [[theDoc windowControllers] objectEnumerator];
    while ( aController = [mainViewEnum nextObject] )
    {
        aTArmeDoc = [[aController document] armeDocument];
        
        if ( !aTArmeDoc->IsPlaying() )
            continue;
		
		if ( [[aController window] isMiniaturized] )
			continue;
		
		[aController updatePlayBar];
		[((MIOverviewController*)[MIOverviewController sharedOverviewController])->fView setNeedsDisplay:YES];

		if ( fFollowPlayBar )
			[[aController topLevelContainerView] moveVisibleRectToPlayBar];
	}
}

/*
- (MIObject*)createClipFromFileName:(NSString*)fileName atPos:(NSPoint)aPoint bopTask:(BOPTask**)bopTask;
{
    TArmeDoc				*armeDoc;
    MIMontageDocument		*mDoc;
    TArmeWSObj				*newArmeObj;
	TArmeWSObjPos			newArmeObjPos;
    MIObject				*newObj;
    NSPoint					destPoint;
    
	verntl(interfaceStephan,0,"\n************ WARNING ******************\n\n USE OF THE createClipFromFileName function is deprecated. use commands!\n\n******************************" );
	ZzAssert(0);
	
    //mDoc = [[currentWindow delegate] document];
    mDoc = [[NSDocumentController sharedDocumentController] currentDocument];
    armeDoc = [mDoc armeDocument];
		
	ZzAssert(armeDoc != NULL);
	
	ArmeErr err;
    newArmeObj = armeDoc->CreateWSObjFromFile( [fileName cString], armeDoc->GetZeroPos(), bopTask, &err );
    
    if ( !newArmeObj || err.type != kArmeOK )
    {
		NSString* msg1 = @"Error while opening clip file : ";
		NSString* msg2 = [msg1 stringByAppendingString:fileName];
		NSString* msg3 = [msg2 stringByAppendingString:@"\nThe error is : "];
		NSString* msg4 = [msg3 stringByAppendingString:[NSString stringWithCString:ArmeGetStringFromError(err)]];
		NSRunAlertPanel(@"Error", msg4, @"Continue", nil, nil);
		return nil;
    }
    
    newObj = [[MIObject alloc] initWithArmeWSObj:newArmeObj];
    [newObj setMontageDoc:mDoc];    
    [[mDoc topLevelContainer] addChild:newObj];
    
    if ( NSEqualPoints(aPoint, NSZeroPoint) )
    {
        destPoint = [[[NSApp mainViewController] topLevelContainerView] visibleRect].origin;
    }
    else
    {
        destPoint = aPoint;
    }
    
	armeDoc->LockRead();
		newArmeObjPos = newArmeObj->GetPos();
	armeDoc->UnlockRead();
	
	newArmeObjPos.fStartTime = [[NSApp mainViewController] pixelToArmeTime:destPoint.x];
	
	newArmeObj->SetPos(newArmeObjPos);
	
	[mDoc lockWrite];
		[newObj setVerticalPos:[[NSApp mainViewController] pixelToMIVPos:destPoint.y]];
	[mDoc unlockWrite];
	
	[newObj createAllRepresentations];
    [newObj setRepresentationsNeedDisplay];
	[self modelChangedForDocument:[[NSDocumentController sharedDocumentController] currentDocument]];

    
    return newObj;
}
*/

- (MIPlayStopPauseController*)playStopPauseController
{
    NSLog(@"MIApplicationController - playStopPauseController  is deprecated. Please use  [MIPlayStopPauseController sharedPlayStopPauseController] instead.");
    [[MIApplicationController sharedApplicationController] beep];
    return [MIPlayStopPauseController sharedPlayStopPauseController];
}

- (MIOverviewController*)overViewController
{
//	return fOverviewController;
	ZzAssert(0);
	return nil;
}

#if 0 //$$$TOOLBOX
- (MIToolBoxController*)toolBoxController
{
    NSLog(@"MIApplicationController - toolBoxController  is deprecated. Please use  [MIToolBoxController sharedToolBoxController] instead.");
    [[MIApplicationController sharedApplicationController] beep];

	return [MIToolBoxController sharedToolBoxController];
}
#endif 0

- (MITimePaletteController*)timePaletteController
{
	return [MITimePaletteController sharedTimePaletteController];
}

- (void)registerPanelToLayoutManagers
{
	/*MIMainViewController	*ctrl;
	NSEnumerator			*ctrlEnum;
	
	ctrlEnum = [[[[NSDocumentController sharedDocumentController] currentDocument] windowControllers] objectEnumerator];
	
	while ( ctrl = [ctrlEnum nextObject] )
	{
            [[ctrl panelLayoutManager] setPanel:[[MIPlayStopPauseController sharedPlayStopPauseController] window] ofType:MIPlayStopPausePanel];
            [[ctrl panelLayoutManager] setPanel:[[MITimePaletteController sharedTimePaletteController] window] ofType:MITimePanel];
            [[ctrl panelLayoutManager] setPanel:[[MIToolBoxController sharedToolBoxController] window] ofType:MIToolBoxPanel];
            [[ctrl panelLayoutManager] setPanel:[[MIOverviewController sharedOverviewController] window] ofType:MIOverviewPanel];

            [[ctrl panelLayoutManager] reinitNotifications];
            [[ctrl panelLayoutManager] checkPanelsStickyness];
        }*/
}


- (void)modelChangedForDocument:(MIMontageDocument*)doc
{
	[[MIOverviewController sharedOverviewController] refreshCachedImage];
}


///////////////////////////////////////////////////////////////////////////
/////																  /////
//								ACTIONS												 
/////																  /////
///////////////////////////////////////////////////////////////////////////


- (BOOL)validateMenuItem:(NSMenuItem *)menuItem
{
	// add checks for stuff that are not commands here.
	SEL action = [menuItem action];


	if ( action == @selector(nudgeRight:) ) {
		if ( [[[NSDocumentController sharedDocumentController] currentDocument] hasContent] ) {
			id controller = [[NSApp keyWindow] windowController];
			if ( controller ) {
				if ( [controller respondsToSelector:@selector(ignoreArrowKeys)] ) {
					BOOL flag;
					NSMethodSignature *signature = [controller methodSignatureForSelector:@selector(ignoreArrowKeys)];
					NSInvocation *invocation = [NSInvocation invocationWithMethodSignature:signature];
					[invocation setSelector:@selector(ignoreArrowKeys)];
					[invocation setTarget:controller];
					[invocation invoke];
					[invocation getReturnValue:&flag];
					if ( flag ) return YES;
					else return NO;
				} else return NO;
			} else return NO;
		}
		return NO;
	}
	
	if ( action == @selector(nudgeLeft:) ) {
		if ( [[[NSDocumentController sharedDocumentController] currentDocument] hasContent] ) {
			id controller = [[NSApp keyWindow] windowController];
			if ( controller ) {
				if ( [controller respondsToSelector:@selector(ignoreArrowKeys)] ) {
					BOOL flag;
					NSMethodSignature *signature = [controller methodSignatureForSelector:@selector(ignoreArrowKeys)];
					NSInvocation *invocation = [NSInvocation invocationWithMethodSignature:signature];
					[invocation setSelector:@selector(ignoreArrowKeys)];
					[invocation setTarget:controller];
					[invocation invoke];
					[invocation getReturnValue:&flag];
					if ( flag ) return YES;
					else return NO;
				} else return NO;
			} else return NO;
		}
		return NO;
	}
	
	if ( action == @selector(setLocatorsFromSelection:) ) {
		MIMainViewController *controller = [NSApp mainViewController];
		if ( controller ) {
			MIWSSelection *selection = [controller selection];
			if ( [selection count] > 0 ) return YES;
		}
		return NO;
	}
	
	if ( action == @selector(find:) ) {
		return [[[NSDocumentController sharedDocumentController] currentDocument] hasContent];
	}
	
	if ( action == @selector(findAgain:) ) {
		return [[[NSDocumentController sharedDocumentController] currentDocument] hasContent];
	}	

	if ( action == @selector(selectAll:) ) {
		return [[[NSDocumentController sharedDocumentController] currentDocument] hasContent];
	}
	
	if ( action == @selector(exportClip:) ) {
//        if( ArboGetLicenseManager()->PluginOrAppLicensed( ) )
            return [[[NSDocumentController sharedDocumentController] currentDocument] hasContent];
//        else
//            return NO;
	}

	if ( action == @selector(exportRawAudio:) ) {
//        if( ArboGetLicenseManager()->PluginOrAppLicensed( ) )
            return [[[NSDocumentController sharedDocumentController] currentDocument] hasContent];
//        else
//            return NO;
	}

	if ( action == @selector(createProcessor:) ) {
		if ( ![[NSDocumentController sharedDocumentController] currentDocument] ) return NO;
		if ( [[NSApp mainWindow] windowController] != [NSApp mainViewController] ) return NO;
		else return YES;
	}
	
	
	if ( action == @selector(toggleLoop:) ) {
		if ( ![[NSDocumentController sharedDocumentController] currentDocument] ) return NO;
		[menuItem setState:[self isLooping] ? NSOnState : NSOffState];
		if ( [[NSApp mainWindow] windowController] == [NSApp mainViewController] ) return YES;
		else return NO;
	}
	
	if ( action == @selector(followPlaybar:) ) {
		[menuItem setState:fFollowPlayBar ? NSOnState : NSOffState];
		if ( [[NSApp mainWindow] windowController] == [NSApp mainViewController] ) return YES;
		else return NO;
	}
	
	if ( action == @selector(showAboutPanel:) ) return YES;


	if ( action == @selector(selectVAO:) )
	{
		MIMainViewController *controller = [NSApp mainViewController];
		if ( controller ) {
			MIWSSelection *selection = [controller selection];
			return [MIChangeVAOCommand supportsSelection:selection];
		}
		return NO;
	}

	if ( action == @selector(createNewContainer:) )
	{
		MIMainViewController *controller = [NSApp mainViewController];
		if ( controller ) {
			MIWSSelection *selection = [controller selection];
			return [MICreateContainerCommand supportsSelection:selection];
		}
		return NO;
	}
	
	
	if ( action == @selector(viewControls:) )
	{
		MIMainViewController *controller = [NSApp mainViewController];
		if ( controller ) {
			MIWSSelection *selection = [controller selection];
			if ( selection ) {
				if ( [selection count] > 0 ) {
					if ([selection type] == kMIWSSelectionTypeObjects) {
						TArmeWSObj* wsobj = [[[selection objectAtIndex:0] representedObject] WSObject];
						SInt32 type = wsobj->GetType();
						if(	(type == TArmeWSObj::kVideoEffect) || (type == TArmeWSObj::kAudioProcessor) )
							return YES;
					}
				}
			}
		}
		return NO;
	}

	if ( action == @selector(showGeneralGridVerticalLines:) )
	{
		MIPreferences* prefs = [MIPreferences sharedPrefs];
		UInt32 mask = [prefs gridShowMask];
		if ( ![[NSDocumentController sharedDocumentController] currentDocument] ) return NO;
		if ( mask & kPrefsGridShowVLines )
			[menuItem setTitle:@"Hide Vertical Lines"];
		else [menuItem setTitle:@"Show Vertical Lines"];
		return YES;
	}

	
	if ( action == @selector(showGeneralGridHorizontalLines:) )
	{
		MIPreferences* prefs = [MIPreferences sharedPrefs];
		UInt32 mask = [prefs gridShowMask];
		if ( ![[NSDocumentController sharedDocumentController] currentDocument] ) return NO;
		if ( mask & kPrefsGridShowHLines )
			[menuItem setTitle:@"Hide Horizontal Lines"];
		else [menuItem setTitle:@"Show Horizontal Lines"];
		return YES;
	}
	

	if ( action == @selector(showObjectGridVerticalLines:) )
	{
		MIPreferences* prefs = [MIPreferences sharedPrefs];
		UInt32 mask = [prefs gridShowMask];
		if ( ![[NSDocumentController sharedDocumentController] currentDocument] ) return NO;
		if ( mask & kPrefsGridShowVObjLines )
			[menuItem setTitle:@"Hide Vertical Lines"];
		else [menuItem setTitle:@"Show Vertical Lines"];
		return YES;
	}

	if ( action == @selector(showObjectGridHorizontalLines:) )
	{
		MIPreferences* prefs = [MIPreferences sharedPrefs];
		UInt32 mask = [prefs gridShowMask];
		if ( ![[NSDocumentController sharedDocumentController] currentDocument] ) return NO;
		if ( mask & kPrefsGridShowHObjLines )
			[menuItem setTitle:@"Hide Horizontal Lines"];
		else [menuItem setTitle:@"Show Horizontal Lines"];
		return YES;
	}
	
	if ( action == @selector(snapToGeneralGrid:) )
	{
		MIPreferences* prefs = [MIPreferences sharedPrefs];
		UInt32 mask = [prefs snapOptionsMask];
		if ( ![[NSDocumentController sharedDocumentController] currentDocument] ) return NO;
		if ( ( mask & kPrefsSnapHGrid ) && (mask & kPrefsSnapVGrid) )
			[menuItem setState:NSOnState];
		else if ( ( mask & kPrefsSnapHGrid ) || (mask & kPrefsSnapVGrid) )
			[menuItem setState:NSMixedState];
		else [menuItem setState:NSOffState];
		return YES;
	}

	if ( action == @selector(snapToObjectsGrid:) )
	{
		MIPreferences* prefs = [MIPreferences sharedPrefs];
		UInt32 mask = [prefs snapOptionsMask];
		if ( ![[NSDocumentController sharedDocumentController] currentDocument] ) return NO;
		if ( ( mask & kPrefsSnapHObjects ) && (mask & kPrefsSnapVObjects) )
			[menuItem setState:NSOnState];
		else if ( ( mask & kPrefsSnapHObjects ) || (mask & kPrefsSnapVObjects) )
			[menuItem setState:NSMixedState];
		else [menuItem setState:NSOffState];
		return YES;
	}
	
	
	if ( action == @selector(showPreferences:) )
	{
		return YES;
	}
	
	if ( action == @selector(chain:) )
	{
		MIMainViewController *controller = [NSApp mainViewController];
		if ( controller )
		{
			MIWSSelection *selection = [controller selection];
			if ( [selection count] > 1 ) return YES;
		}
		return NO;
	}
	
	if ( action == @selector(stack:) ) {
		MIMainViewController *controller = [NSApp mainViewController];
		if ( controller )
		{
			MIWSSelection *selection = [controller selection];
			if ( [selection count] > 1 ) return YES;
		}
		return NO;
	}

	if ( action == @selector(alignLeft:) )
	{
		MIMainViewController *controller = [NSApp mainViewController];
		if ( controller )
		{
			MIWSSelection *selection = [controller selection];
			if ( [selection count] > 1 ) return YES;
		}
		return NO;
	}

	if ( action == @selector(alignRight:) )
	{
		MIMainViewController *controller = [NSApp mainViewController];
		if ( controller )
		{
			MIWSSelection *selection = [controller selection];
			if ( [selection count] > 1 ) return YES;
		}
		return NO;
	}
	
	
	return [[NSApp mainViewController] canMenuItemActionBePerformedOnCurrentSelection:menuItem];
}


///////////////////////////////////////////////////////////////////////////
/////																  /////
//								FILE MENU												 
/////																  /////
///////////////////////////////////////////////////////////////////////////



extern void GetMutedStatusBeforeCommand(MIObject* obj, ZzArray<SInt32>* mutedStatusBeforeCommand);
extern void SetMutedStatusAsTheyWereBeforeCommand(MIObject* obj, ZzArray<SInt32>* mutedStatusBeforeCommand, int& index);

const EventTime kMCIdleDuration = kEventDurationSecond / 30;

- (IBAction)exportClip:(id)sender
{
	Media						audioMedia = NULL;
	Track						audioTrack = NULL;
	float* 						tmpBuffer = NULL;
	SoundDescriptionHandle		audioSampleDesc = NULL;
	char 						filePath[kMaxPathLength];
//	char 						fullPath[kMaxPathLength];
	char						dvStreamPath[kMaxPathLength];
	short 						highResResourceId = 0;
	short 						resourceRefNum = 0;
	Movie 						exportMovie = NULL;
	MEMTGAAudioFile 			file;
	
	bool						exportVideo = false;
	bool						exportAudio = false;
	bool						exportSliceSelection = false;
	
	OSErr						myErr = noErr;
	OSErr						res = noErr;
	OSStatus					st = noErr;
	TArmeDoc*					armeDoc;
	MIMontageDocument*			montageDoc;
	ZzArchPtrArray 				objectsToExport;

	dvStreamPath[0] = 0;

	montageDoc = [[NSDocumentController sharedDocumentController] currentDocument];
	armeDoc = [montageDoc armeDocument];

	ArmeTime movieStartTime = ARME_TIME_MAX;
	ArmeTime movieEndTime = ARME_TIME_MIN;

	

        //display the dialog to choose the kind of export
        MIExportSetting exportSetting = MIExportSelection;
        MIExportCodec exportCodec = MIWorkingCodec;
    
        MIExportContent	selectionContent = MIEmpty;
        MIExportContent documentContent = MIEmpty;	
        bool exportToCamera = true;
        if (![MICamera isCameraConnected])
            exportToCamera = false;
    
        //if no video, can't use the project's codec
        if (!armeDoc->GetTopContainer()->ContainsVideo(false, true))
            documentContent = MINoVideo;
        else
            documentContent = MIVideo;
    
        // if there is no selection
        if ([[[NSApp mainViewController] selection] count] == 0)
        {
            exportSetting = MIExportWholeDocument;
            selectionContent = MIEmpty;
        }
        else
        {
            exportSetting = MIExportSelection;
    
            // if there is only audio, we can not use the project's codec
            if ([[[NSApp mainViewController] selection] containsOnlyAudio])
                selectionContent = MIOnlyAudio;
            else
                selectionContent = MIVideo;
        }
        
        if (! MIRunExportPanel( &exportSetting, &exportCodec, &exportToCamera, selectionContent, documentContent ) ) 
        {
            // user pushed the CANCEL button
            return;
        }
        
        if (exportSetting == MIExportSelection)
        {
            if ([[[NSApp mainViewController] selection] type] == kMIWSSelectionTypeTimeSlice)
            {
                exportSliceSelection = true;
            }
    
            if ([[[NSApp mainViewController] selection] containsAudio])
                exportAudio = true;
    
            if ([[[NSApp mainViewController] selection] containsVideo:false andTransparentTextClip:true])
                exportVideo = true;
        }
        else
        {
			if (armeDoc->GetTopContainer()->ContainsAudio())
                exportAudio = true;
    
            if (armeDoc->GetTopContainer()->ContainsVideo(false, true))
                exportVideo = true;
        }
    
        try
        {
            VUSERCMD3("Export (selection = %d), (toCamera = %d), (project codec = %d)", (exportSetting == MIExportSelection), exportToCamera, (exportCodec == MIWorkingCodec));
            if (exportSetting == MIExportSelection)
            {
                VUSERCMD_PRINT_SELECTION([[NSApp mainViewController] selection]);
            }
    
            // create the export movie
            ZzAppendFileNameToPath(filePath, kMaxPathLength, armeDoc->GetProjectFolderPath(), "Temp.Mov");
            
            FSSpec fsSpec = FSSpecFromNonExistingFilePath(filePath, FOUR_CHAR_CODE('TVOD'), FOUR_CHAR_CODE('MooV'), res);
            ThrowOnOSStatus(res);
        
            res = CreateMovieFile(&fsSpec, FOUR_CHAR_CODE('TVOD'), smCurrentScript, 
                    createMovieFileDontCreateResFile | newMovieActive | createMovieFileDeleteCurFile, &resourceRefNum, &exportMovie);
            ThrowOnOSStatus(res);
                    
            highResResourceId = movieInDataForkResID;
            res = AddMovieResource(exportMovie, resourceRefNum, &highResResourceId, NULL);
            ThrowOnOSStatus(res);
        
            SetMovieTimeScale(exportMovie, armeDoc->fMasterMovie->GetVideoTimeScale());
            ThrowOnOSStatus(res);
            
            if (exportSetting == MIExportSelection)
            {
                NSArray* selArray = [[[NSApp mainViewController] selection] putRepsInArrayAndRemoveDescendants];
                
                //put all objects in the selection in an array
                for(UInt32 i = 0; i < [selArray count]; i++)
                {
                    TArmeWSObj* armeObj = [[[selArray objectAtIndex:i] representedObject] WSObject];
                    if ((armeObj->GetType() != TArmeWSObj::kVideoXFadeEffect) && (!armeObj->IsMuted()))
                    {
                        objectsToExport.AddItem(armeObj);
                        if (armeObj->GetAbsoluteStartTime() < movieStartTime)
                            movieStartTime = armeObj->GetAbsoluteStartTime();
                        if (armeObj->GetAbsoluteEndxTime() > movieEndTime)
                            movieEndTime = armeObj->GetAbsoluteEndxTime();
                    }
                }
        
                if (exportSliceSelection)
                {
                    movieStartTime = [[[NSApp mainViewController] selection] startTime];
                    movieEndTime = [[[NSApp mainViewController] selection] endxTime];
                }
                            
                if (exportVideo)
                {
                    // Solo the objectsToExport
                    //---------------------------
                    TArmeWSObj* previousVideoSoloObject = armeDoc->GetVideoSoloObject();
                    armeDoc->SetVideoSoloObject(NULL);
            
                    ZzArray<SInt32> mutedStatusBeforeCommand;
                    GetMutedStatusBeforeCommand([montageDoc topLevelContainer], &mutedStatusBeforeCommand);
            
                    //mute all objects
                    armeDoc->GetTopContainer()->SetMuted(true);
                
                    //except the ones in the selection
                    for (int i=0; i < objectsToExport.CountItems(); ++i)
                    {
                        ((TArmeWSObj*)(objectsToExport.ItemAt(i)))->SetMuted(false);
                    }
            
                    res = CopyMovieSegment(exportMovie, armeDoc->fMasterMovie->GetQTMovie(), 0, armeDoc->TimeValueFromArmeTime(movieStartTime), armeDoc->TimeValueFromArmeTime(movieEndTime - movieStartTime)); 
                    ThrowOnOSStatus(res);
            
                    // Un-solo the objectsToExport
                    //----------------------------
            
                    int index = 0;
                    SetMutedStatusAsTheyWereBeforeCommand([montageDoc topLevelContainer], &mutedStatusBeforeCommand, index);
                    
                    armeDoc->SetVideoSoloObject(previousVideoSoloObject);
                }
            }
            else
            {	
                //export all childrens of TLC
                for(SInt32 i = 0; i < armeDoc->GetTopContainer()->GetChildrenCount(); i++)
                {
                    objectsToExport.AddItem(armeDoc->GetTopContainer()->GetChild(i));
                    if (armeDoc->GetTopContainer()->GetChild(i)->GetAbsoluteStartTime() < movieStartTime)
                        movieStartTime = armeDoc->GetTopContainer()->GetChild(i)->GetAbsoluteStartTime();
                    if (armeDoc->GetTopContainer()->GetChild(i)->GetAbsoluteEndxTime() > movieEndTime)
                        movieEndTime = armeDoc->GetTopContainer()->GetChild(i)->GetAbsoluteEndxTime();
                }
        
                if (exportVideo)
                {
                    res = CopyMovieSegment(exportMovie, armeDoc->fMasterMovie->GetQTMovie(), 0, armeDoc->TimeValueFromArmeTime(movieStartTime), armeDoc->TimeValueFromArmeTime(movieEndTime - movieStartTime)); 
                    ThrowOnOSStatus(res);
                }
            }
        
            //It seems that there are some problems with the background track (when exporting only audio, there is nothing in the background track)
        /*	DisposeMovieTrack(GetMovieTrack(exportMovie, 2)); //remove original track
            DisposeMovieTrack(GetMovieTrack(exportMovie, 1)); //remove background track
            Track newBackgroundTrack = CreateNewTrackInMovie(exportMovie, armeDoc->fMasterMovie->GetVideoTimeScale(), armeDoc->fMasterMovie->GetWidth(), armeDoc->fMasterMovie->GetHeight());
            armeDoc->fMasterMovie->CopyBackgroundInTrack(newBackgroundTrack, movieStartTime, movieEndTime);
            SetTrackLayer(newBackgroundTrack, 1500);*/
        
            if (exportAudio)
            {
                audioTrack = NewMovieTrack(exportMovie, 0, 0, kFullVolume);
                ThrowOnOSStatus(GetMoviesError());
    
                // set new track to be a sound track
                audioMedia = NewTrackMedia(audioTrack, SoundMediaType, (UInt32)armeDoc->GetAudioFrameRate(), NULL, 0);
                ThrowOnOSStatus(GetMoviesError());
    
                st = BeginMediaEdits(audioMedia);
                ThrowOnOSStatus(st);
    
                SInt32 nbChannels = 2; //$$$TODO it doesn't really matter but 
    
                // create a sound sample description
                audioSampleDesc = (SoundDescriptionHandle)NewHandle(sizeof(SoundDescription));
                if (!audioSampleDesc)
                {
                    ZZ_THROW(ZzExceptionOutOfMemory);
                }
            
                (**audioSampleDesc).descSize			= sizeof(SoundDescription);
                (**audioSampleDesc).dataFormat			= kFloat32Format;
                (**audioSampleDesc).resvd1				= 0;
                (**audioSampleDesc).resvd2				= 0;
                (**audioSampleDesc).dataRefIndex		= 0;
                (**audioSampleDesc).version				= 0;
                (**audioSampleDesc).revlevel			= 0;
                (**audioSampleDesc).vendor				= 0;
                (**audioSampleDesc).numChannels			= nbChannels;
                (**audioSampleDesc).sampleSize			= 32;
                (**audioSampleDesc).compressionID		= 0;
                (**audioSampleDesc).packetSize			= 0;
                (**audioSampleDesc).sampleRate			= (UInt32)armeDoc->GetAudioFrameRate() << 16; //fixed
    
                MIProgressBarDlgManager progressMngr("Exporting audio...");
                MEAudioExportInQTTrack(&objectsToExport,exportSliceSelection,GET_VAL(movieStartTime),GET_VAL(movieEndTime),nbChannels,&audioMedia,(SampleDescriptionHandle)audioSampleDesc,&progressMngr);
            
                st = EndMediaEdits(audioMedia);
                ThrowOnOSStatus(st);
            
                // insert the media into the track
                TimeValue mediaDuration = GetMediaDuration(audioMedia);
                myErr = InsertMediaIntoTrack(audioTrack, 0, 0, mediaDuration, fixed1);  
                ThrowOnOSStatus(myErr);
            }
            
        
            myErr = noErr;
    
            if (exportToCamera)
            {
                using namespace dts; 
                CVideoOutput* videoOutput = NULL;
                MICamera* camera = NULL;
                Movie dvStreamMovie = NULL;
                
                while (![MICamera isCameraConnected])
                {
                    if (NSRunAlertPanel(@"Warning", @"No Camera Detected", @"Retry", @"Cancel", nil) != NSAlertDefaultReturn)
                    {
                        ZZ_THROW(ZzExceptionCancel);
                    }
                }
            
                    
    /*			while (!camera)
                {
                    if (NSRunAlertPanel(@"Warning",@"No camera detected",@"Retry",@"Cancel",nil) != NSAlertDefaultReturn)
                    {
                        ZZ_THROW(ZzExceptionCancel);
                    }
                }*/
            
                try
                {
                
                    //****************
                    //EXPORT TO CAMERA
                    //****************
                
                    MIProgressBarDlgManager progressMngr2("Preparing Export");
                    progressMngr2.SetNbSubTasks(1);
                    progressMngr2.EnterSubTask();
                
                    ::SetMovieProgressProc(exportMovie, QTMovieProgressProc, (long)&progressMngr2);
        
                
                    //to export to a camera, first, export to a DVStream :
                    ZzAppendFileNameToPath(dvStreamPath, kMaxPathLength, armeDoc->GetProjectFolderPath(), "dvStream.dv");
                    
                    //erase the previous file if any
                    remove(dvStreamPath);
                    FSSpec outputFile = Path2FSspec(dvStreamPath, false);//FSSpecFromNonExistingFilePath(dvStreamPath, FOUR_CHAR_CODE('TVOD'), FOUR_CHAR_CODE('MooV'), res);
                    ThrowOnOSStatus(res);
                                
                    //$$TODO : if working codec is MJPEG, ask if he wants to export to NTSC or PAL
                    WorkingCodecType codec;
                    codec = armeDoc->fMasterMovie->GetWorkingCodecType();
                    //select type of output to the camera (NTSC or PAL)
                    bool isNTSC;
                    if (codec == kDvNtsc720_480)
                        isNTSC = true;
                    else if (codec == kDvPal720_576)
                        isNTSC = false;
                    else
                    {
                        if (NSRunAlertPanel(@"Warning", @"What type of camera are you using ?", @"NTSC", @"PAL", nil) == NSAlertDefaultReturn)
                        {
                            isNTSC = true;
                            codec = kDvNtsc720_480;
                        }
                        else
                        {
                            isNTSC = false;
                            codec = kDvPal720_576;
                        }
                    }
    
        
                    myErr = ConvertMovieToFile(exportMovie, 0, &outputFile, 0, 0, -1, 0, createMovieFileDeleteCurFile, GetCodecInstance(codec)); 
                    ThrowOnOSStatus(myErr);
                    
                    progressMngr2.LeaveSubTask();
                    
                    short movieResFile = 0;
                    //short movieResId;
                    ArmeErr armeErr;
                    
                    dvStreamMovie = LoadMovie(outputFile, 0, &movieResFile, &armeErr, false);
                    
                    // Instantiate a VideoOutput object without a specific Movie
                    
                    videoOutput = new(std::nothrow) CVideoOutput( "\pSimpleVideoOut", isNTSC );
                    if ( videoOutput == NULL || videoOutput->GetError() ) 
                    {
                        ZZ_THROW_1ARG(ZzGenericException, "No camera detected");
                    }
    
                    // Select an output component to use
                    OSErr err = videoOutput->SelectVideoOutputComponent();
                    if ( err ) 
                    { 
                        ZZ_THROW_1ARG(ZzGenericException, "Could not select camera");
                    }
                    
                    // Open the video output component
                    err = videoOutput->Open();
                    if ( err ) 
                    { 
                        ZZ_THROW_1ARG(ZzGenericException, "Could not connect to camera");
                    }
                    
                    //attach dvStream to current viewer
                    [[MIViewerController sharedViewerController] setMovie:dvStreamMovie];
        
                    videoOutput->SetMovie(dvStreamMovie);
                    
                    //change dvStream playback quality
                    bool isHighQuality = [[MIPreferences sharedPrefs] isHighQuality];
                    bool isSingleField = [[MIPreferences sharedPrefs] isSingleField];
                    long flags = 0;
                    if (isHighQuality)
                        flags |= hintsHighQuality;
                    if (isSingleField)
                        flags |= hintsSingleField;
                    SetMediaPlayHints(GetTrackMedia(GetFirstVideoTrack(dvStreamMovie)), flags, hintsSingleField | hintsHighQuality);
                    
                    // Tell the output component to begin
                    err = videoOutput->Begin();
                    if ( err ) 
                    { 
                        ZZ_THROW_1ARG(ZzGenericException, "Could not start playing to camera");
                    }
        
                    MIProgressBarDlgManager progressMngr3("Exporting to Camera");
                    progressMngr3.DisplayDlg();
                    progressMngr3.SetNbSubTasks(1);
                    progressMngr3.EnterSubTask();
                                
                    camera = [[MICamera alloc] init];  //note : the MICamera must be created after the CVideoOutput (don't know why)
        
                    [camera deviceControl:deviceRecordOnCamera];
                    sleep(4);  //black 5 seconds (and some time needed for the camera to start recording)
        
                    videoOutput->SetEchoPort(MCGetControllerPort((ComponentInstanceRecord*)[[[MIViewerController sharedViewerController] movieView] movieController]));
        
                    [[[MIViewerController sharedViewerController] movieView] start:self];
        
                    TimeValue dvStreamDuration = GetMovieDuration(dvStreamMovie);
                    
                    while (!IsMovieDone(dvStreamMovie))
                    {
                        [[[MIViewerController sharedViewerController] window] display];
                        
                        TimeRecord streamTime;
                        GetMovieTime(dvStreamMovie, &streamTime);
                        progressMngr3.Update(float(streamTime.value.lo)/float(dvStreamDuration));
                    }
        
                    [camera deviceControl:deviceStop];
        
                    [camera close];
                    [camera release];
                    
                    [[MIViewerController sharedViewerController] setMovie:armeDoc->fMasterMovie->GetQTMovie()];
                    
                    videoOutput->SetEchoPort();
                    videoOutput->Close();
                    delete videoOutput;
                    
                    DisposeMovie(dvStreamMovie);
                    progressMngr3.LeaveSubTask();
                }
                catch(...)
                {
                    [camera deviceControl:deviceStop];
        
                    [camera close];
                    [camera release];
                    
                    [[MIViewerController sharedViewerController] setMovie:armeDoc->fMasterMovie->GetQTMovie()];
                    
                    if (videoOutput)
                    {
                        videoOutput->SetEchoPort();
                        videoOutput->Close();
                        delete videoOutput;
                    }
                    
                    DisposeMovie(dvStreamMovie);
                    throw;
                }
            }
            else
            {
                //**************
                //EXPORT TO FILE
                //**************
                
                MIProgressBarDlgManager progressMngr2("Exporting project");
                progressMngr2.SetNbSubTasks(1);
                progressMngr2.EnterSubTask();
            
                ::SetMovieProgressProc(exportMovie, QTMovieProgressProc, (long)&progressMngr2);
                
                char exportFileName[1024];
                strcpy(exportFileName, armeDoc->GetProjectFolderPath());
                exportFileName[strlen(exportFileName)-1] = 0;  //remove trailing '/'
                ZzRemoveFileExtensionFromPath(exportFileName, 1024);
                ZzAppendExtension(exportFileName, 1024, ".mov");
            
                FSSpec outputFile = Path2FSspec(exportFileName, false); //FSSpecFromNonExistingFilePath(exportFileName, FOUR_CHAR_CODE('TVOD'), FOUR_CHAR_CODE('MooV'), res);
                ThrowOnOSStatus(res);
                
                do
                {
                    if (exportCodec == MIWorkingCodec)
                    {
                        WorkingCodecType codec;
                        //It seems it not yet perfect, keep it for post 1.0 ?
                /*		if (armeDoc->fMasterMovie->GetWorkingCodecType() == kDvNtsc720_480)
                            codec = kExportDvNtsc720_480;
                        else if (armeDoc->fMasterMovie->GetWorkingCodecType() == kDvPal720_576)
                            codec = kExportDvPal720_576;
                        else*/
                            codec = armeDoc->fMasterMovie->GetWorkingCodecType();
                        
                        myErr = ConvertMovieToFile(exportMovie, 0, &outputFile, 0, 0, -1, 0, createMovieFileDeleteCurFile | showUserSettingsDialog, GetCodecInstance(codec)); 			
                    }
                    else
                    {
                        myErr = ConvertMovieToFile(exportMovie, 0, &outputFile, 0, 0, -1, 0, createMovieFileDeleteCurFile | showUserSettingsDialog, nil); 
                    }
            
                    if (myErr == -47)
                    {
                        char title[1024];
                        char fileName[1024];
                        FSpPathFromLocation(&outputFile, fileName);
                        
                        sprintf(title, "cannot overwrite %s because the file is currently in use", fileName);
                        NSRunAlertPanel(@"Error",[NSString stringWithCString:title],@"OK",nil,nil);
                    }
                } while (myErr == -47);
            
                if(myErr != noErr)
                {
                    if (myErr != -128)  // (pressed on cancel returns -128)
                    {
                        ThrowOnOSStatus(myErr);
                    }
                }
        
                //try to set the movie in highQuality :
                SavePlaybackHintsToMovie(&outputFile, [[MIPreferences sharedPrefs] isHighQuality], [[MIPreferences sharedPrefs] isSingleField]);
    
                progressMngr2.LeaveSubTask();
            }
        }
        catch (ZzException* e)
        {
            e->DefaultCatcher();
            goto cleanup_and_exit;
        }
    
    cleanup_and_exit :
        if (audioSampleDesc != NULL)	
            DisposeHandle((Handle)audioSampleDesc);
    
        DisposeTrackMedia(audioMedia);
        DisposeMovieTrack(audioTrack);
        DisposeMovie(exportMovie);
        CloseMovieFile(resourceRefNum);
        remove(filePath);
        file.Close();
        remove(dvStreamPath);
        delete [] tmpBuffer;
}

- (IBAction)exportRawAudio:(id)sender
{
	[MIExportRAWAudioController exportRawAudio];
}

///////////////////////////////////////////////////////////////////////////
/////																  /////
//								EDIT MENU												 
/////																  /////
///////////////////////////////////////////////////////////////////////////

- (IBAction)delete:(id)sender
{
        vertl(commandsBenjamin,5,"INFO : the 'Edit | Del' menu entry has been selected.\n");
	[[NSApp mainViewController] executeCommand:@"del" ];
}

- (IBAction)deleteContainer:(id)sender
{
	vertl(commandsBenjamin,5,"INFO : the 'Edit | Delete Container' menu entry has been selected.\n");
	[[NSApp mainViewController] executeCommand:@"deleteContainer" ];
}

- (IBAction)rename:(id)sender
{
	vertl(commandsBenjamin,5,"INFO : the 'Edit | Rename' menu entry has been selected.\n");
	[[NSApp mainViewController] executeCommand:@"rename" ];
}

- (IBAction)delBenja:(id)sender
{
	vertl(commandsBenjamin,5,"INFO : the 'Edit | DelBenja' menu entry has been selected.\n");
	[[NSApp mainViewController] executeCommand:@"delBenja" ];
}


- (IBAction)cut:(id)sender
{
	vertl(commandsBenjamin,5,"INFO : the 'Edit | Cut' menu entry has been selected.\n");
	[[NSApp mainViewController] executeCommand:@"cut" ];
}

- (IBAction)copy:(id)sender
{
	vertl(commandsBenjamin,5,"INFO : the 'Edit | Copy' menu entry has been selected.\n");
	[[NSApp mainViewController] executeCommand:@"copy" ];
}

- (IBAction)paste:(id)sender
{
	vertl(commandsBenjamin,5,"INFO : the 'Edit | Paste' menu entry has been selected.\n");
	[[NSApp mainViewController] executeCommand:@"paste" ];
}

- (IBAction)duplicate:(id)sender
{
	vertl(commandsBenjamin,5,"INFO : the 'Edit | Duplicate' menu entry has been selected.\n");
	[[NSApp mainViewController] executeCommand:@"duplicate" ];
}

- (IBAction)duplicateMany:(id)sender
{
	vertl(commandsBenjamin,5,"INFO : the 'Edit | Duplicate Many' menu entry has been selected.\n");
	[[NSApp mainViewController] executeCommand:@"duplicateMany" ];
}

- (IBAction)trim:(id)sender
{
	vertl(commandsBenjamin,5,"INFO : the 'Edit | Trim' menu entry has been selected.\n");
	[[NSApp mainViewController] executeCommand:@"trim" ];
}

- (IBAction)crop:(id)sender
{
	vertl(commandsBenjamin,5,"INFO : the 'Edit | Crop' menu entry has been selected.\n");
	[[NSApp mainViewController] executeCommand:@"crop" ];
}

- (IBAction)changeVirtualAudioOutput:(id)sender
{
	vertl(commandsBenjamin,5,"INFO : the 'Edit | Change Virtual Audio Output' menu entry has been selected.\n");
	[[NSApp mainViewController] executeCommand:@"changeVirtualAudioOutput" ];
}

- (IBAction)dirtyResampler:(id)sender
{
	vertl(commandsBenjamin,5,"INFO : the 'Edit | Dirty Resampler' menu entry has been selected.\n");
	[[NSApp mainViewController] executeCommand:@"dirtyResampler" ];
}

- (IBAction)resample:(id)sender
{
	vertl(commandsBenjamin,5,"INFO : the 'Edit | Resample' menu entry has been selected.\n");
	[[NSApp mainViewController] executeCommand:@"resample" ];
}

- (IBAction)reverse:(id)sender
{
	vertl(commandsBenjamin,5,"INFO : the 'Edit | Reverse' menu entry has been selected.\n");
	[[NSApp mainViewController] executeCommand:@"reverse" ];
}

- (IBAction)silence:(id)sender
{
	vertl(commandsBenjamin,5,"INFO : the 'Edit | Silence' menu entry has been selected.\n");
	[[NSApp mainViewController] executeCommand:@"silence" ];
}

- (IBAction)normalize:(id)sender;
{
	vertl(commandsBenjamin,5,"INFO : the 'Edit | Normalize' menu entry has been selected.\n");
	[[NSApp mainViewController] executeCommand:@"normalize" ];
}

- (IBAction)changeGain:(id)sender;
{
	vertl(commandsBenjamin,5,"INFO : the 'Edit | Change Gain' menu entry has been selected.\n");
	[[NSApp mainViewController] executeCommand:@"gain" ];
}

- (IBAction)timeStretch:(id)sender;
{
	vertl(commandsBenjamin,5,"INFO : the 'Edit | Time Stretch' menu entry has been selected.\n");
	[[NSApp mainViewController] executeCommand:@"timeStretch" ];
}

- (IBAction)split:(id)sender;
{
	vertl(commandsBenjamin,5,"INFO : the 'Edit | Split' menu entry has been selected.\n");
	[[NSApp mainViewController] executeCommand:@"split" ];
}

- (IBAction)swapChannels:(id)sender
{
	vertl(commandsBenjamin,5,"INFO : the 'Edit | Swap Channels' menu entry has been selected.\n");
	[[NSApp mainViewController] executeCommand:@"swapChannels" ];
}

- (IBAction)adjustDCOffset:(id)sender
{
	vertl(commandsBenjamin,5,"INFO : the 'Edit | Adjust DC Offset' menu entry has been selected.\n");
	[[NSApp mainViewController] executeCommand:@"adjustDCOffset" ];
}

- (IBAction)invertPhase:(id)sender
{
	vertl(commandsBenjamin,5,"INFO : the 'Edit | Invert Phase' menu entry has been selected.\n");
	[[NSApp mainViewController] executeCommand:@"invertPhase" ];
}
- (IBAction)loopsmooth:(id)sender
{
	vertl(commandsBenjamin,5,"INFO : the 'Edit | Loop Smoothing' menu entry has been selected.\n");
	[[NSApp mainViewController] executeCommand:@"loopsmooth" ];
}

- (IBAction)insertSilence:(id)sender
{
	vertl(commandsBenjamin,5,"INFO : the 'Edit | Insert Silence' menu entry has been selected.\n");
	[[NSApp mainViewController] executeCommand:@"insertSilence" ];
}

- (IBAction)merge:(id)sender
{
	vertl(commandsBenjamin,5,"INFO : the 'Edit | Merge' menu entry has been selected.\n");
	[[NSApp mainViewController] executeCommand:@"merge" ];
}

- (IBAction)clear:(id)sender
{
	vertl(commandsBenjamin,5,"INFO : the 'Edit | Clear' menu entry has been selected.\n");
	[[NSApp mainViewController] executeCommand:@"clear" ];
}

- (IBAction)selectAll:(id)sender
{
	vertl(commandsBenjamin,5,"INFO : the 'Edit | Select All' menu entry has been selected.\n");
	[[NSApp mainViewController] selectAll];
}

///////////////////////////////////////////////////////////////////////////
/////																  /////
//								RECORD/PLAYBACK MENU												 
/////																  /////
///////////////////////////////////////////////////////////////////////////

- (IBAction)moveToDocumentStart:(id)sender
{
}

- (IBAction)moveToDocumentEnd:(id)sender
{
}

- (IBAction)startPlaying:(id)sender
{
    // $$$BENJA play button was pressed
    TArmeDoc	*theDocument;

	// MIMontageDoc             //TArmeDoc
    theDocument = [[[NSDocumentController sharedDocumentController] currentDocument] armeDocument];
	if ( theDocument )
	{
		if ( !theDocument->IsPlaying() )
		{
			[[NSNotificationCenter defaultCenter] postNotificationName:MTGMovieStartedPlayingNotification object:self];
			theDocument->Play();
		}
		else
		{
			[[NSNotificationCenter defaultCenter] postNotificationName:MTGMovieStoppedPlayingNotification object:self];
			theDocument->Stop();
			[[[NSDocumentController sharedDocumentController] currentDocument] restartGenerationOfThumbnails];
		}
		[self updatePlayMenuItem];
	}
	[[MITimePaletteController sharedTimePaletteController] startContinualUpdate];
}

- (void)updatePlayMenuItem
{
	static NSMenuItem* menuItem = [NSApp searchForMenuItemWithTag:MTGPlayPauseMenuTag];
	TArmeDoc	*theDocument = [[[NSDocumentController sharedDocumentController] currentDocument] armeDocument];
	if ( theDocument->IsPlaying() )
	{
		[menuItem setTitle:@"Pause                                         space"];
	}
	else {
		[menuItem setTitle:@"Play                                           space"];
	}

}

- (void)spaceKeyCommand:(NSString*)key withModifiers:(unsigned int)modifiers {
	[self startPlaying:nil];
}

- (IBAction)pausePlaying:(id)sender
{
	// $$$BENJA play/pause button was pressed
    TArmeDoc	*theDocument;

	// MIMontageDoc             //TArmeDoc
    theDocument = [[[NSDocumentController sharedDocumentController] currentDocument] armeDocument];
    if ( theDocument->IsPlaying() )
	{
		[[NSNotificationCenter defaultCenter] postNotificationName:MTGMovieStoppedPlayingNotification object:self];
		theDocument->Stop();
		[[[NSDocumentController sharedDocumentController] currentDocument] restartGenerationOfThumbnails];
	}
	[self updatePlayMenuItem];
	[[MITimePaletteController sharedTimePaletteController] stopContinualUpdate];
}

- (void)returnKeyCommand:(NSString*)key withModifiers:(unsigned int)modifiers {
	[self stopPlaying:nil];
}

- (void)backspaceKeyCommand:(NSString*)key withModifiers:(unsigned int)modifiers 
{
	[self delete:nil];
//	ZzAssertMsg(0,"backspace has been pressed");
}

- (IBAction)stopPlaying:(id)sender
{
	// $$$BENJA stop button was pressed
    TArmeDoc	*theDocument;

	// MIMontageDoc             //TArmeDoc
    theDocument = [[[NSDocumentController sharedDocumentController] currentDocument] armeDocument];
	if ( theDocument ) {
		if ( theDocument->IsPlaying() )
		{
			[[NSNotificationCenter defaultCenter] postNotificationName:MTGMovieStoppedPlayingNotification object:self];
			theDocument->Stop();
			[[[NSDocumentController sharedDocumentController] currentDocument] restartGenerationOfThumbnails];
		}
		[self updatePlayMenuItem];
		if ( theDocument->IsLooping() )
			[self goToLeftLocator:sender];
		else
			[self gotoDocStart:sender];		
	}
	[[MITimePaletteController sharedTimePaletteController] stopContinualUpdate];
}

- (IBAction)gotoDocEnd:(id)sender
{
    TArmeDoc				*theDocument;
    NSEnumerator			*mainViewEnum;
    MIMainViewController	*aController;

    theDocument = [[[NSDocumentController sharedDocumentController] currentDocument] armeDocument];
	theDocument->SeekToEnd();
	[[NSApp mainViewController] updatePlayBar];
	[[NSApp mainViewController] updateLocators];

    mainViewEnum = [[[[NSDocumentController sharedDocumentController] currentDocument] windowControllers] objectEnumerator];
    while ( aController = [mainViewEnum nextObject] )
    {
		[aController updatePlayBar];
		[((MIOverviewController*)[MIOverviewController sharedOverviewController])->fView setNeedsDisplay:YES];

		if ( fFollowPlayBar )
			[[aController topLevelContainerView] moveVisibleRectToPlayBar];
	}
	
}

- (IBAction)toggleLoop:(id)sender
{
	MIMontageDocument*	montageDocument;
    TArmeDoc*			theDocument;

	montageDocument = [[NSDocumentController sharedDocumentController] currentDocument];
    theDocument = [montageDocument armeDocument];
	theDocument->ToggleLoop();
	[montageDocument updateDocumentChangeCount:MIChangeDone];
	[[MIPlayStopPauseController sharedPlayStopPauseController] updateLoopButton:[self isLooping]];
}

- (BOOL)isLooping
{
	MIMontageDocument*	montageDocument;
    TArmeDoc*			theDocument;

	montageDocument = [[NSDocumentController sharedDocumentController] currentDocument];
    theDocument = [montageDocument armeDocument];
	if ( theDocument && theDocument->IsLooping() ) 
		return YES ;
	return NO;
}

/*
- (IBAction)gotoLeftLocator:(id)sender
{
    TArmeDoc				*theDocument;
    NSEnumerator			*mainViewEnum;
    MIMainViewController	*aController;

    theDocument = [[[NSDocumentController sharedDocumentController] currentDocument] armeDocument];
	
	theDocument->Seek([[[NSApp mainViewController] topLevelContainerView] getLeftLocatorTime]);
	[[NSApp mainViewController] updatePlayBar];
	[[NSApp mainViewController] updateLocators];

    mainViewEnum = [[[[NSDocumentController sharedDocumentController] currentDocument] windowControllers] objectEnumerator];
    while ( aController = [mainViewEnum nextObject] )
    {
		[aController updatePlayBar];
		[((MIOverviewController*)[MIOverviewController sharedOverviewController])->fView setNeedsDisplay:YES];

		if ( fFollowPlayBar )
			[[aController topLevelContainerView] moveVisibleRectToPlayBar];
	}
}
*/

- (IBAction)gotoDocStart:(id)sender
{
    TArmeDoc				*theDocument;
    NSEnumerator			*mainViewEnum;
    MIMainViewController	*aController;

    theDocument = [[[NSDocumentController sharedDocumentController] currentDocument] armeDocument];
	theDocument->SeekToStart();
	theDocument->Preroll(theDocument->GetPlayPos());
	[[NSApp mainViewController] updatePlayBar];
	[[NSApp mainViewController] updateLocators];

    mainViewEnum = [[[[NSDocumentController sharedDocumentController] currentDocument] windowControllers] objectEnumerator];
    while ( aController = [mainViewEnum nextObject] )
    {
		[aController updatePlayBar];
		[((MIOverviewController*)[MIOverviewController sharedOverviewController])->fView setNeedsDisplay:YES];

		if ( fFollowPlayBar )
			[[aController topLevelContainerView] moveVisibleRectToPlayBar];
	}
}


- (IBAction)goToLeftLocator:(id)sender
{
    TArmeDoc				*theDocument;
    NSEnumerator			*mainViewEnum;
    MIMainViewController	*aController;

    theDocument = [[[NSDocumentController sharedDocumentController] currentDocument] armeDocument];
	theDocument->Seek(theDocument->GetLLocator());
	theDocument->Preroll(theDocument->GetLLocator());
	[[NSApp mainViewController] updatePlayBar];
	[[NSApp mainViewController] updateLocators];

    mainViewEnum = [[[[NSDocumentController sharedDocumentController] currentDocument] windowControllers] objectEnumerator];
    while ( aController = [mainViewEnum nextObject] )
    {
		[aController updatePlayBar];
		[((MIOverviewController*)[MIOverviewController sharedOverviewController])->fView setNeedsDisplay:YES];

		if ( fFollowPlayBar )
			[[aController topLevelContainerView] moveVisibleRectToPlayBar];
	}	
}

- (IBAction)goToRightLocator:(id)sender
{
    TArmeDoc				*theDocument;
    NSEnumerator			*mainViewEnum;
    MIMainViewController	*aController;

    theDocument = [[[NSDocumentController sharedDocumentController] currentDocument] armeDocument];
	theDocument->Seek(theDocument->GetRLocator());
	theDocument->Preroll(theDocument->GetRLocator());
	[[NSApp mainViewController] updatePlayBar];
	[[NSApp mainViewController] updateLocators];

    mainViewEnum = [[[[NSDocumentController sharedDocumentController] currentDocument] windowControllers] objectEnumerator];
    while ( aController = [mainViewEnum nextObject] )
    {
		[aController updatePlayBar];
		[((MIOverviewController*)[MIOverviewController sharedOverviewController])->fView setNeedsDisplay:YES];

		if ( fFollowPlayBar )
			[[aController topLevelContainerView] moveVisibleRectToPlayBar];
	}	
}


///////////////////////////////////////////////////////////////////////////
/////																  /////
//								CREATE MENU												 
/////																  /////
///////////////////////////////////////////////////////////////////////////

- (void)createProcessor:(id)sender
{
	verntl(audioEffectsTom,6,"createProcessor (MIApplicationController)");

	MICreateAudioProcessorCommand	*cmd;
	MIWSSelection *sel = [[NSApp mainViewController] selection];

	cmd = [[[MICreateAudioProcessorCommand alloc] initWithDocument:[[NSDocumentController sharedDocumentController] currentDocument] selection:sel] autorelease];
	[cmd setParent:nil];
	[cmd setSet:[sender tag]/100 index:[sender tag]%100 name:[sender title]];
	[cmd setPosition:[[[NSApp mainViewController] topLevelContainerView] newObjectLocation]];

	VUSERCMD("Create AudioProcessor");
	VUSERCMD_PRINT_SELECTION([[NSApp mainViewController] selection]);

	[[[NSDocumentController sharedDocumentController] currentDocument] doCmd:cmd];

	[[cmd getCreatedObject] setRepresentationsNeedDisplay];
	if ( [[MIPreferences sharedPrefs] gridShowMask] & kPrefsGridShowVObjLines ||
		 [[MIPreferences sharedPrefs] gridShowMask] & kPrefsGridShowHObjLines )
		[[MIApplicationController sharedApplicationController] redrawAllMainWindows];

	[self modelChangedForDocument:[[NSDocumentController sharedDocumentController] currentDocument]];
	if ( [sel count] == 0 )
		[[[NSApp mainViewController] topLevelContainerView] scrollToNewObject];
}

- (IBAction)createAudioClip:(id)sender
{
    int						result;
    NSMutableArray			*fileTypes;
    NSOpenPanel				*oPanel = [NSOpenPanel openPanel];
	int						i;
	int						count;
//	MIObject				*theNewObject;
//	NSRect					newObjFrame;
	MICreateWSObjsFromFilesCommand	*createClipsCommand;
	
	fileTypes = [[NSMutableArray alloc] initWithObjects:nil];
    [fileTypes autorelease];

	count = TArmeMediaEngine::GetSupportedExtensionsCount();
	for ( i = 0; i < count ; i++ )
	{
		[fileTypes addObject:[NSString stringWithCString:(TArmeMediaEngine::GetSupportedExtension(i))]];
	}
	
    [oPanel setAllowsMultipleSelection:YES];
    result = [oPanel runModalForDirectory:nil file:nil types:fileTypes];
    if (result == NSOKButton)
	{
        NSArray 			*filesToOpen = [oPanel filenames];


		createClipsCommand = [[[MICreateWSObjsFromFilesCommand alloc] 
			initWithFiles:filesToOpen controller:[NSApp mainViewController]] autorelease];
		
		MIPoint position = [[[NSApp mainViewController] topLevelContainerView] newObjectLocation];
			
		[createClipsCommand setFirstPosition:position];
		[createClipsCommand setParent:[[[NSApp mainViewController] topLevelContainerView] representedObject]];
		[[[NSDocumentController sharedDocumentController] currentDocument] doCmd:createClipsCommand];
		if ( [[MIPreferences sharedPrefs] gridShowMask] & kPrefsGridShowVObjLines ||
			[[MIPreferences sharedPrefs] gridShowMask] & kPrefsGridShowHObjLines )
			[[MIApplicationController sharedApplicationController] redrawAllMainWindows];
		[[[NSApp mainViewController] topLevelContainerView] scrollToNewObject];
    }
	
	[self modelChangedForDocument:[[NSDocumentController sharedDocumentController] currentDocument]];

}

- (IBAction)createNewContainer:(id)sender
{
	MICreateContainerCommand	*cmd;

	cmd = [[[MICreateContainerCommand alloc] initWithSelection:[[NSApp mainViewController] selection]] autorelease];
	[cmd setParent:nil];
	[cmd setPosition:[[[NSApp mainViewController] topLevelContainerView] newObjectLocation]];

	VUSERCMD("Create Container");
	VUSERCMD_PRINT_SELECTION([[NSApp mainViewController] selection]);

	[[[NSDocumentController sharedDocumentController] currentDocument] doCmd:cmd];
	if ( [[MIPreferences sharedPrefs] gridShowMask] & kPrefsGridShowVObjLines ||
		 [[MIPreferences sharedPrefs] gridShowMask] & kPrefsGridShowHObjLines )
		[[MIApplicationController sharedApplicationController] redrawAllMainWindows];

	[[cmd getCreatedObject] setRepresentationsNeedDisplay];

	[self modelChangedForDocument:[[NSDocumentController sharedDocumentController] currentDocument]];

	if ( ![[[NSApp mainViewController] selection] count] )
		[[[NSApp mainViewController] topLevelContainerView] scrollToNewObject];
}

- (IBAction)createVideoClip:(id)sender
{
	// called by the Create Clip From Selection menuItem
	vertl(commandsBenjamin,5,"INFO : the 'Edit | Create Clip From Seleciton' menu entry has been selected.\n");
	[[NSApp mainViewController] executeCommand:@"createClipFromSelection" ];
}

- (IBAction)createTextClip:(id)sender
{
	vertl(commandsBenjamin,5,"INFO : the 'Text | new text clip' menu entry has been selected.\n");
	[[NSApp mainViewController] executeCommand:@"createTextClip" ];


/*	TArmeDoc				*armeDoc;
	MIMontageDocument		*mDoc;
	TArmeWSObj				*newArmeObj;
	TArmeWSObjPos			newPos;
	MIObject				*newObj;
	NSPoint					visibleRectOrigin;

	mDoc = [[NSDocumentController sharedDocumentController] currentDocument];
    armeDoc = [mDoc armeDocument];

	// GP : release the MITextController.
	MITextController *	theMITextController = [[MITextController alloc] init];
	newArmeObj = [theMITextController doTextClipDialog:armeDoc];
	[theMITextController release];

	if ( !newArmeObj )
	{
		verntl(interfaceStephan,0,"Null object returned while trying to create a text clip\n\r");
		return;
	}

	visibleRectOrigin = [[[NSApp mainViewController] topLevelContainerView] visibleRect].origin;

	newArmeObj->GetDoc()->LockRead();
	newPos = newArmeObj->GetPos();
	newArmeObj->GetDoc()->UnlockRead();

	newPos.fStartTime = [[[NSApp mainViewController] topLevelContainerView] pixelToArmeTime:(visibleRectOrigin.x+10)];
	newPos.fVerticalPos = GET_VALUE([[[NSApp mainViewController] topLevelContainerView] pixelToMIVPos:(visibleRectOrigin.y+10)]);

	newArmeObj->SetPos(newPos);


    newObj = [[MIObject alloc] initWithArmeWSObj:newArmeObj];
    [newObj setMontageDoc:mDoc];
	[[mDoc topLevelContainer] addChild:newObj];

	[mDoc lockWrite];
	[newObj setVerticalPos:[[[NSApp mainViewController] topLevelContainerView] pixelToMIVPos:(visibleRectOrigin.y+10)]];
	[mDoc unlockWrite];

	[newObj createAllRepresentations];
    [newObj setRepresentationsNeedDisplay];

	[self modelChangedForDocument:[[NSDocumentController sharedDocumentController] currentDocument]];*/
}

- (IBAction)createMidiClip:(id)sender
{
}

- (IBAction)createMarker:(id)sender
{
}

- (IBAction)createAutoLoopPoints:(id)sender
{
}

- (IBAction)advancedFadeIn:(id)sender
{
	vertl(commandsBenjamin,5,"INFO : the 'Edit | Fade In (advanced)' menu entry has been selected.\n");
	[[NSApp mainViewController] executeCommand:@"advancedFadeIn" ];
}

- (IBAction)advancedFadeOut:(id)sender
{
	vertl(commandsBenjamin,5,"INFO : the 'Edit | Fade Out (advanced)' menu entry has been selected.\n");
	[[NSApp mainViewController] executeCommand:@"advancedFadeOut" ];

}

- (IBAction)fadeIn:(id)sender
{
	vertl(commandsBenjamin,5,"INFO : the 'Edit | Fade In' menu entry has been selected.\n");
	[[NSApp mainViewController] executeCommand:@"fadeIn" ];
}

- (IBAction)fadeOut:(id)sender
{
	vertl(commandsBenjamin,5,"INFO : the 'Edit | Fade Out' menu entry has been selected.\n");
	[[NSApp mainViewController] executeCommand:@"fadeOut" ];

}

- (IBAction)customFade:(id)sender
{
}

- (IBAction)createVideoEffect:(id)sender
{
	vertl(commandsBenjamin,5,"INFO : the 'Debug | Create Generic Video Processor' menu entry has been selected.\n");
	[[NSApp mainViewController] executeCommand:@"createVideoProc" ];

#if 0


//	int 					tag = [sender tag];
//	int 					setIndex = tag/100;
//	int 					pluginIndex = tag%100;
	TArmeDoc				*armeDoc = NULL; // replace with the current doc
	TArmeWSObj				*newArmeObj;
	MIMontageDocument		*mDoc;
	TArmeWSObjPos			newPos;
	MIObject				*newObj;
	NSEnumerator			*mainViewEnumerator;
	MIMainViewController	*aMainViewController;
	NSPoint					visibleRectOrigin;


/*

			THIS SHOULD GO IN A COMMAND
			
			PLEASE LOOK AT MICREATEAUDIOPROCESSORCOMMAND FOR INFO
			
			STEPHAN

*/


	
	mDoc = [[NSDocumentController sharedDocumentController] currentDocument];
	armeDoc = [mDoc armeDocument];
	
	newArmeObj = new TVideoEffect(armeDoc, NULL ); // <--- armeDoc : ??????

	if ( !newArmeObj )
	{
		verntl(interfaceStephan,0,"Null objct returned while trying to create a processor");
		return;
	}
	
	// ... create the MIObject 
	visibleRectOrigin = [[[NSApp mainViewController] topLevelContainerView] visibleRect].origin;
	
	newArmeObj->GetDoc()->LockRead();
		newPos = newArmeObj->GetPos();
	newArmeObj->GetDoc()->UnlockRead();
	
	newPos.fParent = newArmeObj->GetDoc()->GetTopContainer();
	newPos.fInput = 0;
	newPos.fStartTime = [[[NSApp mainViewController] topLevelContainerView] pixelToArmeTime:(visibleRectOrigin.x+10)];
	
    newArmeObj->SetPos(newPos);
    newArmeObj->_SetLength([[NSApp mainViewController] pixelToArmeTime:200.0]);
	
    newObj = [[MIObject alloc] initWithArmeWSObj:newArmeObj];
    [newObj setMontageDoc:mDoc];
	[[mDoc topLevelContainer] addChild:newObj];
    
	[mDoc lockWrite];
		[newObj setVerticalPos:[[[NSApp mainViewController] topLevelContainerView] pixelToMIVPos:(visibleRectOrigin.y+10)]];
	[mDoc unlockWrite];
	
	mainViewEnumerator	= [[mDoc windowControllers] objectEnumerator];
	while ( aMainViewController = [mainViewEnumerator nextObject] )
	{
		[newObj createRepresentationInView:[aMainViewController topLevelContainerView]];
	}
	[[newObj findRepresentationInTLC:[[NSApp mainViewController] topLevelContainerView]] makeSelfEncloseCurrentSelection];
	
    [newObj setRepresentationsNeedDisplay];

	[self modelChangedForDocument:[[NSDocumentController sharedDocumentController] currentDocument]];
#endif
}



///////////////////////////////////////////////////////////////////////////
/////																  /////
//								VIEW MENU												 
/////																  /////
///////////////////////////////////////////////////////////////////////////

- (IBAction)showPreferences:(id)sender
{
	NSModalSession session;
	NSWindow* 	win;

	MIPreferencesController* dlgController = [MIPreferencesController alloc];
	[dlgController initWithWindowNibName:@"MIPreferencesDlg" owner:dlgController];
	win = [dlgController window];
	[win setDelegate:dlgController];
	
	session = [NSApp beginModalSessionForWindow:win];
	while([NSApp runModalSession:session] == NSRunContinuesResponse)
	{
	}
	[NSApp endModalSession:session];					
	[win orderOut:0];
	[dlgController release];
	[[MIPreferences sharedPrefs] saveToUserSettings];
	
	// tell all the views to redraw themselves
	
	
}

- (IBAction)showDVCapture:(id)sender
{
	[[MIDVCaptureController sharedDVCaptureController] show:sender];
}

- (IBAction)showAudioRecording:(id)sender
{
	[[MIAudioRecController sharedAudioRecController] show:sender];
}



///////////////////////////////////////////////////////////////////////////
/////																  /////
//								ARRANGE MENU												 
/////																  /////
///////////////////////////////////////////////////////////////////////////

- (IBAction)shrinkToFit:(id)sender
{
	vertl(commandsBenjamin,5,"INFO : the 'Arrange | shrink to fit' menu entry has been selected.\n");
	[[NSApp mainViewController] executeCommand:@"shrinkToFit" ];
}

- (IBAction)collapse:(id)sender
{
	vertl(commandsBenjamin,5,"INFO : the 'Arrange | collapse' menu entry has been selected.\n");
	[[NSApp mainViewController] executeCommand:@"collapse" ];
}

- (IBAction)expand:(id)sender
{
	vertl(commandsBenjamin,5,"INFO : the 'Arrange | expand' menu entry has been selected.\n");
	[[NSApp mainViewController] executeCommand:@"expand" ];
}

- (IBAction)chain:(id)sender
{
	MIMainViewController *controller = [NSApp mainViewController];
	if ( controller )
	{
		MIMontageDocument *document;
		MIMacroCommand *macroCommand;
		MIMoveCommand *moveCommand;
		MIWSSelection *selection = [controller selection];
		MIObjectRep* representation;
		MIObject *object;
		TArmeWSObj * wsObject;
		ArmeTime absoluteEndxTime;
		ArmeTime absoluteStartTime;
		ArmeTime duration;
		MIPoint offset;
		unsigned index;
		
		if ( [selection count] < 2 ) return ;

		VUSERCMD("Chain");
		VUSERCMD_PRINT_SELECTION(selection);

		macroCommand = [[[MIMacroCommand alloc] init] autorelease];
		
		representation = [selection objectAtIndex:0];
		object = [representation representedObject];
		wsObject = [object WSObject];
		
		wsObject->GetDoc()->LockRead();
		absoluteEndxTime = wsObject->GetAbsoluteEndxTime();
		wsObject->GetDoc()->UnlockRead();
		
		[selection discardVideoTransitions];
		
		for ( index = 1 ; index < [selection count] ; index++ ) {
			representation = [selection objectAtIndex:index];
			object = [representation representedObject];
			wsObject = [object WSObject];

			wsObject->GetDoc()->LockRead();
			absoluteStartTime = wsObject->GetAbsoluteStartTime();
			duration = wsObject->GetDuration();
			wsObject->GetDoc()->UnlockRead();

			GET_VALUE(offset.time) = GET_VALUE(absoluteEndxTime) - GET_VALUE(absoluteStartTime);
			GET_VALUE(offset.vpos) = 0;

			moveCommand = [[[MIMoveCommand alloc] initWithSelection:nil] autorelease];
			[moveCommand setObject:[representation representedObject]];
			[moveCommand setOffset:offset];
			[moveCommand setNewParent:[object parentObject]];

			[macroCommand addCommand:moveCommand];
				
			absoluteEndxTime += duration;
		}

		document = [controller document];
		[macroCommand setCommandName:@"Chain"];
		[macroCommand modifiedAudioHierarchy];
		[macroCommand modifiedVideoHierarchy];
		[macroCommand overviewNeedsUpdate];

		[document doCmd:macroCommand];
	}
}

- (NSArray*)childrenOfObjectRep:(MIObjectRep*)rep
{
	MIObject *obj = [rep representedObject];
	NSEnumerator *enumerator = [[obj getChildren] objectEnumerator];
	NSMutableArray *oArray = [NSMutableArray array];
	MITopLevelContainerRep *TLC = [rep getTopLevelContainerRep];

	while (( obj = [enumerator nextObject] ))
	{
		[oArray addObject:[obj findRepresentationInTLC:TLC]];
	}
	return oArray;
}

- (MIRect)absoluteModelFrameForObjectRep:(MIObjectRep*)rep
{
	MIObject *obj = [rep representedObject];
	if ( [obj isContainer] )
	{
		NSArray *children = [self childrenOfObjectRep:rep];
		MIObjectRep *rep2 = [self highestObjectRepFromArray:children];
		MIRect childRect = [self absoluteModelFrameForObjectRep:rep2];
		MIRect containerRect = [obj absoluteModelFrame];
		MIRect returnRect = MIMakeRect( containerRect.origin.time, containerRect.origin.vpos, containerRect.size.duration, (childRect.origin.vpos + childRect.size.height) - containerRect.origin.vpos );
		return returnRect;
	}
	else return [obj absoluteModelFrame];
}

- (MIObjectRep*)highestObjectRepFromArray:(NSArray*)array
{
	NSEnumerator *enumerator = [array objectEnumerator];
	MIObjectRep *rep;
	MIRect resultRect = MIMakeRect( ARME_TIME_MIN, VPOS_MIN, ARME_TIME_ZERO, VPOS_ZERO) ;
	MIObjectRep *resultRep = nil;

	while (( rep = [enumerator nextObject] ))
	{
		//MIRect rect = [[rep representedObject] absoluteModelFrame];
		MIRect rect = [self absoluteModelFrameForObjectRep:rep];

		if ( GET_VALUE(rect.origin.vpos) + GET_VALUE(rect.size.height) > GET_VALUE(resultRect.origin.vpos)  + GET_VALUE(resultRect.size.height) )
		{
			resultRect = rect;
			resultRep = rep;
		}
	}
	return resultRep;
}



- (MIRect)stackRect:(MIRect)refRect underLowestRectFromArray:(NSArray*)array
{
	NSEnumerator *enumerator = [array objectEnumerator];
	MIRectObject *rectObject;
	MIRect resultRect = refRect ; // refRect must have it's origin.vpos at the same value than the invariant object's origin.vpos
	BOOL wasMoved = NO;
	
	while (( rectObject = [enumerator nextObject] ))
	{
		MIRect rect = [rectObject rect];

		if ( !( ( GET_VALUE(rect.origin.time) + GET_VALUE(rect.size.duration) <= GET_VALUE(refRect.origin.time) )
		  || ( GET_VALUE(rect.origin.time) >= GET_VALUE(refRect.origin.time) + GET_VALUE(refRect.size.duration) ) ) )
		{
			if ( GET_VALUE(rect.origin.vpos) <= GET_VALUE(resultRect.origin.vpos) )
			{
				GET_VALUE(resultRect.origin.vpos) = GET_VALUE(rect.origin.vpos);
				wasMoved = YES;
			}
		}
	}

	if ( !wasMoved ) return resultRect;

	GET_VALUE(resultRect.origin.vpos) = GET_VALUE(resultRect.origin.vpos) - GET_VALUE(resultRect.size.height);
	return resultRect;
}


- (IBAction)stack:(id)sender
{
	MIMainViewController *controller = [NSApp mainViewController];
	if ( controller ) {
		MIWSSelection *selection = [controller selection];
		NSMutableArray *oArray;
		NSMutableArray *vArray;
		MIObjectRep* rep;
		MIRect topObjectRect;
		MIRect objectRect;
		MIRect newObjectRect;
		MIMontageDocument *document;
		MIMacroCommand *macroCommand;
		MIMoveCommand *moveCommand;
		MIPoint offset;
		MIObject *obj;
		
		[selection discardVideoTransitions];
		
		if ( [selection count] < 2 ) return ;

		VUSERCMD("Stack");
		VUSERCMD_PRINT_SELECTION(selection);

		document = [controller document];
		macroCommand = [[[MIMacroCommand alloc] init] autorelease];
		oArray = [[NSMutableArray alloc] initWithArray:[selection objArray]];
		vArray = [[NSMutableArray alloc] initWithCapacity:[oArray count]];

		rep = [self highestObjectRepFromArray:oArray];
		obj = [rep representedObject];

		topObjectRect = [self absoluteModelFrameForObjectRep:rep];

		[vArray addObject:[MIRectObject rectObjectWithRect:topObjectRect]];
		[oArray removeObject:rep];
		
		while ( [oArray count] > 0 ) {
			rep = [self highestObjectRepFromArray:oArray];
			objectRect = [self absoluteModelFrameForObjectRep:rep];
			newObjectRect = objectRect;
			GET_VALUE(newObjectRect.origin.vpos) = GET_VALUE(topObjectRect.origin.vpos);
			newObjectRect = [self stackRect:newObjectRect underLowestRectFromArray:vArray];
			[vArray addObject:[MIRectObject rectObjectWithRect:newObjectRect]];

			moveCommand = [[[MIMoveCommand alloc] initWithSelection:nil] autorelease];
			[moveCommand setObject:[rep representedObject]];
			GET_VALUE(offset.time) = 0;
			GET_VALUE(offset.vpos) = GET_VALUE(newObjectRect.origin.vpos) - GET_VALUE(objectRect.origin.vpos);
			[moveCommand setOffset:offset];
			[moveCommand setNewParent:[[rep representedObject] parentObject]];
			[macroCommand addCommand:moveCommand];
			
			[oArray removeObject:rep];
		}
		
		[oArray release];
		[vArray release];

		[macroCommand setCommandName:@"Stack"];
		[document doCmd:macroCommand];
	}
}


- (IBAction)paint:(id)sender
{
}

- (IBAction)alignLeft:(id)sender
{
	MIMainViewController *controller = [NSApp mainViewController];
	if ( controller ) {
		MIMontageDocument *document;
		MIMacroCommand *macroCommand;
		MIMoveCommand *moveCommand;
		MIWSSelection *selection = [controller selection];
		MIObjectRep* representation;
		MIObject *object;
		TArmeWSObj * wsObject;
		ArmeTime absoluteStartTime;
		ArmeTime referenceAbsoluteStartTime;
		MIPoint offset;
		unsigned index;

		[selection discardVideoTransitions];
		if ( [selection count] < 2 ) return ;

		VUSERCMD("Align Left");
		VUSERCMD_PRINT_SELECTION(selection);

		macroCommand = [[[MIMacroCommand alloc] init] autorelease];

		representation = [selection objectAtIndex:0];
		object = [representation representedObject];
		wsObject = [object WSObject];

		wsObject->GetDoc()->LockRead();
		referenceAbsoluteStartTime = wsObject->GetAbsoluteStartTime();
		wsObject->GetDoc()->UnlockRead();

		for ( index = 1 ; index < [selection count] ; index++ ) {
			representation = [selection objectAtIndex:index];
			object = [representation representedObject];
			wsObject = [object WSObject];

			wsObject->GetDoc()->LockRead();
			absoluteStartTime = wsObject->GetAbsoluteStartTime();
			wsObject->GetDoc()->UnlockRead();

			GET_VALUE(offset.time) = GET_VALUE(referenceAbsoluteStartTime) - GET_VALUE(absoluteStartTime);
			GET_VALUE(offset.vpos) = 0;

			moveCommand = [[[MIMoveCommand alloc] initWithSelection:nil] autorelease];
			[moveCommand setObject:[representation representedObject]];

			[moveCommand setOffset:offset];
			[moveCommand setNewParent:[object parentObject]];

			[macroCommand addCommand:moveCommand];
		}

		document = [controller document];
		[macroCommand setCommandName:@"Align Left"];
		[document doCmd:macroCommand];
	}
}

- (IBAction)alignRight:(id)sender
{
	MIMainViewController *controller = [NSApp mainViewController];
	if ( controller ) {
		MIMontageDocument *document;
		MIMacroCommand *macroCommand;
		MIMoveCommand *moveCommand;
		MIWSSelection *selection = [controller selection];
		MIObjectRep* representation;
		MIObject *object;
		TArmeWSObj * wsObject;
		ArmeTime absoluteEndxTime;
		ArmeTime referenceAbsoluteEndxTime;
		MIPoint offset;
		unsigned index;

		[selection discardVideoTransitions];
		if ( [selection count] < 2 ) return ;

		VUSERCMD("Align Right");
		VUSERCMD_PRINT_SELECTION(selection);

		macroCommand = [[[MIMacroCommand alloc] init] autorelease];

		representation = [selection objectAtIndex:0];
		object = [representation representedObject];
		wsObject = [object WSObject];

		wsObject->GetDoc()->LockRead();
		referenceAbsoluteEndxTime = wsObject->GetAbsoluteStartTime() + wsObject->GetDuration();
		wsObject->GetDoc()->UnlockRead();

		for ( index = 1 ; index < [selection count] ; index++ ) {
			representation = [selection objectAtIndex:index];
			object = [representation representedObject];
			wsObject = [object WSObject];

			wsObject->GetDoc()->LockRead();
			absoluteEndxTime = wsObject->GetAbsoluteStartTime() + wsObject->GetDuration();
			wsObject->GetDoc()->UnlockRead();

			GET_VALUE(offset.time) = GET_VALUE(referenceAbsoluteEndxTime) - GET_VALUE(absoluteEndxTime);
			GET_VALUE(offset.vpos) = 0;

			moveCommand = [[[MIMoveCommand alloc] initWithSelection:nil] autorelease];
			[moveCommand setObject:[representation representedObject]];

			[moveCommand setOffset:offset];
			[moveCommand setNewParent:[object parentObject]];

			[macroCommand addCommand:moveCommand];
		}

		document = [controller document];
		[macroCommand setCommandName:@"Align Right"];
		[document doCmd:macroCommand];
	}
}

- (IBAction)lock:(id)sender
{
	vertl(commandsBenjamin,5,"INFO : the 'Arrange | lock' menu entry has been selected.\n");
	[[NSApp mainViewController] executeCommand:@"lock" ];
}

- (IBAction)unlock:(id)sender
{
	vertl(commandsBenjamin,5,"INFO : the 'Arrange | unlock' menu entry has been selected.\n");
	[[NSApp mainViewController] executeCommand:@"unlock" ];
}


- (IBAction)vZoomIn:(id)sender
{
    [[NSApp mainViewController] vZoomIn];
}

- (IBAction)vZoomOut:(id)sender;
{
    [[NSApp mainViewController] vZoomOut];
}

- (IBAction)hZoomIn:(id)sender
{
    [[NSApp mainViewController] hZoomIn];
}

- (IBAction)hZoomOut:(id)sender;
{
    [[NSApp mainViewController] hZoomOut];
}

/*
- (void)displayBOPProgress:(BOPTask*)bopTask
{
	NSMutableArray* array = [[NSMutableArray alloc] init];
	[array addObject:[NSValue valueWithPointer:(void*)bopTask]];
	[self displayBOPsProgress:array];
	[array release];
}

- (void)displayBOPsProgress:(NSMutableArray*)bopTaskArray
{
	NSModalSession session;
	NSWindow* 	win;

	if ( !fProgressDlgController )
	{
		fProgressDlgController = [MIProgressDlgController alloc];
		[fProgressDlgController initWithWindowNibName:@"ProgressDlg" owner:fProgressDlgController];
	}
	win = [fProgressDlgController window];
		
	for( SInt32 i = 0; i < (SInt32)[bopTaskArray count]; i++)
	{
		BOPTask* bopTask = (BOPTask*)[[bopTaskArray objectAtIndex:i] pointerValue];
		session = [NSApp beginModalSessionForWindow:win];
		[fProgressDlgController setText:[NSString stringWithCString:bopTask->GetName()]];
		float progress;
		while([NSApp runModalSession:session] == NSRunContinuesResponse)
		{
			progress = bopTask->GetProgress();
			[fProgressDlgController setProgress:progress];
			if(progress >= 1.0)
			{	
				[fProgressDlgController setDoubleValue:0.0];	
				[NSApp stopModal];
			}
			usleep(100000); // wait for 0.1 sec otherwise it's a busy loop
		}
		[NSApp endModalSession:session];					
	}
	[win orderOut:self];
}
*/

- (SInt32)displayChooseStackingMethodForMainViewController:(MIMainViewController*)controller
{
	NSPanel* 		sheet;

	if ( !fChooseStackingDlgController )
	{
		fChooseStackingDlgController = [MIChooseStackingDlgController alloc];
		[fChooseStackingDlgController initWithWindowNibName:@"ChooseStacking" owner:fChooseStackingDlgController];
	}
	sheet = (NSPanel*)[fChooseStackingDlgController window];

	[NSApp beginSheet:sheet modalForWindow:[controller window] modalDelegate:self didEndSelector:nil contextInfo:nil];
	
	[NSApp runModalForWindow:sheet];
	[NSApp endSheet:sheet];
	[sheet orderOut:nil];	// Causes sheet to roll back up
	return [fChooseStackingDlgController getChosenStackingMethod];
}

- (BOOL)blindlyExecuteAndDeleteSplitTask:(ZzSplitTask*)splitTask
{
	TArmeMediaEngine::BlindlyExecuteAndDeleteSplitTask(splitTask);
	return YES;
}

- (BOOL)executeAndDeleteSplitTask:(ZzSplitTask*)splitTask forMainViewController:(MIMainViewController*)controller
{
	if(!splitTask)
		return YES;
	
	MIProgressBarDlgManager mngr(splitTask->GetDesc());
	
	ArmeErr err = ZzSplitTask::ExecuteInProgressManagerContext(splitTask,&mngr);
	// ***marcp*** il faut grer les erreurs
	if(err.type != kArmeOK)
		return NO;
	
	return YES;
	
/*	
	if(!splitTask->IsLengthy())
		return [self blindlyExecuteAndDeleteSplitTask:(ZzSplitTask*)splitTask];
	
	NSModalSession 	session;

	if ( !fProgressDlgController )
	{
		fProgressDlgController = [MIProgressDlgController alloc];
		[fProgressDlgController initWithWindowNibName:@"ProgressDlg" owner:fProgressDlgController];
	}
	
	// if "controller" has a value, it means we the split task "comes" from a speficic doc window and 
	// we can attach a sheet to this window
	int retCode;
	float progress;
	
	if(controller)
	{
		NSPanel* 		sheet;
		sheet = [fProgressDlgController window];
		NSWindow* theWindow = [controller window];
		[NSApp beginSheet:sheet modalForWindow:[controller window] modalDelegate:self didEndSelector:nil contextInfo:nil];
		session = [NSApp beginModalSessionForWindow:theWindow];
		while((retCode = [NSApp runModalSession:session]) == NSRunContinuesResponse)
		{
			vernl(6,"executeAndDeleteSplitTask about to call top-level splitTask (" << (void*)splitTask << ") ::  GetProgress()");
			splitTask->_PerformNextBit();
			progress = splitTask->GetProgress();
			[fProgressDlgController setProgress:progress];
			if(progress >= 1.0)
			{	
				[fProgressDlgController setDoubleValue:0.0];	
				vernl(6,"****** executeAndDeleteSplitTask : PROGRESS > 1.0 ********");
				[NSApp stopModalWithCode:1];
			}
		}
		vernl(6,"****** executeAndDeleteSplitTask : SHEET OVER ********");
	
		[NSApp endModalSession:session];					
		[NSApp endSheet:sheet];
		[sheet orderOut:nil];	// Causes sheet to roll back up
	}
	else
	{
		NSWindow* dlgWindow = [fProgressDlgController window];
		session = [NSApp beginModalSessionForWindow:dlgWindow];
		while((retCode = [NSApp runModalSession:session]) == NSRunContinuesResponse)
		{
			vernl(6,"executeAndDeleteSplitTask about to call top-level splitTask (" << (void*)splitTask << ") ::  GetProgress()");
			splitTask->_PerformNextBit();
			progress = splitTask->GetProgress();
			[fProgressDlgController setProgress:progress];
			if(progress >= 1.0)
			{	
				[fProgressDlgController setDoubleValue:0.0];	
				[NSApp stopModalWithCode:1];
			}
		}
		[NSApp endModalSession:session];
		[dlgWindow orderOut:self];
	}
		
	if(retCode == 1) // stopped because progress >= 1.0
	{
		vernl(6,"executeAndDeleteSplitTask : modal session stopped because progress >= 1.0");
		delete splitTask;
		return YES;
	}
	else if(retCode == 0) // stopped because cancel button pressed
	{
		vernl(6,"executeAndDeleteSplitTask : modal session stopped because cancel button pressed");
		splitTask->Cancel();
		delete splitTask;
		return NO;
	}
	else
	{
		vernl(6,"executeAndDeleteSplitTask : modal session stopped because <UNKNOWN> return code");
		delete splitTask;
		ZzAssert(0);
		return NO;
	}
	*/
}

/*
- (void)deleteBOPs:(NSMutableArray*)bopTaskArray
{
	for( SInt32 i = 0; i < (SInt32)[bopTaskArray count]; i++)
	{
		BOPTask* bopTask = (BOPTask*)[[bopTaskArray objectAtIndex:i] pointerValue];
		delete bopTask;
	}
}
*/

/*
#ifndef TWO_THREADS_NOTIFICATIONS
static void* NotificationThreadEntryPoint(void* cookie)
{
	MIApplicationController* appController = (MIApplicationController*)cookie;
	MPQueueID queueID = [appController notificationQueueID];
	
	NSAutoreleasePool *pool =[[NSAutoreleasePool alloc] init];
	
	id proxy = [[NSConnection rootProxyForConnectionWithRegisteredName:@"Montager_MIApp" host:nil] retain];
	[proxy setProtocolForProxy:@protocol(MIApplicationControllerListener)];	
	
	[pool release];

	NSMutableArray* packedParams = [[NSMutableArray alloc] init];
	
	while(1)
	{
		id object;
		UInt32 code;
		void* param;
		pool = [[NSAutoreleasePool alloc] init];
		OSStatus st = MPWaitOnQueue(queueID, (void**)&code, (void**)&object, &param, kDurationForever);
		ZzAssert(st == noErr);
		if (st == noErr)
		{
			if( code == 0 && object == nil && param == NULL)
			{
				[pool release];
				[proxy release];
				[packedParams release];
				return NULL;
			}
			
			[packedParams removeAllObjects];
			// create data objects
			NSMutableData* dataCode = [NSMutableData dataWithLength:sizeof(int)];
			NSMutableData* dataObject = [NSMutableData dataWithLength:sizeof(id)];
			NSMutableData* dataParam = [NSMutableData dataWithLength:sizeof(void*)];
		
			// store our data in them
			memcpy([dataCode mutableBytes], &code, sizeof(int));
			memcpy([dataObject mutableBytes], &object, sizeof(id));
			memcpy([dataParam mutableBytes], &param, sizeof(void*));
			
			// add them to the array
			[packedParams addObject:dataCode];
			[packedParams addObject:dataObject];
			[packedParams addObject:dataParam];
			
			// send the message to the proxy. the array is automatically
			// marshalled
			[proxy notificationReceived:packedParams];
			[pool release];
		}
	}
}

- (void)notificationReceived:(NSArray*)packedParams
{
	// NSArray contains 3 NSValue
	// code which is int
	// object which is id
	// param which is void*

	int code;
	id object;
	void* param;
	
	NSData* dataCode = [packedParams objectAtIndex:0];
	NSData* dataObject = [packedParams objectAtIndex:1];
	NSData* dataParam = [packedParams objectAtIndex:2];
	
	memcpy(&code, [dataCode bytes], sizeof(int));
	memcpy(&object, [dataObject bytes], sizeof(id));
	memcpy(&param, [dataParam bytes], sizeof(void*));

			//NSTimeInterval
	[object performSelector:@selector(notificationReceived:) withObject:packedParams afterDelay:0.0];
}
	
- (void)sendNotification:(UInt32)code toObject:(id)obj withParam:(void*)param
{
//	OSStatus err = MPNotifyQueue(fNotificationQueueID,(void*)code,(void*)obj,param);
	ZzAssert(err == noErr);
}

- (MPQueueID)notificationQueueID
{
	return fNotificationQueueID;
}

- (void)filterQueue:(UInt32)code param:(void*)param
{
	// noErr or kMPTimeoutErr
	OSStatus err;
	
	const SInt32 kMaxQueueSizeForFiltering = 1024;
	
	SInt32	iMsg = 0;
	SInt32	i;
	UInt32	codes[kMaxQueueSizeForFiltering];
	id		objs[kMaxQueueSizeForFiltering];
	void*	params[kMaxQueueSizeForFiltering];
			
	do
	{
		err = MPWaitOnQueue(
			fNotificationQueueID,
			(void**)&(codes[iMsg]), (void**)&(objs[iMsg]), &(params[iMsg]),
			kDurationImmediate);
		if(err == noErr)
			iMsg++;
		else
		{
			ZzAssert(err == kMPTimeoutErr);
		}
	} while(err == noErr);
	
	for(i = 0; i < iMsg; i++)
	{
		if(codes[i] == code && params[i] == param)
			objs[i] = nil;
	}

	for(i = 0; i < iMsg; i++)
	{
		if(objs[i] != nil)
		{
//			err = MPNotifyQueue(fNotificationQueueID,(void*)(codes[i]),(void*)(objs[i]),params[i]);
			ZzAssert(err == noErr);
		}
	}
}
*/

- (void)notificationReceived:(UInt32)code param:(void*)param
{
	switch(code)
	{
	case kNotification_AdjustMovieArmeTime :
		{	
//LATER		vernl(6,"[MIApplicationController notificationReceived:kNotification_AdjustMovieArmeTime]");
			TArmeMediaEngine::LockReadDocArray();
			for(int iDoc = 0; iDoc < TArmeMediaEngine::GetNbDocs(); iDoc++)
			{
				TArmeDoc* doc = TArmeMediaEngine::GetDoc(iDoc);
				if(doc->DoesUpdateMovieArmeTime())
					AdjustMovieArmeTime(doc->fMasterMovie->GetQTMovie(), doc, doc->GetPlayPos());
			}
			TArmeMediaEngine::UnlockReadDocArray();
		}
		break;
	}
}

//#else //#ifndef TWO_THREADS_NOTIFICATIONS

- (void)sendNotification:(UInt32)code toObject:(id<MINotificationListener>)object withParam:(void*)param
{
	id pool = [[NSAutoreleasePool alloc] init];
	NSMutableArray* packedParams = [[NSMutableArray alloc] init];
	// create data objects
	NSMutableData* dataCode = [NSMutableData dataWithLength:sizeof(int)];
	NSMutableData* dataObject = [NSMutableData dataWithLength:sizeof(id)];
	NSMutableData* dataParam = [NSMutableData dataWithLength:sizeof(void*)];
		
	// store our data in them
	memcpy([dataCode mutableBytes], &code, sizeof(int));
	memcpy([dataObject mutableBytes], &object, sizeof(id));
	memcpy([dataParam mutableBytes], &param, sizeof(void*));
			
	// add them to the array
	[packedParams addObject:dataCode];
	[packedParams addObject:dataObject];
	[packedParams addObject:dataParam];
			
	// send the message to the proxy. the array is automatically
	// marshalled
	[self performSelectorOnMainThread:@selector(forwardNotification:) withObject:packedParams waitUntilDone:NO];
	[pool release];

	// this is executed in any thread context.
	// now we need to 

//	id pool = [[NSAutoreleasePool alloc] init];
//	[fProxySelf forwardNotificationTo:obj code:code param:(unsigned int)param];
//	[pool release];
}

// this is executed in the app thead context.
- (void)forwardNotification:(id)packedParams
{
	int code;
	id object;
	void* param;
	
	NSData* dataCode = [packedParams objectAtIndex:0];
	NSData* dataObject = [packedParams objectAtIndex:1];
	NSData* dataParam = [packedParams objectAtIndex:2];
	
	memcpy(&code, [dataCode bytes], sizeof(int));
	memcpy(&object, [dataObject bytes], sizeof(id));
	memcpy(&param, [dataParam bytes], sizeof(void*));
	[object notificationReceived:code param:param];
}

//#endif //#ifndef TWO_THREADS_NOTIFICATIONS

/*
- (void)purgeQueueMessagesFor:(id)object
{
#ifndef TWO_THREADS_NOTIFICATIONS
	// noErr or kMPTimeoutErr
	OSStatus err;
	
	const SInt32 kMaxQueueSizeForFiltering = 1024;
	
	SInt32	iMsg = 0;
	SInt32	i;
	UInt32	codes[kMaxQueueSizeForFiltering];
	id		objs[kMaxQueueSizeForFiltering];
	void*	params[kMaxQueueSizeForFiltering];
			
	do
	{
		err = MPWaitOnQueue(
			fNotificationQueueID,
			(void**)&(codes[iMsg]), (void**)&(objs[iMsg]), &(params[iMsg]),
			kDurationImmediate);
		if(err == noErr)
			iMsg++;
		else
		{
			ZzAssert(err == kMPTimeoutErr);
		}
	} while(err == noErr);
	
	for(i = 0; i < iMsg; i++)
	{
		if(objs[i] == object)
			objs[i] = nil;
	}
	
	for(i = 0; i < iMsg; i++)
	{
		if(objs[i] != nil)
		{
//			err = MPNotifyQueue(fNotificationQueueID,(void*)(codes[i]),(void*)(objs[i]),params[i]);
			ZzAssert(err == noErr);
		}
	}
#endif //#ifndef TWO_THREADS_NOTIFICATIONS
}
*/

//- (void)createNotificationResources
//{
//	NSConnection *theConnection;
//		
//	theConnection = [NSConnection defaultConnection];
//	[theConnection setRootObject:self];
//	if ([theConnection registerName:@"Montager_MIApp"] == NO) 
//	{
//		NSRunAlertPanel(@"Error", @"Sorry, only one instance of HyperEngine-AV can run at the same time.", @"OK", nil, nil);
//		exit(0);
//	}
//	[theConnection enableMultipleThreads];
//	
//#ifndef TWO_THREADS_NOTIFICATIONS
//	OSStatus st = MPCreateQueue(&fNotificationQueueID);
//	ZzAssert(st == noErr);
//	pthread_create(&fNotificationThread,NULL,NotificationThreadEntryPoint,(void*)self);
//#else //#ifndef TWO_THREADS_NOTIFICATIONS
//	fProxySelf = [[NSConnection rootProxyForConnectionWithRegisteredName:@"Montager_MIApp" host:nil] retain];
//	[fProxySelf setProtocolForProxy:@protocol(MIApplicationControllerListener)];	
//#endif //#ifndef TWO_THREADS_NOTIFICATIONS
//}

-(void)emptyMethod:(id)object
{
}

- (IBAction)disableMainViewRedraw:(id)sender
{
	int state = [sender state];
	if(state == NSOffState)
	{
		[sender setState:NSOnState];
		fMainViewRedrawDisabled = YES;
	}
	else if(state == NSOnState)
	{
		[sender setState:NSOffState];
		fMainViewRedrawDisabled = NO;
	}
	else
		ZzAssert(0);
}

- (IBAction)showRedrawRects:(id)sender
{
	[[NSApp mainViewController] topLevelContainerView]->fDrawRedrawRects = ![[NSApp mainViewController] topLevelContainerView]->fDrawRedrawRects;
}


- (BOOL)mainViewRedrawDisabled
{
	return fMainViewRedrawDisabled;
}

- (MIObject*) clipBoard 
{
	return fClipBoard;
}

- (NSString*) tempDirectory
{
	return @"/tmp";
}

- (IBAction)displayVerboseConfigDlg:(id)sender
{
#ifndef NO_VERBOSE_AT_ALL
	NSModalSession	session;
	NSWindow* 		win;

	id dlgController = [MIVerboseConfigDlgController alloc];
	[dlgController initWithWindowNibName:@"MIVerboseConfigDlg" owner:dlgController];
	win = [dlgController window];

	session = [NSApp beginModalSessionForWindow:win];
	while([NSApp runModalSession:session] == NSRunContinuesResponse)
	{
		// you can avoid CPU usage by not polling the event loop all the time. Useful
		// if you have another thread performing some background work
//		usleep(200000);
	}
	[NSApp endModalSession:session];
	[win orderOut:0];
	[dlgController release];
	// saves the settings in the database
	[self saveVerboseSettings];
#else //NO_VERBOSE_AT_ALL
	NSRunAlertPanel(@"Error", @"This feature is not implemented. Please undefine NO_VERBOSE_AT_ALL in ZzVerbose.h to enable verbose debug.", @"OK", nil, nil );
#endif //NO_VERBOSE_AT_ALL
}

-(void)saveVerboseSettings
{
#ifndef NO_VERBOSE_AT_ALL
	for(SInt32 i = 0; i < kVERTopic_LastTopic - 1; i++)
	{
		int level = ZzVerbose::fsTopicLevels[i+1];
		// create an NSNumber with index
		NSString* topicKey = [NSString stringWithCString:ZzVerbose::fsTopicStrings[i+1]];
		[[NSUserDefaults standardUserDefaults] setInteger:level forKey:topicKey];
	}
	// add the output settings
	[[NSUserDefaults standardUserDefaults] setInteger:ZzVerbose::fsActiveOutputs forKey:@"activeOutputs"];

	[[NSUserDefaults standardUserDefaults] setObject:@"test_1" forKey:@"testString"];
#endif //NO_VERBOSE_AT_ALL
}

// this saves the default for the first time
+ (void)addVerboseDefaultsToDictionary:(NSMutableDictionary*)appDefaults 
{	
#ifndef NO_VERBOSE_AT_ALL
    
	for(SInt32 i = 0; i < kVERTopic_LastTopic - 1; i++)
	{
		int level = ZzVerbose::fsTopicLevels[i+1];
		// create an NSNumber with index
		NSString* topicKey = [NSString stringWithCString:ZzVerbose::fsTopicStrings[i+1]];
		[appDefaults setObject:[NSNumber numberWithInt:level] forKey:topicKey];
	}
	// add the output settings
	[appDefaults setObject:[NSNumber numberWithInt:ZzVerbose::fsActiveOutputs] forKey:@"activeOutputs"];
	
	[appDefaults setObject:@"test_default" forKey:@"testString"];
#endif //#ifndef NO_VERBOSE_AT_ALL
}

-(void)restoreVerboseSettings
{
#ifndef NO_VERBOSE_AT_ALL
	for(SInt32 i = 0; i < kVERTopic_LastTopic - 1; i++)
	{
		NSString* topicKey;
		topicKey= [NSString stringWithCString:(ZzVerbose::fsTopicStrings[i+1])];
		ZzVerbose::fsTopicLevels[i+1] = [[NSUserDefaults standardUserDefaults] integerForKey:topicKey];
		//$$$TODO handle the NULLTopic
	}
	//NSString* testString = [[NSUserDefaults standardUserDefaults] stringForKey:@"testString"];
	ZzVerbose::fsActiveOutputs = [[NSUserDefaults standardUserDefaults] integerForKey:@"activeOutputs"];
#endif //#ifndef NO_VERBOSE_AT_ALL
}

#undef VER_DEF_TOPIC
#define VER_DEF_TOPIC verboseTestBenja

#ifdef ALLOW_AUDIO_REP_DRAWING_DISABLED
extern BOOL gAudioRepDrawingDisabled;
#endif //ALLOW_AUDIO_REP_DRAWING_DISABLED

- (IBAction)benjaDebugActionOne:(id)sender
{
#ifdef ALLOW_AUDIO_REP_DRAWING_DISABLED
	int state = [sender state];
	if(state == NSOffState)
	{
		[sender setState:NSOnState];
		gAudioRepDrawingDisabled = YES;
	}
	else if(state == NSOnState)
	{
		[sender setState:NSOffState];
		gAudioRepDrawingDisabled = NO;
	}
	else
		ZzAssert(0);
#else //#ifdef ALLOW_AUDIO_REP_DRAWING_DISABLED
	NSRunAlertPanel(@"Error", @"This feature is implemented by #defining ALLOW_AUDIO_REP_DRAWING_DISABLED in MIAudioCLipRep.h", @"OK", nil, nil);
#endif //#ifdef ALLOW_AUDIO_REP_DRAWING_DISABLED
}

- (IBAction)benjaDebugActionTwo:(id)sender
{
	verntl(verboseTestBenja,0,"test verbose level 0");
	verntl(verboseTestBenja,1,"test verbose level 1");
	verntl(verboseTestBenja,2,"test verbose level 2");
	verntl(verboseTestBenja,3,"test verbose level 3");
	verntl(verboseTestBenja,4,"test verbose level 4");
	verntl(verboseTestBenja,5,"test verbose level 5");
	verntl(verboseTestBenja,6,"test verbose level 6");
}

- (IBAction)benjaDebugActionThree:(id)sender
{
	MIProgressBarDlgManager mngr("Testing Progress Bars...");
	mngr.SetNbSubTasks(3);
	mngr.DisplayDlg();
	bool bContinue = NO;
	for (int j=0; j < 3; j++)
	{
		mngr.EnterSubTask();
		for(int i = 0; i < 20; i++)
		{
//			usleep(100000);
			mngr.Update((float)(i+1)/20.0);
			//$$$EXC CATCH EXCEPTIONS !!!!!
		}
		mngr.LeaveSubTask();
	}
	mngr.Delete();
	if(!bContinue)
		NSRunAlertPanel(@"Information",@"This task has been cancelled",nil,nil,nil);
}


- (IBAction)toggleRedrawRects:(id)sender
{
	BOOL redrawRects = [[NSApp mainViewController] topLevelContainerView]->fDrawRedrawRects;
	[[NSApp mainViewController] topLevelContainerView]->fDrawRedrawRects = !redrawRects;
	if(redrawRects)
		[sender setState:NSOffState];
	else
		[sender setState:NSOffState];
}

void  AddExtensions(NSMutableArray* ioTypes, QTAtomContainer inMimeList)
{
    QTAtom mimeAtom;
    short mimeTypeInfoCount;
    short mimeTypeInfoIndex;
    long typeStringLength;
    char typeString[256];
    OSErr err = noErr;
    NSString* s;

    mimeTypeInfoCount = ::QTCountChildrenOfType( inMimeList, kParentAtomIsContainer, kMimeInfoFileExtensionTag);
    for (mimeTypeInfoIndex = 1; mimeTypeInfoIndex <= mimeTypeInfoCount; mimeTypeInfoIndex++ )
    {
        mimeAtom = ::QTFindChildByIndex( inMimeList, kParentAtomIsContainer, 
                kMimeInfoFileExtensionTag, mimeTypeInfoIndex, nil );
    
        // mime data is a comma-deliminated text string of extensions, size is its length
        err = ::QTCopyAtomDataToPtr( inMimeList, mimeAtom, true,
                sizeof(typeString)-1, typeString, &typeStringLength );
        if (err == noErr)
        {
            typeString[typeStringLength] = '\0';
            s = [[NSString alloc] initWithCString:typeString];
            if (strchr(typeString, ','))
            {
                NSArray *extensions = [s componentsSeparatedByString:@","];
                //[ioTypes addObjectsFromArray:extensions];
                id type;
                NSEnumerator *enumerator = [extensions objectEnumerator];
                while ((type = [enumerator nextObject]))
                    if ( [ioTypes indexOfObject:type] == NSNotFound )
                        [ioTypes addObject:type];
            }
            else if ( [ioTypes indexOfObject:s] == NSNotFound )
                [ioTypes addObject:s];
            [s release];
        }
    }
    ::QTDisposeAtomContainer( inMimeList );
}

- (NSArray*)quickTimeFileExtensions
{
    if ( !fQuickTimeFileExtensions )
    {
        NSMutableArray* extensions = [[NSMutableArray alloc] initWithObjects:@"mov", nil];

        ComponentDescription looking;
        looking.componentSubType = 0; // Any subtype is OK
        looking.componentManufacturer = 0; // Any manufacturer is OK
        Component c = nil;
        OSErr err = noErr;
        QTAtomContainer mimeList = nil;

        // first, we'll look for movies
        looking.componentType = MovieImportType;
        looking.componentFlags = canMovieImportFiles | hasMovieImportMIMEList;
        looking.componentFlagsMask = canMovieImportFiles | movieImportSubTypeIsFileExtension | hasMovieImportMIMEList;

        while( c = ::FindNextComponent( c, &looking ))
        {
            err = ::MovieImportGetMIMETypeList((MovieImportComponent)c, &mimeList );

            if (err == noErr)
                AddExtensions(extensions, mimeList);
        }

        // then, we'll look for graphics files
        looking.componentType = GraphicsImporterComponentType;;
        looking.componentFlags = hasMovieImportMIMEList;
        looking.componentFlagsMask = hasMovieImportMIMEList;

        while( c = ::FindNextComponent( c, &looking ))
        {
            err = ::GraphicsImportGetMIMETypeList((MovieImportComponent)c, &mimeList );
            if (!err)
                AddExtensions(extensions, mimeList);
        }

		// remove unwanted extensions
		//[extensions removeObjectsInArray:[NSArray arrayWithObjects:@"FLC", @"flc", @"Flc", nil]];
		
        // all done, return the list
        fQuickTimeFileExtensions = [[NSArray arrayWithArray:extensions] retain];
        [extensions release];
    }
    return fQuickTimeFileExtensions;
}



// returns an array of NSStrings of file Types QuickTime can import
- (NSArray*)quickTimeFileTypes
{
    if ( !fQuickTimeFileTypes )
    {
	NSMutableArray* types = [[NSMutableArray alloc] initWithObjects:@"'MooV'", nil];

	ComponentDescription looking;
	looking.componentSubType = 0; // Any subtype is OK
	looking.componentManufacturer = 0; // Any manufacturer is OK
	Component c = nil;

	// first, we'll look for movies
	looking.componentType = MovieImportType;
	looking.componentFlags = canMovieImportFiles ;
	looking.componentFlagsMask = canMovieImportFiles ;

	while( c = ::FindNextComponent( c, &looking ))
        {
            ComponentDescription cd;
            OSType fileType;
            NSString *str;
            ::GetComponentInfo( c, &cd, nil, nil, nil );
            fileType = cd.componentSubType;
            str = [NSString stringWithFormat:@"'%@'",[NSString stringWithCString:(char*)&fileType length:4]];
            if ( [types indexOfObject:str] == NSNotFound )
                [types addObject:str];
        }

	// then, we'll look for graphics files
	looking.componentType = GraphicsImporterComponentType;;
	looking.componentFlags = hasMovieImportMIMEList;
	looking.componentFlagsMask = hasMovieImportMIMEList;

	while( c = ::FindNextComponent( c, &looking ))
	{
            ComponentDescription cd;
            OSType fileType;
            NSString *str;
            ::GetComponentInfo( c, &cd, nil, nil, nil );
            fileType = cd.componentSubType;
            str = [NSString stringWithFormat:@"'%@'",[NSString stringWithCString:(char*)&fileType length:4]];
            if ( [types indexOfObject:str] == NSNotFound )
                [types addObject:str];
        }

	// remove unwanted file types
	//[types removeObjectsInArray:[NSArray arrayWithObjects:@"'FLI '", nil]];

	// all done, return the list
	fQuickTimeFileTypes = [[NSArray arrayWithArray:types] retain];
	[types release];
    }
    return fQuickTimeFileTypes;
}

- (NSArray*)fileExtensions
{
    if (!fFileExtensions)
    {
	NSMutableArray *array = [NSMutableArray arrayWithObjects:nil];
	NSEnumerator *enumerator = [[self quickTimeFileExtensions] objectEnumerator];
	id type;

	while ((type = [enumerator nextObject]))
	{
	    if ( [array indexOfObject:type] == NSNotFound )
	    {
		[array addObject:type];
	    }
	}

        int32 count = TArmeMediaEngine::GetMediaFileHandlerCount();
        VArmeMediaFileHandler* handler;
        int32 handlerCount;
        NSString *str;
        
        for ( int32 x = 0 ; x < count ; x++ )
        {
            handler = TArmeMediaEngine::GetMediaFileHandler( x );
            handlerCount = handler->GetSupportedExtensionsCount();
            for ( int32 y = 0 ; y < handlerCount ; y++ )
            {
                str = [NSString stringWithCString:handler->GetSupportedExtension(y)];
                if ( [array indexOfObject:str] == NSNotFound )
                {
                    [array addObject:str];
                }
            }
        }
        
	fFileExtensions = [[NSArray arrayWithArray:array] retain];
	//NSLog( @"fileExtensions = %@", fFileExtensions );
    }
	
    return fFileExtensions;
}


- (NSArray*)fileTypes
{
    if (!fFileTypes)
    {
        NSMutableArray *array = [NSMutableArray arrayWithObjects:nil];
        NSEnumerator *enumerator = [[self quickTimeFileTypes] objectEnumerator];
        id type;

        while ((type = [enumerator nextObject]))
        {
            if ( [array indexOfObject:type] == NSNotFound )
            {
                [array addObject:type];
            }
        }

        int32 count = TArmeMediaEngine::GetMediaFileHandlerCount();
        VArmeMediaFileHandler* handler;
        int32 handlerCount;
        NSString *str;
        
        for ( int32 x = 0 ; x < count ; x++ )
        {
            handler = TArmeMediaEngine::GetMediaFileHandler( x );
            handlerCount = handler->GetSupportedTypesCount();
            for ( int32 y = 0 ; y < handlerCount ; y++ )
            {
                str = [NSString stringWithFormat:@"'%@'", [NSString stringWithCString:handler->GetSupportedType(y)]];
                if ( [array indexOfObject:str] == NSNotFound )
                {
                    [array addObject:str];
                }
            }
        }
        
        fFileTypes = [[NSArray arrayWithArray:array] retain];
		//NSLog( @"fileTypes = %@", fFileTypes );
    }
    return fFileTypes;
}

- (NSArray*)fileTypesAndExtensions
{
    if ( !fFileTypesAndExtensions )
    {
        NSMutableArray *array = [NSMutableArray arrayWithArray:[self fileTypes]];
        [array addObjectsFromArray:[self fileExtensions]];
        fFileTypesAndExtensions = [[NSArray arrayWithArray:array] retain];
    }
    return fFileTypesAndExtensions;
}

- (NSArray*)pasteboardTypes
{
    if ( !fPasteboardTypes )
    {
		NSMutableArray *array = [NSMutableArray arrayWithObjects:NSFilenamesPboardType, MILibraryElementsPboardType, nil];

		// ***marcp*** this code is commented since the d&d routines don't handle data types
		/*NSEnumerator *enumerator = [[self fileTypes] objectEnumerator];
		NSString *type;

		while ((type = (NSString*)[enumerator nextObject]))
		{
			NSRange range = { 1, 4 };
			NSString *code = [type substringWithRange:range];
			[array addObject:[NSString stringWithFormat:@"CorePasteboardFlavorType 0x%X", (*(int*)[code cString])]];
		}*/
		
		fPasteboardTypes = [[NSArray arrayWithArray:array] retain];
	}
    return fPasteboardTypes;
}

- (void)applicationWillTerminate:(NSNotification *)notification 
{
	vernl(6,"[MIApplicationController applicationWillTerminate] time stamp = " << [[NSApp currentEvent] timestamp]);
	
	// ok now we are sure that this method will only be called when the application
	// will REALLY terminate. 
	
	// we make sure to release all shared resources here.
	[[MIPreferences sharedPrefs] saveToUserSettings];
	
	// please note this does not release the few resources that are still needed
	TArmeMediaEngine::Deinitialize();
}

- (void)applicationWillFinishLaunching:(NSNotification *)notification {
#ifndef MTG_DEMO_VERSION	
	[NSApp registerKeyCommand:@"v" modifiers:NSCommandKeyMask forTarget:self selector:@selector(tmppaste:)];
#if defined( MTG_USE_LICENSER_KIT)
#if defined( MTG_FULL_VERSION ) || defined( MTG_LIGHT_VERSION ) || defined( MTG_LICENSER_KIT_DEBUG )
	[NSApp applicationWillFinishLaunching:nil];
#endif
#endif //defined( MTG_USE_LICENSER_KIT)
	[NSApp unregisterKeyCommand:@"v" modifiers:NSCommandKeyMask forTarget:self];
#endif
	[MIPaletteController sharedPaletteController];
	
	// create an MEApplicationServices instance linking Arme to us
	MEApplicationServices* applicationServices = new MEApplicationServices(self);

	//- (NSString*)audioOutputDeviceStringCode;
	//- (NSString*)audioInputDeviceStringCode;
	//- (UInt32)audioOutputDeviceBufferNbFrames;
	

	// step 1 : we retrieve the output device string code and the number of output frames
	// as defined in the prerefences
	NSString* outDevStringCode = [[MIPreferences sharedPrefs] audioOutputDeviceStringCode];
	UInt32 outNbFrames = [[MIPreferences sharedPrefs] audioOutputDeviceBufferNbFrames];

	// step 2 : translate the string code into a CoreAudio AudioDeviceID
	AudioDeviceID outDevID = MEAudioDeviceDesc::GetAudioDeviceIDFromStringCode((CFStringRef)outDevStringCode);
	bool problemOpeningChosenDevice = false;
	if(outDevID == 0x00000000) // this means the device hasn't been found
	{
		problemOpeningChosenDevice = true;
		NSRunAlertPanel(@"Warning", @"The Audio Output Device chosen in the preferences cannot be opened. Please open the preferences panel to choose another device. HyperEngine-AV now uses the default output device.", @"OK", nil, nil);
		NSString* defaultOutDevStringCode = [NSString stringWithString:(NSString*)MEAudioDeviceDesc::GetDefaultOutputStringCode()]; // returns a code that is 'always use default device'
		[[MIPreferences sharedPrefs] setAudioOutputDeviceStringCode:defaultOutDevStringCode];
		outDevID = MEAudioDeviceDesc::GetAudioDeviceIDFromStringCode((CFStringRef)defaultOutDevStringCode);
		ZzAssert(outDevID != 0x00000000);
	}
	UInt32 realOutNbFrames = outNbFrames;
TArmeMediaEngine::Initialize(applicationServices,outDevID,&realOutNbFrames);

	// now this instance of the application controller needs to listen to notifications
	// posted by the MEApplicationServices instance
	fProxyListener = new MIObjCProxyListener(self);
TArmeMediaEngine::ApplicationServices()->Subscribe(fProxyListener);

	// if the chosen device was ok but if the frames set up couldn't have been set, let's display a message and
	// change the default value in the preferences
	if((realOutNbFrames != outNbFrames) && (!problemOpeningChosenDevice))
	{
		if(outNbFrames != 0)
		{
			NSRunAlertPanel(@"Warning", @"The number of audio output samples chosen in the preferences couldn't be used. Please open the preferences panel to choose another value. This setting has been reverted to a suitable value.", @"OK", nil, nil);
		}
		[[MIPreferences sharedPrefs] setAudioOutputDeviceBufferNbFrames:realOutNbFrames];
	}

	//create the clipboard
	fClipBoard = [[MIObject alloc] initWithArmeWSObj:(new TArmeContainer("clipboard"))];
	[self addStuffToMenus];
	[self fileTypesAndExtensions];
}

- (IBAction)tmppaste:(id)sender {
	[NSApp sendAction:@selector(paste:) to:nil from:sender];
}

- (void)applicationDidFinishLaunching:(NSNotification *)notification {

/*#ifdef DEBUG	

// create a QD View Panel
	NSRect rect = NSMakeRect( 0, 0, 640, 480 );
	MIQuickDrawView *QDView = [[MIQuickDrawView alloc] initWithFrame:rect];
	NSPanel *panel = [[NSPanel alloc] initWithContentRect:rect styleMask:NSTitledWindowMask backing:NSBackingStoreBuffered defer:NO];

	[panel setTitle:@"Default QD Port"];
	[panel setContentView:QDView];
	[panel orderFront:nil];
	[QDView setBounds:rect];
	[QDView startUpdating];
	[QDView release];
#endif*/
	
	if ( [[[NSDocumentController sharedDocumentController] documents] count] == 0 )
		[[MIStartingPoint startingPoint] documentRemoved:nil];
	else {
		[MIStartingPoint startingPoint];
		[[MIPaletteController sharedPaletteController] showAllPalettes:nil];
	}
}

- (IBAction)showAboutPanel:(id)sender {
	[MISplash aboutbox];
}
	

- (BOOL)applicationShouldOpenUntitledFile:(NSApplication *)sender {
    return NO;
}

- (IBAction)viewControls:(id)sender
{
	NSEnumerator 	*objEnum;
	MIObjectRep		*aRep;

	if ( [[[NSApp mainViewController] selection] type] != kMIWSSelectionTypeObjects )
		return;
	
	objEnum = [[[NSApp mainViewController] selection] objectEnumerator];
	while ( aRep = [objEnum nextObject] )
	{
		[aRep showProperties];
	}
}

- (void)nudgeSelectionLeftSmall:(NSString*)key withModifiers:(unsigned int)modifiers
{
		[[NSApp mainViewController] nudgeSelection:NSLeftArrowFunctionKey withModifiers:NSAlternateKeyMask];
}
- (void)nudgeSelectionRightSmall:(NSString*)key withModifiers:(unsigned int)modifiers
{
		[[NSApp mainViewController] nudgeSelection:NSRightArrowFunctionKey withModifiers:NSAlternateKeyMask];
}

- (void)nudgeSelectionLeftMedium:(NSString*)key withModifiers:(unsigned int)modifiers
{
		[[NSApp mainViewController] nudgeSelection:NSLeftArrowFunctionKey withModifiers:NSAlternateKeyMask|NSShiftKeyMask];
}
- (void)nudgeSelectionRightMedium:(NSString*)key withModifiers:(unsigned int)modifiers
{
		[[NSApp mainViewController] nudgeSelection:NSRightArrowFunctionKey withModifiers:NSAlternateKeyMask|NSShiftKeyMask];
}

- (void)nudgeSelectionLeftLarge:(NSString*)key withModifiers:(unsigned int)modifiers
{
		[[NSApp mainViewController] nudgeSelection:NSLeftArrowFunctionKey withModifiers:NSAlternateKeyMask|NSControlKeyMask];
}
- (void)nudgeSelectionRightLarge:(NSString*)key withModifiers:(unsigned int)modifiers
{
		[[NSApp mainViewController] nudgeSelection:NSRightArrowFunctionKey withModifiers:NSAlternateKeyMask|NSControlKeyMask];
}

- (IBAction)nudgeRight:(id)sender
{
    TArmeDoc	*theDocument;
    theDocument = [[[NSDocumentController sharedDocumentController] currentDocument] armeDocument];
	ArmeTime currentTime = theDocument->GetPlayPos();
	currentTime += _ArmeTime((sample_t)(theDocument->GetAudioFrameRate()/theDocument->GetVideoFrameRate()));
	theDocument->Seek(currentTime);
	theDocument->Preroll(currentTime);
	[[NSApp mainViewController] updatePlayBar];
	[[NSApp mainViewController] updateLocators];
}

- (IBAction)nudgeLeft:(id)sender
{
    TArmeDoc	*theDocument;
    theDocument = [[[NSDocumentController sharedDocumentController] currentDocument] armeDocument];
	ArmeTime currentTime = theDocument->GetPlayPos();
	currentTime -= _ArmeTime((sample_t)(theDocument->GetAudioFrameRate()/theDocument->GetVideoFrameRate()));
	
	if ( GET_VALUE(currentTime) < 0 )
		currentTime = _ArmeTime(0);

	theDocument->Seek(currentTime);
	theDocument->Preroll(currentTime);
	[[NSApp mainViewController] updatePlayBar];
	[[NSApp mainViewController] updateLocators];
}

- (IBAction)setGrid:(id)sender
{
}

- (IBAction)stickToGrid:(id)sender
{
}

- (IBAction)showGrid:(id)sender
{
}

- (IBAction)interleaveToSlit:(id)sender
{
}

- (IBAction)slitToInterleave:(id)sender
{
}

- (IBAction)convertSampleWidth:(id)sender
{
}

- (IBAction)convertSampleRate:(id)sender
{
}

- (IBAction)processorCustomize:(id)sender
{
}

- (IBAction)pictFromMovieView:(id)sender
{
	vertl(commandsBenjamin,5,"INFO : the 'Pict from frame' menu entry has been selected.\n");
	[[NSApp mainViewController] executeCommand:@"pictFromFrame" ];
}

- (IBAction)allOn:(id)sender
{
	vertl(commandsBenjamin,5,"INFO : the 'Playback | All On' menu entry has been selected.\n");
	[[NSApp mainViewController] executeCommand:@"all on" ];
}

- (IBAction)mute:(id)sender
{
	vertl(commandsBenjamin,5,"INFO : the 'Playback | Mute' menu entry has been selected.\n");
	if ( [[sender title] isEqualToString:@"Mute"] )
		[[NSApp mainViewController] executeCommand:@"mute" ];
	else [[NSApp mainViewController] executeCommand:@"unmute" ];
}

- (IBAction)solo:(id)sender
{
	vertl(commandsBenjamin,5,"INFO : the 'Playback | Solo' menu entry has been selected.\n");
	[[NSApp mainViewController] executeCommand:@"solo" ];
}

- (IBAction)followPlaybar:(id)sender
{
	fFollowPlayBar = !fFollowPlayBar;
}

- (void)setShouldFollowPlaybar:(BOOL)flag {
	fFollowPlayBar = flag;
}

- (BOOL)shouldFollowPlaybar {
	return fFollowPlayBar;
}

- (void)deferAllRedraws
{
	gDeferRedrawsUntilCommandEnd = YES;
}

- (void)flushAllDeferedRedrawRects
{
	gDeferRedrawsUntilCommandEnd = NO;
	NSEnumerator* docEnum = [[[NSDocumentController sharedDocumentController] documents] objectEnumerator];
	MIMontageDocument* doc;
	while(doc = [docEnum nextObject])
	{
		NSEnumerator* mainViewEnum = [[doc windowControllers] objectEnumerator];
		MIMainViewController* mainViewController;
		while(mainViewController = [mainViewEnum nextObject])
		{
			[[mainViewController topLevelContainerView] flushDeferedRedraws];
		}
	}	
}

-(void)redrawAllMainWindows
{
	NSEnumerator* docEnum = [[[NSDocumentController sharedDocumentController] documents] objectEnumerator];
	MIMontageDocument* doc;
	while(doc = [docEnum nextObject])
	{
		NSEnumerator* mainViewEnum = [[doc windowControllers] objectEnumerator];
		MIMainViewController* mainViewController;
		while(mainViewController = [mainViewEnum nextObject])
		{
			[[mainViewController topLevelContainerView] setNeedsDisplay:YES];
		}
	}
}


- (IBAction)showGeneralGridVerticalLines:(id)sender
{
	MIPreferences *prefs = [MIPreferences sharedPrefs];
	UInt32 mask = [prefs gridShowMask];

	mask ^= kPrefsGridShowVLines;

	[prefs setGridShowMask:mask];

	[self redrawAllMainWindows];
}

- (IBAction)showGeneralGridHorizontalLines:(id)sender
{
	MIPreferences *prefs = [MIPreferences sharedPrefs];
	UInt32 mask = [prefs gridShowMask];

	mask ^= kPrefsGridShowHLines ;

	[prefs setGridShowMask:mask];

	[self redrawAllMainWindows];	
}

- (IBAction)showObjectGridVerticalLines:(id)sender
{
	MIPreferences *prefs = [MIPreferences sharedPrefs];
	UInt32 mask = [prefs gridShowMask];

	mask ^= kPrefsGridShowVObjLines ;

	[prefs setGridShowMask:mask];

	[self redrawAllMainWindows];
}

- (IBAction)showObjectGridHorizontalLines:(id)sender
{
	MIPreferences *prefs = [MIPreferences sharedPrefs];
	UInt32 mask = [prefs gridShowMask];

	mask ^= kPrefsGridShowHObjLines ;

	[prefs setGridShowMask:mask];

	[self redrawAllMainWindows];
}

- (IBAction)snapToGeneralGrid:(id)sender
{
	MIPreferences* prefs = [MIPreferences sharedPrefs];
	UInt32 mask = [prefs snapOptionsMask];

	if ( (!( mask & kPrefsSnapHGrid )) || (!(mask & kPrefsSnapVGrid) )) {
		mask |= kPrefsSnapHGrid ;
		mask |= kPrefsSnapVGrid ;
	} else {
		mask &= ~kPrefsSnapHGrid ;
		mask &= ~kPrefsSnapVGrid ;
	}

	[prefs setSnapOptionsMask:mask];

	[self redrawAllMainWindows];
}

- (IBAction)snapToObjectsGrid:(id)sender
{
	MIPreferences* prefs = [MIPreferences sharedPrefs];
	UInt32 mask = [prefs snapOptionsMask];

	if ( (!( mask & kPrefsSnapHObjects )) || (!(mask & kPrefsSnapVObjects) )) {
		mask |= kPrefsSnapHObjects ;
		mask |= kPrefsSnapVObjects ;
	} else {
		mask &= ~kPrefsSnapHObjects ;
		mask &= ~kPrefsSnapVObjects ;
	}

	[prefs setSnapOptionsMask:mask];

	[self redrawAllMainWindows];
}



- (IBAction)automaticSceneDetection:(id)sender
{
}

- (IBAction)startRecord:(id)sender
{
}

- (IBAction)findAgain:(id)sender
{
	[[MIFindDlgController sharedFindController] findAgain];
}

- (IBAction)find:(id)sender
{
	id tlc = [[NSApp mainViewController] topLevelContainerView];
	[[MIFindDlgController sharedFindController] showWindow:self];
	[[MIFindDlgController sharedFindController] setTLC:tlc];
}

- (IBAction)print:(id)sender
{
}

- (IBAction)pageSetup:(id)sender
{
}

- (IBAction)showInformation:(id)sender
{
}

- (IBAction)documentOpenRecent:(id)sender
{
}

- (IBAction)renderClip:(id)sender
{
	vertl(commandsBenjamin,5,"INFO : the 'Render Clip' menu entry has been selected.\n");
	[[NSApp mainViewController] executeCommand:@"freezeSelection" ];
}

- (IBAction)freezeSelection:(id)sender
{
	//this is a debug menyentry.  
	
	VUSERCMD("Make the project self-contained");
	//make the project self contained (to be put in the right function when it is available)
	MIProgressBarDlgManager mngr("Copying media files into project");
	if ([[[NSDocumentController sharedDocumentController] currentDocument] armeDocument]->MakeProjectSelfContained(&mngr))
	{
		//the make self-contained has succeeded; save the document
		[[[NSDocumentController sharedDocumentController] currentDocument] saveDocument:self];

		//Note : if we ever decide not to save the doc at this time, we should at least mark the doc as 'modified'
	}

	vertl(commandsBenjamin,5,"INFO : the 'Debug | Freeze Selection' menu entry has been selected.\n");
	[[NSApp mainViewController] executeCommand:@"freezeSelection" ];
}

- (IBAction)unfreezeSelection:(id)sender
{
	vertl(commandsBenjamin,5,"INFO : the 'Debug | Unfreeze Selection' menu entry has been selected.\n");
	[[NSApp mainViewController] executeCommand:@"unfreezeSelection" ];
}

- (IBAction)djSwitch:(id)sender
{
	[[NSApp mainViewController] executeCommand:@"djSwitch" ];
}

- (IBAction)makeDocumentSelfContained:(id)sender
{
	
	VUSERCMD("Make the project self-contained");
	//make the project self contained (to be put in the right function when it is available)
	MIProgressBarDlgManager mngr("Copying media files into project");
	if ([[[NSDocumentController sharedDocumentController] currentDocument] armeDocument]->MakeProjectSelfContained(&mngr))
	{
		//the make self-contained has succeeded; save the document
		[[[NSDocumentController sharedDocumentController] currentDocument] saveDocument:self];

		//Note : if we ever decide not to save the doc at this time, we should at least mark the doc as 'modified'
	}

	vertl(commandsBenjamin,5,"INFO : the 'File | makeDocumentSelfContained' menu entry has been selected.\n");
	[[NSApp mainViewController] executeCommand:@"freezeSelection" ];
}

- (void)beep
{
#ifdef DEBUG
    NSBeep();
#endif
}

- (IBAction)recreateProcessorMenuAction:(id)sender
{
	[self recreateProcessorMenu];
}

- (IBAction)testAudioRecording:(id)sender
{
}

- (IBAction)testGenSineWave:(id)sender
{
}

- (IBAction)configureTransition:(id)sender
{
	vertl(commandsBenjamin,5,"INFO : the 'Debug | Configure Transition' menu entry has been selected.\n");
	[[NSApp mainViewController] executeCommand:@"configureTransition" ];
}

- (IBAction)forceAudioTrackUpdate:(id)sender
{
	TArmeDoc* armeDoc = [[[NSDocumentController sharedDocumentController] currentDocument] armeDocument];
	armeDoc->GetPlayer()->GetWSObjAudioPlayer(0)->RecreateTrackMapping();	
}

- (IBAction)setLocatorsFromSelection:(id)sender
{
	[[NSApp mainViewController] setLocatorsFromSelection];
}

- (void)selectVAO:(id)sender
{
	verntl(audioEffectsTom,6,"selectVAO (MIApplicationController)");

	MIChangeVAOCommand* cmd = [[[MIChangeVAOCommand alloc] initWithSelection:[[NSApp mainViewController] selection]] autorelease];
	
	MEVAORouting newRouting;
	SInt32 tag = [sender tag];
	if(tag < 1000)
	{
		newRouting.bStereo = true;
		newRouting.index = (tag-1)/2;
	}
	else
	{
		newRouting.bStereo = false;
		newRouting.index = tag-1000-1;
	}
			
	[cmd setNewVAORouting:newRouting];
	
	VUSERCMD2("Select VAO (index = %d), (isStereo: %d)", (int)(newRouting.index), newRouting.bStereo);
	VUSERCMD_PRINT_SELECTION([[NSApp mainViewController] selection]);
	
	[[[NSDocumentController sharedDocumentController] currentDocument] doCmd:cmd];
}

- (IBAction)audioClipInspector:(id)sender
{
	[[NSApp mainViewController] audioClipInspector ];
}

- (IBAction)debugAudioRecorder:(id)sender
{
	[[NSApp mainViewController] debugAudioRecorder];
}

bool gEnableCursorRectsUpdate = true;

- (IBAction)toggleCursorRectsUpdate:(id)sender
{
	if( [sender state] == NSOffState)
	{
		[sender setState:NSOnState];
		gEnableCursorRectsUpdate = false; 
	}
	else
	{	
		[sender setState:NSOffState];
		gEnableCursorRectsUpdate = true;
	}
}

bool gNoObjectTitlesAtAll = false;

- (IBAction)toggleUseTitles:(id)sender
{
	if( [sender state] == NSOffState)
	{
		[sender setState:NSOnState];
		gNoObjectTitlesAtAll = true;
	}
	else
	{	
		[sender setState:NSOffState];
		gNoObjectTitlesAtAll = false;
	}
}

bool gDisableRulerDrawing = false;

- (IBAction)toggleRulerDrawing:(id)sender
{
	if( [sender state] == NSOffState)
	{
		[sender setState:NSOnState];
		gDisableRulerDrawing = true;
	}
	else
	{	
		[sender setState:NSOffState];
		gDisableRulerDrawing = false;
	}
}

bool gLazyOverviewUpdateOnSelChanged = false;

- (IBAction)toggleLazyOverviewUpdate:(id)sender
{
	if( [sender state] == NSOffState)
	{
		[sender setState:NSOnState];
		gLazyOverviewUpdateOnSelChanged = true;
	}
	else
	{	
		[sender setState:NSOffState];
		gLazyOverviewUpdateOnSelChanged = false;
	}
}

- (void)applicationDidBecomeActive:(NSNotification *)aNotification
{
	[[MIOverviewController sharedOverviewController] refreshCachedImage];	
}

- (IBAction)close:(id)sender {
	//[[[NSApp mainViewController] window] performClose:sender];
	[[NSApp mainWindow] performClose:sender];
}

- (IBAction)showHelp:(id)sender {
	[[NSWorkspace sharedWorkspace] openFile:[[[NSBundle mainBundle] resourcePath] stringByAppendingString:@"/index.htm"]];
}

@end

@implementation MIQuickDrawView

- (id)initWithFrame:(NSRect)frame {
	if (( self = [super initWithFrame:frame] )) {
		Rect boundsRect = { 0, 0, 480, 640 };
		QDErr err = NewGWorld(
						&fGWorld,
						32,
						&boundsRect,
						nil,
						nil,
						0);
		if ( err == noErr ) {
			SetGWorld ( fGWorld, GetGWorldDevice( fGWorld ) );
			gGWorld = fGWorld;
		} else {
			NSLog(@"unable to create GWorld. error = %d", err);
			[self release];
			return nil;
		}
	}
	return self;
}

- (void)dealloc {
	[self stopUpdating];
	DisposeGWorld( fGWorld ) ;
	[super dealloc];
}

- (void)drawRect:(NSRect)rect {
	Rect srcRect = {0, 0, 480, 640};
	Rect dstRect;
	
	GetPortBounds((CGrafPtr)[self qdPort], &dstRect);
	PixMapHandle pixmap = GetGWorldPixMap( fGWorld );
	
	if ( LockPixels( pixmap ) ) {
		CopyBits( (BitMap *)*pixmap, GetPortBitMapForCopyBits((CGrafPtr)[self qdPort]), &srcRect, &dstRect, srcCopy, NULL );
		UnlockPixels( pixmap );
	}
}

- (void)update:(NSTimer*)timer {
	[self display];
	SetGWorld( fGWorld, GetGWorldDevice( fGWorld ) );
}

- (void)startUpdating {
	fTimer = [NSTimer scheduledTimerWithTimeInterval:.5 target:self selector:@selector(update:) userInfo:nil repeats:YES];
	fModalTimer = [NSTimer timerWithTimeInterval:.5 target:self selector:@selector(update:) userInfo:nil repeats:YES];
	[[NSRunLoop currentRunLoop] addTimer:fModalTimer forMode:NSModalPanelRunLoopMode];
}

- (void)stopUpdating {
	if ( fTimer ) {
		[fTimer invalidate];
		fTimer = nil;
		[fModalTimer invalidate];
	}
}


@end


