/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#include "MIAudioClipInspectorController.h"
#import "MIWSSelection.h"
#import "MIObject.h"
#import "MIObjectRep.h"

#ifndef MEAudioClip_h
#include "MEAudioClip.h"
#endif //MEAudioClip_h

@implementation MIAudioClipInspectorController

- (void)displaySelectionInfo:(MIWSSelection*)sel
{
	char* text = new char[1024*1024]; // et allez ! pourquoi se priver ? j'ai un giga de RAM !
	char* curline = text; // the pointer
	MIObjectRep* rep;
	MIObject* miobj;
	TArmeWSObj* wsobj;
	MEAudioClip* audioClip;
	SInt32 count = [sel count];
	if(count == 0)
	{
		sprintf(text,"Selection is empty.");
	}
	else
	{
		for(SInt32 i = 0; i < count; i++)
		{
			rep = [sel objectAtIndex:i];
			miobj = [rep representedObject];
			wsobj = [miobj WSObject];
			curline += sprintf(curline,"object %ld is %s",i,wsobj->GetName());
			audioClip = dynamic_cast<MEAudioClip*>(wsobj);
			if(audioClip)
			{
				SInt32 iChunk;
				curline += sprintf(curline,"\n");
				curline += sprintf(curline,"  wsobj->GetDuration() = %lld\n", GET_VAL(wsobj->GetDuration()));
				curline += sprintf(curline,"  wsobj->GetStartTrim() = %lld\n", GET_VAL(wsobj->GetStartTrim()));
				curline += sprintf(curline,"  wsobj->GetEndTrim() = %lld\n", GET_VAL(wsobj->GetEndTrim()));
				MEAudioClipData* data = audioClip->GetData();
				curline += sprintf(curline,"  data->GetDuration() = %lld\n", data->GetDuration());
				const MEAudioChunkList& preTrimChunkList = data->GetPreTrimChunkList();
				const MEAudioChunkList& trimmedChunkList = data->GetChunkList();
				curline += sprintf(curline,"  preTrimChunkList.GetDuration() = %lld\n", preTrimChunkList.GetDuration());
				curline += sprintf(curline,"  preTrimChunkList.GetNbChunks() = %ld\n", preTrimChunkList.GetNbChunks());
				for(iChunk = 0; iChunk < preTrimChunkList.GetNbChunks(); iChunk++)
				{
					const MEAudioChunk& chunk = preTrimChunkList.GetChunk(iChunk);
					curline += sprintf(curline,"    chunk %ld = (index = %ld startInFile = %lld duration = %lld doc = %p)\n", iChunk, chunk.GetFileIndex().data, chunk.startInFile, chunk.duration, chunk.doc);
				}
				curline += sprintf(curline,"  trimmedChunkList.GetDuration() = %lld\n", trimmedChunkList.GetDuration());
				curline += sprintf(curline,"  trimmedChunkList.GetNbChunks() = %ld\n", trimmedChunkList.GetNbChunks());
				for(iChunk = 0; iChunk < trimmedChunkList.GetNbChunks(); iChunk++)
				{
					const MEAudioChunk& chunk = trimmedChunkList.GetChunk(iChunk);
					curline += sprintf(curline,"    chunk %ld = (index = %ld startInFile = %lld duration = %lld doc = %p)\n", iChunk, chunk.GetFileIndex().data, chunk.startInFile, chunk.duration, chunk.doc);
				}
			}
			else
			{
				curline += sprintf(curline," (not an audio clip)\n");
			}
		}
	}
	NSString* name = [NSString stringWithUTF8String:text];
	[fTextField setStringValue:name];
	delete [] text;
}

- (IBAction)dismissClicked:(id)sender
{
	[NSApp stopModal];
}

+ (void)displayAudioClipInspector:(MIWSSelection*)sel
{
	MIAudioClipInspectorController* ctrller = [[MIAudioClipInspectorController alloc] initWithWindowNibName:@"MIAudioClipInspector"];
	[ctrller window]; // trigger the actual creation
	[ctrller displaySelectionInfo:sel];
	[NSApp runModalForWindow:[ctrller window]];
	[[ctrller window] orderOut:nil];
	[ctrller release];
}

@end

