/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
#ifndef ZzVerbose_h
//#include "ZzVerbose.h"
#endif //ZzVerbose_h

#import "MIAudioClipRep.h"
#import "MIImageController.h"
#import "MIObject.h"
#import "MITopLevelContainerRep.h"
#import "MIMainViewTrackers.h"
#import "MIApplicationController.h"
#import "MIMainViewController.h"
#import "MITimeStretchTracker.h"
#import "MITrimTracker.h"
#import "MIActiveCursor.h"

#ifndef TArmeDoc_h
#include "TArmeDoc.h"
#endif //TArmeDoc_h

#ifdef TRACK_BASED_AUDIO_CLIPS
#ifndef MEAudioClip_h
#include "MEAudioClip.h"
#endif //MEAudioClip_h
#else //#ifdef TRACK_BASED_AUDIO_CLIPS
#ifndef TAudioClip2_h
#include "TAudioClip2.h"
#endif //TAudioClip2_h
#endif //#ifdef TRACK_BASED_AUDIO_CLIPS

#ifndef MEAudioClipData_h
#include "MEAudioClipData.h"
#endif //MEAudioClipData_h

#ifndef ZzVerbose_h
//#include "ZzVerbose.h"
#endif //ZzVerbose_h

//#ifndef ZzDisableVerbose_h
//#include "ZzDisableVerbose.h"
//#endif //ZzDisableVerbose_h

#import "MISliceSelectionTracker.h"

// the size of the cached image for audio clips
//#define kImageHeigth 516

#ifdef TRACK_BASED_AUDIO_CLIPS
BOOL gAudioRepDrawingDisabled = NO;
NSRect gAudioRepPlaceholderImageRect;
NSImage* gAudioRepPlaceholderImage = nil;
#endif //ALLOW_AUDIO_REP_DRAWING_DISABLED

@interface MIMouseTracker (MTGLiveTrackingUnformalProtocol)

- (void)drawCachedImage:(NSImage*)fCachedImage inFrame:(NSRect)frameInDrawCoords clippedTo:(NSRect)clippingRect;

@end

@implementation MIAudioClipRep
- (id)initWithFrame:(NSRect)frame inTLC:(MITopLevelContainerRep*)theTLC
{
/*#ifdef TRACK_BASED_AUDIO_CLIPS
	if(gAudioRepPlaceholderImage = nil)
	{
		NSBundle*	theBundle;
		theBundle = [NSBundle mainBundle];
		// create placeholder image
		gAudioRepPlaceholderImage = [[NSImage alloc] initWithContentsOfFile:[theBundle pathForResource:@"audioclip_placeholder" ofType:@"tiff"]];
		NSSize size = [gAudioRepPlaceholderImage size];
		gAudioRepPlaceholderImageRect = NSMakeRect(0,0, size.width, size.height);
	}
#endif //#ifdef TRACK_BASED_AUDIO_CLIPS*/
	self = [super initWithFrame:frame inTLC:theTLC];

//	fLeftBarImage = mainImageController->fAudioClipBarOnLeft;
//    fCenterBarImage = mainImageController->fAudioClipBarOnCenter;
//    fRightBarImage = mainImageController->fAudioClipBarOnRight;

	fHasTopBottomHandles = NO;
	fHasLeftRightHandles = NO;
	fHasCollapseButton = NO;
	fHasPropertiesButton = NO;
	fHasOnOffButton = NO;
	fCachedImage = nil;
	fStopImageUpdates = NO;
	
    return self;
}
- (void)setImagesAndColor
{
	if ( [self isSelected] && [[[fTLCRep mainViewController] selection] type] == kMIWSSelectionTypeObjects )
	{
		[self setBackgroundColor:mainImageController->fAudioClipNegativeColor];
		fLeftTitleImage = mainImageController->fLeftAudioClipNegativeTitleBarImage;
		fCenterTitleImage = mainImageController->fCenterAudioClipNegativeTitleBarImage;
		fRightTitleImage = mainImageController->fRightAudioClipNegativeTitleBarImage;
	}
	else if ( [fRepresentedObject isMuted] )
	{
		[self setBackgroundColor:mainImageController->fAudioClipDimColor];
		fLeftTitleImage = mainImageController->fLeftAudioClipDimTitleBarImage;
		fCenterTitleImage = mainImageController->fCenterAudioClipDimTitleBarImage;
		fRightTitleImage = mainImageController->fRightAudioClipDimTitleBarImage;
	}
	else 
	{
		[self setBackgroundColor:mainImageController->fAudioClipBrightColor];
		fLeftTitleImage = mainImageController->fLeftAudioClipBrightTitleBarImage;
		fCenterTitleImage = mainImageController->fCenterAudioClipBrightTitleBarImage;
		fRightTitleImage = mainImageController->fRightAudioClipBrightTitleBarImage;
	}
}

- (void)setSelected:(BOOL)selected
{
	if ( selected != fSelected )
	{
		[fCachedImage release];
		fCachedImage = nil;
	}
	[super setSelected:selected];
}


- (void)refreshCachedBodyImage:(BOOL)createIfAbsent
{
	// don't do anything, we don't cache the body
}

/*
- (void)addCursorRects
{
	NSPoint 	frOrigin = [self getTLCFrame].origin;

	if ( !fMouseIsOverMe )
		return;

	if ( [[NSApp currentEvent] modifierFlags] & NSAlternateKeyMask )
	{
		[fTLCRep addCursorRect:[self rectForZone:MIObjectRepLeftTimeStretchZone withObjectOrigin:frOrigin] cursor:mainImageController->fTimeStretchLeftCursor];
		[fTLCRep addCursorRect:[self rectForZone:MIObjectRepRightTimeStretchZone withObjectOrigin:frOrigin] cursor:mainImageController->fTimeStretchRightCursor];
	}
	else
	{
		[fTLCRep addCursorRect:[self rectForZone:MIObjectRepLeftTimeStretchZone withObjectOrigin:frOrigin] cursor:mainImageController->fTrimLeftCursor];
		[fTLCRep addCursorRect:[self rectForZone:MIObjectRepRightTimeStretchZone withObjectOrigin:frOrigin] cursor:mainImageController->fTrimRightCursor];
	}

	[super addCursorRects];
}

- (void)removeCursorRects
{
	[super removeCursorRects];
	
}

- (void)addActiveCursors
{
	return;

	MIActiveCursor* activeCursor;
	NSRect tlcFrame = [self getTLCFrame];
	NSRect tmpRect[2];

	[super addActiveCursors];
	
	tmpRect[0] = [self rectForZone:MIObjectRepRightTimeStretchZone withObjectOrigin:[self getTLCFrame].origin];
	tmpRect[1] = [self rectForZone:MIObjectRepLeftTimeStretchZone withObjectOrigin:[self getTLCFrame].origin];

//	NSDivideRect( tlcFrame, &tmpRect[0],  &tlcFrame, 5, NSMaxXEdge );
//	NSDivideRect( tlcFrame, &tmpRect[1],  &tlcFrame, 5,NSMinXEdge );
	
	activeCursor = [[MIActiveCursor alloc] initWithPlainCursor:mainImageController->fTrimRightCursor forActiveRect:tmpRect[0]];
	[activeCursor setAlternateCursor:mainImageController->fTimeStretchRightCursor];
	[self addActiveCursor:activeCursor];
	[activeCursor release];

	activeCursor = [[MIActiveCursor alloc] initWithPlainCursor:mainImageController->fTrimLeftCursor forActiveRect:tmpRect[1]];
	[activeCursor setAlternateCursor:mainImageController->fTimeStretchLeftCursor];
	[self addActiveCursor:activeCursor];
	[activeCursor release];
}
*/
- (MIObjectMouseHandling)handleMoreMouseTracking:(NSPoint)mouseLoc modifier:(unsigned int)modifier type:(NSEventType)type
{
    [self invalidateTheMouseTracker];

    // if the parent does NOT handle it, and if we are in the body, let's create a selection tracker
	if ( NSPointInRect(mouseLoc, [self rectForZone:MIObjectRepRightTimeStretchZone]) )
	{
		if ( modifier & NSAlternateKeyMask )
		{
			fTemporaryTracker = [[MITimeStretchTracker alloc] initWithView:fTLCRep atPos:[self convertPointToTLCCoords:mouseLoc] withModifiers:nil ];
			[fTemporaryTracker setEdge:NSMaxXEdge];
		}
        else
		{
			fTemporaryTracker = [[MITrimTracker alloc] initWithView:fTLCRep atPos:[self convertPointToTLCCoords:mouseLoc] withModifiers:nil ];
			[fTemporaryTracker setEdge:NSMaxXEdge];
		}
        return MIAskMeATracker;
	}
	else if ( NSPointInRect(mouseLoc, [self rectForZone:MIObjectRepLeftTimeStretchZone]) )
	{
		if ( modifier & NSAlternateKeyMask )
		{
			fTemporaryTracker = [[MITimeStretchTracker alloc] initWithView:fTLCRep atPos:[self convertPointToTLCCoords:mouseLoc] withModifiers:nil ];
			[fTemporaryTracker setEdge:NSMinXEdge];
		}
        else
		{
			fTemporaryTracker = [[MITrimTracker alloc] initWithView:fTLCRep atPos:[self convertPointToTLCCoords:mouseLoc] withModifiers:nil ];
			[fTemporaryTracker setEdge:NSMinXEdge];
		}
        return MIAskMeATracker;
	}
	else if ( NSPointInRect(mouseLoc, [self rectForZone:MIObjectRepRightBodySelectionZone]) )
	{ // modified tracker to start farther
		fTemporaryTracker = [[MISliceSelectionTracker alloc] initWithObjectRep:self inView:fTLCRep atPos:[self convertPointToTLCCoords:mouseLoc] withModifiers:nil ] ;
		[fTemporaryTracker setStartTime:[fRepresentedObject WSObject]->GetAbsoluteEndxTime()];
        return MIAskMeATracker;
	}
	else if ( NSPointInRect(mouseLoc, [self rectForZone:MIObjectRepLeftBodySelectionZone]) )
	{ // modified tracker to start farther
		fTemporaryTracker = [[MISliceSelectionTracker alloc] initWithObjectRep:self inView:fTLCRep atPos:[self convertPointToTLCCoords:mouseLoc] withModifiers:nil ] ;
		[fTemporaryTracker setStartTime:[fRepresentedObject WSObject]->GetAbsoluteStartTime()];
        return MIAskMeATracker;
	}
    else return [super handleMoreMouseTracking:mouseLoc modifier:modifier type:type];
}



struct Pixel
{
    UInt8 r; 
    UInt8 g;
    UInt8 b;
	UInt8 a;
};
        
// pack 4 bytes into a word
// use a union to prevent endian bugs
union uPixel
{
    struct Pixel pixel;
    SInt32 packed;
};


// this function draws (more spec: adds to) a greylevel intensity ramp in a float buffer
void draw_ramp(int length, float *buf, float begin, float end, float intensity)
{

    // a small assert zoo for the inconfident..
    ZzAssert((0.0 <= begin) && (begin < (float)length)); 
    ZzAssert((0.0 <= end)   && (end   < (float)length)); 
    ZzAssert(begin < end);

    int ibegin = (int)begin;
    int iend   = (int)end;

    // begin and end share the same pixel
    // draw while and black lines and one grey pixel
    if (ibegin == iend)
	{
        float greylevel = 1.0f + (float)iend - 0.5f * (end + begin);
        // skip zero contribution (+= 0)
        int i = ibegin;
        // set grey pixel
        buf[i++] += intensity * greylevel;
        // fill black part
        while(i<length) 
		{
			buf[i++] += intensity;
		}
    }
	// begin and end of ramp are in different pixels
	// now we need to fill a grey region
    else{
        float slope = 1.0f / (end - begin);
        float slope_intensity = slope * intensity;
        float frac_begin = (1.0f + (float)ibegin - begin);
        float frac_end = (end - (float)iend);
        float greylevel_intensity = intensity * frac_begin * frac_begin * slope * 0.5f;
        float greylevel_end_intensity = intensity * (1.0f - frac_end * frac_end * slope * 0.5f);
        // skip zero contribution (+= 0)
        int i = ibegin;
        // set first grey pixel
        buf[i++] += greylevel_intensity;
        // fill grey part
        while (i<iend)
		{
            greylevel_intensity += slope_intensity; 
            buf[i++] += greylevel_intensity;
        }
        // set last grey pixel
        buf[i++] += greylevel_end_intensity;
        // fill black part
        while(i<length)
		{
			buf[i++] += intensity;
		}
    }
}


// draws waveform in uPixel packed image using a preview point array with specified stride
void draw_waveform
(
    SInt32 *startPixel,
    SInt32 width,
    SInt32 height,
    MEAudioClipDataPreviewPoint *preview,
    SInt32 previewStride,
    union uPixel cFG,
    union uPixel cBG,
    SInt32 *endxPixel
)
{
    MEAudioClipDataPreviewPoint *curPreviewPoint = preview;
    SInt32 *pix = startPixel; 
	ZzAssert(height > 0);
    float *vline = new float[height];

    // waveform display properties
    const float half_pixwidth = 0.75f;
    const float half_rampwidth = 0.25;

    // scaling reserves space for pixel width and ramp width
    // plus a 5 % roundoff margin
    float scale = ((float)(height) - (2.0f*(half_pixwidth + half_rampwidth) + 0.05f));
    float offset = (half_pixwidth + half_rampwidth);
    float liftmax = offset + half_pixwidth;
    float liftmin = offset - half_pixwidth;

    while (pix < startPixel+width) 
	{
		
		// saturate between (-1) and (1)
		#define SATURATE(x) ( ((x) < (-1.0f)) ? (-1.0f) : ( ((x) > (1.0f)) ? (1.0f) : (x)) )
		
		// affine transform from (-1,1) to (0,1)
		#define AFFINETRANSFORM(x) (((x) + 1.0f) * 0.5f)
		
		// make sure distance between min and max is at least 1.5 pixels
		float pmax = liftmax + AFFINETRANSFORM(SATURATE(curPreviewPoint[0].max)) * scale; 
		float max  = liftmax + AFFINETRANSFORM(SATURATE(curPreviewPoint[previewStride].max)) * scale;
		float pmin = liftmin + AFFINETRANSFORM(SATURATE(curPreviewPoint[0].min)) * scale;
		float min  = liftmin + AFFINETRANSFORM(SATURATE(curPreviewPoint[previewStride].min)) * scale;
		
		// test for nan
		//ZzAssert((pmin <= 0.0f)||(pmin >= 0.0f));
		//ZzAssert((pmax <= 0.0f)||(pmax >= 0.0f));
		//ZzAssert((min <= 0.0f)||(min >= 0.0f));
		//ZzAssert((max <= 0.0f)||(max >= 0.0f));
		
		// our ramp drawing algorithm relies on the fact that pmax <= max and pmin <= min
		float temp;
		if (pmax > max) {temp = pmax; pmax = max; max = temp;}
		if (pmin > min) {temp = pmin; pmin = min; min = temp;}
		
		// make sure the intensity ramp (transition region from 0 to 1 (full intensity))
		// is at least half a pixel long (there is a divide by difference in there)
		// also this ensures a smooth line
		pmax -= half_rampwidth;
		pmin -= half_rampwidth;
		max += half_rampwidth;
		min += half_rampwidth;
				
		union uPixel pixel;
		
		// clear line buffer to zero intensity
		for (int i=0; i<height; i++) vline[i]=0.0f;
		
		// add a  (0) ~~~~~~~~~~~\_____ (-1) ramp for maximum
		draw_ramp(height, vline, pmax, max, -1.0f);
		
		// add a  (0) ____/~~~~~~~~~~~~ (1) ramp for minimum
		draw_ramp(height, vline, pmin, min, 1.0f);
		
		// together this is a (0)____/~~~~(1)~~~~\___(0) intensity ramp
		// representing an anti aliased vertical slice of a waveform
		
		pixel.pixel.a = 0xff;
		
		float r1 = (float)cBG.pixel.r;
		float g1 = (float)cBG.pixel.g;
		float b1 = (float)cBG.pixel.b;
		
		float r2 = (float)cFG.pixel.r - r1;
		float g2 = (float)cFG.pixel.g - g1;
		float b2 = (float)cFG.pixel.b - b1;
				
		SInt32 vPixelIndex=0;
		SInt32 row=0;
		
		while (vPixelIndex < height)
		{
			float blend = vline[vPixelIndex];
            
			pixel.pixel.r = (UInt8)(r1 + blend*r2);
			pixel.pixel.g = (UInt8)(g1 + blend*g2);
			pixel.pixel.b = (UInt8)(b1 + blend*b2);
#ifdef DEBUG			
			ZzRtAssert(pix + row < endxPixel);
#endif //#ifdef DEBUG			
			pix[row] = pixel.packed;
			row += width;
			vPixelIndex++;
		}
		pix++;
		curPreviewPoint += previewStride;
    }
    
    delete [] vline;

}

- (void)updateCachedImageFrom:(SInt64)startTime to:(SInt64)endxTime wPixels:(float)imWidth hPixels:(float)imHeight
{
	// startTime and endxTime are expressed in clip coordinates
	// which always start at 0 and end at GetDuration()
	
	MEAudioClip* clip = dynamic_cast<MEAudioClip*>([[self representedObject] WSObject]);
	ZzAssert(clip);

#ifdef ALLOW_AUDIO_REP_DRAWING_DISABLED
	if(gAudioRepDrawingDisabled)
		return;
#endif //#ifdef ALLOW_AUDIO_REP_DRAWING_DISABLED

    if ( fStopImageUpdates ) return ; // No updates during time stretch

	// this is the screen zoom value
	double requestedFramesPerPixel = (double)(endxTime-startTime)/imWidth;

	ZzAssert(requestedFramesPerPixel > 0);
    		
	MEAudioClipData* data = clip->GetData();

	if( requestedFramesPerPixel > 0.5 )
	{
		// we have an integer ratio
		SInt32 finalRatio;
			
		if(requestedFramesPerPixel <= 1.0)
			finalRatio = 1; // one frame per pixel

        // +0.5f to eliminate roundoff errors
        finalRatio = (int)(requestedFramesPerPixel + 0.5f);
		
 		// if the already-existing cached image has the correct ratio
        // and the height requested is equal to the current height
        // and the image at least covers the requested range, don't do anything...
 		if(fCachedImage != nil)
		{
			double framesPerPixelInCachedImage = 
                (fCachedImageEndxTime-fCachedImageStartTime)/([fCachedImage size].width);
		
            // another rounding workaround. a "cache hit performance hack"
			if( (0.9999*framesPerPixelInCachedImage <= (double)finalRatio) &&
				fCachedImageStartTime <= startTime &&
				fCachedImageEndxTime >= endxTime && 
                [fCachedImage size].height == imHeight)
				return;
		}
		
		// so, the width (in pixels) of the image is the number
		// of clip data sample points (those whose first bin starts
		// at 0) needed to contain the first time sample point and
		// the last time sample point
		SInt32 width = (endxTime-1)/finalRatio - startTime/finalRatio + 1;
        SInt32 height = (SInt32)imHeight;
        
        // make sure cached image is at least 32 pixels heigh, rescale if necessary
        if (height < 16) height = 16;
				
		// the last pixel will represent the bin index 
		// that contains endTime
				
		NSBitmapImageRep* bitmap = [[NSBitmapImageRep alloc]
                initWithBitmapDataPlanes:	NULL	// Let the class allocate it
                pixelsWide:			width
                pixelsHigh:			height
                bitsPerSample:		8	// Each component is 8 bits (one byte)
                samplesPerPixel:	4	// Number of components (R, G, B, alpha)
                hasAlpha:			YES
                isPlanar:			NO
                colorSpaceName:		NSDeviceRGBColorSpace
                bytesPerRow:		width*4	// 0 means: Let the class figure it out
                bitsPerPixel:		32	// 0 means: Let the class figure it out
                ];
//		vernl(6,"***PTRPTR bitmap = " << (void*)bitmap);

		
		SInt32* dstPixels = (SInt32 *)[bitmap bitmapData];
		SInt32* endAddress = dstPixels + width*height;
		
        union uPixel cBG;	// background
        union uPixel cFG;	// foreground (waveform)
        union uPixel cZERO;	// zero line
        union uPixel cSEP;	// separator line
            
		float r,g,b,a;
		[fBackgroundColor getRed:&r green:&g blue:&b alpha:&a];
            
        // setup the colours
        cBG.pixel.r = (UInt8)(255.0f*r);
        cBG.pixel.g = (UInt8)(255.0f*g);
        cBG.pixel.b = (UInt8)(255.0f*b);
        cBG.pixel.a = (UInt8)(255.0f*a);
		
		if ( [self isSelected] && [[[fTLCRep mainViewController] selection] type] == kMIWSSelectionTypeObjects )
		{
			cFG.pixel.r = 
			cFG.pixel.g = 
			cFG.pixel.b = 0xff;
		}
		else
		{
			cFG.pixel.r = 
			cFG.pixel.g = 
			cFG.pixel.b = 0x00;
		}
		cFG.pixel.a = 0xff; 
		
        cZERO.pixel.r = 
        cZERO.pixel.g = 
        cZERO.pixel.b = 0x60;
        cZERO.pixel.a = 0xff;

        cSEP.pixel.r = 
        cSEP.pixel.g = 
        cSEP.pixel.b = 0x80;
        cSEP.pixel.a = 0xff;
            
		if(data->GetMainPreview() != NULL)
		{
        
            int chans;
            if (data->IsStereo()) chans=2;
            else chans=1;
			
			// now we need to know the averaged (by finalRatio) bins between
			SInt64 reqStartTimeBinIndex = (startTime)/finalRatio;
			// and
			SInt64 reqEndTimeBinIndex = (endxTime-1)/finalRatio;
			
			// nb bins is the number of pixels of the image
			SInt64 nbBins = (reqEndTimeBinIndex - reqStartTimeBinIndex + 1);
			
            // one extra bin due to the boundary condition
            // we use 2 preview points per pixel, and draw a 1 pixel wide polygon
            // so in short, one more element than the number of pixels
			MEAudioClipDataPreviewPoint* previewPoints = new MEAudioClipDataPreviewPoint[(nbBins+1)*chans]; //$$$stereo
        
			data->FillPreviewPointArray(
				previewPoints,
				(nbBins)*chans, // this param is for debugging
				finalRatio,
				nbBins,
				reqStartTimeBinIndex,
				reqEndTimeBinIndex,
				kCallingThread_AppKit
            );
            // solve boundary condition by copying last previewpoint
            // not very clean but it prevents rewriting the entire preview generation code
            // lets call it an acceptable hack
            // $$$TODO low priority: rethink everything

            if(data->IsStereo()){
                previewPoints[nbBins*2].min   = previewPoints[(nbBins-1)*2].min;
                previewPoints[nbBins*2].max   = previewPoints[(nbBins-1)*2].max;
                previewPoints[nbBins*2+1].min = previewPoints[(nbBins-1)*2+1].min;
                previewPoints[nbBins*2+1].max = previewPoints[(nbBins-1)*2+1].max;
            }
            else{
                previewPoints[nbBins].min   = previewPoints[(nbBins-1)].min;
                previewPoints[nbBins].max   = previewPoints[(nbBins-1)].max;
            }
            
            // reserve 3 lines for separators
            SInt32 waveformtotal = height - 3;
            SInt32 waveformleft = waveformtotal >> 1;
            SInt32 waveformright = waveformtotal - waveformleft;

            // make sure there is at least one line per channel
            // might be better to keep some minimal height internally
            //ZzAssert(waveformtotal >= 2);
            
            // setup the destination of separator lines
            // right waveform starts at bottom
            SInt32 rwavebot = 0;
            // reserve waveformright pixels for right waveform
            SInt32 rwavetopx = rwavebot + width * (waveformright);
            // blank (background colour) separation line between right and left
            SInt32 blankmid =  rwavetopx;
            
            // left waveform starts above blank separation line0
            SInt32 lwavebot = blankmid + width;
            // reserve waveformleft pixels for left waveform
            SInt32 lwavetopx = lwavebot + width * (waveformleft);

            // top of pixel are 2 separator lines
            SInt32 septop1 = lwavetopx;
            SInt32 septop2 = lwavetopx + width;

#ifdef DEBUG
            {
                SInt32 expected_top2 = width * (height - 1);
                ZzAssert(septop2 == expected_top2);
            }
#endif //DEBUG

            SInt32 *pix;
            
            // draw vertical lines
           for (pix = dstPixels; pix < dstPixels+width; pix++){

                // top separators
               pix[septop1]  = cSEP.packed;                
               pix[septop2]  = cSEP.packed;

                // blank separator line between left and right
               pix[blankmid] = cBG.packed;
            }

                     
			MEAudioClipDataPreviewPoint* curPreviewPoint = previewPoints;

            // THE FOLLOWING WAVEFORM DRAWING CODE CALL HAS A POINTER BUG $$$TODO	 <------ ??????????????
			if (data->IsStereo())
            {
				draw_waveform(dstPixels+lwavebot, width, waveformleft, curPreviewPoint, chans, cFG, cBG,endAddress);            
				draw_waveform(dstPixels+rwavebot, width, waveformright, curPreviewPoint+1, chans, cFG, cBG,endAddress);
			}
			else
			{
				draw_waveform(dstPixels, width, height, curPreviewPoint, chans, cFG, cBG, endAddress);
			}

			delete [] previewPoints;

			// store the cached image start and end times so that the drawing 
			// engine knows when to regenerate the image
			fCachedImageStartTime = reqStartTimeBinIndex*finalRatio;
			fCachedImageEndxTime = (reqEndTimeBinIndex+1)*finalRatio;
		}
		else
		{
			// if there is no available preview yet, we simply fill the cached image
			// to background colour.
			for(int i=0; i < width*height; i++) dstPixels[i] = cBG.packed;
			fCachedImageStartTime = startTime;
			fCachedImageEndxTime = endxTime;
		}
		NSImage* image = [[NSImage alloc] initWithSize:NSMakeSize(width,imHeight)];

		[image addRepresentation:bitmap];
		// the image has retained the rep so wa can safely delete it
		[bitmap release];
		[fCachedImage release];
		fCachedImage = image;
		// think about it... binNumber*finalRatio = the FIRST time in the
		// bin, while the end time of this image can be seen as the last
		// time of the bin, which is the first time of the NEXT bin minus one	
	}
	else
	{
		//  NOT IMPLEMENTED
		// since at this moment we don't use sub audio frame waveform display resolution
		// i broke this, see cvs for prev version [TS]
		ZzAssert(0);	
	}
}

//#define VERBOSE_drawBaseBodyToPoint

- (void)drawBaseBodyRect:(NSRect)frameInDrawCoords clippedTo:(NSRect)clippingRect
{
//	return;  ////$$$DEBUG___TEST___DEBUG___TEST remove this line !!!!!!!!!!!!!

//	NSBezierPath		*aPath;
//	static NSImage		*lTrimIndicator = nil;
//	static NSImage		*rTrimIndicator = nil;
	
	if(clippingRect.size.width == 0.0 || clippingRect.size.height == 0.0 )
		return;

	MIMouseTracker *mouseTracker = [fTLCRep mouseTracker];
	if ( ( mouseTracker ) && ( [mouseTracker respondsToSelector:@selector(drawCachedImage:inFrame:clippedTo:)] ) && (self == [(MIResizeObjectTracker*)mouseTracker resizedObject]) )
	{
		[mouseTracker drawCachedImage:fCachedImage inFrame:frameInDrawCoords clippedTo:clippingRect];
		return;
	}

	// clippingRect is the rect where we need to draw in draw coords
	// aPoint is the origin of our whole frame in draw coords.
	// clipStart is the x origin of this rect, for syntax clarity
	// and clipEnd is the x end of this rect
	float clipStart = clippingRect.origin.x;
	float clipEndx = clippingRect.origin.x + clippingRect.size.width;
	// wsObj is the Arme object of the MIObject we represent
	TArmeWSObj* wsObj = [[self representedObject] WSObject];
	SInt64 armeTotalLength = GET_VAL(wsObj->GetDuration());
	ZzAssert(armeTotalLength > 0);
	// we are going to use a proportional rule to find
	// the start time and time between which we need to
	// draw. In our coords,
	// pixel ourFrameInDrawCoords.origin.x is arme time 0
	// pixel ourFrameInDrawCoords.origin.x+ourFrameInDrawCoords.size.width is arme time armeTotalLength
	// thus, this means 
	double framesPerPixelInDrawCoords = (double)armeTotalLength/frameInDrawCoords.size.width;
	/*
		ROUNDING ERROR HERE
		since the drawStartTime and drawEndxTime are used to recompute the frame rate per pixel in the called
		function, we better make sure we get the same result.
		for non integral values of clipStart - frameInDrawCoords and the other one this can lead to a difference.
		So, either we convert the thing here
		or, we can make sure we request always and only integral values.
		or, we can pass the correct frame rate along with the other arguments
		or, we can brute force convert the framerate to be a multiple of kMainPreviewBinSize on the other end.
	*/
	// solution 1: ask to draw more and pass the correct values
	// thus, the start and end time of the region to draw are :
	SInt64 drawStartTime = (SInt64)(floor((clipStart - frameInDrawCoords.origin.x))*framesPerPixelInDrawCoords);
	if(drawStartTime < 0)
		drawStartTime = 0;
	if(drawStartTime > armeTotalLength)
		drawStartTime = armeTotalLength;
	SInt64 drawEndxTime = (SInt64)(ceil((clipEndx - frameInDrawCoords.origin.x))*framesPerPixelInDrawCoords);	
	if(drawEndxTime < 1)
		drawEndxTime = 1;
	if(drawEndxTime > armeTotalLength)
		drawEndxTime = armeTotalLength; // $$$AUDIO_DRAW_FIX  these awful hacks are needed because of rounding errors. they will trigger when displaying clips
										// at a very very low zoom level (bird's eye view)
		
	// extra conversion to make sure we get the right frame rate on the other side
	double wPixels = (drawEndxTime - drawStartTime)/framesPerPixelInDrawCoords;
	
	// SAFETY CHECK
	if(fStopImageUpdates)
		return; ///$$$TODO2002-11-07-AUDIOCLIPREP_DRAW_PROBLEM why do we reach this point ?????
	
	// now, we are going to ask a new image that at least contains these
	// two start times
	[self	updateCachedImageFrom:drawStartTime 
			to:drawEndxTime 
			wPixels:wPixels
			hPixels:frameInDrawCoords.size.height];
	ZzAssert(fCachedImage != nil);
	// make sure the image contains those two points : 
	ZzAssert(fCachedImageStartTime <= drawStartTime);
	ZzAssert(fCachedImageEndxTime >= drawEndxTime);
	// we are now going to compute the start and end pixel in the source image
	// pixel 0 is time 	fCachedImageStartTime
	// pixel imageWidth is time fCachedImageEndTime
	// thus, this means
	double framesPerPixelInCachedImage = (fCachedImageEndxTime-fCachedImageStartTime)/([fCachedImage size].width);
	//... draw the image...
	float startPixelInCachedImage = (drawStartTime-fCachedImageStartTime)/framesPerPixelInCachedImage;
	float endxPixelInCachedImage = (drawEndxTime-fCachedImageStartTime)/framesPerPixelInCachedImage;
	float bottomPixelToDraw = NSMinY(clippingRect) - NSMinY(frameInDrawCoords);
	float topPixelToDraw = NSMaxY(clippingRect) - NSMinY(frameInDrawCoords);
	[fCachedImage drawInRect:clippingRect fromRect:NSMakeRect(startPixelInCachedImage,bottomPixelToDraw,endxPixelInCachedImage-startPixelInCachedImage,topPixelToDraw-bottomPixelToDraw) operation:NSCompositeCopy fraction:1.0];


// draw trim indicators

	if ( GET_VALUE([fRepresentedObject WSObject]->GetStartTrim()) )
		[mainImageController->fTrimIndicatorLeft compositeToPoint:frameInDrawCoords.origin operation:NSCompositeSourceAtop];
	else
		[mainImageController->fNonTrimIndicatorLeft compositeToPoint:frameInDrawCoords.origin operation:NSCompositeSourceAtop];		

	if ( GET_VALUE([fRepresentedObject WSObject]->GetEndTrim()) )
		[mainImageController->fTrimIndicatorRight compositeToPoint:NSMakePoint(NSMaxX(frameInDrawCoords)-10, NSMinY(frameInDrawCoords)) operation:NSCompositeSourceAtop];
	else
		[mainImageController->fNonTrimIndicatorRight compositeToPoint:NSMakePoint(NSMaxX(frameInDrawCoords)-10, NSMinY(frameInDrawCoords)) operation:NSCompositeSourceAtop];		


	if ( [self isSelected] && [[[fTLCRep mainViewController] selection] type] == kMIWSSelectionTypeObjects )
		[[NSColor colorWithCalibratedRed:0.8 green:0.8 blue:0.8 alpha:0.5] set];
	else
		[[NSColor colorWithCalibratedRed:0.2 green:0.2 blue:0.2 alpha:0.5] set];
    [NSBezierPath strokeLineFromPoint:NSMakePoint(NSMinX(frameInDrawCoords),NSMinY(frameInDrawCoords)) toPoint:NSMakePoint(NSMinX(frameInDrawCoords), NSMaxY(frameInDrawCoords))];
    [NSBezierPath strokeLineFromPoint:NSMakePoint(NSMaxX(frameInDrawCoords),NSMinY(frameInDrawCoords)) toPoint:NSMakePoint(NSMaxX(frameInDrawCoords), NSMaxY(frameInDrawCoords))];

}

- (void)invalidateCachedImage
{
	// this is always called in the AppKit thread so.. GO !
	[fCachedImage release];
	fCachedImage = nil;
}

- (void)setEnableCachedImageUpdates:(BOOL)flag
{
    fStopImageUpdates = !flag;
}

- (void)dealloc
{
	[fCachedImage release];
	[super dealloc];
}

@end
