/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import <AppKit/AppKit.h>
#import "MISequenceGrabberView.h"
#import "TVideoClip.h"
#import "TArmeDoc.h"
#import "MIMontageDocument.h"
#import "ArmeDataFileRef.h"
#import "MIObject.h"
#import "MovieHelper.h"
#import "MIMainViewController.h"
#import "MITopLevelContainerRep.h"
#import "MIPlayStopPauseController.h"

#ifndef MEAudioRecordManager_h
#include "MEAudioRecordManager.h"
#endif //MEAudioRecordManager_h

enum MIAudioModes
{
	kAudioGrabbing,
	kAudioRecording,
	kAudioPlayingback
};

@interface MIAudioTake : NSObject {
	SInt32				fAudioTakeIndex;
	BOOL				fSelected;
}

- (void)setAudioTakeIndex:(SInt32)audioTakeIndex;
- (SInt32)audioTakeIndex;

- (void)setSelected:(BOOL)flag;
- (BOOL)isSelected;
	
@end

@interface MIAudioRecController : NSWindowController 
{
@public
	IBOutlet id 		fCapturePlayButton;
	IBOutlet id 		fCaptureStopButton;
	IBOutlet id 		fCapturePauseButton;
	IBOutlet id 		fCaptureRecordButton;
	
	IBOutlet id 		fTakesPopupButton;
	IBOutlet id 		fDiscardButton;
	IBOutlet id 		fKeepCheckbox;
	IBOutlet id 		fImportButton;
	IBOutlet id 		fCancelButton;
	
	long long			fDiskFreeBytes;

    FSSpec 				fCaptureFile;
	NSMutableArray		*fAudioRecParts;
	IBOutlet id 		fAudioRecPartContainer;
	BOOL				fThreadRunning;
    BOOL				fThreadContinue;
	MEAudioRecordManager *fAudioManager;
	NSMutableArray		*fTakes;
	MIAudioTake			*fCurrentTake;
	IBOutlet id			fDurationField;
	IBOutlet id			fDiskFreeSpace;
	MIAudioModes		fMode;
	NSTimer				*fTimeTimer;
	NSTimer				*fCameraTimer;
    NSTimer				*fFreeSpaceTimer;
    NSTimer				*fPlayTimer;
	
	long				fTime;
	SInt32				fCurrentTakeIndex;
}

+ (id)sharedAudioRecController;
- (id)init;
- (void)dealloc;
	
- (IBAction)stopRecording:(id)sender;

- (IBAction)show:(id)sender;

- (IBAction)record:(id)sender;

- (IBAction)keepTake:(id)sender;
- (IBAction)keepAllTakes:(id)sender;
- (IBAction)keepNoTakes:(id)sender;
- (IBAction)discardTake:(id)sender;

- (IBAction)importSelectedTakes:(id)sender;

- (IBAction)playTake:(id)sender;
- (IBAction)stopTake:(id)sender;
- (IBAction)pauseTake:(id)sender;

- (IBAction)takeSelected:(id)sender;

- (void)createAudioRecParts;
- (void)terminateDrawingThread;
- (void)sprawnDrawingThread;
- (void)perpetualRedrawLoop:(id)argument;
- (void)updateTakesPopupButton;
- (void)setMode:(MIAudioModes)mode;
- (void)freeSpaceTimer:(NSTimer*)theTimer;

- (IBAction)importAndClose:(id)sender;
- (IBAction)cancelAndClose:(id)sender;
	
@end