/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import <Cocoa/Cocoa.h>

#define _setjmp setjmp
#import <AppKit/AppKit.h>
#import "MIAudioRecController.h"
#import "MIImageController.h"
#import "MIAudioRecPartController.h"
#import "MIGenericBackgroundView.h"
#import "MIMacroCommand.h"
#import "MICreateWSObjsFromFileCommand.h"
#import "MICreateAudioClipsFromRecordedTakesCommand.h"
#import "MITime.h"
#import "MIMontageDocument.h"
#import "MIApplication.h"
#include <math.h>

#if !defined(__MWERKS__)
#import <setjmp.h>
#include <sys/param.h>
#include <sys/mount.h>
#endif //!defined(__MWERKS__)

#ifndef MovieHelper_h
#include "MovieHelper.h"
#endif //MovieHelper_h

#ifndef MEAudioDeviceDesc_h
#include "MEAudioDeviceDesc.h"
#endif //MEAudioDeviceDesc_h

#import "MIPreferences.h"
#import "MIApplicationController.h"


@implementation MIAudioTake

- (id)init
{
	if (( self = [super init] ))
	{
		fSelected = NO;
	}
	return self;
}

- (void)dealloc 
{
	MEAudioRecordManager::SharedAudioRecordManager()->DiscardTake(fAudioTakeIndex);
	[super dealloc];
}

- (void)setAudioTakeIndex:(SInt32)audioTakeIndex {
	fAudioTakeIndex = audioTakeIndex;
}

- (SInt32)audioTakeIndex {
	return fAudioTakeIndex;
}

- (void)setSelected:(BOOL)flag {
	fSelected = flag;
}

- (BOOL)isSelected {
	return fSelected;
}

@end


@implementation MIAudioRecController

+ (id)sharedAudioRecController {
    static id sharedAVCaptureController = nil;
    if ( sharedAVCaptureController == nil ) {
        sharedAVCaptureController = [[self alloc] init];
    }
    return sharedAVCaptureController;
}

- (id)init {
	if (( self = [super initWithWindowNibName:@"AudioRec"] )) {
		fThreadRunning = NO;
		fAudioRecParts = [[NSMutableArray alloc] initWithObjects:nil];
		fTakes = [[NSMutableArray alloc] initWithObjects:nil];
	
		[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(applicationWillTerminate:) name:NSApplicationWillTerminateNotification object:nil];
		
		fAudioManager = MEAudioRecordManager::SharedAudioRecordManager();
		
		fMode = kAudioGrabbing;
    }
    return self;
}

- (void)dealloc {
	[[NSNotificationCenter defaultCenter] removeObserver:self name:NSApplicationWillTerminateNotification object:nil];

	[fAudioRecParts release];
	[fTakes release];

	[super dealloc];
}

- (void)applicationWillTerminate:(NSNotification*)theNotification {
    [self release];
}

- (void)awakeFromNib {
	[fCapturePlayButton setImage:mainImageController->fPlayBarButtonPlayOff];
    [fCapturePlayButton setAlternateImage:mainImageController->fPlayBarButtonPlayOn];

    [fCaptureStopButton setImage:mainImageController->fPlayBarButtonStopOff];
    [fCaptureStopButton setAlternateImage:mainImageController->fPlayBarButtonStopOn];

    [fCapturePauseButton setImage:mainImageController->fPlayBarButtonPauseOff];
    [fCapturePauseButton setAlternateImage:mainImageController->fPlayBarButtonPauseOn];
	
    [fCaptureRecordButton setImage:mainImageController->fPlayBarButtonRecordOff];
    [fCaptureRecordButton setAlternateImage:mainImageController->fPlayBarButtonRecordOn];
}

- (void)createAudioRecParts
{
	SInt32 i;
	MIAudioRecPartController *ctrl= nil;
	float lastPos = 0;

	for ( i = 0 ; i < fAudioManager->GetNbInputs(); i += 2 )
	{
		ctrl = [[MIAudioRecPartController alloc] initWithIndex:i];
		[fAudioRecPartContainer addSubview:ctrl->fAudioRecPartView];
		NSString* partTitle = [NSString stringWithFormat:@"Input %d/%d",i+1,i+2];
		[ctrl->fTitleText setStringValue:partTitle];
		[ctrl->fAudioRecPartView setFrameOrigin:NSMakePoint(lastPos, 0)];
		lastPos = NSMaxX([ctrl->fAudioRecPartView frame]) - 1;
		[fAudioRecParts addObject:ctrl];
		[ctrl release];
	}

	// fill if less than 3 parts
	for ( ; i < 6; i += 2 )
	{
		id view = [[MIGenericBackgroundView alloc] initWithFrame:NSMakeRect( lastPos, 0, 100, 252 )];
		[fAudioRecPartContainer addSubview:view];
		lastPos += 100;
		[view release];
	}

	if ( lastPos )
	{
		[fAudioRecPartContainer setFrameSize:NSMakeSize(lastPos - 1, NSHeight([ctrl->fAudioRecPartView frame]))];
	}

	[self updateTakesPopupButton];
	[self takeSelected:nil];
}

- (void)deleteAudioRecParts {
	NSArray *array = [fAudioRecPartContainer subviews];
	int x;

	for ( x = [array count] - 1 ; x >= 0  ; x-- ) {
		[[array objectAtIndex:x] removeFromSuperview];
	}
	
	[fAudioRecParts removeAllObjects];
}

- (IBAction)show:(id)sender 
{
	[self window]; // force window to be loaded
	MIMontageDocument* document = [[NSDocumentController sharedDocumentController] currentDocument];
	TArmeDoc* doc = [document armeDocument];

	fCurrentTakeIndex = -1;
	fMode = kAudioGrabbing;

	// step 1 : we retrieve the audio input device string code as defined in the prerefences
	NSString* inDevStringCode = [[MIPreferences sharedPrefs] audioInputDeviceStringCode];
//	vernl(0,"AVCap inDevStringCode = " << [inDevStringCode UTF8String]);
	
	// step 2 : translate the audio device string code into a CoreAudio AudioDeviceID
	AudioDeviceID inDevID = MEAudioDeviceDesc::GetAudioDeviceIDFromStringCode((CFStringRef)inDevStringCode);
//	vernl(0,"AVCap inDevID = " << inDevID);
	
	if(inDevID == 0x00000000) // this means the device hasn't been found
	{	
		NSString* defaultInDevStringCode = [NSString stringWithString:(NSString*)MEAudioDeviceDesc::GetDefaultInputStringCode()]; // returns a code that is 'always use default device'
		[[MIPreferences sharedPrefs] setAudioInputDeviceStringCode:defaultInDevStringCode];
		inDevID = MEAudioDeviceDesc::GetAudioDeviceIDFromStringCode((CFStringRef)defaultInDevStringCode);
		
		if(inDevID == 0x00000000)
		{
			NSRunAlertPanel(@"Error", @"No suitable Audio Input Device found.", @"OK", nil, nil);
			return;
		}
		else
		{
			NSRunAlertPanel(@"Warning", @"The Audio Input Device chosen in the preferences cannot be opened. HyperEngine-AV now uses the default input device.", @"OK", nil, nil);
		}
	}
	
	if(inDevID != 0)
	{
		fAudioManager->PrepareRecordingForDoc(doc,inDevID);
		if(fAudioManager->GetNbInputs() < 2)
		{
			NSRunAlertPanel(@"Warning", @"Sorry, your device input channels cannot be opened for recording.", @"OK", nil, nil);
			return;
		}	
	}
		
	// now the hardware recorder engine is created. 
	// let's check if sample rates match
	if(fAudioManager->GetAudioFrameRate() != doc->GetAudioFrameRate())
	{
		NSRunAlertPanel(@"Warning", @"We're sorry, but the recording sample rate does not match the document sample rate. You will need to switch the sample rate on your input device or resample your recorded clips.", @"OK", nil, nil);
	}

	if(inDevID != 0)
		[self createAudioRecParts];
	
	[self setMode:kAudioGrabbing];
	[self updateTakesPopupButton];

	[fKeepCheckbox setState:NSOffState];
				
	fFreeSpaceTimer = [NSTimer timerWithTimeInterval:5 target:self selector:@selector(freeSpaceTimer:) userInfo:nil repeats:YES];
	[[NSRunLoop currentRunLoop] addTimer:fFreeSpaceTimer forMode:NSModalPanelRunLoopMode];
	[self freeSpaceTimer:nil];
	
	// let's start the mixer drawing thread and enable the audio monitoring
	[self sprawnDrawingThread];
	fAudioManager->EnableMonitoring();		

	[fAudioRecParts makeObjectsPerformSelector:@selector(prepareControls)];
	[fDurationField setStringValue:@"--:--:--"];
	
	// Window MUST be visible before calling [self openAll]
	[[self window] center];
	[[self window] makeKeyAndOrderFront:sender];
		
	[NSApp runModalForWindow:[self window]];
	[[self window] orderOut:self];
}


- (void)setFreeDiskSpace:(long long)diskFreeBytes {
	double size;
	NSString *unit;

	if ( diskFreeBytes < pow( 1024, 1 ) ) {
		size = diskFreeBytes; unit = @"bytes";
	} else if ( diskFreeBytes < pow( 1024, 2 ) ) {
		size = diskFreeBytes / pow( 1024, 1 ); unit = @"Kb";
	} else if ( diskFreeBytes < pow( 1024, 3 ) ) {
		size = diskFreeBytes / pow( 1024, 2 ); unit = @"Mb";
	} else if ( diskFreeBytes < pow( 1024, 4 ) ) {
		size = diskFreeBytes / pow( 1024, 3 ); unit = @"Gb";
	} else if ( diskFreeBytes < pow( 1024, 5 ) ) {
		size = diskFreeBytes / pow( 1024, 4 ); unit = @"Tb";
	} else if ( diskFreeBytes < pow( 1024, 6 ) ) {
		size = diskFreeBytes / pow( 1024, 5 ); unit = @"Hb";
	}
	[fDiskFreeSpace setStringValue:[NSString stringWithFormat:@"%.2lf %@", size, unit]];
}


#define kMinimumFreeSpace 40
- (void)freeSpaceTimer:(NSTimer*)theTimer {
#if !defined(__MWERKS__)

	NSString *mountPoint = [[[[NSDocumentController sharedDocumentController] currentDocument] fileName] mountPoint];

	if ( fMode != kAudioRecording ) {
		struct statfs buffer;
		int ret;

		ret = statfs( [mountPoint fileSystemRepresentation], &buffer );

		if ( ret == 0 ) {
			fDiskFreeBytes = ((long long)buffer.f_bsize) * ((long long)buffer.f_bfree);
			[self setFreeDiskSpace:fDiskFreeBytes];
		}  else [fDiskFreeSpace setStringValue:@"???"];

	} else {
		long long remainingFreeDiskBytes;

		struct statfs buffer;
		int ret;

		ret = statfs( [mountPoint fileSystemRepresentation], &buffer );

		if ( ret == 0 ) {
			remainingFreeDiskBytes = ((long long)buffer.f_bsize) * ((long long)buffer.f_bfree);
		}

		[self setFreeDiskSpace:remainingFreeDiskBytes];

		if ( remainingFreeDiskBytes <  kMinimumFreeSpace ) {
			[self stopRecording:nil];
			[[self window] setLevel:[[self window] level] - 1];
			NSRunAlertPanel( @"Recording has stopped.", @"Free disk space is %@.", @"OK", nil, nil, [fDiskFreeSpace stringValue] );
			[[self window] setLevel:[[self window] level] + 1];
		}
	}
#else //#if !defined(__MWERKS__)
	[fDiskFreeSpace setStringValue:@"???"];
#endif //#if !defined(__MWERKS__)
}

- (void)timeTimer:(NSTimer*)theTimer {
	int seconds;
	int minutes;
	int hours;

	fTime++;
	hours = fTime / 3600 ;
	minutes = ( fTime - (hours * 3600) ) / 60 ;
	seconds = fTime - (hours * 3600) - (minutes * 60);

	[fDurationField setStringValue:[NSString stringWithFormat:@"%02d:%02d:%02d", hours, minutes, seconds]];
}


- (void)cleanup {
	[self setMode:kAudioGrabbing];
	[fFreeSpaceTimer invalidate];
	[self terminateDrawingThread];
	[[self window] orderOut:self];
}

- (void)finalCleanup {
	[self deleteAudioRecParts];
	[fTakes removeAllObjects];
	fAudioManager->PutInIdleState();
	[[[NSDocumentController sharedDocumentController] currentDocument] deleteUnusedFiles];
}

- (IBAction)importAndClose:(id)sender {
	BOOL displayAlert = YES;
	unsigned int x;

	for ( x = 0 ; x < [fTakes count] ; x++ ) {
		if ( [[fTakes objectAtIndex:x] isSelected] ) displayAlert = NO;
	}

	if ( ( displayAlert ) && ( [fTakes count] > 0 ) ) {
		[[self window] setLevel:[[self window] level] - 1];
		int reply = NSRunAlertPanel(@"No take will be imported.", @"Discard all takes ?", @"Cancel", @"Discard", nil);
		[[self window] setLevel:[[self window] level] + 1];

		if ( NSAlertDefaultReturn == reply ) {
			return ;
		} else {
			[NSApp stopModal];
			[self cleanup];
			[self finalCleanup];
			return;
		}
	}

	[NSApp stopModal];
	[self cleanup];
	[self performSelector:@selector(importSelectedTakes:) withObject:sender afterDelay:.2 inModes:[NSArray arrayWithObject:NSDefaultRunLoopMode]];
}


- (IBAction)cancelAndClose:(id)sender {

	if ( [fTakes count] > 0 ) {
		[[self window] setLevel:[[self window] level] - 1];
		int reply = NSRunAlertPanel(@"All takes are about to be deleted.", @"Discard all takes ?", @"Cancel", @"Discard", nil) ;
		[[self window] setLevel:[[self window] level] + 1];
		if ( NSAlertDefaultReturn == reply ) {
			return ;
		} else {
			[NSApp stopModal];
			[self cleanup];
			[self finalCleanup];
		}
	} else {
		[NSApp stopModal];
		[self cleanup];
		[self finalCleanup];
	}
}


- (void)terminateDrawingThread {
    if ( fThreadRunning ) {
        fThreadContinue = NO;
        while ( !fThreadContinue ) {
            //[NSThread sleepUntilDate:[NSDate dateWithTimeIntervalSinceNow:0.2]];
			usleep( 200000 );
        }
        fThreadRunning = NO;
    }
}

- (void)sprawnDrawingThread {
	if ( !fThreadRunning ) {
		fThreadContinue = YES;
		fThreadRunning = YES;
		[NSApplication detachDrawingThread:@selector(perpetualRedrawLoop:) toTarget:self withObject:nil];
	}
}


- (void)perpetualRedrawLoop:(id)argument {
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];

	usleep( 500000 );

    while ( fThreadContinue ) {
		timeval tps, tpe;
		gettimeofday(&tps, nil);

        [fAudioRecParts makeObjectsPerformSelector:@selector(updateVUMeters)];
		gettimeofday(&tpe, nil);
		tpe.tv_sec = tpe.tv_sec - tps.tv_sec;
		if ( tpe.tv_sec == 0 ) {
			tpe.tv_usec = tpe.tv_usec - tps.tv_usec;
			if ( tpe.tv_usec < 41666 ) {
				usleep( 41666 - tpe.tv_usec );
			}
		}
    }
	[pool release];
    fThreadContinue = YES;
}

- (void)updateTakesPopupButton {
	unsigned int i;
	NSString* str;

	[fTakesPopupButton removeAllItems];

    for ( i = 0 ; i < [fTakes count]; i++ ) {
		MIAudioTake *takeObject = [fTakes objectAtIndex:i];
		if ( [takeObject isSelected] ) str = [NSString stringWithFormat:@"Take %d - import", i+1];
		else str = [NSString stringWithFormat:@"Take %d", i+1];
		[fTakesPopupButton addItemWithTitle:str];
    }

	if ( [fTakes count] > 0 ) {
		[fTakesPopupButton selectItemAtIndex:0];
		[fTakesPopupButton setEnabled:YES];
		[fCapturePlayButton setEnabled:YES];
		[fCaptureStopButton setEnabled:YES];
		[fCapturePauseButton setEnabled:YES];
		[fDiscardButton setEnabled:YES];
		[fKeepCheckbox setEnabled:YES];
	} else {
		[fTakesPopupButton addItemWithTitle:@"No Takes"];
		[fTakesPopupButton setEnabled:NO];
		[fCapturePlayButton setEnabled:NO];
		[fCapturePauseButton setEnabled:NO];
		[fCaptureStopButton setEnabled:NO];
		[fDiscardButton setEnabled:NO];
		[fKeepCheckbox setEnabled:NO];
	}
}

- (IBAction)takeSelected:(id)sender 
{
	MIMontageDocument* document = [[NSDocumentController sharedDocumentController] currentDocument];
	SInt32 index = [fTakesPopupButton indexOfSelectedItem];
	if ( ( index >= 0 ) && ( [fTakes count] > 0 ) )
	{
		MIAudioTake *takeObject = [fTakes objectAtIndex:index];
		if ( fCurrentTakeIndex != -1 ) [self pauseTake:nil];

		if ( [takeObject isSelected] ) {
			[fKeepCheckbox setState:NSOnState];
		} else {
			[fKeepCheckbox setState:NSOffState];
		}

		fCurrentTakeIndex = index;

		SInt32 take = [takeObject audioTakeIndex];
		ArmeTime takeDuration = fAudioManager->GetTakeDuration(take);
		MITime* time = [MITime timeWithDocument:document];
		[time setArmeTime:takeDuration];
		NSString *str = [time getSMPTEString];
		str = [str substringToIndex:[str length] - 3];
		[fDurationField setStringValue:str];

		[self setMode:kAudioPlayingback];
	} 
	else
	{
		[fDurationField setStringValue:@"--:--:--"];
		fCurrentTakeIndex = -1;
	}
}

- (IBAction)keepTake:(id)sender {
	MIAudioTake *takeObject = [fTakes objectAtIndex:fCurrentTakeIndex];
	[takeObject setSelected:![takeObject isSelected]];
	if ( [takeObject isSelected] ) {
		[fKeepCheckbox setState:NSOnState];
	} else {
		[fKeepCheckbox setState:NSOffState];
	}
	[self updateTakesPopupButton];
	[fTakesPopupButton selectItemAtIndex:fCurrentTakeIndex];
}

- (IBAction)keepAllTakes:(id)sender {
	for ( unsigned int index = 0 ; index < [fTakes count]; index++ ) {
		MIAudioTake *takeObject = [fTakes objectAtIndex:index];
		[takeObject setSelected:YES];
		[self updateTakesPopupButton];
		[fKeepCheckbox setState:NSOnState];
		[fTakesPopupButton selectItemAtIndex:fCurrentTakeIndex];
	}
}

- (IBAction)keepNoTakes:(id)sender {
	for ( unsigned int index = 0 ; index < [fTakes count]; index++ ) {
		MIAudioTake *takeObject = [fTakes objectAtIndex:index];
		[takeObject setSelected:NO];
		[self updateTakesPopupButton];
		[fKeepCheckbox setState:NSOffState];
		[fTakesPopupButton selectItemAtIndex:fCurrentTakeIndex];
	}
}


- (IBAction)importSelectedTakes:(id)sender
{
	unsigned int takeIndex;
	MIMontageDocument* document = [[NSDocumentController sharedDocumentController] currentDocument];
	bool bStackVertically;
	int count = 0 ;
	
	VUSERCMD("Importe Selected Takes (audio)");
	
	MIMacroCommand* command = [[[MIMacroCommand alloc] init] autorelease];
	[command setCommandName:@"Create clips"];

	ArmeTime movieDuration;
	MIPoint location = [[[NSApp mainViewController] topLevelContainerView] newObjectLocation];

	for ( takeIndex = 0 ; takeIndex < [fTakes count]; takeIndex++ )
	{
		MIAudioTake *takeObject = [fTakes objectAtIndex:takeIndex];
		if ( [takeObject isSelected] ) count++;
	}
			

	SInt32 val = [[MIPreferences sharedPrefs] stackingMethod];
//	vernl(6,"[[MIPreferences sharedPrefs] stackingMethod] returned " << val);
	if(val == kPrefsStackingMethodAlwaysAsk)
	{
		if ( count > 1 )
		val = [[MIApplicationController sharedApplicationController] displayChooseStackingMethodForMainViewController:[NSApp mainViewController]];
		else val = kPrefsStackingMethodVertically;
	}
//	vernl(6,"displayChooseStackingMethodForMainViewController: returned " << val);

	if(val == kPrefsStackingMethodCancelled) {
		[self finalCleanup];
		return;
	}
	else if(val == kPrefsStackingMethodVertically)
		bStackVertically = true;
	else if(val == kPrefsStackingMethodHorizontally)
		bStackVertically = false;
	else
		ZzAssert(0);
	

	for ( takeIndex = 0 ; takeIndex < [fTakes count]; takeIndex++ )
	{
		MIPoint originalLocation = location;
		MIAudioTake *takeObject = [fTakes objectAtIndex:takeIndex];
		if ( [takeObject isSelected] )
		{
			MICreateAudioClipsFromRecordedTakesCommand* cmd = [[MICreateAudioClipsFromRecordedTakesCommand alloc] initWithDoc:document];
			SInt32 take = [takeObject audioTakeIndex];
			//ArmeTime takeStart = fAudioManager->GetTakeStartTime(take);
			movieDuration = fAudioManager->GetTakeDuration(take);
			for(SInt32 file = 0; file < fAudioManager->GetTakeNbAudioFiles(take); file++) {
				MEAudioChunkFileIndex index = fAudioManager->GetTakeFile(take,file);
				SInt32 nbChannels = fAudioManager->GetTakeFileNbChannels(take,file);
				NSString* str = [NSString stringWithFormat:@"Take_%d", takeIndex + 1];
				[cmd addFile:index nbChannels:nbChannels position:location duration:movieDuration name:str]; // duration is for debug purposes only since it's known by the file
				GET_VALUE(location.vpos) -= (
#ifdef SMALL_TITLEBAR
								  GET_VALUE(kDefaultClipRepresentationHeight)
#else
								  kDefaultClipRepresentationHeight
#endif
								  + GET_VALUE(kRepTitleHeight) );

			}

			[command addCommand:cmd];
			[cmd release];

			if(!bStackVertically)
			{
				location.vpos = originalLocation.vpos;
				location.time += movieDuration;
			}
			else
			{
				GET_VALUE(location.vpos) -= GET_VALUE(kBetweenClipsHeight);
			}
		}
	}

	[command setCommandName:@"Create Clips"];
	[document doCmd:command];

	if ( [[MIPreferences sharedPrefs] gridShowMask] & kPrefsGridShowVObjLines ||
		 [[MIPreferences sharedPrefs] gridShowMask] & kPrefsGridShowHObjLines )
		[[MIApplicationController sharedApplicationController] redrawAllMainWindows];

	[[[NSApp mainViewController] topLevelContainerView] scrollToNewObject];
	[self finalCleanup];
}


- (IBAction)discardTake:(id)sender {
	[self stopTake:sender];
	[fTakes removeObjectAtIndex:fCurrentTakeIndex];
	[self updateTakesPopupButton];
	[self takeSelected:nil];
}

- (IBAction)record:(id)sender
{
    if ( fMode != kAudioRecording ) {
		fTime = 0;
		[fDurationField setStringValue:@"00:00:00"];

		[fTakesPopupButton removeAllItems];
		[fTakesPopupButton addItemWithTitle:[NSString stringWithFormat:@"Take %d - import", [fTakes count]+1]];
		[fTakesPopupButton selectItemAtIndex:0];
		[fKeepCheckbox setState:NSOnState];
		
		[self setMode:kAudioRecording];

		fCurrentTake = [[MIAudioTake alloc] init];

		fAudioManager->StartRecording();

		
		fTimeTimer = [NSTimer timerWithTimeInterval:1.0 target:self selector:@selector(timeTimer:) userInfo:nil repeats:YES];
		[[NSRunLoop currentRunLoop] addTimer:fTimeTimer forMode:NSModalPanelRunLoopMode];
		
		[fCaptureStopButton setEnabled:YES];
		[fCapturePlayButton setEnabled:NO];
		[fCapturePauseButton setEnabled:NO];
    } else {
        NSBeep();
    }
}

- (IBAction)stopRecording:(id)sender {
	if ( fMode == kAudioRecording ) {
		[fTimeTimer invalidate];
		fTimeTimer = nil;

		[fCurrentTake setAudioTakeIndex:fAudioManager->StopRecording()];

		[fCurrentTake setSelected:YES];
		[fTakes addObject:fCurrentTake];
		[fCurrentTake release];
		fCurrentTake = nil;

		[fCaptureRecordButton setState:NSOffState];

		[self updateTakesPopupButton];
		[fTakesPopupButton selectItemAtIndex:[fTakes count] - 1];
		[self takeSelected:nil];
		[self setMode:kAudioPlayingback];
    }
}


- (IBAction)playTake:(id)sender {
	if ( fMode != kAudioRecording ) {
		if ( fPlayTimer ) [self pauseTake:sender];
		else {
			MIAudioTake *takeObject = [fTakes objectAtIndex:fCurrentTakeIndex];

			[self setMode:kAudioPlayingback];

			SInt32 take = [takeObject audioTakeIndex];
			ArmeTime takePosition = fAudioManager->GetTakeMonitoringPosition(take);
			ArmeTime takeDuration = fAudioManager->GetTakeDuration(take);
			MIMontageDocument* document = [[NSDocumentController sharedDocumentController] currentDocument];
			MITime* time = [MITime timeWithDocument:document];
			/*[time setArmeTime:takeDuration];
			id duration = [time getSMPTEString];
			[time setArmeTime:takePosition];
			id position = [time getSMPTEString];*/
			[time setArmeTime:_ArmeTime(GET_VAL(takeDuration) - GET_VAL(takePosition))];
			//id remaining = [time getSMPTEString];
			//NSLog(@"duration = %@, position = %@, remaining = %@", duration, position, remaining );
			fPlayTimer = [NSTimer timerWithTimeInterval:[time getSeconds] target:self selector:@selector(playTimer:) userInfo:nil repeats:NO];
			[[NSRunLoop currentRunLoop] addTimer:fPlayTimer forMode:NSModalPanelRunLoopMode];

			fAudioManager->StartTakeMonitoring(take);
			[fCapturePlayButton setState:NSOnState];
		}
	}
}

- (void)playTimer:(NSTimer*)timer {
	fPlayTimer = nil;
	[self stopTake:nil];
}

- (IBAction)stopTake:(id)sender {
	if ( fMode == kAudioPlayingback ) {
		if ( fCurrentTakeIndex != -1 ) {
			MIAudioTake *takeObject = [fTakes objectAtIndex:fCurrentTakeIndex];

			if ( fPlayTimer ) {
				[fPlayTimer invalidate];
				fPlayTimer = nil;
			}

			SInt32 take = [takeObject audioTakeIndex];
			fAudioManager->StopTakeMonitoring(take);
			fAudioManager->SeekTakeMonitoringPosition(take, _ArmeTime(0));
			[fCapturePlayButton setState:NSOffState];
		}
	}
}

- (IBAction)pauseTake:(id)sender {
	if ( fMode == kAudioPlayingback ) {
		MIAudioTake *takeObject = [fTakes objectAtIndex:fCurrentTakeIndex];

		if ( fPlayTimer ) {
			[fPlayTimer invalidate];
			fPlayTimer = nil;
		}		

		SInt32 take = [takeObject audioTakeIndex];
		fAudioManager->StopTakeMonitoring(take);
		[fCapturePlayButton setState:NSOffState];
	}
}


- (void)setMode:(MIAudioModes)mode {
	// things to be done even if mode == fMode
	switch (fMode) {
		case kAudioPlayingback:
			if ( mode == kAudioPlayingback ) [self pauseTake:nil];
			break;
		case kAudioRecording:
			break;
		case kAudioGrabbing:
			break;
	}
	
	if ( mode == fMode ) return ;


	switch (fMode) {
		case kAudioGrabbing:
			break;
		case kAudioRecording:
			[fAudioRecParts makeObjectsPerformSelector:@selector(enableJoinAndEnable)];
			break;
		case kAudioPlayingback:
			[self stopTake:nil];
			break;
	}

	fMode = mode;

	switch (fMode) {
		case kAudioGrabbing:
			[fImportButton setEnabled:YES];
			[fCancelButton setEnabled:YES];
			break;
		case kAudioRecording:
			[fCaptureStopButton setAction:@selector(stopRecording:)];
			[fImportButton setEnabled:NO];
			[fCancelButton setEnabled:NO];
			[fAudioRecParts makeObjectsPerformSelector:@selector(disableJoinAndEnable)];
			break;
		case kAudioPlayingback:
			[fCaptureStopButton setAction:@selector(stopTake:)];
			[fImportButton setEnabled:YES];
			[fCancelButton setEnabled:YES];
			break;
	}
}


@end

