/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
#import <Cocoa/Cocoa.h>
#import "MIVUMeterView.h"
#import "MIContinuousSlider.h"

#ifndef MEAudioRecordManager_h
#include "MEAudioRecordManager.h"
#endif //MEAudioRecordManager_h


@interface MIAudioRecPartController : NSObject
{
	@public
	IBOutlet id fAudioRecPartView;
    IBOutlet id fTitleText;
    IBOutlet MIContinuousSlider *fLeftSlider;
    IBOutlet MIVUMeterView *fLeftVUMeter;
    IBOutlet id fLeftOverloadButton;
	IBOutlet id fLeftInputField;
	IBOutlet id fLeftEnableCheckbox;

    IBOutlet MIContinuousSlider *fRightSlider;
    IBOutlet MIVUMeterView *fRightVUMeter;
	IBOutlet id fRightOverloadButton;
	IBOutlet id fRightInputField;
	IBOutlet id fRightEnableCheckbox;

	IBOutlet id fJoinCheckbox;
    IBOutlet id fLockButton;
    IBOutlet id fZeroButton;
    IBOutlet id fOverloadButton;
	
	BOOL	fOverloadButtonEnabled; 
	BOOL	fLeftOverloadButtonIsOn; 
	BOOL	fRightOverloadButtonIsOn; 

	SInt32		fIndex;

	MEAudioRecordManager* fAudioManager;
}

- (id)initWithIndex:(SInt32)index;

- (float)getLeftVUMeterValue:(id)sender;
- (float)getRightVUMeterValue:(id)sender;

- (IBAction)leftSliderMoved:(id)sender;
- (IBAction)rightSliderMoved:(id)sender;

- (IBAction)enableInput:(id)sender;
- (IBAction)joinInputs:(id)sender;

- (IBAction)leftOverloadButtonPressed:(id)sender;
- (IBAction)rightOverloadButtonPressed:(id)sender;

- (IBAction)lockButtonPressed:(id)sender;
- (IBAction)zeroButtonPressed:(id)sender;
- (IBAction)overloadButtonPressed:(id)sender;

- (IBAction)leftInputFieldModified:(id)sender;
- (IBAction)rightInputFieldModified:(id)sender;

- (void)rightSliderChanged;
- (void)leftSliderChanged;

- (void)disableJoinAndEnable;
- (void)enableJoinAndEnable;



@end
