/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
#import "MIAudioRecPartController.h"

#ifndef TArmeDoc_h
#include "TArmeDoc.h"
#endif //TArmeDoc_h
#include <stdlib.h>


@implementation MIAudioRecPartController

- (id)initWithIndex:(SInt32)index {
	if (( self = [super init] )) {
		fIndex = index;
		if ( ![NSBundle loadNibNamed:@"MIAudioRecPart" owner:self] ) {
			[self release];
			return nil;
		}
		fAudioManager = MEAudioRecordManager::SharedAudioRecordManager();
	}
	return self;
}


- (void)dealloc {

	[super dealloc];
}

- (void)awakeFromNib {
}

- (void)prepareControls {
	float dB;

	[fLeftOverloadButton setState:NSOnState];
	fLeftOverloadButtonIsOn = YES; 
	
	[fRightOverloadButton setState:NSOnState];
	fRightOverloadButtonIsOn = YES;
	
	[fOverloadButton setEnabled:NO];
	fOverloadButtonEnabled = NO; 

	// set the lock state
	[fLockButton setState:NSOnState];

	// set left and right fader value and dB
	dB = fAudioManager->GetInputAttenuationDB(fIndex);
	[fLeftSlider setFloatValue:MEMixingInfo::dBToFaderValue(dB)];
	[fLeftInputField setFloatValue:dB];

	dB = fAudioManager->GetInputAttenuationDB(fIndex + 1);
	[fRightSlider setFloatValue:MEMixingInfo::dBToFaderValue(dB)];
	[fRightInputField setFloatValue:dB];

	// Enabled
	if ( fAudioManager->IsInputEnabled(fIndex) ) {
		[fLeftEnableCheckbox setState:NSOnState];
	} else {
		[fLeftEnableCheckbox setState:NSOffState];
	}

	if ( fAudioManager->IsInputEnabled(fIndex + 1) ) {
		[fRightEnableCheckbox setState:NSOnState];
	} else {
		[fRightEnableCheckbox setState:NSOffState];
	}

	if ( fAudioManager->IsInputJoined(fIndex) ) {
		[fJoinCheckbox setState:NSOnState];
	} else {
		[fJoinCheckbox setState:NSOffState];
	}
}

- (IBAction)enableInput:(id)sender {
	if ( [sender state] == NSOnState ) {
		if ( sender == fLeftEnableCheckbox ) {
			fAudioManager->SetInputEnabled(fIndex, true);
			[fLeftSlider setEnabled:YES];
			[fLeftVUMeter setEnabled:YES];
			[fLeftOverloadButton setEnabled:YES];
			[fLeftInputField setEnabled:YES];
		} else {
			fAudioManager->SetInputEnabled(fIndex + 1, true);
			[fRightSlider setEnabled:YES];
			[fRightVUMeter setEnabled:YES];
			[fRightOverloadButton setEnabled:YES];
			[fRightInputField setEnabled:YES];
		}
	} else {
		if ( sender == fLeftEnableCheckbox ) {
			fAudioManager->SetInputEnabled(fIndex, false);
			[fLeftSlider setEnabled:NO];
			[fLeftVUMeter setEnabled:NO];
			[fLeftOverloadButton setEnabled:NO];
			[fLeftInputField setEnabled:NO];
		} else {
			fAudioManager->SetInputEnabled(fIndex + 1, false);
			[fRightSlider setEnabled:NO];
			[fRightVUMeter setEnabled:NO];
			[fRightOverloadButton setEnabled:NO];
			[fRightInputField setEnabled:NO];
		}
	}
}

- (IBAction)joinInputs:(id)sender
{
	if ( [sender state] == NSOnState ) 
	{
		fAudioManager->SetInputJoined(fIndex, true);
	}
	else 
	{
		fAudioManager->SetInputJoined(fIndex, false);
	}
}

- (float)getLeftVUMeterValue:(id)sender 
{
	float value;
	//return ((float)random())/((float)LONG_MAX);
	if ( !fAudioManager->IsInputEnabled(fIndex) ) 
		return 0;

	value = fAudioManager->GetInputDBFSValue(fIndex);

	return (MEMixingInfo::dBToVULevel(value)*0.01);
}

- (float)getRightVUMeterValue:(id)sender 
{
	float value;
	//return ((float)random())/((float)LONG_MAX);
	if ( !fAudioManager->IsInputEnabled(fIndex + 1) ) 
		return 0;

	value = fAudioManager->GetInputDBFSValue(fIndex + 1);

	return (MEMixingInfo::dBToVULevel(value)*0.01);
}


- (IBAction)leftOverloadButtonPressed:(id)sender 
{
	fAudioManager->SetInputNotSaturated(fIndex);
	if ( [fRightOverloadButton state] == NSOnState ) [fOverloadButton setEnabled:NO];
}

- (IBAction)rightOverloadButtonPressed:(id)sender 
{
	fAudioManager->SetInputNotSaturated(fIndex + 1);
	if ( [fLeftOverloadButton state] == NSOnState ) [fOverloadButton setEnabled:NO];
}


- (IBAction)lockButtonPressed:(id)sender 
{
	
}

- (IBAction)zeroButtonPressed:(id)sender 
{
	float sliderVal = MEMixingInfo::dBToFaderValue(0);
	[fRightSlider setFloatValue:sliderVal];
	[self rightSliderMoved:fRightSlider];
	[fLeftSlider setFloatValue:sliderVal];
	[self leftSliderMoved:fLeftSlider];
}

- (IBAction)overloadButtonPressed:(id)sender 
{
	if ( [fLeftOverloadButton state] == NSOffState ) 
	{
		[fLeftOverloadButton setState:NSOnState];
		fLeftOverloadButtonIsOn = true;
		[self leftOverloadButtonPressed:fLeftOverloadButton];
	}
	if ( [fRightOverloadButton state] == NSOffState ) 
	{
		[fRightOverloadButton setState:NSOnState];
		fRightOverloadButtonIsOn = true;
		[self rightOverloadButtonPressed:fRightOverloadButton];
	}
	[fOverloadButton setEnabled:NO];
}

- (IBAction)leftInputFieldModified:(id)sender 
{
	float value = [fLeftInputField floatValue];
	ZzAssert(value >= -192.0 && value <= 6.0);

	float sliderVal = MEMixingInfo::dBToFaderValue(value);

	fAudioManager->SetInputAttenuationDB(fIndex, value);
	[fLeftSlider setFloatValue:sliderVal];

	if ( [fLockButton state] == NSOnState ) 
	{
		fAudioManager->SetInputAttenuationDB(fIndex + 1, value);
		[fRightSlider setFloatValue:sliderVal];
		[fRightInputField setFloatValue:value];
	}
}

- (IBAction)rightInputFieldModified:(id)sender 
{
	float value = [fRightInputField floatValue];
	ZzAssert(value >= -192.0 && value <= 6.0);

	float sliderVal = MEMixingInfo::dBToFaderValue(value);

	fAudioManager->SetInputAttenuationDB(fIndex + 1, value);
	[fRightSlider setFloatValue:sliderVal];
	
	if ( [fLockButton state] == NSOnState ) 
	{
		fAudioManager->SetInputAttenuationDB(fIndex, value);
		[fLeftSlider setFloatValue:sliderVal];
		[fLeftInputField setFloatValue:value];
	}
}



- (IBAction)leftSliderMoved:(id)sender 
{
	// 0 -> 100
	float sliderVal = [sender floatValue];
	ZzAssert(sliderVal >= 0.0 && sliderVal <= 100.0);


	fAudioManager->SetInputAttenuationDB(fIndex, MEMixingInfo::FaderValueTodB(sliderVal));
	[fLeftInputField setFloatValue:MEMixingInfo::FaderValueTodB(sliderVal)];
	
	if ( [fLockButton state] == NSOnState ) 
	{
		fAudioManager->SetInputAttenuationDB(fIndex + 1, MEMixingInfo::FaderValueTodB(sliderVal));
		[fRightSlider setFloatValue:sliderVal];
		[fRightInputField setFloatValue:MEMixingInfo::FaderValueTodB(sliderVal)];
	}
}

- (IBAction)rightSliderMoved:(id)sender 
{
	// 0 -> 100
	float sliderVal = [sender floatValue];
	ZzAssert(sliderVal >= 0.0 && sliderVal <= 100.0);

	fAudioManager->SetInputAttenuationDB(fIndex + 1, MEMixingInfo::FaderValueTodB(sliderVal));
	[fRightInputField setFloatValue:MEMixingInfo::FaderValueTodB(sliderVal)];

	if ( [fLockButton state] == NSOnState )
	{
		fAudioManager->SetInputAttenuationDB(fIndex, MEMixingInfo::FaderValueTodB(sliderVal));
		[fLeftSlider setFloatValue:sliderVal];
		[fLeftInputField setFloatValue:MEMixingInfo::FaderValueTodB(sliderVal)];
	}
	
}

- (void)leftSliderChanged 
{
	/*float value;
	
	if ( fIsRAO ) 
		value = fAudioManager->GetStereoRAOFaderDB(fRAOVAOIndex, true);
	else if ( fStereo ) 
		value = fAudioManager->GetStereoVAOFaderDB(fRAOVAOIndex, true);
	else 
		value = fAudioManager->GetMonoVAOFaderDB(fRAOVAOIndex);
	
	[fLeftSlider setFloatValue:MEMixingInfo::dBToFaderValue(value)];*/
}

- (void)rightSliderChanged 
{
	/*float value;
	
	if ( fIsRAO ) 
		value = fAudioManager->GetStereoRAOFaderDB(fRAOVAOIndex, false);
	else if ( fStereo ) 
		value = fAudioManager->GetStereoVAOFaderDB(fRAOVAOIndex, false);
	else 
		value = fAudioManager->GetMonoVAOFaderDB(fRAOVAOIndex);

	[fRightSlider setFloatValue:MEMixingInfo::dBToFaderValue(value)];*/
}


- (void)updateVUMeters {
	if ( !NSIsEmptyRect([fLeftVUMeter visibleRect]) ) 
	{
		[fLeftVUMeter setLevelAndDisplay:[self getLeftVUMeterValue:fLeftVUMeter]];
	}

	if ( !NSIsEmptyRect([fRightVUMeter visibleRect]) ) 
	{
		[fRightVUMeter setLevelAndDisplay:[self getRightVUMeterValue:fRightVUMeter]];
	}

	if ( fAudioManager->IsInputSaturated(fIndex) ) 
	{
		if(fLeftOverloadButtonIsOn)
		{
			fLeftOverloadButtonIsOn = NO;
			[fLeftOverloadButton setState:NSOffState];
			[fOverloadButton setEnabled:YES];
		}
	}
	else
	{
		if(!fLeftOverloadButtonIsOn)
		{
			fLeftOverloadButtonIsOn = YES;
			[fLeftOverloadButton setState:NSOnState];
			[fOverloadButton setEnabled:YES];
		}
	}
	
	if ( fAudioManager->IsInputSaturated(fIndex + 1) ) 
	{
		if(fRightOverloadButtonIsOn)
		{
			fRightOverloadButtonIsOn = NO;
			[fRightOverloadButton setState:NSOffState];
			[fOverloadButton setEnabled:YES];
		}
	}
	else
	{
		if(!fRightOverloadButtonIsOn)
		{
			fRightOverloadButtonIsOn = YES;
			[fRightOverloadButton setState:NSOnState];
			[fOverloadButton setEnabled:YES];
		}
	}
}

- (void)disableJoinAndEnable {
	[fJoinCheckbox setEnabled:NO];
	[fRightEnableCheckbox setEnabled:NO];
	[fLeftEnableCheckbox setEnabled:NO];
}

- (void)enableJoinAndEnable {
	[fJoinCheckbox setEnabled:YES];
	[fRightEnableCheckbox setEnabled:YES];
	[fLeftEnableCheckbox setEnabled:YES];
}


- (void)disable 
{
    [fLeftSlider setEnabled:NO];
    [fLeftVUMeter setEnabled:NO];
    [fLeftOverloadButton setEnabled:NO];
	[fLeftInputField setEnabled:NO];
	[fLeftEnableCheckbox setEnabled:NO];
    [fRightSlider setEnabled:NO];
    [fRightVUMeter setEnabled:NO];
	[fRightOverloadButton setEnabled:NO];
	[fRightInputField setEnabled:NO];
	[fRightEnableCheckbox setEnabled:NO];
	[fJoinCheckbox setEnabled:NO];
    [fLockButton setEnabled:NO];
	[fOverloadButton setEnabled:NO];
	[fZeroButton setEnabled:NO];
}

- (void)enable {
	[fLeftEnableCheckbox setEnabled:YES];
	if ( [fLeftEnableCheckbox state] == NSOnState ) 
	{
		[fLeftSlider setEnabled:YES];
		[fLeftVUMeter setEnabled:YES];
		[fLeftOverloadButton setEnabled:YES];
		[fLeftInputField setEnabled:YES];
	}

	[fRightEnableCheckbox setEnabled:YES];

	if ( [fRightEnableCheckbox state] == NSOnState ) 
	{
		[fRightSlider setEnabled:YES];
		[fRightVUMeter setEnabled:YES];
		[fRightOverloadButton setEnabled:YES];
		[fRightInputField setEnabled:YES];
	}

	[fJoinCheckbox setEnabled:YES];
    [fLockButton setEnabled:YES];
	[fZeroButton setEnabled:YES];
	if ( ( [fLeftOverloadButton state] == NSOffState ) || ( [fRightOverloadButton state] == NSOffState ) )
		[fOverloadButton setEnabled:YES];
	
}
@end
