/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
#import "MIBackupPasteboard.h"


@implementation MIBackupPasteboard

+ (MIBackupPasteboard*) backupPasteboard
{
    return [[self alloc] init];
}

- (MIBackupPasteboard*) init
{
    if ((self = [super init]))
		{
		// Retrieve the general pasteboard.
		fGeneralPasteboard = [NSPasteboard generalPasteboard];
		// Create a backup pasteboard.
		fBackupPasteboard = [NSPasteboard pasteboardWithUniqueName];
		}
    return self;
}

- (void) dealloc
{
	[fGeneralPasteboardTypesArray release];
	[fBackupPasteboard release];
    [super dealloc];
}

- (void) backup
{
	// Retrieve the types available in the general pasteboard data.
	fGeneralPasteboardTypesArray = [fGeneralPasteboard types];
	[fGeneralPasteboardTypesArray retain];
	unsigned int	generalPasteboardTypesCount = [fGeneralPasteboardTypesArray count];
	[fBackupPasteboard declareTypes:fGeneralPasteboardTypesArray owner:nil];
	for (unsigned int i = 0 ; i < generalPasteboardTypesCount ; i++)
		{
		id	arrayType = [fGeneralPasteboardTypesArray objectAtIndex:i];
		NSData *	backupData = [fGeneralPasteboard dataForType:arrayType];
		if (backupData != nil)
			// Set the backup pasteboard data to the current general pasteboard data.
			[fBackupPasteboard setData:backupData forType:arrayType];
		}
}

- (void) retrieve
{
	NSArray *		backupPasteboardTypesArray = [fBackupPasteboard types];
	unsigned int	backupPasteboardTypesCount = [backupPasteboardTypesArray count];
	[fGeneralPasteboard declareTypes:fGeneralPasteboardTypesArray owner:nil];
	for (unsigned int i = 0 ; i < backupPasteboardTypesCount ; i++)
		{
		id	arrayType = [backupPasteboardTypesArray objectAtIndex:i];
		NSData *	backupData = [fBackupPasteboard dataForType:arrayType];
		if (backupData != nil)
			// Set the general pasteboard data to the backup one.
			[fGeneralPasteboard setData:backupData forType:arrayType];
		}
}

- (BOOL)setData:(NSData *)data forType:(NSString *)dataType
{
	[fGeneralPasteboard declareTypes:[NSArray arrayWithObject:dataType] owner:nil];
	return [fGeneralPasteboard setData:data forType:dataType];
}

- (BOOL)setString:(NSString *)string forType:(NSString *)dataType
{
	[fGeneralPasteboard declareTypes:[NSArray arrayWithObject:dataType] owner:nil];
	return [fGeneralPasteboard setString:string forType:dataType];
}

@end