/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
#import "MIBlueWindowParamController.h"
#import "MIBlueWindowController.h"

@implementation MIBlueWindowParamController

- (id)init
{
	self = [super init];
	fLinkType = MIBlueWindowLinkNone;
	fController = nil;
	fParamNumber = -1;
	
	fLinkButtonImages[MIBlueWindowLinkNone] = nil;
	
	fLinkButtonImages[MIBlueWindowLinkUpDown] = [[NSImage alloc] initWithSize:NSMakeSize(16,16)];
	fLinkButtonImages[MIBlueWindowLinkLeftRight] = [[NSImage alloc] initWithSize:NSMakeSize(16,16)];
	fLinkButtonImages[MIBlueWindowLinkDownUp] = [[NSImage alloc] initWithSize:NSMakeSize(16,16)];
	fLinkButtonImages[MIBlueWindowLinkRightLeft] = [[NSImage alloc] initWithSize:NSMakeSize(16,16)];
	
	[[NSColor blackColor] set];
	
	[fLinkButtonImages[MIBlueWindowLinkUpDown] lockFocus];
		[NSBezierPath strokeLineFromPoint:NSMakePoint(7.5, 0.5) toPoint:NSMakePoint(7.5, 15.5)];
		[NSBezierPath strokeLineFromPoint:NSMakePoint(7.5, 0.5) toPoint:NSMakePoint(3.5, 4.5)];
		[NSBezierPath strokeLineFromPoint:NSMakePoint(7.5, 0.5) toPoint:NSMakePoint(11.5, 4.5)];
	[fLinkButtonImages[MIBlueWindowLinkUpDown] unlockFocus];
	
	[fLinkButtonImages[MIBlueWindowLinkLeftRight] lockFocus];
		[NSBezierPath strokeLineFromPoint:NSMakePoint(0.5, 7.5) toPoint:NSMakePoint(15.5, 7.5)];
		[NSBezierPath strokeLineFromPoint:NSMakePoint(15.5, 7.5) toPoint:NSMakePoint(11.5, 3.5)];
		[NSBezierPath strokeLineFromPoint:NSMakePoint(15.5, 7.5) toPoint:NSMakePoint(11.5, 11.5)];
	[fLinkButtonImages[MIBlueWindowLinkLeftRight] unlockFocus];
	
	[fLinkButtonImages[MIBlueWindowLinkDownUp] lockFocus];
		[NSBezierPath strokeLineFromPoint:NSMakePoint(7.5, 0.5) toPoint:NSMakePoint(7.5, 15.5)];	
		[NSBezierPath strokeLineFromPoint:NSMakePoint(7.5, 15.5) toPoint:NSMakePoint(3.5, 11.5)];
		[NSBezierPath strokeLineFromPoint:NSMakePoint(7.5, 15.5) toPoint:NSMakePoint(11.5, 11.5)];
	[fLinkButtonImages[MIBlueWindowLinkDownUp] unlockFocus];

	[fLinkButtonImages[MIBlueWindowLinkRightLeft] lockFocus];
		[NSBezierPath strokeLineFromPoint:NSMakePoint(0.5, 7.5) toPoint:NSMakePoint(15.5, 7.5)];
		[NSBezierPath strokeLineFromPoint:NSMakePoint(0.5, 7.5) toPoint:NSMakePoint(4.5, 11.5)];
		[NSBezierPath strokeLineFromPoint:NSMakePoint(0.5, 7.5) toPoint:NSMakePoint(4.5, 3.5)];
	[fLinkButtonImages[MIBlueWindowLinkRightLeft] unlockFocus];
	
	return self;
}

- (void)setBlueWindowController:(MIBlueWindowController*)blueWindowController
{
	fController = blueWindowController;
}

- (void)setParameterNumber:(int)number
{
	fParamNumber = number;
}

- (void)initialize
{
	[fMinParamValue setFloatValue:[fController getMinValueForParam:fParamNumber]];
	[fMaxParamValue setFloatValue:[fController getMaxValueForParam:fParamNumber]];
	[fParamCurrentValue setFloatValue:[fController getValueForParam:fParamNumber]];
	[fParamSlider setMinValue:[fMinParamValue floatValue]];
	[fParamSlider setMaxValue:[fMaxParamValue floatValue]];
	[fParamSlider setFloatValue:[fParamCurrentValue floatValue]];
	[fParamTitle setStringValue:[fController getNameForParam:fParamNumber]];
}

- (void)syncValuesFromPlugin
{
	[fParamCurrentValue setFloatValue:[fController getValueForParam:fParamNumber]];
	[fParamSlider setFloatValue:[fParamCurrentValue floatValue]];
}

- (void)sendValueToPlugin:(float)value
{
	[fController setValue:value forParam:fParamNumber];
}

- (void)syncPluginFromValue
{
	[fController setValue:[fParamCurrentValue floatValue] forParam:fParamNumber];
}

- (float)getValue
{
	return [fParamCurrentValue floatValue];
}

- (void)reflectBlueWindowVPos:(float)vPos
{
	float range;
	float newValue;
	
	range = [fMaxParamValue floatValue] - [fMinParamValue floatValue];
	if ( fLinkType == MIBlueWindowLinkUpDown )
	{
		newValue = [fMaxParamValue floatValue] - (range * vPos);
		[self sendValueToPlugin:newValue];
		[self syncValuesFromPlugin];
	}
	else if ( fLinkType == MIBlueWindowLinkDownUp )
	{
		newValue = [fMinParamValue floatValue] + (range * vPos);
		[self sendValueToPlugin:newValue];
		[self syncValuesFromPlugin];
	}
}

- (void)reflectBlueWindowHPos:(float)hPos
{
	float range;
	float newValue;
	
	range = [fMaxParamValue floatValue] - [fMinParamValue floatValue];
	if ( fLinkType == MIBlueWindowLinkRightLeft )
	{
		newValue = [fMaxParamValue floatValue] - (range * hPos);
		[self sendValueToPlugin:newValue];
		[self syncValuesFromPlugin];
	}
	else if ( fLinkType == MIBlueWindowLinkLeftRight )
	{
		newValue = [fMinParamValue floatValue] + (range * hPos);
		[self sendValueToPlugin:newValue];
		[self syncValuesFromPlugin];
	}
}

- (IBAction)currentValueChanged:(id)sender
{
	float value;
	
	value = [sender floatValue];
	
	if ( value > [fMaxParamValue floatValue] )
		value = [fMaxParamValue floatValue];

	if ( value < [fMinParamValue floatValue] )
		value = [fMinParamValue floatValue];
		
	[self sendValueToPlugin:[sender floatValue]];
	[self syncValuesFromPlugin];
}

- (IBAction)paramSliderMoved:(id)sender
{
	[self sendValueToPlugin:[sender floatValue]];
	[self syncValuesFromPlugin];
}

- (IBAction)paramButtonClicked:(id)sender
{
	fLinkType = fLinkType + 1;
	if ( fLinkType == MIMaxBlueWindowLinkTypes )
		fLinkType = MIBlueWindowLinkNone;
	else
		
	[sender setImage:fLinkButtonImages[fLinkType]];
}


@end
