/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MICamera.h"
#include <unistd.h>

@implementation MICamera

+ (BOOL)isCameraConnected
{
	ComponentInstance camera = ::OpenDefaultComponent(kIDHComponentType, kIDHSubtypeDV);

	QTAtomContainer deviceList = nil;
	::IDHGetDeviceList( camera, &deviceList);
	int numberDVDevices = ::QTCountChildrenOfType(deviceList, kParentAtomIsContainer, kIDHDeviceAtomType);

	::CloseComponent(camera);

	return (numberDVDevices >= 1);
}

- (id)init //WithNotification:(BOOL)withNotification
{
	OSErr err = noErr;
//	ComponentResult result;
	
    // try to open a camera component
	fDVCamera = ::OpenDefaultComponent(kIDHComponentType, kIDHSubtypeDV);
	if (!fDVCamera) 
		return nil;

/*	if (withNotification)
	{
		result = ::IDHNewNotification( fDVCamera, kIDHDeviceIDEveryDevice, SGNotificationProc, self, &fNotificationID);
		result = ::IDHNotifyMeWhen( fDVCamera, fNotificationID, kIDHEventDeviceAdded | kIDHEventDeviceRemoved );	
	}*/

	QTAtomContainer deviceList = nil;
	::IDHGetDeviceList( fDVCamera, &deviceList);
	int numberDVDevices = ::QTCountChildrenOfType(deviceList, kParentAtomIsContainer, kIDHDeviceAtomType);
	if (numberDVDevices < 1)
		return nil;

	// get the atom to first device
	QTAtom deviceAtom = ::QTFindChildByIndex( deviceList, kParentAtomIsContainer, kIDHDeviceAtomType, 1, nil);
	// find the isoch characteristics for this device
	QTAtom isochAtom = ::QTFindChildByIndex( deviceList, deviceAtom, kIDHIsochServiceAtomType, 1, nil);
	// how many configs exist for this device -- we'll just assume at least 1 and first is default
	// int nConfigs = ::QTCountChildrenOfType( deviceList, isochAtom, kIDHIsochModeAtomType);
	// get first configs atom
	QTAtom configAtom = ::QTFindChildByIndex( deviceList, isochAtom, kIDHIsochModeAtomType, 1, nil);
	// find the media type atom
	QTAtom mediaAtom = ::QTFindChildByIndex( deviceList, configAtom, kIDHIsochMediaType, 1, nil);
	::QTLockContainer( deviceList);
	// get the value of the mediaType atom
	OSType mediaType;
	long size = sizeof(mediaType);
	::QTCopyAtomDataToPtr( deviceList, mediaAtom, true, sizeof(mediaType), &mediaType, &size);
	::QTUnlockContainer( deviceList);
	// is this config a video config?
	if( mediaType == kIDHVideoMediaAtomType) // found video device
	{
		fCameraConfig.container = deviceList; // save this config
		fCameraConfig.atom = configAtom;
	}
	::IDHSetDeviceConfiguration( fDVCamera, &fCameraConfig);
	err = ::IDHOpenDevice( fDVCamera, kIDHOpenForReadTransactions);

	return self;	
}

- (void)close
{
//	::IDHCancelNotification( fDVCamera, fNotificationID);
	::IDHCloseDevice(fDVCamera);
	::CloseComponent(fDVCamera);
	fDVCamera = nil;
}

- (int)deviceControl:(int)command;
{
	ComponentInstance controlInst;
	ComponentResult result;
	DVCTransactionParams pParams;
	int o_in[2];
	char o_out[16];
	int i;
	int reply;
	char *cReply = (char*)&reply;
	int inSize, outSize;
	int timeout = 5;

	if ( !fDVCamera ) NSLog(@"fDVCamera = nil");
	
	result = ::IDHGetDeviceControl(fDVCamera, &controlInst);
	if ( result ) {
		NSLog(@"error %u in IDHGetDeviceControl()", result ) ;
		return -1;
	}

	o_in[0] = command;
	o_in[1] = 0xffffffff;

	inSize = sizeof(int);
	if ( command == deviceTime ) inSize *= 2;

	outSize = sizeof(o_out);
	
	// fill up the transaction parameter block
	pParams.commandBufferPtr = (char*)o_in;
	pParams.commandLength = inSize;
	pParams.responseBufferPtr = o_out;
	pParams.responseBufferSize = outSize;
	pParams.responseHandler = nil;
	
	do
	{
		for (i=0; i < outSize; i++) o_out[i] = 0;
		reply = 0;
		result = ::DeviceControlDoAVCTransaction( controlInst, &pParams);
		timeout--;

		if ( command == deviceQuery ) {
		cReply[0] = 0x00;
		cReply[1] = 0x20;
		cReply[2] = o_out[2];
		cReply[3] = o_out[3];
		} else if ( command == deviceTime ) {
			cReply[0] = o_out[4];
			cReply[1] = o_out[5];
			cReply[2] = o_out[6];
			cReply[3] = o_out[7];
		}
		
		if (result == kIOReturnOffline)
		{
			sleep(1);
			continue;
		}
		if(result)
			break;
	}
	while ( (result != kIOReturnSuccess) && ( timeout > 0 ) ) ;
	return reply;
}

/*- (IDHNotificationID)notificationID
{
	return fNotificationID;
}*/

- (ComponentInstance)cameraComponent
{
	return fDVCamera;
}
@end
