/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
#import "MIChangeVAOCommandDlgController.h"

#ifndef TArmeDoc_h
#include "TArmeDoc.h"
#endif //TArmeDoc_h

#ifndef MEMixingInfo_h
#include "MEMixingInfo.h"
#endif //MEMixingInfo_h

#ifndef ZzVerbose_h
//#include "ZzVerbose.h"
#endif //ZzVerbose_h

// as checked in on 5/31 these aren't declared anywhere my compiler can find ...alex
extern int min(int a, int b);
int max(int a, int b) {int r = (a>b) ? (a) : (b); return r;}


@implementation MIChangeVAOCommandDlgController

- (IBAction)cancelClicked:(id)sender
{
	fVAORouting.index = -2; // means cancel clicked
	[[NSApplication sharedApplication] stopModal];
}

- (IBAction)okClicked:(id)sender
{
	fVAORouting.index = [fPopUpButton indexOfSelectedItem]/2;
	fVAORouting.bStereo = (([fPopUpButton indexOfSelectedItem] % 2) == 1);
	[[NSApplication sharedApplication] stopModal];
}

- (void)populatePopUpButton:(TArmeDoc*)doc
{
	// we need to remove the NULL item (we can't build
	// an empty popup menu in IB)
	[fPopUpButton removeAllItems];
	SInt32 maxMonoVAOUsed = doc->GetMixingInfo()->GetMaxMixerDisplayMonoVAOCount();
	SInt32 maxStereoVAOUsed = doc->GetMixingInfo()->GetMaxMixerDisplayStereoVAOCount();
	SInt32 firstImpossibleIndex = min(kMEMixingInfo_NbMaxMonoVAO,kMEMixingInfo_NbMaxStereoVAO);
	SInt32 lastIndex = max(max(maxMonoVAOUsed,maxStereoVAOUsed),firstImpossibleIndex-8);
	
	for( SInt32 vaoIndex = 0; vaoIndex < lastIndex; vaoIndex++)
	{
		NSString* string1 = [NSString stringWithFormat:@"Mono %d",vaoIndex + 1];
		NSString* string2 = [NSString stringWithFormat:@"Stereo Pair %d",vaoIndex + 1];
		[fPopUpButton addItemWithTitle:string1];
		[fPopUpButton addItemWithTitle:string2];
	}
}

- (void)awakeFromNib
{
	fVAORouting.index = -3;
}

- (MEVAORouting)getVAORouting
{
	ZzAssert(fVAORouting.index > -3);
	return fVAORouting;
}

@end
