/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MIClipRep.h"
#import "MISliceSelectionTracker.h"
#import "MIImageController.h"
#import "MITopLevelContainerRep.h"
#import "MIMainViewController.h"
#import "MIWSSelection.h"

#ifndef Zz_GlobalDefines_h
#include "Zz_GlobalDefines.h"
#endif //Zz_GlobalDefines_h

#import "MITextClipRep.h"

@implementation MIClipRep

extern bool gEnableCursorRectsUpdate;

- (void)addCursorRects
{
	[super addCursorRects];
#ifdef TEST_BENJA_ALLOW_CURSOR_RECTS_DISABLING
	if(gEnableCursorRectsUpdate && ![self isKindOfClass:[MITextClipRep class]] )
	{
#endif //TEST_BENJA_ALLOW_CURSOR_RECTS_DISABLING
		NSRect center, left, right;
		NSRect tmpRect;
		
		left = [self rectForZone:MIObjectRepLeftBodySelectionZone withObjectOrigin:[self getTLCFrame].origin];
		right = [self rectForZone:MIObjectRepRightBodySelectionZone withObjectOrigin:[self getTLCFrame].origin];
		center = [self rectForZone:MIObjectRepBodySelectionZone withObjectOrigin:[self getTLCFrame].origin];
		
		[fTLCRep addCursorRect:center cursor:mainImageController->fCarretCursor];
		[fTLCRep addCursorRect:right cursor:mainImageController->fRightCarretCursor];
		[fTLCRep addCursorRect:left cursor:mainImageController->fLeftCarretCursor];
		
		MIWSSelection* sel = [[[self getTopLevelContainerRep] mainViewController] selection];
		if(([sel type] == kMIWSSelectionTypeTimeSlice) && [sel containsObject:self])
		{
			tmpRect.size.height = NSHeight(center);
			tmpRect.size.width = 0;
			tmpRect.origin.y = NSMinY(center);
			tmpRect.origin.x = [fTLCRep armeTimeToPixel:[sel startTime]];
			tmpRect = NSInsetRect( tmpRect, -5, 0);
			
			[fTLCRep addCursorRect:tmpRect cursor:mainImageController->fLeftRightCarretCursor];
				
			tmpRect.size.width = 0;
			tmpRect.origin.x = [fTLCRep armeTimeToPixel:[sel endxTime]];
			tmpRect = NSInsetRect( tmpRect, -5, 0);

			[fTLCRep addCursorRect:tmpRect cursor:mainImageController->fLeftRightCarretCursor];
		}
#ifdef TEST_BENJA_ALLOW_CURSOR_RECTS_DISABLING
	}
#endif //TEST_BENJA_ALLOW_CURSOR_RECTS_DISABLING
}
/*
- (void)removeCursorRects
{
	[super removeCursorRects];
#ifdef TEST_BENJA_ALLOW_CURSOR_RECTS_DISABLING
	if(gEnableCursorRectsUpdate)
	{
#endif //TEST_BENJA_ALLOW_CURSOR_RECTS_DISABLING
		[fTLCRep removeCursorRect:[self rectForZone:MIObjectRepBodySelectionZone withObjectOrigin:[self getTLCFrame].origin] cursor:mainImageController->fCarretCursor];
		[fTLCRep removeCursorRect:[self rectForZone:MIObjectRepRightBodySelectionZone withObjectOrigin:[self getTLCFrame].origin] cursor:mainImageController->fRightCarretCursor];
		[fTLCRep removeCursorRect:[self rectForZone:MIObjectRepLeftBodySelectionZone withObjectOrigin:[self getTLCFrame].origin] cursor:mainImageController->fLeftCarretCursor];
#ifdef TEST_BENJA_ALLOW_CURSOR_RECTS_DISABLING
	}
#endif //TEST_BENJA_ALLOW_CURSOR_RECTS_DISABLING
}
*/
- (MIObjectMouseHandling)handleMoreMouseTracking:(NSPoint)mouseLoc modifier:(unsigned int)modifier type:(NSEventType)type
{
	MIObjectMouseHandling answer;
	
	// our frame
	NSRect myFrame = [self frame];
	myFrame.origin = NSZeroPoint;

	answer = [super handleMoreMouseTracking:mouseLoc modifier:modifier type:type];
	// if the parent does NOT handle it, and if we are in the body, let's create a selection tracker
	if( ![self isKindOfClass:[MITextClipRep class]] && answer == MIContinueMouseHandling && mouseLoc.y < (NSMaxY(myFrame) - [fCenterTitleImage size].height) )
	{
		NSPoint		selStartPos = [self convertPointToTLCCoords:mouseLoc];
		// check if a slice selection already exists for this object.
		
		if ( [[[fTLCRep mainViewController] selection] type] == kMIWSSelectionTypeTimeSlice &&
			 [[[fTLCRep mainViewController] selection] containsObject:self] )
		{
			float	st, en;
			st = [fTLCRep armeTimeToPixel:[[[fTLCRep mainViewController] selection] startTime]];
			en = [fTLCRep armeTimeToPixel:[[[fTLCRep mainViewController] selection] endxTime]];
			
			if ( fabs(st- selStartPos.x) <= 5 )
				selStartPos.x = en;
			else if ( fabs(en- selStartPos.x) <= 5 )
				selStartPos.x = st;
		}
	
		fTemporaryTracker = [[MISliceSelectionTracker alloc] initWithObjectRep:self inView:fTLCRep 
		atPos:selStartPos withModifiers:nil ];
		
		return MIAskMeATracker;
	}
	else
	{
		return answer;
	}
}
@end
