/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import <AppKit/AppKit.h>
#import "MISequenceGrabberView.h"
#import "TVideoClip.h"
#import "TArmeDoc.h"
#import "MIMontageDocument.h"
#import "ArmeDataFileRef.h"
#import "MIObject.h"
#import "MovieHelper.h"
#import "MIMainViewController.h"
#import "MITopLevelContainerRep.h"
#import "MIPlayStopPauseController.h"
#import "MICamera.h"

/*// DV device control stuff
#include <DVComponentGlue/IsochronousDataHandler.h>
#include <DVComponentGlue/DeviceControl.h>
#include <iokit/IOKitLib.h>
// from ISOCHRONOUS COMPONENTS REFERENCE v1.0 3/16/2001
//#define kIDHComponentType 	'ihlr'		Component type
//#define	kIDHSubtypeDV		'dv  '		Subtype for DV (over FireWire)
*/
#ifndef MEAudioRecordManager_h
#include "MEAudioRecordManager.h"
#endif //MEAudioRecordManager_h

enum MIDVModes
{
	kDVGrabbing,
	kDVRecording,
	kDVPlayingback,
	kDVIdle
};

@interface MIDVTake : NSObject {
	ArmeDataFileRef*	fMovieFileRef;
	BOOL				fSelected;
}

- (void)setMovieFileRef:(ArmeDataFileRef*)movieFileRef;
- (ArmeDataFileRef*)movieFileRef;

- (void)setSelected:(BOOL)flag;
- (BOOL)isSelected;
	
@end

@interface MIDVCaptureController : NSWindowController 
{
@public
	IBOutlet id 		fVideoSettingsButton;
	IBOutlet id 		fVideoPlayButton;
	IBOutlet id 		fVideoStopButton;
	IBOutlet id 		fVideoPauseButton;
	IBOutlet id 		fVideoFFButton;
	IBOutlet id 		fVideoRewButton;
	IBOutlet id 		fCapturePlayButton;
	IBOutlet id 		fCaptureStopButton;
	IBOutlet id 		fCapturePauseButton;
	IBOutlet id 		fCaptureRecordButton;
	
	IBOutlet id 		fTakesPopupButton;
	IBOutlet id 		fDiscardButton;
	IBOutlet id 		fKeepCheckbox;
	IBOutlet id 		fImportButton;
	IBOutlet id 		fCancelButton;
	
	IBOutlet id 		fSequenceGrabberView;

	IBOutlet id 		fDeviceTextField;

	long long			fDiskFreeBytes;

    SeqGrabComponent	fSequenceGrabber;
    SGChannel 			fSGVideoChannel;
    SGChannel 			fSGAudioChannel;
    UserData 			fSGSettings;
    Rect				fNativeSize;

	BOOL				fCameraConnected; 
/*	ComponentInstance	fDVCamera;
	QTAtomSpec			fCameraConfig;*/
	MICamera*			fCamera;
	BOOL				fReset;
    FSSpec 				fCaptureFile;
	ArmeDataFileRef* 	fMovieFileRef;
	IDHNotificationID	fNotificationID;

	NSMutableArray		*fTakes;
	MIDVTake			*fCurrentTake;
	IBOutlet id			fPlaybackView;
	IBOutlet id			fMovieContainer;
	IBOutlet id			fDurationField;
	IBOutlet id			fDiskFreeSpace;
	BOOL				fSGViewInstalled;
	MIDVModes			fMode;
	NSTimer				*fTimeTimer;
	NSTimer				*fCameraTimer;
	NSTimer				*fIdleSGTimer;
	NSTimer				*fNotificationTimer;
    NSTimer				*fFreeSpaceTimer;
	NSTimer				*fPlayTimer;
	
	long				fTime;
	int					fCameraState;
	SInt32				fCurrentTakeIndex;
	unsigned int		fVideoTakesLength;
}

+ (id)sharedDVCaptureController;
- (id)init;
- (void)dealloc;

- (IBAction)play:(id)sender ;
- (IBAction)stop:(id)sender;
- (IBAction)pause:(id)sender;
- (IBAction)rewind:(id)sender;
- (IBAction)fastForward:(id)sender;
	
- (IBAction)record:(id)sender;
- (IBAction)stopRecording:(id)sender;
- (IBAction)videoSettings:(id)sender;

- (IBAction)show:(id)sender;

- (IBAction)record:(id)sender;

- (IBAction)keepTake:(id)sender;
- (IBAction)keepAllTakes:(id)sender;
- (IBAction)keepNoTakes:(id)sender;
- (IBAction)discardTake:(id)sender;

- (IBAction)importSelectedTakes:(id)sender;

- (IBAction)playTake:(id)sender;
- (IBAction)stopTake:(id)sender;
- (IBAction)pauseTake:(id)sender;

- (IBAction)takeSelected:(id)sender;

- (void)openSequenceGrabberComponent;
- (void)closeSequenceGrabberComponent;
- (void)openChannels;
- (void)openCameraComponent;
- (void)closeCameraComponent;
//- (void)checkDevices;
- (void)forceDeviceReset;
- (void)closeAll;
- (void)openAll;
- (void)updateDeviceMenu;
- (IBAction)deviceFromMenu:(id)sender;
- (void)startIdling;
- (void)stopIdling;
- (void)stopRecording;
- (void)startRecording;
- (BOOL)isDeviceControllable;
- (int)deviceControl:(int)command;
- (void)updateTakesPopupButton;
- (void)setMode:(MIDVModes)mode;
- (void)freeSpaceTimer:(NSTimer*)theTimer;

- (void)updateCameraState;
- (void)updateVideoState;

- (void)setIsCameraConnected:(BOOL)flag;
- (BOOL)isCameraConnected;

- (IBAction)importAndClose:(id)sender;
- (IBAction)cancelAndClose:(id)sender;
	
@end