/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import <Cocoa/Cocoa.h>

#define _setjmp setjmp
#import <AppKit/AppKit.h>
#import "MIDVCaptureController.h"
#import "MIImageController.h"
#import "MIMacroCommand.h"
#import "MICreateWSObjsFromFileCommand.h"
#import "MIMontageDocument.h"
#import "MIApplication.h"
#include <math.h>

#if !defined(__MWERKS__)
#import <setjmp.h>
#include <sys/param.h>
#include <sys/mount.h>
#endif //!defined(__MWERKS__)

#ifndef MovieHelper_h
#include "MovieHelper.h"
#endif //MovieHelper_h

#ifndef MEAudioDeviceDesc_h
#include "MEAudioDeviceDesc.h"
#endif //MEAudioDeviceDesc_h

#import "MIPreferences.h"
#import "MIApplicationController.h"





pascal OSErr ReceiveSGDataProc(SGChannel channel, Ptr data, long len, long *offset, long chRefCon, TimeValue time, short writeType, long refCon) {
    MISequenceGrabberView *previewer = (MISequenceGrabberView*)refCon;
    if (!previewer) return noErr;
    return [previewer receiveSGData:channel data:data length:len];
}

OSStatus SGNotificationProc(IDHGenericEvent *event, void *userData) {
	MIDVCaptureController *controller = (MIDVCaptureController *)userData;
	controller->fReset = YES;
	return noErr;
}



@implementation MIDVTake

- (id)init
{
	if (( self = [super init] ))
	{
		fMovieFileRef = nil;
		fSelected = NO;
	}
	return self;
}

- (void)dealloc 
{
	if ( fMovieFileRef ) fMovieFileRef->Release();
	[super dealloc];
}

- (void)setMovieFileRef:(ArmeDataFileRef*)movieFileRef {
	fMovieFileRef = movieFileRef;
	movieFileRef->AddRef();
}

- (ArmeDataFileRef*)movieFileRef {
	return fMovieFileRef;
}


- (void)setSelected:(BOOL)flag {
	fSelected = flag;
}

- (BOOL)isSelected {
	return fSelected;
}

@end


@implementation MIDVCaptureController

+ (id)sharedDVCaptureController {
    static id sharedAVCaptureController = nil;
    if ( sharedAVCaptureController == nil ) {
        sharedAVCaptureController = [[self alloc] init];
    }
    return sharedAVCaptureController;
}

- (id)init {
	if (( self = [super initWithWindowNibName:@"DVCapture"] )) {
		fTakes = [[NSMutableArray alloc] initWithObjects:nil];
	
		[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(applicationWillTerminate:) name:NSApplicationWillTerminateNotification object:nil];

		fPlaybackView = [[NSMovieView alloc] initWithFrame:NSMakeRect(0, 0, 320, 240)];
		[fPlaybackView setLoopMode:NSQTMovieNormalPlayback];
		[fPlaybackView showController:NO adjustingSize:NO];
		fSGViewInstalled = YES;
		fMode = kDVIdle;
    }
    return self;
}

- (void)dealloc {
	[[NSNotificationCenter defaultCenter] removeObserver:self name:NSApplicationWillTerminateNotification object:nil];
	[self closeAll];
	if (fSGSettings)
	{
		::DisposeUserData(fSGSettings);
	}
	[fTakes release];
	[fPlaybackView release];
	[super dealloc];
}

- (void)applicationWillTerminate:(NSNotification*)theNotification {
    [self release];
}

- (void)awakeFromNib {
	[fVideoPlayButton setImage:mainImageController->fPlayBarButtonPlayOff];
    [fVideoPlayButton setAlternateImage:mainImageController->fPlayBarButtonPlayOn];
	[fCapturePlayButton setImage:mainImageController->fPlayBarButtonPlayOff];
    [fCapturePlayButton setAlternateImage:mainImageController->fPlayBarButtonPlayOn];

	[fVideoStopButton setImage:mainImageController->fPlayBarButtonStopOff];
    [fVideoStopButton setAlternateImage:mainImageController->fPlayBarButtonStopOn];
    [fCaptureStopButton setImage:mainImageController->fPlayBarButtonStopOff];
    [fCaptureStopButton setAlternateImage:mainImageController->fPlayBarButtonStopOn];

    [fVideoPauseButton setImage:mainImageController->fPlayBarButtonPauseOff];
    [fVideoPauseButton setAlternateImage:mainImageController->fPlayBarButtonPauseOn];
    [fCapturePauseButton setImage:mainImageController->fPlayBarButtonPauseOff];
    [fCapturePauseButton setAlternateImage:mainImageController->fPlayBarButtonPauseOn];
	
    [fCaptureRecordButton setImage:mainImageController->fPlayBarButtonRecordOff];
    [fCaptureRecordButton setAlternateImage:mainImageController->fPlayBarButtonRecordOn];

	[fVideoFFButton setImage:mainImageController->fPlayBarButtonforwardOff];
    [fVideoFFButton setAlternateImage:mainImageController->fPlayBarButtonforwardOn];

    [fVideoRewButton setImage:mainImageController->fPlayBarButtonBackOff];
    [fVideoRewButton setAlternateImage:mainImageController->fPlayBarButtonBackOn];
}


- (void)openSequenceGrabberComponent
{
    // try to open a capture component
    ComponentResult result = noErr;

    // get the capture source component
    ComponentDescription theDesc = { 0 };
    theDesc.componentType = SeqGrabComponentType;
    theDesc.componentSubType = 0;
    theDesc.componentManufacturer = 0;
    theDesc.componentFlags = 0;
    theDesc.componentFlagsMask = 0;
    Component sgCompID = ::FindNextComponent( sgCompID, &theDesc );

NS_DURING
    if ( sgCompID != nil ) 
	{
		fSequenceGrabber = ::OpenComponent( sgCompID );
	}
    if ( fSequenceGrabber == nil )
	{
		[NSException raise:@"MTGGenericException" format:@"MTG Generic Exception"];
	}
	
    result = ::SGInitialize( fSequenceGrabber );
	if ( result != noErr )
	{
		[NSException raise:@"MTGGenericException" format:@"MTG Generic Exception"];
	}
		
    if (fSGSettings) 
	{
		::DisposeUserData(fSGSettings);
	}
	
    result = ::SGGetSettings(fSequenceGrabber, &fSGSettings, 0);
	if ( result != noErr )
	{
		[NSException raise:@"MTGGenericException" format:@"MTG Generic Exception"];
	}
	
    result = ::SGSetDataRef(fSequenceGrabber, 0, 0, seqGrabDontMakeMovie);
	if ( result != noErr ) 
	{
		[NSException raise:@"MTGGenericException" format:@"MTG Generic Exception"];
	}

NS_HANDLER
	if (fSGSettings) 
	{
		::DisposeUserData(fSGSettings);
		fSGSettings = nil;
	}
	
	if ( fSequenceGrabber ) 
	{
		::CloseComponent(fSequenceGrabber);
		fSequenceGrabber = nil;
	}
NS_ENDHANDLER
}

- (void)closeSequenceGrabberComponent
{
	if (fSequenceGrabber) 
	{
		::SGStop(fSequenceGrabber);
		::CloseComponent(fSequenceGrabber);
		fSequenceGrabber = nil;
		fSGVideoChannel = nil;
		fSGAudioChannel = nil;
	}
}

- (void)openChannels
{
	if ( !fSequenceGrabber ) 
		return;

	MIMontageDocument* document = [[NSDocumentController sharedDocumentController] currentDocument];
	TArmeDoc* armeDoc = [document armeDocument];
	ZzAssertMsg(armeDoc,"Windows focus problem");

	ComponentResult result = noErr;
    OSType channelType = VideoMediaType;
    short channelIndex = 1;
	Rect projectRect;

NS_DURING
    // get a video channel
    do 
	{
        result = ::SGGetIndChannel(fSequenceGrabber, channelIndex, &fSGVideoChannel, &channelType);
		channelIndex++;
	} while (!result && (VideoMediaType != channelType));
	
    if (result) fSGVideoChannel = nil;

    if (!fSGVideoChannel) 
	{
        result = ::SGNewChannel(fSequenceGrabber, VideoMediaType, &fSGVideoChannel);
		if ( result != noErr ) 
			[NSException raise:@"MTGGenericException" format:@"MTG Generic Exception"];
	}

	
	if (fSGVideoChannel) 
	{
		result = ::SGSetChannelUsage( fSGVideoChannel, seqGrabRecord
								/*| seqGrabPreview | seqGrabPlayDuringRecord*/ );
		if ( result != noErr )
			[NSException raise:@"MTGGenericException" format:@"MTG Generic Exception"];

		result = ::SGGetSrcVideoBounds( fSGVideoChannel, &fNativeSize );
		if ( result != noErr ) 
			[NSException raise:@"MTGGenericException" format:@"MTG Generic Exception"];

		//result = ::SGSetChannelBounds( fSGVideoChannel, &fPreviewBounds);
		result = ::SGSetDataProc(fSequenceGrabber, NewSGDataUPP(ReceiveSGDataProc), (long)fSequenceGrabberView);
		if ( result != noErr ) 
			[NSException raise:@"MTGGenericException" format:@"MTG Generic Exception"];

		// right now these are hardcoded, should eventually be part of project settings
		// http://developer.apple.com/techpubs/quicktime/qtdevdocs/APIREF/SOURCESV/cationfsforvchannelcps.htm
		// http://developer.apple.com/techpubs/quicktime/qtdevdocs/APIREF/SOURCESV/utilityfsforsgchannelcps.htm
			// before : hardcoded kMotionJPEGACodecType
		OSType videoType;
		if(armeDoc->fMasterMovie->GetWorkingCodecType() == kDvNtsc720_480)
		{
			videoType = kDVCNTSCCodecType;
			MacSetRect( &projectRect, 0, 0, 720, 480 );                                             
		}
		else if(armeDoc->fMasterMovie->GetWorkingCodecType() == kDvPal720_576)
		{
			videoType = kDVCPALCodecType;
			MacSetRect( &projectRect, 0, 0, 720, 576 );
		}
		else
		{
			videoType = kMotionJPEGACodecType;
			MacSetRect( &projectRect, 0, 0, 320, 240 );
		}
		
		result = ::SGSetVideoCompressorType(fSGVideoChannel, videoType);
		if ( result != noErr )
			[NSException raise:@"MTGGenericException" format:@"MTG Generic Exception"];

//		long projectFPS = [document FPS];
//		result = ::SGSetFrameRate(fSGVideoChannel, projectFPS << 16);
		result = ::SGSetFrameRate(fSGVideoChannel, 0); // o means use default
		if ( result != noErr )
			[NSException raise:@"MTGGenericException" format:@"MTG Generic Exception"];

		// projectRect = { 0, 0, 240, 320 }; /*NSRect2Rect( [fSequenceGrabberView bounds] );*/ 
		result = ::SGSetChannelBounds( fSGVideoChannel, &projectRect);
		if ( result != noErr ) 
			[NSException raise:@"MTGGenericException" format:@"MTG Generic Exception"];
	}

    // get an audio channel
	channelType = SoundMediaType;
    channelIndex = 1;
    do
    {
        result = ::SGGetIndChannel(fSequenceGrabber, channelIndex++, &fSGAudioChannel, &channelType);
    }
    while (!result and (SoundMediaType != channelType));
    if (result)
		fSGAudioChannel = nil;
    
	if (!fSGAudioChannel) 
	{
        result = ::SGNewChannel(fSequenceGrabber, SoundMediaType, &fSGAudioChannel);
		if ( result != noErr )
			[NSException raise:@"MTGGenericException" format:@"MTG Generic Exception"];
	}
	
    if (fSGAudioChannel)
	{
		result = ::SGSetChannelUsage( fSGAudioChannel, seqGrabRecord
								/*| seqGrabPreview | seqGrabPlayDuringRecord*/ );
		if ( result != noErr )
			[NSException raise:@"MTGGenericException" format:@"MTG Generic Exception"];
		result = ::SGSetDataProc(fSequenceGrabber, NewSGDataUPP(ReceiveSGDataProc), (long)fSequenceGrabberView);
		if ( result != noErr ) 
			[NSException raise:@"MTGGenericException" format:@"MTG Generic Exception"];

		// right now these are hardcoded, should eventually be part of project settings
		// http://developer.apple.com/techpubs/quicktime/qtdevdocs/APIREF/SOURCESV/cationfsforsoundchannelcps.htm
		// note that changes here need synchronizing with MISequenceGrabberView -receiveSGData
		//long frequency = armeDoc;//$$$QTSGAUDIOMON
		long frequency = armeDoc->GetAudioFrameRate(); //$$$QTSGAUDIOMON
		
		result = ::SGSetSoundInputRate( fSGAudioChannel, frequency << 16 );
		if ( result != noErr ) 
			[NSException raise:@"MTGGenericException" format:@"MTG Generic Exception"];
//		short bits = 16; //$$$QTSGAUDIOMON
		short bits = 32; //$$$QTSGAUDIOMON
		short channels = 2; 
//		OSType soundType = k16BitBigEndianFormat; //$$$QTSGAUDIOMON
//		OSType soundType = kSoundNotCompressed;
		OSType soundType = kFloat32Format;
		result = ::SGSetSoundInputParameters(fSGAudioChannel, bits, channels, soundType );
		if ( result != noErr ) 
		{
			std::cerr << "::SGSetSoundInputParameters failed : " << ArmeGetQTError(result) << " ( " << result << ")" << std::endl;
			[NSException raise:@"MTGGenericException" format:@"MTG Generic Exception"];
		}
		
		result = ::SGSetSoundRecordChunkSize(fSGAudioChannel,-16384); // 1/4 sec
		if ( result != noErr ) 
		{
			std::cerr << "::SGGetSoundRecordChunkSize failed : " << ArmeGetQTError(result) << " ( " << result << ")" << std::endl;
			[NSException raise:@"MTGGenericException" format:@"MTG Generic Exception"];
		}
	}

NS_HANDLER
	fSGVideoChannel = nil;
	fSGAudioChannel = nil;
NS_ENDHANDLER
}

- (void)openCameraComponent
{
/*	OSErr err = noErr;
	ComponentResult result;
	
    // try to open a camera component
	fDVCamera = ::OpenDefaultComponent(kIDHComponentType, kIDHSubtypeDV);
	if (!fDVCamera) return;

	result = ::IDHNewNotification( fDVCamera, kIDHDeviceIDEveryDevice, SGNotificationProc, self, &fNotificationID);

	result = ::IDHNotifyMeWhen( fDVCamera, fNotificationID, kIDHEventDeviceAdded | kIDHEventDeviceRemoved );	

	QTAtomContainer deviceList = nil;
	::IDHGetDeviceList( fDVCamera, &deviceList);
	int numberDVDevices = ::QTCountChildrenOfType(deviceList, kParentAtomIsContainer, kIDHDeviceAtomType);
	if (numberDVDevices < 1)
	{
		//[self closeCameraComponent];
		[self setIsCameraConnected:NO];
		[self updateVideoState];
		return;
	}

	// get the atom to first device
	QTAtom deviceAtom = ::QTFindChildByIndex( deviceList, kParentAtomIsContainer, kIDHDeviceAtomType, 1, nil);
	// find the isoch characteristics for this device
	QTAtom isochAtom = ::QTFindChildByIndex( deviceList, deviceAtom, kIDHIsochServiceAtomType, 1, nil);
	// how many configs exist for this device -- we'll just assume at least 1 and first is default
	// int nConfigs = ::QTCountChildrenOfType( deviceList, isochAtom, kIDHIsochModeAtomType);
	// get first configs atom
	QTAtom configAtom = ::QTFindChildByIndex( deviceList, isochAtom, kIDHIsochModeAtomType, 1, nil);
	// find the media type atom
	QTAtom mediaAtom = ::QTFindChildByIndex( deviceList, configAtom, kIDHIsochMediaType, 1, nil);
	::QTLockContainer( deviceList);
	// get the value of the mediaType atom
	OSType mediaType;
	long size = sizeof(mediaType);
	::QTCopyAtomDataToPtr( deviceList, mediaAtom, true, sizeof(mediaType), &mediaType, &size);
	::QTUnlockContainer( deviceList);
	// is this config a video config?
	if( mediaType == kIDHVideoMediaAtomType) // found video device
	{
		fCameraConfig.container = deviceList; // save this config
		fCameraConfig.atom = configAtom;
	}
	::IDHSetDeviceConfiguration( fDVCamera, &fCameraConfig);
	err = ::IDHOpenDevice( fDVCamera, kIDHOpenForReadTransactions);
	//if (err)
		//[self closeCameraComponent];
*/	
	ComponentResult result;
	result = ::IDHNewNotification( [fCamera cameraComponent], kIDHDeviceIDEveryDevice, SGNotificationProc, self, &fNotificationID);

	result = ::IDHNotifyMeWhen( [fCamera cameraComponent], fNotificationID, kIDHEventDeviceAdded | kIDHEventDeviceRemoved );	

	fCamera = [[MICamera alloc] init];
	if (!fCamera)
	{
		[self setIsCameraConnected:NO];
		[self updateVideoState];
		return;		
	}
	
	fCameraTimer = [NSTimer timerWithTimeInterval:1 target:self selector:@selector(cameraTimer:) userInfo:nil repeats:YES];
	[[NSRunLoop currentRunLoop] addTimer:fCameraTimer forMode:NSModalPanelRunLoopMode];

	[fSequenceGrabberView setConnected:YES];
	[self setIsCameraConnected:YES];
	[self updateVideoState];
}


- (void)closeCameraComponent
{
//	if (fDVCamera) {
	if (fCamera) {
		[fCameraTimer invalidate];
		fCameraTimer = nil;
		/*::IDHCancelNotification( fDVCamera, fNotificationID);
		::IDHCloseDevice(fDVCamera);
		::CloseComponent(fDVCamera);
		fDVCamera = nil;*/
		::IDHCancelNotification([fCamera cameraComponent], fNotificationID);
		[fCamera close];
		[fSequenceGrabberView setConnected:NO];
		[self setIsCameraConnected:NO];
		[self updateVideoState];
	}
}

- (void)cameraTimer:(NSTimer*)timer {
	int oldCameraState = fCameraState;
	fCameraState = [self deviceControl:deviceQuery];
	if ( fCameraState == -1 ) {
		if ( fMode != kDVPlayingback ) {
			[self setMode:kDVIdle];
			return ;
		} else return ;
	}
	//NSLog(@"camera state == %x", fCameraState);
	if ( oldCameraState != fCameraState ) {
		[self updateCameraState];

		if ( deviceCameraRec == fCameraState ) {
			[self setMode:kDVGrabbing];
			[fVideoPlayButton setEnabled:NO];
			[fVideoStopButton setEnabled:NO];
			[fVideoPauseButton setEnabled:NO];
			[fVideoFFButton setEnabled:NO];
			[fVideoRewButton setEnabled:NO];
		} else if ( oldCameraState == deviceCameraRec ) {
			[fVideoPlayButton setEnabled:YES];
			[fVideoStopButton setEnabled:YES];
			[fVideoPauseButton setEnabled:YES];
			[fVideoFFButton setEnabled:YES];
			[fVideoRewButton setEnabled:YES];
		}


		if ( ( fMode == kDVRecording ) && ( fCameraState != devicePlay ) ) {
			[self stopRecording:nil];

			if ( fCameraState != deviceStop ) {
				if ( fMode == kDVPlayingback ) [self setMode:kDVGrabbing];
				if ( fMode == kDVIdle ) [self setMode:kDVGrabbing];
			}

			if ( ( ( oldCameraState != devicePlay ) && ( oldCameraState != deviceRewindPlay ) && ( oldCameraState != deviceFastForwardPlay ) )
		&& ( ( fCameraState == devicePlay ) || ( fCameraState == deviceRewindPlay ) || ( fCameraState == deviceFastForwardPlay ) ) )
				[fSequenceGrabberView setShowBlueScreen:NO];
			else [fSequenceGrabberView setShowBlueScreen:YES];
		} else {

			if ( ( fMode == kDVGrabbing ) && ( fCameraState == deviceStop ) && ( oldCameraState == devicePlay ) ) {
				[self setMode:kDVIdle];
				[fSequenceGrabberView setShowBlueScreen:YES];
			}
			
			else if ( fMode != kDVRecording ) {
				if ( ( ( oldCameraState != devicePlay ) && ( oldCameraState != deviceRewindPlay ) && ( oldCameraState != deviceFastForwardPlay ) )
		 && ( ( fCameraState == devicePlay ) || ( fCameraState == deviceRewindPlay ) || ( fCameraState == deviceFastForwardPlay ) ) ) {
					[self setMode:kDVGrabbing];
				}
				
			}
		}
	}
}

- (void)updateCameraState {
	[fVideoPlayButton setState:NSOffState];
	[fVideoStopButton setState:NSOffState];
	[fVideoPauseButton setState:NSOffState];
	[fVideoFFButton setState:NSOffState];
	[fVideoRewButton setState:NSOffState];
	//NSLog(@"camera state = %x", fCameraState );
	switch ( fCameraState ) {
		case devicePlay :
			[fVideoPlayButton setState:NSOnState];
			break;
		case deviceFastForward :
			[fVideoFFButton setState:NSOnState];
			break;
		case deviceRewind :
			[fVideoRewButton setState:NSOnState];
			break;
		case deviceStop :
			[fVideoStopButton setState:NSOnState];
			break;
		case devicePause :
			[fVideoPauseButton setState:NSOnState];
			break;
		case deviceRewindPlay :
			[fVideoRewButton setState:NSOnState];
			[fVideoPlayButton setState:NSOnState];
			break;
		case deviceFastForwardPlay :
			[fVideoFFButton setState:NSOnState];
			[fVideoPlayButton setState:NSOnState];
			break;
	}
}

- (IBAction)show:(id)sender 
{
	[self window]; // force window to load

	fCurrentTakeIndex = -1;
	
	fMode = kDVIdle;
	[self setMode:kDVIdle];
	
	[self updateTakesPopupButton];

	[fKeepCheckbox setState:NSOffState];
				
	fNotificationTimer = [NSTimer timerWithTimeInterval:2 target:self selector:@selector(notificationTimer:) userInfo:nil repeats:YES];
	fFreeSpaceTimer = [NSTimer timerWithTimeInterval:5 target:self selector:@selector(freeSpaceTimer:) userInfo:nil repeats:YES];

	[[NSRunLoop currentRunLoop] addTimer:fNotificationTimer forMode:NSModalPanelRunLoopMode];
	[[NSRunLoop currentRunLoop] addTimer:fFreeSpaceTimer forMode:NSModalPanelRunLoopMode];
	[self freeSpaceTimer:nil];
	
	[fDurationField setStringValue:@"--:--:--"];
	
	// Window MUST be visible before calling [self openAll]
	[[self window] center];
	[[self window] makeKeyAndOrderFront:sender];
	[self openAll];
		
	[NSApp runModalForWindow:[self window]];
	[[self window] orderOut:self];
}

- (void)notificationTimer:(NSTimer*)theTimer {
	if (fReset) {
		fReset = NO;
//		::IDHNotifyMeWhen( fDVCamera, fNotificationID, kIDHEventDeviceAdded | kIDHEventDeviceRemoved );
		::IDHNotifyMeWhen( [fCamera cameraComponent], fNotificationID, kIDHEventDeviceAdded | kIDHEventDeviceRemoved );
		[self forceDeviceReset];
		if (fCameraTimer) [self cameraTimer:nil];
	}
}

- (void)setFreeDiskSpace:(long long)diskFreeBytes {
	double size;
	NSString *unit;

	if ( diskFreeBytes < pow( 1024, 1 ) ) {
		size = diskFreeBytes; unit = @"bytes";
	} else if ( diskFreeBytes < pow( 1024, 2 ) ) {
		size = diskFreeBytes / pow( 1024, 1 ); unit = @"Kb";
	} else if ( diskFreeBytes < pow( 1024, 3 ) ) {
		size = diskFreeBytes / pow( 1024, 2 ); unit = @"Mb";
	} else if ( diskFreeBytes < pow( 1024, 4 ) ) {
		size = diskFreeBytes / pow( 1024, 3 ); unit = @"Gb";
	} else if ( diskFreeBytes < pow( 1024, 5 ) ) {
		size = diskFreeBytes / pow( 1024, 4 ); unit = @"Tb";
	} else if ( diskFreeBytes < pow( 1024, 6 ) ) {
		size = diskFreeBytes / pow( 1024, 5 ); unit = @"Hb";
	}
	[fDiskFreeSpace setStringValue:[NSString stringWithFormat:@"%.2lf %@", size, unit]];
}


#define kMinimumFreeSpace 40
- (void)freeSpaceTimer:(NSTimer*)theTimer {
#if !defined(__MWERKS__)

	NSString *mountPoint = [[[[NSDocumentController sharedDocumentController] currentDocument] fileName] mountPoint];

	if ( fMode != kDVRecording ) {
		struct statfs buffer;
		int ret;

		ret = statfs( [mountPoint fileSystemRepresentation], &buffer );

		if ( ret == 0 ) {
			fDiskFreeBytes = ((long long)buffer.f_bsize) * ((long long)buffer.f_bfree);
			[self setFreeDiskSpace:fDiskFreeBytes];
		}  else [fDiskFreeSpace setStringValue:@"???"];

	} else {
		long long remainingFreeDiskBytes;

		if ( [mountPoint length] == 1 ) { // we are on the root volume
			struct statfs buffer;
			int ret;

			ret = statfs( [mountPoint fileSystemRepresentation], &buffer );

			if ( ret == 0 ) {
				remainingFreeDiskBytes = ((long long)buffer.f_bsize) * ((long long)buffer.f_bfree);
			}
		} else { // we are on mounted volume
			char path[kMaxPathLength + 1] = { 0 };
			fMovieFileRef->GetFilePath(path);

			NSString* moviePath = [[NSFileManager defaultManager] stringWithFileSystemRepresentation:path length:strlen(path)];

			long long fileSize = [[[[NSFileManager defaultManager] fileAttributesAtPath:moviePath traverseLink:NO] objectForKey:NSFileSize] longLongValue];
			remainingFreeDiskBytes = fDiskFreeBytes - fileSize ;
		}

		[self setFreeDiskSpace:remainingFreeDiskBytes];

		long long minfreespace = ( ( fVideoTakesLength + fTime ) * 180LL + ( [fTakes count] + 1 ) * 1024LL + kMinimumFreeSpace * 1024LL ) * 1024LL ;

		if ( remainingFreeDiskBytes <  minfreespace ) {
			[self stopRecording:nil];
			[[self window] setLevel:[[self window] level] - 1];
			NSRunAlertPanel( @"Recording has stopped.", @"Free disk space is %@.", @"OK", nil, nil, [fDiskFreeSpace stringValue] );
			[[self window] setLevel:[[self window] level] + 1];
		}
	}
#else //#if !defined(__MWERKS__)
	[fDiskFreeSpace setStringValue:@"???"];
#endif //#if !defined(__MWERKS__)
}

- (void)timeTimer:(NSTimer*)theTimer {
	int seconds;
	int minutes;
	int hours;

	fTime++;
	hours = fTime / 3600 ;
	minutes = ( fTime - (hours * 3600) ) / 60 ;
	seconds = fTime - (hours * 3600) - (minutes * 60);

	[fDurationField setStringValue:[NSString stringWithFormat:@"%02d:%02d:%02d", hours, minutes, seconds]];
}


- (void)cleanup {
	if ( ( [self isCameraConnected] ) && ( fCameraState == devicePlay ) ) [self deviceControl:deviceStop];
	[self setMode:kDVIdle];
	[fNotificationTimer invalidate];
	[fFreeSpaceTimer invalidate];
	[self closeAll];
	[[self window] orderOut:self];
}

- (void)finalCleanup {
	[fTakes removeAllObjects];
	
	// Crash in the following code :
	/*MIMontageDocument *document = [[NSDocumentController sharedDocumentController] currentDocument];
	TArmeDoc *doc = [document armeDocument];
	doc->DeleteUnusedFiles();*/
	//added  13/02/03 by
	[[[NSDocumentController sharedDocumentController] currentDocument] deleteUnusedFiles];
}

- (IBAction)importAndClose:(id)sender {
	NSEnumerator *enumerator = [fTakes objectEnumerator];
	MIDVTake *takeObject;
	BOOL displayAlert = YES;

	while (( takeObject = [enumerator nextObject] )) {
		if ( [takeObject isSelected] ) displayAlert = NO;
	}

	if ( ( displayAlert ) && ( [fTakes count] > 0 ) ) {
		[[self window] setLevel:[[self window] level] - 1];
		int reply = NSRunAlertPanel(@"No take will be imported.", @"Discard all takes ?", @"Cancel", @"Discard", nil);
		[[self window] setLevel:[[self window] level] + 1];

		if ( NSAlertDefaultReturn == reply ) {
			return ;
		} else {
			[NSApp stopModal];
			[self cleanup];
			[self finalCleanup];
			return;
		}
	}

	[NSApp stopModal];
	[self cleanup];
	[self performSelector:@selector(importSelectedTakes:) withObject:sender afterDelay:.2 inModes:[NSArray arrayWithObject:NSDefaultRunLoopMode]];
}


- (IBAction)cancelAndClose:(id)sender {

	if ( [fTakes count] > 0 ) {
		[[self window] setLevel:[[self window] level] - 1];
		int reply = NSRunAlertPanel(@"All takes are about to be deleted.", @"Discard all takes ?", @"Cancel", @"Discard", nil) ;
		[[self window] setLevel:[[self window] level] + 1];
		if ( NSAlertDefaultReturn == reply ) {
			return ;
		} else {
			[NSApp stopModal];
			[self cleanup];
			[self finalCleanup];
		}
	} else {
		[NSApp stopModal];
		[self cleanup];
		[self finalCleanup];
	}
}

/*- (void)checkDevices {
	[self stopIdling];
	
	if ( [fSequenceGrabberView previewing] ) [fSequenceGrabberView stopPreviewing];
	[fSequenceGrabberView setNeedsDisplay:YES];
	
	if ( !fSequenceGrabber || !fSGVideoChannel || !fSGAudioChannel ) {
		[self openSequenceGrabberComponent];
		[self openChannels];
	}

	[self updateDeviceMenu];
	
    if (fSGVideoChannel || fSGAudioChannel) {
		[fSequenceGrabberView startPreviewing:fSequenceGrabber video:fSGVideoChannel audio:fSGAudioChannel];
		[self startIdling];
	}
}*/

- (void)forceDeviceReset {
	[self closeAll];
	[self openAll];
}

- (void)closeAll {
	[self stopIdling];

	if ( fMode == kDVPlayingback ) {
		[self updateCameraState];
	} else {
		[self setMode:kDVIdle];
		[fSequenceGrabberView setNeedsDisplay:YES];
	}

	[self closeSequenceGrabberComponent];
	[self closeCameraComponent];

	[self updateDeviceMenu];
}

- (void)openAll {
	[self openSequenceGrabberComponent];
	[self openChannels];
	[self openCameraComponent];

	[self updateDeviceMenu];

	//if ( [self isCameraConnected] ) [self setMode:kDVGrabbing];
}


- (void)updateDeviceMenu {
	ComponentResult result;

	// get currently selected if any
	// no, we'll coordinate with SG
	//NSString* selected = [fDevicePopup titleOfSelectedItem];

	// remove all except 'None' at top
	/*for( int i = [fDevicePopup numberOfItems]; i > 1; i--)
		[fDevicePopup removeItemAtIndex:1];*/

	if (!fSGVideoChannel) {
		[fDeviceTextField setStringValue:@"None"];
		return;
	}

	// add devices, select current one if any
	SGDeviceList list = nil;
	result = ::SGGetChannelDeviceList(fSGVideoChannel, 0, &list);
	if (list) {
		SGDeviceName sgName = (**list).entry[(**list).selectedIndex];
		char cName[64];
		p2cstrcpy(cName, sgName.name);
		NSString* text = [[[NSString alloc] initWithCString:cName] autorelease];
		[fDeviceTextField setStringValue:text];

		/*for( int j = 0; j < (**list).count; j++) {
			SGDeviceName sgName = (**list).entry[j];
			char cName[64];
			p2cstrcpy(cName, sgName.name);
			NSString* text = [[NSString alloc] initWithCString:cName];
			[fDevicePopup addItemWithTitle:text];
			NSMenuItem* item = [fDevicePopup itemAtIndex:[fDevicePopup numberOfItems] - 1];
			[item setTarget:self];
			[item setAction:@selector(deviceFromMenu:)];
			//if (selected == text)
			if (j == (**list).selectedIndex)
				[fDevicePopup selectItemAtIndex:[fDevicePopup numberOfItems] - 1];
		}*/

		result = ::SGDisposeDeviceList(fSequenceGrabber, list);
	}
}

- (IBAction)deviceFromMenu:(id)sender
{
	ComponentResult result;
	Str255 str;

	::c2pstrcpy( str, [[sender title] UTF8String] );
	result = ::SGSetChannelDevice( fSGVideoChannel, str);
}

- (void)startIdling {
	if ( !fIdleSGTimer ) {
		float interval = 1./30.;

		fIdleSGTimer = [[NSTimer timerWithTimeInterval:interval target:self selector:@selector(idleSG:) userInfo:nil repeats:YES] retain];

		[[NSRunLoop currentRunLoop] addTimer:fIdleSGTimer forMode:NSModalPanelRunLoopMode];
	}
}

- (void)stopIdling {
    if (fIdleSGTimer) {
        [fIdleSGTimer invalidate];
		[fIdleSGTimer release];
        fIdleSGTimer = nil;
    }
}

- (void)idleSG:(NSTimer*)theTimer {
    OSErr err = ::SGIdle(fSequenceGrabber);
    if (err) {
		NSLog(@"error %d in idleSG:", err);
		[self forceDeviceReset];
    }
}

- (IBAction)videoSettings:(id)sender {
    if (!fSequenceGrabber or !fSGVideoChannel) {
        NSBeep();
        return;
    }

    Rect newActiveVideoRect;
    Rect curBounds, curVideoRect, newVideoRect;
    ComponentResult err;

    short numPanels = 0;
    Component* panelList = nil;

	// as per 2002-04-16-recording_specs.pdf,
	// get the video capture source components only
	ComponentDescription theDesc;
	theDesc.componentType = SeqGrabPanelType;
	theDesc.componentSubType = VideoMediaType; // also soun text
	theDesc.componentManufacturer = SeqGrabSourcePanelType; // also cmpr imag
	theDesc.componentFlags = 0;
	theDesc.componentFlagsMask = 0;
	numPanels = ::CountComponents(&theDesc);
	if (!numPanels) {
		NSBeep();
		return;
	}
	panelList = new Component[numPanels];
	Component lastID = nil;
	for (int i = 0; i < numPanels ;i++) {
		lastID = panelList[i] = ::FindNextComponent( lastID, &theDesc );

		ComponentDescription cd;
		::GetComponentInfo(lastID, &cd, nil, nil, nil);
	}

	// Get our current state
    err = ::SGGetChannelBounds (fSGVideoChannel, &curBounds);
    err = ::SGGetVideoRect (fSGVideoChannel, &curVideoRect);

	MIDVModes mode = fMode;
	[self setMode:kDVIdle];

    err = ::SGSettingsDialog(fSequenceGrabber, fSGVideoChannel,
							 numPanels, panelList,
							 0L, nil, 0L);
    delete [] panelList;

    if (!err) {
        err = ::SGGetVideoRect (fSGVideoChannel, &newVideoRect);
        err = ::SGGetSrcVideoBounds (fSGVideoChannel, &newActiveVideoRect);

        if (fSGSettings) {
			::DisposeUserData(fSGSettings);
			fSGSettings = nil;
		}
        err = ::SGGetSettings(fSequenceGrabber, &fSGSettings, 0);
	}

	[self setMode:mode];
}

- (void)stopRecording
{
    // close up the recording, start preview again

	[fSequenceGrabberView stopPreviewing];
	::SGStop(fSequenceGrabber);
	[self stopIdling];

	if ( deviceCameraRec != fCameraState ) {
		[self deviceControl:deviceStop];
	}

    /*OSErr err = */::SGSetDataRef(fSequenceGrabber, 0, 0, seqGrabDontMakeMovie);
    ::SGSetMaximumRecordTime(fSequenceGrabber, 0);

	ZzAssert(fMovieFileRef); // Is your camera/vcr connected ? it crashed IN the call below so let's check first !!
	[fCurrentTake setMovieFileRef:fMovieFileRef];
	fMovieFileRef->Release();
	fMovieFileRef = nil;
}

- (void)startRecording
{
    if (!fSGVideoChannel)
	{
		NSBeep();
        return;
	}

    MIMontageDocument* frontDoc = [[NSDocumentController sharedDocumentController] currentDocument];
    if (!frontDoc)
    {
        NSBeep();
        return;
    }

	OSErr err = noErr;

	//OSStatus err = MakeUniqueFSSpec(fCaptureFile, [frontDoc armeDocument]->GetProjectFolderPath(), "Video Capture<%ld>.mov");

	//fMovieFileRef = new ArmeVideoTakeDataFileRef([fTakes count] + 1, [frontDoc armeDocument]);
	fMovieFileRef = [frontDoc armeDocument]->GetNewVideoDataFileRef();
	char path[kMaxPathLength + 1] = { 0 };
	fMovieFileRef->GetFilePath(path);
	//NSLog(@"mov path = %s", path );

	// make sure the file doesn't exist
	remove(path);
	
	fCaptureFile = FSSpecFromNonExistingFilePath(path, FOUR_CHAR_CODE('TVOD'), FOUR_CHAR_CODE('MooV'), err);
	if (ThrowOnOSStatus(err))
    {
        NSBeep();
        return;
    }
	else
    {
		err = ::CreateMovieFile(&fCaptureFile, 'TVOD', smSystemScript,
						  createMovieFileDeleteCurFile | createMovieFileDontOpenFile
						  | createMovieFileDontCreateMovie | createMovieFileDontCreateResFile,
						  nil, nil);
    }
	fMovieFileRef->AddRef();

    // start recording
	if ( ![fSequenceGrabberView previewing] ) {
		[fSequenceGrabberView startPreviewing:fSequenceGrabber video:fSGVideoChannel audio:fSGAudioChannel];
	}	

	::SGStop(fSequenceGrabber);
    [self stopIdling];

    err = ::SGSetDataOutput(fSequenceGrabber, &fCaptureFile, seqGrabToDisk);

    // we need to specify a max time of 10 hours (max time handled by the master movie)
    unsigned long captureTicks = 10 * 60 * 60 * 60;
	::SGSetMaximumRecordTime(fSequenceGrabber, captureTicks);

	if ( deviceCameraRec != fCameraState ) {
		if ( fCameraState != devicePlay ) {
			BOOL onPause = ( fCameraState == devicePause ) ;
			[self deviceControl:devicePlay];
			if ( !onPause ) sleep(1);
			int reftime = [self deviceControl:deviceTime];
			do {
			} while ( reftime == [self deviceControl:deviceTime] ) ;
			usleep(800000);
		}
		usleep(200000);
	}

/*	TimeBase soundTimeBase=NULL, sgTimeBase=NULL;
	err = ::SGPrepare( fSequenceGrabber, true, true );
	if ( !err ) {
		err = SGGetTimeBase( fSequenceGrabber, &sgTimeBase );
		if( !err ) {
			err = SGGetChannelTimeBase( fSGAudioChannel, &soundTimeBase );
			if ( !err && soundTimeBase ) {
				SetTimeBaseMasterClock( sgTimeBase,
						   (Component)GetTimeBaseMasterClock(soundTimeBase), nil );
            }
		}
	}*/
	

    err = ::SGStartRecord(fSequenceGrabber);
    [self startIdling];
}

- (BOOL)isDeviceControllable
{
	return fCamera != nil;
//	return fDVCamera != nil;
}

- (int)deviceControl:(int)command
{
	return [fCamera deviceControl:command];
/*	ComponentInstance controlInst;
	ComponentResult result;
	DVCTransactionParams pParams;
	int o_in[2];
	char o_out[16];
	int i;
	int reply;
	char *cReply = (char*)&reply;
	int inSize, outSize;
	int timeout = 5;

	if ( !fDVCamera ) NSLog(@"fDVCamera = nil");
	
	result = ::IDHGetDeviceControl(fDVCamera, &controlInst);
	if ( result ) {
		NSLog(@"error %u in IDHGetDeviceControl()", result ) ;
		return -1;
	}

	o_in[0] = command;
	o_in[1] = 0xffffffff;

	inSize = sizeof(int);
	if ( command == deviceTime ) inSize *= 2;

	outSize = sizeof(o_out);
	
	// fill up the transaction parameter block
	pParams.commandBufferPtr = (char*)o_in;
	pParams.commandLength = inSize;
	pParams.responseBufferPtr = o_out;
	pParams.responseBufferSize = outSize;
	pParams.responseHandler = nil;
	
	do
	{
		for (i=0; i < outSize; i++) o_out[i] = 0;
		reply = 0;
		result = ::DeviceControlDoAVCTransaction( controlInst, &pParams);
		timeout--;

		if ( command == deviceQuery ) {
		cReply[0] = 0x00;
		cReply[1] = 0x20;
		cReply[2] = o_out[2];
		cReply[3] = o_out[3];
		} else if ( command == deviceTime ) {
			cReply[0] = o_out[4];
			cReply[1] = o_out[5];
			cReply[2] = o_out[6];
			cReply[3] = o_out[7];
		}
		
		if (result == kIOReturnOffline)
		{
			sleep(1);
			continue;
		}
		if(result)
			break;
	}
	while ( (result != kIOReturnSuccess) && ( timeout > 0 ) ) ;
	return reply;*/
}

- (void)setIsCameraConnected:(BOOL)flag {
	fCameraConnected = flag;
}

- (BOOL)isCameraConnected {
	return fCameraConnected;
}

- (void)updateVideoState {
	if ( [self isCameraConnected] ) {
		[fVideoSettingsButton setEnabled:YES];
		if ( deviceCameraRec != fCameraState ) {
			[fVideoPlayButton setEnabled:YES];
			[fVideoStopButton setEnabled:YES];
			[fVideoPauseButton setEnabled:YES];
			[fVideoFFButton setEnabled:YES];
			[fVideoRewButton setEnabled:YES];
		}
		[fCaptureRecordButton setEnabled:YES];
	} else {
		[fVideoSettingsButton setEnabled:NO];
		[fVideoPlayButton setEnabled:NO];
		[fVideoStopButton setEnabled:NO];
		[fVideoPauseButton setEnabled:NO];
		[fVideoFFButton setEnabled:NO];
		[fVideoRewButton setEnabled:NO];
		[fCaptureRecordButton setEnabled:NO];
	}
}

- (void)updateTakesPopupButton {
	unsigned int i;
	NSString* str;

	[fTakesPopupButton removeAllItems];

    for ( i = 0 ; i < [fTakes count]; i++ ) {
		MIDVTake *takeObject = [fTakes objectAtIndex:i];
		if ( [takeObject isSelected] ) str = [NSString stringWithFormat:@"Take %d - import", i+1];
		else str = [NSString stringWithFormat:@"Take %d", i+1];
		[fTakesPopupButton addItemWithTitle:str];
    }

	if ( [fTakes count] > 0 ) {
		[fTakesPopupButton selectItemAtIndex:0];
		[fTakesPopupButton setEnabled:YES];
		[fCapturePlayButton setEnabled:YES];
		[fCaptureStopButton setEnabled:YES];
		[fCapturePauseButton setEnabled:YES];
		[fDiscardButton setEnabled:YES];
		[fKeepCheckbox setEnabled:YES];
	} else {
		[fTakesPopupButton addItemWithTitle:@"No Takes"];
		[fTakesPopupButton setEnabled:NO];
		[fCapturePlayButton setEnabled:NO];
		[fCapturePauseButton setEnabled:NO];
		[fCaptureStopButton setEnabled:NO];
		[fDiscardButton setEnabled:NO];
		[fKeepCheckbox setEnabled:NO];
	}
}

- (unsigned int)videoTakesLength {
	unsigned int length = 0 ;
	for ( unsigned int x = 0 ; x < [fTakes count] ; x++ ) {
		MIDVTake *takeObject = [fTakes objectAtIndex:x];
		char temp[kMaxPathLength];

		[takeObject movieFileRef]->GetFilePath(temp);
		NSString *path = [NSString stringWithUTF8String:temp];
		NSURL *url = [NSURL fileURLWithPath:path];
		NSMovie *movie = [[NSMovie alloc] initWithURL:url byReference:NO];
		length += (unsigned int)GetMovieDurationSec( (Movie)[movie QTMovie] );
		[movie release];

	}
	return length;
}

- (IBAction)takeSelected:(id)sender 
{
	SInt32 index = [fTakesPopupButton indexOfSelectedItem];

	if ( ( index >= 0 ) && ( [fTakes count] > 0 ) )
	{
		MIDVTake *takeObject = [fTakes objectAtIndex:index];
		if ( fCurrentTakeIndex != -1 ) [self pauseTake:nil];

		if ( [takeObject isSelected] ) {
			[fKeepCheckbox setState:NSOnState];
		} else {
			[fKeepCheckbox setState:NSOffState];
		}

		fCurrentTakeIndex = index;

		char temp[kMaxPathLength];

		[takeObject movieFileRef]->GetFilePath(temp);
		NSString *path = [NSString stringWithUTF8String:temp];
		NSURL *url = [NSURL fileURLWithPath:path];
		NSMovie *movie = [[NSMovie alloc] initWithURL:url byReference:NO];

		[fPlaybackView setMovie:movie];
		[fPlaybackView gotoBeginning:nil];

		int length = (int)GetMovieDurationSec( (Movie)[movie QTMovie] );
		int hours = length / 3600 ;
		int minutes = ( length - (hours * 3600) ) / 60 ;
		int seconds = length - (hours * 3600) - (minutes * 60);

		[fDurationField setStringValue:[NSString stringWithFormat:@"%02d:%02d:%02d", hours, minutes, seconds]];
		
		[movie release];
		[self setMode:kDVPlayingback];
	} 
	else
	{
		[fDurationField setStringValue:@"--:--:--"];
	}
}

- (IBAction)keepTake:(id)sender {
	MIDVTake *takeObject = [fTakes objectAtIndex:fCurrentTakeIndex];
	[takeObject setSelected:![takeObject isSelected]];
	if ( [takeObject isSelected] ) {
		[fKeepCheckbox setState:NSOnState];
	} else {
		[fKeepCheckbox setState:NSOffState];
	}
	[self updateTakesPopupButton];
	[fTakesPopupButton selectItemAtIndex:fCurrentTakeIndex];
}

- (IBAction)keepAllTakes:(id)sender {
	for ( unsigned int index = 0 ; index < [fTakes count]; index++ ) {
		MIDVTake *takeObject = [fTakes objectAtIndex:index];
		[takeObject setSelected:YES];
		[self updateTakesPopupButton];
		[fKeepCheckbox setState:NSOnState];
		[fTakesPopupButton selectItemAtIndex:fCurrentTakeIndex];
	}
}

- (IBAction)keepNoTakes:(id)sender {
	for ( unsigned int index = 0 ; index < [fTakes count]; index++ ) {
		MIDVTake *takeObject = [fTakes objectAtIndex:index];
		[takeObject setSelected:NO];
		[self updateTakesPopupButton];
		[fKeepCheckbox setState:NSOffState];
		[fTakesPopupButton selectItemAtIndex:fCurrentTakeIndex];
	}
}


- (IBAction)importSelectedTakes:(id)sender
{
	unsigned int takeIndex;
	MIMontageDocument* document = [[NSDocumentController sharedDocumentController] currentDocument];
	TArmeDoc *armeDoc = [document armeDocument];
	bool bStackVertically;
	int count = 0 ;


	VUSERCMD("Import Selected Takes (DV)");

	MIMacroCommand* command = [[[MIMacroCommand alloc] init] autorelease];
	[command setCommandName:@"Capture"];

	ArmeTime movieDuration;
	MIPoint location = [[[NSApp mainViewController] topLevelContainerView] newObjectLocation];

	for ( takeIndex = 0 ; takeIndex < [fTakes count]; takeIndex++ )
	{
		MIDVTake *takeObject = [fTakes objectAtIndex:takeIndex];
		if ( [takeObject isSelected] ) count++;
	}
			

	SInt32 val = [[MIPreferences sharedPrefs] stackingMethod];
	vernl(6,"[[MIPreferences sharedPrefs] stackingMethod] returned " << val);
	if(val == kPrefsStackingMethodAlwaysAsk)
	{
		if ( count > 1 )
		val = [[MIApplicationController sharedApplicationController] displayChooseStackingMethodForMainViewController:[NSApp mainViewController]];
		else val = kPrefsStackingMethodVertically;
	}
	vernl(6,"displayChooseStackingMethodForMainViewController: returned " << val);

	if(val == kPrefsStackingMethodCancelled) {
		[self finalCleanup];
		return;
	}
	else if(val == kPrefsStackingMethodVertically)
		bStackVertically = true;
	else if(val == kPrefsStackingMethodHorizontally)
		bStackVertically = false;
	else
		ZzAssert(0);
	

	for ( takeIndex = 0 ; takeIndex < [fTakes count]; takeIndex++ )
	{
		MIDVTake *takeObject = [fTakes objectAtIndex:takeIndex];
		if ( [takeObject isSelected] )
		{
			char temp[kMaxPathLength];
			MICreateWSObjsFromFileCommand *createClipCommand = [[MICreateWSObjsFromFileCommand alloc] initWithDocument:document];

			[takeObject movieFileRef]->GetFilePath(temp);
			[createClipCommand setFileName:[NSString stringWithUTF8String:temp]];
			[createClipCommand setClipName:[NSString stringWithFormat:@"Take_%d", takeIndex + 1]];

			([takeObject movieFileRef])->AddRef();
			[createClipCommand setFileRef:[takeObject movieFileRef]];

			NSURL *url = [NSURL fileURLWithPath:[NSString stringWithUTF8String:temp]];
			NSMovie *movie = [[NSMovie alloc] initWithURL:url byReference:NO];
			movieDuration = armeDoc->SecondsToArmeTime( GetMovieDurationSec( (Movie)[movie QTMovie] ) );
			[movie release];

			[createClipCommand setParent:[document topLevelContainer]];
			[createClipCommand setOrigin:location];
			[command addCommand:createClipCommand];
			[createClipCommand release];

			if(bStackVertically)
				GET_VALUE(location.vpos) -= ( GET_VALUE(kBetweenClipsHeight) +
#ifdef SMALL_TITLEBAR
								  GET_VALUE(kDefaultClipRepresentationHeight)
#else
								  kDefaultClipRepresentationHeight
#endif
								  + GET_VALUE(kRepTitleHeight) ) * 2 ;
			else
				location.time += movieDuration;
		}
	}

	[command setCommandName:@"Capture Clips"];
	[document doCmd:command];

	if ( [[MIPreferences sharedPrefs] gridShowMask] & kPrefsGridShowVObjLines ||
		 [[MIPreferences sharedPrefs] gridShowMask] & kPrefsGridShowHObjLines )
		[[MIApplicationController sharedApplicationController] redrawAllMainWindows];

	[[[NSApp mainViewController] topLevelContainerView] scrollToNewObject];
	[self finalCleanup];
}


- (IBAction)discardTake:(id)sender {
	[self stopTake:sender];
	[fTakes removeObjectAtIndex:fCurrentTakeIndex];
	[self updateTakesPopupButton];
	[self takeSelected:nil];
}

- (IBAction)record:(id)sender
{
    if ( fMode != kDVRecording ) {
		fVideoTakesLength = [self videoTakesLength];
		fTime = 0;
		[fDurationField setStringValue:@"00:00:00"];

		[fTakesPopupButton removeAllItems];
		[fTakesPopupButton addItemWithTitle:[NSString stringWithFormat:@"Take %d - import", [fTakes count]+1]];
		[fTakesPopupButton selectItemAtIndex:0];
		[fKeepCheckbox setState:NSOnState];
		
		[self setMode:kDVRecording];

		fCurrentTake = [[MIDVTake alloc] init];

		if (fSGVideoChannel and fSGAudioChannel) {
			[self startRecording];
		}

		
		fTimeTimer = [NSTimer timerWithTimeInterval:1.0 target:self selector:@selector(timeTimer:) userInfo:nil repeats:YES];
		[[NSRunLoop currentRunLoop] addTimer:fTimeTimer forMode:NSModalPanelRunLoopMode];
		
		[fCaptureStopButton setEnabled:YES];
		[fCapturePlayButton setEnabled:NO];
		[fCapturePauseButton setEnabled:NO];
    } else {
        NSBeep();
    }
}

- (IBAction)stopRecording:(id)sender {
	if ( fMode == kDVRecording ) {
		[fTimeTimer invalidate];
		fTimeTimer = nil;

        [self stopRecording];

		[fCurrentTake setSelected:YES];
		[fTakes addObject:fCurrentTake];
		[fCurrentTake release];
		fCurrentTake = nil;

		[fCaptureRecordButton setState:NSOffState];

		[self updateTakesPopupButton];
		[fTakesPopupButton selectItemAtIndex:[fTakes count] - 1];
		[self takeSelected:nil];

		[self setMode:kDVGrabbing];
		fVideoTakesLength = 0;
    }
}

- (IBAction)play:(id)sender 
{
	if ( fMode != kDVRecording ) 
	{
		if ( fCameraState != devicePlay ) {
		[self setMode:kDVGrabbing];
		[self deviceControl:devicePlay];
		} else [self deviceControl:devicePause];
	}
	else 
		NSBeep();
}

- (IBAction)stop:(id)sender 
{
	if (fMode != kDVRecording) 
	{
		[self setMode:kDVIdle];
		[self deviceControl:deviceStop];
	} 
	else
		NSBeep();
}

- (IBAction)pause:(id)sender {
	if (fMode != kDVRecording) {
		if ( fCameraState == devicePlay )
			[self deviceControl:devicePause];
		else [self play:sender];
	} else NSBeep();
}

- (IBAction)rewind:(id)sender {
	if ( fMode != kDVRecording ) {
		if ( fCameraState == devicePlay )
			[self deviceControl:deviceRewindPlay];
		else
			[self deviceControl:deviceRewind];
	} else NSBeep();
}

- (IBAction)fastForward:(id)sender {
	if ( fMode != kDVRecording ) {
		if ( fCameraState == devicePlay )
			[self deviceControl:deviceFastForwardPlay];
		else
			[self deviceControl:deviceFastForward];
	} else NSBeep();
}


- (IBAction)playTake:(id)sender {
	if ( fMode != kDVRecording ) {
		if ( fPlayTimer ) [self pauseTake:sender];
		else {
			[self setMode:kDVPlayingback];
			[fPlaybackView start:nil];
			fPlayTimer = [NSTimer timerWithTimeInterval:.5 target:self selector:@selector(playTimer:) userInfo:nil repeats:YES];
			[[NSRunLoop currentRunLoop] addTimer:fPlayTimer forMode:NSModalPanelRunLoopMode];
			[fCapturePlayButton setState:NSOnState];
		}
	}
}

- (void)playTimer:(NSTimer*)timer {
	if ( ![fPlaybackView isPlaying] ) {
		fPlayTimer = nil;
		[self stopTake:nil];
	}
}


- (IBAction)stopTake:(id)sender {
	if ( fMode == kDVPlayingback ) {
		if ( fPlayTimer ) {
			[fPlayTimer invalidate];
			fPlayTimer = nil;
		}
		[fPlaybackView stop:nil];
		[fPlaybackView gotoBeginning:nil];
		[fCapturePlayButton setState:NSOffState];
	}
}

- (IBAction)pauseTake:(id)sender {
	if ( fMode == kDVPlayingback ) {
		if ( fPlayTimer ) {
			[fPlayTimer invalidate];
			fPlayTimer = nil;
		}
		[fPlaybackView stop:nil];
		[fCapturePlayButton setState:NSOffState];
	}
}


- (void)setMode:(MIDVModes)mode {
	MIDVModes oMode = fMode ;

	// things to be done even if mode == fMode
	switch (fMode) {
		case kDVPlayingback:
			if ( mode == kDVPlayingback ) [self pauseTake:nil];
			break;
		case kDVRecording:
			break;
		case kDVGrabbing:
			break;
		case kDVIdle:
			break;
	}
	
	if ( mode == fMode ) return ;


	switch (fMode) {
		case kDVGrabbing:
			if ( mode != kDVGrabbing )
				if (fSGVideoChannel || fSGAudioChannel) {
					[self stopIdling];
					::SGStop(fSequenceGrabber);
					if ( mode != kDVRecording ) [fSequenceGrabberView stopPreviewing];
				}
					break;
		case kDVRecording:
			[self stopTake:nil];
			if ( deviceCameraRec != fCameraState ) {
				[fVideoPlayButton setEnabled:YES];
				[fVideoStopButton setEnabled:YES];
				[fVideoPauseButton setEnabled:YES];
				[fVideoFFButton setEnabled:YES];
				[fVideoRewButton setEnabled:YES];
			}
			[fVideoSettingsButton setEnabled:YES];
			[fKeepCheckbox setEnabled:YES];
			[fTakesPopupButton setEnabled:YES];
			[fDiscardButton setEnabled:YES];
			
			break;
		case kDVPlayingback:
			if ( mode == kDVPlayingback ) [self pauseTake:nil];
				break;
		case kDVIdle:
				break;
	}

	fMode = mode;
	//NSLog(@"mode == %@", fMode == kDVGrabbing ? @"kDVGrabbing" : ( fMode == kDVRecording ? @"kDVRecording" : ( fMode == kDVPlayingback ? @"kDVPlayingback" : @"kDVIdle" ) ) );

	switch (fMode) {
		case kDVGrabbing:
			if ( !fSGViewInstalled ) {
				[fPlaybackView removeFromSuperview];
				[fMovieContainer addSubview:fSequenceGrabberView];
				[fSequenceGrabberView release];
				fSGViewInstalled = YES;
			}
			[fImportButton setEnabled:YES];
			[fCancelButton setEnabled:YES];

			if (fSGVideoChannel || fSGAudioChannel) {
				[fSequenceGrabberView startPreviewing:fSequenceGrabber video:fSGVideoChannel audio:fSGAudioChannel];
				[self startIdling];
			}
				break;
		case kDVRecording:
			if ( !fSGViewInstalled ) {
				[fPlaybackView removeFromSuperview];
				[fMovieContainer addSubview:fSequenceGrabberView];
				[fSequenceGrabberView release];
				fSGViewInstalled = YES;
			}
			if ( ( oMode == kDVPlayingback ) || ( oMode == kDVIdle ) )
				if (fSGVideoChannel || fSGAudioChannel) {
					[fSequenceGrabberView startPreviewing:fSequenceGrabber video:fSGVideoChannel audio:fSGAudioChannel];
				}
					
			[fCaptureStopButton setAction:@selector(stopRecording:)];
			[fVideoPlayButton setEnabled:NO];
			[fVideoStopButton setEnabled:NO];
			[fVideoPauseButton setEnabled:NO];
			[fVideoFFButton setEnabled:NO];
			[fVideoRewButton setEnabled:NO];
			[fImportButton setEnabled:NO];
			[fCancelButton setEnabled:NO];
			[fVideoSettingsButton setEnabled:NO];
			[fKeepCheckbox setEnabled:NO];
			[fTakesPopupButton setEnabled:NO];
			[fDiscardButton setEnabled:NO];
			break;
		case kDVPlayingback:
			if ( fSGViewInstalled ) {
				[fSequenceGrabberView retain];
				[fSequenceGrabberView removeFromSuperview];
				[fMovieContainer addSubview:fPlaybackView];
				fSGViewInstalled = NO;
			}
			[fCaptureStopButton setAction:@selector(stopTake:)];
			[fImportButton setEnabled:YES];
			[fCancelButton setEnabled:YES];
			break;
		case kDVIdle:
			if ( !fSGViewInstalled ) {
				[fPlaybackView removeFromSuperview];
				[fMovieContainer addSubview:fSequenceGrabberView];
				[fSequenceGrabberView release];
				fSGViewInstalled = YES;
			}

			break;
	}
}


@end

