/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MIDebugRecordCtrller.h"
#import "MIMontageDocument.h"
#import "MICreateAudioClipsFromRecordedTakesCommand.h"

#ifndef MEAudioRecordManager_h
#include "MEAudioRecordManager.h"
#endif //MEAudioRecordManager_h

#ifndef MEAudioDeviceDesc_h
#include "MEAudioDeviceDesc.h"
#endif //MEAudioDeviceDesc_h

#import "MIPreferences.h"
#import "MIApplicationController.h"

@implementation MIDebugRecordCtrller

- (IBAction)recordButtonPressed:(id)sender
{
	if([sender state] == NSOnState)
	{
		MEAudioRecordManager::SharedAudioRecordManager()->StartRecording();
	}
	else
	{
		MEAudioRecordManager::SharedAudioRecordManager()->StopRecording();
	}
}

- (IBAction)donePressed:(id)sender
{
	[NSApp stopModal];
}

- (BOOL)setDoc:(MIMontageDocument*)montageDoc
{
	// step 1 : we retrieve the input device string code as defined in the prerefences
	NSString* inDevStringCode = [[MIPreferences sharedPrefs] audioInputDeviceStringCode];
	
	// step 2 : translate the string code into a CoreAudio AudioDeviceID
	AudioDeviceID inDevID = MEAudioDeviceDesc::GetAudioDeviceIDFromStringCode((CFStringRef)inDevStringCode);
	
	if(inDevID == 0x00000000) // this means the device hasn't been found
	{	
		NSString* defaultInDevStringCode = [NSString stringWithString:(NSString*)MEAudioDeviceDesc::GetDefaultInputStringCode()]; // returns a code that is 'always use default device'
		[[MIPreferences sharedPrefs] setAudioInputDeviceStringCode:[NSString stringWithString:defaultInDevStringCode]];
		inDevID = MEAudioDeviceDesc::GetAudioDeviceIDFromStringCode((CFStringRef)defaultInDevStringCode);
		
		if(inDevID == 0x00000000)
		{
			NSRunAlertPanel(@"Error", @"No suitable Audio Input Device found.", @"OK", nil, nil);
			return NO;
		}
		else
		{
			NSRunAlertPanel(@"Warning", @"The Audio Input Device chosen in the preferences cannot be opened. HyperEngine-AV now uses the default input device.", @"OK", nil, nil);
		}
	}
	
	MEAudioRecordManager::SharedAudioRecordManager()->PrepareRecordingForDoc([montageDoc armeDocument],inDevID);
	return YES;
}

+ (void)displayDebugRecordDlg:(MIMontageDocument*)montageDoc
{
	// notation shortcut
	MEAudioRecordManager* arm = MEAudioRecordManager::SharedAudioRecordManager();
	MIDebugRecordCtrller* ctrller = [[MIDebugRecordCtrller alloc] initWithWindowNibName:@"MIDebugRecord"];
	[ctrller window];
	BOOL ok = [ctrller setDoc:montageDoc];
	if(!ok)
	{
		// then nothing else is needed
		[ctrller release];
	}
	else
	{
		// init stuff here
	
		[NSApp runModalForWindow:[ctrller window]];
		[[ctrller window] orderOut:nil];
		[ctrller release];
		
		// create clips from the takes
		if(arm->GetNbTakes() > 0)
		{
			if (NSRunAlertPanel(@"Debug Record", @"Do you want to create audio clips for the recorded takes ?", @"Yes", @"No", nil) == NSAlertDefaultReturn)
			{
				// MARC : you should create a macro command that deals with video takes, too... so that
				// the user only sees one entry in the undo stack
				MICreateAudioClipsFromRecordedTakesCommand* cmd = [[[MICreateAudioClipsFromRecordedTakesCommand alloc] initWithDoc:montageDoc] autorelease];
				for(SInt32 take = 0; take < arm->GetNbTakes(); take++)
				{
					ArmeTime takeStart = arm->GetTakeStartTime(take);
					ArmeTime takeDuration = arm->GetTakeDuration(take);
					for(SInt32 file = 0; file < arm->GetTakeNbAudioFiles(take); file++)
					{
						MEAudioChunkFileIndex index = arm->GetTakeFile(take,file);
						ZzAssert(0); // uncomment following calls but convert to the correct addFile:XXX:XXX selector !
						//SInt32 nbChannels = arm->GetTakeFileNbChannels(take,file);
					//	[cmd addFile:index nbChannels:nbChannels startTime:takeStart duration:takeDuration]; // duration is for debug purposes only since it's known by the file
					}
				}
				[montageDoc doCmd:cmd];

				if ( [[MIPreferences sharedPrefs] gridShowMask] & kPrefsGridShowVObjLines ||
					[[MIPreferences sharedPrefs] gridShowMask] & kPrefsGridShowHObjLines )
					[[MIApplicationController sharedApplicationController] redrawAllMainWindows];
			
				[cmd release];
			}
		}
		arm->PutInIdleState();
	}
}

@end
