/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
#import "MIDockableItem.h"
#import "MIApplicationController.h"
#import "MIMainViewController.h"
//#import "MIOverview.h"
#import "MIDockBar.h"

NSString *kDockableItemBoardType = @"MontageInterfaceDockableItem";

@implementation MIDockableItem

-(id)initWithFrame:(NSRect)frame isStretchable:(BOOL)flag minSize:(NSSize)newMinSize
{
	self = [self initWithFrame:frame];
	_minSize = newMinSize;
	[self setStretchable:flag];
	return self;
}

-(id)initWithFrame:(NSRect)frame isStretchable:(BOOL)flag
{
	self = [self initWithFrame:frame];
	[self setStretchable:flag];
	return self;
}

- (id)initWithFrame:(NSRect)frame
{
    self = [super initWithFrame:frame];
    if (self)
	{
		_isStretchable = NO;
		_minSize = NSMakeSize(100,120);
//		[self setAutoresizingMask:NSViewWidthSizable|NSViewHeightSizable|NSViewMaxXMargin];
		[self setAutoresizingMask:NSViewMaxXMargin];
    }
    return self;
}

-(void)setApplicationController:(MIApplicationController*)newApplicationController
{
	applicationController = newApplicationController;
}

-(BOOL)isStretchable
{
	return _isStretchable;
}

-(void)setStretchable:(BOOL)flag
{
	_isStretchable = flag;
	if ( flag = YES )
	{
//		[self setAutoresizingMask:NSViewWidthSizable|NSViewHeightSizable];
	}
}

-(void)setMinSize:(NSSize)newMinSize;
{
	_minSize = newMinSize;
}

-(NSSize)minSize
{
	return _minSize;
}

- (void)drawRect:(NSRect)rect
{
	NSBezierPath *aPath;

//	[[NSColor redColor] set];
//	NSRectFill(rect);

	[[NSColor blackColor] set];
	aPath = [NSBezierPath bezierPath];
	[aPath setLineWidth:0];
	[aPath moveToPoint:NSMakePoint(5,1)];
	[aPath lineToPoint:NSMakePoint(5,NSHeight([self bounds])-1)];
	[aPath stroke];

	aPath = [NSBezierPath bezierPath];
	[[NSColor grayColor] set];
	[aPath setLineWidth:0];
	[aPath moveToPoint:NSMakePoint(3,0)];
	[aPath lineToPoint:NSMakePoint(3,NSHeight([self bounds]))];
	[aPath stroke];

	// Vertical line with a ">>" sign next to the top
}

// Drag and Drop things:


- (void)mouseDown:(NSEvent *)theEvent
{
	NSSize 			dragOffset = NSMakeSize(0, 0);
	NSPasteboard 	*pboard;
	NSImage 		*image = nil;
	
	/* 
	image = [NSImage alloc];
	image = [image initWithData:[self dataWithPDFInsideRect:[self frame]]];
	*/

	/*
	image = [[[NSImage alloc] initWithSize:[[[self subviews] objectAtIndex:0] bounds].size] autorelease];
	[image lockFocus];
    [self drawSubviews:self];
	[image unlockFocus];
	*/
	
	/*
	image = [[[NSImage alloc] initWithSize:[[[self subviews] objectAtIndex:0] bounds].size] autorelease];
	[image lockFocus];
	NSCopyBits([self gState], [self bounds], [self bounds].origin);
	[image unlockFocus];
	*/

	/*
	for ( i = 0 ; i < NSWidth([self frame]) ; i++ )
		for ( j = 0 ; j < NSHeight([self frame]) ; j++ )
		{
			color = NSReadPixel(NSMakePoint(i,j));
		}
	*/
	
	// figuring out how to create an image from the view is still a problem that has to be looked into....
	
	pboard = [NSPasteboard pasteboardWithName:NSDragPboard];
	[pboard declareTypes:[NSArray arrayWithObject:kDockableItemBoardType] owner:self];
	[pboard setData:nil forType:kDockableItemBoardType];

	[self dragImage:image at:NSMakePoint(0,0) offset:dragOffset event:theEvent pasteboard:pboard source:self slideBack:NO];

	return;
}

- (void)drawSubviews:(NSView*)aView
{
    NSArray 		*subviews = [aView subviews];
    unsigned int 	i;
    NSView 			*aSubview;
    
    for ( i = 0 ; i < [subviews count] ; i++ )
    {
        aSubview = [subviews objectAtIndex:i]; 
        [aSubview drawRect:[aSubview bounds]];
        if ( [[aSubview subviews] count] > 0 )
            [self drawSubviews:aSubview];
    }
}

- (unsigned int)draggingSourceOperationMaskForLocal:(BOOL)flag
{
	return NSDragOperationPrivate;
}

- (void)draggedImage:(NSImage *)anImage endedAt:(NSPoint)aPoint deposited:(BOOL)flag
{
	NSPanel		*newWindow;
	NSSize		tmpSize;
	
	if ( flag == NO )
	{
		// there is no receiver, but the mouse has been "released".
		// If the DockableItem (self) was not in the main dockbar, do nothing,
		// otherwise create a new window to put the DockableItem in.
		vverntl(interfaceStephan,0,"about to use MIMainViewController::mainDockBar which is not implemented");
		ZzAssert(0);
				
		if ( !IsNSPointInRect(aPoint, [self bounds]) )
		{
			if ( [self isDescendantOf:[[applicationController currentMainViewController] mainDockBar]] )
			{
				newWindow = CreatePalette( aPoint, [self bounds], @"RadarView" );
				// retain myself, I'm moving, not disappearing.
				[self retain];
				[(MIDockBar*)[self superview] removeDockableItem:self];

				// put myself in the new window
				[[newWindow contentView] insertDockableItem:self inRow:0 atPos:0];
				
				tmpSize = [newWindow frame].size;
				[newWindow setMinSize:tmpSize];

				[newWindow makeKeyAndOrderFront:self];
				[self setNeedsDisplay:YES];
				[newWindow display];
			}
		}
	}
	else
	{
		// remove myself from my old dock bar because I've been put in another one.
		// Check if my old dock bar was not the main one and is now empty, if that is the case, delete it.
	}
}

@end
