/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
#import "MIDockBar.h"
#import "MIDockableItem.h"
#import "MIApplicationController.h"
#import "MIMainViewController.h"
//#import "MIClipSheetWindowView.h"

@implementation MIDockBar

- (id)initWithFrame:(NSRect)frame
{
    self = [super initWithFrame:frame];
	
    if ( self )
	{
		_isInMainView = YES;
		_isMinimized = NO;
		[self registerForDraggedTypes:[NSArray arrayWithObject:kDockableItemBoardType]];
		_rows = [[NSMutableArray alloc] init];
    }
	
    return self;
}

- (BOOL)isInMainView
{
	return _isInMainView;
}

- (void)setIsInMainView:(BOOL)flag
{
	_isInMainView = flag;
}

- (BOOL)isEmpty
{
	if ( ![[self subviews] count] )
		return YES;
	else
		return NO;
}

- (BOOL)isMinimized
{
	return _isMinimized;
}

- (void)expand
{
	if ( !_isMinimized )
		return;
}

- (void)minimize
{
	_isMinimized = YES;
}

- (void)drawRect:(NSRect)rect
{
//	[[NSColor greenColor] set];
//	NSRectFill(rect);
}

- (void)prepareToDockItem:(MIDockableItem*)item atPos:(NSPoint)pos
{
	// this function will display the impact on other dockitems if the new item is inserted
}

- (NSPoint)insertDockableItem:(MIDockableItem*)item inRow:(unsigned int)rowNbr atPos:(unsigned int)pos
{	
	NSRect newItemFrame;
	NSMutableArray *newRow;
	
	[self addSubview:item];
	
	newItemFrame.size = [item minSize];
	if ( pos == 0 )
		newItemFrame.origin.x = 4;
	else
		newItemFrame.origin.x = NSMaxX([[[_rows objectAtIndex:rowNbr] objectAtIndex:(pos - 1)] frame])+4;
	
	[item setFrame:newItemFrame];
	if ( [_rows count] <= pos )
	{
		newRow = [[NSMutableArray alloc] init];
		[_rows insertObject:newRow atIndex:pos];
	}
	[[_rows objectAtIndex:rowNbr] insertObject:item atIndex:pos];
	[self rearrangeItems];
	
	return [item frame].origin;
}

- (void)prepareToRemoveItem:(MIDockableItem*)item
{
	// this function will display the impact on other dockitems if the item is removed
}

- (void)removeDockableItem:(MIDockableItem*)item
{
	unsigned int	i;
	NSMutableArray	*aRow;
	
	[item removeFromSuperview];
	for ( i = 0 ; i < [_rows count] ; i++ )
	{
		aRow = [_rows objectAtIndex:i];
		if ( [aRow containsObject:item] )
			[aRow removeObject:item];
	}
	
	if ( !_isInMainView && [self isEmpty] )
	{
		// remove myself and my window
		[[self window] performClose:[self window]];
	}
	else
	{
		[self rearrangeItems];
	}
	
}

- (unsigned int)draggingEntered:(id <NSDraggingInfo>)sender
{
	NSPasteboard *pboard;
    NSDragOperation sourceDragMask;

    sourceDragMask = [sender draggingSourceOperationMask];
    pboard = [sender draggingPasteboard];

    if ([[pboard types] indexOfObject:kDockableItemBoardType] != NSNotFound)
        if (sourceDragMask & NSDragOperationPrivate)
		{
			[self prepareToDockItem:[sender draggingSource]
				  atPos:[self convertPoint:[[self window] convertScreenToBase:[sender draggedImageLocation]] toView:self]];
            return NSDragOperationPrivate;
		}

    return NSDragOperationNone;
}

- (unsigned int)draggingUpdated:(id <NSDraggingInfo>)sender
{
	NSPasteboard *pboard;
    NSDragOperation sourceDragMask;

    sourceDragMask = [sender draggingSourceOperationMask];
    pboard = [sender draggingPasteboard];

    if ([[pboard types] indexOfObject:kDockableItemBoardType] != NSNotFound)
        if (sourceDragMask & NSDragOperationPrivate)
		{
			[self prepareToDockItem:[sender draggingSource]
				  atPos:[self convertPoint:[[self window] convertScreenToBase:[sender draggedImageLocation]] toView:self]];
            return NSDragOperationPrivate;
		}

    return NSDragOperationNone;
}

- (void)draggingExited:(id <NSDraggingInfo>)sender
{
}

- (BOOL)prepareForDragOperation:(id <NSDraggingInfo>)sender
{
	return YES;
}

- (BOOL)performDragOperation:(id <NSDraggingInfo>)sender
{
	NSPoint newPos;
	
	[self retain];
	[(MIDockBar*)[[sender draggingSource] superview] removeDockableItem:[sender draggingSource]];

	newPos = [self findAppropriateLocationForItem:[sender draggingSource]
			       atPos:[self convertPoint:[[self window] convertScreenToBase:[sender draggedImageLocation]] toView:self]];
	newPos = [self insertDockableItem:[sender draggingSource] inRow:(int)newPos.y atPos:(int)newPos.y];
	[sender slideDraggedImageTo:[[self window] convertBaseToScreen:[self convertPoint:newPos toView:nil]]];			
		
	return YES;
}

- (void)concludeDragOperation:(id <NSDraggingInfo>)sender
{
}

- (NSRect)rearrangeItemsWithNewItem:(MIDockableItem*)newItem
{
	return [self frame];
}


- (NSPoint)findAppropriateLocationForItem:(MIDockableItem*)item atPos:(NSPoint)pos
{
	unsigned int	i,j;
	NSArray			*aRow;
	MIDockableItem	*anItem;
	int				addInRowNbr = 0;
	float			addAtPos = 0;
	float			rowLength;
	
	for ( i = 0 ; i < [_rows count] ; i++ )
	{
		aRow = [_rows objectAtIndex:i];
		if ( pos.y > NSMinY([[aRow objectAtIndex:0] frame]) &&
			 pos.y < NSMaxY([[aRow objectAtIndex:0] frame]) )
		{
			// Item should be placed on this row if possible, otherwise put in a new row between this one and the next
			rowLength = NSWidth([self frame]);
			
			for ( j = 0 ; j < [aRow count] ; j++ )
				rowLength -= [[aRow objectAtIndex:j] minSize].width;
			
			if ( rowLength < [item minSize].width )
			{
				addInRowNbr = i + 1;
				addAtPos = 0;
			}
			else
			{
				addInRowNbr = i;
				for ( j = 0; j < [aRow count] && addAtPos == -1 ; j++ )
				{
					anItem = [aRow objectAtIndex:j];
					if ( pos.x > NSMinX([anItem frame]) &&
					     pos.x < NSMaxX([anItem frame])+8 )
					{
						if ( pos.x < NSMidX([anItem frame]) )
							addAtPos = i;
						else
							addAtPos = i + 1;
						break;
					}
				}
			}
			break;
		}
	}
	
	return NSMakePoint(addAtPos, addInRowNbr);
}


- (NSRect)rearrangeItems
{
	NSRect			oldBarFrame, oldBarBounds;
	NSRect			tmpRect;
	NSSize			newBarSize = NSMakeSize(0,0);
	NSMutableArray	*aRow;
	NSArray			*subv;
	MIDockableItem	*anItem = nil;
	int 			oldMask;
	unsigned int	i,j;
	BOOL			barWidthCanDecrease = YES;
	float			minBarWidth = 0;
	unsigned int	resizableItemsInRow;
	BOOL			rowWidthWillDecrease = YES;
	float			totalItemsWidth = 0;
	float			extraWidthPerItem = 0;
	float			currentHPosition = 0;	
	
	// calculate the new sizes according to what items left I have
	
	// If I am the mainDockBar, only the height matters, but I calculate the width all the same
	// If an item in the dockbar is flexible, the width does not change, the flexible items taking the width left
	// and share it between them.
	
	oldBarFrame = [self frame];
	oldBarBounds = [self bounds];
	
	for ( i = 0 ; i < [_rows count] ; i++ )
	{
		// remove the emtpy rows first
		if ( [[_rows objectAtIndex:i] count] == 0 )
		{
			[_rows removeObjectAtIndex:i];
			i--;
		}
	}
	
	for ( i = 0 ; i < [_rows count] ; i++ )
	{
		// increment the height of the dockbar, reposition each item vertically accordingly
		// then calculate the horizontal position of each item, considering that unused space
		// is to be redistributed among the stretchable dockBarItems
		newBarSize.height += 8;
		aRow = [_rows objectAtIndex:i];

		rowWidthWillDecrease = YES;
		resizableItemsInRow = 0;
		totalItemsWidth = 8;
		for ( j = 0 ; j < [aRow count] ; j++ )
		{
			// position each row item's origin at a new position 
			anItem = [aRow objectAtIndex:j];

			// set the autoresizing mask to 0 to avoid problems
			[anItem setAutoresizingMask:0];
			
			// don't care about frame X for the time being
			[anItem setFrameOrigin:NSMakePoint(NSMinX([anItem frame]), newBarSize.height)];

			// count total width occupied by items, number of stretchable items, ...
			totalItemsWidth += ([anItem minSize].width + 8);
			if ( [anItem isStretchable] )
			{
				resizableItemsInRow++;
				rowWidthWillDecrease = NO;
			}
		}

		if ( !rowWidthWillDecrease )
			barWidthCanDecrease = NO;
		else
			minBarWidth = max( minBarWidth, totalItemsWidth );
		
		// calculate extra width available per item
		if ( resizableItemsInRow )
			extraWidthPerItem = ( NSWidth(oldBarFrame) - totalItemsWidth ) / resizableItemsInRow;

		// reposition each item horizontally and set their width
		for ( j = 0 ; j < [aRow count] ; j++ )
		{
			anItem = [aRow objectAtIndex:j];
			currentHPosition += 8;

			[anItem setFrameOrigin:NSMakePoint( currentHPosition, newBarSize.height )];
			if ( [anItem isStretchable] )
				[anItem setFrameSize:NSMakeSize( [anItem minSize].width + extraWidthPerItem, [anItem minSize].height)];
			else
				[anItem setFrameSize:NSMakeSize( [anItem minSize].width, [anItem minSize].height )];
			currentHPosition += NSWidth([anItem frame]);
		}
		// increase the new bar height
		newBarSize.height += NSHeight([anItem frame]);
	}
	
	if ( _isInMainView )
		barWidthCanDecrease = NO;
		
	// set the bar width
	if ( barWidthCanDecrease )
		newBarSize.width = minBarWidth;
	else
		newBarSize.width = NSWidth(oldBarFrame);
	newBarSize.height += 8; // upper border
	

	// All the dockable items are now positioned correctly, the new bar size is in the variable newBarSize
	// oldBarFrame and oldBarBounds are initialized
	
	oldMask = [self autoresizingMask];
	[self setAutoresizingMask:0];
	[self setFrameSize:newBarSize];
	if ( _isInMainView )
	{
//		tmpRect = [[[[applicationController currentMainViewController] clipSheetWindowView] enclosingScrollView] frame];
		tmpRect.size.height += NSHeight(oldBarFrame);
		tmpRect.size.height -= newBarSize.height;
		[self setFrameOrigin:NSMakePoint(0, NSMaxY(tmpRect)+1)];
//		[[[[applicationController clipSheetController] clipSheetWindowView] enclosingScrollView] setFrame:tmpRect];
	}
	else
		[[self window] setContentSize:newBarSize];
	
	subv = [self subviews];
	for ( i = 0 ; i < [subv count] ; i++ )
	{
		if ( [[subv objectAtIndex:i] isStretchable] )
			[[subv objectAtIndex:i] setAutoresizingMask:NSViewWidthSizable];
	}
	[[self window] setViewsNeedDisplay:YES];
	[[self window] display];
	
	[self setAutoresizingMask:oldMask];
	return [self frame];
}


@end