/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import <Foundation/Foundation.h>

#ifndef LibraryToggle_h
#include "LibraryToggle.h"
#endif //LibraryToggle_h

#ifndef WorkingCodec_h
#include "WorkingCodec.h"
#endif //WorkingCodec_h

@class MIObject;
@class MIMontageDocument;
@class MISnapperDB;
#if USE_LIBRARY
@class MILibraryElement;
#endif //USE_LIBRARY

class TArmeDoc;

@interface MIMarker : NSObject
{
	ArmeTime	fTime;
}

- (id)initWithArmeTime:(ArmeTime)time;
- (id)initFromArchive:(ArmeArchiveHandler*)unarchiver;
- (BOOL)writeToArchive:(ArmeArchiveHandler*)archiver;
- (void)setTime:(ArmeTime)time;
- (ArmeTime)time;

@end

enum MIFPS
{
	k24FPS,
	k25FPS,
	k30FPSNDF,
	k30FPSDF,
};


@interface MIDocumentModel : NSObject
{
@public
    TArmeDoc				*fDocPtr;
    MIObject				*fTopLevelContainer;
    NSMutableArray			*fUndoStack;
    NSMutableArray			*fRedoStack;
	NSMutableArray			*fMarkers;
    MIMontageDocument		*fDocument;
	MISnapperDB				*fSnapperDB;
    NSMutableArray			*fSavedViews;
	NSMutableArray			*fOuttaHierarchyTransitionsPool;
#if USE_LIBRARY
	MILibraryElement		*fLibraryRoot;
#endif //USE_LIBRARY
	int						fSignatureNumerator;
	int						fSignatureDenominator;
	int						fTempo;
	BOOL					fDidUnarchivingDirtifyDoc;
}


- (void)setSignatureNumerator:(int)num;
- (void)setSignatureDenominator:(int)den;
- (void)setTempo:(int)tempo;
- (int)signatureNumerator;
- (int)signatureDenominator;
- (int)tempo;

- (id)initWithDocument:(MIMontageDocument*)document andFormat:(int)format;
- (void)dealloc;

- (id)initFromArchive:(ArmeArchiveHandler*)unarchiver;
- (BOOL)writeToArchive:(ArmeArchiveHandler*)archiver;

- (void)initAllMembersToNil;
- (void)releaseAllMembers;

- (TArmeDoc*)armeDocument;

- (MIObject*)topLevelContainer;
- (void)setTopLevelContainer:(MIObject*)topLevelContainer;

#if USE_LIBRARY
- (MILibraryElement*)libraryRoot;
#endif //USE_LIBRARY

- (NSMutableArray*)undoStack;
- (NSMutableArray*)redoStack;
- (NSMutableArray*)savedViews;
- (NSMutableArray*)outtaHierarchyTransitionsPool;

- (void)setDocument:(MIMontageDocument*)document;

// marker management
- (UInt32)addMarkerAtTime:(ArmeTime)marker;
- (void)removeMarker:(UInt32)marker;
- (void)moveMarker:(UInt32)marker toTime:(ArmeTime)newTime;
- (UInt32)getMarkerCount;
- (ArmeTime)getMarker:(UInt32)i;

// this is called by the parent MIMontageDocument, after unarchiving, to check 
// whether unarchiving did dirtify the doc. This can happen when audio files have
// been moved and the user has been prompted for a new location
- (BOOL)didUnarchivingDirtifyDoc;

@end

