/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MIMontageDocument.h"
#import "MIDocumentModel.h"
#import "MIApplicationController.h"
#import "MISnapperDB.h"

#ifndef LibraryToggle_h
#include "LibraryToggle.h"
#endif //LibraryToggle_h

#if USE_LIBRARY
#import "MILibraryElement.h"
#endif //USE_LIBRARY

#import "MIStartingPoint.h"
#import "MIViewerController.h"
#import "MIMainViewController.h"


#ifndef TArmeDoc_h
#include "TArmeDoc.h"
#endif //TArmeDoc_h

#ifndef ZzTypes_h
#include "ZzTypes.h"
#endif //ZzTypes_h

#ifndef TArmeMediaEngine_h
#include "TArmeMediaEngine.h"
#endif //TArmeMediaEngine_h

#ifndef ZzException_h
#include "ZzException.h"
#endif //ZzException_h

@implementation MIMarker

- (id)initWithArmeTime:(ArmeTime)time
{
	self = [super init];
	if(self)
	{
		fTime = time;
	}
	return self;
}
	
- (id)initFromArchive:(ArmeArchiveHandler*)unarchiver
{
	verntl(archivingEngine,6,"MIMarker initFromArchive");
	self = [super initFromArchive:unarchiver];
	if(self)
	{
		fTime = unarchiver->ReadArmeTime();
		verntl(archivingEngine,6,"MIMarker initFromArchive fTime = " << fTime);
	}
	return self;
}

- (BOOL)writeToArchive:(ArmeArchiveHandler*)archiver
{
	verntl(archivingEngine,6,"MIMarker writeToArchive");
	[super writeToArchive:archiver];
	archiver->WriteArmeTime(fTime);
	return YES;
}

- (void)setTime:(ArmeTime)time
{
	fTime = time;
}

- (ArmeTime)time
{
	return fTime;
}

@end

@implementation MIDocumentModel

- (id)retain
{
	id Self = [super retain];
	return Self;
}
	
- (void)release
{
	[super release];
}

- (id)initWithDocument:(MIMontageDocument*)document andFormat:(int)format
{
    if (( self = [super init] )) 
	{
		NSDictionary *attributes;
		fDocument = document ;

		NSString *filename = ((MIStartingPoint*)[MIStartingPoint startingPoint])->fFileName;
		BOOL hideExtention = ((MIStartingPoint*)[MIStartingPoint startingPoint])->fHideExtension;

		NSString* path = [filename stringByAppendingPathExtension:@"HAV"];
			
		attributes = [NSDictionary dictionaryWithObject:[NSNumber numberWithBool:hideExtention] forKey:NSFileExtensionHidden];
							 
		if ( ![[NSFileManager defaultManager] createDirectoryAtPath:path attributes:attributes] )
		{
			vernl(6,"Unable to create folder... Aborting creation of new document\n\r");
			[self release];
			return nil;
		}
		try
		{
			fDocPtr = new TArmeDoc([path fileSystemRepresentation],true, (WorkingCodecType)format);	
			if ( !fDocPtr )
			{
				vernl(6,"Null Arme document returned... Aborting creation of new document\n\r");
				[self release];
				return nil;
			}
		}
		catch (ZzException& e)
		{
			e.DefaultCatcher();
			[self release];
			return nil;
		}

		[fDocument setFileName:path];
		
		fTopLevelContainer = nil;
	
		fUndoStack		= [[NSMutableArray alloc] initWithObjects:nil];
		fRedoStack		= [[NSMutableArray alloc] initWithObjects:nil];
		fMarkers 		= [[NSMutableArray alloc] initWithObjects:nil];
        fSavedViews 	= [[NSMutableArray alloc] initWithObjects:nil];
		fOuttaHierarchyTransitionsPool = [[NSMutableArray alloc] initWithObjects:nil];
		
#if USE_LIBRARY
		fLibraryRoot = [[MILibraryElement alloc] initWithObject:nil];
#endif //USE_LIBRARY

		fSignatureNumerator = 4;
		fSignatureDenominator = 4;
		fTempo = 120;
		//fFPS = k30FPSNDF;
    }
    return self ;
}

- (void)initAllMembersToNil
{
	fDocPtr = NULL;
	fTopLevelContainer = NULL;
	fUndoStack = nil;
	fRedoStack = nil;
	fMarkers = nil;
	fSavedViews = nil;
	fOuttaHierarchyTransitionsPool = nil;
#if USE_LIBRARY
	fLibraryRoot = nil;
#endif// USE_LIBRARY
	fDidUnarchivingDirtifyDoc = NO;
}

- (void)releaseAllMembers
{
	vernl(6,"[MIDocumentModel dealloc]");
    [fUndoStack release];
    [fRedoStack release];
	[fMarkers release];
    [fSavedViews release];
	[fOuttaHierarchyTransitionsPool release];
#ifdef DEBUG
//	printf("about to release fTopLevelContainer in [MIDocumentModel releaseAllMembers]\n");
#endif //#ifdef DEBUG
	[fTopLevelContainer release]; // retain in setTopLevelContainer

/*	if (fDocPtr)  //might be null if there was an exception during the creation
		fDocPtr->DeleteUnusedFiles(); done somewhere else */
	
	//ArmeDocument MUST be deleted last (because other objects like command have references on ArmeDoc so it needs to be there when they are deleted)
    delete fDocPtr;
}

- (id)initFromArchive:(ArmeArchiveHandler*)unarchiver {

    if (( self = [super initFromArchive:unarchiver] ))
    {
		[self initAllMembersToNil];
	
		try
		{
			ArmeArchivable* archObj = unarchiver->ReadCPPObject();
			fDocPtr = dynamic_cast<TArmeDoc*>(archObj);
			
			if (fDocPtr->GetAudioChunkFileManagerDidDirtifyDoc())
				fDidUnarchivingDirtifyDoc = YES;
			
			ZzAssert(fDocPtr); // can't be NULL
	
			fTopLevelContainer = [unarchiver->ReadObjCObject() retain];
			fUndoStack = [unarchiver->ReadObjCObject() retain];
			fRedoStack = [unarchiver->ReadObjCObject() retain];
			if (fUndoStack == nil)
			{
				fUndoStack = [[NSMutableArray alloc] initWithObjects:nil];
			}
			if (fRedoStack == nil)
			{
				fRedoStack = [[NSMutableArray alloc] initWithObjects:nil];
			}
			fMarkers = [unarchiver->ReadObjCObject() retain];
			fSavedViews = [unarchiver->ReadObjCObject() retain];
			fOuttaHierarchyTransitionsPool = [unarchiver->ReadObjCObject() retain];
						
	#if USE_LIBRARY
			fLibraryRoot = [unarchiver->ReadObjCObject() retain];
			if(!fLibraryRoot) 
			{
				// apps compiled with "USE_LIBRARY = 0" will write NULL here instead
				// so if we save a doc with a post 0x10002 version compiled with "USE_LIBRARY = 0"
				// and read it with a post 0x10002 version with "USE_LIBRARY = 1", we need to recreate
				// the library.
				fLibraryRoot = [[MILibraryElement alloc] initWithObject:nil];
			}
	#else
			#pragma unused (dummyLibWeDontUseIt)
			//id dummyLibWeDontUseIt = unarchiver->ReadObjCObject(); // we don't release it since it can safely be deleted....
			unarchiver->ReadObjCObject(); // we don't release it since it can safely be deleted....
			// if the saved doc was using USE_LIBRARY = 1 then a real object will be deleted (the lib root) otherwise nil
			// will be released which is ok.
	#endif //USE_LIBRARY
	
			// not in use
			fSignatureNumerator = unarchiver->ReadSInt32();
			fSignatureDenominator = unarchiver->ReadSInt32();
			fTempo = unarchiver->ReadSInt32();
	
			// obsolete
			unarchiver->ReadSInt32();
			unarchiver->ReadSInt32();
		}
		catch(ZzException* e)
		{
			[self releaseAllMembers];
			return nil;
		}
		catch(...)
		{
			[self releaseAllMembers];
			return nil;
		}
    }
    return self;
}

- (void)dealloc 
{
	[self releaseAllMembers];
    [super dealloc];
	//ZzAssert(TArmeMediaEngine::GetNbDocs() == 0); ///$$$TODO REMOVE REMOVE REMOVE REMOVE REMOVE
}

- (BOOL)writeToArchive:(ArmeArchiveHandler*)archiver {
	try
	{
		[super writeToArchive:archiver];
		archiver->WriteCPPObject(fDocPtr);
		archiver->WriteObjCObject(fTopLevelContainer);
		archiver->WriteObjCObject(nil); //archiver->WriteObjCObject(fUndoStack);
		archiver->WriteObjCObject(nil); //archiver->WriteObjCObject(fRedoStack);
		archiver->WriteObjCObject(fMarkers);
		archiver->WriteObjCObject(fSavedViews);
		archiver->WriteObjCObject(fOuttaHierarchyTransitionsPool);
		
	#if USE_LIBRARY
		$$$
		archiver->WriteObjCObject(fLibraryRoot);
	#else
		archiver->WriteObjCObject(nil);
	#endif //USE_LIBRARY
	
		// not in use
		archiver->WriteSInt32(fSignatureNumerator);
		archiver->WriteSInt32(fSignatureDenominator);
		archiver->WriteSInt32(fTempo);
	
		// obsolete
		archiver->WriteSInt32(0);
		archiver->WriteSInt32(0);
	
//		fDocPtr->DeleteUnusedFiles();  //done somewhere else
	
		return true;
	}
	catch (ZzException& e)
	{
		e.DefaultCatcher();
		return false;
	}
}

- (TArmeDoc*)armeDocument {
    return fDocPtr ;
}

- (MIObject*)topLevelContainer {
    return fTopLevelContainer;
}

- (void)setTopLevelContainer:(MIObject*)topLevelContainer 
{
    fTopLevelContainer = [topLevelContainer retain];
}

- (void)setSignatureNumerator:(int)num
{
	fSignatureNumerator = num;
}

- (void)setSignatureDenominator:(int)den
{
	fSignatureDenominator = den;
}

- (void)setTempo:(int)tempo
{
	fTempo = tempo;
}

- (int)signatureNumerator
{
	return fSignatureNumerator;
}

- (int)signatureDenominator
{
	return fSignatureDenominator;
}

- (int)tempo
{
	return fTempo;
}

#if USE_LIBRARY
- (MILibraryElement*)libraryRoot
{
	return fLibraryRoot;
}
#endif //USE_LIBRARY

- (NSMutableArray*)undoStack {
    return fUndoStack;
}

- (NSMutableArray*)redoStack {
    return fRedoStack;
}

- (NSMutableArray*)savedViews {
    return fSavedViews;
}

- (NSMutableArray*)outtaHierarchyTransitionsPool {
    return fOuttaHierarchyTransitionsPool;
}

- (void)setDocument:(MIMontageDocument*)document {
    fDocument = document ;
}

- (BOOL)didUnarchivingDirtifyDoc
{
	return fDidUnarchivingDirtifyDoc;
}

- (UInt32)addMarkerAtTime:(ArmeTime)time
{
	[fMarkers addObject:[[[MIMarker alloc] initWithArmeTime:time] autorelease]];
	return [fMarkers count]-1;
}

- (void)removeMarker:(UInt32)marker
{
	[fMarkers removeObjectAtIndex:marker];
}

- (void)moveMarker:(UInt32)marker toTime:(ArmeTime)newTime
{
	[[fMarkers objectAtIndex:marker] setTime:newTime];
}

- (UInt32)getMarkerCount
{
	return [fMarkers count];
}

- (ArmeTime)getMarker:(UInt32)i
{
	return [[fMarkers objectAtIndex:i] time];
}

@end
