/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MIDuplicateManyController.h"
#import "MIPreferences.h"

@implementation MIDuplicateManyController

- (void)resetFields:(BOOL)fullDialog {
	fSMTPView = [[MIPreferences sharedPrefs] timeDisplay] & kTimeDisplaySMPTE ;
	
    [fDuplications setIntValue:1];
	
    [fHorizontalEvery setState:NSOffState];
    [fHorizontalImmediately setState:NSOnState];
	[fVerticalImmediately setState:NSOffState];
    [fVerticalNone setState:NSOnState];

    //[fKeepInside setState:NSOnState];
	[fHour setStringValue:@""];
    [fMinute setStringValue:@""];
    [fSecond setStringValue:@""];
    [fFrame setStringValue:@""];

	[fBar setStringValue:@""];
    [fBeat setStringValue:@""];
    [fSixteenth setStringValue:@""];
    [fPPQ setStringValue:@""];

	[fBPMView removeFromSuperview];
	[fSMPTEView removeFromSuperview];

	if ( fSMTPView ) {
		[fSuperview addSubview:fSMPTEView];
	} else {
		[fSuperview addSubview:fBPMView];
	}


    [fHorizontalEvery setEnabled:fullDialog];
    [fHorizontalImmediately setEnabled:fullDialog];
	[fVerticalImmediately setEnabled:fullDialog];
    [fVerticalNone setEnabled:fullDialog];
	[fHour setEnabled:fullDialog];
    [fMinute setEnabled:fullDialog];
    [fSecond setEnabled:fullDialog];
    [fFrame setEnabled:fullDialog];

}

+ (id)duplicateManyPanel:(BOOL)fullDialog {
	static id duplicateManyPanel = nil;
	if ( !duplicateManyPanel ) {
		duplicateManyPanel = [[self alloc] init];
	}
	[duplicateManyPanel resetFields:fullDialog];
	return duplicateManyPanel;
}

- (id)init {
    if (( self = [super initWithWindowNibName:@"DuplicateMany"] )) {
		[self window];
		[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(textDidBeginEditing:) name:NSControlTextDidBeginEditingNotification object:fHour];
		[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(textDidBeginEditing:) name:NSControlTextDidBeginEditingNotification object:fMinute];
		[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(textDidBeginEditing:) name:NSControlTextDidBeginEditingNotification object:fSecond];
		[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(textDidBeginEditing:) name:NSControlTextDidBeginEditingNotification object:fFrame];
		[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(textDidBeginEditing:) name:NSControlTextDidBeginEditingNotification object:fBar];
		[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(textDidBeginEditing:) name:NSControlTextDidBeginEditingNotification object:fBeat];
		[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(textDidBeginEditing:) name:NSControlTextDidBeginEditingNotification object:fSixteenth];
		[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(textDidBeginEditing:) name:NSControlTextDidBeginEditingNotification object:fPPQ];
		[fBPMView retain];
		[fSMPTEView retain];
    }
    return self;
}

- (void)dealloc {
	[[NSNotificationCenter defaultCenter] removeObserver:self name:NSControlTextDidBeginEditingNotification object:fHour];
	[[NSNotificationCenter defaultCenter] removeObserver:self name:NSControlTextDidBeginEditingNotification object:fMinute];
	[[NSNotificationCenter defaultCenter] removeObserver:self name:NSControlTextDidBeginEditingNotification object:fSecond];
	[[NSNotificationCenter defaultCenter] removeObserver:self name:NSControlTextDidBeginEditingNotification object:fFrame];
	[[NSNotificationCenter defaultCenter] removeObserver:self name:NSControlTextDidBeginEditingNotification object:fBar];
	[[NSNotificationCenter defaultCenter] removeObserver:self name:NSControlTextDidBeginEditingNotification object:fBeat];
	[[NSNotificationCenter defaultCenter] removeObserver:self name:NSControlTextDidBeginEditingNotification object:fSixteenth];
	[[NSNotificationCenter defaultCenter] removeObserver:self name:NSControlTextDidBeginEditingNotification object:fPPQ];
	[fBPMView release];
	[fSMPTEView release];

    [super dealloc];
}

- (int)runModal {
	return [NSApp runModalForWindow:[self window]];
}


- (IBAction)cancelDuplicate:(id)sender {
	[NSApp stopModalWithCode:NSCancelButton];
	[[self window] orderOut:nil];
}

- (IBAction)duplicate:(id)sender {
	[NSApp stopModalWithCode:NSOKButton];
	[[self window] orderOut:nil];
}

- (BOOL)horizontalEvery {
	if ( [fHorizontalEvery state] == NSOnState ) return YES;
	else return NO;
}

- (BOOL)horizontalImmediately {
	if ( [fHorizontalImmediately state] == NSOnState ) return YES;
	else return NO;
}

- (BOOL)verticalImmediately {
	if ( [fVerticalImmediately state] == NSOnState ) return YES;
	else return NO;
}

- (BOOL)verticalNone {
	if ( [fVerticalNone state] == NSOnState ) return YES;
	else return NO;
}

- (BOOL)shouldKeepInsideContainer {
	/*if ( [fKeepInside state] == NSOnState ) return YES;
	else return NO;*/
	return YES;
}

- (int)numberOfDuplications {
	return [fDuplications intValue];
}

- (void)textDidBeginEditing:(NSNotification *)aNotification {
	[fHorizontalEvery setState:NSOnState];
	[fHorizontalImmediately setState:NSOffState];
}

- (MITime*)time {
	if ( fSMTPView ) {
		return [[[MITime alloc] initWithH:[fHour intValue] m:[fMinute intValue] s:[fSecond intValue] vFr:[fFrame intValue] ms:0 inDocument:[[NSDocumentController sharedDocumentController] currentDocument]] autorelease];
	}
	else {
		MITime *time = [[[MITime alloc] initWithDocument:[[NSDocumentController sharedDocumentController] currentDocument]] autorelease];
		[time setBars:[fBar intValue] beats:[fBeat intValue] nb16ths:[fSixteenth intValue] ppq:[fPPQ intValue]];
		return time;
	}
}

@end
