/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MIExportPanel.h"

#define MIOKButton 1
#define MICancelButton 0

@implementation MIExportPanel


- (id)init {
    if (( self = [super initWithWindowNibName:@"MIExportPanel"] )) 
	{
		[self window];
		
		//by default, export to camera so, grey out what needs to be
		[self setExportCodec:MIWorkingCodec];
		[fWorkingCodec setEnabled:NO];
		[fChooseCodec setEnabled:NO];
    }
    return self;
}

- (void)dealloc {

	[super dealloc];
}

- (int)runModal {
	return [NSApp runModalForWindow:[self window]];
}

//[[fDestination selectedCell] tag]

- (IBAction)destinationChanged:(id)sender
{
  if ( [fExportDestination selectedCell] == fCamera )
    [self setDestinationToCamera:YES];
  else
    [self setDestinationToCamera:NO];
}

- (IBAction)codecChanged:(id)sender
{
  if ( [fExportCodec selectedCell] == fChooseCodec  )
    [self setExportCodec:MIChooseCodec];
  else
    [self setExportCodec:MIWorkingCodec];
}

- (IBAction)selectionChanged:(id)sender
{
  if ( [fExportSetting selectedCell] == fExportWholeDocument)
    [self setExportSetting:MIExportWholeDocument];
  else
    [self setExportSetting:MIExportSelection];
}

// $$$ALAIN appelle cette fonction pour savoir si c'est camra qui est slection
- (BOOL)destinationIsCamera
{
  if ( [fCamera state] == NSOnState )
    return YES;
  else
    return NO;
}

- (IBAction)ok:(id)sender {
	[NSApp stopModalWithCode:MIOKButton];
	[[self window] orderOut:sender];
}

- (IBAction)cancel:(id)sender {
	[NSApp stopModalWithCode:MICancelButton];
	[[self window] orderOut:sender];
}

- (void)setExportSetting:(MIExportSetting)exportSetting {
	if (fSelectionContent==MIEmpty)
	{
		[fExportWholeDocument setState:NSOnState];
		[fExportSelection setState:NSOffState];
		[fExportSelection setEnabled:NO];
		exportSetting = MIExportWholeDocument;
	}
	if ( MIExportWholeDocument == exportSetting ) {
		[fExportWholeDocument setState:NSOnState];
		[fExportSelection setState:NSOffState];
		switch(fDocumentContent)
		{
			case MIOnlyAudio :
			case MINoVideo :
				[self setExportCodec:MIChooseCodec];
				[fChooseCodec setEnabled:YES];
				[fWorkingCodec setEnabled:NO];

				[self setDestinationToCamera:NO];
				[fCamera setEnabled:NO];
				break;
			case MIVideo : 
				//leave as is 
				[fCamera setEnabled:YES];
				[fWorkingCodec setEnabled:YES];
				if ([fCamera state] == NSOnState)
					[fChooseCodec setEnabled:NO];
				else
					[fChooseCodec setEnabled:YES];
				break;
			case MIEmpty :
				break;
		};	
	} else {
		[fExportWholeDocument setState:NSOffState];
		[fExportSelection setState:NSOnState];
		switch(fSelectionContent)
		{
			case MIOnlyAudio :
			case MINoVideo :
				[self setExportCodec:MIChooseCodec];
				[fChooseCodec setEnabled:YES];
				[fWorkingCodec setEnabled:NO];

				[self setDestinationToCamera:NO];
				[fCamera setEnabled:NO];
				break;
			case MIVideo : 
				//leave as is 
				[fCamera setEnabled:YES];
				[fWorkingCodec setEnabled:YES];
				if ([fCamera state] == NSOnState)
					[fChooseCodec setEnabled:NO];
				else
					[fChooseCodec setEnabled:YES];
				break;
			case MIEmpty :
				break;
		};	
	}
//	[self setExportCodec:MIAutoCodec];
}

- (void)setDestinationToCamera:(BOOL)camera {
	if (camera) {
		[fCamera setState:NSOnState];
		[fFile setState:NSOffState];
		[self setExportCodec:MIWorkingCodec];
		[fChooseCodec setEnabled:NO];
	} else {
		[fCamera setState:NSOffState];
		[fFile setState:NSOnState];
		[self setExportCodec:MIAutoCodec];
	}
}

- (MIExportSetting)exportSetting {
	if ( [fExportWholeDocument state] == NSOnState ) return MIExportWholeDocument;
	else return MIExportSelection;
}

- (void)setExportCodec:(MIExportCodec)exportCodec {
	if ( MIWorkingCodec == exportCodec ) {
		[fWorkingCodec setState:NSOnState];
		[fChooseCodec setState:NSOffState];
	} else if ( MIChooseCodec == exportCodec )
	{
		[fWorkingCodec setState:NSOffState];
		[fChooseCodec setState:NSOnState];
	} else { //auto
		if ([self exportSetting] == MIExportSelection)
		{
			switch(fSelectionContent)
			{
				case MIOnlyAudio :
				case MINoVideo :
					[fChooseCodec setEnabled:YES];
					[fWorkingCodec setEnabled:NO];
					[self setExportCodec:MIChooseCodec];
					break;
				case MIVideo : 
					//leave as is 
					[fWorkingCodec setEnabled:YES];
					if ([self destinationIsCamera])
						[fChooseCodec setEnabled:NO];
					else
						[fChooseCodec setEnabled:YES];
					break;

				case MIEmpty :
					break;
			};
		}
		else
		{
			switch(fDocumentContent)
			{
				case MIOnlyAudio :
				case MINoVideo :
					[fChooseCodec setEnabled:YES];
					[fWorkingCodec setEnabled:NO];
					[self setExportCodec:MIChooseCodec];
					break;
				case MIVideo : 
					//leave as is 
					[fWorkingCodec setEnabled:YES];
					if ([self destinationIsCamera])
						[fChooseCodec setEnabled:NO];
					else
						[fChooseCodec setEnabled:YES];
					break;
				case MIEmpty :
					break;
			};
		}
	}
}

- (MIExportCodec)exportCodec {
	if ( [fWorkingCodec state] == NSOnState ) return MIWorkingCodec;
	else return MIChooseCodec;
}

- (void)setSelectionContent:(MIExportContent)selContent
{fSelectionContent = selContent;}
- (void)setDocumentContent:(MIExportContent)docContent
{fDocumentContent = docContent;}

@end

bool MIRunExportPanel( MIExportSetting *exportSetting, MIExportCodec *exportCodec, bool* exportToCamera, MIExportContent selection, MIExportContent document ) {
	MIExportPanel* panel = [[MIExportPanel alloc] init];
	bool returnCode = false;

	[panel setSelectionContent:selection];
	[panel setDocumentContent:document];

	[panel setExportCodec:*exportCodec];
	[panel setDestinationToCamera:*exportToCamera];
	[panel setExportSetting:*exportSetting];

	if ( MIOKButton == [panel runModal] ) {
		*exportSetting = [panel exportSetting];
		*exportCodec = [panel exportCodec];
		*exportToCamera = [panel destinationIsCamera];
		returnCode = true;
	}
	[panel release];
	return returnCode;
}
