/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MIExportRAWAudioController.h"

#ifndef TArmeDoc_h
#include "TArmeDoc.h"
#endif //TArmeDoc_h

#ifndef ME_AudioExportFunctions_h
#include "ME_AudioExportFunctions.h"
#endif //ME_AudioExportFunctions_h

#ifndef ZzHelpers_h
#include "ZzHelpers.h"
#endif //ZzHelpers_h

#import "MIMontageDocument.h"
#import "MIWSSelection.h"
#import "MIObject.h"
#import "MIApplication.h"
#import "MIMainViewController.h"

#ifndef TArmeWSObj_h
#include "TArmeWSObj.h"
#endif //TArmeWSObj_h

#ifndef Zz_PathHelpers_h
#include "Zz_PathHelpers.h"
#endif //Zz_PathHelpers_h

@implementation MIExportRAWAudioController

- (int)displaySavePanel
{
	NSSavePanel* savePanel = [NSSavePanel savePanel];
	[savePanel setAccessoryView:fAccessoryView];
	
	// enforce the type selected according to the 
	// fileType.
	switch(fFileType)
	{
	case kMEExportAIFF :
		switch(fSampleType)
		{
		case kMEExport16Bits :
			[savePanel setRequiredFileType:@"AIFF"];
			break;
		default :
			[savePanel setRequiredFileType:@"AIFC"];
			break;
		};
		break;
	case kMEExportWAVE :
		[savePanel setRequiredFileType:@"WAV"];
		break;
	};
	
	// read the initial directory off the prefs
	NSString* directoryName = [self retrieveDirectoryPreference];
	
	int retCode = [savePanel runModalForDirectory:directoryName file:fInitialFileName];
	if(retCode != NSFileHandlingPanelOKButton)
		return retCode;
	
	// read the directory in the prefs
	[self storeDirectoryPreference:[savePanel directory]];
	
	fFilePath = [savePanel filename];
	[self retrieveValueFromControls];
	[savePanel setAccessoryView:nil];
	[fAccessoryView release];
	fAccessoryView = nil;
	return retCode;
}

- (void)setInitialFileName:(NSString*)fileName
{
	fInitialFileName = fileName;
}

- (void)setSelectionContainsSeveralObjects:(BOOL)several
{
	if(several)
	{
		[fMultipleFilesRadioButton setEnabled:YES];
	}
	else
	{
		[fSingleFileRadioButton setState:NSOnState];
		[fMultipleFilesRadioButton setState:NSOffState];
		[fMultipleFilesRadioButton setEnabled:NO];
	}
}

- (void)setAllowIncludeAllFX:(BOOL)allow
{
	if(!allow)
		[fIncludeAllFXCheckBox setState:NSOffState];
	[fIncludeAllFXCheckBox setEnabled:allow];
}

- (void)setAllowExportSelection:(BOOL)allow
{
	if(allow)
	{
		[fExportWholeDocRadioButton setEnabled:YES];
		[fExportSelectionRadioButton setEnabled:YES];
		[fExportWholeDocRadioButton setState:NSOffState];
		[fExportSelectionRadioButton setState:NSOnState];
		//[fIncludeAllFXCheckBox setState:NSOnState];
		[fIncludeAllFXCheckBox setEnabled:YES];
	}
	else
	{
		[fExportWholeDocRadioButton setEnabled:YES];
		[fExportSelectionRadioButton setEnabled:NO];
		[fExportWholeDocRadioButton setState:NSOnState];
		[fExportSelectionRadioButton setState:NSOffState];
		[fIncludeAllFXCheckBox setState:NSOnState];
		[fIncludeAllFXCheckBox setEnabled:NO];
		
		// only allow exporting as a single file
		[fSingleFileRadioButton setState:NSOnState];
		[fMultipleFilesRadioButton setState:NSOffState];
		[fMultipleFilesRadioButton setEnabled:NO];
	}
}

- (void)storeDirectoryPreference:(NSString*)directory
{
	[[NSUserDefaults standardUserDefaults] setObject:directory forKey:@"RAE_directory"];
}

- (NSString*)retrieveDirectoryPreference
{
	NSString* directory = [[NSUserDefaults standardUserDefaults] objectForKey:@"RAE_directory"];
	if(directory)
	{
		return directory;
	}
	else
	{
		return NSHomeDirectory();
	}
}

- (void)retrieveValueFromControls
{
/*
	NSString*	fFilePath; //done in the caller
	BOOL		fShouldIncludeAllFX;
*/
	switch([fNbChannelsPopup indexOfSelectedItem])
	{
	case 0 :
		fNbChannels = 2;
		break;
	case 1 :
		fNbChannels = 1;
		break;
	case 2 :
		fNbChannels = 0;
		break;
	default :
		ZZ_THROW_1ARG(ZzGenericException,"wrong [fNbChannelsPopup indexOfSelectedItem]");
		break;
	}
		
/*
	[fFileFormatPopup addItemWithTitle:@"AIFF 8 bit"];
	[fFileFormatPopup addItemWithTitle:@"AIFF 16 bit"];
	[fFileFormatPopup addItemWithTitle:@"AIFF 24 bit"];
	[fFileFormatPopup addItemWithTitle:@"AIFF 32 bit float"];
	[fFileFormatPopup addItemWithTitle:@"WAV 8 bit"];
	[fFileFormatPopup addItemWithTitle:@"WAV 16 bit"];
	[fFileFormatPopup addItemWithTitle:@"WAV 24 bit"];
*/
	switch([fFileFormatPopup indexOfSelectedItem])
	{
	case 0 :
		fFileType = kMEExportAIFF;
		fSampleType = kMEExport8Bits;
		break;
	case 1 :
		fFileType = kMEExportAIFF;
		fSampleType = kMEExport16Bits;
		break;
	case 2 :
		fFileType = kMEExportAIFF;
		fSampleType = kMEExport24Bits;
		break;
	case 3 :
		fFileType = kMEExportAIFF;
		fSampleType = kMEExport32BitsFloat;
		break;
	case 4 :
		fFileType = kMEExportWAVE;
		fSampleType = kMEExport8Bits;
		break;
	case 5 :
		fFileType = kMEExportWAVE;
		fSampleType = kMEExport16Bits;
		break;
	case 6 :
		fFileType = kMEExportWAVE;
		fSampleType = kMEExport24Bits;
		break;
	default :
		ZZ_THROW_1ARG(ZzGenericException,"wrong [v indexOfSelectedItem]");
		break;
	}
	
/*
	[fSampleRatePopup addItemWithTitle:@"22.05 Khz"]; //0
	[fSampleRatePopup addItemWithTitle:@"32 Khz"]; //1
	[fSampleRatePopup addItemWithTitle:@"44.1 Khz"]; //2
	[fSampleRatePopup addItemWithTitle:@"48 Khz"]; //3
	[fSampleRatePopup addItemWithTitle:@"64 Khz"]; //4
	[fSampleRatePopup addItemWithTitle:@"88.2 Khz"]; //5
	[fSampleRatePopup addItemWithTitle:@"96 Khz"]; //6
	[fSampleRatePopup addItemWithTitle:@"192 Khz"]; //7
*/	
	switch([fSampleRatePopup indexOfSelectedItem])
	{
	case 0 :
		fAudioFrameRate = 22050;
		break;
	case 1 :
		fAudioFrameRate = 32000;
		break;
	case 2 :
		fAudioFrameRate = 44100;
		break;
	case 3 :
		fAudioFrameRate = 48000;
		break;
	case 4 :
		fAudioFrameRate = 64000;
		break;
	case 5 :
		fAudioFrameRate = 88200;
		break;
	case 6 :
		fAudioFrameRate = 96000;
		break;
	case 7 :
		fAudioFrameRate = 192000;
		break;
	default :
		ZZ_THROW_1ARG(ZzGenericException,"wrong [fSampleRatePopup indexOfSelectedItem]");
		break;
	}
		
/*
	switch([fSampleFormatPopup indexOfSelectedItem])
	{
	case 0 :
		fSampleType = kMEExport8Bits;
		break;
	case 1 :
		fSampleType = kMEExport16Bits;
		break;
	case 2 :
		fSampleType = kMEExport24Bits;
		break;
	case 3 :
		fSampleType = kMEExport32BitsFloat;
		break;
	default :
		ZZ_THROW_1ARG(ZzGenericException,"wrong [fSampleFormatPopup indexOfSelectedItem]");
		break;
	};
	
	switch([fFileTypePopup indexOfSelectedItem])
	{
	case 0:
		fFileType = kMEExportWAVE;
		break;
	case 1:
		fFileType = kMEExportAIFF;
		break;
	default :
		ZZ_THROW_1ARG(ZzGenericException,"wrong [fFileTypePopup indexOfSelectedItem]");
		break;
	};
*/
	if([fExportWholeDocRadioButton state] == NSOnState)
	{
		ZzAssert([fExportSelectionRadioButton state] == NSOffState);
		fShouldExportSelection = false;
	}
	if([fExportSelectionRadioButton state] == NSOnState)
	{
		ZzAssert([fExportWholeDocRadioButton state] == NSOffState);
		fShouldExportSelection = true;
	}
		
	if([fSingleFileRadioButton state] == NSOnState)
	{
		ZzAssert([fMultipleFilesRadioButton state] == NSOffState);
		fShouldExportAsMultipleFiles = false;
	}
	if([fMultipleFilesRadioButton state] == NSOnState)
	{
		ZzAssert([fSingleFileRadioButton state] == NSOffState);
		fShouldExportAsMultipleFiles = true;
	}
	
	if([fIncludeAllFXCheckBox state] == NSOnState)
	{
		fShouldIncludeAllFX = YES;
	}
	else
	{
		fShouldIncludeAllFX = NO;
	}
	
	if([fUseOriginalNamesCheckBox state] == NSOnState)
		fShouldUseOriginalNames = YES;
	else
		fShouldUseOriginalNames = NO;
}

- (void)setNbChannels:(int)nbChannels
{
	[fNbChannelsPopup removeAllItems];
	[fNbChannelsPopup addItemWithTitle:@"Stereo"];
	[fNbChannelsPopup addItemWithTitle:@"Mono"];
	[fNbChannelsPopup addItemWithTitle:@"Default"];
	switch(nbChannels)
	{
	case 0 :
		[fNbChannelsPopup selectItemAtIndex:2];
		break;
	case 1 :
		[fNbChannelsPopup selectItemAtIndex:1];
		break;
	case 2 :
		[fNbChannelsPopup selectItemAtIndex:0];
		break;
	default :
		ZZ_THROW_1ARG(ZzGenericException,"wrong nbChannels in fillControls");
		break;
	};
}

- (void)fillControls:(TArmeDoc*)doc sampleType:(UInt32)sampleType fileType:(UInt32)fileType
{
/*	[fSampleFormatPopup removeAllItems];
	[fSampleFormatPopup addItemWithTitle:@"8 bit"];
	[fSampleFormatPopup addItemWithTitle:@"16 bit"];
	[fSampleFormatPopup addItemWithTitle:@"24 bit"];
	[fSampleFormatPopup addItemWithTitle:@"32 bit float"];
			
	switch(sampleType)
	{
	case kMEExport8Bits :
		[fSampleFormatPopup selectItemAtIndex:0];
		break;
	case kMEExport16Bits :
		[fSampleFormatPopup selectItemAtIndex:1];
		break;
	case kMEExport24Bits :
		[fSampleFormatPopup selectItemAtIndex:2];
		break;
	case kMEExport32BitsFloat :
		[fSampleFormatPopup selectItemAtIndex:3];
		break;
	default :
		ZZ_THROW_1ARG(ZzGenericException,"wrong sampleType in fillControls");
		break;
	};

	[fFileTypePopup removeAllItems];
	[fFileTypePopup addItemWithTitle:@".WAV"];
	[fFileTypePopup addItemWithTitle:@".AIFF/.AIFC"];
	switch(fileType)
	{
	case kMEExportAIFF :
		[fFileTypePopup selectItemAtIndex:1];
		break;
	case kMEExportWAVE :
		[fFileTypePopup selectItemAtIndex:0];
		break;
	default :
		ZZ_THROW_1ARG(ZzGenericException,"wrong doc->GetAudioFrameRate() in fillControls");
		break;
	};

*/	

	[fFileFormatPopup removeAllItems];
	[fFileFormatPopup addItemWithTitle:@"AIFF 8 bit"];
	[fFileFormatPopup addItemWithTitle:@"AIFF 16 bit"];
	[fFileFormatPopup addItemWithTitle:@"AIFF 24 bit"];
	[fFileFormatPopup addItemWithTitle:@"AIFF 32 bit float"];
	[fFileFormatPopup addItemWithTitle:@"WAV 8 bit"];
	[fFileFormatPopup addItemWithTitle:@"WAV 16 bit"];
	[fFileFormatPopup addItemWithTitle:@"WAV 24 bit"];

	int index;
	switch(fileType)
	{
	case kMEExportAIFF :
		index = 0;
		break;
	case kMEExportWAVE :
		index = 4;
		break;
	default :
		ZZ_THROW_1ARG(ZzGenericException,"wrong doc->GetAudioFrameRate() in fillControls");
		break;
	};
	switch(sampleType)
	{
	case kMEExport8Bits :
		index += 0;
		break;
	case kMEExport16Bits :
		index += 1;
		break;
	case kMEExport24Bits :
		index += 2;
		break;
	case kMEExport32BitsFloat :
		ZzAssert(fileType == kMEExportAIFF);
		index += 3;
		break;
	default :
		ZZ_THROW_1ARG(ZzGenericException,"wrong sampleType in fillControls");
		break;
	};
	[fFileFormatPopup selectItemAtIndex:index];

	[fSampleRatePopup removeAllItems];
	[fSampleRatePopup addItemWithTitle:@"22.05 Khz"]; //0
	[fSampleRatePopup addItemWithTitle:@"32 Khz"]; //1
	[fSampleRatePopup addItemWithTitle:@"44.1 Khz"]; //2
	[fSampleRatePopup addItemWithTitle:@"48 Khz"]; //3
	[fSampleRatePopup addItemWithTitle:@"64 Khz"]; //4
	[fSampleRatePopup addItemWithTitle:@"88.2 Khz"]; //5
	[fSampleRatePopup addItemWithTitle:@"96 Khz"]; //6
	[fSampleRatePopup addItemWithTitle:@"192 Khz"]; //7
		
	fAudioFrameRate = doc->GetAudioFrameRate();
	switch(fAudioFrameRate)
	{
	case 22050 :
		[fSampleRatePopup selectItemAtIndex:0];
		break;
	case 32000 :
		[fSampleRatePopup selectItemAtIndex:1];
		break;
	case 44100 :
		[fSampleRatePopup selectItemAtIndex:2];
		break;
	case 48000 :
		[fSampleRatePopup selectItemAtIndex:3];
		break;
	case 64000 :
		[fSampleRatePopup selectItemAtIndex:4];
		break;
	case 88200 :
		[fSampleRatePopup selectItemAtIndex:5];
		break;
	case 96000 :
		[fSampleRatePopup selectItemAtIndex:6];
		break;
	case 192000 :
		[fSampleRatePopup selectItemAtIndex:7];
		break;
	default :
		ZZ_THROW_1ARG(ZzGenericException,"wrong doc->GetAudioFrameRate() in fillControls");
		break;
	}
	[fSampleRatePopup setEnabled:YES];
}

- (BOOL)shouldExportSelection
{
	return fShouldExportSelection;
}

- (NSString*)getFilePath
{
	return fFilePath;
}

- (UInt32)getFileType
{
	return fFileType;
}

- (UInt32)getSampleType;
{
	return fSampleType;
}

- (void)dealloc
{
	[super dealloc];
}

- (id)init
{
	self = [super init];
	if(self)
	{
		fAccessoryView = nil;
		fFilePath = @"";
		fFileType = 0xffffffff;
	}
	return self;
}

- (BOOL)shouldExportMultipleFiles
{
	return fShouldExportAsMultipleFiles;
}

- (BOOL)includeAllFX
{
	return fShouldIncludeAllFX;
}

- (UInt32)getAudioFrameRate
{
	return fAudioFrameRate;
}

- (UInt32)getNbChannels
{
	return fNbChannels;
}

- (IBAction)useSingleFile:(id)sender
{
}

- (IBAction)useMultipleFiles:(id)sender
{
}

- (IBAction)exportWholeDoc:(id)sender
{
}

- (IBAction)exportSelection:(id)sender
{
}

- (IBAction)useOriginalNames:(id)sender
{
}

- (void)setNbAudioRepsInSelection:(int)nbAudioReps
{
	if(nbAudioReps == 0)
	{
		[fExportWholeDocRadioButton setEnabled:YES];
		[fExportSelectionRadioButton setEnabled:NO];
		[fExportWholeDocRadioButton setState:NSOnState];
		[fExportSelectionRadioButton setState:NSOffState];
		[fIncludeAllFXCheckBox setState:NSOnState];
		[fIncludeAllFXCheckBox setEnabled:NO];
		
		// allow multi export
		[fSingleFileRadioButton setState:NSOnState];
		[fMultipleFilesRadioButton setState:NSOffState];
		[fMultipleFilesRadioButton setEnabled:YES];
		
		// allow to use orig names
		[fUseOriginalNamesCheckBox setState:NSOffState];
		[fUseOriginalNamesCheckBox setEnabled:YES];
	}
	else if(nbAudioReps ==1)
	{
		// if there is a single rep selected, let's set to "export selection" first.
		[fExportWholeDocRadioButton setEnabled:YES];
		[fExportSelectionRadioButton setEnabled:YES];
		[fExportWholeDocRadioButton setState:NSOffState];
		[fExportSelectionRadioButton setState:NSOnState];
		
		// only allow exporting as a single file
		[fSingleFileRadioButton setState:NSOnState];
		[fMultipleFilesRadioButton setState:NSOffState];
		[fMultipleFilesRadioButton setEnabled:NO];
		
		// useless
		[fUseOriginalNamesCheckBox setState:NSOffState];
		[fUseOriginalNamesCheckBox setEnabled:NO];
	}
	else
	{
		// if there are several reps selected, let's set to "export selection" first.
		[fExportWholeDocRadioButton setEnabled:YES];
		[fExportSelectionRadioButton setEnabled:YES];
		[fExportWholeDocRadioButton setState:NSOffState];
		[fExportSelectionRadioButton setState:NSOnState];
		
		// allow exporting to a single file or to multiple files
		[fSingleFileRadioButton setEnabled:YES];
		[fSingleFileRadioButton setState:NSOffState];
		[fMultipleFilesRadioButton setEnabled:YES];
		[fMultipleFilesRadioButton setState:NSOnState];
		
		// allow to use the clip name OR another name
		[fUseOriginalNamesCheckBox setState:NSOffState];
		[fUseOriginalNamesCheckBox setEnabled:YES];
	}
	
}

+ (void)exportRawAudio
{
	// +----------------------------------+
	// | retrieve the document pointers   |
	// +----------------------------------+
	
	MIMontageDocument*	montageDoc = [[NSDocumentController sharedDocumentController] currentDocument];
	TArmeDoc*			armeDoc = [montageDoc armeDocument];
	ZzArchPtrArray 		objectsToExport;

	// +----------------------------------+
	// | check for audioless document     |
	// +----------------------------------+
	if(!armeDoc->GetTopContainer()->ContainsAudio())
	{
		ZzWarningMessageBox("Sorry, this document does not contain audio.");
		return;
	}
	
	// +-------------------------------------------------+
	// | create the controller instance and load the nib |
	// | (this loads the view into fAccessoryView)       |
	// +-------------------------------------------------+
	MIExportRAWAudioController* ctrller = [[[MIExportRAWAudioController alloc] init] autorelease];
	[NSBundle loadNibNamed:@"ExportRAWAudioAccessory" owner:ctrller];

	// +-------------------------------------------------+
	// | fill the controller with correct default values |
	// +-------------------------------------------------+
	int nbChannels,fileType,sampleType;
	[ctrller retrieveStoredFormatPreferencesNbChannels:&nbChannels sampleType:&sampleType fileType:&fileType];
	// we're not using the stored prefs nbChannels
	
	[ctrller fillControls:armeDoc sampleType:sampleType fileType:fileType];
						
	// +-----------------------------------------------------------------+
	// | examine the selection to configure possible options accordingly |
	// +-----------------------------------------------------------------+
	NSArray* audioRepsFlattened = [[[NSApp mainViewController] selection] putRepsThatContainAudioInArrayAndRemoveDescendants];
//	if([audioRepsFlattened count] == 0)
//	{
//		ZzWarningMessageBox("Sorry, this document does not contain audio.");
//		return;
//	}
	
	// +-----------------------------------------------------------------+
	// | the ctrller will behave differently depending upon the number   |
	// | of selection objects that contain audio. let's pass this number |
	// | to the controller.                                              |
	// +-----------------------------------------------------------------+
	[ctrller setNbAudioRepsInSelection:[audioRepsFlattened count]];
	
	// +-------------------------------------------------------------------+
	// | if there is only ONE rep in the sel, its name needs to be set as  |
	// | the default name. otherwise, let's use the document name          |
	// +-------------------------------------------------------------------+
	if([audioRepsFlattened count] == 1)
	{
		TArmeWSObj* theWSObj = [[[audioRepsFlattened objectAtIndex:0] representedObject] WSObject];
		ZzString objName(theWSObj->GetName());
		char szObjName[1024];
		strcpy(szObjName,objName.getUTF8String());
		ZzRemoveFileExtensionFromPath(szObjName,1024);
		NSString* nssObjName = [NSString stringWithUTF8String:szObjName];
		[ctrller setInitialFileName:nssObjName];
		if(theWSObj->GetTopParentVAORouting().bStereo)
			[ctrller setNbChannels:2];
		else
			[ctrller setNbChannels:1];
	}
	else
	{
		ZzString zzDocFolderPath(armeDoc->GetProjectFolderPath());
		ZzString docFolderName = ZzLastPathComponent(zzDocFolderPath);
		char szDocName[1024];
		strcpy(szDocName,docFolderName.getUTF8String());
		ZzRemoveFileExtensionFromPath(szDocName,1024);
		NSString* nssDocName = [NSString stringWithUTF8String:szDocName];
		[ctrller setInitialFileName:nssDocName];
		[ctrller setNbChannels:0]; // 0 means "use object routing"
	}
			
	if([audioRepsFlattened count] > 0)
	{
		// +---------------------------------------------------------------------+
		// | if there are overlapping objects or non siblings objects in the     |
		// | sel, then the "include all fx" options cannot be enabled. this      |
		// | is a technical limitation.                                          | 
		// +---------------------------------------------------------------------+
		BOOL selIsMadeOfNonOverlappingSiblings = [[[NSApp mainViewController] selection] audioMembersAreMadeOfNonOverlappingSiblings];
	
		if(!selIsMadeOfNonOverlappingSiblings)
		{
			TArmeWSObj* aSibling = [[[audioRepsFlattened objectAtIndex:0] representedObject] WSObject];
			if(aSibling->GetParent() != aSibling->GetDoc()->GetTopContainer())
			
			NSRunAlertPanel(
				@"Info", @"Your selection contains overlapping objects and/or objects at different hierarchy levels. You won't be allowed to export using the \"include enclosing effects\" feature",
				@"OK", nil, nil);
			[ctrller setAllowIncludeAllFX:NO];
		}
		// +---------------------------------------------------------------------+
		// | if there are NO overlapping objects in the sel and if ALL sel      |
		// | objects are siblings, then the "include all fx" options can be      |
		// | enabled. However, it shouldn't be enabled if the selected objects   |
		// | are already top level                                               |
		// +---------------------------------------------------------------------+
		else
		{	
			TArmeWSObj* aSibling = [[[audioRepsFlattened objectAtIndex:0] representedObject] WSObject];
			if(aSibling->GetParent() == aSibling->GetDoc()->GetTopContainer())
				[ctrller setAllowIncludeAllFX:NO];
			else
				[ctrller setAllowIncludeAllFX:YES];
		}
	}
				
	int retCode = [ctrller displaySavePanel];
	if(retCode != NSFileHandlingPanelOKButton)
		return;

	// if the selection does not contain audio, we need to prevent 
	MIObject* mitlc = [[NSApp mainViewController] getTLC];
	TArmeWSObj* tlc = [mitlc WSObject];
//	TArmeDoc* doc = tlc->GetDoc();
		
	UInt32 		newSampleType = [ctrller getSampleType];
	UInt32 		newNbChannels = [ctrller getNbChannels];
	UInt32 		newFileType = [ctrller getFileType];
	UInt32 		audioFrameRate = [ctrller getAudioFrameRate];
	bool		exportSelection = [ctrller shouldExportSelection];
	bool		multipleFiles = [ctrller shouldExportMultipleFiles];
	ZzString	basePathName([ctrller getFilePath]);
	bool		includeAllFX = [ctrller includeAllFX];
	bool		useOriginalNames = [ctrller shouldUseOriginalNames];
	
	[ctrller storeFormatPreferencesNbChannels:newNbChannels sampleType:newSampleType fileType:newFileType];
	
	// for the slice
	ArmeTime	movieStartTime = ARME_TIME_MAX;
	ArmeTime	movieEndxTime = ARME_TIME_MIN;
		
	bool exportSliceSelection = false;
	
	if(exportSelection)
	{
		exportSliceSelection = ([[[NSApp mainViewController] selection] type] == kMIWSSelectionTypeTimeSlice);
	
		NSArray* selArray = [[[NSApp mainViewController] selection] putRepsInArrayAndRemoveDescendants];
		
		//put all objects in the selection in an array
		for(UInt32 i = 0; i < [selArray count]; i++)
		{
			TArmeWSObj* armeObj = [[[selArray objectAtIndex:i] representedObject] WSObject];
			if (armeObj->GetType() != TArmeWSObj::kVideoXFadeEffect)
			{
				objectsToExport.AddItem(armeObj);
				if (armeObj->GetAbsoluteStartTime() < movieStartTime)
					movieStartTime = armeObj->GetAbsoluteStartTime();
				if (armeObj->GetAbsoluteEndxTime() > movieEndxTime)
					movieEndxTime = armeObj->GetAbsoluteEndxTime();
			}
		}
		if (exportSliceSelection)
		{
			movieStartTime = [[[NSApp mainViewController] selection] startTime];
			movieEndxTime = [[[NSApp mainViewController] selection] endxTime];
		}
	}
	else
	{
		exportSliceSelection = false;
		for(SInt32 i = 0; i < tlc->GetChildrenCount(); i++)
		{
			objectsToExport.AddItem(tlc->GetChild(i));
		}
	}
	
	MIProgressBarDlgManager progressMngr("Exporting Audio");
	try
	{
		MEExportToExternalFile(
			&objectsToExport,basePathName,exportSliceSelection,
			GET_VAL(movieStartTime),GET_VAL(movieEndxTime),
			multipleFiles,includeAllFX,useOriginalNames,
			newNbChannels,audioFrameRate,newFileType,newSampleType,
			&progressMngr);
	}
	catch(ZzExceptionCancel* exc)
	{
	}
	catch(ZzException* ex)
	{
		ex->DefaultCatcher();
	}
	catch(...)
	{
		throw;
	}
}

- (void)retrieveStoredFormatPreferencesNbChannels:(int*)nbChannels sampleType:(int*)sampleType fileType:(int*)fileType
{
	// | nbChannels |
	NSString* nssNbChannels = [[NSUserDefaults standardUserDefaults] stringForKey:@"RAE_nbChannels"];
	if(nssNbChannels)
		*nbChannels = [nssNbChannels intValue];
	else	
		*nbChannels = 2;
	if(*nbChannels != 0 && *nbChannels != 1 && *nbChannels != 2)
		*nbChannels = 2;

	// | sample type |
	NSString* nssSampleType = [[NSUserDefaults standardUserDefaults] stringForKey:@"RAE_sampleType"];
	if(nssSampleType)
	{
		if([nssSampleType isEqualToString:@"8 bits"])
			*sampleType = kMEExport8Bits;
		else if([nssSampleType isEqualToString:@"16 bits"])
			*sampleType = kMEExport16Bits;
		else if([nssSampleType isEqualToString:@"24 bits"])
			*sampleType = kMEExport24Bits;
		else if([nssSampleType isEqualToString:@"32 bits float"])
			*sampleType = kMEExport32BitsFloat;
		else
			*sampleType = kMEExport16Bits;
	}
	else
	{
		*sampleType = kMEExport16Bits;
	}

	NSString* nssFileType = [[NSUserDefaults standardUserDefaults] stringForKey:@"RAE_fileType"];
	if(nssFileType)
	{
		if([nssSampleType isEqualToString:@"AIFF"])
			*fileType = kMEExportAIFF;
		else if([nssSampleType isEqualToString:@"WAVE"])
			*fileType = kMEExportWAVE;
		else 
			*fileType = kMEExportAIFF;
	}
	else 
	{
		*fileType = kMEExportAIFF;
	}
}

- (void)storeFormatPreferencesNbChannels:(int)nbChannels sampleType:(int)sampleType fileType:(int)fileType
{
	[[NSUserDefaults standardUserDefaults] setInteger:nbChannels forKey:@"RAE_nbChannels"];
	switch(sampleType)
	{
	case kMEExport8Bits :
		[[NSUserDefaults standardUserDefaults] setObject:@"8 bits" forKey:@"RAE_sampleType"];
		break;
	case kMEExport16Bits :
		[[NSUserDefaults standardUserDefaults] setObject:@"16 bits" forKey:@"RAE_sampleType"];
		break;
	case kMEExport24Bits :
		[[NSUserDefaults standardUserDefaults] setObject:@"24 bits" forKey:@"RAE_sampleType"];
		break;
	case kMEExport32BitsFloat :
		[[NSUserDefaults standardUserDefaults] setObject:@"32 bits float" forKey:@"RAE_sampleType"];
		break;
	default :
		fprintf(stderr,"WARNING : wrong sampleType in storeFormatPreferencesNbChannels\n");
		[[NSUserDefaults standardUserDefaults] setObject:@"16 bits" forKey:@"RAE_sampleType"];
		break;
	};
	switch(fileType)
	{
	case kMEExportAIFF : 
		[[NSUserDefaults standardUserDefaults] setObject:@"AIFF" forKey:@"RAE_fileType"];
		break;
	case kMEExportWAVE : 
		[[NSUserDefaults standardUserDefaults] setObject:@"WAVE" forKey:@"RAE_fileType"];
		break;
	default :
		fprintf(stderr,"WARNING : wrong sampleType in storeFormatPreferencesNbChannels\n");
		[[NSUserDefaults standardUserDefaults] setObject:@"AIFF" forKey:@"RAE_fileType"];
		break;
	}
}

- (BOOL)shouldUseOriginalNames
{
	return fShouldUseOriginalNames;
}

@end	
	
	
	