/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MIExportSettingsController.h"


@implementation MIExportSettingsController

+ (MIExportSettingsController*)sharedExportSettingsController
{
    static MIExportSettingsController *sharedExportSettingsController = nil;
    if ( sharedExportSettingsController == nil )
    {
        sharedExportSettingsController = [[MIExportSettingsController alloc] init];
	}
    return sharedExportSettingsController;
}

- (MIExportSettingsController*) init
{
    if (( self = [super initWithWindowNibName:@"MIExportSettings"] ))
    {
	fImageSettings = nil;
	::OpenADefaultComponent(StandardCompressionType, StandardCompressionSubType, &fImageSettings);
	fAudioSettings = nil;
	::OpenADefaultComponent(StandardCompressionType, StandardCompressionSubTypeSound, &fAudioSettings);
    }
    
    return self;
}

- (void)describeImage
{
	if (!fImageSettings)
		return;
	Handle text = nil;
	::ImageCodecGetSettingsAsText(fImageSettings, &text);
	if (text)
	{
		NSString *asString = [NSString stringWithCString:*text length:GetHandleSize(text)];
		[fImageDescription selectAll:self];
		NSRange range = [fImageDescription selectedRange];
		[fImageDescription replaceCharactersInRange:range withString:asString];
		::DisposeHandle(text);
	}
}

- (void)describeAudio
{
	if (!fAudioSettings)
		return;
	Handle text = nil;
	::ImageCodecGetSettingsAsText(fAudioSettings, &text);
	if (text)
	{
		NSString *asString = [NSString stringWithCString:*text length:GetHandleSize(text)];
		[fAudioDescription selectAll:self];
		NSRange range = [fAudioDescription selectedRange];
		[fAudioDescription replaceCharactersInRange:range withString:asString];
		::DisposeHandle(text);
	}
}

- (BOOL)requestSettings
{
	fSuccess = false;
	
    [NSApp runModalForWindow:[self window]];
    [[self window] orderOut:self];

    return fSuccess;
}

- (IBAction)cancelClicked:(id)sender
{
	fSuccess = false;
	[NSApp stopModal];
}

- (IBAction)exportClicked:(id)sender
{
	fStreaming = [fStreamingCheckbox intValue];
	fQT3Compatible = [fQT3CompatibleCheckbox intValue];
	fSize.width = [fWidthField intValue];
	fSize.height = [fHeightField intValue];

	fSuccess = true;
	[NSApp stopModal];
}

- (IBAction)fixed43RatioClicked:(id)sender
{
	if ([sender intValue])
		[fHeightField setIntValue:[fWidthField intValue] * 3/4];
}

- (IBAction)imageSettingsClicked:(id)sender
{
	OSErr err = ::SCRequestSequenceSettings(fImageSettings);
	if (err != userCanceledErr)
		[self describeImage];
}

- (IBAction)audioSettingsClicked:(id)sender
{
	OSErr err = ::SCRequestImageSettings(fAudioSettings);
	if (err != userCanceledErr)
		[self describeAudio];
}

- (void)awakeFromNib
{
	[fWidthField setDelegate:self];
	[fHeightField setDelegate:self];
	[self describeImage];
	[self describeAudio];
}

- (void)textDidChange:(NSNotification *)aNotification
{
	if (![fFixed43RatioCheckbox intValue])
		return;

	if (fWidthField == [aNotification object])
		[fHeightField setIntValue:[fWidthField intValue] * 3/4];
	else if (fHeightField == [aNotification object])
		[fWidthField setIntValue:[fHeightField intValue] * 4/3];
}

- (BOOL)getStreaming
{
	return fStreaming;
}

- (BOOL)getQT3Compatible
{
	return fQT3Compatible;
}

- (NSSize)getSize
{
	return fSize;
}

- (ComponentInstance)getImageSettings
{
	return fImageSettings;
}

- (ComponentInstance)getAudioSettings
{
	return fAudioSettings;
}

@end
