/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MontageInterface.h"

// this class is a placeholder for all the images
@interface MIImageController : NSObject
{
@public
//////////////////
    // clips
//////////////////
    // clip external shadow
    IBOutlet NSImage	*fClipExternalShadowTopLeft;
    IBOutlet NSImage	*fClipExternalShadowLeft;
    IBOutlet NSImage	*fClipExternalShadowBottomLeftCorner;
    IBOutlet NSImage	*fClipExternalShadowBottomLeft;
    IBOutlet NSImage	*fClipExternalShadowBottomCenter;
    IBOutlet NSImage	*fClipExternalShadowBottomRight;
    IBOutlet NSImage	*fClipExternalShadowBottomRightCorner;
    IBOutlet NSImage	*fClipExternalShadowTopRight;
    IBOutlet NSImage	*fClipExternalShadowRight;
    
    // clip internal shadow
    IBOutlet NSImage	*fClipInternalShadowLeft;
    IBOutlet NSImage	*fClipInternalShadowCenter;
    IBOutlet NSImage	*fClipInternalShadowRight;
    
    // clip handles
    IBOutlet NSImage	*fClipHandleLeft;
    IBOutlet NSImage	*fClipHandleBottom;
    IBOutlet NSImage	*fClipHandleRight;
    IBOutlet NSImage	*fClipHandleTop;
	
	IBOutlet NSImage	*fClipLock;

    // clip bars
    // audio clip bars
    IBOutlet NSImage	*fAudioClipBarOnLeft;
    IBOutlet NSImage	*fAudioClipBarOnCenter;
    IBOutlet NSImage	*fAudioClipBarOnRight;
    IBOutlet NSImage	*fAudioClipBarOffLeft;
    IBOutlet NSImage	*fAudioClipBarOffCenter;
    IBOutlet NSImage	*fAudioClipBarOffRight;
    IBOutlet NSImage	*fAudioClipBarNegativeLeft;
    IBOutlet NSImage	*fAudioClipBarNegativeCenter;
    IBOutlet NSImage	*fAudioClipBarNegativeRight;
    
    // midi clip bars
    IBOutlet NSImage	*fMidiClipBarOnLeft;
    IBOutlet NSImage	*fMidiClipBarOnCenter;
    IBOutlet NSImage	*fMidiClipBarOnRight;
    IBOutlet NSImage	*fMidiClipBarOffLeft;
    IBOutlet NSImage	*fMidiClipBarOffCenter;
    IBOutlet NSImage	*fMidiClipBarOffRight;
    IBOutlet NSImage	*fMidiClipBarNegativeLeft;
    IBOutlet NSImage	*fMidiClipBarNegativeCenter;
    IBOutlet NSImage	*fMidiClipBarNegativeRight;
    
    // text clip bars
    IBOutlet NSImage	*fTextClipBarOnLeft;
    IBOutlet NSImage	*fTextClipBarOnCenter;
    IBOutlet NSImage	*fTextClipBarOnRight;
    IBOutlet NSImage	*fTextClipBarOffLeft;
    IBOutlet NSImage	*fTextClipBarOffCenter;
    IBOutlet NSImage	*fTextClipBarOffRight;
    IBOutlet NSImage	*fTextClipBarNegativeLeft;
    IBOutlet NSImage	*fTextClipBarNegativeCenter;
    IBOutlet NSImage	*fTextClipBarNegativeRight;

    // video clip bars
    IBOutlet NSImage	*fVideoClipBarOnLeft;
    IBOutlet NSImage	*fVideoClipBarOnCenter;
    IBOutlet NSImage	*fVideoClipBarOnRight;
    IBOutlet NSImage	*fVideoClipBarOffLeft;
    IBOutlet NSImage	*fVideoClipBarOffCenter;
    IBOutlet NSImage	*fVideoClipBarOffRight;
    IBOutlet NSImage	*fVideoClipBarNegativeLeft;
    IBOutlet NSImage	*fVideoClipBarNegativeCenter;
    IBOutlet NSImage	*fVideoClipBarNegativeRight;

//////////////////
    // containers
//////////////////
    // the shadows are the same than for clips
    IBOutlet NSImage	*fContainerGroupBarOnLeft;
    IBOutlet NSImage	*fContainerGroupBarOnCenter;
    IBOutlet NSImage	*fContainerGroupBarOnRight;
    IBOutlet NSImage	*fContainerGroupBarOffLeft;
    IBOutlet NSImage	*fContainerGroupBarOffCenter;
    IBOutlet NSImage	*fContainerGroupBarOffRight;
    IBOutlet NSImage	*fContainerGroupBarNegativeLeft;
    IBOutlet NSImage	*fContainerGroupBarNegativeCenter;
    IBOutlet NSImage	*fContainerGroupBarNegativeRight;

    IBOutlet NSImage	*fContainerProcessorBarOnLeft;
    IBOutlet NSImage	*fContainerProcessorBarOnCenter;
    IBOutlet NSImage	*fContainerProcessorBarOnRight;
    IBOutlet NSImage	*fContainerProcessorBarOffLeft;
    IBOutlet NSImage	*fContainerProcessorBarOffCenter;
    IBOutlet NSImage	*fContainerProcessorBarOffRight;
    IBOutlet NSImage	*fContainerProcessorBarNegativeLeft;
    IBOutlet NSImage	*fContainerProcessorBarNegativeCenter;
    IBOutlet NSImage	*fContainerProcessorBarNegativeRight;
    
    IBOutlet NSImage	*fContainerResizeBarLeft;
    IBOutlet NSImage	*fContainerResizeBarCenter;
    IBOutlet NSImage	*fContainerResizeBarRight;
    
    IBOutlet NSImage	*fContainerButtonArrowHorizontal;
    IBOutlet NSImage	*fContainerButtonArrowVertical;
    
    IBOutlet NSImage	*fContainerButtonOn;
    IBOutlet NSImage	*fContainerButtonOff;
    
    IBOutlet NSImage	*fContainerButtonPropertiesOn;
    IBOutlet NSImage	*fContainerButtonPropertiesOff;
    
//////////////////
    // buttons 
//////////////////
#if 0 //$$$TOOLBOX
    IBOutlet NSImage	*fToolBoxButtonAlignmentOn;
    IBOutlet NSImage	*fToolBoxButtonAlignmentOff;
    IBOutlet NSImage	*fToolBoxButtonAudioOn;
    IBOutlet NSImage	*fToolBoxButtonAudioOff;
    IBOutlet NSImage	*fToolBoxButtonChainOn;
    IBOutlet NSImage	*fToolBoxButtonChainOff;
    IBOutlet NSImage	*fToolBoxButtonClipBoardOn;
    IBOutlet NSImage	*fToolBoxButtonClipBoardOff;
    IBOutlet NSImage	*fToolBoxButtonColorizeOn;
    IBOutlet NSImage	*fToolBoxButtonColorizeOff;
    IBOutlet NSImage	*fToolBoxButtonGridOn;
    IBOutlet NSImage	*fToolBoxButtonGridOff;
    IBOutlet NSImage	*fToolBoxButtonHistoryOn;
    IBOutlet NSImage	*fToolBoxButtonHistoryOff;
    IBOutlet NSImage	*fToolBoxButtonLockOn;
    IBOutlet NSImage	*fToolBoxButtonLockOff;
    IBOutlet NSImage	*fToolBoxButtonMarkerOn;
    IBOutlet NSImage	*fToolBoxButtonMarkerOff;
    IBOutlet NSImage	*fToolBoxButtonMidiOn;
    IBOutlet NSImage	*fToolBoxButtonMidiOff;
    IBOutlet NSImage	*fToolBoxButtonMixerOn;
    IBOutlet NSImage	*fToolBoxButtonMixerOff;
    IBOutlet NSImage	*fToolBoxButtonPackOn;
    IBOutlet NSImage	*fToolBoxButtonPackOff;
    IBOutlet NSImage	*fToolBoxButtonProcessorOn;
    IBOutlet NSImage	*fToolBoxButtonProcessorOff;
    IBOutlet NSImage	*fToolBoxButtonSearchOn;
    IBOutlet NSImage	*fToolBoxButtonSearchOff;
    IBOutlet NSImage	*fToolBoxButtonSnapOn;
    IBOutlet NSImage	*fToolBoxButtonSnapOff;
    IBOutlet NSImage	*fToolBoxButtonSortOn;
    IBOutlet NSImage	*fToolBoxButtonSortOff;
    IBOutlet NSImage	*fToolBoxButtonTextOn;
    IBOutlet NSImage	*fToolBoxButtonTextOff;
    IBOutlet NSImage	*fToolBoxButtonVideoOn;
    IBOutlet NSImage	*fToolBoxButtonVideoOff;
#endif  // 0 //$$$TOOLBOX

    IBOutlet NSImage	*fPlayBarButtonBackOn;
    IBOutlet NSImage	*fPlayBarButtonBackOff;
    IBOutlet NSImage	*fPlayBarButtonCameraOn;
    IBOutlet NSImage	*fPlayBarButtonCameraOff;
    IBOutlet NSImage	*fPlayBarButtonforwardOn;
    IBOutlet NSImage	*fPlayBarButtonforwardOff;
    IBOutlet NSImage	*fPlayBarButtonLoopOn;
    IBOutlet NSImage	*fPlayBarButtonLoopOff;
    IBOutlet NSImage	*fPlayBarButtonMarkOn;
    IBOutlet NSImage	*fPlayBarButtonMarkOff;
    IBOutlet NSImage	*fPlayBarButtonMicroOn;
    IBOutlet NSImage	*fPlayBarButtonMicroOff;
    IBOutlet NSImage	*fPlayBarButtonMidiOn;
    IBOutlet NSImage	*fPlayBarButtonMidiOff;
    IBOutlet NSImage	*fPlayBarButtonPauseOn;
    IBOutlet NSImage	*fPlayBarButtonPauseOff;
    IBOutlet NSImage	*fPlayBarButtonPlayOn;
    IBOutlet NSImage	*fPlayBarButtonPlayOff;
    IBOutlet NSImage	*fPlayBarButtonRecordOn;
    IBOutlet NSImage	*fPlayBarButtonRecordOff;
    IBOutlet NSImage	*fPlayBarButtonStopOn;
    IBOutlet NSImage	*fPlayBarButtonStopOff;
    IBOutlet NSImage	*fPlayBarButtonTextOn;
    IBOutlet NSImage	*fPlayBarButtonTextOff;
    
/////////////////////////////////////
    // time palette & hyperprism
/////////////////////////////////////

	IBOutlet NSImage	*fTimePaletteBackground;
	IBOutlet NSImage	*fHyperprismTextBoxLeft;
	IBOutlet NSImage	*fHyperprismTextBoxCenter;
	IBOutlet NSImage	*fHyperprismTextBoxRight;
	IBOutlet NSImage	*fHyperPrismLogo;
	IBOutlet NSImage	*fBlueWindowLogo;
/////////////////
// locators
/////////////////
	
	IBOutlet NSImage	*fRulerPlayLocator;
	IBOutlet NSImage	*fRulerLeftLocator;
	IBOutlet NSImage	*fRulerRightLocator;
	IBOutlet NSImage	*fRulerGenericLocator;
	IBOutlet NSImage	*fRulerDottedLine;
/////////////////////////
// clip partial images
/////////////////////////

	// bright images
	IBOutlet NSImage	*fLeftAudioClipBrightTitleBarImage;
	IBOutlet NSImage	*fCenterAudioClipBrightTitleBarImage;
	IBOutlet NSImage	*fRightAudioClipBrightTitleBarImage;

	IBOutlet NSImage	*fLeftVideoClipBrightTitleBarImage;
	IBOutlet NSImage	*fCenterVideoClipBrightTitleBarImage;
	IBOutlet NSImage	*fRightVideoClipBrightTitleBarImage;

	IBOutlet NSImage	*fLeftTextClipBrightTitleBarImage;
	IBOutlet NSImage	*fCenterTextClipBrightTitleBarImage;
	IBOutlet NSImage	*fRightTextClipBrightTitleBarImage;

	IBOutlet NSImage	*fLeftMidiClipBrightTitleBarImage;
	IBOutlet NSImage	*fCenterMidiClipBrightTitleBarImage;
	IBOutlet NSImage	*fRightMidiClipBrightTitleBarImage;

	IBOutlet NSImage	*fLeftProcessorBrightTitleBarImage;
	IBOutlet NSImage	*fCenterProcessorBrightTitleBarImage;
	IBOutlet NSImage	*fRightProcessorBrightTitleBarImage;

	IBOutlet NSImage	*fLeftGroupBrightTitleBarImage;
	IBOutlet NSImage	*fCenterGroupBrightTitleBarImage;
	IBOutlet NSImage	*fRightGroupBrightTitleBarImage;


	// dim images
	IBOutlet NSImage	*fLeftAudioClipDimTitleBarImage;
	IBOutlet NSImage	*fCenterAudioClipDimTitleBarImage;
	IBOutlet NSImage	*fRightAudioClipDimTitleBarImage;

	IBOutlet NSImage	*fLeftVideoClipDimTitleBarImage;
	IBOutlet NSImage	*fCenterVideoClipDimTitleBarImage;
	IBOutlet NSImage	*fRightVideoClipDimTitleBarImage;

	IBOutlet NSImage	*fLeftTextClipDimTitleBarImage;
	IBOutlet NSImage	*fCenterTextClipDimTitleBarImage;
	IBOutlet NSImage	*fRightTextClipDimTitleBarImage;

	IBOutlet NSImage	*fLeftMidiClipDimTitleBarImage;
	IBOutlet NSImage	*fCenterMidiClipDimTitleBarImage;
	IBOutlet NSImage	*fRightMidiClipDimTitleBarImage;

	IBOutlet NSImage	*fLeftProcessorDimTitleBarImage;
	IBOutlet NSImage	*fCenterProcessorDimTitleBarImage;
	IBOutlet NSImage	*fRightProcessorDimTitleBarImage;

	IBOutlet NSImage	*fLeftGroupDimTitleBarImage;
	IBOutlet NSImage	*fCenterGroupDimTitleBarImage;
	IBOutlet NSImage	*fRightGroupDimTitleBarImage;


	// negative images
	IBOutlet NSImage	*fLeftAudioClipNegativeTitleBarImage;
	IBOutlet NSImage	*fCenterAudioClipNegativeTitleBarImage;
	IBOutlet NSImage	*fRightAudioClipNegativeTitleBarImage;

	IBOutlet NSImage	*fLeftVideoClipNegativeTitleBarImage;
	IBOutlet NSImage	*fCenterVideoClipNegativeTitleBarImage;
	IBOutlet NSImage	*fRightVideoClipNegativeTitleBarImage;

	IBOutlet NSImage	*fLeftTextClipNegativeTitleBarImage;
	IBOutlet NSImage	*fCenterTextClipNegativeTitleBarImage;
	IBOutlet NSImage	*fRightTextClipNegativeTitleBarImage;

	IBOutlet NSImage	*fLeftMidiClipNegativeTitleBarImage;
	IBOutlet NSImage	*fCenterMidiClipNegativeTitleBarImage;
	IBOutlet NSImage	*fRightMidiClipNegativeTitleBarImage;

	IBOutlet NSImage	*fLeftProcessorNegativeTitleBarImage;
	IBOutlet NSImage	*fCenterProcessorNegativeTitleBarImage;
	IBOutlet NSImage	*fRightProcessorNegativeTitleBarImage;

	IBOutlet NSImage	*fLeftGroupNegativeTitleBarImage;
	IBOutlet NSImage	*fCenterGroupNegativeTitleBarImage;
	IBOutlet NSImage	*fRightGroupNegativeTitleBarImage;
	
	IBOutlet NSImage	*fTextClipCheckboard;
	IBOutlet NSImage	*fTextClipCheckboardNegative;
	
	IBOutlet NSColor	*fAudioClipBrightColor;
	IBOutlet NSColor	*fAudioClipDimColor;
	IBOutlet NSColor	*fAudioClipNegativeColor;

	IBOutlet NSColor	*fVideoClipBrightColor;
	IBOutlet NSColor	*fVideoClipDimColor;
	IBOutlet NSColor	*fVideoClipNegativeColor;

	IBOutlet NSColor	*fTextClipBrightColor;
	IBOutlet NSColor	*fTextClipDimColor;
	IBOutlet NSColor	*fTextClipNegativeColor;

	IBOutlet NSColor	*fMidiClipBrightColor;
	IBOutlet NSColor	*fMidiClipDimColor;
	IBOutlet NSColor	*fMidiClipNegativeColor;

	IBOutlet NSColor	*fProcessorBrightColor;
	IBOutlet NSColor	*fProcessorDimColor;
	IBOutlet NSColor	*fProcessorNegativeColor;

	IBOutlet NSColor	*fGroupBrightColor;
	IBOutlet NSColor	*fGroupDimColor;
	IBOutlet NSColor	*fGroupNegativeColor;
	
	
/////////////////
    // cursors 
/////////////////

    NSCursor			*fResizeUpCursor;
    NSCursor			*fResizeDownCursor;
    NSCursor			*fResizeLeftCursor;
    NSCursor			*fResizeRightCursor;
    NSCursor			*fMoveOpenCursor;
    NSCursor			*fMoveCloseCursor;
    NSCursor			*fMoveCloseCopyCursor;
	NSCursor			*fResizeDownLeftCursor;
	NSCursor			*fResizeDownRightCursor;
	NSCursor			*fTimeStretchLeftCursor;
	NSCursor			*fTimeStretchRightCursor;
	NSCursor			*fTrimLeftCursor;
	NSCursor			*fTrimRightCursor;
	NSCursor			*fCarretCursor;
	NSCursor			*fLeftCarretCursor;
	NSCursor			*fRightCarretCursor;
	NSCursor			*fLeftRightCarretCursor;
	
	
	NSImage				*fTrimIndicatorRight;
	NSImage				*fTrimIndicatorLeft;
	NSImage				*fNonTrimIndicatorRight;
	NSImage				*fNonTrimIndicatorLeft;
	
	NSImage				*fResizeIndicatorDownRight;
	NSImage				*fResizeIndicatorDownLeft;
	NSImage				*fResizeIndicatorUpRight;
	NSImage				*fResizeIndicatorUpLeft;
}

- (void)loadImages;
- (void)checkImages;
- (void)setRepresentationColor;
- (void)setHandCursors:(BOOL)onOff;
- (void)setLocatorIcons:(BOOL)triangles;
- (void)createRepresentationPartialImages;
- (void)createImagesLeft:(NSImage**)leftImage
				  center:(NSImage**)centerImage
					right:(NSImage**)rightImage
					fromLeftBarImage:(NSImage*)leftBarImage
							  center:(NSImage*)centerBarImage
							   right:(NSImage*)rightBarImage
								color:(NSColor*)color;

- (NSImage*)getVAOPicture:(int)vao;
@end
