/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MIImageController.h"

#ifndef ZzVerbose_h
//#include "ZzVerbose.h"
#endif //ZzVerbose_h

#ifdef SMALL_TITLEBAR
const float	kRepTitleHeightTemplate = 10;
#else
const float	kRepTitleHeightTemplate = 10;
#endif


@implementation MIImageController

- (id)init
{
    self = [super init];
    if ( self )
    {
        // clip external shadow
        fClipExternalShadowTopLeft = nil;
        fClipExternalShadowLeft = nil;
        fClipExternalShadowBottomLeft = nil;
        fClipExternalShadowBottomCenter = nil;
        fClipExternalShadowTopRight = nil;
        fClipExternalShadowRight = nil;
        fClipExternalShadowBottomRight = nil;
        fClipExternalShadowBottomRightCorner = nil;
        fClipExternalShadowBottomRightCorner = nil;
        
        // clip internal shadow
        fClipInternalShadowLeft = nil;
        fClipInternalShadowCenter = nil;
        fClipInternalShadowRight = nil;
        
        // clip handles
        fClipHandleLeft = nil;
        fClipHandleBottom = nil;
        fClipHandleRight = nil;
        fClipHandleTop = nil;

		fClipLock = nil;
    
        // clip bars
        // audio clip bars
        fAudioClipBarOnLeft = nil;
        fAudioClipBarOnCenter = nil;
        fAudioClipBarOnRight = nil;
        fAudioClipBarOffLeft = nil;
        fAudioClipBarOffCenter = nil;
        fAudioClipBarOffRight = nil;
        fAudioClipBarNegativeLeft = nil;
        fAudioClipBarNegativeCenter = nil;
        fAudioClipBarNegativeRight = nil;
        
        // midi clip bars
        fMidiClipBarOnLeft = nil;
        fMidiClipBarOnCenter = nil;
        fMidiClipBarOnRight = nil;
        fMidiClipBarOffLeft = nil;
        fMidiClipBarOffCenter = nil;
        fMidiClipBarOffRight = nil;
        fMidiClipBarNegativeLeft = nil;
        fMidiClipBarNegativeCenter = nil;
        fMidiClipBarNegativeRight = nil;
        
        // text clip bars
        fTextClipBarOnLeft = nil;
        fTextClipBarOnCenter = nil;
        fTextClipBarOnRight = nil;
        fTextClipBarOffLeft = nil;
        fTextClipBarOffCenter = nil;
        fTextClipBarOffRight = nil;
        fTextClipBarNegativeLeft = nil;
        fTextClipBarNegativeCenter = nil;
        fTextClipBarNegativeRight = nil;
    
        // video clip bars
        fVideoClipBarOnLeft = nil;
        fVideoClipBarOnCenter = nil;
        fVideoClipBarOnRight = nil;
        fVideoClipBarOffLeft = nil;
        fVideoClipBarOffCenter = nil;
        fVideoClipBarOffRight = nil;
        fVideoClipBarNegativeLeft = nil;
        fVideoClipBarNegativeCenter = nil;
        fVideoClipBarNegativeRight = nil;
    
    //////////////////
        // containers
    //////////////////
        // the shadows are the same than for clips
        fContainerGroupBarOnLeft = nil;
        fContainerGroupBarOnCenter = nil;
        fContainerGroupBarOnRight = nil;
        fContainerGroupBarOffLeft = nil;
        fContainerGroupBarOffCenter = nil;
        fContainerGroupBarOffRight = nil;
        fContainerGroupBarNegativeLeft = nil;
        fContainerGroupBarNegativeCenter = nil;
        fContainerGroupBarNegativeRight = nil;
    
        fContainerProcessorBarOnLeft = nil;
        fContainerProcessorBarOnCenter = nil;
        fContainerProcessorBarOnRight = nil;
        fContainerProcessorBarOffLeft = nil;
        fContainerProcessorBarOffCenter = nil;
        fContainerProcessorBarOffRight = nil;
        fContainerProcessorBarNegativeLeft = nil;
        fContainerProcessorBarNegativeCenter = nil;
        fContainerProcessorBarNegativeRight = nil;
        
        fContainerResizeBarLeft = nil;
        fContainerResizeBarCenter = nil;
        fContainerResizeBarRight = nil;
        
        fContainerButtonArrowHorizontal = nil;
        fContainerButtonArrowVertical = nil;
        
        fContainerButtonOn = nil;
        fContainerButtonOff = nil;
        
        fContainerButtonPropertiesOn = nil;
        fContainerButtonPropertiesOff = nil;    
 
//////////////////
    // buttons
//////////////////
#if 0 //$$$TOOLBOX
    	fToolBoxButtonAlignmentOn = nil;
    	fToolBoxButtonAlignmentOff = nil;
    	fToolBoxButtonAudioOn = nil;
    	fToolBoxButtonAudioOff = nil;
    	fToolBoxButtonChainOn = nil;
    	fToolBoxButtonChainOff = nil;
    	fToolBoxButtonClipBoardOn = nil;
    	fToolBoxButtonClipBoardOff = nil;
    	fToolBoxButtonColorizeOn = nil;
    	fToolBoxButtonColorizeOff = nil;
    	fToolBoxButtonGridOn = nil;
    	fToolBoxButtonGridOff = nil;
    	fToolBoxButtonHistoryOn = nil;
    	fToolBoxButtonHistoryOff = nil;
    	fToolBoxButtonLockOn = nil;
    	fToolBoxButtonLockOff = nil;
    	fToolBoxButtonMarkerOn = nil;
    	fToolBoxButtonMarkerOff = nil;
    	fToolBoxButtonMidiOn = nil;
    	fToolBoxButtonMidiOff = nil;
    	fToolBoxButtonMixerOn = nil;
    	fToolBoxButtonMixerOff = nil;
    	fToolBoxButtonPackOn = nil;
    	fToolBoxButtonPackOff = nil;
    	fToolBoxButtonProcessorOn = nil;
    	fToolBoxButtonProcessorOff = nil;
    	fToolBoxButtonSearchOn = nil;
    	fToolBoxButtonSearchOff = nil;
    	fToolBoxButtonSnapOn = nil;
    	fToolBoxButtonSnapOff = nil;
    	fToolBoxButtonSortOn = nil;
    	fToolBoxButtonSortOff = nil;
    	fToolBoxButtonTextOn = nil;
    	fToolBoxButtonTextOff = nil;
    	fToolBoxButtonVideoOn = nil;
    	fToolBoxButtonVideoOff = nil;
#endif // 0 //$$$TOOLBOX
    	fPlayBarButtonBackOn = nil;
    	fPlayBarButtonBackOff = nil;
    	fPlayBarButtonCameraOn = nil;
    	fPlayBarButtonCameraOff = nil;
    	fPlayBarButtonforwardOn = nil;
    	fPlayBarButtonforwardOff = nil;
    	fPlayBarButtonLoopOn = nil;
    	fPlayBarButtonLoopOff = nil;
    	fPlayBarButtonMarkOn = nil;
    	fPlayBarButtonMarkOff = nil;
    	fPlayBarButtonMicroOn = nil;
    	fPlayBarButtonMicroOff = nil;
    	fPlayBarButtonMidiOn = nil;
    	fPlayBarButtonMidiOff = nil;
    	fPlayBarButtonPauseOn = nil;
    	fPlayBarButtonPauseOff = nil;
        fPlayBarButtonPlayOn = nil;
        fPlayBarButtonPlayOff = nil;
    	fPlayBarButtonRecordOn = nil;
    	fPlayBarButtonRecordOff = nil;
    	fPlayBarButtonStopOn = nil;
    	fPlayBarButtonStopOff = nil;
    	fPlayBarButtonTextOn = nil;
    	fPlayBarButtonTextOff = nil;
        
/////////////////////////////////////
    // time palette & hyperprism
/////////////////////////////////////

		fTimePaletteBackground = nil;
		fHyperprismTextBoxLeft = nil;
		fHyperprismTextBoxCenter = nil;
		fHyperprismTextBoxRight = nil;
		fHyperPrismLogo = nil;
        
		
		fResizeUpCursor = nil;
        fResizeDownCursor = nil;
        fResizeLeftCursor = nil;
        fResizeRightCursor = nil;
        fMoveOpenCursor = nil;
		fMoveCloseCursor = nil;
		fMoveCloseCopyCursor = nil;
		fRulerPlayLocator = nil;
		fRulerLeftLocator = nil;
		fRulerRightLocator = nil;
		fRulerDottedLine = nil;
		fCarretCursor = nil;
		fLeftCarretCursor = nil;
		fRightCarretCursor = nil;
		fLeftRightCarretCursor = nil;
    }
    return self;
}

#define CHECK_IMG( img ) { if ( !img ) { verntl(interfaceStephan, 0, "Missing image:" #img ); } }
#define CHECK_CURSOR( curs )  { if ( !curs ) { verntl(interfaceStephan, 0, "Missing cursor:img"); } } 
- (void)checkImages
{
    CHECK_IMG(fClipExternalShadowTopLeft)
    CHECK_IMG(fClipExternalShadowLeft)
    CHECK_IMG(fClipExternalShadowBottomLeftCorner)
    CHECK_IMG(fClipExternalShadowBottomLeft)
    CHECK_IMG(fClipExternalShadowBottomCenter)
    CHECK_IMG(fClipExternalShadowBottomRight)
    CHECK_IMG(fClipExternalShadowBottomRightCorner)
    CHECK_IMG(fClipExternalShadowTopRight)
    CHECK_IMG(fClipExternalShadowRight)
    
    // clip internal shadow
    CHECK_IMG(fClipInternalShadowLeft)
    CHECK_IMG(fClipInternalShadowCenter)
    CHECK_IMG(fClipInternalShadowRight)
    
    // clip handles
    CHECK_IMG(fClipHandleLeft)
    CHECK_IMG(fClipHandleBottom)
    CHECK_IMG(fClipHandleRight)
    CHECK_IMG(fClipHandleTop)

	CHECK_IMG(fClipLock)

    // clip bars
    // audio clip bars
    CHECK_IMG(fAudioClipBarOnLeft)
    CHECK_IMG(fAudioClipBarOnCenter)
    CHECK_IMG(fAudioClipBarOnRight)
    CHECK_IMG(fAudioClipBarOffLeft)
    CHECK_IMG(fAudioClipBarOffCenter)
    CHECK_IMG(fAudioClipBarOffRight)
    CHECK_IMG(fAudioClipBarNegativeLeft)
    CHECK_IMG(fAudioClipBarNegativeCenter)
    CHECK_IMG(fAudioClipBarNegativeRight)
    
    // midi clip bars
    CHECK_IMG(fMidiClipBarOnLeft)
    CHECK_IMG(fMidiClipBarOnCenter)
    CHECK_IMG(fMidiClipBarOnRight)
    CHECK_IMG(fMidiClipBarOffLeft)
    CHECK_IMG(fMidiClipBarOffCenter)
    CHECK_IMG(fMidiClipBarOffRight)
    CHECK_IMG(fMidiClipBarNegativeLeft)
    CHECK_IMG(fMidiClipBarNegativeCenter)
    CHECK_IMG(fMidiClipBarNegativeRight)
    
    // text clip bars
    CHECK_IMG(fTextClipBarOnLeft)
    CHECK_IMG(fTextClipBarOnCenter)
    CHECK_IMG(fTextClipBarOnRight)
    CHECK_IMG(fTextClipBarOffLeft)
    CHECK_IMG(fTextClipBarOffCenter)
    CHECK_IMG(fTextClipBarOffRight)
    CHECK_IMG(fTextClipBarNegativeLeft)
    CHECK_IMG(fTextClipBarNegativeCenter)
    CHECK_IMG(fTextClipBarNegativeRight)

    // video clip bars
    CHECK_IMG(fVideoClipBarOnLeft)
    CHECK_IMG(fVideoClipBarOnCenter)
    CHECK_IMG(fVideoClipBarOnRight)
    CHECK_IMG(fVideoClipBarOffLeft)
    CHECK_IMG(fVideoClipBarOffCenter)
    CHECK_IMG(fVideoClipBarOffRight)
    CHECK_IMG(fVideoClipBarNegativeLeft)
    CHECK_IMG(fVideoClipBarNegativeCenter)
    CHECK_IMG(fVideoClipBarNegativeRight)

//////////////////
    // containers
//////////////////
    // the shadows are the same than for clips
    CHECK_IMG(fContainerGroupBarOnLeft)
    CHECK_IMG(fContainerGroupBarOnCenter)
    CHECK_IMG(fContainerGroupBarOnRight)
    CHECK_IMG(fContainerGroupBarOffLeft)
    CHECK_IMG(fContainerGroupBarOffCenter)
    CHECK_IMG(fContainerGroupBarOffRight)
    CHECK_IMG(fContainerGroupBarNegativeLeft)
    CHECK_IMG(fContainerGroupBarNegativeCenter)
    CHECK_IMG(fContainerGroupBarNegativeRight)

    CHECK_IMG(fContainerProcessorBarOnLeft)
    CHECK_IMG(fContainerProcessorBarOnCenter)
    CHECK_IMG(fContainerProcessorBarOnRight)
    CHECK_IMG(fContainerProcessorBarOffLeft)
    CHECK_IMG(fContainerProcessorBarOffCenter)
    CHECK_IMG(fContainerProcessorBarOffRight)
    CHECK_IMG(fContainerProcessorBarNegativeLeft)
    CHECK_IMG(fContainerProcessorBarNegativeCenter)
    CHECK_IMG(fContainerProcessorBarNegativeRight)
    
    CHECK_IMG(fContainerResizeBarLeft)
    CHECK_IMG(fContainerResizeBarCenter)
    CHECK_IMG(fContainerResizeBarRight)
    
    CHECK_IMG(fContainerButtonArrowHorizontal)
    CHECK_IMG(fContainerButtonArrowVertical)
    
    CHECK_IMG(fContainerButtonOn)
    CHECK_IMG(fContainerButtonOff)
    
    CHECK_IMG(fContainerButtonPropertiesOn)
    CHECK_IMG(fContainerButtonPropertiesOff)
    
//////////////////
    // buttons 
//////////////////
#if 0 //$$$TOOLBOX
    CHECK_IMG(fToolBoxButtonAlignmentOn)
    CHECK_IMG(fToolBoxButtonAlignmentOff)
    CHECK_IMG(fToolBoxButtonAudioOn)
    CHECK_IMG(fToolBoxButtonAudioOff)
    CHECK_IMG(fToolBoxButtonChainOn)
    CHECK_IMG(fToolBoxButtonChainOff)
    CHECK_IMG(fToolBoxButtonClipBoardOn)
    CHECK_IMG(fToolBoxButtonClipBoardOff)
    CHECK_IMG(fToolBoxButtonColorizeOn)
    CHECK_IMG(fToolBoxButtonColorizeOff)
    CHECK_IMG(fToolBoxButtonGridOn)
    CHECK_IMG(fToolBoxButtonGridOff)
    CHECK_IMG(fToolBoxButtonHistoryOn)
    CHECK_IMG(fToolBoxButtonHistoryOff)
    CHECK_IMG(fToolBoxButtonLockOn)
    CHECK_IMG(fToolBoxButtonLockOff)
    CHECK_IMG(fToolBoxButtonMarkerOn)
    CHECK_IMG(fToolBoxButtonMarkerOff)
    CHECK_IMG(fToolBoxButtonMidiOn)
    CHECK_IMG(fToolBoxButtonMidiOff)
    CHECK_IMG(fToolBoxButtonMixerOn)
    CHECK_IMG(fToolBoxButtonMixerOff)
    CHECK_IMG(fToolBoxButtonPackOn)
    CHECK_IMG(fToolBoxButtonPackOff)
    CHECK_IMG(fToolBoxButtonProcessorOn)
    CHECK_IMG(fToolBoxButtonProcessorOff)
    CHECK_IMG(fToolBoxButtonSearchOn)
    CHECK_IMG(fToolBoxButtonSearchOff)
    CHECK_IMG(fToolBoxButtonSnapOn)
    CHECK_IMG(fToolBoxButtonSnapOff)
    CHECK_IMG(fToolBoxButtonSortOn)
    CHECK_IMG(fToolBoxButtonSortOff)
    CHECK_IMG(fToolBoxButtonTextOn)
    CHECK_IMG(fToolBoxButtonTextOff)
    CHECK_IMG(fToolBoxButtonVideoOn)
    CHECK_IMG(fToolBoxButtonVideoOff)
#endif // 0 //$$$TOOLBOX
    CHECK_IMG(fPlayBarButtonBackOn)
    CHECK_IMG(fPlayBarButtonBackOff)
    CHECK_IMG(fPlayBarButtonCameraOn)
    CHECK_IMG(fPlayBarButtonCameraOff)
    CHECK_IMG(fPlayBarButtonforwardOn)
    CHECK_IMG(fPlayBarButtonforwardOff)
    CHECK_IMG(fPlayBarButtonLoopOn)
    CHECK_IMG(fPlayBarButtonLoopOff)
    CHECK_IMG(fPlayBarButtonMarkOn)
    CHECK_IMG(fPlayBarButtonMarkOff)
    CHECK_IMG(fPlayBarButtonMicroOn)
    CHECK_IMG(fPlayBarButtonMicroOff)
    CHECK_IMG(fPlayBarButtonMidiOn)
    CHECK_IMG(fPlayBarButtonMidiOff)
    CHECK_IMG(fPlayBarButtonPauseOn)
    CHECK_IMG(fPlayBarButtonPauseOff)
    CHECK_IMG(fPlayBarButtonPlayOn)
    CHECK_IMG(fPlayBarButtonPlayOff)
    CHECK_IMG(fPlayBarButtonRecordOn)
    CHECK_IMG(fPlayBarButtonRecordOff)
    CHECK_IMG(fPlayBarButtonStopOn)
    CHECK_IMG(fPlayBarButtonStopOff)
    CHECK_IMG(fPlayBarButtonTextOn)
    CHECK_IMG(fPlayBarButtonTextOff)
    
/////////////////////////////////////
    // time palette & hyperprism
/////////////////////////////////////

	CHECK_IMG(fTimePaletteBackground )
	CHECK_IMG(fHyperprismTextBoxLeft)
	CHECK_IMG(fHyperprismTextBoxCenter)
	CHECK_IMG(fHyperprismTextBoxRight)
	CHECK_IMG(fHyperPrismLogo)
	CHECK_IMG(fBlueWindowLogo)

	CHECK_IMG(fRulerPlayLocator)
	CHECK_IMG(fRulerLeftLocator)
	CHECK_IMG(fRulerRightLocator)
	CHECK_IMG(fRulerDottedLine )


    CHECK_CURSOR(fResizeUpCursor )
    CHECK_CURSOR(fResizeDownCursor )
    CHECK_CURSOR(fResizeLeftCursor )
    CHECK_CURSOR(fResizeRightCursor )
    CHECK_CURSOR(fMoveOpenCursor )
    CHECK_CURSOR(fMoveCloseCursor )
    CHECK_CURSOR(fMoveCloseCopyCursor )
	CHECK_CURSOR(fCarretCursor )
	CHECK_CURSOR(fLeftCarretCursor )
	CHECK_CURSOR(fRightCarretCursor )
	CHECK_CURSOR(fLeftRightCarretCursor )
}

#define LOAD_IMG( name, ext) [[NSImage alloc] initWithContentsOfFile:[theBundle pathForResource:name ofType:ext]]

- (void)loadImages
{
    NSBundle	*theBundle = [NSBundle mainBundle];
	fClipExternalShadowTopLeft = LOAD_IMG(@"ombr_ext_sup_g", @"png");
    fClipExternalShadowLeft = LOAD_IMG(@"ombr_ext_gauche", @"png");
    fClipExternalShadowBottomLeftCorner = LOAD_IMG(@"ombr_ext_inf_g", @"png");
    fClipExternalShadowBottomLeft = LOAD_IMG(@"ombr_ext_bas_g", @"png");
    fClipExternalShadowBottomCenter = LOAD_IMG(@"ombr_ext_bas_c", @"png");
    fClipExternalShadowBottomRight = LOAD_IMG(@"ombr_ext_bas_d", @"png");
    fClipExternalShadowBottomRightCorner = LOAD_IMG(@"ombr_ext_inf_d", @"png");
    fClipExternalShadowTopRight = LOAD_IMG(@"ombr_ext_sup_d", @"png");
    fClipExternalShadowRight = LOAD_IMG(@"ombr_ext_droite", @"png");
    
    // clip internal shadow
    fClipInternalShadowLeft = LOAD_IMG(@"ombr_int_g", @"png");
    fClipInternalShadowCenter = LOAD_IMG(@"ombr_int_c", @"png");
    fClipInternalShadowRight = LOAD_IMG(@"ombr_int_d", @"png");
    
    // clip handles
    fClipHandleLeft = LOAD_IMG(@"handle_gauche", @"png");
    fClipHandleBottom = LOAD_IMG(@"handle_bas", @"png");
    fClipHandleRight = LOAD_IMG(@"handle_droite", @"png");
    fClipHandleTop = LOAD_IMG(@"handle_haut", @"png");

	fClipLock = LOAD_IMG(@"lock", @"png");

    // clip bars
    // audio clip bars
    fAudioClipBarOnLeft = LOAD_IMG(@"bar_on_audio_g", @"png");
    fAudioClipBarOnCenter = LOAD_IMG(@"bar_on_audio_c", @"png");
    fAudioClipBarOnRight = LOAD_IMG(@"bar_on_audio_d", @"png");
    fAudioClipBarOffLeft = LOAD_IMG(@"bar_off_audio_g", @"png");
    fAudioClipBarOffCenter = LOAD_IMG(@"bar_off_audio_c", @"png");
    fAudioClipBarOffRight = LOAD_IMG(@"bar_off_audio_d", @"png");
    fAudioClipBarNegativeLeft = LOAD_IMG(@"bar_neg_audio_g", @"png");
    fAudioClipBarNegativeCenter = LOAD_IMG(@"bar_neg_audio_c", @"png");
    fAudioClipBarNegativeRight = LOAD_IMG(@"bar_neg_audio_d", @"png");
    
    // midi clip bars
    fMidiClipBarOnLeft = LOAD_IMG(@"bar_on_midi_g", @"png");
    fMidiClipBarOnCenter = LOAD_IMG(@"bar_on_midi_c", @"png");
    fMidiClipBarOnRight = LOAD_IMG(@"bar_on_midi_d", @"png");
    fMidiClipBarOffLeft = LOAD_IMG(@"bar_off_midi_g", @"png");
    fMidiClipBarOffCenter = LOAD_IMG(@"bar_off_midi_c", @"png");
    fMidiClipBarOffRight = LOAD_IMG(@"bar_off_midi_d", @"png");
    fMidiClipBarNegativeLeft = LOAD_IMG(@"bar_neg_midi_g", @"png");
    fMidiClipBarNegativeCenter = LOAD_IMG(@"bar_neg_midi_c", @"png");
    fMidiClipBarNegativeRight = LOAD_IMG(@"bar_neg_midi_d", @"png");
    
    // text clip bars
    fTextClipBarOnLeft = LOAD_IMG(@"bar_on_txt_g", @"png");
    fTextClipBarOnCenter = LOAD_IMG(@"bar_on_txt_c", @"png");
    fTextClipBarOnRight = LOAD_IMG(@"bar_on_txt_d", @"png");
    fTextClipBarOffLeft = LOAD_IMG(@"bar_off_txt_g", @"png");
    fTextClipBarOffCenter = LOAD_IMG(@"bar_off_txt_c", @"png");
    fTextClipBarOffRight = LOAD_IMG(@"bar_off_txt_d", @"png");
    fTextClipBarNegativeLeft = LOAD_IMG(@"bar_neg_txt_g", @"png");
    fTextClipBarNegativeCenter = LOAD_IMG(@"bar_neg_txt_c", @"png");
    fTextClipBarNegativeRight = LOAD_IMG(@"bar_neg_txt_d", @"png");

    // video clip bars
    fVideoClipBarOnLeft = LOAD_IMG(@"bar_on_video_g", @"png");
    fVideoClipBarOnCenter = LOAD_IMG(@"bar_on_video_c", @"png");
    fVideoClipBarOnRight = LOAD_IMG(@"bar_on_video_d", @"png");
    fVideoClipBarOffLeft = LOAD_IMG(@"bar_off_video_g", @"png");
    fVideoClipBarOffCenter = LOAD_IMG(@"bar_off_video_c", @"png");
    fVideoClipBarOffRight = LOAD_IMG(@"bar_off_video_d", @"png");
    fVideoClipBarNegativeLeft = LOAD_IMG(@"bar_neg_video_g", @"png");
    fVideoClipBarNegativeCenter = LOAD_IMG(@"bar_neg_video_c", @"png");
    fVideoClipBarNegativeRight = LOAD_IMG(@"bar_neg_video_d", @"png");

//////////////////
    // containers
//////////////////
    // the shadows are the same than for clips
    fContainerGroupBarOnLeft = LOAD_IMG(@"bar_on_group_g", @"png");
    fContainerGroupBarOnCenter = LOAD_IMG(@"bar_on_group_c", @"png");
    fContainerGroupBarOnRight = LOAD_IMG(@"bar_on_group_d", @"png");
    fContainerGroupBarOffLeft = LOAD_IMG(@"bar_off_group_g", @"png");
    fContainerGroupBarOffCenter = LOAD_IMG(@"bar_off_group_c", @"png");
    fContainerGroupBarOffRight = LOAD_IMG(@"bar_off_group_d", @"png");
    fContainerGroupBarNegativeLeft = LOAD_IMG(@"bar_neg_group_g", @"png");
    fContainerGroupBarNegativeCenter = LOAD_IMG(@"bar_neg_group_c", @"png");
    fContainerGroupBarNegativeRight = LOAD_IMG(@"bar_neg_group_d", @"png");

    fContainerProcessorBarOnLeft = LOAD_IMG(@"bar_on_proces_g", @"png");
    fContainerProcessorBarOnCenter = LOAD_IMG(@"bar_on_proces_c", @"png");
    fContainerProcessorBarOnRight = LOAD_IMG(@"bar_on_proces_d", @"png");
    fContainerProcessorBarOffLeft = LOAD_IMG(@"bar_off_proces_g", @"png");
    fContainerProcessorBarOffCenter = LOAD_IMG(@"bar_off_proces_c", @"png");
    fContainerProcessorBarOffRight = LOAD_IMG(@"bar_off_proces_d", @"png");
    fContainerProcessorBarNegativeLeft = LOAD_IMG(@"bar_neg_proces_g", @"png");
    fContainerProcessorBarNegativeCenter = LOAD_IMG(@"bar_neg_proces_c", @"png");
    fContainerProcessorBarNegativeRight = LOAD_IMG(@"bar_neg_proces_d", @"png");
    
    fContainerResizeBarLeft = LOAD_IMG(@"bar_resizing_g", @"png");
    fContainerResizeBarCenter = LOAD_IMG(@"bar_resizing_c", @"png");
    fContainerResizeBarRight = LOAD_IMG(@"bar_resizing_d", @"png");
    
    fContainerButtonArrowHorizontal = LOAD_IMG(@"bt_right_expan", @"png");
    fContainerButtonArrowVertical = LOAD_IMG(@"bt_down_expan", @"png");
    
    fContainerButtonOn = LOAD_IMG(@"bt_on", @"png");
    fContainerButtonOff = LOAD_IMG(@"bt_off", @"png");
    
    fContainerButtonPropertiesOn = LOAD_IMG(@"bt_on_reglage", @"png");
    fContainerButtonPropertiesOff = LOAD_IMG(@"bt_off_reglage", @"png");    

    // buttons
#if 0 //$$$TOOLBOX
    fToolBoxButtonAlignmentOn = LOAD_IMG(@"button_align_on", @"png");
    fToolBoxButtonAlignmentOff = LOAD_IMG(@"button_align_off", @"png");
    fToolBoxButtonAudioOn = LOAD_IMG(@"button_audio_on", @"png");
    fToolBoxButtonAudioOff = LOAD_IMG(@"button_audio_off", @"png");
    fToolBoxButtonChainOn = LOAD_IMG(@"button_chain_on", @"png");
    fToolBoxButtonChainOff = LOAD_IMG(@"button_chain_off", @"png");
    fToolBoxButtonClipBoardOn = LOAD_IMG(@"button_clipboard_on", @"png");
    fToolBoxButtonClipBoardOff = LOAD_IMG(@"button_clipboard_off", @"png");
    fToolBoxButtonColorizeOn = LOAD_IMG(@"button_colorize_on", @"png");
    fToolBoxButtonColorizeOff = LOAD_IMG(@"button_colorize_off", @"png");
    fToolBoxButtonGridOn = LOAD_IMG(@"button_grid_on", @"png");
    fToolBoxButtonGridOff = LOAD_IMG(@"button_grid_off", @"png");
    fToolBoxButtonHistoryOn = LOAD_IMG(@"button_history_on", @"png");
    fToolBoxButtonHistoryOff = LOAD_IMG(@"button_history_off", @"png");
    fToolBoxButtonLockOn = LOAD_IMG(@"button_lock_on", @"png");
    fToolBoxButtonLockOff = LOAD_IMG(@"button_lock_off", @"png");
    fToolBoxButtonMarkerOn = LOAD_IMG(@"button_marker_on", @"png");
    fToolBoxButtonMarkerOff = LOAD_IMG(@"button_marker_off", @"png");
    fToolBoxButtonMidiOn = LOAD_IMG(@"button_midi_on", @"png");
    fToolBoxButtonMidiOff = LOAD_IMG(@"button_midi_off", @"png");
    fToolBoxButtonMixerOn = LOAD_IMG(@"button_mixer_on", @"png");
    fToolBoxButtonMixerOff = LOAD_IMG(@"button_mixer_off", @"png");
    fToolBoxButtonPackOn = LOAD_IMG(@"button_pack_on", @"png");
    fToolBoxButtonPackOff = LOAD_IMG(@"button_pack_off", @"png");
    fToolBoxButtonProcessorOn = LOAD_IMG(@"button_proces_on", @"png");
    fToolBoxButtonProcessorOff = LOAD_IMG(@"button_proces_off", @"png");
    fToolBoxButtonSearchOn = LOAD_IMG(@"button_search_on", @"png");
    fToolBoxButtonSearchOff = LOAD_IMG(@"button_search_off", @"png");
    fToolBoxButtonSnapOn = LOAD_IMG(@"button_snap_on", @"png");
    fToolBoxButtonSnapOff = LOAD_IMG(@"button_snap_off", @"png");
    fToolBoxButtonSortOn = LOAD_IMG(@"button_sort_on", @"png");
    fToolBoxButtonSortOff = LOAD_IMG(@"button_sort_off", @"png");
    fToolBoxButtonTextOn = LOAD_IMG(@"button_text_on", @"png");
    fToolBoxButtonTextOff = LOAD_IMG(@"button_text_off", @"png");
    fToolBoxButtonVideoOn = LOAD_IMG(@"button_video_on", @"png");
    fToolBoxButtonVideoOff = LOAD_IMG(@"button_video_off", @"png");
#endif // 0 //$$$TOOLBOX


    fPlayBarButtonBackOn = LOAD_IMG(@"playbar_button_back_on", @"png");
    fPlayBarButtonBackOff = LOAD_IMG(@"playbar_button_back_off", @"png");
    fPlayBarButtonCameraOn = LOAD_IMG(@"playbar_button_camera_on", @"png");
    fPlayBarButtonCameraOff = LOAD_IMG(@"playbar_button_camera_off", @"png");
    fPlayBarButtonforwardOn = LOAD_IMG(@"playbar_button_forward_on", @"png");
    fPlayBarButtonforwardOff = LOAD_IMG(@"playbar_button_forward_off", @"png");
    fPlayBarButtonLoopOn = LOAD_IMG(@"playbar_button_loop_on", @"png");
    fPlayBarButtonLoopOff = LOAD_IMG(@"playbar_button_loop_off", @"png");
    fPlayBarButtonMarkOn = LOAD_IMG(@"playbar_button_mark_on", @"png");
    fPlayBarButtonMarkOff = LOAD_IMG(@"playbar_button_mark_off", @"png");
    fPlayBarButtonMicroOn = LOAD_IMG(@"playbar_button_micro_on", @"png");
    fPlayBarButtonMicroOff = LOAD_IMG(@"playbar_button_micro_off", @"png");
    fPlayBarButtonMidiOn = LOAD_IMG(@"playbar_button_midi_on", @"png");
    fPlayBarButtonMidiOff = LOAD_IMG(@"playbar_button_midi_off", @"png");
    fPlayBarButtonPauseOn = LOAD_IMG(@"playbar_button_pause_on", @"png");
    fPlayBarButtonPauseOff = LOAD_IMG(@"playbar_button_pause_off", @"png");
    fPlayBarButtonPlayOn = LOAD_IMG(@"playbar_button_play_on", @"png");
    fPlayBarButtonPlayOff = LOAD_IMG(@"playbar_button_play_off", @"png");
    fPlayBarButtonRecordOn = LOAD_IMG(@"playbar_button_record_on", @"png");
    fPlayBarButtonRecordOff = LOAD_IMG(@"playbar_button_record_off", @"png");
    fPlayBarButtonStopOn = LOAD_IMG(@"playbar_button_stop_on", @"png");
    fPlayBarButtonStopOff = LOAD_IMG(@"playbar_button_stop_off", @"png");
    fPlayBarButtonTextOn = LOAD_IMG(@"playbar_button_text_on", @"png");
    fPlayBarButtonTextOff = LOAD_IMG(@"playbar_button_text_off", @"png");
    

/////////////////////////////////////
    // time palette & hyperprism
/////////////////////////////////////

	fTimePaletteBackground = LOAD_IMG(@"BWbackground", @"png");
	fHyperprismTextBoxLeft = LOAD_IMG(@"BWboite_txt_g", @"png");
	fHyperprismTextBoxCenter = LOAD_IMG(@"BWboite_txt_c", @"png");
	fHyperprismTextBoxRight = LOAD_IMG(@"BWboite_txt_d", @"png");
	fHyperPrismLogo = LOAD_IMG(@"BWlogo_hyper", @"png");
	fBlueWindowLogo = LOAD_IMG(@"Hyperprism", @"png");

	fRulerDottedLine = LOAD_IMG(@"RulerDottedLine", @"png");

	[self setRepresentationColor];
	
	[self setHandCursors:YES];

	[self setLocatorIcons:NO];

	[self createRepresentationPartialImages];
	
    [self checkImages];
}

- (NSImage*)getVAOPicture:(int)vao
{
	return fContainerButtonOn;
}


- (void)setLocatorIcons:(BOOL)triangles
{
    NSBundle	*theBundle = [NSBundle mainBundle];
	
	[fRulerPlayLocator release];
	[fRulerLeftLocator release];
	[fRulerRightLocator release];
	[fRulerGenericLocator release];
	if ( triangles )
	{
		fRulerPlayLocator = LOAD_IMG(@"triangle_bleu", @"png");
		fRulerLeftLocator = LOAD_IMG(@"triangle_vert", @"png");
		fRulerRightLocator = LOAD_IMG(@"triangle_rouge", @"png");
		fRulerGenericLocator = LOAD_IMG(@"triangle_noir", @"png");
	}
	else
	{
		fRulerPlayLocator = LOAD_IMG(@"locatplay", @"png");
		fRulerLeftLocator = LOAD_IMG(@"locatstart", @"png");
		fRulerRightLocator = LOAD_IMG(@"locatend", @"png");
		fRulerGenericLocator = LOAD_IMG(@"locatmarker", @"png");
	}
}

- (void)setHandCursors:(BOOL)onOff
{
    NSBundle	*theBundle = [NSBundle mainBundle];
	
	[fResizeUpCursor release];
	[fResizeDownCursor release];
	[fResizeLeftCursor release];
	[fResizeRightCursor release];
	[fMoveOpenCursor release];
	[fMoveCloseCursor release];
	[fMoveCloseCopyCursor release];
	[fResizeDownLeftCursor release];
	[fResizeDownRightCursor release];

	[fTimeStretchLeftCursor release];
	[fTimeStretchRightCursor release];

	[fTrimLeftCursor release];
	[fTrimRightCursor release];
		
	if ( !onOff )
	{
		fResizeUpCursor = [[NSCursor alloc] initWithImage:
			LOAD_IMG(@"uparrow", @"png")
												hotSpot:NSMakePoint(8,8)];
	
		fResizeDownCursor = [[NSCursor alloc] initWithImage:
			LOAD_IMG(@"downarrow", @"png")
													hotSpot:NSMakePoint(8,8)];
	
		fResizeLeftCursor = [[NSCursor alloc] initWithImage:
			LOAD_IMG(@"leftarrow", @"png")
													hotSpot:NSMakePoint(8,8)];
	
		fResizeRightCursor = [[NSCursor alloc] initWithImage:
			LOAD_IMG(@"rightarrow", @"png")
													hotSpot:NSMakePoint(8,8)];
	
		fMoveOpenCursor = [[NSCursor alloc] initWithImage:
			LOAD_IMG(@"move", @"png")
											hotSpot:NSMakePoint(8,8)];

		fResizeDownLeftCursor = [[NSCursor alloc] initWithImage:
			LOAD_IMG(@"leftarrow", @"png")
													hotSpot:NSMakePoint(8,8)];

		fResizeDownRightCursor = [[NSCursor alloc] initWithImage:
			LOAD_IMG(@"rightarrow", @"png")
													hotSpot:NSMakePoint(8,8)];
	}
	else
	{
		fResizeUpCursor = [[NSCursor alloc] initWithImage:
			LOAD_IMG(@"curhndpointdown", @"png")
																				hotSpot:NSMakePoint(12,15)];

		fResizeDownCursor = [[NSCursor alloc] initWithImage:
			LOAD_IMG(@"curhndpointup", @"png")
																					 hotSpot:NSMakePoint(4,0)];

		fResizeLeftCursor = [[NSCursor alloc] initWithImage:
			LOAD_IMG(@"curhndpointright", @"png")
																					 hotSpot:NSMakePoint(15,12)];

		fResizeRightCursor = [[NSCursor alloc] initWithImage:
			LOAD_IMG(@"curhndpointleft", @"png")
																					  hotSpot:NSMakePoint(0,12)];

		fMoveOpenCursor = [[NSCursor alloc] initWithImage:
			LOAD_IMG(@"curhndopen", @"png")
																		 hotSpot:NSMakePoint(8,8)];

		fMoveCloseCursor = [[NSCursor alloc] initWithImage:
			LOAD_IMG(@"curhndclosed", @"png")
																		 hotSpot:NSMakePoint(8,8)];

		fMoveCloseCopyCursor = [[NSCursor alloc] initWithImage:
			LOAD_IMG(@"curhndclosedplus", @"png")
																								   hotSpot:NSMakePoint(8,8)];		

		fResizeDownLeftCursor = [[NSCursor alloc] initWithImage:
			LOAD_IMG(@"curhndpointrightup", @"png")
																		hotSpot:NSMakePoint(15, 0)];

		fResizeDownRightCursor = [[NSCursor alloc] initWithImage:
			LOAD_IMG(@"curhndpointleftup", @"png")
																		hotSpot:NSMakePoint(0, 0)];
		
	}

	fTimeStretchLeftCursor = [[NSCursor alloc] initWithImage:
		LOAD_IMG(@"cursorstretchleft", @"png")
																		hotSpot:NSMakePoint(0,8)];

	fTimeStretchRightCursor = [[NSCursor alloc] initWithImage:
		LOAD_IMG(@"cursorstretchright", @"png")
																		hotSpot:NSMakePoint(15,8)];
	

	fTrimLeftCursor = [[NSCursor alloc] initWithImage:
		LOAD_IMG(@"cursortrimleft", @"png")
																		hotSpot:NSMakePoint(8,8)];
	
	fTrimRightCursor = [[NSCursor alloc] initWithImage:
		LOAD_IMG(@"cursortrimright", @"png")
																		hotSpot:NSMakePoint(8,8)];

	fCarretCursor = [[NSCursor alloc] initWithImage:
		LOAD_IMG(@"cursorcarret", @"png")
																		hotSpot:NSMakePoint(8,8)];

	fLeftCarretCursor = [[NSCursor alloc] initWithImage:
		LOAD_IMG(@"cursorleftcarret", @"png")
																		hotSpot:NSMakePoint(8,8)];

	fRightCarretCursor = [[NSCursor alloc] initWithImage:
		LOAD_IMG(@"cursorrightcarret", @"png")
																		hotSpot:NSMakePoint(8,8)];
																		
	fLeftRightCarretCursor = [[NSCursor alloc] initWithImage:
		LOAD_IMG(@"cursorextendcarret", @"png")
																		hotSpot:NSMakePoint(8,8)];

}


- (void)setRepresentationColor
{
	fAudioClipBrightColor = [[NSColor colorWithCalibratedRed:.827 green:.914 blue:.973 alpha:1] retain];
	fAudioClipDimColor = [[NSColor colorWithCalibratedRed:.913 green:.937 blue:.984 alpha:1] retain];
	fAudioClipNegativeColor = [NegativeOfColor(fAudioClipBrightColor) retain];

	fVideoClipBrightColor = [[NSColor colorWithCalibratedRed:.796 green:.835 blue:.929 alpha:1] retain];
	fVideoClipDimColor = [[NSColor colorWithCalibratedRed:.914 green:.929 blue:.973 alpha:1] retain];
	fVideoClipNegativeColor = [NegativeOfColor(fVideoClipBrightColor) retain];

	fTextClipBrightColor = [[NSColor colorWithCalibratedRed:.839 green:.839 blue:.839 alpha:1] retain];
	fTextClipDimColor = [[NSColor colorWithCalibratedRed:.839 green:.839 blue:.839 alpha:1] retain];
	fTextClipNegativeColor = [NegativeOfColor(fTextClipBrightColor) retain];

	fMidiClipBrightColor = [[NSColor colorWithCalibratedRed:.977 green:.886 blue:.706 alpha:1] retain];
	fMidiClipDimColor = [[NSColor colorWithCalibratedRed:.988 green:.945 blue:.855 alpha:1] retain];
	fMidiClipNegativeColor = [NegativeOfColor(fMidiClipBrightColor) retain];

	fProcessorBrightColor = [[NSColor colorWithCalibratedRed:1 green:.851 blue:.757 alpha:1] retain];
	fProcessorDimColor= [[NSColor colorWithCalibratedRed:1 green:.925 blue:.878 alpha:1] retain];
	fProcessorNegativeColor = [NegativeOfColor(fProcessorBrightColor) retain];

	fGroupBrightColor = [[NSColor colorWithCalibratedRed:.788 green:.843 blue:.835 alpha:1] retain];
	fGroupDimColor = [[NSColor colorWithCalibratedRed:.894 green:.922 blue:.918 alpha:1] retain];
	fGroupNegativeColor = [NegativeOfColor(fGroupBrightColor) retain];
}

- (void)createRepresentationPartialImages
{

	// Audio Clip
	[self createImagesLeft:&fLeftAudioClipBrightTitleBarImage
					   center:&fCenterAudioClipBrightTitleBarImage
						right:&fRightAudioClipBrightTitleBarImage
			 fromLeftBarImage:fAudioClipBarOnLeft
					   center:fAudioClipBarOnCenter
						right:fAudioClipBarOnRight
						color:fAudioClipBrightColor];

	[self createImagesLeft:&fLeftAudioClipDimTitleBarImage
					center:&fCenterAudioClipDimTitleBarImage
					 right:&fRightAudioClipDimTitleBarImage
		  fromLeftBarImage:fAudioClipBarOffLeft
					center:fAudioClipBarOffCenter
					 right:fAudioClipBarOffRight
					 color:fAudioClipDimColor];
/* this one is all negative
	[self createImagesLeft:&fLeftAudioClipNegativeTitleBarImage
					center:&fCenterAudioClipNegativeTitleBarImage
					 right:&fRightAudioClipNegativeTitleBarImage
		  fromLeftBarImage:fAudioClipBarNegativeLeft
					center:fAudioClipBarNegativeCenter
					 right:fAudioClipBarNegativeRight
					 color:fAudioClipNegativeColor];
*/
	[self createImagesLeft:&fLeftAudioClipNegativeTitleBarImage
					center:&fCenterAudioClipNegativeTitleBarImage
					 right:&fRightAudioClipNegativeTitleBarImage
		  fromLeftBarImage:fAudioClipBarOnLeft
					center:fAudioClipBarOnCenter
					 right:fAudioClipBarOnRight
					 color:fAudioClipNegativeColor];


	// video clips
	[self createImagesLeft:&fLeftVideoClipBrightTitleBarImage
										 center:&fCenterVideoClipBrightTitleBarImage
										  right:&fRightVideoClipBrightTitleBarImage
				fromLeftBarImage:fVideoClipBarOnLeft
										 center:fVideoClipBarOnCenter
										  right:fVideoClipBarOnRight
										  color:fVideoClipBrightColor];

	[self createImagesLeft:&fLeftVideoClipDimTitleBarImage
										 center:&fCenterVideoClipDimTitleBarImage
										  right:&fRightVideoClipDimTitleBarImage
				fromLeftBarImage:fVideoClipBarOffLeft
										 center:fVideoClipBarOffCenter
										  right:fVideoClipBarOffRight
										  color:fVideoClipDimColor];

/* this one is all negative
	[self createImagesLeft:&fLeftVideoClipNegativeTitleBarImage
										 center:&fCenterVideoClipNegativeTitleBarImage
										  right:&fRightVideoClipNegativeTitleBarImage
				fromLeftBarImage:fVideoClipBarNegativeLeft
										 center:fVideoClipBarNegativeCenter
										  right:fVideoClipBarNegativeRight
										  color:fVideoClipNegativeColor];
*/
	[self createImagesLeft:&fLeftVideoClipNegativeTitleBarImage
										 center:&fCenterVideoClipNegativeTitleBarImage
										  right:&fRightVideoClipNegativeTitleBarImage
				fromLeftBarImage:fVideoClipBarOnLeft
										 center:fVideoClipBarOnCenter
										  right:fVideoClipBarOnRight
										  color:fVideoClipNegativeColor];


	// text clips

	[self createImagesLeft:&fLeftTextClipBrightTitleBarImage
										 center:&fCenterTextClipBrightTitleBarImage
										  right:&fRightTextClipBrightTitleBarImage
				fromLeftBarImage:fTextClipBarOnLeft
										 center:fTextClipBarOnCenter
										  right:fTextClipBarOnRight
												color:fTextClipBrightColor];

	[self createImagesLeft:&fLeftTextClipDimTitleBarImage
										 center:&fCenterTextClipDimTitleBarImage
										  right:&fRightTextClipDimTitleBarImage
				fromLeftBarImage:fTextClipBarOffLeft
										 center:fTextClipBarOffCenter
										  right:fTextClipBarOffRight
										  color:fTextClipDimColor];

/* this one is all negative
	[self createImagesLeft:&fLeftTextClipNegativeTitleBarImage
										 center:&fCenterTextClipNegativeTitleBarImage
										  right:&fRightTextClipNegativeTitleBarImage
				fromLeftBarImage:fTextClipBarNegativeLeft
										 center:fTextClipBarNegativeCenter
										  right:fTextClipBarNegativeRight
										  color:fTextClipNegativeColor];
*/
	[self createImagesLeft:&fLeftTextClipNegativeTitleBarImage
										 center:&fCenterTextClipNegativeTitleBarImage
										  right:&fRightTextClipNegativeTitleBarImage
				fromLeftBarImage:fTextClipBarOnLeft
										 center:fTextClipBarOnCenter
										  right:fTextClipBarOnRight
										  color:fTextClipNegativeColor];


	// midi clips


	[self createImagesLeft:&fLeftMidiClipBrightTitleBarImage
										 center:&fCenterMidiClipBrightTitleBarImage
										  right:&fRightMidiClipBrightTitleBarImage
				fromLeftBarImage:fMidiClipBarOnLeft
										 center:fMidiClipBarOnCenter
										  right:fMidiClipBarOnRight
										  color:fMidiClipBrightColor];

	[self createImagesLeft:&fLeftMidiClipDimTitleBarImage
										 center:&fCenterMidiClipDimTitleBarImage
										  right:&fRightMidiClipDimTitleBarImage
				fromLeftBarImage:fMidiClipBarOffLeft
										 center:fMidiClipBarOffCenter
										  right:fMidiClipBarOffRight
										  color:fMidiClipDimColor];

/* this one is all negative
	[self createImagesLeft:&fLeftMidiClipNegativeTitleBarImage
										 center:&fCenterMidiClipNegativeTitleBarImage
										  right:&fRightMidiClipNegativeTitleBarImage
				fromLeftBarImage:fMidiClipBarNegativeLeft
										 center:fMidiClipBarNegativeCenter
										  right:fMidiClipBarNegativeRight
										  color:fMidiClipNegativeColor];
*/
	[self createImagesLeft:&fLeftMidiClipNegativeTitleBarImage
										 center:&fCenterMidiClipNegativeTitleBarImage
										  right:&fRightMidiClipNegativeTitleBarImage
				fromLeftBarImage:fMidiClipBarOnLeft
										 center:fMidiClipBarOnCenter
										  right:fMidiClipBarOnRight
										  color:fMidiClipNegativeColor];

	// processors

	[self createImagesLeft:&fLeftProcessorBrightTitleBarImage
										 center:&fCenterProcessorBrightTitleBarImage
										  right:&fRightProcessorBrightTitleBarImage
				fromLeftBarImage:fContainerProcessorBarOnLeft
										 center:fContainerProcessorBarOnCenter
										  right:fContainerProcessorBarOnRight
										  color:fProcessorBrightColor];

	[self createImagesLeft:&fLeftProcessorDimTitleBarImage
										 center:&fCenterProcessorDimTitleBarImage
										  right:&fRightProcessorDimTitleBarImage
				fromLeftBarImage:fContainerProcessorBarOffLeft
										 center:fContainerProcessorBarOffCenter
										  right:fContainerProcessorBarOffRight
										  color:fProcessorDimColor];
	
/* this one is all negative
	[self createImagesLeft:&fLeftProcessorNegativeTitleBarImage
										 center:&fCenterProcessorNegativeTitleBarImage
										  right:&fRightProcessorNegativeTitleBarImage
				fromLeftBarImage:fContainerProcessorBarNegativeLeft
										 center:fContainerProcessorBarNegativeCenter
										  right:fContainerProcessorBarNegativeRight
										  color:fProcessorNegativeColor];
*/
	[self createImagesLeft:&fLeftProcessorNegativeTitleBarImage
										 center:&fCenterProcessorNegativeTitleBarImage
										  right:&fRightProcessorNegativeTitleBarImage
				fromLeftBarImage:fContainerProcessorBarOnLeft
										 center:fContainerProcessorBarOnCenter
										  right:fContainerProcessorBarOnRight
										  color:fProcessorNegativeColor];
	
	// Group
	[self createImagesLeft:&fLeftGroupBrightTitleBarImage
					center:&fCenterGroupBrightTitleBarImage
					right:&fRightGroupBrightTitleBarImage
		  fromLeftBarImage:fContainerGroupBarOnLeft
					center:fContainerGroupBarOnCenter
					right:fContainerGroupBarOnRight
					color:fGroupBrightColor];

	[self createImagesLeft:&fLeftGroupDimTitleBarImage
					center:&fCenterGroupDimTitleBarImage
					 right:&fRightGroupDimTitleBarImage
		  fromLeftBarImage:fContainerGroupBarOffLeft
					center:fContainerGroupBarOffCenter
					 right:fContainerGroupBarOffRight
					 color:fGroupDimColor];

/* this one is all negative
	[self createImagesLeft:&fLeftGroupNegativeTitleBarImage
					center:&fCenterGroupNegativeTitleBarImage
					 right:&fRightGroupNegativeTitleBarImage
		  fromLeftBarImage:fContainerGroupBarNegativeLeft
					center:fContainerGroupBarNegativeCenter
					 right:fContainerGroupBarNegativeRight
					 color:fGroupNegativeColor];
*/
	[self createImagesLeft:&fLeftGroupNegativeTitleBarImage
					center:&fCenterGroupNegativeTitleBarImage
					 right:&fRightGroupNegativeTitleBarImage
		  fromLeftBarImage:fContainerGroupBarOnLeft
					center:fContainerGroupBarOnCenter
					 right:fContainerGroupBarOnRight
					 color:fGroupNegativeColor];

/*
	[self createImagesLeft:&fLeftXXXXBrightTitleBarImage
								center:&fCenterXXXXBrightTitleBarImage
								 right:&fRightXXXXBrightTitleBarImage
			 fromLeftBarImage:fYYYYBarOnLeft
								center:fYYYYBarOnCenter
								 right:fYYYYBarOnRight
								 color:[NSColor colorWithCalibratedRed:.788 green:.843 blue:.835 alpha:1]];

	[self createImagesLeft:&fLeftXXXXDimTitleBarImage
								center:&fCenterXXXXDimTitleBarImage
								 right:&fRightXXXXDimTitleBarImage
			 fromLeftBarImage:fYYYYBarOffLeft
								center:fYYYYBarOffCenter
								 right:fYYYYBarOffRight
								 color:[NSColor colorWithCalibratedRed:.788 green:.843 blue:.835 alpha:1]];
 */

	NSBezierPath *aPath;
	
	fTrimIndicatorRight = [[NSImage alloc] initWithSize:NSMakeSize(10, 10)];
	[fTrimIndicatorRight lockFocus];
		aPath = [NSBezierPath bezierPath];
		[aPath moveToPoint:NSMakePoint(0, 2 )];
		[aPath lineToPoint:NSMakePoint(0, 10 )]; 
		[aPath lineToPoint:NSMakePoint(8, 6 )];
		[aPath closePath];
		[[NSColor redColor] set];
		[aPath fill];
	[fTrimIndicatorRight unlockFocus];
	
	fTrimIndicatorLeft = [[NSImage alloc] initWithSize:NSMakeSize(10, 10)];
	[fTrimIndicatorLeft lockFocus];
		aPath = [NSBezierPath bezierPath];
		[aPath moveToPoint:NSMakePoint(10, 2 )];
		[aPath lineToPoint:NSMakePoint(10, 10 )]; 
		[aPath lineToPoint:NSMakePoint(2, 6 )];
		[aPath closePath];
		[[NSColor redColor] set];
		[aPath fill];
	[fTrimIndicatorLeft unlockFocus];
	
	fNonTrimIndicatorRight = [[NSImage alloc] initWithSize:NSMakeSize(10, 10)];
	[fNonTrimIndicatorRight lockFocus];
		aPath = [NSBezierPath bezierPath];
		[aPath moveToPoint:NSMakePoint(0, 2 )];
		[aPath lineToPoint:NSMakePoint(0, 10 )]; 
		[aPath lineToPoint:NSMakePoint(8, 6 )];
		[aPath closePath];
		[[NSColor grayColor] set];
		[aPath fill];
	[fNonTrimIndicatorRight unlockFocus];
	
	fNonTrimIndicatorLeft = [[NSImage alloc] initWithSize:NSMakeSize(10, 10)];
	[fNonTrimIndicatorLeft lockFocus];
		aPath = [NSBezierPath bezierPath];
		[aPath moveToPoint:NSMakePoint(10, 2 )];
		[aPath lineToPoint:NSMakePoint(10, 10 )]; 
		[aPath lineToPoint:NSMakePoint(2, 6 )];
		[aPath closePath];
		[[NSColor grayColor] set];
		[aPath fill];
	[fNonTrimIndicatorLeft unlockFocus];
	
	fResizeIndicatorDownRight = [[NSImage alloc] initWithSize:NSMakeSize(10, 10)];
	[fResizeIndicatorDownRight lockFocus];
		aPath = [NSBezierPath bezierPath];
		[aPath moveToPoint:NSMakePoint(2,2)];
		[aPath lineToPoint:NSMakePoint(10,10)];
		[aPath lineToPoint:NSMakePoint(10,2)];
		[aPath closePath];
		[[NSColor grayColor] set];
		[aPath fill];
		aPath = [NSBezierPath bezierPath];
		[aPath moveToPoint:NSMakePoint(4,3)];
		[aPath lineToPoint:NSMakePoint(9,8)];
		[aPath lineToPoint:NSMakePoint(9,3)];
		[aPath closePath];
		[[NSColor whiteColor] set];
		[aPath fill];
	[fResizeIndicatorDownRight unlockFocus];
	
	fResizeIndicatorDownLeft = [[NSImage alloc] initWithSize:NSMakeSize(10, 10)];
	[fResizeIndicatorDownLeft lockFocus];
		aPath = [NSBezierPath bezierPath];
		[aPath moveToPoint:NSMakePoint(2,2)];
		[aPath lineToPoint:NSMakePoint(2,10)];
		[aPath lineToPoint:NSMakePoint(10,2)];
		[aPath closePath];
		[[NSColor grayColor] set];
		[aPath fill];
		aPath = [NSBezierPath bezierPath];
		[aPath moveToPoint:NSMakePoint(3,3)];
		[aPath lineToPoint:NSMakePoint(3,8)];
		[aPath lineToPoint:NSMakePoint(8,3)];
		[aPath closePath];
		[[NSColor whiteColor] set];
		[aPath fill];
	[fResizeIndicatorDownLeft unlockFocus];
	
	fResizeIndicatorUpRight = [[NSImage alloc] initWithSize:NSMakeSize(10, 10)];
	[fResizeIndicatorUpRight lockFocus];
		aPath = [NSBezierPath bezierPath];
		[aPath moveToPoint:NSMakePoint(2,10)];
		[aPath lineToPoint:NSMakePoint(10,10)];
		[aPath lineToPoint:NSMakePoint(10,2)];
		[aPath closePath];
		[[NSColor grayColor] set];
		[aPath fill];
		aPath = [NSBezierPath bezierPath];
		[aPath moveToPoint:NSMakePoint(4,9)];
		[aPath lineToPoint:NSMakePoint(9,9)];
		[aPath lineToPoint:NSMakePoint(9,4)];
		[aPath closePath];
		[[NSColor whiteColor] set];
		[aPath fill];
	[fResizeIndicatorUpRight unlockFocus];
	
	fResizeIndicatorUpLeft = [[NSImage alloc] initWithSize:NSMakeSize(10, 10)];
	[fResizeIndicatorUpLeft lockFocus];
		aPath = [NSBezierPath bezierPath];
		[aPath moveToPoint:NSMakePoint(2,2)];
		[aPath lineToPoint:NSMakePoint(2,10)];
		[aPath lineToPoint:NSMakePoint(10,10)];
		[aPath closePath];
		[[NSColor grayColor] set];
		[aPath fill];
		aPath = [NSBezierPath bezierPath];
		[aPath moveToPoint:NSMakePoint(3,4)];
		[aPath lineToPoint:NSMakePoint(3,9)];
		[aPath lineToPoint:NSMakePoint(9,9)];
		[aPath closePath];
		[[NSColor whiteColor] set];
		[aPath fill];
	[fResizeIndicatorUpLeft unlockFocus];
	
	float checkboardsize = 10;
	fTextClipCheckboard = [[NSImage alloc] initWithSize:NSMakeSize(2*checkboardsize, 2*checkboardsize)];
	[fTextClipCheckboard lockFocus];
		[[NSColor whiteColor] set];
		[NSBezierPath fillRect:NSMakeRect(0,0, checkboardsize, checkboardsize)];
		[NSBezierPath fillRect:NSMakeRect(checkboardsize,checkboardsize, checkboardsize+1, checkboardsize+1)];
		[fTextClipBrightColor set];
		[NSBezierPath fillRect:NSMakeRect(0,checkboardsize, checkboardsize+1, checkboardsize+1)];
		[NSBezierPath fillRect:NSMakeRect(checkboardsize,0, checkboardsize+1, checkboardsize+1)];
	[fTextClipCheckboard unlockFocus];
	
	
	fTextClipCheckboardNegative = [[NSImage alloc] initWithSize:NSMakeSize(2*checkboardsize, 2*checkboardsize)];
	[fTextClipCheckboardNegative lockFocus];
		[[NSColor blackColor] set];
		[NSBezierPath fillRect:NSMakeRect(0,0, checkboardsize, checkboardsize)];
		[NSBezierPath fillRect:NSMakeRect(checkboardsize,checkboardsize, checkboardsize+1, checkboardsize+1)];
		[NegativeOfColor(fTextClipBrightColor) set];
		[NSBezierPath fillRect:NSMakeRect(0,checkboardsize, checkboardsize, checkboardsize+1)];
		[NSBezierPath fillRect:NSMakeRect(checkboardsize,0, checkboardsize+1, checkboardsize)];
	[fTextClipCheckboardNegative unlockFocus];
}

- (void)createImagesLeft:(NSImage**)leftImage center:(NSImage**)centerImage right:(NSImage**)rightImage
				   fromLeftBarImage:(NSImage*)leftBarImage center:(NSImage*)centerBarImage right:(NSImage*)rightBarImage
							color:(NSColor*)bgcolor
{
//	NSColor		*white = [NSColor whiteColor];
//	NSColor		*someGray = [NSColor colorWithCalibratedRed:.699 green:.699 blue:.699 alpha:1.0f];

	*leftImage = [[NSImage alloc] initWithSize:NSMakeSize([leftBarImage size].width, kRepTitleHeightTemplate)];
	[*leftImage lockFocus];
	{
		// draw the background color
		[bgcolor set];
		[NSBezierPath fillRect:NSMakeRect(0,0, [*leftImage size].width, kRepTitleHeightTemplate )];

		// draw the left bar image
		[leftBarImage compositeToPoint:NSMakePoint(0, kRepTitleHeightTemplate-[leftBarImage size].height) operation:NSCompositeCopy];

		// draw the bar shadow
		[fClipInternalShadowLeft compositeToPoint:
			NSMakePoint(0,(kRepTitleHeightTemplate-[centerBarImage size].height-[fClipInternalShadowCenter size].height))
											 operation:NSCompositeSourceAtop];

		[fClipInternalShadowCenter drawInRect:
			NSMakeRect([fClipInternalShadowLeft size].width, (kRepTitleHeightTemplate-[leftBarImage size].height) - [fClipInternalShadowCenter size].height
			  ,[*leftImage size].width - [fClipInternalShadowLeft size].width, [*leftImage size].height)
			fromRect:NSMakeRect(0,0,1,kRepTitleHeightTemplate) operation:NSCompositeSourceAtop fraction:1];

	}
	[*leftImage unlockFocus];
	


	
	*centerImage = [[NSImage alloc] initWithSize:NSMakeSize(1, kRepTitleHeightTemplate)];
	[*centerImage lockFocus];
	{
		// draw the background color
		[bgcolor set];
		[NSBezierPath fillRect:NSMakeRect(0,0, 1, kRepTitleHeightTemplate )];
	
		// draw the bar
		[centerBarImage compositeToPoint:NSMakePoint(0, kRepTitleHeightTemplate-[centerBarImage size].height) operation:NSCompositeCopy];
	
		// draw the bar shadow
		[fClipInternalShadowCenter compositeToPoint:
			NSMakePoint(0,(kRepTitleHeightTemplate-[centerBarImage size].height-[fClipInternalShadowCenter size].height))
			operation:NSCompositeSourceAtop];
	
	}
	[*centerImage unlockFocus];



	*rightImage = [[NSImage alloc] initWithSize:NSMakeSize([rightBarImage size].width, kRepTitleHeightTemplate)];
	[*rightImage lockFocus];
	{
		// draw the background color
		[bgcolor set];
		[NSBezierPath fillRect:NSMakeRect(0,0, [rightBarImage size].width, kRepTitleHeightTemplate )];

		// draw the left bar image
		[rightBarImage compositeToPoint:NSMakePoint(0, kRepTitleHeightTemplate-[rightBarImage size].height) operation:NSCompositeCopy];

		// draw the bar shadow
		[fClipInternalShadowCenter drawInRect:
			NSMakeRect(0,((kRepTitleHeightTemplate-[rightBarImage size].height)-[fClipInternalShadowRight size].height),
			  [*rightImage size].width - [fClipInternalShadowRight size].width, [fClipInternalShadowCenter size].height)
				fromRect:NSMakeRect(0,0,1,[fClipInternalShadowCenter size].height) operation:NSCompositeSourceAtop fraction:1];

		[fClipInternalShadowRight compositeToPoint:
			NSMakePoint([*rightImage size].width - [fClipInternalShadowRight size].width,
			   (kRepTitleHeightTemplate-[rightBarImage size].height-[fClipInternalShadowRight size].height))
				 operation:NSCompositeSourceAtop];

	}
	[*rightImage unlockFocus];
	
}

@end
