/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
#import "MIImportPanelController.h"

@implementation MIImportPanelController

+ (MIImportPanelController*)sharedImportPanelController {
	
    static id sharedImportPanelController = nil;
    if ( sharedImportPanelController == nil ) {
        sharedImportPanelController = [[self alloc] init];
    }
	
	[sharedImportPanelController setDelegate:nil];
	[sharedImportPanelController setPrompt:@"Select the media files to import :"];
	[sharedImportPanelController setAllowsMultipleSelection:NO];
	
    return sharedImportPanelController;
}

- (id)init {
	if (( self = [super initWithWindowNibName:@"Import"] )) {
		fIgnoreNotification = NO;
		[self loadWindow];
	}
	return self;
}

- (void)awakeFromNib {
	[[fTableView tableColumnWithIdentifier:@"icon"] setDataCell:[[[NSImageCell alloc] init] autorelease]];
}

- (void)buildPopupMenu {
	unsigned int i;
	
	[fDocumentPopupButton removeAllItems];

	for ( i = 0 ; i < [fDocuments count]; i++ ) {
		[fDocumentPopupButton addItemWithTitle:[[[fDocuments objectAtIndex:i] lastPathComponent] stringByDeletingPathExtension]];
    }
	[fDocumentPopupButton selectItemAtIndex:0];
}

- (IBAction)select:(id)sender {
	[fTableView reloadData];
	
	fIgnoreNotification = YES;
	
	[fTableView deselectAll:nil];
	
	NSArray *files = [fFileInfo objectForKey:[fDocuments objectAtIndex:[fDocumentPopupButton indexOfSelectedItem]]];
	unsigned int count = [files count];
	unsigned int index;

	for ( index = 0 ; index < count ; index++ ) {
		if ( [[[files objectAtIndex:index] objectForKey:@"selected"] boolValue] )
			[fTableView selectRow:index byExtendingSelection:YES];
	}
	fIgnoreNotification = NO;
	[self tableViewSelectionDidChange:nil];
}

- (int)numberOfRowsInTableView:(NSTableView *)aTableView {
	return [[fFileInfo objectForKey:[fDocuments objectAtIndex:[fDocumentPopupButton indexOfSelectedItem]]] count];
}

- (id)tableView:(NSTableView *)aTableView objectValueForTableColumn:(NSTableColumn *)aTableColumn row:(int)rowIndex {
	return [[[fFileInfo objectForKey:[fDocuments objectAtIndex:[fDocumentPopupButton indexOfSelectedItem]]] objectAtIndex:rowIndex] objectForKey:[aTableColumn identifier]];
}

- (void)tableViewSelectionDidChange:(NSNotification*)notification {
	if ( !fIgnoreNotification ) {
		NSArray *files = [fFileInfo objectForKey:[fDocuments objectAtIndex:[fDocumentPopupButton indexOfSelectedItem]]];
		unsigned int count = [files count];
		unsigned int index;

		for ( index = 0 ; index < count ; index++ ) {
			[[files objectAtIndex:index] setObject:[NSNumber numberWithBool:[fTableView isRowSelected:index]] forKey:@"selected"];
		}

		if ( fMovieView ) {
			[fMovieView removeFromSuperview];
			[fMovieView setMovie:nil];
			[fMovieSuperview setNeedsDisplay:YES];
			[fMovieView release];
			fMovieView = nil;
		}

		if ( [fTableView numberOfSelectedRows] == 1 ) {
			NSString *documentPath = [fDocuments objectAtIndex:[fDocumentPopupButton indexOfSelectedItem]];

			NSString *movieName = [[[fFileInfo objectForKey:documentPath] objectAtIndex:[fTableView selectedRow]] objectForKey:@"path"];
			NSURL *url = [NSURL fileURLWithPath:[documentPath stringByAppendingPathComponent:movieName]];
			NSMovie *movie = [[NSMovie alloc] initWithURL:url byReference:NO];

			fMovieView = [[NSMovieView alloc] initWithFrame:NSMakeRect( 20, 20, 186, 138 )];
			[fMovieView setMovie:movie];
			[movie release];

			[fMovieView setLoopMode:NSQTMovieLoopingPlayback];
			[fMovieView showController:NO adjustingSize:YES];

			[fMovieSuperview addSubview:fMovieView];

			[fMovieView start:nil];
		}
	}
}

- (BOOL)importPanel:(id)sender shouldShowFilename:(NSString *)filename {
	return NO;
}

- (void)cacheFileInformation {
	unsigned int index;
	NSFileManager *fileManager = [NSFileManager defaultManager];
	NSWorkspace *workspace = [NSWorkspace sharedWorkspace];
	NSString *documentPath;

	if ( fDelegate ) {
		if ( ![fDelegate respondsToSelector:@selector(importPanel:shouldShowFilename:)] ) {
			fDelegate = nil;
		}
	}
	
	fFileInfo = [[NSMutableDictionary alloc] init];

	for ( index = 0 ; index < [fDocuments count] ; index++ ) {

		documentPath = [fDocuments objectAtIndex:index];
		
		NSMutableArray *files = [NSMutableArray array];
		
		NSArray *unfilteredContents = [fileManager directoryContentsAtPath:documentPath];
		
		if ( fDelegate ) {
			NSEnumerator *enumerator = [unfilteredContents objectEnumerator];
			NSString *path;

			while (( path = [enumerator nextObject] )) {
				if ( ![path isEqualToString:@".DS_Store"] ) {
					if ( [fDelegate importPanel:self shouldShowFilename:[documentPath stringByAppendingPathComponent:path]] ) {
						[files addObject:path];
					}
				}
			}
		} else {
			NSEnumerator *enumerator = [unfilteredContents objectEnumerator];
			NSString *path;

			while (( path = [enumerator nextObject] )) {
				if ( ![path isEqualToString:@".DS_Store"] ) {
					[files addObject:path];
				}
			}
		}

		NSMutableArray *info = [NSMutableArray arrayWithCapacity:[files count]];
		NSEnumerator *enumerator = [files objectEnumerator];
		NSString *path;
		
		while (( path = [enumerator nextObject] )) {
			NSMutableDictionary *fileDesc = [NSMutableDictionary dictionary];
			[fileDesc setObject:path forKey:@"path"];
			[fileDesc setObject:[workspace iconForFile:[documentPath stringByAppendingPathComponent:path]] forKey:@"icon"];
			[fileDesc setObject:[NSNumber numberWithBool:NO] forKey:@"selected"];
			[info addObject:fileDesc];
		}

		[fFileInfo setObject:info forKey:documentPath];
	}
}

- (void)beginSheetForDocuments:(NSArray *)documents modalForWindow:(NSWindow *)docWindow modalDelegate:(id)modalDelegate didEndSelector:(SEL)didEndSelector contextInfo:(void *)contextInfo {
	fDocuments = [documents retain];
	fModalDelegate = modalDelegate;
	fDidEndSelector = didEndSelector;
	
	id sheet = [self window];
	[self buildPopupMenu];
	[self cacheFileInformation];
	[fTableView reloadData];
	
	[NSApp beginSheet:sheet modalForWindow:docWindow modalDelegate:self didEndSelector:@selector(panelDidEnd:returnCode:contextInfo:) contextInfo:contextInfo];
}


- (void)panelDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void  *)contextInfo {
	NSMethodSignature *signature = [fModalDelegate methodSignatureForSelector:fDidEndSelector];
	NSInvocation *invocation = [NSInvocation invocationWithMethodSignature:signature];
	[invocation setTarget:fModalDelegate];
	[invocation setSelector:fDidEndSelector];
	[invocation setArgument:&self atIndex:2];
	[invocation setArgument:&returnCode atIndex:3];
	[invocation setArgument:&contextInfo atIndex:4];
	[invocation setArgument:&contextInfo atIndex:4];
	[invocation invoke];
	[fDocuments release];
	fDocuments = nil;
	[fFileInfo release];
	fFileInfo = nil;
	if ( fMovieView ) {
		[fMovieView removeFromSuperview];
		[fMovieView setMovie:nil];
		[fMovieSuperview setNeedsDisplay:YES];
		[fMovieView release];
		fMovieView = nil;
	}	
}


- (void)setPrompt:(NSString *)prompt {
	[fPromptTextField setStringValue:prompt];
}

- (NSString *)prompt {
	return [fPromptTextField stringValue];
}

- (void)setDelegate:(id)anObject {
	fDelegate = anObject;
}

- (BOOL)allowsMultipleSelection {
	return [fTableView allowsMultipleSelection];
}

- (void)setAllowsMultipleSelection:(BOOL)flag {
	[fTableView setAllowsMultipleSelection:flag];
}

- (NSArray*)fileNames {
	if ( fDocuments ) {
		NSMutableArray *result = [NSMutableArray array];
		unsigned docIndex;
		unsigned fileIndex;

		for ( docIndex = 0 ; docIndex < [fDocuments count] ; docIndex++ ) {
			NSString* docPath = [fDocuments objectAtIndex:docIndex];
			NSArray* files = [fFileInfo objectForKey:docPath];
			for ( fileIndex = 0 ; fileIndex < [files count]; fileIndex++ ) {
				NSDictionary *info = [files objectAtIndex:fileIndex];
				if ( [[info objectForKey:@"selected"] boolValue] ) {
					[result addObject:[docPath stringByAppendingPathComponent:[info objectForKey:@"path"]]];
				}
			}
		}
		return result;
	} else return nil;
}

- (IBAction)import:(id)sender {
	[[self window] orderOut:sender];
	[NSApp endSheet:[self window] returnCode:NSOKButton];
}

- (IBAction)cancel:(id)sender {
	[[self window] orderOut:sender];
	[NSApp endSheet:[self window] returnCode:NSCancelButton];
}

- (IBAction)orderOut:(id)sender {
	[[self window] orderOut:sender];
}

@end
