/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MIInspectorController.h"
#import "MIMainViewController.h"
#import "MIMontageDocument.h"
#import "MIObjectRep.h"
#import "MIObject.h"
#import "MITime.h"
#import "MIApplicationController.h"
#import "TArmeDoc.h"
#import "MenuTags.h"
#import "MIApplication.h"

NSString *MTGNoTime = @"--:--:--:--";
NSString *MTGNoSampleTime = @"";

@implementation MIInspectorController

+ (id)sharedInspectorController {
    static id sharedInspectorController = nil;
    if ( sharedInspectorController == nil ) {
        sharedInspectorController = [[self alloc] init];
    }
    return sharedInspectorController;
}

+ (NSString*)visibleDefaultName {
    return @"Inspector Visible";
}

- (NSString*)nibName {
    return @"MIInspector";
}

- (int)menuTag {
	return MTGInspectorMenuTag;
}

- (NSString*)menuName {
	// There is a space after Info because of a name collision with a Finder Service menu item
	// that caused ours to be disabled.
	return @"Info ";
}


- (id)init {
    if (( self = [super init] )) {
		[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(windowDidBecomeMain:) name:NSWindowDidBecomeMainNotification object:nil];
		[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(windowDidBecomeMain:) name:NSApplicationDidBecomeActiveNotification object:nil];

		MIApplicationController *appController = [NSApp delegate];
		NSMenu* menu = [appController localEditMenu];
		NSMenuItem* menuItem = [menu itemWithTag:MTGInspectorLocalMenuTag];
		if ( menuItem ) {
			[menuItem setTarget:self];
			[menuItem setAction:@selector(show:)];
			[menuItem setTitle:[NSString stringWithFormat:@"Show %@",[self menuName]]];
		} else NSLog(@"unable to find the item \"Show Info\" with tag 14 in the localEditMenu");
	}
    return self;
}

- (void)dealloc {
    [[NSNotificationCenter defaultCenter] removeObserver:self name:NSWindowDidBecomeMainNotification object:nil];
    [[NSNotificationCenter defaultCenter] removeObserver:self name:NSApplicationDidBecomeActiveNotification object:nil];
    [super dealloc];
}

- (void)awakeFromNib {

}

- (IBAction)show:(id)sender {
	[super show:sender];
	[self selectionChanged:nil];
}

- (void)windowDidBecomeMain:(NSNotification*)notification {
    MIMainViewController *mvc = [NSApp mainViewController];

    [[NSNotificationCenter defaultCenter] removeObserver:self name:MTGSelectionChangedNotification object:nil];

    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(selectionChanged:) name:MTGSelectionChangedNotification object:[mvc selection]];
    [self selectionChanged:nil];
}


- (void)selectionChanged:(NSNotification*)notification {
    MIMainViewController *mvc = [NSApp mainViewController];
    MIMontageDocument *montageDoc = [mvc document];
    if ( mvc ) {
        MIWSSelection *selection = [mvc selection];
        unsigned count = [selection count];
        if (count == 0) {
            [fNameField setStringValue:@"Empty selection"];
            [fRelativeStartField setStringValue:MTGNoTime];
            [fRelativeSampleStartField setStringValue:MTGNoSampleTime];
            [fRelativeEndField setStringValue:MTGNoTime];
            [fRelativeSampleEndField setStringValue:MTGNoSampleTime];
            [fAbsoluteStartField setStringValue:MTGNoTime];
            [fAbsoluteSampleStartField setStringValue:MTGNoSampleTime];
            [fAbsoluteEndField setStringValue:MTGNoTime];
            [fAbsoluteSampleEndField setStringValue:MTGNoSampleTime];
            [fDurationField setStringValue:MTGNoTime];
            [fSampleDurationField setStringValue:MTGNoSampleTime];
            [fMuteCheckbox setState:NSOffState];
            [fSoloCheckbox setState:NSOffState];
            [fSliceStartField setStringValue:MTGNoTime];
            [fSliceSampleStartField setStringValue:MTGNoSampleTime];
            [fSliceEndField setStringValue:MTGNoTime];
            [fSliceSampleEndField setStringValue:MTGNoSampleTime];
            [fSliceDurationField setStringValue:MTGNoTime];
            [fSliceSampleDurationField setStringValue:MTGNoSampleTime];
        }
        if (count == 1) {
            MIObject* object = [[selection objectAtIndex:0] representedObject];
            TArmeWSObj *armeObject = [object WSObject];
            NSString *str = [NSString stringWithCString:armeObject->GetName()];
            [fNameField setStringValue:[NSString stringWithFormat:@"Selected element: %@", str]];
            TArmeWSObjPos pos = armeObject->GetPos();
            MITime* time = [MITime timeWithDocument:montageDoc];
            [time setArmeTime:pos.fStartTime];
            [fRelativeStartField setStringValue:[time getSMPTEString]];
            [fRelativeSampleStartField setStringValue:[self stringFromArmeTime:pos.fStartTime]];
            [time setArmeTime:pos.fStartTime+armeObject->GetDuration()];
            [fRelativeEndField setStringValue:[time getSMPTEString]];
            [fRelativeSampleEndField setStringValue:[self stringFromArmeTime:pos.fStartTime+armeObject->GetDuration()]];
            [time setArmeTime:armeObject->GetAbsoluteStartTime()];
            [fAbsoluteStartField setStringValue:[time getSMPTEString]];
            [fAbsoluteSampleStartField setStringValue:[self stringFromArmeTime:armeObject->GetAbsoluteStartTime()]];
            [time setArmeTime:armeObject->GetAbsoluteStartTime()+armeObject->GetDuration()];
            [fAbsoluteEndField setStringValue:[time getSMPTEString]];
            [fAbsoluteSampleEndField setStringValue:[self stringFromArmeTime: armeObject->GetAbsoluteStartTime()+armeObject->GetDuration()]];
            [time setArmeTime:armeObject->GetDuration()];
            [fDurationField setStringValue:[time getSMPTEString]];
            [fSampleDurationField setStringValue:[self stringFromArmeTime:armeObject->GetDuration()]];
            [fMuteCheckbox setState:armeObject->IsMuted() ? NSOnState : NSOffState];
            [fSoloCheckbox setState:(armeObject->IsTheVideoSolo() || armeObject->IsTheAudioSolo()) ? NSOnState : NSOffState];
            if ( [selection type] == kMIWSSelectionTypeTimeSlice ) {
                [time setArmeTime:[selection startTime]];
                [fSliceStartField setStringValue:[time getSMPTEString]];
                [fSliceSampleStartField setStringValue:[self stringFromArmeTime:[selection startTime]]];
                [time setArmeTime:[selection endxTime]];
                [fSliceEndField setStringValue:[time getSMPTEString]];
                [fSliceSampleEndField setStringValue:[self stringFromArmeTime:[selection endxTime]]];
                [time setArmeTime:[selection endxTime]-[selection startTime]];
                [fSliceDurationField setStringValue:[time getSMPTEString]];
                [fSliceSampleDurationField setStringValue:[self stringFromArmeTime:[selection endxTime] - [selection startTime]]];
            } else {
                [fSliceStartField setStringValue:MTGNoTime];
                [fSliceSampleStartField setStringValue:MTGNoSampleTime];
                [fSliceEndField setStringValue:MTGNoTime];
                [fSliceSampleEndField setStringValue:MTGNoSampleTime];
                [fSliceDurationField setStringValue:MTGNoTime];
                [fSliceSampleDurationField setStringValue:MTGNoSampleTime];
            }
        }
        if (count > 1) {
            MITime* time = [MITime timeWithDocument:montageDoc];
            NSEnumerator *enumerator = [selection objectEnumerator];
            MIObjectRep* objectRep;
            MIObject* object;
            TArmeWSObj *armeObject;
            ArmeTime startTime = ARME_TIME_MAX;
            ArmeTime endTime = ARME_TIME_MIN;
            ArmeTime t;
            unsigned mute = 0;
            BOOL solo = NO;

            while (( objectRep = [enumerator nextObject] )) {
                object = [objectRep representedObject];
                armeObject = [object WSObject];
                t = (armeObject->GetPos()).fStartTime;
                if ( t < startTime ) startTime = t;
                t = armeObject->GetDuration() + t;
                if ( t > endTime ) endTime = t;
                if ( armeObject->IsMuted() ) mute++;
                if ( armeObject->IsTheVideoSolo() || armeObject->IsTheAudioSolo() ) solo = YES;
            }

            [fNameField setStringValue:[NSString stringWithFormat:@"%d elements selected", count]];
            [fRelativeStartField setStringValue:MTGNoTime];
            [fRelativeSampleStartField setStringValue:MTGNoSampleTime];
            [fRelativeEndField setStringValue:MTGNoTime];
            [fRelativeSampleEndField setStringValue:MTGNoSampleTime];

            [time setArmeTime:startTime];
            [fAbsoluteStartField setStringValue:[time getSMPTEString]];
            [fAbsoluteStartField setStringValue:[self stringFromArmeTime:startTime]];
            [time setArmeTime:endTime];
            [fAbsoluteEndField setStringValue:[time getSMPTEString]];
            [fAbsoluteSampleEndField setStringValue:[self stringFromArmeTime:endTime]];
            [time setArmeTime:endTime - startTime];
            [fDurationField setStringValue:[time getSMPTEString]];
            [fSampleDurationField setStringValue:[self stringFromArmeTime:endTime - startTime]];

            if (mute == 0) [fMuteCheckbox setState:NSOffState];
            else if (mute == [selection count]) [fMuteCheckbox setState:NSOnState];
            else [fMuteCheckbox setState:NSMixedState];
            if ( solo ) [fSoloCheckbox setState:NSMixedState];
            else [fSoloCheckbox setState:NSOffState];

            if ( [selection type] == kMIWSSelectionTypeTimeSlice ) {
                [time setArmeTime:[selection startTime]];
                [fSliceStartField setStringValue:[time getSMPTEString]];
                [fSliceSampleStartField setStringValue:[self stringFromArmeTime:[selection startTime]]];
                [time setArmeTime:[selection endxTime]];
                [fSliceEndField setStringValue:[time getSMPTEString]];
                [fSliceSampleEndField setStringValue:[self stringFromArmeTime:[selection endxTime]]];
                [time setArmeTime:[selection endxTime]-[selection startTime]];
                [fSliceDurationField setStringValue:[time getSMPTEString]];
                [fSliceSampleDurationField setStringValue:[self stringFromArmeTime:[selection endxTime]-[selection startTime]]];
            } else {
                [fSliceStartField setStringValue:MTGNoTime];
                [fSliceSampleStartField setStringValue:MTGNoSampleTime];
                [fSliceEndField setStringValue:MTGNoTime];
                [fSliceSampleEndField setStringValue:MTGNoSampleTime];
                [fSliceDurationField setStringValue:MTGNoTime];
                [fSliceSampleDurationField setStringValue:MTGNoSampleTime];
            }
        }
    } else {
        [fNameField setStringValue:@"No document"];
        [fRelativeStartField setStringValue:MTGNoTime];
        [fRelativeSampleStartField setStringValue:MTGNoSampleTime];
        [fRelativeEndField setStringValue:MTGNoTime];
        [fRelativeSampleEndField setStringValue:MTGNoSampleTime];
        [fAbsoluteStartField setStringValue:MTGNoTime];
        [fAbsoluteSampleStartField setStringValue:MTGNoSampleTime];
        [fAbsoluteEndField setStringValue:MTGNoTime];
        [fAbsoluteSampleEndField setStringValue:MTGNoSampleTime];
        [fDurationField setStringValue:MTGNoTime];
        [fSampleDurationField setStringValue:MTGNoSampleTime];
        [fMuteCheckbox setState:NSOffState];
        [fSoloCheckbox setState:NSOffState];
        [fSliceStartField setStringValue:MTGNoTime];
        [fSliceSampleStartField setStringValue:MTGNoSampleTime];
        [fSliceEndField setStringValue:MTGNoTime];
        [fSliceSampleEndField setStringValue:MTGNoSampleTime];
        [fSliceDurationField setStringValue:MTGNoTime];
        [fSliceSampleDurationField setStringValue:MTGNoSampleTime];
    }
}

- (NSString*)stringFromArmeTime:(ArmeTime)time {
	return [NSString stringWithFormat:@"%lld", GET_VAL(time)];
}

- (BOOL)validateMenuItem:(NSMenuItem *)menuItem {
	if ( ( ![[[NSDocumentController sharedDocumentController] currentDocument] hasContent] ) && ( ![self isVisible] ) ) return NO;
	return [super validateMenuItem:menuItem];
}

- (BOOL)ignoreArrowKeys {
	return YES;
}

- (BOOL)ignoresUndoRedoActions {
	return YES;
}



@end
