/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
#ifndef LibraryToggle_h
#include "LibraryToggle.h"
#endif //LibraryToggle_h

#if USE_LIBRARY

#import "MIMontagePalette.h"
#import "MILibraryController.h"
#import "MontageInterface.h"
#import "MIApplicationController.h"

#import "MILibraryElement.h"

#import "MIMontageDocument.h"

#import "MIDeleteCommand.h"
#import "MIObject.h"

@implementation MILibraryController

+ (MILibraryController*)sharedLibraryController
{
    static id sharedLibraryController = nil;
    if ( !sharedLibraryController )
	{
        sharedLibraryController = [[self alloc] init];
    }
    return sharedLibraryController;
}

+ (NSString*)visibleDefaultName
{
    return @"Library Visible";
}

- (NSString*)nibName
{
    return @"MILibrary";
}

- (int)menuTag
{
	return 70;
}

- (NSString*)menuName
{
	return @"Library";
}


- (id)init
{
    if (( self = [super init] ))
	{
	}
    return self;
}

- (void)dealloc
{
    [super dealloc];
}

- (void)awakeFromNib
{
//	[[self window] registerForDraggedTypes:[[MIApplicationController sharedApplicationController] pasteboardTypes]];
	[fOutlineView registerForDraggedTypes:[[MIApplicationController sharedApplicationController] pasteboardTypes]];

    [[NSNotificationCenter defaultCenter] removeObserver:self name:MTGSelectionChangedNotification object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(selectionChanged:) name:MTGSelectionChangedNotification object:nil];

	[fOutlineView reloadData];
}

- (void)selectionChanged:(NSNotification*)notification
{
	[self reloadData];
}

- (void)windowDidBecomeKey:(NSNotification *)aNotification
{
	[self reloadData];
}

- (void)tableView:(NSTableView*)tableView mouseDownInHeaderOfTableColumn:(NSTableColumn *)tableColumn
{
	id identifier = [[tableColumn headerCell] stringValue];
	
	[fRootElement sortChildrenWithCriteria:identifier];
	[fOutlineView reloadData];
	[fOutlineView setHighlightedTableColumn:tableColumn];
}

- (IBAction)itemClicked:(id)sender
{
	NSTableColumn	*tableColumn = [[sender tableColumns] objectAtIndex:[sender clickedColumn]];
	id				identifier = [[tableColumn headerCell] stringValue];
	
	if ( [sender clickedRow] == -1 )
	{
		[fRootElement sortChildrenWithCriteria:identifier];
		[fOutlineView reloadData];
		[fOutlineView setHighlightedTableColumn:tableColumn];
	}
}

- (void)recreateData
{
	BOOL refresh = NO;
	
	fRootElement = [[[NSDocumentController sharedDocumentController] currentDocument] libraryRoot];
	
	if ( [[fRootElement children] count] == 0 )
	{
		if ( [[[NSDocumentController sharedDocumentController] currentDocument] topLevelContainer] )
		{
			[fRootElement addChild:[MILibraryElement elementWithObject:[[[NSDocumentController sharedDocumentController] currentDocument] topLevelContainer]]];
			[fRootElement addChild:[MILibraryElement elementWithPath:@"Objects"]];
			[fRootElement addChild:[MILibraryElement elementWithPath:@"Files"]];
			
			refresh = YES;
		}
	}
	else
		refresh = YES;
	
	if ( refresh )
		[[[fRootElement children] objectAtIndex:0] refreshData];
	
}

- (void)reloadData
{
	[self recreateData];
	[fOutlineView reloadData];
}

- (BOOL)textShouldBeginEditing:(NSText *)textObject
{
	return NO;
}

- (int)outlineView:(NSOutlineView*)ov numberOfChildrenOfItem:(id)item
{
	int					children;
	
	if ( !item )
		item = fRootElement;
	
	children =  [[item children] count];
	
	return children;
}

- (BOOL)outlineView:(NSOutlineView*)ov isItemExpandable:(id)item
{
	BOOL exp;
	
	if ( !item )
		item = fRootElement;

	exp = ([[item children] count] > 0 );
	
	return exp;
}

- (id)outlineView:(NSOutlineView *)outlineView objectValueForTableColumn:(NSTableColumn *)tableColumn byItem:(id)theItem
{
	id					value;
	NSString			*header = [[tableColumn headerCell] stringValue];
	MILibraryElement	*item;
	
	if ( !theItem )
		item = fRootElement;

	// item is supposed to be a MIObject, but in the case of an item not already created in the workspace, it can be something else.
	
	item = (MILibraryElement*)theItem;

	if ( [header isEqualToString:@"Title"] )
	{
		value = [item title];
	}
	else if ( [header isEqualToString:@"Type"] )
	{
		value = [item type];
	}
	else if ( [header isEqualToString:@"Duration"] )
	{
		value = [item duration];
	}
	else if ( [header isEqualToString:@"Start Time"] )
	{
		value = [item startTime];
	}
	else if ( [header isEqualToString:@"End Time"] )
	{
		value = [item endTime];
	}
	else if ( [header isEqualToString:@"Status"] )
	{
		value = [item status];
	}
	else if ( [header isEqualToString:@"Size (Kb)"] )
	{
		value = @"some";
	}
	else
	{
		value = @"------";
	}
	
	
	return value;
}

- (id)outlineView:(NSOutlineView *)outlineView child:(int)index ofItem:(id)item
{
	id retVal;
	
	if ( !item )
		item = fRootElement;

	retVal = [[item children] objectAtIndex:index];

	return retVal;
}

- (void)outlineViewSelectionDidChange:(NSNotification *)notification
{
	
}


- (void)keyDown:(NSEvent *)theEvent
{
	static NSMutableCharacterSet	*cSet = nil;
	NSEnumerator			*selectedRowsEnum;
	MILibraryElement		*elem;
	NSNumber				*num;
	NSMutableArray			*arr = [[[NSMutableArray alloc] initWithObjects:nil] autorelease];
	
	if ( !cSet )
	{
		cSet = [[NSMutableCharacterSet alloc] init];
		[cSet addCharactersInRange:NSMakeRange(NSDeleteFunctionKey, 1)];
		[cSet addCharactersInRange:NSMakeRange(NSDeleteLineFunctionKey, 1)];
		[cSet addCharactersInRange:NSMakeRange(NSDeleteCharFunctionKey, 1)];
	}
//								[NSString stringWithFormat:@"%c%c%c",NSDeleteFunctionKey,NSDeleteLineFunctionKey,NSDeleteCharFunctionKey]] retain];	}
	
	if ( [[theEvent characters] rangeOfCharacterFromSet:cSet].location != NSNotFound )
	{
		selectedRowsEnum = [fOutlineView selectedRowEnumerator];
		// delete an object from the list
		while ( num = [selectedRowsEnum nextObject] )
		{
			elem = [fOutlineView itemAtRow:[num intValue]];
			if ( [elem object] && [elem parent] == [[fRootElement children] objectAtIndex:1] )
			{
				// delete from library
				[[elem parent] removeChild:elem];
			}
			else if ( [elem object] && ![[elem object] isTLC] )
			{
				// delete from workspace
				[arr addObject:[elem object]];
			}
			else if ( [elem path] && ![[elem path] isEqualToString:@"Objects"] && ![[elem path] isEqualToString:@"Files"] )
			{
				// delete from library
				[[elem parent] removeChild:elem];
			}
		}
		
		if ( [arr count] > 0 )
		{
			[[[arr objectAtIndex:0] montageDoc] doCmd:[[[MIDeleteCommand alloc] initWithListOfObjects:arr] autorelease]];
		}
	}

}



// dragging destination
- (NSDragOperation)outlineView:(NSOutlineView*)outlineView validateDrop:(id <NSDraggingInfo>)info proposedItem:(id)item proposedChildIndex:(int)index
{
    NSPasteboard	*pboard;
    unsigned int	returnVal;
    returnVal = NSDragOperationNone;
    pboard = [info draggingPasteboard];
    
    if ([[pboard types] containsObject:NSFilenamesPboardType])
    {
/*        fileNames = [pboard propertyListForType:NSFilenamesPboardType];
        n = [fileNames count];
        for ( i = 0; i < n && returnVal == NSDragOperationNone; i++ )
        {
            name = [fileNames objectAtIndex:i];
            if ([self isSoundDirectoryOrFile:name])
*/
            returnVal = NSDragOperationCopy;
//        }
    }
    
    fLastDraggingAnswer = returnVal;
	
	return fLastDraggingAnswer;
}

- (BOOL)outlineView:(NSOutlineView*)outlineView acceptDrop:(id <NSDraggingInfo>)info item:(id)item childIndex:(int)index
{
	NSPasteboard			*pboard;
    NSArray					*fileNames;
	NSString				*aFile;
	NSEnumerator			*fileEnum;
	MILibraryElement		*elem;
	
    pboard = [info draggingPasteboard];
	
	// check if the information contains filenames
	if ([[pboard types] containsObject:NSFilenamesPboardType])
    {
        fileNames = [pboard propertyListForType:NSFilenamesPboardType];
		fileEnum = [fileNames objectEnumerator];
		while( aFile = [fileEnum nextObject] )
		{
			elem = [MILibraryElement elementWithPath:aFile];
			if ( [[fRootElement children] count] < 2 )
				[self recreateData];
			[(MILibraryElement*)[[fRootElement children] objectAtIndex:2] addChild:elem];
		}
    }
	
	[fOutlineView reloadData];
	
	return YES;
}

- (BOOL)outlineView:(NSOutlineView *)outlineView writeItems:(NSArray*)items toPasteboard:(NSPasteboard*)pboard
{
	NSData *theData;
	// put the relevant data in the pasteboard
	// the application is responsible for publishing the data in as many types as possible in general when dragging outside of the
	// application, but we can use something standard when staying in the application
	
//	NSLog( [items description] );
	theData = [NSArchiver archivedDataWithRootObject:items];
	[pboard declareTypes:[NSArray arrayWithObject:MILibraryElementsPboardType] owner:self];
	[pboard setData:theData forType:(NSString*)MILibraryElementsPboardType];
	
	return YES;
}


@end

#endif //USE_LIBRARY

