/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#ifndef LibraryToggle_h
#include "LibraryToggle.h"
#endif //LibraryToggle_h

#import <Foundation/Foundation.h>

extern NSString *MILibraryElementsPboardType;

@class MIObject;

@interface MILibraryElement : NSObject
{
	NSString			*fPath;
	MIObject			*fObject;
	NSMutableArray		*fChildren;
	MILibraryElement	*fParent;
}

- (id)initWithPath:(NSString*)path;
- (id)initWithObject:(MIObject*)object;

#if USE_LIBRARY

+ (id)elementWithPath:(NSString*)path;
+ (id)elementWithObject:(MIObject*)object;

- (void)setObject:(MIObject*)object;
- (void)setPath:(NSString*)path;


- (MIObject*)object;
- (NSString*)path;

- (void)setParent:(MILibraryElement*)parent;
- (MILibraryElement*)parent;

- (NSArray*)children;
- (void)addChild:(MILibraryElement*)child;
- (void)removeChild:(MILibraryElement*)child;

- (int)compareTitle:(MILibraryElement*)other;
- (int)compareStartTime:(MILibraryElement*)other;
- (int)compareEndTime:(MILibraryElement*)other;
- (int)compareDuration:(MILibraryElement*)other;
- (int)compareStatus:(MILibraryElement*)other;
- (int)compareType:(MILibraryElement*)other;

- (NSString*)title;
- (NSString*)startTime;
- (NSString*)endTime;
- (NSString*)duration;
- (NSString*)status;
- (NSString*)type;
- (void)refreshData;
- (void)sortChildrenWithCriteria:(id)criteria;
#endif //#if USE_LIBRARY

@end

