/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MontageInterface.h"


@class MIMainViewController;

/*!
    @class MIMainScrollView
    @abstract the MIMainScrollView is a NSScrollView subclass with more buttons
    @discussion
	the MIMainScrollView is the ScrollView that encapsulates the topLevelContainerRep. the large majority of the functionalities
	of this class are that of it's base class: NSScrollView. Having it subclassed gives us the ability to add a dsp bar, zoom
	buttons and have a bit of control over when scrolling occurs (in order to notify the topLevelContainerRep that it should
	update the ruler and overviews). 
*/

@interface MIMainScrollView : NSScrollView
{
    /*! @var fVZoomSlider	the vertical zoom slider */
	NSSlider							*fVZoomSlider;

    /*! @var fVZoomPlusButton	the vertical zoomIn button */
	NSButton							*fVZoomPlusButton;

    /*! @var fVZoomMinusButton	the vertical zoomOut button */
	NSButton							*fVZoomMinusButton;

    /*! @var fHZoomSlider	the horizontal zoom slider */
	NSSlider							*fHZoomSlider;

    /*! @var fHZoomPlusButton	the horizontal zoomIn button */
	NSButton							*fHZoomPlusButton;

    /*! @var fHZoomMinusButton	the horizontal zoomOut button */
	NSButton							*fHZoomMinusButton;

    /*! @var fDspBar	the DSP bar */
	NSProgressIndicator					*fDspBar;

    /*! @var fMainViewController a pointer to the main view controller */
	IBOutlet	MIMainViewController	*fMainViewController;
}

/*! @function tile an overriden version of the NSSCrollView method, it organises the sliders, buttons, etc
    at the good position */
- (void)tile;

/*! @function makeVSliderAndButtons creates the vertical slider and buttons */
- (void)makeVSliderAndButtons;
/*! @function makeHSliderAndButtons creates the horizontal slider and buttons */
- (void)makeHSliderAndButtons;


- (void)vZoomSliderAction:(id)sender;
- (void)vZoomPlusButtonAction:(id)sender;
- (void)vZoomMinusButtonAction:(id)sender;
- (void)hZoomSliderAction:(id)sender;
- (void)hZoomPlusButtonAction:(id)sender;
- (void)hZoomMinusButtonAction:(id)sender;

/*! @function updateZoomSliders this function updates the zoom sliders to put the knobs at the right positions */
- (void)updateZoomSliders;

@end


