/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
 
#import "MIMainScrollView.h"
#import "MIMainViewController.h"
#import "MIMontageDocument.h"
#import "MIOverviewController.h"

@implementation MIMainScrollView
/*
- (void)reflectScrolledClipView:(NSClipView *)aClipView
{
	[super reflectScrolledClipView:aClipView];
	[[[self documentView] clipSheetController] scrollingOccured]; 
}
*/
- (id)initWithFrame:(NSRect)frame
{
	if ( self = [super initWithFrame:frame] )
	{
	}
	[self tile];
	return self;
}

- (void)awakeFromNib
{
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(clipViewBoundsChanged:) name:NSViewBoundsDidChangeNotification object:[self contentView]];
}

- (void)dealloc
{
    [[NSNotificationCenter defaultCenter] removeObserver:self name:NSViewBoundsDidChangeNotification object:[self contentView]];
	[super dealloc];
}

- (void)clipViewBoundsChanged:(NSNotification*)notification {
	[[[[self window] windowController] document] setDocumentSetting:NSStringFromPoint([[self contentView] bounds].origin) forKey:@"scrollPoint"];
}

- (void)makeVSliderAndButtons
{
	NSBundle *bundle = [NSBundle mainBundle];
	NSImage *image;
	if ( !fVZoomSlider )
	{
		fVZoomSlider = [[NSSlider allocWithZone:[self zone]] initWithFrame:NSMakeRect(0,0,12,100)];
		[fVZoomSlider setMinValue:0];
		[fVZoomSlider setMaxValue:[MIMainViewController getNbVZoomIndices]-1];
		[fVZoomSlider setFloatValue:[MIMainViewController getDefaultVZoomIndex]];
//		[fVZoomSlider setKnobThickness:12];
		[fVZoomSlider setNumberOfTickMarks:[MIMainViewController getNbVZoomIndices]];
		[fVZoomSlider setAllowsTickMarkValuesOnly:YES];
		[[fVZoomSlider cell] setControlSize:NSSmallControlSize];
		[fVZoomSlider setTarget:self];
		[fVZoomSlider setAction:@selector(vZoomSliderAction:)];
		[self addSubview:fVZoomSlider];
		[fVZoomSlider release]; 
		
		fVZoomPlusButton = [[NSButton allocWithZone:[self zone]] initWithFrame:NSMakeRect(0,0,16,10)];
		[fVZoomPlusButton setButtonType:NSMomentaryChangeButton];
		[fVZoomPlusButton setImagePosition:NSImageOnly];
		[fVZoomPlusButton setBordered:NO];
		
		image = [[NSImage alloc] initWithContentsOfFile:[bundle pathForImageResource:@"bt-_horiz_gris.png"]];
		[fVZoomPlusButton setImage:image];
		[image release];
		image = [[NSImage alloc] initWithContentsOfFile:[bundle pathForImageResource:@"bt-_horiz_bleu.png"]];
		[fVZoomPlusButton setAlternateImage:image];
		[image release];

//		[fVZoomPlusButton setControlSize:NSSmallControlSize];
		//[fVZoomPlusButton setTitle:@"-"];
		[fVZoomPlusButton setTarget:self];
		[fVZoomPlusButton setAction:@selector(vZoomPlusButtonAction:)];
		[self addSubview:fVZoomPlusButton];
		[fVZoomPlusButton release]; 

		fVZoomMinusButton = [[NSButton allocWithZone:[self zone]] initWithFrame:NSMakeRect(0,0,16,10)];
		[fVZoomMinusButton setButtonType:NSMomentaryChangeButton];
		[fVZoomMinusButton setImagePosition:NSImageOnly];
		[fVZoomMinusButton setBordered:NO];

		image = [[NSImage alloc] initWithContentsOfFile:[bundle pathForImageResource:@"bt+_horiz_gris.png"]];
		[fVZoomMinusButton setImage:image];
		[image release];
		image = [[NSImage alloc] initWithContentsOfFile:[bundle pathForImageResource:@"bt+_horiz_bleu.png"]];
		[fVZoomMinusButton setAlternateImage:image];
		[image release];
		//		[fVZoomMinusButton setControlSize:NSSmallControlSize];
		//[fVZoomMinusButton setTitle:@"+"];
		[fVZoomMinusButton setTarget:self];
		[fVZoomMinusButton setAction:@selector(vZoomMinusButtonAction:)];
		[self addSubview:fVZoomMinusButton];
		[fVZoomMinusButton release]; 
	}
}

- (void)makeHSliderAndButtons
{
	NSBundle *bundle = [NSBundle mainBundle];
	NSImage *image;
	if ( !fHZoomSlider )
	{
		fHZoomSlider = [[MIHSlider allocWithZone:[self zone]] initWithFrame:NSMakeRect(0,0,100,16)];
		[fHZoomSlider setMinValue:0];
		[fHZoomSlider setMaxValue:[MIMainViewController getNbHZoomIndices]-1 ];
		[fHZoomSlider setIntValue:[MIMainViewController getDefaultHZoomIndex]];

//		[fHZoomSlider setNumberOfTickMarks:[MIMainViewController getNbHZoomIndices]];
//		[fHZoomSlider setAllowsTickMarkValuesOnly:YES];
		[[fHZoomSlider cell] setControlSize:NSSmallControlSize];
		[fHZoomSlider setTarget:self];
		[fHZoomSlider setAction:@selector(hZoomSliderAction:)];
		[self addSubview:fHZoomSlider];
		[fHZoomSlider release]; 
		
		fHZoomPlusButton = [[NSButton allocWithZone:[self zone]] initWithFrame:NSMakeRect(0,0,10,16)];
		[fHZoomPlusButton setButtonType:NSMomentaryChangeButton];
		[fHZoomPlusButton setImagePosition:NSImageOnly];
		[fHZoomPlusButton setBordered:NO];

		image = [[NSImage alloc] initWithContentsOfFile:[bundle pathForImageResource:@"bt+_vert_gris.png"]];
		[fHZoomPlusButton setImage:image];
		[image release];
		image = [[NSImage alloc] initWithContentsOfFile:[bundle pathForImageResource:@"bt+_vert_bleu.png"]];
		[fHZoomPlusButton setAlternateImage:image];
		[image release];
		
//		[fHZoomPlusButton setControlSize:NSSmallControlSize];
		//[fHZoomPlusButton setTitle:@"+"];
		[fHZoomPlusButton setTarget:self];
		[fHZoomPlusButton setAction:@selector(hZoomPlusButtonAction:)];
		[self addSubview:fHZoomPlusButton];
		[fHZoomPlusButton release]; 

		fHZoomMinusButton = [[NSButton allocWithZone:[self zone]] initWithFrame:NSMakeRect(0,0,10,16)];
		[fHZoomMinusButton setButtonType:NSMomentaryChangeButton];
		[fHZoomMinusButton setImagePosition:NSImageOnly];
		[fHZoomMinusButton setBordered:NO];

		image = [[NSImage alloc] initWithContentsOfFile:[bundle pathForImageResource:@"bt-_vert_gris.png"]];
		[fHZoomMinusButton setImage:image];
		[image release];
		image = [[NSImage alloc] initWithContentsOfFile:[bundle pathForImageResource:@"bt-_vert_bleu.png"]];
		[fHZoomMinusButton setAlternateImage:image];
		[image release];
		//		[fHZoomMinusButton setControlSize:NSSmallControlSize];
		//[fHZoomMinusButton setTitle:@"-"];
		[fHZoomMinusButton setTarget:self];
		[fHZoomMinusButton setAction:@selector(hZoomMinusButtonAction:)];
		[self addSubview:fHZoomMinusButton];
		[fHZoomMinusButton release]; 

/*
		fDspBar = [[NSProgressIndicator allocWithZone:[self zone]] initWithFrame:NSMakeRect(0,0,100,16)];
		[fDspBar setMinValue:0];
		[fDspBar setMaxValue:100];
		[fDspBar setDoubleValue:10];
		[fDspBar setControlSize:NSRegularControlSize];
		[self addSubview:fDspBar];
		[fDspBar release]; 
*/
	}
}

- (void)tile
{
    // Let the superclass do most of the work.
    [super tile];

	// arrange horizontal stuff
/*
    if (![self hasHorizontalScroller])
	{
        if ( fDspBar )
			[fDspBar removeFromSuperview];
		if ( fHZoomSlider )
			[fHZoomSlider removeFromSuperview];
		if ( fHZoomPlusButton )
			[fHZoomPlusButton removeFromSuperview];
		if ( fHZoomMinusButton )
			[fHZoomMinusButton removeFromSuperview];

        fDspBar = nil;
		fHZoomSlider = nil;
		fHZoomPlusButton = nil;
		fHZoomMinusButton = nil;
    }
	else
	{
		NSScroller *horizScroller;
		NSRect horizScrollerFrame, plusButtonFrame, sliderFrame, minusButtonFrame, dspBarFrame;
	
        if (!fHZoomSlider)
			[self makeHSliderAndButtons];

        horizScroller = [self horizontalScroller];
        horizScrollerFrame = [horizScroller frame];
        plusButtonFrame = [fHZoomPlusButton frame];
		minusButtonFrame = [fHZoomMinusButton frame];
		sliderFrame = [fHZoomSlider frame];
//		dspBarFrame = [fDspBar frame];
		
        // Now we'll just adjust the horizontal scroller size and set the buttons size and location.
		
//		dspBarFrame.origin.y = NSHeight([self bounds]) - NSHeight(dspBarFrame) +1;
//		[fDspBar setFrame:dspBarFrame];
		
        horizScrollerFrame.size.width -= NSWidth(plusButtonFrame);
		horizScrollerFrame.size.width -= NSWidth(minusButtonFrame);
		horizScrollerFrame.size.width -= NSWidth(sliderFrame);
//		horizScrollerFrame.size.width -= NSWidth(dspBarFrame);
		horizScrollerFrame.origin.x = NSMaxX(dspBarFrame);
		
        [horizScroller setFrame:horizScrollerFrame];

		minusButtonFrame.origin.x = NSMaxX(horizScrollerFrame);
		minusButtonFrame.size.height = NSHeight(horizScrollerFrame) + 1;
		minusButtonFrame.origin.y = NSHeight([self bounds]) - NSHeight(minusButtonFrame) + 1;
		[fHZoomMinusButton setFrame:minusButtonFrame];

		sliderFrame.origin.x = NSMaxX(minusButtonFrame);
		sliderFrame.size.height = NSHeight(horizScrollerFrame) + 1;
		sliderFrame.origin.y = NSHeight([self bounds]) - NSHeight(sliderFrame) + 1;
		[fHZoomSlider setFrame:sliderFrame];

		plusButtonFrame.origin.x = NSMaxX(sliderFrame);
		plusButtonFrame.size.height = NSHeight(horizScrollerFrame) + 1;
		plusButtonFrame.origin.y = NSHeight([self bounds]) - NSHeight(plusButtonFrame) + 1;
		[fHZoomPlusButton setFrame:plusButtonFrame];
    }
*/
/*    if (![self hasVerticalScroller])
	{
		if ( fVZoomSlider )
			[fVZoomSlider removeFromSuperview];
		if ( fVZoomPlusButton )
			[fVZoomPlusButton removeFromSuperview];
		if ( fVZoomMinusButton )
			[fVZoomMinusButton removeFromSuperview];

		fVZoomSlider = nil;
		fVZoomPlusButton = nil;
		fVZoomMinusButton = nil;
    }
	else
*//*	{
		NSScroller *vertScroller;
		NSRect vertScrollerFrame, plusButtonFrame, sliderFrame, minusButtonFrame;
	
        if (!fVZoomSlider)
			[self makeVSliderAndButtons];

        vertScroller = [self verticalScroller];
        vertScrollerFrame = [vertScroller frame];
        plusButtonFrame = [fVZoomPlusButton frame];
		minusButtonFrame = [fVZoomMinusButton frame];
		sliderFrame = [fVZoomSlider frame];
		
        // Now we'll just adjust the horizontal scroller size and set the buttons size and location.
        vertScrollerFrame.size.height -= NSHeight(plusButtonFrame);
		vertScrollerFrame.size.height -= NSHeight(minusButtonFrame);
		vertScrollerFrame.size.height -= NSHeight(sliderFrame);
		
        [vertScroller setFrame:vertScrollerFrame];

		minusButtonFrame.origin.y = NSMaxY(vertScrollerFrame);
		minusButtonFrame.size.width = NSWidth(vertScrollerFrame) + 1;
		minusButtonFrame.origin.x = NSWidth([self bounds]) - NSWidth(minusButtonFrame) + 1;
		[fVZoomMinusButton setFrame:minusButtonFrame];

		sliderFrame.origin.y = NSMaxY(minusButtonFrame);
		sliderFrame.size.width = NSWidth(vertScrollerFrame) + 1;
		sliderFrame.origin.x = NSWidth([self bounds]) - NSWidth(sliderFrame) + 1;
		[fVZoomSlider setFrame:sliderFrame];

		plusButtonFrame.origin.y = NSMaxY(sliderFrame);
		plusButtonFrame.size.width = NSWidth(vertScrollerFrame) + 1;
		plusButtonFrame.origin.x = NSWidth([self bounds]) - NSWidth(plusButtonFrame) + 1;
		[fVZoomPlusButton setFrame:plusButtonFrame];
    }
*/
}

- (void)vZoomSliderAction:(id)sender
{
	[fMainViewController setVZoom:[sender intValue]];
}

- (void)vZoomPlusButtonAction:(id)sender
{
	[fMainViewController vZoomIn];
}

- (void)vZoomMinusButtonAction:(id)sender
{
	[fMainViewController vZoomOut];
}

- (void)hZoomSliderAction:(id)sender
{
//	[fMainViewController setHZoom:[sender intValue]];
	// do not change the zoom now, simply update the overview to show what the frame would look like
	
	[[MIOverviewController sharedOverviewController] setFutureHZoomIndex:[sender intValue]];
}

- (void)hZoomPlusButtonAction:(id)sender
{
	[fMainViewController hZoomIn];
}

- (void)hZoomMinusButtonAction:(id)sender
{
	[fMainViewController hZoomOut];
}

- (void)updateZoomSliders
{
	[fHZoomSlider setFloatValue:[fMainViewController hZoomIndex]];
	[fVZoomSlider setFloatValue:[fMainViewController vZoomIndex]];
}

@end

