/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MIMixerController.h"
#import "MIMixerPartController.h"
#import "MIMontageDocument.h"
#import "MIPaletteController.h"
#import "MIDocumentController.h"
#import "MIObjCProxyListener.h"
#import "MIApplicationController.h"

#import "ArmeNotifications.h"
#import "MenuTags.h"

#ifndef MEMixingInfo_h
#include "MEMixingInfo.h"
#endif //MEMixingInfo_h

#ifndef TArmeDoc_h
#include "TArmeDoc.h"
#endif //TArmeDoc_h

#include <unistd.h>

//#define MTGEnableTimeLogs 1

/*
#ifdef MTGEnableTimeLogs
#include <sys/time.h>

#define MTGStartTimeLog {timeval tps,tpe;gettimeofday(&tps, nil);
#define //MTGEndTimeLog(C,M) gettimeofday(&tpe, nil); NSLog(@"%@ - %@ : %d seconds %d microseconds",C,M,tpe.tv_sec-tps.tv_sec, abs(tpe.tv_usec-tps.tv_usec));}
#else
#define MTGStartTimeLog
#define //MTGEndTimeLog(C,M)
#endif
    */


#define kMIMixerPartWidth 100.0

@implementation MIMixerController

+ (MIMixerController*)sharedMixerController {
    static id sharedMixerController = nil;
    if ( sharedMixerController == nil ) {
        sharedMixerController = [[self alloc] init];
    }
    return sharedMixerController;
}

+ (NSString*)visibleDefaultName {
    return @"Mixer Visible";
}

- (NSString*)nibName {
    return @"MIMixer";
}

- (int)menuTag {
    return MTGMixerMenuTag;
}

- (NSString*)menuName {
    return @"Mixer";
}

- (id)init {
    if (( self = [super init] )) {
        fRealMixerParts = [[NSMutableArray alloc] initWithObjects:nil];
        fVirtualMixerParts = [[NSMutableArray alloc] initWithObjects:nil];
        fVirtualMonoMixerParts = [[NSMutableArray alloc] initWithObjects:nil];
        fUptoDate = NO ;
        fThreadRunning = NO;
        fListener = new MIObjCProxyListener(self);
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(currentDocumentChanged:) name:MTGCurrentDocumentChangedNotification object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(documentRemoved:) name:MTGDocumentRemovedNotification object:nil];
    }
    return self;
}

- (void)dealloc
{
    [self terminateDrawingThread];
    [fRealMixerParts release];
    [fVirtualMixerParts release];
    [fVirtualMonoMixerParts release];
    delete fListener;
    [[NSNotificationCenter defaultCenter] removeObserver:self name:MTGCurrentDocumentChangedNotification object:nil];
    [[NSNotificationCenter defaultCenter] removeObserver:self name:MTGDocumentRemovedNotification object:nil];

    [super dealloc];
}


- (void)createRealMixerParts:(MIMontageDocument*)document {
    SInt32 i;
    MIMixerPartController *ctrl= nil;
    float lastPos = 0;

    for ( i = 0 ; i < fArmeDoc->GetMixingInfo()->GetMaxStereoRAOCount(); i++ ) 
	{
        ctrl = [[MIMixerPartController alloc] initWithDocument:document andRAOIndex:i];
        [fRealMixerPartContainer addSubview:ctrl->fMixerPartView];
        NSString* partTitle = [NSString stringWithFormat:@"%@ %d/%d",fArmeDoc->GetMixingInfo()->GetCurrentOutputDeviceShortName().getNSString(),(i*2)+1,(i*2)+2];
        [ctrl->fTitleText setStringValue:partTitle];
        [ctrl->fMixerPartView setFrameOrigin:NSMakePoint(lastPos, 0)];
        lastPos = NSMaxX([ctrl->fMixerPartView frame]) - 1;
        [fRealMixerParts addObject:ctrl];
        [ctrl release];
    }

    if ( lastPos ) {
        [fRealMixerPartContainer setFrameSize:NSMakeSize(lastPos, NSHeight([ctrl->fMixerPartView frame]))];
    }
}

- (void)createVirtualMixerParts:(MIMontageDocument*)document {
    SInt32 i,j;
    MIMixerPartController *ctrl = nil;
    float lastPos = 0;

    for ( i = 0 ; i < fArmeDoc->GetMixingInfo()->GetMaxMixerDisplayStereoVAOCount(); i++ ) {
        ctrl = [[MIMixerPartController alloc] initWithDocument:document andStereoVAOIndex:i];
        [fVirtualMixerPartContainer addSubview:ctrl->fMixerPartView];
        NSString* partTitle = [NSString stringWithFormat:@"Internal Stereo %d",i+1];
        [ctrl->fTitleText setStringValue:partTitle];
        [ctrl->fMixerPartView setFrameOrigin:NSMakePoint(lastPos, 0)];
        lastPos = NSMaxX([ctrl->fMixerPartView frame]) - 1;
        [fVirtualMixerParts addObject:ctrl];
        [ctrl release];
    }

    for ( j = 0 ; j < fArmeDoc->GetMixingInfo()->GetMaxMixerDisplayMonoVAOCount(); j++ ) {
        ctrl = [[MIMixerPartController alloc] initWithDocument:document andMonoVAOIndex:j];
        [fVirtualMixerPartContainer addSubview:ctrl->fMixerPartView];
        NSString* partTitle = [NSString stringWithFormat:@"Internal Mono %d",j+1];
        [ctrl->fTitleText setStringValue:partTitle];
        [ctrl->fMixerPartView setFrameOrigin:NSMakePoint(lastPos, 0)];
        lastPos = NSMaxX([ctrl->fMixerPartView frame]) - 1;
        [fVirtualMonoMixerParts addObject:ctrl];
        [ctrl release];
    }

    if ( lastPos ) {
        [fVirtualMixerPartContainer setFrameSize:NSMakeSize(lastPos, NSHeight([ctrl->fMixerPartView frame]))];
    }
}

- (void)terminateDrawingThread {
    if ( fThreadRunning ) {
        fThreadContinue = NO;
        while ( !fThreadContinue ) {
            //[NSThread sleepUntilDate:[NSDate dateWithTimeIntervalSinceNow:0.4]];
			usleep( 400000 );
        }
        fThreadRunning = NO;
    }
}

- (void)emptyContent {
    fUptoDate = NO ;
    [self terminateDrawingThread];
    [fRealMixerParts makeObjectsPerformSelector:@selector(removeFromSuperview)];
    [fVirtualMixerParts makeObjectsPerformSelector:@selector(removeFromSuperview)];
    [fVirtualMonoMixerParts makeObjectsPerformSelector:@selector(removeFromSuperview)];
    [fRealMixerParts removeAllObjects];
    [fVirtualMixerParts removeAllObjects];
    [fVirtualMonoMixerParts removeAllObjects];
}

- (void)sprawnDrawingThread {
    if ( !fThreadRunning ) {
        fThreadContinue = YES;
        fThreadRunning = YES;
        [NSApplication detachDrawingThread:@selector(perpetualRedrawLoop:) toTarget:self withObject:nil];
    }
}


- (void)perpetualRedrawLoop:(id)argument {
    NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];

    while ( fThreadContinue ) {
        timeval tps, tpe;
		gettimeofday(&tps, nil);
        [fRealMixerParts makeObjectsPerformSelector:@selector(updateVUMeters)];
        [fVirtualMixerParts makeObjectsPerformSelector:@selector(updateVUMeters)];
        [fVirtualMonoMixerParts makeObjectsPerformSelector:@selector(updateVUMeters)];
		gettimeofday(&tpe, nil);
		tpe.tv_sec = tpe.tv_sec - tps.tv_sec;
		if ( tpe.tv_sec == 0 ) {
		tpe.tv_usec = tpe.tv_usec - tps.tv_usec;
			if ( tpe.tv_usec < 41666 ) {
				usleep( 41666 - tpe.tv_usec );
			}
		}
    }
    fThreadContinue = YES;
    [pool release];
}

- (void)resizeSubviewsOfSplitView:(NSSplitView *)sender {
    NSArray* subviews = [sender subviews];
    NSView *view1 = [subviews objectAtIndex:0];
    NSView *view2 = [subviews objectAtIndex:1];
    NSRect frame = [view1 frame];
    float split = NSMaxX( frame );

    split = [self splitView:sender constrainSplitPosition:split ofSubviewAt:0];

    if ( split > NSWidth([fVirtualMixerPartContainer frame]) ) {
        split = NSWidth([fVirtualMixerPartContainer frame]);
        split = [self splitView:sender constrainSplitPosition:split ofSubviewAt:0];
    }

    if ( split < kMIMixerPartWidth ) {
        split = kMIMixerPartWidth;
        split = [self splitView:sender constrainSplitPosition:split ofSubviewAt:0];
    }

    if ( NSWidth([sender bounds]) - split > NSWidth([fRealMixerPartContainer frame]) ) {
        split = NSWidth([sender bounds]) - NSWidth([fRealMixerPartContainer frame]) ;
        split = [self splitView:sender constrainSplitPosition:split ofSubviewAt:0];
    }

    if (  NSWidth([sender bounds]) - split < kMIMixerPartWidth ) {
        split = NSWidth([sender bounds]) - kMIMixerPartWidth ;
        split = [self splitView:sender constrainSplitPosition:split ofSubviewAt:0];
    }

    [view1 setFrame:NSMakeRect( NSMinX( frame ), NSMinY( frame ), split, NSHeight( frame ) )];
    [view2 setFrame:NSMakeRect( split + [sender dividerThickness], NSMinY( frame ), NSMaxX( [sender frame] ) - ( split + [sender dividerThickness] ), NSHeight( frame ) )];
}

- (void)updateContent:(MIMontageDocument*)doc 
{
    [self terminateDrawingThread];
    if ( fArmeDoc ) 
	{
        // make an unsubscribe() if the document still exists
        NSEnumerator *enumerator = [[[MIDocumentController sharedDocumentController] documents] objectEnumerator];
        MIMontageDocument *document;
        while (( document = [enumerator nextObject] )) 
		{
            if ( [document armeDocument] == fArmeDoc ) 
			{
                fArmeDoc->Unsubscribe( fListener ) ;
                fArmeDoc = nil;
                break;
            }
        }
    }

    [self emptyContent];
    if ( doc ) 
	{
        fArmeDoc = [doc armeDocument];
        fArmeDoc->Subscribe( fListener ) ;
        //[[self window] useOptimizedDrawing:YES];
        [self createRealMixerParts:doc];
        [self createVirtualMixerParts:doc];
        [self resizeSubviewsOfSplitView:fSplitView];
        fUptoDate = YES ;
        [self sprawnDrawingThread];
    }
}

- (void)documentRemoved:(NSNotification*)notification {
	[self terminateDrawingThread];
	[self updateContent:nil];
	fArmeDoc = nil;
}

- (void)currentDocumentChanged:(NSNotification*)notification {
    if ( [self isVisible] )
        [self updateContent:[[notification userInfo] objectForKey:MTGCurrentDocumentKey]];
}

- (IBAction)show:(id)sender {
    [super show:sender];
    if ( !fUptoDate ) {
        MIMontageDocument* doc = [[NSDocumentController sharedDocumentController] currentDocument];
        [self updateContent:doc];
    } else [self sprawnDrawingThread];
}

- (IBAction)hide:(id)sender {
    [self terminateDrawingThread];
    [super hide:sender];
}

- (void)windowWillClose:(NSNotification *)aNotification {
    [self terminateDrawingThread];
    [super windowWillClose:aNotification];
}

- (void)windowWasLoaded {
    NSWindow *window = [self window];
    NSRect frame = [window frame];
    [window setFrame:NSMakeRect( NSMinX( frame ), NSMinY( frame ), 6 * kMIMixerPartWidth + [fSplitView dividerThickness], [window minSize].height ) display:NO];
    [window setResizeIncrements:NSMakeSize( kMIMixerPartWidth - 1, kMIMixerPartWidth )];
    [fScrollRealMixerPartView setLineScroll:kMIMixerPartWidth - 1];
    [fScrollVirtuelMixerPartView setLineScroll:kMIMixerPartWidth - 1];
    [fScrollRealMixerPartView setPageScroll:0];
    [fScrollVirtuelMixerPartView setPageScroll:0];
}

- (NSSize)windowWillResize:(NSWindow *)sender toSize:(NSSize)frameSize {
    float maxWidth = ( [fVirtualMixerParts count] + [fVirtualMonoMixerParts count] + [fRealMixerParts count] ) * kMIMixerPartWidth + [fSplitView dividerThickness];
    return NSMakeSize( frameSize.width < maxWidth ? frameSize.width : maxWidth, [sender minSize].height ) ;
}

- (float)splitView:(NSSplitView *)sender constrainMaxCoordinate:(float)proposedMax ofSubviewAt:(int)offset {
    float realWidth = NSWidth( [sender bounds] ) - kMIMixerPartWidth - [sender dividerThickness] ;
    float virtualWidth = ( [fVirtualMixerParts count] + [fVirtualMonoMixerParts count] ) * kMIMixerPartWidth;
    return [self splitView:sender constrainSplitPosition:realWidth < virtualWidth ? realWidth : virtualWidth ofSubviewAt:offset];
}

- (float)splitView:(NSSplitView *)sender constrainMinCoordinate:(float)proposedMin ofSubviewAt:(int)offset {
    float virtualWidth = kMIMixerPartWidth;
    float realWidth = NSWidth( [sender bounds] ) - ( [fRealMixerParts count] * kMIMixerPartWidth ) - [sender dividerThickness]  ;
    return [self splitView:sender constrainSplitPosition:realWidth > virtualWidth ? realWidth : virtualWidth ofSubviewAt:offset];

}

- (float)splitView:(NSSplitView *)splitView constrainSplitPosition:(float)proposedPosition ofSubviewAt:(int)offset {
    return rint( proposedPosition / kMIMixerPartWidth ) * ( kMIMixerPartWidth - 1 ) + 3 ;
}

- (void)splitView:(NSSplitView *)sender resizeSubviewsWithOldSize:(NSSize)oldSize {
    [self resizeSubviewsOfSplitView:sender];
}

/*
#ifndef TWO_THREADS_NOTIFICATIONS
 - (void)notificationReceived:(NSArray*)packedParams {
     int code;
     id object;
     void* param;

     NSData* dataCode = [packedParams objectAtIndex:0];
     NSData* dataObject = [packedParams objectAtIndex:1];
     NSData* dataParam = [packedParams objectAtIndex:2];

     memcpy(&code, [dataCode bytes], sizeof(int));
     memcpy(&object, [dataObject bytes], sizeof(id));
     memcpy(&param, [dataParam bytes], sizeof(void*));

#else // #ifndef TWO_THREADS_NOTIFICATIONS*/
- (void)notificationReceived:(int)code param:(unsigned int)_param
{
    void* param = (void*)_param;
    //#endif // #ifndef TWO_THREADS_NOTIFICATIONS
    unsigned int x;
    BOOL state;

    verntl(notifications,5,"[MIMixerController notificationReceived]");
    switch(code) {

        case kNotification_Mixer_LeftVAOFader :
            [[fVirtualMixerParts objectAtIndex:(int)param] leftSliderChanged];
            break;
        case kNotification_Mixer_RightVAOFader :
            [[fVirtualMixerParts objectAtIndex:(int)param] rightSliderChanged];
            break;
        case kNotification_Mixer_LeftRAOFader :
            [[fRealMixerParts objectAtIndex:(int)param] leftSliderChanged];
            break;
        case kNotification_Mixer_RightRAOFader :
            [[fRealMixerParts objectAtIndex:(int)param] rightSliderChanged];
            break;
        case kNotification_Mixer_SoloedMonoVAOChanged :
            if ( (int)param == -1 ) state = YES;
            else state = NO;
            for ( x = 0 ; x < [fVirtualMonoMixerParts count] ; x++ ) {
                if ( x != (unsigned int)param ) {
                    [[fVirtualMonoMixerParts objectAtIndex:x] setSoloAndMuteButtonState:state];
                }
            }
                break;
        case kNotification_Mixer_SoloedStereoVAOChanged :
            if ( (int)param == -1 ) state = YES;
            else state = NO;
            for ( x = 0 ; x < [fVirtualMixerParts count] ; x++ ) {
                if ( x != (unsigned int)param ) {
                    [[fVirtualMixerParts objectAtIndex:x] setSoloAndMuteButtonState:state];
                }
            }
                break;
        case kNotification_Mixer_SoloedStereoRAOChanged :
            if ( (int)param == -1 ) state = YES;
            else state = NO;
            for ( x = 0 ; x < [fRealMixerParts count] ; x++ ) {
                if ( x != (unsigned int)param ) {
                    [[fRealMixerParts objectAtIndex:x] setSoloAndMuteButtonState:state];
                }
            }
                break;

        case kNotification_Mixer :
            [self updateContent:[[MIDocumentController sharedDocumentController] currentDocument]];
            break;
    }
}

- (BOOL)ignoreArrowKeys {
	id firstResponder = [[self window] firstResponder];
	if ( firstResponder ) {
		if ( [firstResponder isKindOfClass:NSClassFromString(@"NSTextView")] ) return NO;
	}
	return YES;
}

- (BOOL)ignoresUndoRedoActions {
	return YES;
}


@end

@implementation MISplitView

- (float)dividerThickness {
    return 6.0;
}

- (void)drawDividerInRect:(NSRect)aRect {
    int aqua = [[NSUserDefaults standardUserDefaults] integerForKey:@"AppleAquaColorVariant"];
    float x = rint(NSMinX(aRect)) + 0.5;
    if ( aqua == 0 ) aqua = 1; // pref is not defined
    if ( aqua == 1 ) [[NSColor colorWithCalibratedRed:19/255.0 green:85/255.0 blue:176/255.0 alpha:1.0] set];
    else [[NSColor colorWithCalibratedRed:89/255.0 green:103/255.0 blue:116/255.0 alpha:1.0] set];
    [NSBezierPath strokeLineFromPoint:NSMakePoint( x, NSMinY(aRect) ) toPoint:NSMakePoint( x, NSMaxY(aRect) )];

    if ( aqua == 1 ) [[NSColor colorWithCalibratedRed:164/255.0 green:197/255.0 blue:229/255.0 alpha:1.0] set];
    else [[NSColor colorWithCalibratedRed:199/255.0 green:204/255.0 blue:208/255.0 alpha:1.0] set];
    [NSBezierPath strokeLineFromPoint:NSMakePoint( x+1, NSMinY(aRect) ) toPoint:NSMakePoint( x+1, NSMaxY(aRect) )];

    if ( aqua == 1 ) [[NSColor colorWithCalibratedRed:183/255.0 green:209/255.0 blue:239/255.0 alpha:1.0] set];
    else [[NSColor colorWithCalibratedRed:212/255.0 green:217/255.0 blue:220/255.0 alpha:1.0] set];
    [NSBezierPath strokeLineFromPoint:NSMakePoint( x+2, NSMinY(aRect) ) toPoint:NSMakePoint( x+2, NSMaxY(aRect) )];

    if ( aqua == 1 ) [[NSColor colorWithCalibratedRed:120/255.0 green:173/255.0 blue:232/255.0 alpha:1.0] set];
    else [[NSColor colorWithCalibratedRed:179/255.0 green:187/255.0 blue:196/255.0 alpha:1.0] set];
    [NSBezierPath strokeLineFromPoint:NSMakePoint( x+3, NSMinY(aRect) ) toPoint:NSMakePoint( x+3, NSMaxY(aRect) )];

    if ( aqua == 1 ) [[NSColor colorWithCalibratedRed:153/255.0 green:204/255.0 blue:255/255.0 alpha:1.0] set];
    else [[NSColor colorWithCalibratedRed:207/255.0 green:216/255.0 blue:224/255.0 alpha:1.0] set];
    [NSBezierPath strokeLineFromPoint:NSMakePoint( x+4, NSMinY(aRect) ) toPoint:NSMakePoint( x+4, NSMaxY(aRect) )];

    if ( aqua == 1 ) [[NSColor colorWithCalibratedRed:19/255.0 green:84/255.0 blue:173/255.0 alpha:1.0] set];
    else [[NSColor colorWithCalibratedRed:90/255.0 green:104/255.0 blue:118/255.0 alpha:1.0] set];
    [NSBezierPath strokeLineFromPoint:NSMakePoint( x+5, NSMinY(aRect) ) toPoint:NSMakePoint( x+5, NSMaxY(aRect) )];

    [[NSColor colorWithCalibratedRed:128/255.0 green:128/255.0 blue:128/255.0 alpha:1.0] set];
    [NSBezierPath strokeLineFromPoint:NSMakePoint( x+2, NSMidY(aRect) - 5 ) toPoint:NSMakePoint( x+2, NSMidY(aRect) + 5 )];

    [[NSColor colorWithCalibratedRed:170/255.0 green:170/255.0 blue:170/255.0 alpha:1.0] set];
    [NSBezierPath strokeLineFromPoint:NSMakePoint( x+3, NSMidY(aRect) - 5 ) toPoint:NSMakePoint( x+3, NSMidY(aRect) + 5 )];

    if ( aqua == 1 ) [[NSColor colorWithCalibratedRed:48/255.0 green:114/255.0 blue:192/255.0 alpha:1.0] set];
    else [[NSColor colorWithCalibratedRed:118/255.0 green:130/255.0 blue:142/255.0 alpha:1.0] set];
    [NSBezierPath strokeLineFromPoint:NSMakePoint( x+1, NSMinY(aRect) ) toPoint:NSMakePoint( x+1, NSMinY(aRect) + 1 )];
    [NSBezierPath strokeLineFromPoint:NSMakePoint( x+1, NSMaxY(aRect) ) toPoint:NSMakePoint( x+1, NSMaxY(aRect) - 1 )];
    if ( aqua == 1 ) [[NSColor colorWithCalibratedRed:87/255.0 green:146/255.0 blue:213/255.0 alpha:1.0] set];
    else [[NSColor colorWithCalibratedRed:149/255.0 green:160/255.0 blue:170/255.0 alpha:1.0] set];
    [NSBezierPath strokeLineFromPoint:NSMakePoint( x+2, NSMinY(aRect) ) toPoint:NSMakePoint( x+2, NSMinY(aRect) + 1 )];
    [NSBezierPath strokeLineFromPoint:NSMakePoint( x+2, NSMaxY(aRect) ) toPoint:NSMakePoint( x+2, NSMaxY(aRect) - 1 )];
}


@end
