
#import "MIMixerPartController.h"

#ifndef TArmeDoc_h
    #include "TArmeDoc.h"
#endif //TArmeDoc_h

#include <stdlib.h>
#include "ArboLicenseManager.h"

#ifdef MTG_USE_LICENSER_KIT
    #if defined( MTG_LIGHT_VERSION ) || defined( MTG_FULL_VERSION )
        #import "CSLicensing.h"
        #import "MainApp.h"
    #endif
#endif //#ifdef MTG_USE_LICENSER_KIT

@implementation MIMixerPartController

NSString *MTGMixerLabelChangedNotification = @"MTG Mixer Label Changed";

- (id)initWithDocument:(MIMontageDocument*)document andStereoVAOIndex:(SInt32)index {
	if (( self = [super init] )) 
	{
		[self setDocument:document];
		fRAOVAOIndex = index;
		fIsRAO = NO;
		fStereo = YES;
		fOverloadButtonEnabled = false; 
		fLeftOverloadButtonIsOn = true; 
		fRightOverloadButtonIsOn = true; 
		
		if ( ![NSBundle loadNibNamed:@"MIMixerPart" owner:self] ) 
		{
			[self release];
			return nil;
		}
	}
	return self;
}

- (id)initWithDocument:(MIMontageDocument*)document andMonoVAOIndex:(SInt32)index {
	if (( self = [super init] )) {
		[self setDocument:document];
		fRAOVAOIndex = index;
		fIsRAO = NO;
		fStereo = NO;
		fOverloadButtonEnabled = false; 
		fLeftOverloadButtonIsOn = true; 
		fRightOverloadButtonIsOn = true; 

		if ( ![NSBundle loadNibNamed:@"MIMixerPart" owner:self] ) {
			[self release];
			return nil;
		}
	}
	return self;
}

- (id)initWithDocument:(MIMontageDocument*)document andRAOIndex:(SInt32)index 
{
	if (( self = [super init] )) {
		[self setDocument:document];
		fRAOVAOIndex = index;
		fIsRAO = YES;
		fStereo = YES;
		fOverloadButtonEnabled = false; 
		fLeftOverloadButtonIsOn = true; 
		fRightOverloadButtonIsOn = true; 

		if ( ![NSBundle loadNibNamed:@"MIMixerPart" owner:self] ) {
			[self release];
			return nil;
		}
	}
	return self;
}

- (void)dealloc {
	if ( !fIsRAO ) {
		[[NSNotificationCenter defaultCenter] removeObserver:self name:MTGMixerLabelChangedNotification object:nil];
	}
	[super dealloc];
}

- (void)awakeFromNib
{
	bool bMuted;
	float dB;

	[fLeftOverloadButton setState:NSOnState];
	[fRightOverloadButton setState:NSOnState];
	[fOverloadButton setEnabled:NO];
	
	if ( fIsRAO ) 
	{
		if ( fMixingInfo->GetSoloedStereoRAO() == -1 ) 
		{
			[fSoloButton setEnabled:YES];
			[fSoloButton setState:NSOffState];
		}
		else if ( fMixingInfo->GetSoloedStereoRAO() == fRAOVAOIndex ) 
		{
			[fSoloButton setEnabled:YES];
			[fSoloButton setState:NSOnState];
		} 
		else
		{
			[fSoloButton setEnabled:NO];
			[fSoloButton setState:NSOffState];
		}
		
		[fPhysicalOutputButton removeFromSuperview];
		[fPanSlider removeFromSuperview];

		// set the muted state
		bMuted = fMixingInfo->IsStereoRAOMuted(fRAOVAOIndex);
		if ( bMuted ) 
			[fMuteButton setState:NSOnState];
		else 
			[fMuteButton setState:NSOffState];
			
		// set the lock state
		[fLockButton setState:(fMixingInfo->GetStereoRAOLock(fRAOVAOIndex) ? NSOnState : NSOffState)];
		
		// set left and right fader value and dB
		dB = fMixingInfo->GetStereoRAOFaderDB(fRAOVAOIndex, true);
		[fLeftSlider setFloatValue:MEMixingInfo::dBToFaderValue(dB)];
		[fLeftInputField setFloatValue:dB];
		dB = fMixingInfo->GetStereoRAOFaderDB(fRAOVAOIndex, false);
		[fRightSlider setFloatValue:MEMixingInfo::dBToFaderValue(dB)];
		[fRightInputField setFloatValue:dB];

		[fLabelText setStringValue:[NSString stringWithCString:fMixingInfo->GetStereoRAOName(fRAOVAOIndex)]];
		if ( fRAOVAOIndex > 0 ) {
            if( !ArboGetLicenseManager()->PluginOrAppLicensed( ) ) {
                printf("!ArboGetLicenseManager()->PluginOrAppLicensed( ) --> disabling MIMixerPartController items\n");
				[fMuteButton setEnabled:NO];
				[fPanSlider setEnabled:NO];
				[fLeftSlider setEnabled:NO];
				[fRightSlider setEnabled:NO];
				[fRightInputField setEnabled:NO];
				[fLeftInputField setEnabled:NO];
				[fLabelText setEnabled:NO];
				[fPhysicalOutputButton setEnabled:NO];
				[fLeftOverloadButton setEnabled:NO];
				[fRightOverloadButton setEnabled:NO];
				[fLockButton setEnabled:NO];
				[fSoloButton setEnabled:NO];
				[fLeftVUMeter setEnabled:NO];
				[fRightVUMeter setEnabled:NO];
			}
            else {
				[fMuteButton setEnabled:YES];
				[fPanSlider setEnabled:YES];
				[fLeftSlider setEnabled:YES];
				[fRightSlider setEnabled:YES];
				[fRightInputField setEnabled:YES];
				[fLeftInputField setEnabled:YES];
				[fLabelText setEnabled:YES];
				[fPhysicalOutputButton setEnabled:YES];
				[fLeftOverloadButton setEnabled:YES];
				[fRightOverloadButton setEnabled:YES];
				[fLockButton setEnabled:YES];
				[fSoloButton setEnabled:YES];
				[fLeftVUMeter setEnabled:YES];
				[fRightVUMeter setEnabled:YES];
			}
		}
	}
	else if( fStereo ) 
	{
		if ( fMixingInfo->GetSoloedStereoVAO() == -1 ) 
		{
			[fSoloButton setEnabled:YES];
			[fSoloButton setState:NSOffState];
		}
		else if ( fMixingInfo->GetSoloedStereoVAO() == fRAOVAOIndex ) 
		{
			[fSoloButton setEnabled:YES];
			[fSoloButton setState:NSOnState];
		}
		else
		{
			[fSoloButton setEnabled:NO];
			[fSoloButton setState:NSOffState];
		}
		
		// set muted state
		bMuted = fMixingInfo->IsStereoVAOMuted(fRAOVAOIndex);
		if ( bMuted ) 
			[fMuteButton setState:NSOnState];
		else 
			[fMuteButton setState:NSOffState];
		
		// set panning
		[fPanSlider setFloatValue:fMixingInfo->GetStereoVAOPanning(fRAOVAOIndex) + 1.0];

		// set the muted state
		[fLockButton setState:(fMixingInfo->GetStereoVAOLock(fRAOVAOIndex) ? NSOnState : NSOffState)];

		// set left and right fader value and dB
		dB = fMixingInfo->GetStereoVAOFaderDB(fRAOVAOIndex, true);
		[fLeftSlider setFloatValue:MEMixingInfo::dBToFaderValue(dB)];
		[fLeftInputField setFloatValue:dB];
		dB = fMixingInfo->GetStereoVAOFaderDB(fRAOVAOIndex, false);
		[fRightSlider setFloatValue:MEMixingInfo::dBToFaderValue(dB)];
		[fRightInputField setFloatValue:dB];

		[fLabelText setStringValue:[NSString stringWithCString:fMixingInfo->GetStereoVAOName(fRAOVAOIndex)]];
		[self buildPopupMenu];
		[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(labelDidChanged:) name:MTGMixerLabelChangedNotification object:nil];
        if( !ArboGetLicenseManager()->PluginOrAppLicensed( ) ) {
            printf("!ArboGetLicenseManager()->PluginOrAppLicensed( ) --> disabling MIMixerPartController items\n");
            [fMuteButton setEnabled:NO];
            [fPanSlider setEnabled:NO];
            [fLeftSlider setEnabled:NO];
            [fRightSlider setEnabled:NO];
            [fRightInputField setEnabled:NO];
            [fLeftInputField setEnabled:NO];
            [fLabelText setEnabled:NO];
            [fPhysicalOutputButton setEnabled:NO];
            [fLeftOverloadButton setEnabled:NO];
            [fRightOverloadButton setEnabled:NO];
            [fLockButton setEnabled:NO];
            [fSoloButton setEnabled:NO];
            [fLeftVUMeter setEnabled:NO];
            [fRightVUMeter setEnabled:NO];
		}
	}
	else {
		if ( fMixingInfo->GetSoloedMonoVAO() == -1 ) {
			[fSoloButton setEnabled:YES];
			[fSoloButton setState:NSOffState];
		} else if ( fMixingInfo->GetSoloedMonoVAO() == fRAOVAOIndex ) {
			[fSoloButton setEnabled:YES];
			[fSoloButton setState:NSOnState];
		} else {
			[fSoloButton setEnabled:NO];
			[fSoloButton setState:NSOffState];
		}

		[fLockButton setState:NSOnState];
		[fLockButton setEnabled:NO];
		bMuted = fMixingInfo->IsMonoVAOMuted(fRAOVAOIndex);
		if ( bMuted ) [fMuteButton setState:NSOnState];
		else [fMuteButton setState:NSOffState];
		[fPanSlider setFloatValue:fMixingInfo->GetMonoVAOPanning(fRAOVAOIndex) + 1.0];

		dB = fMixingInfo->GetMonoVAOFaderDB(fRAOVAOIndex);
		[fLeftSlider setFloatValue:MEMixingInfo::dBToFaderValue(dB)];
		[fLeftInputField setFloatValue:dB];
		[fRightSlider setFloatValue:MEMixingInfo::dBToFaderValue(dB)]; //$$$TODO REMOVE RIGHT SLIDER
		[fRightInputField setFloatValue:dB];

		[fLabelText setStringValue:[NSString stringWithCString:fMixingInfo->GetMonoVAOName(fRAOVAOIndex)]];
		[self buildPopupMenu];
		[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(labelDidChanged:) name:MTGMixerLabelChangedNotification object:nil];
        if( !ArboGetLicenseManager()->PluginOrAppLicensed( ) ) {
            printf("!ArboGetLicenseManager()->PluginOrAppLicensed( ) --> disabling MIMixerPartController items\n");
            [fMuteButton setEnabled:NO];
            [fPanSlider setEnabled:NO];
            [fLeftSlider setEnabled:NO];
            [fRightSlider setEnabled:NO];
            [fRightInputField setEnabled:NO];
            [fLeftInputField setEnabled:NO];
            [fLabelText setEnabled:NO];
            [fPhysicalOutputButton setEnabled:NO];
            [fLeftOverloadButton setEnabled:NO];
            [fRightOverloadButton setEnabled:NO];
            [fLockButton setEnabled:NO];
            [fSoloButton setEnabled:NO];
            [fLeftVUMeter setEnabled:NO];
            [fRightVUMeter setEnabled:NO];
		}
	}
}

- (void)buildPopupMenu 
{
	SInt32 i;
	NSString* partTitle;

	[fPhysicalOutputButton removeAllItems];

    for ( i = 0 ; i < fArmeDoc->GetMixingInfo()->GetMaxStereoRAOCount(); i++ )
	{
		NSString* label = [NSString stringWithCString:fMixingInfo->GetStereoRAOName(i)];
		if ( [label length] > 0 ) 
		{
			partTitle = label;//[label stringByAppendingFormat:@" %d/%d",(i*2)+1,(i*2)+2];
		}
		else
		{
			partTitle = [NSString stringWithFormat:@"Output %d/%d",(i*2)+1,(i*2)+2];
		}
		[fPhysicalOutputButton addItemWithTitle:partTitle];
    }

	if(fArmeDoc->GetMixingInfo()->GetMaxStereoRAOCount() > 0)
	{
		if(fStereo)
			[fPhysicalOutputButton selectItemAtIndex:fMixingInfo->GetStereoRAOForStereoVAO(fRAOVAOIndex)];
		else
			[fPhysicalOutputButton selectItemAtIndex:fMixingInfo->GetStereoRAOForMonoVAO(fRAOVAOIndex)];
	}
}

- (void)labelDidChanged:(NSNotification*)notification {
	[self buildPopupMenu];
}

- (void)removeFromSuperview {
	[fMixerPartView removeFromSuperview];
}

- (float)getLeftVUMeterValue:(id)sender {
	float value;
	//return ((float)random())/((float)LONG_MAX);
	if ( fIsRAO ) 
	{
		value = fMixingInfo->GetDBFSLevelForStereoRAO(fRAOVAOIndex, true);
	}
	else if ( fStereo ) 
	{
		value = fMixingInfo->GetDBFSLevelForStereoVAO(fRAOVAOIndex, true);
	}
	else 
	{
		value = fMixingInfo->GetDBFSLevelForMonoVAO(fRAOVAOIndex);
	}
//	verntl(MEMixingInfo,6,"MIMixerPartController getLeftVUMeterValue dBFS = " << value << " ---> returning " << MEMixingInfo::dBToVULevel(value));
	return (MEMixingInfo::dBToVULevel(value)*0.01);
}

- (float)getRightVUMeterValue:(id)sender {
	float value;
	//return ((float)random())/((float)LONG_MAX);
	if ( fIsRAO ) 
	{
		value = fMixingInfo->GetDBFSLevelForStereoRAO(fRAOVAOIndex, false);
	}
	else if ( fStereo ) 
	{
		value = fMixingInfo->GetDBFSLevelForStereoVAO(fRAOVAOIndex, false);
	}
	else 
	{	
		value = fMixingInfo->GetDBFSLevelForMonoVAO(fRAOVAOIndex);
	}

	return (MEMixingInfo::dBToVULevel(value)*0.01);
}

- (void)setDocument:(MIMontageDocument*)document
{
	fDocument = document;
	fArmeDoc = [document armeDocument];
	fMixingInfo = fArmeDoc->GetMixingInfo();
}

- (IBAction)controllerChanged:(id)sender
{
}

- (IBAction)labelChanged:(id)sender
{
	fCommand = [(MIMixerCommand*)[MIMixerCommand alloc] initWithDocument:fArmeDoc];

	if(fIsRAO)
	{
		[fCommand setStereoRAOName:fRAOVAOIndex name:[[sender stringValue] UTF8String]];
		VUSERCMD2("Mixer : Set Stereo RAO Name %d -> %s", (int)fRAOVAOIndex, [[sender stringValue] cString]);
	}
	else if(fStereo)
	{
		[fCommand setStereoVAOName:fRAOVAOIndex name:[[sender stringValue] UTF8String]];
		VUSERCMD2("Mixer : Set Stereo VAO Name %d -> %s", (int)fRAOVAOIndex, [[sender stringValue] cString]);
	}
	else
	{
		[fCommand setMonoVAOName:fRAOVAOIndex name:[[sender stringValue] UTF8String]];
		VUSERCMD2("Mixer : Set Mono VAO Name %d -> %s", (int)fRAOVAOIndex, [[sender stringValue] cString]);
	}
	
	[fCommand executeInDoc:fDocument];
	[fCommand release];
	fCommand = nil;

	if ( fIsRAO ) 
	{
		[[NSNotificationCenter defaultCenter] postNotificationName:MTGMixerLabelChangedNotification object:self];
	}
}

- (IBAction)leftOverloadButtonPressed:(id)sender 
{
	if ( fIsRAO ) 
	{
		fMixingInfo->ResetStereoRAOSaturation(fRAOVAOIndex, true);
	}
	else if ( fStereo ) 
	{
		fMixingInfo->ResetStereoVAOSaturation(fRAOVAOIndex, true);
	} 
	else 
	{
		fMixingInfo->ResetMonoVAOSaturation(fRAOVAOIndex);
		[fRightOverloadButton setState:NSOnState];
	}

	if ( [fRightOverloadButton state] == NSOnState ) [fOverloadButton setEnabled:NO];
}

- (IBAction)rightOverloadButtonPressed:(id)sender 
{
	if ( fIsRAO ) 
	{
		fMixingInfo->ResetStereoRAOSaturation(fRAOVAOIndex, false);
	} 
	else if ( fStereo ) 
	{
		fMixingInfo->ResetStereoVAOSaturation(fRAOVAOIndex, false);
	} 
	else
	{
		fMixingInfo->ResetMonoVAOSaturation(fRAOVAOIndex);
		[fLeftOverloadButton setState:NSOnState];
	}

	if ( [fLeftOverloadButton state] == NSOnState ) [fOverloadButton setEnabled:NO];
}

- (IBAction)muteButtonPressed:(id)sender {
	bool bMuted;
	
	if ( [fMuteButton state] == NSOnState ) bMuted = true;
	else bMuted = false;

	fCommand = [(MIMixerCommand*)[MIMixerCommand alloc] initWithDocument:fArmeDoc];

	if ( fIsRAO )
	{
		[fCommand setStereoRAOMute:fRAOVAOIndex muted:bMuted];
		VUSERCMD2("Mixer : Set Stereo RAO Mute %d -> %d", (int)fRAOVAOIndex, bMuted);
	}
	else if ( fStereo )
	{	
		[fCommand setStereoVAOMute:fRAOVAOIndex muted:bMuted];
		VUSERCMD2("Mixer : Set Stereo VAO Mute %d -> %d",(int) fRAOVAOIndex, bMuted);
	}
	else 
	{
		[fCommand setMonoVAOMute:fRAOVAOIndex muted:bMuted];
		VUSERCMD2("Mixer : Set Mono VAO Mute %d -> %d", (int)fRAOVAOIndex, bMuted);
	}
	
	[fCommand executeInDoc:fDocument];
	[fCommand release];
	fCommand = nil;
}

- (IBAction)soloButtonPressed:(id)sender {
	SInt32 index = fRAOVAOIndex ;

	if ( [fSoloButton state] == NSOffState ) index = -1;

	fCommand = [(MIMixerCommand*)[MIMixerCommand alloc] initWithDocument:fArmeDoc];

	if ( fIsRAO ) 
	{
		[fCommand setSoloedStereoRAO:index];
		VUSERCMD1("Mixer : Set Soloed Stereo RAO %d", (int)index);
	}
	else if ( fStereo ) 
	{
		[fCommand setSoloedStereoVAO:index];
		VUSERCMD1("Mixer : Set Soloed Stereo VAO %d", (int)index);
	}
	else 
	{
		[fCommand setSoloedMonoVAO:index];
		VUSERCMD1("Mixer : Set Soloed Mono VAO %d", (int)index);
	}

	[fCommand executeInDoc:fDocument];
	[fCommand release];
	fCommand = nil;
}

- (void)setSoloAndMuteButtonState:(BOOL)state {
	if ( state ) {
		BOOL bMuted;
		if ( fIsRAO ) {
			bMuted = fMixingInfo->IsStereoRAOMuted(fRAOVAOIndex);
		}
		else if ( fStereo ) {
			bMuted = fMixingInfo->IsStereoVAOMuted(fRAOVAOIndex);
		}
		else {
			bMuted = fMixingInfo->IsMonoVAOMuted(fRAOVAOIndex);
		}
		if ( bMuted ) [fMuteButton setState:NSOnState];
		else [fMuteButton setState:NSOffState];
	} else [fMuteButton setState:NSOnState];
	[fMuteButton setEnabled:state];
	[fSoloButton setEnabled:state];
}

- (IBAction)lockButtonPressed:(id)sender {
	fCommand = [(MIMixerCommand*)[MIMixerCommand alloc] initWithDocument:fArmeDoc];

	if ( fIsRAO )
	{
		[fCommand setStereoRAOLock:fRAOVAOIndex locked:( [fLockButton state] == NSOnState )];
		VUSERCMD2("Mixer : Set Stereo RAO Lock %d -> %d", (int)fRAOVAOIndex, ( [fLockButton state] == NSOnState ));
	}
	else if ( fStereo ) 
	{	
		[fCommand setStereoVAOLock:fRAOVAOIndex locked:( [fLockButton state] == NSOnState )];
		VUSERCMD2("Mixer : Set Stereo VAO Lock %d -> %d", (int)fRAOVAOIndex, ( [fLockButton state] == NSOnState ));
	}

	[fCommand executeInDoc:fDocument];
	[fCommand release];
	fCommand = nil;
}

- (IBAction)zeroButtonPressed:(id)sender {
	float sliderVal = fMixingInfo->dBToFaderValue(0);
	[fRightSlider setFloatValue:sliderVal];
	[self rightSliderMoved:fRightSlider];
	[fLeftSlider setFloatValue:sliderVal];
	[self leftSliderMoved:fLeftSlider];
}

- (IBAction)overloadButtonPressed:(id)sender {
	if ( [fLeftOverloadButton state] == NSOffState ) {
		[fLeftOverloadButton setState:NSOnState];
		[self leftOverloadButtonPressed:fLeftOverloadButton];
	}
	if ( [fRightOverloadButton state] == NSOffState ) {
		[fRightOverloadButton setState:NSOnState];
		[self rightOverloadButtonPressed:fRightOverloadButton];
	}
	[fOverloadButton setEnabled:NO];
}


- (IBAction)leftInputFieldModified:(id)sender {
	float value = [fLeftInputField floatValue];
	ZzAssert(value >= -192.0 && value <= 6.0);

	float sliderVal = fMixingInfo->dBToFaderValue(value);

	fCommand = [(MIMixerCommand*)[MIMixerCommand alloc] initWithDocument:fArmeDoc];

	if ( fIsRAO ) 
	{
		[fCommand setStereoRAOFaderValue:fRAOVAOIndex value:sliderVal left:YES];
		VUSERCMD2("Mixer : Set Left Stereo RAO Fader Value %d -> %f", (int)fRAOVAOIndex, sliderVal);
	}
	else if ( fStereo ) 
	{
		[fCommand setStereoVAOFaderValue:fRAOVAOIndex value:sliderVal left:YES];
		VUSERCMD2("Mixer : Set Left Stereo VAO Fader Value %d -> %f", (int)fRAOVAOIndex, sliderVal);
	}
	else {
		[fCommand setMonoVAOFaderValue:fRAOVAOIndex value:sliderVal];
		VUSERCMD2("Mixer : Set Mono VAO Fader Value %d -> %f", (int)fRAOVAOIndex, sliderVal);
		[fRightInputField setFloatValue:value];
		[fRightSlider setFloatValue:sliderVal];
	}

	[fLeftSlider setFloatValue:sliderVal];

	[fCommand executeInDoc:fDocument];
	[fCommand release];
	fCommand = nil;
}

- (IBAction)rightInputFieldModified:(id)sender {
	float value = [fRightInputField floatValue];
	ZzAssert(value >= -192.0 && value <= 6.0);

	float sliderVal = fMixingInfo->dBToFaderValue(value);

	fCommand = [(MIMixerCommand*)[MIMixerCommand alloc] initWithDocument:fArmeDoc];

	if ( fIsRAO ) 
	{
		[fCommand setStereoRAOFaderValue:fRAOVAOIndex value:sliderVal left:NO];
		VUSERCMD2("Mixer : Set Right Stereo RAO Fader Value %d -> %f", (int)fRAOVAOIndex, sliderVal);
	}
	else if ( fStereo ) 
	{
		[fCommand setStereoVAOFaderValue:fRAOVAOIndex value:sliderVal left:NO];
		VUSERCMD2("Mixer : Set Right Stereo VAO Fader Value %d -> %f", (int)fRAOVAOIndex, sliderVal);
	}
	else {
		[fCommand setMonoVAOFaderValue:fRAOVAOIndex value:sliderVal];
		VUSERCMD2("Mixer : Set Mono VAO Fader Value %d -> %f", (int)fRAOVAOIndex, sliderVal);
		[fLeftInputField setFloatValue:value];
		[fLeftSlider setFloatValue:sliderVal];
	}

	[fRightSlider setFloatValue:sliderVal];

	[fCommand executeInDoc:fDocument];
	[fCommand release];
	fCommand = nil;
}

- (IBAction)physicalOutputChanged:(id)sender {
	fCommand = [(MIMixerCommand*)[MIMixerCommand alloc] initWithDocument:fArmeDoc];

	if ( fIsRAO )  { ZzAssert(false); }
	else if ( fStereo ) 
	{
		[fCommand setStereoRAOForStereoVAO:fRAOVAOIndex RAO:[fPhysicalOutputButton indexOfSelectedItem]];
		VUSERCMD2("Mixer : Set Stereo RAO For Stereo VAO %d -> %d", (int)fRAOVAOIndex, [fPhysicalOutputButton indexOfSelectedItem]);
	}
	else 
	{
		[fCommand setStereoRAOForMonoVAO:fRAOVAOIndex RAO:[fPhysicalOutputButton indexOfSelectedItem]];
		VUSERCMD2("Mixer : Set Stereo RAO For Mono VAO %d -> %d", (int)fRAOVAOIndex, [fPhysicalOutputButton indexOfSelectedItem]);
	}

	[fCommand executeInDoc:fDocument];
	[fCommand release];
	fCommand = nil;
}

- (IBAction)panSliderMoved:(id)sender 
{
	// 0 is left, 1 is middle, 2 is right
	float panVal = [sender floatValue] - 1.0;
	ZzAssert(panVal >= -1.0 && panVal <= 1.0);

	if ( !fCommand ) {
		fCommand = [(MIMixerCommand*)[MIMixerCommand alloc] initWithDocument:fArmeDoc];
	}

	if ( fIsRAO )
	{
		ZzAssert(false); 
	}
	else if ( fStereo ) 
	{
		[fCommand setStereoVAOPanning:fRAOVAOIndex panning:panVal];
	}
	else
	{
		[fCommand setMonoVAOPanning:fRAOVAOIndex panning:panVal];
	}

	if ( ![sender isTracking] ) 
	{
		[fCommand executeInDoc:fDocument];
        [fCommand release];
		fCommand = nil;
	}
}

- (IBAction)leftSliderMoved:(id)sender {
	// 0 -> 100
	float sliderVal = [sender floatValue];
	ZzAssert(sliderVal >= 0.0 && sliderVal <= 100.0);

	if ( !fCommand ) 
	{
		fCommand = [(MIMixerCommand*)[MIMixerCommand alloc] initWithDocument:fArmeDoc];
	}

	if ( fIsRAO ) 
	{
		[fCommand setStereoRAOFaderValue:fRAOVAOIndex value:sliderVal left:YES];
	}
	else if ( fStereo ) 
	{
		[fCommand setStereoVAOFaderValue:fRAOVAOIndex value:sliderVal left:YES];
	}
	else 
	{
		[fCommand setMonoVAOFaderValue:fRAOVAOIndex value:sliderVal];
		[fRightInputField setFloatValue:MEMixingInfo::FaderValueTodB(sliderVal)];
	}

	[fLeftInputField setFloatValue:MEMixingInfo::FaderValueTodB(sliderVal)];
	
	if ( [fLockButton state] == NSOnState ) 
	{
		[fRightSlider setFloatValue:sliderVal];
		[fRightInputField setFloatValue:MEMixingInfo::FaderValueTodB(sliderVal)];
	}

	if ( ![sender isTracking] ) {
		[fCommand executeInDoc:fDocument];
        [fCommand release];
		fCommand = nil;
	}
}

- (IBAction)rightSliderMoved:(id)sender 
{
	// 0 -> 100
	float sliderVal = [sender floatValue];
	ZzAssert(sliderVal >= 0.0 && sliderVal <= 100.0);

	if ( !fCommand ) 
	{
		fCommand = [(MIMixerCommand*)[MIMixerCommand alloc] initWithDocument:fArmeDoc];
	}

	if ( fIsRAO ) 
	{
		[fCommand setStereoRAOFaderValue:fRAOVAOIndex value:sliderVal left:NO];
	}
	else if ( fStereo )
	{
		[fCommand setStereoVAOFaderValue:fRAOVAOIndex value:sliderVal left:NO];
	}
	else 
	{
		[fCommand setMonoVAOFaderValue:fRAOVAOIndex value:sliderVal];
		[fLeftInputField setFloatValue:MEMixingInfo::FaderValueTodB(sliderVal)];
	}

	[fRightInputField setFloatValue:MEMixingInfo::FaderValueTodB(sliderVal)];
	if ( [fLockButton state] == NSOnState )
	{
		[fLeftSlider setFloatValue:sliderVal];
		[fLeftInputField setFloatValue:MEMixingInfo::FaderValueTodB(sliderVal)];
	}
	
	if ( ![sender isTracking] ) 
	{
		[fCommand executeInDoc:fDocument];
        [fCommand release];
		fCommand = nil;
	}
}

- (void)leftSliderChanged 
{
	float value;
	
	if ( fIsRAO ) 
		value = fMixingInfo->GetStereoRAOFaderDB(fRAOVAOIndex, true);
	else if ( fStereo ) 
		value = fMixingInfo->GetStereoVAOFaderDB(fRAOVAOIndex, true);
	else 
		value = fMixingInfo->GetMonoVAOFaderDB(fRAOVAOIndex);
	
	[fLeftSlider setFloatValue:MEMixingInfo::dBToFaderValue(value)];
	[fLeftInputField setFloatValue:value];
}

- (void)rightSliderChanged {
	float value;
	
	if ( fIsRAO ) 
		value = fMixingInfo->GetStereoRAOFaderDB(fRAOVAOIndex, false);
	else if ( fStereo ) 
		value = fMixingInfo->GetStereoVAOFaderDB(fRAOVAOIndex, false);
	else 
		value = fMixingInfo->GetMonoVAOFaderDB(fRAOVAOIndex);

	[fRightSlider setFloatValue:MEMixingInfo::dBToFaderValue(value)];
	[fRightInputField setFloatValue:value];
}

- (void)panSliderChanged {
	if ( fStereo ) {
		[fPanSlider setFloatValue:fMixingInfo->GetStereoVAOPanning(fRAOVAOIndex)];
	}
	else {
		[fPanSlider setFloatValue:fMixingInfo->GetMonoVAOPanning(fRAOVAOIndex)];
	}
}

- (void)updateVUMeters {
	if ( !NSIsEmptyRect([fLeftVUMeter visibleRect]) ) {
		[fLeftVUMeter setLevelAndDisplay:[self getLeftVUMeterValue:fLeftVUMeter]];
	}

	if ( !NSIsEmptyRect([fRightVUMeter visibleRect]) ) {
		[fRightVUMeter setLevelAndDisplay:[self getRightVUMeterValue:fRightVUMeter]];
	}

	if ( fIsRAO ) 
	{
		if ( fMixingInfo->IsStereoRAOSaturated(fRAOVAOIndex, true) ) 
		{
			if(fLeftOverloadButtonIsOn)
			{
				fLeftOverloadButtonIsOn = NO;
				[fLeftOverloadButton setState:NSOffState];
				[fOverloadButton setEnabled:YES];
			}
		}
		else
		{
			if(!fLeftOverloadButtonIsOn)
			{
				fLeftOverloadButtonIsOn = YES;
				[fLeftOverloadButton setState:NSOnState];
				[fOverloadButton setEnabled:YES];
			}
		}
		if ( fMixingInfo->IsStereoRAOSaturated(fRAOVAOIndex, false) ) 
		{
			if(fRightOverloadButtonIsOn)
			{
				fRightOverloadButtonIsOn = NO;
				[fRightOverloadButton setState:NSOffState];
				[fOverloadButton setEnabled:YES];
			}
		}
		else
		{
			if(!fRightOverloadButtonIsOn)
			{
				fRightOverloadButtonIsOn = YES;
				[fRightOverloadButton setState:NSOnState];
				[fOverloadButton setEnabled:YES];
			}
		}
	}
	else if ( fStereo ) 
	{
		if ( fMixingInfo->IsStereoVAOSaturated(fRAOVAOIndex, true) ) 
		{
			if(fLeftOverloadButtonIsOn)
			{
				fLeftOverloadButtonIsOn = NO;
				[fLeftOverloadButton setState:NSOffState];
				[fOverloadButton setEnabled:YES];
			}
		}
		else
		{
			if(!fLeftOverloadButtonIsOn)
			{
				fLeftOverloadButtonIsOn = YES;
				[fLeftOverloadButton setState:NSOnState];
				[fOverloadButton setEnabled:YES];
			}
		}
		if ( fMixingInfo->IsStereoVAOSaturated(fRAOVAOIndex, false) ) 
		{
			if(fRightOverloadButtonIsOn)
			{
				fRightOverloadButtonIsOn = NO;
				[fRightOverloadButton setState:NSOffState];
				[fOverloadButton setEnabled:YES];
			}
		}
		else
		{
			if(!fRightOverloadButtonIsOn)
			{
				fRightOverloadButtonIsOn = YES;
				[fRightOverloadButton setState:NSOnState];
				[fOverloadButton setEnabled:YES];
			}
		}
	} 
	else 
	{
		if ( fMixingInfo->IsMonoVAOSaturated(fRAOVAOIndex) ) 
		{
			if(fLeftOverloadButtonIsOn || fRightOverloadButtonIsOn)
			{
				fLeftOverloadButtonIsOn = false;
				fRightOverloadButtonIsOn = false;
				[fLeftOverloadButton setState:NSOffState];
				[fRightOverloadButton setState:NSOffState];
				[fOverloadButton setEnabled:YES];
			}
		}
		else
		{
			if(!fLeftOverloadButtonIsOn || !fRightOverloadButtonIsOn)
			{
				fLeftOverloadButtonIsOn = YES;
				fRightOverloadButtonIsOn = YES;
				[fLeftOverloadButton setState:NSOnState];
				[fRightOverloadButton setState:NSOnState];
				[fOverloadButton setEnabled:YES];
			}
		}
	}
}

@end
