/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

//#import "MontageInterface.h"

#ifndef LibraryToggle_h
#include "LibraryToggle.h"
#endif //LibraryToggle_h

#ifndef ArmeArchiveHandler_h
#include "../Arme/ArmeArchiveHandler.h"
#endif //ArmeArchiveHandler_h

#ifndef ArmeArchiveHandler_h
#include "../Arme/ArmeArchiveHandler.h"
#endif //ArmeArchiveHandler_h

#ifndef ArmeObjCArchivable_h
#include "ArmeObjCArchivable.h"
#endif //ArmeObjCArchivable_h

#import "MIDocumentModel.h"

#ifndef WorkingCodec_h
#include "WorkingCodec.h"
#endif //WorkingCodec_h


extern NSString *MTGArchiveName;
extern NSString *MTGArchiveBackupName;
extern NSString *MTGArchiveCorruptedName;

extern NSString *MTGMasterName;
extern NSString *MTGMasterBackupName;
extern NSString *MTGMasterCorruptedName;
extern NSString *MTGWorkingMasterName;

extern NSString *MTGDocumentSettingsName;


typedef enum fNSDocumentChangeType {
    MIChangeDone = 0,
    MIChangeUndone = 1,
    MIChangeCleared = 2,
    MIChangeRedone = 4
} MIDocumentChangeType;

class MIObjCProxyListener;
class TArmeWSObj;
class TArmeDoc;

@class MIDocumentModel;
@class MIMainViewController;
@class MILock;
@class MICommand;
@class MIObject;
@class MISnapperDB;
#if USE_LIBRARY
$$$$$
@class MILibraryElement;
#endif //USE_LIBRARY

@interface MIMontageDocument : NSDocument
{
@public
	MILock*					fReadWriteLock;
	MIObjCProxyListener*	fListener;
	MIDocumentModel*		fModel;
	MISnapperDB*			fSnapperDB;
	BOOL					fShouldUpdateChangeCount;
	int						fChangeCount;
	NSMutableDictionary		*fDocumentSettings;
	BOOL					fIsInited;
	
	NSMutableArray			*fVideoClipRepIncompletelyDrawn;
	NSMutableArray			*fObjectsSetOuttaHierarchyRecently;
}

- (id)initWithVideoFormat:(int)format;

- (void)deleteUnusedFiles;
- (bool)copyWorkingMasterInMaster:(NSString*)documentPath;

- (TArmeDoc*)armeDocument;
- (bool)recreateNonArchivedCachedDataWithController:(MIMainViewController*)mainViewController;

- (NSString*)projectFolderPath;
- (NSString*)projectFilePath;

- (MIObject*)topLevelContainer;
- (void)setTopLevelContainer:(MIObject*)tlc;

- (void)setSignatureNumerator:(int)num;
- (void)setSignatureDenominator:(int)den;
- (void)setTempo:(int)tempo;
- (int)signatureNumerator;
- (int)signatureDenominator;
- (int)tempo;

#if USE_LIBRARY
- (MILibraryElement*)libraryRoot;
#endif //USE_LIBRARY

// as soon as an operation on a MIObject potentially changes the snapping database 
// (a  flat list of all the object guides), we need to udpate it. 
- (void)refreshSnapDB:(MIObject*)obj;
- (void)deleteFromSnapDB:(MIObject*)obj;
- (MISnapperDB*)snapperDB;

- (NSMutableArray*)undoStack;
- (NSMutableArray*)redoStack;
- (void)addCommandToUndoStack:(MICommand*)cmd;
- (void)removeCommandFromUndoStack:(MICommand*)cmd;
- (NSString*)nameOfUndoCommand;
- (NSString*)nameOfRedoCommand;

- (MIObject*)findObject:(TArmeWSObj*)armeObj;

// marker management
- (UInt32)addMarkerAtTime:(ArmeTime)marker;
- (void)removeMarker:(UInt32)marker;
- (void)moveMarker:(UInt32)marker toTime:(ArmeTime)newTime;
- (UInt32)getMarkerCount;
- (ArmeTime)getMarker:(UInt32)i;
- (UInt32)getMarkerAtTime:(ArmeTime)time;
- (UInt32)getClosestMarkerForTime:(ArmeTime)time;
- (ArmeTime)ArmeTimeWidth:(ArmeTime)a withArmeTime:(ArmeTime)b;

/*! 
	@function notificationReceived 
	This function is called when someone has requested you receive
	it through MIApplicationController's sendNotification function.
	Don't worry, this function will always execute in the AppKit thread
	(just like a mouseDown or drawRect method)
*/
- (void)notificationReceived:(UInt32)code param:(void*)param;

- (void)lockRead;
- (void)unlockRead;
- (void)lockWrite;
- (void)unlockWrite;

- (void)doCmd:(MICommand*)cmd;
- (IBAction)undo:(id)sender;
- (IBAction)redo:(id)sender;

- (NSMutableArray*)savedViews;

- (void)addToOuttaHierarchyTransitionsPool:(MIObject*)object;
- (void)removeFromOuttaHierarchyTransitionsPool:(MIObject*)object;
- (NSMutableArray*)outtaHierarchyTransitionsPool;

- (void)createTopContainerObjectWithWindowController:(MIMainViewController*)mainViewController;
- (void)updateDocumentChangeCount:(MIDocumentChangeType)changeType;

- (void)setDocumentSetting:(id)setting forKey:(NSString*)key;
- (id)documentSettingForKey:(NSString*)key;
- (void)saveDocumentSettings;

- (void)restartGenerationOfThumbnails;
//- (long long)freeDiskSpace;
- (BOOL)hasContent;
- (BOOL)validateMenuItem:(NSMenuItem *)anItem;

- (void)playbackQualityHasChanged:(NSNotification*)notification;
@end


