/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#ifndef LibraryToggle_h
#include "LibraryToggle.h"
#endif //LibraryToggle_h

#import "MIMontageDocument.h"
#import "MIMainViewController.h"
#import "MIDocumentModel.h"
#import "MIObjectRep.h"
#import "MIObject.h"
#import "MIClipRep.h"
#import "MITopLevelContainerRep.h"
#import "MILock.h"
#import "MIApplicationController.h"
#import "MIApplication.h"
#import "MIRuler.h"
#import "MICommand.h"
#import "MISnapperDB.h"
#import "MIPreferences.h"
#import "MIOverviewController.h"
#import "MIStartingPoint.h"
#import "MIDocumentController.h"
#import "MenuTags.h"
#import "MIUpdateVideoTransitionsCommand.h"
#import "MIApplication.h"
#import "MITimePaletteController.h"
#import "MICopyCommand.h"

#if USE_LIBRARY
#import "MILibraryController.h"
#endif //USE_LIBRARY

#import "MITemporaryDirectory.h"

#include <stdlib.h>

#ifndef TArmeMasterMovie_h
#include "TArmeMasterMovie.h"
#endif //TArmeMasterMovie_h

#ifndef MovieHelper_h
#include "MovieHelper.h"
#endif //MovieHelper_h

#ifndef MIObjCProxyListener_h
#include "MIObjCProxyListener.h"
#endif //MIObjCProxyListener_h

#ifndef ArmeNotifications_h
#include "../Arme/ArmeNotifications.h"
#endif //ArmeNotifications_h

#ifndef TArmeDoc_h
#include "TArmeDoc.h"
#endif //TArmeDoc_h

#ifndef MontageArchiveHandler_h
#include "MontageArchiveHandler.h"
#endif //MontageArchiveHandler_h

#ifndef ZzHelpers_h
#include "ZzHelpers.h"
#endif //ZzHelpers_h

#ifndef Zz_PathHelpers_h
#include "Zz_PathHelpers.h"
#endif //Zz_PathHelpers_h

#ifndef ArmeDataFileRef_h
#include "ArmeDataFileRef.h"
#endif //ArmeDataFileRef_h

#ifndef MEAudioClip_h
#include "MEAudioClip.h"
#endif //MEAudioClip_h

#ifndef MESplitTaskComputeAudioClipDataPreview_h
#include "MESplitTaskComputeAudioClipDataPreview.h"
#endif //MESplitTaskComputeAudioClipDataPreview_h

#ifndef MIProgressBarDlgManager_h
#include "MIProgressBarDlgManager.h"
#endif //MIProgressBarDlgManager_h

#ifndef ZzException_h
#include "ZzException.h"
#endif //ZzException_h

#undef VER_DEF_TOPIC
#define VER_DEF_TOPIC MIMontageDocument

NSString *MTGArchiveName = @"project.mtg";
NSString *MTGArchiveBackupName = @"project~.mtg";
NSString *MTGArchiveCorruptedName = @"project.corrupted";

NSString *MTGMasterName = @"Master.mov";
NSString *MTGMasterBackupName = @"Master~.mov";
NSString *MTGMasterCorruptedName = @"Master.corrupted";
NSString *MTGWorkingMasterName = @"WorkingMaster.mov";

NSString *MTGDocumentSettingsName = @"Settings.plist";

bool gbVideoClipRepIncompletelyDrawnIsReleased = false;  //see the $$HACK below

#if USE_LIBRARY
bool gEnableLibraryUpdates = true;
#endif //USE_LIBRARY

@implementation MIMontageDocument


- (MIObject*)topLevelContainer
{
	return [fModel topLevelContainer];
}

- (void)setTopLevelContainer:(MIObject*)tlc
{
    [fModel setTopLevelContainer:tlc];
}

#if USE_LIBRARY
- (MILibraryElement*)libraryRoot
{
	return [fModel libraryRoot];
}
#endif //USE_LIBRARY

- (void)setSignatureNumerator:(int)num
{
	[fModel setSignatureNumerator:num];
}

- (void)setSignatureDenominator:(int)den
{
	[fModel setSignatureDenominator:den];
}

- (void)setTempo:(int)tempo
{
	[fModel setTempo:tempo];
}

- (void)close
{
	VUSERCMD("Close document");

	[[MIApplicationController sharedApplicationController] pausePlaying:nil];
	[self saveDocumentSettings];
	vernl(6,"-[MIMontageDocument close] called");
		// let's remove the selections THIS ABSOLUTELY 
		// need to be done BEFORE the fModel is released 
		// otherwise there would be reps with references 
		// to non-existing objects !!!!!!!
		MIObject* tlc = [fModel topLevelContainer];
		for(UInt32 i = 0; i < [[tlc getRepresentations] count]; i++)
		{
			MITopLevelContainerRep* tlcRep = [[tlc getRepresentations] objectAtIndex:i];
			MIMainViewController* mainViewController = [tlcRep mainViewController];
			[mainViewController clearSelection];
		}
	// this hack is done so that the document is closed DIRECTLY !
	// so that the applicationWillTerminate notifications are sent
	// AFTER the docs are ALL close and then we can safely release
	// all the threads, blah blah..
	NSAutoreleasePool* pool = [[NSAutoreleasePool alloc] init];
	[super close];
	[pool release];
}

- (int)signatureNumerator
{
	return [fModel signatureNumerator];
}
- (int)signatureDenominator
{
	return [fModel signatureDenominator];
}

- (int)tempo
{
	return [fModel tempo];
}

/*
- (NSArray*)objects
{
	return fObjects;
}
- (void)addObject:(MIObject*)object
{
	NSEnumerator			*mainViewEnumerator;
	MIMainViewController	*aMainViewController;
	
	
	if ( !object )
	{
		verntl(interfaceStephan,0,"Null object passed to addObject in MIMontageDocument\n\r");
		return;
	}
	
	[object retain];
	[fObjects addObject:object];
	
	[object setMontageDoc:self];

	// now we create the representations of this object
	mainViewEnumerator	= [fMainViewControllers objectEnumerator];
	while ( aMainViewController = [mainViewEnumerator nextObject] )
	{
		[object createRepresentationInView:[aMainViewController topLevelContainerView] atPos:NSMakePoint(10,10)];
		[[aMainViewController topLevelContainerView] setNeedsDisplay:YES];
	}
}

- (void)removeObject:(MIObject*)object
{
if ( !object )
	{
		verntl(interfaceStephan,0,"Null object passed to removeObject in MIMontageDocument\n\r");
		return;
	}
	
	[fObjects removeObject:object];
	[object release];

}
*/

- (MIObject*)findObject:(TArmeWSObj*)armeObj
{
	return  [[fModel topLevelContainer] findObjectRecursively:armeObj];
}

- (NSString*)printObject:(MIObject*)anObject withIndent:(int)indentLevel
{
	NSString		*desc;
	NSString		*spaces;
	NSEnumerator	*childrenEnum;
	MIObject		*aChild;
	int				i;
    
    spaces = [NSString stringWithCString:""];
    for ( i = 0; i <= indentLevel ; i++ )
    {
        spaces = [spaces stringByAppendingString:@"  "];
    }
    
	desc = [NSString stringWithFormat:@"%@(MIObject:%p:WSObject:%p)%@\n%@{\n", 
		spaces, anObject, [anObject WSObject], [anObject title], spaces];
	
//	repsEnum = [[anObject getRepresentations] objectEnumerator];
	childrenEnum = [[anObject getChildren] objectEnumerator];
	
/*	while ( aRep = [repsEnum nextObject] )
	{
		desc = [desc stringByAppendingFormat:@"%@  | +--> 1 rep\n", spaces];
	}
*/	
	indentLevel++;
	while ( aChild = [childrenEnum nextObject] )
	{
		desc = [desc stringByAppendingString:[self printObject:aChild withIndent:indentLevel]];
	}
	indentLevel--;
    
    desc = [desc stringByAppendingFormat:@"%@}\n",spaces];
	
	return desc;
}

- (NSString*)description
{
	NSString		*theDescription;
/*	NSEnumerator	*childrenEnum;
	MIObject		*aChild;

	childrenEnum = [[fTopLevelContainer getChildren] objectEnumerator];
	theDescription = [NSString stringWithFormat:@"MontageDocument:%p\nChildren:\n{\n"];

	while ( aChild = [childrenEnum nextObject] )
	{
		theDescription = [theDescription stringByAppendingString:[self printObject:aChild withIndent:0]];
	}
*/

	theDescription = [NSString stringWithFormat:@"MontageDocument:%p\nChildren:\n{\n"];
    theDescription = [theDescription stringByAppendingString:
		[self printObject:[fModel topLevelContainer] withIndent:0]];

    theDescription = [theDescription stringByAppendingString:@"}"];
    
	return theDescription;
}

- (void)lockRead
{
	[fReadWriteLock lockRead];
}

- (void)unlockRead
{
	[fReadWriteLock unlockRead];
}

- (void)lockWrite
{
	[fReadWriteLock lockWrite];
}

- (void)unlockWrite
{
	[fReadWriteLock unlockWrite];
}

/*
//#ifndef TWO_THREADS_NOTIFICATIONS
- (void)notificationReceived:(NSArray*)packedParams
{
	int code;
	id object;
	void* param;
	
	NSData* dataCode = [packedParams objectAtIndex:0];
	NSData* dataObject = [packedParams objectAtIndex:1];
	NSData* dataParam = [packedParams objectAtIndex:2];
	
	memcpy(&code, [dataCode bytes], sizeof(int));
	memcpy(&object, [dataObject bytes], sizeof(id));
	memcpy(&param, [dataParam bytes], sizeof(void*));
#if 0
} // for enabling the popup button to display all methods 
#endif

//#else // #ifndef TWO_THREADS_NOTIFICATIONS*/
- (void)notificationReceived:(UInt32)code param:(void*)param
{
//#endif // #ifndef TWO_THREADS_NOTIFICATIONS

	verntl(notifications,5,"[MIMontageDocument notificationReceived]");
	switch(code)
	{
		case kNotification_Mixer :
			// rebuild mixer view
			break;
		case kNotification_Mixer_LeftVAOFader :
			break;
		case kNotification_Mixer_RightVAOFader :
			break;
		case kNotification_Mixer_LeftRAOFader :
			break;
		case kNotification_Mixer_RightRAOFader :
			break;
		case kNotification_Mixer_SoloedMonoVAOChanged :
			break;
		case kNotification_Mixer_SoloedStereoVAOChanged :
			break;
		case kNotification_Mixer_SoloedStereoRAOChanged :
			break;

		case kNotification_ModelHasChanged :
			// update whole document;
			break;
		case kNotification_DocAutoStop :
			[[MIApplicationController sharedApplicationController] stopPlaying:nil];
			break;
		case kNotification_AudioClipDataHasChanged :
		{
			MEAudioClip* audioClip = (MEAudioClip*)param;
			// we can't receive a null notification for stand alone
			// audio data... they must be filtered at the source....
			ZzAssert(audioClip);
			MIObject* object = [self findObject:audioClip];
			[object invalidateRepresentationsCachedImages];
			[object invalidateRep];
			[object setRepresentationsNeedDisplay];
		}
			break;
		case kNotification_AdjustMovieArmeTime :
		{
			ZzAssert(0);
			TArmeDoc* armeDoc = [fModel armeDocument];
			AdjustMovieArmeTime(armeDoc->fMasterMovie->GetQTMovie(), armeDoc, armeDoc->GetPlayPos());
		}
			break;
		case kNotification_AudioClipMoved :
			// nothing to do here. The various MEWSObjAudioPlayers do listen
			break;
		case kNotification_VAOChangeToMonoVAO :
			// nothing to do here. MEMixingInfo is listening.
			break;
		case kNotification_VAOChangeToStereoVAO :
			// nothing to do here. MEMixingInfo is listening.
			break;
		default :
			ZzAssert(0);
			break;
	}
}

/*- (BOOL)writeToArchive:(ArmeArchiveHandler*)archiver
{
    MIApplicationController *mainController = [MIApplicationController sharedApplicationController];
    NSEnumerator *enumerator;
    NSWindowController *windowCtl;
    
	[super writeToArchive:archiver];
	archiver->WriteCPPObject(fDocPtr);
//	archiver->WriteObjCObject(fMainViewControllers);
	archiver->WriteObjCObject(fTopLevelContainer);
	archiver->WriteObjCObject(fUndoStack);
	archiver->WriteObjCObject(fRedoStack);
	
	[self writeWindow:[[MIOverviewController sharedOverviewController] window] toArchive:archiver];
	[self writeWindow:[[mainController playStopPauseController] window] toArchive:archiver];
	[self writeWindow:[[mainController toolBoxController] window] toArchive:archiver];
	[self writeWindow:[[mainController timePaletteController] window] toArchive:archiver];
	
	enumerator = [[self windowControllers] objectEnumerator];
	while ( windowCtl = [enumerator nextObject] )
	{
	    [self writeWindow:[windowCtl window] toArchive:archiver];
	}
	return true;
}*/


- (NSMutableArray*)undoStack
{
    return [fModel undoStack];
}

- (NSMutableArray*)redoStack
{
    return [fModel redoStack];
}

- (NSMutableArray*)savedViews
{
    return [fModel savedViews];
}

- (void)addToOuttaHierarchyTransitionsPool:(MIObject*)object
{
	[[fModel outtaHierarchyTransitionsPool] addObject:object];
}

- (void)removeFromOuttaHierarchyTransitionsPool:(MIObject*)object
{
	[[fModel outtaHierarchyTransitionsPool] removeObject:object];
}

- (NSMutableArray*)outtaHierarchyTransitionsPool
{
	return [fModel outtaHierarchyTransitionsPool];
}

- (NSString*)nameOfUndoCommand
{
	id object = [[fModel undoStack] lastObject];
	if ( object )
	{
		return [NSString stringWithFormat:@"Undo %@", [object commandName]];
	}
	else return @"Can't Undo";
}
- (NSString*)nameOfRedoCommand
{
	id object = [[fModel redoStack] lastObject];
	if ( object )
	{
		return [NSString stringWithFormat:@"Redo %@", [object commandName]];
	}
	else return @"Can't Redo";
}

- (void)addCommandToUndoStack:(MICommand*)cmd
{
	int maxDepth, count;
	
	if ( !cmd )
		return;
	
	maxDepth = [[MIPreferences sharedPrefs] undoStackDepth]; 
	count = [[fModel undoStack] count];
	if ( maxDepth > 0 && count >= maxDepth )
	{
		[[fModel undoStack] removeObjectsInRange:NSMakeRange(0, (count-maxDepth)+1)];
	}
	[[fModel undoStack] addObject:cmd];
	[[fModel redoStack] removeAllObjects];
	[self updateDocumentChangeCount:MIChangeDone];
}

- (void)removeCommandFromUndoStack:(MICommand*)cmd
{
	if ( !cmd )
		return;
	// $$$TODO verify that the command is on top of the heap
	
	[[fModel undoStack] removeObject:cmd];
}

- (void)doCmd:(MICommand*)cmd
{
	NSString* nsStrCommandName = [cmd commandName];
	
	// create a progress bar dialog with the 
	MIProgressBarDlgManager progressDlog([nsStrCommandName cString]);
	
	[MICommand setCurrentProgressDialog:&progressDlog];

	[[MIApplicationController sharedApplicationController] deferAllRedraws];
	
	MICmdStatus st = [cmd _doIt];

	[[MIApplicationController sharedApplicationController] flushAllDeferedRedrawRects];
	
	// if command was a macro and it has been cancelled, it might have modified everything. 
	if ([cmd hasModifiedAudioHierarchy])
	{
		[self armeDocument]->GetPlayer()->RecreateTrackMapping();
		[self armeDocument]->Preroll([self armeDocument]->GetPlayPos());
	}
	if ([cmd hasModifiedVideoHierarchy])
	{
		// update the transitions $$TEMP$$ (reduce the time range)
		MIUpdateVideoTransitionsCommand* update = [[MIUpdateVideoTransitionsCommand alloc] initWithDocument:self from:ARME_TIME_UNDEF to:ARME_TIME_UNDEF];
		[update doIt];
		[update release];
	}
	if([cmd causedNeedOfOverviewUpdate] || [cmd hasModifiedVideoHierarchy] || [cmd hasModifiedAudioHierarchy] )
	{
		[[MIOverviewController sharedOverviewController] refreshCachedImage];
#if USE_LIBRARY
		if(gEnableLibraryUpdates)
			[[MILibraryController sharedLibraryController] reloadData];
#endif //USE_LIBRARY
	}

	[MICommand setCurrentProgressDialog:NULL];

	//if the command has been cancelled (or if it is a copy command), don't add it to the undo stack
	if ((st == kMICmdStatusCancelled) || ([cmd isKindOfClass:[MICopyCommand class]]))
	{
		return;
	}
	else
	{	
		[fSnapperDB refreshWholeSnapDB:self];
		
		[self addCommandToUndoStack:cmd];
	}
	
	[[MITimePaletteController sharedTimePaletteController] getValuesFromMainView];
}

- (NSString*)projectFolderPath
{
    return [self fileName];
}

- (NSString*)projectFilePath
{
    return [[self fileName] stringByAppendingPathComponent:MTGArchiveName];
}

- (NSString*)projectFileName
{
    return [[self fileName] lastPathComponent];
}


- (bool)recreateNonArchivedCachedDataWithController:(MIMainViewController*)mainViewController
{
	try
	{
		MIProgressBarDlgManager progressMngr("Opening project",true);
		([fModel armeDocument])->RecreateNonArchivedCachedData(&progressMngr);
		[self deleteUnusedFiles];
	}
	catch (ZzException& e)
	{
		e.DefaultCatcher();
		return false;
	}
	
	return true;
}

- (id)initWithVideoFormat:(int)format
{
    if (( self = [super init] ))
    {
        TArmeWSObj	*ArmeTlc;
		MIObject	*tlcObj;

		fShouldUpdateChangeCount = YES;
		fChangeCount = 0;
		[self setHasUndoManager:NO];
		fModel = [[MIDocumentModel alloc] initWithDocument:self andFormat:format];
		if (!fModel)
		{
			[self release];
			return nil;
		}
		fReadWriteLock = [[MILock alloc] init];
	
		fListener = new MIObjCProxyListener(self);
		([fModel armeDocument])->Subscribe(fListener);
        
        TArmeDoc* doc = [fModel armeDocument];
        doc->LockRead();
        {
            ArmeTlc = doc->GetTopContainer();
        }
        doc->UnlockRead();
		
		fSnapperDB = [[MISnapperDB alloc] init];
		
		tlcObj = [[MIObject alloc] initWithArmeWSObj:ArmeTlc];
        [self setTopLevelContainer:tlcObj];
		[tlcObj release]; // we don't need it here

		[tlcObj setHeight:_MIVPos(1000)];
		ArmeTlc->_SetLength(_ArmeTime(13230000));
		
		fVideoClipRepIncompletelyDrawn = [[NSMutableArray arrayWithCapacity:5] retain];
		fObjectsSetOuttaHierarchyRecently = [[NSMutableArray arrayWithCapacity:5] retain];
		
		[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(playbackQualityHasChanged:) name:MIHighQualityHasChangedNotification object:nil];
		[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(playbackQualityHasChanged:) name:MISingleFieldHasChangedNotification object:nil];

		[self playbackQualityHasChanged:nil];
		fIsInited = YES;
    }

    return self;
}

- (id)initWithContentsOfFile:(NSString*)fileName ofType:(NSString*)docType
{
#ifdef LOG_USER_COMMAND_TO_LOG_FILE
	UserCommandsLogFileName = new char[1024];
//	strcpy(UserCommandsLogFileName, ZzGetLogFileFolder());
//	ZzGetDirectory(UserCommandsLogFileName, 1024, [fileName cString]);
	ZzAppendFileNameToPath(UserCommandsLogFileName, 1024, ZzGetLogFileFolder().getUTF8String(), "HyperEngine-AV.txt");
#endif //LOG_USER_COMMAND_TO_LOG_FILE
	VUSERCMD1("Opening document %s", [fileName cString]);

	vernl(6,"[MIMontageDocument initWithContentsOfFile]");

#ifndef NO_VERBOSE_AT_ALL
	ZzVerbose::SetScreenOutputStream(std::cout);
#endif //#ifndef NO_VERBOSE_AT_ALL
    if (( self = [super init] ))
    {
		NSFileManager *fileManager = [NSFileManager defaultManager];
		if ( ![fileManager isWritableFileAtPath:[fileName stringByAppendingPathComponent:MTGArchiveName]] )
		{
			NSString *mountPoint = [fileName mountPoint];
			BOOL removableFlag;
			BOOL writableFlag;
			BOOL unmountableFlag;
			NSString *description;
			NSString *fileSystemType;
			if ( [[NSWorkspace sharedWorkspace] getFileSystemInfoForPath:mountPoint isRemovable:&removableFlag isWritable:&writableFlag isUnmountable:&unmountableFlag description:&description type:&fileSystemType] ) {
				if ( !writableFlag ) {
					NSRunAlertPanel( @"Unable to open document file.", @"The file \"%@\" is located on a read-only volume.", @"OK", nil, nil, fileName );
					fIsInited = NO;
					[self release];
					return nil;
				}
			}
			NSRunAlertPanel(@"Unable to open document file.", @"The file \"%@\" can't be modified. Please check the file permissions.", @"OK", nil, nil, fileName );
			fIsInited = NO;
			[self release];
			return nil;
		}
		fIsInited = YES;

		fShouldUpdateChangeCount = YES;

		[self setHasUndoManager:NO];
		NSString* projectPath = [fileName stringByAppendingPathComponent:MTGArchiveName];

		fReadWriteLock = [[MILock alloc] init];

		fSnapperDB = [[MISnapperDB alloc] init];

		NSDictionary *attributes = [fileManager fileAttributesAtPath:fileName traverseLink:NO];
			
		// make a copy of Master.mov in WorkingMaster.mov
		verntl(dataFileRef, 4, "Making a copy of Master.mov in WorkingMaster.mov");
		[fileManager removeFileAtPath:[fileName stringByAppendingPathComponent:MTGWorkingMasterName] handler:nil];
		[fileManager copyPath:[fileName stringByAppendingPathComponent:MTGMasterName] toPath:[fileName stringByAppendingPathComponent:MTGWorkingMasterName] handler:nil];

		[fileManager changeFileAttributes:[NSDictionary dictionaryWithObject:[attributes fileModificationDate] forKey:NSFileModificationDate] atPath:fileName];

		if (![self readFromFile:projectPath ofType:docType])
		{
			if ( [fileManager fileExistsAtPath:[fileName stringByAppendingPathComponent:MTGArchiveBackupName]] ) {
				NSString *title = [NSString stringWithFormat:@"Unable to open document \"%@\"", [fileName lastPathComponent]];
				if ( NSAlertDefaultReturn == NSRunAlertPanel( title, @"Read from backup files ?", @"OK", @"Cancel", nil ) ) {
					NSString *src = nil;
					NSString *dst;

					// rename corrupted files
					src = [fileName stringByAppendingPathComponent:MTGArchiveName];
					dst = [fileName stringByAppendingPathComponent:MTGArchiveCorruptedName];
					if ( -1 == rename([src fileSystemRepresentation], [dst fileSystemRepresentation]) ) {
						[self release];
						return nil;
					}
					
					src = [fileName stringByAppendingPathComponent:MTGMasterName];
					dst = [fileName stringByAppendingPathComponent:MTGMasterCorruptedName];
					if ( -1 == rename([src fileSystemRepresentation], [dst fileSystemRepresentation]) ) {
						[self release];
						return nil;
					}
					
					// rename backup files
					src = [fileName stringByAppendingPathComponent:MTGArchiveBackupName];
					dst = [fileName stringByAppendingPathComponent:MTGArchiveName];
					if ( -1 == rename([src fileSystemRepresentation], [dst fileSystemRepresentation]) ) {
						[self release];
						return nil;
					}

					src = [fileName stringByAppendingPathComponent:MTGMasterBackupName];
					dst = [fileName stringByAppendingPathComponent:MTGMasterName];
					if ( -1 == rename([src fileSystemRepresentation], [dst fileSystemRepresentation]) ) {
						[self release];
						return nil;
					}

					// make a copy of Master.mov in WorkingMaster.mov
					verntl(dataFileRef, 4, "Making a copy of Master.mov in WorkingMaster.mov");
					[fileManager removeFileAtPath:[fileName stringByAppendingPathComponent:MTGWorkingMasterName] handler:nil];
					[fileManager copyPath:[fileName stringByAppendingPathComponent:MTGMasterName] toPath:[fileName stringByAppendingPathComponent:MTGWorkingMasterName] handler:nil];

					if (![self readFromFile:projectPath ofType:docType]) {
						NSRunAlertPanel( title, @"Unable to read from backup files.", @"OK", nil, nil );
						[fileManager removeFileAtPath:[fileName stringByAppendingPathComponent:MTGArchiveName] handler:nil];
						[fileManager removeFileAtPath:[fileName stringByAppendingPathComponent:MTGMasterName] handler:nil];

						src = [fileName stringByAppendingPathComponent:MTGArchiveCorruptedName];
						dst = [fileName stringByAppendingPathComponent:MTGArchiveName];
						if ( -1 == rename([src fileSystemRepresentation], [dst fileSystemRepresentation]) ) {
							[self release];
							return nil;
						}
						src = [fileName stringByAppendingPathComponent:MTGMasterCorruptedName];
						dst = [fileName stringByAppendingPathComponent:MTGMasterName];
						if ( -1 == rename([src fileSystemRepresentation], [dst fileSystemRepresentation]) ) {
							[self release];
							return nil;
						}
						
						[fileManager removeFileAtPath:[fileName stringByAppendingPathComponent:MTGArchiveCorruptedName] handler:nil];
						[fileManager removeFileAtPath:[fileName stringByAppendingPathComponent:MTGMasterCorruptedName] handler:nil];

						[self release];
						return nil;
					} else {
						[fileManager removeFileAtPath:[fileName stringByAppendingPathComponent:MTGArchiveCorruptedName] handler:nil];
						[fileManager removeFileAtPath:[fileName stringByAppendingPathComponent:MTGMasterCorruptedName] handler:nil];
					}
					
				} else {
					[self release];
					return nil;
				}
			} else {
				[self release];
				return nil;
			}
		}

		fListener = new MIObjCProxyListener(self);
		([fModel armeDocument])->Subscribe(fListener);
		[self setFileName:fileName];

		fVideoClipRepIncompletelyDrawn = [[NSMutableArray arrayWithCapacity:5] retain];
		fObjectsSetOuttaHierarchyRecently = [[NSMutableArray arrayWithCapacity:5] retain];
		
		[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(playbackQualityHasChanged:) name:MIHighQualityHasChangedNotification object:nil];
		[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(playbackQualityHasChanged:) name:MISingleFieldHasChangedNotification object:nil];

		[self playbackQualityHasChanged:nil];
    }
	
    return self;
}

bool g_MIMontageDocument_dealloc_bodystarted = false; //$$$DEBUG STUFF
bool g_MIMontageDocument_dealloc_bodyfinished = false;


-(void)dealloc
{
	if ( fIsInited )
	{
		g_MIMontageDocument_dealloc_bodystarted = true;
		vernl(6,"[MIMontageDocument dealloc]");
		//NSLog(@"MIMontageDocument - dealloc");
		if ( fListener ) {
			([fModel armeDocument])->Unsubscribe(fListener);
			delete fListener;
		}
		vernl(6,"[MIMontageDocument dealloc] : [fModel retainCount] = " << [fModel retainCount] << " before release");
		[fDocumentSettings release];
		
/*

        OBSOLETE : THIS DID NOT WORK !
        
        WHY : the "close" method of MIMontageDocument is obviously called before "dealloc". However, shortly after that "close" method
        (in the parent version, actually), the document WINDOW controller is released. And this controller is the MIMainViewController
        !!!!!!!!!!!!
    

		// let's remove the selections THIS ABSOLUTELY 
		// need to be done BEFORE the fModel is released 
		// otherwise there would be reps with references 
		// to non-existing objects !!!!!!!
		MIObject* tlc = [fModel topLevelContainer];
		for(UInt32 i = 0; i < [[tlc getRepresentations] count]; i++)
		{
			MITopLevelContainerRep* tlcRep = [[tlc getRepresentations] objectAtIndex:i];
			MIMainViewController* mainViewController = [tlcRep mainViewController];
			[mainViewController clearSelection];
		}
*/								
		[fModel release];
		[fSnapperDB release];
		fSnapperDB = nil;
		
        //$$HACK of the day:
        //it happens that some MIVideoClipRep are dealloced once the doc has already been dealloced (I don't know who else keep a ref on VideoClipRep, maybe the pasteboard
        //or some tracker) and, in the dealloc of the MIVideoClipRep, they must remove themselves from the fVideoClipRepIncompletelyDrawn array (which does not exist
        //anymore at that time) => crash on quit.
        gbVideoClipRepIncompletelyDrawnIsReleased = true;
		[fVideoClipRepIncompletelyDrawn release];
		[fObjectsSetOuttaHierarchyRecently release];
		[fReadWriteLock release]; // must be AFTER all MIObject dealloc because their dealloc uses the fReadWriteLock
	}
#ifdef MTG_DEMO_VERSION
	if ( ![[NSFileManager defaultManager] fileExistsAtPath:[[self fileName] stringByAppendingPathComponent:MTGArchiveName]] ) {
		[[NSFileManager defaultManager] removeFileAtPath:[self fileName] handler:nil];
		[[MIDocumentController sharedDocumentController] clearRecentDocuments:nil];
	}
#endif
	[super dealloc];
	if ( fIsInited )
	{
		g_MIMontageDocument_dealloc_bodyfinished = true;
	}
	
    [[NSNotificationCenter defaultCenter] removeObserver:self name:MIHighQualityHasChangedNotification object:nil];
    [[NSNotificationCenter defaultCenter] removeObserver:self name:MISingleFieldHasChangedNotification object:nil];
	
}

- (IBAction)saveDocument:(id)sender 
{
   [super saveDocument:sender];
}

- (IBAction)saveDocumentAs:(id)sender {
	NSRunAlertPanel(@"NOT IMPLEMENTED", @"Save As is not implemented in this version of HyperEngine-AV.", @"OK", nil, nil);
}

- (IBAction)saveDocumentTo:(id)sender {

        NSSavePanel *savePanel = [NSSavePanel savePanel];
        [savePanel setTitle:@"Save to"];
        [savePanel setPrompt:@"Save"];
        [savePanel beginSheetForDirectory:nil file:[[[self displayName] stringByDeletingPathExtension] stringByAppendingString:@" copy"] modalForWindow:[NSApp mainWindow] modalDelegate:self didEndSelector:@selector(savePanelDidEnd:returnCode:contextInfo:) contextInfo:nil];
}

- (void)savePanelDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void  *)contextInfo {
	NSString *src;
	NSString *dst;
	[sheet orderOut:self];
	
	if ( NSOKButton == returnCode ) 
	{
		MIProgressBarDlgManager *progress = new MIProgressBarDlgManager("Saving Document");

		BOOL isDirectory;
		NSFileManager *fileManager = [NSFileManager defaultManager];
//		NSString *filename = [[((NSSavePanel*)sheet) filename] stringByAppendingPathExtension:@"montage"];
		NSString *filename = [[((NSSavePanel*)sheet) filename] stringByAppendingPathExtension:@"HAV"];
		MITemporaryDirectory* tmp = [MITemporaryDirectory temporaryDirectory];

		if ( [fileManager fileExistsAtPath:filename isDirectory:&isDirectory] )
		{
			if ( NSAlertDefaultReturn == NSRunAlertPanel(@"Save To", @"A file named \"%@\" already exists in %@. Do you want to replace it ?", @"Replace", @"Cancel", nil, [filename lastPathComponent], [filename stringByDeletingLastPathComponent]) )
			{
				[fileManager removeFileAtPath:filename handler:nil];
			}
			else return;
		}

		VUSERCMD1("Save To %s", [filename cString]);

		[self saveDocumentSettings];

//		progress->SetNbSubTasks(7);
//		progress->DisplayDlg();
		
		// backup of the archive file
//		progress->EnterSubTask(); // ENTER 1
		src = [[self fileName] stringByAppendingPathComponent:MTGArchiveName];
		if ( [fileManager fileExistsAtPath:src] )
		{
			dst = [[tmp path] stringByAppendingPathComponent:MTGArchiveBackupName];

			if ( ![fileManager copyPath:src toPath:dst handler:nil] ) return;
		}
//		progress->Update(1);
//		progress->LeaveSubTask();
		
		// backup of the master.mov file
//		progress->EnterSubTask();  // ENTER 2
		src = [[self fileName] stringByAppendingPathComponent:MTGMasterName];
		if ( [fileManager fileExistsAtPath:src] )
		{
			dst = [[tmp path] stringByAppendingPathComponent:MTGMasterBackupName];

			if ( ![fileManager copyPath:src toPath:dst handler:nil] ) 
				return;
		}
		//progress->Update(1);
		//progress->LeaveSubTask();

		//progress->EnterSubTask(); // ENTER 3
		if ( [self isDocumentEdited] ) 
		{
			fShouldUpdateChangeCount = NO;
			[self saveDocument:nil];
			fShouldUpdateChangeCount = YES;
		}
		//progress->Update(1);
		//progress->LeaveSubTask();
		//progress->EnterSubTask(); // ENTER 4
		
		[self lockWrite];

		NSArray *paths = [fileManager directoryContentsAtPath:[self fileName]];
		int itemsToCopy = [paths count];
		int x;
		NSDictionary *attributes = [fileManager fileAttributesAtPath:[self fileName] traverseLink:NO];
		NSMutableDictionary* newDirAttributes = [NSMutableDictionary dictionaryWithCapacity:4];
		
		[newDirAttributes setObject:[attributes objectForKey:NSFileOwnerAccountName] forKey:NSFileOwnerAccountName];
		[newDirAttributes setObject:[attributes objectForKey:NSFileGroupOwnerAccountName] forKey:NSFileGroupOwnerAccountName];
		[newDirAttributes setObject:[attributes objectForKey:NSFilePosixPermissions] forKey:NSFilePosixPermissions];
		[newDirAttributes setObject:[attributes objectForKey:NSFileExtensionHidden] forKey:NSFileExtensionHidden];
		

		if ( ![fileManager createDirectoryAtPath:filename attributes:newDirAttributes] ) 
		{
			NSRunAlertPanel(@"An error has occured", @"Couldn't copy file \"%@\"", @"OK", nil, nil, [[self fileName] lastPathComponent] );
//			progress->Delete();
			return;
		}

//		progress->Update(1);
//		progress->LeaveSubTask();

//		progress->EnterSubTask(); // ENTER 5
		

		progress->SetNbSubTasks(itemsToCopy);

		for( x = 0 ; x < itemsToCopy ; x++ ) 
		{
			NSString* fileName = [paths objectAtIndex:x];
			if ( ![fileName isEqualToString:@".DS_Store"] ) 
			{
				src = [[self fileName] stringByAppendingPathComponent:[paths objectAtIndex:x]];
				dst = [filename stringByAppendingPathComponent:[paths objectAtIndex:x]];
#if 1
				progress->EnterSubTask();
				bool result = ZzCopyMacFile([src fileSystemRepresentation], [dst fileSystemRepresentation], progress);
				progress->LeaveSubTask();
				if (!result) 
				{
					NSRunAlertPanel(@"An error has occured", @"Couldn't copy file \"%@\"", @"OK", nil, nil, [[self fileName] lastPathComponent] );
					progress->Delete();
					progress = NULL;
					return;
				}
			}
#else //#if 1
				/*char tmp[512] = {0};
				sprintf( tmp, "cp -p %s %s", [src fileSystemRepresentation], [dst fileSystemRepresentation] );
				if ( -1 == system( tmp ) ) {*/
				
				NSTask *task = [NSTask launchedTaskWithLaunchPath:@"/bin/cp" arguments:[NSArray arrayWithObjects:@"-p", src, dst, nil]];
				[task waitUntilExit];
				if ( 0 < [task terminationStatus] ) 
				{
					//if ( ![fileManager copyPath:src toPath:dst handler:nil] ) 
					// {
						//if ( !ZzCopyFile( [src fileSystemRepresentation], [dst fileSystemRepresentation] ) ) 
					//	{
					NSRunAlertPanel(@"An error has occured", @"Couldn't copy file \"%@\"", @"OK", nil, nil, [[self fileName] lastPathComponent] );
					progress->Delete();
					return;
					//}
				}
#endif //#if 1
			//progress->Update( ( x + 1 ) / itemsToCopy );
		}
		
		[self unlockWrite];
//		progress->LeaveSubTask();

		// restaure archive file
//		progress->EnterSubTask(); // ENTER 6
		dst = [[self fileName] stringByAppendingPathComponent:MTGArchiveName];
		[fileManager removeFileAtPath:dst handler:nil];
		src = [[tmp path] stringByAppendingPathComponent:MTGArchiveBackupName];
		if ( ![fileManager copyPath:src toPath:dst handler:nil] ) return;
//		progress->Update(1);
//		progress->LeaveSubTask();

		// restaure master file
//		progress->EnterSubTask(); // ENTER 7
		dst = [[self fileName] stringByAppendingPathComponent:MTGMasterName];
		[fileManager removeFileAtPath:dst handler:nil];
		src = [[tmp path] stringByAppendingPathComponent:MTGMasterBackupName];
		if ( ![fileManager copyPath:src toPath:dst handler:nil] ) return;
//		progress->Update(1);
//		progress->LeaveSubTask();

		NSURL *url = [NSURL fileURLWithPath:filename];
		[[NSDocumentController sharedDocumentController] noteNewRecentDocumentURL:url];
		progress->Delete();
	}
}

// do not call this method : Use updateDocumentChangeCount: instead.
- (void)updateChangeCount:(NSDocumentChangeType)changeType 
{
}

- (void)updateDocumentChangeCount:(MIDocumentChangeType)changeType 
{
	if ( fShouldUpdateChangeCount ) {
		switch ( changeType ) {
			case MIChangeDone :
				if ( fChangeCount < 0 ) {
					[super updateChangeCount:NSChangeCleared];
					fChangeCount = [[fModel undoStack] count];
					for ( int x = 0 ; x <= fChangeCount ; x++ ) {
						[super updateChangeCount:NSChangeDone];
					}
				} else {
					fChangeCount++;
					[super updateChangeCount:NSChangeDone];
				}
				break;
			case MIChangeUndone :
				fChangeCount--;
				[super updateChangeCount:NSChangeUndone];
				break;
			case MIChangeCleared :
				fChangeCount = 0;
				[super updateChangeCount:NSChangeCleared];
				break;
			case MIChangeRedone :
				fChangeCount++;
				[super updateChangeCount:NSChangeDone];
				break;
		}
	}
}

- (IBAction)undo:(id)sender
{
	MICommand *cmd = [[fModel undoStack] lastObject];
	if ( !cmd )
		return;

	NSString* nsStrCommandName = [cmd commandName];
	MIProgressBarDlgManager progressDlog([nsStrCommandName cString], true);
	[MICommand setCurrentProgressDialog:&progressDlog];

	VUSERCMD1("Undo %s", [nsStrCommandName cString]);

	if ( [cmd retainCount] > 1 ) NSLog(@"***This command should have a retain count of 1 ; but it has a retain count of %d***", [cmd retainCount] );

	// as the cmd retain count should be 1, we must add it to the redo stack BEFORE removing it from the undo stack
	[[fModel redoStack] addObject:cmd];
	[[fModel undoStack] removeObject:cmd];

	[[MIApplicationController sharedApplicationController] deferAllRedraws];
	[cmd _undoIt];
	[[MIApplicationController sharedApplicationController] flushAllDeferedRedrawRects];

	//update hierarchy if needed
	if ([cmd hasModifiedAudioHierarchy])
	{
		[self armeDocument]->GetPlayer()->RecreateTrackMapping();
	}
	if ([cmd hasModifiedVideoHierarchy])
	{
		// update the transitions $$TEMP$$ (reduce the time range)
		MIUpdateVideoTransitionsCommand* update = [[MIUpdateVideoTransitionsCommand alloc] initWithDocument:self from:ARME_TIME_UNDEF to:ARME_TIME_UNDEF];
		[update doIt];
		[update release];
	}
	if ( [cmd causedNeedOfOverviewUpdate] || [cmd hasModifiedVideoHierarchy] || [cmd hasModifiedAudioHierarchy] )
	{
		[[MIOverviewController sharedOverviewController] refreshCachedImage];
#if USE_LIBRARY
		if(gEnableLibraryUpdates)
			[[MILibraryController sharedLibraryController] reloadData];
#endif //USE_LIBRARY
	}
	[fSnapperDB refreshWholeSnapDB:self];
	[self updateDocumentChangeCount:MIChangeUndone];
	[MICommand setCurrentProgressDialog:NULL];
	
	[[MITimePaletteController sharedTimePaletteController] getValuesFromMainView];

}

- (IBAction)redo:(id)sender
{
	MICommand *cmd = [[fModel redoStack] lastObject];
	if ( !cmd )
		return;

	NSString* nsStrCommandName = [cmd commandName];
	MIProgressBarDlgManager progressDlog([nsStrCommandName cString], true);
	[MICommand setCurrentProgressDialog:&progressDlog];

	VUSERCMD1("Redo %s", [nsStrCommandName cString]);

	if ( [cmd retainCount] > 1 ) NSLog(@"***This command should have a retain count of 1 ; but it has a retain count of %d***", [cmd retainCount] );

	// as the cmd retain count should be 1, we must add it to the undo stack BEFORE removing it from the redo stack
	[[fModel undoStack] addObject:cmd];
	[[fModel redoStack] removeObject:cmd];

	[[MIApplicationController sharedApplicationController] deferAllRedraws];
	
	[cmd _redoIt];

	[[MIApplicationController sharedApplicationController] flushAllDeferedRedrawRects];

	//update hierarchy if needed
	if ([cmd hasModifiedAudioHierarchy])
	{
		[self armeDocument]->GetPlayer()->RecreateTrackMapping();
	}
	if ([cmd hasModifiedVideoHierarchy])
	{
		// update the transitions $$TEMP$$  (reduce the time range)
		MIUpdateVideoTransitionsCommand* update = [[MIUpdateVideoTransitionsCommand alloc] initWithDocument:self from:ARME_TIME_UNDEF to:ARME_TIME_UNDEF];
		[update doIt];
		[update release];
	}
	if ( [cmd causedNeedOfOverviewUpdate] || [cmd hasModifiedVideoHierarchy] || [cmd hasModifiedAudioHierarchy] )
	{
		[[MIOverviewController sharedOverviewController] refreshCachedImage];
#if USE_LIBRARY
		if(gEnableLibraryUpdates)
			[[MILibraryController sharedLibraryController] reloadData];
#endif //USE_LIBRARY
	}
	[fSnapperDB refreshWholeSnapDB:self];
	[self updateDocumentChangeCount:MIChangeRedone];
	[MICommand setCurrentProgressDialog:NULL];
	
	[[MITimePaletteController sharedTimePaletteController] getValuesFromMainView];
}


- (BOOL)readFromFile:(NSString*)fileName ofType:(NSString*)docType
{
    BOOL readOK = YES;
    MontageArchiveHandler unarchiver;
	ZzGetDirectory(unarchiver.fProjectDirectory, kMaxPathLength, [fileName fileSystemRepresentation]);
    fModel = [unarchiver.ReadArchiveWithObjCHead([fileName fileSystemRepresentation], self) retain];
    if ( fModel )
    {
		if([fModel didUnarchivingDirtifyDoc])
			[self updateDocumentChangeCount:MIChangeDone];
		[fModel setDocument:self];
		fChangeCount = 0;
    }
    else readOK = NO;
	
	// refresh the snapping guides.
	[fSnapperDB refreshWholeSnapDB:self];

    return readOK;
}

- (BOOL)revertToSavedFromFile:(NSString *)fileName ofType:(NSString *)type {
#ifdef MTG_DEMO_VERSION
	NSRunAlertPanel(@"Demo Version", @"Revert is not enabled in this version of HyperEngine-AV.", @"OK", nil, nil);
	return YES;
#else //MTG_DEMO_VERSION
	NSDocumentController *documentController = [NSDocumentController sharedDocumentController];
	MIStartingPoint *startingPoint = [MIStartingPoint startingPoint];
	
	[startingPoint setShouldDisplayStartingPoint:NO];
	
	[self close];

	MIMontageDocument *document = [documentController makeDocumentWithContentsOfFile:fileName ofType:@"HyperEngine-AV document"];
	[documentController addDocument:document];
	[document makeWindowControllers];

	[startingPoint setShouldDisplayStartingPoint:YES];
	
	return YES;
#endif //MTG_DEMO_VERSION
}

- (BOOL)validateMenuItem:(NSMenuItem *)anItem {

	SEL action = [anItem action];

	if ( action == @selector(saveDocument:) ) {
		if ( [[NSApp mainWindow] windowController] != [NSApp mainViewController] ) return NO;
		if ( [self isDocumentEdited] ) return YES;
		else return NO;
	}
	
	if ( action == @selector(saveDocumentTo:) ) {
		if ( [[NSApp mainWindow] windowController] != [NSApp mainViewController] ) return NO;
		else return YES;
	}
	
	if ( action == @selector(revertDocumentToSaved:) ) {
		if ( [[NSApp mainWindow] windowController] != [NSApp mainViewController] ) return NO;
		return [self isDocumentEdited];
	}
	
	if ( action == @selector(undo:) ) {
		[anItem setTitle:[self nameOfUndoCommand]];
		if ( [[self undoStack] count] > 0 ) {
			return YES;
		} else return NO;
	}

	if ( action == @selector(redo:) ) {
		[anItem setTitle:[self nameOfRedoCommand]];
		if ( [[self redoStack] count] > 0 ) {
			return YES;
		} else return NO;
	}

	if ( action == NSSelectorFromString(@"pageSetup:") ) {
		return NO;
	}

	if ( action == NSSelectorFromString(@"print:") ) {
		return NO;
	}
	
	
	return [super validateMenuItem:anItem];
}

- (IBAction)pageSetup:(id)sender {
}

- (IBAction)print:(id)sender {
}

- (void)deleteUnusedFiles
{
	//First, remove all references to unused files in the WorkingMaster.mov
	[fModel armeDocument]->RemoveReferencesOnUnusedFilesInMasterMovie();
	
	//then, copy the WorkingMaster.mov in Master.mov
	if ([self copyWorkingMasterInMaster:[NSString stringWithCString:[fModel armeDocument]->GetProjectFolderPath()]])
	{
		//then, remove the files
		[fModel armeDocument]->DeleteUnusedFiles();
	}
}

- (bool)copyWorkingMasterInMaster:(NSString*)documentPath
{
	NSFileManager *fileManager = [NSFileManager defaultManager];
	NSString *src = nil;
	NSString *dst;

	//make a backup of Master.mov in Master~.mov (by changing it's name)
	dst = [documentPath stringByAppendingPathComponent:MTGMasterBackupName];
	[fileManager removeFileAtPath:dst handler:nil];

	src = [documentPath stringByAppendingPathComponent:MTGMasterName];
	rename([src fileSystemRepresentation], [dst fileSystemRepresentation]);
	
	// make a copy of WorkingMaster.mov in Master.mov
	verntl(dataFileRef, 4, "Making a copy of WorkingMaster.mov in Master.mov");
	src = [documentPath stringByAppendingPathComponent:MTGWorkingMasterName];
	dst = [documentPath stringByAppendingPathComponent:MTGMasterName];

	if ( ![fileManager copyPath:src toPath:dst handler:nil] )
	{
		// restore backup files
		dst = [documentPath stringByAppendingPathComponent:MTGArchiveName];
		src = [documentPath stringByAppendingPathComponent:MTGArchiveBackupName];

		[fileManager removeFileAtPath:dst handler:nil];
		rename([src fileSystemRepresentation], [dst fileSystemRepresentation]);

		dst = [documentPath stringByAppendingPathComponent:MTGMasterName];
		src = [documentPath stringByAppendingPathComponent:MTGMasterBackupName];

		[fileManager removeFileAtPath:dst handler:nil];
		rename([src fileSystemRepresentation], [dst fileSystemRepresentation]);
		return NO;
	}
	
	return YES;
}

- (BOOL)writeWithBackupToFile:(NSString*)documentPath ofType:(NSString*)docType saveOperation:(NSSaveOperationType)saveOperationType
{
/*
The purpose of this method is to backup the archive file so that if archiving fails,
it is able to restore the original archive file. The media files are left untouched.
The WorkingMaster.mov is duplicated as Master.mov (and Master.mov is the one that is
read at document opening time).
*/
		NSFileManager *fileManager = [NSFileManager defaultManager];
		NSString *src = nil;
		NSString *dst;

		// backup of the archive file (by changing it's name)
		src = [documentPath stringByAppendingPathComponent:MTGArchiveName];
		if ( [fileManager fileExistsAtPath:src] )
		{
			// delete previous backup file
			dst = [documentPath stringByAppendingPathComponent:MTGArchiveBackupName];
			[fileManager removeFileAtPath:dst handler:nil];
			
			dst = [documentPath stringByAppendingPathComponent:MTGArchiveBackupName];
			
			if ( -1 == rename([src fileSystemRepresentation], [dst fileSystemRepresentation]) )
			{
				return NO;
			}
		}
		
		// write the new archive file
		dst = [documentPath stringByAppendingPathComponent:MTGArchiveName];
		if ( ![self writeToFile:dst ofType:docType originalFile:src saveOperation:saveOperationType] )
		{
			// restore backup file
			dst = [documentPath stringByAppendingPathComponent:MTGArchiveName];
			src = [documentPath stringByAppendingPathComponent:MTGArchiveBackupName];
			
			[fileManager removeFileAtPath:dst handler:nil];
			rename([src fileSystemRepresentation], [dst fileSystemRepresentation]);
			return NO;
		}

		[self copyWorkingMasterInMaster:[NSString stringWithCString:[fModel armeDocument]->GetProjectFolderPath()]];
		
		// delete backup files if not needed anymore
		if ( ![self keepBackupFile] )
		{
			dst = [documentPath stringByAppendingPathComponent:MTGArchiveBackupName];
			[fileManager removeFileAtPath:dst handler:nil];

			dst = [documentPath stringByAppendingPathComponent:MTGMasterBackupName];
			[fileManager removeFileAtPath:dst handler:nil];
		}

		return YES;
}

- (NSDictionary *)fileAttributesToWriteToFile:(NSString *)fullDocumentPath ofType:(NSString *)documentTypeName saveOperation:(NSSaveOperationType)saveOperationType
{
    NSDictionary *dict = [super fileAttributesToWriteToFile:fullDocumentPath ofType:documentTypeName saveOperation:saveOperationType];
    NSMutableDictionary *mdict = [NSMutableDictionary dictionaryWithDictionary:dict];
    [mdict setObject:@"'MtgA'" forKey:NSFileHFSCreatorCode];
    [mdict setObject:@"'MtgD'" forKey:NSFileHFSTypeCode];
    return [NSDictionary dictionaryWithDictionary:mdict] ;
}

- (void)setFileName:(NSString*)fileName
{
    [super setFileName:fileName];
    TArmeDoc *armeDoc = [self armeDocument];
	if ( armeDoc ) armeDoc->SetProjectFolderPath([fileName fileSystemRepresentation]);
}


- (BOOL)writeToFile:(NSString*)fileName ofType:(NSString*)docType
{
        VUSERCMD("Saved");
    
        MontageArchiveHandler archiver;
        ArmeArchiveHandler::Error error = archiver.WriteArchiveWithObjCHead([fileName fileSystemRepresentation], fModel, self);
        if ( error != ArmeArchiveHandler::NoErr ) return NO;
    
        [self deleteUnusedFiles];
        [self updateDocumentChangeCount:MIChangeCleared];
        return YES;
}

- (TArmeDoc*)armeDocument
{
    return [fModel armeDocument];
}

- (void)makeWindowControllers
{
	id zoomH = [[self documentSettingForKey:@"hZoomIndex"] retain];
	id zoomV = [[self documentSettingForKey:@"vZoomIndex"] retain];
	id scroll = [[self documentSettingForKey:@"scrollPoint"] retain];
	id frame = [[self documentSettingForKey:@"windowFrame"] retain];
	
    MIMainViewController *mainViewController = [[MIMainViewController alloc] init];
    [self addWindowController:mainViewController];
	
	if ( zoomH ) {
		mainViewController->fHZoomIndex = [zoomH unsignedIntValue];
		[zoomH release];
	} else {
		mainViewController->fHZoomIndex = [MIMainViewController getDefaultHZoomIndex];
	}

	if ( zoomV ) {
		mainViewController->fVZoomIndex = [zoomV unsignedIntValue];
		[zoomV release];
	} else {
		mainViewController->fVZoomIndex = [MIMainViewController getDefaultVZoomIndex];
	}
	
    [mainViewController setShouldCloseDocument:YES];

	if ( frame ) {
		[[mainViewController window] setFrame:NSRectFromString(frame) display:NO];
		[frame release];
	}		

	[self createTopContainerObjectWithWindowController:mainViewController];

	if ( scroll ) {
		[[mainViewController topLevelContainerView] scrollPoint:NSPointFromString(scroll)];
		[scroll release];
	} else {
		NSPoint p = NSZeroPoint;
		p.x = 0;
		p.y = NSMidY([[mainViewController topLevelContainerView] bounds]);
		p.y -= NSHeight([[mainViewController topLevelContainerView] visibleRect])/2;

		[[mainViewController topLevelContainerView] scrollPoint:p];
	}

	id flag = [self documentSettingForKey:@"FollowPlaybar"];
	if ( flag ) {
		[[MIApplicationController sharedApplicationController] setShouldFollowPlaybar:[flag boolValue]];
	} else [[MIApplicationController sharedApplicationController] setShouldFollowPlaybar:YES];
	
    [[mainViewController window] makeKeyAndOrderFront:self];
	[mainViewController release];
}

- (void)createTopContainerObjectWithWindowController:(MIMainViewController*)mainViewController
{
    TArmeDoc* doc = [self armeDocument];
    TArmeWSObj *ArmeTlc;
    MIObject *tlc;
    MITopLevelContainerRep *tlcView;
    ArmeTime theTime;
//    ArmeTime TLCLength;
    
    doc->LockRead();
    {
        ArmeTlc = doc->GetTopContainer();
    }
    doc->UnlockRead();
    
    tlc = [self topLevelContainer];

    tlcView = [[MITopLevelContainerRep alloc] initWithFrame:NSMakeRect(0, 0, 10, 10)];
    [tlcView setMainViewController:mainViewController];
    
    [tlc addRepresentation:tlcView];

    [mainViewController setTopLevelContainerView:tlcView];
    [[mainViewController ruler] setTopLevelContainerView:tlcView];
    [[mainViewController ruler] setNeedsDisplay:YES];
/*
    TLCLength = [tlcView pixelToArmeTime:3000];
    [self lockWrite];
    [tlc setHeight:[tlcView pixelToMIVPos:1000]];
    [self unlockWrite];
    ArmeTlc->_SetLength(TLCLength);
*/
	[fSnapperDB refreshSnapDB:tlc];

    // $$$BENJA
    //	theDoc->LockWrite();
    //		theDoc->SetRLocator(TLCLength);
    //	theDoc->UnlockWrite();

    // this one is to make sure the model and the UI are consistent
    [[mainViewController topLevelContainerView] recreateFrame];
    [[mainViewController topLevelContainerView] setNeedsDisplay:YES];
	
	ArmeTime oldPlayPos = doc->GetPlayPos();
	ArmeTime oldLL = doc->GetLLocator();
	ArmeTime oldRL = doc->GetRLocator();
	
	if(oldLL == _ArmeTime(-1)) // means not an init from archive but a new doc
	{
		theTime = _ArmeTime(0);
		doc->Preroll(theTime);
		doc->Seek(theTime);
		theTime = [mainViewController pixelToArmeTime:NSMinX([[mainViewController topLevelContainerView] visibleRect])+5];
		doc->SetLLocator(theTime);
		theTime = [mainViewController pixelToArmeTime:NSMaxX([[mainViewController topLevelContainerView] visibleRect])-5];
		doc->SetRLocator(theTime);
	}
	else
	{
		id pos = [self documentSettingForKey:@"playpos"];
		if ( pos ) {
			doc->Seek(_ArmeTime(SInt64([pos doubleValue])));
		}
	}
	
    if (![self recreateNonArchivedCachedDataWithController:mainViewController]) // the controller is passed
	{
		//TODO : Cancel the loading of the file and go back to starting point (?)
	
	}
	
	// because split tasks need a window to root their progress bar sheets.
    [tlcView createChildrenRepresentationsRecursivelyInSelf];
	
	[[MIOverviewController sharedOverviewController] refreshCachedImage];
	[[fModel topLevelContainer] setRepresentationsNeedDisplay];
    [mainViewController updatePlayBar];
    [mainViewController updateLocators];

    [tlcView release];
}

- (void)refreshSnapDB:(MIObject*)obj
{
	[fSnapperDB refreshSnapDB:obj];
}

- (void)deleteFromSnapDB:(MIObject*)obj
{
	[fSnapperDB refreshSnapDB:obj];
}

- (MISnapperDB*)snapperDB
{
	return fSnapperDB;
}

- (UInt32)addMarkerAtTime:(ArmeTime)marker
{
	return [fModel addMarkerAtTime:marker];
}

- (void)removeMarker:(UInt32)marker
{
	[fModel removeMarker:marker];
}

- (void)moveMarker:(UInt32)marker toTime:(ArmeTime)newTime
{
	[fModel moveMarker:marker toTime:newTime];
}

- (UInt32)getMarkerCount
{
	return [fModel getMarkerCount];
}

- (ArmeTime)getMarker:(UInt32)i
{
	return [fModel getMarker:i];
}

- (UInt32)getMarkerAtTime:(ArmeTime)time
{
	ArmeTime markerTime;
	UInt32 i;
	
	for ( i = 0 ; i < [fModel getMarkerCount] ; i++ )
	{
		markerTime = [fModel getMarker:i];
		if ( markerTime == time ) return i;
	}
	return (UInt32)-1;
}

- (UInt32)getClosestMarkerForTime:(ArmeTime)time
{
	ArmeTime markerTime;
	ArmeTime absoluteTime;
	ArmeTime bestAbsoluteTime = ARME_TIME_MAX;
	UInt32 bestMarker = (UInt32)-1;
	UInt32 i;

	for ( i = 0 ; i < [fModel getMarkerCount] ; i++ )
	{
		markerTime = [fModel getMarker:i];
		absoluteTime = [self ArmeTimeWidth:markerTime withArmeTime:time];
		if ( absoluteTime < bestAbsoluteTime )
		{
			bestAbsoluteTime = absoluteTime;
			bestMarker = i;
		}
	}
	return bestMarker;	
}

- (ArmeTime)ArmeTimeWidth:(ArmeTime)a withArmeTime:(ArmeTime)b
{
	ArmeTime t1 = a-b;
	ArmeTime t2 = b-a;
	return ( t1 > t2 ? t1 : t2 ) ;
}

// Document settings should be used to save data that don't belong to the document model
// (ie : window size, viewer location...)
- (void)setDocumentSetting:(id)setting forKey:(NSString*)key {
	if ( !fDocumentSettings ) {
		fDocumentSettings = [[NSMutableDictionary dictionaryWithContentsOfFile:[[self fileName] stringByAppendingPathComponent:MTGDocumentSettingsName]] retain];
		if ( !fDocumentSettings ) {
			fDocumentSettings = [[NSMutableDictionary alloc] init];
		}
	}
	
	[fDocumentSettings setObject:setting forKey:key];
}

- (id)documentSettingForKey:(NSString*)key {
	if ( !fDocumentSettings ) {
		fDocumentSettings = [[NSMutableDictionary dictionaryWithContentsOfFile:[[self fileName] stringByAppendingPathComponent:MTGDocumentSettingsName]] retain];
		if ( !fDocumentSettings ) {
			fDocumentSettings = [[NSMutableDictionary alloc] init];
		}
	}
	return [fDocumentSettings objectForKey:key];
}

- (void)saveDocumentSettings {
	TArmeDoc				*aTArmeDoc;
    ArmeTime				newArmeTimePos;
	
    aTArmeDoc = [self armeDocument];

    aTArmeDoc->LockRead();
    newArmeTimePos = aTArmeDoc->GetPlayPos();
    aTArmeDoc->UnlockRead();

	[self setDocumentSetting:[NSNumber numberWithDouble:newArmeTimePos.data] forKey:@"playpos"];

	[self setDocumentSetting:[NSNumber numberWithBool:[[MIApplicationController sharedApplicationController] shouldFollowPlaybar]] forKey:@"FollowPlaybar"];
	
	[fDocumentSettings writeToFile:[[self fileName] stringByAppendingPathComponent:MTGDocumentSettingsName] atomically:NO];
}

- (void)restartGenerationOfThumbnails
{
	NSEnumerator* repEnum = [fVideoClipRepIncompletelyDrawn objectEnumerator];
	MIObjectRep* rep;
	while (rep = [repEnum nextObject])
		[rep setNeedsDisplay];
}

/*- (long long)freeDiskSpace {
	NSString *mountPoint = [[self fileName] mountPoint];
	struct statfs buffer;
	int ret;

	ret = statfs( [mountPoint fileSystemRepresentation], &buffer );

	if ( ret == 0 ) {
		// return the free disk space (in bytes)
		return ((long long)buffer.f_bsize) * ((long long)buffer.f_bfree);
	}
	return 0;
}*/

- (BOOL)hasContent {
	if ( [self armeDocument]->GetTopContainer()->GetChildrenCount() > 0 ) 
        return YES;
	else
        return NO;
}

- (void)playbackQualityHasChanged:(NSNotification*)notification {
	bool isHighQuality = [[MIPreferences sharedPrefs] isHighQuality];
	bool isSingleField = [[MIPreferences sharedPrefs] isSingleField];

	[self armeDocument]->fMasterMovie->SetPlaybackQuality(isHighQuality, isSingleField);
}
@end


