/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import <AppKit/AppKit.h>
#import "MIPaletteController.h"

@interface MIMontagePalette : NSWindowController {
	NSMenuItem *fMenuItem;
	BOOL fWindowJustLoaded;
}

// should be overriden : returns a unique string that is used to save
// if the palette is visible in the user defaults
+ (NSString*)visibleDefaultName;

// should be overriden if the palette should be visible the first time HyperEngine-AV
// is launch and return YES
+ (BOOL)factoryVisibleDefault;

// could be overriden if a palette wants to save it's own user defaults.
// it should then call super's implementation
+ (void)addDefaultValuesToDictionary:(NSMutableDictionary*)defaultValues;

// This is the designated initializer that should be overidden.
- (id)init;

- (void)dealloc;

// subclasses should NOT use this method. They should use - (void)windowWasLoaded instead.
- (void)windowDidLoad;

// subclasses that need to perform window initialization after if has been loaded should subclass this method. There is no need to call super's implementation.
- (void)windowWasLoaded;

// should be overriden to return the subclass's nib file name
- (NSString*)nibName;

// if overidden should call super's implementation
- (void)applicationWillTerminate:(NSNotification*)theNotification;

// should be overriden. returns the menu tag of the menu that shows/hides the palette
- (int)menuTag;

// should be overriden. returns the name of the palette to be used as the menu item's title.
- (NSString*)menuName;

- (void)registerSelf;

- (IBAction)show:(id)sender;
- (IBAction)hide:(id)sender;
- (IBAction)toggleShowHide:(id)sender;
- (BOOL)isVisible;

- (void)saveUserDefaults:(BOOL)shouldBeVisible;
@end
