/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MIObject.h"
#import "MIObjectRep.h"
#import "MIClipRep.h"
#import "MIAudioClipRep.h"
#import "MIVideoClipRep.h"
#import "MITextClipRep.h"
#import "MIContainerRep.h"
#import "MIProcessorRep.h"
#import "MIVideoTransitionRep.h"
#import "MITopLevelContainerRep.h"
#import "MIMainViewController.h"
#import "MIApplicationController.h"
#import "MIVideoEffectRep.h"
#import "MIMontageDocument.h"
#import "TArmeDoc.h"
#import "MIImageController.h"
#import "MISnapperDB.h"
#import "MIPreferences.h"
#import "VARPIPluginClass.h"

#ifndef MEAudioClip_h
#include "MEAudioClip.h"
#endif //MEAudioClip_h

#ifndef TArmeDoc_h
#include "TArmeDoc.h"
#endif //TArmeDoc_h

#ifndef MIProgressBarDlgManager_h
#include "MIProgressBarDlgManager.h"
#endif //MIProgressBarDlgManager_h

@implementation MIObject

- (id)initFromArchive:(ArmeArchiveHandler*)unarchiver
{
	self = [super initFromArchive:unarchiver];
	if(self)
	{
		ArmeArchivable* archObj;
//		fRepresentations = unarchiver->ReadObjCObject();
		fParentObject = unarchiver->ReadObjCObject();
		fOldParentObject = unarchiver->ReadObjCObject();
		fFreezeSource = [unarchiver->ReadObjCObject() retain];
		fFrozenObject = [unarchiver->ReadObjCObject() retain];
		fContentHasBeenModifiedSinceLastFreezeOperation = unarchiver->ReadBOOL();	
		fChildren = [unarchiver->ReadObjCObject() retain];
		fIsOpaque = unarchiver->ReadBOOL();
		archObj = unarchiver->ReadCPPObject();
		fWSObject = dynamic_cast<TArmeWSObj*>(archObj);
		fWSObject->AddMIObjectRef();
//		vernl(0,"(initFromArchive) MIObject (" << (void*)self << ") created with fWSObject (" << (void*)fWSObject);
		ZzAssert(fWSObject);
		//if((SInt32)(GET_VAL(fWSObject->GetStartTime())) == 793800) Debugger();
		fMontageDoc = unarchiver->ReadObjCObject();
		[self setVerticalPos:_MIVPos(unarchiver->ReadSInt32())];
		GET_VALUE(fHeight) = unarchiver->ReadSInt32();
    if ( unarchiver->GetFileVer() >= 0x00010034 )
    {
      GET_VALUE(fOldHeight) = unarchiver->ReadSInt32();
      fIsCollapsed = unarchiver->ReadBOOL();
    }
    else
    {
      fIsCollapsed = FALSE;
    }
//		fIsTurnedOn = unarchiver->ReadBOOL();
		fBackgroundColor = unarchiver->ReadObjCObject();
		[fBackgroundColor retain];
		fClipIsLocked = unarchiver->ReadBOOL();
		if ( fClipIsLocked )
			fLockedToTime = unarchiver->ReadArmeTime();

		[[fMontageDoc snapperDB] refreshSnapDB:self];
	}
	return self;
}

- (BOOL)writeToArchive:(ArmeArchiveHandler*)archiver
{
//	archiver->WriteObjCObject(fRepresentations);
	archiver->WriteObjCObject(fParentObject);
	archiver->WriteObjCObject(fOldParentObject);
	archiver->WriteObjCObject(fFreezeSource);
	archiver->WriteObjCObject(fFrozenObject);
	archiver->WriteBOOL(fContentHasBeenModifiedSinceLastFreezeOperation);
	archiver->WriteObjCObject(fChildren);
	archiver->WriteBOOL(fIsOpaque);
	archiver->WriteCPPObject(fWSObject);
	archiver->WriteObjCObject(fMontageDoc);
	archiver->WriteSInt32(GET_VALUE([self getVerticalPos]));
	archiver->WriteSInt32(GET_VALUE(fHeight));
  archiver->WriteSInt32(GET_VALUE(fOldHeight));
  archiver->WriteBOOL(fIsCollapsed);
//	archiver->WriteBOOL(fIsTurnedOn);
	archiver->WriteObjCObject( fBackgroundColor );
	archiver->WriteBOOL(fClipIsLocked);
	if ( fClipIsLocked )
		archiver->WriteArmeTime(fLockedToTime);
	return YES;
}

- (id)init
{
	self = [super init];
	if ( self )
	{
		fRepresentations = [[NSMutableArray alloc] initWithObjects:nil];
		fChildren = [[NSMutableArray alloc] initWithObjects:nil];
		fIsOpaque = TRUE;
		fParentObject = nil;
		fOldParentObject = nil;
		fFreezeSource = nil;
		fFrozenObject = nil;
		fContentHasBeenModifiedSinceLastFreezeOperation = FALSE;
		fIsBeingDealloced = FALSE;
		[self setBackgroundColor:[NSColor whiteColor]];
		
		[self setVerticalPos:_MIVPos(0)];
#ifdef SMALL_TITLEBAR
		fHeight = kDefaultGroupRepresentationHeight;
#else
		GET_VALUE(fHeight) = kDefaultGroupRepresentationHeight;
#endif
	}	
//	vernl(0,"(init) MIObject (" << (void*)self << ") created without fWSObject");
	return self;
}

-(id)retain
{
#ifdef DEBUG
//	printf("about to [retain] MIObject(%p,%s). Ref count will go from %d to %d\n",(void*)self,[self WSObject]->GetName(),[self retainCount],[self retainCount]+1);
#endif //#ifdef DEBUG
	return [super retain];
}		
	
-(void)release
{
#ifdef DEBUG
//	printf("about to [release] MIObject(%p,%s). Ref count will go from %d to %d\n",(void*)self,[self WSObject]->GetName(),[self retainCount],[self retainCount]-1);
#endif //#ifdef DEBUG
	[super release];
}

-(id)autorelease
{
//	SInt32 startTime;
//	startTime = (SInt32)(GET_VAL(fWSObject->GetStartTime()));
//	if(startTime == 793800)
//	{
//		vernl(0,"autoreleasing MIObject (" << (void*)self << ") with fWSObject = " << (void*)fWSObject << " fWSObject->GetName() = " << fWSObject->GetName() << " current retain count is " << [self retainCount] );
//		//Debugger();
//	}
	return [super autorelease];
}
	
- (void)dealloc
{
//#ifdef DEBUG
	
//	bool isDeallocatingTLC = false;
//	if([self WSObject] == [self WSObject]->GetDoc()->GetTopContainer())
//	{
//		printf("******** ABOUT TO DEALLOC THE TLC MIOBJECT ******* \n");
//		isDeallocatingTLC = true;
//	}

//	printf("MIObject (%p,%s) [dealloc]\n",(void*)self,[self WSObject]->GetName());
//#endif //#ifdef DEBUG
	fIsBeingDealloced = TRUE;
	[fBackgroundColor release];
	if (fFrozenObject && !fFrozenObject->fIsBeingDealloced) //HACK : to avoid infinite calls from fFrozenObject to fFreezeSource to fFrozenObject ...
		[fFrozenObject release];
	if (fFreezeSource && !fFreezeSource->fIsBeingDealloced) //HACK : to avoid infinite calls from fFrozenObject to fFreezeSource to fFrozenObject ...
		[fFreezeSource release];
	if([fMontageDoc snapperDB])
		[[fMontageDoc snapperDB] deleteFromSnapDB:self];
	
	// remove representations
//	NSEnumerator* 	repEnum = [fRepresentations objectEnumerator];
#ifdef DEBUG
	MIObjectRep*	aRep;
	printf("about to release reps from MIObject %p :\n",(void*)self);
	if([fRepresentations count] == 0)
		printf("---- no reps to remove\n");

	for(UInt32 i = 0; i < [fRepresentations count]; i++)
//	while ( aRep = [repEnum nextObject] )
	{
		aRep = [fRepresentations objectAtIndex:i];
		// removes the objects and the guides
		printf("    rep : %p\n",(void*)aRep);
//		[fMontageDoc lockWrite];
//			[fRepresentations removeObject:aRep];
///		[fMontageDoc unlockWrite];
		// we do NOT need to close and delete the blue windows because
		// this is handled in the rep dealloc call
		//[aRep release]; it's already released in [fRepresentations removeObject:aRep];
	}
#endif //#ifdef DEBUG

	[fMontageDoc lockWrite];
	[fRepresentations removeAllObjects];
	[fMontageDoc unlockWrite];
	

	#ifdef DEBUG
//	printf("MIObject is about to remove those objects from fChildren in dealloc : \n");
//	if([fChildren count] == 0)
//		printf("---- no objects to remove\n");
//	for(UInt32 debug_i = 0; debug_i < [fChildren count]; debug_i++)
//	{
//		printf("    %d : %p\n",debug_i, (void*)[fChildren objectAtIndex:debug_i]);
//	}
	#endif //#ifdef DEBUG
	
	[fChildren release];
	
	// if the fWSObject is a transition, other objects might still have references on it
/*	if ((fWSObject->GetType() == TArmeWSObj::kVideoXFadeEffect) && (fWSObject->GetDoc()->fOuttaHierarchyTransitionsPool->FindIndexForItem((TVideoXFadeEffect*)fWSObject) != -1))
		return;
	else*/
	fWSObject->ReleaseMIObjectRef();
	//delete fWSObject;  // deletion of the Arme hieararchy is taken care of by ~TArmeDoc

//	if(isDeallocatingTLC)
//		printf("******** END OF TLC MIOBJECT DEALLOC !!!!!!!******* \n");
    [super dealloc];
}

- (BOOL)deleteSelfAndChildren
{
	NSEnumerator	*childEnum;
	MIObject		*aChild;
	NSEnumerator	*repEnum;
	MIObjectRep		*aRep;
	
	childEnum = [fChildren objectEnumerator];
	while ( aChild = [childEnum nextObject] )
	{
		[aChild deleteSelfAndChildren];
	}
	
	// actual deletion code
	// remove representations
	repEnum = [fRepresentations objectEnumerator];
	while ( aRep = [repEnum nextObject] )
	{
		// removes the objects and the guides
		[fMontageDoc lockWrite];
			[fRepresentations removeObject:aRep];
		[fMontageDoc unlockWrite];
		// we do NOT need to close and delete the blue windows because
		// this is handled in the rep dealloc call
		//[aRep release]; it's already released in [fRepresentations removeObject:aRep];
	}
	
	// remove from parent
	[fMontageDoc lockWrite];
		[fParentObject removeChild:self];
	[fMontageDoc unlockWrite];
	
	// remove the WSObject from its parent
	[fMontageDoc armeDocument]->LockWrite();
	if (fParentObject)  // parent object may be null if object is outtaHierarchy
		[fParentObject WSObject]->_RemoveChild(fWSObject);
	[fMontageDoc armeDocument]->UnlockWrite();
	[[fMontageDoc snapperDB] deleteFromSnapDB:self];
	return YES;
}

- (id)initWithArmeWSObj:(TArmeWSObj*)armeObj
{
	self = [self init];
	[self setArmeWSObject:armeObj];
//	vernl(0,"(initWithArmeWSObj) MIObject (" << (void*)self << ") created with fWSObject (" << (void*)fWSObject);
	return self;
}

- (void)invalidateRep
{
	// the object has changed, all its representation needs to be updated
	MIObjectRep* rep;
	NSEnumerator* enumRep = [[self getRepresentations] objectEnumerator];
	
	while (rep = [enumRep nextObject])
		[rep invalidate];
		
	MIObject* child;
	NSEnumerator* childEnum = [[self getChildren] objectEnumerator];
	
	while (child = [childEnum nextObject])
		[child invalidateRep];
		
	[[fMontageDoc snapperDB] refreshSnapDB:self];
}

- (MIObject*)copyInClipBoard:(MIObject*)parentInClipBoard;
{
	MIProgressBarDlgManager progressMngr("Copying");
	TArmeWSObj* wsCopy = [self WSObject]->GetCopy([[[MIApplicationController sharedApplicationController] tempDirectory] UTF8String], &progressMngr);
	if (!wsCopy)
		return NULL;
	
	MIObject* copy = [[[MIObject alloc] initWithArmeWSObj:wsCopy] autorelease];
	[copy setVerticalPos:[self getVerticalPos]];
	[copy setHeight:[self getHeight]];
  copy->fOldHeight = fOldHeight;
	
	[parentInClipBoard addChild:copy];

	//recursive calls
	// if the object is a container, copy its childs too
	TArmeContainer* container = dynamic_cast<TArmeContainer*>([self WSObject]);
	if (container)
	{
		NSEnumerator* childEnum = [[self getChildren] objectEnumerator];
		MIObject* object;
		while (object = [childEnum nextObject])
		{
			[object copyInClipBoard:copy];
			verntl(MICopyCommand,6,"MIOBject copyInClipBoard (4)");
		}
	}
	
	return copy;
}

- (MIObject*)sliceCopyInClipBoard:(MIObject*)parentInClipBoard from:(ArmeTime)startTime to:(ArmeTime)endTime;
{
	ZzSplitTask* splitTask = NULL;

	MIProgressBarDlgManager progressMngr("Copying the object");
	TArmeWSObj* wsCopy = [self WSObject]->GetSliceCopy(startTime, endTime, [[[MIApplicationController sharedApplicationController] tempDirectory] UTF8String], &progressMngr);
	if (!wsCopy)
		return NULL;
	
	[[MIApplicationController sharedApplicationController] executeAndDeleteSplitTask:splitTask forMainViewController:nil];
			
	MIObject* copy = [[[MIObject alloc] initWithArmeWSObj:wsCopy] autorelease];
	[copy setVerticalPos:[self getVerticalPos]];
	[copy setHeight:[self getHeight]];
	
	[parentInClipBoard addChild:copy];

	//recursive calls
	// if the object is a container, copy its childs too
	TArmeContainer* container = dynamic_cast<TArmeContainer*>([self WSObject]);
	if (container)
	{
		NSEnumerator* childEnum = [[self getChildren] objectEnumerator];
		MIObject* object;
		while (object = [childEnum nextObject])
		{
			[object sliceCopyInClipBoard:copy from:startTime to:endTime];
		}
	}
	
	return copy;
}

- (BOOL)isOpaque
{
	return fIsOpaque;
}

- (void)setIsOpaque:(BOOL)flag
{
	fIsOpaque = flag;
}

- (BOOL)isContainer
{
	SInt32 type = fWSObject->GetType();
/*
		kArmeWSObj,
		kContainer,
		kAudioProcessor,
		kTLC,
		kAudioClip,
		kVideoClip,
		kVideoEffect,
		kVideoXFadeEffect,
		kTextClip,
		kMIDIClip
*/
	return ( type == TArmeWSObj::kArmeWSObj || type == TArmeWSObj::kContainer || type == TArmeWSObj::kAudioProcessor || type == TArmeWSObj::kTLC || type == TArmeWSObj::kVideoEffect);// TArmeWSObj::kVideoXFadeEffect is not considered as a container
}

- (TArmeWSObj*)WSObject
{
	return fWSObject;
}

- (void)setArmeWSObject:(TArmeWSObj*)WSObject
{
#ifdef AUTOMATION_DATA
	int nbParams;
#endif
	
	if (fWSObject)
	{
		fWSObject->ReleaseMIObjectRef();
	}
	
	fWSObject = WSObject;
	fWSObject->AddMIObjectRef();
	
	switch ( [self getType] )
	{
		case (TArmeWSObj::kVideoXFadeEffect):
		case (TArmeWSObj::kVideoEffect):
			break;
		case (TArmeWSObj::kArmeWSObj):
		case (TArmeWSObj::kAudioProcessor):
#ifdef AUTOMATION_DATA
			nbParams = fWSObject->GetAudioPluginClass()->GetNbParams();
			fHeight = kRepAutomationCurveHeight*nbParams;
			fHeight += _MIVPos(kDefaultGroupRepresentationHeight);
			break;
#endif
		case (TArmeWSObj::kContainer):
#ifdef SMALL_TITLEBAR
            fHeight = kDefaultGroupRepresentationHeight;
#else
            GET_VALUE(fHeight) = kDefaultGroupRepresentationHeight;
#endif
			break;
		case (TArmeWSObj::kAudioClip):
		case (TArmeWSObj::kVideoClip):
		case (TArmeWSObj::kTextClip):
#ifdef SMALL_TITLEBAR
            fHeight = kDefaultClipRepresentationHeight;
#else
            GET_VALUE(fHeight) = kDefaultClipRepresentationHeight;
#endif
			break;
		case (TArmeWSObj::kTLC):
			break;
		default:
			verntl(interfaceStephan,0,"Unsupported object type in setArmeWSObject: " << (unsigned int)[self getType]);
			break;
	}
}

- (MIMontageDocument*)montageDoc
{
	return fMontageDoc;
}

- (void)setMontageDoc:(MIMontageDocument*)montageDoc
{
	if ( !montageDoc )
	{
		verntl(interfaceStephan,0,"Null montageDoc passed to setMontageDoc in MIObject");
	}
	fMontageDoc = montageDoc;
}

- (NSColor*)backgroundColor
{
	return fBackgroundColor;
}

- (void)setBackgroundColor:(NSColor*)color
{
	[color retain];
	[fBackgroundColor release];
	fBackgroundColor = color;
}


- (ArmeTime)getLength
{
    ArmeTime t;
	if ( !fWSObject->GetDoc() )
        t = fWSObject->GetDuration();
	else
	{
		fWSObject->GetDoc()->LockRead();
			t = fWSObject->GetDuration();
		fWSObject->GetDoc()->UnlockRead();
	}
    
    return t;
}

- (NSString*)title
{
	NSString	*theTitle;
	NSString*	vaoOutput = @"";
	BOOL		displayVAO = NO;
	SInt32		type;
	
	type = [self getType];
	
	// if the object is a direct children of the TLC, we need to display routing
	if( ( fWSObject->GetParent() == fWSObject->GetDoc()->GetTopContainer() ) &&
		( type == TArmeWSObj::kAudioClip || type == TArmeWSObj::kContainer || 
			type == TArmeWSObj::kAudioProcessor || type == TArmeWSObj::kVideoEffect ) )
	{
		displayVAO = YES;
	}
	
	if ( fWSObject->GetDoc() )
		fWSObject->GetDoc()->LockRead();
	theTitle = [NSString stringWithUTF8String:fWSObject->GetName()];
	if([theTitle isEqualToString:@""])
		theTitle = @"<UNKNOWN>";
	
	if ( displayVAO )
	{
//		MEAudioClip* audioClip = dynamic_cast<MEAudioClip*>(fWSObject);
//		ZzAssert(audioClip);
		MEVAORouting vaoRouting = fWSObject->GetVAORouting();
		if(vaoRouting.bStereo)
			vaoOutput = [NSString stringWithFormat:@",%d-%d",2*vaoRouting.index+1,2*vaoRouting.index+2];
		else
			vaoOutput = [NSString stringWithFormat:@",%d",vaoRouting.index+1];
		theTitle = [theTitle stringByAppendingString:vaoOutput];
	}
	if ( fWSObject->GetDoc() )
		fWSObject->GetDoc()->UnlockRead();
	return theTitle;
}

- (BOOL)isTLC
{
	if ( self == [[self montageDoc] topLevelContainer] )
		return YES;
	else
		return NO;
}


- (SInt32)getType
{
	SInt32	type;

	ZzAssert( fWSObject );
	
	if (fWSObject->GetDoc())
	{
		fWSObject->GetDoc()->LockRead();
		type = fWSObject->GetType();
		fWSObject->GetDoc()->UnlockRead();
	}
	else
		type = fWSObject->GetType(); // object in the clipboard are not attached to any document
	
	return type;
}

- (ArmeTime)getHorizontalPos
{
    ArmeTime	t;
    
	if ( !fWSObject->GetDoc() )
        t = fWSObject->GetPos().fStartTime;
	else
	{
		fWSObject->GetDoc()->LockRead();
        t = fWSObject->GetPos().fStartTime;
		fWSObject->GetDoc()->UnlockRead();
	}
    
    return t;
}

- (ArmeTime)getMaxHorizontalPos
{
	ArmeTime	t;

	t = [self getHorizontalPos];
	GET_VALUE(t) += GET_VALUE([self getLength]);

	return t;
}

- (void)setVerticalPos:(MIVPos)verticalPos
{
	fVerticalPos = verticalPos;
	if ([self WSObject])
		[self WSObject]->SetVerticalPos(GET_VALUE(verticalPos));
	[[fMontageDoc snapperDB] refreshSnapDB:self];
}

- (MIVPos)getVerticalPos
{
	return fVerticalPos;
/*	if ([self WSObject])
	{
		ZzAssert([self WSObject]->GetVerticalPos() >= 0);
		return _MIVPos([self WSObject]->GetVerticalPos());
	}
	else
		return _MIVPos(0);*/
}

- (MIVPos)getAbsoluteVerticalPos
{
	if ([self parentObject])
		return [self getVerticalPos] + [[self parentObject] getAbsoluteVerticalPos];
	else
		return [self getVerticalPos];
}

- (MIVPos)getMaxVerticalPos
{
	MIVPos	pos;

	GET_VALUE(pos) = GET_VALUE(fHeight) + GET_VALUE([self getVerticalPos]);

	return pos;
}

- (void)setHeight:(MIVPos)height
{
/*	if ( !height.value )
	{
		verntl(interfaceStephan,0,"height 0 passed to setHeight in MIObject, setting it to a default value");
		height.value = kDefaultGroupRepresentationHeight;
	}
*/	
	fHeight = height;
	[[fMontageDoc snapperDB] refreshSnapDB:self];
}

- (MIVPos)getHeight
{
	if ( !GET_VALUE(fHeight) )
	{
		if (fWSObject->GetType() == TArmeWSObj::kVideoXFadeEffect)
		{
			fHeight = _MIVPos(1);
		}
		else
		{
			verntl(interfaceStephan,0,"This object's height was not set, setting it to the default value");
#ifdef SMALL_TITLEBAR
			fHeight = kDefaultGroupRepresentationHeight;
#else
			GET_VALUE(fHeight) = kDefaultGroupRepresentationHeight;
#endif
		}
	}
	return fHeight;
}

- (MIRect)modelFrame
{
	MIRect	fr;

	fr.origin.time = [self getHorizontalPos];
	fr.origin.vpos = [self getVerticalPos];
	fr.size.height = [self getHeight];
	fr.size.duration = [self getLength];

	return fr;
}

- (MIRect)absoluteModelFrame
{
	MIRect	fr;

	fr.origin.time = fWSObject->GetAbsoluteStartTime();
	fr.origin.vpos = [self getAbsoluteVerticalPos];
	fr.size.height = [self getHeight];
	fr.size.duration = [self getLength];

	return fr;
}

- (BOOL)isMuted
{
	bool 		isMuted;
	//TArmeWSObj	*solo;

	if(fWSObject->GetDoc()) // doc may be null for clipboard clips
		fWSObject->GetDoc()->LockRead();
	isMuted = fWSObject->IsMuted();
//	solo = fWSObject->GetDoc()->getSolo();
	if(fWSObject->GetDoc())
		fWSObject->GetDoc()->UnlockRead();

	return isMuted;
}

- (void)setMuted:(BOOL)flag
{
	NSEnumerator	*repEnum;
	MIObjectRep		*aRep;
	
	if (flag != fWSObject->GetMuted())
	{
		fWSObject->SetMuted(flag);
	
		repEnum = [fRepresentations objectEnumerator];
		while ( aRep = [repEnum nextObject] )
		{
			[aRep setImagesAndColor];
			[aRep refreshCachedTitleImage:NO];
			[aRep refreshCachedAutomationImage:NO];
			[aRep refreshCachedBodyImage:NO];
			[aRep refreshCachedTitleStringImage];
		}
	}
}

- (MIObject*)parentObject
{
	return fParentObject;
}

- (void)setParentObject:(MIObject*)parentObject
{
	if ( !parentObject )
	{
		verntl(interfaceStephan,0,"Null parentObject passed to setParentObject in MIObject");
	}
	fParentObject = parentObject;
	[[fMontageDoc snapperDB] refreshSnapDB:self];
}

- (MIPoint)convertPointFromTLCCoords:(MIPoint)point
{
	MIObject	*parent = self;
	MIPoint		p = point;

	p.time -= fWSObject->GetAbsoluteStartTime();
	p.vpos -= [self getAbsoluteVerticalPos];

	return p;
	
	do
	{
		MIRect parentFrame = [parent modelFrame];
		GET_VALUE(p.vpos) -= GET_VALUE( parentFrame.origin.vpos);
		GET_VALUE(p.time) -= GET_VALUE( parentFrame.origin.time);
	} while ( parent = [parent parentObject] );
	
	return p;
}

- (MIPoint)convertPointToTLCCoords:(MIPoint)point
{
	MIPoint p = point;
	
	p.time += fWSObject->GetAbsoluteStartTime();
	p.vpos += [self getAbsoluteVerticalPos];

	return p;
}

- (MIRect)convertRectToTLCCoords:(MIRect)rect
{
	MIRect		r = rect;
	
	r.origin.time += fWSObject->GetAbsoluteStartTime();
	r.origin.vpos += [self getAbsoluteVerticalPos];
	
	return r;
}	

- (MIRect)convertRectFromTLCCoords:(MIRect)rect
{
	MIRect		r = rect;
	
	r.origin.time -= fWSObject->GetAbsoluteStartTime()	;
	r.origin.vpos -= [self getAbsoluteVerticalPos];
	
	return r;
}

- (MIObject*)freezeSource {return fFreezeSource;}
- (void)setFreezeSource:(MIObject*)freezeSource {fFreezeSource = [freezeSource retain]; }
- (BOOL)isFrozenObject {return fFreezeSource != nil;};
- (BOOL)hasParentFrozenObject
{
	if ([self isFrozenObject])
		return YES;
	else if ([self parentObject])
		return [[self parentObject] hasParentFrozenObject];
	else 
		return NO;
}

- (MIObject*)frozenObject {return fFrozenObject;}
- (void)setFrozenObject:(MIObject*)frozenObject {fFrozenObject = [frozenObject retain]; }

- (void)markModified 
{
	fContentHasBeenModifiedSinceLastFreezeOperation = true; 
	[self markParentModified];
}
- (void)cleanModifiedFlag {fContentHasBeenModifiedSinceLastFreezeOperation = false; }
- (BOOL)hasBeenModified {return fContentHasBeenModifiedSinceLastFreezeOperation;}
- (void)markParentModified
{
	if ([self parentObject])
		[[self parentObject] markModified];
}

- (void)_fillArrayWithModifiedParentsFlag:(ZzArray<bool>*)parentsFlag
{
	parentsFlag->AddItem([self hasBeenModified]);
	if ([self parentObject])
		[[self parentObject] _fillArrayWithModifiedParentsFlag:parentsFlag];

}

- (ZzArray<bool>*)getCurrentModifiedParentsFlag
{
	ZzArray<bool>* parentsFlag = new ZzArray<bool>(32);
	[self _fillArrayWithModifiedParentsFlag:parentsFlag];
	
	return parentsFlag;
}

- (void)_resetModifiedFlag:(ZzArray<bool>*)parentsFlag indexInArray:(unsigned int)index
{
	fContentHasBeenModifiedSinceLastFreezeOperation = parentsFlag->ItemAt(index);
	if ([self parentObject])
		[[self parentObject] _resetModifiedFlag:parentsFlag indexInArray:(++index)];
}

- (void)resetModifiedFlag:(ZzArray<bool>*)parentsFlag
{
	[self _resetModifiedFlag:parentsFlag indexInArray:0];
}

- (void)addRepresentation:(id)newRepresentation
{
	if ( !newRepresentation )
	{
		verntl(interfaceStephan,0,"MIObject addRepresentation: nil passed as argument");
		return;
	}
		
	[newRepresentation setRepresentedObject:self];
	[fRepresentations addObject:newRepresentation];
}

- (void)removeRepresentation:(id)representation
{
	if ( !representation )
	{
		verntl(interfaceStephan,0,"MIObject removeRepresentation:nil passed as argument");
		return;
	}
	
	[fRepresentations removeObject:representation];
}

- (NSArray*)getRepresentations
{
	return fRepresentations;
}

- (MIObjectRep*)createRepresentationsRecursively:(id)parentView
{
	MIObjectRep* selfNewRep = [self createRepresentationInView:parentView];
	MIObject* childObj;
	NSEnumerator* childObjEnum = [fChildren objectEnumerator];
	while ( childObj = [childObjEnum nextObject] )
	{
		[childObj createRepresentationsRecursively:selfNewRep];
	}
	return selfNewRep;
}

- (void)createAllRepresentations
{
	NSEnumerator* mainViewEnumerator = [[[self montageDoc] windowControllers] objectEnumerator];
	MIMainViewController	*aMainViewController;
	MIObjectRep* newObjectRep;
	
	while ( aMainViewController = [mainViewEnumerator nextObject] )
	{
		newObjectRep = [self createRepresentationInView:[aMainViewController topLevelContainerView]];
	}	
}

- (MIObjectRep*)createRepresentationInView:(MITopLevelContainerRep*)parentView
{
	MIObjectRep	*newRep = nil;

	NSRect		newFrame;
	
	if ( !parentView )
	{
		verntl(interfaceStephan,0,"No parentView passed to createRepresentationInView");
		return nil;
	}

#ifdef SCALABLE_TITLEBAR
	newFrame = [[[parentView getTopLevelContainerRep] mainViewController] MIRectToNSRect:[self modelFrame]];
#else	
	newFrame = NSMakeRect(0,0,	
		[[[parentView getTopLevelContainerRep] mainViewController] 
			armeTimeToPixel:[self getLength]],
		[[[parentView getTopLevelContainerRep] mainViewController] 
			MIVPosToPixel:[self getHeight]]+kRepTitleHeight);
	newFrame.origin.x = [[[parentView getTopLevelContainerRep] mainViewController] 
		armeTimeToPixel:[self getHorizontalPos]];
	newFrame.origin.y = [[[parentView getTopLevelContainerRep] mainViewController] 
		MIVPosToPixel:[self getVerticalPos]];
#endif

	switch ( [self getType] )
	{
		case (TArmeWSObj::kArmeWSObj):
		case (TArmeWSObj::kContainer):
			newRep = [[MIContainerRep alloc] initWithFrame:newFrame inTLC:[parentView getTopLevelContainerRep]];
			break;
		case (TArmeWSObj::kAudioProcessor):
			newRep = [[MIProcessorRep alloc] initWithFrame:newFrame inTLC:[parentView getTopLevelContainerRep]];
			break;
		case (TArmeWSObj::kAudioClip):
			newRep = [[MIAudioClipRep alloc] initWithFrame:newFrame inTLC:[parentView getTopLevelContainerRep]];
			break;
		case (TArmeWSObj::kVideoClip):
			newRep = [[MIVideoClipRep alloc] initWithFrame:newFrame inTLC:[parentView getTopLevelContainerRep]];
			break;
		case (TArmeWSObj::kTextClip):
			newRep = [[MITextClipRep alloc] initWithFrame:newFrame inTLC:[parentView getTopLevelContainerRep]];
			break;
		case (TArmeWSObj::kVideoEffect):
			newRep = [[MIVideoEffectRep alloc] initWithFrame:newFrame inTLC:[parentView getTopLevelContainerRep]];
			break;
		case (TArmeWSObj::kVideoXFadeEffect):
			newRep = [[MIVideoTransitionRep alloc] initWithFrame:newFrame inTLC:[parentView getTopLevelContainerRep]];
			break;
		default:
			verntl(interfaceStephan,0,"Unsupported object type in createRepresentations : " << [self getType]);
			break;
	}
	[self addRepresentation:newRep];

//	[self setParentObject:[parentView representedObject]];
	
	[newRep cacheImagesIfSizeSmallerThan:kCacheingMinSize frameIsInRect:NSInsetRect([[parentView getTopLevelContainerRep] visibleRect],-kCacheingHorizontalExtra, -kCacheingVerticalExtra)];
/*    if ( USE_CACHED_IMAGES )
    {
        [newRep refreshCachedBodyImage];
        [newRep refreshCachedTitleImage];
    }*/
	[[fMontageDoc snapperDB] refreshSnapDB:self];
	
	return [newRep autorelease];
}

- (void)addToSelections
{
	NSEnumerator		*repEnum;
	MIObjectRep			*aRep;
	
	repEnum = [fRepresentations objectEnumerator];
	while ( aRep = [repEnum nextObject] )
	{
		[[[aRep getTopLevelContainerRep] mainViewController] addObjectToSelection:aRep];
	}
}

- (void)setSelectionsFromTime:(ArmeTime)from toTime:(ArmeTime)to
{
	NSEnumerator		*repEnum;
	MIObjectRep			*aRep;
	
	repEnum = [fRepresentations objectEnumerator];
	while ( aRep = [repEnum nextObject] )
	{
		[[[aRep getTopLevelContainerRep] mainViewController] clearSelection];
		[[[aRep getTopLevelContainerRep] mainViewController] selectionSetTimeSliceType];
		[[[aRep getTopLevelContainerRep] mainViewController] addObjectToSelection:aRep];
		[[[aRep getTopLevelContainerRep] mainViewController] setSelectionStartTime:from];
		[[[aRep getTopLevelContainerRep] mainViewController] setSelectionEndxTime:to];
	}
}

- (void)moveChildToTopOfStack:(MIObject*)aChild
{
	[aChild retain]; // to prevent retainCount to reach 0
	[fChildren removeObject:aChild];
	[fChildren addObject:aChild];
	[aChild release];
}

- (void)addChild:(MIObject*)aChild
{
	if ( [fChildren indexOfObject:aChild] != NSNotFound )
	{
		verntl(interfaceStephan,0,"asked to add an already existing child");
		return;
	}

#ifdef DEBUG
//	printf("MIObject retaining MIObject %p in addChild by adding it to fChildren\n",(void*)aChild);
#endif //#ifdef DEBUG
	[fChildren addObject:aChild];
	[aChild setParentObject:self];	
}

- (void)removeChild:(MIObject*)aChild
{
//	[aChild setParentObject:nil];
#ifdef DEBUG
//	printf("MIObject releasing MIObject %p in removeChild ([fChildren removeObject:aChild])\n",(void*)aChild);
#endif //#ifdef DEBUG
	[fChildren removeObject:aChild];
}

- (NSArray*)getChildren
{
	return fChildren;
}

- (void)addCursorRectsInTLC:(MITopLevelContainerRep*)theTLC
{
	MIObject		*aChild;
	NSEnumerator	*childEnum;

	[[self findRepresentationInTLC:theTLC] addCursorRects];

	childEnum = [fChildren objectEnumerator];
	while ( aChild = [childEnum nextObject] )
	{
		[aChild addCursorRectsInTLC:theTLC];
	}
}

- (void)recursivelyRecreateFrame
{
	MIObject		*aChild;
	NSEnumerator	*childEnum;
	
	[self recreateFrame];
	
	childEnum = [fChildren objectEnumerator];
	while ( aChild = [childEnum nextObject] )
	{
		[aChild recursivelyRecreateFrame];
	}
}

- (void)refreshTrackingRect
{
	NSEnumerator	*viewEnumerator;
	MIObjectRep		*aRep;

	viewEnumerator = [fRepresentations objectEnumerator];
	while ( aRep = [viewEnumerator nextObject] )
    {
        [aRep refreshTrackingRect];
    }
}

- (void)recursivelyRefreshTrackingRect
{
	MIObject		*aChild;
	NSEnumerator	*childEnum;
	
	[self recreateFrame];
	
	childEnum = [fChildren objectEnumerator];
	while ( aChild = [childEnum nextObject] )
	{
		[aChild recursivelyRefreshTrackingRect];
	}
}

- (void)recreateFrame
{
	NSEnumerator	*viewEnumerator;
	MIObjectRep		*aRep;

	viewEnumerator = [fRepresentations objectEnumerator];
	while ( aRep = [viewEnumerator nextObject] )
    {
        [aRep recreateFrame];
		[aRep refreshTrackingRect];
    }
}

- (MIRect)minimalRectToFitContent
{
	NSEnumerator				*childEnum;
	MIObject					*aChild;
	MIRect						minRect;

	minRect = MIZeroRect;

	// bail out if no children
	if ( ![[self getChildren] count] )
		return MIZeroRect;

	if ( [self isTLC] )
	{
		GET_VALUE(minRect.size.duration) = 13230000;
		GET_VALUE(minRect.size.height) = 1000;
	}
	
	childEnum = [[self getChildren] objectEnumerator];
	while ( aChild = [childEnum nextObject] )
	{
		if ( MIEqualRects(minRect, MIZeroRect) )
			minRect = [aChild modelFrame];
		else
			minRect = MIUnionRect(minRect, [aChild modelFrame]);
	}

	// add, as per george request, a margin left and right of 8 pixels at the default zoom level
/*
	if ( ![self isTLC] )
	{
		ArmeTime t = _ArmeTime((UInt64)8*[MIMainViewController getFramesPerPixelFromHZoomIndex:[MIMainViewController getDefaultHZoomIndex]]); // assumption: the last index is 1
		minRect.size.duration += (t+t);
		minRect.origin.time -= t;
	}
	
	COMMENTED BY BENJA 2002-08-24
*/

	if ( ![self isTLC] )
	{
#ifdef FOOTER_EXISTS
	// add the footer and title bar
		minRect.origin.vpos -= kRepFooterHeight;
		minRect.size.height += kRepFooterHeight;
		minRect.size.height += kRepHeaderHeight;
#endif

#ifdef AUTOMATION_DATA
		minRect.size.height += [self getAutomationCurvesHeight];
#endif

#ifdef SCALABLE_TITLEBAR
		minRect.size.height += [self getTitleHeight];
#else
		GET_VALUE(minRect.size.height) += kRepTitleHeight;
#endif
	}

	if ( [self parentObject] )
	{
		GET_VALUE(minRect.origin.time) += GET_VALUE([self getHorizontalPos]);
		GET_VALUE(minRect.origin.vpos) += GET_VALUE([self getVerticalPos]);
	}
	
	if ( [self isTLC] )
	{
		minRect.size.duration = _ArmeTime((SInt64)(GET_VALUE(minRect.size.duration) * 1.05));
		minRect.size.height = _MIVPos((SInt32)(GET_VALUE(minRect.size.height) * 1.05));
	}
	return minRect;
}

- (void)setModelFrameFromRect:(NSRect)newFrame inTLC:(MITopLevelContainerRep*)theTLC
{
	MIRect newModelFrame;
	
	newModelFrame = [theTLC NSRectToMIRect:newFrame];
	[self setModelFrame:newModelFrame];
}

- (void)setModelFrame:(MIRect)newFrame
{
	TArmeWSObjPos		theArmePos;

	if ( fWSObject->GetDoc() )
	{
		fWSObject->GetDoc()->LockRead();
		theArmePos = fWSObject->GetPos();
		fWSObject->GetDoc()->UnlockRead();
	}
	else
		theArmePos = fWSObject->GetPos();
	
	if ( [self isTLC] )
	{
		if ( GET_VALUE(newFrame.origin.vpos) < 0 )
			newFrame.origin.vpos = _MIVPos(0);
		if ( GET_VALUE(newFrame.origin.time) < 0 )
			newFrame.origin.time = _ArmeTime(0);
	}
	theArmePos.fStartTime = MIMinX(newFrame);

	[[self montageDoc] lockWrite];
	[self setVerticalPos:MIMinY(newFrame)];
	[self setHeight:MIHeight(newFrame)];
	[[self montageDoc] unlockWrite];
    
	theArmePos.fVerticalPos = GET_VALUE([self getVerticalPos]);
    fWSObject->SetPos(theArmePos);
    fWSObject->_SetLength(MIWidth(newFrame));
	[[fMontageDoc snapperDB] refreshSnapDB:self];
    [self recreateFrame];
}


- (void)refreshHierarchy
{
//	NSEnumerator	*aChild;
//	TArmeWSObj		*aChildWSObj;

	// check for each children if it is a subview of a representation,
	// if not, add it
	// if unknown objects are present, remove them
	// then ask to each children to check itself too..
	
	
}

- (void)setTitlesNeedDisplay
{
	MIObjectRep			*aRepresentation;
	MIObject			*aChild;
	NSEnumerator		*childEnum;
	
	for(UInt32 i = 0; i < [fRepresentations count]; i++ )
	{
		aRepresentation = [fRepresentations objectAtIndex:i];
#ifdef SCALABLE_TITLEBAR
		float				titleHeight;
		NSRect				titleRect = [aRepresentation getTLCFrame];
		titleHeight = [[aRepresentation getTopLevelContainerRep] MIVPosToPixel:kRepTitleHeight];
		titleRect.origin.y = NSMaxY(titleRect) - titleHeight;
		titleRect.size.height = titleHeight;
#else
		NSRect titleRect = [aRepresentation getTLCFrame];
		titleRect.origin.y = NSMaxY(titleRect) - kRepTitleHeight;
		titleRect.size.height = kRepTitleHeight;
#endif
		[[aRepresentation getTopLevelContainerRep] setNeedsDisplayInRect:titleRect];
	}

	childEnum = [fChildren objectEnumerator];

	while ( aChild = [childEnum nextObject] )
	{
		[aChild setTitlesNeedDisplay];
	}
}

- (void)refreshCachedTitleStringImageInReps
{
	NSEnumerator		*repEnumerator;
	id					aRepresentation;
	BOOL				isTLC = [self isTLC];
	
	repEnumerator = [fRepresentations objectEnumerator];
	
	while ( aRepresentation = [repEnumerator nextObject] )
	{
		if( !isTLC )
			[aRepresentation refreshCachedTitleStringImage];
	}
}

- (void)forceImmediateDisplayOfGuides:(MIRect)rect
{
	NSEnumerator		*repEnumerator;
	id					aRepresentation;
//	BOOL				isTLC = [self isTLC];
	NSRect				vRect;
	NSRect				tmpRect;
	NSRect				dRect;
	
	return;

	repEnumerator = [fRepresentations objectEnumerator];
	
	while ( aRepresentation = [repEnumerator nextObject] )
	{
		vRect = [[aRepresentation getTopLevelContainerRep] visibleRect];
		
		tmpRect = [[aRepresentation getTopLevelContainerRep] MIRectToNSRect:rect];

		// left vertical guide
		dRect = NSMakeRect(NSMinX(tmpRect), NSMinY(vRect), 1, NSHeight(vRect));
		dRect = NSInsetRect(dRect, -5,0);
		[[aRepresentation getTopLevelContainerRep] setNeedsDisplayInRect:dRect];
		[[aRepresentation getTopLevelContainerRep] displayIfNeeded];

		// right vertical guide
		dRect = NSMakeRect(NSMaxX(tmpRect), NSMinY(vRect), 1, NSHeight(vRect));
		dRect = NSInsetRect(dRect, -5,0);
		[[aRepresentation getTopLevelContainerRep] setNeedsDisplayInRect:dRect];
		[[aRepresentation getTopLevelContainerRep] displayIfNeeded];
		
		// bottom horizontal guide
		dRect = NSMakeRect(NSMinX(vRect), NSMinY(tmpRect), NSWidth(vRect), 1);
		dRect = NSInsetRect(dRect, 0,-5);
		[[aRepresentation getTopLevelContainerRep] setNeedsDisplayInRect:dRect];
		[[aRepresentation getTopLevelContainerRep] displayIfNeeded];

		// top horizontal guide
		dRect = NSMakeRect(NSMinX(vRect), NSMaxY(tmpRect), NSWidth(vRect), 1);
		dRect = NSInsetRect(dRect, 0,-5);
		[[aRepresentation getTopLevelContainerRep] setNeedsDisplayInRect:dRect];
		[[aRepresentation getTopLevelContainerRep] displayIfNeeded];
	}
}

- (void)setRepresentationsNeedDisplay
{
	NSEnumerator		*repEnumerator;
	id					aRepresentation;
	BOOL				isTLC = [self isTLC];
	
	repEnumerator = [fRepresentations objectEnumerator];
	
	while ( aRepresentation = [repEnumerator nextObject] )
	{
		if ( isTLC )
			[aRepresentation setNeedsDisplay:YES];
		else
			[aRepresentation setNeedsDisplay];
	}
}


- (void)moveToObject:(MIObject*)newParent atTime:(ArmeTime)newPos withVPos:(MIVPos)VPos
{
	MIObject		*oldParent;
	TArmeWSObjPos   newArmePos;
	MIObject		*aParent;
	MIObject		*aParentsParent;

	// this function does not create a command or whatever, it justs moves the view...
	// it does not take the media engine into account here.

	if ( !newParent )
	{
		verntl(interfaceStephan,0,"moveToObject called with NULL newParent");
		return;
	}
	
	oldParent = fParentObject;
	
#ifdef DEBUG
//	printf("MIObject retaining itself %p in moveToObject\n",(void*)self);
#endif //#ifdef DEBUG
	[self retain]; //to prevent retainCount to reach 0
	[oldParent removeChild:self];
	// if the object didn't change parent, just position, this is not a problem...
	[newParent addChild:self];
#ifdef DEBUG
//	printf("MIObject releasing itself %p in moveToObject\n",(void*)self);
#endif //#ifdef DEBUG
	[self release];
	
	if ( [newParent isTLC] )
	{
		if ( GET_VALUE(newPos) < 0 )
			newPos = _ArmeTime(0);
		if ( GET_VALUE(VPos) < 0 )
			VPos = _MIVPos(0);
	}

	[self setVerticalPos:VPos];
	
	newArmePos.fParent = [fParentObject WSObject];
	newArmePos.fInput = 0; // support for multiple entries will come, although not right now
	newArmePos.fStartTime = newPos;
	newArmePos.fVerticalPos = GET_VALUE(VPos);

//	DumpArmeHierarchy(fWSObject->GetDoc());
	fWSObject->SetPos(newArmePos);
    // END OF TO BE PUT IN COMMAND
//	DumpArmeHierarchy(fWSObject->GetDoc());
    
	[self recreateFrame];
	[self recursivelyRefreshTrackingRect];
//	[newParent setRepresentationsNeedDisplay];
/*    
    if ( [newParent resizeFrameToFullyIncludeChild:self] )
    {
        // parentFrame changed, modify myself accordingly
        // practically, that means that any negative value should be set to 0
        // since the parent has changed itself to fit exactly the child
        
    }
*/	
	// move parent to the top of the display list
	aParent = newParent;
	aParentsParent = [newParent parentObject];
	while ( aParentsParent )
	{
		[aParent retain]; //to prevent retainCount to reach 0
		[aParentsParent removeChild:aParent];
		[aParentsParent addChild:aParent];
		[aParent release];
		
		aParent = aParentsParent;
		aParentsParent = [aParent parentObject];
	}
	
	
	[[fMontageDoc snapperDB] refreshSnapDB:self];
//	[newParent setRepresentationsNeedDisplay];
}

- (MIObjectRep*)findRepresentationInWindow:(NSWindow*)theWindow
{
	NSEnumerator	*viewEnum;
	MIObjectRep		*aRep;
	
	viewEnum = [fRepresentations objectEnumerator];
	
	while ( aRep = [viewEnum nextObject] )
	{
		if ( [[aRep getTopLevelContainerRep] window] == theWindow )
			return aRep;
	}
	
	return nil;
}

- (MIObjectRep*)findRepresentationInTLC:(MITopLevelContainerRep*)theTLC
{
	MIObjectRep		*aRep;
	
	for(UInt32 i = 0; i < [fRepresentations count]; i++)
	{
		aRep = [fRepresentations objectAtIndex:i];
		if ( [aRep getTopLevelContainerRep] == theTLC )
			return aRep;
	}
	
	return nil;
}

- (MIObjectRep*)hitTest:(NSPoint)aPoint inTLC:(MITopLevelContainerRep*)theTLC
{
	NSEnumerator	*childEnum = nil;
	MIObjectRep		*myRep = nil;
	MIObjectRep		*theRep = nil;
	MIObject		*aChild = nil;
	MIObjectRep		*transition = nil;
	
	
	myRep = [self findRepresentationInTLC:theTLC];
	if ( NSPointInRect(aPoint, [myRep frame]) )
	{
		// this point is within my frame. So either a child or self
		NSPoint		pointInMyCoords = [myRep convertPointFromParent:aPoint];

		if ( [self isCollapsed] )
			return myRep;
		
		childEnum = [fChildren reverseObjectEnumerator];
		while ( (aChild = [childEnum nextObject]) && !theRep )
		{
			if ( [aChild getType] == TArmeWSObj::kVideoXFadeEffect )
			{
				if ( !transition )
					transition = [aChild hitTest:pointInMyCoords inTLC:theTLC];
//					transition = aChild; // transitions are second class citizens, they get selected only if no other thing can be selected.
				continue;
			}
			
			theRep = [aChild hitTest:pointInMyCoords inTLC:theTLC];
		}
		
		if ( !theRep && transition )
			theRep = transition;
			

		if ( theRep )
			return theRep;
		else
			return myRep;
	}
	else
		return nil;
}

- (void)drawRect:(NSRect)aRect inTLC:(MITopLevelContainerRep*)theTLC withOffset:(NSPoint)parentOrigin
{
	//MTGStartTimeLog
    NSEnumerator	*childrenEnum;
    MIObject		*aChild;
    NSPoint			myRepOrigin;
    MIObjectRep		*myRepresentation;
	NSRect			myRepFrame;

    // draw myself first

    // find my representation
    myRepresentation = [self findRepresentationInTLC:theTLC];

	myRepFrame = [myRepresentation frame];
    myRepOrigin = TranslateNSPoint(myRepFrame.origin, parentOrigin);
    myRepFrame.origin = myRepOrigin;
	
	if ( [MIPreferences shadowsAroundObject] && [fParentObject isTLC])
		[myRepresentation drawShadowClippedTo:aRect withObjectOrigin:myRepOrigin];
		
    [myRepresentation drawRect:aRect withOffset:myRepOrigin];
	
    if ( ![self isCollapsed] && NSIntersectsRect( myRepFrame, aRect ) )
    {
//        newClippingRect.origin = myRepOrigin;
//		newClippingRect.size = [myRepresentation frame].size;
//        newClippingRect.size.height -= kRepTitleHeight;
        
//        newClippingRect = NSIntersectionRect(newClippingRect,aRect);
    
        // draw my children;
		// first render all the video transitions
        childrenEnum = [fChildren objectEnumerator];	
        while ( aChild = [childrenEnum nextObject] )
        {
			if ([aChild WSObject]->GetType() == TArmeWSObj::kVideoXFadeEffect)
				[aChild drawRect:aRect inTLC:theTLC withOffset:myRepOrigin];
        }
		// then the other children
        childrenEnum = [fChildren objectEnumerator];	
        while ( aChild = [childrenEnum nextObject] )
        {
			if ([aChild WSObject]->GetType() != TArmeWSObj::kVideoXFadeEffect)
				[aChild drawRect:aRect inTLC:theTLC withOffset:myRepOrigin];
        }
		
		//if there is a slice selection, draw it
		[myRepresentation drawSelection:aRect drawNow:YES]; //actually, we don't take aRect into account here, the selection rectangle is already computed, we just need to draw it
    }
	//MTGEndTimeLog(@"MIObject",@"drawRect:inTLC:withOffset:")
}

- (NSString*)description
{
    NSString		*desc;
    NSString		*children;
    NSString		*representations;
    NSEnumerator	*objEnum;
    MIObject		*aChild;
    MIObjectRep		*aRep;
    
    children = [NSString stringWithUTF8String:""];
    objEnum = [fChildren objectEnumerator];
    while ( aChild = [objEnum nextObject] )
    {
        children = [children stringByAppendingFormat:@"  %p(%@)\n", aChild, [aChild title]]; 
    }
    
    representations = [NSString stringWithUTF8String:""];
    objEnum = [fRepresentations objectEnumerator];
    while ( aRep = [objEnum nextObject] )
    {
        representations = [representations stringByAppendingFormat:@" Rep:%p\n", aRep];
    }
    
	MIRect fr = [self modelFrame];
    desc = [NSString stringWithFormat:@"\nTitle:%@\nMIObject:%p (TArmeWSObject:%p)\nIn Document:%p ParentObject:%p\nChildren:\n{\n%@}\nFrame Origin: {%f,%f} Size:{%f, %f}\n",
                                        [self title], self, fWSObject, fMontageDoc, fParentObject, children, (float)GET_VALUE(MIMinX(fr)), (float)GET_VALUE(MIMinY(fr)), (float)GET_VALUE(MIWidth(fr)), (float)GET_VALUE(MIHeight(fr))];
    
    return desc;
}

////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
////////////////////////////  Dragging
////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////

/*
As the image is dragged into the destination's boundaries, the destination is sent a draggingEntered: message.
  While the image remains within the destination, a series of draggingUpdated: messages are sent.
  If the image is dragged out of the destination, draggingExited: is sent and the sequence of NSDraggingDestination messages stops. If it re-enters, the sequence begins again (with a new draggingEntered: message).
  When the image is released, it either slides back to its source (and breaks the sequence) or a prepareForDragOperation: message is sent to the destination, depending on the value returned by the most recent invocation of draggingEntered: or draggingUpdated:.
  If the prepareForDragOperation: message returned YES, a performDragOperation: message is sent.
  Finally, if performDragOperation: returned YES, concludeDragOperation: is sent.
*/

// The clipsheetwindowView is already registered as a dragging recipient on initialization of the clipSheetController
- (unsigned int)draggingEntered:(id <NSDraggingInfo>)sender inView:(NSView*)aView
{
	return NSDragOperationNone;
}

- (unsigned int)draggingUpdated:(id <NSDraggingInfo>)sender inView:(NSView*)aView
{
	return NSDragOperationNone;
}

- (void)draggingExited:(id <NSDraggingInfo>)sender inView:(NSView*)aView
{
}

- (BOOL)prepareForDragOperation:(id <NSDraggingInfo>)sender inView:(NSView*)aView
{
	return FALSE;
}

- (BOOL)performDragOperation:(id <NSDraggingInfo>)sender inView:(NSView*)aView
{
	return FALSE;
}

- (void)concludeDragOperation:(id <NSDraggingInfo>)sender inView:(NSView*)aView
{
}

- (void)invalidateRepresentationsCachedImages
{
	NSEnumerator* repEnum = [[self getRepresentations] objectEnumerator];
	MIObjectRep* rep;
	while(rep = [repEnum nextObject])
	{
		[rep invalidateCachedImage];
	}
}

- (BOOL)isDescendantOf:(MIObject*)object
{
	MIObject* ascendant = [self parentObject];
	while(ascendant != nil)
	{
		if(object==ascendant)
			return YES;
		ascendant = [ascendant parentObject];
	}
	return NO;
}

-(void)putAllFamilyInFlatArrayIncludingSelf:(NSMutableArray*)whereToAdd
{
	[whereToAdd addObject:self];
	NSEnumerator* childrenEnum = [fChildren objectEnumerator];
	MIObject* child;
	while(child = [childrenEnum nextObject])
	{
		[child putAllFamilyInFlatArrayIncludingSelf:whereToAdd];
	}
}

- (BOOL)isCollapsed
{
  if ( GET_VALUE(fHeight) == GET_VALUE([self getTitleHeight]) )
    return YES;
  
  return NO;
}

- (void)toggleCollapsed
{
    [self setCollapsed:![self isCollapsed]];
}

- (void)setCollapsed:(BOOL)flag
{
    if ( flag == [self isCollapsed] )
        return;

    fIsCollapsed = flag;

    if ( flag )
    { // we now collapse the clip
      fOldHeight = fHeight;
      [self setVerticalPos:_MIVPos(GET_VALUE([self getVerticalPos]) + (GET_VALUE(fHeight) - GET_VALUE([self getTitleHeight])))];
      [self setHeight:[self getTitleHeight]];
    }
    else
    {
      [self setVerticalPos:_MIVPos(GET_VALUE([self getVerticalPos]) - (GET_VALUE(fOldHeight) - GET_VALUE([self getTitleHeight])))];
      [self setHeight:fOldHeight];
    }

}

- (ArmeTime)lockedToTime
{
	return fLockedToTime;
}

- (BOOL)isLocked
{
	return fClipIsLocked;
}

- (void)setLocked:(BOOL)flag
{
	if ( flag == fClipIsLocked )
		return;
	
	fClipIsLocked = flag;
	
	if ( !fClipIsLocked )
		fLockedToTime = _ArmeTime(0);
	else
		fLockedToTime = fWSObject->GetAbsoluteStartTime();

	[self refreshCachedTitleStringImageInReps];

}

- (MIVPos)getTitleHeight
{
#ifdef SCALABLE_TITLEBAR
	return kRepTitleHeight;
#else
	ZzAssert(0);
	// only to get rid of the warning at compilation
	return _MIVPos(0);
#endif
}

- (MIVPos)getAutomationCurvesHeight
{
#ifdef AUTOMATION_DATA
	if ( [self getType] == (TArmeWSObj::kAudioProcessor) )
	{
#ifdef AUTOMATION_DATA
		return _MIVPos(0);
#else //#ifdef AUTOMATION_DATA
		int nbParams = fWSObject->GetAudioPluginClass()->GetNbParams();
		return kRepAutomationCurveHeight*nbParams;
#endif //#ifdef AUTOMATION_DATA
	}
	else
		return _MIVPos(0);
#else
	return _MIVPos(0);
#endif
}

- (MIVPos)getBodyHeight
{
	MIVPos bHeight;
	
	bHeight = fHeight;
	bHeight -= [self getTitleHeight];
	bHeight -= [self getAutomationCurvesHeight];
	
	return bHeight;
}

- (void)setOuttaHierarchy
{
	NSEnumerator* repEnum = [[self getRepresentations] objectEnumerator];
	MIObjectRep* rep;
	while(rep = [repEnum nextObject])
	{
		// close and delete the blue window if needed
	//	if( [self WSObject]->GetType() == TArmeWSObj::kAudioProcessor)
	//	{
		//	[rep closeAndDeleteBlueWindow];
	//	}
		// OBSOLETE : this is taken care of from TArmeWSObj::_NotifyParentOuttaHierarchy()
		// which is called recursively while this one is only called recursively when
		// deleting, but not when rendering for example.
		MITopLevelContainerRep* tlcView = [rep getTopLevelContainerRep];
		MIMainViewController* mainViewController = [tlcView mainViewController];
		[mainViewController removeObjectFromSelection:rep];
		if ( rep->fTrackingRectTag != -1 )
		{
			[tlcView removeTrackingRect:rep->fTrackingRectTag];
			rep->fTrackingRectTag = -1;
			[[tlcView window] resetCursorRects];
		}
	}
	[fRepresentations removeAllObjects];
	fOldParentObject = fParentObject;
	[fParentObject removeChild:self];
	fParentObject = nil;
	fWSObject->SetOuttaHierarchy();
	[[fMontageDoc snapperDB] refreshSnapDB:self];

	#ifdef DEBUG
//	printf("MIObject is about to retain object %p in setOuttaHierarchy ([fMontageDoc->fObjectsSetOuttaHierarchyRecently addObject:self])\n",(void*)self);
	#endif //#ifdef DEBUG
	[fMontageDoc->fObjectsSetOuttaHierarchyRecently addObject:self];
}

- (void)putBackInHierarchy
{
	if (fWSObject->GetType() == TArmeWSObj::kVideoXFadeEffect)
		return;
	fParentObject = fOldParentObject;
	MIObjectRep* parentRep;
	fOldParentObject = nil;
	ZzAssert([fParentObject isInHierarchy]);
	fWSObject->SetBackInHierarchy([fParentObject WSObject]);
	NSEnumerator* repEnum = [[fParentObject getRepresentations] objectEnumerator];
	[fParentObject addChild:self];
	while(parentRep = [repEnum nextObject])
	{
		[self createRepresentationsRecursively:parentRep];
	}
	[[fMontageDoc snapperDB] refreshSnapDB:self];
	[fMontageDoc->fObjectsSetOuttaHierarchyRecently removeObject:self]; // added by benja on 2003-05-30
}

-(BOOL)isInHierarchy
{
	return (fOldParentObject == nil);
}

- (MIObjectMemento*)getMemento
{
	MIObjectMemento* memento = [[MIObjectMemento alloc] autorelease];
	[memento setVerticalPos:[self getVerticalPos]];
	[memento setHeight:[self getHeight]];
	[memento setMuted:[self isMuted]];
//	[memento setIsContainer:[self isContainer]];
//	[memento setIsCollapsed: $$TODO
	[memento setWSObjMemento:[self WSObject]->GetMemento()];
	[memento setMontageDoc:[self montageDoc]];
	memento->fFreezeSource = [fFreezeSource retain];
	memento->fFrozenObject = [fFrozenObject retain];
	memento->fContentHasBeenModifiedSinceLastFreezeOperation = fContentHasBeenModifiedSinceLastFreezeOperation;
	
	return memento;
}

- (void)setMemento:(MIObjectMemento*)memento
{
	[self setRepresentationsNeedDisplay];  //invalidates old rect

	[self WSObject]->SetMemento([memento getWSObjMemento]);

	[self setVerticalPos:[memento getVerticalPos]];
	[self setHeight:[memento getHeight]];
  fOldHeight = [memento getOldHeight];
	[self setMuted:[memento isMuted]];
	[self setMontageDoc:[memento montageDoc]];
//	[self setIsCollapsed: $$TODO];
	[[fMontageDoc snapperDB] refreshSnapDB:self];
	[fFreezeSource release];
	[fFrozenObject release];
	fFreezeSource = [memento->fFreezeSource retain];
	fFrozenObject = [memento->fFrozenObject retain];
	fContentHasBeenModifiedSinceLastFreezeOperation = memento->fContentHasBeenModifiedSinceLastFreezeOperation;
	
	[self invalidateRepresentationsCachedImages];
	[self recreateFrame];
	[self setRepresentationsNeedDisplay];
}

- (id)initWithMemento:(MIObjectMemento*)memento
{
	[self init];
	[self setMemento:memento];
	[[fMontageDoc snapperDB] refreshSnapDB:self];
	return self;
}

- (MIObject*)findObjectRecursively:(TArmeWSObj*)wsObj
{
	if ([self WSObject] == wsObj)
		return self;
		
	NSEnumerator* childEnum = [[self getChildren] objectEnumerator];
	MIObject* child;
	while ( child = [childEnum nextObject] )
	{
		MIObject* result = [child findObjectRecursively:wsObj];
		if (result)
			return result;
	}
	
	return nil;
}

int SortChildrenArrayByVPos(id child1, id child2, void *context)
{
    MIVPos vpos1 = [child1 getVerticalPos];
    MIVPos vpos2 = [child2 getVerticalPos];
    if (GET_VALUE(vpos1) < GET_VALUE(vpos2))
        return NSOrderedAscending;
    else if (GET_VALUE(vpos1) > GET_VALUE(vpos2))
        return NSOrderedDescending;
    else
        return NSOrderedSame;
}

- (NSArray*)getVideoChildrenAtTime:(ArmeTime)time
{
	NSMutableArray* result = [[[NSMutableArray alloc] initWithCapacity:1] autorelease];
	NSEnumerator* childEnum = [[self getChildren] objectEnumerator];
	MIObject* child;
	while (child = [childEnum nextObject])
	{
		if (([child WSObject]->GetType() == TArmeWSObj::kVideoClip) && ([child WSObject]->ContainsVideoDuring(time, time, true)))
			[result addObject:child];
	} 

	//sort the object according to their VPOS
	[result sortUsingFunction:SortChildrenArrayByVPos context:NULL];

	[result retain];
	return result;
}
- (NSArray*)getAudioChildrenAtTime:(ArmeTime)time
{
	NSMutableArray* result = [[[NSMutableArray alloc] initWithCapacity:1] autorelease];
	NSEnumerator* childEnum = [[self getChildren] objectEnumerator];
	MIObject* child;
	while (child = [childEnum nextObject])
	{
		if (([child WSObject]->GetType() == TArmeWSObj::kAudioClip) && ([child WSObject]->ContainsAudioDuring(time, time, true)))
			[result addObject:child];
	} 

	//sort the object according to their VPOS
	[result sortUsingFunction:SortChildrenArrayByVPos context:NULL];

	[result retain];
	return result;
}
- (NSArray*)getVideoChildren
{
	NSMutableArray* result = [[[NSMutableArray alloc] initWithCapacity:1] autorelease];
	NSEnumerator* childEnum = [[self getChildren] objectEnumerator];
	MIObject* child;
	while (child = [childEnum nextObject])
	{
		if ([child WSObject]->GetType() == TArmeWSObj::kVideoClip)
			[result addObject:child];
	} 

	//sort the object according to their VPOS
	[result sortUsingFunction:SortChildrenArrayByVPos context:NULL];

	[result retain];
	return result;
}
- (NSArray*)getAudioChildren
{
	NSMutableArray* result = [[[NSMutableArray alloc] initWithCapacity:1] autorelease];
	NSEnumerator* childEnum = [[self getChildren] objectEnumerator];
	MIObject* child;
	while (child = [childEnum nextObject])
	{
		if ([child WSObject]->GetType() == TArmeWSObj::kAudioClip)
			[result addObject:child];
	} 

	//sort the object according to their VPOS
	[result sortUsingFunction:SortChildrenArrayByVPos context:NULL];

	[result retain];
	return result;
}

@end

@implementation MIObjectMemento

- (void)dealloc
{
	delete fWSObjectMemento;
	
	[fFrozenObject release];
	[fFreezeSource release];

	[super dealloc];
}

- (void)setVerticalPos:(MIVPos)verticalPos 
{
	fVerticalPos = verticalPos;
}

- (MIVPos)getVerticalPos
{
	return fVerticalPos;
}

- (void)setHeight:(MIVPos)height 
{
	fHeight = height;
}

- (MIVPos)getHeight
{
	return fHeight;
}

- (void)setOldHeight:(MIVPos)height
{
  fOldHeight = height;
}

- (MIVPos)getOldHeight
{
  return fOldHeight;
}

- (BOOL)isMuted
{
	return fIsMuted;
}

- (void)setMuted:(BOOL)flag
{
	fIsMuted = flag;
}

- (BOOL)isCollapsed
{
	return fIsCollapsed;
}

- (void)setIsCollapsed:(BOOL)flag
{
	fIsCollapsed = flag;
}

- (void)setWSObjMemento:(TArmeWSObjMemento*)memento
{
	fWSObjectMemento = memento;
}

- (TArmeWSObjMemento*)getWSObjMemento
{
	return fWSObjectMemento;
}

- (MIMontageDocument*)montageDoc
{
	return fMontageDoc;
}

- (void)setMontageDoc:(MIMontageDocument*)doc
{
	fMontageDoc = doc;
}

- (id)initFromArchive:(ArmeArchiveHandler*)unarchiver
{
	self = [super initFromArchive:unarchiver];
	if(self)
	{
		ArmeArchivable* archObj;
		archObj = unarchiver->ReadCPPObject();
		fWSObjectMemento = dynamic_cast<TArmeWSObjMemento*>(archObj);
		ZzAssert(fWSObjectMemento);

		GET_VALUE(fVerticalPos) = unarchiver->ReadSInt32();
		GET_VALUE(fHeight) = unarchiver->ReadSInt32();
    if ( unarchiver->GetFileVer() >= 0x00010034 )
    {
      GET_VALUE(fOldHeight) = unarchiver->ReadSInt32();
    }

		fMontageDoc = unarchiver->ReadObjCObject();
		fFreezeSource = [unarchiver->ReadObjCObject() retain];
		fFrozenObject = [unarchiver->ReadObjCObject() retain];
		fContentHasBeenModifiedSinceLastFreezeOperation = unarchiver->ReadBOOL();
	}
	return self;
}

- (BOOL)writeToArchive:(ArmeArchiveHandler*)archiver
{
	archiver->WriteCPPObject(fWSObjectMemento);
	
	archiver->WriteSInt32(GET_VALUE(fVerticalPos));
	archiver->WriteSInt32(GET_VALUE(fHeight));
  archiver->WriteSInt32(GET_VALUE(fOldHeight));
	archiver->WriteObjCObject(fMontageDoc);
	archiver->WriteObjCObject(fFreezeSource);
	archiver->WriteObjCObject(fFrozenObject);
	archiver->WriteBOOL(fContentHasBeenModifiedSinceLastFreezeOperation);
	return YES;
}
@end
