/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
 #import <Cocoa/Cocoa.h>


/*!
    @header MIObjectRep.h
    
    defines the MIObjectRep class
*/
@class MIObject;
@class MITopLevelContainerRep;
@class MIMouseTracker;
@class MIActiveCursor;

#define TEST_BENJA_ALLOW_CURSOR_RECTS_DISABLING

class TArmeDoc;

#ifdef SMALL_TITLEBAR
extern const MIVPos	kDefaultClipRepresentationHeight;
extern const MIVPos	kDefaultGroupRepresentationHeight;
#else
extern const int	kDefaultClipRepresentationHeight;
extern const int	kDefaultGroupRepresentationHeight;
#endif
extern bool			gMovingTitle;
extern const int	kRepDragZoneSize;

/*! 
@enum MIObjectMouseHandling
    this enumeration is used by the top level container to see if this object has done
    something with the mouse event.
@constant MIContinueMouseHandling the object did nothing with the event.

@constant MIStopMouseHandling the object has done something with the mouse event and
this should stop the checks about the event (example: the event was on a button)

@constant MIAskMeATracker the event is something that needs more processing than just one
click and a tracker has to be created to process it. The TLC should then ask the object
for a tracker using the "getTheMouseTracker" function
*/ 

enum MIObjectMouseHandling
{
    MIContinueMouseHandling = 0,
    MIStopMouseHandling = 1,
    MIAskMeATracker = 2
};

// used in the rectForZone function to get the rectangles for the different interactive parts
// of the object rep
enum MIObjectRepZone
{
	MIObjectRepResizeUpZone,
	MIObjectRepResizeUpLeftZone,
	MIObjectRepResizeLeftZone,
	MIObjectRepResizeDownLeftZone,
	MIObjectRepResizeDownZone,
	MIObjectRepResizeDownRightZone,
	MIObjectRepResizeRightZone,
	MIObjectRepResizeUpRightZone,
	MIObjectRepDragBarZone,
	MIObjectRepTitleZone,
	MIObjectRepAutomationZone,
	MIObjectRepBodyZone,
	MIObjectRepCollapseButtonZone,
	MIObjectRepPropertiesButtonZone,
	MIObjectRepOnOffButtonZone,
	MIObjectRepBodySelectionZone,
	MIObjectRepRightBodySelectionZone,
	MIObjectRepLeftBodySelectionZone,
	MIObjectRepRightTimeStretchZone,
	MIObjectRepLeftTimeStretchZone,
};

/*!
    @class MIObjectRep
    @abstract this class is responsible for drawing a MIObject
    @discussion
    the sole purpose of this class is to provide a graphical representation for the MIObject it is
    linked to.
    there is a 1 to many correspondance between a MIObject and MIObjectRep(resentations).
    Actually, there is (or should be) exactly one representation per window opened who has the MIObject
    in it.
    each MIObjectRep holds two NSImages, one for the body, one for the title in order to speed up
    (tremendously, may I add) the rendering times 
*/

@class MIGuide;

@interface MIObjectRep : NSObject
{
    /*! @var fBackgroundColor holds the general background color */
	NSColor					*fBackgroundColor;
    
    /*! @var fRepresentedObject pointer to the object this MIObjectRep represents */
	MIObject				*fRepresentedObject;
	
    /*! @var fTLCRep pointer to the Top level container view */
    MITopLevelContainerRep	*fTLCRep;
    
    /*! @var fFrame the frame of the representation (a change to this does not change the media engine data) */
	NSRect					fFrame;
	
	/*! @var fSliceSelectionRect This is just a temporary rect for optimization (the slice selection rectangle is computed in the drawBaseBodyRect
		while it is displayed once the children have been drawned.  To avoid some cliping rect re-computation, we store it in this variable*/
	NSRect					fSliceSelectionRect;
	      
	/*! @var fHasTopBottomHandles
		determines whether the object has top and bottom resize handles, taken into account by the basic drawing
		and mouse handling functions */
	BOOL					fHasTopBottomHandles;
	
	/*! @var fHasLeftRightHandles
		determines whether the object has left and right resize handles, taken into account by the basic drawing
		and mouse handling functions */
	BOOL					fHasLeftRightHandles;

	/*! @var fHasCollapseButton
		determines whether the object has a collapse button, taken into account by the basic drawing
		and mouse handling functions */
	BOOL					fHasCollapseButton;
	
	/*! @var fHasPropertiesButton
		determines whether the object has a properties button, taken into account by the basic drawing
		and mouse handling functions (if the button is hit, the showProperties method is called) */
	BOOL					fHasPropertiesButton;
	
	/*! @var fHasOnOffButton
		determines whether the object has a on/off button, taken into account by the basic drawing
		and mouse handling functions (if the button is hit, the represented object's toggleOnOff function is called) */
	BOOL					fHasOnOffButton;

	/*! @var fDrawVirtualOutputs
		unused */
	BOOL 					fDrawVirtualOutputs;

	NSImage					*fLeftTitleImage;
	NSImage					*fCenterTitleImage;
	NSImage					*fRightTitleImage;
	
	/*! @var fLeftBarImage the left bar image for this representation */
//	NSImage					*fLeftBarImage;

	/*! @var fCenterBarImage the left bar image for this representation */
//	NSImage					*fCenterBarImage;

	/*! @var fRightBarImage the left bar image for this representation */
//	NSImage					*fRightBarImage;
    
    /*! @var fCachedTitleImage the cached title image */
    NSImage					*fCachedTitleImage;

    /*! @var fCachedBodyImage the cached body image */
    NSImage					*fCachedBodyImage;
	
	NSImage					*fCachedAutomationImage;

	/*! @var fCachedTitleStringImage the cached title string image */
	NSImage					*fCachedTitleStringImage;

	NSRect					fLastTitleImageRect;

	/*! @var fCachedLeftTitleImage the cached left part of the title */	
//	NSImage					*fCachedLeftTitleImage;
	
	/*! @var fCachedRightTitleImage the cached left part of the title */	
//	NSImage					*fCachedRightTitleImage;

	/*! @var fCachedStretchableTitleImage a 1 pixel wide image to stretch to obtain the title */
//	NSImage					*fCachedStretchableTitleImage;
	
	/*! @var fTemporaryTracker a mouseTracker that has been created and that will be returned if asked by whoever. Will dissipate with the runloop */
	id						fTemporaryTracker;
	
	/*! @var fSelected The selection state of the current object */
	BOOL					fSelected;

	/*! @var fIsInvalid YES : the represented object has changed, the representation needs to be regenerated*/
	bool					fIsInvalid;

@public
	NSMutableArray 			*fTrackingRectTagArray;
	NSTrackingRectTag		fTrackingRectTag;

@protected
	MIActiveCursor 			*fActiveCursor;
	NSTimer					*fTimer;
	
	// the following 2 are used to prevent unnecessary recaching of the object titles.
	BOOL					fLastLockedStatus; 
	NSString				*fLastTitle;
	BOOL					fLastSelectedStatus;

	BOOL					fMouseIsOverMe;
	
	BOOL					fIsBeingTimeStretched;
}

- (void)setImagesAndColor;

/*! @function invalidate Tells that represented object has changed, the representation needs to be regenerated*/
- (void)invalidate;

/*! @function setSelected Tells whether the object is currently selected or not */
- (void)setSelected:(BOOL)selected;

/*! @function initWithFrame the function is the prefered initializer of the class,
    it sets the frame and the TLC */
- (id)initWithFrame:(NSRect)frame inTLC:(MITopLevelContainerRep*)tlc;

/*! @function toggleOnOff toggle the ON/OFF state of the representation */
- (void)toggleOnOff;

/*! @function setBackgroundColor sets the global background color for this representation
    it is usually not needed as all representation of a given kind share the same background color
    */
- (void)setBackgroundColor:(NSColor*)color;

/*! @function backgroundColor returns the background color for this object */
- (NSColor*)backgroundColor;

/*! @function setRepresentedObject sets the MIObject this MIObjectRep represents */ 
- (void)setRepresentedObject:(MIObject*)object;

/*! @function representedObject returns the MIObject this MIObjectRep represents */
- (MIObject*)representedObject;

/*! @function setTopLevelContainerRep sets the top level container in which the representation is */
- (void)setTopLevelContainerRep:(MITopLevelContainerRep*)newTLC;

/*! @function setNeedsDisplay sets the top level container in which the representation is to redisplay
	the object representation, including the shadow */
- (void)setNeedsDisplay;

/*! @function setDataNeedsDisplay sets the top level container in which the representation is to redisplay
	the object representation, without the shadow */
- (void)setDataNeedsDisplay;

/*! @function getTopLevelContainerRep returns the top level container in which the representation is */ 
- (MITopLevelContainerRep*)getTopLevelContainerRep;

/*! @function getTLCFrame this function returns the representation's frame in the TLC coordinate system */
- (NSRect)getTLCFrame;

/*! @function frame returns the representation's frame */ 
- (NSRect)frame;

- (NSRect)frameIncludingShadow;

- (void)setTempFrameWithoutRegeneratingImages:(NSRect)frame;

/*! @function setFrame sets the representation's frame.
    This function does NOT change the media engine data in any way*/
- (void)setFrame:(NSRect)aFrame;

/*! @function setFrameOrigin sets the representation's frame origin */
- (void)setFrameOrigin:(NSPoint)frameOrigin;

/*! @function setFrameSize sets the representation's frame size */
- (void)setFrameSize:(NSSize)frameSize;

- (NSRect)getBodyRect;
- (NSRect)getTitleRect;

- (NSRect)minimalRectToFitContent;

/*! @function recreateFrame this function takes the data from the MIObject and its associated TArmeWSObj
    and recreates the representation frame, taking into account the zoom and the collapsed state */
- (void)recreateFrame;

/*! @function removeCursorRects removes the TLC's cursor rectangles for this object */
- (void)removeCursorRects;

/*! @function addCursorRects adds the TLC's cursor rectangles for this object */
- (void)addCursorRects;

/*! @function drawRect this functions draws the representation at point offset, displaying only
    the parts of the representation that are inside aRect */
- (void)drawRect:(NSRect)aRect withOffset:(NSPoint)offset;


- (void)drawSoloStateInRect:(NSRect)rect;
- (void)drawMutedStateInRect:(NSRect)rect lineLength:(unsigned)lineLength;

/*! @function drawShadowClippedTo draws the shadow around the object clipped to the clipping rectangle. The shadow
    changes if the object is selected or not */
- (void)drawShadowClippedTo:(NSRect)clippingRect withObjectOrigin:(NSPoint)objOrigin;

/*! @function refreshCachedBodyImage refreshes the cached body image. The extra parameter specifies whether the image is to be created if it doesn't exist yet*/
- (void)refreshCachedBodyImage:(BOOL)createIfAbsent;

/*! @function refreshCachedTitleImage refreshes the cached title image. The extra parameter specifies whether the image is to be created if it doesn't exist yet */
- (void)refreshCachedTitleImage:(BOOL)createIfAbsent;

- (void)refreshCachedAutomationImage:(BOOL)createIfAbsent;

- (void)refreshCachedTitleStringImage;

- (void)drawCachedAutomationImageToPoint:(NSPoint)aPoint fromRect:(NSRect)rect;

/*! @function drawCachedBodyImageToPoint
    this function displays the part of the cached body image inside 'rect' (in the image coordinate system)
    at point aPoint */
- (void)drawCachedBodyImageToPoint:(NSPoint)aPoint fromRect:(NSRect)rect;

/*! @function drawCachedTitleImageToPoint
    this function displays the part of the cached title image inside 'rect' (in the image coordinate system)
    at point aPoint */
- (void)drawCachedTitleImageToPoint:(NSPoint)aPoint fromRect:(NSRect)rect;

/*! 
	@function drawBaseBodyRect
    this function draws the base content of the body to whatever destination (screen, image, printer, ...).
	frameInDrawCoords is the frame of the body in draw coords, but clipped to clippingRect (also in draw coords)
*/
- (void)drawBaseAutomationRect:(NSRect)zoneInDrawCoords clippedTo:(NSRect)clippingRect;
- (void)drawBaseBodyRect:(NSRect)frameInDrawCoords clippedTo:(NSRect)clippingRect;

- (void)drawSelection:(NSRect)clippingRect drawNow:(BOOL)drawItNow;

- (void)drawTitleToPoint:(NSPoint)aPoint clippedTo:(NSRect)clippingRect;

- (void)drawScrollingTitleWithTitleRect:(NSRect)titleRect;

/*! @function drawBaseTitleToPoint
    this function draws the base content of the title to whatever destination (screen, image, printer, ...) at point
	aPoint and clips the drawing to rect
	*/
//- (void)drawBaseTitleToPoint:(NSPoint)aPoint fromRect:(NSRect)rect;

/*! @function drawClippedImage
	this function draws an image at point aPoint, clipped by clippingRect.
	returns the value of aPoint plus the width of the point, useful for drawing multiple images sequentially */
- (NSPoint)drawClippedImage:(NSImage*)img atPoint:(NSPoint)aPoint clippingRect:(NSRect)clippingRect;

/*! @function convertPointFromParent converts aPoint from the parent's coordinate system to the representation's */
- (NSPoint)convertPointFromParent:(NSPoint)aPoint;

/*! @function convertRectFromParent converts aRect from the parent's coordinate system to the representation's */ 
- (NSRect)convertRectFromParent:(NSRect)aRect;

/*! @function convertPointFromTLCCoords converts aPoint from the TLC's coordinate system to the local coordinate system */
- (NSPoint)convertPointFromTLCCoords:(NSPoint)aPoint;

/*! @function convertRectFromTLCCoords converts aRect from the TLC's coordinate system to the local coordinate system */
- (NSRect)convertRectFromTLCCoords:(NSRect)aRect;

/*! @function convertPointToParentCoords converts aPoint to the parent coordinate system */
- (NSPoint)convertPointToParentCoords:(NSPoint)aPoint;

/*! @function convertRectToParentCoords converts aRect to the parent coordinate system */
- (NSRect)convertRectToParentCoords:(NSRect)aRect;

/*! @function convertPointToTLCCoords converts aPoint from the local coordinate system to that of the Top level container */
- (NSPoint)convertPointToTLCCoords:(NSPoint)aPoint;

/*! @function convertRectToTLCCoords converts aRect from the local coordinate system to that of the Top level container */
- (NSRect)convertRectToTLCCoords:(NSRect)aRect;

- (void)popUpMenu:(NSMenu*)menu atPos:(NSPoint)pos;

- (NSRect)rectForZone:(MIObjectRepZone)zone withObjectOrigin:(NSPoint)origin;
// same as withObjectOrigin with origin being NSZeroPoint
- (NSRect)rectForZone:(MIObjectRepZone)zone;

/*! @function handleMoreMouseTracking checks if the receiver does an action with the mouse event.
    mouseLoc is in the local coordinate system */
- (MIObjectMouseHandling)handleMoreMouseTracking:(NSPoint)mouseLoc modifier:(unsigned int)modifier type:(NSEventType)type;

/*! @function doubleClick checks if the receiver does an action with the mouse event.
    mouseLoc is in the local coordinate system */
- (BOOL)doubleClick:(NSPoint)mouseLoc modifier:(unsigned int)modifier;

/*! @function getTheMouseTracker if the representation answered 'MIAskMeAMouseTracker' to the handleMoreMouseTracking
    function, this function gets called and the receiver can create an appropriate mouse tracker to handle mouse
    movement */
- (MIMouseTracker*)getTheMouseTracker:(NSPoint)mouseLoc;

/*! @function invalidateTheMouseTracker if the representation answered 'MIAskMeAMouseTracker' to the handleMoreMouseTracking:
    method, and after using the tracker returned by the getTheMouseTracker: method, this method should be called when the tracker is no longer needed.*/
- (void)invalidateTheMouseTracker;

/*! @function isSelected returns YES if the representation is selected */
- (BOOL)isSelected;

/*! @function resetFrameWithHZoom modifies the receiver's frame according to the given horizontal zoom.
    This function does NOT ask the receiver to be redrawn */ 
- (void)resetFrameWithHZoom:(float)hZoom;

/*! @function resetFrameWithVZoom modifies the receiver's frame according to the given vertical zoom
    This function does NOT ask the receiver to be redrawn */ 
- (void)resetFrameWithVZoom:(float)vZoom;

/*! @function makeSelfEncloseCurrentSelection modifies the receiver so that it becomes parent of the current selection
    without changing the current selection's global position (on the screen) returns TRUE in case of success*/
// DEPRECATED - (BOOL)makeSelfEncloseCurrentSelection;

/*! @function showProperties display something (inspector?) when the properties button is hit */
- (void)showProperties;

/*! This function is called when the data of an object has changed and, thus, the cached image is
    no longer valid. Please note a better $$$TODO implementation might provide an invalidation range */
- (void)invalidateCachedImage;

/*! @function cacheImagesIfSizeSmallerThan caches the images if they are inside a given rectangle and under a maximum size. Frees the existing image if the conditions are not met*/
- (void)cacheImagesIfSizeSmallerThan:(NSSize)maxSize frameIsInRect:(NSRect)enclosingRect;

/*! @function freeCachedImages free the cached images */
- (void)freeCachedImages;

/*! @function getDoc returns the attached doc */
- (TArmeDoc*)getDoc;

/*! @function createOrRefreshImage create an image or modify its size if needed */
- (NSImage*)createOrRefreshImage:(NSImage*)image ofSize:(NSSize)size;

- (void)resetActiveCursorRects;
- (void)addActiveCursors;
- (void)addActiveCursor:(MIActiveCursor*)activeCursor;
- (void)mouseEntered:(NSEvent *)theEvent;
- (void)trigger:(NSTimer*)timer;
- (void)mouseExited:(NSEvent *)theEvent;
- (void)refreshTrackingRect;


@end

