/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
 #import "MIOfflineAudioTimeStretchController.h"

@implementation MIOfflineAudioTimeStretchController

- (IBAction)clickedCancel:(id)sender
{
	fWasCancelled = YES;
	[[NSApplication sharedApplication] stopModal];
}

- (IBAction)clickedOK:(id)sender
{	
	fWasCancelled = NO;
	fNewDuration = _ArmeTime([newNbSamples intValue]); 
	[[NSApplication sharedApplication] stopModal];
}

- (IBAction)selectAlgo:(id)sender
{
   fAlgoType = [sender indexOfSelectedItem];
}

- (void)controlTextDidChange:(NSNotification*)aNotification
{
	if( fTextChangeNotificationsDisabled)
		return;
	id obj = [aNotification object];

/*	
	#define TEST_NOTIF_OBJ(a) \
		if(obj == a)\
		{\
			verntl(offlineProcBenjamin,5,"notification from " << #a);\
		}
		
	TEST_NOTIF_OBJ(newDurationFFTextField)
	TEST_NOTIF_OBJ(newDurationHHTextField)
	TEST_NOTIF_OBJ(newDurationMMTextField)
	TEST_NOTIF_OBJ(newDurationMSTextField)
	TEST_NOTIF_OBJ(newDurationSSTextField)
	TEST_NOTIF_OBJ(newMusicalDurationBARS)
	TEST_NOTIF_OBJ(newMusicalDurationBEATS)
	TEST_NOTIF_OBJ(newMusicalDurationPPQ)
	TEST_NOTIF_OBJ(newMusicalDurationQ)
	TEST_NOTIF_OBJ(newNbSamples)
	TEST_NOTIF_OBJ(newPercentOfOrigDuration)
	TEST_NOTIF_OBJ(newSampleRateTextField)
	TEST_NOTIF_OBJ(newTempo)
*/
    double oldDur = (float)GET_VAL(fOrigDuration);

    if (obj == newPercentOfOrigDuration) 
        fNewDuration = _ArmeTime(SInt64([obj doubleValue]/100.0 * oldDur));

    if (obj == newNbSamples) 
        fNewDuration = _ArmeTime(SInt64([obj doubleValue]));
    
    
    [self updateView:obj];
}

- (void)awakeFromNib
{
	fWasCancelled = YES;
	fTextChangeNotificationsDisabled = NO;
	[newSampleRateTextField setDelegate:self];  
}

- (void)updateView:(id)notifier // update from fNewDuration
{
    double oldDur = (float)GET_VAL(fOrigDuration);
    double newDur = (float)GET_VAL(fNewDuration);
    
    if (newNbSamples != notifier) [newNbSamples setFloatValue:newDur];
    if (newPercentOfOrigDuration != notifier) [newPercentOfOrigDuration setDoubleValue:newDur/oldDur*100.0];
}

- (BOOL)wasCancelled
{
	return fWasCancelled;
}

- (ArmeTime)getDuration
{
	return fNewDuration;
}

- (int)getAlgoType
{
    return fAlgoType;
}

- (void)setAlgoType:(int)t
{
    fAlgoType = t;
}

- (void) setOrigDuration: (ArmeTime)dur
{
    fOrigDuration = dur;
    fNewDuration = dur;
    [origNbSamples setFloatValue:(float)GET_VAL(dur)];
    [self updateView:nil];
    
}

- (void)setDuration:(ArmeTime)dur
{
	vverntl(offlineProcTom,0,"please implement MIOfflineAudioTimeStretchController::setDuration:");
	ZzAssert(0);
}

@end
