/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
#import "MontageInterface.h"

#ifndef ArmeBasicTypes_h
#include "ArmeBasicTypes.h"
#endif //ArmeBasicTypes_h

#ifndef MontageArchiveHandler_h
#include "MontageArchiveHandler.h"
#endif //MontageArchiveHandler_h
#import "MIPaletteController.h"
#import "MIMontagePalette.h"

#define kNumberRecordedPositions 10

@interface MIOverviewSavedView : NSObject
{
	NSString			*fName;
	unsigned int		fVZoomLevel;
	unsigned int		fHZoomLevel;
	ArmeTime			fHOrigin;
	MIVPos				fVOrigin;
}

- (NSString*)name ;
- (void)setName:(NSString*)name ;

- (unsigned int)vZoomLevel ;
- (void)setVZoomLevel:(unsigned int)zoom ;

- (unsigned int)hZoomLevel ;
- (void)setHZoomLevel:(unsigned int)zoom ;

- (ArmeTime)hOrigin ;
- (void)setHOrigin:(ArmeTime)time ;

- (MIVPos)vOrigin ;
- (void)setVOrigin:(MIVPos)position ;

- (BOOL)writeToArchive:(ArmeArchiveHandler*)archiver;
- (id)initFromArchive:(ArmeArchiveHandler*)unarchiver;

@end

@class MIApplicationController;
@class MITopLevelContainerView;

@interface MIOverviewController : MIMontagePalette
{
@public
    IBOutlet id 				fPopupButton;
    IBOutlet id 				fSaveViewButton;
    IBOutlet id 				fView;
    IBOutlet id 				fNamePanel;
    IBOutlet id 				fNameField;
    IBOutlet id 				fEditPanel;
    IBOutlet id 				fRenameButton;
    IBOutlet id 				fDeleteButton;
    IBOutlet id 				fTableView;
	IBOutlet id					fHZoomSliderView;
	IBOutlet id					fVZoomSliderView;
	IBOutlet id					fHZoomSlider;
	IBOutlet id					fVZoomSlider;
	IBOutlet id					fHZoomButton;
	IBOutlet id					fVZoomButton;
	IBOutlet id					fHZoomText;
	IBOutlet id					fVZoomText;
	
    int							fEditedRow;
	int							fFutureHZoomIndex;
	int							fFutureVZoomIndex;
    NSMutableArray				*fEditedCbbDataSource;
}

+ (id)sharedOverviewController;
- (id)init;

- (void)myBoundsChanged:(NSNotification*)theNotification;
- (void)refreshCachedImage;
- (void)setViewNeedsDisplay;


- (void)setFutureHZoomIndex:(int)newValue;
- (void)setFutureVZoomIndex:(int)newValue;
- (int)futureHZoomIndex;
- (int)futureVZoomIndex;

- (void)reflectScrolling:(MITopLevelContainerView*)theView;

    // update of the Popup Button
- (void)updatePopupButton ;
- (void)buildPopupButton ;

    // Action called when selecting an item in the popup button
- (IBAction)savedViewSelected:(id)sender;

    // methods for managing the name panel
- (IBAction)saveView:(id)sender;
- (void)sheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void  *)contextInfo ;

    // Actions for the buttons of the name panel
- (IBAction)saveViewName:(id)sender ;
- (IBAction)cancelViewName:(id)sender ;

    // Gestion du panel d'dition
- (IBAction)editView:(id)sender ;
- (void)editSheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void  *)contextInfo ;

    // Actions for the buttons of the Edit Panel
- (IBAction)renameSelection:(id)sender ;
- (IBAction)deleteSelection:(id)sender ;
- (IBAction)saveEdit:(id)sender ;
- (IBAction)cancelEdit:(id)sender ;

- (IBAction)vSliderMoved:(id)sender;
- (IBAction)hSliderMoved:(id)sender;
- (IBAction)vZoomButtonPressed:(id)sender;
- (IBAction)hZoomButtonPressed:(id)sender;


    // methods for managing the NSTableView of the Edit Panel
- (int)numberOfRowsInTableView:(NSTableView *)aTableView ;
- (id)tableView:(NSTableView *)aTableView objectValueForTableColumn:(NSTableColumn *)aTableColumn row:(int)rowIndex ;
- (void)tableView:(NSTableView *)aTableView setObjectValue:(id)anObject forTableColumn:(NSTableColumn *)aTableColumn row:(int)rowIndex ;
- (void)tableViewSelectionDidChange:(NSNotification *)aNotification ;
- (BOOL)tableView:(NSTableView *)aTableView shouldEditTableColumn:(NSTableColumn *)aTableColumn row:(int)rowIndex ;
- (BOOL)selectionShouldChangeInTableView:(NSTableView *)aTableView ;


@end

@interface MIHSlider : NSSlider
{
}
@end
@interface MIVSlider : NSSlider
{
}
@end



