/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MIApplication.h"
#import "MIPaletteController.h"
#import "MIMixerController.h"
#import "MIViewerController.h"
#import "MIOverviewController.h"
#import "MITimePaletteController.h"
#import "MIPlayStopPauseController.h"
//#import "MIToolBoxController.h"
#import "MIInspectorController.h"
#import "MIPerfMeter.h"

#ifndef LibraryToggle_h
#include "LibraryToggle.h"
#endif //LibraryToggle_h

#if USE_LIBRARY
#import "MILibraryController.h"
#endif //USE_LIBRARY

NSString *MTGFirstLaunch = @"First Launch";

@implementation MIPaletteController


+ (MIPaletteController*)sharedPaletteController {
    static MIPaletteController* sharedPaletteController = nil;
    if ( sharedPaletteController == nil ) {
        sharedPaletteController = [[self alloc] init];
		[sharedPaletteController registerPalettes];
    }
    return sharedPaletteController;
}

- (MIPaletteController*)init {
    if (( self = [super init] )) {
		NSString *firstLaunch;
        fDontShowPalettes = NO;
        fPalettes = [[NSMutableArray alloc] initWithCapacity:10];
        fHiddenPalettes = [[NSMutableArray alloc] initWithCapacity:10];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(applicationWillTerminate:) name:NSApplicationWillTerminateNotification object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(applicationDidChangeScreenParameters:) name:NSApplicationDidChangeScreenParametersNotification object:nil];
		firstLaunch = [[NSUserDefaults standardUserDefaults] objectForKey:MTGFirstLaunch];
		if ( !firstLaunch ) {
			fShouldRepositionAllPalettes = YES;
			/*NSString *str = @"";
			NSEnumerator* enumerator = [[NSScreen screens] objectEnumerator];
			NSScreen *screen;

			while (( screen = [enumerator nextObject] )) {
				str = [str stringByAppendingFormat:@"%@ ", NSStringFromRect([screen frame])];
			}

			NSRunAlertPanel(@"Palette Debug", @"This is the first Launch : forcing all palettes to be re-organised. Screens = %@", @"OK", nil, nil, str);*/
		}
#if !defined(__MWERKS__)
		[NSApp registerKeyCommand:@"\t" modifiers:0 forTarget:self selector:@selector(TabKeyCommand:withModifiers:)];
		[NSApp registerKeyCommand:@"\t" modifiers:NSAlternateKeyMask forTarget:self selector:@selector(optionTabKeyCommand:withModifiers:)];
#endif //#if !defined(__MWERKS__)
    }
    return self;
}

- (void)dealloc 
{
	[NSApp unregisterKeyCommandsForTarget:self];
    [fHiddenPalettes release];
	[fPalettes release];
    [[NSNotificationCenter defaultCenter] removeObserver:self name:NSApplicationWillTerminateNotification object:nil];
    [[NSNotificationCenter defaultCenter] removeObserver:self name:NSApplicationDidChangeScreenParametersNotification object:nil];
    [super dealloc];
}

- (void)saveUserDefaults {
	NSEnumerator *enumerator;
	MIMontagePalette *object;

	//NSRunAlertPanel(@"Palette Debug", @"All palettes are now writing their frames.", @"OK", nil, nil);
	
    [[NSUserDefaults standardUserDefaults] setObject:@"NO" forKey:MTGFirstLaunch];

	if ( [fHiddenPalettes count] == 0 ) [self hideAllPalettes:nil];
	
	enumerator = [fHiddenPalettes objectEnumerator];
	while (( object = [enumerator nextObject] )) {
		[object saveUserDefaults:YES];
		[fPalettes removeObject:object];
	}
	
	enumerator = [fPalettes objectEnumerator];
	while (( object = [enumerator nextObject] )) {
		[object saveUserDefaults:NO];
	}
}

- (void)applicationWillTerminate:(NSNotification*)theNotification
{
	vernl(6,"[MIPaletteController applicationWillTerminate]");
	[self saveUserDefaults];
    [self release];
}

- (void)applicationDidChangeScreenParameters:(NSNotification*)theNotification
{
    [self repositionAllPalettes:nil];
}

- (void)registerPalettes {
	[MIViewerController sharedViewerController];
	[MIPlayStopPauseController sharedPlayStopPauseController];
	[MIOverviewController sharedOverviewController];
	[MITimePaletteController sharedTimePaletteController];
	[MIInspectorController sharedInspectorController];
	[MIMixerController sharedMixerController];
	//[MIToolBoxController sharedToolBoxController];
	[MIPerfMeter sharedPerfMeterController];
#if USE_LIBRARY
	[MILibraryController sharedLibraryController];
#endif //USE_LIBRARY
}

- (void)registerPaletteController:(id)paletteController visibleAfterLaunch:(BOOL)visible {
    [fPalettes addObject:paletteController];
    if ( visible ) [fHiddenPalettes addObject:paletteController];
    [self checkPalettePosition:paletteController];
}

- (IBAction)hideAllPalettes:(id)sender {
    if ( [fHiddenPalettes count] == 0 ) {
        NSEnumerator *enumerator = [fPalettes objectEnumerator];
        id paletteController;

        while (( paletteController = [enumerator nextObject] )) {
            if ( [paletteController isVisible] ) {
                [paletteController hide:nil];
                [fHiddenPalettes addObject:paletteController];
            }
        }
    }
}

- (IBAction)showAllPalettes:(id)sender {
    if ( !fDontShowPalettes ) {
        if ( fShouldRepositionAllPalettes ) {
            [self repositionAllPalettes:nil];
            fShouldRepositionAllPalettes = NO;
        }
        [fHiddenPalettes makeObjectsPerformSelector:@selector(show:) withObject:self];
        [fHiddenPalettes removeAllObjects];
    }
}

- (IBAction)toggleShowHideAllPalettes:(id)sender {
    if ( [fHiddenPalettes count] == 0 ) [self hideAllPalettes:sender];
    else [self showAllPalettes:sender];
}

- (IBAction)repositionAllPalettes:(id)sender {
#define kInterSpace 4
	NSScreen *screen = [NSScreen mainScreen];
	NSRect screenFrame = [screen frame];
    NSRect visibleScreenFrame = [screen visibleFrame];
	NSRect previousWindowFrame = NSZeroRect;
	float minX = NSMaxX(visibleScreenFrame);

	//NSRunAlertPanel(@"Palette Debug", @"All palettes are now re-organised.", @"OK", nil, nil);

    NSEnumerator *enumerator = [fPalettes objectEnumerator];
    id paletteController;
	NSWindow* window;
	NSRect windowFrame;
	BOOL isVisible;
	NSRect newFrame;
	BOOL shouldCenterRemainingPalettes = NO;

    while (( paletteController = [enumerator nextObject] )) {
		window = (NSWindow*)[paletteController window];
		windowFrame = [window frame];
		isVisible = [window isVisible];
		newFrame.size.width = NSWidth( windowFrame );
		newFrame.size.height = NSHeight( windowFrame );
		if ( NSEqualRects( previousWindowFrame, NSZeroRect ) ) {
			newFrame.origin.y = NSMaxY( visibleScreenFrame ) - NSHeight( windowFrame ) - kInterSpace;
			newFrame.origin.x = minX - NSWidth( windowFrame ) - kInterSpace;
		} else {
			newFrame.origin.y = NSMinY( previousWindowFrame ) - NSHeight( windowFrame ) - kInterSpace;
			newFrame.origin.x = NSMaxX( previousWindowFrame ) - NSWidth( windowFrame );
			if ( !NSContainsRect(visibleScreenFrame, newFrame) ) {
				newFrame.origin.y = NSMaxY( visibleScreenFrame ) - NSHeight( windowFrame ) - kInterSpace;
				newFrame.origin.x = minX - NSWidth( windowFrame ) - kInterSpace;
			}
		}
		if ( ( ![self isFrameOnScreen:newFrame] ) || ( shouldCenterRemainingPalettes ) ) {
			// center on main screen
			newFrame = NSMakeRect( NSMidX(screenFrame) - NSWidth(newFrame) / 2, NSMidY(screenFrame) - NSHeight(newFrame) / 2, NSWidth(newFrame), NSHeight(newFrame) );
			shouldCenterRemainingPalettes = YES;
		}
		[window setFrame:newFrame display:isVisible animate:isVisible];
		previousWindowFrame = newFrame;
		if ( NSMinX( newFrame ) < minX ) minX = NSMinX( newFrame );
    }
}

- (BOOL)isFrameOnScreen:(NSRect)frame {
	NSEnumerator* enumerator = [[NSScreen screens] objectEnumerator];
    NSScreen *screen;
	NSBezierPath *screens = [NSBezierPath bezierPath];

    while (( screen = [enumerator nextObject] )) {
		[screens appendBezierPath:[NSBezierPath bezierPathWithRect:[screen visibleFrame]]];
    }

	if (
	 [screens containsPoint:NSMakePoint( NSMinX(frame), NSMinY(frame) )] &&
	 [screens containsPoint:NSMakePoint( NSMinX(frame), NSMaxY(frame) )] &&
	 [screens containsPoint:NSMakePoint( NSMaxX(frame), NSMinY(frame) )] &&
	 [screens containsPoint:NSMakePoint( NSMaxX(frame), NSMaxY(frame) )] )
		return YES;
	return NO;
}

- (void)checkPalettePosition:(id)paletteController {
	NSRect windowFrame;
	NSString *windowFrameString = [[NSUserDefaults standardUserDefaults] objectForKey:[NSString stringWithFormat:@"NSWindow Frame %@",[paletteController nibName]]];
	if ( windowFrameString ) windowFrame = NSRectFromString(windowFrameString);
	else windowFrame = [[paletteController window] frame];
	//NSLog(@"%@ frame : %@", paletteController, NSStringFromRect(windowFrame));
    NSRect leftCorner;
    NSRect rightCorner;
    NSEnumerator* enumerator = [[NSScreen screens] objectEnumerator];
    NSScreen *screen;
    BOOL isVisible = NO;

    leftCorner = NSMakeRect( NSMinX( windowFrame), NSMaxY( windowFrame ) - 10, 10, 10 );
    rightCorner = NSMakeRect( NSMaxX( windowFrame) - 10, NSMaxY( windowFrame ) - 10, 10, 10 );
    while (( screen = [enumerator nextObject] )) {
        if ( ( NSContainsRect([screen visibleFrame], leftCorner) ) || ( NSContainsRect([screen visibleFrame], rightCorner) ) )
            isVisible = YES;
    }

    if (!isVisible) {
		fShouldRepositionAllPalettes = YES;
		/*NSString *str = @"";
		enumerator = [[NSScreen screens] objectEnumerator];
		
		while (( screen = [enumerator nextObject] )) {
			str = [str stringByAppendingFormat:@"%@ ", NSStringFromRect([screen frame])];
		}
		
		NSLog(@"palette %@ has forced all palettes to be re-arranged. Frame = %@, Screens = %@",paletteController, NSStringFromRect(windowFrame), str);

		NSRunAlertPanel(@"Palette Debug", @"palette %@ has forced all palettes to be re-arranged. Frame = %@, Screens = %@", @"OK", nil, nil, paletteController, NSStringFromRect(windowFrame), str);*/
	}
}

- (void)setDontShowPalettes:(BOOL)flag
{
    fDontShowPalettes = flag;
}

- (void)TabKeyCommand:(NSString*)key withModifiers:(unsigned int)modifiers {
	[self toggleShowHideAllPalettes:nil];
}

- (void)optionTabKeyCommand:(NSString*)key withModifiers:(unsigned int)modifiers {
	[self showAllPalettes:nil];
	[self repositionAllPalettes:nil];
}

@end

