/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
#import "MIPaletteLayoutManager.h"

@implementation MIPaletteLayoutManager

- (id)init
{
	int i;

	self = [super init];
	
	for ( i = 0 ; i < kNumPanels; i++)
	{
		fPanels[i] = nil;
		for ( int j = 0; j < 4; j++ )
			fPanelSticksTo[i][j] = MINoPanel;
	}
	
	return self;
}

- (void)setPanel:(NSWindow*)panelWindow ofType:(MIPanelType)type
{
	fPanels[type] = panelWindow;
}

- (void)mainWindowMoved:(NSNotification*)notification
{
	[self makePanelsStickyFrom:MIMainWindowPanel];	
	[self checkPanelsStickyness];
}

- (void)mainWindowResized:(NSNotification*)notification
{
	[self makePanelsStickyFrom:MIMainWindowPanel];	
	[self checkPanelsStickyness];
}

- (void)panelMoved:(NSNotification*)notification
{
	for ( int i = 0 ; i < kNumPanels ; i++ )
	{
		if ( fPanels[i] && fPanels[i] == [notification object] )
		{
			[self makePanelsStickyFrom:(MIPanelType)i];
		}
	}
	[self checkPanelsStickyness];
	
//	[self makePanelsStickyFrom:MIMainWindowPanel];	
}

- (void)panelResized:(NSNotification*)notification
{
	for ( int i = 0 ; i < kNumPanels ; i++ )
	{
		if ( fPanels[i] && fPanels[i] == [notification object] )
		{
			[self makePanelsStickyFrom:(MIPanelType)i];
		}
	}
	[self checkPanelsStickyness];
//	[self makePanelsStickyFrom:MIMainWindowPanel];	
}

- (void)reinitNotifications
{
#ifdef MONTAGE_SNAPPING_PANELS
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	
	// move notification
	[[NSNotificationCenter defaultCenter]	addObserver:self
											selector:@selector(mainWindowMoved:)
											name:NSWindowDidMoveNotification
											object:fMainWindow];
	// resize notification
	[[NSNotificationCenter defaultCenter]	addObserver:self
											selector:@selector(mainWindowResized:)
											name:NSWindowDidResizeNotification
											object:fMainWindow];
	
	for ( int i = 0 ; i < kNumPanels ; i++ )
	{
		if ( fPanels[i] )
		{
			// move notification
			[[NSNotificationCenter defaultCenter]	addObserver:self
													selector:@selector(panelMoved:)
													name:NSWindowDidMoveNotification
													object:fPanels[i]];
			// resize notification
			[[NSNotificationCenter defaultCenter]	addObserver:self
													selector:@selector(panelResized:)
													name:NSWindowDidResizeNotification
													object:fPanels[i]];
			// NSWindowDidMiniaturizeNotification
			// NSWindowWillCloseNotification
			// NSWindowDidChangeScreenNotification
		}
	}
#endif
}

- (void)makePanelsStickyFrom:(MIPanelType)panel
{
	NSRect	panelFrame;
	int 	edge;
	
	if ( panel < kNumPanels )
		panelFrame = [fPanels[panel] frame];
	else if ( panel == MIMainWindowPanel )
	{
		panelFrame = [fMainWindow frame];
	}
	
	if ( panel == MINoPanel )
	{
	}
	else
	{
		NSRect	oldPanelFrames[kNumPanels];
		NSRect	newPanelFrames[kNumPanels];
		
		for ( int nPanel = 0 ; nPanel < kNumPanels ; nPanel++ )
		{
			oldPanelFrames[nPanel] = [fPanels[nPanel] frame];
			newPanelFrames[nPanel] = oldPanelFrames[nPanel];
		}
				
		for ( int nPanel = 0 ; nPanel < kNumPanels ; nPanel++ )
		{
			if ( nPanel != panel)
			{
				NSRect	newPanelFrame;
				newPanelFrame = oldPanelFrames[nPanel];
				
				for ( edge = 0 ; edge < 4 ; edge++ )
				{
					if ( fPanelSticksTo[nPanel][edge] == panel )
					{
						switch ( edge )
						{
							case NSMinXEdge:
								if ( NSMinX(newPanelFrame) != NSMaxX(panelFrame))
								{
									newPanelFrame.origin.x = NSMaxX(panelFrame);
//									newPanelFrame.origin.x += kSpaceBetweenPanels;
								}
								break;
							case NSMaxXEdge:
								if ( NSMaxX(newPanelFrame) != NSMinX(panelFrame) )
								{
									newPanelFrame.origin.x = NSMinX(panelFrame) - NSWidth(newPanelFrame);
//									newPanelFrame.origin.x += kSpaceBetweenPanels;
								}
								break;
							case NSMinYEdge:
								if ( NSMinY(newPanelFrame) != NSMaxY(panelFrame) )
								{
									newPanelFrame.origin.y = NSMaxY(panelFrame );
//									newPanelFrame.origin.x += kSpaceBetweenPanels;
								}
								break;
							case NSMaxYEdge:
								if ( NSMaxY(newPanelFrame) != NSMinY(panelFrame) )
								{
									newPanelFrame.origin.y = NSMinY(panelFrame) - NSHeight(newPanelFrame);
//									newPanelFrame.origin.x += kSpaceBetweenPanels;
								}
								break;
						}
					}					
				}
				newPanelFrames[nPanel] = newPanelFrame;
			} 
		}
		for ( int nPanel = 0 ; nPanel < kNumPanels ; nPanel++ )
		{
			if ( !NSEqualRects(newPanelFrames[nPanel], oldPanelFrames[nPanel]) )
			{
				// window screen visibleFrame
				if ( NSMaxX([[fMainWindow screen] visibleFrame]) < NSMaxX(newPanelFrames[nPanel]) )
				{
					newPanelFrames[nPanel].origin.x = NSMaxX([[fMainWindow screen] visibleFrame]) - NSWidth(newPanelFrames[nPanel]);
				}

				[fPanels[nPanel] setFrame:newPanelFrames[nPanel] display:YES];
			}
		}
	}
}

- (void)checkPanelsStickyness
{
	for ( int i = 0 ; i < kNumPanels ; i++ )
	{
		if ( fPanels[i] )
		{
			NSRect			panelFrame;
			NSRect			otherPanelFrame;
			
			panelFrame = [fPanels[i] frame];
			
			for ( int j = 0 ; j < 4 ; j++ )
				fPanelSticksTo[i][j] = MINoPanel;
			
			for ( int j = 0 ; j < kNumPanels ; j++ )
			{
				if ( fPanels[j] && i != j)
				{
					otherPanelFrame = [fPanels[j] frame];
					
					if ( NSIntersectsRect(NSInsetRect(panelFrame, -kStickyPanelsThreshold, -kStickyPanelsThreshold),
										  NSInsetRect(otherPanelFrame, -kStickyPanelsThreshold, -kStickyPanelsThreshold)) )
					{
						// check below
						if ( [self coord:NSMinX(panelFrame) sticksTo:NSMaxX(otherPanelFrame)] )
							fPanelSticksTo[i][NSMinXEdge] = (MIPanelType)j;
						
	/*					// check above
						if ( [self coord:NSMaxX(panelFrame) sticksTo:NSMinX(otherPanelFrame)] )
							fPanelSticksTo[i][NSMaxXEdge] = (MIPanelType)j;
	*/					
						// check left
						if ( [self coord:NSMinY(panelFrame) sticksTo:NSMaxY(otherPanelFrame)] )
							fPanelSticksTo[i][NSMinYEdge]  = (MIPanelType)j;
	/*					
						// check right
						if ( [self coord:NSMaxY(panelFrame) sticksTo:NSMinY(otherPanelFrame)] )
							fPanelSticksTo[i][NSMaxYEdge] = (MIPanelType)j;
*/					}
				}
			}
			
			otherPanelFrame = [fMainWindow frame];
			
			if ( [self coord:NSMinX(panelFrame) sticksTo:NSMaxX(otherPanelFrame)] )
				fPanelSticksTo[i][NSMinXEdge] = MIMainWindowPanel;

			if ( [self coord:NSMaxX(panelFrame) sticksTo:NSMinX(otherPanelFrame)] )
				fPanelSticksTo[i][NSMaxXEdge] = MIMainWindowPanel;
			
			if ( [self coord:NSMinY(panelFrame) sticksTo:NSMaxY(otherPanelFrame)] )
				fPanelSticksTo[i][NSMinYEdge] = MIMainWindowPanel;
			
			if ( [self coord:NSMaxY(panelFrame) sticksTo:NSMinY(otherPanelFrame)] )
				fPanelSticksTo[i][NSMaxYEdge] = MIMainWindowPanel;
		}
	}	
}

- (BOOL)coord:(float)coord sticksTo:(float)otherCoord
{
	if ( (coord > otherCoord - kStickyPanelsThreshold) &&
		 (coord < otherCoord + kStickyPanelsThreshold) )
		return YES;
	else
		return NO;
}
@end
