/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
#import "MIPlayStopPauseController.h"
#import "MIImageController.h"
#import "MIApplicationController.h"
#import "MIPaletteController.h"
#import "MIMainViewController.h"
#import "MITopLevelContainerRep.h"
#import "MIMainViewController.h"
#import "MIMontageDocument.h"
#import "MIDocumentController.h"
#import "MenuTags.h"
//#import "MIRecordPreviewController.h"

#ifndef TArmeDoc_h
#include "TArmeDoc.h"
#endif //TArmeDoc_h

@implementation MIPlayStopPauseController

+ (MIPlayStopPauseController*)sharedPlayStopPauseController
{
    static id sharedPlayStopPauseController;
    if ( sharedPlayStopPauseController == nil )
    {
        sharedPlayStopPauseController = [[self alloc] init];
    }
    return sharedPlayStopPauseController;
}

+ (BOOL)factoryVisibleDefault {
    return YES;
}

+ (NSString*)visibleDefaultName {
    return @"PlayStopPauseVisible";
}

- (NSString*)nibName {
    return @"PlayStopPause";
}

- (int)menuTag {
	return MTGTransportMenuTag;
}

- (NSString*)menuName {
	return @"Playback";
}


- (id)init
{
    if (( self = [super init] )) {
		[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(movieStoppedPlaying:) name:MTGMovieStoppedPlayingNotification object:nil];
		[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(movieStartedPlaying:) name:MTGMovieStartedPlayingNotification object:nil];
    }
    
    return self;
}

- (void)dealloc
{
	[[NSNotificationCenter defaultCenter] removeObserver:self name:MTGMovieStoppedPlayingNotification object:nil];
	[[NSNotificationCenter defaultCenter] removeObserver:self name:MTGMovieStartedPlayingNotification object:nil];
    [super dealloc];
}

- (void)movieStartedPlaying:(NSNotification*)notification {
	[playButton setState:NSOnState];
}

- (void)movieStoppedPlaying:(NSNotification*)notification {
	[playButton setState:NSOffState];
}


- (void)awakeFromNib
{
    [self setImages];
	
	fControllingCamera = NO;

    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(currentDocumentChanged:) name:MTGCurrentDocumentChangedNotification object:nil];
}

- (void)currentDocumentChanged:(NSNotification*)theNotification
{
	MIMontageDocument* mtgDoc = [[theNotification userInfo] objectForKey:MTGCurrentDocumentKey];
	TArmeDoc* doc = [mtgDoc armeDocument];
	if(doc->IsLooping())
	{
		[loopButton setState:NSOnState];
	}
	else
	{	
		[loopButton setState:NSOffState];
	}
}

- (void)setImages
{
    if(!mainImageController)
        return;
    [backButton setImage:mainImageController->fPlayBarButtonBackOff];
    [backButton setAlternateImage:mainImageController->fPlayBarButtonBackOn];
//    [cameraButton setImage:mainImageController->fPlayBarButtonCameraOff];
//    [cameraButton setAlternateImage:mainImageController->fPlayBarButtonCameraOn];
    [forwardButton setImage:mainImageController->fPlayBarButtonforwardOff];
    [forwardButton setAlternateImage:mainImageController->fPlayBarButtonforwardOn];
    [loopButton setImage:mainImageController->fPlayBarButtonLoopOff];
    [loopButton setAlternateImage:mainImageController->fPlayBarButtonLoopOn];
    [markButton setImage:mainImageController->fPlayBarButtonMarkOff];
    [markButton setAlternateImage:mainImageController->fPlayBarButtonMarkOn];
    [microButton setImage:mainImageController->fPlayBarButtonMicroOff];
    [microButton setAlternateImage:mainImageController->fPlayBarButtonMicroOn];
    [midiButton setImage:mainImageController->fPlayBarButtonMidiOff];
    [midiButton setAlternateImage:mainImageController->fPlayBarButtonMidiOn];
    [pauseButton setImage:mainImageController->fPlayBarButtonPauseOff];
    [pauseButton setAlternateImage:mainImageController->fPlayBarButtonPauseOn];
    [playButton setImage:mainImageController->fPlayBarButtonPlayOff];
    [playButton setAlternateImage:mainImageController->fPlayBarButtonPlayOn];
    [recordButton setImage:mainImageController->fPlayBarButtonRecordOff];
    [recordButton setAlternateImage:mainImageController->fPlayBarButtonRecordOn];
    [stopButton setImage:mainImageController->fPlayBarButtonStopOff];
    [stopButton setAlternateImage:mainImageController->fPlayBarButtonStopOn];
    [textButton setImage:mainImageController->fPlayBarButtonTextOff];
    [textButton setAlternateImage:mainImageController->fPlayBarButtonTextOn];

/*
    [backButton setImage:mainImageController->fPlayBarButtonBackOn];
    [backButton setAlternateImage:mainImageController->fPlayBarButtonBackOff];
    [cameraButton setImage:mainImageController->fPlayBarButtonCameraOn];
    [cameraButton setAlternateImage:mainImageController->fPlayBarButtonCameraOff];
    [forwardButton setImage:mainImageController->fPlayBarButtonforwardOn];
    [forwardButton setAlternateImage:mainImageController->fPlayBarButtonforwardOff];
    [loopButton setImage:mainImageController->fPlayBarButtonLoopOn];
    [loopButton setAlternateImage:mainImageController->fPlayBarButtonLoopOff];
    [markButton setImage:mainImageController->fPlayBarButtonMarkOn];
    [markButton setAlternateImage:mainImageController->fPlayBarButtonMarkOff];
    [microButton setImage:mainImageController->fPlayBarButtonMicroOn];
    [microButton setAlternateImage:mainImageController->fPlayBarButtonMicroOff];
    [midiButton setImage:mainImageController->fPlayBarButtonMidiOn];
    [midiButton setAlternateImage:mainImageController->fPlayBarButtonMidiOff];
    [pauseButton setImage:mainImageController->fPlayBarButtonPauseOn];
    [pauseButton setAlternateImage:mainImageController->fPlayBarButtonPauseOff];
    [playButton setImage:mainImageController->fPlayBarButtonPlayOn];
    [playButton setAlternateImage:mainImageController->fPlayBarButtonPlayOff];
    [recordButton setImage:mainImageController->fPlayBarButtonRecordOn];
    [recordButton setAlternateImage:mainImageController->fPlayBarButtonRecordOff];
    [stopButton setImage:mainImageController->fPlayBarButtonStopOn];
    [stopButton setAlternateImage:mainImageController->fPlayBarButtonStopOff];
    [textButton setImage:mainImageController->fPlayBarButtonTextOn];
    [textButton setAlternateImage:mainImageController->fPlayBarButtonTextOff];
    */
}

- (BOOL)canBecomeKeyWindow
{
    return NO;
}

- (BOOL)canBecomeMainWindow
{
    return NO;
}

- (IBAction)backButtonPressed:(id)sender
{
	//if ([[MIRecordPreviewController sharedRecordPreviewController] isDeviceControllable])
	/*if (fControllingCamera)
	{
		NSEvent* current = [NSApp currentEvent];
		unsigned int modifiers = [current modifierFlags];

		if ( modifiers & NSCommandKeyMask )
			[[MIRecordPreviewController sharedRecordPreviewController] deviceRewindPlay];
		else if ( modifiers & NSAlternateKeyMask)
			[[MIRecordPreviewController sharedRecordPreviewController] deviceSlowRewind];
		else if ( modifiers & NSControlKeyMask)
			[[MIRecordPreviewController sharedRecordPreviewController] devicePreviousFrame];
		else
			[[MIRecordPreviewController sharedRecordPreviewController] deviceRewind];
	}
	else*/
		[[MIApplicationController sharedApplicationController] gotoDocStart:sender];
}

- (IBAction)cameraButtonPressed:(id)sender
{
	fControllingCamera = NSOnState == [sender state];
	if (fControllingCamera /*and ![[MIRecordPreviewController sharedRecordPreviewController] isDeviceControllable]*/)
	{
		NSBeep();
		fControllingCamera = NO;
		[sender setState:NSOffState];
	}
	if (sender == cameraButton) {}
		/*[[MIRecordPreviewController sharedRecordPreviewController] toggleCameraControl:sender];*/
	else
		[cameraButton setState:(fControllingCamera ? NSOnState : NSOffState)];
}

- (IBAction)forwardButtonPressed:(id)sender
{
	//if ([[MIRecordPreviewController sharedRecordPreviewController] isDeviceControllable])
	/*if (fControllingCamera)
	{
		NSEvent* current = [NSApp currentEvent];
		unsigned int modifiers = [current modifierFlags];

		if ( modifiers & NSCommandKeyMask )
			[[MIRecordPreviewController sharedRecordPreviewController] deviceFastForwardPlay];
		else if ( modifiers & NSAlternateKeyMask)
			[[MIRecordPreviewController sharedRecordPreviewController] deviceSlowForward];
		else if ( modifiers & NSControlKeyMask)
			[[MIRecordPreviewController sharedRecordPreviewController] deviceNextFrame];
		else
			[[MIRecordPreviewController sharedRecordPreviewController] deviceFastForward];
	}
	else*/
		[[MIApplicationController sharedApplicationController] gotoDocEnd:sender];
}

- (IBAction)loopButtonPressed:(id)sender
{
	// $$$TODO make sure the initial state is the same for the engine
	// and for the playstoppause panel
	[[MIApplicationController sharedApplicationController] toggleLoop:sender];
}

- (void)updateLoopButton:(BOOL)flag {
	[loopButton setState:flag ? NSOnState : NSOffState];
}

- (IBAction)markButtonPressed:(id)sender
{
}

- (IBAction)microButtonPressed:(id)sender
{
}

- (IBAction)midiButtonPressed:(id)sender
{
}

- (IBAction)pauseButtonPressed:(id)sender
{
	//if ([[MIRecordPreviewController sharedRecordPreviewController] isDeviceControllable])
	if (fControllingCamera)
	{
		//[[MIRecordPreviewController sharedRecordPreviewController] devicePause];
	}
	else
		[[MIApplicationController sharedApplicationController] pausePlaying:sender];
}

- (IBAction)playButtonPressed:(id)sender
{
	//if ([[MIRecordPreviewController sharedRecordPreviewController] isDeviceControllable])
	if (fControllingCamera)
	{
		//[[MIRecordPreviewController sharedRecordPreviewController] devicePlay];
	}
	else
		[[MIApplicationController sharedApplicationController] startPlaying:sender];
}

- (IBAction)recordButtonPressed:(id)sender
{
	//if (fControllingCamera)
		//[[MIRecordPreviewController sharedRecordPreviewController] startRecording];
}

- (IBAction)stopButtonPressed:(id)sender
{
	//[[MIRecordPreviewController sharedRecordPreviewController] stopRecording];

	//if ([[MIRecordPreviewController sharedRecordPreviewController] isDeviceControllable])
	/*if (fControllingCamera)
	{
		if ([[MIRecordPreviewController sharedRecordPreviewController] isRecording])
			[[MIRecordPreviewController sharedRecordPreviewController] stopRecording];
		else
			[[MIRecordPreviewController sharedRecordPreviewController] deviceStop];
	}
	else*/
		[[MIApplicationController sharedApplicationController] stopPlaying:sender];
}

- (IBAction)textButtonPressed:(id)sender
{
}

- (BOOL)ignoreArrowKeys {
	return YES;
}

- (BOOL)ignoresUndoRedoActions {
	return YES;
}



@end
