/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MIPreferences.h"
#import "MIObjectRep.h"

#ifndef MEAudioDeviceDesc_h
#include "MEAudioDeviceDesc.h"
#endif //MEAudioDeviceDesc_h

#ifndef ArmeMemArchiveHandler_h
#include "ArmeMemArchiveHandler.h"
#endif //ArmeMemArchiveHandler_h

NSString *MIHighQualityHasChangedNotification = @"High Quality Has Changed Notification";
NSString *MISingleFieldHasChangedNotification = @"Single Field Has Changed Notification";

static MIPreferences* gPrefs = nil;

#ifdef SMALL_TITLEBAR
static MIVPos g_vPosSpacing[4] = {_MIVPos(GET_VALUE(kDefaultClipRepresentationHeight)/2),_MIVPos(GET_VALUE(kDefaultClipRepresentationHeight)),_MIVPos(GET_VALUE(kDefaultClipRepresentationHeight)*2),_MIVPos(100)};
#else
static SInt32 g_vPosSpacing[4] = {kDefaultClipRepresentationHeight/2,kDefaultClipRepresentationHeight,kDefaultClipRepresentationHeight*2,100};
#endif
@implementation MIPreferences

-(MIVPos)gridVSpacing
{
	MIVPos ret;
#ifdef SMALL_TITLEBAR
	ret = g_vPosSpacing[fGridVSpacing];
#else
	ret.data = g_vPosSpacing[fGridVSpacing];
#endif
	return ret;
}

+(MIPreferences*)sharedPrefs
{
	if(!gPrefs)
		gPrefs = [[MIPreferences alloc] init];
	return gPrefs;
}

+(BOOL)isGenerateThumbnailsWhenPlayingDoc 
{
	return ([[MIPreferences sharedPrefs] perfOptionsMask] & kPrefsPerfThumbsWhenPlaying) != 0;
}

+(BOOL)shadowsAroundObject
{
	return ([[MIPreferences sharedPrefs] perfOptionsMask] & kPrefsPerfObjectShadows) != 0;
}

+(BOOL)uniformBackground
{
	return ([[MIPreferences sharedPrefs] perfOptionsMask] & kPrefsPerfUniformBackground) != 0;
}

+(BOOL)movingTitles
{
	return ([[MIPreferences sharedPrefs] perfOptionsMask] & kPrefsPerfMovingTitles) != 0;
}

-(id)init
{
	self = [super init];
	
	fHyperEngineAVSerial = nil;
    fRayGunProSerial = nil;
    fRayGunSerial = nil;
    fHyperprismSilverSerial = nil;
    fHyperprismGoldSerial = nil;
	fDefaultTransitionEffectDesc = NULL;
	
	return self;
}

-(UInt32)snapOptionsMask
{
	return fSnapOptionsMask;
}
	
-(void)setSnapOptionsMask:(UInt32)mask
{
	fSnapOptionsMask = mask;
}

/*
-(UInt32)shiftSnapOptionsMask
{
	return fShiftSnapOptionsMask;
}
	
-(void)setShiftSnapOptionsMask:(UInt32)mask
{
	fShiftSnapOptionsMask = mask;
}
*/

-(UInt32)gridShowMask
{
	return fGridShowMask;
}

-(void)setGridShowMask:(UInt32)mask
{
	fGridShowMask = mask;
}

-(UInt32)gridVSpacingIndex
{
	return fGridVSpacing;
}
	
-(void)setGridVSpacing:(UInt32)spacing
{
	fGridVSpacing = spacing;
}

-(UInt32)perfOptionsMask
{
    return fPerfOptionsMask;
}

-(void)setPerfOptionsMask:(UInt32)mask
{
    fPerfOptionsMask = mask;
}


- (void)setUndoStackDepth:(SInt32)depth
{
	fUndoStackDepth = depth;
}

- (SInt32)undoStackDepth
{
	return fUndoStackDepth;
}

- (void)setStackingMethod:(SInt32)stackingMethod
{	
	ZzAssert(stackingMethod == kPrefsStackingMethodVertically || stackingMethod == kPrefsStackingMethodHorizontally || stackingMethod == kPrefsStackingMethodAlwaysAsk);
	fStackingMethod = stackingMethod;
}

- (SInt32)stackingMethod
{
	return fStackingMethod;
}

- (void)setTimeDisplay:(SInt32)td
{
	fTimeDisplay = td;
}

- (SInt32)timeDisplay
{
	return fTimeDisplay;
}

- (void)setWorkspaceBGColor:(NSColor*)color
{
	[fWorkspaceBGColor release];
	fWorkspaceBGColor = [color retain];
}

- (NSColor*)workspaceBGColor
{
	return fWorkspaceBGColor;
}

- (void)setAudioOutputDeviceStringCode:(NSString*)code
{
	[fAudioOutputDeviceStringCode release];
	fAudioOutputDeviceStringCode = [[NSString alloc] initWithString:code];
	
	// let's change the output device here
}

- (void)setAudioInputDeviceStringCode:(NSString*)code
{
	[fAudioInputDeviceStringCode release];
	fAudioInputDeviceStringCode = [[NSString alloc] initWithString:code];
	// we don't have to change anything here because the recorder window is modal
}

- (void)setAudioOutputDeviceBufferNbFrames:(UInt32)nbFrames
{
	fAudioOutputDeviceBufferNbFrames = nbFrames;
}

// ASCII string code that describe the audio deviuce
- (NSString*)audioOutputDeviceStringCode
{
	return fAudioOutputDeviceStringCode;
}

- (NSString*)audioInputDeviceStringCode
{
	return fAudioInputDeviceStringCode;
}

- (UInt32)audioOutputDeviceBufferNbFrames
{
	return fAudioOutputDeviceBufferNbFrames;
}

- (void)setAskNewDocProperties:(BOOL)val
{
	fAskNewDocProperties = val;
}

- (BOOL)askNewDocProperties
{
	return fAskNewDocProperties;
}

- (void)setHorizontalGridSpacing:(UInt32)val
{
	fHorizontalGridSpacing = val;
}

- (UInt32)getHorizontalGridSpacing
{
	return fHorizontalGridSpacing;
}

/*- (void)setDefaultFPS:(UInt32)val
{
	fFPS = val;
}

- (UInt32)getDefaultFPS
{
	return fFPS;
}*/

- (void)setDefaultTempo:(UInt32)val
{
	fTempo = val;
}
- (UInt32)getDefaultTempo
{
	return fTempo;
}

- (void)setDefaultSignatureNum:(UInt32)val
{
	fSignatureNum = val;
}

- (UInt32)getDefaultSignatureNum
{
	return fSignatureNum;
}

- (void)setDefaultSignatureDen:(UInt32)val
{
	fSignatureDen = val;
}

- (UInt32)getDefaultSignatureDen
{
	return fSignatureDen;
}

- (void)setDefaultRenderFormat:(WorkingCodecType)codec {
	fRenderCodec = codec;
}

- (void)setDefaultPreviewFormat:(WorkingCodecType)codec {
	fPreviewCodec = codec;
}

- (WorkingCodecType)defaultRenderFormat {
	return fRenderCodec;
}

- (WorkingCodecType)defaultPreviewFormat {
	return fPreviewCodec;
}

+(void)addDefaultValuesToDictionary:(NSMutableDictionary*)appDefaults 
{
    [appDefaults setObject:[NSNumber numberWithInt:0] forKey:@"Serialized"];

#if defined(MTG_FULL_VERSION)
    [appDefaults setObject:@"" forKey:@"hyperEngineAVSerial"];
    [appDefaults setObject:@"" forKey:@"rayGunProSerial"];
    [appDefaults setObject:@"" forKey:@"rayGunSerial"];
    [appDefaults setObject:@"" forKey:@"hyperprismSilverSerial"];
    [appDefaults setObject:@"" forKey:@"hyperprismGoldSerial"];
#elif defined(MTG_DEMO_VERSION) 
    [appDefaults setObject:@"Demo version : cannot be registered" forKey:@"hyperEngineAVSerial"];
    [appDefaults setObject:@"Demo version : cannot be registered" forKey:@"rayGunProSerial"];
    [appDefaults setObject:@"Demo version : cannot be registered" forKey:@"rayGunSerial"];
    [appDefaults setObject:@"Demo version : cannot be registered" forKey:@"hyperprismSilverSerial"];
    [appDefaults setObject:@"Demo version : cannot be registered" forKey:@"hyperprismGoldSerial"];
#else
    #error("MTG_DEMO_VERSION or MTG_FULL_VERSION should be defined....");
#endif //MTG_***_VERSION
 
    [appDefaults setObject:[NSNumber numberWithInt:
		kPrefsSnapHLocators | 
		kPrefsSnapHObjects |
		kPrefsSnapVObjects] 
		forKey:@"snapOptionsMask"];

/*
	[appDefaults setObject:[NSNumber numberWithInt:
		kPrefsSnapHLocators ] 
		forKey:@"shiftSnapOptionsMask"];
*/
	[appDefaults setObject:[NSNumber numberWithInt:0] forKey:@"gridShowMask"];
	
	[appDefaults setObject:[NSNumber numberWithInt:
		kPrefsGridsVSpacingMedium] 
		forKey:@"gridVSpacingIndex"];

	[appDefaults setObject:[NSNumber numberWithInt:
		kPrefsPerfMovingTitles|kPrefsPerfUniformBackground|kPrefsPerfObjectShadows] 
		forKey:@"perfOptionsMask"];
	
	[appDefaults setObject:[NSNumber numberWithInt:65536]
		forKey:@"undoStackDepth"];

	[appDefaults setObject:[NSNumber numberWithInt:kPrefsStackingMethodAlwaysAsk]
		forKey:@"stackingMethod"];

	[appDefaults setObject:[NSNumber numberWithInt:kTimeDisplaySMPTE] forKey:@"timeDisplay"];
	
	[appDefaults setObject:[NSArchiver archivedDataWithRootObject:[NSColor colorWithCalibratedRed:0.686 green:0.737 blue:0.808 alpha:1.0]] forKey:@"MIWorkspaceBackgroundColor"];
	
	// defaults for audio interface
	// let's retrieve the codes that mean "default devices"
	NSString* defaultAudioOutputDeviceStringCode = [NSString stringWithString:(NSString*)MEAudioDeviceDesc::GetDefaultOutputStringCode()];
	NSString* defaultAudioInputDeviceStringCode = [NSString stringWithString:(NSString*)MEAudioDeviceDesc::GetDefaultInputStringCode()];
	UInt32 defaultAudioOutputDeviceBufferNbFrames = MEAudioDeviceDesc::GetDefaultOutputNbFrames();
	[appDefaults setObject:defaultAudioOutputDeviceStringCode forKey:@"audioOutputDeviceStringCode"];
	[appDefaults setObject:defaultAudioInputDeviceStringCode forKey:@"audioInputDeviceStringCode"];
	[appDefaults setObject:[NSNumber numberWithInt:defaultAudioOutputDeviceBufferNbFrames] forKey:@"audioOutputDeviceBufferNbFrames"];
	
	[appDefaults setObject:[NSNumber numberWithBool:YES] forKey:@"askNewDocProperties"];
	[appDefaults setObject:[NSNumber numberWithInt:0] forKey:@"horizontalGridSpacing"];
	//[appDefaults setObject:[NSNumber numberWithInt:1] forKey:@"framesPerSecond"];
	[appDefaults setObject:[NSNumber numberWithInt:120] forKey:@"tempo"];
	[appDefaults setObject:[NSNumber numberWithInt:4] forKey:@"signatureNum"];
	[appDefaults setObject:[NSNumber numberWithInt:4] forKey:@"signatureDen"];
	[appDefaults setObject:[NSNumber numberWithInt:kMjpeg320_240] forKey:@"previewCodec"];
	[appDefaults setObject:[NSNumber numberWithInt:kDvNtsc720_480] forKey:@"renderCodec"];
	
	[appDefaults setObject:[NSNumber numberWithBool:YES] forKey:@"TCFixed"];
	[appDefaults setObject:[NSNumber numberWithFloat:5.0] forKey:@"TCDuration"];
	[appDefaults setObject:[NSNumber numberWithFloat:.2] forKey:@"TCSpeed"];

	[appDefaults setObject:[NSNumber numberWithFloat:2.0] forKey:@"SCDuration"];
	
	[appDefaults setObject:[NSNumber numberWithBool:NO] forKey:@"HighQualityPlayback"];
	[appDefaults setObject:[NSNumber numberWithBool:NO] forKey:@"SingleFieldPlayback"];
	[appDefaults setObject:[NSNumber numberWithBool:YES] forKey:@"ReferenceExternalFiles"];
	[appDefaults setObject:[NSNumber numberWithBool:YES] forKey:@"automaticTransitionParameters"];
	[appDefaults setObject:[self createDefaultTransition] forKey:@"defaultTransition"];
	
	
}

-(void)saveToUserSettings
{
/*
    fHyperEngineAVSerial = [[NSUserDefaults standardUserDefaults] stringForKey:@"hyperEngineAVSerial"];
    fRayGunProSerial = [[NSUserDefaults standardUserDefaults] stringForKey:@"rayGunProSerial"];
    fRayGunSerial = [[NSUserDefaults standardUserDefaults] stringForKey:@"rayGunSerial"];
    fHyperprismSilverSerial = [[NSUserDefaults standardUserDefaults] stringForKey:@"hyperprismSilverSerial"];
    fHyperprismGoldSerial = [[NSUserDefaults standardUserDefaults] stringForKey:@"hyperprismGoldSerial"];
*/
    [[NSUserDefaults standardUserDefaults] setObject:fHyperEngineAVSerial forKey:@"hyperEngineAVSerial"];
    [[NSUserDefaults standardUserDefaults] setObject:fRayGunProSerial forKey:@"rayGunProSerial"];
    [[NSUserDefaults standardUserDefaults] setObject:fRayGunSerial forKey:@"rayGunSerial"];
    [[NSUserDefaults standardUserDefaults] setObject:fHyperprismSilverSerial forKey:@"hyperprismSilverSerial"];
    [[NSUserDefaults standardUserDefaults] setObject:fHyperprismGoldSerial forKey:@"hyperprismGoldSerial"];
   

	[[NSUserDefaults standardUserDefaults] setInteger:fSnapOptionsMask forKey:@"snapOptionsMask"];
//	[[NSUserDefaults standardUserDefaults] setInteger:fShiftSnapOptionsMask forKey:@"shiftSnapOptionsMask"];
	[[NSUserDefaults standardUserDefaults] setInteger:fGridShowMask forKey:@"gridShowMask"];
	[[NSUserDefaults standardUserDefaults] setInteger:fGridVSpacing forKey:@"gridVSpacingIndex"];
	[[NSUserDefaults standardUserDefaults] setInteger:fPerfOptionsMask forKey:@"perfOptionsMask"];
	[[NSUserDefaults standardUserDefaults] setInteger:fUndoStackDepth forKey:@"undoStackDepth"];
	[[NSUserDefaults standardUserDefaults] setInteger:fStackingMethod forKey:@"stackingMethod"];
	[[NSUserDefaults standardUserDefaults] setInteger:fTimeDisplay forKey:@"timeDisplay"];
	[[NSUserDefaults standardUserDefaults] setObject:fAudioOutputDeviceStringCode forKey:@"audioOutputDeviceStringCode"];
	[[NSUserDefaults standardUserDefaults] setObject:fAudioInputDeviceStringCode forKey:@"audioInputDeviceStringCode"];
	[[NSUserDefaults standardUserDefaults] setInteger:fAudioOutputDeviceBufferNbFrames forKey:@"audioOutputDeviceBufferNbFrames"];
	[[NSUserDefaults standardUserDefaults] removeObjectForKey:@"MIWorkspaceBackgroundColor"];
	[[NSUserDefaults standardUserDefaults] setObject:[NSNumber numberWithInt:fAskNewDocProperties] forKey:@"askNewDocProperties"];
	[[NSUserDefaults standardUserDefaults] setObject:[NSNumber numberWithInt:fHorizontalGridSpacing] forKey:@"horizontalGridSpacing"];
	//[[NSUserDefaults standardUserDefaults] setObject:[NSNumber numberWithInt:fFPS] forKey:@"framesPerSecond"];
	[[NSUserDefaults standardUserDefaults] setObject:[NSNumber numberWithInt:fTempo] forKey:@"tempo"];
	[[NSUserDefaults standardUserDefaults] setObject:[NSNumber numberWithInt:fSignatureNum] forKey:@"signatureNum"];
	[[NSUserDefaults standardUserDefaults] setObject:[NSNumber numberWithInt:fSignatureDen] forKey:@"signatureDen"];
	[[NSUserDefaults standardUserDefaults] setObject:[NSNumber numberWithInt:fPreviewCodec] forKey:@"previewCodec"];
	[[NSUserDefaults standardUserDefaults] setObject:[NSNumber numberWithInt:fRenderCodec] forKey:@"renderCodec"];

// $$$MARC: why does the following call cause EXC_BAD_ACCESS ???? (stephan)
// reply : because NSColor cannot be put in a plist
	
//	[[NSUserDefaults standardUserDefaults] setObject:fWorkspaceBGColor forKey:@"MIWorkspaceBackgroundColor"];
	[[NSUserDefaults standardUserDefaults] setObject:[NSArchiver archivedDataWithRootObject:fWorkspaceBGColor] forKey:@"MIWorkspaceBackgroundColor"];
	[[NSUserDefaults standardUserDefaults] setObject:[NSNumber numberWithBool:fFixedDuration] forKey:@"TCFixed"];
	[[NSUserDefaults standardUserDefaults] setObject:[NSNumber numberWithFloat:fDefaultDuration] forKey:@"TCDuration"];
	[[NSUserDefaults standardUserDefaults] setObject:[NSNumber numberWithFloat:fScrollSpeed] forKey:@"TCSpeed"];

	[[NSUserDefaults standardUserDefaults] setObject:[NSNumber numberWithFloat:fStillDefaultDuration] forKey:@"SCDuration"];


	[[NSUserDefaults standardUserDefaults] setObject:[NSNumber numberWithBool:fHighQuality] forKey:@"HighQualityPlayback"];
	[[NSUserDefaults standardUserDefaults] setObject:[NSNumber numberWithBool:fSingleField] forKey:@"SingleFieldPlayback"];	
	[[NSUserDefaults standardUserDefaults] setObject:[NSNumber numberWithBool:fReferenceExternalFiles] forKey:@"ReferenceExternalFiles"];
  
  [[NSUserDefaults standardUserDefaults] setObject:[NSNumber numberWithBool:fAutomaticTransitionParameters] forKey:@"automaticTransitionParameters"];
  [[NSUserDefaults standardUserDefaults] setObject:fDefaultTransition forKey:@"defaultTransition"];

//	[[NSUserDefaults standardUserDefaults] setObject:fFirstRun forKey:@"firstRun"];
}
	
-(void)restoreFromUserSettings
{
    fHyperEngineAVSerial = [[[NSUserDefaults standardUserDefaults] stringForKey:@"hyperEngineAVSerial"] retain];
    fRayGunProSerial = [[[NSUserDefaults standardUserDefaults] stringForKey:@"rayGunProSerial"] retain];
    fRayGunSerial = [[[NSUserDefaults standardUserDefaults] stringForKey:@"rayGunSerial"] retain];
    fHyperprismSilverSerial = [[[NSUserDefaults standardUserDefaults] stringForKey:@"hyperprismSilverSerial"] retain];
    fHyperprismGoldSerial = [[[NSUserDefaults standardUserDefaults] stringForKey:@"hyperprismGoldSerial"] retain];

	fSnapOptionsMask = [[NSUserDefaults standardUserDefaults] integerForKey:@"snapOptionsMask"];
//	fShiftSnapOptionsMask = [[NSUserDefaults standardUserDefaults] integerForKey:@"shiftSnapOptionsMask"];
	fGridShowMask = [[NSUserDefaults standardUserDefaults] integerForKey:@"gridShowMask"];
	fGridVSpacing = [[NSUserDefaults standardUserDefaults] integerForKey:@"gridVSpacingIndex"];
	fPerfOptionsMask = [[NSUserDefaults standardUserDefaults] integerForKey:@"perfOptionsMask"];
	fUndoStackDepth = [[NSUserDefaults standardUserDefaults] integerForKey:@"undoStackDepth"];
	fStackingMethod = [[NSUserDefaults standardUserDefaults] integerForKey:@"stackingMethod"];
	fTimeDisplay = [[NSUserDefaults standardUserDefaults] integerForKey:@"timeDisplay"];
	
	NSData *data = [[NSUserDefaults standardUserDefaults] objectForKey:@"MIWorkspaceBackgroundColor"];
	if ( data )
	{
		NSColor	*color = [NSUnarchiver unarchiveObjectWithData:data];
		[self setWorkspaceBGColor:color];
	}
	[self setAudioOutputDeviceStringCode:[[NSUserDefaults standardUserDefaults] objectForKey:@"audioOutputDeviceStringCode"]];
	[self setAudioInputDeviceStringCode:[[NSUserDefaults standardUserDefaults] objectForKey:@"audioInputDeviceStringCode"]];
	fAudioOutputDeviceBufferNbFrames = [[NSUserDefaults standardUserDefaults] integerForKey:@"audioOutputDeviceBufferNbFrames"];

	fAskNewDocProperties = [[[NSUserDefaults standardUserDefaults] objectForKey:@"askNewDocProperties"] intValue];
	fHorizontalGridSpacing = [[[NSUserDefaults standardUserDefaults] objectForKey:@"horizontalGridSpacing"] intValue];
	//fFPS = [[[NSUserDefaults standardUserDefaults] objectForKey:@"framesPerSecond"] intValue];
	fTempo = [[[NSUserDefaults standardUserDefaults] objectForKey:@"tempo"] intValue];
	fSignatureNum = [[[NSUserDefaults standardUserDefaults] objectForKey:@"signatureNum"] intValue];
	fSignatureDen = [[[NSUserDefaults standardUserDefaults] objectForKey:@"signatureDen"] intValue];
	fPreviewCodec = (WorkingCodecType)[[[NSUserDefaults standardUserDefaults] objectForKey:@"previewCodec"] intValue];
	fRenderCodec = (WorkingCodecType)[[[NSUserDefaults standardUserDefaults] objectForKey:@"renderCodec"] intValue];

	fFixedDuration = [[[NSUserDefaults standardUserDefaults] objectForKey:@"TCFixed"] boolValue];
	fDefaultDuration = [[[NSUserDefaults standardUserDefaults] objectForKey:@"TCDuration"] floatValue];
	fScrollSpeed = [[[NSUserDefaults standardUserDefaults] objectForKey:@"TCSpeed"] floatValue];

	fStillDefaultDuration = [[[NSUserDefaults standardUserDefaults] objectForKey:@"SCDuration"] floatValue];

	fHighQuality = [[[NSUserDefaults standardUserDefaults] objectForKey:@"HighQualityPlayback"] boolValue];
	fSingleField = [[[NSUserDefaults standardUserDefaults] objectForKey:@"SingleFieldPlayback"] boolValue];
	fReferenceExternalFiles = [[[NSUserDefaults standardUserDefaults] objectForKey:@"ReferenceExternalFiles"] boolValue];
//  fDefaultTransition = [[[NSUserDefaults standardUserDefaults] objectForKey:@"defaultTransition"] retain];
  fAutomaticTransitionParameters = [[[NSUserDefaults standardUserDefaults] objectForKey:@"automaticTransitionParameters"] boolValue];
	data = [[NSUserDefaults standardUserDefaults] objectForKey:@"defaultTransition"];
	
	[self setDefaultTransition:data];
/*	if ( data )
	{
		NSData	*value = [NSUnarchiver unarchiveObjectWithData:data];
		[self setDefaultTransition:value];
	}*/

}

- (void)dealloc
{
	QTDisposeAtomContainer(fDefaultTransitionEffectDesc);
	[fWorkspaceBGColor release];
//	[fFirstRun release];
    [super dealloc];
}

- (void)setTextClipDefaultDuration:(float)duration {
	fDefaultDuration = duration;
}

- (float)textClipDefaultDuration {
	return fDefaultDuration;
}

- (void)setTextClipScrollSpeed:(float)speed {
	fScrollSpeed = speed;
}

- (float)textClipScrollSpeed {
	return fScrollSpeed;
}

- (BOOL)isFixedDuration {
	return fFixedDuration;
}

- (void)setIsFixedDuration:(BOOL)flag {
	fFixedDuration = flag;
}


- (void)setStillClipDefaultDuration:(float)duration {
	fStillDefaultDuration = duration;
}

- (float)stillClipDefaultDuration {
	return fStillDefaultDuration;
}

- (void)setIsHighQuality:(BOOL)flag {
	fHighQuality = flag;
	[[NSNotificationCenter defaultCenter] postNotificationName:MIHighQualityHasChangedNotification object:self];
}

- (BOOL)isHighQuality {
	return fHighQuality;
}

- (void)setIsSingleField:(BOOL)flag {
	fSingleField = flag;
	[[NSNotificationCenter defaultCenter] postNotificationName:MISingleFieldHasChangedNotification object:self];
}

- (BOOL)isSingleField {
	return fSingleField;
}

- (void)setReferenceExternalFiles:(BOOL)flag
{
	fReferenceExternalFiles = flag;
}

- (BOOL)referenceExternalFiles
{
	return fReferenceExternalFiles;
}

- (void)setAutomaticTransitionParameters:(BOOL)flag
{
  fAutomaticTransitionParameters = flag;
}

- (BOOL)automaticTransitionParameters
{
  return fAutomaticTransitionParameters;
}

- (void)setDefaultTransition:(id)value
{
	fDefaultTransition = [value retain];
	
	int* effectBuffer = new int[4096];
	[fDefaultTransition getBytes:effectBuffer];
	ArmeMemArchiveHandler effectArchive(effectBuffer, 4096, false);
	
	QTDisposeAtomContainer(fDefaultTransitionEffectDesc);
	QTNewAtomContainer(&fDefaultTransitionEffectDesc);
	effectArchive.ReadQTAtomContainer(&fDefaultTransitionEffectDesc);

	delete effectBuffer;
}

- (id)getDefaultTransition
{
  return fDefaultTransition;
}

- (QTAtomContainer)getDefaultTransitionEffectDesc
{
	return fDefaultTransitionEffectDesc;
}

+ (id)createDefaultTransition
{
    OSType myType;
    OSErr res=0;
    QTAtomID newAtom;
    Fixed	percentage[2];
    
	QTAtomContainer effectDesc;
	QTNewAtomContainer(&effectDesc);


    // create the effect ID atom: the atom type is kParameterWhatName, and the atom ID is kParameterWhatID
    myType = EndianU32_NtoB(kCrossFadeTransitionType);
    res = QTInsertChild(effectDesc, kParentAtomIsContainer, kParameterWhatName, kParameterWhatID, 0, sizeof(myType), &myType, NULL);
    ThrowOnOSStatus(res);

    /* add the percentage ('pcnt') atom for the effect */
    res = QTInsertChild(effectDesc, kParentAtomIsContainer, kParameterUsagePercent, 1, 0, 0, 0, &newAtom);
    ThrowOnOSStatus(res);

    /* this atom specifies the format of the tween data */
    myType = EndianU32_NtoB(kParameterTypeDataFixed);
    res = QTInsertChild(effectDesc, newAtom, kTweenType, 1 , 0, sizeof(OSType), &myType, NULL);
    ThrowOnOSStatus(res);

    percentage[0] =  EndianS32_NtoB( X2Fix(0.0f) );	/* percentage of mix at the end of the transition */
    percentage[1] =  EndianS32_NtoB( X2Fix(1.0f) );	/* percentage of mix at the start of the transition */

    /* here's the data for the tween  */
    res = QTInsertChild(effectDesc, newAtom, kTweenData, 1 , 0, sizeof(long) * 2, percentage, NULL);
    ThrowOnOSStatus(res);
	
	int* effectBuffer = new int[4096];
	ArmeMemArchiveHandler effectArchive(effectBuffer, 4096, true);
	effectArchive.WriteQTAtomContainer(&effectDesc);
	
	NSData* value = [[NSData dataWithBytes:effectBuffer length:effectArchive.GetSizeWritten()] retain];

	delete effectBuffer;
	QTDisposeAtomContainer(effectDesc);

	return value;
}

-(const char*)getSerialFor:(int)product
{
        return "";
}

-(void)setSerial:(NSString*)serial forProduct:(int)product;
{
}        

-(bool)isSerialized
{
    int serialized = [[NSUserDefaults standardUserDefaults] integerForKey:@"Serialized"];
    return (serialized != 0);
}

-(void)setSerialized
{
    [[NSUserDefaults standardUserDefaults] setInteger:1 forKey:@"Serialized"];
}

- (NSCalendarDate*)getFirstRunDate
{
	return fFirstRun;
}
@end
