/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import <Cocoa/Cocoa.h>

@class MIPreferences;

@interface MIPreferencesController : NSWindowController
{
    IBOutlet id fHyperEngineAVSerialTextBox;
    IBOutlet id fRayGunProSerialTextBox;
    IBOutlet id fRayGunSerialTextBox;
    IBOutlet id fHyperprismSilverSerialTextBox;
    IBOutlet id fHyperprismGoldSerialTextBox;
    
    IBOutlet id fValidateSerialNumbersButton;
	
    // snapping pane
	IBOutlet id fSnapHGridCheckBox;
	IBOutlet id fSnapHObjCheckBox;
	IBOutlet id fSnapHLocatorCheckBox;
	IBOutlet id fSnapVGridCheckBox;
	IBOutlet id fSnapVObjCheckBox;
	IBOutlet id fGridShowVLinesCheckBox;
	IBOutlet id fGridShowHLinesCheckBox;
	IBOutlet id fGridShowVObjLinesCheckBox;
	IBOutlet id fGridShowHObjLinesCheckBox;
	IBOutlet id fGridVSpacingPopUp;

	// performance pane
	IBOutlet id fPerfThumbsWhenPlayingCheckBox;
	IBOutlet id	fPerfShadowsAroundObjectsCheckBox;
	IBOutlet id fPerfUniformBackgroundCheckBox;
	IBOutlet id	fPerfMovingTitlesCheckBox;
	IBOutlet id	fWorkspaceBackgroundColor;

	// Undo Stack pane
	IBOutlet id fUndoStackDepthSlider;
	IBOutlet id	fUndoStackDepthCaption;
	
	// Radio buttons for stacking method
	IBOutlet id fStackVerticallyRadioButton;
	IBOutlet id fStackHorizontallyRadioButton;
	IBOutlet id fStackAlwaysAskRadioButton;
	IBOutlet id fTimeDisplayMatrix;
	
	// audio hardware stuff
	IBOutlet id fAudioOutputDevicePopUp;
	IBOutlet id fAudioOutputBufferSizeSlider;
	IBOutlet id fAudioOutputBufferSizeTextBox;

	// audio hardware stuff
	IBOutlet id fAudioInputDevicePopUp;
	IBOutlet id fAudioInputBufferSizeSlider;
	IBOutlet id fAudioInputBufferSizeTextBox;
	// tmp stuff for audio hardware
	NSMutableArray*	fOutputDevicesStringCodes;
	NSMutableArray*	fInputDevicesStringCodes;
	
	IBOutlet id fAskNewDocProperties;
	IBOutlet id	fTempo;
	IBOutlet id fSignatureNum;
	IBOutlet id fSignatureDen;
	IBOutlet id fHorizontalGridSpacing;
	IBOutlet id fPreviewFormatMenu;
	IBOutlet id fPreviewImageSizeTextField;
	IBOutlet id fPreviewFrameRateTextField;
	IBOutlet id fRenderFormatMenu;
	IBOutlet id fRenderImageSizeTextField;
	IBOutlet id fRenderFrameRateTextField;

	// Text Clips
	IBOutlet id fFixed;
	IBOutlet id fAutomatic;
	IBOutlet id fSeconds;
	IBOutlet id fMinutes;
	IBOutlet id fTenth;
	IBOutlet id fSpeedSlider;
	IBOutlet id fSpeedTextField;

	// Still Clips
	IBOutlet id fSCSeconds;
	IBOutlet id fSCMinutes;
	IBOutlet id fSCTenth;

	// Playback Quality
	IBOutlet id fHighQualityCheckbox;
	IBOutlet id fSingleFieldCheckbox;
	
	IBOutlet id fReferenceExternalFilesRadio;
	IBOutlet id fCopyExternalFilesRadio;
  
  IBOutlet id fAutomaticCrossFadeRadio;
  IBOutlet id fFixedCrossFadeRadio;
  
  IBOutlet id fDefaultTransitionText;
}

- (IBAction)validateSerialNumbers:(id)sender;

- (IBAction)snapHGridCheckBox:(id)sender;
- (IBAction)snapHObjCheckBox:(id)sender;
- (IBAction)snapHLocatorCheckBox:(id)sender;
- (IBAction)snapVGridCheckBox:(id)sender;
- (IBAction)snapVObjCheckBox:(id)sender;
- (IBAction)gridShowVLinesCheckBox:(id)sender;
- (IBAction)gridShowHLinesCheckBox:(id)sender;
- (IBAction)gridShowVObjLinesCheckBox:(id)sender;
- (IBAction)gridShowHObjLinesCheckBox:(id)sender;
- (IBAction)gridVSpacingPopUp:(id)sender;


- (IBAction)perfThumbsWhenPlayingCheckBox:(id)sender;
- (IBAction)perfShadowsAroundObjectsCheckBox:(id)sender;
- (IBAction)perfUniformBackgroundCheckBox:(id)sender;
- (IBAction)perfMovingTitlesCheckBox:(id)sender;
- (IBAction)perfWorkspaceBGColor:(id)sender;


//- (IBAction)chooseScratchDiskFolder:(id)sender;

- (IBAction)undoStackDepthSlid:(id)sender;
- (IBAction)purgeUndoStack:(id)sender;

- (IBAction)stackVerticallyRadioButton:(id)sender;
- (IBAction)stackHorizontallyRadioButton:(id)sender;
- (IBAction)stackAlwaysAskRadioButton:(id)sender;

- (IBAction)audioOutputDevicePopUp:(id)sender;
- (IBAction)audioOutputBufferSizeSlider:(id)sender;
- (IBAction)audioInputDevicePopUp:(id)sender;
- (IBAction)audioInputBufferSizeSlider:(id)sender;

- (IBAction)timeDisplayChanged:(id)sender;

- (IBAction)askNewDocProperties:(id)sender;
- (IBAction)gridDefinitionChanged:(id)sender;
- (IBAction)defaultFPSChanged:(id)sender;
- (IBAction)defaultTempoChanged:(id)sender;
- (IBAction)defaultSignatureNumChanged:(id)sender;
- (IBAction)defaultSignatureDenChanged:(id)sender;

- (IBAction)setIsFixed:(id)sender;
- (IBAction)setIsAutomatic:(id)sender;
	
- (IBAction)speedSliderChanged:(id)sender;
- (IBAction)speedTextFieldChanged:(id)sender;

- (IBAction)durationTextFieldChanged:(id)sender;

- (IBAction)durationSCTextFieldChanged:(id)sender;
	
//- (void)openPanelDidEnd:(NSOpenPanel *)sheet returnCode:(int)returnCode contextInfo:(void  *)contextInfo;
//- (void)controlTextDidChange:(NSNotification*)notification;

- (void)awakeFromNib;

// the methods that populate each tab controls with values from the preferences
- (void)fillDlgWithSnapAndGridValues:(MIPreferences*)prefs;
- (void)fillDlgWithPerformanceValues:(MIPreferences*)prefs;
- (void)fillDlgWithUndoStackValues:(MIPreferences*)prefs;
- (void)fillDlgWithStackingMethodValues:(MIPreferences*)pref;
- (void)fillDlgWithAudioDeviceValues:(MIPreferences*)pref;
- (void)fillDlgWithTextClipValues:(MIPreferences*)prefs;
- (void)fillDlgWithStillClipValues:(MIPreferences*)prefs;
- (void)fillDlgWithPlaybackQualityValues:(MIPreferences*)prefs;
- (void)fillDlgWithReferenceStyle:(MIPreferences*)prefs;
- (void)fillDlgWithDefaultTransition:(MIPreferences*)prefs;
- (void)fillDlgWithSerialNumbers:(MIPreferences*)prefs;

- (void)redrawAllMainWindows;
- (BOOL)windowShouldClose:(id)sender;

- (IBAction)formatPopupSelectedItemChanged:(id)sender;

- (IBAction)highQualityChanged:(id)sender;
- (IBAction)singleFieldChanged:(id)sender;

- (IBAction)referenceExternalFiles:(id)sender;
- (IBAction)copyExternalFile:(id)sender;

- (IBAction)setDefaultTransition:(id)sender;
- (IBAction)automaticTransitionParameters:(id)sender;
- (IBAction)fixedTransitionParameters:(id)sender;

@end
