/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MIPreferencesController.h"
#import "MIPreferences.h"
#import "MIDocumentController.h"
#import "MIMontageDocument.h"
#import "MIMainViewController.h"
#import "MITopLevelContainerRep.h"
#import "MIRuler.h"
#import "MITimePaletteController.h"
#import "MIMixerController.h"
#import "MIApplication.h"
#import "MIApplicationController.h"

#ifndef MEAudioDeviceDesc_h
#include "MEAudioDeviceDesc.h"
#endif //MEAudioDeviceDesc_h

#ifndef TArmeMediaEngine_h
#include "TArmeMediaEngine.h"
#endif //TArmeMediaEngine_h

#ifndef MovieHelper_h
#include "MovieHelper.h"
#endif //MovieHelper_h

#ifndef ArmeMemArchiveHandler_h
#include "ArmeMemArchiveHandler.h"
#endif //ArmeMemArchiveHandler_h


@implementation MIPreferencesController
-(NSString*)MaskSerial:(NSString*)serial
{
	if ([serial length] > 9)
	{
		char mask[256];
		[serial getCString:mask];
		for (uint i=0; i < ([serial length] - 9); i++)
		{
			mask[i] = '*';
		}
		NSString* maskedSerial = [NSString stringWithCString:mask];

		return maskedSerial;
	}
	return serial;
}


-(IBAction)snapHGridCheckBox:(id)sender
{
	MIPreferences* prefs = [MIPreferences sharedPrefs];
	UInt32 snapOptionsMask = [prefs snapOptionsMask];
	if([sender state] == NSOnState)
		[prefs setSnapOptionsMask:snapOptionsMask | kPrefsSnapHGrid];
	else
		[prefs setSnapOptionsMask:snapOptionsMask & ~kPrefsSnapHGrid];
	[self redrawAllMainWindows];
}

-(IBAction)snapHObjCheckBox:(id)sender
{
	MIPreferences* prefs = [MIPreferences sharedPrefs];
	UInt32 snapOptionsMask = [prefs snapOptionsMask];
	if([sender state] == NSOnState)
		[prefs setSnapOptionsMask:snapOptionsMask | kPrefsSnapHObjects];
	else
		[prefs setSnapOptionsMask:snapOptionsMask & ~kPrefsSnapHObjects];
	[self redrawAllMainWindows];
}

-(IBAction)snapHLocatorCheckBox:(id)sender
{
	MIPreferences* prefs = [MIPreferences sharedPrefs];
	UInt32 snapOptionsMask = [prefs snapOptionsMask];
	if([sender state] == NSOnState)
		[prefs setSnapOptionsMask:snapOptionsMask | kPrefsSnapHLocators];
	else
		[prefs setSnapOptionsMask:snapOptionsMask & ~kPrefsSnapHLocators];
	[self redrawAllMainWindows];
}

-(IBAction)snapVGridCheckBox:(id)sender
{
	MIPreferences* prefs = [MIPreferences sharedPrefs];
	UInt32 snapOptionsMask = [prefs snapOptionsMask];
	if([sender state] == NSOnState)
		[prefs setSnapOptionsMask:snapOptionsMask | kPrefsSnapVGrid];
	else
		[prefs setSnapOptionsMask:snapOptionsMask & ~kPrefsSnapVGrid];
	[self redrawAllMainWindows];
}


- (IBAction)stackVerticallyRadioButton:(id)sender
{
	MIPreferences* prefs = [MIPreferences sharedPrefs];
	if([fStackVerticallyRadioButton state] == NSOnState)
		[prefs setStackingMethod:kPrefsStackingMethodVertically];
}

- (IBAction)stackHorizontallyRadioButton:(id)sender
{
	MIPreferences* prefs = [MIPreferences sharedPrefs];
	if([fStackHorizontallyRadioButton state] == NSOnState)
		[prefs setStackingMethod:kPrefsStackingMethodHorizontally];
}

- (IBAction)stackAlwaysAskRadioButton:(id)sender
{
	MIPreferences* prefs = [MIPreferences sharedPrefs];
	if([fStackAlwaysAskRadioButton state] == NSOnState)
		[prefs setStackingMethod:kPrefsStackingMethodAlwaysAsk];
}

-(IBAction)snapVObjCheckBox:(id)sender
{
	MIPreferences* prefs = [MIPreferences sharedPrefs];
	UInt32 snapOptionsMask = [prefs snapOptionsMask];
	if([sender state] == NSOnState)
		[prefs setSnapOptionsMask:snapOptionsMask | kPrefsSnapVObjects];
	else
		[prefs setSnapOptionsMask:snapOptionsMask & ~kPrefsSnapVObjects];
	[self redrawAllMainWindows];
}

/*
-(IBAction)snapShiftHGridCheckBox:(id)sender
{
	MIPreferences* prefs = [MIPreferences sharedPrefs];
	UInt32 shiftSnapOptionsMask = [prefs shiftSnapOptionsMask];
	if([sender state] == NSOnState)
		[prefs setShiftSnapOptionsMask:shiftSnapOptionsMask | kPrefsSnapHGrid];
	else
		[prefs setShiftSnapOptionsMask:shiftSnapOptionsMask & ~kPrefsSnapHGrid];
	[self redrawAllMainWindows];
}

-(IBAction)snapShiftHObjCheckBox:(id)sender
{
	MIPreferences* prefs = [MIPreferences sharedPrefs];
	UInt32 shiftSnapOptionsMask = [prefs shiftSnapOptionsMask];
	if([sender state] == NSOnState)
		[prefs setShiftSnapOptionsMask:shiftSnapOptionsMask | kPrefsSnapHObjects];
	else
		[prefs setShiftSnapOptionsMask:shiftSnapOptionsMask & ~kPrefsSnapHObjects];
	[self redrawAllMainWindows];
}

-(IBAction)snapShiftHLocatorCheckBox:(id)sender
{
	MIPreferences* prefs = [MIPreferences sharedPrefs];
	UInt32 shiftSnapOptionsMask = [prefs shiftSnapOptionsMask];
	if([sender state] == NSOnState)
		[prefs setShiftSnapOptionsMask:shiftSnapOptionsMask | kPrefsSnapHLocators];
	else
		[prefs setShiftSnapOptionsMask:shiftSnapOptionsMask & ~kPrefsSnapHLocators];
	[self redrawAllMainWindows];
}

-(IBAction)snapShiftVGridCheckBox:(id)sender
{
	MIPreferences* prefs = [MIPreferences sharedPrefs];
	UInt32 shiftSnapOptionsMask = [prefs shiftSnapOptionsMask];
	if([sender state] == NSOnState)
		[prefs setShiftSnapOptionsMask:shiftSnapOptionsMask | kPrefsSnapVGrid];
	else
		[prefs setShiftSnapOptionsMask:shiftSnapOptionsMask & ~kPrefsSnapVGrid];
	[self redrawAllMainWindows];
}

-(IBAction)snapShiftVObjCheckBox:(id)sender
{
	MIPreferences* prefs = [MIPreferences sharedPrefs];
	UInt32 shiftSnapOptionsMask = [prefs shiftSnapOptionsMask];
	if([sender state] == NSOnState)
		[prefs setShiftSnapOptionsMask:shiftSnapOptionsMask | kPrefsSnapVObjects];
	else
		[prefs setShiftSnapOptionsMask:shiftSnapOptionsMask & ~kPrefsSnapVObjects];
	[self redrawAllMainWindows];
}
*/

-(IBAction)gridShowVLinesCheckBox:(id)sender
{
	MIPreferences* prefs = [MIPreferences sharedPrefs];
	UInt32 gridShowMask = [prefs gridShowMask];
	if([sender state] == NSOnState)
		[prefs setGridShowMask:gridShowMask | kPrefsGridShowVLines];
	else
		[prefs setGridShowMask:gridShowMask & ~kPrefsGridShowVLines];
	[self redrawAllMainWindows];
}

-(IBAction)gridShowHLinesCheckBox:(id)sender
{
	MIPreferences* prefs = [MIPreferences sharedPrefs];
	UInt32 gridShowMask = [prefs gridShowMask];
	if([sender state] == NSOnState)
		[prefs setGridShowMask:gridShowMask | kPrefsGridShowHLines];
	else
		[prefs setGridShowMask:gridShowMask & ~kPrefsGridShowHLines];
	[self redrawAllMainWindows];
}

-(IBAction)gridShowVObjLinesCheckBox:(id)sender
{
	MIPreferences* prefs = [MIPreferences sharedPrefs];
	UInt32 gridShowMask = [prefs gridShowMask];
	if([sender state] == NSOnState)
		[prefs setGridShowMask:gridShowMask | kPrefsGridShowVObjLines];
	else
		[prefs setGridShowMask:gridShowMask & ~kPrefsGridShowVObjLines];
	[self redrawAllMainWindows];
}

-(IBAction)gridShowHObjLinesCheckBox:(id)sender
{
	MIPreferences* prefs = [MIPreferences sharedPrefs];
	UInt32 gridShowMask = [prefs gridShowMask];
	if([sender state] == NSOnState)
		[prefs setGridShowMask:gridShowMask | kPrefsGridShowHObjLines];
	else
		[prefs setGridShowMask:gridShowMask & ~kPrefsGridShowHObjLines];
	[self redrawAllMainWindows];
}

-(IBAction)gridVSpacingPopUp:(id)sender
{
	MIPreferences* prefs = [MIPreferences sharedPrefs];
	UInt32 index = [sender indexOfSelectedItem];
	[prefs setGridVSpacing:index];
	[self redrawAllMainWindows];
}

- (IBAction)timeDisplayChanged:(id)sender
{
	MIPreferences* prefs = [MIPreferences sharedPrefs];
	NSEnumerator* docEnum = [[[MIDocumentController sharedDocumentController] documents] objectEnumerator];
	MIMontageDocument* doc;

	[prefs setTimeDisplay:[[sender selectedCell] tag]];

	[self redrawAllMainWindows];
	[[MITimePaletteController sharedTimePaletteController] checkSMPTEBPM];

	while(doc = [docEnum nextObject])
	{
		NSEnumerator* mainViewEnum = [[doc windowControllers] objectEnumerator];
		MIMainViewController* mainViewController;
		while(mainViewController = [mainViewEnum nextObject])
		{
			[[mainViewController ruler] refreshMarksImage];
			[[mainViewController ruler] setNeedsDisplay:YES];
		}
	}

}

- (IBAction)undoStackDepthSlid:(id)sender
{
	MIPreferences*	prefs = [MIPreferences sharedPrefs];
	SInt32			depth;
	NSString*		msg;
	
//	int test = [sender intValue];
	
	switch( [sender intValue] )
	{
		case 0:		depth = 0; break;
		case 1:		depth = 1; break;
		case 2:		depth = 10; break;
		case 3:		depth = 20; break;
		case 4:		depth = 30; break;
		case 5:		depth = 50; break;
		case 6:		depth = 100; break;
		case 7:		depth = 200; break;
		case 8:		depth = 300; break;
		case 9:		depth = 500; break;
		case 10:	depth = -1; break;
		default:	depth = 50; break;
	}
	
	[prefs setUndoStackDepth:depth];
	if (depth == -1)
		msg = @"Infinite";
	else if (depth <= 1)
		msg = [NSString stringWithFormat:@"%d Command", depth];
	else 
		msg = [NSString stringWithFormat:@"%d Commands", depth];
	
	[fUndoStackDepthCaption setStringValue:msg];
}

- (IBAction)purgeUndoStack:(id)sender
{
	[[[[NSApp mainViewController] document] undoStack] removeAllObjects];
}    

-(void)redrawAllMainWindows
{
	NSEnumerator* docEnum = [[[MIDocumentController sharedDocumentController] documents] objectEnumerator];
	MIMontageDocument* doc;
	while(doc = [docEnum nextObject])
	{
		NSEnumerator* mainViewEnum = [[doc windowControllers] objectEnumerator];
		MIMainViewController* mainViewController;
		while(mainViewController = [mainViewEnum nextObject])
		{
			[[mainViewController topLevelContainerView] setNeedsDisplay:YES];
		}
	}
}

-(IBAction)perfThumbsWhenPlayingCheckBox:(id)sender
{
	MIPreferences* prefs = [MIPreferences sharedPrefs];
	UInt32 perfOptionsMask = [prefs perfOptionsMask];
	if([sender state] == NSOnState)
		[prefs setPerfOptionsMask:perfOptionsMask | kPrefsPerfThumbsWhenPlaying];
	else
		[prefs setPerfOptionsMask:perfOptionsMask & ~kPrefsPerfThumbsWhenPlaying];
}

- (IBAction)perfShadowsAroundObjectsCheckBox:(id)sender
{
	MIPreferences* prefs = [MIPreferences sharedPrefs];
	UInt32 perfOptionsMask = [prefs perfOptionsMask];
	if([sender state] == NSOnState)
		[prefs setPerfOptionsMask:perfOptionsMask | kPrefsPerfObjectShadows];
	else
		[prefs setPerfOptionsMask:perfOptionsMask & ~kPrefsPerfObjectShadows];
		
	[self redrawAllMainWindows];
}

- (IBAction)perfUniformBackgroundCheckBox:(id)sender
{
	MIPreferences* prefs = [MIPreferences sharedPrefs];
	UInt32 perfOptionsMask = [prefs perfOptionsMask];
	if([sender state] == NSOnState)
		[prefs setPerfOptionsMask:perfOptionsMask | kPrefsPerfUniformBackground];
	else
		[prefs setPerfOptionsMask:perfOptionsMask & ~kPrefsPerfUniformBackground];
}

- (IBAction)perfMovingTitlesCheckBox:(id)sender
{
	MIPreferences* prefs = [MIPreferences sharedPrefs];
	UInt32 perfOptionsMask = [prefs perfOptionsMask];
	if([sender state] == NSOnState)
		[prefs setPerfOptionsMask:perfOptionsMask | kPrefsPerfMovingTitles];
	else
		[prefs setPerfOptionsMask:perfOptionsMask & ~kPrefsPerfMovingTitles];
}

- (IBAction)perfWorkspaceBGColor:(id)sender
{
	MIPreferences* prefs = [MIPreferences sharedPrefs];

	[prefs setWorkspaceBGColor:[fWorkspaceBackgroundColor color]];
	
	[self redrawAllMainWindows];
}

- (IBAction)askNewDocProperties:(id)sender
{
	MIPreferences* prefs = [MIPreferences sharedPrefs];
	
	if ( [sender state] == NSOnState )
		[prefs setAskNewDocProperties:YES];
	else
		[prefs setAskNewDocProperties:NO];
}

- (IBAction)gridDefinitionChanged:(id)sender
{
	MIPreferences* prefs = [MIPreferences sharedPrefs];
	[prefs setHorizontalGridSpacing:[[sender itemAtIndex:[sender indexOfSelectedItem]] tag]];

	[self redrawAllMainWindows];
}

- (IBAction)defaultFPSChanged:(id)sender
{
//	MIPreferences* prefs = [MIPreferences sharedPrefs];
	ZzAssert(0);
	//[prefs setDefaultFPS:[sender indexOfSelectedItem]];
}

- (IBAction)defaultTempoChanged:(id)sender
{
	MIPreferences* prefs = [MIPreferences sharedPrefs];

	[prefs setDefaultTempo:[sender intValue]];
}

- (IBAction)defaultSignatureNumChanged:(id)sender
{
	MIPreferences* prefs = [MIPreferences sharedPrefs];

	[prefs setDefaultSignatureNum:[sender intValue]];
}

- (IBAction)defaultSignatureDenChanged:(id)sender
{
	MIPreferences* prefs = [MIPreferences sharedPrefs];

	[prefs setDefaultSignatureDen:[sender intValue]];
}

- (void)fillDlgWithMiscValues:(MIPreferences*)prefs
{
	UInt32 tempo = [prefs getDefaultTempo];
	UInt32 signNum = [prefs getDefaultSignatureNum];
	UInt32 signDen = [prefs getDefaultSignatureDen];
	//UInt32 fps = [prefs getDefaultFPS];
	BOOL newDocProp = [prefs askNewDocProperties];
	UInt32 timeDisplay = [prefs timeDisplay];
	UInt32 hGridSpacing = [prefs getHorizontalGridSpacing];
//	WorkingCodecType previewCodec = [prefs defaultPreviewFormat];
	WorkingCodecType renderCodec = [prefs defaultRenderFormat];

	[fTempo setIntValue:tempo];
	[fSignatureNum setIntValue:signNum];
	[fSignatureDen setIntValue:signDen];
	if ( newDocProp )
		[fAskNewDocProperties setState:NSOnState];
	else
		[fAskNewDocProperties setState:NSOffState];
	//[fFPS selectItemAtIndex:fps];
	[fHorizontalGridSpacing selectItemAtIndex:[fHorizontalGridSpacing indexOfItemWithTag:hGridSpacing]];
	[[fTimeDisplayMatrix cellWithTag:(timeDisplay == 1 ? 2 : 1)] setState:NSOffState];
	[[fTimeDisplayMatrix cellWithTag:timeDisplay] setState:NSOnState];
	
/*	switch ( previewCodec ) {
		case kDvPal720_576 :
			[fPreviewFormatMenu selectItemAtIndex:0];
			break;
		case kDvPal16_9 :
			[fPreviewFormatMenu selectItemAtIndex:1];
			break;
		case kDvNtsc720_480 :
			[fPreviewFormatMenu selectItemAtIndex:2];
			break;
		case kDvNtsc16_9 :
			[fPreviewFormatMenu selectItemAtIndex:3];
			break;
		case kMjpeg160_120 :
			[fPreviewFormatMenu selectItemAtIndex:4];
			break;
		case kMjpeg320_240 :
			[fPreviewFormatMenu selectItemAtIndex:5];
			break;
		case kAiff :
		default :
			ZzAssert(0);
			break;
	}

	[self formatPopupSelectedItemChanged:fPreviewFormatMenu];*/

	switch ( renderCodec ) {
		case kDvPal720_576 :
			[fRenderFormatMenu selectItemAtIndex:0];
			break;
		case kDvPal16_9 :
			[fRenderFormatMenu selectItemAtIndex:1];
			break;
		case kDvNtsc720_480 :
			[fRenderFormatMenu selectItemAtIndex:2];
			break;
		case kDvNtsc16_9 :
			[fRenderFormatMenu selectItemAtIndex:3];
			break;
		case kMjpeg160_120 :
			[fRenderFormatMenu selectItemAtIndex:4];
			break;
		case kMjpeg320_240 :
			[fRenderFormatMenu selectItemAtIndex:5];
			break;
		case kAiff :
		default :
			ZzAssert(0);
			break;
	}

	[self formatPopupSelectedItemChanged:fRenderFormatMenu];
}

- (void)setPrefsWithPopupSelection:(id)popupmenu andCodec:(WorkingCodecType)codec {
	MIPreferences* prefs = [MIPreferences sharedPrefs];
	
/*	if ( popupmenu == fPreviewFormatMenu )
		[prefs setDefaultPreviewFormat:codec];
	else*/ 
	[prefs setDefaultRenderFormat:codec];
}

- (IBAction)formatPopupSelectedItemChanged:(id)sender {
	id imageSizeTextField;
	id frameRateTextField;
	int index = [sender indexOfSelectedItem];
	

/*	if ( sender == fPreviewFormatMenu ) {
		imageSizeTextField = fPreviewImageSizeTextField;
		frameRateTextField = fPreviewFrameRateTextField;
	} else {*/
		imageSizeTextField = fRenderImageSizeTextField;
		frameRateTextField = fRenderFrameRateTextField;
//	}	

	switch (index ) {
		case 0 :
			[imageSizeTextField setStringValue:@"720 x 576 pixels"];
			[frameRateTextField setStringValue:@"25 fps"];
			[self setPrefsWithPopupSelection:sender andCodec:kDvPal720_576];
			break;
		case 1 :
			[imageSizeTextField setStringValue:@"720 x 576 pixels"];
			[frameRateTextField setStringValue:@"25 fps"];
			[self setPrefsWithPopupSelection:sender andCodec:kDvPal16_9];
			break;
		case 2 :
			[imageSizeTextField setStringValue:@"720 x 480 pixels"];
			[frameRateTextField setStringValue:@"29.97 fps"];
			[self setPrefsWithPopupSelection:sender andCodec:kDvNtsc720_480];
			break;
		case 3 :
			[imageSizeTextField setStringValue:@"720 x 480 pixels"];
			[frameRateTextField setStringValue:@"29.97 fps"];
			[self setPrefsWithPopupSelection:sender andCodec:kDvNtsc16_9];
			break;
		case 4 :
			[imageSizeTextField setStringValue:@"160 x 120 pixels"];
			[frameRateTextField setStringValue:@"24 fps"];
			[self setPrefsWithPopupSelection:sender andCodec:kMjpeg160_120];
			break;
		case 5 :
			[imageSizeTextField setStringValue:@"320 x 240 pixels"];
			[frameRateTextField setStringValue:@"30 fps"];
			[self setPrefsWithPopupSelection:sender andCodec:kMjpeg320_240];
			break;
	}
}


- (void)fillDlgWithSnapAndGridValues:(MIPreferences*)prefs
{
	UInt32 snapOptionsMask = [prefs snapOptionsMask];
//	UInt32 shiftSnapOptionsMask = [prefs shiftSnapOptionsMask];
	UInt32 gridShowMask = [prefs gridShowMask];
	UInt32 gridVSpacingIndex = [prefs gridVSpacingIndex];
	UInt32 gridHSpacingIndex = [prefs getHorizontalGridSpacing];
	
	// SNAP OPTIONS
	
	if(snapOptionsMask & kPrefsSnapHGrid)
		[fSnapHGridCheckBox setState:NSOnState];
	else
		[fSnapHGridCheckBox setState:NSOffState];

	if(snapOptionsMask & kPrefsSnapHObjects)
		[fSnapHObjCheckBox setState:NSOnState];
	else
		[fSnapHObjCheckBox setState:NSOffState];

	if(snapOptionsMask & kPrefsSnapHLocators)
		[fSnapHLocatorCheckBox setState:NSOnState];
	else
		[fSnapHLocatorCheckBox setState:NSOffState];

	if(snapOptionsMask & kPrefsSnapVGrid)
		[fSnapVGridCheckBox setState:NSOnState];
	else
		[fSnapVGridCheckBox setState:NSOffState];

	if(snapOptionsMask & kPrefsSnapVObjects)
		[fSnapVObjCheckBox setState:NSOnState];
	else
		[fSnapVObjCheckBox setState:NSOffState];
		
	// SHIFT SNAP OPTIONS

/*	
	if(shiftSnapOptionsMask & kPrefsSnapHGrid)
		[fSnapShiftHGridCheckBox setState:NSOnState];
	else
		[fSnapShiftHGridCheckBox setState:NSOffState];

	if(shiftSnapOptionsMask & kPrefsSnapHObjects)
		[fSnapShiftHObjCheckBox setState:NSOnState];
	else
		[fSnapShiftHObjCheckBox setState:NSOffState];

	if(shiftSnapOptionsMask & kPrefsSnapHLocators)
		[fSnapShiftHLocatorCheckBox setState:NSOnState];
	else
		[fSnapShiftHLocatorCheckBox setState:NSOffState];

	if(shiftSnapOptionsMask & kPrefsSnapVGrid)
		[fSnapShiftVGridCheckBox setState:NSOnState];
	else
		[fSnapShiftVGridCheckBox setState:NSOffState];

	if(shiftSnapOptionsMask & kPrefsSnapVObjects)
		[fSnapShiftVObjCheckBox setState:NSOnState];
	else
		[fSnapShiftVObjCheckBox setState:NSOffState];
*/
	// GRID DISPLAY OPTIONS

	if(gridShowMask & kPrefsGridShowVLines)
		[fGridShowVLinesCheckBox setState:NSOnState];
	else
		[fGridShowVLinesCheckBox setState:NSOffState];

	if(gridShowMask & kPrefsGridShowHLines)
		[fGridShowHLinesCheckBox setState:NSOnState];
	else
		[fGridShowHLinesCheckBox setState:NSOffState];
	
	if(gridShowMask & kPrefsGridShowVObjLines)
		[fGridShowVObjLinesCheckBox setState:NSOnState];
	else
		[fGridShowVObjLinesCheckBox setState:NSOffState];

	if(gridShowMask & kPrefsGridShowHObjLines)
		[fGridShowHObjLinesCheckBox setState:NSOnState];
	else
		[fGridShowHObjLinesCheckBox setState:NSOffState];

	[fGridVSpacingPopUp selectItemAtIndex:gridVSpacingIndex];
	[fHorizontalGridSpacing selectItemAtIndex:[fHorizontalGridSpacing indexOfItemWithTag:gridHSpacingIndex]];
	
}

- (void)fillDlgWithPerformanceValues:(MIPreferences*)prefs
{
	UInt32 perfOptionsMask = [prefs perfOptionsMask];
	// PERFORMANCE OPTIONS
	if ( perfOptionsMask & kPrefsPerfThumbsWhenPlaying )
		[fPerfThumbsWhenPlayingCheckBox setState:NSOnState];
	else
		[fPerfThumbsWhenPlayingCheckBox setState:NSOffState];
		
	if ( perfOptionsMask & kPrefsPerfObjectShadows )
		[fPerfShadowsAroundObjectsCheckBox setState:NSOnState];
	else
		[fPerfShadowsAroundObjectsCheckBox setState:NSOffState];

	if ( perfOptionsMask & kPrefsPerfUniformBackground )
		[fPerfUniformBackgroundCheckBox setState:NSOnState];
	else
		[fPerfUniformBackgroundCheckBox setState:NSOffState];

	if ( perfOptionsMask & kPrefsPerfMovingTitles )
		[fPerfMovingTitlesCheckBox setState:NSOnState];
	else
		[fPerfMovingTitlesCheckBox setState:NSOffState];

	[fWorkspaceBackgroundColor setColor:[prefs workspaceBGColor]];
}

- (void)fillDlgWithUndoStackValues:(MIPreferences*)prefs
{
	// UNDO STACK OPTIONS
	int level;
	NSString *msg;
	switch( [prefs undoStackDepth] )
	{
		case 0:		level = 0; break;
		case 1:		level = 1; break;
		case 10:	level = 2; break;
		case 20:	level = 3; break;
		case 30:	level = 4; break;
		case 50:	level = 5; break;
		case 100:	level = 6; break;
		case 200:	level = 7; break;
		case 300:	level = 8; break;
		case 500:	level = 9; break;
		case -1:	level = 10; break;
		default: level = 10; break;
	}
	
	if ( level <= 1)
		msg = [NSString stringWithFormat:@"%d Command", [prefs undoStackDepth]];
	else if ( level != 10 )
		msg = [NSString stringWithFormat:@"%d Commands", [prefs undoStackDepth]];
	else
		msg = @"Infinite";
	[fUndoStackDepthSlider setIntValue:level];
	[fUndoStackDepthCaption setStringValue:msg];
}

- (void)fillDlgWithStackingMethodValues:(MIPreferences*)prefs
{
	// populate the stacking method radio buttons according to the specs
	[fStackVerticallyRadioButton setState:NSOffState];
	[fStackHorizontallyRadioButton setState:NSOffState];
	[fStackAlwaysAskRadioButton setState:NSOffState];

	switch([prefs stackingMethod])
	{
	case kPrefsStackingMethodVertically :
		[fStackVerticallyRadioButton setState:NSOnState];
		break;
	case kPrefsStackingMethodHorizontally :
		[fStackHorizontallyRadioButton setState:NSOnState];
		break;
	case kPrefsStackingMethodAlwaysAsk :
		[fStackAlwaysAskRadioButton setState:NSOnState];
		break;
	default :
		ZzAssert(0);
		break;
	}
}

- (void)fillDlgWithAudioDeviceValues:(MIPreferences*)prefs
{
	MEAudioDeviceDescEnumerator enumerator;
	[fAudioOutputDevicePopUp removeAllItems];
	[fAudioInputDevicePopUp removeAllItems];
	
	// create the arrays that store the codes
	fOutputDevicesStringCodes = [[NSMutableArray alloc] init];
	fInputDevicesStringCodes = [[NSMutableArray alloc] init];
	
	// cache the string codes that are chosen in the prefs
	NSString* chosenInputStringCode = [prefs audioInputDeviceStringCode];
	vernl(6,"fillDlgWithAudioDeviceValues chosenInputStringCode = " << [chosenInputStringCode UTF8String]);

	NSString* chosenOutputStringCode = [prefs audioOutputDeviceStringCode];
	
	// the current string code we're adding
	NSString* currentlyAddedOutputStringCode;
	NSString* currentlyAddedInputStringCode;
					
	// add the "default" entries	
	SInt32 indexOfChosenInput = -1;
	SInt32 indexOfChosenOutput = -1;
	
	// add the popup string
	[fAudioOutputDevicePopUp addItemWithTitle:@"Use the default device"];
	
	// store the soon-to-be-added string code
	currentlyAddedOutputStringCode = [NSString stringWithString:(NSString*)MEAudioDeviceDesc::GetDefaultOutputStringCode()];
	
	// add the string code to the array
	[fOutputDevicesStringCodes addObject:currentlyAddedOutputStringCode];
	
	// check for "chosen output device" index
	if([chosenOutputStringCode isEqualToString:currentlyAddedOutputStringCode])
	{
		ZzAssert(indexOfChosenOutput == -1); // make sure we only find it once
		indexOfChosenOutput = 0;
	//	vernl(0,"index of chosenOutputStringCode is " << indexOfChosenOutput);
	}
		
	// add the popup string
	[fAudioInputDevicePopUp addItemWithTitle:@"Use the default device"];
	
	// store the soon-to-be-added string code
	currentlyAddedInputStringCode = [NSString stringWithString:(NSString*)MEAudioDeviceDesc::GetDefaultInputStringCode()];
	
	// add the string code to the array
	[fInputDevicesStringCodes addObject:currentlyAddedInputStringCode];

	// check for "chosen output device" index
	if([chosenInputStringCode isEqualToString:currentlyAddedInputStringCode])
	{
		ZzAssert(indexOfChosenInput == -1); // make sure we only find it once
		indexOfChosenInput = 0;
//		vernl(0,"index of chosenOutputStringCode is " << indexOfChosenInput);
	}
			
	SInt32 currentIndex = 1; // index 0 in the popup is meant for the "use default device"
	for(SInt32 i = 0; i < enumerator.GetNbInputDevices(); i++)
	{
		// add the popup string
		[fAudioInputDevicePopUp addItemWithTitle:[NSString stringWithString:(NSString*)(enumerator.GetInputDeviceName(i))]];

		// store the soon-to-be-added string code
		currentlyAddedInputStringCode = [NSString stringWithString:(NSString*)(enumerator.GetInputDeviceStringCode(i))];

		// add the string code to the array
		[fInputDevicesStringCodes addObject:currentlyAddedInputStringCode];

		// check for "chosen input device" index
		if([chosenInputStringCode isEqualToString:currentlyAddedInputStringCode])
		{
			ZzAssert(indexOfChosenInput == -1); // make sure we only find it once
			indexOfChosenInput = currentIndex;
		}
		currentIndex++;
	}
	
	currentIndex = 1; // index 0 in the popup is meant for the "use default device"
	for(SInt32 i = 0; i < enumerator.GetNbOutputDevices(); i++)
	{
			// add the popup string
		[fAudioOutputDevicePopUp addItemWithTitle:[NSString stringWithString:(NSString*)(enumerator.GetOutputDeviceName(i))]];
		
		// store the soon-to-be-added string code
		currentlyAddedOutputStringCode = [NSString stringWithString:(NSString*)(enumerator.GetOutputDeviceStringCode(i))];
		
		// add the string code to the array
		[fOutputDevicesStringCodes addObject:currentlyAddedOutputStringCode];

		// check for "chosen output device" index
		if([chosenOutputStringCode isEqualToString:currentlyAddedOutputStringCode])
		{
			ZzAssert(indexOfChosenOutput == -1); // make sure we only find it once
			indexOfChosenOutput = currentIndex;
		}
		currentIndex++;
	}
	
	if(indexOfChosenInput == -1 && indexOfChosenOutput == -1  )
	{
		// this means the string code chosen in the preferences doesn't match
		// an existing device.
		indexOfChosenInput = 0;
		indexOfChosenOutput = 0;
		NSRunAlertPanel(@"Error", @"The Audio Input and Output devices chosen in the preferences cannot be found anymore. The default device will be used. Please choose another device here if you wish.", @"OK", nil, nil);
		[prefs setAudioOutputDeviceStringCode:[NSString stringWithString:(NSString*)MEAudioDeviceDesc::GetDefaultOutputStringCode()]];
		[prefs setAudioInputDeviceStringCode:[NSString stringWithString:(NSString*)MEAudioDeviceDesc::GetDefaultInputStringCode()]];
	}
	else if(indexOfChosenInput == -1)
	{
		// this means the string code chosen in the preferences doesn't match
		// an existing device.
		indexOfChosenInput = 0;
		NSRunAlertPanel(@"Error", @"The Audio Input device chosen in the preferences cannot be found anymore. The default device will be used. Please choose another device here if you wish.", @"OK", nil, nil);
		[prefs setAudioInputDeviceStringCode:[NSString stringWithString:(NSString*)MEAudioDeviceDesc::GetDefaultInputStringCode()]];
	}
	else if(indexOfChosenOutput == -1  )
	{
		// this means the string code chosen in the preferences doesn't match
		// an existing device.
		indexOfChosenOutput = 0;
		NSRunAlertPanel(@"Error", @"The Audio Output device chosen in the preferences cannot be found anymore. The default device will be used. Please choose another device here if you wish.", @"OK", nil, nil);
		[prefs setAudioOutputDeviceStringCode:[NSString stringWithString:(NSString*)MEAudioDeviceDesc::GetDefaultOutputStringCode()]];
	}
	
	[fAudioOutputDevicePopUp selectItemAtIndex:indexOfChosenOutput];
	[fAudioInputDevicePopUp selectItemAtIndex:indexOfChosenInput];
		
/*
	// audio hardware stuff
	IBOutlet id fAudioOutputDevicePopUp;
	IBOutlet id fAudioOutputBufferSizeSlider;
	IBOutlet id fAudioOutputBufferSizeTextBox;

	// audio hardware stuff
	IBOutlet id fAudioInputDevicePopUp;
	IBOutlet id fAudioInputBufferSizeSlider;
	IBOutlet id fAudioInputBufferSizeTextBox;
*/
	

}

-(void)awakeFromNib
{
	MIPreferences* prefs = [MIPreferences sharedPrefs];

	[self fillDlgWithSnapAndGridValues:prefs];
	[self fillDlgWithPerformanceValues:prefs];
	[self fillDlgWithUndoStackValues:prefs];
	[self fillDlgWithStackingMethodValues:prefs];
	[self fillDlgWithAudioDeviceValues:prefs];
	[self fillDlgWithMiscValues:prefs];
	[self fillDlgWithTextClipValues:prefs];
	[self fillDlgWithStillClipValues:prefs];
	[self fillDlgWithPlaybackQualityValues:prefs];
	[self fillDlgWithReferenceStyle:prefs];
    [self fillDlgWithDefaultTransition:prefs];
    [self fillDlgWithSerialNumbers:prefs];
	[[NSColorPanel sharedColorPanel] setFloatAboveModalWindows:YES];
}

- (void)dealloc
{
	[[NSColorPanel sharedColorPanel] setFloatAboveModalWindows:NO];
	[[NSColorPanel sharedColorPanel] orderOut:self];
    [super dealloc];
}

-(BOOL)windowShouldClose:(id)sender
{
	// delete tmp resources for hardware enumeration
	[fOutputDevicesStringCodes release];
	[fInputDevicesStringCodes release];

	fOutputDevicesStringCodes = nil;
	fInputDevicesStringCodes = nil;
	
	[[NSApplication sharedApplication] stopModal];
	return YES;
}

- (IBAction)audioOutputDevicePopUp:(id)sender
{
	// 1st let's shut the mixer down
	[[MIMixerController sharedMixerController] terminateDrawingThread];
	
	MIPreferences* prefs = [MIPreferences sharedPrefs];
	ZzAssert(fOutputDevicesStringCodes != nil);
	NSString* chosenStringCode = [fOutputDevicesStringCodes objectAtIndex:[fAudioOutputDevicePopUp indexOfSelectedItem]];
	[prefs setAudioOutputDeviceStringCode:chosenStringCode];
	
	// step 1 : we retrieve the output device string code as defined in the prerefences
	NSString* outDevStringCode = [[MIPreferences sharedPrefs] audioOutputDeviceStringCode];
	
	// step 2 : translate the string code into a CoreAudio AudioDeviceID
	AudioDeviceID outDevID = MEAudioDeviceDesc::GetAudioDeviceIDFromStringCode((CFStringRef)outDevStringCode);
	
	if(outDevID == 0x00000000) // this means the device hasn't been found
	{	
		NSString* defaultOutDevStringCode = [NSString stringWithString:(NSString*)MEAudioDeviceDesc::GetDefaultOutputStringCode()]; // returns a code that is 'always use default device'
		[[MIPreferences sharedPrefs] setAudioOutputDeviceStringCode:defaultOutDevStringCode];
		outDevID = MEAudioDeviceDesc::GetAudioDeviceIDFromStringCode((CFStringRef)defaultOutDevStringCode);
		
		if(outDevID == 0x00000000)
		{
			NSRunAlertPanel(@"Error", @"No suitable Audio Output Device found.", @"OK", nil, nil);
		}
		else
		{
			NSRunAlertPanel(@"Warning", @"The Audio Output Device chosen in the preferences cannot be opened. HyperEngine-AV now uses the default device.", @"OK", nil, nil);
		}
	}
	
	UInt32 outNbFrames = [prefs audioOutputDeviceBufferNbFrames];
	TArmeMediaEngine::ChangeOutputDevice(outDevID,&outNbFrames);
	if(outNbFrames != [prefs audioOutputDeviceBufferNbFrames])
	{
		// $$$TODO display message here and update the buffer size slider
		[prefs setAudioOutputDeviceBufferNbFrames:outNbFrames];
	}
//	NSRunAlertPanel(@"Info", @"You need to restart HyperEngine-AV for the changes to be applied.", @"OK", nil, nil); //$$$TODO fix this.

	// relaunch mixer
	[[MIMixerController sharedMixerController] updateContent:[[NSDocumentController sharedDocumentController] currentDocument]];
}

- (IBAction)audioOutputBufferSizeSlider:(id)sender
{
	NSRunAlertPanel(@"Error", @"Sorry, but this is not allowed with this device", @"OK", nil, nil);
}

- (IBAction)audioInputDevicePopUp:(id)sender
{
	MIPreferences* prefs = [MIPreferences sharedPrefs];
	ZzAssert(fInputDevicesStringCodes != nil);
	NSString* chosenStringCode = [fInputDevicesStringCodes objectAtIndex:[fAudioInputDevicePopUp indexOfSelectedItem]];
	[prefs setAudioInputDeviceStringCode:chosenStringCode];
	vernl(0,"audioInputDevicePopUp chosenStringCode = " << [chosenStringCode UTF8String]);
}

- (IBAction)audioInputBufferSizeSlider:(id)sender
{
	NSRunAlertPanel(@"Error", @"Sorry, but this is not allowed with this device", @"OK", nil, nil);
}


- (IBAction)setIsFixed:(id)sender {
	[fSeconds setEnabled:YES];
	[fMinutes setEnabled:YES];
	[fTenth setEnabled:YES];
	[fSpeedSlider setEnabled:NO];
	[fSpeedTextField setEnabled:NO];
	[[MIPreferences sharedPrefs] setIsFixedDuration:YES];
}

- (IBAction)setIsAutomatic:(id)sender {
	[fSeconds setEnabled:NO];
	[fMinutes setEnabled:NO];
	[fTenth setEnabled:NO];
	[fSpeedSlider setEnabled:YES];
	[fSpeedTextField setEnabled:YES];
	[[MIPreferences sharedPrefs] setIsFixedDuration:NO];
}


- (IBAction)speedTextFieldChanged:(id)sender {
	float speed = [fSpeedTextField floatValue];
	[fSpeedSlider setFloatValue:speed];
	[[MIPreferences sharedPrefs] setTextClipScrollSpeed:speed];
}

- (IBAction)speedSliderChanged:(id)sender {
	float speed = [fSpeedSlider floatValue];
	speed = lrint( speed * 10.0 ) / 10.0 ;
	[fSpeedTextField setStringValue:[NSString stringWithFormat:@"%.1f", speed]];
	[[MIPreferences sharedPrefs] setTextClipScrollSpeed:speed];
}


- (IBAction)durationTextFieldChanged:(id)sender {
	float duration = [fSeconds intValue] + [fMinutes intValue] * 60 + ((float)[fTenth intValue]) / 10.0 ;
	if ( duration > 3599.9 ) duration = 3599.9 ;
	if ( duration < 0.1 ) duration = 0.1 ;
	[[MIPreferences sharedPrefs] setTextClipDefaultDuration:duration];
	int minutes = (int)(duration / 60);
	int seconds = (int)(duration - minutes * 60);
	[fMinutes setIntValue:minutes];
	[fSeconds setIntValue:seconds];
	[fTenth setIntValue:(int)((duration - minutes * 60 - seconds) * 10)];
}


- (void)fillDlgWithTextClipValues:(MIPreferences*)prefs {
	float duration = [prefs textClipDefaultDuration];
	int minutes = (int)(duration / 60);
	int seconds = (int)(duration - minutes * 60);
	[fSeconds setIntValue:seconds];
	[fMinutes setIntValue:minutes];
	[fTenth setIntValue:(int)((duration - minutes * 60 - seconds) * 10)];

	[fSpeedSlider setFloatValue: [prefs textClipScrollSpeed]];
	[self speedSliderChanged:nil];

	if ( [prefs isFixedDuration] ) {
		[fFixed setState:NSOnState];
		[fAutomatic setState:NSOffState];
		[self setIsFixed:nil];
	} else {
		[fFixed setState:NSOffState];
		[fAutomatic setState:NSOnState];
		[self setIsAutomatic:nil];
	}
}


- (IBAction)durationSCTextFieldChanged:(id)sender {
	float duration = [fSCSeconds intValue] + [fSCMinutes intValue] * 60 + ((float)[fSCTenth intValue]) / 10.0 ;
	if ( duration > 3599.9 ) duration = 3599.9 ;
	if ( duration < 0.1 ) duration = 0.1 ;
	[[MIPreferences sharedPrefs] setStillClipDefaultDuration:duration];
	int minutes = (int)(duration / 60);
	int seconds = (int)(duration - minutes * 60);
	[fSCMinutes setIntValue:minutes];
	[fSCSeconds setIntValue:seconds];
	[fSCTenth setIntValue:(int)((duration - minutes * 60 - seconds) * 10)];
}

- (void)fillDlgWithStillClipValues:(MIPreferences*)prefs {
	float duration = [prefs stillClipDefaultDuration];
	int minutes = (int)(duration / 60);
	int seconds = (int)(duration - minutes * 60);
	[fSCSeconds setIntValue:seconds];
	[fSCMinutes setIntValue:minutes];
	[fSCTenth setIntValue:(int)((duration - minutes * 60 - seconds) * 10)];
}

- (IBAction)highQualityChanged:(id)sender
{
	[[MIPreferences sharedPrefs] setIsHighQuality:( [fHighQualityCheckbox state] == NSOnState )];
}

- (IBAction)singleFieldChanged:(id)sender
{
	[[MIPreferences sharedPrefs] setIsSingleField:( [fSingleFieldCheckbox state] == NSOnState )];
}

- (void)fillDlgWithPlaybackQualityValues:(MIPreferences*)prefs {
	if ( [prefs isHighQuality] ) [fHighQualityCheckbox setState:NSOnState];
	else [fHighQualityCheckbox setState:NSOffState];
	if ( [prefs isSingleField] ) [fSingleFieldCheckbox setState:NSOnState];
	else [fSingleFieldCheckbox setState:NSOffState];
}


- (void)fillDlgWithSerialNumbers:(MIPreferences*)prefs {
}

- (IBAction)referenceExternalFiles:(id)sender
{
	[[MIPreferences sharedPrefs] setReferenceExternalFiles:YES];
}

- (IBAction)copyExternalFile:(id)sender
{
	[[MIPreferences sharedPrefs] setReferenceExternalFiles:NO];
}

- (void)fillDlgWithReferenceStyle:(MIPreferences*)prefs
{
	if ( [prefs referenceExternalFiles] )
	{
		[fReferenceExternalFilesRadio setState:NSOnState];
		[fCopyExternalFilesRadio setState:NSOffState];
	}
	else
	{
		[fReferenceExternalFilesRadio setState:NSOffState];
		[fCopyExternalFilesRadio setState:NSOnState];
	}
	
}

- (void)fillDlgWithDefaultTransition:(MIPreferences*)prefs
{
	if ( [prefs automaticTransitionParameters] == YES )
	{
		[fAutomaticCrossFadeRadio setState:NSOnState];
		[fFixedCrossFadeRadio setState:NSOffState];
	}
	else
	{
		[fAutomaticCrossFadeRadio setState:NSOffState];
		[fFixedCrossFadeRadio setState:NSOnState];
	}
	
	char effectName[256];
	QTAtomContainer effectDesc;
	QTNewAtomContainer(&effectDesc);
	QTCopyAtom([[MIPreferences sharedPrefs] getDefaultTransitionEffectDesc], kParentAtomIsContainer, &effectDesc);

	GetEffectName(effectDesc, effectName);
	[fDefaultTransitionText setStringValue:[NSString stringWithCString:effectName]];
	
	QTDisposeAtomContainer(effectDesc);
}

- (IBAction)setDefaultTransition:(id)sender
{
	QTAtomContainer effectDesc;
	QTNewAtomContainer(&effectDesc);
	QTCopyAtom([[MIPreferences sharedPrefs] getDefaultTransitionEffectDesc], kParentAtomIsContainer, &effectDesc);

	if (ConfigureEffect(effectDesc, 2, NULL, NULL))
	{
		char effectName[256];
		GetEffectName(effectDesc, effectName);

		int* effectBuffer = new int[4096];
		ArmeMemArchiveHandler effectArchive(effectBuffer, 4096, true);
		effectArchive.WriteQTAtomContainer(&effectDesc);
		
		NSData* value = [NSData dataWithBytes:effectBuffer length:effectArchive.GetSizeWritten()];
		[fDefaultTransitionText setStringValue:[NSString stringWithCString:effectName]];
  
		[[MIPreferences sharedPrefs] setDefaultTransition:value];
		
		delete effectBuffer;
	}
}

- (IBAction)automaticTransitionParameters:(id)sender
{
	[[MIPreferences sharedPrefs] setAutomaticTransitionParameters:YES];
}
- (IBAction)fixedTransitionParameters:(id)sender
{
	[[MIPreferences sharedPrefs] setAutomaticTransitionParameters:NO];
}

- (IBAction)validateSerialNumbers:(id)sender
{
}

@end
