/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
#import "MIProcessorRep.h"
#import "MIImageController.h"
#import "MIObject.h"
#import "ARPI.h"
#import "MITopLevelContainerRep.h"

#ifndef TArmeWSObj_h
#include "TArmeWSObj.h"
#endif //TArmeWSObj_h

#import "MIBlueWindowController.h"

#ifndef TArmeMediaEngine_h
#include "TArmeMediaEngine.h"
#endif //TArmeMediaEngine_h

#import "MIApplicationController.h"
#include "cSpline.h"
#import "MIAutomationTrackers.h"
#import "MIMainViewController.h"
#import "MIWSSelection.h"

#import "ARPIPluginView.h"

const float kAutomationKnotHalfSize = 3.0f;


@implementation MIProcessorRep
- (id)initWithFrame:(NSRect)frame inTLC:(MITopLevelContainerRep*)theTLC
{
    self = [super initWithFrame:frame inTLC:theTLC];
	
//	fLeftBarImage = mainImageController->fContainerProcessorBarOnLeft;
//	fCenterBarImage = mainImageController->fContainerProcessorBarOnCenter;
//	fRightBarImage = mainImageController->fContainerProcessorBarOnRight;

	
    return self; 
}

- (void)setImagesAndColor
{
	if ( [self isSelected] && [[[fTLCRep mainViewController] selection] type] == kMIWSSelectionTypeObjects )
	{
		[self setBackgroundColor:mainImageController->fProcessorNegativeColor];
		fLeftTitleImage = mainImageController->fLeftProcessorNegativeTitleBarImage;
		fCenterTitleImage = mainImageController->fCenterProcessorNegativeTitleBarImage;
		fRightTitleImage = mainImageController->fRightProcessorNegativeTitleBarImage;
	}
	else if ( [fRepresentedObject isMuted] )
	{
		[self setBackgroundColor:mainImageController->fProcessorDimColor];
		fLeftTitleImage = mainImageController->fLeftProcessorDimTitleBarImage;
		fCenterTitleImage = mainImageController->fCenterProcessorDimTitleBarImage;
		fRightTitleImage = mainImageController->fRightProcessorDimTitleBarImage;
	}
	else 
	{
		[self setBackgroundColor:mainImageController->fProcessorBrightColor];
		fLeftTitleImage = mainImageController->fLeftProcessorBrightTitleBarImage;
		fCenterTitleImage = mainImageController->fCenterProcessorBrightTitleBarImage;
		fRightTitleImage = mainImageController->fRightProcessorBrightTitleBarImage;
	}
}

- (BOOL)doubleClick:(NSPoint)mouseLoc modifier:(unsigned int)modifier
{
	if ( NSPointInRect(mouseLoc, [self rectForZone:MIObjectRepTitleZone]) ||
		 NSPointInRect(mouseLoc, [self rectForZone:MIObjectRepDragBarZone]) )
	{
		[self showProperties];
		return YES;
	}
	
	return NO;
}


- (MIObjectMouseHandling)handleMoreMouseTracking:(NSPoint)mouseLoc modifier:(unsigned int)modifier type:(NSEventType)type
{
    MIObjectMouseHandling		status = MIContinueMouseHandling;
    NSRect						myTLCFrame = [self getTLCFrame];
	NSRect						automationRect = [self rectForZone:MIObjectRepAutomationZone];
	
	[self invalidateTheMouseTracker]; // au cas ou un tracker n'aurait pas t releas
	
#ifdef AUTOMATION_DATA
	float						curveHeight;
	SInt32						numParam;
	cSpline						*theSpline;
	cSplineKnot					*aKnot;

	if ( NSPointInRect(mouseLoc, automationRect) )
	{
		if ( type == NSRightMouseDown || (modifier & NSControlKeyMask ) )
		{
				// bring up contextual menu
				status = MIStopMouseHandling;
				// $$$TODO create a different pop-up menu
				[self popUpMenu:[[MIApplicationController sharedApplicationController] localEditMenu] atPos:[self convertPointToTLCCoords:mouseLoc]];
		}
		
		// check in what part of the automation data we lie
		curveHeight = [fTLCRep MIVPosToPixel:kRepAutomationCurveHeight];
		numParam = floor((mouseLoc.y - NSMinY(automationRect))/curveHeight);
		
		// get the spline we clicked in
		theSpline = [fRepresentedObject WSObject]->GetAudioAutomationData()->GetSplineForParameter(numParam);

		// check if an existing point is arund the mouse position
		aKnot = theSpline->knotAround( [fTLCRep pixelToArmeTime:mouseLoc.x],
										fmod(mouseLoc.y - NSMinY(automationRect),curveHeight)/curveHeight,
										[fTLCRep pixelToArmeTime:kAutomationKnotHalfSize],
										kAutomationKnotHalfSize/curveHeight );
		
		// move the knot if it exists
		// or create a new one and move it if it doesn't
		if ( !aKnot )
		{
			aKnot = new cSplineKnot( [fTLCRep pixelToArmeTime:mouseLoc.x], fmod(mouseLoc.y - NSMinY(automationRect), curveHeight));
			aKnot->setInterpolationType( cSplineCubicInterpolation );
			theSpline->addKnot(aKnot);
		}
		
		status = MIAskMeATracker;
		NSRect bRect = [self rectForZone:MIObjectRepAutomationZone withObjectOrigin:myTLCFrame.origin];
		bRect.origin.y += numParam * curveHeight;
		bRect.size.height = curveHeight;
		
		fTemporaryTracker = [[MIMoveAutomationKnotTracker alloc] initWithView:fTLCRep atPos:[self convertPointToTLCCoords:mouseLoc] withModifiers:modifier];
		[fTemporaryTracker setSpline:theSpline knot:aKnot obj:self boundRect:bRect];
		
	}
#endif

	if ( status == MIContinueMouseHandling )
		status = [super handleMoreMouseTracking:mouseLoc modifier:modifier type:type];

	return status;
}

- (void)drawBaseAutomationRect:(NSRect)zoneInDrawCoords clippedTo:(NSRect)clippingRect
{
#ifdef AUTOMATION_DATA
	cSpline		*aSpline;
	ArmeTime	t;
	ArmeTime	inc = [fTLCRep pixelToArmeTime:1];
	ArmeTime	start = [fTLCRep pixelToArmeTime:NSMinX(zoneInDrawCoords)];
	ArmeTime	stop = [fTLCRep pixelToArmeTime:NSMaxX(zoneInDrawCoords)];
	int			nbParams = [fRepresentedObject WSObject]->GetAudioPluginClass()->GetNbParams();
	float		splineHeight = [fTLCRep MIVPosToPixel:kRepAutomationCurveHeight];
	float		lastValue, newValue, x;
	float		baseY;
	float		baseX = NSMinX(zoneInDrawCoords);
	bool		isRecorded;
	
	
	stop -= start;
	GET_VALUE(start) = 0;
	
	if ( [self isSelected] && [[[fTLCRep mainViewController] selection] type] == kMIWSSelectionTypeObjects )
		[NegativeOfColor([NSColor blueColor]) set];
	else
		[[NSColor blueColor] set];
	[NSBezierPath fillRect:NSIntersectionRect(zoneInDrawCoords, clippingRect)];

	for ( int i = 0; i < nbParams; i++ )
	{
		aSpline = [fRepresentedObject WSObject]->GetAudioAutomationData()->GetSplineForParameter(i);
		
		// draw the spline
		x = 0;
		lastValue = aSpline->splineValue(start, isRecorded) * splineHeight;
		baseY = NSMinY(zoneInDrawCoords)+i*splineHeight;
		[[NSColor grayColor] set];
		[NSBezierPath strokeLineFromPoint:NSMakePoint(baseX, baseY) toPoint:NSMakePoint(NSMaxX(zoneInDrawCoords), baseY)];
		if ( [self isSelected] && [[[fTLCRep mainViewController] selection] type] == kMIWSSelectionTypeObjects )
			[NegativeOfColor([NSColor yellowColor]) set];
		else
			[[NSColor yellowColor] set];
		for ( t = start; t <= stop; t += inc, x++ )
		{
			newValue = aSpline->splineValue(t, isRecorded) * splineHeight;
			
			[NSBezierPath strokeLineFromPoint:NSMakePoint(baseX+x-1, baseY+lastValue)
									toPoint:NSMakePoint(baseX+x, baseY+newValue)];
			lastValue = newValue;
		}

		if ( fSelected )
		{
			// draw the knots
			if ( [self isSelected] && [[[fTLCRep mainViewController] selection] type] == kMIWSSelectionTypeObjects )
				[[NSColor blackColor] set];
			else
				[[NSColor whiteColor] set];
			
			cSplineKnot *aKnot = aSpline->firstKnot();
			while (aKnot)
			{
				NSRect		knotRect = NSZeroRect;
				
				knotRect.origin.x = baseX + [fTLCRep armeTimeToPixel:aKnot->time()]; 
				knotRect.origin.y = baseY + (aKnot->value() * splineHeight);
				
				knotRect = NSInsetRect( knotRect, -kAutomationKnotHalfSize, -kAutomationKnotHalfSize);
				
				[NSBezierPath fillRect:knotRect];
				
				aKnot = aKnot->nextKnot();
			}
		}

/*
		[NSBezierPath strokeLineFromPoint:NSMakePoint(NSMinX(zoneInDrawCoords), NSMinY(zoneInDrawCoords)+[fTLCRep MIVPosToPixel:(i*kRepAutomationCurveHeight)])
								toPoint:NSMakePoint(NSMaxX(zoneInDrawCoords), NSMinY(zoneInDrawCoords)+[fTLCRep MIVPosToPixel:((i+1)*kRepAutomationCurveHeight)])];
*/		
		
	}
#endif
}


- (void)showProperties
{
	ARPIPluginView*			pluginView;
	NSRect 					winContent;
//	VARPIPlugin				*plugin;

	if ( fBlueWindow )
	{
		/*TArmeMediaEngine::ARPIHost()->DeleteView(fBlueWindowView);
		[fBlueWindow close];
                [fBlueWindow release];
		fBlueWindow = nil;*/
            [fBlueWindow makeKeyAndOrderFront:nil];
            return;
	}
	
	TArmeWSObj* processor = [[self representedObject] WSObject];

	if ( processor->GetType() != TArmeWSObj::kAudioProcessor)
	{
		vernl(1,"WARNING : showProperties called on the representation of a non-kAudioProcessor");
		return;
	}
/*
	bwCtrl = [[MIBlueWindowController alloc] initWithWindowNibName:@"MIBlueWindow"];
	[bwCtrl setWSObject:[fRepresentedObject WSObject]];
	[bwCtrl initialize];
	[[bwCtrl window] makeKeyAndOrderFront:nil];
	[[bwCtrl window] setTitle:[fRepresentedObject title]];
	
	fBlueWindow = bwCtrl;
*/

//	return;
	pluginView = (NSView*)(TArmeMediaEngine::ARPIHost()->CreateProcessorView([self representedObject]));
	// and dirtify the document when the user changes the parameters
	
	winContent.origin.x = 100;
	winContent.origin.y = 100;
	winContent.size = [pluginView bounds].size;

	NSWindow* win = [[NSPanel alloc] initWithContentRect:winContent 
		styleMask:NSTitledWindowMask | NSMiniaturizableWindowMask | NSClosableWindowMask
		backing:NSBackingStoreBuffered defer:NO];
	
	fBlueWindow = win;
	fBlueWindowView = pluginView;
	
	[win setContentSize:[pluginView bounds].size];
	[win setTitle:[NSString stringWithFormat:@"Parameters of %@",[fRepresentedObject title]]];
	
	[[win contentView] addSubview:pluginView];
	[win setDelegate:self];
	[win setInitialFirstResponder:[pluginView firstFocusControl]];
	
	[win makeKeyAndOrderFront:nil];

}

- (void)windowWillClose:(NSNotification*)notification
{
	// WARNING : at App quit time, the media engine is destroyed BEFORE the windowWillClose
	// notification, thus we can only use ARPIHost and friends if the media engine is still alive
	if(TArmeMediaEngine::IsAlive())
	{
		TArmeMediaEngine::ARPIHost()->DeleteView(fBlueWindowView);
	}
    [fBlueWindow autorelease];
    fBlueWindow = nil;
}

- (void)closeAndDeleteBlueWindow
{
    if ( fBlueWindow )
    {
        TArmeMediaEngine::ARPIHost()->DeleteView(fBlueWindowView);
        [fBlueWindow close];
        [fBlueWindow release];
        fBlueWindow = nil;
    }
}

- (void)dealloc
{
	[self closeAndDeleteBlueWindow];
    [super dealloc];
}


@end
