/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
#define MIProgressBarDlgManager_h
#ifndef MIProgressBarDlgManager_h
#include "MIProgressBarDlgManager.h"
#endif //MIProgressBarDlgManager_h


#ifndef ZzTypes_h
#include "ZzTypes.h"
#endif //ZzTypes_h

#ifndef ZzVerbose_h
#include "ZzVerbose.h"
#endif //ZzVerbose_h

#ifndef ZzProgressMngrLogic_h
#include "ZzProgressMngrLogic.h"
#endif //ZzProgressMngrLogic_h

#ifndef ZzProgressMngrLogic2_h
#include "ZzProgressMngrLogic2.h"
#endif //ZzProgressMngrLogic2_h


/*
Sample code :
*************

	MIProgressBarDlgManager mngr(3);	// 3 is the number of sub-tasks  (Default value = 1)
	mngr.DisplayDlg("Dlg message",true);	// true to display a 'cancel' button
	bool bContinue;
	
	for (int j=0; j < 3; j++)
	{
		mngr.EnterSubTask(); 
		for(int i = 0; i < 20; i++)
		{
			usleep(100000);
			bContinue = mngr.Update((float)(i+1)/20.0);  // progress is always between 0 and 1 
			if(!bContinue)
				break;
		}
		mngr.LeaveSubTask(); 
	}
	mngr.Delete();	// note : there is an automatic delete in the destructor (but you can call Delete twice)
	if(!bContinue)
		NSRunAlertPanel(@"Information",@"This task has been cancelled",nil,nil,nil);
*/

class MIProgressBarDlgManager
{
	ZzProgressMngrLogic2 fLogic;
	
	bool	fHasBeenDeleted;
	bool	fHasBeenDisplayed;
	void	*fKeyWindow;
	bool	fDisableCancelButton;

public :
	void	DisplayDlg();
	// throws an excepion if cancel has been pressed
	void	Update(float progress);
	void	Delete();
	
	void	SetNbSubTasks(SInt32 nbSubTasks);
	void	Hide();
	void	EnterSubTask();
	void	LeaveSubTask();
	
	MIProgressBarDlgManager(const char* title, bool disableCancelButton = false);
	~MIProgressBarDlgManager();

private :
	bool				fDoNotDisplay;
#ifdef __OBJC__
	static id 			fsProgressBarDlgCtrller;
	NSModalSession 		fSession;
	NSWindow* 			fDlgWindow;
	NSString*			fTitle;
#else //#ifdef __OBJC__
	static void* 		fsProgressBarDlgCtrller;
	void*				fSession; //$$$TODO make sure NSModalSession is a pointer in future Cocoa releases otherwise sizeof will be different
	void*				fDlgWindow;
	void*				fTitle;
#endif //#ifdef __OBJC__
};





