/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MISnapper.h"
#import "MIMainViewController.h"
#import "MIMontageDocument.h"
#import "MIRuler.h"
#import "MIPreferences.h"
#import "MISnapperDB.h"
#import "MITime.h"

#ifndef TArmeDoc_h
#include "TArmeDoc.h"
#endif //TArmeDoc_h

#define kPixelSnapping 14

#undef VER_DEF_TOPIC
#define VER_DEF_TOPIC interfaceBenjamin

@implementation MISnapper

- (id)initWithMainViewController:(MIMainViewController*)controller
{
	self = [super init];
	if(self)
		fController = controller;
	return self;
}

// rectBySnappingRect --> snapRectByShifting

/*! @function snapRectByShifting returns a shifted snapped rectangle */
- (MIRect)snapRectByShifting:(MIRect)aRect modifiers:(unsigned int)modifiers;
{
	if(modifiers & NSCommandKeyMask)
		return aRect;

//	vernl(6,"snapRectByShifting:(aRect = " << aRect << ") modifiers:(modifiers = " << modifiers << ")");
	MIRect newRect;
	ArmeTime start = aRect.origin.time;
	ArmeTime end;
	end.data = start.data + aRect.size.duration.data;
	MIVPos bottom = aRect.origin.vpos;
	MIVPos top;
	top.data = bottom.data + aRect.size.height.data;
	
	ArmeTime newStart,newEnd,snappedStart,snappedEnd;
	MIVPos newTop,newBottom,snappedTop,snappedBottom;
	
	float pixDeltaStart,pixDeltaEnd,pixDeltaTop,pixDeltaBottom; 
				
	snappedStart = [self snapTime:start pixDelta:&pixDeltaStart modifiers:modifiers];
	snappedEnd = [self snapTime:end pixDelta:&pixDeltaEnd modifiers:modifiers];

	if(pixDeltaStart <= pixDeltaEnd) // start has snapped
	{
		newStart = snappedStart;
		newEnd = end;
		// let's shift the end by the same amount
		newEnd.data += (newStart.data - start.data);
	}
	else
	{
		newEnd = snappedEnd;
		newStart = start;
		// let's shift the end by the same amount
		newStart.data += (newEnd.data - end.data);
	}

	snappedBottom = [self snapVPos:bottom pixDelta:&pixDeltaBottom modifiers:modifiers];
	snappedTop = [self snapVPos:top pixDelta:&pixDeltaTop modifiers:modifiers];
	
	if(pixDeltaBottom <= pixDeltaTop) // bottom has snapped
	{
		newBottom = snappedBottom;
		newTop = top;
		// let's shift the end by the same amount
		newTop.data += (newBottom.data - bottom.data);
	}
	else
	{
		newTop = snappedTop;
		newBottom = bottom;
		// let's shift the end by the same amount
		newBottom.data += (newTop.data - top.data);
	}

	newRect.origin.time = newStart;
	newRect.origin.vpos = newBottom;
	newRect.size.duration = _ArmeTime(newEnd.data - newStart.data);
	newRect.size.height = _MIVPos(newTop.data - newBottom.data);
	ZzAssert(newRect.size.duration.data == aRect.size.duration.data);
	ZzAssert(newRect.size.height.data == aRect.size.height.data);
	return newRect;
}

- (ArmeTime)snapTime:(ArmeTime)time
	pixDelta:(float*)pixDelta // if pixDelta is < -1, then it means it hasn't snapped
	modifiers:(unsigned int)modifiers
{
	return [self snapTime:time pixDelta:pixDelta modifiers:modifiers avoidLocatorIndex:-4]; // locator -4 does not exist so it doesn't have any effect (-3 -> +inf do)
}
	
/*! @function snapTime snaps a time horizontally to the grid, the markers and the object bounds */ 
- (ArmeTime)snapTime:(ArmeTime)armeTime
	pixDelta:(float*)pixDelta // if pixDelta is < -1, then it means it hasn't snapped
	modifiers:(unsigned int)modifiers
	avoidLocatorIndex:(SInt32)avoidLocatorIndex // -3 : lloc -2 play -1 rloc >= 0 marker
{
	*pixDelta = 2000.0f;
	ArmeTime retTime = armeTime;
	if(modifiers & NSCommandKeyMask)
		return retTime;
			
//	vernl(6,"snapTime:(armeTime = " << armeTime << ") modifiers:(modifiers = " << modifiers);
	SInt64 time = GET_VAL(armeTime);

	MIMontageDocument* doc = [fController document];
	TArmeDoc* armeDoc = [doc armeDocument];

//	verntl(interfaceStephan,3,"snapTime SRC = " << time);
	UInt32 snapFlags;
	snapFlags = [[MIPreferences sharedPrefs] snapOptionsMask];
	
/*
	if(modifiers & NSShiftKeyMask)
		snapFlags = [[MIPreferences sharedPrefs] shiftSnapOptionsMask];
	else
*/
		
	double quantValPrev,quantValNext,deltaPrev; 

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// snap horizontally to the grid
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
	
//	vernl(6,"checking for h grid snapping snapping");
	if(snapFlags & kPrefsSnapHGrid)
	{
		double gridSpacing; // the pixel delta for grid snapping
		
		if ( [[MIPreferences sharedPrefs] getHorizontalGridSpacing] != 0 )
		{
			MITime	*t;
			// if it's not 0 (automatic), it's a fraction of a bar
			t = [MITime timeWithDocument:[fController document]];
			/*
			[t setBars:2 beats:1 nb16ths:1 ppq:0];
			t->fTheTime /= [[MIPreferences sharedPrefs] getHorizontalGridSpacing];
			*/
			[t setDoubleTime:0];
			switch ( [[MIPreferences sharedPrefs] getHorizontalGridSpacing] )
			{
				case 1:
					[t setSeconds:1];
					break;
				case 2:
					[t setSeconds:10];
					break;
				case 3:
					[t setSeconds:30];
					break;
				case 4:
					[t setMinutes:1];
					break;
				case 5:
					[t setMinutes:2];
					break;
				default:
					[t setSeconds:1];
					break;
			}
			gridSpacing = [t getDoubleTime];			
//			vvernl(6,gridSpacing);
		}
		else if ( [[fController ruler] minSubTimeUnit] )
		{
			gridSpacing = [[[fController ruler] minSubTimeUnit] getDoubleTime];
//			vvernl(6,gridSpacing);
		}
		else
		{
			gridSpacing = [[[fController ruler] minTimeUnit] getDoubleTime];
//			vvernl(6,gridSpacing);
		}

		ZzAssert(gridSpacing != 0);

		quantValPrev = (double)((SInt64)((double)time/gridSpacing))*gridSpacing;
//		vvernl(6,quantValPrev);
		quantValNext = quantValPrev + gridSpacing;
//		vvernl(6,quantValNext);
		deltaPrev = fmod(time, gridSpacing);
//		vvernl(6,deltaPrev);

		double pixDeltaPrev = [fController doubleTimeToPixel:deltaPrev];
//		vvernl(6,pixDeltaPrev);
		double pixHGridSpacing = [fController doubleTimeToPixel:gridSpacing];
//		vvernl(6,pixHGridSpacing);

		if ( pixDeltaPrev < kPixelSnapping )
		{
			// snap if no snap yet  (*pixDelta <= -1) or if this snap is closer
			if( pixDeltaPrev < *pixDelta)
			{
				*pixDelta = pixDeltaPrev;
				retTime = _ArmeTime((SInt64)quantValPrev);
			}
		}
			
		if ( pixHGridSpacing - pixDeltaPrev < kPixelSnapping )
		{
			// snap if no snap yet (*pixDelta <= -1) or if this snap is closer
			if( (pixHGridSpacing - pixDeltaPrev) < *pixDelta)
			{
				*pixDelta = (pixHGridSpacing - pixDeltaPrev);
				retTime = _ArmeTime((SInt64)quantValNext);
			}
		}
//		vvernl(6,retTime);
	}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// snap horizontally to the locators and markers
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
	
//	vernl(6,"checking for locators snapping");
	if(snapFlags & kPrefsSnapHLocators)
	{
		// loop on all markers and make a dynamic array instead;
		ArmeTime locs[3];
		SInt32 i;
		
		armeDoc->LockRead();
		locs[0] = armeDoc->GetLLocator();
		locs[1] = armeDoc->GetPlayPos();
		locs[2] = armeDoc->GetRLocator();
		armeDoc->UnlockRead();
		for(i= 0; i < 3; i++)
		{
			if(i-3 == avoidLocatorIndex)
				continue;
			SInt64 delta = time - locs[i].data;
//			vvernl(6,delta);

			if(delta < 0)
				delta = -delta;
			float locPixDelta = [fController armeTimeToPixel:_ArmeTime(delta)];

//			vvernl(6,pixDelta);
			if(locPixDelta < kPixelSnapping)
			{
				// snap if no snap yet  (*pixDelta <= -1) or if this snap is closer
				if(locPixDelta < *pixDelta)
				{
					*pixDelta = locPixDelta;
					retTime = _ArmeTime((SInt64)locs[i].data);
				}
//				vvernl(6,retTime);
			}
		}
		
		SInt32 nbMarkers = [doc getMarkerCount];
		for(i = 0; i < nbMarkers; i++)
		{
			ArmeTime markStartTime = [doc getMarker:i];
			SInt64 delta = time - markStartTime.data;

			if(delta < 0)
				delta = -delta;
			float locPixDelta = [fController armeTimeToPixel:_ArmeTime(delta)];

			if(locPixDelta < kPixelSnapping)
			{
				// snap if no snap yet  (*pixDelta <= -1) or if this snap is closer
				if(locPixDelta < *pixDelta)
				{
					*pixDelta = locPixDelta;
					retTime = _ArmeTime((SInt64)markStartTime.data);
				}
			}
		}
	}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// snap horizontally to the object bounds
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
	
	if(snapFlags & kPrefsSnapHObjects)
	{
		NSEnumerator* snapObjEnum = [[doc snapperDB] objectEnumerator];
		MIObjectBoundsInfo* snapInfo;
		while(snapInfo = [snapObjEnum nextObject])
		{
			SInt64 delta;
			float objPixDelta;
						
			// snap to start ?
			delta = time - GET_VAL(snapInfo->fStart);
			if(delta < 0)
				delta = -delta;
			objPixDelta = [fController armeTimeToPixel:_ArmeTime(delta)];
			if(objPixDelta < kPixelSnapping)
			{
				// snap if no snap yet  (*pixDelta <= -1) or if this snap is closer
				if(objPixDelta < *pixDelta)
				{
					*pixDelta = objPixDelta;
					retTime = snapInfo->fStart;
				}
			}
			
			// snap to end ?
			delta = time - GET_VAL(snapInfo->fEndx);
			if(delta < 0)
				delta = -delta;
			objPixDelta = [fController armeTimeToPixel:_ArmeTime(delta)];
			if(objPixDelta < kPixelSnapping)
			{
				// snap if no snap yet  (*pixDelta <= -1) or if this snap is closer
				if( (*pixDelta <= -1.0f) || (objPixDelta < *pixDelta))
				{
					*pixDelta = objPixDelta;
					retTime = snapInfo->fEndx;
				}
			}
		}
	}
//	verntl(interfaceStephan,3,"snapTime VALUE = " << retTime);
//	printf("snapTime return value is : %d\n", (SInt32)GET_VAL(retTime));
	return retTime;
}

/*! @function snapVPos snaps a position vertically */ 
- (MIVPos)snapVPos:(MIVPos)_vpos
	pixDelta:(float*)pixDelta // if pixDelta is > 1000.0, then it means it hasn't snapped
	modifiers:(unsigned int)modifiers;
{
	UInt32 snapFlags;
	SInt32 vpos = _vpos.data;
	SInt32 retVPos = vpos;
	*pixDelta = 2000;

	if(modifiers & NSCommandKeyMask)
		return _MIVPos(retVPos);

	MIMontageDocument* doc = [fController document];
//	TArmeDoc* armeDoc = [doc armeDocument];
	
	snapFlags = [[MIPreferences sharedPrefs] snapOptionsMask];

	SInt32 quantValPrev,quantValNext,deltaPrev; 

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// snap vertically to the grid
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////

	if(snapFlags & kPrefsSnapVGrid)
	{
		MIPreferences* prefs = [MIPreferences sharedPrefs];
		SInt32 gridVSpacing = [prefs gridVSpacing].data;
		float pixGridVSpacing = [fController MIVPosToPixel:_MIVPos(gridVSpacing)];
			
		quantValPrev = (vpos/gridVSpacing)*gridVSpacing;
		quantValNext = quantValPrev + gridVSpacing;
		deltaPrev = vpos % gridVSpacing;
		float pixDeltaPrev = [fController MIVPosToPixel:_MIVPos(deltaPrev)];
		if(pixDeltaPrev < kPixelSnapping)
		{
			if(pixDeltaPrev < *pixDelta)
			{
				*pixDelta = pixDeltaPrev;
				retVPos = quantValPrev;
			}
		}
		if(pixGridVSpacing - pixDeltaPrev < kPixelSnapping)
		{
			if( (pixGridVSpacing - pixDeltaPrev) < *pixDelta)
			{
				*pixDelta = (pixGridVSpacing - pixDeltaPrev);
				retVPos = quantValNext;
			}
		}
	}
	
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// snap vertically to the object bounds
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
	
	if(snapFlags & kPrefsSnapVObjects)
	{
		NSEnumerator* snapObjEnum = [[doc snapperDB] objectEnumerator];
		MIObjectBoundsInfo* snapInfo;
		while(snapInfo = [snapObjEnum nextObject])
		{
			SInt32 delta;
			float pixDeltaObj;
						
			// snap to top ?
			delta = vpos - GET_VAL(snapInfo->fTopx);
			if(delta < 0)
				delta = -delta;
			pixDeltaObj = [fController MIVPosToPixel:_MIVPos(delta)];
			if(pixDeltaObj < kPixelSnapping)
			{
				if(pixDeltaObj < *pixDelta)
				{
					*pixDelta = pixDeltaObj;
					retVPos = snapInfo->fTopx.data;
				}
			}

			// snap to body top ?
			delta = vpos - GET_VAL(snapInfo->fBodyTopx);
			if(delta < 0)
				delta = -delta;
			pixDeltaObj = [fController MIVPosToPixel:_MIVPos(delta)];
			if(pixDeltaObj < kPixelSnapping)
			{
				if(pixDeltaObj < *pixDelta)
				{
					*pixDelta = pixDeltaObj;
					retVPos = snapInfo->fBodyTopx.data;
				}
			}

			// snap to bottom ?
			delta = vpos - GET_VAL(snapInfo->fBottom);
			if(delta < 0)
				delta = -delta;
			pixDeltaObj = [fController MIVPosToPixel:_MIVPos(delta)];
			if(pixDeltaObj < kPixelSnapping)
			{
				if(pixDeltaObj < *pixDelta)
				{
					*pixDelta = pixDeltaObj;
					retVPos = snapInfo->fBottom.data;
				}
			}
		}
	}
	return _MIVPos(retVPos);
}


@end
