/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MISnapperDB.h"
#import "MIMontageDocument.h"

#ifndef TArmeWSObj_h
#include "TArmeWSObj.h"
#endif //TArmeWSObj_h

@implementation MIObjectBoundsInfo

-(id)initWithMIObject:(MIObject*)obj
{
	self = [super init];
	if(self)
	{
		fObj = obj;
		[self refresh];
	}
	return self;
}

-(void)refresh
{
	fIsContainer = [fObj isContainer];
	TArmeWSObj* wsobj = [fObj WSObject];
	fStart = wsobj->GetAbsoluteStartTime();
	fEndx = wsobj->GetAbsoluteStartTime() + wsobj->GetDuration();
	MIVPos absVerticalPos = [fObj getAbsoluteVerticalPos];
	fBottom = absVerticalPos;
	fTopx.data = absVerticalPos.data + [fObj getHeight].data;
	// $$$SAIBOT-2002-11-14-01
	// DONE saibot, please implement the new title bar model for MIObjectBoundsInfo::fBodyTopx to be computed correctly");
	fBodyTopx.data = absVerticalPos.data + [fObj getHeight].data - [fObj getTitleHeight].data;
}

@end

@implementation MISnapperDB

-(id)init
{
	self = [super init];
	if(self)
	{
		fObjBoundsInfoArray = [[NSMutableArray alloc] init];
	}
	return self;
}

-(void)dealloc
{
	[fObjBoundsInfoArray release];
	[super dealloc];
}

#define SNAPPER_DB_AS_A_WHOLE

#ifdef SNAPPER_DB_AS_A_WHOLE

-(void)refreshWholeSnapDB:(MIMontageDocument*)doc
{
	// let's empty the array
	[fObjBoundsInfoArray removeAllObjects];
	
	MIObject* tlc = [doc topLevelContainer]; 
	NSArray* tlcChildren = [tlc getChildren];
	int countTLCChildren = [tlcChildren count];
	MIObject* tlcChild = nil;
	for(int iChild = 0; iChild < countTLCChildren; iChild++)
	{
		tlcChild = [tlcChildren objectAtIndex:iChild];
		[self refreshSnapDBRecursive:tlcChild];
	}
}

-(void)refreshSnapDBRecursive:(MIObject*)obj
{
	MIObjectBoundsInfo* objSnapInfo = [[MIObjectBoundsInfo alloc] initWithMIObject:obj]; 
	[fObjBoundsInfoArray addObject:objSnapInfo];
	[objSnapInfo release];
	
	NSArray* children = [obj getChildren];
	int countChildren = [children count];
	MIObject* child = nil;
	for(int iChild = 0; iChild < countChildren; iChild++)
	{
		child = [children objectAtIndex:iChild];
		[self refreshSnapDBRecursive:child];
	}
}

-(void)refreshSnapDB:(MIObject*)obj
{
}

-(void)deleteFromSnapDB:(MIObject*)obj
{
}

#else //#ifdef SNAPPER_DB_AS_A_WHOLE

-(void)refreshWholeSnapDB:(MIMontageDocument*)doc
{
	ZzAssert(0);
}

-(void)refreshSnapDB:(MIObject*)obj
{
	MIObjectBoundsInfo* foundObjBoundsInfo = nil;
	MIObjectBoundsInfo* objBoundsInfo;
	NSEnumerator* enumerator = [fObjBoundsInfoArray objectEnumerator];
	while(objBoundsInfo = [enumerator nextObject])
	{
		if(objBoundsInfo->fObj == obj)
		{
			foundObjBoundsInfo = objBoundsInfo;
			break;
		}
	}
	if(foundObjBoundsInfo == nil)
	{
		[fObjBoundsInfoArray addObject:[[[MIObjectBoundsInfo alloc] initWithMIObject:obj] autorelease]];
	}
	else
	{
		[foundObjBoundsInfo refresh];
	}
}

-(void)deleteFromSnapDB:(MIObject*)obj
{
	MIObjectBoundsInfo* foundObjBoundsInfo = nil;
	MIObjectBoundsInfo* objBoundsInfo;
	NSEnumerator* enumerator = [fObjBoundsInfoArray objectEnumerator];
	while(objBoundsInfo = [enumerator nextObject])
	{
		if(objBoundsInfo->fObj == obj)
		{
			foundObjBoundsInfo = objBoundsInfo;
			break;
		}
	}
	if(foundObjBoundsInfo == nil)
	{
		verntl(interfaceStephan,0,"trying to call deleteFromSnapDB with an object that is NOT in the database -> CRASH");
		ZzAssert(0);
	}
	else
	{
		[fObjBoundsInfoArray removeObject:foundObjBoundsInfo];
	}
}

#endif //#ifdef SNAPPER_DB_AS_A_WHOLE

-(NSEnumerator*)objectEnumerator
{
	return [fObjBoundsInfoArray objectEnumerator];
}

@end
