/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MISplash.h"
#import "MIDocumentController.h"
#import "MIApplicationController.h"
#import "MIApplication.h"

#include <unistd.h>

#ifdef DEBUG
#ifndef MTG_LICENSER_KIT_DEBUG
#define MTG_DEBUG_VERSION
#endif
#endif

#if defined( MTG_USE_LICENSER_KIT)
#if defined( MTG_LIGHT_VERSION ) || defined( MTG_FULL_VERSION ) || defined ( MTG_DEBUG_VERSION )
#import "CSOneWay.h"
#import "MainApp.h"
#endif
#endif //#if defined( MTG_USE_LICENSER_KIT)

NSString *versionString = @"v. open source 1.6.1";

@interface MISplash (MISplashPrivateMethod)
- (void)_showSplash;
@end

@implementation MISplash

+ (void)splash
{
    [[[self alloc] init] _showSplash];
}

- (void)_about {
	if ([NSBundle loadNibNamed:@"Splash" owner:self]) {
		fSplashWindow = [[NSWindow alloc] initWithContentRect:[fSplashView frame] styleMask:NSBorderlessWindowMask backing:NSBackingStoreBuffered defer:NO];
		[fSplashWindow setContentView:fSplashView];
		[fSplashView release];


		[fText setStringValue:@""];
		[fRefNum setStringValue:@"Full Version"];
		[fName setStringValue:@""];
		[fCompany setStringValue:@""];
		[fVersion setStringValue:versionString];



		[fBox retain];
		[fBox removeFromSuperview];
		[fImage addSubview:fBox];
		[fBox setFrame:NSMakeRect(288,256,280,62)];
		[fBox release];
		[fSplashWindow setHasShadow:YES];
		[fSplashWindow centerOnMenubarScreen];
		[fSplashWindow orderFront:nil];
		[fSplashWindow display];
		[NSApp nextEventMatchingMask:NSLeftMouseDownMask | NSKeyDownMask untilDate:[NSDate distantFuture] inMode:NSDefaultRunLoopMode dequeue:YES];
		[fSplashWindow orderOut:nil];
		[self release];
    }
}

+ (void)aboutbox {
	[[[self alloc] init] _about];
}

- (void)_showSplash {
    NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	
    [MIApplication sharedApplication];
   
    [MIDocumentController sharedDocumentController];	
	[NSFontManager setFontManagerFactory:NSClassFromString(@"MIFontManager")];
    if ([NSBundle loadNibNamed:@"Splash" owner:self]) {
        fSplashWindow = [[NSWindow alloc] initWithContentRect:[fSplashView frame] 
            styleMask:NSBorderlessWindowMask backing:NSBackingStoreBuffered defer:NO];
        [fSplashWindow setContentView:fSplashView];
        [fSplashView release];
    
        [fText setStringValue:@""];
    #ifndef MTG_DEBUG_VERSION
    #ifdef MTG_DEMO_VERSION
        [fRefNum setStringValue:@"Demo Version"];
    #else	
        [fRefNum setStringValue:@""];
    #endif
    #else
        [fRefNum setStringValue:@"Debug build"];
    #endif
        [fName setStringValue:@""];
        [fCompany setStringValue:@""];
        [fVersion setStringValue:versionString];
    
        [fBox retain];
        [fBox removeFromSuperview];
        [fImage addSubview:fBox];
        [fBox setFrame:NSMakeRect(288,256,280,62)];
        [fBox release];
        
    
        [fSplashWindow setHasShadow:YES];
        [fSplashWindow centerOnMenubarScreen];
        [fSplashWindow orderFront:nil];
        [fSplashWindow display];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(_applicationWillFinishLaunching:) name:NSApplicationWillFinishLaunchingNotification 						object:NSApp];
    }
    [pool release];
}

- (void)_applicationWillFinishLaunching:(NSNotification *)notification {
	sleep(2);
    [fSplashWindow orderOut:nil];
    [fSplashWindow release];
    [[NSNotificationCenter defaultCenter] removeObserver:self 			name:NSApplicationWillFinishLaunchingNotification object:NSApp];
    [self release];
}


@end
