/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MIStartingPoint.h"
#import "MIDocumentController.h"
#import "MIPaletteController.h"
#import "MIMontageDocument.h"
#import "MIPreferences.h"
#import "MenuTags.h"
#import "MIApplication.h"
#import "MIViewerController.h"

#ifdef MTG_USE_LICENSER_KIT
#ifdef MTG_LIGHT_VERSION
#import "CSOneWay.h"
#import "MainApp.h"
#endif
#endif //#ifdef MTG_USE_LICENSER_KIT

#ifndef WorkingCodec_h
#include "WorkingCodec.h"
#endif //WorkingCodec_h

#ifndef ZzHelpers_h
#include "ZzHelpers.h"
#endif //ZzHelpers_h

#ifndef Zz_PathHelpers_h
#include "Zz_PathHelpers.h"
#endif //Zz_PathHelpers_h

@implementation MIStartingPoint

+ (id)startingPoint
{
    static id startingPoint;
    if (startingPoint == nil)
	{
		startingPoint = [[self alloc] init];
    }
    return startingPoint;
}

- (id)init
{
    if (( self = [super init] ))
	{
		[NSBundle loadNibNamed:@"StartingPoint" owner:self] ;
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(applicationWillTerminate:) name:NSApplicationWillTerminateNotification object:nil];
		[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(documentAdded:) name:MTGDocumentAddedNotification object:nil];
		[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(documentRemoved:) name:MTGDocumentRemovedNotification object:nil];
		fShouldDisplay = YES;
		fShouldDisplay = NO; //$$temp
		id menuItem = [NSApp searchForMenuItemWithTag:MTGNewDocumentMenuTag];
		[menuItem setTarget:self];
		[menuItem setAction:@selector(newDocClicked:)];
		
		[fStartingPointPanel center];
    }
    return self;
}

- (void)dealloc
{
    [[NSNotificationCenter defaultCenter] removeObserver:self name:nil object:nil];
    [super dealloc];
}

- (void)documentAdded:(NSNotification *)notification
{	
	vernl(6,"[MIStartingPoint documentAdded]");
//	NSMenuItem *item;
    [fStartingPointPanel orderOut:nil];
	[[NSNotificationCenter defaultCenter] removeObserver:self name:NSPopUpButtonWillPopUpNotification object:fRecentPopupButton];

    [[MIPaletteController sharedPaletteController] setDontShowPalettes:NO];
    [[MIPaletteController sharedPaletteController] showAllPalettes:nil];
	
	[[NSApp searchForMenuItemWithTag:MTGRecentDocumentMenuTag] setEnabled:NO];
}

- (void)documentRemoved:(NSNotification *)notification
{
	vernl(6,"[MIStartingPoint documentRemoved]");

	if ( [[[NSDocumentController sharedDocumentController] documents] count] <= 1 )
	{
		[[MIPaletteController sharedPaletteController] hideAllPalettes:nil];
		[[MIPaletteController sharedPaletteController] setDontShowPalettes:YES];
		
		if ( fShouldDisplay ) {
			[self updatePopupButton];
			[fStartingPointPanel makeKeyAndOrderFront:nil];
			[[NSApp searchForMenuItemWithTag:MTGRecentDocumentMenuTag] setEnabled:YES];
			[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(buttonWillPopup:) name:NSPopUpButtonWillPopUpNotification object:fRecentPopupButton];
		}
		else
		{		
			[[NSApp searchForMenuItemWithTag:MTGRecentDocumentMenuTag] setEnabled:YES];
		}
	}
}

- (void)setShouldDisplayStartingPoint:(BOOL)flag {
	fShouldDisplay = flag;
	fShouldDisplay = NO;  //$$temp
}

- (BOOL)shouldDisplayStartingPoint {
	return fShouldDisplay;
}

- (void)buttonWillPopup:(NSNotification*)theNotification {
	[[NSDocumentController sharedDocumentController] updateRecentDocuments];
	[self updatePopupButton];
}

- (void)applicationWillTerminate:(NSNotification*)theNotification
{
	vernl(6,"[MIStartingPoint applicationWillTerminate]");
    [self release];
}

- (IBAction)popupMenuSelectedItemChanged:(id)sender
{
    int index = [fRecentPopupButton indexOfSelectedItem];
    if ( index > 0 )
	{
        MIDocumentController *sharedController = [MIDocumentController sharedDocumentController];
		NSArray *urls = [sharedController recentDocumentURLs];		
		NSString *path = [[urls objectAtIndex:index - 1] path];

		if ( (unsigned)([fRecentPopupButton numberOfItems] - 1) != [urls count] )
		{
			[sharedController updateRecentDocuments];
			[self updatePopupButton];
			NSBeep();
			return;
		}
		
		if ( [[NSFileManager defaultManager] fileExistsAtPath:path] )
		{
			try
			{
				[sharedController openDocumentWithContentsOfFile:path display:YES];
			}
			catch(ZzExceptionCancel* exc)
			{
				NSRunAlertPanel(@"Can't open selected document ", @"Sorry, this document cannot be opened because of missing audio files. Please try to reopen it and locate the missing files.",@"OK",nil,nil);
			}
			catch(...)
			{
				throw;
			}
		}	
		else
		{
			[self updatePopupButton];
			NSRunAlertPanel(@"Can't open selected document ", @"The document \"%@\" can't be found.", @"OK", nil, nil, [path lastPathComponent] );
		}
    }
	[fRecentPopupButton selectItemAtIndex:0];
}

- (IBAction)formatPopupSelectedItemChanged:(id)sender {
	id imageSizeTextField;
	id frameRateTextField;
	int index = [sender indexOfSelectedItem];

		imageSizeTextField = fRenderImageSizeTextField;
		frameRateTextField = fRenderFrameRateTextField;

	switch (index ) {
		case 0 :
		case 1 :
			[imageSizeTextField setStringValue:@"720 x 576 pixels"];
			[frameRateTextField setStringValue:@"25 fps"];
			break;
		case 2 :
		case 3 :
			[imageSizeTextField setStringValue:@"720 x 480 pixels"];
			[frameRateTextField setStringValue:@"29.97 fps"];
			break;
		case 4 :
			[imageSizeTextField setStringValue:@"160 x 120 pixels"];
			[frameRateTextField setStringValue:@"24 fps"];
			break;
		case 5 :
			[imageSizeTextField setStringValue:@"320 x 240 pixels"];
			[frameRateTextField setStringValue:@"30 fps"];
			break;
	}
}

- (void)updatePopupButton
{
    NSArray *urls = [[NSDocumentController sharedDocumentController] recentDocumentURLs];

    if ( [urls count] > 0 )
	{
		NSURL *url;
		NSEnumerator *enumerator = [urls objectEnumerator];
        NSString *title = [[[fRecentPopupButton itemAtIndex:0] title] retain];

        [fRecentPopupButton removeAllItems];
		[fRecentPopupButton setEnabled:YES];

		while (( url = [enumerator nextObject] ))
		{
            [fRecentPopupButton addItemWithTitle:[NSString stringWithFormat:@"%@ - .../%@", [[[url path] lastPathComponent] stringByDeletingPathExtension], [[[url path] stringByDeletingLastPathComponent] lastPathComponent]]];
		}
        [fRecentPopupButton insertItemWithTitle:title atIndex:0];
        [title release];
        [fRecentPopupButton selectItemAtIndex:0];
    }
	else
	{
		[fRecentPopupButton setEnabled:NO];
    }
}

- (IBAction)newDocClicked:(id)sender
{
	vernl(6,"[MIStartingPoint newDocClicked]");

	if ( [[MIPreferences sharedPrefs] askNewDocProperties] )
	{
		switch ( [[MIPreferences sharedPrefs] defaultRenderFormat] ) {
			case kDvPal720_576 :
				[fRenderFormatMenu selectItemAtIndex:0];
				break;
			case kDvPal16_9 :
				[fRenderFormatMenu selectItemAtIndex:1];
				break;
			case kDvNtsc720_480 :
				[fRenderFormatMenu selectItemAtIndex:2];
				break;
			case kDvNtsc16_9 :
				[fRenderFormatMenu selectItemAtIndex:3];
				break;
			case kMjpeg160_120 :
				[fRenderFormatMenu selectItemAtIndex:4];
				break;
			case kMjpeg320_240 :
				[fRenderFormatMenu selectItemAtIndex:5];
				break;
			default :
				break;
		}

		[self formatPopupSelectedItemChanged:fRenderFormatMenu];
		[fShouldNotAskAgainCheckbox setState:NSOffState];

		if ( [fStartingPointPanel isVisible] )
		{
			[NSApp beginSheet:fNewDocumentPropertiesView modalForWindow:fStartingPointPanel modalDelegate:self didEndSelector:nil contextInfo:nil];
		}
		else
		{
			[NSApp runModalForWindow:fNewDocumentPropertiesView];
		}
	}
	else
	{
		[self openSavePanel];
	}
}

- (IBAction)closeNewDocProperties:(id)sender
{
	if ( [fStartingPointPanel isVisible] )
	{
		[NSApp endSheet:[sender window]];
		[fNewDocumentPropertiesView orderOut:self];
	}
	else
	{
		[NSApp stopModal];
		[fNewDocumentPropertiesView orderOut:self];
	}
	[self openSavePanel];
}

- (IBAction)cancelNewDocProperties:(id)sender
{
	if ( [fStartingPointPanel isVisible] )
	{
		[NSApp endSheet:[sender window]];
		[fNewDocumentPropertiesView orderOut:self];
	}
	else
	{
		[NSApp stopModal];
		[fNewDocumentPropertiesView orderOut:self];
	}
}


- (void)openSavePanel
{
#ifndef MTG_DEMO_VERSION
	id savePanel = [NSSavePanel savePanel];
	[savePanel setPrompt:@"Create"];
	[savePanel setTitle:@"New Document"];
	[savePanel setCanSelectHiddenExtension:YES];

	if ( [fStartingPointPanel isVisible] )
	{
		[savePanel beginSheetForDirectory:nil file:@"Untitled" modalForWindow:fStartingPointPanel modalDelegate:self didEndSelector:@selector(savePanelDidEnd:returnCode:contextInfo:) contextInfo:nil];
	}
	else
	{
		if ( NSFileHandlingPanelOKButton == [savePanel runModalForDirectory:nil file:@"Untitled"] )
			[self savePanelDidEnd:savePanel returnCode:NSOKButton contextInfo:nil];
	}
#else
	[self savePanelDidEnd:NULL returnCode:NSOKButton contextInfo:NULL];
#endif
}

- (void)savePanelDidEnd:(NSSavePanel *)savePanel returnCode:(int)returnCode contextInfo:(void  *)contextInfo
{
	NSDocumentController *documentController = [NSDocumentController sharedDocumentController];
	MIMontageDocument *document;
	int num, den, tempo;
	WorkingCodecType rFormat;

#ifndef MTG_DEMO_VERSION
	[savePanel orderOut:self];

	if ( NSOKButton == returnCode )
	{
//		NSString *filename = [[((NSSavePanel*)savePanel) filename] stringByAppendingPathExtension:@"montage"];
		NSString *filename = [[((NSSavePanel*)savePanel) filename] stringByAppendingPathExtension:@"HAV"];
#else
//		NSString *filename = @"/tmp/Demo.montage";
		NSString *filename = @"/tmp/Demo.HAV";
#endif
		BOOL isDirectory;
		NSFileManager *fileManager = [NSFileManager defaultManager];

		if ( [fileManager fileExistsAtPath:filename isDirectory:&isDirectory] )
		{
#ifndef MTG_DEMO_VERSION			
			if ( NSAlertDefaultReturn == NSRunAlertPanel(@"Save", @"A file named \"%@\" already exists in %@. Do you want to replace it ?", @"Replace", @"Cancel", nil, [filename lastPathComponent], [filename stringByDeletingLastPathComponent]) )
			{
#endif
				[fileManager removeFileAtPath:filename handler:nil];
#ifndef MTG_DEMO_VERSION				
			}
			else return;
#endif
		}

		[fStartingPointPanel orderOut:nil];

		fFileName = [filename stringByDeletingPathExtension];
#ifndef MTG_DEMO_VERSION
		fHideExtension = [savePanel isExtensionHidden];
#else
		fHideExtension = YES;
#endif

		[[MIViewerController sharedViewerController] setShouldIgnoreNotifications:YES];
		if ( [[MIPreferences sharedPrefs] askNewDocProperties] )
		{
			num = [fSignatureNum intValue];
			den = [fSignatureDen intValue];
			tempo = [fTempo intValue];

			switch ( [fRenderFormatMenu indexOfSelectedItem] ) {
				case 0 :
					rFormat = kDvPal720_576;
					break;
				case 1 :
					rFormat = kDvPal16_9;
					break;
				case 2 :
					rFormat = kDvNtsc720_480;
					break;
				case 3 :
					rFormat = kDvNtsc16_9;
					break;
				case 4 :
					rFormat = kMjpeg160_120;
					break;
				case 5 :
					rFormat = kMjpeg320_240;
					break;
			}

			if ( [fShouldNotAskAgainCheckbox state] == NSOnState ) {
				[[MIPreferences sharedPrefs] setAskNewDocProperties:NO];
				[[MIPreferences sharedPrefs] setDefaultRenderFormat:rFormat];
			}
		}
		else
		{
			num =[[MIPreferences sharedPrefs] getDefaultSignatureNum];
			den = [[MIPreferences sharedPrefs] getDefaultSignatureDen];
			tempo = [[MIPreferences sharedPrefs] getDefaultTempo];
			rFormat = [[MIPreferences sharedPrefs] defaultRenderFormat];
		}

		//document = [documentController openUntitledDocumentOfType:@"HyperEngine-AV document" display:YES];
		document = [[[MIMontageDocument alloc] initWithVideoFormat:rFormat] autorelease];
		if ( document )
		{
			[documentController addDocument:document];
			if ( [documentController shouldCreateUI] ) [document makeWindowControllers];
			//[document setSignatureNumerator:num];
			[document setSignatureNumerator:4];
			//[document setSignatureDenominator:den];
			[document setSignatureDenominator:4];
			//[document setTempo:tempo];
			[document setTempo:120];
			//[document setRenderFormat:(WorkingCodecType)rFormat];
			//[document setPreviewFormat:(WorkingCodecType)pFormat];
			//[document setPreviewFormat:(WorkingCodecType)kDvPal720_576];

			[[MIViewerController sharedViewerController] setShouldIgnoreNotifications:NO];

			// hack to have the viewer at the correct size
			NSNotification *notification = [NSNotification notificationWithName:MTGCurrentDocumentChangedNotification object:[NSDocumentController sharedDocumentController] userInfo:[NSDictionary dictionaryWithObject:document forKey:MTGCurrentDocumentKey]];
			[[MIViewerController sharedViewerController] currentDocumentChanged:notification];
			
#ifdef LOG_USER_COMMAND_TO_LOG_FILE
	UserCommandsLogFileName = new char[1024];
//	strcpy(UserCommandsLogFileName, ZzGetLogFileFolder());
//	ZzGetDirectory(UserCommandsLogFileName, 1024, [fileName cString]);
	ZzAppendFileNameToPath(UserCommandsLogFileName, 1024, ZzGetLogFileFolder().getUTF8String(), "HyperEngine-AV.txt");
#endif //LOG_USER_COMMAND_TO_LOG_FILE
			VUSERCMD2("Creating new doc %s with format %d", [filename cString], (int)rFormat);
		
#ifdef MTG_LIGHT_VERSION
			unsigned int challenge, reply;
			challenge = createTimeChallenge ();
			reply = [NSApp isHostFeatureSubsetModeWithChallenge:challenge];
			if (verifyChallenge (reply) == challenge) {
				[document saveDocument:self];
			}
#else
#ifndef MTG_DEMO_VERSION			
			[document saveDocument:self];
#endif			
#endif
		}
		fFileName = nil;
#ifndef MTG_DEMO_VERSION			
	}
#endif
}

- (BOOL)validateMenuItem:(NSMenuItem *)anItem {
	if ( [[anItem title] isEqualToString:@"New Document"] ) {
		if ( [[[NSDocumentController sharedDocumentController] documents] count] == 0 ) return YES;
		else return NO;
	}
	return YES;
}


@end
