/*
 *  MISynthesizeClipDlgController.mm
 *  HyperEngine AV
 *
 *  Created by benjamin on Wed Jun 12 2002.
 *  Copyright (c) 2002 Arboretum Systems, Inc.. All rights reserved.
 *
 */

#import <Cocoa/Cocoa.h>
#import "MIMontageDocument.h"
#import "MIMainViewController.h"
#import "MITopLevelContainerRep.h"
#import "MISynthesizeClipDlgController.h"
#import "MICreateAudioClipFromChunkCommand.h"
#import "MIApplicationController.h"
#import "MIPreferences.h"

#ifndef ArmeErrors_h
#include "ArmeErrors.h"
#endif //ArmeErrors_h

#ifndef MEMTGAAudioFile_h
#include "MEMTGAAudioFile.h"
#endif //MEMTGAAudioFile_h

#ifndef MEAudioChunkList_h
#include "MEAudioChunkList.h"
#endif //MEAudioChunkList_h

#ifndef TArmeDoc_h
#include "TArmeDoc.h"
#endif //TArmeDoc_h

@implementation MISynthesizeClipDlgController

- (void)setMontageDoc:(MIMontageDocument*)doc
{
	fMontageDoc = doc;
}

#define kSineWaveTmpBufferNbFrames 32768

- (IBAction)OKClicked:(id)sender
{
	MEMTGAAudioFile file;
	MEAudioChunk chunk;
	chunk.startInFile = 0;
	chunk.doc = [fMontageDoc armeDocument];
	chunk.duration = 0; // to be set later on.
	chunk.SetFileIndex(chunk.doc->GetNewAudioChunkFileIndex());
	
	SInt32 nbChannels = [[fNbChannelsRadioGroup selectedCell] tag];
	
	// open the destination chunk file
	ArmeErr err = file.OpenSaveNewChunkFile([fMontageDoc armeDocument],chunk,nbChannels);
	if(err.type != kArmeOK)
	{
		vernl(1,"MESplitTaskImportAndResampleAudioTrackInAudioClip::PerformOnce() : ERROR in OpenSaveNewChunkFile()");
		return;
	}
	
	double durationSec = [fDurationTextBox doubleValue];
	double startFreq = [fStartFrequencyTextBox doubleValue];
	double endFreq = [fEndFrequencyTextBox doubleValue];
//	double startAttenuation = [fStartAttenuationTextBox doubleValue];
//	double endAttenuation = [fEndAttenuationTextBox doubleValue];

	const double pi=3.14159265359;
	double frameRate = (double)(chunk.doc->GetAudioFrameRate());

	SInt64 duration = GET_VAL(chunk.doc->SecondsToArmeTime(durationSec));
	
	
	// we want to compute a new duration so that it ends on 2pi
	
	// phase(0) = 0
	// phase(i) = phase(i-1) + cstPhaseIncr + i*varPhaseIncrIncr
	// if we expand cstPhaseIncr and varPhaseIncrIncr
	// phase(i) = phase(i-1) + 2*pi*startFreq/frameRate + i*(2*pi*(endFreq-startFreq)/(frameRate*(double)duration));
	
	// if we concatenate the recursive relation, it leads to :
	// phase(i) = i*2*pi*startFreq/frameRate + (1+2+3+...i)*(2*pi*(endFreq-startFreq)/(frameRate*(double)duration));
	// phase(i) = i*2*pi*startFreq/frameRate + i*(i+1)*(pi*(endFreq-startFreq)/(frameRate*(double)duration));

	// let's call A = startFreq/frameRate
	// and B = endFreq/frameRate
	// and D = duration;
	// and Pd = the phase when sample index == duration
	// thus Pd = 2*pi*D*A + pi*D*(D+1)*(B-A)/D
	// Pd = D*pi*(2*A +(D+1)*(B-A)/D)
	// Pd = pi*(2*A*D + D*B + B - D*A - A)
	// = pi*(B*D+A*D+B-A)
	// = pi*(D*(A+B) + B-A)
	// if gamma = (D*(A+B) + B-A)/2
	// 
	// thus if we want this to be k*2*pi it takes (D*(A+B) + B-A)/2 to be an integer
	// let's call this gamma
	
	// this D = (2*gamma + A-B)/(A+B)
	
	double A = startFreq/frameRate;
	double B = endFreq/frameRate;
	double D = (double)duration;
//	vvernl(0,(double)duration);
	
	double gammaD1 = (D*(A+B) + B-A)/2.0;
	double gammaInt = rint(gammaD1);
	double newDurationDbl = (2.0*gammaInt + A-B)/(A+B);
//	vvernl(0,newDurationDbl);
	SInt32 newDuration = (SInt32)round(newDurationDbl);
//	vvernl(0,newDuration);
//	double newGammaInt = (newDuration*(A+B) + B-A)/2.0;
//	vvernl(0,newGammaInt);
//	double debugEndPhase = 2*pi*newGammaInt;
//	vvernl(0,debugEndPhase);
	
	SInt64 curFrame = 0;
	double phase = 0; // the current phase
	double cstPhaseIncr = 2*pi*startFreq/frameRate; // the fixed value that's added to the phase at each frame
	double varPhaseIncr = 0; // the variable value that's added to the phase at each frame
	double varPhaseIncrIncr = 2*pi*(endFreq-startFreq)/(frameRate*(double)newDuration); // the incr that's added to varPhaseIncr at each frame
	
	float* tmpBuffer = new float [kSineWaveTmpBufferNbFrames*nbChannels];
	SInt32 debugIFrame = 0;
	while(curFrame < newDuration)
	{
		SInt32 framesInTmpBuffer = kSineWaveTmpBufferNbFrames;
		SInt32 remainingDuration = newDuration - curFrame;
		if(framesInTmpBuffer > remainingDuration)
			framesInTmpBuffer = remainingDuration;
		
		if(nbChannels == 1)
		{
			float* dst = tmpBuffer;
			for(SInt32 frameInBuffer = 0; frameInBuffer < framesInTmpBuffer; frameInBuffer++)
			{
				float val = (float)sin(phase);
				*dst = val;
				dst++;
				phase += cstPhaseIncr;
				phase += varPhaseIncr;
				varPhaseIncr += varPhaseIncrIncr;
			}
		}
		else if(nbChannels == 2)
		{
			float* dst = tmpBuffer;
			for(SInt32 frameInBuffer = 0; frameInBuffer < framesInTmpBuffer; frameInBuffer++)
			{
				float val = (float)sin(phase);
				*dst = val;
				dst++;
				*dst = val;
				dst++;
				phase += cstPhaseIncr;
				phase += varPhaseIncr;
				varPhaseIncr += varPhaseIncrIncr;
				debugIFrame++;
				if(debugIFrame == newDuration)
				{
					vernl(6,"");
				}
			}
		}
		else
		{
			ZzAssert(0);
		}
		file.WriteFloat32Frames(tmpBuffer,framesInTmpBuffer);
		curFrame += kSineWaveTmpBufferNbFrames;
	}
			
	file.Close();
	err = file.OpenReadChunk([fMontageDoc armeDocument],chunk);
	chunk.duration = file.GetNbFrames();
	file.Close();
	
	MICreateAudioClipFromChunkCommand* createAudioClipCommand = [[[MICreateAudioClipFromChunkCommand alloc] initWithChunk:chunk montageDoc:fMontageDoc] autorelease];
	[createAudioClipCommand setParent:[fMontageDoc topLevelContainer]];
	[createAudioClipCommand setPosition:[[[NSApp mainViewController] topLevelContainerView] newObjectLocation]];
	[createAudioClipCommand setNbChannels:nbChannels];
	[fMontageDoc doCmd:createAudioClipCommand];


	if ( [[MIPreferences sharedPrefs] gridShowMask] & kPrefsGridShowVObjLines ||
		 [[MIPreferences sharedPrefs] gridShowMask] & kPrefsGridShowHObjLines )
		[[MIApplicationController sharedApplicationController] redrawAllMainWindows];


	[[[NSApp mainViewController] topLevelContainerView] scrollToNewObject];
	vernl(6,"");
}

- (IBAction)cancelClicked:(id)sender
{
}

@end

