/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
#import "MITextClipRep.h"
#import "MIImageController.h"
#import "TArmeTextClip.h"
#import "MIObject.h"
#import "MITopLevelContainerRep.h"
#import "MIMainViewController.h"
#import "MIWSSelection.h"
#import "MITimeStretchTracker.h"
#import "MITrimTracker.h"

@implementation MITextClipRep

- (id)initWithFrame:(NSRect)frame inTLC:(MITopLevelContainerRep*)theTLC
{
    self = [super initWithFrame:frame inTLC:theTLC];
    [self setBackgroundColor:mainImageController->fTextClipBrightColor];
    
//    fLeftBarImage = mainImageController->fTextClipBarOnLeft;
//    fCenterBarImage = mainImageController->fTextClipBarOnCenter;
//    fRightBarImage = mainImageController->fTextClipBarOnRight;

	fLeftTitleImage = mainImageController->fLeftTextClipBrightTitleBarImage;
    fCenterTitleImage = mainImageController->fCenterTextClipBrightTitleBarImage;
    fRightTitleImage = mainImageController->fRightTextClipBrightTitleBarImage;

	/*
	fHasTopBottomHandles = YES;
	fHasLeftRightHandles = YES;
	fHasCollapseButton = YES;
	fHasPropertiesButton = NO;
	fHasOnOffButton = YES;
	*/
	
	return self;
}

- (void)setSelected:(BOOL)flag
{
	if ( flag == fSelected )
		return;
	
	[super setSelected:flag];
	[fStringRep release];
	fStringRep = nil;
}

- (void)setImagesAndColor
{
	if ( [self isSelected] && [[[fTLCRep mainViewController] selection] type] == kMIWSSelectionTypeObjects )
	{
		[self setBackgroundColor:mainImageController->fTextClipNegativeColor];
		fLeftTitleImage = mainImageController->fLeftTextClipNegativeTitleBarImage;
		fCenterTitleImage = mainImageController->fCenterTextClipNegativeTitleBarImage;
		fRightTitleImage = mainImageController->fRightTextClipNegativeTitleBarImage;
	}
	else if ( [fRepresentedObject isMuted] )
	{
		[self setBackgroundColor:mainImageController->fTextClipDimColor];
		fLeftTitleImage = mainImageController->fLeftTextClipDimTitleBarImage;
		fCenterTitleImage = mainImageController->fCenterTextClipDimTitleBarImage;
		fRightTitleImage = mainImageController->fRightTextClipDimTitleBarImage;
	}
	else
	{
		[self setBackgroundColor:mainImageController->fTextClipBrightColor];
		fLeftTitleImage = mainImageController->fLeftTextClipBrightTitleBarImage;
		fCenterTitleImage = mainImageController->fCenterTextClipBrightTitleBarImage;
		fRightTitleImage = mainImageController->fRightTextClipBrightTitleBarImage;
	}
}

- (MIObjectMouseHandling)handleMoreMouseTracking:(NSPoint)mouseLoc modifier:(unsigned int)modifier type:(NSEventType)type
{
    [self invalidateTheMouseTracker];

    // if the parent does NOT handle it, and if we are in the body, let's create a selection tracker
	if ( NSPointInRect(mouseLoc, [self rectForZone:MIObjectRepRightTimeStretchZone]) )
	{
		if ( modifier & NSAlternateKeyMask )
		{
			fTemporaryTracker = [[MITimeStretchTracker alloc] initWithView:fTLCRep atPos:[self convertPointToTLCCoords:mouseLoc] withModifiers:nil ];
			[fTemporaryTracker setEdge:NSMaxXEdge];
		}
        else
		{
			fTemporaryTracker = [[MITrimTracker alloc] initWithView:fTLCRep atPos:[self convertPointToTLCCoords:mouseLoc] withModifiers:nil ];
			[fTemporaryTracker setEdge:NSMaxXEdge];
		}
        return MIAskMeATracker;
	}
	else if ( NSPointInRect(mouseLoc, [self rectForZone:MIObjectRepLeftTimeStretchZone]) )
	{
		if ( modifier & NSAlternateKeyMask )
		{
			fTemporaryTracker = [[MITimeStretchTracker alloc] initWithView:fTLCRep atPos:[self convertPointToTLCCoords:mouseLoc] withModifiers:nil ];
			[fTemporaryTracker setEdge:NSMinXEdge];
		}
        else
		{
			fTemporaryTracker = [[MITrimTracker alloc] initWithView:fTLCRep atPos:[self convertPointToTLCCoords:mouseLoc] withModifiers:nil ];
			[fTemporaryTracker setEdge:NSMinXEdge];
		}
        return MIAskMeATracker;
	}
    else
		return [super handleMoreMouseTracking:mouseLoc modifier:modifier type:type];
}



- (void)drawBaseBodyRect:(NSRect)frameInDrawCoords clippedTo:(NSRect)clippingRect
{
	float		titleHeight;
	float		titleBarHeight;
	float		footerHeight;
	NSRect		srcRect, dstRect;
	NSImage		*img;


	// wsObj is the Arme object of the MIObject we represent
	TArmeTextClip* clip = (TArmeTextClip*)[[self representedObject] WSObject];

	if ( !fLastString || !fStringRep || ![fLastString isEqualToAttributedString:clip->GetStyledText()] )
		[self recreateWorkingAttributedString];
	
#ifdef SCALABLE_TITLEBAR
	titleHeight = [[self getTopLevelContainerRep] MIVPosToPixel:kRepTitleHeight];
	titleBarHeight = [[self getTopLevelContainerRep] MIVPosToPixel:kRepTitleBarHeight];
#else
	titleHeight = kRepTitleHeight;
	titleBarHeight = kRepTitleBarHeight;
#endif
	footerHeight = [[self getTopLevelContainerRep] MIVPosToPixel:kRepFooterHeight];

	[super drawBaseBodyRect:frameInDrawCoords clippedTo:clippingRect];

	if ( ![self isSelected] )
		img = mainImageController->fTextClipCheckboard;
	else
		img = mainImageController->fTextClipCheckboardNegative;
		
	if ( ((TArmeTextClip*)[[self representedObject] WSObject])->IsKeyed() )
	{
		NSSize s = [mainImageController->fTextClipCheckboard size];
		for ( float i = NSMinX(frameInDrawCoords); i < NSMaxX(frameInDrawCoords) ; i += s.width )
			for ( float j = NSMinY(frameInDrawCoords); j < NSMaxY(frameInDrawCoords) ; j += s.height )
			{
				dstRect = NSMakeRect( i, j, s.width, s.height );
				srcRect = NSMakeRect(0,0, s.width, s.height);
				if ( NSMaxX(dstRect) > NSMaxX( frameInDrawCoords) )
				{
					srcRect.size.width += (NSMaxX(frameInDrawCoords) - NSMaxX(dstRect));
					dstRect.size.width += (NSMaxX(frameInDrawCoords) - NSMaxX(dstRect));
				}
				if ( NSMaxY(dstRect) > NSMaxY( frameInDrawCoords) )
				{
					srcRect.size.height += (NSMaxY(frameInDrawCoords) - NSMaxY(dstRect));
					dstRect.size.height += (NSMaxY(frameInDrawCoords) - NSMaxY(dstRect));
				}
				
				[img drawInRect:dstRect fromRect:srcRect operation:NSCompositeCopy fraction:1];
			}
	}
	
	
	if(clippingRect.size.width <= 0.1 || clippingRect.size.height <= 0.1 )
		return;

	//float duration = clip->GetDurationSec();
	
	// clippingRect is the rect where we need to draw in draw coords
	// aPoint is the origin of our whole frame in draw coords.
	// clipStart is the x origin of this rect, for syntax clarity
	// and clipEnd is the x end of this rect
	//float clipStart = clippingRect.origin.x - frameInDrawCoords.origin.x;
	//float clipEndx = clipStart + clippingRect.size.width;

/*
	NSString* string = clip->GetPlainText();
	[string drawInRect:clippingRect withAttributes:nil];
*/
	
	[fStringRep drawInRect:frameInDrawCoords];


// draw trim indicators

	if ( GET_VALUE([fRepresentedObject WSObject]->GetStartTrim()) )
		[mainImageController->fTrimIndicatorLeft compositeToPoint:frameInDrawCoords.origin operation:NSCompositeSourceAtop];
	else
		[mainImageController->fNonTrimIndicatorLeft compositeToPoint:frameInDrawCoords.origin operation:NSCompositeSourceAtop];		

	if ( GET_VALUE([fRepresentedObject WSObject]->GetEndTrim()) )
		[mainImageController->fTrimIndicatorRight compositeToPoint:NSMakePoint(NSMaxX(frameInDrawCoords)-10, NSMinY(frameInDrawCoords)) operation:NSCompositeSourceAtop];
	else
		[mainImageController->fNonTrimIndicatorRight compositeToPoint:NSMakePoint(NSMaxX(frameInDrawCoords)-10, NSMinY(frameInDrawCoords)) operation:NSCompositeSourceAtop];		

}

- (void)recreateWorkingAttributedString
{
	// wsObj is the Arme object of the MIObject we represent
	TArmeTextClip* clip = (TArmeTextClip*)[[self representedObject] WSObject];
	
	[fLastString release];
	fLastString = [clip->GetStyledText() copy];

	[fStringRep release];
	fStringRep = [[NSMutableAttributedString alloc] init];

	NSMutableParagraphStyle*	style = [[[NSMutableParagraphStyle alloc] init] autorelease];
	NSDictionary*				att;
	NSString*					tmpString = [clip->GetStyledText() string];
	NSCharacterSet				*cset = [NSCharacterSet characterSetWithCharactersInString:@"\n\r"];
	NSRange						rng;
	unsigned					len = [tmpString length];
	unsigned					curPos;
	
	[fStringRep setAttributedString:clip->GetStyledText()];
	
	rng.location = 0;
	curPos = 0;
	while ( rng.location != NSNotFound )
	{
		rng = [tmpString rangeOfCharacterFromSet:cset options:0 range:NSMakeRange(curPos, len- curPos)];
		if ( rng.location != NSNotFound )
		{
			curPos = NSMaxRange(rng);
			[fStringRep replaceCharactersInRange:rng withString:@" "];
		}
	}
	
	// set the colors to negative in case the clip is selected
	if ( [self isSelected] )
	{
		NSRange 			rng;
		unsigned int		length = [fStringRep length];
		NSMutableDictionary	*dic = [NSMutableDictionary dictionaryWithCapacity:5];
		
//NSBackgroundColorAttributeName
//NSForegroundColorAttributeName
		rng.location = 0;
		rng.length = 0;
		while ( NSMaxRange(rng) < length )
		{
			[dic setDictionary:[fStringRep attributesAtIndex:NSMaxRange(rng) effectiveRange:&rng]];
			[dic setObject:NegativeOfColor([dic objectForKey:NSForegroundColorAttributeName]) forKey:NSForegroundColorAttributeName];
			[fStringRep setAttributes:dic range:rng];
		}
//		(NSDictionary *)attributesAtIndex:(unsigned)index effectiveRange:(NSRangePointer)aRange
	}
	
	
	// realign the text (as in, remove all \n and \r and set the wrapping position)
	[style setLineBreakMode:NSLineBreakByWordWrapping];
	att = [NSDictionary dictionaryWithObject:style forKey:NSParagraphStyleAttributeName];
	[fStringRep addAttributes:att range:NSMakeRange(0, [[fStringRep string] length])];
}


- (void)refreshCachedBodyImage:(BOOL)createIfAbsent
{
	// silly thing doesn't draw if we let it try to cache, what's up with that?
}

- (BOOL)doubleClick:(NSPoint)mouseLoc modifier:(unsigned int)modifier
{
	[[NSApp mainViewController] executeCommand:@"editTextClip" ];	


/*	TArmeTextClip* clip = (TArmeTextClip*)[[self representedObject] WSObject];
	[self invalidate];
	if (clip and clip->EditText())
	{
		[self setNeedsDisplay];
		[self invalidate];
		return YES;
	}
	
	return NO;*/
	return YES;
}

@end
