/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import <Cocoa/Cocoa.h>
//#import <Quicktime/Quicktime.h>

class TArmeDoc;
class TArmeWSObj;
class ArmeDataFileRef;
class TArmeTextClip;

typedef enum _MIVerticalTextAlignment {
    MITopTextAlignment		= 0,	/* Visually top aligned */
    MICenterTextAlignment	= 1,	/* Visually center aligned */
    MIBottomTextAlignment	= 2		/* Visually bottom aligned */
} MIVerticalTextAlignment;

typedef enum _MIHorizontalTextAlignment {
    MILeftTextAlignment		= -2,	/* Visually left aligned */
    MIHCenterTextAlignment	= 1,	/* Visually center aligned */
    MIRightTextHAlignment	= -1	/* Visually right aligned */
} MIHorizontalTextAlignment;

typedef enum _MIBackgroundMatrixTags
{
	MIBackgroundSolidColor = 0,
	MIBackgroundTransparent
} MIBackgroundMatrixTags;

typedef struct TextParams
{
	int						scrollType;
    bool					textKeyed;
    NSColor *				trackBackground;

    ArmeTime				clipDuration;
	short					textJustify;
	short					verticalAlignment;

	NSData *				richText;
	NSString *				plainText;
	NSAttributedString *	styledText;
} TextParams;
// GP

@interface MITextController : NSWindowController 
{
@public
	ArmeDataFileRef* fMovieFileRef;
    FSSpec		fMovieFile;
    short		fRefNum;
    Movie		fMovie;
	short		fTextJustify;
	BOOL		fSetSize;
    BOOL		fOKClicked;

	TArmeDoc* 	fCurrentDoc;
    
    IBOutlet id fOKButton;
    IBOutlet id fPreviewButton;

    IBOutlet id fMoviePreviewView;
    IBOutlet id fTextView;
    IBOutlet id fScrollingView;

	IBOutlet id fScrollingMatrix;

	IBOutlet id fBackgroundMatrix;
	IBOutlet id fBackgroundTransparent;
	IBOutlet id fBackgroundSolidColor;
    IBOutlet id fBackgroundColorWell;

    IBOutlet id fTrackDurationMinutes;
    IBOutlet id fTrackDurationSeconds;
    IBOutlet id fTrackDurationTenthSecond;
	IBOutlet id fTrackDurationSecByCar;
	IBOutlet id fCalculateDurationButton;

	IBOutlet id fFontPopupButton;
	IBOutlet id fSizePopupButton;
	IBOutlet id fCustomSizeTextField;
	NSRect		fCustomSizeTextFieldFrame;
	IBOutlet id fStylePopupButton;
	IBOutlet id fTextColorWell;
	
	IBOutlet id fHorizontalAlignmentPopupButton;
	IBOutlet id fVerticalAlignmentPopupButton;
	MIVerticalTextAlignment fVerticalTextAlignment;
}

//+ (MITextController*) sharedTextController;
- (MITextController*) init;
- (void)windowDidLoad;

- (TArmeWSObj*) doTextClipDialog:(TArmeDoc*)doc;
// GP : old : - (ArmeDataFileRef*) editTextClip:(TArmeTextClip*)owner;
- (BOOL) editTextClip:(TArmeTextClip*)owner;

- (void) updateButtonStatus:(IBOutlet id)button;
- (void)updateCalculateButtonStatus;

/*- (BOOL) oneTextFieldActive;*/

- (BOOL) textView:(NSTextView *)textView shouldChangeTextInRange:(NSRange)affectedCharRange replacementString:(NSString *)replacementString;
/*- (BOOL)textShouldBeginEditing:(NSText *)textObject;
- (BOOL)textShouldEndEditing:(NSText *)textObject;*/
- (void) textDidChange:(NSNotification *)aNotification;
- (void) controlTextDidChange:(NSNotification *)aNotification;

- (void) secondsToMinSecTenthSec:(float) floatSeconds;
- (float) minSecTenthSecToSeconds;

- (IBAction) makePreview:(id)sender;

- (IBAction) cancelClicked:(id)sender;
- (IBAction) okClicked:(id)sender;

- (void) makeMovieForPreview;
- (void) cleanMovie;

- (BOOL)isBackgroundColorWellActive;
- (BOOL)isTextColorWellActive;

- (void) updateBackgroundColorRadioButtons;

- (void)setShouldShowCustomSizeTextField:(BOOL)flag;
- (void)setupAlignPopupButtons;
- (void)setupFontPopupButton;
- (void)setupStylePopupButton;
- (void)setupSizePopupButton;

/* GP : not needed anymore.
// why does this need to be here? Shouldn't NSApp pick it up automagically?
- (void)orderFrontColorPanel:(id)sender;
GP */
- (IBAction)setVAlignmentTop:(id)sender;
- (IBAction)setVAlignmentBottom:(id)sender;
- (IBAction)setVAlignmentCenter:(id)sender;
- (IBAction)setHAlignmentLeft:(id)sender;
- (IBAction)setHAlignmentRight:(id)sender;
- (IBAction)setHAlignmentCenter:(id)sender;
- (void) setHAlignment:(NSTextAlignment)mode;
- (void) updateHAlignment;

- (IBAction)fontChanged:(id)sender;
- (IBAction)scrollingChanged:(id)sender;
- (IBAction)calculateDuration:(id)sender;

- (IBAction)textColorChanged:(id)sender;
- (IBAction)backgroundColorChanged:(id)sender;

- (IBAction)transparentBackgroundClicked:(id)sender;
- (IBAction)solidColorBackgroundClicked:(id)sender;

@end