/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MITextController.h"
#import "MIFontManager.h"
#import "TArmeDoc.h"
#import "TVideoClip.h"
#import "ZzHelpers.h"
#import "MovieHelper.h"

#import "MIBackupPasteboard.h"

#ifndef ArmeDataFileRef_h
#include "ArmeDataFileRef.h"
#endif //ArmeDataFileRef_h
#ifndef TArmeTextClip_h
#include "TArmeTextClip.h"
#endif //TArmeTextClip_h

#ifndef MIProgressBarDlgManager_h
#include "MIProgressBarDlgManager.h"
#endif //MIProgressBarDlgManager_h

#import "MIApplication.h" 

#import "MIPreferences.h"

#ifndef Zz_PathHelpers_h
#include "Zz_PathHelpers.h"
#endif //Zz_PathHelpers_h

const short kKeyedTextLayer = -32000;
const short kUnkeyedTextLayer = -31000;

int sortStrings(id str1, id str2, void *context);

@implementation MITextController

// GP : the text controller must be a new one when we create one.
// The class method sharedTextController has no reason to exist anymore.
// Just call this way : [[MITextController alloc] init].
/*+ (MITextController*)sharedTextController
{
    static MITextController *sharedTextController = nil;
    if ( sharedTextController == nil )
    {
        sharedTextController = [[MITextController alloc] init];
    }
    return sharedTextController;
	return [[MITextController alloc] init];
}*/

- (MITextController*) init
{
    if ( self = [super initWithWindowNibName:@"MICreateText"] )
		{
        fMovie = nil;
		fTextJustify = teCenter;
		fVerticalTextAlignment = MICenterTextAlignment;
		fSetSize = false;
		[self window];
		[[NSColorPanel sharedColorPanel] setFloatAboveModalWindows:YES];
		}
    return self;
}

- (void)dealloc
{
	//remove the temp file used to build the preview
	char path[kMaxPathLength];
	ZzAppendFileNameToPath(path, kMaxPathLength, fCurrentDoc->GetProjectFolderPath(), "temptext.mov");
	// GP : use FSspecDelete to delete the file we created. Comment : remove(path);
	FSSpec theFsspec = Path2FSspec(path, true);

    [[NSNotificationCenter defaultCenter] removeObserver:self name:NSApplicationWillTerminateNotification object:nil];
    [[NSNotificationCenter defaultCenter] removeObserver:self name:MTGSelectedFontChangedNotification object:nil];
	
	[[NSColorPanel sharedColorPanel] setFloatAboveModalWindows:NO];
	[[NSColorPanel sharedColorPanel] orderOut:self];

    [super dealloc];
}

- (void)awakeFromNib
{
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(applicationWillTerminate:) name:NSApplicationWillTerminateNotification object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(selectedFontChanged:) name:MTGSelectedFontChangedNotification object:nil];

	fCustomSizeTextFieldFrame = [fCustomSizeTextField frame];
	[self setShouldShowCustomSizeTextField:NO];
	
	// Resize the NSMovieView to be with the same ratio than the current format.
	/*Rect movieBounds;
	::GetMovieBox(QTMovie, &movieBounds);
			
	NSRect movieFrame = ScaleNSRectToNSRect(NSMakeRect(	movieBounds.left,
														movieBounds.top,
														movieBounds.right-movieBounds.left,
														movieBounds.bottom-movieBounds.top),
											[screen frame],
											true);
	
	short		theMovieHeight = fCurrentDoc->fMasterMovie->GetHeight();
	short		theMovieWidth = fCurrentDoc->fMasterMovie->GetWidth();

	*/

	/*NSDecimalNumber *	zero = [NSDecimalNumber zero];
	NSDecimalNumber *	maxMinutes = [NSDecimalNumber decimalNumberWithString:@"99"];
	NSDecimalNumber *	maxSeconds = [NSDecimalNumber decimalNumberWithString:@"59"];
	NSDecimalNumber *	maxTenthSecond = [NSDecimalNumber decimalNumberWithString:@"9"];
	NSDecimalNumber *	minSecBycar = [NSDecimalNumber decimalNumberWithString:@"0.1"];
	NSDecimalNumber *	maxSecBycar = [NSDecimalNumber decimalNumberWithString:@"1.0"];

	NSNumberFormatter *minutesFormatter = [[[NSNumberFormatter alloc] init] autorelease];
	[minutesFormatter setFormat:@"00;00"];
	[minutesFormatter setMinimum:zero];
	[minutesFormatter setMaximum:maxMinutes];
	[[fTrackDurationMinutes cell] setFormatter:minutesFormatter];
	
	NSNumberFormatter *secondsFormatter = [[[NSNumberFormatter alloc] init] autorelease];
	[secondsFormatter setFormat:@"00;00"];
	[secondsFormatter setMinimum:zero];
	[secondsFormatter setMaximum:maxSeconds];
	[[fTrackDurationSeconds cell] setFormatter:secondsFormatter];

	NSNumberFormatter *tenthSecondFormatter = [[[NSNumberFormatter alloc] init] autorelease];
	[tenthSecondFormatter setFormat:@"0;0"];
	[tenthSecondFormatter setMinimum:zero];
	[tenthSecondFormatter setMaximum:maxTenthSecond];
	[[fTrackDurationTenthSecond cell] setFormatter:tenthSecondFormatter];
	
	NSNumberFormatter *secByCarFormatter = [[[NSNumberFormatter alloc] init] autorelease];
	[secByCarFormatter setFormat:@"0.0"];
	[secByCarFormatter setMinimum:minSecBycar];
	[secByCarFormatter setMaximum:maxSecBycar];
	[[fTrackDurationSecByCar cell] setFormatter:secByCarFormatter];*/
	
	NSString *	secByCharLabel = [fTrackDurationSecByCar stringValue];
	NSMutableString *	secByCharNewLabel = [[NSMutableString alloc] initWithCapacity:[secByCharLabel length]];
	[secByCharNewLabel setString:secByCharLabel];
	[secByCharNewLabel	replaceOccurrencesOfString:@"0.0"
						withString:[NSString stringWithFormat:@"%.1f",[[MIPreferences sharedPrefs] textClipScrollSpeed]] 															options:NSLiteralSearch
						range:NSMakeRange(0, [secByCharNewLabel length])];
	[fTrackDurationSecByCar setStringValue:secByCharNewLabel];
	[secByCharNewLabel release];

	[self secondsToMinSecTenthSec:[[MIPreferences sharedPrefs] textClipDefaultDuration]];
}

- (void)applicationWillTerminate:(NSNotification*)theNotification
{
	vernl(6,"[MITextController applicationWillTerminate]");
   // [self release];
}

- (void)windowDidLoad
{
	//GP : not needed anymore : delegation already done in IB. : [fTextView setDelegate:self];
	//[fTextView insertText:@""];
	[self updateButtonStatus:fOKButton];
	[self updateButtonStatus:fPreviewButton];
	[self updateCalculateButtonStatus];
	[self updateBackgroundColorRadioButtons];
}

- (void)updateButtonStatus:(IBOutlet id)button
{
	// Enable or disable the button according to the NSTextView text length (empty or not).
	NSString* string = [fTextView string];
	if (	([string cStringLength] == 0)
		||	(	([fTrackDurationMinutes intValue] == 0)
			&&	([fTrackDurationSeconds intValue] == 0)
			&&	([fTrackDurationTenthSecond intValue] == 0)))
		[button setEnabled:false];
	else
		if (![button isEnabled])
			[button setEnabled:true];
}

- (void)updateCalculateButtonStatus
{
	// Enable or disable the calculate button according to the NSTextView text length (empty or not).
	if ([[fTextView string] cStringLength] == 0)
		[fCalculateDurationButton setEnabled:false];
	else
		if (![fCalculateDurationButton isEnabled])
			[fCalculateDurationButton setEnabled:true];
}

/*- (BOOL) oneTextFieldActive
{
	return	(	([fTrackDurationMinutes currentEditor] != nil)
			||	([fTrackDurationSeconds currentEditor] != nil)
			||	([fTrackDurationTenthSecond currentEditor] != nil)
			||	([fCustomSizeTextField currentEditor] != nil));
}*/

// NSTextView behavior in a dialog with an OK button :
// If the return key is pressed, insert a new line.
// If the enter key is pressed, validate the dialog -> ok button.
- (BOOL)textView:(NSTextView *)textView shouldChangeTextInRange:(NSRange)affectedCharRange replacementString:(NSString *)replacementString
{
	/*id firstResponder = [[self window] firstResponder];
	if (firstResponder)
		if (![firstResponder isKindOfClass:[NSTextView class]])
			return true;*/
	// Check if one of our NSTextfield is active.
	/*if ([self oneTextFieldActive])
		return true;*/

	/*unsigned short	keyCode = [[[self window] currentEvent] keyCode];
	if (keyCode == 76NSEnterCharacter)*/
	NSEvent *	theCurrentEvent = [[self window] currentEvent];
	if ([theCurrentEvent type] != NSKeyDown)
		return true;

	NSString *		key = [theCurrentEvent charactersIgnoringModifiers];
	const char *	keyCStr = [key UTF8String/*cString*/];
	if (*keyCStr == NSEnterCharacter)
		{
		[fOKButton performClick:self];
		return false;
		}
	return true;
}


/*
- (BOOL)textShouldBeginEditing:(NSText *)textObject
{
	//[fOKButton setKeyEquivalent:@""];
	return true;
}


- (BOOL)textShouldEndEditing:(NSText *)textObject
{
	//[fOKButton setKeyEquivalent:@"\r"];
	return true;
}
*/

// GP : delegate from NSTextField.
// Why is it not textDidChange, as written in apple's doc? -> BUG?
// Or is it the NSFormater linked with the NSTextField?
- (void)controlTextDidChange:(NSNotification *)aNotification
{
	[self updateButtonStatus:fOKButton];
	[self updateButtonStatus:fPreviewButton];
}

// Delegate from NSTextEditView.
- (void)textDidChange:(NSNotification *)aNotification
{
	// Force the horizontal alignment in cas of a not same aligned paste.
	[self updateHAlignment];
	// Update Buttons.
	[self updateButtonStatus:fOKButton];
	[self updateButtonStatus:fPreviewButton];
	
	// Enable or disable the calculate button according to the NSTextView text length (empty or not).
	if ([[fTextView string] cStringLength] == 0)
		[fCalculateDurationButton setEnabled:false];
	else
		if (![fCalculateDurationButton isEnabled])
			[fCalculateDurationButton setEnabled:true];

	// are we going to account for formatting, or just characters?
	// if so need to use [fTextView RTFFromRange:range] or something
	// at default font/size/work area zoom, 5 characters a second looks about right
	/*NSString* string = [fTextView string];
	float duration = [string length] * [[MIPreferences sharedPrefs] textClipScrollSpeed];
	CalculateMinSecTenthSec(duration);*/
}

- (void) secondsToMinSecTenthSec:(float) floatSeconds
{
	int	minutes, seconds, tenthSeconds;
	minutes = int(floatSeconds / 60.0);
	seconds = int(floatSeconds - (minutes * 60));
	tenthSeconds = int((floatSeconds - (int)floatSeconds) * 10.0);
	[fTrackDurationMinutes setIntValue:minutes];
	[fTrackDurationSeconds setIntValue:seconds];
	[fTrackDurationTenthSecond setIntValue:tenthSeconds];
}

- (float) minSecTenthSecToSeconds
{
	return		[fTrackDurationMinutes floatValue] * 60.0
			+ 	[fTrackDurationSeconds floatValue]
			+	[fTrackDurationTenthSecond floatValue] / 10.0;
}


/* GP : not needed anymore;
- (void)orderFrontColorPanel:(id)sender;
{
// why does this need to be here? Shouldn't NSApp pick it up automagically?
    [NSApp orderFrontColorPanel:self];
}
GP */

- (TArmeWSObj*)doTextClipDialog:(TArmeDoc*)doc
{
	fCurrentDoc = doc;
    TArmeTextClip* newClip = nil;
    fOKClicked = NO;

	[self scrollingChanged:nil];
	[self setupFontPopupButton];
	[self setupAlignPopupButtons];

	NSFontManager *fontManager = [NSFontManager sharedFontManager];
	[fontManager fontPanel:YES];

	[fTextView setUsesFontPanel:YES];
	[fTextView setAlignment:NSCenterTextAlignment];
	
	// Georges wants the text always selected
    [[self window] makeFirstResponder:fTextView];
    [NSApp runModalForWindow:[self window]];
    [[self window] orderOut:self];

    if (fOKClicked)
		{
		// GP : fill the TextParams struct.
		TextParams	textParams;
		textParams.scrollType = [[fScrollingMatrix selectedCell] tag];
		textParams.textKeyed = [[fBackgroundMatrix selectedCell] tag];

		textParams.trackBackground = [fBackgroundColorWell color];
	
		textParams.clipDuration = doc->SecondsToArmeTime([self minSecTenthSecToSeconds]);
		textParams.textJustify = fTextJustify;
		textParams.verticalAlignment = fVerticalTextAlignment;
		
		[fTextView selectAll:self];
		NSRange range = [fTextView selectedRange];

		textParams.richText = [fTextView RTFFromRange:range];
		textParams.plainText = [fTextView string];
		textParams.styledText = nil;

		newClip = new TArmeTextClip("", fCurrentDoc, doc->GetZeroPos(), textParams);
		}

    return newClip;
}

/// GP : old : - (ArmeDataFileRef*) editTextClip:(TArmeTextClip*)owner;
- (BOOL) editTextClip:(TArmeTextClip*)owner
{
	[self window];
	
	fCurrentDoc = owner->GetDoc();
    fOKClicked = NO;
	
	[fScrollingMatrix selectCellWithTag:(owner->GetScrollType())];
	[fBackgroundMatrix selectCellWithTag:(owner->GetTextKeyed())];

	[fBackgroundColorWell setColor:owner->GetTrackBackground()];
    NSColor* textColor = [owner->GetStyledText() attribute:NSForegroundColorAttributeName atIndex:0 effectiveRange:nil];
    if (textColor == nil) textColor=[NSColor blackColor];
    [fTextColorWell setColor:textColor];

    [self secondsToMinSecTenthSec:fCurrentDoc->ArmeTimeToSeconds(owner->GetDuration())];
	[fTextView selectAll:self];
	NSRange range = [fTextView selectedRange];
	if (owner->GetRichText() != nil)
		[fTextView replaceCharactersInRange:range withRTF:owner->GetRichText()];
	else
		if (owner->GetPlainText() != nil)
			[fTextView replaceCharactersInRange:range withString:owner->GetPlainText()];
		else
			if (owner->GetStyledText() != nil)
				{
				NSAttributedString *	theStyledText = owner->GetStyledText();
				NSRange		theRange = NSMakeRange(0, [theStyledText length]);
				NSData *	theNSData = [theStyledText RTFFromRange:theRange documentAttributes:nil];
				[fTextView replaceCharactersInRange:range withRTF:theNSData];
				}
	fTextJustify = owner->GetTextJustify();
	fVerticalTextAlignment = (MIVerticalTextAlignment)owner->GetVerticalAlignment();

	[self scrollingChanged:nil];
	[self setupFontPopupButton];
	[self setupAlignPopupButtons];
	[self updateButtonStatus:fOKButton];
	[self updateButtonStatus:fPreviewButton];
	[self updateBackgroundColorRadioButtons];
	[fCalculateDurationButton setEnabled:true];

	NSFontManager *fontManager = [NSFontManager sharedFontManager];
	[fontManager fontPanel:YES];
	[fTextView setUsesFontPanel:YES];	

	// Georges wants the text always selected
    [[self window] makeFirstResponder:fTextView];
 	[fTextView selectAll:self];
    [NSApp runModalForWindow:[self window]];
    [[self window] orderOut:self];

    if (fOKClicked)
		{
		//the clips has beenn modified, we first need to remove it from the master movie
		owner->fMasterMovie->RemoveSubMovie(owner);

		//then modify the object
		
		[fTextView selectAll:self];
		NSRange range = [fTextView selectedRange];

		owner->SetTextSpecs([[fScrollingMatrix selectedCell] tag],
							[[fBackgroundMatrix selectedCell] tag],
                            [fBackgroundColorWell color],
							fCurrentDoc->SecondsToArmeTime([self minSecTenthSecToSeconds]),
							[fTextView RTFFromRange:range],
							[fTextView string],
							nil,
							fTextJustify,
							fVerticalTextAlignment);
		
//		owner->_SetLength(fCurrentDoc->SecondsToArmeTime(owner->GetTrackDuration()));

		//then, insert the clip in the mastermovie once it has been modified
		owner->fMasterMovie->AddSubMovie(owner);
		owner->SetMuted(!owner->IsVisible());
		
		return true;
		}

	return false;
}

- (void) makeMovieForPreview;
{
	// The height and the width of the the movie.
	/*short		theMovieHeight = fCurrentDoc->fMasterMovie->GetHeight();
	short		theMovieWidth = fCurrentDoc->fMasterMovie->GetWidth();*/
	// As the viewer ratio is always 1,33333, we keep the same proportions .
	short		theMovieHeight = fCurrentDoc->fMasterMovie->GetViewerHeight();
	short		theMovieWidth = fCurrentDoc->fMasterMovie->GetViewerWidth();
	Rect		textRect = {0, 0, theMovieHeight, theMovieWidth};

	//to make the movie, we need to add media to the movie.  For this, there must be a file associated.
	// so we create a temporary file that will be deleted at the end.
	char path[kMaxPathLength];
	ZzAppendFileNameToPath(path, kMaxPathLength, fCurrentDoc->GetProjectFolderPath(), "temptext.mov");
	
	//remove the previous temp file
	// GP : already removed in FSSpecFromNonExistingFilePath. Comment : remove(path);
	// GP : CreateMovieFile already creates the file. Comment : fMovieFile = FSSpecFromNonExistingFilePath(path, FOUR_CHAR_CODE('TVOD'), FOUR_CHAR_CODE('MooV'));
	fMovieFile = Path2FSspec(path, true);

	OSStatus err = ::CreateMovieFile(	&fMovieFile,
										'TVOD',
										smSystemScript, 
										createMovieFileDeleteCurFile | createMovieFileDontCreateResFile,
										&fRefNum,
										&fMovie);
    if (err)
		{
        vernl(0,"makeMovieForPreview : error during CreatingMovieFile");
        return;
		}
	SetMovieTimeScale(fMovie, fCurrentDoc->fMasterMovie->GetVideoTimeScale());                
    
	// Set the beginning and the end of the clip as TArmeTime.
	ArmeTime startTime, endTime;
	startTime = fCurrentDoc->SecondsToArmeTime(0.0);
	endTime = fCurrentDoc->SecondsToArmeTime([self minSecTenthSecToSeconds]);

    //$$Test
    // GP : ? : projectRect.bottom /= 2;
    TimeScale projectTimeScale = GetMovieTimeScale(fCurrentDoc->fMasterMovie->GetQTMovie());

    // make the track
    Track textTrack = ::NewMovieTrack(	fMovie,
										(long)(textRect.right - textRect.left) << 16,
										(long)(textRect.bottom - textRect.top) << 16,
										0);
    Media textMedia = ::NewTrackMedia(textTrack, TextMediaType, projectTimeScale, 0, 0);
	
	SetMediaPlayHints(textMedia, hintsHighQuality, hintsHighQuality);

    MediaHandler textHandler = ::GetMediaHandler(textMedia);

    // get background color
	RGBColor whiteRGBColor = {0xFFFF, 0xFFFF, 0xFFFF};
	RGBColor blackRGBColor = {0, 0, 0};
    RGBColor background;
    NSColor* bg = [fBackgroundColorWell color];
    background.red = (short)(0xFFFF * [bg redComponent]);
    background.green = (short)(0xFFFF * [bg greenComponent]);
    background.blue = (short)(0xFFFF * [bg blueComponent]);
    
    // get flags
    long flags = 0; // dfShrinkTextBoxToFit?

    /* BUG IN PREVIEW!
	if ([[fBackgroundMatrix selectedCell] tag])
        flags += dfKeyedText;
	*/

	//flags += dfClipToTextBox;
	flags += dfShrinkTextBoxToFit;
	//flags += dfDontAutoScale;

	MIBackupPasteboard	* theBackupPasteboard = [MIBackupPasteboard backupPasteboard];
	[theBackupPasteboard backup];

	short	theScrollType = [[fScrollingMatrix selectedCell] tag];
 
	// Create a new GWorld where we'll do our text things...
	GrafPtr		savePort;
	GWorldPtr	offscreenGWorld = nil;
	QDErr		theQDErr = ::NewGWorld (&offscreenGWorld, 0, &textRect, NULL, NULL, 0);
	if (theQDErr == noErr)
		{
		::GetPort(&savePort);
		::SetGWorld(offscreenGWorld, NULL);
		}

	// create text source
	//    Rect emptyRect = { 0 };
	//    TEHandle theTE = ::TEStyleNew(&projectRect, &emptyRect);
	/*Rect	textRect2 = textRect;
	textRect2.right *= 2;*/
	Rect	destRect = textRect;
	if ((theScrollType == scrollHorizontal) || (theScrollType == scrollHorizontalReverse))
		destRect.right = 32767;
    TEHandle theTE = ::TEStyleNew(&destRect, &textRect);
	// Don't wrap the text in case of horizontal scrolling.
	if ((theScrollType == scrollHorizontal) || (theScrollType == scrollHorizontalReverse))
		(**theTE).crOnly = -1;

    // no formatting, LF/CR conflict
    //::TESetText([[fTextView string] cString], [[fTextView string] cStringLength], theTE);
    NSRange oldRange = [fTextView selectedRange];
    [fTextView selectAll:self];
    [fTextView copy:self];
    [fTextView setSelectedRange:oldRange];
    ::TEStylePaste(theTE);

	[theBackupPasteboard retrieve];
	[theBackupPasteboard release];

	// Get the text height.
	long	theTextHeight = ::TEGetHeight(32767, 0, theTE);

	// Find the longuest line length of the text.
	int	theTextWidth = 0;
	for (int i = 0 ; i < (**theTE).nLines ; i++)
		{
		::TESetSelect((**theTE).lineStarts[i], (**theTE).lineStarts[i + 1], theTE);
		if (((**theTE).selRect.right - (**theTE).selRect.left) > theTextWidth)
			theTextWidth = (**theTE).selRect.right - (**theTE).selRect.left;
		}

	if (	(theScrollType == scrollHorizontal)
		||	(theScrollType == scrollHorizontalReverse)
		||	(theScrollType == noScroll))
		switch ([[fVerticalAlignmentPopupButton selectedCell] tag])
			{
			case MITopTextAlignment :
				break;
			case MICenterTextAlignment :
				textRect.top = theMovieHeight/2 - theTextHeight/2;
				textRect.bottom = textRect.top + theTextHeight;
				break;
			case MIBottomTextAlignment :
				textRect.bottom = theMovieHeight;
				textRect.top = textRect.bottom - theTextHeight;
				break;
			}

	short	textJustify = fTextJustify;
	if ((theScrollType == scrollHorizontal) || (theScrollType == scrollHorizontalReverse))
		textJustify = teFlushLeft;

	// add text sample
	TimeValue duration = fCurrentDoc->TimeValueFromArmeTime(endTime - startTime);
    TimeValue sampleTime = 0;

    if (theScrollType == noScroll)
		{
		::BeginMediaEdits(textMedia);

		::TextMediaAddTESample(	textHandler,
								theTE,
								[[fBackgroundMatrix selectedCell] tag] ? &whiteRGBColor : &background,
								textJustify,
								&textRect,
								flags,        
								0,/*scrollDelay,*/
								0,
								0,
								&blackRGBColor,
								duration,
								&sampleTime);
    
		OSErr res1 = GetMoviesError();
	
		// finish movie up
		::EndMediaEdits(textMedia);
		OSErr res2 = GetMoviesError();
		ThrowOnOSStatus(res1);
		ThrowOnOSStatus(res2);
		::InsertMediaIntoTrack(	textTrack,
								fCurrentDoc->TimeValueFromArmeTime(startTime),
								sampleTime, 
								duration,//videoFrameDuration,//::GetMediaDuration(textMedia),
								fixed1);
		res2 = GetMoviesError();
		ThrowOnOSStatus(res2);
		}
	else
		{
		// Frame duration.
		TimeValue	videoFrameDuration = fCurrentDoc->GetVideoFrameDuration();

		// Absolute times.
		ArmeTime	theAbsoluteStart = startTime;//GetAbsoluteStartTimeOfVideoClip() - fStartTrim;
		ArmeTime	theAbsoluteEnd = endTime;//GetAbsoluteEndxTimeOfVideoClip() + fEndTrim;
		ArmeTime	theAbsoluteLength = theAbsoluteEnd - theAbsoluteStart;
	
		// Project Frames per second.
		float		projectFPS = fCurrentDoc->fMasterMovie->GetFPS();
	
		// Number of frames.
		long		theNumberOfFrames = (long)ceil((fCurrentDoc->ArmeTimeToSeconds(endTime - startTime) * projectFPS) - 0.5);
	
		// Percentage of this part from the total time of the text.
		float		thePercentage = float(endTime.data - startTime.data) / float(theAbsoluteLength.data);
	
		// The current height and width;
		short		theRectHeight = short((theMovieHeight + theTextHeight) * thePercentage);
		short		theRectWidth = short((theMovieWidth + theTextWidth) * thePercentage);
		
		// Increment of each frame in pixels.
		float		theVPixelsIncrement = float(theRectHeight) / float(theNumberOfFrames);
		float		theHPixelsIncrement = float(theRectWidth) / float(theNumberOfFrames);
	
		// The starting text box rect.
		float		theStartFrame;
		if ((theScrollType == scrollVertical) || (theScrollType == scrollHorizontal))
			theStartFrame = ceil((fCurrentDoc->ArmeTimeToSeconds(theAbsoluteEnd - endTime) * projectFPS) - 0.5);
		else
			if ((theScrollType == scrollVerticalReverse) || (theScrollType == scrollHorizontalReverse))
				theStartFrame = ceil((fCurrentDoc->ArmeTimeToSeconds(endTime - theAbsoluteStart) * projectFPS) - 0.5);
	
		short		startTop = short(theStartFrame * theVPixelsIncrement);
		short		startLeft = short(theStartFrame * theHPixelsIncrement);

		MIProgressBarDlgManager progDlgManager("Creating Text Clip preview...");

		for (int i = 0 ; i <= theNumberOfFrames ; i++)
			{
			progDlgManager.Update(float(i + 1) / theNumberOfFrames);
			if (theScrollType == scrollVertical)
				{
				textRect.top = (startTop + short(i * theVPixelsIncrement)) - theTextHeight;
				textRect.bottom = textRect.top + theTextHeight;
				}
			else
				if (theScrollType == scrollVerticalReverse)
					{
					textRect.top = (startTop - short(i * theVPixelsIncrement)) - theTextHeight;
					textRect.bottom = textRect.top + theTextHeight;
					}
				else
					if (theScrollType == scrollHorizontal)
						{
						textRect.left = (startLeft + short(i * theHPixelsIncrement)) - theTextWidth;
						textRect.right = textRect.left + theTextWidth;
						}
					else
						if (theScrollType == scrollHorizontalReverse)
							{
							textRect.left = (startLeft - short(i * theHPixelsIncrement)) - theTextWidth;
							textRect.right = textRect.left + theTextWidth;
							}
	
			::BeginMediaEdits(textMedia);

			::TextMediaAddTESample(	textHandler,
									theTE,
									[[fBackgroundMatrix selectedCell] tag] ? &whiteRGBColor : &background,
									textJustify,
									&textRect,
									flags,        
									0,/*scrollDelay,*/
									0,
									0,
									&blackRGBColor,
									videoFrameDuration,
									&sampleTime);
			
			OSErr res1 = GetMoviesError();
	
			// finish movie up
			::EndMediaEdits(textMedia);
			OSErr res2 = GetMoviesError();
			ThrowOnOSStatus(res1);
			ThrowOnOSStatus(res2);
			::InsertMediaIntoTrack(	textTrack,
									fCurrentDoc->TimeValueFromArmeTime(startTime),
									sampleTime, // the time where the new media has been added
									videoFrameDuration, // the duration of the new media added
									fixed1);

			res2 = GetMoviesError();
			ThrowOnOSStatus(res2);
			}
		progDlgManager.Delete();
		}

	//::SetTrackLayer(textTrack, kUnkeyedTextLayer);
    short resID = movieInDataForkResID;
    ::AddMovieResource(fMovie, fRefNum, &resID, nil);
    ::CloseMovieFile(fRefNum);
    fRefNum = 0;
	
	::TEDispose(theTE);

	if (theQDErr == noErr)
		{
		::SetPort(savePort);
		::DisposeGWorld(offscreenGWorld);	
		}
}


/*
[10:18]  saibot: 		NSRange 			rng;
[10:18]  saibot: 		int					length = [theString length];
[10:18]  saibot: 		NSMutableDictionary	*dic = [NSMutableDictionary dictionaryWithCapacity:5];
[10:18]  saibot: 		NSFont				*oldFont;		
[10:18]  saibot: 		float				scaleFactor; //factor by which the fonts will be scaled. 
[10:18]  saibot: 		rng.location = 0;
[10:18]  saibot: 		rng.length = 0;
[10:18]  saibot: 		while ( NSMaxRange(rng) < length )
[10:19]  saibot: 		{
[10:19]  saibot: 			[dic setDictionary:[theString attributesAtIndex:NSMaxRange(rng) effectiveRange:&rng]];
[10:19]  saibot: 			oldFont = [dic objectForKey:NSFontAttributeName];
[10:19]  saibot: 			[dic setObject:[NSFont fontWithName:[oldFont fontName] size:[oldFont pointSize]*scaleFactor]];
[10:19]  saibot: 			[theString setAttributes:dic range:rng];
[10:19]  saibot: 		}
[10:19]  Alain: ok merci
[10:19]  saibot: theString doit tre une NSMutableAtributedString
[10:20]  saibot: si tu as une NSAttributedString (non mutable)
[10:20]  saibot: tu fais
[10:21]  saibot: 	NSMutableAttributedString *theString = [[NSMutableAttributedString alloc] init];
[10:21]  saibot: 	[theString setAttributedString:clip->GetStyledText()];
*/

- (IBAction) makePreview:(id)sender
{
	/*Point where = {-1, -1};
	RGBColor inColor = {0, 0, 0}, outColor;
	Boolean getColor = ::GetColor(where, "\pChoose the background color :", &inColor, &outColor);*/

	/*ColorPickerInfo	theColorInfo;
	
	OSErr theErr = ::PickColor(&theColorInfo);
	if (theErr != noErr)
		NSBeep();*/

	if ([fMoviePreviewView movie] != nil)
		if ([fMoviePreviewView isPlaying])
			[fMoviePreviewView stop:sender];

	[self cleanMovie];
	[self makeMovieForPreview];

	// and set to preview
    [fMoviePreviewView showController:NO adjustingSize:NO];
    NSMovie *movie = [[NSMovie alloc] initWithMovie:fMovie];
    [fMoviePreviewView setMovie:movie];	// GP : BUG : WRITE THE TEXT NON STYLED IN VIEWER!!!
    [fMoviePreviewView start:sender];

    /*[fMoviePreviewView setMovie:nil];

	//[self makeMovieForPreview];

	// retrieve our movie file from our bundle resource.
	NSBundle *	theBundle = [NSBundle mainBundle];
    NSString *filePath = [theBundle pathForResource:@"temptext" ofType:@"mov"];
    NSURL *movieUrl = [NSURL fileURLWithPath:filePath];
    // create an NSMovie object from our QuickTime movie;
    NSMovie *theMovie = [[NSMovie alloc] initWithURL:movieUrl byReference:YES];*/

	/*ZzString zzString(fCurrentDoc->GetProjectFolderPath());
	//zzString += ZzString("/");
	zzString += ZzString("temptext.mov");
	NSString* filePath = zzString.getNSString();

	NSURL *movieUrl = [NSURL fileURLWithPath:filePath];
	// create an NSMovie object from our QuickTime movie;
    NSMovie *theMovie = [[NSMovie alloc] initWithURL:movieUrl byReference:YES];

    // assign the NSMovie object (which we made above) for our NSMovieView object.
    [fMoviePreviewView setMovie:theMovie];
	
	[fMoviePreviewView start:sender];*/
}

- (IBAction) cancelClicked:(id)sender
{
    [NSApp stopModal];

    [self cleanMovie];	
}
    
- (IBAction) okClicked:(id)sender
{
    [NSApp stopModal];

    [self cleanMovie];
	
	// we're just sending file
	::DisposeMovie(fMovie);
     fMovie = nil;
   
    fOKClicked = YES;
}

- (void) cleanMovie
{
    if (fMovie == nil)
        return;
        
    // Stupid, stupid NSMovie insists on a DisposeMovie() apparently
    NSMovie *movie = [fMoviePreviewView movie];
    [fMoviePreviewView setMovie:nil];

    [movie release];
    fMovie = nil;
}

- (IBAction)setVAlignmentCenter:(id)sender
{
	fVerticalTextAlignment = MICenterTextAlignment;
}

- (IBAction)setVAlignmentTop:(id)sender
{
	fVerticalTextAlignment = MITopTextAlignment;
}

- (IBAction)setVAlignmentBottom:(id)sender
{
	fVerticalTextAlignment = MIBottomTextAlignment;
}

- (IBAction)setHAlignmentCenter:(id)sender
{
	[self setHAlignment:NSCenterTextAlignment];
}

- (IBAction)setHAlignmentLeft:(id)sender
{
	[self setHAlignment:NSLeftTextAlignment];
}

- (IBAction)setHAlignmentRight:(id)sender
{
	[self setHAlignment:NSRightTextAlignment];
}

- (void)setHAlignment:(NSTextAlignment)mode
{
	[fTextView setAlignment:mode];
	
	switch (mode)
	{
		case NSRightTextAlignment :
			fTextJustify = teFlushRight;
			break;
		case NSCenterTextAlignment :
			fTextJustify = teCenter; 
			break;
		case NSLeftTextAlignment :
		default:
			fTextJustify = teFlushLeft;
			break;
	}
}

- (void) updateHAlignment
{
	switch ([[fHorizontalAlignmentPopupButton selectedCell]tag])
		{
		case MILeftTextAlignment :
			[fTextView setAlignment:NSLeftTextAlignment];
			break;
		case MIHCenterTextAlignment :
			[fTextView setAlignment:NSCenterTextAlignment];
			break;
		case MIRightTextHAlignment :
			[fTextView setAlignment:NSRightTextAlignment];
			break;
		}
}

- (IBAction)scrollingChanged:(id)sender
{
	// Retrieve the current selected cell tag from the scrolling matrix.
	int	scrollingState = [[fScrollingMatrix selectedCell] tag];

	// If no scrolling, allow vertical and horizontal alignment.
	if (scrollingState == noScroll)
		{
		[fHorizontalAlignmentPopupButton setEnabled:YES];
		[fVerticalAlignmentPopupButton setEnabled:YES];
		}
	else
		// GP : If HScroll, VAlignment allowed
		if ((scrollingState == scrollHorizontal) || (scrollingState == scrollHorizontalReverse))
			{
			[fHorizontalAlignmentPopupButton setEnabled:NO];
			[fVerticalAlignmentPopupButton setEnabled:YES];
			}
		else
			// GP : If VScroll, HAlignment allowed
			{
			[fHorizontalAlignmentPopupButton setEnabled:YES];
			[fVerticalAlignmentPopupButton setEnabled:NO];
			}
}

- (void)setupAlignPopupButtons
{
	switch(fTextJustify)
		{
		case - 2 : // Left.
			[fHorizontalAlignmentPopupButton selectItemAtIndex:0];
			break;
		case 1 : // Center.
			[fHorizontalAlignmentPopupButton selectItemAtIndex:1];
			break;
		case - 1 : // Right.
			[fHorizontalAlignmentPopupButton selectItemAtIndex:2];
			break;
		};
	[fVerticalAlignmentPopupButton selectItemAtIndex:fVerticalTextAlignment];
}

- (void) updateBackgroundColorRadioButtons
{
	if ([fBackgroundTransparent state] == NSOnState)
		{
		if ([fBackgroundColorWell isEnabled])
			[fBackgroundColorWell setEnabled:false];
		}
	else
		if (![fBackgroundColorWell isEnabled])
			[fBackgroundColorWell setEnabled:true];
}

- (void) setupFontPopupButton
{
	NSFontManager *fontManager = [NSFontManager sharedFontManager];
	NSCharacterSet *set = [[NSCharacterSet letterCharacterSet] invertedSet];
	[fFontPopupButton removeAllItems];
	[fFontPopupButton addItemsWithTitles:[[fontManager availableFontFamilies] sortedArrayUsingFunction:sortStrings context:set]];
}

- (void) setupStylePopupButton
{
	NSFontManager *fontManager = [NSFontManager sharedFontManager];
	[fStylePopupButton removeAllItems];
	NSEnumerator *enumerator = [[fontManager availableMembersOfFontFamily:[fFontPopupButton titleOfSelectedItem]] objectEnumerator];
	NSArray *array;
	while (( array = [enumerator nextObject] ))
		[fStylePopupButton addItemWithTitle:[array objectAtIndex:1]];
}

- (void) setupSizePopupButton
{
	[fSizePopupButton removeAllItems];
	[fSizePopupButton addItemsWithTitles:[@"9,10,11,12,13,14,18,24,36,48,64,72,96,144,288" componentsSeparatedByString:@","]];
	[[fSizePopupButton menu] addItem:[NSMenuItem separatorItem]];
	[fSizePopupButton addItemWithTitle:@"Other"];
}

- (void) setShouldShowCustomSizeTextField: (BOOL)flag
{
	if (flag)
		{
		[fCustomSizeTextField setDrawsBackground:YES];
		[fCustomSizeTextField setBezeled:YES];
		[fCustomSizeTextField setEnabled:YES];
		[fCustomSizeTextField setFrame:fCustomSizeTextFieldFrame];
		[fCustomSizeTextField setNeedsDisplay:YES];
		}
	else
		{
		[fCustomSizeTextField setEnabled:NO];
		[fCustomSizeTextField setBezeled:NO];
		[fCustomSizeTextField setDrawsBackground:NO];
		[fCustomSizeTextField setStringValue:@""];
		}
}

- (void) selectedFontChanged: (NSNotification *)theNotification
{
	NSFontManager *fontManager = [NSFontManager sharedFontManager];
	NSFont *font = [fontManager convertFont:[fontManager selectedFont]];
	//NSFont *font = [fontManager selectedFont];

	if ([fontManager isMultiple])
		{
		// Retrieve 
		//NSAttributedString

		/*[fSizePopupButton removeAllItems];
		[fStylePopupButton removeAllItems];*/
			{
			[fFontPopupButton selectItemAtIndex:-1];
			[fStylePopupButton removeAllItems];
			[fStylePopupButton addItemsWithTitles:[@"Regular,Bold,Oblique,Bold Oblique" componentsSeparatedByString:@","]];
			[fStylePopupButton selectItemAtIndex:-1];
			if (![[fSizePopupButton titleOfSelectedItem] isEqualToString:@"Other"])
				[fSizePopupButton selectItemAtIndex:-1];
			}
			/*{
			int size = (int)[font pointSize];
			int index = [fSizePopupButton indexOfItemWithTitle:[NSString stringWithFormat:@"%d", size]];
			if (index == -1)
				{
				index = [fSizePopupButton indexOfItemWithTitle:@"Other"];
				[fCustomSizeTextField setIntValue:size];
				[self setShouldShowCustomSizeTextField:YES];
				}
			else
				[self setShouldShowCustomSizeTextField:NO];
			[fSizePopupButton selectItemAtIndex:index];
			}*/
		}
	else
		{
		[fFontPopupButton selectItemWithTitle:[font familyName]];

		[self setupSizePopupButton];
		int size = (int)[font pointSize];
		int index = [fSizePopupButton indexOfItemWithTitle:[NSString stringWithFormat:@"%d", size]];
		if (index == -1)
			{
			index = [fSizePopupButton indexOfItemWithTitle:@"Other"];
			[fCustomSizeTextField setIntValue:size];
			[self setShouldShowCustomSizeTextField:YES];
			}
		else
			[self setShouldShowCustomSizeTextField:NO];
		[fSizePopupButton selectItemAtIndex:index];

		[self setupStylePopupButton];
		
		NSEnumerator *enumerator = [[fontManager availableMembersOfFontFamily:[font familyName]] objectEnumerator];
		NSArray *array;
		while ((array = [enumerator nextObject]))
			if ([[array objectAtIndex:0] isEqualToString:[font fontName]])
				{
				[fStylePopupButton selectItemWithTitle:[array objectAtIndex:1]];
				break;
				}
		}
}

- (IBAction) fontChanged: (id)sender
{
	MIFontManager *fontManager = [MIFontManager sharedFontManager];
	NSString *title = [fSizePopupButton titleOfSelectedItem];
	float size;

	if ([title isEqualToString:@"Other"])
		{
		if (sender == fSizePopupButton)
			{
			[self setShouldShowCustomSizeTextField:YES];
			return ;
			}
		size = [fCustomSizeTextField intValue];
		[[self window] makeFirstResponder:fTextView];
		}
	else
		{
		if (sender == fSizePopupButton)
			[self setShouldShowCustomSizeTextField:NO];
		size = [title intValue];
		}

	/*if ([fontManager isMultiple])
		{
		if (	((sender == fSizePopupButton) || (sender == fCustomSizeTextField))
			&&	(size != 0))
			{
			fSetSize = true;
			NSFont *font = [fontManager convertFont:[fontManager selectedFont]];
			//NSFont * font = [fontManager selectedFont];
			NSFont * newFont = [fontManager convertFont:font toSize:size];
			[fontManager selectFontSize:newFont isMultiple:true];
			//[fontManager setSelectedFont:newFont isMultiple:true];
			//[fontManager modifyFontViaPanel:nil];
			}
		}
	else*/
		{
		NSArray *members = [fontManager availableMembersOfFontFamily:[fFontPopupButton titleOfSelectedItem]];
		NSEnumerator *enumerator = [members objectEnumerator];
		NSArray *array;
		while ((array = [enumerator nextObject]))
			if ([[array objectAtIndex:1] isEqualToString:[fStylePopupButton titleOfSelectedItem]])
				{
				[fontManager selectFondNamed:[array objectAtIndex:0] size:size];
				return;
				}
		[fontManager selectFondNamed:[[members objectAtIndex:0] objectAtIndex:0] size:size];
		}
}

- (IBAction)calculateDuration:(id)sender
{
	NSString* string = [fTextView string];
	float duration = [string length] * [[MIPreferences sharedPrefs] textClipScrollSpeed];
	[self secondsToMinSecTenthSec:duration];
	
	[self updateButtonStatus:fOKButton];
	[self updateButtonStatus:fPreviewButton];
}

- (BOOL)isBackgroundColorWellActive
{
	return [fBackgroundColorWell isActive];
}

- (BOOL)isTextColorWellActive
{
	return [fTextColorWell isActive];
}

- (IBAction)textColorChanged:(id)sender
{
	[[NSColorPanel sharedColorPanel] setTitle:@"Text Color"];
}

- (IBAction)backgroundColorChanged:(id)sender
{
	[[NSColorPanel sharedColorPanel] setTitle:@"Background Color"];
}

- (IBAction)transparentBackgroundClicked:(id)sender
{
	if ([fBackgroundColorWell isEnabled])
		[fBackgroundColorWell setEnabled:false];
}

- (IBAction)solidColorBackgroundClicked:(id)sender
{
	if (![fBackgroundColorWell isEnabled])
		[fBackgroundColorWell setEnabled:true];
}

@end

int sortStrings(id str1, id str2, void *context)
{
	NSCharacterSet *set = (NSCharacterSet*)context;
	return [[str1 stringByTrimmingCharactersInSet:set] compare:[str2 stringByTrimmingCharactersInSet:set] options:NSCaseInsensitiveSearch];
}