/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
/*
	MITime is a class to transform ArmeTime and ArmeTime into humanly readable values
	
	(BEGIN DEPRECATED ):
		it also offers getPixels/setPixels functions to transform pixel values into time, depending
		on the kTimeUnitPerPixel //DEPRECATED conversion factor. Zoom has nothing to do with this class, so values
		gotten and set have to apply the zoom factors after/before using the functions.
	(END DEPRECATED)
	
	The values returned by the miliseconds and audioframes are not each the amount of whichever remains
	in the "time modulo timeUnitsInASecond". The miliseconds are how many miliseconds remain after the
	integer amount of frames have been removed from the portions of seconds.
	
	In the same fashion, getMinutes for "1 hour and 20 minutes" returns 20, not 80
	
	this class also provides functionality to easily add or substract a time to an existing time,
	either in the form of separate HMSF factors or as an ArmeWhateverTime structure.
*/


#ifndef __ARMETIME_H__

#import "MontageInterface.h"

#ifndef ArmeBasicTypes_h
#include "ArmeBasicTypes.h"
#endif //ArmeBasicTypes_h

#ifndef ArmeArchiveHandler_h
#include "ArmeArchiveHandler.h"
#endif //ArmeArchiveHandler_h

#define __ARMETIME_H__

@class MIMontageDocument;

#import "MIDocumentModel.h"

@interface MITime : NSObject
{
@public
	double				fTheTime;
	int					fAudioFrameRate;
	MIFPS				fVideoFrameRate;
	int					fTempo;
	double				fSignatureNum;
	double				fSignatureDen;
	MIMontageDocument	*fMontageDoc;
}
+ (id)timeWithDocument:(MIMontageDocument*)doc;
+ (id)timeWithArmeTime:(ArmeTime)time inDocument:(MIMontageDocument*)doc;
- (id)initWithDocument:(MIMontageDocument*)doc;
- (id)initWithArmeTime:(ArmeTime)time inDocument:(MIMontageDocument*)doc;
- (id)initWithArmeTime:(ArmeTime)time inDocument:(MIMontageDocument*)doc;
- (id)initWithH:(int)hours m:(int)minutes s:(int)seconds vFr:(int)videoFrames ms:(int)miliseconds inDocument:(MIMontageDocument*)doc;
- (id)initWithH:(int)hours m:(int)minutes s:(int)seconds aFr:(int)audioFrames inDocument:(MIMontageDocument*)doc;

- (NSString*)getSMPTEString;
- (NSString*)getBPMString;
- (NSString*)getBPMDurationString;

- (void)getFrameRatesFromDocument;
- (void)getTempoAndSignatureForDocument;

- (ArmeTime)getArmeTime;
- (double)getDoubleTime;
- (int)getHours;
- (int)getMinutes;
- (int)getSeconds;
- (int)getMiliseconds;
- (int)getAudioFrames;
- (int)getVideoFrames;
- (int)getAudioFrameRate;
- (MIFPS)getVideoFrameRate;
- (int)getAbsoluteVideoFrames;

- (void)setArmeTime:(ArmeTime)time;
- (void)setDoubleTime:(double)time;
- (void)setH:(int)hours m:(int)minutes s:(int)seconds vFr:(int)videoFrames ms:(int)miliseconds;
- (void)setH:(int)hours m:(int)minutes s:(int)seconds vFr:(int)videoFrames;
- (void)setH:(int)hours m:(int)minutes s:(int)seconds aFr:(int)audioFrames;
- (void)setH:(int)hours m:(int)minutes s:(int)seconds;

//- (void)setPixels:(float)pixels;
- (void)setHours:(int)hours;
- (void)setMinutes:(int)minutes;
- (void)setSeconds:(int)seconds;
- (void)setMiliseconds:(int)miliseconds;
- (void)setAudioFrames:(int)audioFrames;
- (void)setVideoFrames:(int)videoFrames;
- (void)setAudioFrameRate:(int)audioFrameRate;
- (void)setVideoFrameRate:(MIFPS)videoFrameRate;

- (int)getBars;
- (int)getBeats;
- (int)get16ths; // double-croche
- (int)getPPQs; // parts per quarter (left-overs)
- (void)setBars:(int)bars;
- (void)setBeats:(int)beats;
- (void)set16ths:(int)nb16ths;
- (void)setPPQs:(int)ppq;


- (void)addBars:(int)bars beats:(int)beats n16ths:(int)nb16ths ppq:(int)ppq;
- (void)setBars:(int)bars beats:(int)beats nb16ths:(int)nb16ths ppq:(int)ppq;


- (void)addH:(int)hours m:(int)minutes s:(int)seconds vFr:(int)videoFrames ms:(int)miliseconds;
- (void)addH:(int)hours m:(int)minutes s:(int)seconds aFr:(int)audioFrames;
- (void)addH:(int)hours m:(int)minutes s:(int)seconds;
- (void)addArmeTime:(ArmeTime)time;
- (void)addDoubleTime:(double)time;
- (void)addTime:(MITime*)time;

- (void)subH:(int)hours m:(int)minutes s:(int)seconds vFr:(int)videoFrames ms:(int)miliseconds;
- (void)subH:(int)hours m:(int)minutes s:(int)seconds aFr:(int)audioFrames;
- (void)subArmeTime:(ArmeTime)time;
- (void)subTime:(MITime*)time;

@end
#endif
