/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
#import "MITime.h"
#import "MIMontageDocument.h"

#ifndef TArmeDoc_h
#include "TArmeDoc.h"
#endif //TArmeDoc_h

@implementation MITime
+ (id)timeWithDocument:(MIMontageDocument*)doc
{
	MITime	*theTime;
	
	theTime = [[MITime alloc] initWithDocument:doc];
	[theTime setDoubleTime:0];
	[theTime autorelease];
	return theTime;
}

+ (id)timeWithArmeTime:(ArmeTime)time inDocument:(MIMontageDocument*)doc
{
	MITime	*theTime;
	
	theTime = [[MITime alloc] initWithDocument:doc];
	[theTime setArmeTime:time];
	[theTime autorelease];

	return theTime;
}

- (id)initWithDocument:(MIMontageDocument*)doc
{
	self = [super init];
	if ( !doc )
		return nil;
	
	fMontageDoc = doc;
	fTheTime = 0;
	[self getFrameRatesFromDocument];
	[self getTempoAndSignatureForDocument];
	return self;
}

- (id)copyWithZone:(NSZone*)zone
{
	MITime *t = [MITime allocWithZone:zone];
	
	[t initWithArmeTime:[self getArmeTime] inDocument:fMontageDoc];
	
	return t;
}

- (id)initWithArmeTime:(ArmeTime)time inDocument:(MIMontageDocument*)doc
{
	self = [self initWithDocument:doc];
	
	fTheTime = GET_VALUE(time);

	return self;
}

- (id)initWithH:(int)hours m:(int)minutes s:(int)seconds vFr:(int)videoFrames ms:(int)miliseconds inDocument:(MIMontageDocument*)doc
{
	self = [self initWithDocument:doc];
	
	[self setH:hours m:minutes s:seconds vFr:videoFrames ms:miliseconds];
	
	return self;
}

- (id)initWithH:(int)hours m:(int)minutes s:(int)seconds aFr:(int)audioFrames inDocument:(MIMontageDocument*)doc
{
	self = [self initWithDocument:doc];
	
	[self setH:hours m:minutes s:seconds aFr:audioFrames];
	
	return self;
}

- (void)getFrameRatesFromDocument
{
	fAudioFrameRate = GET_VALUE([fMontageDoc armeDocument]->SecondsToArmeTime(1));
//	fVideoFrameRate = [fMontageDoc document]->GetVideoFrameRate();
/*
	switch( [fMontageDoc FPS] )
	{
		case k24FPS:
		case k25FPS:
		case k30FPSNDF:
		case k30FPSDF:
	}
*/	
	switch( [fMontageDoc armeDocument]->fMasterMovie->GetWorkingCodecType() )
	{
	default:
		fVideoFrameRate = k30FPSNDF;
		break;
	case kDvPal720_576:
		fVideoFrameRate = k25FPS;
		break;
	case kDvNtsc720_480:
		fVideoFrameRate = k30FPSDF;
		break;
	case kMjpeg320_240:
		fVideoFrameRate = k30FPSNDF;
		break;
	case kMjpeg160_120:
		fVideoFrameRate = k24FPS;
		break;
	}
}

- (void)getTempoAndSignatureForDocument
{
	fSignatureNum = [fMontageDoc signatureNumerator];
	fSignatureDen = [fMontageDoc signatureDenominator];
	fTempo = [fMontageDoc tempo];
}

- (ArmeTime)getArmeTime
{
	return _ArmeTime((SInt64)fTheTime);
}

- (int)getHours
{
	int h;
	
	h = (int)(fTheTime / (fAudioFrameRate*60*60));
	
	if (fVideoFrameRate == k30FPSDF)
	{
		float frameCount = [self getAbsoluteVideoFrames];
		h = (int)(frameCount/107892.0f);
	}
	else
		h = (int)(fTheTime / (fAudioFrameRate*60*60));

	return h;
}

- (int)getMinutes
{
	int m;
	
	if (fVideoFrameRate == k30FPSDF)
	{
		float hours = float([self getHours]);
		float frameCount = [self getAbsoluteVideoFrames];
		float frameCountForMinutes = (int)(frameCount - float(hours*107892.0f));
		
		m = (int)((frameCount + float(2*int(frameCountForMinutes/1800.0f)) - float(2*int(frameCountForMinutes/18000.0f)) - 107892.0f*hours)/1800.0f);
	}
	else
		m = (int)(fTheTime / (fAudioFrameRate*60))%60;

	return m;
}

- (int)getSeconds
{
	int s;
	
	if (fVideoFrameRate == k30FPSDF)
	{
		float hours = float([self getHours]);
		float minutes = float([self getMinutes]);
		float frameCount = [self getAbsoluteVideoFrames];
		
		s = int((frameCount - 1798.0f*minutes - float(2*int(minutes/10.0f)) - 107892.0f*hours)/30.0f);
	}
	else
	{
		s = (int)(fTheTime / (fAudioFrameRate))%60;
	}
	
	return s;
}

- (int)getVideoFrames
{
	int vFr;

	switch( fVideoFrameRate )
	{
		case k24FPS:
			vFr = (int)((fmod(fTheTime,fAudioFrameRate)/fAudioFrameRate) * 24);
			break;
		case k25FPS:
			vFr = (int)((fmod(fTheTime, fAudioFrameRate)/fAudioFrameRate) * 25);
			break;
		case k30FPSNDF:
			vFr = (int)((fmod(fTheTime, fAudioFrameRate)/fAudioFrameRate) * 30);
			break;
		case k30FPSDF:
			float hours = float([self getHours]);
			float minutes = float([self getMinutes]);
			float seconds = float([self getSeconds]);
			float frameCount = [self getAbsoluteVideoFrames];
			
			vFr = int(frameCount - 30.0f*seconds - 1798*minutes - float(2*int(minutes/10.0f)) - 107892.0f*hours);
//			if (minutes >= 1)  //the first minutes has 1800 frames; it is not taken into account here !
//				vFr -= 2;

					
/*			int absoluteVideoFrames = [self getAbsoluteVideoFrames];
			int nbMinutes = 0;
			int nbSeconds = 0;
			
			while (absoluteVideoFrames > 0)
			{
				if (((nbMinutes%10)==0) && (absoluteVideoFrames>=1800))
				{
					nbMinutes++;
					absoluteVideoFrames-=1800;
				}
				else if (((nbMinutes%10)!=0) && (absoluteVideoFrames>=1798))
				{
					nbMinutes++;
					absoluteVideoFrames-=1798;
				}
				else if ((nbMinutes%10)!=0)
				{
					if ((nbSeconds == 0) && (absoluteVideoFrames>= 28))
					{
						nbSeconds++;
						absoluteVideoFrames-=28;
					}
					else if (absoluteVideoFrames >= 30)
					{
						nbSeconds++;
						absoluteVideoFrames-=30;
					}
					else
					{
						break;
					}
				}
				else if ((nbMinutes%10)==0)
				{
					if (absoluteVideoFrames >= 30)
					{
						nbSeconds++;
						absoluteVideoFrames-=30;
					}
					else
					{
						break;
					}
				}
			}
			if (((nbMinutes%10)!=0) && (nbSeconds == 0))
			{
				absoluteVideoFrames += 2;
			}

			vFr = absoluteVideoFrames;*/
/*			if ( ([self getMinutes] %10 ) != 0)
				vFr = (int)((fmod(fTheTime, fAudioFrameRate)/fAudioFrameRate) * 28) + 2;
			else
				vFr = (int)((fmod(fTheTime, fAudioFrameRate)/fAudioFrameRate) * 30);
			break;*/
	}

	
	return vFr;
}

- (int)getAbsoluteVideoFrames
{
	int nbFrames;
	double div;
	switch( fVideoFrameRate )
	{
		case k24FPS:
			div = ((double)fAudioFrameRate/24);
			break;
		case k25FPS:
			div = ((double)fAudioFrameRate/25);
			break;
		case k30FPSNDF:
			div = ((double)fAudioFrameRate/30);
			break;
		case k30FPSDF:
			div = ((double)fAudioFrameRate/29.97);
			break;
	}
	nbFrames = int((fTheTime+1)/div);
	return nbFrames;
}

- (int)getMiliseconds
{
	float ms;
	
	ms = (fmod(fTheTime, fAudioFrameRate)/ fAudioFrameRate);
	ms -= (int)ms;
	switch( fVideoFrameRate )
	{
		case k24FPS:
			ms -= ((float)[self getVideoFrames]/24);
			break;
		case k25FPS:
			ms -= ((float)[self getVideoFrames]/25);
			break;
		case k30FPSNDF:
			ms -= ((float)[self getVideoFrames]/30);
			break;
		case k30FPSDF:
			ms = (fTheTime+1)/double([self getAudioFrameRate]);
			ms -= double([self getAbsoluteVideoFrames])/29.97;
			/*
		
			if ( ([self getMinutes] %10 ) != 0)
				ms -= ((float)([self getVideoFrames]-2)/28);
			else
				ms -= ((float)[self getVideoFrames]/30);
*/
			break;
	}

	ms *= 1000;
	
	return (int)rint(ms);
}

- (int)getAudioFrames
{
	int aFr;
	
	aFr = (int)fTheTime % (fAudioFrameRate);
	
	return aFr;
}

- (int)getBars
{
	// samples persecond: fAudioFrameRate
	// beats per second	: ((float)fTempo/60)
	// beats per bar	: (fSignatureNum)
	// seconds per bar	: beats per bar/ beats per second
	// seconds per bar	: (((fSignatureNum)/((float)fTempo/60))
	// samples per bar	: samples per second / seconds per bar
	// samples per bar	: (fAudioFrameRate / (((fSignature*4)/((float)fTempo/60))/((float)fTempo/60)))
	// samples per beat	: (fAudioFrameRate / (fTempo/60))
	int bars;
	double beats;
	
	beats = (fTheTime/fAudioFrameRate)*((float)fTempo/60);
	bars = (int)(beats/(fSignatureNum));
	
	return bars +1;
}


- (int)getBeats
{
	// samples persecond: fAudioFrameRate
	// beats per second	: ((float)fTempo/60)
	// beats per bar	: (fSignatureNum)
	// seconds per bar	: beats per bar/ beats per second
	// seconds per bar	: (((fSignatureNum)/((float)fTempo/60))
	// samples per bar	: samples per second / seconds per bar
	// samples per bar	: (fAudioFrameRate / (((fSignature*4)/((float)fTempo/60))/((float)fTempo/60)))
	// samples per beat	: (fAudioFrameRate / (fTempo/60))
	double	numBeats;
	int		numBars;
	// remove the number of complete bars times the number of samples per bar

	numBeats = ((double)fTheTime/fAudioFrameRate)*((float)fTempo/60);
	numBars = (int)(numBeats/(fSignatureNum));
	numBeats -= ((double)numBars*fSignatureNum);
	
	numBeats++;
	return (int)numBeats;
}

- (int)get16ths // double-croche
{
	int		nb16ths;
	double	numBeats;
	
	numBeats = ((double)fTheTime/fAudioFrameRate)*((float)fTempo/60);
	numBeats -= (int)numBeats;
	nb16ths = (int)(numBeats*(16/fSignatureDen));
	
	return nb16ths +1;
}

- (int)getPPQs // parts per quarter (left-overs)
{
	double	nb16ths;
	int		ppq;
	
	nb16ths = ((double)fTheTime/fAudioFrameRate)*((double)fTempo/60)*(16/fSignatureDen);
	nb16ths -= (int)nb16ths;
	
	ppq = (int)(nb16ths*2048);

	return ppq;
}

- (int)getAudioFrameRate
{
	return fAudioFrameRate;
}

- (MIFPS)getVideoFrameRate
{
	return fVideoFrameRate;
}

- (void)setArmeTime:(ArmeTime)time
{
	fTheTime = GET_VALUE(time);
}

- (void)setDoubleTime:(double)time
{
	fTheTime = time;
}

- (void)setBars:(int)bars beats:(int)beats nb16ths:(int)nb16ths ppq:(int)ppq
{
	fTheTime = 0;
	[self setBars:bars];
	[self setBeats:beats];
	[self set16ths:nb16ths];
	[self setPPQs:ppq]; 
}

- (void)setH:(int)hours m:(int)minutes s:(int)seconds vFr:(int)videoFrames ms:(int)miliseconds
{
	double		theTime = 0;
	int			mult;
	
	mult = fAudioFrameRate;
	
	switch( fVideoFrameRate )
	{
		case k24FPS:
			theTime = (double)(mult * ((float)videoFrames/24));
			break;
		case k25FPS:
			theTime = (double)(mult * ((float)videoFrames/25));
			break;
		case k30FPSNDF:
			theTime = (double)(mult * ((float)videoFrames/30));
			break;
		case k30FPSDF:
			double absoluteVideoFrames = videoFrames;
			absoluteVideoFrames += 30*seconds;
			absoluteVideoFrames += 1798*minutes;
			absoluteVideoFrames += 2*int(minutes/10);
			absoluteVideoFrames += 107892*hours;
			
			if (minutes >= 1)  //the first minutes has 1800 frames; it is not taken into account here !
				absoluteVideoFrames += 2;
			
			theTime = mult * absoluteVideoFrames / 29.97;
			theTime += (double)(mult * ((float)miliseconds/1000));

/*			theTime = (double)(mult * (double(videoFrames) + 30*double(seconds) + 1798*double(minutes) + 2*int(float(minutes)/10.0f) + 107892*double(hours))/29.97);*/
			fTheTime = theTime;
			
			return;
		
/*			if ( ([self getMinutes] %10 ) != 0)
					theTime = (double)(mult * ((float)(videoFrames-2)/28));
			else
				theTime = (double)(mult * ((float)videoFrames/30));*/
			break;
	}

	theTime += (double)(mult * ((float)miliseconds/1000));
	theTime += mult * seconds;
	
	mult *= 60;
	theTime += mult * minutes;
	
	mult *= 60;
	theTime += mult * hours;
	
	fTheTime = theTime;
	
}

- (void)setH:(int)hours m:(int)minutes s:(int)seconds vFr:(int)videoFrames
{
	[self setH:hours m:minutes s:seconds vFr:videoFrames ms:0];
}

- (void)setH:(int)hours m:(int)minutes s:(int)seconds aFr:(int)audioFrames
{
	double		theTime;
	int			mult;

	mult = fAudioFrameRate;
	theTime = (mult * ((float)audioFrames/fAudioFrameRate));
	theTime += mult * seconds;
	
	mult *= 60;
	theTime += mult * minutes;
	
	mult *= 60;
	theTime += mult * hours;

	fTheTime = theTime;
}
- (void)setH:(int)hours m:(int)minutes s:(int)seconds
{
	[self setH:hours m:minutes s:seconds aFr:0];
}


- (void)setHours:(int)hours
{
	// I think we should use the k30FPSDF implementation but, since we are one day to the release, I don't touch the generic code.

	if (fVideoFrameRate == k30FPSDF)
	{
		[self setH:hours m:[self getMinutes] s:[self getSeconds] vFr:[self getVideoFrames] ms:[self getMiliseconds]];
	}
	else
	{
		int oldH;
		float mult;
		
		mult = fAudioFrameRate * 60 * 60;
		
		oldH = [self getHours];
		fTheTime -= (oldH * mult);
		fTheTime += (hours * mult); 
	}
}

- (void)setMinutes:(int)minutes
{
	// I think we should use the k30FPSDF implementation but, since we are one day to the release, I don't touch the generic code.

	if (fVideoFrameRate == k30FPSDF)
	{
		[self setH:[self getHours] m:minutes s:[self getSeconds] vFr:[self getVideoFrames] ms:[self getMiliseconds]];
	}
	else
	{
		int oldM;
		int mult;
		
		mult = fAudioFrameRate * 60;
		
		oldM = [self getMinutes];
		fTheTime -= (oldM * mult);
		fTheTime += (minutes * mult); 
	}
}

- (void)setSeconds:(int)seconds
{
	// I think we should use the k30FPSDF implementation but, since we are one day to the release, I don't touch the generic code.

	if (fVideoFrameRate == k30FPSDF)
	{
		[self setH:[self getHours] m:[self getMinutes] s:seconds vFr:[self getVideoFrames] ms:[self getMiliseconds]];
	}
	else
	{
		int oldS;
		float mult;
		
		mult = fAudioFrameRate;
		
		oldS = [self getSeconds];
		fTheTime -= (oldS * mult);
		fTheTime += (seconds * mult); 
	}
}

- (void)setMiliseconds:(int)miliseconds
{
	// I think we should use the k30FPSDF implementation but, since we are one day to the release, I don't touch the generic code.

	if (fVideoFrameRate == k30FPSDF)
	{
		[self setH:[self getHours] m:[self getMinutes] s:[self getSeconds] vFr:[self getVideoFrames] ms:miliseconds];
	}
	else
	{
	
		int oldMS;
		float mult;
		
		mult = (fAudioFrameRate/1000);
		
		oldMS = [self getMiliseconds];
		fTheTime -= (oldMS * mult);
		fTheTime += (miliseconds * mult); 
	}
}

- (void)setAudioFrames:(int)audioFrames
{
	int oldAFR;
	
	oldAFR = [self getAudioFrames];
	fTheTime -= oldAFR;
	fTheTime += audioFrames;
}

- (void)setVideoFrames:(int)videoFrames
{
	// I think we should use the k30FPSDF implementation but, since we are one day to the release, I don't touch the generic code.

	if (fVideoFrameRate == k30FPSDF)
	{
		[self setH:[self getHours] m:[self getMinutes] s:[self getSeconds] vFr:videoFrames ms:[self getMiliseconds]];
	}
	else
	{
	
		int oldVFR;
		double mult;
		
		switch( fVideoFrameRate )
		{
			case k24FPS:
				mult = ((double)fAudioFrameRate/24);
				break;
			case k25FPS:
				mult = ((double)fAudioFrameRate/25);
				break;
			case k30FPSNDF:
				mult = ((double)fAudioFrameRate/30);
				break;
			case k30FPSDF:
				mult = ((double)fAudioFrameRate/30);
				break;
		}
		if ( fVideoFrameRate != k30FPSDF || ([self getMinutes] % 10) == 0)
		{
			oldVFR = [self getVideoFrames];
			fTheTime -= (oldVFR * mult);
			fTheTime += (videoFrames * mult); 
		}
		else
		{
			oldVFR = [self getVideoFrames] - 2;
			fTheTime -= (oldVFR * (double)fAudioFrameRate/28 );
			fTheTime += ((videoFrames-2) * (double)fAudioFrameRate/28);
		}
	}
}

- (void)setAudioFrameRate:(int)audioFrameRate
{
	fAudioFrameRate = audioFrameRate;
}

- (void)setVideoFrameRate:(MIFPS)videoFrameRate
{
	fVideoFrameRate = videoFrameRate;
}


- (void)setBars:(int)bars
{
	// samples persecond: fAudioFrameRate
	// beats per second	: ((float)fTempo/60)
	// beats per bar	: (((float)fSignatureNumerator/fSignatureDenominator)*4)
	// seconds per bar	: beats per bar/ beats per second
	// seconds per bar	: ((((float)fSignatureNumerator/fSignatureDenominator)*4)/((float)fTempo/60))
	// samples per bar	: samples per second / seconds per bar
	// samples per bar	: (fAudioFrameRate / ((((float)fSignatureNumerator/fSignatureDenominator)*4)/((float)fTempo/60)))
	// samples per beat	: (fAudioFrameRate / ((float)fTempo/60))

	int	oldBars = [self getBars];
	fTheTime += (double)((bars - oldBars)*((double)fAudioFrameRate/((float)fTempo/60)*fSignatureNum));
}

- (void)setBeats:(int)beats
{
	// 1 beat = fAudioFrameRate/ (fTempo/60)
	int oldBeats = [self getBeats];
	
	fTheTime += (double)((beats - oldBeats)*((double)fAudioFrameRate/((float)fTempo/60)));
}

- (void)set16ths:(int)nb16ths
{
	// 1 16th = 1 beat/4
	
	int old16ths = [self get16ths];
	fTheTime += (double)((nb16ths - old16ths)*( ((double)fAudioFrameRate / ((float)fTempo/60))/4 ));
}

- (void)setPPQs:(int)ppq
{
	// ppqs are 2048s of a 16th
	int oldPPQ = [self getPPQs];
	fTheTime += (double)((ppq -oldPPQ)*(((double)fAudioFrameRate / ((float)fTempo/60))/(4*2048) ));
}

- (void)addBars:(int)bars beats:(int)beats n16ths:(int)nb16ths ppq:(int)ppq
{
	[self setBars:[self getBars]+bars];
	[self setBeats:[self getBeats]+beats];
	[self set16ths:[self get16ths]+nb16ths];
	[self setPPQs:[self getPPQs]+ppq];
}

- (void)addH:(int)hours m:(int)minutes s:(int)seconds vFr:(int)videoFrames ms:(int)miliseconds
{
	MITime	*t = [MITime timeWithDocument:fMontageDoc];
	[t setH:hours m:minutes s:seconds vFr:videoFrames ms:miliseconds];
	fTheTime += [t getDoubleTime];
/*	
	SInt64		theTime;
	int			mult;
	
	mult = fAudioFrameRate;
	theTime = mult * (videoFrames/fVideoFrameRate);
	theTime += mult * (miliseconds/1000);
	theTime += mult * seconds;
	
	mult *= 60;
	theTime += mult * minutes;
	
	mult *= 60;
	theTime += mult * hours;
	
	fTheTime += theTime;
*/
}

- (void)addH:(int)hours m:(int)minutes s:(int)seconds
{
	[self addH:hours m:minutes s:seconds aFr:0];
}

- (void)addH:(int)hours m:(int)minutes s:(int)seconds aFr:(int)audioFrames
{
	double		theTime;
	int			mult;

	mult = fAudioFrameRate;
	theTime = mult * (audioFrames/fAudioFrameRate);
	theTime += mult * seconds;
	
	mult *= 60;
	theTime += mult * minutes;
	
	mult *= 60;
	theTime += mult * hours;

	fTheTime += theTime;
}

- (void)addArmeTime:(ArmeTime)time
{
	fTheTime += GET_VALUE(time);
}

- (void)addTime:(MITime*)t
{
	fTheTime += [t getDoubleTime];
}

- (void)subTime:(MITime*)t
{
	fTheTime -= [t getDoubleTime];
}


- (void)addDoubleTime:(double)t
{
	fTheTime += t;
}

- (double)getDoubleTime
{
	return fTheTime;
}

- (void)subH:(int)hours m:(int)minutes s:(int)seconds vFr:(int)videoFrames ms:(int)miliseconds
{
	MITime	*t = [MITime timeWithDocument:fMontageDoc];
	[t setH:hours m:minutes s:seconds vFr:videoFrames ms:miliseconds];
	fTheTime -= [t getDoubleTime];
/*
	SInt64		theTime;
	int			mult;
	
	mult = fAudioFrameRate;
	theTime = mult * (videoFrames/fVideoFrameRate);
	theTime += mult * (miliseconds/1000);
	theTime += mult * seconds;
	
	mult *= 60;
	theTime += mult * minutes;
	
	mult *= 60;
	theTime += mult * hours;
	
	fTheTime -= theTime;
*/
}

- (void)subH:(int)hours m:(int)minutes s:(int)seconds aFr:(int)audioFrames
{
	double		theTime;
	int			mult;

	mult = fAudioFrameRate;
	theTime = mult * (audioFrames/fAudioFrameRate);
	theTime += mult * seconds;
	
	mult *= 60;
	theTime += mult * minutes;
	
	mult *= 60;
	theTime += mult * hours;

	fTheTime -= theTime;
}

- (void)subArmeTime:(ArmeTime)time
{
	fTheTime -= GET_VALUE(time);
}

- (NSString*)getBPMString
{
	NSString	*string;
	
	string = [NSString stringWithFormat:@"%.2d:%.2d:%.2d:%.4d",
				[self getBars],
				[self getBeats],
				[self get16ths],
				[self getPPQs]];
	
	return string;
}

- (NSString*)getBPMDurationString
{
	NSString	*string;
	
	string = [NSString stringWithFormat:@"%.2d:%.2d:%.2d:%.4d",
				[self getBars] - 1,
				[self getBeats] - 1,
				[self get16ths] - 1,
				[self getPPQs]];
	
	return string;
}

- (NSString*)getSMPTEString;
{
	NSString	*string;

	if ( fVideoFrameRate != k30FPSDF )
	{
		string = [NSString stringWithFormat:@"%.2d:%.2d:%.2d:%.2d",
					[self getHours],
					[self getMinutes],
					[self getSeconds],
					[self getVideoFrames]];
	}
	else
	{
		string = [NSString stringWithFormat:@"%.2d:%.2d:%.2d;%.2d",
					[self getHours],
					[self getMinutes],
					[self getSeconds],
					[self getVideoFrames]];
	}
	
	return string;
}

- (NSString*)description
{
	NSString	*desc;
	desc = [NSString stringWithFormat:@"Time:%f\nHours:%d Minutes:%d Seconds:%d video frames:%d miliseconds:%d audio frames:%d\nBars:%d Beats:%d 16ths:%d ppq:%d\naudio Frame Rate:%d video Frame Rate:%d",
										fTheTime,
										[self getHours],
										[self getMinutes],
										[self getSeconds],
										[self getVideoFrames],
										[self getMiliseconds],
										[self getAudioFrames],
										[self getBars],
										[self getBeats],
										[self get16ths],
										[self getPPQs],
										[self getAudioFrameRate],
										[self getVideoFrameRate]];

	return desc;
}

@end