/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
#import "MontageInterface.h"

#ifndef ArmeBasicTypes_h
#include "ArmeBasicTypes.h"
#endif //ArmeBasicTypes_h

#import "MIPaletteController.h"
#import "MIDocumentController.h"
#import "MIMontagePalette.h"

enum MITimePaletteFeedbackType
{
	MITimePaletteLocatorsFeedback,
	MITimePaletteStartLocatorFeedback,
	MITimePaletteEndLocatorFeedback,
	MITimePalettePlayLocatorFeedback,
	MITimePaletteOtherLocatorFeedback,
	MITimePaletteSelectionFeedback,
	MITimePaletteTrackerFeedback,
};

@class MIApplicationController;

@interface MITimePaletteController : MIMontagePalette
{
    IBOutlet id					fDFrame;
    IBOutlet id					fDHour;
    IBOutlet id					fDMilisecond;
    IBOutlet id					fDMinute;
    IBOutlet id					fDSecond;
    
	IBOutlet id					fEFrame;
    IBOutlet id					fEHour;
    IBOutlet id					fEMilisecond;
    IBOutlet id					fEMinute;
    IBOutlet id					fESecond;
    
	IBOutlet id					fSFrame;
    IBOutlet id					fSHour;
    IBOutlet id					fSMilisecond;
    IBOutlet id					fSMinute;
    IBOutlet id					fSSecond;

/*
	IBOutlet id					fBigFrame;
	IBOutlet id					fBigHour;
    IBOutlet id					fBigMilisecond;
    IBOutlet id					fBigMinute;
    IBOutlet id					fBigSecond;
*/
	IBOutlet id					fBigTextView;
	
	IBOutlet id					fSBar;
	IBOutlet id					fSBeat;
	IBOutlet id					fS16th;
	IBOutlet id					fSPPQ;

	IBOutlet id					fEBar;
	IBOutlet id					fEBeat;
	IBOutlet id					fE16th;
	IBOutlet id					fEPPQ;

	IBOutlet id					fDBar;
	IBOutlet id					fDBeat;
	IBOutlet id					fD16th;
	IBOutlet id					fDPPQ;
	
	IBOutlet id					fBigTextView2;
	
    IBOutlet id					fView;
	
	IBOutlet id					fSMPTEView;
	IBOutlet id					fBPMView;
		
	ArmeTime					fStartTime;
	ArmeTime					fEndTime;
	ArmeTime					fBigTime;
	
	id							fFeedbackObject;
	MITimePaletteFeedbackType	fFeedbackType;
	
	BOOL						fTimerRunning;
  NSTimer*        fUpdateTimer;
	
	IBOutlet id					fTemporaryTestBox;
}

+ (MITimePaletteController*)sharedTimePaletteController;
//- (id)init;

- (IBAction)durationChanged:(id)sender;
- (IBAction)endChanged:(id)sender;
- (IBAction)startChanged:(id)sender;
- (IBAction)bigChanged:(id)sender;

- (void)setStartTime:(ArmeTime)time;
- (void)setEndTime:(ArmeTime)time;
- (void)setBigTime:(ArmeTime)time;

- (void)refreshStartBoxes;
- (void)refreshEndBoxes;
- (void)refreshDurationBoxes;
- (void)refreshBigBoxes;

- (void)startBarMoved:(NSNotification*)notification;
- (void)endBarMoved:(NSNotification*)notification;
- (void)playBarMoved:(NSNotification*)notification;
- (void)selectionDidChange:(NSNotification*)notification;

- (void)getValuesFromMainView;
- (void)checkSMPTEBPM;

- (void)setFeedbackType:(MITimePaletteFeedbackType)type;
- (void)setFeedbackObject:(id)object;
- (MITimePaletteFeedbackType)feedbackType;

- (void)startContinualUpdate;
- (void)continualUpdate:(id)argument;
- (void)stopContinualUpdate;
- (ArmeTime)getRealPlaybarTime;


@end
