/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
#import "MITimePaletteController.h"
#import "MIApplicationController.h"
#import "MIMainViewController.h"
#import "MITopLevelContainerRep.h"
#import "MITime.h"
#import "MIPaletteController.h"
#import "MIPreferences.h"
#import "MIMoveCommand.h"
#import "MIObject.h"
#import "MIMontageDocument.h"
#import "MIMouseTracker.h"
#import "MenuTags.h"
#import "TArmeDoc.h"
#import "MontageVerbose.h"

@implementation MITimePaletteController

+ (NSString*)visibleDefaultName {
    return @"TimePaletteVisible";
}

+ (BOOL)factoryVisibleDefault {
    return YES;
}

- (NSString*)nibName {
    return @"MITimePalette";
}

- (int)menuTag {
	return MTGTimeMenuTag;
}

- (NSString*)menuName {
	return @"Time";
}


+ (MITimePaletteController*)sharedTimePaletteController
{
    static id sharedTimePaletteController;
    if ( sharedTimePaletteController == nil )
    {
        sharedTimePaletteController = [[self alloc] init];
    }
    return sharedTimePaletteController;
}

- (id)init 
{
    if (( self = [super init] )) 
    {
		  // marcp : this HAS to be done in the init
      [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(documentRemoved:) name:MTGDocumentRemovedNotification object:nil];
    }
    return self;
}

- (void)dealloc
{
  [[NSNotificationCenter defaultCenter] removeObserver:self name:MTGDocumentRemovedNotification object:nil];
  [super dealloc];
}

- (void)setFeedbackType:(MITimePaletteFeedbackType)type
{
	fFeedbackType = type;
	fFeedbackObject = nil;
}

- (MITimePaletteFeedbackType)feedbackType
{
	return fFeedbackType;
}

- (void)setFeedbackObject:(id)object
{
	fFeedbackObject = object;
}

- (IBAction)show:(id)sender
{
    [super show:sender];
	[self checkSMPTEBPM];
}


- (void)awakeFromNib
{
/*	NSFont	*theFont;
	
	theFont = [NSFont menuFontOfSize:24];
	[fBigMinute setFont:theFont];
	[fBigHour setFont:theFont];
	[fBigSecond setFont:theFont];
	[fBigMilisecond setFont:theFont];
	[fBigFrame setFont:theFont];
*/	
	fFeedbackType = MITimePaletteLocatorsFeedback;
	fFeedbackObject = nil;
	fTimerRunning = NO;
  fUpdateTimer = nil;
	[self setStartTime:_ArmeTime(0)];
	[self setEndTime:_ArmeTime(0)];
	[self setBigTime:_ArmeTime(0)];
}

- (void)documentRemoved:(NSNotification*)notification 
{
	vernl(6,"[MITimePaletteController documentRemoved] event time stamp = %f" << [[NSApp currentEvent] timestamp]);
	fFeedbackType = MITimePaletteLocatorsFeedback;
	fFeedbackObject = nil;
}

- (void)setStartTime:(ArmeTime)time
{
	fStartTime = time;
	[self refreshStartBoxes];
	[self refreshDurationBoxes];
}

- (void)setEndTime:(ArmeTime)time
{
	fEndTime = time;
	[self refreshEndBoxes];
	[self refreshDurationBoxes];
}

- (void)setBigTime:(ArmeTime)time;
{
	fBigTime = time;
	[self refreshBigBoxes];
}

- (IBAction)durationChanged:(id)sender
{
	MITime	*time = [MITime timeWithDocument:[[NSDocumentController sharedDocumentController] currentDocument]];
	
	if ([[MIPreferences sharedPrefs] timeDisplay] & kTimeDisplaySMPTE)
	{
		[time setHours:[fDHour intValue]];
		[time setMinutes:[fDMinute intValue]];
		[time setSeconds:[fDSecond intValue]];
		[time setVideoFrames:[fDFrame intValue]];
		[time setMiliseconds:[fDMilisecond intValue]];
	}
	else
	{
		[time setBars:[fDBar intValue] beats:[fDBeat intValue] nb16ths:[fD16th intValue] ppq:[fDPPQ intValue]];
	}
	
}

- (IBAction)endChanged:(id)sender
{
	MITime	*time = [MITime timeWithDocument:[[NSDocumentController sharedDocumentController] currentDocument]];
	MIMoveCommand		*cmd;
	NSEnumerator		*objEnum;
	MIRect				selRect = MIZeroRect;
	MIObject			*anObj;
	MIPoint				offset = MIZeroPoint;
	
	if ([[MIPreferences sharedPrefs] timeDisplay] & kTimeDisplaySMPTE)
	{
		[time setHours:[fEHour intValue]];
		[time setMinutes:[fEMinute intValue]];
		[time setSeconds:[fESecond intValue]];
		[time setVideoFrames:[fEFrame intValue]];
		[time setMiliseconds:[fEMilisecond intValue]];
	}
	else
	{
		[time setBars:[fEBar intValue] beats:[fEBeat intValue] nb16ths:[fE16th intValue] ppq:[fEPPQ intValue]];
	}
	
	switch( fFeedbackType )
	{
		case MITimePaletteLocatorsFeedback:
			[[[NSApp mainViewController] topLevelContainerView] setRightLocatorPosition:[time getArmeTime]];
			break;
		case MITimePaletteStartLocatorFeedback:
			[[[NSApp mainViewController] topLevelContainerView] setLeftLocatorPosition:[time getArmeTime]];
			break;
		case MITimePaletteEndLocatorFeedback:
			[[[NSApp mainViewController] topLevelContainerView] setRightLocatorPosition:[time getArmeTime]];
			break;
		case MITimePalettePlayLocatorFeedback:
			[[[NSApp mainViewController] topLevelContainerView] setCurrentPlayBarPosition:[time getArmeTime]];
			break;
		case MITimePaletteSelectionFeedback:
			if ( [(MIWSSelection*)fFeedbackObject type] == kMIWSSelectionTypeObjects )
			{
                                MIMontageDocument	*doc;
        
                                // move the selection
				objEnum = [fFeedbackObject objectEnumerator];
				
				// get start ArmeTime of the selection
				while ( anObj = [[objEnum nextObject] representedObject] )
				{
					if ( !MIEqualRects( selRect, MIZeroRect) )
						selRect = MIUnionRect(MIZeroRect, [anObj modelFrame]);
					else
						selRect = [anObj modelFrame];
					doc = [anObj montageDoc];
				}
				
				offset.time = _ArmeTime((SInt64)([time getDoubleTime] - GET_VALUE(MIMaxX(selRect))));
				if ( GET_VALUE(offset.time) != 0 )
				{
					VUSERCMD1("Move (through Time Palette), offset = %d", (int)(offset.time.data));
					VUSERCMD_PRINT_SELECTION((MIWSSelection*)fFeedbackObject);

					cmd = [[[MIMoveCommand alloc] initWithSelection:fFeedbackObject] autorelease];
					[cmd setOffset:offset];
					[cmd setNewParent:nil]; // keep the same parent that each children already has.
					[doc doCmd:cmd];
				}
			}
			else if ( [(MIWSSelection*)fFeedbackObject type] == kMIWSSelectionTypeTimeSlice )
			{
				[fFeedbackObject setEndxTime:[time getArmeTime]];
				[[[NSApp mainViewController] topLevelContainerView] setNeedsDisplayInRect:[[fFeedbackObject objectAtIndex:0] getTLCFrame]];
			}
			
			[self getValuesFromMainView];
			break;
		case MITimePaletteOtherLocatorFeedback:
			// move or resize 
			break;
		default:
			[[[NSApp mainViewController] topLevelContainerView] setRightLocatorPosition:[time getArmeTime]];
			break;
	}
}

- (IBAction)startChanged:(id)sender
{
	MITime				*time = [MITime timeWithDocument:[[NSDocumentController sharedDocumentController] currentDocument]];
	MIMoveCommand		*cmd;
	NSEnumerator		*objEnum;
	MIRect				selRect = MIZeroRect;
	MIObject			*anObj;
	MIPoint				offset = MIZeroPoint;
	MIMontageDocument	*doc = nil;
	
	if ([[MIPreferences sharedPrefs] timeDisplay] & kTimeDisplaySMPTE)
	{
		[time setHours:[fSHour intValue]];
		[time setMinutes:[fSMinute intValue]];
		[time setSeconds:[fSSecond intValue]];
		[time setVideoFrames:[fSFrame intValue]];
		[time setMiliseconds:[fSMilisecond intValue]];
	}
	else
	{
		[time setBars:[fSBar intValue] beats:[fSBeat intValue] nb16ths:[fS16th intValue] ppq:[fSPPQ intValue]];
	}
	
	switch( fFeedbackType )
	{
		case MITimePaletteLocatorsFeedback:
			[[[NSApp mainViewController] topLevelContainerView] setLeftLocatorPosition:[time getArmeTime]];
			break;
		case MITimePaletteStartLocatorFeedback:
			[[[NSApp mainViewController] topLevelContainerView] setLeftLocatorPosition:[time getArmeTime]];
			break;
		case MITimePaletteEndLocatorFeedback:
			[[[NSApp mainViewController] topLevelContainerView] setRightLocatorPosition:[time getArmeTime]];
			break;
		case MITimePalettePlayLocatorFeedback:
			[[[NSApp mainViewController] topLevelContainerView] setCurrentPlayBarPosition:[time getArmeTime]];
			break;
		case MITimePaletteSelectionFeedback:
			// move the selection
			if ( [(MIWSSelection*)fFeedbackObject type] == kMIWSSelectionTypeObjects )
			{
				objEnum = [fFeedbackObject objectEnumerator];
				
				// get start ArmeTime of the selection
				while ( anObj = [[objEnum nextObject] representedObject] )
				{
					if ( !MIEqualRects( selRect, MIZeroRect) )
						selRect = MIUnionRect(selRect, [anObj modelFrame]);
					else
						selRect = [anObj modelFrame];
					doc = [anObj montageDoc];
				}
				
				offset.time = _ArmeTime((SInt64)([time getDoubleTime] - GET_VALUE(MIMinX(selRect))));
				if ( GET_VALUE(offset.time) != 0 )
				{
					VUSERCMD1("Move (through Time Palette), offset = %d",(int)(offset.time.data));
					VUSERCMD_PRINT_SELECTION((MIWSSelection*)fFeedbackObject);

					cmd = [[[MIMoveCommand alloc] initWithSelection:fFeedbackObject] autorelease];
					[cmd setOffset:offset];
					[cmd setNewParent:nil]; // keep the same parent that each children already has.
					[doc doCmd:cmd];
				}
			}
			else if ( [(MIWSSelection*)fFeedbackObject type] == kMIWSSelectionTypeTimeSlice )
			{
				[fFeedbackObject setStartTime:[time getArmeTime]];
				[[[NSApp mainViewController] topLevelContainerView] setNeedsDisplayInRect:[[fFeedbackObject objectAtIndex:0] getTLCFrame]];
			}
			
			[self getValuesFromMainView];
			break;
		case MITimePaletteOtherLocatorFeedback:
			// move
			break;
		default:
			[[[NSApp mainViewController] topLevelContainerView] setLeftLocatorPosition:[time getArmeTime]];
			break;
	}
}

- (void)getValuesFromMainView
{
//	NSEnumerator		*objEnum;
	MIRect				selRect = MIZeroRect;
	MIObject			*anObj;
	MIPoint				offset = MIZeroPoint;
	MIMontageDocument	*doc;
	ArmeTime			t;

//	NSLog([NSString stringWithFormat:@"Refreshing with feedbackType:%d", fFeedbackType]);

	switch( fFeedbackType )
	{
/*		case MITimePaletteLocatorsFeedback:
			[self setStartTime:[[[NSApp mainViewController] topLevelContainerView] getLeftLocatorTime]];
			[self setEndTime:[[[NSApp mainViewController] topLevelContainerView] getRightLocatorTime]];
			[self setBigTime:[[[NSApp mainViewController] topLevelContainerView] getCurrentPlayBarTime]];
			break;
*/		case MITimePaletteStartLocatorFeedback:
			[self setStartTime:_ArmeTime(0)];
			[self setEndTime:_ArmeTime(0)];
			[self setBigTime:[[[NSApp mainViewController] topLevelContainerView] getLeftLocatorTime]];
			break;
		case MITimePaletteEndLocatorFeedback:
			[self setStartTime:_ArmeTime(0)];
			[self setEndTime:_ArmeTime(0)];
			[self setBigTime:[[[NSApp mainViewController] topLevelContainerView] getRightLocatorTime]];
			break;
		default:
		case MITimePaletteLocatorsFeedback:
		case MITimePalettePlayLocatorFeedback:
			[self setStartTime:_ArmeTime(0)];
			[self setEndTime:_ArmeTime(0)];
//			[self setBigTime:[[[NSApp mainViewController] topLevelContainerView] getCurrentPlayBarTime]];
			[self setBigTime:[self getRealPlaybarTime]];
			break;
		case MITimePaletteSelectionFeedback:
			if ( [(MIWSSelection*)fFeedbackObject type] == kMIWSSelectionTypeObjects )
			{
				//objEnum = [fFeedbackObject objectEnumerator];
				
				// get start ArmeTime of the selection
				for(UInt32 i = 0; i < [fFeedbackObject count]; i++)
				{
					anObj = [[fFeedbackObject objectAtIndex:i] representedObject];
					if ( !MIEqualRects( selRect, MIZeroRect) )
						selRect = MIUnionRect(selRect, [anObj absoluteModelFrame]);
					else
						selRect = [anObj absoluteModelFrame];
					doc = [anObj montageDoc];
				}
				[self setStartTime:MIMinX(selRect)];
				[self setEndTime:MIMaxX(selRect)];
				[self setBigTime:[[[NSApp mainViewController] topLevelContainerView] getCurrentPlayBarTime]];
			}
			else if ( [(MIWSSelection*)fFeedbackObject type] == kMIWSSelectionTypeTimeSlice )
			{
				[self setStartTime:[(MIWSSelection*)fFeedbackObject startTime]];
				[self setEndTime:[(MIWSSelection*)fFeedbackObject endxTime]];
				[self setBigTime:[[[NSApp mainViewController] topLevelContainerView] getCurrentPlayBarTime]];
			}
			break;
		case MITimePaletteOtherLocatorFeedback:
			// move
			break;
		case MITimePaletteTrackerFeedback:
			[self setStartTime:[fFeedbackObject trackerStartTime]];
			[self setEndTime:[fFeedbackObject trackerEndTime]];
			t = [fFeedbackObject trackerMainTime];
			if ( GET_VALUE(t) != 0 )
				[self setBigTime:t];			
			else
				[self setBigTime:[[[NSApp mainViewController] topLevelContainerView] getCurrentPlayBarTime]];
			break;
	}
}

- (IBAction)bigChanged:(id)sender
{
	// do nothing.
}


- (void)selectionDidChange:(NSNotification*)notification
{
}

- (void)playBarMoved:(NSNotification*)notification
{
}

- (void)startBarMoved:(NSNotification*)notification
{
}

- (void)endBarMoved:(NSNotification*)notification
{
}

- (void)refreshStartBoxes
{
	MITime	*time = [MITime timeWithDocument:[[NSDocumentController sharedDocumentController] currentDocument]];
	
	if ( !time )
		return;

	[time setArmeTime:fStartTime]; 

	if ([[MIPreferences sharedPrefs] timeDisplay] & kTimeDisplaySMPTE)
	{
		[fSHour setIntValue:[time getHours]];
		[fSMinute setIntValue:[time getMinutes]];
		[fSSecond setIntValue:[time getSeconds]];
		[fSFrame setIntValue:[time getVideoFrames]];
		[fSMilisecond setIntValue:[time getMiliseconds]];
	}
	else
	{
		[fSBar setIntValue:[time getBars]];
		[fSBeat setIntValue:[time getBeats]];
		[fS16th setIntValue:[time get16ths]];
		[fSPPQ setIntValue:[time getPPQs]];
	}
}

- (void)refreshEndBoxes
{
	MITime	*time = [MITime timeWithDocument:[[NSDocumentController sharedDocumentController] currentDocument]];
	
	if ( !time )
		return;

	[time setArmeTime:fEndTime]; 
	if ([[MIPreferences sharedPrefs] timeDisplay] & kTimeDisplaySMPTE)
	{
		[fEHour setIntValue:[time getHours]];
		[fEMinute setIntValue:[time getMinutes]];
		[fESecond setIntValue:[time getSeconds]];
		[fEFrame setIntValue:[time getVideoFrames]];
		[fEMilisecond setIntValue:[time getMiliseconds]];
	}
	else
	{
		[fEBar setIntValue:[time getBars]];
		[fEBeat setIntValue:[time getBeats]];
		[fE16th setIntValue:[time get16ths]];
		[fEPPQ setIntValue:[time getPPQs]];
	}
}

- (void)refreshDurationBoxes
{
	MITime	*time = [MITime timeWithDocument:[[NSDocumentController sharedDocumentController] currentDocument]];
	
	if ( !time )
		return;

	[time setArmeTime:fEndTime];
	[time subArmeTime:fStartTime];
	if ([[MIPreferences sharedPrefs] timeDisplay] & kTimeDisplaySMPTE)
	{
		[fDHour setIntValue:[time getHours]];
		[fDMinute setIntValue:[time getMinutes]];
		[fDSecond setIntValue:[time getSeconds]];
		[fDFrame setIntValue:[time getVideoFrames]];
		[fDMilisecond setIntValue:[time getMiliseconds]];
	}
	else
	{
		[fDBar setIntValue:[time getBars] - 1];
		[fDBeat setIntValue:[time getBeats] - 1];
		[fD16th setIntValue:[time get16ths] - 1];
		[fDPPQ setIntValue:[time getPPQs]];
	}
}

- (void)refreshBigBoxes
{
	MITime	*time = [MITime timeWithDocument:[[NSDocumentController sharedDocumentController] currentDocument]];
	
	if ( !time )
		return;
	
	[time setArmeTime:fBigTime];
	if ( [time getVideoFrameRate] != k30FPSDF )
		[fTemporaryTestBox setStringValue:[NSString stringWithFormat:@"%@:%.3d", [time getSMPTEString], [time getMiliseconds]]];
	else
		[fTemporaryTestBox setStringValue:[NSString stringWithFormat:@"%@;%.3d", [time getSMPTEString], [time getMiliseconds]]];
	
	if ([[MIPreferences sharedPrefs] timeDisplay] & kTimeDisplaySMPTE)
	{
		[fBigTextView setStringValue:[NSString stringWithFormat:@"%@:%.3d", [time getSMPTEString], [time getMiliseconds]]];
	}
	else
	{
		[fBigTextView2 setStringValue:[time getBPMString]];
	}
/*	[fBigHour setIntValue:[time getHours]];
	[fBigMinute setIntValue:[time getMinutes]];
	[fBigSecond setIntValue:[time getSeconds]];
	[fBigFrame setIntValue:[time getVideoFrames]];
	[fBigMilisecond setIntValue:[time getMiliseconds]];*/
}

- (void)checkSMPTEBPM
{
	if ([[MIPreferences sharedPrefs] timeDisplay] & kTimeDisplaySMPTE)
	{
		if ( fView != fSMPTEView )
		{
			[[fView superview] addSubview:fSMPTEView];
			[fView removeFromSuperview];
			fView = fSMPTEView;
		}
	}
	else
	{
		if ( fView != fBPMView )
		{
			[[fView superview] addSubview:fBPMView];
			[fView removeFromSuperview];
			fView = fBPMView;
		}
	}
}

- (void)startContinualUpdate
{
	if ( !fTimerRunning )
	{
/*
		fRedrawTimer = [NSTimer	scheduledTimerWithTimeInterval:0.5 // interval in seconds
							target:self
							selector:@selector(redrawMyself:)
							userInfo:nil					// if extra information is wanted, put an object here
							repeats:YES];					// does the timer repeat or is once only
*/
		//[NSApplication detachDrawingThread:@selector(continualUpdate:) toTarget:self withObject:nil];
    fUpdateTimer = [NSTimer	scheduledTimerWithTimeInterval:0.05 // interval in seconds
							target:self
							selector:@selector(continualUpdate:)
							userInfo:nil					// if extra information is wanted, put an object here
							repeats:YES];					// does the timer repeat or is once only
	[fUpdateTimer retain];
	}
}

- (void)continualUpdate:(id)argument
{
	fTimerRunning  = YES;
  timeval tps, tpe;
  gettimeofday(&tps, nil);
  [self getValuesFromMainView];
	gettimeofday(&tpe, nil);
	tpe.tv_sec = tpe.tv_sec - tps.tv_sec;
	if ( tpe.tv_sec == 0 ) 
  {
    tpe.tv_usec = tpe.tv_usec - tps.tv_usec;
	  if ( tpe.tv_usec < 41666 ) 
    {
      usleep( 41666 - tpe.tv_usec );
    }
  }
}

- (ArmeTime)getRealPlaybarTime
{
	TArmeDoc	*aTArmeDoc = [[[NSApp mainViewController] document] armeDocument];
	ArmeTime	t;
	
	if ( !aTArmeDoc )
		return _ArmeTime(0);
		
    aTArmeDoc->LockRead();
    t = aTArmeDoc->GetPlayPos();
    aTArmeDoc->UnlockRead();
	
	return t;
}

- (void)stopContinualUpdate
{
  [fUpdateTimer release];
  fUpdateTimer = nil;
}

- (void)windowWillClose:(NSNotification*)notification
{
	[self stopContinualUpdate];
}

- (BOOL)ignoresUndoRedoActions {
	return YES;
}

@end
