/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
//#define MTGEnableTimeLogs 1

#ifdef MTGEnableTimeLogs
#include <sys/time.h>
#endif

#import "MontageInterface.h"
#import "MIObjectRep.h"

#define USE_NEW_DRAW_GRID

#ifndef ArmeBasicTypes_h
#include "ArmeBasicTypes.h"
#endif //ArmeBasicTypes_h

@class MIMouseTracker;
@class MIMainViewController;

/*
#ifdef MTGEnableTimeLogs
#define MTGStartTimeLog {timeval tps,tpe;gettimeofday(&tps, nil);
#define //MTGEndTimeLog(C,M) gettimeofday(&tpe, nil); NSLog(@"%@ - %@ : %d seconds %d microseconds",C,M,tpe.tv_sec-tps.tv_sec, abs(tpe.tv_usec-tps.tv_usec));}
#else
#define MTGStartTimeLog
#define //MTGEndTimeLog(C,M) 
#endif
*/

/*!
    @class MITopLevelContainerRep
    @abstract
    this class is the top level container in which all the other objects are drawn,
    where all user events occur, ...
    @discussion
    the MITopLevelContainerRep is a NSView subclass, the only object's representation that
    has this 'particularity'
    It is enclosed in an NSClipView and an NSScrollView
    All of the drawing is done in this view, although each representation is responsible for drawing
    itself when asked.
    This class is also where the mouse trackers take their origin and all coordinate systems are based
    on this one's.
*/

#define DISPLAY_QUEUES

@interface MITopLevelContainerRep : NSView
{
@public
    /*! @var fMouseTracker a mouse tracker, used to control the mouse within the view */
	MIMouseTracker				*fMouseTracker;
    
    /*! @var fBackgroundColor unused atm, could be here to specify the background color */
	NSColor						*fBackgroundColor;
    
	/*! @var fRepresentedObject pointer to the TLC MIObject */
    MIObject					*fRepresentedObject;
    
    /*! @var fMainViewController the main view controller associated with this TLC (and window) */
    MIMainViewController		*fMainViewController;
    
    /*! @var fCurrentPlayBarPosition the current play bar position */
    ArmeTime					fCurrentPlayBarPosition;

	/*! @var fLeftLocatorPosition the current left locator position */
	ArmeTime					fLeftLocatorPosition;
	
	/*! @var fRightLocatorPosition the current right locator position */
	ArmeTime					fRightLocatorPosition;
	
	BOOL						fDrawRedrawRects;

	NSBitmapImageRep			*fPlaybarCacheImageRep;
	int							fPlaybarCachePosition;
	NSRect						fCachedVisibleRect;
	BOOL						fShouldDrawPlaybar;
	
	unsigned int				fLastDraggingAnswer;
	BOOL						fDropEnabled;
}

/*! @function setBackgroundColor  sets the background color (unused atm) */
- (void)setBackgroundColor:(NSColor*)color;

/*! @function getTopLevelContainerRep  Returns self. Useful for recursive calls */
- (MITopLevelContainerRep*)getTopLevelContainerRep;

/*! @function getBackgroundCOlor returns the background color (unused atm) */
- (NSColor*)getBackgroundColor;

/*! @function setRepresentedObject sets the TLC MIObject */
- (void)setRepresentedObject:(MIObject*)object;

/*! @function representedObject returns the TLC MIObject */
- (MIObject*)representedObject;

/*! @function setMainViewController sets the main view controller associated with this TLC */
- (void)setMainViewController:(MIMainViewController*)theMainViewController;

/*! @function mainViewController gets the main view controller associated with this TLC */
- (MIMainViewController*)mainViewController;

/*! @function getCurrentPlayBarTime get the current play bar position in ArmeTime */
- (ArmeTime)getCurrentPlayBarTime;

/*! @function getLeftLocatorTime get the left locator position in ArmeTime */
- (ArmeTime)getLeftLocatorTime;

/*! @function getRightLocatorTime get the right locator position in ArmeTime */
- (ArmeTime)getRightLocatorTime;

/*! @function getCurrentPlayBarPosition get the current play bar position in pixels*/
- (float)getCurrentPlayBarPosition;

/*! @function setCurrentPlayBarPosition sets the current play bar position 
    if the playbar position passed is negative, the play bar won't be redrawn next time*/
- (void)setCurrentPlayBarPosition:(ArmeTime)newPlayBarPosition;

/*! @function getLeftLocatorPosition returns the left locator position in pixels*/
- (float)getLeftLocatorPosition;

/*! @function getRightLocatorPosition returns the right locator position in pixels*/
- (float)getRightLocatorPosition;

/*! @function setLeftLocatorPosition sets the left locator position */
- (void)setLeftLocatorPosition:(ArmeTime)position;

/*! @function setRightLocatorPosition sets the right locator position */
- (void)setRightLocatorPosition:(ArmeTime)position;

#ifdef USE_NEW_DRAW_GRID
- (void)drawGrid:(NSRect)aRect;
#endif //#ifdef USE_NEW_DRAW_GRID

/*! @function pixelToArmeTime convert pixels to ArmeTime taking the zoom into account */
- (ArmeTime)pixelToArmeTime:(float)pixels;

/*! @function armeTimeToPixel convert ArmeTime to pixels taking the zoom into account */ 
- (float)armeTimeToPixel:(ArmeTime)time;

- (double)doubleTimeToPixel:(double)time;
- (double)pixelToDoubleTime:(float)pixels;


/*! @function pixelToMIVPos converts pixels to a vertical position the model uses taking zoom into account */
- (MIVPos)pixelToMIVPos:(float)pixels;

/*! @function MIVPosToPixel converts MIVpos that the model uses to pixels taking zoom into account */
- (float)MIVPosToPixel:(MIVPos)vpos;

- (NSRect)MIRectToNSRect:(MIRect)rect;
- (MIRect)NSRectToMIRect:(NSRect)rect;
- (MIPoint)NSPointToMIPoint:(NSPoint)point;


- (void)CenterOnPoint:(MIPoint)center;
- (MIPoint)currentCenter;

/*! @function hitTestForMIObject retrieve the object representation that lies at a given point in the TLC */
- (MIObjectRep*)hitTestForMIObject:(NSPoint)point;

/*! @function createChildrenRepresentationsRecursivelyInSelf Creates representations in this tlc rep for t
              the whole object netwerk */
- (void)createChildrenRepresentationsRecursivelyInSelf;

- (void)focusCacheAndDrawPlaybar:(NSRect)rect;
- (void)cacheAndDrawPlaybar:(NSRect)rect;
- (void)drawPlaybar:(NSRect)rect;
- (void)redrawBehindPlaybar;
- (void)flushDeferedRedraws;

- (void)drawLocators;

- (void)drawMarkers;

/*! @function getTLCFrame
    this function returns the TLC's frame, it is used because an MIObject does not know if a
    representation is the TLC or not */
- (NSRect)getTLCFrame;

/*! @function recreateFrame
	this function recreates the TLC's frame from the model data */
- (void)recreateFrame;

//$$$TODO remove this call both here and in the mm and in drawRect:
-(void)debugDrawSnapperDBLines:(NSRect)aRect;

/*! @function moveVisibleRectToPlayBar
    this function moves the visible rectangle in order to see the playbar as it moves */
- (void)moveVisibleRectToPlayBar;

- (BOOL)prepareForDragOperation:(id <NSDraggingInfo>)sender;
- (BOOL)performDragOperation:(id <NSDraggingInfo>)sender;
- (void)concludeDragOperation:(id <NSDraggingInfo>)sender;

- (MIMouseTracker*)mouseTracker;

- (void)refreshTrackingRect;

// Drag and Drop
- (BOOL)canFileBeImported:(NSString*)name;
- (NSArray*)discardUnsupportedFiles:(NSArray*)files;

- (MIPoint)newObjectLocation;
- (void)scrollToNewObject;

@end
