/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MITrimDlgCtrller.h"
#import "MontageInterface.h"
#import "MITime.h"
#import "MIMontageDocument.h"

@implementation MITrimDlgCtrller

/*
	MITime : 
	
	- (int)getHours;
	- (int)getMinutes;
	- (int)getSeconds;
	- (int)getMiliseconds;
	- (int)getAudioFrames;
	- (int)getVideoFrames;
*/

- (void)setStartTrim:(ArmeTime)startTrim
{
	MITime* time = [[[MITime alloc] initWithArmeTime:startTrim inDocument:fMontageDocument] autorelease];
	[startTrimHHTextField setIntValue:[time getHours]];
	[startTrimMMTextField setIntValue:[time getMinutes]];
	[startTrimSSTextField setIntValue:[time getSeconds]];
	[startTrimFFTextField setIntValue:[time getVideoFrames]];
	[startTrimMSTextField setIntValue:[time getMiliseconds]];
}

- (void)setEndTrim:(ArmeTime)endTrim
{
	MITime* time = [[[MITime alloc] initWithArmeTime:endTrim inDocument:fMontageDocument] autorelease];
	[endTrimHHTextField setIntValue:[time getHours]];
	[endTrimMMTextField setIntValue:[time getMinutes]];
	[endTrimSSTextField setIntValue:[time getSeconds]];
	[endTrimFFTextField setIntValue:[time getVideoFrames]];
	[endTrimMSTextField setIntValue:[time getMiliseconds]];
}

- (void)setTotalObjectDuration:(ArmeTime)duration
{
	fTotalObjectDuration = duration;
}

- (ArmeTime)startTrim
{
	return fFinalStartTrim;
}

- (ArmeTime)endTrim
{
	return fFinalEndTrim;
}

- (IBAction)OKPressed:(id)sender
{
	MITime* time = [[MITime alloc] initWithDocument:fMontageDocument];
//- (void)setH:(int)hours m:(int)minutes s:(int)seconds vFr:(int)videoFrames ms:(int)miliseconds;
	int HH = [startTrimHHTextField intValue];
	int MM = [startTrimMMTextField intValue];
	int SS = [startTrimSSTextField intValue];
	int FF = [startTrimFFTextField intValue];
	int MS = [startTrimMSTextField intValue];
	[time setH:HH m:MM s:SS vFr:FF ms:MS];
	fFinalStartTrim = [time getArmeTime];

	HH = [endTrimHHTextField intValue];
	MM = [endTrimMMTextField intValue];
	SS = [endTrimSSTextField intValue];
	FF = [endTrimFFTextField intValue];
	MS = [endTrimMSTextField intValue];
	[time setH:HH m:MM s:SS vFr:FF ms:MS];
	fFinalEndTrim = [time getArmeTime];
		
	[NSApp stopModal];
}

- (IBAction)CancelPressed:(id)sender
{
	fFinalStartTrim = _ArmeTime(-1);
	fFinalEndTrim = _ArmeTime(-1);
	[NSApp stopModal];
}

- (void)setMontageDocument:(MIMontageDocument*)montageDocument
{
	fMontageDocument = montageDocument;
}

@end
