/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
#import "MIVUMeterView.h"

#define USE_NSIMAGES 1

void CalculateDisplayRectangles( float level, BOOL isVertical, NSRect bounds, NSRect *colorRect, NSRect *blackRect );

@implementation MIVUMeterView

#ifdef USE_NSIMAGES
- (void)makeCacheImages {
	NSRect bounds = [self bounds];
	NSRect square;
	float squareHeight;
	float squareWidth;
	int x;

	[fColorImage release];
	[fBlackImage release];


	fBlackImage = [[NSImage alloc] initWithSize:bounds.size];
	[fBlackImage lockFocus];
	[[NSColor blackColor] set];
	[NSBezierPath fillRect:bounds];
	[[NSColor lightGrayColor] set];

	[NSBezierPath strokeLineFromPoint:NSMakePoint( NSMinX( bounds ), NSMaxY( bounds ) ) toPoint:NSMakePoint( NSMaxX( bounds ), NSMaxY( bounds ) )];
	[NSBezierPath strokeLineFromPoint:NSMakePoint( NSMinX( bounds ), NSMinY( bounds ) ) toPoint:NSMakePoint( NSMinX( bounds ), NSMaxY( bounds ) )];
	[fBlackImage unlockFocus];

	fColorImage = [[NSImage alloc] initWithSize:bounds.size];
	[fColorImage lockFocus];
	[[NSColor blackColor] set];
	[NSBezierPath fillRect:bounds];
	[[NSColor lightGrayColor] set];
	[NSBezierPath strokeLineFromPoint:NSMakePoint( NSMinX( bounds ), NSMaxY( bounds ) ) toPoint:NSMakePoint( NSMaxX( bounds ), NSMaxY( bounds ) )];
	[NSBezierPath strokeLineFromPoint:NSMakePoint( NSMinX( bounds ), NSMinY( bounds ) ) toPoint:NSMakePoint( NSMinX( bounds ), NSMaxY( bounds ) )];

	if ( fIsVertical ) {
		squareHeight = ( NSHeight( bounds ) - 3 ) / 15;
		squareWidth = 0;

		square = NSMakeRect( NSMinX( bounds ) + 2, NSMinY( bounds ) + 2, NSWidth( bounds ) - 4, squareHeight - 2 );
	} else {
		squareHeight = 0;
		squareWidth = NSWidth( bounds ) / 15;

		square = NSMakeRect( NSMinX( bounds ) + 2, NSMinY( bounds ) + 2, squareWidth - 2, NSHeight( bounds ) - 4 );
	}

	[[NSColor colorWithCalibratedRed:0/255.0 green:167/255.0 blue:24/255.0 alpha:1.0] set];
	for ( x = 0; x <  10 ; x++ ) {
		[NSBezierPath fillRect:square];
		square = NSOffsetRect( square, squareWidth, squareHeight );
	}

	[[NSColor colorWithCalibratedRed:254/255.0 green:179/255.0 blue:0/255.0 alpha:1.0] set];
	for ( ; x <  12 ; x++ ) {
		[NSBezierPath fillRect:square];
		square = NSOffsetRect( square, squareWidth, squareHeight );
	}

	[[NSColor colorWithCalibratedRed:255/255.0 green:0/255.0 blue:0/255.0 alpha:1.0] set];
	for ( ; x <  15 ; x++ ) {
		[NSBezierPath fillRect:square];
		square = NSOffsetRect( square, squareWidth, squareHeight );
	}

	[fColorImage unlockFocus];
}
#endif


- (id)initWithFrame:(NSRect)frameRect {
	if (( self = [super initWithFrame:frameRect] )) {
		if ( NSWidth( frameRect ) < NSHeight( frameRect ) ) {
			fIsVertical = YES;
		} else {
			fIsVertical = NO;
		}
#ifdef USE_NSIMAGES		
		[self makeCacheImages];
#endif		
		fValue = 0.0;
		fIsEnabled = YES;
		[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(viewFrameDidChange:) name:NSViewFrameDidChangeNotification object:self];
	}
	return self;
}

- (void)dealloc {
	//NSLog(@"MIVUMeterView - dealloc");
	[[NSNotificationCenter defaultCenter] removeObserver:self name:NSViewFrameDidChangeNotification object:self];
	[fColorImage release];
	[fBlackImage release];
	[super dealloc];
}

- (void)setLevel:(float)level {
	if ( level != fValue ) {
		fValue = level;
		[self setNeedsDisplay:YES];
	}
}

- (void)setLevelAndDisplay:(float)level {
	if ( level != fValue ) {
		fValue = level;

		if ( [self lockFocusIfCanDraw] ) {
			[self drawRect:[self bounds]];
			[[NSGraphicsContext currentContext] flushGraphics];
			[self unlockFocus];
		}
	}
}

- (void)drawRect:(NSRect)rect {
#ifdef USE_NSIMAGES
	NSRect newColorRect;
	NSRect newBlackRect;
	NSRect bounds = NSIntegralRect([self bounds]);

	if ( fIsEnabled ) {
		CalculateDisplayRectangles( fValue, fIsVertical, bounds, &newColorRect, &newBlackRect );

		[fBlackImage drawInRect:newBlackRect fromRect:newBlackRect operation:NSCompositeCopy fraction:1];
		[fColorImage drawInRect:newColorRect fromRect:newColorRect operation:NSCompositeCopy fraction:1];
	} else {
		CalculateDisplayRectangles( 0, fIsVertical, bounds, &newColorRect, &newBlackRect );

		[[NSColor colorWithCalibratedRed:100/256.0 green:102/256.0 blue:109/256.0 alpha:1
			] set];
		[NSBezierPath fillRect:newBlackRect];
	}	
#else	
	NSRect newColorRect;
	NSRect newBlackRect;
	NSRect bounds = NSIntegralRect([self bounds]);
	NSRect square;
	float squareHeight;
	float squareWidth;
	int x;
	int value = (int)(fValue * 15);

	if ( fIsEnabled ) {
		// adjust bounds
		
		[[NSColor blackColor] set];
		[NSBezierPath fillRect:bounds];
		[[NSColor lightGrayColor] set];

		// make right and top side lighter
		[NSBezierPath strokeLineFromPoint:NSMakePoint( NSMinX( bounds ) + .5, NSMaxY( bounds ) + .5 ) toPoint:NSMakePoint( NSMaxX( bounds ) + .5, NSMaxY( bounds ) + .5 )];

		[NSBezierPath strokeLineFromPoint:NSMakePoint( NSMinX( bounds ) + .5, NSMinY( bounds ) + .5 ) toPoint:NSMakePoint( NSMinX( bounds ) + .5, NSMaxY( bounds ) + .5 )];

		// calculate square size
		if ( fIsVertical ) {
			squareHeight = ( NSHeight( bounds ) - 3 ) / 15;
			squareWidth = 0;

			square = NSMakeRect( NSMinX( bounds ) + 2, NSMinY( bounds ) + 2, NSWidth( bounds ) - 4, squareHeight - 2 );
		} else {
			squareHeight = 0;
			squareWidth = ( NSWidth( bounds ) - 3 ) / 15;

			square = NSMakeRect( NSMinX( bounds ) + 2, NSMinY( bounds ) + 2, squareWidth - 2, NSHeight( bounds ) - 4 );
		}

		[[NSColor colorWithCalibratedRed:0/255.0 green:167/255.0 blue:24/255.0 alpha:1.0] set];
		for ( x = 0; x <  10 && x < value ; x++ ) {
			[NSBezierPath fillRect:square];
			square = NSOffsetRect( square, squareWidth, squareHeight );
		}

		[[NSColor colorWithCalibratedRed:254/255.0 green:179/255.0 blue:0/255.0 alpha:1.0] set];
		for ( ; x <  12 && x < value ; x++ ) {
			[NSBezierPath fillRect:square];
			square = NSOffsetRect( square, squareWidth, squareHeight );
		}

		[[NSColor colorWithCalibratedRed:255/255.0 green:0/255.0 blue:0/255.0 alpha:1.0] set];
		for ( ; x <  15 && x < value ; x++ ) {
			[NSBezierPath fillRect:square];
			square = NSOffsetRect( square, squareWidth, squareHeight );
		}
	} else {
		CalculateDisplayRectangles( 0, fIsVertical, bounds, &newColorRect, &newBlackRect );

		[[NSColor colorWithCalibratedRed:100/256.0 green:102/256.0 blue:109/256.0 alpha:1
			] set];
		[NSBezierPath fillRect:newBlackRect];
	}
#endif	
}

- (void)viewFrameDidChange:(NSNotification*)notification {
	if ( NSWidth( [self frame] ) < NSHeight( [self frame] ) ) {
		fIsVertical = YES;
	} else {
		fIsVertical = NO;
	}
#ifdef USE_NSIMAGES	
	[self makeCacheImages];
#endif
	[self setNeedsDisplayInRect:[self visibleRect]];
}

- (void)setEnabled:(BOOL)enabled {
	fIsEnabled = enabled;
	[self setNeedsDisplay:YES];
}

- (BOOL)isEnabled {
	return fIsEnabled;
}

@end

void CalculateDisplayRectangles( float level, BOOL isVertical, NSRect bounds, NSRect *colorRect, NSRect *blackRect ) {
	if ( isVertical ) {
		float squareHeight = ( NSHeight( bounds ) - 3 ) / 15;
		float limit = NSHeight( bounds ) * level;
		int squareNumber = (int)( limit / squareHeight ) ;
		limit = (int) (squareNumber * squareHeight) ;

		NSDivideRect( bounds, colorRect, blackRect, limit, NSMinYEdge );
	}
	else {
		float squareWidth = ( NSWidth( bounds ) - 3 ) / 15;
		float limit = NSWidth( bounds ) * level;
		int squareNumber = (int)( limit / squareWidth ) ;
		limit = (int) (squareNumber * squareWidth) ;

		NSDivideRect( bounds, colorRect, blackRect, limit, NSMinXEdge );
	}
}
