/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
#ifndef NO_VERBOSE_AT_ALL

#import "MIVerboseConfigDlgController.h"

#ifndef ZzVerbose_h
//#include "ZzVerbose.h"
#endif //ZzVerbose_h

@implementation MIVerboseConfigDlgController

- (IBAction)outputToLogFileCheckBox:(id)sender
{
		if([sender state] == NSOffState)
			fActiveOutputs &= ~kVERFile;
		else
			fActiveOutputs |= kVERFile;
}

- (IBAction)outputToCoutCheckBox:(id)sender
{
		if([sender state] == NSOffState)
			fActiveOutputs &= ~kVERScreen;
		else
			fActiveOutputs |= kVERScreen;
}

- (IBAction)verboseLevelSlider:(id)sender
{
	int sel = [fVerboseTopicDropDown indexOfSelectedItem];
	ZzAssert(sel != -1);
	int val = [sender intValue];
	fTopicLevels[sel] = val;
}

- (IBAction)verboseTopicDropDown:(id)sender
{
	int sel = [fVerboseTopicDropDown indexOfSelectedItem];
	ZzAssert(sel != -1);
	[fVerboseLevelSlider setIntValue:fTopicLevels[sel]];
}

- (IBAction)maxAllTopics:(id)sender
{
	int sel = [fVerboseTopicDropDown indexOfSelectedItem];
	for(SInt32 i = 0; i < kVERTopic_LastTopic - 1; i++)
	{
		fTopicLevels[i] = 6;
	}
	[fVerboseLevelSlider setIntValue:fTopicLevels[sel]];
}

- (IBAction)soloCurrentTopic:(id)sender
{
	int sel = [fVerboseTopicDropDown indexOfSelectedItem];
	for(SInt32 i = 0; i < kVERTopic_LastTopic - 1; i++)
	{
		fTopicLevels[i] = 0;
	}
	fTopicLevels[sel] = 6;
	[fVerboseLevelSlider setIntValue:fTopicLevels[sel]];
}

- (IBAction)allOne:(id)sender
{
	int sel = [fVerboseTopicDropDown indexOfSelectedItem];
	for(SInt32 i = 0; i < kVERTopic_LastTopic - 1; i++)
	{
		fTopicLevels[i] = 1;
	}
	[fVerboseLevelSlider setIntValue:fTopicLevels[sel]];
}

- (IBAction)allZero:(id)sender
{
	int sel = [fVerboseTopicDropDown indexOfSelectedItem];
	for(SInt32 i = 0; i < kVERTopic_LastTopic - 1; i++)
	{
		fTopicLevels[i] = 0;
	}
	[fVerboseLevelSlider setIntValue:fTopicLevels[sel]];
}

- (void)awakeFromNib
{
	// copy the current settings here :
	fActiveOutputs = ZzVerbose::fsActiveOutputs;
	for(SInt32 i = 0; i < kVERTopic_LastTopic - 1; i++)
	{
		fTopicLevels[i] = ZzVerbose::fsTopicLevels[i+1];
	}
	
	for(SInt32 i = 0; i < kVERTopic_LastTopic - 1; i++)
	{
		const char* topicName = ZzVerbose::fsTopicStrings[i+1];
		[fVerboseTopicDropDown addItemWithObjectValue:[NSString stringWithCString:topicName]];
	}
	[fVerboseTopicDropDown selectItemAtIndex:0];
	[fVerboseLevelSlider setMinValue:0];
	[fVerboseLevelSlider setMaxValue:6];
	[fVerboseLevelSlider setIntValue:fTopicLevels[0]];
	
	if(fActiveOutputs & kVERFile)
		[fOutputToLogFileCheckBox setState:NSOnState];
	else
		[fOutputToLogFileCheckBox setState:NSOffState];

	if(fActiveOutputs & kVERScreen)
		[fOutputToCoutCheckBox setState:NSOnState];
	else
		[fOutputToCoutCheckBox setState:NSOffState];

	[fVerboseLevelSlider setNumberOfTickMarks:7];
}

- (IBAction)OKClicked:(id)sender
{
	// copy the current settings here :
	ZzVerbose::fsActiveOutputs = fActiveOutputs;
	for(SInt32 i = 0; i < kVERTopic_LastTopic - 1; i++)
	{
		ZzVerbose::fsTopicLevels[i+1] = fTopicLevels[i];
	}
	[[NSApplication sharedApplication] stopModal];
}

- (IBAction)CancelClicked:(id)sender
{
	[[NSApplication sharedApplication] stopModal];
}


@end

#endif //NO_VERBOSE_AT_ALL
