/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
#import <Cocoa/Cocoa.h>

#import "MIVideoClipRep.h"
#import "MIImageController.h"
#import "MIObject.h"
#import "MIApplicationController.h"
#import "MITimeStretchTracker.h"
#import "MITrimTracker.h"
#import "MIPreferences.h"
#import "MISliceSelectionTracker.h"
#import "MIActiveCursor.h"
#import "MIMontageDocument.h"
#import "MIMovie.h"

#ifndef TVideoClip_h
#include "TVideoClip.h"
#endif //TVideoClip_h

#ifndef MovieHelper_h
#include "MovieHelper.h"
#endif //MovieHelper_h

#import <unistd.h>
#import "MITopLevelContainerRep.h"
#import "MIMainViewController.h"
#import "MIWSSelection.h"

#undef VER_DEF_TOPIC
#define VER_DEF_TOPIC videoDrawing

#define MAX_NB_FRAME_PER_SEC_AT_MAXIMUM_ZOOM	8

extern bool gbVideoClipRepIncompletelyDrawnIsReleased;

@interface MIMouseTracker (MTGLiveTrackingUnformalProtocol)

- (void)drawCachedImage:(NSImage*)fCachedImage inFrame:(NSRect)frameInDrawCoords clippedTo:(NSRect)clippingRect;

@end


@implementation MIVideoClipRep

-(id)retain
{
	return [super retain];
}		

-(id)autorelease
{
	return [super autorelease];
}

- (void)release
{
	[super release];
}
- (id)initWithFrame:(NSRect)frame inTLC:(MITopLevelContainerRep*)theTLC
{
    self = [super initWithFrame:frame inTLC:theTLC];

	fHasGeneratedCurrentViewCompletely = NO;

	fThumbnailsArray = nil;

	fVideoObject = nil;
//	fCachedImage = nil;
//	fStopImageUpdates = NO;

	fCurrentTimeForInAdvanceCaching = ARME_TIME_ZERO;
	fCurrentStepForInAdvanceCaching = 0;
		
    return self;
}

- (void)setImagesAndColor
{
	if ( [self isSelected] && [[[fTLCRep mainViewController] selection] type] == kMIWSSelectionTypeObjects )
	{
		[self setBackgroundColor:mainImageController->fVideoClipNegativeColor];
		fLeftTitleImage = mainImageController->fLeftVideoClipNegativeTitleBarImage;
		fCenterTitleImage = mainImageController->fCenterVideoClipNegativeTitleBarImage;
		fRightTitleImage = mainImageController->fRightVideoClipNegativeTitleBarImage;
	}
	else if ( [fRepresentedObject isMuted] )
	{
		[self setBackgroundColor:mainImageController->fVideoClipDimColor];
		fLeftTitleImage = mainImageController->fLeftVideoClipDimTitleBarImage;
		fCenterTitleImage = mainImageController->fCenterVideoClipDimTitleBarImage;
		fRightTitleImage = mainImageController->fRightVideoClipDimTitleBarImage;
	}
	else 
	{
		[self setBackgroundColor:mainImageController->fVideoClipBrightColor];
		fLeftTitleImage = mainImageController->fLeftVideoClipBrightTitleBarImage;
		fCenterTitleImage = mainImageController->fCenterVideoClipBrightTitleBarImage;
		fRightTitleImage = mainImageController->fRightVideoClipBrightTitleBarImage;
	}
}
/*
- (void)setSelected:(BOOL)selected
{
	if ( selected == fSelected )
		return;
		
	fSelected = selected;
	[self setImagesAndColor];
	[self refreshCachedTitleImage:NO];
	[self refreshCachedAutomationImage:NO];
	[self refreshCachedTitleStringImage];

//	[self invalidateCachedImage];
//	[self refreshCachedBodyImage:NO];

	if ( fCachedBodyImage )
	{
		NSImage *tmpImg = [[NSImage alloc] initWithSize:[fCachedBodyImage size]];
		[tmpImg lockFocus];
			[[NSColor grayColor] set];
			[NSBezierPath fillRect:NSMakeRect(0,0,[fCachedBodyImage size].width,[fCachedBodyImage size].height)];
			[fCachedBodyImage compositeToPoint:NSZeroPoint operation:NSCompositeXOR];
		[tmpImg unlockFocus];
		[fCachedBodyImage release];
		fCachedBodyImage = tmpImg;
	}
}
*/

-(void)dealloc
{
	vernl(2, "(" << self << ") purge message queue for MIVideoClipRep");
//	[[MIApplicationController sharedApplicationController] purgeQueueMessagesFor:self];
	[self deleteThumbnailsArray];
	
	// remove the clip rep from the list of clip rep being drawn
    if (!gbVideoClipRepIncompletelyDrawnIsReleased)
	{
		MIMontageDocument * montageDoc = [[self representedObject] montageDoc];
        [montageDoc->fVideoClipRepIncompletelyDrawn removeObject:self];
	}
    
    [super dealloc];
}

-(void)deleteThumbnailsArray
{
	vernl(6, "(" << self << ") delete thumbnails array");

	if (fThumbnailsArray)
	{
		for (int i = 0; i < fNbThumbnailsInClip; ++i)
		{
			//$$ HACK it happens that the release of some thumbnails fails, I don't know why but let's ignore the exception and try to continue
			try
			{
				[fThumbnailsArray[i] release];
			}
			catch (NSException* e)
			{
				vernl(6, [e reason]);
			}
            fThumbnailsArray[i] = nil;
		}
		free(fThumbnailsArray);
		fThumbnailsArray = nil;
	}
}



- (void)invalidate
{
	vernl(6, "(" << self << ") Invalidate");

	fHasGeneratedCurrentViewCompletely = NO;
	[super invalidate];
	[self deleteThumbnailsArray];
	[self invalidateCachedImage];
}

- (void)recreateFrame
{
	fHasGeneratedCurrentViewCompletely = NO;
	[self invalidateCachedImage];
	[super recreateFrame];
}

- (void)setFrameSize:(NSSize)frameSize
{
    if ( NSEqualSizes(frameSize, fFrame.size) )
		return;
	
	[super setFrameSize:frameSize];
}

- (void)refreshCachedBodyImage:(BOOL)createIfAbsent
{
	if (fHasGeneratedCurrentViewCompletely)
		[super refreshCachedBodyImage:createIfAbsent];
}

- (void)invalidateCachedImage
{
	[fCachedBodyImage release];
	fCachedBodyImage = nil;
	//il y a un blme ici
	[super refreshCachedBodyImage:NO];
//	[fCachedImage release];
//	fCachedImage = nil;
}

/*- (void)updateCachedImageFrom:(SInt64)startTime to:(SInt64)endxTime wPixels:(float)imWidth hPixels:(float)imHeight
{
	// startTime and endxTime are expressed in clip coordinates
	// which always start at 0 and end at GetDuration()
	
	TVideoClip* clip = dynamic_cast<TVideoClip*>([[self representedObject] WSObject]);
	ZzAssert(clip);

    if ( fStopImageUpdates ) return ; // No updates during time stretch

}*/

//#define VERBOSE_DrawBaseBodyRect


- (void)drawBaseBodyRect:(NSRect)frameInDrawCoords clippedTo:(NSRect)clippingRect
{
	float		titleHeight;
	float		titleBarHeight;
	float		footerHeight;

	MIMouseTracker *mouseTracker = [fTLCRep mouseTracker];
	if ( ( mouseTracker ) && fCachedBodyImage && ( [mouseTracker respondsToSelector:@selector(drawCachedImage:inFrame:clippedTo:)] ) )
	{
		[mouseTracker drawCachedImage:fCachedBodyImage inFrame:frameInDrawCoords clippedTo:clippingRect];
		return;
	}


#ifdef SCALABLE_TITLEBAR
	titleHeight = [[self getTopLevelContainerRep] MIVPosToPixel:kRepTitleHeight];
	titleBarHeight = [[self getTopLevelContainerRep] MIVPosToPixel:kRepTitleBarHeight];
#else
	titleHeight = kRepTitleHeight;
	titleBarHeight = kRepTitleBarHeight;
#endif
	footerHeight = [[self getTopLevelContainerRep] MIVPosToPixel:kRepFooterHeight];

	[super drawBaseBodyRect:frameInDrawCoords clippedTo:clippingRect];

	if(clippingRect.size.width <= 0.1 || clippingRect.size.height <= 0.1 )
		return;

	// wsObj is the Arme object of the MIObject we represent
	fVideoObject = (TVideoClip*)[[self representedObject] WSObject];
	ArmeTime duration = fVideoObject->GetDuration();

	if (duration <= ARME_TIME_ZERO)
	{
//		TimeValue offsetFirst = GetTrackOffset(fVideoObject->GetTrack(0)->fQTTrack);
//		TimeValue offsetLast = GetTrackOffset(fVideoObject->GetLastVideoTrack()->fQTTrack);
//		TimeValue durationFirst = GetTrackDuration(fVideoObject->GetTrack(0)->fQTTrack);
//		TimeValue durationLast = GetTrackDuration(fVideoObject->GetLastVideoTrack()->fQTTrack);
	}


	// first time we come here ? need to initialize a few stuffs
	if (fThumbnailsArray == nil)
	{
		fNbThumbnailsInClip = int(duration.data/[[self representedObject] WSObject]->GetDoc()->GetVideoFrameDuration());
		fThumbnailsArray = (NSImage**)malloc(sizeof(NSImage*)*fNbThumbnailsInClip);
		vernl(6, "(" << self << ") draw BaseBodyRect : creating thumbsArray for " << fNbThumbnailsInClip << " clips");
		
		for (int i = 0; i < fNbThumbnailsInClip; ++i)
			fThumbnailsArray[i] = nil;
	}
	
	// (Stephan) leave one pixel above and below the images
	NSSize sizeOfThumbnail = NSMakeSize((frameInDrawCoords.size.height - 3) *1.333f, frameInDrawCoords.size.height - 3);
	const int minSpaceBetweenTwoThumbnails = 2;
	
	float spaceBetweenTwoThumbnails;
	int nbThumbnailsInClip = int(frameInDrawCoords.size.width)/int(sizeOfThumbnail.width + minSpaceBetweenTwoThumbnails);
	if (nbThumbnailsInClip <= 1)
	{
		nbThumbnailsInClip = 1;
		spaceBetweenTwoThumbnails = 0;
	}
	else
		spaceBetweenTwoThumbnails = (frameInDrawCoords.size.width - nbThumbnailsInClip*sizeOfThumbnail.width)/(nbThumbnailsInClip - 1);

	// clippingRect is the rect where we need to draw in draw coords
	// aPoint is the origin of our whole frame in draw coords.
	// clipStart is the x origin of this rect, for syntax clarity
	// and clipEnd is the x end of this rect
	float clipStart = clippingRect.origin.x - frameInDrawCoords.origin.x;
	float clipEndx = clipStart + clippingRect.size.width;

	int firstThumbToDraw;
	int lastThumbToDraw;
	if (nbThumbnailsInClip == 1)
	{
		lastThumbToDraw = 0;
		firstThumbToDraw = 0;
	}
	else
	{
		firstThumbToDraw = int(clipStart/(sizeOfThumbnail.width + spaceBetweenTwoThumbnails));
		lastThumbToDraw = int(clipEndx/(sizeOfThumbnail.width + spaceBetweenTwoThumbnails)) + 1;
		if (lastThumbToDraw >= nbThumbnailsInClip)
			lastThumbToDraw = nbThumbnailsInClip-1;
	}

	ArmeTime thumbTime = ARME_TIME_ZERO;
	fSizeOfThumbImg = NSMakeSize(133, 100);

	int iThumb;
	bool	bAllThumbnailsOfThisViewAreAvailable = YES;

	vernl(6, "(" << self << ") Drawing Thumbs (first = " << firstThumbToDraw << "), (last = " << lastThumbToDraw << ")");

	bool bAddedOneThumbInRequestList = false;

	for (iThumb = firstThumbToDraw; iThumb <= lastThumbToDraw; iThumb++)
	{
		// get the rect where to draw the thumbnail :
		// (Stephan) leave one pixel above and below the images
		NSRect thumbRect = NSMakeRect(frameInDrawCoords.origin.x + iThumb*(sizeOfThumbnail.width + spaceBetweenTwoThumbnails),
									  frameInDrawCoords.origin.y + 1,
									  sizeOfThumbnail.width,
									  sizeOfThumbnail.height);

		// get the time position of the thumbnail in the movie
		if (nbThumbnailsInClip >=2)
		{
			thumbTime = _ArmeTime(long(float(thumbRect.origin.x - frameInDrawCoords.origin.x)/float(frameInDrawCoords.size.width)*duration.data));
			thumbTime = [[self representedObject] WSObject]->GetDoc()->RoundArmeTimeToNearestVideoFrame(thumbTime);
		}
		else
			thumbTime = ARME_TIME_ZERO;
	
		// get the intersection of this rect and the clipping rect :
		NSRect drawRect = NSIntersectionRect(thumbRect, clippingRect);
	
		// the same rect in local coord 
		NSRect localDrawRect = NSMakeRect(drawRect.origin.x - thumbRect.origin.x,
										drawRect.origin.y - thumbRect.origin.y,
										drawRect.size.width,
										drawRect.size.height);
		//scale it to have image coordinates
		float scaleFactor = sizeOfThumbnail.width / fSizeOfThumbImg.width;
		localDrawRect.origin.x /= scaleFactor;
		localDrawRect.origin.y /= scaleFactor;
		localDrawRect.size.width /= scaleFactor;
		localDrawRect.size.height /= scaleFactor;
		
		int thumbIndexInArray;
		if ((thumbIndexInArray = [self IsThumbnailAvailableForTime:thumbTime]) != -1)
		{
			NSImage* img = fThumbnailsArray[thumbIndexInArray];
			[img drawInRect:drawRect fromRect:localDrawRect operation:NSCompositeCopy fraction:1.0f];
		}
		else
		{
			if (!bAddedOneThumbInRequestList)
			{
				// request to generate this thumbnail
				bAllThumbnailsOfThisViewAreAvailable = NO;
				fThumbRequest.fThumbTime = thumbTime;
				fThumbRequest.fNeedsRedraw = YES;
				fThumbRequest.fThumbRect = clippingRect;
				fThumbRequest.fThumbRect.origin.x = thumbRect.origin.x;
				
				// and draw a black rect
				[[NSColor blackColor] set];
				[NSBezierPath fillRect:drawRect];
				bAddedOneThumbInRequestList = true;
			}
		}
	}

	// if not yet in the list, add the clip rep to the list of clip rep being drawn
	if ([[[self representedObject] montageDoc]->fVideoClipRepIncompletelyDrawn indexOfObject:self] == NSNotFound)
		[[[self representedObject] montageDoc]->fVideoClipRepIncompletelyDrawn addObject:self];


	if ((![self getDoc]->IsPlaying()) ||
		([self getDoc]->IsPlaying() && [MIPreferences isGenerateThumbnailsWhenPlayingDoc]))
	{
		if ((bAllThumbnailsOfThisViewAreAvailable == NO))
		{
			[[MIApplicationController sharedApplicationController] sendNotification:GENERATE_THUMBNAILS toObject:self withParam:nil];
	
			//reset InAdvanceCachingVariables
			fCurrentTimeForInAdvanceCaching = ARME_TIME_ZERO;
			fCurrentStepForInAdvanceCaching = 0;
		}
		else if ((bAllThumbnailsOfThisViewAreAvailable==YES))
		{
			// remove the clip rep from the list of clip rep being drawn
			[[[self representedObject] montageDoc]->fVideoClipRepIncompletelyDrawn removeObject:self];
		
		
			// since all the thumbnails have been drawn, we can start to generate the surrounding thumbnails
	//		vernl(6, "Drawing Thumbs (Ask in advance caching)");
	
			// once this view is generated, we shall generate in advance the views close to this one
			int nbThumbnailsDrawn = 1 + lastThumbToDraw - firstThumbToDraw;
			fThumbDurationOfCurrentView = _ArmeTime(long(float(sizeOfThumbnail.width + spaceBetweenTwoThumbnails)/float(frameInDrawCoords.size.width)*duration.data));
			fStartTimeOfCurrentView = _ArmeTime(thumbTime.data - (nbThumbnailsDrawn-1)*fThumbDurationOfCurrentView.data);
			fEndTimeOfCurrentView = thumbTime;
			
	//		[self generateInAdvanceCachingDescriptionFrom:(thumbTime - (nbThumbnailsDrawn-1)*thumbTimeDuration) To:thumbTime WithInc:thumbTimeDuration];
			[[MIApplicationController sharedApplicationController] sendNotification:GENERATE_IN_ADVANCE_CACHING toObject:self withParam:nil];
			
			//all thumbnails have been drawn, update the cached image
			fHasGeneratedCurrentViewCompletely = YES;
			if ( !fCachedBodyImage )
				[self refreshCachedBodyImage:YES];

/*			SInt64 drawStartTime = (SInt64)(floor(clipStart)/float(frameInDrawCoords.size.width)*duration);
			SInt64 drawEndxTime = (SInt64)(ceil(clipEndx)/float(frameInDrawCoords.size.width)*duration);	
			// extra conversion to make sure we get the right frame rate on the other side
			double wPixels = (drawEndxTime - drawStartTime)*float(frameInDrawCoords.size.width)/duration;
			// now, we are going to ask a new image that at least contains these
			// two start times
			[self	updateCachedImageFrom:drawStartTime 
					to:drawEndxTime 
					wPixels:wPixels
					hPixels:frameInDrawCoords.size.height];
			ZzAssert(fCachedImage != nil);*/
		}
	}

// draw trim indicators

	if ( GET_VALUE([fRepresentedObject WSObject]->GetStartTrim()) )
		[mainImageController->fTrimIndicatorLeft compositeToPoint:frameInDrawCoords.origin operation:NSCompositeSourceAtop];
	else
		[mainImageController->fNonTrimIndicatorLeft compositeToPoint:frameInDrawCoords.origin operation:NSCompositeSourceAtop];		

	if ( GET_VALUE([fRepresentedObject WSObject]->GetEndTrim()) )
		[mainImageController->fTrimIndicatorRight compositeToPoint:NSMakePoint(NSMaxX(frameInDrawCoords)-10, NSMinY(frameInDrawCoords)) operation:NSCompositeSourceAtop];
	else
		[mainImageController->fNonTrimIndicatorRight compositeToPoint:NSMakePoint(NSMaxX(frameInDrawCoords)-10, NSMinY(frameInDrawCoords)) operation:NSCompositeSourceAtop];		
}

/*
#ifndef TWO_THREADS_NOTIFICATIONS
- (void)notificationReceived:(NSArray*)packedParams
{
	int code;
	id object;
	void* param;
	
	NSData* dataCode = [packedParams objectAtIndex:0];
	NSData* dataObject = [packedParams objectAtIndex:1];
	NSData* dataParam = [packedParams objectAtIndex:2];
	
	memcpy(&code, [dataCode bytes], sizeof(int));
	memcpy(&object, [dataObject bytes], sizeof(id));
	memcpy(&param, [dataParam bytes], sizeof(void*));
#else // #ifndef TWO_THREADS_NOTIFICATIONS*/
- (void)notificationReceived:(int)code param:(void*)param
{
//$$$
//#endif // #ifndef TWO_THREADS_NOTIFICATIONS
	switch (code)
	{
		case GENERATE_IN_ADVANCE_CACHING :
		{
			[self generateInAdvanceCaching];
		};break;
		case GENERATE_THUMBNAILS :
		{
			[self generateThumbnails];
		};break;
		default :
			return;
	};
}

- (int)IsThumbnailAvailableForTime:(ArmeTime)aTime
{
	// find the nearest image in the array
	int iThumb = [[self representedObject] WSObject]->GetDoc()->TimeValueFromArmeTime(aTime) / [[self representedObject] WSObject]->GetDoc()->GetVideoFrameDuration();

	if (iThumb < 0)
		iThumb = 0;
	else if (iThumb >= fNbThumbnailsInClip)
		iThumb = fNbThumbnailsInClip - 1;

	NSImage* img = fThumbnailsArray[iThumb];

	if ( img != nil)
		return iThumb;
	else
		return -1;
}

- (void)generateThumbnails
{
	[self getImageRepAtTimeValue:fThumbRequest.fThumbTime];
	if (fThumbRequest.fNeedsRedraw == YES)
		[fTLCRep setNeedsDisplayInRect:fThumbRequest.fThumbRect];
		//[fTLCRep setNeedsDisplay]; //
}


- (NSImage *)getImageRepAtTimeValue:(ArmeTime)aTime
{    
	if (!fThumbnailsArray)
		return nil;

	// find the nearest image in the array
	int iThumb = [[self representedObject] WSObject]->GetDoc()->TimeValueFromArmeTime(aTime) / [[self representedObject] WSObject]->GetDoc()->GetVideoFrameDuration();

	if (iThumb < 0)
		iThumb = 0;
	else if (iThumb >= fNbThumbnailsInClip)
		iThumb = fNbThumbnailsInClip - 1;

	if (fThumbnailsArray[iThumb] != nil)
	{
		NSImage* img = fThumbnailsArray[iThumb];

		// the image has already been generated
		vernl(6, "(" << self << ") Found Thumb for time " << aTime);

		return img;
	}
	else
	{	
		NSImage* img;

		// get the image of the thumbnail :
		SignedByte theState;
		PicHandle thePicHandle;
		NSData *thePictData;
		NSImageRep *rep;
		Size theSize;
		

		vernl(6, "(" << self << ") Generate Thumb for time" << aTime);
		thePicHandle = fVideoObject->GetPict(aTime);
		if (!thePicHandle)
		{
			vernl(4, "(" << self << ") GetMoviePict : error generating thumb " << iThumb << " for time " << aTime);
			return nil;
		}
//		NSAssert( thePicHandle, @"GetMoviePict" );

		theSize = GetHandleSize( (Handle)thePicHandle );
//		NSAssert( theSize, @"GetHandleSize" );

		theState = HGetState( (Handle)thePicHandle );

//		HLock( (Handle)thePicHandle );

		thePictData = [[[NSData dataWithBytes:*thePicHandle length:theSize] copy] autorelease];
		rep = [NSPICTImageRep imageRepWithData:thePictData];

		HSetState( (Handle)thePicHandle, theState );
		//$$$$TEMP KillPicture( thePicHandle );
		
		// normalize its size
		[rep setSize:fSizeOfThumbImg];

		//create the image with the imageRep
		img = [[NSImage alloc] initWithSize:fSizeOfThumbImg];
		[img addRepresentation:rep];
		fThumbnailsArray[iThumb] = img;
		//[img retain]; img a dj un retainCount de 1 avec l'alloc init

		return img;
	}
}

- (void)generateInAdvanceCaching
{
/*	float duration = fEndTimeOfCurrentView - fStartTimeOfCurrentView;

	if (fCurrentStepForInAdvanceCaching == 0)
	{
		fCurrentTimeForInAdvanceCaching = fEndTimeOfCurrentView + fThumbDurationOfCurrentView;
		fCurrentStepForInAdvanceCaching++;
		// and send a notification to generate the first thumb
		[[MIApplicationController sharedApplicationController] sendNotification:GENERATE_IN_ADVANCE_CACHING toObject:self withParam:nil];
	}
	// first, we ask to generate the view that is lying at the right side of the last one
	else if (fCurrentStepForInAdvanceCaching == 1)
	{
		if (fCurrentTimeForInAdvanceCaching < fVideoObject->GetDurationSec())
		{
			// generate thumb
			[self getImageRepAtTimeValue:fCurrentTimeForInAdvanceCaching];

			// and send a notification to generate next one
			fCurrentTimeForInAdvanceCaching += fThumbDurationOfCurrentView;
			[[MIApplicationController sharedApplicationController] sendNotification:GENERATE_IN_ADVANCE_CACHING toObject:self withParam:nil];
		}
		if (fCurrentTimeForInAdvanceCaching > (fEndTimeOfCurrentView + duration))
		{
			fCurrentStepForInAdvanceCaching++;
			fCurrentTimeForInAdvanceCaching = fStartTimeOfCurrentView - duration;
		}
	}
	else if (fCurrentStepForInAdvanceCaching == 2)
	{
		if (fCurrentTimeForInAdvanceCaching >= 0)
		{
			// generate thumb
			[self getImageRepAtTimeValue:fCurrentTimeForInAdvanceCaching];

			// and send a notification to generate next one
			fCurrentTimeForInAdvanceCaching += fThumbDurationOfCurrentView;
			[[MIApplicationController sharedApplicationController] sendNotification:GENERATE_IN_ADVANCE_CACHING toObject:self withParam:nil];
		}
		if (fCurrentTimeForInAdvanceCaching >= fStartTimeOfCurrentView)
		{
			fCurrentStepForInAdvanceCaching++;
			fCurrentTimeForInAdvanceCaching = fStartTimeOfCurrentView - duration;
		}
	}
	else if (fCurrentStepForInAdvanceCaching == 3)
	{
		//reset InAdvanceCachingVariables
		fCurrentTimeForInAdvanceCaching = 0.0f;
		fCurrentStepForInAdvanceCaching = 0;
	}*/
/*
	//then, we ask to generate the view that is on the right of the current one at a low res
	start = lastThumbTime + duration;
	if (start < fVideoObject->GetDurationSec())
	{
		end = start + 4*duration;
		for (t = start; t <= end; t+= thumbIncTime*2)
		{
			[self addToRequestList:t NeedsRedraw:NO];
		}
	}

	//then, we ask to generate the view that is on the left of the current one at a low res
	end = firstThumbTime - duration;
	if (end > 0.0f)
	{
		start = end - 4*duration;
		for (t = start; t <= end; t+= thumbIncTime*2)
		{
			[self addToRequestList:t NeedsRedraw:NO];
		}
	}

	//at last, we ask to generate all the thumbnails at a lower resolution than the one we have now
	start = 0;
	end = fVideoObject->GetDurationSec();
	for (t = start; t <= end; t+= thumbIncTime*4)
	{
		[self addToRequestList:t NeedsRedraw:NO];
	}*/
}
/*
- (void)addActiveCursors
{
	return;

	MIActiveCursor* activeCursor;
	NSRect tlcFrame = [self getTLCFrame];
	NSRect tmpRect[2];

	[super addActiveCursors];
	
	tmpRect[0] = [self rectForZone:MIObjectRepRightTimeStretchZone withObjectOrigin:[self getTLCFrame].origin];
	tmpRect[1] = [self rectForZone:MIObjectRepLeftTimeStretchZone withObjectOrigin:[self getTLCFrame].origin];

//	NSDivideRect( tlcFrame, &tmpRect[0],  &tlcFrame, 5, NSMaxXEdge );
//	NSDivideRect( tlcFrame, &tmpRect[1],  &tlcFrame, 5,NSMinXEdge );
	
	activeCursor = [[MIActiveCursor alloc] initWithPlainCursor:mainImageController->fTrimRightCursor forActiveRect:tmpRect[0]];
	[activeCursor setAlternateCursor:mainImageController->fTimeStretchRightCursor];
	[self addActiveCursor:activeCursor];
	[activeCursor release];

	activeCursor = [[MIActiveCursor alloc] initWithPlainCursor:mainImageController->fTrimLeftCursor forActiveRect:tmpRect[1]];
	[activeCursor setAlternateCursor:mainImageController->fTimeStretchLeftCursor];
	[self addActiveCursor:activeCursor];
	[activeCursor release];
}

- (void)addCursorRects
{
	NSPoint 	frOrigin = [self getTLCFrame].origin;

	if ( !fMouseIsOverMe )
		return;

	if ( [[NSApp currentEvent] modifierFlags] & NSAlternateKeyMask )
	{
		[fTLCRep addCursorRect:[self rectForZone:MIObjectRepLeftTimeStretchZone withObjectOrigin:frOrigin] cursor:mainImageController->fTimeStretchLeftCursor];
		[fTLCRep addCursorRect:[self rectForZone:MIObjectRepRightTimeStretchZone withObjectOrigin:frOrigin] cursor:mainImageController->fTimeStretchRightCursor];
	}
	else
	{
		[fTLCRep addCursorRect:[self rectForZone:MIObjectRepLeftTimeStretchZone withObjectOrigin:frOrigin] cursor:mainImageController->fTrimLeftCursor];
		[fTLCRep addCursorRect:[self rectForZone:MIObjectRepRightTimeStretchZone withObjectOrigin:frOrigin] cursor:mainImageController->fTrimRightCursor];
	}

	[super addCursorRects];

}

- (void)removeCursorRects
{
	[super removeCursorRects];
	
}
*/

// copied from MIAudioClipRep ...alex
- (MIObjectMouseHandling)handleMoreMouseTracking:(NSPoint)mouseLoc modifier:(unsigned int)modifier type:(NSEventType)type
{
    [self invalidateTheMouseTracker];

    // if the parent does NOT handle it, and if we are in the body, let's create a selection tracker
	if ( NSPointInRect(mouseLoc, [self rectForZone:MIObjectRepRightTimeStretchZone]) )
	{
		if ( modifier & NSAlternateKeyMask )
		{
			fTemporaryTracker = [[MITimeStretchTracker alloc] initWithView:fTLCRep atPos:[self convertPointToTLCCoords:mouseLoc] withModifiers:nil ];
			[fTemporaryTracker setEdge:NSMaxXEdge];
		}
        else
		{
			fTemporaryTracker = [[MITrimTracker alloc] initWithView:fTLCRep atPos:[self convertPointToTLCCoords:mouseLoc] withModifiers:nil ];
			[fTemporaryTracker setEdge:NSMaxXEdge];
		}
        return MIAskMeATracker;
	}
	else if ( NSPointInRect(mouseLoc, [self rectForZone:MIObjectRepLeftTimeStretchZone]) )
	{
		if ( modifier & NSAlternateKeyMask )
		{
			fTemporaryTracker = [[MITimeStretchTracker alloc] initWithView:fTLCRep atPos:[self convertPointToTLCCoords:mouseLoc] withModifiers:nil ];
			[fTemporaryTracker setEdge:NSMinXEdge];
		}
        else
		{
			fTemporaryTracker = [[MITrimTracker alloc] initWithView:fTLCRep atPos:[self convertPointToTLCCoords:mouseLoc] withModifiers:nil ];
			[fTemporaryTracker setEdge:NSMinXEdge];
		}
        return MIAskMeATracker;
	}
	else if ( NSPointInRect(mouseLoc, [self rectForZone:MIObjectRepRightBodySelectionZone]) )
	{ // modified tracker to start farther
		fTemporaryTracker = [[MISliceSelectionTracker alloc] initWithObjectRep:self inView:fTLCRep atPos:[self convertPointToTLCCoords:mouseLoc] withModifiers:nil ] ;
		[fTemporaryTracker setStartTime:[fRepresentedObject WSObject]->GetAbsoluteEndxTime()];
        return MIAskMeATracker;
	}
	else if ( NSPointInRect(mouseLoc, [self rectForZone:MIObjectRepLeftBodySelectionZone]) )
	{ // modified tracker to start farther
		fTemporaryTracker = [[MISliceSelectionTracker alloc] initWithObjectRep:self inView:fTLCRep atPos:[self convertPointToTLCCoords:mouseLoc] withModifiers:nil ] ;
		[fTemporaryTracker setStartTime:[fRepresentedObject WSObject]->GetAbsoluteStartTime()];
        return MIAskMeATracker;
	}
    else return [super handleMoreMouseTracking:mouseLoc modifier:modifier type:type];
}

- (void) setEffectiveDuration:(ArmeTime)newDuration
{
    fVideoObject->SetEffectiveDuration(newDuration);
}

- (void) setBeginOffset:(ArmeTime)newOffset
{
    fVideoObject->SetBeginOffset(newOffset);
}

- (void) setEndOffset:(ArmeTime)newOffset
{
    fVideoObject->SetEndOffset(newOffset);
}

/*- (void)setEnableCachedImageUpdates:(BOOL)flag
{
    fStopImageUpdates = !flag;
}*/

@end
