/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MIVideoTransitionRep.h"
#import "MIObject.h"
#import "MITopLevelContainerRep.h"
#import "MIMainViewController.h"
#import "MIWSSelection.h"

@implementation MIVideoTransitionRep

- (MIObjectMouseHandling)handleMoreMouseTracking:(NSPoint)mouseLoc modifier:(unsigned int)modifier type:(NSEventType)type
{
    NSRect						myFrame = [self frame];
    BOOL						bNeedsRedisplay = NO;
    MIObjectMouseHandling		status = MIContinueMouseHandling;
    NSRect						myTLCFrame = [self getTLCFrame];

	[self invalidateTheMouseTracker]; // au cas ou un tracker n'aurait pas t releas

	modifier &= (NSHelpKeyMask| NSControlKeyMask| NSShiftKeyMask| NSCommandKeyMask| NSAlternateKeyMask);

	// check for selection (only if it's about full objects)
	if ( [[[fTLCRep mainViewController] selection] type] == kMIWSSelectionTypeObjects )
	{
		if ([[fTLCRep mainViewController] isObjectInSelection:self])
		{
			// if the object is already in the selection and the COMMAND modifier is there, remove the object from the selection
			if ( modifier & NSCommandKeyMask )
			{
				[[fTLCRep mainViewController] removeObjectFromSelection:self];
				status = MIStopMouseHandling;
			}
		}
		else
		{
			// the object is not in the selection, so we clear the selection, add this object
			if ( !modifier )
				[[fTLCRep mainViewController] clearSelection];
			
			[[fTLCRep mainViewController] addObjectToSelection:self];
			status = MIStopMouseHandling;
		}
	}


	/*
	precedence:
		contextual menu
		buttons
		resize up, down, left, right
		resize corners
		move/copy/alias
	*/
		
// check for contextual menu
	if ( (type == NSRightMouseDown || (modifier & NSControlKeyMask )) &&
		 NSPointInRect(mouseLoc, [self rectForZone:MIObjectRepDragBarZone]) )
	{
			// bring up contextual menu
			status = MIStopMouseHandling;
//			[self popUpMenu:[[MIApplicationController sharedApplicationController] localEditMenu] atPos:[self convertPointToTLCCoords:mouseLoc]];
	}
	
// check for selection
	
    if ( bNeedsRedisplay )
    {
		[self setNeedsDisplay];
    }

    return status;
}


- (NSRect)rectForZone:(MIObjectRepZone)zone withObjectOrigin:(NSPoint)origin
{
	/*
					+-------------------------------------------+
		grab bar	|                                           |
					+-------------------------------------------+
		title bar	|                                           |
					|                                           |
					+---+-----------------------------------+---+
		resize up	|   |                                   |   |
					+---+-----------------------------------+---+
					|   |                                   |   |
					|   |                                   |   |
					|   |                                   |   |
		body		|   |                                   |   |
					|   |                                   |   |
					|   |                                   |   |
					|   |                                   |   |
					+---+-----------------------------------+---+
		resize down	| 1 |                                   | 1 |
					+---+-----------------------------------+---+
	*/
	
	NSRect	myFrame = fFrame;
	NSRect	rectForZone = NSZeroRect;
	
	myFrame.origin = NSZeroPoint;

	switch( zone )
	{
		default:
			rectForZone = NSZeroRect;
			break;
		case MIObjectRepBodyZone:
			rectForZone = myFrame;
			break;
	}
	if ( !NSContainsRect(myFrame, rectForZone) )
		rectForZone = NSIntersectionRect(myFrame, rectForZone);
		
	rectForZone.origin = TranslateNSPoint( rectForZone.origin, origin);
	
	return rectForZone;
}



- (id)initWithFrame:(NSRect)frame inTLC:(MITopLevelContainerRep*)theTLC
{
    self = [super initWithFrame:frame inTLC:theTLC];
	//fLeftBarImage = mainImageController->fContainerGroupBarOnLeft;
    //fCenterBarImage = mainImageController->fContainerGroupBarOnCenter;
    //fRightBarImage = mainImageController->fContainerGroupBarOnRight;

	fHasTopBottomHandles = NO;
	fHasLeftRightHandles = NO;
	fHasCollapseButton = NO;
	fHasPropertiesButton = NO;
	fHasOnOffButton = NO;
	
	fBackgroundColor = [[NSColor colorWithCalibratedRed:.699 green:.1 blue:.1 alpha:0.5f] retain];

    return self;
}

- (void)setImagesAndColor
{
	if ( [self isSelected] && [[[fTLCRep mainViewController] selection] type] == kMIWSSelectionTypeObjects )
	{
		[self setBackgroundColor:NegativeOfColor([NSColor colorWithCalibratedRed:.699 green:.1 blue:.1 alpha:0.5f])];
	}
	else
	{
		[self setBackgroundColor:[NSColor colorWithCalibratedRed:.699 green:.1 blue:.1 alpha:0.5f]];
	}
}

- (void)drawBaseBodyRect:(NSRect)frameInDrawCoords clippedTo:(NSRect)clippingRect
{
    // draw the background color
	[fBackgroundColor set];
	[NSBezierPath fillRect:NSIntersectionRect(frameInDrawCoords, clippingRect)];
}

- (void)drawTitleToPoint:(NSPoint)aPoint clippedTo:(NSRect)clippingRect
{
	NSRect			aRect;
	NSRect			imgClipRect = NSZeroRect;
	float			titleHeight, titleBarHeight;
	
#ifdef SCALABLE_TITLEBAR
	titleHeight = [[self getTopLevelContainerRep] MIVPosToPixel:[fRepresentedObject getTitleHeight]];
	titleBarHeight = [[self getTopLevelContainerRep] MIVPosToPixel:kRepTitleBarHeight];
#else
	titleBarHeight = kRepTitleBarHeight;
	titleHeight = kRepTitleHeight;
#endif


	aRect = [self frame];
	aRect.origin = aPoint;
//	aRect.size.height = titleHeight;

#ifndef SMALL_TITLEBAR
//	[fBackgroundColor set];
//	[NSBezierPath fillRect:aRect];
//	[[NSColor grayColor] set];
//	[NSBezierPath strokeLineFromPoint:NSMakePoint(NSMinX(aRect), NSMinY(aRect)) toPoint:NSMakePoint(NSMaxX(aRect), NSMinY(aRect))];
//	aRect.origin.y += (titleHeight - titleBarHeight);
//	aRect.size.height = titleBarHeight;
#endif


	// take the left size of the image from aRect, draw it
/*	NSDivideRect(aRect, &tmpRect, &aRect, [fLeftTitleImage size].width, NSMinXEdge);
	[fLeftTitleImage drawInRect:tmpRect fromRect:NSMakeRect(0,0,[fLeftTitleImage size].width,[fLeftTitleImage size].height) operation:NSCompositeCopy fraction:1];

	// do the same for the right part
	NSDivideRect(aRect, &tmpRect, &tmpRect2, [fRightTitleImage size].width, NSMaxXEdge);
	[fRightTitleImage drawInRect:tmpRect fromRect:NSMakeRect(0,0,[fRightTitleImage size].width,[fRightTitleImage size].height) operation:NSCompositeCopy fraction:1];

	// draw the center image where in the remaining rect
	[fCenterTitleImage drawInRect:tmpRect2 fromRect:NSMakeRect(0,0,[fCenterTitleImage size].width, [fCenterTitleImage size].height) operation:NSCompositeCopy fraction:1];
*/
}

- (void)refreshCachedTitleStringImage
{
}

- (void)refreshCachedBodyImage:(BOOL)createIfAbsent
{

}

- (BOOL)doubleClick:(NSPoint)mouseLoc modifier:(unsigned int)modifier
{
	vertl(commandsBenjamin,5,"INFO : the 'configure video transition' menu entry has been selected (by double-clicking the object).\n");
	[[NSApp mainViewController] executeCommand:@"configureTransition" ];	
	return YES;
}
@end
