//
//  MIVideoWindow.mm
//  Montage
//
//  Created by alainm on Sat Nov 17 2001.
//  Copyright (c) 2001-2002 Arboretum. All rights reserved.
//

#import "MIVideoWindow.h"
#import "MIVideoController.h"
#import "MIApplicationController.h"
#import "MIPickScreenController.h"


#ifndef TArmeDoc_h
#include "TArmeDoc.h"
#endif //TArmeDoc_h

#ifndef ZzVerbose_h
//#include "ZzVerbose.h"
#endif //ZzVerbose_h

// this is above the 'kill task panel' level
// but so is the Dock, so doesn't look like there's much effective to do about that.
const int kTheaterLevel = NSScreenSaverWindowLevel;

@implementation MIMovieView

- (void)drawRect:(NSRect)rect {
	[[NSColor whiteColor] set];  //Georges wants the background to be white when there is no video media
	[NSBezierPath fillRect:rect];
		//[super drawRect:rect];
}

@end


@implementation MIVideoWindow

//TODO
//NSMovieView est drive de NSView, donc drawRect:(NSRect)rect s'applique l aussi. Tu peux forcer un if ( !fIsPlaying ) {[[NSColor blackColor] set]; [NSBezierPath fillRect:[self bounds];} else { [super drawRect:rect] }

/*
- (void)display
{
	[super display];
}
- (void)displayIfNeeded
{
	[super display];
}
*/

- (id)initWithFrame:(NSRect)frameRect
{
	//create the window
//	self = [super initWithContentRect:frameRect styleMask:(NSUtilityWindowMask|NSTitledWindowMask|NSClosableWindowMask|NSMiniaturizableWindowMask|NSResizableWindowMask) backing:NSBackingStoreBuffered defer:YES];
	self = [super initWithContentRect:frameRect

/* NB:
The wacky-text-track-drawing issue is directly correlated with NSUtilityWindowMask.
I deduce this because making the text clip creation dialog use it makes the 
problem show up there too.
I've asked on qt-api if anyone knows about this issue, so hopefully I'll
have some more info from the QT engineers soon ...alex
*/
	styleMask:(/*NSUtilityWindowMask|*/NSTitledWindowMask)
        backing:NSBackingStoreBuffered defer:NO];

	[self setAutodisplay:YES];

//	[self setFloatingPanel:YES];
//	[self setHidesOnDeactivate:YES];
	//create the movie view without controller
        NSRect movieRect = frameRect;
        movieRect.origin = NSZeroPoint;
		movieRect.origin.y = -22;  //try this to avoid the movie to be drawn behind the title bar
	fMovieView = [[MIMovieView alloc] initWithFrame:movieRect];
	[fMovieView showController:NO adjustingSize:YES]; 

	[self setFrame:frameRect display:YES];
//	[super makeKeyAndOrderFront:self];

	[[self contentView] addSubview:fMovieView];
	[fMovieView setAutoresizingMask:NSViewWidthSizable | NSViewHeightSizable];
	[[self contentView] setAutoresizesSubviews:YES];
	//[self setContentView:fMovieView];
        
	fMovie = nil;
	//fIsFullScreen = NO;
        fTrackingRect = 0;
        fMagnification = 1;
		
	return self;
}

- (void)dealloc
{
	// this appears to avoid the crashing on quit, bug #192
	[self detachMovie];
	[fMovieView release];
	[super dealloc];
}

- (void)detachMovie
{
	if (fMovie)
	{
	//	[fMovie release];  //$$ why does it crashes when I release the fMovie before re-allocating it ?
	// Possibly because somewhere else calls ::DisposeMovie, and so does NSMovie.
	// That means this is a leak that should be fixed, probably ...alex
	fMovie = nil;
	}

	[fMovieView setMovie:nil];
}

- (void)attachMovie:(Movie)aMovie
{
	TimeValue 	timeNow;
	Fixed 		playRate;

	[self detachMovie];
	
	if (aMovie)
	{
		fMovie = [[NSMovie alloc] initWithMovie:aMovie];
		[fMovieView setMovie:fMovie];
	}


	fQTMovie = aMovie;
        /* this appears empty when initialized!
        Rect movieBounds;
        ::GetMovieBox(fQTMovie, &movieBounds);
        NSSize movieSize;
        movieSize.width = movieBounds.right - movieBounds.left;
        movieSize.height = movieBounds.bottom - movieBounds.top;
        [self setContentSize:movieSize];
        */
	
	if (aMovie)
	{
		timeNow = GetMovieTime(aMovie, nil);
		playRate = GetMoviePreferredRate(aMovie);
		PrePrerollMovie(aMovie, timeNow, playRate, nil, nil);
		PrerollMovie(aMovie, timeNow, playRate);
	}
	
	// change the size of the window to the size of the movie
	//$$ marche pas !
//	GetMovieBox(aMovie, &rect);
//	[self setFrame:(NSRectFromRect(rect)) display:YES];

}

- (void)setMagnification:(float)magnification
{
    if (!magnification or (magnification == fMagnification))
        return;
    

    NSRect frame = [NSWindow contentRectForFrameRect:[self frame] styleMask:NSTitledWindowMask];
    NSSize newSize = frame.size;
/*    newSize.width /= fMagnification;
    newSize.width *= magnification;
    newSize.height /= fMagnification;
    newSize.height *= magnification;
*/
	newSize.width = [[[NSDocumentController sharedDocumentController] currentDocument] armeDocument]->fMasterMovie->GetWidth() * magnification;
	newSize.height = [[[NSDocumentController sharedDocumentController] currentDocument] armeDocument]->fMasterMovie->GetHeight() * magnification;
    [self setContentSize:newSize];
    
    fMagnification = magnification;

	[[self contentView] setNeedsDisplay:YES];
}

- (void)takeMovieView:(NSMovieView*)theMovieView
{
	if ( fMovieView != theMovieView ) {
		[fMovieView release];
		fMovieView = [theMovieView retain];
	}
    fQTMovie = (MovieRecord*)([[fMovieView movie] QTMovie]);
	
	[[self contentView] addSubview:fMovieView];
	[fMovieView setAutoresizingMask:NSViewWidthSizable | NSViewHeightSizable];
	[[self contentView] setAutoresizesSubviews:YES];

    [self orderFront:nil];
}

- (NSMovieView*)giveMovieView;
{
    NSMovieView* givenView = fMovieView;
    fMovieView = nil;
    fQTMovie = nil;
    
   // [givenView retain];
    [givenView removeFromSuperviewWithoutNeedingDisplay];

    [self orderOut:nil];

    // On the off chance cursor was still hidden from full screening
    [NSCursor setHiddenUntilMouseMoves:NO];
    // get rid of tracking rect if any
    if (fTrackingRect)
    {
        [[self contentView] removeTrackingRect:fTrackingRect];
        fTrackingRect = 0;
    }

    return givenView;
}

#if OBSOLETE
-(BOOL)setFullScreen:(BOOL)isFullScreen
{
    verntl(videoAlex,5,"setFullScreen(" << isFullScreen << ") called");
    
    fIsFullScreen = isFullScreen;
    
    NSScreen* theScreen = nil;
    if (fIsFullScreen)
    {
        theScreen = [[MIPickScreenController sharedPickScreenController] pickScreen];
        if (!theScreen)
            {
            fIsFullScreen = false;
            // this is a cancel, setting as below isn't needed
            return false;
            }
    }

    NSRect movieFrame;
    if (theScreen)
    {
        fNormalFrame = [self frame];
	[self setFrame:[theScreen frame] display:YES];
	[self setLevel:kTheaterLevel];
	[self makeKeyAndOrderFront:nil];
        movieFrame = ScaleNSRectToNSRect(fNormalFrame, [self frame], true);
    }
    else
    {
	[self setFrame:fNormalFrame display:YES];
	[self setLevel:NSNormalWindowLevel];
        movieFrame = fNormalFrame;
        movieFrame.origin.x = 0;
        movieFrame.origin.y = 0;
    }
    [fMovieView setBounds:movieFrame];
           
    return fIsFullScreen;
}

-(BOOL)isFullScreen
{
	return fIsFullScreen;
}

#endif OBSOLETE

/*
- (BOOL)canBecomeKeyWindow
{
    return NO;
}

- (BOOL)canBecomeMainWindow
{
    return NO;
}
*/
@end

@implementation MIFullScreenVideoWindow

- (id)initWithFrame:(NSRect)frameRect
{
    //create the window
    self = [super initWithContentRect:frameRect
    styleMask: NSBorderlessWindowMask
    backing:NSBackingStoreNonretained defer:NO];
    
    [self setFrame:frameRect display:NO];
    [self setBackgroundColor:[NSColor blackColor]];
    
    fMovieView = nil;
    fMovie = nil;
            
    return self;
}

- (void)takeMovieViewAndFill:(NSMovieView*)theMovieView onScreen:(NSScreen*)theScreen
{
    [self setFrame:[theScreen frame] display:YES];

	if ( fMovieView != theMovieView ) {
		[fMovieView release];
		fMovieView = [theMovieView retain];
	}
	
    fQTMovie = (MovieRecord*)([[fMovieView movie] QTMovie]);
    
    //[self setContentView:fMovieView];
    [[self contentView] addSubview:fMovieView];
    Rect movieBounds;
    ::GetMovieBox(fQTMovie, &movieBounds);
    NSRect movieFrame = NSMakeRect(movieBounds.left, movieBounds.top, movieBounds.right-movieBounds.left, movieBounds.bottom-movieBounds.top);
    movieFrame = ScaleNSRectToNSRect(movieFrame, [theScreen frame], true);
    [fMovieView setFrame:movieFrame];
    
    [self orderFront:nil];
 
    // is this what we need to get the top and bottom to draw in background color?
    [[self contentView] setNeedsDisplay:TRUE];
 
    [self makeKeyAndOrderFront:nil];
    [self setLevel:kTheaterLevel];
    
    // Georges wants cursor hidden until mouse moved
    [NSCursor setHiddenUntilMouseMoves:YES];
    // and menu bar to show when moved over
    if (theScreen == [NSScreen mainScreen])
        [self setTrackingRect:NO];
}

- (BOOL)canBecomeKeyWindow
{
    return YES;
}

- (void)keyDown:(NSEvent *)theEvent
{
    [[MIVideoController sharedVideoController] restoreBackstageView:self];
}

- (void)mouseDown:(NSEvent *)theEvent
{
    [[MIVideoController sharedVideoController] restoreBackstageView:self];
}

- (void) setTrackingRect:(BOOL)forHidingMenuBar
{
    // need this to avoid flicker it seems
    if (fTrackingRect)
        [[self contentView] removeTrackingRect:fTrackingRect];

    NSRect tracking = [self frame];
    if (forHidingMenuBar)
    {
        tracking.size.height -= [NSMenuView menuBarHeight];
    }
    else
    {
        tracking.origin.y += tracking.size.height - [NSMenuView menuBarHeight];
        tracking.size.height = [NSMenuView menuBarHeight];
   }
    
    fTrackingRect = [[self contentView] addTrackingRect:tracking owner:self userData:nil assumeInside:NO];
}

- (void) mouseEntered:(NSEvent*)event
{
    if ([self level] == kTheaterLevel)
    {
        [self setLevel:NSMainMenuWindowLevel - 1];
        [self setTrackingRect:YES];
    }
    else
    {
        [self setLevel:kTheaterLevel];
        [self setTrackingRect:NO];
    }
}

/*
- (void)mouseExited:(NSEvent *)theEvent
{
    [self setLevel:kTheaterLevel];
    [self setTrackingRect:NO];
}
*/

@end

#if DONT_USE_THIS

// kept around in case we ever need it

#include <DrawSprocket/DrawSprocket.h>

// Compile-time flags for various fullscreen related aesthetic preferences

// Use gamma fadetoblack around presentation?
// If off, resolution changing will cause a flash.
#define FADEINFADEOUT 0
// Fade just presentation monitor, or all of them?
#define FADEALLSCREENS 0
// Switch to a more appropriate display resolution or not?
// Note that not changing will involve centering/scaling issues.
// But they'll happen anyway when(if?) project output dimensions change from 320x240.
#define SWITCHRESOLUTION 1


static bool AnyKeyPressed()
{
    unsigned char theKeyMap[16];
    ::GetKeys((long*)theKeyMap);
    for (int k = 0; k < 16; k++)
            if (theKeyMap[k] != 0)
                    return true;
    return false;
}
        
- (void)presentWithDrawSprocket
{
        OSStatus theError = noErr;

        // this should be part of application initialization, really,
        // and the fullscreen option in the menu should be unavailable if it fails. (Not that it should, but...)
        // What is an Objective-C equivalent of static class variables/constructors/destructors in C++?
        if (::DSpStartup() != noErr)
        {
            fIsFullScreen = false;
            return;
        }

        // get attributes of movie here that we'll restore so as to not confuse the rest of the program
        Rect movieBounds;
        ::GetMovieBox(fQTMovie, &movieBounds);
        CGrafPtr moviePort;
        GDHandle movieGDH;
        ::GetMovieGWorld(fQTMovie, &moviePort, &movieGDH);
        RgnHandle movieClip = ::GetMovieDisplayClipRgn(fQTMovie);
        Fixed movieRate = ::GetMovieRate(fQTMovie);
        TimeValue movieTime = ::GetMovieTime(fQTMovie, nil);
        
        // Output context should perhaps be a preference so multiple monitor users don't have to select every time?
        DSpContextAttributes contextAttr = { 0 };
#if SWITCHRESOLUTION
        // seems that OS X DrawSprocket will not do a best fit on your size request
        // so we'll just ask for 640x480 for now. (Output is currently fixed at 320x240.)
        contextAttr.displayWidth		= 640; // movieBounds.right - movieBounds.left;
        contextAttr.displayHeight		= 480; //movieBounds.bottom - movieBounds.top;
#else
        // we'll ask for the first video device's size
        GDHandle firstGDH =  ::DMGetFirstScreenDevice(true);
        contextAttr.displayWidth		= (**firstGDH).gdRect.right - (**firstGDH).gdRect.left;
        contextAttr.displayHeight		= (**firstGDH).gdRect.bottom - (**firstGDH).gdRect.top;
#endif SWITCHRESOLUTION
        contextAttr.colorNeeds		= kDSpColorNeeds_Require;
        contextAttr.backBufferDepthMask	= kDSpDepthMask_All;
        contextAttr.displayDepthMask	= kDSpDepthMask_All;
        contextAttr.backBufferBestDepth	= 32;
        contextAttr.displayBestDepth	= 32;
        contextAttr.pageCount		= 1;
        // apparently VBL synchronization is not an OS X feature of DrawSprocket
        contextAttr.contextOptions		= 0; // | kDSpContextOption_DontSyncVBL;
        DSpContextReference context = nil;
        
        // hmmmm. DSpUserSelectContext() is not available in OS X.
        // We'll need another way to select monitor when there's more than one.
        //theError = ::DSpUserSelectContext(&contextAttr, 0L, nil, &context);
        theError = ::DSpFindBestContext(&contextAttr, &context);
        if (!theError)
            theError = ::DSpContext_Reserve(context, &contextAttr);
        
        if (!theError)
        {
#if FADEINFADEOUT
#if FADEALLSCREENS
            DSpContextReference fadeDisplays = nil;
#else
            DSpContextReference fadeDisplays = context;
#endif FADEALLSCREENS
#endif FADEINFADEOUT
            // fade and blank to DSp context
            const RGBColor rgbBlack = { 0x0000, 0x0000, 0x0000 };
            ::DSpSetBlankingColor(&rgbBlack);
            ::HideCursor();
#if FADEINFADEOUT 
            ::DSpContext_FadeGammaOut(fadeDisplays, nil);
#endif FADEINFADEOUT 
            ::DSpContext_SetState(context, kDSpContextState_Active);
#if FADEINFADEOUT 
            ::DSpContext_FadeGammaIn(fadeDisplays, nil);
#endif FADEINFADEOUT 
            GrafPtr	savePort;
            GDHandle saveGDH;
            ::GetPort(&savePort);
            saveGDH = ::GetGDevice();
            CGrafPtr dspGrafPtr;
            ::DSpContext_GetFrontBuffer(context, &dspGrafPtr);
            DisplayIDType displayID;
            ::DSpContext_GetDisplayID(context, &displayID);
            GDHandle dspGDH = nil;
            ::DMGetGDeviceByDisplayID(displayID, &dspGDH, false);
            ::SetPort((GrafPtr)dspGrafPtr);
            ::SetGDevice(dspGDH);
            ::ForeColor(blackColor);
            ::BackColor(whiteColor);

            // play the movie in the context, taking over machine completely for the duration
            ::SetMovieGWorld(fQTMovie, dspGrafPtr, nil);
            Rect screenBounds;
            ::GetPortBounds(dspGrafPtr, &screenBounds);
#if (!SWITCHRESOLUTION)
            #warning we really need to calculate a properly centered/scaled movieBox here
#endif SWITCHRESOLUTION
            ::SetMovieBox(fQTMovie, &screenBounds);
            RgnHandle screenClip = ::NewRgn();
            //::RectRgn(screenClip, &screenBounds);
            // what on earth is up with needing to inset by 1, I have no idea. Some OS X thing? Something about fQTMovie?
            ::SetRectRgn(screenClip, 0, 0, screenBounds.right - 1, screenBounds.bottom - 1);
            ::SetMovieDisplayClipRgn(fQTMovie, screenClip);
            ::GoToBeginningOfMovie(fQTMovie);
            ::StartMovie(fQTMovie);
            ::FlushEvents(mDownMask | mUpMask | keyDownMask | keyUpMask | autoKeyMask, 0);
            while(!Button() and !AnyKeyPressed())
                    {
                    // hmmmm. Video plays, but sound doesn't. Apparently somebody needs to get processing time here.
                    ::MoviesTask(nil, DoTheRightThing);
                    ::MoviesTask(nil, DoTheRightThing);
                    ::MoviesTask(nil, DoTheRightThing);
                    if (::IsMovieDone(fQTMovie))
                            break;
                    }
            while (Button() or AnyKeyPressed())
                    ; // pause and flush to eliminate spurious release messages, switching apps, etc.
            ::FlushEvents(mDownMask | mUpMask | keyDownMask | keyUpMask | autoKeyMask, 0);
            
            // restore movie to state before we started messing with it
            ::SetMovieBox(fQTMovie, &movieBounds);
            if (movieClip)
            {
                ::SetMovieDisplayClipRgn(fQTMovie, movieClip);
                ::DisposeRgn(movieClip);
            }
            ::DisposeRgn(screenClip); // looks like a bug in OS X QuickTime to me, needing to keep this around
            ::SetMovieRate(fQTMovie, movieRate);
            ::SetMovieTimeValue(fQTMovie, movieTime);
            ::SetMovieGWorld(fQTMovie, moviePort, movieGDH);

            // close down the DSp context, resume normal display
            ::SetPort(savePort);
            ::SetGDevice(saveGDH);
#if FADEINFADEOUT 
            ::DSpContext_FadeGammaOut(fadeDisplays, nil);
#endif FADEINFADEOUT 
            ::DSpContext_SetState(context, kDSpContextState_Inactive);
#if FADEINFADEOUT 
            ::DSpContext_FadeGammaIn(fadeDisplays, nil);
#endif FADEINFADEOUT 
            ::ShowCursor();
            ::DrawMenuBar();
        }
        // this is balanced with DSpStartup call(s), as per note above it should be in an application termination routine
        ::DSpShutdown();
        
        if (theError != noErr)
        {
            ::SysBeep(1);
            // what is a Montage methodology for reporting runtime errors to the user?
        }
}

#endif DONT_USE_THIS
