/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
/*!

	This header defines he MIWSelection class
	that describes a workspace selection
	
	See the class for a torough description
	
*/

#import <Cocoa/Cocoa.h>

#ifndef ArmeBasicTypes_h
#include "../Arme/ArmeBasicTypes.h"
#endif //ArmeBasicTypes_h

#ifndef ZzArray_h
#include "ZzArray.h"
#endif //ZzArray_h

#import "MontageInterface.h"

#ifdef AUTOMATION_DATA
#import "cSpline.h"
#endif

extern NSString *MTGSelectionChangedNotification;

enum EMIWSSelectionType
{
	kMIWSSelectionTypeTimeSlice,
	kMIWSSelectionTypeObjects,
#ifdef AUTOMATION_DATA
	kMIWSSelectionTypeAutomationPoints,
	kMIWSSelectionTypeAutomationCurves,
#endif
};

/*!
	@class MIWSSelectionItem
	
	A selection can be :
	
	- a time range and a set of objects
	OR
	- a set of objects
	OR
	- a set of automation points
	OR
	- a set of automation curves
	
*/

#ifndef ArmeObjCArchivable_h
#include "ArmeObjCArchivable.h"
#endif //ArmeObjCArchivable_h

@class MIObjectRep;
@class MIObject;

@interface MIWSSelection : NSObject 
{
    SInt32				fType;
	NSMutableArray*		fObjectArray;
	ArmeTime			fStartTime;
	ArmeTime			fEndxTime;
}

-(BOOL)writeToArchive:(ArmeArchiveHandler*)archiver;
-(id)initFromArchive:(ArmeArchiveHandler*)unarchiver;

/*! @function containsObject Returns YES if the selection is a full objects selection
    and if "object" is part of this selection. if the selection is not a full objects 
	selection, it crashes */
- (BOOL)containsObject:(id)object;

- (BOOL)containsVideo:(BOOL)considerAlsoMutedObjects andTransparentTextClip:(BOOL)considerAlsoTransparentTextClips;
- (BOOL)containsAudio;
- (BOOL)containsVideoTransitions;
- (BOOL)containsOnlyVideoTransitions;
- (BOOL)containsOnlyTextClips;
- (BOOL)containsOnlyAudio;
- (BOOL)containsTextClips;

- (BOOL)audioMembersAreMadeOfNonOverlappingSiblings;

- (id)discardVideoTransitions;
- (id)keepOnlyVideoTransitions;
- (id)discardVideoClips;
- (id)discardTextClips;

- (void)adjustSelectionTimesToVideo;

/*! @function clear empties the selection without changing its mode */
- (void)clear;

/*! @function initEmpty designated initializer */
- (id)initEmpty;

/*! @function initWitExistingSelection initializes the object with the content of another selection */
- (id)initWithExistingSelection:(MIWSSelection*)selection;

/*! @function putRepsInArrayAndRemoveDescendants This function copies the selection into an NSArray, removing the
objects that have one or more ancestors in the selection. For example, if we have container C1 that contains
container C2 that contains clips CL1 and CL2, and if C1 and CL1 are selected, then the result of the function
will be C1 only. This is meant to be used in the move selection tracker where we CAN'T move an object AND its
children (or grandchildren, or grandgrandchildren...). Moving an object implies movings all its descendants
so we don't have to explicitely move the descendants and that's why we remove them from the selection 

THE RETURNED ARRAY IS AUTORELEASED. YOU MAY LEAVE IT 'AS THIS'.
*/
- (NSArray*)putRepsInArrayAndRemoveDescendants;


/*! @function putRepsThatContainAudioInArrayAndRemoveDescendants Same as putRepsInArrayAndRemoveDescendants
but only add object that reply YES to TArmeWSObj::ContainsAudio() */
- (NSArray*)putRepsThatContainAudioInArrayAndRemoveDescendants;

/*! @function objectEnumerator Returns an enumerator for fObjectArray */
- (NSEnumerator*)objectEnumerator;

/*! @function addObject Adds an object to the selection */
- (void)addObject:(MIObjectRep*)rep;

#ifdef AUTOMATION_DATA
- (void)addAutomationPoints:(NSArray*)points;
- (void)addAutomationCurves:(NSArray*)curves;
#endif


- (void)removeObject:(MIObjectRep*)rep;

#ifdef AUTOMATION_DATA
- (void)removePoint:(cSplineKnot*)aPoint;
- (void)removeCurve:(cSpline*)curve;
#endif

- (unsigned)indexOfObject:(id)anObject;

- (ArmeTime)duration;

- (MIObjectRep*)objectAtIndex:(unsigned)index;

- (unsigned)count;

// getters
- (NSArray*)objArray;
- (ArmeTime)startTime;
- (ArmeTime)endxTime;
- (ArmeTime)length;
- (SInt32)type;

- (MIObject*)getLeftMostObject;
- (MIObject*)getRightMostObject;

// setters
- (void)setStartTime:(ArmeTime)startTime;  
- (void)setEndxTime:(ArmeTime)endxTime;  

- (void)setTimeSliceType;
- (void)setObjectsType;

@end              

