/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MIWSSelection.h"
#import "MIObject.h"
#import "MIObjectRep.h"
#import "MIVideoTransitionRep.h"
#import "MIVideoClipRep.h"
#import "MITextClipRep.h"

#ifndef ArmeArchiveHandler_h
#include "ArmeArchiveHandler.h"
#endif //ArmeArchiveHandler_h

#ifndef TArmeWSObj_h
#include "TArmeWSObj.h"
#endif //TArmeWSobj_h

#ifndef TArmeDoc_h
#include "TArmeDoc.h"
#endif //TArmeDoc_h


NSString *MTGSelectionChangedNotification = @"Selection Changed";


@implementation MIWSSelection

-(BOOL)writeToArchive:(ArmeArchiveHandler*)archiver
{
	[super writeToArchive:archiver];
	archiver->WriteSInt32(fType);
//	archiver->WriteObjCObject(fObjectArray);
	archiver->WriteArmeTime(fStartTime);
	archiver->WriteArmeTime(fEndxTime);
	return YES;
}
	
-(id)initFromArchive:(ArmeArchiveHandler*)unarchiver
{
	self = [super initFromArchive:unarchiver];
	if(self)
	{
		fType = unarchiver->ReadSInt32();
//		fObjectArray = unarchiver->ReadObjCObject();
		fObjectArray = nil;
		fStartTime = unarchiver->ReadArmeTime();
		fEndxTime = unarchiver->ReadArmeTime();
	}
	return self;
}

- (BOOL)containsObject:(id)object
{
//	ZzAssert(fType == kMIWSSelectionTypeObjects);
	if([fObjectArray indexOfObject:object] != NSNotFound)
		return YES;
	return NO;
}

- (BOOL)containsVideo:(BOOL)considerAlsoMutedObjects andTransparentTextClip:(BOOL)considerAlsoTransparentTextClips
{
	MIObjectRep*	objRep;
	
	for(UInt32 i = 0; i <	[fObjectArray count]; i++)
	{
		objRep = [fObjectArray objectAtIndex:i];
		if ([[objRep representedObject] WSObject]->ContainsVideo(considerAlsoMutedObjects, considerAlsoTransparentTextClips))
			return TRUE;
	}
	
	return FALSE;
}

- (BOOL)containsAudio
{
	NSEnumerator*	objEnum = [fObjectArray objectEnumerator];
	MIObjectRep*	objRep;
	while (objRep = [objEnum nextObject])
	{
		if ([[objRep representedObject] WSObject]->ContainsAudio())
			return TRUE;
	}
	
	return FALSE;
}

- (BOOL)containsVideoTransitions
{
	NSEnumerator*	objEnum = [fObjectArray objectEnumerator];
	MIObjectRep*	objRep;
	while (objRep = [objEnum nextObject])
	{
		if ([objRep isKindOfClass:[MIVideoTransitionRep class]])
			return TRUE;
	}
	
	return FALSE;
}

- (BOOL)containsOnlyVideoTransitions
{
	MIObjectRep*	objRep;
	
	if ( [fObjectArray count] == 0 ) return NO;
	
	for(UInt32 i = 0; i < [fObjectArray count]; i++)
	{
		objRep = [fObjectArray objectAtIndex:i];
		if (![objRep isKindOfClass:[MIVideoTransitionRep class]])
			return FALSE;
	}
	
	return TRUE;
}

- (id)discardVideoTransitions
{
	NSEnumerator*	objEnum = [fObjectArray objectEnumerator];
	MIObjectRep*	objRep;
	while (objRep = [objEnum nextObject])
	{
		if ([objRep isKindOfClass:[MIVideoTransitionRep class]])
		{
			//remove the video transition
			[fObjectArray removeObject:objRep];
			//and restart enumerating at the beginning of the array (don't know what it does if I delete the object that is pointed by the enumerator)
			objEnum = [fObjectArray objectEnumerator];
		}
	}
	return self;
}

- (id)keepOnlyVideoTransitions
{
	NSEnumerator*	objEnum = [fObjectArray objectEnumerator];
	MIObjectRep*	objRep;
	while (objRep = [objEnum nextObject])
	{
		if (![objRep isKindOfClass:[MIVideoTransitionRep class]])
		{
			//remove the video transition
			[fObjectArray removeObject:objRep];
			//and restart enumerating at the beginning of the array (don't know what it does if I delete the object that is pointed by the enumerator)
			objEnum = [fObjectArray objectEnumerator];
		}
	}
	return self;
}

int MIObjectRepSortAccordingToStartTime(id obj1, id obj2, void* context)
{
	SInt64 start1 = GET_VAL([[obj1 representedObject] WSObject]->GetAbsoluteStartTime());
	SInt64 start2 = GET_VAL([[obj2 representedObject] WSObject]->GetAbsoluteStartTime());
	if (start1 < start2)
        return NSOrderedAscending;
    else if (start1 > start2)
        return NSOrderedDescending;
    else
        return NSOrderedSame;
}

- (BOOL)audioMembersAreMadeOfNonOverlappingSiblings
{
	// this function returns YES only if all the objects
	// in the selection, ******after putRepsInArrayAndRemoveDescendants has been called****
	// are 
	// - siblings
	// - non-overlapping

	NSArray* selObjs = [self putRepsThatContainAudioInArrayAndRemoveDescendants];

	if([selObjs count] == 0)
	{
		ZZ_THROW_1ARG(ZzGenericException,"Sorry you are not allowed to call this with an empty selection (audioMembersAreMadeOfNonOverlappingSiblings)");
		return YES;
	}

	if([selObjs count] == 1)
		return YES;
		
	TArmeWSObj* firstParent = [[[selObjs objectAtIndex:0] representedObject] WSObject]->GetParent();

	for(UInt32 i = 1; i < [selObjs count]; i++)
	{
		TArmeWSObj* thisParent = [[[selObjs objectAtIndex:i] representedObject] WSObject]->GetParent();
		if(thisParent != firstParent)
		{
			return NO;
		}
	}
	
	NSArray* sortedObjs = [selObjs sortedArrayUsingFunction:MIObjectRepSortAccordingToStartTime context:NULL];
		
	for(UInt32 i = 0; i < [sortedObjs count]-1; i++)
	{
		MIObjectRep* rep = [sortedObjs objectAtIndex:i];
		MIObjectRep* nextRep = [sortedObjs objectAtIndex:(i+1)];
		TArmeWSObj* obj = [[rep representedObject] WSObject];
		TArmeWSObj* nextObj = [[nextRep representedObject] WSObject];
		ZzAssert(nextObj->GetAbsoluteStartTime() >= obj->GetAbsoluteStartTime());
		if(nextObj->GetAbsoluteStartTime() < obj->GetAbsoluteEndxTime())
			return NO;
	}
	return YES; 
}

- (BOOL)containsTextClips
{
	//NSEnumerator*	objEnum = [fObjectArray objectEnumerator];
	MIObjectRep*	objRep;
	
	for(UInt32 i = 0; i < [fObjectArray count]; i++)
	//while (objRep = [objEnum nextObject])
	{
		objRep = [fObjectArray objectAtIndex:i];
		if ([objRep isKindOfClass:[MITextClipRep class]])
			return TRUE;
	}
	
	return FALSE;
}

- (BOOL)containsOnlyTextClips
{
	NSEnumerator*	objEnum = [fObjectArray objectEnumerator];
	MIObjectRep*	objRep;
	while (objRep = [objEnum nextObject])
	{
		if (![objRep isKindOfClass:[MITextClipRep class]])
			return FALSE;
	}
	
	return TRUE;
}

- (BOOL)containsOnlyAudio
{
	NSEnumerator*	objEnum = [fObjectArray objectEnumerator];
	MIObjectRep*	objRep;
	while (objRep = [objEnum nextObject])
	{
		if ([[objRep representedObject] WSObject]->ContainsVideo(false, true))
			return FALSE;
	}
	
	return TRUE;
}

- (id)discardTextClips
{
	NSEnumerator*	objEnum = [fObjectArray objectEnumerator];
	MIObjectRep*	objRep;
	while (objRep = [objEnum nextObject])
	{
		if ([objRep isKindOfClass:[MITextClipRep class]])
		{
			//remove the text clip from selection
			[fObjectArray removeObject:objRep];
			//and restart enumerating at the beginning of the array (don't know what it does if I delete the object that is pointed by the enumerator)
			objEnum = [fObjectArray objectEnumerator];
		}
	}
	return self;
}

- (id)discardVideoClips
{
	NSEnumerator*	objEnum = [fObjectArray objectEnumerator];
	MIObjectRep*	objRep;
	while (objRep = [objEnum nextObject])
	{
		if ([objRep isKindOfClass:[MIVideoClipRep class]])
		{
			//remove the videoclip
			[fObjectArray removeObject:objRep];
			//and restart enumerating at the beginning of the array (don't know what it does if I delete the object that is pointed by the enumerator)
			objEnum = [fObjectArray objectEnumerator];
		}
	}
	return self;
}

- (id)initWithExistingSelection:(MIWSSelection*)selection
{
	[self initEmpty];
	fType = [selection type];
	fObjectArray = [[NSMutableArray alloc] initWithArray:selection->fObjectArray];
	fStartTime = [selection startTime];
	fEndxTime = [selection endxTime];
	return self;
}

- (void)dealloc
{
	[fObjectArray release];
    [super dealloc];
}

- (id)copyWithZone:(NSZone*)zone
{
	MIWSSelection *copy = [[MIWSSelection alloc] initWithExistingSelection:self];

	return copy;
}

- (NSArray*)objArray
{
	return [[fObjectArray copy] autorelease];
}

- (NSArray*)putRepsThatContainAudioInArrayAndRemoveDescendants
{
	NSArray* flattened = [self putRepsInArrayAndRemoveDescendants];
	NSMutableArray* audioFlattened = [[[NSMutableArray alloc] init] autorelease];
	for(UInt32 i = 0; i < [flattened count]; i++)
	{
		MIObjectRep* rep = [flattened objectAtIndex:i];
		if([[rep representedObject] WSObject]->ContainsAudio())
			[audioFlattened addObject:rep];
	}
	return audioFlattened;
}

- (NSArray*)putRepsInArrayAndRemoveDescendants
{
	//ZzAssert(fType == kMIWSSelectionTypeObjects);
	// we copy the selection in another array
	NSMutableArray* array = [[NSMutableArray alloc] initWithArray:fObjectArray];
	NSMutableArray* objectsToRemove = [[NSMutableArray alloc] init];
	NSEnumerator* enumerator = [array objectEnumerator];
	MIObjectRep* obj;
	while(obj = [enumerator nextObject])
	{
		// if this object is the descendant of another one in the selection, let's remove it...
		NSEnumerator* selEnumerator = [fObjectArray objectEnumerator];
		MIObjectRep* selObj;
		while(selObj = [selEnumerator nextObject])
		{
			if([[obj representedObject] isDescendantOf:[selObj representedObject]] && ![objectsToRemove containsObject:obj])
				[objectsToRemove addObject:obj];
		}
//		[selEnumerator release];
	}
//	[enumerator release];
	NSEnumerator* objectsToRemoveEnumerator = [objectsToRemove objectEnumerator];
	MIObject* objToRemove;
	while(objToRemove = [objectsToRemoveEnumerator nextObject])
	{
		[array removeObject:objToRemove];
	}
//	[objectsToRemoveEnumerator release];
	[objectsToRemove release];
	return [array autorelease];
}

- (NSEnumerator*)objectEnumerator
{
	return [fObjectArray objectEnumerator];
}

- (void)addObject:(MIObjectRep*)rep
{
	if ( [fObjectArray containsObject:rep] )
		return;

    [fObjectArray addObject:rep];
	[rep setSelected:YES];
	[rep setNeedsDisplay];
    [[NSNotificationCenter defaultCenter] postNotificationName:MTGSelectionChangedNotification object:self];
}

- (void)removeObject:(MIObjectRep*)rep
{
	[fObjectArray removeObject:rep];
	[rep setSelected:NO];
	[rep setNeedsDisplay];
	
	[[NSNotificationCenter defaultCenter] postNotificationName:MTGSelectionChangedNotification object:self];
}

- (unsigned)count
{
    return [fObjectArray count];
}

- (MIObjectRep*)objectAtIndex:(unsigned)index
{
    return [fObjectArray objectAtIndex:index];
}

- (ArmeTime)startTime
{
	if (fStartTime < fEndxTime)
		return fStartTime;
	else
		return fEndxTime;
}

- (ArmeTime)endxTime
{
	if (fStartTime < fEndxTime)
		return fEndxTime;
	else
		return fStartTime;
}

- (ArmeTime)length
{
	ArmeTime ret;
	if (fType == kMIWSSelectionTypeTimeSlice)
	{
		ret = [self endxTime] - [self startTime];
	}
	else
	{
		ret = [[self getRightMostObject] WSObject]->GetAbsoluteEndxTime() - [[self getLeftMostObject] WSObject]->GetAbsoluteStartTime();
	}
	return ret;
}

- (SInt32)type
{
	return fType;
}

- (id)initEmpty
{
	[super init];
	fObjectArray = [[NSMutableArray alloc] initWithObjects:nil];
	fType = kMIWSSelectionTypeObjects;
	return self;
}

- (void)clear
{
	if ( [fObjectArray count] == 0 )
		return;
	
	MIObjectRep		*selObj;
	//NSEnumerator	*selEnumerator = [fObjectArray objectEnumerator];
	
	for(UInt32 i = 0; i < [fObjectArray count]; i++)
	//while(selObj = [selEnumerator nextObject])
	{
		selObj = [fObjectArray objectAtIndex:i];
		[selObj setSelected:NO];
		[selObj setNeedsDisplay];
	}
	
    [fObjectArray removeAllObjects];
    [[NSNotificationCenter defaultCenter] postNotificationName:MTGSelectionChangedNotification object:self];
}

- (unsigned)indexOfObject:(id)anObject
{
	return [fObjectArray indexOfObject:anObject];
}

- (void)setTimeSliceType
{
    // this clears the array if type changes
    if(fType != kMIWSSelectionTypeTimeSlice)
    {
        [fObjectArray removeAllObjects];
        fType = kMIWSSelectionTypeTimeSlice;
        [[NSNotificationCenter defaultCenter] postNotificationName:MTGSelectionChangedNotification object:self];
    }
}

- (void)setObjectsType
{
    // this clears the array if type changes
    if(fType != kMIWSSelectionTypeObjects)
    {
        [fObjectArray removeAllObjects];
        fType = kMIWSSelectionTypeObjects;
        [[NSNotificationCenter defaultCenter] postNotificationName:MTGSelectionChangedNotification object:self];
    }
}

- (void)setStartTime:(ArmeTime)startTime
{
    fStartTime = startTime;
    [[NSNotificationCenter defaultCenter] postNotificationName:MTGSelectionChangedNotification object:self];
}

- (void)setEndxTime:(ArmeTime)endxTime
{
    fEndxTime = endxTime;
    [[NSNotificationCenter defaultCenter] postNotificationName:MTGSelectionChangedNotification object:self];
}

- (MIObject*)getLeftMostObject
{
	//NSEnumerator* 	selEnumerator = [fObjectArray objectEnumerator];
	MIObjectRep* 	selObj;
	ArmeTime 		leftMostStartTime = ARME_TIME_MAX;
	MIObject*		leftMostObject = nil;

	// no sense to call this function if there are no object in the selection
	ZzAssert([fObjectArray count] > 0);

	for(UInt32 i = 0; i < [fObjectArray count]; i++)
	//while(selObj = [selEnumerator nextObject])
	{
		selObj = [fObjectArray objectAtIndex:i];
		if ([[selObj representedObject] WSObject]->GetAbsoluteStartTime() < leftMostStartTime)
		{
			leftMostObject = [selObj representedObject];
			leftMostStartTime = [leftMostObject WSObject]->GetAbsoluteStartTime();
		}
	}
	
	return leftMostObject;
}

- (MIObject*)getRightMostObject
{
	NSEnumerator* 	selEnumerator = [fObjectArray objectEnumerator];
	MIObjectRep* 	selObj;
	ArmeTime 		rightMostEndTime = ARME_TIME_MIN;
	MIObject*		rightMostObject = nil;

	// no sense to call this function if there are no object in the selection
	ZzAssert([fObjectArray count] > 0);

	while(selObj = [selEnumerator nextObject])
	{
		if ([[selObj representedObject] WSObject]->GetAbsoluteEndxTime() > rightMostEndTime)
		{
			rightMostObject = [selObj representedObject];
			rightMostEndTime = [rightMostObject WSObject]->GetAbsoluteEndxTime();
		}
	}
	
	return rightMostObject;
}

- (ArmeTime)duration
{
	ArmeTime duration =  _ArmeTime(GET_VAL(fEndxTime) - GET_VAL(fStartTime));
	if(GET_VAL(duration) < 0)
		GET_VAL(duration) = -GET_VAL(duration);
	return duration;
}

- (void)adjustSelectionTimesToVideo
{
	if ([self containsVideo:true andTransparentTextClip:YES] && (fType == kMIWSSelectionTypeTimeSlice))
	{
		TArmeDoc* doc = [[[fObjectArray objectAtIndex:0] representedObject] WSObject]->GetDoc();
		[self setStartTime:doc->RoundArmeTimeToNearestVideoFrame(fStartTime)];
		[self setEndxTime:doc->RoundArmeTimeToNearestVideoFrame(fEndxTime)];	
	}
}


@end
