/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
#import "MontageInterface.h"
#import "MIDockBar.h"
#import "MIObject.h"
#import "MIObjectRep.h"

#ifndef MEMixingInfo_h
#include "MEMixingInfo.h"
#endif //MEMixingInfo_h

#ifndef TArmeWSObj_h
#include "TArmeWSObj.h"
#endif //TArmeWSObj_h

#if defined(__MWERKS__)
inline NSRect BenjaSSCenteredRectInRect(NSRect innerRect, NSRect outerRect)
{
    innerRect.origin.x = floor((outerRect.size.width - innerRect.size.width) / 2);
    innerRect.origin.y = floor((outerRect.size.height - innerRect.size.height) / 2);
    return innerRect;
}
#else //#if defined(__MWERKS__)
#include <ScreenSaver/ScreenSaverView.h>
#endif //#if defined(__MWERKS__)

#import "ArmeBasicTypes.h"

//const int 						kTimeUnitPerPixel = 256;
const int 							kVUnitPerPixel	= 1;
#if !defined(__MWERKS__)
const NSSize 						kCacheingMinSize = NSMakeSize(1500,512);
#endif //#if !defined(__MWERKS__)
const float							kCacheingHorizontalExtra = 0;
const float							kCacheingVerticalExtra = 0;
NSColor								*selectedClipHighlightColor;
NSColor								*selectedProcessorHighlightColor;
MIImageController					*mainImageController;


const MISize MIZeroSize = {_ArmeTime(0), _MIVPos(0)};
const MIPoint MIZeroPoint = {_ArmeTime(0), _MIVPos(0)};
const MIRect MIZeroRect = { MIZeroPoint, MIZeroSize };

MIRect MIMakeRect(ArmeTime x, MIVPos y, ArmeTime w, MIVPos h)
{
    MIRect r;
    r.origin.time = x;
    r.origin.vpos = y;
    r.size.duration = w;
    r.size.height = h;
    return r;
}

MIPoint	MIMakePoint(ArmeTime x, MIVPos y)
{
	MIPoint p;
	p.time = x;
	p.vpos = y;
	return p;
}

MISize	MIMakeSize(ArmeTime length, MIVPos height)
{
	MISize s;
	s.duration = length;
	s.height = height;
	
	return s;
}

ArmeTime MIMaxX(MIRect aRect)
{
	ArmeTime t;
	GET_VALUE(t) = GET_VALUE(aRect.origin.time) + GET_VALUE(aRect.size.duration);
    return t;
}

MIVPos MIMaxY(MIRect aRect)
{
	MIVPos pos;
	GET_VALUE(pos) = GET_VALUE(aRect.origin.vpos) + GET_VALUE(aRect.size.height);
    return pos;
}

ArmeTime MIMinX(MIRect aRect)
{
    return (aRect.origin.time);
}

MIVPos MIMinY(MIRect aRect)
{
    return (aRect.origin.vpos);
}

ArmeTime MIWidth(MIRect aRect)
{
    return (aRect.size.duration);
}

MIVPos MIHeight(MIRect aRect)
{
    return (aRect.size.height);
}


BOOL MIContainsRect( MIRect rect1, MIRect rect2 )
{
	/// contains or is equal

	if ( GET_VALUE(rect1.origin.time) <= GET_VALUE(rect2.origin.time) && // origin of rect1 is lower than origin of rect 2
	  GET_VALUE(rect1.origin.vpos) <= GET_VALUE(rect2.origin.vpos) &&
	  // maxX and maxY of rect1 are greater than maxX and maxY of rect2
	  (GET_VALUE(rect1.origin.time) + GET_VALUE(rect1.size.duration) >= GET_VALUE(rect2.origin.time) + GET_VALUE(rect2.size.duration) ) &&
	  (GET_VALUE(rect1.origin.vpos) + GET_VALUE(rect1.size.height) >= GET_VALUE(rect2.origin.vpos) + GET_VALUE(rect2.size.height) ) )
		return YES;
	else
		return NO;
}

BOOL MIEqualRects( MIRect rect1, MIRect rect2 )
{
	if ( !MIEqualPoints(rect1.origin, rect2.origin) ||
		 !MIEqualSizes(rect1.size, rect2.size) )
		return NO;
	else
		return YES;
}

BOOL MIEqualPoints( MIPoint p1, MIPoint p2)
{
	if ( GET_VALUE(p1.time) != GET_VALUE(p2.time) ||
	  GET_VALUE(p1.vpos) != GET_VALUE(p2.vpos) )
		return NO;
	else
		return YES;
}

BOOL MIEqualSizes( MISize s1, MISize s2 )
{
	if ( GET_VALUE(s1.height) != GET_VALUE(s2.height) ||
	  GET_VALUE(s1.duration) != GET_VALUE(s2.duration) )
		return NO;
	else
		return YES;
}


MIRect MIUnionRect( MIRect rect1, MIRect rect2 )
{
	MIRect ur;

	// if either is NULL, return the other one
	if ( !GET_VALUE(rect1.size.duration) && !GET_VALUE(rect1.size.height) )
	{ 
		ur = rect2;
	}
	else if ( !GET_VALUE(rect2.size.duration) && !GET_VALUE(rect2.size.height) )
	{
		ur = rect1;
	}
	else
	{
		// take lesser origin
		ur.origin.time = (GET_VALUE(rect1.origin.time) < GET_VALUE( rect2.origin.time) ) ? rect1.origin.time : rect2.origin.time;
		ur.origin.vpos = (GET_VALUE(rect1.origin.vpos) < GET_VALUE( rect2.origin.vpos) ) ? rect1.origin.vpos : rect2.origin.vpos;
		// check for the greater maxX and set the width to that maxX - origin
		GET_VALUE(ur.size.duration) = (GET_VALUE(rect1.origin.time) + GET_VALUE(rect1.size.duration) > GET_VALUE( rect2.origin.time) + GET_VALUE(rect2.size.duration) )
			? (GET_VALUE(rect1.origin.time) + GET_VALUE(rect1.size.duration)) - GET_VALUE( ur.origin.time)
			: (GET_VALUE(rect2.origin.time) + GET_VALUE(rect2.size.duration)) - GET_VALUE( ur.origin.time);
		GET_VALUE(ur.size.height) = (GET_VALUE(rect1.origin.vpos) + GET_VALUE(rect1.size.height) > GET_VALUE( rect2.origin.vpos) + GET_VALUE(rect2.size.height) )
			? (GET_VALUE(rect1.origin.vpos) + GET_VALUE(rect1.size.height)) - GET_VALUE( ur.origin.vpos)
			: (GET_VALUE(rect2.origin.vpos) + GET_VALUE(rect2.size.height)) - GET_VALUE( ur.origin.vpos);
	}

	return ur;
}

MIPoint TranslateMIPoint( MIPoint p1, MIPoint p2 )
{
	MIPoint p;
	
	GET_VALUE(p.time) = GET_VALUE(p1.time) + GET_VALUE(p2.time);
	GET_VALUE(p.vpos) = GET_VALUE(p1.vpos) + GET_VALUE(p2.vpos);
	
	return p;
}

NSRect ClipNSRect( NSRect rect, NSRect clip ) {
	if ( NSMinX( rect ) < NSMinX( clip ) ) rect.origin.x = NSMinX( clip );
	if ( NSMinY( rect ) < NSMinY( clip ) ) rect.origin.x = NSMinX( clip );
	if ( NSMaxX( rect ) > NSMaxX( clip ) ) rect.size.width = NSMaxX( clip ) - NSMinX( rect );
	if ( NSMaxY( rect ) > NSMaxY( clip ) ) rect.size.height = NSMaxY( clip ) - NSMinY( rect );
	return rect;
}

NSRect TranslateNSRect( NSRect rect, NSPoint vector )
{
  rect.origin.x += vector.x;
  rect.origin.y += vector.y;
  
  return rect;
}

NSRect ScaleNSRect(NSRect rect, float hZoom, float vZoom )
{
  rect.origin.x *= hZoom;
  rect.size.width *= hZoom;
  rect.origin.y *= vZoom;
  rect.size.height *= vZoom;
  
  return rect;
}


NSPoint TranslateNSPoint( NSPoint origin, NSPoint vector )
{
  origin.x += vector.x;
  origin.y += vector.y;
  
  return origin;
}

NSRect RectWithPoints(NSPoint point1, NSPoint point2)
{
	NSRect newRect;
	
	newRect.origin.x = min(point1.x, point2.x);
	newRect.origin.y = min(point1.y, point2.y);  
	newRect.size.width = fabs(point1.x - point2.x );
	newRect.size.height = fabs(point1.y - point2.y );
	
	return newRect;
}

Rect NSRect2Rect(const NSRect& inNSRect)
{
    Rect result;
    ::SetRect( &result, (short)inNSRect.origin.x, (short)inNSRect.origin.y,
            (short)inNSRect.origin.x + (short)inNSRect.size.width,
            (short)inNSRect.origin.y + (short)inNSRect.size.height );
    return result;
}

NSRect MapNSRect(const NSRect& nsr, const NSRect& srcNSRect, const NSRect& dstNSRect)
{
    Rect r = NSRect2Rect(nsr);
    Rect srcRect = NSRect2Rect(srcNSRect);
    Rect dstRect = NSRect2Rect(dstNSRect);
    
    ::MapRect(&r, &srcRect, &dstRect);
    
    return NSMakeRect(r.left, r.top, r.right - r.left, r.bottom - r.top);
}

NSRect ScaleNSRectToNSRect(const NSRect& inSourceRect, const NSRect& inDestRect, bool inCenter)
{
    // make a Rect to output, same as the source
    Rect outRect = NSRect2Rect(inSourceRect);

    // make a Rect to map from
    Rect mapFromRect = outRect;
    
    // fix the ratio of the destination
    Rect mapToRect = NSRect2Rect(inDestRect);
    double xRatio = inDestRect.size.width / inSourceRect.size.width;
    double yRatio = inDestRect.size.height / inSourceRect.size.height;
    if (xRatio > yRatio)
        mapToRect.right = mapToRect.left + (short)(yRatio * inSourceRect.size.width);
    else
        mapToRect.bottom = mapToRect.top + (short)(xRatio * inSourceRect.size.height);
    
    // and map to ratio-corrected destination
    ::MapRect(&outRect, &mapFromRect, &mapToRect);

    NSRect result = NSMakeRect(outRect.left, outRect.top, outRect.right - outRect.left, outRect.bottom - outRect.top);
#if defined(__MWERKS__)
    if (inCenter)
        result = BenjaSSCenteredRectInRect(result, inDestRect);
#else //#if defined(__MWERKS__)
    if (inCenter)
        result = SSCenteredRectInRect(result, inDestRect);
#endif //#if defined(__MWERKS__)

    return result;
}

id CreatePalette( NSPoint position, NSRect dockableItemSize, NSString *title )
{
	NSRect		contentRect;
	NSPanel 	*newWindow;
	MIDockBar	*newDockBar;
	
	contentRect = dockableItemSize;
	contentRect.size.width += 16;
	contentRect.size.height += 16;

	// create a new window (a panel)
	newWindow = [[NSPanel alloc] initWithContentRect:contentRect
				styleMask:NSResizableWindowMask|NSClosableWindowMask|NSTitledWindowMask|NSUtilityWindowMask
				backing:NSBackingStoreBuffered
				defer:YES];
	[newWindow setTitle:title];
	[newWindow setFrameOrigin:position];
	[newWindow setFloatingPanel:YES];
	[newWindow setReleasedWhenClosed:YES]; // non standard behaviour for panels, but these are not static panels

	// Create a new DockBar to set as the contentView of the newly created panel
	newDockBar = [[MIDockBar alloc] initWithFrame:contentRect];
	[newDockBar setIsInMainView:NO];
	[newWindow setContentView:newDockBar];
	
	[newWindow makeKeyAndOrderFront:nil];
	[newDockBar setNeedsDisplay:YES];

	return newWindow;
}

id CreateNewClipSheetWindow( NSPoint position, NSString *title )
{
	NSWindow					*newWindow;
	
	newWindow = [[NSWindow alloc] 	initWithContentRect:NSMakeRect(0,0,400,400)
									styleMask:NSResizableWindowMask|NSClosableWindowMask|NSTitledWindowMask
									backing:NSBackingStoreBuffered
									defer:YES];
	return newWindow;
}

BOOL IsNSPointInRect(NSPoint point, NSRect rect)
{
	if ( point.x >= NSMinX(rect) &&
	     point.x <= NSMaxX(rect) &&
		 point.y >= NSMinY(rect) &&
	     point.y <= NSMaxY(rect) )
		return YES;
	
	return NO;
}

ArmeTime MIArmeTimeFromString(NSString* time)
{
	const char* cString = [time cString];
	SInt64 num = strtoll(cString, (char **)NULL, 10);
	return _ArmeTime(num);
}

NSString* MIStringFromArmeTime(ArmeTime time)
{
	char buffer[256];
	sprintf( buffer, "%lld", GET_VAL(time) );
	return [NSString stringWithCString:buffer];
}

NSColor *NegativeOfColor( NSColor *color)
{
	NSColor *newColor;
	NSColor* rgbcolor = [color colorUsingColorSpaceName:NSDeviceRGBColorSpace];
	
	if ( !color )
		newColor = [NSColor whiteColor];
	else
		newColor = [NSColor colorWithCalibratedRed: 1.0 - [rgbcolor redComponent]
													green: 1.0 - [rgbcolor greenComponent]
													blue: 1.0 - [rgbcolor blueComponent]
													alpha: [rgbcolor alphaComponent]];
	
	return newColor;
}

MEVAORouting MIGetSelectionVAORouting(MIWSSelection* selection)
{
	// the rule is as follows : 

	// if all the objects in the selection have the same vao routing,
	// we return that one

	// if all the objects are mono OR stereo, then the result is the same
	// (mono OR stereo) and has the lowest index of the selection

	// if there are stereo and mono objects, then the result is stereo and
	// has the lowest index of the selection
	
	NSArray* selArray = [selection putRepsInArrayAndRemoveDescendants];
	
	bool containsStereoVao = false;
	SInt32 minimalStereoVAOIndex = LONG_MAX;
	bool containsMonoVao = false;
	SInt32 minimalMonoVAOIndex = LONG_MAX;
	
	for(UInt32 i = 0; i < [selArray count]; i++)
	{
		MIObjectRep* rep = [selArray objectAtIndex:i];
		MIObject* miobj = [rep representedObject];
		TArmeWSObj* wsobj = [miobj WSObject];
		const MEVAORouting& routing = wsobj->GetVAORouting();
		if(routing.bStereo)
		{
			containsStereoVao = true;
			if(routing.index < minimalStereoVAOIndex)
				minimalStereoVAOIndex = routing.index;
		}
		else
		{
			containsMonoVao = true;
			if(routing.index < minimalMonoVAOIndex)
				minimalMonoVAOIndex = routing.index;
		}
	}
	MEVAORouting result;
	if(containsStereoVao)
	{
		result.bStereo = true;
		result.index = minimalStereoVAOIndex;
	}
	else
	{
		if(containsMonoVao)
		{
			result.bStereo = false;
			result.index = minimalMonoVAOIndex;
		}
		else
		{
			result.bStereo = true;
			result.index = 0;
		}
	}
	return result;
}

