/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
#import <Cocoa/Cocoa.h>
#import "MICommand.h"

#ifndef ZzArray_h
#include "ZzArray.h"
#endif //ZzArray_h

#ifndef MEAudioClipData_h
#include "MEAudioClipData.h"
#endif //MEAudioClipData_h

@class MIMainViewController;
class OfflineAudioProcessor;
class OfflineAudioProcessorParams;
class MEAudioClip;
@class MIAudioClipRep;
@class MIMacroCommand;

@interface MIAudioProcessingCommand : MICommand
{
	OfflineAudioProcessor*							fOPC;
	OfflineAudioProcessorParams*					fParams;
	ZzArray<ZzArray<bool>*>*						fModifiedFlagsArray;
	NSMutableArray*									fpModifiedMIObjList;
	ZzArchPtrArray*									fpOriginalMementosList;
	ZzArchPtrArray*									fpNewMementosList;
	MIMainViewController*							fController; // used only for doIt, to display a progress bar...
	MIMacroCommand*									fEnlargeParentsCommand;
}

-(id)initFromArchive:(ArmeArchiveHandler*)handler;
-(BOOL)writeToArchive:(ArmeArchiveHandler*)handler;

- (id)initWithSelection:(MIWSSelection*)sel offlineAudioProcessor:(OfflineAudioProcessor*)opc;
- (id)initWithSingleClipRepAndParams:	(MIAudioClipRep*)audioClipRep 
		offlineAudioProcessor:			(OfflineAudioProcessor*)opc 
		params:							(OfflineAudioProcessorParams*)params;
- (void)dealloc;
- (BOOL)displaySetupDialog;
- (MICmdStatus)doIt;
- (void)undoIt;
- (void)redoIt;
// this function allows to get access to the processor params for special customization
- (OfflineAudioProcessorParams*)processorParams;

// this function tell ALL THE TLC REPS to redraw in a given model rect
- (void)setTLCRepsNeedDisplayInModelRect:(MIRect)modelRect;

// this function computes the union of the fpModifiedMIObjList model frames at a given time
- (MIRect)getUnionModelRect;

// this function tells the audio clip its stored image is not valid anymore.
// it also tells the rep to recompute its frame according to the new object
// data and it also tells the object to tell its reps to redisplay themselves
- (void)invalidateCachedImagesAndInvalidateRepAndSetDisplay;

- (void)setObjectsRepsNeedDisplay;

/*! @function closeAndDeleteNewlyWrittenMTGAFiles closes then deletes the partially or totally written MTGA files*/
-(void)closeAndDeleteNewlyWrittenMTGAFiles;

/*! @function applyOriginalMementoWithoutRefresh resets the data to its orig state. this can be done before new preview computing has started */
-(void)applyOriginalMementoWithoutRefresh;

/*! @function replaceRangesWithChunks actually modifies the MEAudioClipData instances */
-(void)replaceRangesWithChunks;

/*! @function deleteNewlyWrittenMTGAFiles deletes the partially or totally written MTGA files */
-(void)deleteNewlyWrittenMTGAFiles;

@end
