/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
//

#import "MIWSSelection.h"
#import "MIAudioProcessingCommand.h"
#import "MIMainViewController.h"
#import "OfflineAudioProcessing.h"
#import "MIApplicationController.h"
#import "MIObject.h"
#import "MIObjectRep.h"
#import "MITopLevelContainerRep.h"
#import "MIMontageDocument.h"
#import "MIDocumentController.h"
#import "MIMacroCommand.h"
#import "MIResizeCommand.h"

#ifndef TArmeDoc_h
#include "TArmeDoc.h"
#endif //TArmeDoc_h

#ifndef TArmeWSObj_h
#include "TArmeWSObj.h"
#endif //TArmeWSObj_h

#ifdef TRACK_BASED_AUDIO_CLIPS

#ifndef MEAudioClip_h
#include "MEAudioClip.h"
#endif //MEAudioClip_h

#else //#ifdef TRACK_BASED_AUDIO_CLIPS

#ifndef TAudioClip2_h
#include "TAudioClip2.h"
#endif //TAudioClip2_h

#endif //#ifdef TRACK_BASED_AUDIO_CLIPS

#import "MIAudioClipRep.h"

#ifndef MESplitTaskAudioOfflineProc_h
#include "MESplitTaskAudioOfflineProc.h"
#endif //MESplitTaskAudioOfflineProc_h

#ifndef MESplitTaskComputeAudioClipDataPreview_h
#include "MESplitTaskComputeAudioClipDataPreview.h"
#endif //MESplitTaskComputeAudioClipDataPreview_h

#undef VER_DEF_TOPIC
#define VER_DEF_TOPIC offlineProcBenjamin

@implementation MIAudioProcessingCommand

-(id)initFromArchive:(ArmeArchiveHandler*)handler
{
	self = [super initFromArchive:handler];
	if(self)
	{
		fOPC = NULL; // we don't need it for undo redo
		fParams = NULL; // we don't need it for undo redo
		fController = NULL; // used only for doIt, to display a progress bar...
	
		/*fModifiedFlagsArray = new ZzArray<bool>;
		SInt32 count = handler->ReadSInt32();
		for(SInt32 i = 0; i < count; i++)
		{
			bool val = handler->ReadBool();
			fModifiedFlagsArray->AddItem(val);
		}*/
		
		// read the list of modified clips and the associated mementos
		fpModifiedMIObjList = [handler->ReadObjCObject("NSMutableArray") retain];
		ZzAssert(fpModifiedMIObjList);
		ArmeArchivable* archObj;
		archObj = handler->ReadCPPObject();
		fpOriginalMementosList = dynamic_cast<ZzArchPtrArray*>(archObj);
		ZzAssert(fpOriginalMementosList);
		archObj = handler->ReadCPPObject();
		fpNewMementosList = dynamic_cast<ZzArchPtrArray*>(archObj);
		ZzAssert(fpNewMementosList);
		
		fEnlargeParentsCommand = [handler->ReadObjCObject() retain];
	}
	[self modifiedAudioHierarchy];
	return self;
}
	
-(BOOL)writeToArchive:(ArmeArchiveHandler*)handler
{
	[super writeToArchive:handler];
	
/*	handler->WriteSInt32(fModifiedFlagsArray->CountItems()); 
	for(SInt32 i = 0; i < fModifiedFlagsArray->CountItems(); i++)
	{
		bool val = fModifiedFlagsArray->ItemAt(i);
		handler->WriteBool(val);
	}*/
	handler->WriteObjCObject(fpModifiedMIObjList);
	handler->WriteCPPObject(fpOriginalMementosList);
	handler->WriteCPPObject(fpNewMementosList);
	handler->WriteObjCObject(fEnlargeParentsCommand);
	return YES;
}

+ (BOOL)supportsSelection:(MIWSSelection*)sel
{
	if ( [sel type] == kMIWSSelectionTypeObjects
		 || [sel type] == kMIWSSelectionTypeTimeSlice
		 )
	{
		if ([sel count] == 0) 
			return NO;
		
		NSEnumerator* selEnum = [sel objectEnumerator];
		MIObject* obj;
		while (obj = [[selEnum nextObject] representedObject])
		{
			if (([obj WSObject]->GetType() != TArmeWSObj::kAudioClip))
				return NO;
		}
		return YES;
	}
	else
		return NO;
}

- (id)initWithSelection:(MIWSSelection*)sel offlineAudioProcessor:(OfflineAudioProcessor*)opc
{
	self = [super initWithSelection:sel];
	
	if ( self )
	{
		// create the arrays
		fModifiedFlagsArray = new ZzArray<ZzArray<bool>*>;
		fpModifiedMIObjList = [[NSMutableArray alloc] init];
		fEnlargeParentsCommand = [[MIMacroCommand alloc] initWithSelection:nil];
		fpOriginalMementosList = new ZzArchPtrArray;
		fpNewMementosList = new ZzArchPtrArray;
		
		// save the processor, because we need it later
		fOPC = opc;
		
		// create default params for this processing command
		fParams = opc->CreateDefaultParams();
			
		// let's enumerate all the selection objects.
		NSEnumerator* objEnum = [sel objectEnumerator];
		MIObjectRep* objectRep;
		while ( objectRep = [objEnum nextObject] )
		{
			MIObject* miobj = [objectRep representedObject];
			fModifiedFlagsArray->AddItem([miobj getCurrentModifiedParentsFlag]);
			TArmeWSObj* wsobj = [miobj WSObject];
			// if it's an audio clip, then we add a reader and writer object
			// to the processing parameters
			if(wsobj->GetType() == TArmeWSObj::kAudioClip)
			{
				ArmeTime	startTime;
				ArmeTime	endxTime;
				if([sel type] == kMIWSSelectionTypeTimeSlice)
				{
					// it's a time slice selection. so probably
					// only a part of the object is selected
					// PLEASE NOTE we specify the time values
					// relative to each clip
					startTime = 
						_ArmeTime(GET_VAL([sel startTime]) - GET_VAL(wsobj->GetAbsoluteStartTime()));
					endxTime = 
						_ArmeTime(GET_VAL([sel endxTime]) - GET_VAL(wsobj->GetAbsoluteStartTime()));
				}
				else
				{
					// the whole object is selected
					startTime = _ArmeTime(0);
					endxTime = wsobj->GetDuration();
				}
				// add a reader and a writer for this processing
				
				// first, we rearrange selection slice times so that
				// the greatest time is endxTime
				if(GET_VAL(endxTime) < GET_VAL(startTime))
				{
					ArmeTime tmp = startTime;
					startTime = endxTime;
					endxTime = tmp;
				}
				MEAudioClip* _wsobj = dynamic_cast<MEAudioClip*>(wsobj);
				
				[fpModifiedMIObjList addObject:miobj];
				
				ZzAssert(_wsobj);
				fParams->AdoptReader(new OfflineAudioClipReader(
					(MEAudioClip*)wsobj,startTime,_ArmeTime(GET_VAL(endxTime)-GET_VAL(startTime))));
				fParams->AdoptWriter(new OfflineAudioClipWriter(
					(MEAudioClip*)wsobj,startTime,_ArmeTime(GET_VAL(endxTime)-GET_VAL(startTime))));
			}
		}
		if ([sel containsAudio])
			[self modifiedAudioHierarchy];
	}
	return self;
}

- (OfflineAudioProcessorParams*)processorParams
{
	return fParams;
}

- (id)initWithSingleClipRepAndParams:	(MIAudioClipRep*)audioClipRep 
		offlineAudioProcessor:			(OfflineAudioProcessor*)opc 
		params:							(OfflineAudioProcessorParams*)params
{
	self = [super initWithSelection:nil];
	
	if(self)
	{
		fModifiedFlagsArray = new ZzArray<ZzArray<bool>*>;
		fpModifiedMIObjList = [[NSMutableArray alloc] init];
		fEnlargeParentsCommand = [[MIMacroCommand alloc] initWithSelection:nil];

		fpOriginalMementosList = new ZzArchPtrArray;
		fpNewMementosList = new ZzArchPtrArray;
		
		MIObject* miobj = [audioClipRep representedObject];
		fModifiedFlagsArray->AddItem([miobj getCurrentModifiedParentsFlag]);

		TArmeWSObj* baseModifiedClip = [miobj WSObject];
		MEAudioClip* modifiedClip = dynamic_cast<MEAudioClip*>(baseModifiedClip);
	
		ZzAssert(modifiedClip != NULL);
	
		// save the processor, because we need it later
		fOPC = opc;
		
		// use the supplied parameters
		fParams = params;
			
		if(modifiedClip->GetType() == TArmeWSObj::kAudioClip)
		{
			ArmeTime startTime = _ArmeTime(0);;
			ArmeTime endxTime = modifiedClip->GetDuration();
			// add a reader and a writer for this processing
			
			// first, we rearrange selection slice times so that
			// the greatest time is endxTime
			if(GET_VAL(endxTime) < GET_VAL(startTime))
			{
				ArmeTime tmp = startTime;
				startTime = endxTime;
				endxTime = tmp;
			}
			
			[fpModifiedMIObjList addObject:miobj];
			
			fParams->AdoptReader(new OfflineAudioClipReader(
				modifiedClip,startTime,_ArmeTime(GET_VAL(endxTime)-GET_VAL(startTime))));
			fParams->AdoptWriter(new OfflineAudioClipWriter(
				modifiedClip,startTime,_ArmeTime(GET_VAL(endxTime)-GET_VAL(startTime))));
		}
	}
	return self;
}


- (BOOL)displaySetupDialog
{
	if ([self CheckFrozenObjectInSelectionAndAskIfUserWantsToContinue:fSelection] == NO)
		return NO;

	BOOL ok = fOPC->DisplayAudioProcessorConfiguration(fParams);
	return ok;
}

- (MICmdStatus)doIt
{
	UInt32 i;
	// let's save the clips data mementos for the undo action and save the old
	// rect to schedule a redisplay later.
	NSRect firstRect = NSZeroRect;

	// compute the old model frame of the objects
	MIRect oldModelFrameUnion = [self getUnionModelRect];
	
	// save the mementos for the undo operation
	for(i = 0; i < [fpModifiedMIObjList count]; i++)
	{
		MIObject* miobj = [fpModifiedMIObjList objectAtIndex:i];
		[miobj markModified];
		MEAudioClip* clip = dynamic_cast<MEAudioClip*>([miobj WSObject]);
		ZzAssert(clip);
		fpOriginalMementosList->AddItem(clip->GetData()->CreateMemento());
	}
	
	// this executs the offlineproc, which computes a new chunk file
	if(fOPC->IsLengthy(fParams))
	{
		MESplitTaskAudioOfflineProc* splitTask = NULL;
		try
		{
			// this call allows the user to watch the operation progress and 
			splitTask = new MESplitTaskAudioOfflineProc(fOPC,fParams);
			BOOL notCancelled = [[MIApplicationController sharedApplicationController] executeAndDeleteSplitTask:splitTask forMainViewController:fController];
			if(!notCancelled)
			{
				ZZ_THROW(ZzException);
			}
		}
		catch(...)
		{
			// first : tell the thread to stop processing. it doesn't do anything more clever than
			// simply stop writing in the files.
			splitTask->Cancel();
			
			// some files may have been totally created already.
			// we'll simply close them all and delete them all.
			[self closeAndDeleteNewlyWrittenMTGAFiles];
			
			delete splitTask;
			
			[[MIApplicationController sharedApplicationController] redrawAllMainWindows];
			
			// now we need to delete the file that has been partially written, then delete the fOPC itself
			throw;
		}
	}
	else
	{
		try
		{
			fOPC->PerformAudioProcessing(fParams);
		}
		catch(...)
		{
			[self closeAndDeleteNewlyWrittenMTGAFiles]; // although they will be automatically deleted since there are no reference, we delete them
			[[MIApplicationController sharedApplicationController] redrawAllMainWindows];
			throw;
		}
	}
	
	try
	{
		[self replaceRangesWithChunks]; // WARNING : this closes the writer files. THROW RISK : HIGH !
	}
	catch(...)
	{
		// we don't need to delete the files...
		[self applyOriginalMementoWithoutRefresh];
		[self deleteNewlyWrittenMTGAFiles]; // although they will be automatically deleted since there are no reference, we delete them
		
		[[MIApplicationController sharedApplicationController] redrawAllMainWindows];
		throw;
	}

	fParams->SetProgressRange(0.0,1.0);
	fParams->SetProgress(1.1);
	
	// let's save the clips data mementos for the undo action
	
	// first tell the OLD model frame to redraw in all the TLC reps for the objects' doc
	[self setTLCRepsNeedDisplayInModelRect:oldModelFrameUnion];
							
	MIObject *parent;
	for(i = 0; i < [fpModifiedMIObjList count]; i++)
	{
		MIObject* miobj = [fpModifiedMIObjList objectAtIndex:i];
		MEAudioClip* clip = dynamic_cast<MEAudioClip*>([miobj WSObject]);
		ZzAssert(clip);
		// save the modified mementos for the redo action
		fpNewMementosList->AddItem(clip->GetData()->CreateMemento());
		
		// check if parent needs to be resized and 
		// create an enlarging command in case it's needed
		parent = [miobj parentObject];
		if ( parent && !MIContainsRect( [parent modelFrame], [parent minimalRectToFitContent] ) )
		{
			MIResizeCommand *cmd;
			cmd = [[[MIResizeCommand alloc] initWithSelection:nil] autorelease];
			[cmd setObject:parent newFrame:MIUnionRect([parent modelFrame], [parent minimalRectToFitContent])];
			[fEnlargeParentsCommand addCommand:cmd];
		}
	}
	if ( fEnlargeParentsCommand )
		[fEnlargeParentsCommand _doIt];
		
	// now tell the objects to invalidate the cached image and display themselves..
	// since the cached image is destroyed, a new image is computed from the actual
	// data....
	[self invalidateCachedImagesAndInvalidateRepAndSetDisplay];

	return kMICmdStatusDone;
}

- (void)setTLCRepsNeedDisplayInModelRect:(MIRect)modelRect
{
	MIObject* tlc = [[[fpModifiedMIObjList objectAtIndex:0] montageDoc] topLevelContainer];
	NSEnumerator* tlcRepEnum = [[tlc getRepresentations] objectEnumerator];
	MITopLevelContainerRep* tlcRep;
	while(tlcRep = [tlcRepEnum nextObject])
	{
		NSRect visRect = [tlcRep MIRectToNSRect:modelRect];
		[tlcRep setNeedsDisplayInRect:visRect];
	}
}

- (MIRect)getUnionModelRect
{
	MIRect unionRect = MIZeroRect;
	for(unsigned int i = 0; i < [fpModifiedMIObjList count]; i++)
	{
		MIObject* miobj = [fpModifiedMIObjList objectAtIndex:i];
		
		if ( !MIEqualRects( MIZeroRect, unionRect ) )
			unionRect = MIUnionRect(unionRect, [miobj absoluteModelFrame]);
		else
			unionRect = [miobj absoluteModelFrame];
	}
	return unionRect;
}
	
- (void)invalidateCachedImagesAndInvalidateRepAndSetDisplay
{
	for(UInt32 i = 0; i < [fpModifiedMIObjList count]; i++)
	{
		MIObject* miobj = [fpModifiedMIObjList objectAtIndex:i];
		[miobj invalidateRepresentationsCachedImages];
		[miobj invalidateRep];
		[miobj setRepresentationsNeedDisplay];
	}
}	

- (void)setObjectsRepsNeedDisplay
{
	for(UInt32 i = 0; i < [fpModifiedMIObjList count]; i++)
	{
		MIObject* miobj = [fpModifiedMIObjList objectAtIndex:i];
		[miobj setRepresentationsNeedDisplay];
	}
}	
	
- (void)undoIt
{
	// first, we compute the old model frame union for all the objects 
	MIRect oldModelFrameUnion = [self getUnionModelRect];

	// then we actually modify the objects. please note the splitTask that SetMemento 'returns' is the main preview
	// computation splittask for a given object
	ZzMacroSplitTask* macroSplitTask = new ZzMacroSplitTask;
	for(UInt32 i = 0; i < [fpModifiedMIObjList count]; i++)
	{
		MIObject* miobj = [fpModifiedMIObjList objectAtIndex:i];
		
//		BOOL wasMarkedModifiedBefore = ;
		[miobj resetModifiedFlag:fModifiedFlagsArray->ItemAt(i)];
/*		if(wasMarkedModifiedBefore)
			[miobj markModified];
		else
			[miobj cleanModifiedFlag];*/
		
		//[miobj markModified];
		[miobj setRepresentationsNeedDisplay];
		MEAudioClip* clip = dynamic_cast<MEAudioClip*>([miobj WSObject]);
		ZzAssert(clip);
		ZzSplitTask* splitTask;
		MEAudioClipDataMemento* memento = dynamic_cast<MEAudioClipDataMemento*>(fpOriginalMementosList->ItemAt(i));
		ZzAssert(memento);
		clip->GetData()->SetMemento(memento,&splitTask);
		macroSplitTask->AdoptSubTask(splitTask);
	}
	// execute the preview split task.
	BOOL notCancelled = [[MIApplicationController sharedApplicationController] executeAndDeleteSplitTask:macroSplitTask forMainViewController:nil];
	ZzAssert(notCancelled);
	
	[fEnlargeParentsCommand _undoIt];
	
	// now tell the tlc to redisplay itself in the old zone
	[self setTLCRepsNeedDisplayInModelRect:oldModelFrameUnion];
	
	// now tell the objects to redisplay themselves
	[self invalidateCachedImagesAndInvalidateRepAndSetDisplay];
		
}

- (void)redoIt
{
	// first, we compute the old model frame union for all the objects 
	MIRect oldModelFrameUnion = [self getUnionModelRect];
	
	// then we actually modify the objects. please note the splitTask that SetMemento 'returns' is the main preview
	// computation splittask for a given object
	ZzMacroSplitTask* macroSplitTask = new ZzMacroSplitTask;
	for(UInt32 i = 0; i < [fpModifiedMIObjList count]; i++)
	{
		MIObject* miobj = [fpModifiedMIObjList objectAtIndex:i];
		[miobj markModified];
		MEAudioClip* clip = dynamic_cast<MEAudioClip*>([miobj WSObject]);
		ZzAssert(clip);
		ZzSplitTask* splitTask;
		MEAudioClipDataMemento* memento = dynamic_cast<MEAudioClipDataMemento*>(fpNewMementosList->ItemAt(i));
		ZzAssert(memento);
		clip->GetData()->SetMemento(memento,&splitTask);
		macroSplitTask->AdoptSubTask(splitTask);
	}
	// execute the preview split task.
	BOOL notCancelled = [[MIApplicationController sharedApplicationController] executeAndDeleteSplitTask:macroSplitTask forMainViewController:nil];
	ZzAssert(notCancelled);
	
	[fEnlargeParentsCommand _redoIt];
	
	// now tell the tlc to redisplay itself in the old zone
	[self setTLCRepsNeedDisplayInModelRect:oldModelFrameUnion];
	
	// now tell the objects to redisplay themselves
	[self invalidateCachedImagesAndInvalidateRepAndSetDisplay];
}

- (void)dealloc
{
//	printf("-[MIAudioProcessingCommand dealloc]\n");
	delete fParams;
	[fpModifiedMIObjList release];
	[fEnlargeParentsCommand release];
	
	for(SInt32 i = 0; i < fpOriginalMementosList->CountItems(); i++)
	{
		MEAudioClipDataMemento* memento = dynamic_cast<MEAudioClipDataMemento*>(fpOriginalMementosList->ItemAt(i));
		delete memento;
	}
	delete fpOriginalMementosList;
	
	for(SInt32 i = 0; i < fpNewMementosList->CountItems(); i++)
	{
		MEAudioClipDataMemento* memento = dynamic_cast<MEAudioClipDataMemento*>(fpNewMementosList->ItemAt(i));
		delete memento;
	}
	delete fpNewMementosList;
	delete fModifiedFlagsArray;
    
    [super dealloc];
}

-(void)closeAndDeleteNewlyWrittenMTGAFiles
{
	for(UInt32 i = 0; i < [fpModifiedMIObjList count]; i++)
	{
		OfflineAudioClipWriter* writer = dynamic_cast<OfflineAudioClipWriter*>(fParams->GetWriter(i));
		OfflineAudioClipReader* reader = dynamic_cast<OfflineAudioClipReader*>(fParams->GetReader(i));
		ZzRtAssert(writer);
		ZzRtAssert(reader);
		// close the writer file
		writer->CloseAudioFile();
		char newChunkFilePath[1024];
		writer->GetChunk().GetPath(newChunkFilePath,1024);
		remove(newChunkFilePath);
	}
}

-(void)deleteNewlyWrittenMTGAFiles
{
	for(UInt32 i = 0; i < [fpModifiedMIObjList count]; i++)
	{
		OfflineAudioClipWriter* writer = dynamic_cast<OfflineAudioClipWriter*>(fParams->GetWriter(i));
		OfflineAudioClipReader* reader = dynamic_cast<OfflineAudioClipReader*>(fParams->GetReader(i));
		ZzRtAssert(writer);
		ZzRtAssert(reader);
		char newChunkFilePath[1024];
		writer->GetChunk().GetPath(newChunkFilePath,1024);
		remove(newChunkFilePath);
	}
}


-(void)replaceRangesWithChunks
{
	// actually replace the modified range with the new chunk file
	// please note MEAudioClipData::ReplaceRangeWithChunk returns a new preview splitask
	// that we add to a macro split task
	// add the same time, we first tell the object to redisplay itself (with the old
	// data and old size), then we tell the object to redisplay with the new size and
	// no preview data...
	ZzMacroSplitTask* mainSplitTask = new ZzMacroSplitTask();
	ZzSplitTask* splitTask = NULL;
	for(UInt32 i = 0; i < [fpModifiedMIObjList count]; i++)
	{
		OfflineAudioClipWriter* writer = dynamic_cast<OfflineAudioClipWriter*>(fParams->GetWriter(i));
		OfflineAudioClipReader* reader = dynamic_cast<OfflineAudioClipReader*>(fParams->GetReader(i));
		ZzAssert(writer);
		ZzAssert(reader);
		MEAudioClipData* data = writer->GetClip()->GetData();
		// close the writer file
		writer->CloseAudioFile();
		ZzAssert(data == reader->GetClip()->GetData());
		data->ReplaceRangeWithChunk(writer->GetChunk(),reader->GetStartTimeInClip(), reader->GetDuration(),&splitTask);
		mainSplitTask->AdoptSubTask(splitTask);
	}
	// execute the macro task (this perhaps displays a progress dlg box)
	BOOL notCancelled = [[MIApplicationController sharedApplicationController] executeAndDeleteSplitTask:mainSplitTask forMainViewController:fController];
	ZzAssert(notCancelled);
}

-(void)applyOriginalMementoWithoutRefresh
{
	for(UInt32 i = 0; i < [fpModifiedMIObjList count]; i++)
	{
		MIObject* miobj = [fpModifiedMIObjList objectAtIndex:i];
		
//		BOOL wasMarkedModifiedBefore = fModifiedFlagsArray->ItemAt(i);
		[miobj resetModifiedFlag:fModifiedFlagsArray->ItemAt(i)];
/*		if(wasMarkedModifiedBefore)
			[miobj markModified];
		else
			[miobj cleanModifiedFlag];*/
		
		MEAudioClip* clip = dynamic_cast<MEAudioClip*>([miobj WSObject]);
		ZzAssert(clip);
		MEAudioClipDataMemento* memento = dynamic_cast<MEAudioClipDataMemento*>(fpOriginalMementosList->ItemAt(i));
		ZzAssert(memento);
		clip->GetData()->SetMementoWithoutPreviewComputing(memento);
	}
}

@end
