/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MIChangeVAOCommand.h"
#import "MIChangeVAOCommandDlgController.h"

#ifndef TArmeWSObj_h
#include "TArmeWSObj.h"
#endif //TArmeWSObj_h

#ifndef TArmeDoc_h
#include "TArmeDoc.h"
#endif //TArmeDoc_h

#import "MIMainViewController.h"
#import "MIMontageDocument.h"
#import "MIObjectRep.h"
#import "MIObject.h"

#ifndef ArmeNotifications_h
#include "ArmeNotifications.h"
#endif //ArmeNotifications_h

/****************************************************************************************************

MIChangeVAOCommand

****************************************************************************************************/

@implementation MIChangeVAOCommand

-(id)initFromArchive:(ArmeArchiveHandler*)handler
{
	self = [super initFromArchive:handler];
	if(self)
	{
		// create the array that will hold the modified clips
		fModifiedMIObjs = [handler->ReadObjCObject() retain];
		fModifiedWSObjsVAORouting = new ZzArray<MEVAORouting>;
		
		fNewVAORouting.ReadFromArchive(handler);
		
		for(UInt32 i = 0; i < [fModifiedMIObjs count]; i++)
		{
			MEVAORouting vaoRouting;
			vaoRouting.ReadFromArchive(handler);
			fModifiedWSObjsVAORouting->AddItem(vaoRouting);
		}
	}
	return self;
}

-(BOOL)writeToArchive:(ArmeArchiveHandler*)handler
{
	[super writeToArchive:handler];
	handler->WriteObjCObject(fModifiedMIObjs);
	fNewVAORouting.WriteToArchive(handler);
	for(UInt32 i = 0; i < [fModifiedMIObjs count]; i++)
	{
		MEVAORouting vaoRouting = fModifiedWSObjsVAORouting->ItemAt(i);
		vaoRouting.WriteToArchive(handler);
	}
	return NO;
}

- (NSString*)commandName
{
	return @"Change Internal Audio Routing";
}

- (id)initWithSelection:(MIWSSelection*)sel
{
	self = [super initWithSelection:sel];
	if ( self )
	{
		[fSelection discardVideoTransitions];
		[fSelection discardVideoClips];

		fNewVAORouting.index = -3;
		// create the array that will hold the modified clips
		fModifiedMIObjs = [[NSMutableArray alloc] init];
		fModifiedWSObjsVAORouting = new ZzArray<MEVAORouting>;
	}
	return self;
}

+ (BOOL)supportsSelection:(MIWSSelection*)sel
{
	MIObject		*anObj;
	NSEnumerator	*objEnum = [sel objectEnumerator];

	if ( [sel type] == kMIWSSelectionTypeTimeSlice
#ifdef AUTOMATION_DATA
		|| [sel type] == kMIWSSelectionTypeAutomationPoints
		|| [sel type] == kMIWSSelectionTypeAutomationCurves
#endif
		 )
		return NO;

	if ( [sel count] == 0 ) return NO;

	while ( anObj = [[objEnum nextObject] representedObject] )
	{
		SInt32 type = [anObj getType];
		
		if (![[anObj parentObject] isTLC])
			return NO;
					
		if( type != (TArmeWSObj::kAudioClip) && type != (TArmeWSObj::kContainer) && type != (TArmeWSObj::kVideoEffect) && type != (TArmeWSObj::kArmeWSObj) && type != (TArmeWSObj::kAudioProcessor) )
			return NO;
	}
	return YES;
}

- (void)setNewVAORouting:(MEVAORouting)vaoRouting
{
	ZzAssert(fNewVAORouting.index == -3);
	fNewVAORouting = vaoRouting;
}

- (BOOL)displaySetupDialog
{
	NSModalSession session;
	NSWindow* 	win;

	ZzAssert(fNewVAORouting.index == -3);
	MIChangeVAOCommandDlgController* dlgController =
		[MIChangeVAOCommandDlgController alloc];
	[dlgController initWithWindowNibName:@"MIChangeVAOCommandDlg" owner:dlgController];
	win = [dlgController window];
//	[dlgController populatePopUpButton:[[fController document] armeDocument]];
	
	session = [NSApp beginModalSessionForWindow:win];
	while([NSApp runModalSession:session] == NSRunContinuesResponse)
	{
	}
	[NSApp endModalSession:session];					
	[win orderOut:0];
	fNewVAORouting = [dlgController getVAORouting];
	[dlgController release];
	if(fNewVAORouting.index == -2)
		return NO;
	else
		return YES;
}

- (MICmdStatus)doIt
{
	MIWSSelection* sel = fSelection;
	
	ZzAssert(fNewVAORouting.index > -3);
	
	// let's enumerate all the selection objects.
	NSEnumerator* objEnum = [sel objectEnumerator];
	MIObjectRep* objectRep;
	
	while ( objectRep = [objEnum nextObject] )
	{
		MIObject* miobj = [objectRep representedObject];
		TArmeWSObj* wsobj = [miobj WSObject];
		ZzAssert(wsobj)
		[fModifiedMIObjs addObject:miobj];
		MEVAORouting originalVAORouting = wsobj->GetVAORouting();
		fModifiedWSObjsVAORouting->AddItem(originalVAORouting);
		wsobj->SetVAORouting(fNewVAORouting);
		[miobj refreshCachedTitleStringImageInReps];
		[miobj setRepresentationsNeedDisplay];
	}
	return kMICmdStatusDone;
}

- (void)undoIt
{
	for(UInt32 i = 0; i < [fModifiedMIObjs count]; i++)
	{
		MIObject* miobj = [fModifiedMIObjs objectAtIndex:i];
		TArmeWSObj* wsobj = dynamic_cast<TArmeWSObj*>([miobj WSObject]);
		ZzAssert(wsobj);
		wsobj->SetVAORouting(fModifiedWSObjsVAORouting->ItemAt(i));
		[miobj refreshCachedTitleStringImageInReps];
		[miobj setRepresentationsNeedDisplay];
	}
}

- (void)redoIt
{
	for(UInt32 i = 0; i < [fModifiedMIObjs count]; i++)
	{
		MIObject* miobj = [fModifiedMIObjs objectAtIndex:i];
		TArmeWSObj* wsobj = dynamic_cast<TArmeWSObj*>([miobj WSObject]);
		ZzAssert(wsobj);
		wsobj->SetVAORouting(fNewVAORouting);
		[miobj refreshCachedTitleStringImageInReps];
		[miobj setRepresentationsNeedDisplay];
	}
}

- (void)dealloc
{
	[fModifiedMIObjs release];
	delete fModifiedWSObjsVAORouting;
    
    [super dealloc];
}

@end


