/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MICollapseCommand.h"
#import "MIMainViewController.h"
#import "MIObjectRep.h"
#import "MIObject.h"

#import "MISingleCollapseExpandCommand.h"

/****************************************************************************************************

MICollapseCommand

****************************************************************************************************/


@implementation MICollapseCommand

-(void)dealloc
{
	[super dealloc];
}

- (NSString*)commandName
{
	return @"Collapse";
}

- (BOOL)displaySetupDialog
{
	return YES;
}

- (MICmdStatus)doIt
{
	[fSelection discardVideoTransitions];

	NSEnumerator*	objEnum = [fSelection objectEnumerator];
	MIObjectRep*	objectRep;

	while ( objectRep = [objEnum nextObject] )
	{
		if ( ![[objectRep representedObject] isCollapsed] )
		{
			MISingleCollapseCommand	*cmd;
			
			cmd = [[[MISingleCollapseCommand alloc] initWithSelection:nil] autorelease];
			[cmd setObject:[objectRep representedObject]];
			[self addCommand:cmd];
		}
	}
	
	return [super doIt];
}

+ (BOOL)supportsSelection:(MIWSSelection*)sel
{
	if ( [sel containsOnlyVideoTransitions])
		return NO;

	if ( [sel type] != kMIWSSelectionTypeObjects )
		return NO;
	else
	{
		NSEnumerator	*objEnum = [sel objectEnumerator];
		MIObject		*anObj;
		
		while ( anObj = [[objEnum nextObject] representedObject] )
		{
			if ( ![anObj isCollapsed] )
				return YES;
		}
		return NO;
	}
}

@end


