/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MICommand.h"
#import "MIObjectRep.h"
#import "MIObject.h"
#import "MIApplicationController.h"

#ifndef ArmeArchiveHandler_h
#include "ArmeArchiveHandler.h"
#endif //ArmeArchiveHandler_h

#ifndef MIProgressBarDlgManager_h
#include "MIProgressBarDlgManager.h"
#endif //MIProgressBarDlgManager_h

#ifndef ZzException_h
#include "ZzException.h"
#endif //ZzException_h


MIProgressBarDlgManager* _mainCommandProgressBarDialog = NULL;

/****************************************************************************************************

MICommand

****************************************************************************************************/

@implementation MICommand

- (id)init
{
	self = [super init];
	if ( self )
	{
		fState = kMICommandStateCanOnlyBeDone;
		fSelection = nil;
		fModifiedAudioHierarchy = FALSE;
		fModifiedVideoHierarchy = FALSE;
		fOverviewUpdate = NO;
	}
	return self;
}

- (void)dealloc
{
	[fSelection release];
    [super dealloc];
}

- (id)initWithSelection:(MIWSSelection*)sel
{
	self = [super init];
	if ( self )
	{
		fState = kMICommandStateCanOnlyBeDone;
		fSelection = [sel copy];
	}
	
	return self;
}

-(id)initFromArchive:(ArmeArchiveHandler*)handler
{
    if (( self = [super initFromArchive:handler] ))
    {
		fState = handler->ReadSInt32();
		fSelection = [handler->ReadObjCObject() retain];
		fModifiedAudioHierarchy = handler->ReadBOOL();
		fModifiedVideoHierarchy = handler->ReadBOOL();
		fOverviewUpdate = handler->ReadBOOL();
    }
    return self;
}

-(BOOL)writeToArchive:(ArmeArchiveHandler*)handler
{
    if ( [super writeToArchive:handler] )
    {
		handler->WriteSInt32(fState);
		handler->WriteObjCObject(fSelection);
		handler->WriteBOOL(fModifiedAudioHierarchy);
		handler->WriteBOOL(fModifiedVideoHierarchy);
		handler->WriteBOOL(fOverviewUpdate);
		return YES;
    }
    else return NO;
}

- (BOOL)displaySetupDialog
{
	return YES;
}

- (BOOL)CheckFrozenObjectInSelectionAndAskIfUserWantsToContinue:(MIWSSelection*)sel
{
	NSEnumerator* selEnum = [sel objectEnumerator];
	MIObjectRep* objRep;
	BOOL bContainsFrozenObject = NO;

	while (objRep = [selEnum nextObject])
	{
		if ([[objRep representedObject] hasParentFrozenObject])
		{
			bContainsFrozenObject = YES;
			break;
		}
	}

	if (bContainsFrozenObject)
	{
		if (NSRunAlertPanel(nil, @"Your selection contains some object that are the result of a render operation.  The command you are going to accomplish will forbid you to un-render your object.\n Do you want to continue ?", @"continue", @"cancel", nil) == NSAlertDefaultReturn)
			return YES;
		else
			return NO;
	}
	else
		return YES;
}

- (BOOL)CheckFrozenObjectAndAskIfUserWantsToContinue:(MIObject*)obj
{
	if ([obj hasParentFrozenObject])
	{
		if (NSRunAlertPanel(nil, @"Your selection contains some object that are the result of a render operation.  The command you are going to accomplish will forbid you to un-render your object.\n Do you want to continue ?", @"continue", @"cancel", nil) == NSAlertDefaultReturn)
			return YES;
		else
			return NO;
	}
	else
		return YES;
}


- (SInt32)state
{
	return fState;
}

- (void)setSelection:(MIWSSelection*)sel
{
	id selection = [sel copy];
	[fSelection release];
	fSelection = selection;
}

- (MICmdStatus)_doIt
{
	try
	{
		MICmdStatus st;
		ZzAssert(fState == kMICommandStateCanOnlyBeDone);
		st = [self doIt];
		fState = kMICommandStateCanOnlyBeUndone;
		return st;
	}
	catch (ZzException* e)
	{
		e->DefaultCatcher();
		delete e;
		return kMICmdStatusCancelled;
	}
}

- (void)_undoIt
{
	ZzAssert(fState == kMICommandStateCanOnlyBeUndone);
	[self undoIt];
	fState = kMICommandStateCanOnlyBeRedone;
	
	[self selectOriginalSelection];
}

- (void)_redoIt
{
	ZzAssert(fState == kMICommandStateCanOnlyBeRedone);
	[self redoIt];
	fState = kMICommandStateCanOnlyBeUndone;
}

- (MICmdStatus)doIt
{
	ZzAssert(0);
	return kMICmdStatusDone;
}

- (void)undoIt
{
	ZzAssert(0);
}

- (void)redoIt
{
	ZzAssert(0);
}

- (BOOL)canDoIt
{
	return (fState == kMICommandStateCanOnlyBeDone);
}

- (BOOL)canUndoIt
{
	return (fState == kMICommandStateCanOnlyBeUndone);
}

- (BOOL)canRedoIt
{
	return (fState == kMICommandStateCanOnlyBeRedone);
}

+ (BOOL)supportsSelection:(MIWSSelection*)sel
{
	return YES;
}

- (NSString*)commandName
{
	return [self className];
}

- (void)modifiedAudioHierarchy
{
	fModifiedAudioHierarchy = TRUE;
}
- (void)modifiedVideoHierarchy
{
	fModifiedVideoHierarchy = TRUE;
}

- (void)overviewNeedsUpdate
{
	fOverviewUpdate = YES;
}

- (BOOL)hasModifiedAudioHierarchy
{
	return fModifiedAudioHierarchy;
}
- (BOOL)hasModifiedVideoHierarchy
{
	return fModifiedVideoHierarchy;
}

- (BOOL)causedNeedOfOverviewUpdate;
{
	return fOverviewUpdate;
}


+ (MIProgressBarDlgManager*)getCurrentProgressDialog;
{
	return _mainCommandProgressBarDialog;
}
+ (void)setCurrentProgressDialog:(MIProgressBarDlgManager*)dlog
{
	_mainCommandProgressBarDialog = dlog;
}

- (int)getNbLengthyTasks
{
	return 0;
}

- (BOOL)isCancelable
{
	return YES;
}

- (void)selectOriginalSelection
{
	if (!fSelection)
		return;

	[[MIApplicationController sharedApplicationController] clearAllSelections];

	if ([fSelection type] == kMIWSSelectionTypeObjects)
	{
		for (unsigned int i=0; i < [fSelection count]; ++i)
		{
			[[[fSelection objectAtIndex:i] representedObject] addToSelections];
		}
	}
	else
	{
		if ( fSelection && [fSelection count] )
		{
			[[[fSelection objectAtIndex:0] representedObject] setSelectionsFromTime:[fSelection startTime] toTime:[fSelection endxTime]];
		}
	}

}


@end

