/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MIConfigureTransitionCommand.h"
#import "MIObjectRep.h"
#import "MIObject.h"
#import "MIUpdateVideoTransitionsCommand.h"
#import "MIViewerController.h"

#ifndef TArmeWSObj_h
#include "TArmeWSObj.h"
#endif //TArmeWSObj_h

#ifndef TVideoEffect_h
#include "TVideoEffect.h"
#endif //TVideoEffect_h

#ifndef MovieHelper_h
#include "MovieHelper.h"
#endif //MovieHelper_h

@implementation MIConfigureTransitionCommand
-(id)initFromArchive:(ArmeArchiveHandler*)handler
{
	self = [super initFromArchive:handler];
	return self;
}

-(BOOL)writeToArchive:(ArmeArchiveHandler*)handler
{
	return [super writeToArchive:handler];
}

- (void)dealloc
{
	[super dealloc];
}

- (NSString*)commandName
{
	return @"Configure Video Transition";
}

- (BOOL)displaySetupDialog
{
	return YES;
}

- (id)initWithSelection:(MIWSSelection*)sel
{
	self = [super initWithSelection:sel];
	
	if ( self )
	{
		[fSelection keepOnlyVideoTransitions];
		[self modifiedVideoHierarchy];
	}
	return self;
}

+ (BOOL)supportsSelection:(MIWSSelection*)sel
{
	if ( [sel type] != kMIWSSelectionTypeObjects )
		return NO;
	else
	{
		if ( [sel count] == 0 ) return NO;
		if ( [sel containsVideoTransitions] ) return YES;
		return NO;
	}
}

- (MICmdStatus)doIt
{
	//configure the effectDesc of the first object in the selection
	QTAtomContainer tempEffectDesc;

	QTNewAtomContainer(&tempEffectDesc);
	
	QTCopyAtom( ((TVideoXFadeEffect*)[[[fSelection objectAtIndex:0] representedObject] WSObject])->fEffectDesc, kParentAtomIsContainer, &tempEffectDesc);

	float from;
	float to;
	TVideoXFadeEffect* xFade = ((TVideoXFadeEffect*)[[[fSelection objectAtIndex:0] representedObject] WSObject]);

	GetFromAndToParamOfTransition( xFade->fEffectDesc, from, to);
	GetFromAndToParamOfTransition( tempEffectDesc, from, to);

	PicHandle posterSrc1 = NULL;
	PicHandle posterSrc2 = NULL;

	ArmeTime posterTime = xFade->GetAbsoluteStartTime();

	// configure the effectDescription
	posterSrc1 = xFade->fMasterMovie->GetPictAtTime(xFade->fClip1, xFade, posterTime);
	posterSrc2 = xFade->fMasterMovie->GetPictAtTime(xFade->fClip2, xFade, posterTime);
	[[MIViewerController sharedViewerController] setMovie:nil];
	bool flag = ConfigureEffect(tempEffectDesc, 2, posterSrc1, posterSrc2);
	[[MIViewerController sharedViewerController] show:nil];
		if (!flag)
        {
            MIUpdateVideoTransitionsCommand* update = [[[MIUpdateVideoTransitionsCommand alloc] initWithDocument:[[[fSelection objectAtIndex:0] representedObject] montageDoc] from:ARME_TIME_UNDEF to:ARME_TIME_UNDEF] autorelease];
            [update doIt];
            return kMICmdStatusCancelled;
        }

	NSEnumerator*	selEnum = [fSelection objectEnumerator];
	MIObjectRep*	selObj;
	while (selObj = [selEnum nextObject])
	{
		//for each of the object in the selection, create a command
		[self addCommand:[[[MIConfigureTransitionLowLevelCommand alloc] initWithTransition:[selObj representedObject] effect:tempEffectDesc] autorelease]];
	}

	MICmdStatus st = [super doIt];
	
	QTDisposeAtomContainer(tempEffectDesc);
	return st;
}

@end


@implementation MIConfigureTransitionLowLevelCommand

-(id)initFromArchive:(ArmeArchiveHandler*)handler
{
	self = [super initFromArchive:handler];
	
	QTNewAtomContainer(&fPreviousEffectDesc);
	handler->ReadQTAtomContainer(&fPreviousEffectDesc);
	QTNewAtomContainer(&fNewEffectDesc);
	handler->ReadQTAtomContainer(&fNewEffectDesc);
	fClip1 = dynamic_cast<TVideoObject*>(handler->ReadCPPObject());
	fClip2 = dynamic_cast<TVideoObject*>(handler->ReadCPPObject());
	fParent = dynamic_cast<TVideoObject*>(handler->ReadCPPObject());
	return self;
}

-(BOOL)writeToArchive:(ArmeArchiveHandler*)handler
{
	[super writeToArchive:handler];
	
	handler->WriteQTAtomContainer(&fPreviousEffectDesc);
	handler->WriteQTAtomContainer(&fNewEffectDesc);
	handler->WriteCPPObject(fClip1);
	handler->WriteCPPObject(fClip2);
	handler->WriteCPPObject(fParent);
	return YES;
}

- (void)dealloc
{
	QTDisposeAtomContainer(fPreviousEffectDesc);
	QTDisposeAtomContainer(fNewEffectDesc);
	[super dealloc];
}

- (NSString*)commandName
{
	return @"Configure Video Transition Low Level";
}


- (id)initWithTransition:(MIObject*)xFade effect:(QTAtomContainer)effectDesc
{
	self = [super initWithSelection:nil];
	
	if ( self )
	{
		QTNewAtomContainer(&fPreviousEffectDesc);
		QTNewAtomContainer(&fNewEffectDesc);
		QTCopyAtom(((TVideoXFadeEffect*)[xFade WSObject])->fEffectDesc, kParentAtomIsContainer, &fPreviousEffectDesc);
		QTCopyAtom(effectDesc, kParentAtomIsContainer, &fNewEffectDesc);

		TVideoXFadeEffect*	wsxFade = (TVideoXFadeEffect*)([xFade WSObject]);

		fClip1 = wsxFade->fClip1;
		fClip2 = wsxFade->fClip2;
		fParent = wsxFade->GetPos().fParent;

		[self modifiedVideoHierarchy];
	}
	return self;
}


- (MICmdStatus)doIt
{
	TVideoXFadeEffect*	xFade = ((TVideoObject*)fParent)->FindTransitionBetweenChildren(fClip1, fClip2);
	
	//copy the effectDesc
	QTCopyAtom(fNewEffectDesc, kParentAtomIsContainer, &(xFade->fEffectDesc));

	//update the from an to parameters
	GetFromAndToParamOfTransition(xFade->fEffectDesc, xFade->fFrom, xFade->fTo);

	xFade->fMasterMovie->UpdateSubMovie(xFade);

	return kMICmdStatusDone;
}

- (void)undoIt
{
	TVideoXFadeEffect*	xFade = ((TVideoObject*)fParent)->FindTransitionBetweenChildren(fClip1, fClip2);
	
	//copy the effectDesc
	QTCopyAtom(fPreviousEffectDesc, kParentAtomIsContainer, &(xFade->fEffectDesc));

	//update the from an to parameters
	GetFromAndToParamOfTransition(xFade->fEffectDesc, xFade->fFrom, xFade->fTo);
	
	xFade->fMasterMovie->UpdateSubMovie(xFade);	
}

- (void)redoIt
{
	[self doIt];
}

@end