/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MIConfigureVideoEffectCommand.h"
#import "MIObjectRep.h"
#import "MIObject.h"
#import "MIUpdateVideoTransitionsCommand.h"
#import "MIMontageDocument.h"

#ifndef TArmeWSObj_h
#include "TArmeWSObj.h"
#endif //TArmeWSObj_h

#ifndef TVideoEffect_h
#include "TVideoEffect.h"
#endif //TVideoEffect_h

#ifndef MovieHelper_h
#include "MovieHelper.h"
#endif //MovieHelper_h

@implementation MIConfigureVideoEffectCommand
-(id)initFromArchive:(ArmeArchiveHandler*)handler
{
	self = [super initFromArchive:handler];
	return self;
}

-(BOOL)writeToArchive:(ArmeArchiveHandler*)handler
{
	return [super writeToArchive:handler];
}

- (void)dealloc
{
	[super dealloc];
}

- (NSString*)commandName
{
	return @"Configure Video Effect";
}

- (BOOL)displaySetupDialog
{
	return YES;
}

- (id)initWithSelection:(MIWSSelection*)sel
{
	self = [super initWithSelection:sel];
	
	if ( self )
	{
		[self modifiedVideoHierarchy];
	}
	return self;
}

+ (BOOL)supportsSelection:(MIWSSelection*)sel
{
	if ( [sel type] != kMIWSSelectionTypeObjects )
		return NO;
	else
	{
		if ( [sel count] == 0 ) return NO;
		// the selection can only contain xfadeEffects
		NSEnumerator*	objEnum = [sel objectEnumerator];
		MIObjectRep*	objectRep;
	
		while ( objectRep = [objEnum nextObject] )
		{
			if (( [[objectRep representedObject] WSObject]->GetType() != TArmeWSObj::kVideoEffect ) &&
	                     ([[objectRep representedObject] WSObject]->GetType() != TArmeWSObj::kVideoClip))
				return NO;
		}
		return YES;
	}
}

- (MICmdStatus)doIt
{
	//configure the effectDesc of the first object in the selection
	QTAtomContainer tempEffectDesc;
	QTNewAtomContainer(&tempEffectDesc);
	
	QTCopyAtom( ((TVideoEffect*)[[[fSelection objectAtIndex:0] representedObject] WSObject])->fEffectDesc, kParentAtomIsContainer, &tempEffectDesc);

	TVideoEffect* container = ((TVideoEffect*)[[[fSelection objectAtIndex:0] representedObject] WSObject]);


	// configure the effect
        PicHandle posterSrc1 = NULL;
    
    
        for (int i=0; i < container->GetChildrenCount(); i++)
        {
            if (container->GetChild(i)->IsVideoObject() && (container->GetChild(i)->GetType() != TArmeWSObj::kVideoXFadeEffect))  //actually, this also includes xFade effect since the trackRep of the clips will be the track of the xFade
            {
                TVideoObject* video = (TVideoObject*)(container->GetChild(i));


                if (!posterSrc1)
                    posterSrc1 = video->fMasterMovie->GetPictAtTime(video, container, video->GetAbsoluteStartTimeOfVideoClip());
            }
        }
        
	if (!ConfigureEffect(tempEffectDesc, 1, posterSrc1))
        {
            MIUpdateVideoTransitionsCommand* update = [[[MIUpdateVideoTransitionsCommand alloc] initWithDocument:[[[fSelection objectAtIndex:0] representedObject] montageDoc] from:ARME_TIME_UNDEF to:ARME_TIME_UNDEF] autorelease];
            [update doIt];
            return kMICmdStatusCancelled;
        }
	NSEnumerator*	selEnum = [fSelection objectEnumerator];
	MIObjectRep*	selObj;
	while (selObj = [selEnum nextObject])
	{
		//for each of the object in the selection, create a command
		[self addCommand:[[[MIConfigureVideoEffectLowLevelCommand alloc] initWithContainer:[selObj representedObject] effect:tempEffectDesc] autorelease]];
	}

	MICmdStatus st = [super doIt];
	
	QTDisposeAtomContainer(tempEffectDesc);
	return st;
}

@end


@implementation MIConfigureVideoEffectLowLevelCommand

-(id)initFromArchive:(ArmeArchiveHandler*)handler
{
	self = [super initFromArchive:handler];
	
	QTNewAtomContainer(&fPreviousEffectDesc);
	handler->ReadQTAtomContainer(&fPreviousEffectDesc);
	QTNewAtomContainer(&fNewEffectDesc);
	handler->ReadQTAtomContainer(&fNewEffectDesc);
	fContainer = dynamic_cast<TVideoEffect*>(handler->ReadCPPObject());
	if (handler->GetFileVer() >= 0x0001002A)
		fMIContainer = [handler->ReadObjCObject() retain];
	else
		fMIContainer = nil;
	
	return self;
}

-(BOOL)writeToArchive:(ArmeArchiveHandler*)handler
{
	[super writeToArchive:handler];
	
	handler->WriteQTAtomContainer(&fPreviousEffectDesc);
	handler->WriteQTAtomContainer(&fNewEffectDesc);
	handler->WriteCPPObject(fContainer);
	//from version 0x0001002A
	handler->WriteObjCObject(fMIContainer);
	
	return YES;
}

- (void)dealloc
{
	QTDisposeAtomContainer(fPreviousEffectDesc);
	QTDisposeAtomContainer(fNewEffectDesc);
	[super dealloc];
}

- (NSString*)commandName
{
	return @"Configure Video Effect Low Level";
}


- (id)initWithContainer:(MIObject*)container effect:(QTAtomContainer)effectDesc
{
	self = [super initWithSelection:nil];
	
	if ( self )
	{
		QTNewAtomContainer(&fPreviousEffectDesc);
		QTNewAtomContainer(&fNewEffectDesc);
		QTCopyAtom(((TVideoEffect*)[container WSObject])->fEffectDesc, kParentAtomIsContainer, &fPreviousEffectDesc);
		QTCopyAtom(effectDesc, kParentAtomIsContainer, &fNewEffectDesc);

		fContainer = (TVideoEffect*)([container WSObject]);
		fMIContainer = [container retain];
		[self modifiedVideoHierarchy];
	}
	return self;
}


- (MICmdStatus)doIt
{
	//copy the effectDesc
	QTCopyAtom(fNewEffectDesc, kParentAtomIsContainer, &(fContainer->fEffectDesc));
	fContainer->UpdateName();

	fContainer->fMasterMovie->UpdateSubMovie(fContainer);

	if (!fMIContainer)
	{
		fMIContainer = [[[NSDocumentController sharedDocumentController] currentDocument] findObject:fContainer];
	}

	[fMIContainer refreshCachedTitleStringImageInReps];
	[fMIContainer setTitlesNeedDisplay];

	return kMICmdStatusDone;
}

- (void)undoIt
{
	//copy the effectDesc
	QTCopyAtom(fPreviousEffectDesc, kParentAtomIsContainer, &(fContainer->fEffectDesc));
	fContainer->UpdateName();

	if (!fMIContainer)
	{
		fMIContainer = [[[NSDocumentController sharedDocumentController] currentDocument] findObject:fContainer];
	}

	[fMIContainer refreshCachedTitleStringImageInReps];
	[fMIContainer setTitlesNeedDisplay];

	fContainer->fMasterMovie->UpdateSubMovie(fContainer);
}

- (void)redoIt
{
	[self doIt];
}

@end