/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MICommand.h"

#ifndef TArmeDoc_h
#include "TArmeDoc.h"
#endif //TArmeDoc_h

#ifndef ME_AudioChunks_h
#include "ME_AudioChunks.h"
#endif //ME_AudioChunks_h

class MEAudioChunk;
@class MIObject;
@class MIDeleteCommand;
@class MIMontageDocument;
@class MIResizeCommand;
class ZzSplitTask;
struct MIVPos;

// this class creates a bunch of audio clips in the tlc from existing chunk files (created by a record operation)

@interface MICreateAudioClipsFromRecordedTakesCommand : MICommand
{
	NSMutableArray*					fCreatedObjects;
	NSMutableArray*					fDeleteCommands;
	MIMontageDocument*				fMontageDoc;
	MIResizeCommand*				fEnlargeCommand; //this command is used to enlarge the target parent if needed
	ZzArray<MEAudioChunkFileIndex>*	fIndicesZzArray;
	ZzArray<SInt32>*			fNbChannelsZzArray;
	ZzArray<MIPoint>*			fPositionZzArray;
	ZzArray<ArmeTime>*			fDurationZzArray;
	NSMutableArray*				fClipNames;
}

-(id)initWithDoc:(MIMontageDocument*)mtgDoc;
-(id)initFromArchive:(ArmeArchiveHandler*)handler;
-(BOOL)writeToArchive:(ArmeArchiveHandler*)handler;
- (void)addFile:(MEAudioChunkFileIndex)index nbChannels:(SInt32)nbChannels position:(MIPoint)position duration:(ArmeTime)takeDuration name:(NSString*)name; // duration is for debug purposes only since it's known by the file
- (MICmdStatus)doIt;
- (void)undoIt;
- (void)redoIt;
- (void)dealloc;
@end

