/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MICreateAudioProcessorCommand.h"
#import "MIMainViewController.h"
#import "MISingleMoveCommand.h"
#import "MIEncloseSelectionCommand.h"
#import "MITopLevelContainerRep.h"
#import "MIObject.h"
#import "MIMontageDocument.h"
#import "MIDeleteCommand.h"
#import "MITime.h"

#ifndef TArmeDoc_h
#include "TArmeDoc.h"
#endif //TArmeDoc_h

#ifndef TARPIHost_h
#include "TARPIHost.h"
#endif //TARPIHost_h

#ifndef TArmeWSObj_h
#include "TArmeWSObj.h"
#endif //TArmeWSObj_h

#ifndef VARPIPluginSet_h
#include "VARPIPluginSet.h"
#endif //VARPIPluginSet_h

#ifndef VARPIPluginClass_h
#include "VARPIPluginClass.h"
#endif VARPIPluginClass_h

#ifndef VARPIPlugin_h
#include "VARPIPlugin.h"
#endif VARPIPlugin_h

#ifndef TArmeMediaEngine_h
#include "TArmeMediaEngine.h"
#endif //TArmeMediaEngine_h

#ifndef MECoreAudioPlayer_h
#include "MECoreAudioPlayer.h"
#endif MECoreAudioPlayer_h

/****************************************************************************************************

MICreateAudioProcessorCommand

****************************************************************************************************/

@implementation MICreateAudioProcessorCommand : MICommand

-(id)initFromArchive:(ArmeArchiveHandler*)handler
{
	self = [super initFromArchive:handler];
	if ( self )
	{
		fMoveCommand = [handler->ReadObjCObject() retain];
		fEncloseCommand = [handler->ReadObjCObject() retain];
		fDeleteCommand = [handler->ReadObjCObject() retain];
		fCreatedObject = [handler->ReadObjCObject() retain];
		fProcName = [handler->ReadObjCObject() retain];
	}
	return self;
}

-(BOOL)writeToArchive:(ArmeArchiveHandler*)handler
{
	if ( [super writeToArchive:handler] )
	{
		handler->WriteObjCObject( fMoveCommand );
		handler->WriteObjCObject( fEncloseCommand );
		handler->WriteObjCObject( fDeleteCommand );
		handler->WriteObjCObject( fCreatedObject );
		handler->WriteObjCObject( fProcName );
		return YES;
	}
	return NO;
}

- (void)dealloc
{
	[fMoveCommand release];
	[fEncloseCommand release];
	[fCreatedObject release];
	[fDeleteCommand release];
	[fProcName release];
	[super dealloc];
}

- (NSString*)commandName
{
	return [NSString stringWithFormat:@"Create %@", fProcName];
}

- (MIObject*)getCreatedObject
{
	return fCreatedObject;
}

- (id)initWithDocument:(MIMontageDocument*)doc selection:(MIWSSelection*)sel
{
	verntl(audioEffectsTom,6,"initWithController (MICreateAudioProcessorCommand)");
	self = [super initWithSelection:sel];
	fMontageDoc = doc;

	[self modifiedAudioHierarchy];
	
	return self;
}

- (void)setPosition:(MIPoint)point
{
	verntl(audioEffectsTom,6,"setPosition (MICreateAudioProcessorCommand)");
	fPosition = point;
}

- (void)setParent:(MIObject*)parent
{
	verntl(audioEffectsTom,6,"setParent (MICreateAudioProcessorCommand)");
	fParent = parent;
}

- (void)setSet:(int)set index:(int)index name:(NSString*)name
{
	verntl(audioEffectsTom,6,"setSet (MICreateAudioProcessorCommand)");
	fSet = set;
	fIndex = index;
	fProcName = [name retain];
}


- (MICmdStatus)doIt
{
	verntl(audioEffectsTom,6,"doIt (MICreateAudioProcessorCommand)");
	TArmeDoc				*armeDoc;
	MIMontageDocument		*mDoc;
	TArmeWSObj				*newArmeObj;
	MIObject				*newObj;
	MITime					*t;
	
	mDoc = fMontageDoc;
    armeDoc = [mDoc armeDocument];

	newArmeObj = new TArmeContainer( "", armeDoc );

	if ( !newArmeObj )
	{
		NSLog( @"Null object returned while trying to create a container\n\r");
		return kMICmdStatusCancelled;
	}

	t = [MITime timeWithDocument:mDoc];
	
	[t setSeconds:10];
	
	newArmeObj->_SetLength([t getArmeTime]);
	
	// copy the children routing, if any
	MEVAORouting newRouting = MIGetSelectionVAORouting(fSelection);
	newArmeObj->SetVAORouting(newRouting);

	VARPIPluginClass* pluginClass = TArmeMediaEngine::ARPIHost()->GetPluginSet(fSet)->GetPluginClass(fIndex);
	VARPIPlugin* plugin = TArmeMediaEngine::ARPIHost()->CreatePlugin(pluginClass);
	newArmeObj->SetAudioPlugin(plugin);
	newArmeObj->SetPresetAsFirstFromFactory();
	newArmeObj->SyncPluginFromPreset();
	
	UInt32 procIndex = armeDoc->GetAndIncrNextIndexForAudioPluginClass(pluginClass);
	
	char* procName = new char[1024];

	// this is disabled, since the name will be saved with the doc
	// it's not a good idea to have (demo) in there... or (demo) (demo)

	//if (pluginClass->PluginIsDemo())
	//{
	//	sprintf(procName,"%s (Demo) %d",[fProcName UTF8String],(int)procIndex);
	//}
	//else
	//{
	//	sprintf(procName,"%s %d",[fProcName UTF8String],(int)procIndex);
	//}
	
	sprintf(procName,"%s %d",pluginClass->GetPluginName(),(int)procIndex);
	
	newArmeObj->SetNameCopy(procName);
	delete [] procName;				

    // setup plugin environment
    plugin->SetFrameRate(TArmeMediaEngine::CoreAudioPlayer()->GetAudioFrameRate());
    plugin->SetBufferNbFrames(TArmeMediaEngine::CoreAudioPlayer()->OutDeviceBufferNbFrames());
    // send notification
    plugin->FormatChange();

    newObj = [[MIObject alloc] initWithArmeWSObj:newArmeObj];
    [newObj setMontageDoc:mDoc];

	[newObj createAllRepresentations];

	fMoveCommand = [[MISingleMoveCommand alloc] initWithSelection:nil];
	[fMoveCommand setObject:newObj];
	if ( fParent )
		[fMoveCommand setNewParent:fParent];
	else
		[fMoveCommand setNewParent:[mDoc topLevelContainer]];
	[fMoveCommand setNewOrigin:fPosition];

	[fMoveCommand _doIt];

	fCreatedObject = newObj; // no need to retain it since it was created here !

	// make the new container enclose the current selection, if it is possible

	fEncloseCommand = [[MIEncloseSelectionCommand alloc] initWithSelection:fSelection];
	[fEncloseCommand setContainer:fCreatedObject];
	if ( ![MIEncloseSelectionCommand supportsSelection:fSelection] )
	{
		[fEncloseCommand release];
		fEncloseCommand = nil;
	}
	[fEncloseCommand _doIt];
	return kMICmdStatusDone;
}

- (void)undoIt
{
	 [fEncloseCommand _undoIt];

	 if ( !fDeleteCommand )
	 {
		 fDeleteCommand = [[MIDeleteCommand alloc] initWithObject:fCreatedObject];
		 [fDeleteCommand _doIt];
	 }
	 else
	 {
		 [fDeleteCommand _redoIt];
	 }
}

- (void)redoIt
{
	 [fDeleteCommand _undoIt];
	 [fEncloseCommand _redoIt];
}

@end

