/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
#import "MICreateClipsFromLibraryElementsCommand.h"
#import "MILibraryElement.h"
#import "MIObject.h"
#import "TArmeWSObj.h"
#import "MIPasteLowLevelCommand.h"
#import "MICreateWSObjsFromFilesCommand.h"
#import "MIMainViewController.h"

@implementation MICreateClipsFromLibraryElementsCommand

- (id)initWithElements:(NSArray*)elems
{
	self = [super initWithSelection:nil];
	if ( self )
	{
		fFirstPosition = MIZeroPoint;
		fParent = nil;
		fElements = [elems retain];
	}
	return self;
}

-(id)initFromArchive:(ArmeArchiveHandler*)handler
{
	self = [super initFromArchive:handler];
	if(self)
	{
		fElements = [handler->ReadObjCObject() retain];
		fFirstPosition = handler->ReadMIPoint();
		fParent = handler->ReadObjCObject();
//		fMainViewController = handler->ReadObjCObject();
	}
	return self;
}

-(BOOL)writeToArchive:(ArmeArchiveHandler*)handler
{
	[super writeToArchive:handler];
		handler->WriteObjCObject( fElements );
		handler->WriteMIPoint( fFirstPosition );
		handler->WriteObjCObject( fParent );
//		handler->WriteObjCObject( fMainViewController );
	return YES;
}

- (void)dealloc
{
	[fElements release];
	[super dealloc];
}

- (void)setParent:(MIObject*)parent
{
	fParent = parent;
}

- (void)setGlobalFirstPosition:(MIPoint)p
{
	fFirstPosition = p;
}

- (void)setMainViewController:(MIMainViewController*)mvc
{
	fMainViewController = mvc;
}

- (MICmdStatus)doIt
{
#if USE_LIBRARY

	NSEnumerator		*libEnum = [fElements objectEnumerator];
	MILibraryElement	*elem;
	NSMutableArray		*osObjects = [[NSMutableArray alloc] initWithObjects:nil];
	NSMutableArray		*miObjects = [[NSMutableArray alloc] initWithObjects:nil];
	ArmeTime			miObjectLengths = _ArmeTime(0);
	id					cmd;

	
	while ( elem = [libEnum nextObject] )
	{
		if ( [elem object] )
		{
			[miObjects addObject:[elem object]];
			miObjectLengths += [[elem object] WSObject]->GetDuration();
		}
		else
		{
			[osObjects addObject:[elem path]];
		}
	}
	
	if ( [miObjects count] )
	{
		// create a paste command
		cmd = [[[MIPasteLowLevelCommand alloc]
				initWithDocument:[fMainViewController document]
				objectsToPaste:miObjects
				inObject:fParent
				atTime:fFirstPosition.time
				atVPos:fFirstPosition.vpos] autorelease];
		[self addCommand:cmd];
		fFirstPosition.time += miObjectLengths;
	}
	
	if ( [osObjects count] )
	{
		cmd = [[[MICreateWSObjsFromFilesCommand alloc] initWithFiles:osObjects controller:fMainViewController] autorelease];
		[cmd setParent:fParent];
		[cmd setFirstPosition:fFirstPosition];
		[self addCommand:cmd];
			
	}
#endif
	return [super doIt];
}

@end
