/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
#import "MICreateTextClipCommand.h"
#import "MIMainViewController.h"
#import "MISingleMoveCommand.h"
#import "MIEncloseSelectionCommand.h"
#import "MIMontageDocument.h"
#import "MITopLevelContainerRep.h"
#import "MIDeleteCommand.h"
#import "MIUpdateVideoTransitionsCommand.h"
#import "MITextController.h"

#ifndef TArmeDoc_h
#include "TArmeDoc.h"
#endif //TArmeDoc_h

#ifndef TArmeWSObj_h
#include "TArmeWSObj.h"
#endif //TArmeWSObj_h

#ifndef TArmeTextClip_h
#include "TArmeTextClip.h"
#endif //TArmeTextClip_h

/****************************************************************************************************

MICreateTextClipCommand

****************************************************************************************************/

@implementation MICreateTextClipCommand : MICommand

-(id)initFromArchive:(ArmeArchiveHandler*)handler
{
	self = [super initFromArchive:handler];
	if(self)
	{
		fDeleteCommand = [handler->ReadObjCObject() retain];
		fCreatedObject = [handler->ReadObjCObject() retain];
	}
	return self;
}

-(BOOL)writeToArchive:(ArmeArchiveHandler*)handler
{
	if ( [super writeToArchive:handler] )
	{
		handler->WriteObjCObject( fDeleteCommand );
		handler->WriteObjCObject( fCreatedObject );
		return YES;
	}
	else
		return NO;
}

- (void)dealloc
{
	[fCreatedObject release];
	[fDeleteCommand release];
	[super dealloc];
}

- (NSString*)commandName
{
	return @"Create Text Clip";
}

- (id)initWithSelection:(MIWSSelection*)sel
{
	self = [super initWithSelection:sel];

	[self modifiedVideoHierarchy];
	[self overviewNeedsUpdate];

	return self;
}

- (MICmdStatus)doIt
{
	TArmeWSObjPos 			newPos;
	NSPoint					visibleRectOrigin;

	visibleRectOrigin = [[[NSApp mainViewController] topLevelContainerView] visibleRect].origin;

	fArmeDoc->LockRead();
	newPos = fTextClip->GetPos();
	fArmeDoc->UnlockRead();

	newPos.fStartTime = [[[NSApp mainViewController] topLevelContainerView] pixelToArmeTime:(visibleRectOrigin.x+10)];
	newPos.fVerticalPos = GET_VALUE([[[NSApp mainViewController] topLevelContainerView] pixelToMIVPos:(visibleRectOrigin.y+10)]);

	fTextClip->SetPos(newPos);


    fCreatedObject = [[MIObject alloc] initWithArmeWSObj:fTextClip];
    [fCreatedObject setMontageDoc:fMontageDoc];
	[[fMontageDoc topLevelContainer] addChild:fCreatedObject];

	[fMontageDoc lockWrite];
	[fCreatedObject setVerticalPos:[[[NSApp mainViewController] topLevelContainerView] pixelToMIVPos:(visibleRectOrigin.y+10)]];
	[fMontageDoc unlockWrite];

	[fCreatedObject createAllRepresentations];
    [fCreatedObject setRepresentationsNeedDisplay];

//	[self modelChangedForDocument:[[NSDocumentController sharedDocumentController] currentDocument]];
	
	return kMICmdStatusDone;
}

- (void)undoIt
{
	fDeleteCommand = [[MIDeleteCommand alloc] initWithObject:fCreatedObject];
	[fDeleteCommand _doIt];
}

- (void)redoIt
{
	 [fDeleteCommand _undoIt];
	 [fDeleteCommand release];
	 fDeleteCommand = nil;
}

+ (BOOL)supportsSelection:(MIWSSelection*)sel
{
	return YES;
}

- (BOOL)displaySetupDialog
{
	fMontageDoc = [[NSDocumentController sharedDocumentController] currentDocument];
    fArmeDoc = [fMontageDoc armeDocument];

	// GP : release the MITextController.
	MITextController *	theMITextController = [[MITextController alloc] init];
	fTextClip = (TArmeTextClip*)[theMITextController doTextClipDialog:fArmeDoc];
	[theMITextController release];

	if ( !fTextClip )
	{
		return NO;
	}
	else
	{
		return YES;
	}

}
@end


