/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
#import "MICreateVideoProcessorCommand.h"
#import "MIMainViewController.h"
#import "MISingleMoveCommand.h"
#import "MIEncloseSelectionCommand.h"
#import "MITopLevelContainerRep.h"
#import "MIObject.h"
#import "MIMontageDocument.h"
#import "MIDeleteCommand.h"
#import "MIUpdateVideoTransitionsCommand.h"
#import "MIViewerController.h"
#import "MITime.h"

#ifndef TArmeDoc_h
#include "TArmeDoc.h"
#endif //TArmeDoc_h

#ifndef TArmeWSObj_h
#include "TArmeWSObj.h"
#endif //TArmeWSObj_h

#ifndef TVideoEffect_h
#include "TVideoEffect.h"
#endif //TVideoEffect_h

#ifndef MovieHelper_h
#include "MovieHelper.h"
#endif //MovieHelper_h
/****************************************************************************************************

MICreateVideoProcessorCommand

****************************************************************************************************/

@implementation MICreateVideoProcessorCommand : MICommand

-(id)initFromArchive:(ArmeArchiveHandler*)handler
{
	self = [super initFromArchive:handler];
	if ( self )
	{
		fMoveCommand = [handler->ReadObjCObject() retain];
		fEncloseCommand = [handler->ReadObjCObject() retain];
		fDeleteCommand = [handler->ReadObjCObject() retain];
		fCreatedObject = [handler->ReadObjCObject() retain];
		fProcName = [handler->ReadObjCObject() retain];
		
		fNewEffectDesc = NULL;
	}
	return self;
}

-(BOOL)writeToArchive:(ArmeArchiveHandler*)handler
{
	if ( [super writeToArchive:handler] )
	{
		handler->WriteObjCObject( fMoveCommand );
		handler->WriteObjCObject( fEncloseCommand );
		handler->WriteObjCObject( fDeleteCommand );
		handler->WriteObjCObject( fCreatedObject );
		handler->WriteObjCObject( fProcName );
		return YES;
	}
	return NO;
}

- (void)dealloc
{
	[fMoveCommand release];
	[fEncloseCommand release];
	[fCreatedObject release];
	[fProcName release];
	[fDeleteCommand release];
	
	if (fNewEffectDesc)
		QTDisposeAtomContainer(fNewEffectDesc);
	[super dealloc];
}

- (NSString*)commandName
{
	return [NSString stringWithFormat:@"Create %@", fProcName];
}

- (MIObject*)getCreatedObject
{
	return fCreatedObject;
}

- (id)initWithDocument:(MIMontageDocument*)doc selection:(MIWSSelection*)sel
{
	self = [super initWithSelection:sel];
	fMontageDoc = doc;
	
	fNewEffectDesc = NULL;
	
	[self modifiedVideoHierarchy];
	return self;
}

- (void)setPosition:(MIPoint)point
{
	fPosition = point;
}

- (void)setParent:(MIObject*)parent
{
	fParent = parent;
}

- (void)setSet:(int)set index:(int)index name:(NSString*)name
{
	fSet = set;
	fIndex = index;
	fProcName = [name retain];
}


- (MICmdStatus)doIt
{
	TArmeDoc				*armeDoc;
	MIMontageDocument		*mDoc;
	TArmeWSObj				*newArmeObj;
	MIObject				*newObj;
	MITime					*t;

	mDoc = fMontageDoc;
    armeDoc = [mDoc armeDocument];

	newArmeObj = new TVideoEffect(armeDoc, NULL, &fNewEffectDesc ); // <--- armeDoc : ??????

	t = [MITime timeWithDocument:mDoc];
	
	[t setSeconds:10];
	
	newArmeObj->_SetLength([t getArmeTime]);
	
	// copy the children routing, if any
	MEVAORouting newRouting = MIGetSelectionVAORouting(fSelection);
	newArmeObj->SetVAORouting(newRouting);

	char temp[1024];
	GetEffectName(fNewEffectDesc, temp);
	fProcName = [[NSString alloc] initWithCString:temp];

	if ( !newArmeObj )
	{
		verntl(interfaceStephan,0,"Null objct returned while trying to create a processor");
		return kMICmdStatusCancelled;
	}

    newObj = [[MIObject alloc] initWithArmeWSObj:newArmeObj];
    [newObj setMontageDoc:mDoc];
	[newObj createAllRepresentations];

	fMoveCommand = [[MISingleMoveCommand alloc] initWithSelection:nil];
	[fMoveCommand setObject:newObj];
	if ( fParent )
		[fMoveCommand setNewParent:fParent];
	else
		[fMoveCommand setNewParent:[mDoc topLevelContainer]];
	[fMoveCommand setNewOrigin:fPosition];

	[fMoveCommand _doIt];
	
	fCreatedObject = [newObj retain];

	// make the new container enclose the current selection, if it is possible

	fEncloseCommand = [[MIEncloseSelectionCommand alloc] initWithSelection:fSelection];
	[fEncloseCommand setContainer:fCreatedObject];
	if ( ![MIEncloseSelectionCommand supportsSelection:fSelection] )
	{
		[fEncloseCommand release];
		fEncloseCommand = nil;
	}
	[fEncloseCommand _doIt];
	[self overviewNeedsUpdate];
	return kMICmdStatusDone;
}

- (void)undoIt
{
	 [fEncloseCommand _undoIt];

	//we update the transitions now (before deleting the effect) otherwise, the effect is still applied once we have deleted it
	MIUpdateVideoTransitionsCommand* update = [[MIUpdateVideoTransitionsCommand alloc] initWithDocument:[fCreatedObject montageDoc] from:ARME_TIME_UNDEF to:ARME_TIME_UNDEF];
	[update doIt];
	[update release];

	 if ( !fDeleteCommand )
	 {
		 fDeleteCommand = [[MIDeleteCommand alloc] initWithObject:fCreatedObject];
		 [fDeleteCommand _doIt];
	 }
	 else
	 {
		 [fDeleteCommand _redoIt];
	 }
}

- (void)redoIt
{
	 [fDeleteCommand _undoIt];
	 [fEncloseCommand _redoIt];
}

- (BOOL)displaySetupDialog
{
	PicHandle posterSrc1 = NULL;
	TVideoObject* video = NULL;
	unsigned int i=0;

	[[MIViewerController sharedViewerController] setMovie:nil];
	
	QTNewAtomContainer(&fNewEffectDesc);
	
	// configure the effect
		
	while (!video && (i < [fSelection count]))
	{
		if ([[[fSelection objectAtIndex:i] representedObject] WSObject]->ContainsVideo())
			video = (TVideoObject*)([[[fSelection objectAtIndex:i] representedObject] WSObject]);
			
		i++;
	}
	
	if (video)
		posterSrc1 = video->fMasterMovie->GetPictAtTime(video, NULL, video->GetAbsoluteStartTimeOfVideoClip());;

		bool flag = ConfigureEffect(fNewEffectDesc, 1, posterSrc1);
		[[MIViewerController sharedViewerController] show:nil];
		if (!flag)
		{
			return NO;
		}
		else
			return YES;
}

+ (BOOL)supportsSelection:(MIWSSelection*)sel
{
	// it works also without a selection
	if (!sel)
		return YES;

	if ([sel count] == 0)
		return YES;

	if ( [sel containsOnlyVideoTransitions])
		return NO;

	if ( [sel type] != kMIWSSelectionTypeObjects )
		return NO;
		
	return YES;
}

@end