/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MICreateWSObjFromFileCommand.h"
#import "MIMontageDocument.h"
#import "MIShrinkToFitCommand.h"
#import "MISingleMoveCommand.h"
#import "MIObjectRep.h"
#import "MIMainViewController.h"
#import "MIApplicationController.h"
#import "MIDeleteCommand.h"

#ifndef TArmeDoc_h
#include "TArmeDoc.h"
#endif //TArmeDoc_h

#ifndef TArmeWSObj_h
#include "TArmeWSObj.h"
#endif //TArmeWSObj_h

#ifndef ZzSplitTask_h
#include "ZzSplitTask.h"
#endif //ZzSplitTask_h

/****************************************************************************************************

MICreateWSObjFromFileCommand

****************************************************************************************************/


@implementation MICreateWSObjFromFileCommand : MICommand

-(id)initFromArchive:(ArmeArchiveHandler*)handler
{
	ZzAssert(0);
	vverntl(archivingEngine,0,"please implement MICreateWSObjFromFileCommand :: initFromArchive:");
	self = [super initFromArchive:handler];
	if(self)
	{
		fDeleteCommand = [handler->ReadObjCObject() retain];
		fSplitTask = nil;
	}
	return self;
}
#if 0

-(BOOL)writeToArchive:(ArmeArchiveHandler*)handler
{
	vverntl(archivingEngine,0,"please implement MICreateWSObjFromFileCommand :: writeToArchive:");
	[super writeToArchive:handler];
	handler->WriteObjCObject(fDeleteCommand);
	return YES;
}

- (void)dealloc
{
	[fDeleteCommand release];
	[fTheFile release];
}

-(void)setPosition:(MIPoint)position
{
	fPosition = position;
}

- (NSString*)commandName
{
	return @"Create Clip";
}

- (void)setFile:(NSString*)file
{
	fTheFile = [[file copy] retain];
}

- (void)setMontageDoc:(MIMontageDocument*)doc
{
	fMontageDoc = doc;
}

- (MIObject*)getCreatedObject
{
	return fCreatedObject;
}

- (id)initWithDocument:(MIMontageDocument*)doc
{
	self = [super initWithSelection:nil];
	if ( self )
	{
		fMontageDoc = doc;
		fSplitTask = nil;
		fDeleteCommand = nil;
		
		//$$ is there a way to know what is created
		[self modifiedVideoHierarchy];
		[self modifiedAudioHierarchy];
		[self overviewNeedsUpdate];
	
	}
	
	return self;
}

- (MICmdStatus)doIt
{
    TArmeDoc				*armeDoc;
    MIMontageDocument		*mDoc;
    TArmeWSObj				*newArmeObj;
    MIObject				*newObj;
	BOOL					isContainer = NO;
	int						count;
	int						i;

    //mDoc = [[currentWindow delegate] document];
    mDoc = fMontageDoc;
    armeDoc = [mDoc armeDocument];

	ZzAssert(armeDoc != NULL);

	ArmeErr err;
    // FSPathMakeRef doesn't like [fTheFile cString]
    //newArmeObj = armeDoc->CreateWSObjFromFile( [fTheFile cString], armeDoc->GetZeroPos(), fBopTask, &err );
    // NSPathUtilites.h seems to have stuff that addresses a number of path related issues
    // dealt with by hand coding in various places in HyperEngine-AV ...alex
    ZzAssert(0); // the following call is not available anymore
	//newArmeObj = armeDoc->CreateWSObjFromFile( 
	//	[fTheFile fileSystemRepresentation], armeDoc->GetZeroPos(), [MICommand getCurrentProgressDialog], &err );

    if ( !newArmeObj || err.type != kArmeOK )
    {
		NSString* msg1 = @"Error while opening file : ";
		NSString* msg2 = [msg1 stringByAppendingString:fTheFile];
		NSString* msg3 = [msg2 stringByAppendingString:@"\nThe error is : "];
		NSString* msg4 = [msg3 stringByAppendingString:[NSString stringWithCString:ArmeGetStringFromError(err)]];
		NSRunAlertPanel(@"Error", msg4, @"Continue", nil, nil);
		if(fSplitTask)
		{
			fSplitTask->Cancel();
			delete fSplitTask;
			fSplitTask = nil;
		}
		return kMICmdStatusCancelled;
    }
	else
	{
	// don't do anything......the caller will execute the split task
/*

		// $$$TODO need to use document, not view, because a file drop in a document can lead to a progress bar
		// in 2 windows (since the document is created in 2 instances of the doc
		BOOL done = YES;
		if(splitTask)
		{
			done = [[MIApplicationController sharedApplicationController] 
				executeAndDeleteSplitTask:splitTask 
				forMainViewController:[NSApp mainViewController]];
		}
		if(!done)
		{
			delete newArmeObj;
			return kMICmdStatusCancelled;
		}
*/
	}
		
	newObj = [[MIObject alloc] initWithArmeWSObj:newArmeObj];
	fCreatedObject = newObj;
	
	if ( [newObj getType] == TArmeWSObj::kContainer )
		isContainer = YES;
	
    [newObj setMontageDoc:mDoc];
    [[mDoc topLevelContainer] addChild:newObj];
	[newObj createAllRepresentations];

	if ( isContainer )
	{
		MIShrinkToFitCommand 	*shrink;
		MISingleMoveCommand		*cmd;
		MIPoint					newOrigin = MIZeroPoint;
		// create creation commands to create children and move them accordingly (and resize the parent container)
		// no need to register these commands as the delete command will take care of the whole hierarchy of children

		count = newArmeObj->GetChildrenCount();
		for ( i = 0; i < count; i++ )
		{
			newObj = [[MIObject alloc] initWithArmeWSObj:newArmeObj->GetChild(i)];
			[newObj setMontageDoc:mDoc];
//			[fCreatedObject addChild:newObj];
			[newObj createAllRepresentations];
			
			cmd = [[MISingleMoveCommand alloc] initWithSelection:fSelection];
			[cmd setObject:newObj];
			[cmd setNewParent:fCreatedObject];
			[cmd setNewOrigin:newOrigin];
			[cmd _doIt];
			[cmd release];
#ifdef SMALL_TITLEBAR
			GET_VALUE(newOrigin.vpos) += GET_VALUE(kDefaultClipRepresentationHeight);
#else
			GET_VALUE(newOrigin.vpos) += kDefaultClipRepresentationHeight;
#endif

		}

		shrink = [[MIShrinkToFitCommand alloc] initWithSelection:nil];
		[shrink setObject:fCreatedObject];
		[shrink _doIt];
		[shrink release];
	}
	return kMICmdStatusDone;
}

- (void)undoIt
{

	if ( !fDeleteCommand )
	{
		fDeleteCommand = [[MIDeleteCommand alloc] initWithObject:fCreatedObject];
		[fDeleteCommand _doIt];
	}
	else
	{
		[fDeleteCommand _redoIt];
	}
}

- (void)redoIt
{
	[fDeleteCommand _undoIt];
}

- (ZzSplitTask*)getSplitTask
{
	return fSplitTask;
}

#endif //#if 0

@end
