/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
#import "MontageInterface.h"
#import "MICommand.h"

@class MIMainViewController;
@class MIObject;
@class MIDeleteCommand;
@class MIMontageDocument;
@class MIResizeCommand;
class ZzSplitTask;
class ArmeDataFileRef;

@interface MICreateWSObjsFromFileCommand : MICommand
{
	// we only need to archive this one
	NSMutableArray*			fDeleteCommands;
	
	NSString*				fTheFileName;
	ArmeDataFileRef*		fTheFileRef;
	NSMutableArray*			fCreatedObjects;
	NSMutableArray*			fSingleMoveCommands;
	MIMontageDocument*		fMontageDoc;
	MIVPos					fTotalHeight;
	ArmeTime				fMaxDuration;
	MIPoint					fOrigin;
	MIObject*				fParent;
	MIResizeCommand*		fEnlargeCommand;
	NSString*				fTheClipName;
}

// creation stuff
- (id)initFromArchive:(ArmeArchiveHandler*)handler;
- (BOOL)writeToArchive:(ArmeArchiveHandler*)handler;
- (id)initWithDocument:(MIMontageDocument*)doc;
- (void)setMontageDoc:(MIMontageDocument*)doc;

// execution params
- (void)setFileName:(NSString*)fileName;
- (void)setFileRef:(ArmeDataFileRef*)fileRef;
- (void)setOrigin:(MIPoint)pos; // local coords
- (void)setParent:(MIObject*)parent;
- (void)setClipName:(NSString*)name;

// post doIt infos
- (NSMutableArray*)createdObjects;
- (MIVPos)totalHeight;
- (ArmeTime)maxDuration;


// the meat
- (MICmdStatus)doIt;
- (void)undoIt;
- (void)redoIt;

@end