/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MICreateWSObjsFromFileCommand.h"
#import "MIMontageDocument.h"
#import "MIShrinkToFitCommand.h"
#import "MISingleMoveCommand.h"
#import "MIObjectRep.h"
#import "MIMainViewController.h"
#import "MIApplicationController.h"
#import "MIDeleteCommand.h"
#import "MIResizeCommand.h"
#import "MIApplication.h"

#ifndef ZzString_h
#include "ZzString.h"
#endif //ZzString_h

#ifndef TArmeDoc_h
#include "TArmeDoc.h"
#endif //TArmeDoc_h

#ifndef TArmeWSObj_h
#include "TArmeWSObj.h"
#endif //TArmeWSObj_h

#ifndef ZzSplitTask_h
#include "ZzSplitTask.h"
#endif //ZzSplitTask_h

#ifndef ArmeDataFileRef_h
#include "ArmeDataFileRef.h"
#endif //ArmeDataFileRef_h

/****************************************************************************************************

MICreateWSObjsFromFileCommand

****************************************************************************************************/


@implementation MICreateWSObjsFromFileCommand : MICommand

-(id)initFromArchive:(ArmeArchiveHandler*)handler
{
	self = [super initFromArchive:handler];
	if(self)
	{
		fDeleteCommands = [handler->ReadObjCObject() retain];
		fEnlargeCommand = [handler->ReadObjCObject() retain];
		if (handler->GetFileVer() >= 0x0001002B)
		{
			fCreatedObjects = [handler->ReadObjCObject() retain];
		}
		else
		{
			fCreatedObjects = nil;
		}
	}
	fParent = nil; // useless post-arch
	return self;
}

-(BOOL)writeToArchive:(ArmeArchiveHandler*)handler
{
	[super writeToArchive:handler];
	handler->WriteObjCObject(fDeleteCommands);
	handler->WriteObjCObject(fEnlargeCommand);

	//from version 0x0001002B
	handler->WriteObjCObject(fCreatedObjects);
	return YES;
}

/*
-(void)setPosition:(MIPoint)position
{
	fPosition = position;
}
*/
- (NSString*)commandName
{
	return @"Create Clip";
}

- (void)setFileName:(NSString*)fileName
{
	fTheFileName = [[fileName copy] retain];
}

- (void)setFileRef:(ArmeDataFileRef*)fileRef
{
	fTheFileRef = fileRef;
}

- (void)setMontageDoc:(MIMontageDocument*)doc
{
	fMontageDoc = doc;
}

- (NSMutableArray*)createdObjects
{
	return fCreatedObjects;
}

- (id)initWithDocument:(MIMontageDocument*)doc
{
	self = [super initWithSelection:nil];
	if ( self )
	{
		fMontageDoc = doc;
		fDeleteCommands = nil;
		fEnlargeCommand = nil;
		fCreatedObjects = nil;
		fParent = nil;
		fSingleMoveCommands = nil;
		fTotalHeight = _MIVPos(0);
		fMaxDuration = _ArmeTime(0);
		fOrigin = MIZeroPoint;
		fTheFileName = nil;
		fTheClipName = nil;
		fTheFileRef = nil;
		
		//$$ is there a way to know what is created
		[self modifiedVideoHierarchy];
		[self modifiedAudioHierarchy];
		[self overviewNeedsUpdate];
	}
	
	return self;
}

- (MICmdStatus)doIt
{
    TArmeDoc				*armeDoc;
    MIMontageDocument		*mDoc;
//  TArmeWSObj				*newArmeObj;
//  MIObject				*newObj;
//	BOOL					isContainer = NO;
//	int						count;
//	int						i;
	ArmeErr					err;

	
	fTotalHeight = _MIVPos(0);
	fMaxDuration = _ArmeTime(0);


	fCreatedObjects = [[NSMutableArray alloc] init];

    //mDoc = [[currentWindow delegate] document];
    mDoc = fMontageDoc;
    armeDoc = [mDoc armeDocument];

	ZzAssert(armeDoc != NULL);
	
	ZzArchPtrArray theObjects;
	
	TArmeWSObjPos pos;
	pos.fParent = [fParent WSObject];
	pos.fStartTime = fOrigin.time;
	pos.fInput = 0;
	pos.fVerticalPos = GET_VALUE(fOrigin.vpos);
	
	VUSERCMD("Importing Files");
	VUSERCMD3("- at startTime = %d with vpos = %d, importing %s ", (int)(fOrigin.time.data), (int)(fOrigin.vpos.data), [fTheFileName cString]);
	
	ZzAssert(fParent != nil);
	
	ZzString fileName(fTheFileName);
	err = armeDoc->CreateWSObjsFromFile(
		&theObjects, 
		&fileName, 
		fTheFileRef,
		pos, 
		[MICommand getCurrentProgressDialog] );

	if ( err.type != kArmeOK )
	{
		// first we need to delete the created TArmeWSObj objects
		for(SInt32 i = 0; i < theObjects.CountItems(); i++)
		{
			TArmeWSObj* obj = dynamic_cast<TArmeWSObj*> (theObjects.ItemAt(i));
			ZzAssert(obj != NULL);
			obj->SetOuttaHierarchy();
			delete obj;
		}
		// we let the automatic theObjects dtor take care of emptying the array
	
		NSString* title = [NSString stringWithFormat:@"Unable to open file \"%@\"", [fTheFileName lastPathComponent]];
		NSString* msg = [NSString stringWithFormat:@"in file \"%@\"", fTheFileName];
		NSString* error = [NSString stringWithFormat:@"Error %d %@", ((int)err.data) ? ((int)err.data) : err.type, [NSString stringWithCString:ArmeGetStringFromError(err)]];
		NSString* msg2 = [NSString stringWithFormat:@"%@ has occured.", ((int)err.data) ? @"A QuickTime related error" : @"An internal error"];

		MIRunErrorPanel( title, msg2, error, msg );
		return kMICmdStatusCancelled;
	}
	else
	{
		for(SInt32 i = 0; i < theObjects.CountItems(); i++)
		{
			TArmeWSObj* wsobj = dynamic_cast<TArmeWSObj*>(theObjects.ItemAt(i));
			ZzAssert(wsobj);
			if ( fTheClipName ) 
				wsobj->SetNameCopy( [fTheClipName UTF8String] );
			MIObject* obj = [[[MIObject alloc] initWithArmeWSObj:wsobj] autorelease];
			[fCreatedObjects addObject:obj];
			ZzAssert( [obj getType] != TArmeWSObj::kContainer ); // it CANNOT be a container. it must be a clip
			[obj setMontageDoc:mDoc];
			[obj setVerticalPos:fOrigin.vpos + fTotalHeight];
			[fParent addChild:obj];
			[obj createAllRepresentations];
			[obj setRepresentationsNeedDisplay];
			fTotalHeight += [obj getHeight] + kBetweenClipsHeight;
			if(GET_VAL(wsobj->GetDuration()) > GET_VAL(fMaxDuration))
				fMaxDuration = wsobj->GetDuration();
		}

		// check if parent size is big enough
		if ( !MIContainsRect( [fParent modelFrame], [fParent minimalRectToFitContent] ) )
		{
			fEnlargeCommand = [[MIResizeCommand alloc] initWithSelection:nil];
			[fEnlargeCommand setObject:fParent newFrame:MIUnionRect([fParent modelFrame], [fParent minimalRectToFitContent])];
			[fEnlargeCommand _doIt];
		}
		
	}
	return kMICmdStatusDone;
}

- (void)undoIt
{
	if ( !fDeleteCommands )
	{
		fDeleteCommands = [[NSMutableArray alloc] init];
		for(UInt32 i = 0; i < [fCreatedObjects count]; i++)
		{
			MIDeleteCommand* delCmd = [[[MIDeleteCommand alloc] initWithObject:[fCreatedObjects objectAtIndex:i]] autorelease];
			[fDeleteCommands addObject:delCmd];
			[delCmd _doIt];
		}
	}
	else
	{
		for(UInt32 i = 0; i < [fDeleteCommands count]; i++)
		{
			[[fDeleteCommands objectAtIndex:i] _redoIt];
		}
	}
	[fEnlargeCommand _undoIt];
}

- (void)redoIt
{
	for(SInt32 i = [fDeleteCommands count]-1; i >= 0 ; i--)
	{
		[[fDeleteCommands objectAtIndex:i] _undoIt];
	}
	[fEnlargeCommand _redoIt];
}

- (void)dealloc
{
	[fEnlargeCommand release];
	[fDeleteCommands release];
	[fTheFileName release];
	[fTheClipName release];
	if (fTheFileRef)
		fTheFileRef->Release();
	[fCreatedObjects release];
    [super dealloc];
}

- (void)setOrigin:(MIPoint)pos
{
	fOrigin = pos;
}

- (void)setParent:(MIObject*)parent
{
	fParent = parent;
}

- (MIVPos)totalHeight
{
	return fTotalHeight;
}

- (ArmeTime)maxDuration
{
	return fMaxDuration;
}

- (void)setClipName:(NSString*)name {
	if ( name != fTheClipName ) {
		[fTheClipName release];
		fTheClipName = [name copy];
	}

}
@end
