/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
#import "MICreateWSObjsFromFilesCommand.h"
#import "MICreateWSObjsFromFileCommand.h"
#import "MIObject.h"
#import "MIObjectRep.h"
#import "MIMainViewController.h"
#import "MIPreferences.h"
#import "MIApplicationController.h"

#ifndef ZzSplitTask_h
#include "ZzSplitTask.h"
#endif //ZzSplitTask_h

#ifndef ZzHeapWalker_h
#include "ZzHeapWalker.h"
#endif ZzHeapWalker_h

/****************************************************************************************************

MICreateWSObjsFromFilesCommand

****************************************************************************************************/

#undef VER_DEF_TOPIC
#define VER_DEF_TOPIC commandsStephan 

@implementation MICreateWSObjsFromFilesCommand : MICommand

-(id)initFromArchive:(ArmeArchiveHandler*)handler
{
	self = [super initFromArchive:handler];
	if(self)
	{
		fFileArray = [handler->ReadObjCObject() retain];
		fSubCommands = [handler->ReadObjCObject() retain];
	}
	return self;
}

-(BOOL)writeToArchive:(ArmeArchiveHandler*)handler
{
	[super writeToArchive:handler];
	handler->WriteObjCObject(fFileArray);
	handler->WriteObjCObject(fSubCommands);
	return YES;
}

- (void)dealloc
{
	[fFileArray release];
	[fSubCommands release];
	[super dealloc];
}

- (NSString*)commandName
{
	return @"Create Clips";
}

- (id)initWithFiles:(NSArray*)files controller:(MIMainViewController*)controller;
{
	self = [super initWithSelection:nil];
	if ( self )
	{
		fFileArray = [files retain];
		fController = controller;
		fSubCommands = nil;
		
		// is there a way to know what is affected ?
		[self modifiedVideoHierarchy];
		[self modifiedAudioHierarchy];
		[self overviewNeedsUpdate];
	}

	return self;
}

- (void)setParent:(MIObject*)parent
{
	if(parent)
		fParent = parent;
	else // this means TLC
		fParent = [fController getTLC];
	ZzAssert(fParent != nil);
}

- (void)setFirstPosition:(MIPoint)position
{
	fPosition = position;
}

- (void)chooseStackingMethodSheetDidEnd:(NSWindow*)sheet returnCode:(int)returnCode contextInfo:(void*)contextInfo
{
	vernl(6,"chooseStackingMethodSheetDidEnd returnCode = " << returnCode);
	UInt32* pRetCode = (UInt32*)contextInfo;
	*pRetCode = returnCode;
}

-(MICmdStatus)doIt
{
	NSEnumerator*						fileEnum = [fFileArray objectEnumerator];

	// retCode is the value returned by the stackingSeetRetCode
	bool bStackVertically = true;
		
	if([fFileArray count] > 1)
	{
		SInt32 val = [[MIPreferences sharedPrefs] stackingMethod];
		vernl(6,"[[MIPreferences sharedPrefs] stackingMethod] returned " << val);
		if(val == kPrefsStackingMethodAlwaysAsk)
		{
			val = [[MIApplicationController sharedApplicationController] displayChooseStackingMethodForMainViewController:fController];
		}
		vernl(6,"displayChooseStackingMethodForMainViewController: returned " << val);

		if(val == kPrefsStackingMethodCancelled)
			return kMICmdStatusCancelled;
		else if(val == kPrefsStackingMethodVertically)
		{
			fileEnum = [fFileArray reverseObjectEnumerator];
			bStackVertically = true;
		}
		else if(val == kPrefsStackingMethodHorizontally)
			bStackVertically = false;
		else
			ZzAssert(0);
	}
		
	NSString*							aFile;
	MICreateWSObjsFromFileCommand*		createClipsCommand;
	ZzMacroSplitTask*					macroSplitTask;
	heapwalker(macroSplitTask = new ZzMacroSplitTask());
	
	fSubCommands = [[NSMutableArray alloc] init];
	
	// let's create a sub MICreateMoveWSObjFromFileCommand for each file dropped
	[MICommand getCurrentProgressDialog]->SetNbSubTasks([fFileArray count]);
	
	while ( aFile = [fileEnum nextObject] )
	{
		createClipsCommand = [[[MICreateWSObjsFromFileCommand alloc] initWithDocument:[[fController getTLC] montageDoc]] autorelease];
		[createClipsCommand setFileName:aFile];
		[createClipsCommand setOrigin:fPosition];
		[createClipsCommand setParent:fParent];
		[MICommand getCurrentProgressDialog]->EnterSubTask();
		MICmdStatus st = [createClipsCommand doIt];
		[MICommand getCurrentProgressDialog]->LeaveSubTask();
		
		// if the command was cancelled, we can release it and pass to the next
		if(st == kMICmdStatusCancelled)
		{
			return kMICmdStatusCancelled;
		}
		// otherwise, we add the command to the list of command that we keep for undoing
		// purposes, and we increment the "position of next created object" according to
		// the loaded object size and the stacking method
		else if(st == kMICmdStatusDone)
		{
			[fSubCommands addObject:createClipsCommand];
			if(bStackVertically)
				GET_VALUE(fPosition.vpos) += GET_VALUE([createClipsCommand totalHeight]);
			else
				GET_VALUE(fPosition.time) += GET_VALUE([createClipsCommand maxDuration]);
		}
		else
		{
			ZzAssert(0);
		}
	}
	if([fFileArray count] == 0)
	{
		return kMICmdStatusCancelled;
	}
	else
	{
		return kMICmdStatusDone;
	}
}

- (void)undoIt
{
	NSEnumerator* subCmdRevEnum = [fSubCommands reverseObjectEnumerator];
	MICommand* cmd;
	while(cmd = [subCmdRevEnum nextObject])
	{
		[cmd undoIt];
	}
}

- (void)redoIt
{
	NSEnumerator* subCmdEnum = [fSubCommands objectEnumerator];
	MICommand* cmd;
	while(cmd = [subCmdEnum nextObject])
	{
		[cmd redoIt];
	}
}

@end

