/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MIDeleteCommand.h"
#import "MIMainViewController.h"
#import "MIMontageDocument.h"
#import "MIObjectRep.h"
#import "MIObject.h"
#import "MITopLevelContainerRep.h"
#import "MIDeleteLowLevelCommand.h"
#import "MIShrinkToFitCommand.h"
#import "MIPreferences.h"
#import "MIApplication.h"
#import "MIApplicationController.h"

/****************************************************************************************************

MIDeleteCommand

****************************************************************************************************/

@implementation MIDeleteCommand

- (void)dealloc
{
//	printf("-[MIDeleteCommand dealloc]");
	[super dealloc];
}

-(id)initFromArchive:(ArmeArchiveHandler*)handler
{
	self = [super initFromArchive:handler]; // it's a macro command
	return self;
}

-(BOOL)writeToArchive:(ArmeArchiveHandler*)handler
{
	[super writeToArchive:handler]; // it's a macro command
	return YES;
}

- (NSString*)commandName
{
	return @"Delete";
}

- (id)initWithListOfObjects:(NSArray*)listOfObjects;
{
	self = [super initWithSelection:nil];
	fStartTime = ARME_TIME_UNDEF;
	fEndTime = ARME_TIME_UNDEF;

	//for each object in the list (and each of its children), create a low level delete command.
	NSEnumerator* objEnum = [listOfObjects objectEnumerator];
	MIObject* obj;

	while (obj = [objEnum nextObject])
	{
		[self createListOfCommandsToDelete:obj];
	}

	return self;
}

- (id)initWithSelection:(MIWSSelection*)sel
{
	self = [super initWithSelection:sel];
	[fSelection adjustSelectionTimesToVideo];	
	if ( self )
	{
		[fSelection discardVideoTransitions];
	
		if ([fSelection type] == kMIWSSelectionTypeTimeSlice)
		{
			fStartTime = [fSelection startTime];
			fEndTime = [fSelection endxTime];
		}
		else
		{
			fStartTime = ARME_TIME_UNDEF;
			fEndTime = ARME_TIME_UNDEF;
		}
		
		//for each object in the selection (and each of its children), create a low level delete command.
		//NSEnumerator* selEnum = [fSelection objectEnumerator];
		MIObjectRep* objRep;
		for(UInt32 i = 0; i < [fSelection count]; i++)
		//while (objRep = [selEnum nextObject])
		{
			objRep = [fSelection objectAtIndex:i];
			[self createListOfCommandsToDelete:[objRep representedObject]];
		}
	}
	return self;
}

- (id)initWithObject:(MIObject*)object from:(ArmeTime)startTime to:(ArmeTime)endTime
{
	self = [super initWithSelection:nil];
	fStartTime = startTime;
	fEndTime = endTime;
	[self createListOfCommandsToDelete:object];

	return self;
}

- (id)initWithObject:(MIObject*)object
{
	self = [super initWithSelection:nil];
	if ( self )
	{
		fStartTime = ARME_TIME_UNDEF;
		fEndTime = ARME_TIME_UNDEF;
		[self createListOfCommandsToDelete:object];
	}
	return self;
}


- (void)createListOfCommandsToDelete:(MIObject*)object
{
	//NSEnumerator* childEnum = [[object getChildren] objectEnumerator];
	MIObject* child;

	//first, delete the children and then the parents
	for(UInt32 i = 0; i < [[object getChildren] count]; i++)
	//while (child = [childEnum nextObject])
	{
		child = [[object getChildren] objectAtIndex:i];
		[self createListOfCommandsToDelete:child];
	}

//	if ([object WSObject]->GetType() != TArmeWSObj::kVideoXFadeEffect)  // transitions will be deleted automatically
	[self addCommand:[[[MIDeleteLowLevelCommand alloc] initWithObjectToDelete:object  from:fStartTime to:fEndTime] autorelease]];

#undef BENJA_TEST_FIX_BUG
#if BENJA_TEST_FIX_BUG
	if ( [object parentObject] && [[object parentObject] isTLC] )
	{
		// when deleting an object that is direct child of the TLC, shrink the TLC so as not to leave a large white space on the right
		MIShrinkToFitCommand *cmd;
		cmd = [[[MIShrinkToFitCommand alloc] initWithSelection:nil] autorelease];
		[cmd setObject:[object parentObject]];
		[self addCommand:cmd];
	}
#endif //#ifdef BENJA_TEST_FIX_BUG
}

- (BOOL)displaySetupDialog
{
	if ([fSelection type] == kMIWSSelectionTypeTimeSlice)
	{
		if ([self CheckFrozenObjectInSelectionAndAskIfUserWantsToContinue:fSelection] == NO)
			return NO;
	}
	return YES;
}

- (MICmdStatus)doIt
{
	MICmdStatus st = [super doIt];
//	[fController clearSelection];
	if ( [[MIPreferences sharedPrefs] gridShowMask] & kPrefsGridShowVObjLines ||
		 [[MIPreferences sharedPrefs] gridShowMask] & kPrefsGridShowHObjLines )
		[[MIApplicationController sharedApplicationController] redrawAllMainWindows];
	
//	[[fController topLevelContainerView] setNeedsDisplay:YES];
	return st;
}

- (void)undoIt
{
	[super undoIt];
	if ( [[MIPreferences sharedPrefs] gridShowMask] & kPrefsGridShowVObjLines ||
		 [[MIPreferences sharedPrefs] gridShowMask] & kPrefsGridShowHObjLines )
		[[MIApplicationController sharedApplicationController] redrawAllMainWindows];
	
//	[[fController topLevelContainerView] setNeedsDisplay:YES];
}

- (void)redoIt
{
	[super redoIt];
	if ( [[MIPreferences sharedPrefs] gridShowMask] & kPrefsGridShowVObjLines ||
		 [[MIPreferences sharedPrefs] gridShowMask] & kPrefsGridShowHObjLines )
		[[MIApplicationController sharedApplicationController] redrawAllMainWindows];
	
//	[[fController topLevelContainerView] setNeedsDisplay:YES];
}

+ (BOOL)supportsSelection:(MIWSSelection*)sel
{
	// you can't delete video transitions
	if ([sel containsOnlyVideoTransitions])
		return NO;

	if ( [sel type] == kMIWSSelectionTypeTimeSlice ) return YES;
	
	// you can't delete nothing
	if ( [sel count] == 0 ) return NO;
	
	return YES;
}

@end

