/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MIDuplicateCommand.h"


@implementation MIDuplicateCommand

- (void)dealloc
{
	[super dealloc];
}

-(id)initFromArchive:(ArmeArchiveHandler*)handler
{
	self = [super initFromArchive:handler];
	
	return self;
}

-(BOOL)writeToArchive:(ArmeArchiveHandler*)handler
{
	[super writeToArchive:handler];
	return YES;
}

- (NSString*)commandName
{
	return @"Duplicate";
}

- (id)initWithSelection:(MIWSSelection*)sel
{
	self = [super initWithSelectionForSimpleDuplicate:sel];
	[fSelection adjustSelectionTimesToVideo];	
	[fSelection discardVideoTransitions];

	[self createSubCommands];
	return self;
}

- (BOOL)displaySetupDialog
{
	return YES;
}

+ (BOOL)supportsSelection:(MIWSSelection*)sel
{
	// you can't duplicate video transitions
	if ([sel containsOnlyVideoTransitions])
		return NO;

	if ( [sel count] == 0 ) return NO;

	if ( [sel type] == kMIWSSelectionTypeObjects
#ifdef AUTOMATION_DATA
		|| [sel type] == kMIWSSelectionTypeAutomationPoints
#endif
		 )
		return YES;
	else if ( [sel type] == kMIWSSelectionTypeTimeSlice )
	{
		if ([sel duration] > ARME_TIME_ZERO)
			return YES;
	}
	return NO;
}

@end
