/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MIEditTextCommand.h"

#import "MIObject.h"
#import "MIObjectRep.h"

#ifndef TArmeTextClip_h
#include "TArmeTextClip.h"
#endif //TArmeTextClip_h

@implementation MIEditTextCommand

-(id)initFromArchive:(ArmeArchiveHandler*)handler
{
	self = [super initFromArchive:handler];
	
	fMementoBefore = [handler->ReadObjCObject() retain];
	fMementoAfter = [handler->ReadObjCObject() retain];
	fTextClip = [handler->ReadObjCObject() retain]; 
	return self;
}

-(BOOL)writeToArchive:(ArmeArchiveHandler*)handler
{
	[super writeToArchive:handler];
	
	handler->WriteObjCObject(fMementoBefore);
	handler->WriteObjCObject(fMementoAfter);
	handler->WriteObjCObject(fTextClip);
	
	return YES;
}

- (void)dealloc
{
	[fMementoBefore release];
	[fMementoAfter release];
	[fTextClip release];
	[super dealloc];
}

- (NSString*)commandName
{
	return @"Edit Text Clip";
}

- (BOOL)displaySetupDialog
{
	return YES;
}

- (id)initWithSelection:(MIWSSelection*)sel
{
	self = [super initWithSelection:sel];
	
	if ( self )
	{
		fTextClip = [[sel objectAtIndex:0] representedObject];
		[fTextClip retain];
//		printf("STEP 1 [fTextClip retainCount] = %d\n",[fTextClip retainCount]);
		fMementoBefore = nil;
		fMementoAfter = nil;
		[self modifiedVideoHierarchy];
	}
	return self;
}

+ (BOOL)supportsSelection:(MIWSSelection*)sel
{
	if ( [sel type] != kMIWSSelectionTypeObjects )
		return NO;
	else
	{
		if ( [sel count] == 0 ) return NO;
		// the selection can only contain textClips
		NSEnumerator*	objEnum = [sel objectEnumerator];
		MIObjectRep*	objectRep;
	
		while ( objectRep = [objEnum nextObject] )
		{
			if ( [[objectRep representedObject] WSObject]->GetType() != TArmeWSObj::kTextClip )
				return NO;
		}
		if ( [sel count] == 1)
			return YES;
		else
			return NO;
	}
}

- (MICmdStatus)doIt
{
	fMementoBefore = [[fTextClip getMemento] retain];

	if (!((TArmeTextClip*)[fTextClip WSObject])->EditText())
	{
		return kMICmdStatusCancelled;
	}

	[fTextClip setRepresentationsNeedDisplay];
	[fTextClip invalidateRep];
	fMementoAfter = [[fTextClip getMemento] retain];
	
	return kMICmdStatusDone;
}

-(void)undoIt
{
	[fTextClip setMemento:fMementoBefore];
}

-(void)redoIt
{
	[fTextClip setMemento:fMementoAfter];
}
@end
