/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
#import "MIFadeInCommand.h"
#import "MIAudioProcessingCommand.h"
#import "MIObject.h"
#import "MIObjectRep.h"
#import "MICopyCommand.h"
#import "MIDeleteCommand.h"
#import "MIFreezeCommand.h"
#import "MIInsertLowLevelCommand.h"
#import "MIDeleteLowLevelCommand.h"
#import "MIPasteLowLevelCommand.h"
#import "MIMontageDocument.h"

#ifndef OfflineAudioProcessorFadeIn_h
#include "OfflineAudioProcessorFadeIn.h"
#endif 

#ifndef TVideoEffect_h
#include "TVideoEffect.h"
#endif

/****************************************************************************************************

MIFadeInCommand

****************************************************************************************************/

@implementation MIFadeInCommand

- (void)dealloc
{
	[super dealloc];
} 

-(id)initFromArchive:(ArmeArchiveHandler*)handler
{
	self = [super initFromArchive:handler];
	return self;
}

-(BOOL)writeToArchive:(ArmeArchiveHandler*)handler
{
	return [super writeToArchive:handler];
}

- (NSString*)commandName
{
	return @"fadeIn";
}

- (BOOL)displaySetupDialog
{
	if ([self CheckFrozenObjectInSelectionAndAskIfUserWantsToContinue:fSelection] == NO)
		return NO;
	else
		return YES;
}

- (id)initWithSelection:(MIWSSelection*)sel
{
	self = [super initWithSelection:sel];

	[fSelection adjustSelectionTimesToVideo];
	
	if ( self )
	{
		NSEnumerator* selEnum = [sel objectEnumerator];
		MIObject* obj;
		while (obj = [[selEnum nextObject] representedObject])
		{
			if ([obj WSObject]->GetType() == TArmeWSObj::kAudioClip)
			{
				[self addCommand:[[[MIAudioProcessingCommand alloc] initWithSelection:fSelection
				offlineAudioProcessor:OfflineAudioProcessorFadeIn::GetInstance()] autorelease]];
			}
			else if ([obj WSObject]->GetType() == TArmeWSObj::kVideoClip)
			{
				[self modifiedVideoHierarchy];
			
				if ([sel type] == kMIWSSelectionTypeTimeSlice)
				{
					ArmeTime selStartTime = [sel startTime];
					ArmeTime selEndTime = [sel endxTime];
					[obj WSObject]->GetDoc()->RoundArmeTimeToNearestVideoFrame(selStartTime);
					[obj WSObject]->GetDoc()->RoundArmeTimeToNearestVideoFrame(selEndTime);
									
					[self addCommand:[[[MIFadeVideoCommand alloc] initWithObjectToFade:obj from:selStartTime to:selEndTime fadeType:kFadeInFromBlack] autorelease]];
				}
				else
					[self addCommand:[[[MIFadeVideoCommand alloc] initWithObjectToFade:obj from:ARME_TIME_UNDEF to:ARME_TIME_UNDEF fadeType:kFadeInFromBlack] autorelease]];
			}
		}
	}
	return self;
}

+ (BOOL)supportsSelection:(MIWSSelection*)sel
{
	if ( [sel type] == kMIWSSelectionTypeObjects
		 || [sel type] == kMIWSSelectionTypeTimeSlice
		 )
	{
		if ([sel count] == 0) 
			return NO;
		
//		NSEnumerator* selEnum = [sel objectEnumerator];
		MIObject* obj;
		for(UInt32 i = 0; i < [sel count]; i++)
		//while (obj = [[selEnum nextObject] representedObject])
		{
			obj = [[sel objectAtIndex:i] representedObject];
			if (([obj WSObject]->GetType() != TArmeWSObj::kAudioClip) &&
				([obj WSObject]->GetType() != TArmeWSObj::kVideoClip))
				return NO;
		}
		return YES;
	}
	else
		return NO;

}


@end


@implementation MIFadeVideoCommand
- (void)dealloc
{
	[fObject release];
	
	[super dealloc];
} 

- (id)initWithObjectToFade:(MIObject*)obj from:(ArmeTime)from to:(ArmeTime)to fadeType:(SInt32)type
{
	self = [super initWithSelection:nil];
	if (self)
	{
		fObject = [obj retain];
		fFrom = from;
		fTo = to;
		fType = type;
		fCompleteObject = false;		

		if ((fFrom == ARME_TIME_UNDEF) || (fTo == ARME_TIME_UNDEF))
		{
			fFrom = [fObject WSObject]->GetAbsoluteStartTime();
			fTo = [fObject WSObject]->GetAbsoluteEndxTime();
			fCompleteObject = true;
		}
	}
	return self;
}
-(id)initFromArchive:(ArmeArchiveHandler*)handler 
{
	self = [super initFromArchive:handler];
	fObject = [handler->ReadObjCObject() retain];
	fFrom = handler->ReadArmeTime();
	fTo = handler->ReadArmeTime();
	fType = handler->ReadSInt32(); 
	fCompleteObject = handler->ReadBOOL();

	return self;
}
-(BOOL)writeToArchive:(ArmeArchiveHandler*)handler
{
	[super writeToArchive:handler];

	handler->WriteObjCObject(fObject);
	handler->WriteArmeTime(fFrom);
	handler->WriteArmeTime(fTo);
	handler->WriteSInt32(fType);
	handler->WriteBOOL(fCompleteObject);
	
	return YES;
}
- (NSString*)commandName
{
	return @"FadeVideo";
}
- (MICmdStatus)doIt
{
	//note : each command uses the result of the previous one.  That's why they are created in the 'doIt' and why they are executed one by one

	//create the clipboard
	MIObject* clipboard = [[MIObject alloc] initWithArmeWSObj:(new TArmeContainer("temp clipboard for fade"))];

	// copy the selected part
	MICommand* cmd = [[[MICopyCommand alloc] initWithObject:fObject from:fFrom to:fTo usingClipboard:clipboard] autorelease];
	[cmd _doIt];

	MIObject* copy = [[clipboard getChildren] objectAtIndex:0];

	// and paste it at the same position in the parent
	MIPasteLowLevelCommand* pasteCmd = [[[MIPasteLowLevelCommand alloc] initWithDocument:[fObject montageDoc] objectsToPaste:[clipboard getChildren] inObject:[[fObject montageDoc] topLevelContainer] atTime:fFrom atVPos:VPOS_ZERO] autorelease];
	[pasteCmd _doIt];

	copy = [pasteCmd->fListOfObjectsPasted objectAtIndex:0];

	//create an effect around the copied object
	cmd = [[[MIFadeVideoLowLevelCommand alloc] initWithObjectToEncloseInFadeEffect:copy fadeType:fType] autorelease];
	[cmd _doIt];

	MIObject* effect = [copy parentObject];

	//render the effect
	cmd = [[[MIFreezeLowLevelCommand alloc] initWithObjectToFreeze:effect] autorelease];
	if ([cmd _doIt] == kMICmdStatusCancelled)
	{
		 //delete the effect
		MICommand* deleteRenderedEffectCommand = [[[MIDeleteCommand alloc] initWithObject:effect] autorelease];
		[deleteRenderedEffectCommand _doIt];
		
		[clipboard release];

		return kMICmdStatusCancelled;
	}

	if (fCompleteObject)
	{
		NSMutableArray* array = [[[NSMutableArray alloc] initWithCapacity:1] autorelease];
		[array addObject:[effect frozenObject]];

		// give the same name
		[[effect frozenObject] WSObject]->SetNameCopy([fObject WSObject]->GetName());
		
		//paste the rendered effect in the tlc
		MIPasteLowLevelCommand* pasteCmd2 = [[[MIPasteLowLevelCommand alloc] initWithDocument:[fObject montageDoc] objectsToPaste:array inObject:[fObject parentObject] atTime:[fObject WSObject]->GetPos().fStartTime atVPos:[fObject getAbsoluteVerticalPos]] autorelease];
		[self addCommand:pasteCmd2];
		[pasteCmd2 _doIt];
		
		//delete the selection
		cmd = [[[MIDeleteLowLevelCommand alloc] initWithObjectToDelete:fObject from:ARME_TIME_UNDEF to:ARME_TIME_UNDEF] autorelease];
		[self addCommand:cmd];
		[cmd _doIt];
	}
	else
	{
		//delete the selection
		cmd = [[[MIDeleteLowLevelCommand alloc] initWithObjectToDelete:fObject from:fFrom to:fTo] autorelease];
		[self addCommand:cmd];
		[cmd _doIt];
	
		//paste the rendered effect in the original object
		cmd = [[[MIInsertLowLevelCommand alloc] initWithSelection:nil objectToInsert:[effect frozenObject] atPosition:fFrom inObject:fObject] autorelease];
		[self addCommand:cmd];
		[cmd _doIt];
	}
	 
	 //delete the effect
	MICommand* deleteRenderedEffectCommand = [[[MIDeleteCommand alloc] initWithObject:[effect frozenObject]] autorelease];
	[deleteRenderedEffectCommand _doIt];
	
	[clipboard release];

	return kMICmdStatusDone;
}
- (void)undoIt
{
	[super undoIt];
}
- (void)redoIt
{
	[super redoIt];
}

- (int)getNbLengthyTasks
{
	return 1;  //only the freeze is considered as lengthy
}
@end

@implementation MIFadeVideoLowLevelCommand
- (void)dealloc
{
	[fObject release];
	[super dealloc];
} 

- (id)initWithObjectToEncloseInFadeEffect:(MIObject*)obj fadeType:(SInt32)type
{
	self = [super initWithSelection:nil];
	if (self)
	{
		fObject = [obj retain];
		fType = type;
	}
	return self;
}
-(id)initFromArchive:(ArmeArchiveHandler*)handler 
{
	self = [super initFromArchive:handler];
	fObject = [handler->ReadObjCObject() retain];
	fType = handler->ReadSInt32(); 
	
	return self;
}
-(BOOL)writeToArchive:(ArmeArchiveHandler*)handler
{
	[super writeToArchive:handler];
	handler->WriteObjCObject(fObject);
	handler->WriteSInt32(fType);
	
	return YES;
}
- (NSString*)commandName
{
	return @"FadeVideoLowLevel";
}
- (MICmdStatus)doIt
{
	TArmeDoc				*armeDoc;
	MIMontageDocument		*mDoc;
	TVideoEffect			*newArmeEffect;
	MIObject				*newEffect;

	mDoc = [fObject montageDoc];
    armeDoc = [mDoc armeDocument];

	newArmeEffect = new TVideoEffect(armeDoc, NULL, NULL ); // <--- armeDoc : ??????

	if (fType == kFadeInFromBlack)
		newArmeEffect->CreateFadeEffectDescription(0, 100);
	else if (fType == kFadeOutToBlack)
		newArmeEffect->CreateFadeEffectDescription(100, 0);

    newEffect = [[MIObject alloc] initWithArmeWSObj:newArmeEffect];
    [newEffect setMontageDoc:mDoc];

	[fObject moveToObject:newEffect atTime:ARME_TIME_ZERO withVPos:VPOS_ZERO];
	[newEffect moveToObject:[mDoc topLevelContainer] atTime:[fObject WSObject]->GetPos().fStartTime withVPos:VPOS_ZERO];

	newArmeEffect->_SetLength([fObject WSObject]->GetDuration());
	//build the effect

	
	return kMICmdStatusDone;	
}
- (void)undoIt
{
}
- (void)redoIt
{
}
- (BOOL)displaySetupDialog
{
	return YES;
}

@end