/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import <Foundation/Foundation.h>
#import "MIMacroCommand.h"
#include "MontageInterface.h"

@class MIMainViewController;
class ArmeArchiveHandler;
@class MIWSSelection;
@class MIObject;
class TArmeWSObj;
@class MIMontageDocument;
class MIProgressBarDlgManager;
@class MIResizeCommand;

@interface MIFreezeCommand : MIMacroCommand
{
	TArmeWSObj*				fPreviousVideoSoloObject;
	TArmeWSObj*				fPreviousAudioSoloObject;
	MIMontageDocument*		fMontageDoc;
	BOOL					fIsRenderCommand;
}

-(id)initFromArchive:(ArmeArchiveHandler*)handler;
-(BOOL)writeToArchive:(ArmeArchiveHandler*)handler;
- (id)initWithSelection:(MIWSSelection*)sel;
- (MICmdStatus)doIt;
- (void)undoIt;
- (void)redoIt;

+ (BOOL)supportsSelection:(MIWSSelection*)sel;
+ (BOOL)isRenderCommand:(MIWSSelection*)sel;
+ (BOOL)isUnRenderCommand:(MIWSSelection*)sel;
@end

@interface MIBaseFreezeLowLevelCommand : MICommand
{
}
- (void)toggleToSourceObject:(MIObject*)object;
- (void)toggleToFrozenObject:(MIObject*)object;
@end

@interface MIFreezeLowLevelCommand : MIBaseFreezeLowLevelCommand
{
	MIObject* 				fObjectToFreeze;
	MIObject* 				fFreezingResultObject;
	BOOL					fRenderReverse;
	BOOL					fRenderUsingCustomAlgo;
}

-(id)initFromArchive:(ArmeArchiveHandler*)handler;
-(BOOL)writeToArchive:(ArmeArchiveHandler*)handler;
- (id)initWithObjectToFreeze:(MIObject*)object;
- (MICmdStatus)doIt;
- (void)undoIt;
- (void)redoIt;
- (int)getNbLengthyTasks;
- (void)setRenderReverse;
@end

@interface MIUnFreezeLowLevelCommand : MIBaseFreezeLowLevelCommand
{
	MIObject* 				fObjectToUnFreeze;
	MIObject*				fParent;
	ArmeTime				fStartTime;
	MIVPos					fVerticalPos;
	MIResizeCommand*				fEnlargeCommand;
}

-(id)initFromArchive:(ArmeArchiveHandler*)handler;
-(BOOL)writeToArchive:(ArmeArchiveHandler*)handler;
- (id)initWithObjectToUnFreeze:(MIObject*)object;
- (MICmdStatus)doIt;
- (void)undoIt;
- (void)redoIt;

@end


MIObject* Render(
	ArmeErr &err, 
	NSArray* objectsToFreeze, 
	MIObject* parent, 
	MIVPos videoVPos, 
	MIVPos audioVPos, 
	MIProgressBarDlgManager* progressMngr = NULL, 
	bool bConstantMixTransition = false, 
	ArmeTime startTime = ARME_TIME_UNDEF, 
	ArmeTime endTime = ARME_TIME_UNDEF, 
	bool onlyVideo = false,
	bool reverse = false,
	bool useCustomAlgo = false,
	bool stillClipToScale = false);
