/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MIInsertSilenceCommand.h"
#import "MIAudioProcessingCommand.h"
#import "MIObjectRep.h"
#import "MIObject.h"
#import "MISilenceCommand.h"

#ifndef OfflineAudioProcessorSilence_h
#include "OfflineAudioProcessorSilence.h"
#endif //OfflineAudioProcessorSilence_h

/****************************************************************************************************

MIInsertSilenceCommand

****************************************************************************************************/

@implementation MIInsertSilenceCommand

-(id)initFromArchive:(ArmeArchiveHandler*)handler
{
	self = [super initFromArchive:handler];
	return self;
}

-(BOOL)writeToArchive:(ArmeArchiveHandler*)handler
{
	return [super writeToArchive:handler];
}

- (NSString*)commandName
{
	return @"Insert Silence";
}

- (id)initWithSelection:(MIWSSelection*)sel
{
	self = [super initWithSelection:sel];
	
	[fSelection adjustSelectionTimesToVideo];
	
	if ( self )
	{
		NSEnumerator* selEnum = [sel objectEnumerator];
		MIObject* obj;
		while (obj = [[selEnum nextObject] representedObject])
		{
			if ([obj WSObject]->GetType() == TArmeWSObj::kAudioClip)
			{
				MIAudioProcessingCommand* silenceCommand = [[[MIAudioProcessingCommand alloc] initWithSelection:fSelection
					offlineAudioProcessor:OfflineAudioProcessorSilence::GetInstance()] autorelease];
				OfflineAudioProcessorParams* baseParams = [silenceCommand processorParams];
				OfflineSilenceParams* params = dynamic_cast<OfflineSilenceParams*>(baseParams);
				params->fDuration = kOfflineSilenceInsertSilenceWithDlg;
				params->fMontageDoc = [[[sel objectAtIndex:0] representedObject] montageDoc];
				[self addCommand:silenceCommand];
			}
			/* No with video clip in Montage 1.0
			else if ([obj WSObject]->GetType() == TArmeWSObj::kVideoClip)
			{
				[self modifiedVideoHierarchy];
				[self addCommand:[[[MISilenceVideoCommand alloc] initWithObjectToSilence:obj from:[sel startTime] to:[sel endxTime] insert:YES] autorelease]];
			}*/
		}
	}
	return self;
}

+ (BOOL)supportsSelection:(MIWSSelection*)sel
{
	if([sel type] != kMIWSSelectionTypeTimeSlice)
		return NO;
	if([sel duration] != _ArmeTime(0))
		return NO;
	NSEnumerator* selEnum = [sel objectEnumerator];
	MIObject* obj;
	while (obj = [[selEnum nextObject] representedObject])
	{
		if (([obj WSObject]->GetType() != TArmeWSObj::kAudioClip) /* && No with video clip in Montage 1.0
			([obj WSObject]->GetType() != TArmeWSObj::kVideoClip)*/)
			return NO;
	}
	return YES;
}

@end


