/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MILockToTimelineCommand.h"

#ifndef ArmeArchiveHandler_h
#include "ArmeArchiveHandler.h"
#endif //ArmeArchiveHandler_h

#import "MIObject.h"
#import "MIObjectRep.h"

@implementation MILockToTimelineCommand
- (void)dealloc
{
	[fObjects release];
	[fTheObject release];
	[super dealloc];
}

- (NSString*)commandName
{
	return @"Lock To Timeline";
}

- (id)initWithSelection:(MIWSSelection*)sel
{
	self = [super initWithSelection:sel];

	if ( self )
	{
		fObjects = [[NSMutableArray alloc] initWithObjects:nil];
		fTheObject = nil;
	}
	return self;
}

-(id)initFromArchive:(ArmeArchiveHandler*)handler
{
    if (( self = [super initFromArchive:handler] ))
    {
		fObjects = [handler->ReadObjCObject() retain];
		fTheObject = [handler->ReadObjCObject() retain];
    }
    return self ;
}

-(BOOL)writeToArchive:(ArmeArchiveHandler*)handler
{
    if ( [super writeToArchive:handler] )
    {
		handler->WriteObjCObject(fObjects);
		handler->WriteObjCObject(fTheObject);
		return YES;
    }
    else
		return NO;
}

- (BOOL)displaySetupDialog
{
	return YES;
}


+ (BOOL)supportsSelection:(MIWSSelection*)sel
{
	if ( [sel type] != kMIWSSelectionTypeObjects )
		return NO;
	if ( ( sel ) && ( [sel count] > 0 ) ) {
		MIObjectRep *objectRep;
		NSEnumerator *enumerator = [sel objectEnumerator];

		while (( objectRep = [enumerator nextObject] )) {

			MIObject* object = [objectRep representedObject];
			if ( ![object isLocked] ) return YES;
			
		}
	}
	return NO;
}

- (void)setObject:(MIObject*)obj
{
	fTheObject = [obj retain];
}


- (MICmdStatus)doIt
{
	MIObject		*obj;
	NSEnumerator	*objEnum; 

	if ( fTheObject && ![fTheObject isLocked])
	{
		[fObjects addObject:obj];
		[fTheObject setLocked:YES];
		[fTheObject setRepresentationsNeedDisplay];
	}
	else
	{
		objEnum = [fSelection objectEnumerator];
		
		while ( obj = [[objEnum nextObject] representedObject] )
		{
			if ( ![obj isLocked] )
			{
				[fObjects addObject:obj];
				[obj setLocked:YES];
				[obj setRepresentationsNeedDisplay];
			}
		}
	}
	
	return kMICmdStatusDone;
}

- (void)undoIt
{
	NSEnumerator		*objEnum = [fObjects objectEnumerator];
	MIObject			*obj;
	
	while ( obj = [objEnum nextObject] )
	{
		[obj setLocked:NO];
		[obj setRepresentationsNeedDisplay];
	}
}

- (void)redoIt
{
	NSEnumerator		*objEnum = [fObjects objectEnumerator];
	MIObject			*obj;
	
	while ( obj = [objEnum nextObject] )
	{
		[obj setLocked:YES];
		[obj setRepresentationsNeedDisplay];
	}
}

@end



@implementation MIUnlockToTimelineCommand
- (void)dealloc
{
	[fObjects release];
	[fTheObject release];
	[super dealloc];
}

- (NSString*)commandName
{
	return @"Unlock From Timeline";
}

- (id)initWithSelection:(MIWSSelection*)sel
{
	self = [super initWithSelection:sel];

	if ( self )
	{
		fObjects = [[NSMutableArray alloc] initWithObjects:nil];
		fTheObject = nil;
	}
	return self;
}

-(id)initFromArchive:(ArmeArchiveHandler*)handler
{
    if (( self = [super initFromArchive:handler] ))
    {
		fObjects = [handler->ReadObjCObject() retain];
		fTheObject = [handler->ReadObjCObject() retain];
    }
    return self ;
}

-(BOOL)writeToArchive:(ArmeArchiveHandler*)handler
{
    if ( [super writeToArchive:handler] )
    {
		handler->WriteObjCObject(fObjects);
		handler->WriteObjCObject(fTheObject);
		return YES;
    }
    else
		return NO;
}

- (BOOL)displaySetupDialog
{
	return YES;
}

+ (BOOL)supportsSelection:(MIWSSelection*)sel
{
	if ( [sel type] != kMIWSSelectionTypeObjects )
		return NO;
	if ( ( sel ) && ( [sel count] > 0 ) ) {
		MIObjectRep *objectRep;
		NSEnumerator *enumerator = [sel objectEnumerator];

		while (( objectRep = [enumerator nextObject] )) {
			MIObject* object = [objectRep representedObject];
			if ( [object isLocked] ) return YES;
		}
	}
	return NO;
}

- (void)setObject:(MIObject*)obj
{
	fTheObject = [obj retain];
}


- (MICmdStatus)doIt
{
	MIObject		*obj;
	NSEnumerator	*objEnum; 

	if ( fTheObject && [fTheObject isLocked])
	{
		[fObjects addObject:obj];
		[fTheObject setLocked:NO];
		[fTheObject setRepresentationsNeedDisplay];
	}
	else
	{
		objEnum = [fSelection objectEnumerator];
		
		while ( obj = [[objEnum nextObject] representedObject] )
		{
			if ( [obj isLocked] )
			{
				[fObjects addObject:obj];
				[obj setLocked:NO];
				[obj setRepresentationsNeedDisplay];
			}
		}
	}
	
	return kMICmdStatusDone;
}

- (void)undoIt
{
	NSEnumerator		*objEnum = [fObjects objectEnumerator];
	MIObject			*obj;
	
	while ( obj = [objEnum nextObject] )
	{
		[obj setLocked:YES];
		[obj setRepresentationsNeedDisplay];
	}
}

- (void)redoIt
{
	NSEnumerator		*objEnum = [fObjects objectEnumerator];
	MIObject			*obj;
	
	while ( obj = [objEnum nextObject] )
	{
		[obj setLocked:NO];
		[obj setRepresentationsNeedDisplay];
	}
}

@end