/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
/*!
	@class MIMacroCommand
	
	A macro command is a command that executes a set of commands.
	doing the command does all the sub-commands
	undoing the command undoes all the sub-commands
	redoing the command redoes all the sub-commands
*/
#import "MICommand.h"

@interface MIMacroCommand : MICommand
{
	NSMutableArray*	fSubCommands;
	NSString *fCommandName;
}

// arch stuff
-(id)initFromArchive:(ArmeArchiveHandler*)handler;
-(BOOL)writeToArchive:(ArmeArchiveHandler*)handler;

/*! @function init Creates an empty MIMacroCommand */
- (id)init;
- (id)initWithSelection:(MIWSSelection*)sel;

/*! @function addCommand Adds a command to the list of commands */
- (void)addCommand:(MICommand*)command;

- (BOOL)displaySetupDialog;
- (MICmdStatus)doIt;
- (void)undoIt;
- (void)redoIt;

- (NSString*)commandName;
- (void)setCommandName:(NSString*)commandName;

- (int)getNbLengthyTasks;
- (int)getNbSubCommands;
- (BOOL)isCancelable;

@end
