/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import <Foundation/Foundation.h>
#import "MIMacroCommand.h"
@class MIMainViewController;
class ArmeArchiveHandler;
@class MIWSSelection;
@class MIObject;
class TArmeWSObj;
@class MIMontageDocument;

/*! 
	@class MIMergeCommand Merges several objects into a single one. Depends
	on the objects.
	
	- on several clips, it merges them if they don't overlap horizontally and
	  if they are the same kind and if they are siblings.
	- on several containers, it makes a big one with them IF THEY ARE SIBLINGS
	- any other selection : it fails.
*/

@interface MIMergeCommand : MIMacroCommand
{
	MIObject*				fResult;
	NSMutableArray*			fAudioObjectsToMerge;	
	NSMutableArray*			fVideoObjectsToMerge;
	NSMutableArray*			fOtherObjectsToMerge;
	TArmeWSObj*				fPreviousVideoSoloObject;
	TArmeWSObj*				fPreviousAudioSoloObject;
	MIMontageDocument*		fMontageDoc;
	MIVPos					fVPos;
	MIObject*				fParent;
//	MICommand*				fSimpleMergeCommand1;
//	MICommand*				fSimpleMergeCommand2;
}

-(id)initFromArchive:(ArmeArchiveHandler*)handler;
-(BOOL)writeToArchive:(ArmeArchiveHandler*)handler;
- (id)initWithSelection:(MIWSSelection*)sel;
- (MICmdStatus)doIt;
- (void)undoIt;
- (void)redoIt;
- (int)getNbLengthyTasks;
@end

