/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MIMergeCommand.h"
#import "MIObject.h"
#import "MIObjectRep.h"
#import "MIMainViewController.h"
#import "MIMontageDocument.h"
#import "MIFreezeCommand.h"
#import "MIInsertLowLevelCommand.h"
#import "MIDeleteLowLevelCommand.h"
#import "MIApplicationController.h"
#import "MIApplication.h"

//#include "ArmeArchivable.h"
#include "TArmeDoc.h"

@implementation MIMergeCommand


-(void)dealloc
{
	[fOtherObjectsToMerge release];
	[fVideoObjectsToMerge release];
	[fAudioObjectsToMerge release];
	[fResult release];
	
//	[fSimpleMergeCommand1 release];
//	[fSimpleMergeCommand2 release];

	[super dealloc];
}

- (NSString*)commandName
{
	return @"Merge";
}

-(id)initFromArchive:(ArmeArchiveHandler*)handler
{
	self = [super initFromArchive:handler];
	
	if ( self )
	{
/*		fObjectsToMerge = [handler->ReadObjCObject() retain];
		fResult = [handler->ReadObjCObject() retain];
		fPreviousVideoSoloObject = dynamic_cast<TArmeWSObj*>(handler->ReadCPPObject());
		fPreviousAudioSoloObject = dynamic_cast<TArmeWSObj*>(handler->ReadCPPObject());
		
		if (handler->GetFileVer() >= 0x00010026)
		{
			fSimpleMergeCommand1 = [handler->ReadObjCObject() retain];
			fSimpleMergeCommand2 = [handler->ReadObjCObject() retain];
		}
		else
		{
			fSimpleMergeCommand1 = nil;
			fSimpleMergeCommand2 = nil;
		}*/
	}
	return self;
}

-(BOOL)writeToArchive:(ArmeArchiveHandler*)handler
{
	if ( [super writeToArchive:handler] )
	{
/*		handler->WriteObjCObject(fObjectsToMerge);
		handler->WriteObjCObject(fResult);
		handler->WriteCPPObject(fPreviousVideoSoloObject);
		handler->WriteCPPObject(fPreviousAudioSoloObject);

		//from version 0x00010026
		handler->WriteObjCObject(fSimpleMergeCommand1);
		handler->WriteObjCObject(fSimpleMergeCommand2);
*/
		return YES;
	}
	else
		return NO;
}

- (id)initWithSelection:(MIWSSelection*)sel
{
	self = [super initWithSelection:sel];

	if ( self )
	{
		[fSelection discardVideoTransitions];

		NSMutableArray* unsortedVideoObjectsToMerge = [[[NSMutableArray alloc] initWithCapacity:2] autorelease];
		NSMutableArray* unsortedAudioObjectsToMerge = [[[NSMutableArray alloc] initWithCapacity:2] autorelease];
		fOtherObjectsToMerge = [[NSMutableArray alloc] initWithCapacity:2];
	
		//first, separate the video and audio objects in two arrays
		NSArray* repArray = [fSelection putRepsInArrayAndRemoveDescendants];
		NSEnumerator* objRepEnum = [repArray objectEnumerator];
		MIObjectRep* rep;
		MIObject* object;
		while(rep = [objRepEnum nextObject])
		{
			object = [rep representedObject];
			if ([object WSObject]->GetType() == TArmeWSObj::kVideoClip)
				[unsortedVideoObjectsToMerge addObject:object];
			else if ([object WSObject]->GetType() == TArmeWSObj::kAudioClip)
				[unsortedAudioObjectsToMerge addObject:object];
			else
				[fOtherObjectsToMerge addObject:object];
		}

		if ([fOtherObjectsToMerge count] > 0)
		{
			// in this case, we don't really know what the guy is merging so : bazooka solution : render the selection
			[fOtherObjectsToMerge addObjectsFromArray:unsortedVideoObjectsToMerge];
			[fOtherObjectsToMerge addObjectsFromArray:unsortedAudioObjectsToMerge];
		}
		else
		{
			//let's try to make a smart merge
			
			//video :
			//-----
			fVideoObjectsToMerge = [[NSMutableArray alloc] initWithCapacity:[unsortedVideoObjectsToMerge count]];

			while ([unsortedVideoObjectsToMerge count] > 0)
			{
				MIObject* leftMostObject = [unsortedVideoObjectsToMerge objectAtIndex:0];
				ArmeTime leftMostStartTime = [leftMostObject WSObject]->GetAbsoluteStartTime();
				for (unsigned int i = 1; i < [unsortedVideoObjectsToMerge count]; ++i)
				{
					if ([[unsortedVideoObjectsToMerge objectAtIndex:i] WSObject]->GetAbsoluteStartTime() < leftMostStartTime)
					{
						leftMostStartTime = [[unsortedVideoObjectsToMerge objectAtIndex:i] WSObject]->GetAbsoluteStartTime();
						leftMostObject = [unsortedVideoObjectsToMerge objectAtIndex:i];
					}
				}
				[fVideoObjectsToMerge addObject:leftMostObject];
				[unsortedVideoObjectsToMerge removeObject:leftMostObject];
			}
		
			if ([fVideoObjectsToMerge count] > 1)
			{
				//then, look if they are contiguous
				MIObject* leftObject = nil; 
				MIObject* rightObject = [fVideoObjectsToMerge objectAtIndex:0];
				unsigned int index = 1;
				
				do
				{
					leftObject = rightObject;
					rightObject = [fVideoObjectsToMerge objectAtIndex:index];
					if ((abs(GET_VALUE([rightObject WSObject]->GetAbsoluteStartTime()) - GET_VALUE([leftObject WSObject]->GetAbsoluteEndxTime())) > 1))
						break;
					
					index++;
				}
				while (index < [fVideoObjectsToMerge count]);
				
				if (index == [fVideoObjectsToMerge count]) //all video clips are contiguous
				{
					//create the sub-commands to merge them
					for (unsigned int i = [fVideoObjectsToMerge count]-1; i > 0 ; --i)
					{
						[self addCommand:[[[MIInsertLowLevelCommand alloc] initWithSelection:nil objectToInsert:[fVideoObjectsToMerge objectAtIndex:i] atPosition:[[fVideoObjectsToMerge objectAtIndex:i-1] WSObject]->GetAbsoluteEndxTime() inObject:[fVideoObjectsToMerge objectAtIndex:i-1]] autorelease]];
						[self addCommand:[[[MIDeleteLowLevelCommand alloc] initWithObjectToDelete:[fVideoObjectsToMerge objectAtIndex:i] from:ARME_TIME_UNDEF to:ARME_TIME_UNDEF] autorelease]];
					}
				}
				else
				{
					[fOtherObjectsToMerge addObjectsFromArray:fVideoObjectsToMerge];
				}
			}
			else
			{
				[fOtherObjectsToMerge addObjectsFromArray:fVideoObjectsToMerge];
			}

			//audio :
			//-----
			fAudioObjectsToMerge = [[NSMutableArray alloc] initWithCapacity:[unsortedAudioObjectsToMerge count]];

			while ([unsortedAudioObjectsToMerge count] > 0)
			{
				MIObject* leftMostObject = [unsortedAudioObjectsToMerge objectAtIndex:0];
				ArmeTime leftMostStartTime = [leftMostObject WSObject]->GetAbsoluteStartTime();
				for (unsigned int i = 1; i < [unsortedAudioObjectsToMerge count]; ++i)
				{
					if ([[unsortedAudioObjectsToMerge objectAtIndex:i] WSObject]->GetAbsoluteStartTime() < leftMostStartTime)
					{
						leftMostStartTime = [[unsortedAudioObjectsToMerge objectAtIndex:i] WSObject]->GetAbsoluteStartTime();
						leftMostObject = [unsortedAudioObjectsToMerge objectAtIndex:i];
					}
				}
				[fAudioObjectsToMerge addObject:leftMostObject];
				[unsortedAudioObjectsToMerge removeObject:leftMostObject];
			}
		
			if ([fAudioObjectsToMerge count] > 1)
			{
				//then, look if they are contiguous
				MIObject* leftObject = nil; 
				MIObject* rightObject = [fAudioObjectsToMerge objectAtIndex:0];
				unsigned int index = 1;
				
				do
				{
					leftObject = rightObject;
					rightObject = [fAudioObjectsToMerge objectAtIndex:index];
					if ((abs(GET_VALUE([rightObject WSObject]->GetAbsoluteStartTime()) - GET_VALUE([leftObject WSObject]->GetAbsoluteEndxTime())) > 1))
						break;
					
					index++;
				}
				while (index < [fAudioObjectsToMerge count]);
				
				if (index == [fAudioObjectsToMerge count]) //all Audio clips are contiguous
				{
					//create the sub-commands to merge them
					for (unsigned int i = [fAudioObjectsToMerge count]-1; i > 0 ; --i)
					{
						[self addCommand:[[[MIInsertLowLevelCommand alloc] initWithSelection:nil objectToInsert:[fAudioObjectsToMerge objectAtIndex:i] atPosition:[[fAudioObjectsToMerge objectAtIndex:i-1] WSObject]->GetAbsoluteEndxTime() inObject:[fAudioObjectsToMerge objectAtIndex:i-1]] autorelease]];
						[self addCommand:[[[MIDeleteLowLevelCommand alloc] initWithObjectToDelete:[fAudioObjectsToMerge objectAtIndex:i] from:ARME_TIME_UNDEF to:ARME_TIME_UNDEF] autorelease]];
					}
				}
				else
				{
					[fOtherObjectsToMerge addObjectsFromArray:fAudioObjectsToMerge];
				}
			}
			else
			{
				[fOtherObjectsToMerge addObjectsFromArray:fAudioObjectsToMerge];
			}
		}
		
		fVPos = [[fSelection getLeftMostObject] getVerticalPos];
		fParent = [[fSelection getLeftMostObject] parentObject]; 
		
		if ([fSelection containsVideo:true andTransparentTextClip:NO])
			[self modifiedVideoHierarchy];
		if ([fSelection containsAudio])
			[self modifiedAudioHierarchy];
		[self overviewNeedsUpdate];
		
	}
	return self;
}

- (MICmdStatus)doIt
{
	if ([self getNbSubCommands] > 0)
	{
		MICmdStatus st = [super doIt];
		[[NSApp mainViewController] clearSelection];
		if (st == kMICmdStatusCancelled)
			return kMICmdStatusCancelled;
	}
	if ([fOtherObjectsToMerge count] > 0)
	{
		ArmeErr err;
		fResult = [Render(err, fOtherObjectsToMerge, fParent, fVPos, fVPos, [MICommand getCurrentProgressDialog], false) retain];
		
		if ((fResult == nil) || (err.type != kArmeOK))
		{
			NSRunAlertPanel(@"Error", @"Error during the merge operation.  Command Failed.", @"OK", nil, nil);
			[fResult release];
			return kMICmdStatusCancelled;
		}
		
		NSEnumerator* objEnum = [fOtherObjectsToMerge objectEnumerator];
		MIObject* obj;
		while (obj = [objEnum nextObject])
			[obj setOuttaHierarchy];
	}
	return kMICmdStatusDone;
}

- (void)undoIt
{
	if ([self getNbSubCommands] > 0)
	{
		[super undoIt];
	}
	if ([fOtherObjectsToMerge count] > 0)
	{
		NSEnumerator* objEnum = [fOtherObjectsToMerge objectEnumerator];
		MIObject* obj;
		while (obj = [objEnum nextObject])
		{
			[obj putBackInHierarchy];
			[obj setRepresentationsNeedDisplay];
		}
		[fResult setOuttaHierarchy];
	}
}

- (void)redoIt
{
	if ([self getNbSubCommands] > 0)
	{
		[super redoIt];
		[[NSApp mainViewController] clearSelection];
	}
	if ([fOtherObjectsToMerge count] > 0)
	{
		NSEnumerator* objEnum = [fOtherObjectsToMerge objectEnumerator];
		MIObject* obj;
		while (obj = [objEnum nextObject])
			[obj setOuttaHierarchy];
		
		[fResult putBackInHierarchy];
		[fResult setRepresentationsNeedDisplay];
	}
}

- (BOOL)displaySetupDialog
{
	if ([self CheckFrozenObjectInSelectionAndAskIfUserWantsToContinue:fSelection] == NO)
		return NO;
	return YES;
}

- (int)getNbLengthyTasks
{
	return 1;
}
+ (BOOL)supportsSelection:(MIWSSelection*)sel
{
	if ( [sel containsOnlyVideoTransitions])
		return NO;

	if ( [sel type] == kMIWSSelectionTypeObjects
#ifdef AUTOMATION_DATA
	|| [sel type] == kMIWSSelectionTypeAutomationPoints
#endif
	)
	{
		//we need at least 2 objects of the same media type

		NSArray* objectRepArray = [sel putRepsInArrayAndRemoveDescendants];
		int nbObjectsContainingVideo = 0;
		int nbObjectsContainingAudio = 0;
		
		NSEnumerator* objEnum = [objectRepArray objectEnumerator];
		MIObjectRep* obj;
		
		while (obj = [objEnum nextObject])
		{
			if ([[obj representedObject] WSObject]->ContainsVideo( ) && ([[obj representedObject] WSObject]->GetType() != TArmeWSObj::kVideoXFadeEffect))
				nbObjectsContainingVideo++;
			if ([[obj representedObject] WSObject]->ContainsAudio( TRUE ))
				nbObjectsContainingAudio++;
		}
		
		if (nbObjectsContainingVideo >= 2)
			return YES;
		if (nbObjectsContainingAudio >= 2)
			return YES;
	}
	return NO;
}
@end
