/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MICommand.h"

#ifndef MEMixingInfo_h
#include "MEMixingInfo.h"
#endif //MEMixingInfo_h

@class MIMainViewController;
@class MIMontageDocument;
class TArmeDoc;

enum EMixerCommand_CommandType
{
	kMixerCommand_NoCommandSet = 1973,
	kMixerCommand_SetStereoRAOForStereoVAO,
	kMixerCommand_SetStereoRAOForMonoVAO,
	kMixerCommand_SetMonoRAOForMonoVAO,
	kMixerCommand_SetMonoVAOFaderValue,
	kMixerCommand_SetStereoVAOFaderValue,
	kMixerCommand_SetStereoRAOFaderValue,
	kMixerCommand_SetMonoVAOPanning,
	kMixerCommand_SetStereoVAOPanning,
	kMixerCommand_SetMonoVAOName,
	kMixerCommand_SetStereoVAOName,
	kMixerCommand_SetStereoRAOName,
	kMixerCommand_SetStereoVAOLock,
	kMixerCommand_SetStereoRAOLock,
	kMixerCommand_SetMonoVAOMute,
	kMixerCommand_SetStereoVAOMute,
	kMixerCommand_SetStereoRAOMute,
	kMixerCommand_SetSoloedMonoVAO,
	kMixerCommand_SetSoloedStereoVAO,
	kMixerCommand_SetSoloedStereoRAO
};
	
@interface MIMixerCommand : MICommand
{
	EMixerCommand_CommandType	fType;
	
	// the mapping of the modified VAO
	MEVAOInfo					fOriginalVAOInfo;
	// the VAO index of the modified VAO
	SInt32						fModifiedVAOIndex;
	
	// the mapping of the modified RAO
	MERAOInfo					fOriginalRAOInfo;
	// the RAO index of the modified RAO
	SInt32						fModifiedRAOIndex;
	
	TArmeDoc*					fDoc;
	
	SInt32						fOriginallySoloedMonoVAO;
	SInt32						fOriginallySoloedStereoVAO;
	SInt32						fOriginallySoloedStereoRAO;
}

-(id)initFromArchive:(ArmeArchiveHandler*)handler;
-(BOOL)writeToArchive:(ArmeArchiveHandler*)handler;
- (id)initWithDocument:(TArmeDoc*)doc;
- (MICmdStatus)doIt;
- (void)undoIt;
- (void)redoIt;

- (void)executeInDoc:(MIMontageDocument*)doc;

// the actual commands :

- (void)setStereoRAOForStereoVAO:(SInt32)vaoIndex RAO:(SInt32)raoIndex;
- (void)setStereoRAOForMonoVAO:(SInt32)vaoIndex RAO:(SInt32)raoIndex;
- (void)setMonoRAOForMonoVAO:(SInt32)vaoIndex RAO:(SInt32)raoIndex;

// fader values always between 0.0 and 100.0
- (void)setMonoVAOFaderValue:(SInt32)vaoIndex value:(float)value; 
- (void)setStereoVAOFaderValue:(SInt32)vaoIndex value:(float)value left:(bool)bLeft; 
- (void)setStereoRAOFaderValue:(SInt32)raoIndex value:(float)value left:(bool)bLeft;

// panning is always between -1.0 (full left) and 1.0 (full right)
- (void)setMonoVAOPanning:(SInt32)vaoIndex panning:(float)panning;
- (void)setStereoVAOPanning:(SInt32)vaoIndex panning:(float)panning;

- (void)setMonoVAOName:(SInt32)monoVAOIndex name:(const char*)name;
- (void)setStereoVAOName:(SInt32)stereoVAOIndex name:(const char*)name;
- (void)setStereoRAOName:(SInt32)stereoRAOIndex name:(const char*)name;

- (void)setStereoVAOLock:(SInt32)stereoVAOIndex locked:(BOOL)locked;
- (void)setStereoRAOLock:(SInt32)stereoRAOIndex locked:(BOOL)locked;

- (void)setSoloedMonoVAO:(SInt32)soloedMonoVAO;
- (void)setSoloedStereoVAO:(SInt32)soloedStereoVAO;
- (void)setSoloedStereoRAO:(SInt32)soloedStereoRAO;

// muting
- (void)setMonoVAOMute:(SInt32)monoVAOIndex muted:(BOOL)muted;
- (void)setStereoVAOMute:(SInt32)stereoVAOIndex muted:(BOOL)muted;
- (void)setStereoRAOMute:(SInt32)stereoRAOIndex muted:(BOOL)muted;

@end
