/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#include "MIMixerCommand.h"

#ifndef TArmeDoc_h
#include "TArmeDoc.h"
#endif //TArmeDoc_h

#undef VER_DEF_TOPIC
#define VER_DEF_TOPIC MIMixerCommand

@implementation MIMixerCommand

- (id)initFromArchive:(ArmeArchiveHandler*)handler
{
	self = [super initFromArchive:handler];
	if(self)
	{
		ArmeArchivable* archObj = handler->ReadCPPObject();
		fDoc = dynamic_cast<TArmeDoc*>(archObj);
		ZzAssert(fDoc);
	}
	return self;
}

- (BOOL)writeToArchive:(ArmeArchiveHandler*)handler
{
	[super writeToArchive:handler];
	handler->WriteCPPObject(fDoc);
	return YES;
}

- (id)initWithDocument:(TArmeDoc*)doc
{
	self = [super initWithSelection:nil];
	if(self)
	{
		fDoc = doc;
		fType = kMixerCommand_NoCommandSet;
	}
	return self;
}

- (MICmdStatus)doIt
{
	// don't do anything... mixer is already changed.
	return kMICmdStatusDone;
}

- (void)undoIt
{
	ZzAssert(0);
/*
	switch(fType)
	{
	case kMixerCommand_NoCommandSet :
		ZzAssert(0);
		break;
	case 
*/
}

- (void)redoIt
{
	ZzAssert(0);
}

- (void)setStereoRAOForStereoVAO:(SInt32)vaoIndex RAO:(SInt32)raoIndex
{
	if(fType == kMixerCommand_NoCommandSet)
	{
		fType = kMixerCommand_SetStereoRAOForStereoVAO;
		fModifiedVAOIndex = vaoIndex;
		fOriginalVAOInfo = fDoc->GetMixingInfo()->RefStereoVAOMapping(fModifiedVAOIndex);
		fDoc->GetMixingInfo()->_SetStereoRAOForStereoVAO(vaoIndex,raoIndex);
	}
	else
	{
		ZzAssert(0); // cannot change the mapping twice in the same command
	}
}

- (void)setStereoRAOForMonoVAO:(SInt32)vaoIndex RAO:(SInt32)raoIndex
{
	if(fType == kMixerCommand_NoCommandSet)
	{
		fType = kMixerCommand_SetStereoRAOForMonoVAO;
		fModifiedVAOIndex = vaoIndex;
		fOriginalVAOInfo = fDoc->GetMixingInfo()->RefMonoVAOMapping(fModifiedVAOIndex);
		fDoc->GetMixingInfo()->_SetStereoRAOForMonoVAO(vaoIndex,raoIndex);
	}
	else
	{
		ZzAssert(0); // cannot change the mapping twice in the same command
	}
}

- (void)setMonoRAOForMonoVAO:(SInt32)vaoIndex RAO:(SInt32)raoIndex
{
	if(fType == kMixerCommand_NoCommandSet)
	{
		fType = kMixerCommand_SetMonoRAOForMonoVAO;
		fModifiedVAOIndex = vaoIndex;
		fOriginalVAOInfo = fDoc->GetMixingInfo()->RefMonoVAOMapping(fModifiedVAOIndex);
		fDoc->GetMixingInfo()->_SetMonoRAOForMonoVAO(vaoIndex,raoIndex);
	}
	else
	{
		ZzAssert(0); // cannot change the mapping twice in the same command
	}
}

- (void)setMonoVAOFaderValue:(SInt32)vaoIndex value:(float)value
{
	if(fType == kMixerCommand_NoCommandSet)
	{
		fType = kMixerCommand_SetMonoVAOFaderValue;
		fModifiedVAOIndex = vaoIndex;
		fOriginalVAOInfo = fDoc->GetMixingInfo()->RefMonoVAOMapping(fModifiedVAOIndex);
		fDoc->GetMixingInfo()->_SetMonoVAOFaderValue(vaoIndex,value);
	}
	else if(fType == kMixerCommand_SetMonoVAOFaderValue)
	{
		fDoc->GetMixingInfo()->_SetMonoVAOFaderValue(vaoIndex,value);
	}
	else
	{
		ZzAssert(0); // cannot change the mapping twice in the same command
	}
}

- (void)setStereoVAOFaderValue:(SInt32)vaoIndex value:(float)value left:(bool)bLeft
{
	vernl(6,"setStereoVAOFaderValue: vaoIndex = " << vaoIndex << " value = " << value << " left = " << bLeft);
	if(fType == kMixerCommand_NoCommandSet)
	{
		fType = kMixerCommand_SetStereoVAOFaderValue;
		fModifiedVAOIndex = vaoIndex;
		fOriginalVAOInfo = fDoc->GetMixingInfo()->RefStereoVAOMapping(fModifiedVAOIndex);
		fDoc->GetMixingInfo()->_SetStereoVAOFaderValue(vaoIndex,value,bLeft);
	}
	else if(fType == kMixerCommand_SetStereoVAOFaderValue)
	{
		fDoc->GetMixingInfo()->_SetStereoVAOFaderValue(vaoIndex,value,bLeft);
	}
	else
	{
		ZzAssert(0); // cannot change the mapping twice in the same command
	}
}

- (void)setStereoRAOFaderValue:(SInt32)raoIndex value:(float)value left:(bool)bLeft
{
	if(fType == kMixerCommand_NoCommandSet)
	{
		fType = kMixerCommand_SetStereoRAOFaderValue;
		fModifiedRAOIndex = raoIndex;
		fOriginalRAOInfo = fDoc->GetMixingInfo()->RefStereoRAOMapping(fModifiedRAOIndex);
		fDoc->GetMixingInfo()->_SetStereoRAOFaderValue(raoIndex,value,bLeft);
	}
	else if(fType == kMixerCommand_SetStereoRAOFaderValue)
	{
		fDoc->GetMixingInfo()->_SetStereoRAOFaderValue(raoIndex,value,bLeft);
	}
	else
	{
		ZzAssert(0); // cannot change the mapping twice in the same command
	}
}

- (void)setMonoVAOPanning:(SInt32)vaoIndex panning:(float)panning
{
	if(fType == kMixerCommand_NoCommandSet)
	{
		fType = kMixerCommand_SetMonoVAOPanning;
		fModifiedVAOIndex = vaoIndex;
		fOriginalVAOInfo = fDoc->GetMixingInfo()->RefMonoVAOMapping(fModifiedVAOIndex);
		fDoc->GetMixingInfo()->_SetMonoVAOPanning(vaoIndex,panning);
	}
	else if(fType == kMixerCommand_SetMonoVAOPanning)
	{
		fDoc->GetMixingInfo()->_SetMonoVAOPanning(vaoIndex,panning);
	}
	else
	{
		ZzAssert(0); // cannot change the mapping twice in the same command
	}
}

- (void)setStereoVAOPanning:(SInt32)vaoIndex panning:(float)panning
{
	if(fType == kMixerCommand_NoCommandSet)
	{
		fType = kMixerCommand_SetStereoVAOPanning;
		fModifiedVAOIndex = vaoIndex;
		fOriginalVAOInfo = fDoc->GetMixingInfo()->RefStereoVAOMapping(fModifiedVAOIndex);
		fDoc->GetMixingInfo()->_SetStereoVAOPanning(vaoIndex,panning);
	}
	else if(fType == kMixerCommand_SetStereoVAOPanning)
	{
		fDoc->GetMixingInfo()->_SetStereoVAOPanning(vaoIndex,panning);
	}
	else
	{
		ZzAssert(0); // cannot change the mapping twice in the same command
	}
}

- (void)setMonoVAOName:(SInt32)monoVAOIndex name:(const char*)name
{
	if(fType == kMixerCommand_NoCommandSet)
	{
		fType = kMixerCommand_SetMonoVAOName;
		fModifiedVAOIndex = monoVAOIndex;
		fOriginalVAOInfo = fDoc->GetMixingInfo()->RefMonoVAOMapping(fModifiedVAOIndex);
		fDoc->GetMixingInfo()->_SetMonoVAOName(monoVAOIndex,name);
	}
	else
	{
		ZzAssert(0); // cannot change the mapping twice in the same command
	}
}

- (void)setStereoVAOName:(SInt32)stereoVAOIndex name:(const char*)name
{
	if(fType == kMixerCommand_NoCommandSet)
	{
		fType = kMixerCommand_SetStereoVAOName;
		fModifiedVAOIndex = stereoVAOIndex;
		fOriginalVAOInfo = fDoc->GetMixingInfo()->RefStereoVAOMapping(fModifiedVAOIndex);
		fDoc->GetMixingInfo()->_SetStereoVAOName(stereoVAOIndex,name);
	}
	else
	{
		ZzAssert(0); // cannot change the mapping twice in the same command
	}
}

- (void)setStereoRAOName:(SInt32)stereoRAOIndex name:(const char*)name
{
	if(fType == kMixerCommand_NoCommandSet)
	{
		fType = kMixerCommand_SetStereoRAOName;
		fModifiedRAOIndex = stereoRAOIndex;
		fOriginalRAOInfo = fDoc->GetMixingInfo()->RefStereoRAOMapping(fModifiedRAOIndex);
		fDoc->GetMixingInfo()->_SetStereoRAOName(stereoRAOIndex,name);
	}
	else
	{
		ZzAssert(0); // cannot change the mapping twice in the same command
	}
}

- (void)setStereoVAOLock:(SInt32)stereoVAOIndex locked:(BOOL)locked
{
	if(fType == kMixerCommand_NoCommandSet)
	{
		fType = kMixerCommand_SetStereoVAOLock;
		fModifiedVAOIndex = stereoVAOIndex;
		fOriginalVAOInfo = fDoc->GetMixingInfo()->RefStereoVAOMapping(fModifiedVAOIndex);
		fDoc->GetMixingInfo()->_SetStereoVAOLock(stereoVAOIndex,locked);
	}
	else
	{
		ZzAssert(0); // cannot change the mapping twice in the same command
	}
}

- (void)setStereoRAOLock:(SInt32)stereoRAOIndex locked:(BOOL)locked
{
	if(fType == kMixerCommand_NoCommandSet)
	{
		fType = kMixerCommand_SetStereoRAOLock;
		fModifiedRAOIndex = stereoRAOIndex;
		fOriginalRAOInfo = fDoc->GetMixingInfo()->RefStereoRAOMapping(fModifiedRAOIndex);
		fDoc->GetMixingInfo()->_SetStereoRAOLock(stereoRAOIndex,locked);
	}
	else
	{
		ZzAssert(0); // cannot change the mapping twice in the same command
	}
}

- (void)setSoloedMonoVAO:(SInt32)soloedMonoVAO
{
	if(fType == kMixerCommand_NoCommandSet)
	{
		fType = kMixerCommand_SetSoloedMonoVAO;
		fOriginallySoloedMonoVAO = fDoc->GetMixingInfo()->GetSoloedMonoVAO();
		fDoc->GetMixingInfo()->_SetSoloedMonoVAO(soloedMonoVAO);
	}
	else
	{
		ZzAssert(0);
	}
}	

- (void)setSoloedStereoVAO:(SInt32)soloedStereoVAO
{
	if(fType == kMixerCommand_NoCommandSet)
	{
		fType = kMixerCommand_SetSoloedStereoVAO;
		fOriginallySoloedStereoVAO = fDoc->GetMixingInfo()->GetSoloedStereoVAO();
		fDoc->GetMixingInfo()->_SetSoloedStereoVAO(soloedStereoVAO);
	}
	else
	{
		ZzAssert(0);
	}
}

- (void)setSoloedStereoRAO:(SInt32)soloedStereoRAO
{
	if(fType == kMixerCommand_NoCommandSet)
	{
		fType = kMixerCommand_SetSoloedStereoRAO;
		fOriginallySoloedStereoRAO = fDoc->GetMixingInfo()->GetSoloedStereoRAO();
		fDoc->GetMixingInfo()->_SetSoloedStereoRAO(soloedStereoRAO);
	}
	else
	{
		ZzAssert(0);
	}
}

- (void)setMonoVAOMute:(SInt32)monoVAOIndex muted:(BOOL)muted
{
	if(fType == kMixerCommand_NoCommandSet)
	{
		fType = kMixerCommand_SetMonoVAOMute;
		fModifiedVAOIndex = monoVAOIndex;
		fOriginalVAOInfo = fDoc->GetMixingInfo()->RefMonoVAOMapping(fModifiedVAOIndex);
		fDoc->GetMixingInfo()->_SetMonoVAOMute(monoVAOIndex,muted);
	}
	else
	{
		ZzAssert(0);
	}
}

- (void)setStereoVAOMute:(SInt32)stereoVAOIndex muted:(BOOL)muted
{
	if(fType == kMixerCommand_NoCommandSet)
	{
		fType = kMixerCommand_SetStereoVAOMute;
		fModifiedVAOIndex = stereoVAOIndex;
		fOriginalVAOInfo = fDoc->GetMixingInfo()->RefStereoVAOMapping(fModifiedVAOIndex);
		fDoc->GetMixingInfo()->_SetStereoVAOMute(stereoVAOIndex,muted);
	}
	else
	{
		ZzAssert(0);
	}
}

- (void)setStereoRAOMute:(SInt32)stereoRAOIndex muted:(BOOL)muted
{
	if(fType == kMixerCommand_NoCommandSet)
	{
		fType = kMixerCommand_SetStereoRAOMute;
		fModifiedRAOIndex = stereoRAOIndex;
		fOriginalRAOInfo = fDoc->GetMixingInfo()->RefStereoRAOMapping(fModifiedRAOIndex);
		fDoc->GetMixingInfo()->_SetStereoRAOMute(stereoRAOIndex,muted);
	}
	else
	{
		ZzAssert(0);
	}
}

- (void)executeInDoc:(MIMontageDocument*)doc
{
	// right now mixer changes are not commands
	[self _doIt];
}

@end

