/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import <Foundation/Foundation.h>
#import "MICommand.h"
#import "MIMacroCommand.h"

@class MIMainViewController;
class ArmeArchiveHandler;
@class MIWSSelection;
@class MIResizeCommand;
@class MIObject;
@class MIRepositionLockedObjectsCommand;

@interface MIMoveCommand : MICommand
{
	NSMutableArray*						fExtraCommands;
	MIResizeCommand*					fEnlargeCommand;
//	MIPoint								fNewPosition;
	MIPoint								fOffset;
	MIObject*							fNewParent;
	MIObject*							fTheObject;
	MIRepositionLockedObjectsCommand*	fRepositionLockedObjects;
	NSArray*							fObjArray;
	id									fShiftUpCommand;
}

-(id)initFromArchive:(ArmeArchiveHandler*)handler;
-(BOOL)writeToArchive:(ArmeArchiveHandler*)handler;
- (id)initWithSelection:(MIWSSelection*)sel;
- (id)initWithObject:(MIObject*)obj;
- (id)initWithObjArray:(NSArray*)objs;
- (id)initFromArchive:(ArmeArchiveHandler*)handler;
- (BOOL)writeToArchive:(ArmeArchiveHandler*)handler;
- (void)setObject:(MIObject*)obj;
- (void)setObjectArray:(NSArray*)objs; // array of MIObjects
- (void)setOffset:(MIPoint)offset; // offset in global coords
- (void)setNewParent:(MIObject*)newParent;
+ (BOOL)supportsSelection:(MIWSSelection*)sel;
- (BOOL)displaySetupDialog;
- (MICmdStatus)doIt;
- (void)undoIt;
- (void)redoIt;
- (MIObject*)newParent;
@end

@interface MIShiftEverythingUpCommand : MIMacroCommand
{
	MIObject	*fTLC;
	MIPoint		fOffset;
}
- (id)initFromArchive:(ArmeArchiveHandler*)handler;
- (BOOL)writeToArchive:(ArmeArchiveHandler*)handler;
- (id)initWithTLCObject:(MIObject*)obj offset:(MIVPos)offset;
- (void)setTLCObject:(MIObject*)obj offset:(MIVPos)offset;
- (void)preprareExtraCommands;
@end

@interface MIShiftObjectUpCommand : MICommand
{
	MIObject	*fObj;
	MIPoint		fOffset;
}
- (id)initFromArchive:(ArmeArchiveHandler*)handler;
- (BOOL)writeToArchive:(ArmeArchiveHandler*)handler;
- (id)initWithObject:(MIObject*)obj offset:(MIVPos)offset;
- (void)setObject:(MIObject*)obj offset:(MIVPos)offset;
@end