/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MIMoveCommand.h"

#import "MISingleMoveCommand.h"
#import "MIResizeCommand.h"
#import "MIObject.h"
#import "MIObjectRep.h"
#import "MIMainViewController.h"
#import "MIRepositionLockedObjectsCommand.h"
#import "MIUpdateVideoTransitionsCommand.h"
#import "MIMontageDocument.h"

#ifndef TArmeWSObj_h
#include "TArmeWSObj.h"
#endif //TArmeWSObj_h

#ifndef ArmeArchiveHandler_h
#include "ArmeArchiveHandler.h"
#endif //ArmeArchiveHandler_h

/****************************************************************************************************

MIMoveCommand

****************************************************************************************************/


@implementation MIMoveCommand
- (void)dealloc
{
	[fExtraCommands release];
	[fEnlargeCommand release];
	[fTheObject release];
	[fRepositionLockedObjects release];
	[fObjArray release];
	[fNewParent release];
	[super dealloc];
}

- (NSString*)commandName
{
	return @"Move";
}

- (id)initWithSelection:(MIWSSelection*)sel
{
	self = [super initWithSelection:sel];

	[fSelection adjustSelectionTimesToVideo];
	
	if ( self )
	{
		[fSelection discardVideoTransitions];

		fExtraCommands = [[NSMutableArray alloc] initWithObjects:nil];
		fEnlargeCommand = nil;
		
		if ([fSelection containsVideo:true andTransparentTextClip:YES])
			[self modifiedVideoHierarchy];
		if ([fSelection containsAudio])
			[self modifiedAudioHierarchy];
		[self overviewNeedsUpdate];

		fTheObject = nil;
		fObjArray = nil;
		fNewParent = nil;
		fOffset = MIZeroPoint;
		fRepositionLockedObjects = nil;
	}

	return self;
}

- (id)initWithObject:(MIObject*)obj
{
	self = [super initWithSelection:nil];

	if ( self )
	{
		fExtraCommands = [[NSMutableArray alloc] initWithObjects:nil];
		fEnlargeCommand = nil;
		
		if ( [obj WSObject]->ContainsVideo(false, true) )
			[self modifiedVideoHierarchy];
		if ( [obj WSObject]->ContainsAudio() )
			[self modifiedAudioHierarchy];
		[self overviewNeedsUpdate];

		fTheObject = [obj retain];
		fObjArray = nil;
		fNewParent = nil;
		fOffset = MIZeroPoint;
		fRepositionLockedObjects = nil;
	}

	return self;
}

- (id)initWithObjArray:(NSArray*)objs
{
	self = [super initWithSelection:nil];

	if ( self )
	{
		fExtraCommands = [[NSMutableArray alloc] initWithObjects:nil];
		fEnlargeCommand = nil;
		
		[self overviewNeedsUpdate];

		fTheObject = nil;
		fNewParent = nil;
		fOffset = MIZeroPoint;
		fRepositionLockedObjects = nil;
		[self setObjectArray:objs];
	}

	return self;
}


-(id)initFromArchive:(ArmeArchiveHandler*)handler
{
    if (( self = [super initFromArchive:handler] ))
    {
		fExtraCommands = [handler->ReadObjCObject() retain];
		fEnlargeCommand = [handler->ReadObjCObject() retain];
//		fNewPosition = handler->ReadMIPoint();
		fOffset = handler->ReadMIPoint();
		fNewParent = [handler->ReadObjCObject() retain];
		fTheObject = [handler->ReadObjCObject() retain];
		fRepositionLockedObjects = [handler->ReadObjCObject() retain];
		fShiftUpCommand = [handler->ReadObjCObject() retain];
		fObjArray = [handler->ReadObjCObject() retain];
    }
    return self;
}

-(BOOL)writeToArchive:(ArmeArchiveHandler*)handler
{
    if ( [super writeToArchive:handler] )
    {
		//handler->WriteObjCObject(fController);
		handler->WriteObjCObject(fExtraCommands);
		handler->WriteObjCObject(fEnlargeCommand);
//		handler->WriteMIPoint(fNewPosition);
		handler->WriteMIPoint(fOffset);
		handler->WriteObjCObject(fNewParent);
		handler->WriteObjCObject(fTheObject);
		handler->WriteObjCObject(fRepositionLockedObjects);
		handler->WriteObjCObject(fShiftUpCommand);
		handler->WriteObjCObject(fObjArray);
		return YES;
    }
    else return NO;
}

- (void)setOffset:(MIPoint)offset
{
	fOffset = offset;
}

- (void)setNewParent:(MIObject*)newParent
{
	[fNewParent release];
	fNewParent = [newParent retain];
}

- (MIObject*)newParent
{
	return fNewParent;
}

- (void)setObject:(MIObject*)obj
{
	ZzAssert([obj WSObject]->GetType() != TArmeWSObj::kVideoXFadeEffect);
	fTheObject = [obj retain];
	if ([fTheObject  WSObject]->ContainsVideo(true, true))
		[self modifiedVideoHierarchy];
	if ([fTheObject WSObject]->ContainsAudio())
		[self modifiedAudioHierarchy];
}

- (void)setObjectArray:(NSArray*)objs
{
	NSEnumerator	*objEnum;
	MIObject		*anObj;
	
	fObjArray = [[NSArray arrayWithArray:objs] retain];
	objEnum = [fObjArray objectEnumerator];
	
	while ( anObj = [objEnum nextObject] )
	{
		if ( [anObj WSObject]->ContainsVideo(true, true) )
			[self modifiedVideoHierarchy];
		if ( [anObj WSObject]->ContainsAudio() )
			[self modifiedAudioHierarchy];
	}

}


+ (BOOL)supportsSelection:(MIWSSelection*)sel
{
	if ( [sel containsOnlyVideoTransitions])
		return NO;

	if ( [sel type] == kMIWSSelectionTypeObjects
#ifdef AUTOMATION_DATA
		|| [sel type] == kMIWSSelectionTypeAutomationPoints
#endif
		 )
		return YES;
	else
		return NO;
}

- (BOOL)displaySetupDialog
{
	if ([self CheckFrozenObjectAndAskIfUserWantsToContinue:fNewParent])
	{
		if ([self CheckFrozenObjectAndAskIfUserWantsToContinue:[fTheObject parentObject]])
		{
			return YES;
		}
	}
	return NO;
}

- (MICmdStatus)doIt
{
	NSEnumerator		*objEnum;
	MIObject			*anObject;
	NSEnumerator		*cmdEnum;
	MISingleMoveCommand	*cmd;
	MIVPos				min = _MIVPos(0);
	MIPoint				p;
	MIObject			*tlc = nil;
	
	
	// create the array of commands.
	if ( ( fTheObject && (fObjArray || fSelection) ) ||
		 ( fObjArray && fSelection ) )
		verntl(interfaceStephan, 0, "only one of 'setObject', 'setSelection' or 'setObjectArray' can be used" ); 

	if ( fTheObject )
	{
		MIPoint		newGlobalPosition;

		ZzAssert(fTheObject != newParentObject);
		// determine the origin and the destination, as well as the new position

		newGlobalPosition = [fTheObject convertPointToTLCCoords:MIZeroPoint];
		newGlobalPosition = TranslateMIPoint( newGlobalPosition, fOffset );

		if ( GET_VALUE(newGlobalPosition.vpos) < 0 )
		{
			fShiftUpCommand = [[MIShiftEverythingUpCommand alloc] initWithTLCObject:[[fTheObject montageDoc] topLevelContainer] offset:_MIVPos(-GET_VALUE(newGlobalPosition.vpos))];
			[fShiftUpCommand _doIt];
		}
	

		cmd = [[[MISingleMoveCommand alloc] initWithSelection:nil] autorelease];
		[cmd setObject:fTheObject];

		if ( !fNewParent )
			[self setNewParent:[fTheObject parentObject]];
			
		[cmd setNewParent:fNewParent];

		if ( ![fNewParent parentObject] /*fNewParent == [[fController getTopLevelContainerRep] representedObject]*/ )
		{
			// in TLC
			[cmd setNewOrigin:newGlobalPosition];
		}
		else
		{
			// regular object
			p = [fNewParent convertPointFromTLCCoords:newGlobalPosition];
			[cmd setNewOrigin:p];
		}
		[fExtraCommands addObject:cmd];
	}
	else if ( fSelection )
	{
		MIPoint p;
		objEnum = [fSelection objectEnumerator];

		while ( anObject = [[objEnum nextObject] representedObject] )
		{
			if ( [anObject isLocked] )
			{
				fOffset.time = _ArmeTime(0);
			}
			p = [anObject convertPointToTLCCoords:MIZeroPoint];
			p.vpos += fOffset.vpos;
			if ( p.vpos < _MIVPos(0) )
			{
				min = p.vpos;
				tlc = [[anObject montageDoc] topLevelContainer];
			}
		}

		if ( GET_VALUE(min) < 0 )
		{
			fShiftUpCommand = [[MIShiftEverythingUpCommand alloc] initWithTLCObject:tlc offset:_MIVPos(-GET_VALUE(min))];
			[fShiftUpCommand _doIt];
		}
	
		objEnum = [[fSelection putRepsInArrayAndRemoveDescendants] objectEnumerator];

		while ( anObject = [[objEnum nextObject] representedObject] )
		{
			MIPoint		newGlobalPosition;
	
			ZzAssert(anObject != newParentObject);
			// determine the origin and the destination, as well as the new position
	
//			newGlobalPosition = [anObject convertRectToTLCCoords:[anObject modelFrame]].origin;
			newGlobalPosition = [anObject convertPointToTLCCoords:MIZeroPoint];
			if ( [anObject isLocked] )
				newGlobalPosition.vpos += fOffset.vpos;
			else
				newGlobalPosition = TranslateMIPoint( newGlobalPosition, fOffset );
	
			cmd = [[[MISingleMoveCommand alloc] initWithSelection:nil] autorelease];
			[cmd setObject:anObject];
			if ( !fNewParent )
				[self setNewParent:[anObject parentObject]];
				
			[cmd setNewParent:fNewParent];
	
			if ( [fNewParent isTLC] /*fNewParent == [[fController getTopLevelContainerRep] representedObject]*/ )
			{
				// in TLC
				[cmd setNewOrigin:newGlobalPosition];
			}
			else
			{
				// regular object
				p = [fNewParent convertPointFromTLCCoords:newGlobalPosition];
				[cmd setNewOrigin:p];
			}
			[fExtraCommands addObject:cmd];
		}
	}
	else if ( fObjArray )
	{
		objEnum = [fObjArray objectEnumerator];

		while ( anObject = [objEnum nextObject] )
		{
			if ( [anObject isLocked] )
			{
				fOffset.time = _ArmeTime(0);
			}
			p = [anObject convertPointToTLCCoords:MIZeroPoint];
			p.vpos += fOffset.vpos;
			if ( p.vpos < _MIVPos(0) )
			{
				min = p.vpos;
				tlc = [[anObject montageDoc] topLevelContainer];
			}
		}

		if ( GET_VALUE(min) < 0 )
		{
			fShiftUpCommand = [[MIShiftEverythingUpCommand alloc] initWithTLCObject:tlc offset:_MIVPos(-GET_VALUE(min))];
			[fShiftUpCommand _doIt];
		}
	
				
		objEnum = [fObjArray objectEnumerator];

		while ( anObject = [objEnum nextObject] )
		{
			MIPoint		newGlobalPosition;
	
			ZzAssert(anObject != newParentObject);
			// determine the origin and the destination, as well as the new position
	
			newGlobalPosition = [anObject convertPointToTLCCoords:MIZeroPoint];
			if ( [anObject isLocked] )
				newGlobalPosition.vpos += fOffset.vpos;
			else
				newGlobalPosition = TranslateMIPoint( newGlobalPosition, fOffset );
	
			cmd = [[[MISingleMoveCommand alloc] initWithSelection:nil] autorelease];
			[cmd setObject:anObject];
			if ( !fNewParent )
				fNewParent = [anObject parentObject];
				
			[cmd setNewParent:fNewParent];
	
			if ( [fNewParent isTLC] /*fNewParent == [[fController getTopLevelContainerRep] representedObject]*/ )
			{
				// in TLC
				[cmd setNewOrigin:newGlobalPosition];
			}
			else
			{
				// regular object
				p = [fNewParent convertPointFromTLCCoords:newGlobalPosition];
				[cmd setNewOrigin:p];
			}
			[fExtraCommands addObject:cmd];
		}
	}
	
	// if something were to go under the vertical line, shift everything up before.
	
	cmdEnum = [fExtraCommands objectEnumerator];
	while ( cmd = [cmdEnum nextObject] )
	{
		[cmd _doIt];
	}

	fRepositionLockedObjects = [[MIRepositionLockedObjectsCommand alloc] initWithSelection:fSelection];
	[fRepositionLockedObjects _doIt];
	
	// check if parent size is big enough
	if ( !MIContainsRect( [fNewParent modelFrame], [fNewParent minimalRectToFitContent] ) )
	{
		fEnlargeCommand = [[MIResizeCommand alloc] initWithSelection:nil];
		[fEnlargeCommand setObject:fNewParent newFrame:MIUnionRect([fNewParent modelFrame], [fNewParent minimalRectToFitContent])];
		[fEnlargeCommand _doIt];
	}
	
	return kMICmdStatusDone;
}

- (void)undoIt
{
	NSEnumerator		*cmdEnum;
	MISingleMoveCommand	*cmd;

	[fEnlargeCommand _undoIt];

	[fRepositionLockedObjects _undoIt];

	cmdEnum = [fExtraCommands objectEnumerator];
	while ( cmd = [cmdEnum nextObject] )
	{
		[cmd _undoIt];
	}
	
	[fShiftUpCommand _undoIt];
}

- (void)redoIt
{
	NSEnumerator		*cmdEnum;
	MISingleMoveCommand	*cmd;

	cmdEnum = [fExtraCommands objectEnumerator];
	
	[fShiftUpCommand _redoIt];
	while ( cmd = [cmdEnum nextObject] )
	{
		[cmd _redoIt];
	}
	
	[fRepositionLockedObjects _redoIt];

	[fEnlargeCommand _redoIt];
}
@end











@implementation MIShiftEverythingUpCommand : MIMacroCommand
- (id)initFromArchive:(ArmeArchiveHandler*)handler
{
	self = [super initFromArchive:handler];
	if ( self )
	{
		fOffset = handler->ReadMIPoint();
		fTLC = [handler->ReadObjCObject() retain];
	}
	return self;
}
- (BOOL)writeToArchive:(ArmeArchiveHandler*)handler
{
	[super writeToArchive:handler];
	handler->WriteMIPoint(fOffset);
	handler->WriteObjCObject(fTLC);
	
	return YES;
}

- (id)initWithTLCObject:(MIObject*)obj offset:(MIVPos)offset
{
	self = [super initWithSelection:nil];
	if ( self )
	{
		fTLC = nil;
		[self setTLCObject:obj offset:offset];
	}
	
	return self;
}

- (void)setTLCObject:(MIObject*)obj offset:(MIVPos)offset
{
	[fTLC release];
	fTLC = [obj retain];
	fOffset.vpos = offset;
	
	[self preprareExtraCommands];
}

- (void)preprareExtraCommands
{
	NSEnumerator	*objEnum;
	MIObject		*anObj;
	id				cmd;
	
	
	objEnum = [[fTLC getChildren] objectEnumerator];
	
	while ( anObj = [objEnum nextObject] )
	{
		cmd = [[[MIShiftObjectUpCommand alloc] initWithObject:anObj offset:fOffset.vpos] autorelease];
		[self addCommand:cmd];
	}
}
@end

@implementation MIShiftObjectUpCommand : MICommand
- (id)initFromArchive:(ArmeArchiveHandler*)handler
{
	self = [super initFromArchive:handler];
	if ( self )
	{
		fOffset = handler->ReadMIPoint();
		fObj = [handler->ReadObjCObject() retain];
	}
	return self;
}
- (BOOL)writeToArchive:(ArmeArchiveHandler*)handler
{
	[super writeToArchive:handler];
	handler->WriteMIPoint(fOffset);
	handler->WriteObjCObject(fObj);
	return YES;
}

- (id)initWithObject:(MIObject*)obj offset:(MIVPos)offset
{
	self = [super initWithSelection:nil];
	if ( self )
	{
		fObj = nil;
		[self setObject:obj offset:offset];
	}
	
	return self;
}

- (void)setObject:(MIObject*)obj offset:(MIVPos)offset
{
	[fObj release];
	fObj = [obj retain];
	fOffset.vpos = offset;
}

- (MICmdStatus)doIt
{
	[fObj setVerticalPos:([fObj getVerticalPos] + fOffset.vpos)];
	[fObj recreateFrame];
	[fObj setRepresentationsNeedDisplay];

	return kMICmdStatusDone;
}

- (void)undoIt
{
	[fObj setVerticalPos:([fObj getVerticalPos] - fOffset.vpos)];
	[fObj recreateFrame];
	[fObj setRepresentationsNeedDisplay];
}

- (void)redoIt
{
	[self doIt];
}
@end

