/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MIMuteCommand.h"
#import "MIMainViewController.h"
#import "MIObjectRep.h"
#import "MIObject.h"
#import "MIDocumentController.h"
#import "MITopLevelContainerRep.h"

#import "MIMontageDocument.h"
#include "TVideoObject.h"
#include "TArmeDoc.h"


void GetMutedStatusBeforeCommand(MIObject* obj, ZzArray<SInt32>* mutedStatusBeforeCommand)
{
	NSEnumerator*	childEnum = [[obj getChildren] objectEnumerator];
	MIObject*		child;
	
	mutedStatusBeforeCommand->AddItem([obj isMuted]);

	while ( child = [childEnum nextObject] )
		GetMutedStatusBeforeCommand(child, mutedStatusBeforeCommand);
}

void SetMutedStatusAsTheyWereBeforeCommand(MIObject* obj, ZzArray<SInt32>* mutedStatusBeforeCommand, int& index)
{
	NSEnumerator*	childEnum = [[obj getChildren] objectEnumerator];
	MIObject*		child;
	
	[obj setMuted:mutedStatusBeforeCommand->ItemAt(index++)];

	while ( child = [childEnum nextObject] )
		SetMutedStatusAsTheyWereBeforeCommand(child, mutedStatusBeforeCommand, index);
}



/****************************************************************************************************

MIMuteCommand

****************************************************************************************************/


@implementation MIMuteCommand

-(id)initFromArchive:(ArmeArchiveHandler*)handler
{
	self = [super initFromArchive:handler];
	if ( self )
	{
		fObjectsToMute = [handler->ReadObjCObject() retain];
		SInt32 nbStatus = handler->ReadSInt32();
		fObjectsStatusBeforeCommand = new ZzArray<SInt32>(nbStatus);
		for (int i=0; i < nbStatus; ++i)
			fObjectsStatusBeforeCommand->AddItem(handler->ReadSInt32());

		fPreviousVideoSoloObject = dynamic_cast<TArmeWSObj*>(handler->ReadCPPObject());
		fPreviousAudioSoloObject = dynamic_cast<TArmeWSObj*>(handler->ReadCPPObject());
//		fVideoSoloChanged = handler->ReadBOOL();
//		fAudioSoloChanged = handler->ReadBOOL();
	}
	return self;
}

-(BOOL)writeToArchive:(ArmeArchiveHandler*)handler
{
	if ( [super writeToArchive:handler] )
	{
		handler->WriteObjCObject(fObjectsToMute);
		handler->WriteSInt32(fObjectsStatusBeforeCommand->CountItems());
		for (int i=0; i < fObjectsStatusBeforeCommand->CountItems(); ++i)
			handler->WriteSInt32(fObjectsStatusBeforeCommand->ItemAt(i));

		handler->WriteCPPObject(fPreviousVideoSoloObject);
		handler->WriteCPPObject(fPreviousAudioSoloObject);
//		handler->WriteBOOL(fVideoSoloChanged);
//		handler->WriteBOOL(fAudioSoloChanged);
		return YES;
	}
	else
		return NO;
}

- (void)dealloc
{
	[fObjectsToMute release];
	delete fObjectsStatusBeforeCommand;
	[super dealloc];
}

- (NSString*)commandName
{
	return @"Mute";
}
- (id)initWithSelection:(MIWSSelection*)sel
{
	self = [super initWithSelection:sel];

	if ( self )
	{
		[fSelection discardVideoTransitions];

		fObjectsToMute = [[NSMutableArray alloc] initWithCapacity:1];
		
		NSEnumerator*	objEnum = [[fSelection putRepsInArrayAndRemoveDescendants] objectEnumerator];
		MIObjectRep*	objectRep;
	
		while ( objectRep = [objEnum nextObject] )
			[fObjectsToMute addObject:[objectRep representedObject]];
	
		fObjectsStatusBeforeCommand = new ZzArray<SInt32>([fObjectsToMute count]);
		
		if ([fSelection containsVideo:true andTransparentTextClip:YES])
			[self modifiedVideoHierarchy];
		if ([fSelection containsAudio])
			[self modifiedAudioHierarchy];
		[self overviewNeedsUpdate];
		
	}
	return self;
}

- (BOOL)displaySetupDialog
{
	return YES;
}

- (MICmdStatus)doIt
{
	NSEnumerator* objEnum = [fObjectsToMute objectEnumerator];
	MIObject* 	obj;

	fPreviousVideoSoloObject = [[[fObjectsToMute objectAtIndex:0] montageDoc] armeDocument]->GetVideoSoloObject();
//	fVideoSoloChanged = false;
	fPreviousAudioSoloObject = [[[fObjectsToMute objectAtIndex:0] montageDoc] armeDocument]->GetAudioSoloObject();
//	fAudioSoloChanged = false;
	
	
	while ( obj = [objEnum nextObject] )
	{
		GetMutedStatusBeforeCommand(obj, fObjectsStatusBeforeCommand);

		if (fPreviousVideoSoloObject && fPreviousVideoSoloObject->IsDescendantOf([obj WSObject]))
		{
			[[[fObjectsToMute objectAtIndex:0] montageDoc] armeDocument]->SetVideoSoloObject(NULL);
//			fSoloChanged = true;
		}
		if (fPreviousAudioSoloObject && fPreviousAudioSoloObject->IsDescendantOf([obj WSObject]))
		{
			[[[fObjectsToMute objectAtIndex:0] montageDoc] armeDocument]->SetAudioSoloObject(NULL);
//			fSoloChanged = true;
		}

		[obj setMuted:true];
//		[obj setRepresentationsNeedDisplay];
	}

//	((TVideoObject*)[[[fObjectsToMute objectAtIndex:0] montageDoc] armeDocument]->GetTopContainer())->UpdateAffectedHierarchy();

	//update all the views
	NSEnumerator* docEnum = [[[MIDocumentController sharedDocumentController] documents] objectEnumerator];
	MIMontageDocument* doc;
	while(doc = [docEnum nextObject])
	{
		NSEnumerator* mainViewEnum = [[doc windowControllers] objectEnumerator];
		MIMainViewController* mainViewController;
		while(mainViewController = [mainViewEnum nextObject])
		{
			[[mainViewController topLevelContainerView] setNeedsDisplay:YES];
		}
	}
	return kMICmdStatusDone;
}

- (void)undoIt
{
	NSEnumerator* objEnum = [fObjectsToMute objectEnumerator];
	MIObject* 	obj;
	int i=0;
	
	while ( obj = [objEnum nextObject] )
	{
		SetMutedStatusAsTheyWereBeforeCommand(obj, fObjectsStatusBeforeCommand, i);
	}

//	if (fSoloChanged)
	[[[fObjectsToMute objectAtIndex:0] montageDoc] armeDocument]->SetVideoSoloObject(fPreviousVideoSoloObject);
	[[[fObjectsToMute objectAtIndex:0] montageDoc] armeDocument]->SetAudioSoloObject(fPreviousAudioSoloObject);

//	((TVideoObject*)[[[fObjectsToMute objectAtIndex:0] montageDoc] armeDocument]->GetTopContainer())->UpdateAffectedHierarchy();

	//update all the views
	NSEnumerator* docEnum = [[[MIDocumentController sharedDocumentController] documents] objectEnumerator];
	MIMontageDocument* doc;
	while(doc = [docEnum nextObject])
	{
		NSEnumerator* mainViewEnum = [[doc windowControllers] objectEnumerator];
		MIMainViewController* mainViewController;
		while(mainViewController = [mainViewEnum nextObject])
		{
			[[mainViewController topLevelContainerView] setNeedsDisplay:YES];
		}
	}
}

- (void)redoIt
{
	[self doIt];
}

+ (BOOL)supportsSelection:(MIWSSelection*)sel
{
	if ( [sel containsOnlyVideoTransitions])
		return NO;

	if ( [sel type] != kMIWSSelectionTypeObjects )
		return NO;
	else
	{
		// if at least one object in the selection is in the solo, return no
		// if at least one object in the selection is not muted yet, return yes 
		bool bInSolo = false;
		bool bIsNotMuted = false;

		NSEnumerator*	objEnum = [[sel putRepsInArrayAndRemoveDescendants] objectEnumerator];
		MIObjectRep*	objectRep;

		while ( objectRep = [objEnum nextObject] )
		{
			if (![[objectRep representedObject] isMuted])
				bIsNotMuted = true;
			if ([[objectRep representedObject] WSObject]->IsInSolo())
				bInSolo = true;
		}
		
		if (bInSolo)
			return NO;
		else if (bIsNotMuted)
			return YES;
		else
			return NO;
	}
}

@end


/*****************************************************************************************************************************

Unmute

*****************************************************************************************************************************/

@implementation MIUnMuteCommand

-(id)initFromArchive:(ArmeArchiveHandler*)handler
{
	self = [super initFromArchive:handler];
	if ( self )
	{
		fObjectsToUnMute = [handler->ReadObjCObject() retain];
		SInt32 nbStatus = handler->ReadSInt32();
		fObjectsStatusBeforeCommand = new ZzArray<SInt32>(nbStatus);
		for (int i=0; i < nbStatus; ++i)
			fObjectsStatusBeforeCommand->AddItem(handler->ReadSInt32());

		fPreviousVideoSoloObject = dynamic_cast<TArmeWSObj*>(handler->ReadCPPObject());
		fPreviousAudioSoloObject = dynamic_cast<TArmeWSObj*>(handler->ReadCPPObject());
//		fSoloChanged = handler->ReadBOOL();
	}
	return self;
}

-(BOOL)writeToArchive:(ArmeArchiveHandler*)handler
{
	[super writeToArchive:handler];
	handler->WriteObjCObject(fObjectsToUnMute);
	handler->WriteSInt32(fObjectsStatusBeforeCommand->CountItems());
	for (int i=0; i < fObjectsStatusBeforeCommand->CountItems(); ++i)
		handler->WriteSInt32(fObjectsStatusBeforeCommand->ItemAt(i));

	handler->WriteCPPObject(fPreviousVideoSoloObject);
	handler->WriteCPPObject(fPreviousAudioSoloObject);
//	handler->WriteBOOL(fSoloChanged);
	
	return YES;
}

- (void)dealloc
{
	[fObjectsToUnMute release];
	delete fObjectsStatusBeforeCommand;
	[super dealloc];
}

- (NSString*)commandName
{
	return @"Un-Mute";
}
- (id)initWithSelection:(MIWSSelection*)sel
{
	self = [super initWithSelection:sel];
	if ( self )
	{
		[fSelection discardVideoTransitions];

		fObjectsToUnMute = [[NSMutableArray alloc] initWithCapacity:1];
	
		NSEnumerator*	objEnum = [[fSelection putRepsInArrayAndRemoveDescendants] objectEnumerator];
		MIObjectRep*	objectRep;
	
		while ( objectRep = [objEnum nextObject] )
			[fObjectsToUnMute addObject:[objectRep representedObject]];
	
		fObjectsStatusBeforeCommand = new ZzArray<SInt32>([fObjectsToUnMute count]);

		if ([fSelection containsVideo:true andTransparentTextClip:YES])
			[self modifiedVideoHierarchy];
		if ([fSelection containsAudio])
			[self modifiedAudioHierarchy];
	}
	return self;
}

- (BOOL)displaySetupDialog
{
	return YES;
}

- (MICmdStatus)doIt
{
	NSEnumerator* objEnum = [fObjectsToUnMute objectEnumerator];
	MIObject* 	obj;
//	int i=0;
	
	fPreviousVideoSoloObject = [[[fObjectsToUnMute objectAtIndex:0] montageDoc] armeDocument]->GetVideoSoloObject();
	fPreviousAudioSoloObject = [[[fObjectsToUnMute objectAtIndex:0] montageDoc] armeDocument]->GetAudioSoloObject();
//	fSoloChanged = false;

	

	while ( obj = [objEnum nextObject] )
	{
		GetMutedStatusBeforeCommand(obj, fObjectsStatusBeforeCommand);
	
		if (fPreviousVideoSoloObject && fPreviousVideoSoloObject->IsDescendantOf([obj WSObject]))
		{
			[[[fObjectsToUnMute objectAtIndex:0] montageDoc] armeDocument]->SetVideoSoloObject(NULL);
//			fSoloChanged = true;
		}
		if (fPreviousAudioSoloObject && fPreviousAudioSoloObject->IsDescendantOf([obj WSObject]))
		{
			[[[fObjectsToUnMute objectAtIndex:0] montageDoc] armeDocument]->SetAudioSoloObject(NULL);
//			fSoloChanged = true;
		}

		[obj setMuted:false];
//		[obj setRepresentationsNeedDisplay];
	}

//	((TVideoObject*)[[[fObjectsToUnMute objectAtIndex:0] montageDoc] armeDocument]->GetTopContainer())->UpdateAffectedHierarchy();

	//update all the views
	NSEnumerator* docEnum = [[[MIDocumentController sharedDocumentController] documents] objectEnumerator];
	MIMontageDocument* doc;
	while(doc = [docEnum nextObject])
	{
		NSEnumerator* mainViewEnum = [[doc windowControllers] objectEnumerator];
		MIMainViewController* mainViewController;
		while(mainViewController = [mainViewEnum nextObject])
		{
			[[mainViewController topLevelContainerView] setNeedsDisplay:YES];
		}
	}
	return kMICmdStatusDone;
}

- (void)undoIt
{
	NSEnumerator* objEnum = [fObjectsToUnMute objectEnumerator];
	MIObject* 	obj;
	int i=0;
	
	while ( obj = [objEnum nextObject] )
	{
		SetMutedStatusAsTheyWereBeforeCommand(obj, fObjectsStatusBeforeCommand, i);
	}

//	if (fSoloChanged)
	[[[fObjectsToUnMute objectAtIndex:0] montageDoc] armeDocument]->SetVideoSoloObject(fPreviousVideoSoloObject);
	[[[fObjectsToUnMute objectAtIndex:0] montageDoc] armeDocument]->SetAudioSoloObject(fPreviousAudioSoloObject);

	//update all the views
	NSEnumerator* docEnum = [[[MIDocumentController sharedDocumentController] documents] objectEnumerator];
	MIMontageDocument* doc;
	while(doc = [docEnum nextObject])
	{
		NSEnumerator* mainViewEnum = [[doc windowControllers] objectEnumerator];
		MIMainViewController* mainViewController;
		while(mainViewController = [mainViewEnum nextObject])
		{
			[[mainViewController topLevelContainerView] setNeedsDisplay:YES];
		}
	}
}

- (void)redoIt
{
	[self doIt];
}

+ (BOOL)supportsSelection:(MIWSSelection*)sel
{
	if ( [sel containsOnlyVideoTransitions])
		return NO;

	if ( [sel type] != kMIWSSelectionTypeObjects )
		return NO;
	else
	{
		// if at least one object in the selection is in the solo, return no
		// if at least one object in the selection is muted, return yes 
		bool bInSolo = false;
		bool bIsMuted = false;

		NSEnumerator*	objEnum = [[sel putRepsInArrayAndRemoveDescendants] objectEnumerator];
		MIObjectRep*	objectRep;

		while ( objectRep = [objEnum nextObject] )
		{
			if ([[objectRep representedObject] isMuted])
				bIsMuted = true;
			if ([[objectRep representedObject] WSObject]->IsInSolo())
				bInSolo = true;
		}
		
		if (bInSolo)
			return NO;
		else if (bIsMuted)
			return YES;
		else
			return NO;
	}
}

@end


