/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
#import <Foundation/Foundation.h>
#import "MICommand.h"
@class MIMainViewController;
class ArmeArchiveHandler;
@class MIWSSelection;
@class MIObject;
@class MIDeleteLowLevelCommand;
@class MIMontageDocument;
@class MIResizeCommand;
/*! 
	@class MIPasteLowLevelCommand Paste the pasteboard content in the doc (context
	dependant)
*/

enum EMEPasteSuffix
{
    kSuffixSplitLeft = 0,
    kSuffixSplitRight,
    kSuffixCopy,
    kSuffixNumberZero
};

@interface MIPasteLowLevelCommand : MICommand
{
@public
	MIMontageDocument*	 		fMontageDoc;
	NSArray*				fListOfObjectsToPaste;
	ArmeTime				fPastePositionH;
	MIVPos					fPastePositionV;
	MIObject*				fWhereToPaste;
	MIResizeCommand*			fEnlargeCommand;
	SInt32					fSuffix;
	NSMutableArray*				fListOfObjectsPasted;
	MIDeleteLowLevelCommand*	fUndoCommand;
}
-(id)initFromArchive:(ArmeArchiveHandler*)handler;
-(BOOL)writeToArchive:(ArmeArchiveHandler*)handler;

- (id)initWithDocument:(MIMontageDocument*)montageDoc objectsToPaste:(NSArray*)listOfObjectsToPaste inObject:(MIObject*) whereToPaste atTime:(ArmeTime)whenToPaste atVPos:(MIVPos)whereToPasteAbsoluteVPos;
- (MICmdStatus)doIt;
- (void)undoIt;
- (void)redoIt;
/*! 
    @function setSuffix The suffix that has to be added to ALL the pasted objects (recursive). 
    if suffix = kSuffixSplitLeft, then @"\\" is added to the name
    if suffix = kSuffixSplitRight, then @"/" is added to the name
    if suffix = kSuffixCopy, then the first one is added @" (1st Copy)", and the other ones become @" (2nd Copy)", @" (3rd Copy)", etc...
    if suffix >= kSuffixNumberZero, then the number (suffix - kSuffixNumberZero) is added to the string, with a leading space char.
*/
- (void)setSuffix:(SInt32)suffix;

- (MIObject*)pasteRecursive:(MIObject*)objInClipBoard to:(MIObject*)parentInDoc at:(ArmeTime)pasteTime atVPos:(MIVPos)pasteAbsoluteVPos withSuffix:(SInt32)suffix;

+(NSString*)computeNewNameFromOldName:(NSString*)originalName suffix:(SInt32)suffix;

@end


