/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MIPasteLowLevelCommand.h"
#import "MITopLevelContainerRep.h"
#import "MIMontageDocument.h"
#import "MIDeleteLowLevelCommand.h"
#import "MIDeleteCommand.h"
#import "MIMainViewController.h"
#import "MIObject.h"
#import "MIResizeCommand.h"
#import "MIApplicationController.h"
#ifndef TVideoEffect_h
#include "TVideoEffect.h"
#endif //TVideoEffect_h

#ifndef MEAudioClip_h
#include "MEAudioClip.h"
#endif //MEAudioClip_h

#ifndef ZzHelpers_h
#include "ZzHelpers.h"
#endif //ZzHelpers_h

#ifndef TArmeWSObj_h
#include "TArmeWSObj.h"
#endif //TArmeWSObj_h

#ifndef TVideoClip_h
#include "TVideoClip.h"
#endif //TVideoClip_h

#ifndef TArmeTextClip_h
#include "TArmeTextClip.h"
#endif //TArmeTextClip_h

#ifndef TArmeContainer_h
#include "TArmeContainer.h"
#endif //TArmeContainer_h

#ifndef TArmeDoc_h
#include "TArmeDoc.h"
#endif //TArmeDoc_h

#ifndef ArmeDataFileRef_h
#include "ArmeDataFileRef.h"
#endif //ArmeDataFileRef_h

#ifndef VARPIPluginClass_h
#include "VARPIPluginClass.h"
#endif VARPIPluginClass_h

/****************************************************************************************************
MIPasteLowLevelCommand
****************************************************************************************************/

@implementation MIPasteLowLevelCommand

-(void)dealloc
{
	[fListOfObjectsToPaste release];
	[fListOfObjectsPasted release];
	[fEnlargeCommand release];
	[fUndoCommand release];
	
	[super dealloc];
}

-(id)initFromArchive:(ArmeArchiveHandler*)handler
{
	self = [super initFromArchive:handler];
	if(self)
	{
		fListOfObjectsPasted = [handler->ReadObjCObject() retain];
		fUndoCommand = [handler->ReadObjCObject() retain];
		fEnlargeCommand = [handler->ReadObjCObject() retain];
		fSuffix = kSuffixCopy; // we don't need it for undo and redo

		fListOfObjectsToPaste = nil;
	}
	return self;
}

-(BOOL)writeToArchive:(ArmeArchiveHandler*)handler
{
	[super writeToArchive:handler];
	handler->WriteObjCObject(fListOfObjectsPasted);
	handler->WriteObjCObject(fUndoCommand);
	handler->WriteObjCObject(fEnlargeCommand);

	return YES;
}


- (id)initWithDocument:(MIMontageDocument*)montageDoc objectsToPaste:(NSArray*)listOfObjectsToPaste inObject:(MIObject*)whereToPaste atTime:(ArmeTime)whenToPaste atVPos:(MIVPos)whereToPasteAbsoluteVPos
{
	self = [super initWithSelection:nil];
	if ( self )
	{
		fMontageDoc = montageDoc;
		fPastePositionH = whenToPaste;
		fPastePositionV = whereToPasteAbsoluteVPos;
		fWhereToPaste = whereToPaste;
		fListOfObjectsPasted = [[NSMutableArray alloc] initWithCapacity:[listOfObjectsToPaste count]];
		fListOfObjectsToPaste = [listOfObjectsToPaste retain]; 
		fEnlargeCommand = nil;
		fUndoCommand = nil;
		fSuffix = kSuffixCopy;
	}
	return self;
}

- (void)setSuffix:(SInt32)suffix
{
	fSuffix = suffix;
}

- (MICmdStatus)doIt
{
	NSEnumerator* objToPasteEnum = [fListOfObjectsToPaste objectEnumerator];
	MIObject* object;

	// find the leftmost start time in the list of objects to paste
	// and the smallest VPos
	ArmeTime LeftMostStartTimeInPasteboard = ARME_TIME_MAX;
	MIVPos	 SmallestVPosInPasteboard;
	GET_VALUE(SmallestVPosInPasteboard) = 999999;
	while ( object = [objToPasteEnum nextObject] )
	{
		if ([object WSObject]->GetPos().fStartTime < LeftMostStartTimeInPasteboard)
			LeftMostStartTimeInPasteboard = [object WSObject]->GetAbsoluteStartTime(); // was GetPos().fStartTime;
		if (GET_VALUE([object getAbsoluteVerticalPos]) < GET_VALUE(SmallestVPosInPasteboard))
			GET_VALUE(SmallestVPosInPasteboard) = GET_VALUE([object getAbsoluteVerticalPos]);
	}
	
	if (fPastePositionV == VPOS_UNDEF)
		fPastePositionV = SmallestVPosInPasteboard;
	
	objToPasteEnum = [fListOfObjectsToPaste objectEnumerator];
	//paste each children of the clipboard
	[[MIApplicationController sharedApplicationController] clearAllSelections];
	while ( object = [objToPasteEnum nextObject] )
	{
		MIVPos pasteVPos;
		GET_VALUE(pasteVPos) = GET_VALUE(fPastePositionV) + GET_VALUE([object getAbsoluteVerticalPos]) - GET_VALUE(SmallestVPosInPasteboard);
		MIObject* newObject = [self pasteRecursive:object to:fWhereToPaste at:(fPastePositionH + [object WSObject]->GetAbsoluteStartTime() - LeftMostStartTimeInPasteboard) atVPos:pasteVPos withSuffix:fSuffix]; 

		[newObject addToSelections];
		[fListOfObjectsPasted addObject:newObject];
	}

	// check if parent size is big enough
	if ( !MIContainsRect( [fWhereToPaste modelFrame], [fWhereToPaste minimalRectToFitContent] ) )
	{
		fEnlargeCommand = [[MIResizeCommand alloc] initWithSelection:nil];
		[fEnlargeCommand setObject:fWhereToPaste newFrame:MIUnionRect([fWhereToPaste modelFrame], [fWhereToPaste minimalRectToFitContent])];
		[fEnlargeCommand _doIt];
	}
	
	if([fListOfObjectsPasted count] > 0)
		return kMICmdStatusDone;
	else
		return kMICmdStatusCancelled;
}

- (void)undoIt
{
	if (fEnlargeCommand)
		[fEnlargeCommand _undoIt];

	fUndoCommand = [[MIDeleteCommand alloc] initWithListOfObjects:fListOfObjectsPasted];
	[fUndoCommand _doIt];
	
}

- (void)redoIt
{
	[fUndoCommand _undoIt];

	if (fEnlargeCommand)
		[fEnlargeCommand _redoIt];
}

- (MIObject*)pasteRecursive:(MIObject*)objInClipBoard to:(MIObject*)parentInDoc at:(ArmeTime)pasteTime atVPos:(MIVPos)pasteAbsoluteVPos withSuffix:(SInt32)suffix
{
	MIObject* newObject;
	TArmeWSObj* newArmeObj;
	TArmeWSObj* obj = [objInClipBoard WSObject];
	TArmeWSObjPos pos;
	pos.fInput = 0;
	
	//determine the paste position
	pos.fParent = [parentInDoc WSObject];
	if (pasteTime == ARME_TIME_UNDEF)
		pos.fStartTime = [objInClipBoard WSObject]->GetPos().fStartTime;
	else
		pos.fStartTime = pasteTime;
	pos.fVerticalPos = GET_VALUE(pasteAbsoluteVPos) - GET_VALUE([parentInDoc getAbsoluteVerticalPos]);

	switch (obj->GetType())
	{
		case TArmeWSObj::kAudioClip :
		{
			MEAudioClip* clipboardAudioClip = dynamic_cast<MEAudioClip*>(obj);
			ZzAssert(clipboardAudioClip);
			// inter-doc copying is not supported yet
			ZzAssert(clipboardAudioClip->GetData()->GetDoc() == [fMontageDoc armeDocument]);
			
			// now let's create a new audio clip that references the same audio clip data, but in this document
			// instead of the NULL document...
			MEAudioClip* audioClip = new MEAudioClip([fMontageDoc armeDocument], clipboardAudioClip->GetData(), true); // create a copy, not an alias
			audioClip->SetMuted([objInClipBoard WSObject]->GetMuted());
			audioClip->SetPos(pos);
			audioClip->SetNameCopy(obj->GetName());
			audioClip->SetVAORouting(obj->GetVAORouting());
			newArmeObj = audioClip;			
		    newObject = [[[MIObject alloc] initWithArmeWSObj:newArmeObj] autorelease];
			[newObject setMontageDoc:fMontageDoc];
			
			[self modifiedAudioHierarchy];
		}
		break;
		case TArmeWSObj::kVideoClip :
		{
			// if obj in clipboard comes from another document as the current one
			/* Inter-doc copy is not more supported
			if ([objInClipBoard WSObject]->GetDoc() != [fMontageDoc armeDocument])
			{
				//first, we need to copy the movie file from the temp directory to the project folder.  We need to check if such a filename is already
				//in use or not, if yes, we need to generate an alternative filename
			
				ArmeDataFileRef* dataFileRef = [fMontageDoc armeDocument]->GetNewVideoDataFileRef();
				TVideoClip* video = (TVideoClip*)obj;
				char newFilePath[kMaxPathLength];
				dataFileRef->GetFilePath(newFilePath);
				NSString* newPath = [[NSString alloc] initWithCString:newFilePath];
				NSString* oldPath = [[NSString alloc] initWithCString:video->fSourceFile];
		
				// copy
				[[NSFileManager defaultManager] copyPath:oldPath toPath:newPath handler:nil];
	
				ZzSplitTask* splitTask = NULL;
				ArmeErr err;
				
				// create a new object by importing it from its new location
				newArmeObj = new TVideoClip(dataFileRef, [fMontageDoc armeDocument], [MICommand getCurrentProgressDialog], &err, pos);
			}
			else // the object in clipboard comes from the same document as this one
			*/ 
			{
				newArmeObj = new TVideoClip((TVideoClip*)[objInClipBoard WSObject], [fMontageDoc armeDocument], pos);
				newArmeObj->SetMuted([objInClipBoard WSObject]->GetMuted());
			}
			newObject = [[[MIObject alloc] initWithArmeWSObj:newArmeObj] autorelease];
			[newObject setMontageDoc:fMontageDoc];

			[self modifiedVideoHierarchy];
		};break;
		case TArmeWSObj::kTextClip :
		{
			newArmeObj = new TArmeTextClip((TArmeTextClip*)[objInClipBoard WSObject], [fMontageDoc armeDocument], pos);
			newArmeObj->SetMuted([objInClipBoard WSObject]->GetMuted());
			newObject = [[[MIObject alloc] initWithArmeWSObj:newArmeObj] autorelease];
			[newObject setMontageDoc:fMontageDoc];

			[self modifiedVideoHierarchy];
		};break;
		case TArmeWSObj::kContainer :
		case TArmeWSObj::kAudioProcessor :
		case TArmeWSObj::kVideoEffect :
		{
			if (obj->GetType() == TArmeWSObj::kContainer)
			{
				newArmeObj = new TArmeContainer(obj->GetName(), [fMontageDoc armeDocument]);
				newArmeObj->SetVAORouting(obj->GetVAORouting());
			}
			else if (obj->GetType() == TArmeWSObj::kAudioProcessor)
			{
				newArmeObj = new TArmeContainer(obj->GetName(), [fMontageDoc armeDocument]);
				newArmeObj->CopyAudioPluginFrom(obj);
				newArmeObj->SetVAORouting(obj->GetVAORouting());
			}
			else if (obj->GetType() == TArmeWSObj::kVideoEffect)
			{
				ArmeErr err;
				newArmeObj = new TVideoEffect([fMontageDoc armeDocument], &err, NULL);
				newArmeObj->SetNameCopy(obj->GetName());
				newArmeObj->SetVAORouting(obj->GetVAORouting());
				((TVideoEffect*)newArmeObj)->CopyEffectDescFrom((TVideoEffect*)obj);
			}

			newArmeObj->SetMuted([objInClipBoard WSObject]->GetMuted());
			newArmeObj->SetPos(pos);
			newArmeObj->_SetLength([objInClipBoard WSObject]->GetDuration());

		    newObject = [[[MIObject alloc] initWithArmeWSObj:newArmeObj] autorelease];
			[newObject setMontageDoc:fMontageDoc];

			[self modifiedVideoHierarchy];
			[self modifiedAudioHierarchy];
			
		};break;
		default :
			return NULL;
	}	

	NSString* originalName = [NSString stringWithUTF8String:[newObject WSObject]->GetName()]; // autoreleased
        NSString* newName = [MIPasteLowLevelCommand computeNewNameFromOldName:originalName suffix:suffix];
	[newObject WSObject]->SetNameCopy([newName UTF8String]);

	// sets the attributes of the newObject to the one in the clipboard
	[newObject setVerticalPos:_MIVPos(pos.fVerticalPos)];
	[newObject setHeight:[objInClipBoard getHeight]];
	newObject->fOldHeight = objInClipBoard->fOldHeight;
	[newObject createAllRepresentations];
	[newObject setRepresentationsNeedDisplay];
	
	
	// add it to the hierarchy of MIObject
	[parentInDoc addChild:newObject];
	
	//recursive call
	NSEnumerator* childEnum = [[objInClipBoard getChildren] objectEnumerator];
	MIObject* child;
	while (child = [childEnum nextObject])
	{
		[self pasteRecursive:child to:newObject at:ARME_TIME_UNDEF atVPos:([newObject getAbsoluteVerticalPos] + [child getVerticalPos]) withSuffix:suffix];
	}
	[self overviewNeedsUpdate];
	
	return newObject;
}

+(NSString*)computeNewNameFromOldName:(NSString*)originalName suffix:(SInt32)suffix
{
//    kSuffixSplitLeft = 0,
//    kSuffixSplitRight,
//    kSuffixCopy,
//    kSuffixNumberZero
    if(suffix == kSuffixSplitLeft)
    {
        NSString* retString = [NSString stringWithFormat:@"%@\\",originalName];
        return retString;
    }
    if(suffix == kSuffixSplitRight)
    {
        NSString* retString = [NSString stringWithFormat:@"%@/",originalName];
        return retString;
    }
    if(suffix >= kSuffixNumberZero)
    {
        SInt32 number = suffix - kSuffixNumberZero;
        NSString* retString = [NSString stringWithFormat:@"%@ %d",originalName,number];
        return retString;
    }
    if(suffix == kSuffixCopy) // a se corse TRS TRS considrablement !!!!!!!!!!!!!!!!!!!!
    {
        // first we need to check whether the original name
        // complies to something like "WHATEVER(Copy NNNNN)"
        SInt32 length = [originalName length];
        bool isComplyingString = false; // unless we prove this wrong... see below
        
        // the following two variables are valid, after the next "if" block,
        // only if (isPotentialComplyingString != false)
        SInt32 number; // the number NNNNN, 
        SInt32 baseLength; // the name length, minus the "(Copy NNNNNN)" suffix
                        
        if(length >= 1)
        {
            unichar* rawOriginalName = new unichar[length];
            [originalName getCharacters:rawOriginalName]; // now rawOriginalName contains a bunch of unichar
            SInt32 currentIndex = length-1;
            if(rawOriginalName[currentIndex] == ')')
            {
                currentIndex--;
                if(currentIndex >= 0)
                {
                    while( (currentIndex >= 0) && ( (rawOriginalName[currentIndex] >= '0') && (rawOriginalName[currentIndex] <= '9') ) )
                    {
                        currentIndex--;
                    }
                    if( (currentIndex-5) >= 0 && 
                        (rawOriginalName[currentIndex] == ' ') && 
                        (rawOriginalName[currentIndex-1] == 'y') && 
                        (rawOriginalName[currentIndex-2] == 'p') && 
                        (rawOriginalName[currentIndex-3] == 'o') && 
                        (rawOriginalName[currentIndex-4] == 'C') && 
                        (rawOriginalName[currentIndex-5] == '('))
                    {
                        isComplyingString = true;
                        baseLength = currentIndex-5;
                        // the start index of the number is currentIndex+1
                        // the last index of the number (inclusive) is length-2
                        SInt32 baseExponent = 1;
                        number = 0;
                        for(SInt32 numberIndex = length-2; numberIndex >= currentIndex+1; numberIndex--)
                        {
                            number += baseExponent*(rawOriginalName[numberIndex] - '0');
                            baseExponent *= 10;
                        }
                    }
                }
            }
            delete [] rawOriginalName;
        }
        if(isComplyingString)
        {
            NSString* baseName;
            if(baseLength > 0)
                baseName = [originalName substringToIndex:baseLength];
            else
                baseName = @"";
            
            NSString* retString = [NSString stringWithFormat:@"%@(Copy %d)",baseName,number+1];
            return retString;
        }
        else
        {
            NSString* retString = [NSString stringWithFormat:@"%@(Copy 1)",originalName];
            return retString;
        }
    }
	return originalName; // SHOULD NEVER REACH THIS POINT
}

@end




