/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MIRepositionLockedObjectsCommand.h"

#import "MIObject.h"
#import "MIObjectRep.h"
#import "MIMainViewController.h"
#import "MIMoveCommand.h"
#import "TArmeWSObj.h"
#import "MontageInterface.h"

#ifndef ArmeArchiveHandler_h
#include "ArmeArchiveHandler.h"
#endif //ArmeArchiveHandler_h

@implementation MIRepositionLockedObjectsCommand

- (NSString*)commandName
{
	return @"Reposition locked objects";
}

- (MICmdStatus)doIt
{
	NSEnumerator	*objEnum;
	MIObject		*anObj;
	
	// check the hierarchy for things to move (excluding children of things to move)
	objEnum = [[fSelection putRepsInArrayAndRemoveDescendants] objectEnumerator];
	while ( anObj = [[objEnum nextObject] representedObject] )
	{
		[self checkObjectChildren:anObj];
	}

	return [super doIt];
}

- (void)checkObjectChildren:(MIObject*)obj
{
	NSEnumerator	*objEnum;
	MIObject		*anObj;
	
	objEnum = [[obj getChildren] objectEnumerator];
	while ( anObj = [objEnum nextObject] )
	{
		if ( [anObj isLocked] )
		{
			// check that time is correct 
			if ( GET_VALUE([anObj WSObject]->GetAbsoluteStartTime()) != GET_VALUE([anObj lockedToTime]) )
			{
				// create a move command that leaves the parent intact but changes the time
				MIMoveCommand	*cmd = [[[MIMoveCommand alloc] initWithSelection:nil] autorelease];
				[cmd setObject:anObj];
				[cmd setOffset:MIMakePoint([anObj lockedToTime] - [anObj WSObject]->GetAbsoluteStartTime(), _MIVPos(0))];
				[self addCommand:cmd];
			}
		}
		else
		{
			// check children
			[self checkObjectChildren:anObj];
		}
	}
}

@end
