/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
#import "MIResizeCommand.h"
#import "MITopLevelContainerRep.h"
#import "MISingleMoveCommand.h"

#ifndef ArmeArchiveHandler_h
#include "ArmeArchiveHandler.h"
#endif //ArmeArchiveHandler_h

#ifndef TArmeWSObj_h
#include "TArmeWSObj.h"
#endif //TArmeWSObj_h

#import "MIMainViewController.h"
#import "MIObject.h"


/****************************************************************************************************

MIResizeCommand

****************************************************************************************************/

@implementation MIResizeCommand
- (void)dealloc
{
	[fExtraCommands release];
	[fEnlargeParentCommand release];
	[fTheObject release];
	[super dealloc];
}

- (NSString*)commandName
{
	return @"Resize";
}

- (id)initWithObject:(MIObject*)object
{
	self = [super initWithSelection:nil];

	if ( self )
	{
		fExtraCommands = [[NSMutableArray alloc] initWithObjects:nil];
		fNewFrame = MIZeroRect;
		fOldFrame = MIZeroRect;
		fTheObject = [object retain];
	}
	
	return self;
}

- (id)initWithSelection:(MIWSSelection*)sel
{
	self = [super initWithSelection:sel];

	if ( self )
	{
		[fSelection discardVideoTransitions];

		fExtraCommands = [[NSMutableArray alloc] initWithObjects:nil];
		fNewFrame = MIZeroRect;
		fOldFrame = MIZeroRect;
		fTheObject = nil;
		
		[self modifiedVideoHierarchy];
		[self modifiedAudioHierarchy];
		[self overviewNeedsUpdate];
		
	}
	
	return self;
}

-(id)initFromArchive:(ArmeArchiveHandler*)handler
{
    if (( self = [super initFromArchive:handler] ))
    {
		//fController = [handler->ReadObjCObject() retain];
		fExtraCommands = [handler->ReadObjCObject() retain];
		fNewFrame = handler->ReadMIRect();
		fOldFrame = handler->ReadMIRect();
		fTheObject = [handler->ReadObjCObject() retain];
		fEnlargeParentCommand = [handler->ReadObjCObject() retain];
    }
    return self ;
}

-(BOOL)writeToArchive:(ArmeArchiveHandler*)handler
{
    if ( [super writeToArchive:handler] )
    {
		//handler->WriteObjCObject(fController);
		handler->WriteObjCObject(fExtraCommands);
		handler->WriteMIRect(fNewFrame);
		handler->WriteMIRect(fOldFrame);
		handler->WriteObjCObject(fTheObject);
		handler->WriteObjCObject(fEnlargeParentCommand);
		return YES;
    }
    else return NO;
}

- (void)setObject:(MIObject*)theObject newFrame:(MIRect)newFrame
{
	[fTheObject release];
	fTheObject = [theObject retain];
	fNewFrame = newFrame;
}

+ (BOOL)supportsSelection:(MIWSSelection*)sel
{
	if ( [sel containsOnlyVideoTransitions])
		return NO;

	if ( [sel type] != kMIWSSelectionTypeObjects )
		return NO;
	else
		return YES;
}

- (BOOL)displaySetupDialog
{
	return YES;
}

- (BOOL)createMoveCommandsForChildren
{
//	id						theRep;
	MIPoint					oldFrameOrigin;
	MIPoint					newFrameOrigin;
	MIPoint					deltaPos;
//	MITopLevelContainerRep	*theTLC;
	MISingleMoveCommand		*cmd;

	NSEnumerator			*children;
	MIObject				*aChild;
	MIPoint					aChildNewOrigin;
	MIPoint					aChildOldOrigin;


//	theTLC = [fController topLevelContainerView];

//	theRep = [fTheObject findRepresentationInTLC:theTLC];
	oldFrameOrigin = fOldFrame.origin;
	newFrameOrigin = fNewFrame.origin;

	if ( MIEqualPoints(oldFrameOrigin, newFrameOrigin))
		return NO;

	GET_VALUE(deltaPos.time) = GET_VALUE(oldFrameOrigin.time) - GET_VALUE(newFrameOrigin.time);
	GET_VALUE(deltaPos.vpos) = GET_VALUE(oldFrameOrigin.vpos) - GET_VALUE(newFrameOrigin.vpos);

	children = [[fTheObject getChildren] objectEnumerator];

	while ( aChild = [children nextObject] )
	{
		if ([aChild WSObject]->GetType() != TArmeWSObj::kVideoXFadeEffect)
		{
			aChildOldOrigin = [aChild modelFrame].origin;
			GET_VALUE(aChildNewOrigin.time) = GET_VALUE(aChildOldOrigin.time) + GET_VALUE(deltaPos.time);
			GET_VALUE(aChildNewOrigin.vpos) = GET_VALUE(aChildOldOrigin.vpos) + GET_VALUE(deltaPos.vpos);
	
			cmd = [[[MISingleMoveCommand alloc] initWithSelection:nil] autorelease];
			[cmd setObject:aChild];
			[cmd setNewOrigin:aChildNewOrigin];
			[cmd setNewParent:fTheObject];
			[fExtraCommands addObject:cmd];
		}
	}
	return YES;
}

- (MICmdStatus)doIt
{
	NSEnumerator	*cmdEnum;
	MICommand		*cmd;

	fOldFrame = [fTheObject modelFrame];
	
	// create a move subcommand for each children that has to move
	if ( !MIEqualPoints( fOldFrame.origin, fNewFrame.origin ) )
		[self createMoveCommandsForChildren];

	// resize the object itself
	if ( ![fTheObject isTLC] )
	{
		[fTheObject setRepresentationsNeedDisplay];
		[fTheObject setModelFrame:fNewFrame];
		[fTheObject setRepresentationsNeedDisplay];
	}
	else
	{
		fNewFrame.origin = MIZeroPoint;
//		fNewFrame.origin.time = _ArmeTime(0);
		[fTheObject setModelFrame:fNewFrame];
		[fTheObject setRepresentationsNeedDisplay];
	}
	// execute the move commands for each child
	cmdEnum = [fExtraCommands objectEnumerator];

	while ( cmd = [cmdEnum nextObject] )
	{
		[cmd _doIt];
	}

	// resize the parent
	if ( ![fTheObject isTLC] )
	{
		MIObject *parent = [fTheObject parentObject];
		
		if ( parent && !MIContainsRect( [parent modelFrame], [parent minimalRectToFitContent] ) )
		{
			fEnlargeParentCommand = [[MIResizeCommand alloc] initWithSelection:nil];
			[fEnlargeParentCommand setObject:parent newFrame:MIUnionRect([parent modelFrame], [parent minimalRectToFitContent])];
			[fEnlargeParentCommand _doIt];
		}
	}
	return kMICmdStatusDone;
}

- (void)undoIt
{
	NSEnumerator	*cmdEnum;
	MICommand		*cmd;

	[fEnlargeParentCommand _undoIt];

	cmdEnum = [fExtraCommands objectEnumerator];

	while ( cmd = [cmdEnum nextObject] )
	{
		[cmd _undoIt]; 
	}

	if ( ![fTheObject isTLC] )
	{
		[fTheObject setRepresentationsNeedDisplay];
//		[fTheObject setModelFrameFromRect:fOldFrame inTLC:[fController topLevelContainerView]];
		[fTheObject setModelFrame:fOldFrame];
		[fTheObject setRepresentationsNeedDisplay];
	}
	else
	{
		[fTheObject setModelFrame:fOldFrame];
		[fTheObject setRepresentationsNeedDisplay];
	}
}

- (void)redoIt
{
	NSEnumerator	*cmdEnum;
	MICommand		*cmd;

	if ( ![fTheObject isTLC] )
	{
		[fTheObject setRepresentationsNeedDisplay];
//		[fTheObject setModelFrameFromRect:fNewFrame inTLC:[fController topLevelContainerView]];
		[fTheObject setModelFrame:fNewFrame];
		[fTheObject setRepresentationsNeedDisplay];
	}
	else
	{
		[fTheObject setModelFrame:fNewFrame];
		[fTheObject setRepresentationsNeedDisplay];
	}
	
	cmdEnum = [fExtraCommands objectEnumerator];

	while ( cmd = [cmdEnum nextObject] )
	{
		[cmd _redoIt];
	}

	[fEnlargeParentCommand _redoIt];
}

@end
