/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MISilenceCommand.h"
#import "MIAudioProcessingCommand.h"
#import "MIWSSelection.h"
#import "MIObjectRep.h"
#import "MIObject.h"
#import "MIFreezeCommand.h"
#import "MICopyCommand.h"
#import "MIDeleteCommand.h"
#import "MIInsertLowLevelCommand.h"
#import "MIDeleteLowLevelCommand.h"
#import "MIPasteLowLevelCommand.h"
#import "MIMontageDocument.h"
#import "MIInsertSilenceDlgCtrller.h"


#ifndef OfflineAudioProcessorSilence_h
#include "OfflineAudioProcessorSilence.h"
#endif //OfflineAudioProcessorSilence_h

/****************************************************************************************************

MISilenceCommand

****************************************************************************************************/

@implementation MISilenceCommand

-(id)initFromArchive:(ArmeArchiveHandler*)handler
{
	self = [super initFromArchive:handler];
	return self;
}

-(BOOL)writeToArchive:(ArmeArchiveHandler*)handler
{
	return [super writeToArchive:handler];
}

- (NSString*)commandName
{
	return @"Silence";
}

- (id)initWithSelection:(MIWSSelection*)sel
{
	self = [super initWithSelection:sel];
	
	[fSelection adjustSelectionTimesToVideo];
	
	if ( self )
	{
		NSEnumerator* selEnum = [sel objectEnumerator];
		MIObject* obj;
		while (obj = [[selEnum nextObject] representedObject])
		{
			if ([obj WSObject]->GetType() == TArmeWSObj::kAudioClip)
			{
				MIAudioProcessingCommand* silenceCommand = [[[MIAudioProcessingCommand alloc] initWithSelection:fSelection
					offlineAudioProcessor:OfflineAudioProcessorSilence::GetInstance()] autorelease];
				OfflineAudioProcessorParams* baseParams = [silenceCommand processorParams];
				OfflineSilenceParams* params = dynamic_cast<OfflineSilenceParams*>(baseParams);
				params->fDuration = kOfflineSilenceReplaceSelWithSilence;
				params->fMontageDoc = [[[sel objectAtIndex:0] representedObject] montageDoc];
				[self addCommand:silenceCommand];
			}
		/*	Not with video clip in Montage 1.0
			else if ([obj WSObject]->GetType() == TArmeWSObj::kVideoClip)
			{
				[self modifiedVideoHierarchy];
				[self addCommand:[[[MISilenceVideoCommand alloc] initWithObjectToSilence:obj from:[sel startTime] to:[sel endxTime] insert:NO] autorelease]];
			}*/
		}
	}
	return self;
}

- (BOOL)displaySetupDialog
{
	if ([self CheckFrozenObjectInSelectionAndAskIfUserWantsToContinue:fSelection] == NO)
		return NO;
	else
		return YES;
}

+ (BOOL)supportsSelection:(MIWSSelection*)sel
{
	if (( [sel type] == kMIWSSelectionTypeTimeSlice) && ([sel duration] > _ArmeTime(0)))
	{
		NSEnumerator* selEnum = [sel objectEnumerator];
		MIObject* obj;
		while (obj = [[selEnum nextObject] representedObject])
		{
			if (([obj WSObject]->GetType() != TArmeWSObj::kAudioClip) /*&&  // not working with video clips in Montage 1.0
				([obj WSObject]->GetType() != TArmeWSObj::kVideoClip)*/)
				return NO;
		}
		return YES;
	}
	else
		return NO;
}

@end

@implementation MISilenceVideoCommand
- (void)dealloc
{
	[fObject release];
	
	[super dealloc];
} 

- (id)initWithObjectToSilence:(MIObject*)obj from:(ArmeTime)from to:(ArmeTime)to insert:(BOOL)insertSilence
{
	self = [super initWithSelection:nil];
	if (self)
	{
		fObject = [obj retain];
		fFrom = from;
		fTo = to;
		fCompleteObject = false;		
		fInsertSilence = insertSilence;

		if ((fFrom == ARME_TIME_UNDEF) || (fTo == ARME_TIME_UNDEF))
		{
			fFrom = [fObject WSObject]->GetAbsoluteStartTime();
			fTo = [fObject WSObject]->GetAbsoluteEndxTime();
			fCompleteObject = true;
		}
	}
	return self;
}

-(id)initFromArchive:(ArmeArchiveHandler*)handler 
{
	self = [super initFromArchive:handler];
	fObject = [handler->ReadObjCObject() retain];
	fFrom = handler->ReadArmeTime();
	fTo = handler->ReadArmeTime();
	fCompleteObject = handler->ReadBOOL();
	fInsertSilence = handler->ReadBOOL();

	return self;
}
-(BOOL)writeToArchive:(ArmeArchiveHandler*)handler
{
	[super writeToArchive:handler];

	handler->WriteObjCObject(fObject);
	handler->WriteArmeTime(fFrom);
	handler->WriteArmeTime(fTo);
	handler->WriteBOOL(fCompleteObject);
	handler->WriteBOOL(fInsertSilence);
	
	return YES;
}
- (NSString*)commandName
{
	return @"SilenceVideo";
}
- (MICmdStatus)doIt
{
	MICommand* cmd;
	
	//note : each command uses the result of the previous one.  That's why they are created in the 'doIt' and why they are executed one by one
	NSMutableArray* array1 = [[[NSMutableArray alloc] initWithCapacity:1] autorelease];
	[array1 addObject:fObject];

	ArmeErr err;
	// what we do here is : render a movie where it does not contain any data so the output is black (and it should be fast to render)
	MIObject* blackClip = Render(err, array1, [[fObject montageDoc] topLevelContainer], _MIVPos(0), _MIVPos(0), [MICommand getCurrentProgressDialog], false, [fObject WSObject]->GetAbsoluteEndxTime() + fFrom, [fObject WSObject]->GetAbsoluteEndxTime() + fTo, false, false, true);

	//$$TODO, handle the error

	if (fCompleteObject)
	{
		NSMutableArray* array = [[[NSMutableArray alloc] initWithCapacity:1] autorelease];
		[array addObject:blackClip];

		// give the same name
		[blackClip WSObject]->SetNameCopy([fObject WSObject]->GetName());
		
		//paste the rendered copy at the original object location
		MIPasteLowLevelCommand* pasteCmd2 = [[[MIPasteLowLevelCommand alloc] initWithDocument:[fObject montageDoc] objectsToPaste:array inObject:[fObject parentObject] atTime:[fObject WSObject]->GetPos().fStartTime atVPos:[fObject getAbsoluteVerticalPos]] autorelease];
		[self addCommand:pasteCmd2];
		[pasteCmd2 _doIt];
//		MIObject* copy2 = [pasteCmd2->fListOfObjectsPasted objectAtIndex:0];
		
		//delete the selection
		cmd = [[[MIDeleteLowLevelCommand alloc] initWithObjectToDelete:fObject from:ARME_TIME_UNDEF to:ARME_TIME_UNDEF] autorelease];
		[self addCommand:cmd];
		[cmd _doIt];
	}
	else
	{
		if (fInsertSilence)
		{
			//this command does nothing except that it provide a good visual feedback (need to understand why)
			cmd = [[[MIDeleteLowLevelCommand alloc] initWithObjectToDelete:fObject from:fFrom to:fFrom] autorelease];
			[self addCommand:cmd];
			[cmd _doIt];
		}
		else
		{
			//delete the selection
			cmd = [[[MIDeleteLowLevelCommand alloc] initWithObjectToDelete:fObject from:fFrom to:fTo] autorelease];
			[self addCommand:cmd];
			[cmd _doIt];
		}
		

		//paste the rendered copy in the original object
		cmd = [[[MIInsertLowLevelCommand alloc] initWithSelection:nil objectToInsert:blackClip atPosition:fFrom inObject:fObject] autorelease];
		[self addCommand:cmd];
		[cmd _doIt];
	}
	 
	 //delete the blackClip
	MICommand* deleteRenderedCopyCommand = [[[MIDeleteCommand alloc] initWithObject:blackClip] autorelease];
	[deleteRenderedCopyCommand _doIt];
	
	return kMICmdStatusDone;
}
- (void)undoIt
{
	[super undoIt];
}

- (void)redoIt
{
	[super redoIt];
}

- (int)getNbLengthyTasks
{
	return 1;  //only the freeze is considered as lengthy
}
- (BOOL)displaySetupDialog
{
	if (fInsertSilence)
	{
		NSModalSession session;
		NSWindow* 	win;
		
		MIInsertSilenceDlgCtrller* ctrller =
			[MIInsertSilenceDlgCtrller alloc];
		[ctrller initWithWindowNibName:@"MIInsertSilenceDlg" owner:ctrller];
		win = [ctrller window];
		[ctrller setMontageDocument:[fObject montageDoc]];
		[ctrller setInitialDuration:_ArmeTime(0)];
		
		session = [NSApp beginModalSessionForWindow:win];
		while([NSApp runModalSession:session] == NSRunContinuesResponse)
		{
		}
		[NSApp endModalSession:session];					
		bool ret;
		if([ctrller duration] <= _ArmeTime(0))
		{
			ret = false; // duration 0 OR cancelled dialog, which is the same.... we don't insert anythin
		}
		else
		{
			fTo = fFrom + [ctrller duration];
			ret = true;
		}
		[ctrller release];
		[win orderOut:0];
		return ret;
	}
	else
		return YES;
}
@end


