/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MISingleCollapseExpandCommand.h"
#import "MIObject.h"
#import "MIResizeCommand.h"

@implementation MISingleCollapseCommand
-(id)initFromArchive:(ArmeArchiveHandler*)handler
{
	self = [super initFromArchive:handler];
	if ( self )
	{
		fTheObject = [handler->ReadObjCObject() retain];
		fEnlargeCommand = [handler->ReadObjCObject() retain];
	}
	return self;
}

-(BOOL)writeToArchive:(ArmeArchiveHandler*)handler
{
	[super writeToArchive:handler];
	
	handler->WriteObjCObject(fTheObject);
	handler->WriteObjCObject(fEnlargeCommand);
	return YES;
}

- (void)dealloc
{
	[fTheObject release];
	[fEnlargeCommand release];
	[super dealloc];
}

- (NSString*)commandName
{
	return @"Collapse";
}

- (id)initWithSelection:(MIWSSelection*)sel
{
	self = [super initWithSelection:sel];
	if ( self )
	{
		fTheObject = nil;
	}

	return self;
}

- (void)setObject:(MIObject*)object
{
	[fTheObject release];
	fTheObject = [object retain];
}

- (BOOL)displaySetupDialog
{
	return YES;
}

- (MICmdStatus)doIt
{
	[fTheObject setRepresentationsNeedDisplay];
	[fTheObject setCollapsed:YES];
	[fTheObject recreateFrame];
	[fTheObject setRepresentationsNeedDisplay];

	return kMICmdStatusDone;
}

- (void)undoIt
{
	[fTheObject setRepresentationsNeedDisplay];
	[fTheObject setCollapsed:NO];
	[fTheObject recreateFrame];
	[fTheObject setRepresentationsNeedDisplay];

	if ( !fEnlargeCommand )
	{
		// check if parent size is big enough
		if ( !MIContainsRect( [[fTheObject parentObject] modelFrame], [[fTheObject parentObject] minimalRectToFitContent] ) )
		{
			fEnlargeCommand = [[MIResizeCommand alloc] initWithSelection:nil];
			[fEnlargeCommand setObject:[fTheObject parentObject] newFrame:MIUnionRect([[fTheObject parentObject] modelFrame], [[fTheObject parentObject] minimalRectToFitContent])];
			[fEnlargeCommand _doIt];
		}
	}
	else
	{
		[fEnlargeCommand _redoIt];
	}
}

- (void)redoIt
{
	[fTheObject setRepresentationsNeedDisplay];
	[fTheObject setCollapsed:YES];
	[fTheObject recreateFrame];
	[fTheObject setRepresentationsNeedDisplay];
	[fEnlargeCommand _undoIt];

}

+ (BOOL)supportsSelection:(MIWSSelection*)sel
{
	if ( [sel type] != kMIWSSelectionTypeObjects )
		return NO;
	else
		return YES;
}

@end












@implementation MISingleExpandCommand
-(id)initFromArchive:(ArmeArchiveHandler*)handler
{
	self = [super initFromArchive:handler];
	if ( self )
	{
		fTheObject = [handler->ReadObjCObject() retain];
		fEnlargeCommand = [handler->ReadObjCObject() retain];
	}
	return self;
}

-(BOOL)writeToArchive:(ArmeArchiveHandler*)handler
{
	[super writeToArchive:handler];
	
	handler->WriteObjCObject(fTheObject);
	handler->WriteObjCObject(fEnlargeCommand);
	return YES;
}

- (void)dealloc
{
	[fTheObject release];
	[fEnlargeCommand release];
	[super dealloc];
}

- (NSString*)commandName
{
	return @"Expand";
}

- (id)initWithSelection:(MIWSSelection*)sel
{
	self = [super initWithSelection:sel];
	if ( self )
	{
		fTheObject = nil;
	}

	return self;
}

- (void)setObject:(MIObject*)object
{
	[fTheObject release];
	fTheObject = [object retain];
}

- (BOOL)displaySetupDialog
{
	return YES;
}

- (MICmdStatus)doIt
{
	[fTheObject setRepresentationsNeedDisplay];
	[fTheObject setCollapsed:NO];
	[fTheObject recreateFrame];
	[fTheObject setRepresentationsNeedDisplay];


	if ( !fEnlargeCommand )
	{
		// check if parent size is big enough
		if ( !MIContainsRect( [[fTheObject parentObject] modelFrame], [[fTheObject parentObject] minimalRectToFitContent] ) )
		{
			fEnlargeCommand = [[MIResizeCommand alloc] initWithSelection:nil];
			[fEnlargeCommand setObject:[fTheObject parentObject] newFrame:MIUnionRect([[fTheObject parentObject] modelFrame], [[fTheObject parentObject] minimalRectToFitContent])];
			[fEnlargeCommand _doIt];
		}
	}
	return kMICmdStatusDone;
}

- (void)undoIt
{
	[fTheObject setRepresentationsNeedDisplay];
	[fTheObject setCollapsed:YES];
	[fTheObject recreateFrame];
	[fTheObject setRepresentationsNeedDisplay];
	[fEnlargeCommand _undoIt];
}

- (void)redoIt
{
	[fTheObject setRepresentationsNeedDisplay];
	[fTheObject setCollapsed:NO];
	[fTheObject recreateFrame];
	[fTheObject setRepresentationsNeedDisplay];
	[fEnlargeCommand _redoIt];

}

+ (BOOL)supportsSelection:(MIWSSelection*)sel
{
	if ( [sel type] != kMIWSSelectionTypeObjects )
		return NO;
	else
		return YES;
}

@end