/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MISingleMoveCommand.h"
#import "MIObjectPositionCombo.h"
#import "MIMainViewController.h"
#import "MITopLevelContainerRep.h"

#ifndef ArmeArchiveHandler_h
#include "ArmeArchiveHandler.h"
#endif //ArmeArchiveHandler_h

#ifndef TArmeWSObj_h
#include "TArmeWSObj.h"
#endif

/****************************************************************************************************

MISingleMoveCommand

****************************************************************************************************/


@implementation MISingleMoveCommand
- (void)dealloc
{
	[fOldPos release];
	[fNewPos release];
#ifdef DEBUG
//	printf("MISingleMoveCommand releasing MIObject (%p) in dealloc\n",(void*)fMovedObject);
#endif //#ifdef DEBUG
	[fMovedObject release];
	
	delete fModifiedFlagOldParent;
	delete fModifiedFlagNewParent;

	[super dealloc];
}

- (id)initWithSelection:(MIWSSelection*)sel
{
	self = [super initWithSelection:sel];

	[fSelection adjustSelectionTimesToVideo];
	
	if ( self )
	{
		[fSelection discardVideoTransitions];
	
		fOldPos = [[MIObjectPositionCombo alloc] init];
		[fOldPos setParent:nil];
		fNewPos = [[MIObjectPositionCombo alloc] init];
		[fNewPos setParent:nil];
		fMovedObject = nil;
		
		fModifiedFlagOldParent = nil;
		fModifiedFlagNewParent = nil;
		
		if ([fSelection containsVideo:true andTransparentTextClip:YES])
			[self modifiedVideoHierarchy];
		if ([fSelection containsAudio])
			[self modifiedAudioHierarchy];
		[self overviewNeedsUpdate];
	}

	return self;
}

- (id)initWithObject:(MIObject*)obj moveToParent:(MIObject*)parent atTime:(ArmeTime)time atVPos:(MIVPos)vpos
{
	self = [super initWithSelection:nil];

	if ( self )
	{
		fOldPos = [[MIObjectPositionCombo alloc] init];
		fNewPos = [[MIObjectPositionCombo alloc] init];

		[fNewPos setParent:parent];
		fNewPos->fOriginX = time;
		fNewPos->fOriginY = vpos;
		
		fMovedObject = [obj retain];

		fOldPos->fOriginX = [fMovedObject getHorizontalPos];
		fOldPos->fOriginY = [fMovedObject getVerticalPos];
		[fOldPos setParent:[fMovedObject parentObject]];

		fModifiedFlagOldParent = [[fMovedObject parentObject] getCurrentModifiedParentsFlag];
		fModifiedFlagNewParent = [parent getCurrentModifiedParentsFlag];;
		
		if ([obj WSObject]->ContainsVideo(false, true))
			[self modifiedVideoHierarchy];
		if ([obj WSObject]->ContainsAudio())
			[self modifiedAudioHierarchy];
		[self overviewNeedsUpdate];
	}

	return self;
}

-(id)initFromArchive:(ArmeArchiveHandler*)handler
{
    if (( self = [super initFromArchive:handler] ))
    {
		fMovedObject = [handler->ReadObjCObject() retain];
		fOldPos = [handler->ReadObjCObject() retain];
		fNewPos = [handler->ReadObjCObject() retain];

		fModifiedFlagOldParent = nil;
		fModifiedFlagNewParent = nil;
    }
    return self;
}

-(BOOL)writeToArchive:(ArmeArchiveHandler*)handler
{
    if ( [super writeToArchive:handler] )
    {
		handler->WriteObjCObject(fMovedObject);
		handler->WriteObjCObject(fOldPos);
		handler->WriteObjCObject(fNewPos);
		return YES;
    }
    else return NO;
}

- (void)setObject:(MIObject*)movedObject
{
	ZzAssert([movedObject WSObject]->GetType() != TArmeWSObj::kVideoXFadeEffect);
	
#ifdef DEBUG
//	printf("MISingleMoveCommand retaining MIObject (%p) for storage\n",(void*)movedObject);
#endif //#ifdef DEBUG
	fMovedObject = [movedObject retain];

	fOldPos->fOriginX = [movedObject getHorizontalPos];
	fOldPos->fOriginY = [movedObject getVerticalPos];
	[fOldPos setParent:[movedObject parentObject]];
//	fNewPos->fParent = fOldPos->fParent;

	fModifiedFlagOldParent = [[fMovedObject parentObject] getCurrentModifiedParentsFlag];
}

- (void)setNewParent:(MIObject*)newParent
{
	[fNewPos setParent:newParent];
	fModifiedFlagNewParent = [newParent getCurrentModifiedParentsFlag];;
}

- (MIObject*)newParent
{
	return fNewPos->fParent;
}

- (void)setNewOrigin:(ArmeTime)newOriginX vPos:(MIVPos)newOriginY;
{
	fNewPos->fOriginX = newOriginX;
	fNewPos->fOriginY = newOriginY;
}
- (void)setNewOrigin:(MIPoint)newOrigin
{
	fNewPos->fOriginX = newOrigin.time;
	fNewPos->fOriginY = newOrigin.vpos;
}

+ (BOOL)supportsSelection:(MIWSSelection*)sel
{
	if ( [sel containsOnlyVideoTransitions])
		return NO;

	if ( [sel type] == kMIWSSelectionTypeObjects
		 || [sel type] == kMIWSSelectionTypeTimeSlice
#ifdef AUTOMATION_DATA
		|| [sel type] == kMIWSSelectionTypeAutomationPoints
#endif
		 )
		return YES;
	else
		return NO;
}

- (BOOL)displaySetupDialog
{
	return YES;
}

- (MICmdStatus)doIt
{
	MIRect	fr1, fr2;
//	MIRect	tmpRect;

	if ( !fMovedObject )
		vernl( 1, "Error: MISingleMoveCOmmand::doIt called with NULL movedObject" );
	
	if ( !fNewPos->fParent )
		[fNewPos setParent:fOldPos->fParent];

	[fMovedObject setRepresentationsNeedDisplay];
	fr1 = [fMovedObject absoluteModelFrame];
	[fMovedObject moveToObject:fNewPos->fParent	atTime:fNewPos->fOriginX withVPos:fNewPos->fOriginY];
	fr2 = [fMovedObject absoluteModelFrame];
	[fMovedObject setRepresentationsNeedDisplay];

	[fMovedObject markParentModified];
//	[fMovedObject forceImmediateDisplayOfGuides:fr1];
//	[fMovedObject forceImmediateDisplayOfGuides:fr2];
	return kMICmdStatusDone;
}

- (void)undoIt
{
	[fMovedObject setRepresentationsNeedDisplay];
	[fMovedObject moveToObject:fOldPos->fParent	atTime:fOldPos->fOriginX withVPos:fOldPos->fOriginY];
	[fMovedObject setRepresentationsNeedDisplay];
	
	if (fModifiedFlagOldParent) [fOldPos->fParent resetModifiedFlag:fModifiedFlagOldParent];
	if (fModifiedFlagNewParent) [fNewPos->fParent resetModifiedFlag:fModifiedFlagNewParent];
}

- (void)redoIt
{
	[fMovedObject setRepresentationsNeedDisplay];
	[fMovedObject moveToObject:fNewPos->fParent	atTime:fNewPos->fOriginX withVPos:fNewPos->fOriginY];
	[fMovedObject setRepresentationsNeedDisplay];

	[fMovedObject markParentModified];
}

- (NSString*)commandName
{
	return @"SingleMove";
}

@end


