/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MISoloCommand.h"
#import "MIMainViewController.h"
#import "MIObjectRep.h"
#import "MIObject.h"
#import "MIMontageDocument.h"
#import "MIDocumentController.h"
#import "MITopLevelContainerRep.h"
#include "TVideoObject.h"
#include "TArmeDoc.h"


/****************************************************************************************************

MISoloCommand

****************************************************************************************************/


@implementation MISoloCommand

-(id)initFromArchive:(ArmeArchiveHandler*)handler
{
	[super initFromArchive:handler];

	fObjectToSolo = [handler->ReadObjCObject() retain];
	fPreviousVideoSoloObject = dynamic_cast<TArmeWSObj*>(handler->ReadCPPObject());
	fPreviousAudioSoloObject = dynamic_cast<TArmeWSObj*>(handler->ReadCPPObject());
//	fObjectWasMuted = handler->ReadBOOL();
	return self;
}

-(BOOL)writeToArchive:(ArmeArchiveHandler*)handler
{
	[super writeToArchive:handler];

	handler->WriteObjCObject(fObjectToSolo);
	handler->WriteCPPObject(fPreviousVideoSoloObject);
	handler->WriteCPPObject(fPreviousAudioSoloObject);
//	handler->WriteBOOL(fObjectWasMuted);
	return YES;
}

- (void)dealloc
{
	[fObjectToSolo release];
	
	[super dealloc];
}

- (NSString*)commandName
{
	return @"Solo";
}
- (id)initWithSelection:(MIWSSelection*)sel
{
	self = [super initWithSelection:sel];
	
	if ( self )
	{
		fPreviousVideoSoloObject = NULL;
		fPreviousAudioSoloObject = NULL;
		
		if (fSelection)
		{
			ZzAssert([fSelection type] == kMIWSSelectionTypeObjects);
			ZzAssert([fSelection count] == 1);
			fObjectToSolo = [[fSelection objectAtIndex:0] representedObject];
			[fObjectToSolo retain];
			
			[self modifiedVideoHierarchy];
			[self modifiedAudioHierarchy];
			[self overviewNeedsUpdate];
		}
	}
	return self;
}

- (id)initWithObject:(MIObject*)object
{
	fObjectToSolo = [object retain];
	return self;
}


- (BOOL)displaySetupDialog
{
	return YES;
}

- (MICmdStatus)doIt
{
	fPreviousVideoSoloObject = [[fObjectToSolo montageDoc] armeDocument]->GetVideoSoloObject();
	fPreviousAudioSoloObject = [[fObjectToSolo montageDoc] armeDocument]->GetAudioSoloObject();
//	fObjectWasMuted = [fObjectToSolo isMuted];

	if (![fObjectToSolo WSObject]->IsTheVideoSolo() && ![fObjectToSolo WSObject]->IsTheAudioSolo())
	{
		if ([fObjectToSolo WSObject]->ContainsVideo(true, true))
			[[fObjectToSolo montageDoc] armeDocument]->SetVideoSoloObject([fObjectToSolo WSObject]);
		if ([fObjectToSolo WSObject]->ContainsAudio(true))
			[[fObjectToSolo montageDoc] armeDocument]->SetAudioSoloObject([fObjectToSolo WSObject]);
	}
	else
	{	if ([fObjectToSolo WSObject]->IsTheVideoSolo())
			[[fObjectToSolo montageDoc] armeDocument]->SetVideoSoloObject(NULL);
		if ([fObjectToSolo WSObject]->IsTheAudioSolo())
			[[fObjectToSolo montageDoc] armeDocument]->SetAudioSoloObject(NULL);
	}
	
	//update all the views
	NSEnumerator* docEnum = [[[MIDocumentController sharedDocumentController] documents] objectEnumerator];
	MIMontageDocument* doc;
	while(doc = [docEnum nextObject])
	{
		NSEnumerator* mainViewEnum = [[doc windowControllers] objectEnumerator];
		MIMainViewController* mainViewController;
		while(mainViewController = [mainViewEnum nextObject])
		{
			[[mainViewController topLevelContainerView] setNeedsDisplay:YES];
		}
	}
	return kMICmdStatusDone;
}

- (void)undoIt
{
	[[fObjectToSolo montageDoc] armeDocument]->SetVideoSoloObject(fPreviousVideoSoloObject);
	[[fObjectToSolo montageDoc] armeDocument]->SetAudioSoloObject(fPreviousAudioSoloObject);

	//update all the views
	NSEnumerator* docEnum = [[[MIDocumentController sharedDocumentController] documents] objectEnumerator];
	MIMontageDocument* doc;
	while(doc = [docEnum nextObject])
	{
		NSEnumerator* mainViewEnum = [[doc windowControllers] objectEnumerator];
		MIMainViewController* mainViewController;
		while(mainViewController = [mainViewEnum nextObject])
		{
			[[mainViewController topLevelContainerView] setNeedsDisplay:YES];
		}
	}
}

- (void)redoIt
{
	[self doIt];
}

+ (BOOL)supportsSelection:(MIWSSelection*)sel
{
	if ( [sel containsOnlyVideoTransitions])
		return NO;

	if ( [sel type] != kMIWSSelectionTypeObjects )
		return NO;
	else
	{
		if ([sel count] != 1)
			return NO;
		else
			return YES;
	}
}


@end