/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MISplitCommand.h"
#import "MIMainViewController.h"
#import "MIObjectRep.h"
#import "MIObject.h"
#import "MIMontageDocument.h"
#import "MICutCommand.h"
#import "MIPasteLowLevelCommand.h"
#include "TVideoObject.h"
#include "TArmeDoc.h"
#import "MIRenameCommand.h"
#import "MIDeleteLowLevelCommand.h"
#import "MIDeleteCommand.h"
#import "MIApplication.h"
#import "MIApplicationController.h"
#import "MiTrimCommand.h"

/****************************************************************************************************

MISplitCommand

****************************************************************************************************/


@implementation MISplitCommand

-(id)initFromArchive:(ArmeArchiveHandler*)handler
{
	[super initFromArchive:handler];
	return self;
}

-(BOOL)writeToArchive:(ArmeArchiveHandler*)handler
{
	[super writeToArchive:handler];
	return YES;
}

- (void)dealloc
{
	[super dealloc];
}

- (NSString*)commandName
{
	return @"Split";
}

- (id)initWithSelection:(MIWSSelection*)sel
{
	self = [super initWithSelection:sel];
	
	if ( self )
	{
		if (fSelection)
		{
			[fSelection discardVideoTransitions];
			//[fSelection discardTextClips];
			
			if([fSelection type] == kMIWSSelectionTypeTimeSlice)
			{
				NSRunAlertPanel(@"Error", @"Error : The split command only works with whole objects selected, and splits them at the play bar location...", @"OK", nil, nil );
				fSplitTime == _ArmeTime(-1);
			}
			else
			{
				fSplitTime = [[[NSDocumentController sharedDocumentController] currentDocument] armeDocument]->GetPlayPos();
				//if there is video in the selection, we need to split on a frame boundary
				if ([fSelection containsVideo:true andTransparentTextClip:YES])
				{
					fSplitTime = [[[NSDocumentController sharedDocumentController] currentDocument] armeDocument]->RoundArmeTimeToNearestVideoFrame(fSplitTime);
				}
			}
			NSEnumerator* selEnum = [fSelection objectEnumerator];
			MIObjectRep* objRep;
			while (objRep = [selEnum nextObject])
			{
				if ((fSplitTime > [[objRep representedObject] WSObject]->GetAbsoluteStartTime()) && (fSplitTime < [[objRep representedObject] WSObject]->GetAbsoluteEndxTime()))
					[self addCommand:[[[MISplitLowLevelCommand alloc] initWithObject:[objRep representedObject] atTime:fSplitTime] autorelease]];
			}
			
			if ([fSelection containsVideo:true andTransparentTextClip:YES])
				[self modifiedVideoHierarchy];
			if ([fSelection containsAudio])
				[self modifiedAudioHierarchy];
			[self overviewNeedsUpdate];
	
		}
	}
	return self;
}

- (MICmdStatus)doIt
{
	[super doIt];
	[self selectSplitObjects];

	return kMICmdStatusDone;
}

- (void)undoIt
{
	[super undoIt];
	[self selectOriginalSelection];
}

- (void)redoIt
{
	[super redoIt];
	[self selectSplitObjects];	
}

- (void)selectSplitObjects
{
	[[MIApplicationController sharedApplicationController] clearAllSelections];

	for (unsigned int i=0; i < [fSubCommands count]; ++i)
	{
//		[[[fSubCommands objectAtIndex:i] getLeftPart] addToSelections];
		[[[fSubCommands objectAtIndex:i] getRightPart] addToSelections];
	}
}

- (BOOL)displaySetupDialog
{
	if ([self CheckFrozenObjectInSelectionAndAskIfUserWantsToContinue:fSelection] == NO)
		return NO;
	return YES;
}

+ (BOOL)supportsSelection:(MIWSSelection*)sel
{
	if ( [sel count] <= 0)
		return NO;

	if ( [sel containsOnlyVideoTransitions])
		return NO;

	if ( [sel type] != kMIWSSelectionTypeObjects )
		return NO;
	
	ArmeTime splitTime = [[[NSDocumentController sharedDocumentController] currentDocument] armeDocument]->GetPlayPos();
	if ([sel containsVideo:true andTransparentTextClip:YES])
		splitTime = [[[NSDocumentController sharedDocumentController] currentDocument] armeDocument]->RoundArmeTimeToNearestVideoFrame(splitTime);

	if (splitTime <= [[sel getLeftMostObject] WSObject]->GetAbsoluteStartTime())
		return NO;

	if (splitTime >= [[sel getRightMostObject] WSObject]->GetAbsoluteEndxTime())
		return NO;
	
	return YES;
}

-(ArmeTime)splitTime
{
	return fSplitTime;
}
@end


/****************************************************************************************************

MISplitLowLevelCommand

****************************************************************************************************/


@implementation MISplitLowLevelCommand

-(id)initFromArchive:(ArmeArchiveHandler*)handler
{
	self = [super initFromArchive:handler];
	if ( self )
	{
		fObjectToSplit = [handler->ReadObjCObject() retain];
		handler->ReadObjCObject(); //obsolete from version 0x00010017
		handler->ReadObjCObject(); //obsolete from version 0x00010017
		fSplitTime = handler->ReadArmeTime();
		
		if (handler->GetFileVer() >= 0x00010017)
		{
			fDeleteObject = [handler->ReadObjCObject() retain];
			fPasteRightPart = [handler->ReadObjCObject() retain];
			fPasteLeftPart = [handler->ReadObjCObject() retain];
		}
		else
		{
			fDeleteObject = nil;
			fPasteRightPart = nil;
			fPasteLeftPart = nil;
		}
	}
	return self;
}

-(BOOL)writeToArchive:(ArmeArchiveHandler*)handler
{
	[super writeToArchive:handler];

	handler->WriteObjCObject(fObjectToSplit);
	handler->WriteObjCObject(nil); 	//obsolete from version 0x00010017
	handler->WriteObjCObject(nil); //obsolete from version 0x00010017
	handler->WriteArmeTime(fSplitTime);
	
	//from version 0x00010017
	handler->WriteObjCObject(fDeleteObject);
	handler->WriteObjCObject(fPasteRightPart);
	handler->WriteObjCObject(fPasteLeftPart);
	
	return YES;
}

- (void)dealloc
{
	[fObjectToSplit release];
	[fPasteRightPart release];
	[fPasteLeftPart release];
	[fDeleteObject release];		
	
	[super dealloc];
}


- (id)initWithObject:(MIObject*)object atTime:(ArmeTime)splitTime
{
	self = [super initWithSelection:nil];
	
	if ( self )
	{
		fObjectToSplit = object;
		[fObjectToSplit retain];
		fSplitTime = splitTime;
		fDeleteObject = nil;
		fPasteRightPart = nil;
		fPasteLeftPart = nil;
	}

	return self;
}

- (MICmdStatus)doIt
{
	if(fSplitTime == _ArmeTime(-1))
		return kMICmdStatusCancelled;

	//if this is actually a redo, release the old commands
	if (fPasteLeftPart)
	{
		[fPasteRightPart release];
		[fPasteLeftPart release];
		[fDeleteObject release];		
	}

	// for the paste command, we need to put the object to split in an array
	NSMutableArray* objectToSplitInArray = [NSMutableArray arrayWithCapacity:5];
	[objectToSplitInArray addObject:fObjectToSplit];

	[MICommand getCurrentProgressDialog]->SetNbSubTasks(5);

	//copy the left part of the object (copy the complete object and then delete the right part) 
	fPasteLeftPart = [[MIPasteLowLevelCommand alloc] initWithDocument:[fObjectToSplit montageDoc] objectsToPaste:objectToSplitInArray inObject:[fObjectToSplit parentObject] atTime:[fObjectToSplit WSObject]->GetPos().fStartTime atVPos:[fObjectToSplit getAbsoluteVerticalPos]];
	[fPasteLeftPart setSuffix:kSuffixSplitLeft]; 
	[MICommand getCurrentProgressDialog]->EnterSubTask();
	[fPasteLeftPart _doIt];
	[MICommand getCurrentProgressDialog]->LeaveSubTask();

	MIObject* leftPart = [fPasteLeftPart->fListOfObjectsPasted objectAtIndex:0];
//	[[fPasteLeftPart->fListOfObjectsPasted objectAtIndex:0] WSObject]->SetEndTrim(ARME_TIME_ZERO);
	if ( [fObjectToSplit isLocked] )
		[leftPart setLocked:YES];
		
/*	MICommand* deleteLeftPart = [[[MIDeleteCommand alloc] initWithObject:leftPart from:fSplitTime to:[leftPart WSObject]->GetAbsoluteEndxTime()] autorelease];
	[MICommand getCurrentProgressDialog]->EnterSubTask();
	[deleteLeftPart _doIt];
	[MICommand getCurrentProgressDialog]->LeaveSubTask();
*/
	MITrimCommand* trimLeftPart = [[[MITrimCommand alloc] initWithObject:leftPart] autorelease];
	ArmeTime endTrim = [leftPart WSObject]->GetAbsoluteEndxTime() - fSplitTime;
	[trimLeftPart setEndTrim:(endTrim + [leftPart WSObject]->GetEndTrim())];
	[trimLeftPart setStartTrim:[leftPart WSObject]->GetStartTrim()];
	[MICommand getCurrentProgressDialog]->EnterSubTask();
	[trimLeftPart _doIt];
	[MICommand getCurrentProgressDialog]->LeaveSubTask();

	//copy the right part of the object (copy the complete object and then delete the left part) 
	fPasteRightPart = [[MIPasteLowLevelCommand alloc] initWithDocument:[fObjectToSplit montageDoc] objectsToPaste:objectToSplitInArray inObject:[fObjectToSplit parentObject] atTime:[fObjectToSplit WSObject]->GetPos().fStartTime atVPos:[fObjectToSplit getAbsoluteVerticalPos]];
	[fPasteRightPart setSuffix:kSuffixSplitRight]; 
	[MICommand getCurrentProgressDialog]->EnterSubTask();
	[fPasteRightPart _doIt];
	[MICommand getCurrentProgressDialog]->LeaveSubTask();
	
	MIObject* rightPart = [fPasteRightPart->fListOfObjectsPasted objectAtIndex:0];
//	[rightPart WSObject]->SetStartTrim(ARME_TIME_ZERO);
	if ( [fObjectToSplit isLocked] )
		[rightPart setLocked:YES];

/*	MICommand* deleteRightPart = [[[MIDeleteCommand alloc] initWithObject:rightPart from:[rightPart WSObject]->GetAbsoluteStartTime() to:fSplitTime] autorelease];
	[MICommand getCurrentProgressDialog]->EnterSubTask();
	[deleteRightPart _doIt];
	[MICommand getCurrentProgressDialog]->LeaveSubTask();
*/
	MITrimCommand* trimRightPart = [[[MITrimCommand alloc] initWithObject:rightPart] autorelease];
	ArmeTime startTrim = fSplitTime - [leftPart WSObject]->GetAbsoluteStartTime();
	[trimRightPart setStartTrim:(startTrim + [rightPart WSObject]->GetStartTrim())];
	[trimRightPart setEndTrim:[rightPart WSObject]->GetEndTrim()];
	[MICommand getCurrentProgressDialog]->EnterSubTask();
	[trimRightPart _doIt];
	[MICommand getCurrentProgressDialog]->LeaveSubTask();

	[rightPart moveToObject:[rightPart parentObject] atTime:(fSplitTime - [[rightPart parentObject] WSObject]->GetAbsoluteStartTime()) withVPos:[rightPart getVerticalPos]];

	//delete the object that has been split
	fDeleteObject = [[MIDeleteLowLevelCommand alloc] initWithObjectToDelete:fObjectToSplit from:ARME_TIME_UNDEF to:ARME_TIME_UNDEF];
	[MICommand getCurrentProgressDialog]->EnterSubTask();
	[MICommand getCurrentProgressDialog]->Update(1.0);
	[fDeleteObject _doIt];
	[MICommand getCurrentProgressDialog]->LeaveSubTask();

	// $$$HACK $$$STEPHAN
	// UI code has nothing to do with here
	[[NSApp mainViewController] addObjectToSelection:[rightPart findRepresentationInWindow:[[NSApp mainViewController] window]]];
	[[NSApp mainViewController] addObjectToSelection:[leftPart findRepresentationInWindow:[[NSApp mainViewController] window]]];

	return kMICmdStatusDone;
}

- (void)undoIt
{
	[fDeleteObject _undoIt];
	[fPasteLeftPart _undoIt];
	[fPasteRightPart _undoIt];
}

- (void)redoIt
{
	[fPasteRightPart _redoIt];
	[fPasteLeftPart _redoIt];
	[fDeleteObject _redoIt];

//	[self doIt];
}

- (MIObject*)getLeftPart
{
	return [fPasteLeftPart->fListOfObjectsPasted objectAtIndex:0];
}

- (MIObject*)getRightPart
{
	return [fPasteRightPart->fListOfObjectsPasted objectAtIndex:0];
}

@end
