/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
#import "MISwapChannelsCommand.h"
#import "MIAudioProcessingCommand.h"

#ifndef OfflineAudioProcessorSwapChannels_h
#include "OfflineAudioProcessorSwapChannels.h"
#endif //OfflineAudioProcessorSwapChannels_h

#import "MIObject.h"

#ifndef MEAudioClip_h
#include "MEAudioClip.h"
#endif //MEAudioClip_h

/****************************************************************************************************

MISwapChannelsCommand

****************************************************************************************************/

@implementation MISwapChannelsCommand


- (NSString*)commandName
{
	return @"Swap Channels";
}

-(id)initFromArchive:(ArmeArchiveHandler*)handler
{
	self = [super initFromArchive:handler];
	return self;
}

-(BOOL)writeToArchive:(ArmeArchiveHandler*)handler
{
	return [super writeToArchive:handler];
}

- (id)initWithSelection:(MIWSSelection*)sel
{
	self = [super initWithSelection:sel];
	if ( self )
	{
		[self addCommand:[[[MIAudioProcessingCommand alloc] initWithSelection:fSelection
			offlineAudioProcessor:OfflineAudioProcessorSwapChannels::GetInstance()] autorelease]];
	}
	return self;
}

- (BOOL)displaySetupDialog
{
	if ([self CheckFrozenObjectInSelectionAndAskIfUserWantsToContinue:fSelection] == NO)
		return NO;
	else
		return YES;
}

+ (BOOL)supportsSelection:(MIWSSelection*)sel
{
	if( [sel duration] == _ArmeTime(0))
		return NO;
	if( [sel count] == 0)
		return NO;
		
	if ( [sel type] == kMIWSSelectionTypeObjects || [sel type] == kMIWSSelectionTypeTimeSlice )
	{
		NSEnumerator* selEnum = [sel objectEnumerator];
		MIObject* obj;
		while (obj = [[selEnum nextObject] representedObject])
		{
			//note it works for textClips if they are completely selected
			if (([obj WSObject]->GetType() != TArmeWSObj::kAudioClip))
				return NO;
			
			MEAudioClip* audioClip = dynamic_cast<MEAudioClip*>([obj WSObject]);
			ZzAssert(audioClip);
			if(audioClip->GetNbChannels() != 2)
				return NO;
		}
		return YES; // only if all objects are stereo audio clips
	}
	else
		return NO;
}


@end