/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MITimeStretchCommand.h"
#import "MIAudioProcessingCommand.h"
#import "MIObject.h"
#import "MIObjectRep.h"
#import "MIOfflineAudioTimeStretchController.h"
#import "MIResizeCommand.h"

#ifndef OfflineAudioTimeStretch_h
#include "OfflineAudioTimeStretch.h"
#endif //OfflineAudioTimeStretch_h

#ifndef TVideoClip_h
#include "TVideoClip.h"
#endif //TVideoClip_h

#import "MIAudioProcessingCommand.h"

/****************************************************************************************************

MITimeStretchCommand

****************************************************************************************************/


@implementation MITimeStretchCommand

-(void)dealloc
{
	[super dealloc];
}

-(id)initFromArchive:(ArmeArchiveHandler*)handler
{
	self = [super initFromArchive:handler];
	
	fObject = nil;
	
	return self;
}

-(BOOL)writeToArchive:(ArmeArchiveHandler*)handler
{
	return [super writeToArchive:handler];
}

- (NSString*)commandName
{
	return @"Time Stretch";
}

- (id)initWithSingleClipRep:(MIObjectRep*)clipRep newDuration:(ArmeTime)newDuration
{
	self = [super initWithSelection:nil];
	
	if ( self )
	{
		fObject = [clipRep representedObject];
		if ([[clipRep representedObject] WSObject]->GetType() == TArmeWSObj::kAudioClip)
		{
			OfflineAudioProcessorParams* baseParams = OfflineAudioTimeStretch::GetInstance()->CreateDefaultParams();
			OfflineAudioTimeStretchParams* params = dynamic_cast<OfflineAudioTimeStretchParams*>(baseParams);
			ZzAssert(params != NULL);
			
			params->fNewDuration = newDuration;
				
			[self addCommand:[[[MIAudioProcessingCommand alloc] initWithSingleClipRepAndParams:(MIAudioClipRep*)clipRep 
				offlineAudioProcessor:OfflineAudioTimeStretch::GetInstance() params:params] autorelease]];
				
			[self modifiedAudioHierarchy];
		}
		else if (([[clipRep representedObject] WSObject]->GetType() == TArmeWSObj::kVideoClip) || ([[clipRep representedObject] WSObject]->GetType() == TArmeWSObj::kTextClip))
		{
			[self addCommand:[[[MITimeStretchVideoCommand alloc] initWithObjectToStretch:[clipRep representedObject] newDuration:newDuration selection:nil] autorelease]];
		}
	}
	return self;
}

- (id)initWithSelection:(MIWSSelection*)sel
{
	self = [super initWithSelection:nil];
	
	[fSelection adjustSelectionTimesToVideo];
	
	if ( self )
	{
		fObject = nil;
		
		NSEnumerator* selEnum = [sel objectEnumerator];
		MIObject* obj;
		while (obj = [[selEnum nextObject] representedObject])
		{
			if ([obj WSObject]->GetType() == TArmeWSObj::kAudioClip)
			{
				[self addCommand:[[[MIAudioProcessingCommand alloc] initWithSelection:sel offlineAudioProcessor:OfflineAudioTimeStretch::GetInstance()] autorelease]];
			}
			else if (([obj WSObject]->GetType() == TArmeWSObj::kVideoClip) || ([obj WSObject]->GetType() == TArmeWSObj::kTextClip))
			{
				[self modifiedVideoHierarchy];
		
				if ([sel type] == kMIWSSelectionTypeTimeSlice)
					[self addCommand:[[[MITimeStretchVideoCommand alloc] initWithObjectToStretch:obj from:[sel startTime] to:[sel endxTime] newEnd:([sel startTime] + [sel startTime]) selection:sel] autorelease]];
				else
					[self addCommand:[[[MITimeStretchVideoCommand alloc] initWithObjectToStretch:obj from:ARME_TIME_UNDEF to:ARME_TIME_UNDEF newEnd:([sel startTime] + [sel startTime]) selection:sel] autorelease]];
			}
		}
	}
	
	return self;
}

- (BOOL)displaySetupDialog
{
	if ([self CheckFrozenObjectAndAskIfUserWantsToContinue:fObject] == NO)
		return NO;
	else
		return YES;
//		return [super displaySetupDialog];
}

+ (BOOL)supportsSelection:(MIWSSelection*)sel
{
	if ( [sel type] == kMIWSSelectionTypeObjects
		 || [sel type] == kMIWSSelectionTypeTimeSlice
		 )
	{
		//Note : right now, TimeStretch will only work for single clip selection
		if ([sel count] != 1)
			return NO;
	
		NSEnumerator* selEnum = [sel objectEnumerator];
		MIObject* obj;
		while (obj = [[selEnum nextObject] representedObject])
		{
			//note it works for textClips if they are completely selected
			if ([obj WSObject]->GetType() == TArmeWSObj::kTextClip)
			{
				if ([sel type] == kMIWSSelectionTypeObjects)
					return YES;
			}
			if (([obj WSObject]->GetType() != TArmeWSObj::kAudioClip) &&
				([obj WSObject]->GetType() != TArmeWSObj::kVideoClip) &&
				([obj WSObject]->GetType() != TArmeWSObj::kTextClip))
				return NO;
		}
		return YES;
	}
	else
		return NO;
}


@end

@implementation MITimeStretchVideoCommand
- (void)dealloc
{
	[fObject release];
	[fMementoBefore release];
	[fMementoAfter release];
	[fEnlargeParentCommand release];
	
	[super dealloc];
} 

- (id)initWithObjectToStretch:(MIObject*)obj newDuration:(ArmeTime)newDuration selection:(MIWSSelection*)sel
{
	self = [super initWithSelection:sel];
	if (self)
	{
		fObject = [obj retain];
		fFrom = [fObject WSObject]->GetAbsoluteStartTime();
		fTo = [fObject WSObject]->GetAbsoluteEndxTime();
		fNewEnd = fFrom + newDuration;
		fMementoBefore = [[obj getMemento] retain];
		fMementoAfter = nil;
		fEnlargeParentCommand = nil;
	}

	[self modifiedVideoHierarchy];
	return self;
}

- (id)initWithObjectToStretch:(MIObject*)obj from:(ArmeTime)from to:(ArmeTime)to newEnd:(ArmeTime)newEnd selection:(MIWSSelection*)sel
{
	self = [super initWithSelection:sel];
	if (self)
	{
		fObject = [obj retain];
		fFrom = from;
		fTo = to;
		fNewEnd = newEnd;
		fMementoBefore = [[obj getMemento] retain];
		fMementoAfter = nil;

		if ((fFrom == ARME_TIME_UNDEF) || (fTo == ARME_TIME_UNDEF))
		{
			fFrom = [fObject WSObject]->GetAbsoluteStartTime();
			fTo = [fObject WSObject]->GetAbsoluteEndxTime();
		}
	}

	[self modifiedVideoHierarchy];
	return self;
}
-(id)initFromArchive:(ArmeArchiveHandler*)handler 
{
	self = [super initFromArchive:handler];
	fObject = [handler->ReadObjCObject() retain];
	fFrom = handler->ReadArmeTime();
	fTo = handler->ReadArmeTime();
	fNewEnd = handler->ReadArmeTime(); 
	fMementoBefore = [handler->ReadObjCObject() retain];
	fEnlargeParentCommand = [handler->ReadObjCObject() retain];

	if (handler->GetFileVer() >= 0x0001002C)
	{
		fMementoAfter = [handler->ReadObjCObject() retain];
	}
	else
	{
		fMementoAfter = nil;
	}

	return self;
}
-(BOOL)writeToArchive:(ArmeArchiveHandler*)handler
{
	[super writeToArchive:handler];

	handler->WriteObjCObject(fObject);
	handler->WriteArmeTime(fFrom);
	handler->WriteArmeTime(fTo);
	handler->WriteArmeTime(fNewEnd);
	handler->WriteObjCObject(fMementoBefore);
	handler->WriteObjCObject(fEnlargeParentCommand);
	
	//from version 0x0001002C
	handler->WriteObjCObject(fMementoAfter);
	return YES;
}
- (NSString*)commandName
{
	return @"TimeStretchVideo";
}
- (MICmdStatus)doIt
{
	[fObject invalidateRep];

	TVideoObject* video = (TVideoClip*)[fObject WSObject];
	video->TimeStretch(fFrom, fTo, fNewEnd);

	[fObject invalidateRep];
	[fObject invalidateRepresentationsCachedImages];
	[fObject setRepresentationsNeedDisplay];
	[fObject markModified];

	MIObject *parent = [fObject parentObject];
	
	if ( parent && !MIContainsRect( [parent modelFrame], [parent minimalRectToFitContent] ) )
	{
		fEnlargeParentCommand = [[MIResizeCommand alloc] initWithSelection:nil];
		[fEnlargeParentCommand setObject:parent newFrame:MIUnionRect([parent modelFrame], [parent minimalRectToFitContent])];
		[fEnlargeParentCommand _doIt];
	}

	return kMICmdStatusDone;
}

- (void)undoIt
{
	[fObject invalidateRep];
	if (!fMementoAfter)
		fMementoAfter = [[fObject getMemento] retain];
	[fObject setMemento:fMementoBefore];
	
	[fObject invalidateRep];
	[fObject invalidateRepresentationsCachedImages];
	[fObject setRepresentationsNeedDisplay];
	
	[fEnlargeParentCommand _undoIt];
}

- (void)redoIt
{
	[fObject invalidateRep];

/*	TVideoObject* video = (TVideoObject*)[fObject WSObject];
	video->TimeStretch(fFrom, fTo, fNewEnd);*/
	[fObject setMemento:fMementoAfter];

	[fObject invalidateRep];
	[fObject invalidateRepresentationsCachedImages];
	[fObject setRepresentationsNeedDisplay];

	[fEnlargeParentCommand _redoIt];
}

- (BOOL)displaySetupDialog
{
	NSModalSession session;
	NSWindow* 	win;

//    SInt64 duration, maxDuration=0;
    
	id dlgController = [MIOfflineAudioTimeStretchController alloc];
	[dlgController initWithWindowNibName:@"MIOfflineAudioTimeStretch" owner:dlgController];
	win = [dlgController window];

	if ([fSelection type] == kMIWSSelectionTypeObjects)
		[dlgController setOrigDuration:[fObject WSObject]->GetDuration()];
	else
		[dlgController setOrigDuration:[fSelection duration]];
	
    [dlgController setAlgoType:0];
	
	session = [NSApp beginModalSessionForWindow:win];
	while([NSApp runModalSession:session] == NSRunContinuesResponse)
	{
	}
	[NSApp endModalSession:session];					
	[win orderOut:0];

	if ([fSelection type] == kMIWSSelectionTypeObjects)
		fNewEnd = [fObject WSObject]->GetAbsoluteStartTime() + [dlgController getDuration];
	else
		fNewEnd = fFrom + [dlgController getDuration];
	bool wasCancelled = [dlgController wasCancelled];
	[dlgController release];
	if(wasCancelled)
		return NO;
	return YES;
}
@end
