/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MITrimCommand.h"
#import "MIObject.h"
#import "MIObjectRep.h"
#import "MITrimDlgCtrller.h"
#import "MIMoveCommand.h"

#ifndef TArmeWSObj_h
#include "TArmeWSObj.h"
#endif //TArmeWSObj_h

#undef VER_DEF_TOPIC
#define VER_DEF_TOPIC MITrimTracker

@implementation MITrimCommand

- (void)dealloc
{
	[fModifiedObject release];
	[fMoveCommand release];
	
	[super dealloc];
}

-(id)initFromArchive:(ArmeArchiveHandler*)handler
{
	self = [super initFromArchive:handler];
	if(self)
	{
		fOriginalStartTrim = handler->ReadArmeTime();
		fOriginalEndTrim = handler->ReadArmeTime();
		fNewStartTrim = handler->ReadArmeTime();
		fNewEndTrim = handler->ReadArmeTime();
		fModifiedObject = [handler->ReadObjCObject() retain]; 
		fMoveCommand = [handler->ReadObjCObject() retain];
	}
	return self;
}

-(BOOL)writeToArchive:(ArmeArchiveHandler*)handler
{
	[super writeToArchive:handler];
	handler->WriteArmeTime(fOriginalStartTrim);
	handler->WriteArmeTime(fOriginalEndTrim);
	handler->WriteArmeTime(fNewStartTrim);
	handler->WriteArmeTime(fNewEndTrim);
	handler->WriteObjCObject(fModifiedObject);
	handler->WriteObjCObject(fMoveCommand);
	return YES;
}

- (id)initWithSelection:(MIWSSelection*)sel
{
	vernl(6,"******* initWithSelection ******");
	self = [super initWithSelection:sel];
	
	[fSelection adjustSelectionTimesToVideo];
	
	if(self)
	{
		if([MITrimCommand supportsSelection:fSelection])
		{
			fMoveCommand = nil;
			fModifiedObject = [[[fSelection objectAtIndex:0] representedObject] retain];
			ZzAssert(fModifiedObject);
			fOriginalStartTrim = [fModifiedObject WSObject]->GetStartTrim();
			vvernl(6,fOriginalStartTrim);
			fOriginalEndTrim = [fModifiedObject WSObject]->GetEndTrim();
			vvernl(6,fOriginalEndTrim);
			fNewStartTrim = _ArmeTime(-1);
			fNewEndTrim = _ArmeTime(-1);
					
			if ([fSelection containsVideo:true andTransparentTextClip:YES])
				[self modifiedVideoHierarchy];
			if ([fSelection containsAudio])
				[self modifiedAudioHierarchy];
			[self overviewNeedsUpdate];
		}
		else
		{
			fMoveCommand = nil;
			fModifiedObject = nil;
			fOriginalStartTrim = _ArmeTime(-1);
			fOriginalEndTrim = _ArmeTime(-1);
			fNewStartTrim = _ArmeTime(-1);
			fNewEndTrim = _ArmeTime(-1);
		}
	}
	return self;
}

- (id)initWithObject:(MIObject*)obj
{
	self = [super initWithSelection:nil];
	if ( self )
	{
		fMoveCommand = nil;
		fModifiedObject = nil;
		fOriginalStartTrim = _ArmeTime(-1);
		fOriginalEndTrim = _ArmeTime(-1);
		fNewStartTrim = _ArmeTime(-1);
		fNewEndTrim = _ArmeTime(-1);
		[self setObject:obj];
	}
	return self;
}

- (void)setObject:(MIObject*)obj
{
	if ( !obj )
		return;

	fModifiedObject = [obj retain];
	fOriginalStartTrim = [fModifiedObject WSObject]->GetStartTrim();
	vvernl(6,fOriginalStartTrim);
	fOriginalEndTrim = [fModifiedObject WSObject]->GetEndTrim();
	vvernl(6,fOriginalEndTrim);
	fNewStartTrim = _ArmeTime(-1);
	fNewEndTrim = _ArmeTime(-1);
			
	if ([obj WSObject]->ContainsVideo(true, true))
		[self modifiedVideoHierarchy];
	if ([obj WSObject]->ContainsAudio())
		[self modifiedAudioHierarchy];
	[self overviewNeedsUpdate];
}

- (NSString*)commandName
{
	return @"Trim";
}

- (MICmdStatus)doIt
{
	ZzAssert(fModifiedObject);
	[fModifiedObject setRepresentationsNeedDisplay];
	[fModifiedObject WSObject]->SetStartTrim(fNewStartTrim);
	[fModifiedObject WSObject]->SetEndTrim(fNewEndTrim);
	fNewStartTrim = [fModifiedObject WSObject]->GetStartTrim();
	fNewEndTrim = [fModifiedObject WSObject]->GetEndTrim();

	fMoveCommand = [[MIMoveCommand alloc] initWithSelection:nil];
	[fMoveCommand setObject:fModifiedObject];
	[fMoveCommand setOffset:MIMakePoint(_ArmeTime(GET_VAL(fNewStartTrim) - GET_VAL(fOriginalStartTrim)),_MIVPos(0))];
	[fMoveCommand doIt];
	[fModifiedObject invalidateRepresentationsCachedImages];
	[fModifiedObject invalidateRep];
	[fModifiedObject setRepresentationsNeedDisplay];
	
	return kMICmdStatusDone;
}
	
- (void)undoIt
{
	ZzAssert(fModifiedObject);
	[fModifiedObject setRepresentationsNeedDisplay];
	[fModifiedObject WSObject]->SetStartTrim(fOriginalStartTrim);
	[fModifiedObject WSObject]->SetEndTrim(fOriginalEndTrim);
	[fMoveCommand undoIt];  //note, it is mandatory to undo the move after the modification of the trim
	[fModifiedObject invalidateRepresentationsCachedImages];
	[fModifiedObject invalidateRep];
}

- (void)redoIt
{
	ZzAssert(fModifiedObject);
	[fModifiedObject setRepresentationsNeedDisplay];
	[fModifiedObject WSObject]->SetStartTrim(fNewStartTrim);
	[fModifiedObject WSObject]->SetEndTrim(fNewEndTrim);
	[fMoveCommand redoIt];
	[fModifiedObject invalidateRepresentationsCachedImages];
	[fModifiedObject invalidateRep];
	[fModifiedObject setRepresentationsNeedDisplay];
}

+ (BOOL)supportsSelection:(MIWSSelection*)sel
{
	if([sel count] != 1)
		return NO;
	if([sel type] != kMIWSSelectionTypeObjects)
		return NO;
	
	TArmeWSObj* obj = [[[sel objectAtIndex:0] representedObject] WSObject];
	if((obj->GetType() != TArmeWSObj::kAudioClip) && (obj->GetType() != TArmeWSObj::kVideoClip) && (obj->GetType() != TArmeWSObj::kTextClip))
		return NO;
	return YES;
}

- (void)setStartTrim:(ArmeTime)startTrim
{
	fNewStartTrim = startTrim;
}

- (void)setEndTrim:(ArmeTime)endTrim
{
	fNewEndTrim = endTrim;
}

- (BOOL)displaySetupDialog
{
	if ([self CheckFrozenObjectAndAskIfUserWantsToContinue:fModifiedObject] == NO)
		return NO;
	else
		return YES;
		
	//all the code below is to be used with the trim dialog (not available right now)
	NSModalSession session;
	NSWindow* 	win;
	if(![MITrimCommand supportsSelection:fSelection])
	{
		NSRunAlertPanel(@"Error", @"This command does not support the supplied selection", @"OK", nil, nil);
		return NO;
	}

	ZzAssert(fNewStartTrim == _ArmeTime(-1) && fNewEndTrim == _ArmeTime(-1));
	MITrimDlgCtrller* ctrller =
		[MITrimDlgCtrller alloc];
	[ctrller initWithWindowNibName:@"MITrimDlg" owner:ctrller];
	win = [ctrller window];
	[ctrller setMontageDocument:[fModifiedObject montageDoc]];
	[ctrller setStartTrim:fOriginalStartTrim];
	[ctrller setEndTrim:fOriginalEndTrim];
	// the ctrller needs to know the max duration so that it doesn't try to trim past the start or past the end
	[ctrller setTotalObjectDuration:([fModifiedObject WSObject]->GetDuration() + fOriginalStartTrim + fOriginalEndTrim)];
	
	session = [NSApp beginModalSessionForWindow:win];
	while([NSApp runModalSession:session] == NSRunContinuesResponse)
	{
	}
	[NSApp endModalSession:session];					
	fNewStartTrim = [ctrller startTrim];
	fNewEndTrim = [ctrller endTrim];
	[ctrller release];
	// new trim set to -1 means command cancelled
	bool b1 = (GET_VAL(fNewStartTrim) == -1);
	bool b2 = (GET_VAL(fNewEndTrim) == -1);
	bool b3;
	[win orderOut:0];
	b3=(b1||b2);
	if(b3)
		return NO;
	else
	{
		if (fNewStartTrim < ARME_TIME_ZERO)
			fNewStartTrim = ARME_TIME_ZERO;
		if (fNewEndTrim < ARME_TIME_ZERO)
			fNewEndTrim = ARME_TIME_ZERO;
		TArmeWSObj* obj = [fModifiedObject WSObject];
		//ensure that the clip is at least 100 frames long
		if (fNewStartTrim > (obj->GetDuration() - fNewEndTrim - _ArmeTime(100)))
			fNewStartTrim = (obj->GetDuration() - fNewEndTrim - _ArmeTime(100));
		if (fNewEndTrim > (obj->GetDuration() - fNewStartTrim - _ArmeTime(100)))
			fNewEndTrim = (obj->GetDuration() - fNewStartTrim - _ArmeTime(100));
		return YES;
	}
}

@end
