/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MIUnfreezeCommand.h"
#import "MIObject.h"
#import "MIMainViewController.h"
#import "MIObjectRep.h"

#ifndef ArmeArchiveHandler_h
#include "ArmeArchiveHandler.h"
#endif //ArmeArchiveHandler_h

/****************************************************************************************************

MIUnfreezeCommand

****************************************************************************************************/

@implementation MIUnfreezeCommand

-(id)initFromArchive:(ArmeArchiveHandler*)handler
{
	self = [super initFromArchive:handler];
	if(self)
	{
		fFrozenObjectsArray = [handler->ReadObjCObject("NSMutableArray") retain];
		fFreezingResultObjectArray = [handler->ReadObjCObject("NSMutableArray") retain];
	}
	return self;
}

-(BOOL)writeToArchive:(ArmeArchiveHandler*)handler
{
	[super writeToArchive:handler];
	handler->WriteObjCObject(fFrozenObjectsArray);
	handler->WriteObjCObject(fFreezingResultObjectArray);
	return YES;
}

- (void)dealloc
{
	[fFrozenObjectsArray release];
	[fFreezingResultObjectArray release];

	// delete the objects, either the frozen ones, or the freezing results
	[super dealloc];
}

- (NSString*)commandName
{
	return @"Restore rendered objects";
}

- (id)initWithSelection:(MIWSSelection*)sel
{
	self = [super initWithSelection:sel];
	
	if ( self )
	{
		[fSelection discardVideoTransitions];

		fFrozenObjectsArray = [[NSMutableArray alloc] init];
		fFreezingResultObjectArray = nil;
		
		NSArray* repArray = [fSelection putRepsInArrayAndRemoveDescendants];
		NSEnumerator* objRepEnum = [repArray objectEnumerator];
		MIObjectRep* rep;
		while(rep = [objRepEnum nextObject])
		{
			[fFrozenObjectsArray addObject:[rep representedObject]];
		}
		
		if ([fSelection containsVideo:true andTransparentTextClip:NO])
			[self modifiedVideoHierarchy];
		if ([fSelection containsAudio])
			[self modifiedAudioHierarchy];
		
	}
	return self;
}

- (BOOL)displaySetupDialog
{
	return YES;
}

- (MICmdStatus)doIt
{
	NSEnumerator*	objEnum = [fFrozenObjectsArray objectEnumerator];
	MIObject*		object;

	while ( object = [objEnum nextObject] )
	{
	ZzAssert(0);
		// do the actual rendering here
		// add the newly created objects in fFreezingResultObjectArray
		// make sure the fOriginalFrozenObject member of the NEW object
		// is set to the old container/clip (it's useless on clips ,btw)
	}
	return kMICmdStatusDone;
}

- (void)undoIt
{
	NSEnumerator*	objEnum = [fFreezingResultObjectArray objectEnumerator];
	MIObject*	object;

	while ( object = [objEnum nextObject] )
	{
		ZzAssert(0);
	#if 0 //pseudo coe
		object->PutOuttaHierarchy()
		object->fOriginalFrozenObject->PutBackInHiearchy()
	#endif 
	}
}

- (void)redoIt
{
	NSEnumerator*	objEnum = [fFreezingResultObjectArray objectEnumerator];
	MIObject*	object;

	while ( object = [objEnum nextObject] )
	{
		ZzAssert(0);
	#if 0 //pseudo coe
		object->fOriginalFrozenObject->PutOuttaHierarchy()
		object->PutBackInHiearchy()
	#endif 
	}
}

+ (BOOL)supportsSelection:(MIWSSelection*)sel
{
	if ( [sel containsOnlyVideoTransitions])
		return NO;

	if ( [sel type] != kMIWSSelectionTypeObjects )
		return NO;
	else
		return YES;
}


@end

