/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MIUpdateVideoTransitionsCommand.h"
#import "MIDeleteCommand.h"
#import "MIMontageDocument.h"
#import "MIObject.h"

#ifndef TArmeDoc_h
#include "TArmeDoc.h"
#endif //TArmeDoc_h

#ifndef TVideoEffect_h
#include "TVideoEffect.h"
#endif //TVideoEffect_h

#undef VER_DEF_TOPIC
#define VER_DEF_TOPIC videoAlain

static int CompareWSObjVerticalPos(const void* obj1, const void* obj2)
{
	MIObject* miobj1 = (MIObject*)(*((void**)obj1));
	MIObject* miobj2 = (MIObject*)(*((void**)obj2));
	SInt32 vpos1 = GET_VAL([miobj1 getVerticalPos]);
	SInt32 vpos2 = GET_VAL([miobj2 getVerticalPos]);
	if (vpos1 > vpos2)
		return 1;
	else if (vpos1 < vpos2)
		return -1;
	else
		return 0;
}


void sortChildrenByVPos(MIObject* parent)
{
	ZzArray<MIObject*> objArray([[parent getChildren] count]);

	NSEnumerator* childEnum = [[parent getChildren] objectEnumerator];
	MIObject*	child;
	
	while (child = [childEnum nextObject])
	{
		objArray.AddItem(child);
		
		//do the same for all children
		sortChildrenByVPos(child);
	}
	
	// then, let's sort this array according to hpos
	qsort((void*)(objArray.data),objArray.CountItems(),sizeof(MIObject*), CompareWSObjVerticalPos);

	//now, apply the same ordering to the ArmeWSObj objects
	ZzArchPtrArray* children = [parent WSObject]->GetChildrenDangerousFunction();
	ZzAssert(children->CountItems() == objArray.CountItems());
	
	children->RemoveAll();
	
	for(int i = 0; i < objArray.CountItems(); i++)
	{
		children->AddItem([objArray.ItemAt(i) WSObject]);
	}
}

@implementation MIUpdateVideoTransitionsCommand

-(void)dealloc
{
	[fTransitionsCreated release];
	[fTransitionsRemoved release];
    
    [super dealloc];
}


-(id)initFromArchive:(ArmeArchiveHandler*)handler
{
	self = [super initFromArchive:handler];

//	fTransitionsCreated = [handler->ReadObjCObject() retain];
//	fTransitionsRemoved = [handler->ReadObjCObject() retain];
//	fConstantTransition	= handler->ReadBOOL();
	return self;
}

-(BOOL)writeToArchive:(ArmeArchiveHandler*)handler
{
	[super writeToArchive:handler];
	
//	handler->WriteObjCObject(fTransitionsCreated);
//	handler->WriteObjCObject(fTransitionsRemoved);
//	handler->WriteBOOL(fConstantTransition);
	
	return YES;
}

- (id)initWithDocument:(MIMontageDocument*)montageDoc from:(ArmeTime)startTime to:(ArmeTime)endTime
{
	self = [super initWithSelection:nil];
	fTransitionsCreated = [[NSMutableArray alloc] initWithCapacity:1];
	fTransitionsRemoved = [[NSMutableArray alloc] initWithCapacity:1];
	fMontageDoc = montageDoc;
	fConstantTransition = FALSE;
	
	return self;
}

- (MICmdStatus)doIt
{
/*	ZzArray<TVideoXFadeEffect*> transitionsCreated;
	ZzArray<TVideoXFadeEffect*> transitionsToDelete;
	ZzArray<TVideoXFadeEffect*> transitionsToUpdate;
	ZzArray<TVideoXFadeEffect*> transitionsOuttaHierarchy;
			
	//first, sort the children by their vertical pos	
	sortChildrenByVPos([fMontageDoc topLevelContainer]);
	
	// make a list with all the transitions that are currently out of the hierarchy
	NSEnumerator*	objEnum;
	MIObject*		obj;
	objEnum = [[fMontageDoc outtaHierarchyTransitionsPool] objectEnumerator];
	while (obj = [objEnum nextObject])
		transitionsOuttaHierarchy.AddItem((TVideoXFadeEffect*)([obj WSObject]));
	
	//update the Arme hierarchy
	[fMontageDoc armeDocument]->GetTopContainer()->UpdateAffectedHierarchy(fConstantTransition, &transitionsOuttaHierarchy, &transitionsCreated, &transitionsToDelete, &transitionsToUpdate);
	
	//the transitions that were outtaHierarchy and that have been reused to create a new effect must be deleted from that list
	objEnum = [[fMontageDoc outtaHierarchyTransitionsPool] objectEnumerator];
	while (obj = [objEnum nextObject])
	{
		if (transitionsOuttaHierarchy.FindIndexForItem((TVideoXFadeEffect*)([obj WSObject])) == -1)
		{
			//the effect must be deleted
			[fMontageDoc removeFromOuttaHierarchyTransitionsPool:obj];
			[obj release]; //it's not needed anymore
		}
	}*/
		
	[fMontageDoc armeDocument]->fMasterMovie->RebuildMasterMovie();
		
	//setOuttaHierarchy the transitions that are no longer used
	while([fMontageDoc armeDocument]->fTransitionsToRemove->CountItems() > 0)
	{
		MIObject* delObject = [fMontageDoc findObject:[fMontageDoc armeDocument]->fTransitionsToRemove->ItemAt(0)];

		//if not found, the MIObject might have been setOuttaHierarchy in this command
		unsigned int index = 0;
		while (!delObject && (index < [fMontageDoc->fObjectsSetOuttaHierarchyRecently count]))
		{
			delObject = [[fMontageDoc->fObjectsSetOuttaHierarchyRecently objectAtIndex:index++] findObjectRecursively:[fMontageDoc armeDocument]->fTransitionsToRemove->ItemAt(0)];
		}
		[delObject retain]; //if it is not retained, it is dealloc while in the setOuttaHierarchy
		[delObject setOuttaHierarchy];
		[delObject release];
		
//		[fMontageDoc armeDocument]->fOuttaHierarchyTransitionsPool->AddItem([fMontageDoc armeDocument]->fTransitionsToRemove->ItemAt(0));

		// the transition to remove might also be in the transiton created (in case of Macro commands, it might happen).
		// so we need to remove it from the other list
		for (int i=0; i < [fMontageDoc armeDocument]->fNewlyCreatedTransitions->CountItems(); i++) 
		{
			if ([fMontageDoc armeDocument]->fTransitionsToRemove->ItemAt(0) == [fMontageDoc armeDocument]->fNewlyCreatedTransitions->ItemAt(i))
			{
				[fMontageDoc armeDocument]->fNewlyCreatedTransitions->RemoveItemByIndex(i);
				i--;  // all the indexes have been moved left
			}
		}

		[fMontageDoc armeDocument]->fTransitionsToRemove->RemoveItemByIndex(0);
	}	
	
	#ifdef DEBUG
//	printf("MIUpdateVideoTransitionsCommand is about to remove those objects from fMontageDoc->fObjectsSetOuttaHierarchyRecently : \n");
//	if([fMontageDoc->fObjectsSetOuttaHierarchyRecently count] == 0)
//		printf("---- no objects to remove\n");
//	for(int debug_i = 0; debug_i < [fMontageDoc->fObjectsSetOuttaHierarchyRecently count]; debug_i++)
//	{
//		printf("    %d : %p\n",debug_i, (void*)[fMontageDoc->fObjectsSetOuttaHierarchyRecently objectAtIndex:debug_i]);
//	}
	#endif //#ifdef DEBUG
	
	[fMontageDoc->fObjectsSetOuttaHierarchyRecently removeAllObjects];
	
	//create the new transitionsRep
	while ([fMontageDoc armeDocument]->fNewlyCreatedTransitions->CountItems() > 0) 
	{
		MIObject* newObject = [[MIObject alloc] initWithArmeWSObj:[fMontageDoc armeDocument]->fNewlyCreatedTransitions->ItemAt(0)];
		[newObject setMontageDoc:fMontageDoc];

		MIObject* clipTop = [fMontageDoc findObject:[fMontageDoc armeDocument]->fNewlyCreatedTransitions->ItemAt(0)->fClip1];
		MIObject* clipDown = [fMontageDoc findObject:[fMontageDoc armeDocument]->fNewlyCreatedTransitions->ItemAt(0)->fClip2];

		[newObject setVerticalPos:([clipDown getVerticalPos] + [clipDown getHeight])];
		[newObject setHeight:([clipTop getVerticalPos] - [newObject getVerticalPos])];
		[[clipDown parentObject] addChild:newObject];
		[newObject setParentObject:[clipDown parentObject]];
		[newObject createAllRepresentations];
		[newObject setRepresentationsNeedDisplay];

		[fMontageDoc armeDocument]->fNewlyCreatedTransitions->RemoveItemByIndex(0);
	}
	
	//update the transitionsRep that have changed
/*	for (int i=0; i < transitionsToUpdate.CountItems(); ++i)
	{
		// if the object had been put outtaHierarchy before,
		if (transitionsToUpdate.ItemAt(i)->fNeedsToBePutBackInHierarchy)
		{
			transitionsToUpdate.ItemAt(i)->fNeedsToBePutBackInHierarchy = false;
			//find its MIObject in the outtaHierarchyTransitionsPool
			NSEnumerator* objEnum = [[fMontageDoc outtaHierarchyTransitionsPool] objectEnumerator];
			MIObject* obj;
			while (obj = [objEnum nextObject])
			{
				if ([obj WSObject] == transitionsToUpdate.ItemAt(i))
				{
					[obj putBackInHierarchy];
					[fMontageDoc removeFromOuttaHierarchyTransitionsPool:obj];
				}
			}
		}

		MIObject* xFade = [fMontageDoc findObject:transitionsToUpdate.ItemAt(i)];
		TVideoXFadeEffect*	wsxFade = (TVideoXFadeEffect*)([xFade WSObject]);
		
		MIObject* clipDown = [fMontageDoc findObject:transitionsToUpdate.ItemAt(i)->fClip1];
		MIObject* clipTop = [fMontageDoc findObject:transitionsToUpdate.ItemAt(i)->fClip2];
		
		[xFade setRepresentationsNeedDisplay];  //to invalidate old rep

		//change the size of the rep
		MIRect newFrame;
		newFrame.origin.time = [xFade WSObject]->GetPos().fStartTime;
		newFrame.origin.vpos = ([clipDown getVerticalPos] + [clipDown getHeight]);
		newFrame.size.height = ([clipTop getVerticalPos] - newFrame.origin.vpos);
		newFrame.size.duration = [xFade WSObject]->GetDuration();
		[xFade setModelFrame:newFrame];
		
		// draw the new rep
		[xFade setRepresentationsNeedDisplay];
	}
*/
	return kMICmdStatusDone;
}

- (void)undoIt
{
	[self doIt];

	//remove the Transition that have been created by
//	MIDeleteCommand* deleteTransitionsCommand = [[MIDeleteCommand alloc] initWithListOfObjects:fTransitionsCreated];
//	[deleteTransitionsCommand _doIt];
//	[deleteTransitionsCommand release];

/*	NSEnumerator*	objEnum;
	MIObject*		obj;
	
	objEnum = [fTransitionsCreated objectEnumerator];
	while (obj = [objEnum nextObject])
	{
		[obj setOuttaHierarchy];
		[fMontageDoc addToOuttaHierarchyTransitionsPool:obj];
	}*/

/*	objEnum = [fTransitionsRemoved objectEnumerator];
	while (obj = [objEnum nextObject])
	{
		[obj putBackInHierarchy];
		[fMontageDoc removeFromOuttaHierarchyTransitionsPool:obj];
	}*/

	//nothing else is done here
}

- (void)redoIt
{
	[self doIt];
}

@end
