/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MIAudioClipTrackers.h"
#import "MIObjectRep.h"
#import "MITopLevelContainerRep.h"
#import "MIMainViewController.h"

@implementation MIAudioClipFrameSelectorTracker

- (id)initWithObjectRep:(MIObjectRep*)rep inView:(id)inView atPos:(NSPoint)where withModifiers:(unsigned int)modifiers;
{
	fObjRep = rep;
	[super initWithView:inView atPos:where withModifiers:modifiers];
	return self;
}

- (void)start:(NSPoint)where withModifiers:(unsigned int)modifiers
{
	printf("%c\n",7);
	NSPoint localWhere = [fObjRep convertPointFromTLCCoords:where];
	ArmeTime globalTime = [[[fObjRep getTopLevelContainerRep] mainViewController] pixelToArmeTime:where.x];
	
	// destroy global selection
	[[[fInView mainViewController] selection] clear];

	// create a null length selection
	[[[fObjRep getTopLevelContainerRep] mainViewController] selectionSetTimeSliceType];	
	[[[fObjRep getTopLevelContainerRep] mainViewController] addObjectToSelection:fObjRep];
	[[[fObjRep getTopLevelContainerRep] mainViewController] setSelectionStartTime:globalTime];
	[[[fObjRep getTopLevelContainerRep] mainViewController] setSelectionEndxTime:globalTime];
		
	[fInView setNeedsDisplayInRect:[fObjRep frame]];
		
//	printf("MIAudioClipFrameSelectorTracker start where=(%2.2f,%2.2f)\n",localWhere.x,localWhere.y);
}

- (void)stop:(NSPoint)where withModifiers:(unsigned int)modifiers
{
	NSPoint localWhere = [fObjRep convertPointFromTLCCoords:where];
//	printf("MIAudioClipFrameSelectorTracker stop where=(%2.2f,%2.2f)\n",localWhere.x,localWhere.y);
}

- (void)moved:(NSPoint)where withModifiers:(unsigned int)modifiers
{
	NSPoint localWhere = [fObjRep convertPointFromTLCCoords:where];
	ArmeTime globalEndTime = [[fInView mainViewController] pixelToArmeTime:where.x];
	[[fInView mainViewController] setSelectionEndxTime:globalEndTime];
	[fInView setNeedsDisplayInRect:[fObjRep getTLCFrame]];

//	printf("MIAudioClipFrameSelectorTracker moved where=(%2.2f,%2.2f)\n",localWhere.x,localWhere.y);
}

- (void)draw:(NSRect)rect
{
//	printf("MIAudioClipFrameSelectorTracker draw()\n");
}

@end
