/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MIAutomationTrackers.h"

#import "MIObjectRep.h"
#import "MIObject.h"
#import "TArmeWSObj.h"
#import "MITopLevelContainerRep.h"
#import "MIApplicationController.h"
#import "MITimePaletteController.h"

const float kAutomationKnotHalfSize = 3.0f;

@implementation MIMoveAutomationKnotTracker

- (void)setSpline:(cSpline*)spline knot:(cSplineKnot*)knot obj:(MIObjectRep*)obj boundRect:(NSRect)rect
{
	fTheSpline = spline;
	fTheKnot = knot;
	fInObj = obj;
	fBoundRect = rect;
	fObjMinX = [[obj representedObject] WSObject]->GetAbsoluteStartTime();
}

- (void)start:(NSPoint)where withModifiers:(unsigned int)modifiers
{
	// should already have the knot and the spline at this point
	
	
//	[[MITimePaletteController sharedTimePaletteController] setFeedbackType:MITimePaletteSelectionFeedback];
//	[[MITimePaletteController sharedTimePaletteController] setFeedbackObject:[[fInView mainViewController] selection]];
//	[[MITimePaletteController sharedTimePaletteController] getValuesFromMainView];
	
	
}

- (void)moved:(NSPoint)where withModifiers:(unsigned int)modifiers
{
	float		newValue;
	ArmeTime	newTime;
	
	
	// check that a knot and spline exist
	if ( !fTheKnot || !fTheSpline )
		return;
	
	// cap at the box
	
	if ( where.x < NSMinX(fBoundRect) )
		where.x = NSMinX(fBoundRect);

	if ( where.x > NSMaxX(fBoundRect) )
		where.x = NSMaxX(fBoundRect);
	
	if ( where.y < NSMinY(fBoundRect) )
		where.y = NSMinY(fBoundRect);

	if ( where.y > NSMaxY(fBoundRect) )
		where.y = NSMaxY(fBoundRect);
	
	newValue = ((where.y - NSMinY(fBoundRect))/NSHeight(fBoundRect));
	newTime = [fInView pixelToArmeTime:(where.x - NSMinX(fBoundRect))];
	
	fTheSpline->changeKnotValues( fTheKnot, newTime, newValue );
	
	[fInObj setNeedsDisplay];
	[[[MIApplicationController sharedApplicationController] timePaletteController] getValuesFromMainView];
}
	
-(void)stop:(NSPoint)where withModifiers:(unsigned int)modifiers
{
	[fInObj refreshCachedAutomationImage:NO];
}
	
-(void)draw:(NSRect)rect
{
	ArmeTime	t;
	ArmeTime	inc = [fInView pixelToArmeTime:1];
	ArmeTime	start = [fInView pixelToArmeTime:NSMinX(fBoundRect)];
	ArmeTime	stop = [fInView pixelToArmeTime:NSMaxX(fBoundRect)];
	float		lastValue, newValue, x;
	float		baseY;
	float		baseX = NSMinX(fBoundRect);
	bool		isRecorded;
	float		splineHeight = NSHeight(fBoundRect);
	NSRect		knotRect;
	
	stop -= start;
	GET_VALUE(start) = 0;
	
	[[NSColor blueColor] set];
	[NSBezierPath fillRect:fBoundRect];

	[[NSColor yellowColor] set];
		
	// draw the spline
	x = 0;
	lastValue = fTheSpline->splineValue(start, isRecorded) * splineHeight;
	baseY = NSMinY(fBoundRect);
	for ( t = start; t <= stop; t += inc, x++ )
	{
		newValue = fTheSpline->splineValue(t, isRecorded) * splineHeight;
		
		[NSBezierPath strokeLineFromPoint:NSMakePoint(baseX+x-1, baseY+lastValue)
								toPoint:NSMakePoint(baseX+x, baseY+newValue)];
		lastValue = newValue;
	}

	// draw the knots
	[[NSColor whiteColor] set];
	
	cSplineKnot *aKnot = fTheSpline->firstKnot();
	while (aKnot)
	{
		knotRect = NSZeroRect;
		
		knotRect.origin.x = baseX + [fInView armeTimeToPixel:aKnot->time()]; 
		knotRect.origin.y = baseY + (aKnot->value() * splineHeight);
		
		knotRect = NSInsetRect( knotRect, -kAutomationKnotHalfSize, -kAutomationKnotHalfSize);
		
		[NSBezierPath fillRect:knotRect];
		
		aKnot = aKnot->nextKnot();
	}
	
	// draw the moved knot in red:
	[[NSColor redColor] set];
	
	knotRect = NSZeroRect;
	
	knotRect.origin.x = baseX + [fInView armeTimeToPixel:fTheKnot->time()]; 
	knotRect.origin.y = baseY + (fTheKnot->value() * splineHeight);
	
	knotRect = NSInsetRect( knotRect, -kAutomationKnotHalfSize, -kAutomationKnotHalfSize);
	
	[NSBezierPath fillRect:knotRect];
}

- (void)alternateDraw:(NSRect)rect
{
}



@end
