/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MICreateMarkerTracker.h"
#import "MIMainViewController.h"
#import "MIMontageDocument.h"
#import "MIImageController.h"
#import "MIRuler.h"
#import "MITopLevelContainerRep.h"
#import "MICreateDeleteMarkerCommand.h"
#import "MISnapper.h"
#import "MITime.h"

@implementation MICreateMarkerTracker

- (id)initWithView:(id)inView atPos:(NSPoint)where withModifiers:(unsigned int)modifiers
{
	if (( self = [super initWithView:inView atPos:where withModifiers:modifiers] ))
	{
		fMarker = (UInt32) -1;
	}
	return self;
}

- (void)start:(NSPoint)where withModifiers:(unsigned int)modifiers
{
	fMouse = where;
	MIRuler *ruler = fInView ;
	
	if ( NSPointInRect( where, [ruler rulerTopRect] ) )
	{
		fShouldDrawGhost = YES;
	}
	else if ( NSPointInRect( where, [ruler rulerBottomRect] ) )
	{
		fShouldDrawGhost = NO;
		[self createMarkerAtPoint:where withModifiers:modifiers];
	}
}

- (void)moved:(NSPoint)where withModifiers:(unsigned int)modifiers
{
	MIRuler *ruler = fInView ;

	if ( fShouldDrawGhost )
		[self setMarkerNeedsDisplayAtPoint:fMouse updateTLC:NO];
		
	if ( NSPointInRect( where, [ruler rulerTopRect] ) )
	{
		fShouldDrawGhost = YES;
		if ( fMarker != (UInt32) -1 )
		{
			[self removeMarker];
		}
	}
	else if ( NSPointInRect( where, [ruler rulerBottomRect] ) )
	{
		fShouldDrawGhost = NO;
		if ( fMarker == (UInt32) -1 )
			[self createMarkerAtPoint:where withModifiers:modifiers];
		else
			[self moveMarkerToPoint:where withModifiers:modifiers];
	}
	else
		[self mouseOutsideRuler:where withModifiers:modifiers];
		
	fMouse = where;
}

- (void)mouseOutsideRuler:(NSPoint)mouse withModifiers:(unsigned int)modifiers
{
	fShouldDrawGhost = NO;
	if ( fMarker != (UInt32) -1 )
	{
		[self removeMarker];
	}
}

- (void)stop:(NSPoint)where withModifiers:(unsigned int)modifiers
{
	MIRuler *ruler = fInView ;

	if ( fShouldDrawGhost )
		[self setMarkerNeedsDisplayAtPoint:fMouse updateTLC:NO];

	fMouse = where;
	fShouldDrawGhost = NO;
	
	if ( fMarker != (UInt32) -1 )
	{
		[self removeMarker];
	}
	
	if ( NSPointInRect( where, [ruler rulerBottomRect] ) )
	{
		MIMainViewController *controller = [[fInView window] windowController];
		MIMontageDocument *document = [controller document];

		MICreateDeleteMarkerCommand* command = [[MICreateDeleteMarkerCommand alloc] initWithDocument:document andArmeTime:[controller pixelToArmeTime:fMouse.x]];
		VUSERCMD1("Create Marker at time = %d", (int)([controller pixelToArmeTime:fMouse.x].data));
		[document doCmd:command];
		[command release];
	}
}


- (void)draw:(NSRect)rect
{
	if ( fShouldDrawGhost )
	{
		NSImage *img = mainImageController->fRulerGenericLocator;
		NSPoint point = NSMakePoint(fMouse.x-([img size].width/2),fMouse.y-([img size].height/2));
		[img compositeToPoint:point operation:NSCompositeSourceOver fraction:0.5];
	}
	else
	{
		float		markerPos = fMouse.x;
		MITime		*theTime = [MITime timeWithDocument:[[[fInView getTopLevelContainerView] mainViewController] document]];
		NSString	*theString;
		NSSize		stringSize;
		NSRect		tmpRect;

		[theTime setArmeTime:[[fInView getTopLevelContainerView] pixelToArmeTime:markerPos]];
		theString = [theTime getSMPTEString];
		stringSize = [theString sizeWithAttributes:nil];

		tmpRect = NSMakeRect(markerPos+10,(22-(stringSize.height/2)), 10+stringSize.width, stringSize.height);

//		[[NSColor yellowColor] set];
		[[NSColor cyanColor] set];
		[NSBezierPath fillRect:tmpRect];

		[[NSColor blackColor] set];
		[NSBezierPath strokeRect:tmpRect];
		tmpRect.origin.x += 5;
		[theString drawAtPoint:tmpRect.origin withAttributes:nil];
	}
}

- (void)setMarkerNeedsDisplay:(UInt32)marker
{
	MIMainViewController *controller = [[fInView window] windowController];
    MIMontageDocument *document = [controller document];
	NSSize size = [mainImageController->fRulerGenericLocator size];
	float pixels = [controller armeTimeToPixel:[document getMarker:marker]];
	[self setMarkerNeedsDisplayAtPoint:NSMakePoint( pixels, - 2 + size.height / 2 ) updateTLC:YES];
}

- (void)setMarkerNeedsDisplayAtPoint:(NSPoint)point updateTLC:(BOOL)flag
{
	NSSize size = [mainImageController->fRulerGenericLocator size];
	NSRect rect = NSMakeRect( point.x - size.width / 2, point.y  - size.height / 2, size.width, size.height );
	[fInView setNeedsDisplayInRect:rect];
	id view = [fInView getTopLevelContainerView];

	NSRect visibleRect = [view visibleRect];

	if ( flag )
	{
		[view setNeedsDisplayInRect:NSMakeRect(point.x - 0.5, NSMinY(visibleRect), 1.0, NSHeight(visibleRect) )];
		float		markerPos = point.x;
		MITime		*theTime = [MITime timeWithDocument:[[[fInView getTopLevelContainerView] mainViewController] document]];
		NSString	*theString;
		NSSize		stringSize;
		NSRect		tmpRect;

		[theTime setArmeTime:[[fInView getTopLevelContainerView] pixelToArmeTime:markerPos]];
		theString = [theTime getSMPTEString];
		stringSize = [theString sizeWithAttributes:nil];

		tmpRect = NSMakeRect(markerPos+10,(22-(stringSize.height/2)), 12+stringSize.width, stringSize.height + 5);
		[fInView setNeedsDisplayInRect:tmpRect];
	}		
}

- (void)createMarkerAtPoint:(NSPoint)point withModifiers:(unsigned int)modifiers
{
    MIMainViewController *controller = [[fInView window] windowController];
    MIMontageDocument *document = [controller document];

    fMarker = [document addMarkerAtTime:[self snappedArmeTimeFromPixels:point.x withModifiers:modifiers]];
	[self setMarkerNeedsDisplay:fMarker];
}

- (void)moveMarkerToPoint:(NSPoint)point withModifiers:(unsigned int)modifiers
{
    MIMainViewController *controller = [[fInView window] windowController];
    MIMontageDocument *document = [controller document];

	[self setMarkerNeedsDisplay:fMarker];
    [document moveMarker:fMarker toTime:[self snappedArmeTimeFromPixels:point.x withModifiers:modifiers]];
	[self setMarkerNeedsDisplay:fMarker];
}

- (void)removeMarker
{
    MIMainViewController *controller = [[fInView window] windowController];
    MIMontageDocument *document = [controller document];

	[self setMarkerNeedsDisplay:fMarker];
    [document removeMarker:fMarker];
	fMarker = (UInt32) -1;
}

- (ArmeTime)snappedArmeTimeFromPixels:(float)pixels withModifiers:(unsigned int)modifiers
{
	float newPos;
	ArmeTime theTime;
	MIMainViewController *controller = [[fInView window] windowController];
	
	float pixDelta;
	newPos = [controller armeTimeToPixel:[[controller snapper]
			snapTime:[controller pixelToArmeTime:pixels] pixDelta:&pixDelta
		modifiers:modifiers avoidLocatorIndex:-4]];

	theTime = [[fInView getTopLevelContainerView] pixelToArmeTime:newPos];

	GET_VALUE(theTime) = (SInt64)(GET_VALUE(theTime));
	
	return theTime;
}

@end
