/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
/*
	each of the following classes is a mouse tracker that relates to the clipSheetWindowView. Each
	mouse tracker override the "start", "moved" and "stop" method that come from the base class (MIMouseTracker)
	some of them override the draw method when something is to be drawn. 
	
	Most of the trackers still have to be implemented because the action they have are not to be reproduced until
	the media engine is there (creating a temporary replacement for commands is not nice a job)
*/

#import "MontageInterface.h"
#import "MIMouseTracker.h"

extern const int kSnappingThreshold;

@class MIObjectRep;
@class MIObject;
@class MIWSSelection;

enum EMIGlobalSelectionMode
{
	kMIGlobalSelectionModeNewSelection,
	kMIGlobalSelectionModeXORSelection,
	kMIGlobalSelectionModeORSelection
};

@interface MISelectionTracker : MIMouseTracker
{
	// on initialization, put the clipRefPtrs of selected clips in an array
	// and compare that one with the list of clips situated in the rectangle
	// when the mouse moves...

	NSRect					fCurrentSelectionRect;
	BOOL					fSelectionToggle;
	MIWSSelection*			fOriginalSelection;
	int						fSelectionMode;
	NSRect vRect, hRect;

}
@end

@interface MIMoveSelectionTracker : MIMouseTracker
{
    NSRect						fOriginalTLCRect;
	MIObjectRep					*fOldParent;
	NSRect						fLastRect;
	NSPoint						fOriginOffset;
	NSImage						*fDragImage;
    MIObjectRep					*fTmpDest;
    MIObjectRep					*fOriginalObjectRep;
	BOOL						fDisplayMoveFeedback;
	NSPoint						fImageOriginOffset;
	NSPoint						fLastPoint;
	BOOL						fHorizontalLock;
	NSTimer						*fTimer;
}

- (MIObject*)getDestinationForLocation:(NSPoint)aPoint;
- (NSImage*)createImageOfSelectionBasedAtPoint:(NSPoint)thePoint;
@end

@interface MIDragThingsFromOutsideTracker: MIMouseTracker
{
	NSPoint						fLastPoint;
	MIObjectRep					*fTmpDest;
}
- (MIObjectRep*)destination;
@end

@interface MIResizeObjectTracker: MIMouseTracker
{
	NSPoint						fLastPos;
	NSRectEdge					fEdge1;
	NSRectEdge					fEdge2;
	NSRect						fRedrawRect;
	MIObjectRep					*fResizedObject;
	NSColor						*fFrameColor;
}

- (void)setEdge:(NSRectEdge)theEdge;
- (MIRect)calculateNewFrame:(BOOL)inTLC modifiers:(unsigned int)modifiers;
- (MIObjectRep*)resizedObject;

@end
