/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
#import "MIMouseTracker.h"
#import "MIMainViewTrackers.h"
#import "MIMontageDocument.h"
#import "MIObject.h"
#import "MIObjectRep.h"
#import "MIMainViewController.h"
#import "MITopLevelContainerRep.h"
#import "MIImageController.h"
#import "MIApplicationController.h"
#import "MITimePaletteController.h"
#import "MIMoveCommand.h"
#import "MIResizeCommand.h"
#import "MIRuler.h"
#import "MIPreferences.h"
#import "MICreateClipFromSelectionCommand.h"
#import "MIApplicationController.h"
#import "MontageVerbose.h"

////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
//////////////////////////// Selection Tracker /////////////////////////
////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////

extern bool gCurrentlyDrawingToImage;

@implementation MISelectionTracker
- (void)start:(NSPoint)where withModifiers:(unsigned int)modifiers
{
	MIMontageDocument	*mDoc;

	mDoc = [[fInView representedObject] montageDoc];
	modifiers &= (NSHelpKeyMask| NSControlKeyMask| NSShiftKeyMask| NSCommandKeyMask| NSAlternateKeyMask);

	fSelectionToggle = NO;
	if ( fInView )
	{
		if ( !modifiers )
		{
			// if the user clicked without holding CMD, the current selection needs
			// to be erased.
			[[fInView mainViewController] clearSelection];
			[[fInView mainViewController] selectionSetObjectsType];
			fSelectionMode = kMIGlobalSelectionModeNewSelection;
		}      
		else
		{
			if(modifiers & NSCommandKeyMask)
				fSelectionMode = kMIGlobalSelectionModeXORSelection;
			else
				fSelectionMode = kMIGlobalSelectionModeORSelection;
		}
	}
	else
	{
		ZzAssert(0);
	}

	// let's save the current selection (which might have been emptied)
	fOriginalSelection = [[MIWSSelection alloc] initWithExistingSelection:[[fInView mainViewController] selection]];


	MIObjectRep		*aRep;
	NSRect			selRect;
	NSEnumerator	*objEnum = [fOriginalSelection objectEnumerator];
	
	while ( aRep = [objEnum nextObject] )
	{
		if ( !NSEqualRects(selRect, NSZeroRect) )
			selRect = NSUnionRect( selRect, [aRep getTLCFrame] );
		else
			selRect = [aRep getTLCFrame];
	}

	[[MITimePaletteController sharedTimePaletteController] setFeedbackType:MITimePaletteSelectionFeedback];
	[[MITimePaletteController sharedTimePaletteController] setFeedbackObject:[[fInView mainViewController] selection]];
	[[MITimePaletteController sharedTimePaletteController] getValuesFromMainView];
	
	
}

- (void)dealloc
{
	[fOriginalSelection release];
    [super dealloc];
}

- (void)moved:(NSPoint)where withModifiers:(unsigned int)modifiers
{
	// do like the finder does : do not care about the 'modifiers'
	// param when moving. we only care about the initial state
	// of 'modifiers' (the one that was given to the start() method)
	// NOTE : this isn't true for a move/copy/alias tracker where
	// the user can change the policy as he wants until the releases
	// the mousebutton

	// look for views in the rectangle defined by the old and new location...
	// origin is lower left corner.
	
	NSRect  				newRect;
	NSEnumerator			*viewsEnumerator;
	MIObject				*anObject;
	MIObjectRep				*anObjectRep;
	BOOL					toggleSelection = NO;
	NSRect					redrawRect;
	NSMutableArray			*newSel = [[NSMutableArray alloc] initWithObjects:nil];
	NSArray					*oldSel;
	
	
	// the whole new selection
	newRect = RectWithPoints(fStartPoint, where);
	
	// the redraw rect is the old + the new since we need
	// to clear the selection where it WAS and draw it where
	// it IS
	redrawRect = NSUnionRect(newRect, fCurrentSelectionRect);
    
//	NSRect vRect, hRect;
	
	// determine the 2 rectangles that need updating.
	// vRect is the vertical rectangle, hRect the horizontal one
	
	// first check that the two rectangles share a corner
	if ( ( NSMinX(newRect) != NSMinX(fCurrentSelectionRect) && NSMaxX(newRect) != NSMaxX(fCurrentSelectionRect) ) ||
		( NSMinY(newRect) != NSMinY(fCurrentSelectionRect) && NSMaxY(newRect) != NSMaxY(fCurrentSelectionRect) ) )
	{
		vRect = newRect;
		hRect = fCurrentSelectionRect;
	}
	else
	{
		if ( NSMinX(newRect) == NSMinX(fCurrentSelectionRect) )
		{
			// the left part does not change, the right part did
			vRect.origin.x = min(NSMaxX(newRect), NSMaxX(fCurrentSelectionRect));
			vRect.size.width = fabs(NSMaxX(newRect) - NSMaxX(fCurrentSelectionRect));
			hRect.size.width = fabs(min(NSMaxX(newRect), NSMaxX(fCurrentSelectionRect)) - NSMinX(newRect));
			hRect.origin.x = NSMinX(newRect);
		}
		else
		{
			// it has been extended or diminished to the left
			vRect.origin.x = min(NSMinX(newRect), NSMinX(fCurrentSelectionRect));
			vRect.size.width = fabs(NSMinX(newRect) - NSMinX(fCurrentSelectionRect));
			hRect.size.width = fabs(NSMaxX(newRect) - max(NSMinX(newRect), NSMinX(fCurrentSelectionRect)));
			hRect.origin.x = max(NSMinX(newRect), NSMinX(fCurrentSelectionRect));
		}
		
		if ( NSMinY(newRect) == NSMinY(fCurrentSelectionRect) )
		{
			// the bottom part does not change, the top part did
			hRect.origin.y = min(NSMaxY(newRect), NSMaxY(fCurrentSelectionRect));
			hRect.size.height = fabs(NSMaxY(newRect) - NSMaxY(fCurrentSelectionRect));
			vRect.size.height = fabs(max(NSMaxY(newRect), NSMaxY(fCurrentSelectionRect)) - NSMinY(newRect));
			vRect.origin.y = NSMinY(newRect);
		}
		else
		{
			// it has been eytended or diminished to the bottom
			hRect.origin.y = min(NSMinY(newRect), NSMinY(fCurrentSelectionRect));
			hRect.size.height = fabs(NSMinY(newRect) - NSMinY(fCurrentSelectionRect));
			vRect.size.height = fabs(NSMaxY(newRect) - min(NSMinY(newRect), NSMinY(fCurrentSelectionRect)));
			vRect.origin.y = min(NSMinY(newRect), NSMinY(fCurrentSelectionRect));
		}
	}
	
/*
	// Draw a rectangle over this all
	[fInView setNeedsDisplayInRect:redrawRect];
*/	
    
	[[[fInView mainViewController] ruler] setNeedsDisplayInRect:NSMakeRect( NSMinX(fCurrentSelectionRect) - 1, 0, 3, 20)];
	[[[fInView mainViewController] ruler] setNeedsDisplayInRect:NSMakeRect( NSMaxX(fCurrentSelectionRect) - 1, 0, 3, 20)];

	fCurrentSelectionRect = newRect;
	verntl(trackers,5,"MISelectionTracker : newRect = " << newRect);

	[[[fInView mainViewController] ruler] setNeedsDisplayInRect:NSMakeRect( NSMinX(fCurrentSelectionRect) - 1, 0, 3, 20)];
	[[[fInView mainViewController] ruler] setNeedsDisplayInRect:NSMakeRect( NSMaxX(fCurrentSelectionRect) - 1, 0, 3, 20)];
	

	[fInView setNeedsDisplayInRect:NSInsetRect(hRect,-2,-2)];
	[fInView displayIfNeeded];
	[fInView setNeedsDisplayInRect:NSInsetRect(vRect,-2,-2)];
	[fInView displayIfNeeded];
	


	// clear the selection and refill it (possibly
	// merging with the original selection)
	// don't clear the selection, diff it only.
//	[[fInView mainViewController] clearSelection];
	
	if ( modifiers & NSCommandKeyMask || modifiers || modifiers & NSShiftKeyMask)
		toggleSelection = YES;

//	viewsEnumerator = [[[fInView representedObject] getChildren] objectEnumerator];
	anObjectRep = [fInView hitTestForMIObject:fStartPoint];
	if ( anObjectRep )
		viewsEnumerator = [[[anObjectRep representedObject] getChildren] objectEnumerator];
	else
		viewsEnumerator = [[[fInView representedObject] getChildren] objectEnumerator];
		
	while ( anObject = [viewsEnumerator nextObject] )
	{
		anObjectRep = [anObject findRepresentationInTLC:[fInView getTopLevelContainerRep]];
	
		BOOL wasOriginallySelected = NO;
		BOOL isInRect = NO;

		if ( [fOriginalSelection containsObject:anObjectRep])
			wasOriginallySelected = YES;
		
		NSRect objFrame = [anObjectRep getTLCFrame];
			
		if ((NSContainsRect(newRect,objFrame) || NSIntersectsRect(newRect, objFrame)) && !NSContainsRect(objFrame, newRect) )
			isInRect = YES;
		
		switch(fSelectionMode)
		{
		case kMIGlobalSelectionModeNewSelection :
			ZzAssert(wasOriginallySelected == NO);
			if(isInRect)
				[newSel addObject:anObjectRep];
//				[[fInView mainViewController] addObjectToSelection:anObjectRep];
			break;
		case kMIGlobalSelectionModeORSelection :
			if(isInRect || wasOriginallySelected)
				[newSel addObject:anObjectRep];
//				[[fInView mainViewController] addObjectToSelection:anObjectRep];
			break;
		case kMIGlobalSelectionModeXORSelection :
			if(isInRect ^ wasOriginallySelected )
				[newSel addObject:anObjectRep];
//				[[fInView mainViewController] addObjectToSelection:anObjectRep];
			break;
		default :
			ZzAssert(0);
			break;
		}
	}
	
	// first remove all the unnecessary objects from the old selection
	//objEnum = [[[fInView mainViewController] selection] objectEnumerator];
	for(int i = 0; i < [[[fInView mainViewController] selection] count]; i++)
	{
		anObjectRep = [[[fInView mainViewController] selection] objectAtIndex:i];
		if ( ![newSel containsObject:anObjectRep] )
			[[[fInView  mainViewController] selection] removeObject:anObjectRep];
	}
	// then add all the other objects
	
	//objEnum = [newSel objectEnumerator];
	oldSel = [[[fInView mainViewController] selection] objArray];
		
	for(int i = 0; i < [newSel count]; i++) 
	{
		anObjectRep = [newSel objectAtIndex:i];
		if ( ![oldSel containsObject:anObjectRep] )
			[[fInView  mainViewController] addObjectToSelection:anObjectRep];
	}
	
	[[[MIApplicationController sharedApplicationController] timePaletteController] getValuesFromMainView];
	[newSel release];
}
	
-(void)stop:(NSPoint)where withModifiers:(unsigned int)modifiers
{
	[fInView setNeedsDisplayInRect:fCurrentSelectionRect];

//    NSRect newRect, redrawRect;

//	[fInView setNeedsDisplayInRect:NSMakeRect(fStartPoint.x, fStartPoint.y, NSWidth(fCurrentSelectionRect), NSHeight(fCurrentSelectionRect))];
//	[fInView setNeedsDisplayInRect:fCurrentSelectionRect];
	
//	newRect = RectWithPoints(fStartPoint, where);
    
//    redrawRect = NSUnionRect(newRect, fCurrentSelectionRect);

//	[fInView setNeedsDisplayInRect:redrawRect];
}
	
-(void)draw:(NSRect)rect
{
//	MITopLevelContainerRep	*tlc = fInView;

	[[[NSColor selectedTextBackgroundColor] colorWithAlphaComponent:0.7] set];
	[NSBezierPath fillRect:fCurrentSelectionRect];
	[[[NSColor whiteColor] colorWithAlphaComponent:0.7] set];
	[NSBezierPath strokeRect:NSInsetRect(fCurrentSelectionRect,1,1)];

/*
	if ( tlc->fDrawRedrawRects )
	{
		[[NSColor blueColor] set];
		[NSBezierPath strokeRect:hRect];
		[[NSColor redColor] set];
		[NSBezierPath strokeRect:vRect];
	}
*/
	
//	[[NSColor redColor] set];
//	[aPath stroke];
}

- (void)alternateDraw:(NSRect)rect
{
	NSImage	*dots = mainImageController->fRulerDottedLine;
	float p1, p2;
	p1 = NSMinX(fCurrentSelectionRect) - [mainImageController->fRulerDottedLine size].width/2;
	p2 = NSMaxX(fCurrentSelectionRect) - [mainImageController->fRulerDottedLine size].width/2;

	[dots compositeToPoint:NSMakePoint(p1, 0) operation:NSCompositeSourceAtop];
	[dots compositeToPoint:NSMakePoint(p2, 0) operation:NSCompositeSourceAtop];
	
}

@end


////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
//////////////////////////// Move Tracker //////////////////////////////
////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////

@implementation MIMoveSelectionTracker : MIMouseTracker

- (void)start:(NSPoint)where withModifiers:(unsigned int)modifiers
{
	MIObjectRep		*theRep;
	NSRect			theRepFrame;
    NSEnumerator	*objEnum = [[fInView mainViewController] selectionObjectEnumerator];
	MIObject		*anObj;
	
	
	fTmpDest = nil;
	fLastRect.origin = where;
	fDisplayMoveFeedback = YES;	
	fOldParent = nil;
	
	// get the representation being moved
	theRep = [fInView hitTestForMIObject:fLastRect.origin];
	fOriginalObjectRep = theRep;
    
	fOriginOffset = [theRep convertPointToTLCCoords:NSZeroPoint];
	theRepFrame = [theRep frame];
    
	// get the old (current) parent frame in TLC coordinate system
	// if the old parent is the TLC, no need for all this and provide feedback
	if ( [[theRep representedObject] parentObject] == [fInView representedObject] )
		fDisplayMoveFeedback = YES;
	else
		fOldParent = [[[theRep representedObject] parentObject] findRepresentationInTLC:fInView];
	
	// get the original rectangle
	fOriginalTLCRect.origin = fOriginOffset;
	fOriginOffset.x -= where.x;
	fOriginOffset.y -= where.y;
	
	fOriginalTLCRect.size = theRepFrame.size;
	fLastRect.size = theRepFrame.size;
    fLastRect.origin.x = where.x + fOriginOffset.x;
    fLastRect.origin.y = where.y + fOriginOffset.y;

	MIRect modelLastRect = [fInView NSRectToMIRect:fLastRect];
	fStartTime = MIMinX(modelLastRect);
	fEndTime = MIMaxX(modelLastRect);

	while ( anObj = [[objEnum nextObject] representedObject] )
	{
		if ( [anObj isLocked] )
			fHorizontalLock = YES;
	}


	[mainImageController->fMoveCloseCursor set];

	[[[MIApplicationController sharedApplicationController] timePaletteController] setFeedbackType:MITimePaletteTrackerFeedback];
	[[[MIApplicationController sharedApplicationController] timePaletteController] setFeedbackObject:self];
	[[[MIApplicationController sharedApplicationController] timePaletteController] getValuesFromMainView];
	
//	theRepFrame.origin = NSZeroPoint;
	// draw the representation and children in the image

	fTimer = [NSTimer scheduledTimerWithTimeInterval:.2 target:self selector:@selector(adjustCursor:) userInfo:nil repeats:YES];
}

- (NSImage*)createImageOfSelectionBasedAtPoint:(NSPoint)thePoint;
{
	NSRect			tlcFrame;
	NSImage			*theImage;
	NSPoint			offset;
	NSRect			allSelRect;
	NSEnumerator	*selObjectsEnum;
	MIObjectRep		*anObjectRep;
	NSRect			anObjectFrame;
	NSRect			maxImgRect;
	
	// first go all over the selection, check all the objects coordinates and determine maxRect from that
	selObjectsEnum = [[fInView mainViewController] selectionObjectEnumerator];

	tlcFrame = [fInView frame];
	allSelRect = NSZeroRect;

	while ( anObjectRep = [selObjectsEnum nextObject] )
	{
		anObjectFrame = [anObjectRep getTLCFrame];

		if ( NSEqualRects( allSelRect, NSZeroRect) )
		{
			allSelRect = anObjectFrame;
		}
		else
		{
			allSelRect = NSUnionRect( allSelRect, anObjectFrame);
		}
	}
	
	// we got the maximum rectangle, get the visible rect of the TLC and allow maximum 3 times that size vertically and horizontally
	maxImgRect = [fInView visibleRect];
	maxImgRect.origin.x -= NSWidth(maxImgRect);
	maxImgRect.origin.y -= NSHeight(maxImgRect);
	maxImgRect.size.width *= 3;
	maxImgRect.size.height *= 3;

	selObjectsEnum = [[fInView mainViewController] selectionObjectEnumerator];

	if ( NSWidth(maxImgRect) > NSWidth(allSelRect) && NSHeight(maxImgRect) > NSHeight(allSelRect) )
	{
		// everything's ok, the image fits, create it and display everything in it
		offset.x = -allSelRect.origin.x;
		offset.y = -allSelRect.origin.y;

		theImage = [[NSImage alloc] initWithSize:allSelRect.size];
		[theImage lockFocus];
		gCurrentlyDrawingToImage = YES;
		while ( anObjectRep = [selObjectsEnum nextObject] )
		{
			NSPoint 	repOrigin;
			NSPoint		nOffset;
			
			repOrigin = [anObjectRep getTLCFrame].origin;
			nOffset = TranslateNSPoint(repOrigin, offset);
			repOrigin = [anObjectRep frame].origin;
			nOffset.x -= repOrigin.x;
			nOffset.y -= repOrigin.y;

			[[anObjectRep representedObject] drawRect:tlcFrame inTLC:fInView withOffset:nOffset];
		}
		gCurrentlyDrawingToImage = NO;
		[theImage unlockFocus];
	}
	else
	{
		// the sum of the rects is larger than the allowed image size, take the max image size with the current point
		// as origin and draw everything from there...
		allSelRect = ClipNSRect( allSelRect, maxImgRect );

		offset.x = -allSelRect.origin.x;
		offset.y = -allSelRect.origin.y;

		theImage = [[NSImage alloc] initWithSize:allSelRect.size];
		[theImage lockFocus];
		gCurrentlyDrawingToImage = YES;
		while ( anObjectRep = [selObjectsEnum nextObject] )
		{
			NSPoint 	repOrigin;
			NSPoint		nOffset;

			repOrigin = [anObjectRep getTLCFrame].origin;
			nOffset = TranslateNSPoint(repOrigin, offset);
			repOrigin = [anObjectRep frame].origin;
			nOffset.x -= repOrigin.x;
			nOffset.y -= repOrigin.y;

			tlcFrame.size = allSelRect.size;

			[[anObjectRep representedObject] drawRect:tlcFrame inTLC:fInView withOffset:nOffset];
		}
		gCurrentlyDrawingToImage = NO;
		[theImage unlockFocus];
	}

	fImageOriginOffset = TranslateNSPoint(offset,fStartPoint);
	fImageOriginOffset.x = - fImageOriginOffset.x;
	fImageOriginOffset.y = - fImageOriginOffset.y;
	
	
	return theImage;

}

- (void)moved:(NSPoint)where withModifiers:(unsigned int)modifiers
{
    NSRect			newRect;
	NSRect			snappedRect;
    NSRect			unionRect;

	
	if ( !fDragImage && !NSEqualPoints(where,fLastRect.origin) )
		fDragImage = [[self createImageOfSelectionBasedAtPoint:where] retain];
	
	if ( [[NSApp currentEvent] modifierFlags] & NSShiftKeyMask )
	{
		if ( fabs( where.x - fStartPoint.x) < fabs(where.y - fStartPoint.y) )
			where.x = fStartPoint.x;
		else
			where.y = fStartPoint.y;
	}
		
	if ( fHorizontalLock )
		where.x = fStartPoint.x;

	fLastPoint = where;

	if ( fDisplayMoveFeedback && fTmpDest )
	{
		// clear last feedback
		[fTmpDest setNeedsDisplay];
	}


	newRect.size = [fDragImage size];

	newRect.origin = TranslateNSPoint(where, fImageOriginOffset);

	// check new destination
	fTmpDest = [[self getDestinationForLocation:where] findRepresentationInTLC:fInView];

	if ( fTmpDest == fInView )
		fTmpDest = nil;


	// display feedback in the boxes
	[[[MIApplicationController sharedApplicationController] timePaletteController] getValuesFromMainView];


	// check the difference between the snapped and unsnapped version of the thing

	snappedRect = [[fInView mainViewController]
					MIRectToNSRect:[[[fInView mainViewController] snapper]
						snapRectByShifting:[[fInView mainViewController] NSRectToMIRect:newRect]
									   modifiers:modifiers]];

	fLastPoint.x += ( NSMinX(snappedRect) - NSMinX(newRect));
	fLastPoint.y += ( NSMinY(snappedRect) - NSMinY(newRect));


	[[[fInView mainViewController] ruler] setNeedsDisplayInRect:NSMakeRect( NSMinX(fLastRect) - 1, 0, 3, 20)];
	[[[fInView mainViewController] ruler] setNeedsDisplayInRect:NSMakeRect( NSMaxX(fLastRect) - 1, 0, 3, 20)];

	// merge the two
	if ( !NSEqualRects(fLastRect,NSZeroRect) )
		unionRect = NSUnionRect(fLastRect, snappedRect);
	else
		unionRect = snappedRect;

	// add a border
	unionRect = NSInsetRect(unionRect, -2, -2);
	[fInView setNeedsDisplayInRect:unionRect];

	fLastRect = snappedRect;
	
	// prevent move beyond origin
	if ( fLastRect.origin.x < 0 )
		fLastRect.origin.x = 0;
	
//	if ( fLastRect.origin.y < 0 )
//		fLastRect.origin.y = 0;
	
	[[[fInView mainViewController] ruler] setNeedsDisplayInRect:NSMakeRect( NSMinX(fLastRect) - 1, 0, 3, 20)];
	[[[fInView mainViewController] ruler] setNeedsDisplayInRect:NSMakeRect( NSMaxX(fLastRect) - 1, 0, 3, 20)];

	// display need feedback
	if ( fDisplayMoveFeedback && fTmpDest )
	{
		[fTmpDest setNeedsDisplay];
	}


	MIRect modelLastRect = [fInView NSRectToMIRect:fLastRect];
	fStartTime = MIMinX(modelLastRect);
	fEndTime = MIMaxX(modelLastRect);

	[[[MIApplicationController sharedApplicationController] timePaletteController] getValuesFromMainView];
}

- (void)adjustCursor:(NSTimer*)timer
{
	unsigned int modifiers = [[NSApp currentEvent] modifierFlags];
	if ( modifiers & NSAlternateKeyMask )
		[mainImageController->fMoveCloseCopyCursor set];
	else [mainImageController->fMoveCloseCursor set];
}

- (void)stop:(NSPoint)where withModifiers:(unsigned int)modifiers
{
	MIObject		*newParentObject = nil;
	NSPoint			newPosInView;
	NSPoint			moveOffset;
	id				cmd;

	[fTimer invalidate];
	fTimer = nil;
	
	if ( !fHasMoved )
	{
		// this is necessary for the snappers not to move the object when just clicking on it should put it on top of the "stack"
		NSEnumerator	*objEnum = [[[fInView mainViewController] selection] objectEnumerator];
		MIObject		*anObj;
		
		while ( anObj = [[objEnum nextObject] representedObject] )
		{
			[[anObj parentObject] moveChildToTopOfStack:anObj];
			[[anObj parentObject] setRepresentationsNeedDisplay];
		}
	}
	else
	{
		// setting the need for redisplay, at the last position, at the new position, at the original position
		newParentObject = [fTmpDest representedObject];
		
		if ( !newParentObject )
		{
			newParentObject = [fInView representedObject];
		}
		
		
		// determine the new origin
	
		newPosInView = fLastRect.origin;
		// determine the offset
		// fOriginOffset is the offset from the original dragged item...
		// what we need to determine now is the global move offset.
	
		moveOffset.x = fLastPoint.x - fStartPoint.x;
		moveOffset.y = fLastPoint.y - fStartPoint.y;
	

// test

		NSRect allSelRect = NSZeroRect;
		NSEnumerator *selObjectsEnum = [[fInView mainViewController] selectionObjectEnumerator];
		MIObjectRep *anObjectRep;
		NSRect anObjectFrame;
		
		while ( anObjectRep = [selObjectsEnum nextObject] )
		{
			anObjectFrame = [anObjectRep getTLCFrame];
	
			if ( NSEqualRects( allSelRect, NSZeroRect) )
			{
				allSelRect = anObjectFrame;
			}
			else
			{
				allSelRect = NSUnionRect( allSelRect, anObjectFrame);
			}
		}
		
		if ( -moveOffset.x > NSMinX(allSelRect) )
			moveOffset.x = -NSMinX(allSelRect);
//		if ( -moveOffset.y > NSMinY(allSelRect) )
//			moveOffset.y = -NSMinY(allSelRect);

// /test

		if ( modifiers & NSAlternateKeyMask ) {
			moveOffset.y = -moveOffset.y;
			cmd = [[[MICreateClipFromSelectionCommand alloc] initWithSelection:[[fInView mainViewController] selection] containerObject:newParentObject andOffset:[fInView NSPointToMIPoint:moveOffset]] autorelease];

			VUSERCMD1("Create Clip From Selection with offset = %d", (int)([fInView NSPointToMIPoint:moveOffset].time.data));
			VUSERCMD_PRINT_SELECTION([[fInView mainViewController] selection]);
			VUSERCMD("-NewParent = ");
			VUSERCMD_PRINT_OBJECT(newParentObject);
		}
		else
		{
			cmd = [[[MIMoveCommand alloc] initWithSelection:[[fInView mainViewController] selection]] autorelease];
			VUSERCMD1("Move (through main view), offset = %d", (int)([fInView NSPointToMIPoint:moveOffset].time.data));
			VUSERCMD_PRINT_SELECTION([[fInView mainViewController] selection]);
			VUSERCMD("-NewParent = ");
			VUSERCMD_PRINT_OBJECT(newParentObject);
			
			[cmd setOffset:[fInView NSPointToMIPoint:moveOffset]];
			[cmd setNewParent:newParentObject];
		}
		
		if ([cmd displaySetupDialog])
			[[newParentObject montageDoc] doCmd:cmd];
	
		[[MIApplicationController sharedApplicationController] modelChangedForDocument:[[fInView mainViewController] document]];
	
		// reset time boxes
		[[[MIApplicationController sharedApplicationController] timePaletteController] getValuesFromMainView];
	}
	
	[mainImageController->fMoveOpenCursor set];
	
	if ( [[MIPreferences sharedPrefs] gridShowMask] & kPrefsGridShowVObjLines ||
		 [[MIPreferences sharedPrefs] gridShowMask] & kPrefsGridShowHObjLines )
		[[MIApplicationController sharedApplicationController] redrawAllMainWindows];
}

- (MIObject*)getDestinationForLocation:(NSPoint)aPoint
{
    NSEnumerator	*selObjectsEnum;
    MIObjectRep		*tmpDest;
    MIObject		*anObject;
    MIObject		*newParentObject;
    MIObject		*aParentObject;
    
	selObjectsEnum = [[fInView mainViewController] selectionObjectEnumerator];
	
	tmpDest = [fInView hitTestForMIObject:aPoint];
		
	if ( tmpDest )
	{
		BOOL bParentChanged = NO;
		// check if the parentView is not the object itself or a descendant 
		// if that's the case, pick the object's parent
		newParentObject = [tmpDest representedObject];		
		
		
		while ( anObject = [[selObjectsEnum nextObject] representedObject] )
		{
			// check for each object if the destination is self or a descdendant
			// also check to see if the object is a container or not
            
			if ( anObject == newParentObject )
			{
				// the destination is self, make it the parent
				newParentObject = [anObject parentObject];
				bParentChanged = YES;
			}
            else if ( ![newParentObject isContainer] )
            {
            	// the destination is not a container
				newParentObject = [newParentObject parentObject];
				bParentChanged = YES;
			} 
			else
			{
				// check if the object is a parent of the destination object
				// in which case, set the newParent to the object's parent 
				BOOL	bFound = NO;
				
				aParentObject = newParentObject;
				while ( (aParentObject = [aParentObject parentObject]) && !bFound )
					if ( aParentObject == anObject )
						bFound = YES;
				
				if ( bFound )
				{
					newParentObject = [anObject parentObject];
					bParentChanged = YES;
				}
			}
			
			// if the parent changed, start again the whole enumeration with the new parent, longer but prevents 'order' problems
			if ( bParentChanged )
			{
				selObjectsEnum = [[fInView mainViewController] selectionObjectEnumerator];
				bParentChanged = NO;
			}
		}
	}
	else
		newParentObject = nil;
    
    return newParentObject;
}

- (void)draw:(NSRect)rect
{
	NSPoint		blitPoint;
	
//	blitPoint = TranslateNSPoint(fLastPos, fOriginOffset);
	blitPoint = fLastRect.origin;
//	blitPoint = TranslateNSPoint(fLastRect.origin, fImageOriginOffset);



	// display move feedback
    if ( fDisplayMoveFeedback && fTmpDest )
    {
        NSRect	tlcFrame;
        [[NSColor colorWithCalibratedRed:1.0f green:0.0f blue:0.0f alpha:1.0f] set];
        
        tlcFrame = [fTmpDest getTLCFrame];
        [NSBezierPath strokeRect:tlcFrame];

    }

	// display the image
	
	if ( blitPoint.x < 0 )
		blitPoint.x = 0;
	
/*	if ( blitPoint.y < 0 )
		blitPoint.y = 0;
*/	
	[fDragImage dissolveToPoint:blitPoint fraction:0.5f];    
//	[fDragImage compositeToPoint:blitPoint operation:NSCompositeCopy];    
}

- (void)alternateDraw:(NSRect)rect
{
	NSImage	*dots = mainImageController->fRulerDottedLine;
	float p1, p2;
	p1 = fLastRect.origin.x - [mainImageController->fRulerDottedLine size].width/2;
	p2 = fLastRect.origin.x + [fDragImage size].width - [mainImageController->fRulerDottedLine size].width/2;

	[dots compositeToPoint:NSMakePoint(p1, 0) operation:NSCompositeSourceAtop];
	[dots compositeToPoint:NSMakePoint(p2, 0) operation:NSCompositeSourceAtop];
	
}

- (void)dealloc
{
	[fDragImage dealloc];
	[super dealloc];
}

@end

@implementation MIDragThingsFromOutsideTracker: MIMouseTracker
- (void)start:(NSPoint)where withModifiers:(unsigned int)modifiers
{
	fTmpDest = nil;
	fLastPoint = where;
	fTmpDest = [fInView hitTestForMIObject:fLastPoint];
}

- (void)moved:(NSPoint)where withModifiers:(unsigned int)modifiers
{
	fLastPoint = where;
	
	if ( fTmpDest )
	{
		[fTmpDest setNeedsDisplay];
	}
	
	fTmpDest = [fInView hitTestForMIObject:fLastPoint];
	
	if ( ![[fTmpDest representedObject] isContainer] )
	{
		// the destination is not a container
		fTmpDest = [[[fTmpDest representedObject] parentObject] findRepresentationInTLC:fInView];
		if ( fTmpDest == fInView )
			fTmpDest = nil;
	}
	
	if ( fTmpDest )
	{
		[fTmpDest setNeedsDisplay];
	}
	
}

- (void)stop:(NSPoint)where withModifiers:(unsigned int)modifiers
{
	if ( fTmpDest )
	{
		[fTmpDest setNeedsDisplay];
	}
}

- (void)draw:(NSRect)rect
{
	if ( fTmpDest )
	{
		NSRect	tlcFrame;
		[[NSColor colorWithCalibratedRed:1.0f green:0.0f blue:0.0f alpha:1.0f] set];
		
		tlcFrame = [fTmpDest getTLCFrame];
		[NSBezierPath strokeRect:tlcFrame];
	}
}

- (MIObjectRep*)destination
{
	return fTmpDest;
}

@end

////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
////////////////////////// Resize Tracker //////////////////////////////
////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////


@implementation MIResizeObjectTracker : MIMouseTracker

- (id)initWithView:(id)inView atPos:(NSPoint)where withModifiers:(unsigned int)modifiers;
{
	self = [super initWithView:inView atPos:where withModifiers:modifiers];
	if (self )
	{
		// assigning to each edge an impossible value so as not to trigger unwanted moves
		fEdge1 = NSMaxYEdge;
		fEdge2 = NSMaxXEdge;
	}
	return self;
}

- (void)setEdge:(NSRectEdge)theEdge
{
    if ( theEdge == NSMaxXEdge ||
         theEdge == NSMinXEdge )
        fEdge1 = theEdge;
    else
        fEdge2 = theEdge;
}

- (void)start:(NSPoint)where withModifiers:(unsigned int)modifiers
{
	fLastPos = where;
	
	fResizedObject = [fInView hitTestForMIObject:fLastPos];
	fFrameColor = [fResizedObject backgroundColor];
	fFrameColor = [NSColor colorWithCalibratedRed:	1-[fFrameColor redComponent]
											green:	1-[fFrameColor greenComponent]
											blue:	1-[fFrameColor blueComponent]
											alpha: 	1];
	fRedrawRect = NSZeroRect;
	[fFrameColor retain];

	[[[MIApplicationController sharedApplicationController] timePaletteController] setFeedbackType:MITimePaletteTrackerFeedback];
	[[[MIApplicationController sharedApplicationController] timePaletteController] setFeedbackObject:self];
	[[[MIApplicationController sharedApplicationController] timePaletteController] getValuesFromMainView];
}

- (void)moved:(NSPoint)where withModifiers:(unsigned int)modifiers
{
    NSRect		oldRect;
	MIRect		newRect;
    
    oldRect = [fInView MIRectToNSRect:[self calculateNewFrame:YES modifiers:modifiers]];
    [fInView setNeedsDisplayInRect:NSInsetRect(oldRect,-2, -2)];
	fLastPos = where;

	[[[fInView mainViewController] ruler] setNeedsDisplayInRect:NSMakeRect( NSMinX(fRedrawRect) - 1, 0, 3, 20)];
	[[[fInView mainViewController] ruler] setNeedsDisplayInRect:NSMakeRect( NSMaxX(fRedrawRect) - 1, 0, 3, 20)];

	newRect = [self calculateNewFrame:YES modifiers:modifiers];

    fRedrawRect = [fInView MIRectToNSRect:newRect];

	fStartTime = MIMinX(newRect);
	fEndTime = MIMaxX(newRect);

	[fInView setNeedsDisplayInRect:NSInsetRect(fRedrawRect,-2,-2)];

	[[[fInView mainViewController] ruler] setNeedsDisplayInRect:NSMakeRect( NSMinX(fRedrawRect) - 1, 0, 3, 20)];
	[[[fInView mainViewController] ruler] setNeedsDisplayInRect:NSMakeRect( NSMaxX(fRedrawRect) - 1, 0, 3, 20)];
	[[[MIApplicationController sharedApplicationController] timePaletteController] getValuesFromMainView];
}

- (void)stop:(NSPoint)where withModifiers:(unsigned int)modifiers
{
	MIResizeCommand	*cmd;
	MIRect 		newFrame;
	fLastPos = where;
	
    newFrame = [self calculateNewFrame:NO modifiers:modifiers];

	newFrame = MIUnionRect(newFrame, [[fResizedObject representedObject] minimalRectToFitContent] );
	
	cmd = [[[MIResizeCommand alloc] initWithSelection:[[fInView mainViewController] selection]] autorelease];
	VUSERCMD("Resize");
	VUSERCMD_PRINT_OBJECT([fResizedObject representedObject]);

	[cmd setObject:[fResizedObject representedObject] newFrame:newFrame];
	[[[fResizedObject representedObject] montageDoc] doCmd:cmd];
/*	
	[[fResizedObject representedObject] preserveChildrenPositionsWithNewFrame:newFrame inTLC:fInView];
	[[fResizedObject representedObject] setModelFrameFromRect:newFrame inTLC:fInView];
	[[fResizedObject representedObject] setRepresentationsNeedDisplay];

	[[[fResizedObject representedObject] parentObject] resizeFrameToFullyIncludeChildren];
	[[[fResizedObject representedObject] parentObject] resizeFrameToFullyIncludeChild:[fResizedObject representedObject]];
*/
//    [[NSCursor arrowCursor] set];

	[[MIApplicationController sharedApplicationController] modelChangedForDocument:[[fInView mainViewController] document]];
	if ( [[MIPreferences sharedPrefs] gridShowMask] & kPrefsGridShowVObjLines ||
		 [[MIPreferences sharedPrefs] gridShowMask] & kPrefsGridShowHObjLines )
		[[MIApplicationController sharedApplicationController] redrawAllMainWindows];
}

- (void)draw:(NSRect)rect
{
//	ZzAssert(fRedrawRect.origin.x >= 0);
	[fFrameColor set];
	[NSBezierPath strokeRect:[fInView MIRectToNSRect:[self calculateNewFrame:YES modifiers:0]]];
}

- (void)alternateDraw:(NSRect)rect
{
	NSImage	*dots = mainImageController->fRulerDottedLine;
	NSRect	nFrameRect = [fInView MIRectToNSRect:[self calculateNewFrame:YES modifiers:0]];
	float p1, p2;
	p1 = NSMinX(nFrameRect) - [mainImageController->fRulerDottedLine size].width/2;
	p2 = NSMaxX(nFrameRect) - [mainImageController->fRulerDottedLine size].width/2;

	[dots compositeToPoint:NSMakePoint(p1, 0) operation:NSCompositeSourceAtop];
	[dots compositeToPoint:NSMakePoint(p2, 0) operation:NSCompositeSourceAtop];

	
}

- (void)dealloc
{
	[fFrameColor release];
    [super dealloc];
}

- (MIObjectRep*)resizedObject
{
	return fResizedObject;
}

- (MIRect)calculateNewFrame:(BOOL)inTLC modifiers:(unsigned int)modifiers
{
	MIRect		oldFrame;
	MIRect		newFrame;
	ArmeTime	hOtherEdgePos;
	MIVPos		vOtherEdgePos;
		
	oldFrame = [[fResizedObject representedObject] absoluteModelFrame];
	newFrame = oldFrame;
	
	float pixDelta;
	if ( fEdge1 == NSMinXEdge )
	{
		hOtherEdgePos = MIMaxX(oldFrame);
		GET_VALUE(newFrame.origin.time) += GET_VALUE([fInView pixelToArmeTime:(fLastPos.x-fStartPoint.x)]);
		newFrame.origin.time = [[[fInView mainViewController] snapper]
									snapTime:MIMinX(newFrame) pixDelta:&pixDelta modifiers:modifiers];
		GET_VALUE(newFrame.size.duration) = GET_VALUE(hOtherEdgePos) - GET_VALUE(MIMinX(newFrame));
	}

	if ( fEdge1 == NSMaxXEdge )
	{
		hOtherEdgePos = MIMinX(oldFrame);
		GET_VALUE(newFrame.size.duration) += GET_VALUE([fInView pixelToArmeTime:(fLastPos.x-fStartPoint.x)]);
		GET_VALUE(newFrame.size.duration) = GET_VALUE([[[fInView mainViewController] snapper] 
				snapTime:MIMaxX(newFrame) pixDelta:&pixDelta modifiers:modifiers]) - GET_VALUE(MIMinX(newFrame));
	}
	
	if ( fEdge2 == NSMinYEdge )
	{
		vOtherEdgePos = MIMaxY(oldFrame);
		GET_VALUE(newFrame.origin.vpos) += GET_VALUE([fInView pixelToMIVPos:(fLastPos.y-fStartPoint.y)]);
		MIMainViewController* mainViewController= [fInView mainViewController];
		MISnapper* snapper = [mainViewController snapper];
		newFrame.origin.vpos = [snapper snapVPos:MIMinY(newFrame) pixDelta:&pixDelta modifiers:modifiers];
		GET_VALUE(newFrame.size.height) = GET_VALUE(vOtherEdgePos) - GET_VALUE(MIMinY(newFrame));

		// $$$TODO add check for height being at least the size of the title bar
/*
		if ( NSHeight(newFrame) < 20 )
		{
			newFrame.size.height = 20;
			newFrame.origin.y = otherEdgePos - 20;
		}
*/
	}
	
	if ( fEdge2 == NSMaxYEdge )
	{
		vOtherEdgePos = MIMinY(oldFrame);
		GET_VALUE(newFrame.size.height) += GET_VALUE([fInView pixelToMIVPos:(fLastPos.y-fStartPoint.y)]);
		MIMainViewController* mainViewController= [fInView mainViewController];
		MISnapper* snapper = [mainViewController snapper];
		GET_VALUE(newFrame.size.height) = GET_VALUE([snapper snapVPos:MIMaxY(newFrame) pixDelta:&pixDelta  modifiers:modifiers]) - GET_VALUE(MIMinY(newFrame));
		// $$$TODO add check for height being at most  the size of the title bar
/*
		if ( NSHeight(newFrame) < 20 )
		{
			newFrame.size.height = 20;
		}
*/
	}

	// $$$TODO verify that the frame contains all children here.


	if ( inTLC )
	{
/*
		MIObject	*aParent;
		aParent = [[fResizedObject representedObject] parentObject];
		do
		{
			GET_VALUE(newFrame.origin.vpos) += GET_VALUE(MIMinY([aParent modelFrame]));
			GET_VALUE(newFrame.origin.time) += GET_VALUE(MIMinX([aParent modelFrame]));
		} while ( (aParent = [aParent parentObject]) && (aParent != [fInView representedObject]) );
*/
	}
	else
	{
		newFrame = [[[fResizedObject representedObject] parentObject] convertRectFromTLCCoords:newFrame];
	}
	
    return newFrame;
}

@end

