/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
 /*
	This class is a base on which to build real mouse trackers, like moving clips, selecting, ...
	
	on initialization with initWithView:atPos:withModifiers, it sets the local variables which
	are the point at which the dragging started, a boolean to determine if the mouse has actually
	moved from its starting location on mouse release and the view in which it has been started.
	
	the views who have mousetrackers call the mouse trackers
	"start" on mouseDown, "stop" on mouseUp, 'draw' whenever the views get drawn (useful for selection
	rectangles, ...)
	and "mouseMoved" on mouseDragged.
	
	subclasses override the 'moved' method, not the mouseMoved, the mouseMoved method is used to acertain
	that the _hasMoved flag is correctly maintained, although it could hold other things.
	
*/


#ifndef __MIMOUSETRACKERS_H__

#import "MontageInterface.h"
#define __MIMOUSETRACKERS_H__

@class MIObjectRep;

@interface MIMouseTracker : NSObject
{
	NSPoint					fStartPoint;
	BOOL					fHasMoved;
	id				 		fInView;
	ArmeTime				fStartTime;
	ArmeTime				fEndTime;
	ArmeTime				fMainTime;
}

- (id)initWithView:(id)inView atPos:(NSPoint)where withModifiers:(unsigned int)modifiers;
- (void)start:(NSPoint)where withModifiers:(unsigned int)modifiers;
- (void)stop:(NSPoint)where withModifiers:(unsigned int)modifiers;
- (void)mouseMoved:(NSPoint)where withModifiers:(unsigned int)modifiers;
- (void)moved:(NSPoint)where withModifiers:(unsigned int)modifiers;
- (void)draw:(NSRect)rect;
// the following function is used for trackers that are also drawn somewhere else
// like the move tracker is drawn in the TLC and the overview.
- (void)alternateDraw:(NSRect)rect; 
- (MIObjectRep*)destination;

- (ArmeTime)trackerStartTime;
- (ArmeTime)trackerEndTime;
- (ArmeTime)trackerMainTime;

@end
#endif
