/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
 
#import "MIMouseTracker.h"

#ifndef ZzVerbose_h
//#include "ZzVerbose.h"
#endif //ZzVerbose_h

@implementation MIMouseTracker
- (id)initWithView:(id)inView atPos:(NSPoint)where withModifiers:(unsigned int)modifiers;
{
  self = [super init];
  if ( self )
  {
    fInView = inView;
    fStartPoint = where;
    fHasMoved = NO;
	fStartTime = _ArmeTime(0);
	fEndTime = _ArmeTime(0);
	fMainTime = _ArmeTime(0);
  }
  return self;
}

- (void)start:(NSPoint)where withModifiers:(unsigned int)modifiers
{
  // overriden
}

- (void)stop:(NSPoint)where withModifiers:(unsigned int)modifiers
{
  // Overriden
}

- (void)setEdge:(NSRectEdge)theEdge
{
	vverntl(interfaceStephan,0,"MIMouseTracker::-(void)setEdge:(NSRectEdge)theEdge should be overridden in ur concrete class...");
	ZzAssert(0);
}

- (MIObjectRep*)destination
{
	vverntl(interfaceStephan,0,"MIMouseTracker::-(MIObjectRep*)destination should be overridden in ur concrete class...");
	ZzAssert(0);
	return nil;
}

- (void)mouseMoved:(NSPoint)where withModifiers:(unsigned int)modifiers
{
	if ( !NSEqualPoints(where, fStartPoint) )
		fHasMoved = YES;
	[self moved:where withModifiers:modifiers];
}

- (void)moved:(NSPoint)where withModifiers:(unsigned int)modifiers
{
  // do nothing here, this is overridden.
}

- (void)draw:(NSRect)rect
{
  // used only in cases where something has to be drawn, such as selection
}

- (void)alternateDraw:(NSRect)rect
{
	// used for drawing in other view, IE, a TLC tracker could draw itself in the ruler with this method.
}

- (ArmeTime)trackerStartTime
{
	return fStartTime;
}

- (ArmeTime)trackerEndTime
{
	return fEndTime;
}

- (ArmeTime)trackerMainTime
{
	return fMainTime;
}


@end