/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MIMoveMarkerTracker.h"
#import "MIMainViewController.h"
#import "MIMontageDocument.h"
#import "MICreateDeleteMarkerCommand.h"
#import "MIMoveMarkerCommand.h"
#import "MIRuler.h"
#import "MITopLevelContainerRep.h"

@implementation MIMoveMarkerTracker

- (id)initWithView:(id)inView atPos:(NSPoint)where withModifiers:(unsigned int)modifiers andMarker:(UInt32)marker
{
	if (( self = [super initWithView:inView atPos:where withModifiers:modifiers] ))
	{
		MIMainViewController *controller = [[fInView window] windowController];
		MIMontageDocument *document = [controller document];

		fInitialTime = [document getMarker:marker];
		fMarker = marker;
	}
	return self;
}

- (void)start:(NSPoint)where withModifiers:(unsigned int)modifiers
{
	fMouse = where;
	fShouldDrawGhost = NO;
}

- (void)stop:(NSPoint)where withModifiers:(unsigned int)modifiers
{
	MIMainViewController *controller = [[fInView window] windowController];
	MIMontageDocument *document = [controller document];
	MIRuler *ruler = fInView ;

	if ( fShouldDrawGhost )
		[self setMarkerNeedsDisplayAtPoint:fMouse updateTLC:NO];

	fMouse = where;
	fShouldDrawGhost = NO;

	if ( fMarker != (UInt32) -1 )
	{
		[self removeMarker];
	}

	[self createMarkerAtTime:fInitialTime];

	if ( NSPointInRect( where, [ruler rulerBottomRect] ) )
	{
		MIMoveMarkerCommand* command = [[MIMoveMarkerCommand alloc] initWithDocument:document currentArmeTime:fInitialTime andDestinationArmeTime:[self snappedArmeTimeFromPixels:where.x withModifiers:modifiers]];
		VUSERCMD2("Move Marker from %d to %d", (int)(fInitialTime.data), (int)([self snappedArmeTimeFromPixels:where.x withModifiers:modifiers].data));
		[document doCmd:command];
		[command release];
	}
	else if ( NSPointInRect( where, [ruler rulerTopRect] ) )
	{
		MICreateDeleteMarkerCommand *command = [[MICreateDeleteMarkerCommand alloc] initWithDocument:document andArmeTime:fInitialTime];
		VUSERCMD1("Delete Marker at time = %d", (int)(fInitialTime.data));
		[document doCmd:command];
		[command release];
	}
}

- (void)mouseOutsideRuler:(NSPoint)mouse withModifiers:(unsigned int)modifiers
{
	fShouldDrawGhost = NO;
	if ( fMarker != (UInt32) -1 )
	{
		[self removeMarker];
	}
	[self createMarkerAtTime:fInitialTime];
}

- (void)createMarkerAtTime:(ArmeTime)time
{
    MIMainViewController *controller = [[fInView window] windowController];
    MIMontageDocument *document = [controller document];
    fMarker = [document addMarkerAtTime:time];
	[self setMarkerNeedsDisplay:fMarker];
}


@end
