/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
#import "MIOverviewTrackers.h"
#import "MIOverviewController.h"
#import "MIOverview.h"
#import "MITopLevelContainerRep.h"
#import "MIApplicationController.h"
#import "MIMainViewController.h"
#import "MIObject.h"
#import "MIObjectRep.h"
#import "MITimePaletteController.h"
#import "MIMoveCommand.h"
#import "MIMontageDocument.h"
#import "TArmeDoc.h"
#import "MontageVerbose.h"

#ifndef ZzTypes_h
#include "ZzTypes.h"
#endif //ZzTypes_h

extern bool gCurrentlyDrawingToImage;

@implementation MIMoveVisibleRectTracker
- (void)start:(NSPoint)where withModifiers:(unsigned int)modifiers
{
	[fInView refreshRatios];
	[self moved:where withModifiers:modifiers];
}

- (void)moved:(NSPoint)where withModifiers:(unsigned int)modifiers
{
	MITopLevelContainerRep	*theView;
	NSRect					vRect;
	NSPoint					centralPoint;
	
	centralPoint = where;
	
	centralPoint.x /= [fInView getHRatio];
	centralPoint.y /= [fInView getVRatio];
	
	theView = [[NSApp mainViewController] topLevelContainerView];
	
	vRect = [theView visibleRect];
	
	centralPoint.x -= (NSWidth(vRect)/2);
	centralPoint.y -= (NSHeight(vRect)/2);
	
	[theView scrollPoint:centralPoint];
	
	[fInView setNeedsDisplay:YES];
	
}

- (void)stop:(NSPoint)where withModifiers:(unsigned int)modifiers
{
}

@end

@implementation MIResizeVisibleRectTracker
- (void)start:(NSPoint)where withModifiers:(unsigned int)modifiers
{
	NSRect 	vRect = [fInView getVisibleRect];

	[fInView refreshRatios];
	[self moved:where withModifiers:modifiers];
	fTopLeft.x = NSMinX(vRect);
	fTopLeft.y = NSMaxY(vRect);
}

- (void)stop:(NSPoint)where withModifiers:(unsigned int)modifiers
{
	MIPoint 				miTopLeft;
	NSPoint 				nsTopLeft;
	NSRect					vRect;
	int						vZoomLevel, hZoomLevel;
	int						oldVZoomLevel, oldHZoomLevel;
	MIMainViewController	*ctrl = [NSApp mainViewController];
	MITopLevelContainerRep	*tlc = [ctrl topLevelContainerView];
	
	fLastPoint = where;

	oldVZoomLevel = [ctrl vZoomIndex];
	oldHZoomLevel = [ctrl hZoomIndex];

// determine the current top left corner in model coords

	vRect = [tlc visibleRect];
	nsTopLeft.x = NSMinX(vRect);
	nsTopLeft.y = NSMaxY(vRect);
	miTopLeft = [ctrl NSPointToMIPoint:nsTopLeft]; 

	[self findBestSnappedRectAndVZoom:&vZoomLevel HZoom:&hZoomLevel];

	if ( oldVZoomLevel != vZoomLevel )
		[ctrl setVZoom:vZoomLevel];
	
	if ( oldHZoomLevel != hZoomLevel )
		[ctrl setHZoom:hZoomLevel];
	
	if ( oldHZoomLevel != hZoomLevel || oldVZoomLevel != vZoomLevel )
	{
		nsTopLeft = [ctrl MIPointToNSPoint:miTopLeft];
		nsTopLeft.y -= NSHeight(vRect); // nsTopLeft became bottom left corner here
		[tlc scrollPoint:nsTopLeft];
	}
	[fInView setNeedsDisplay:YES];
}

- (NSRect)findBestSnappedRectAndVZoom:(int*)finalvZoom HZoom:(int*)finalhZoom 
{
	NSRect					vRect = [fInView getVisibleRect]; // the current visible portion of the TLC, in OV coords
	MIMainViewController	*ctrl = [NSApp mainViewController];
	NSRect					newRect;
	float					hRatio = [fInView getHRatio];
	float					vRatio = [fInView getVRatio];
	MIRect					desiredRect;
	NSSize					visibleSize;
	double					bestVDelta = 1LL << 62;
//	double					bestHDelta = 1LL << 62;
	SInt32					vZoomLevel, hZoomLevel, oldVZoomLevel, oldHZoomLevel;
	SInt64					testValue;
	SInt32					nbHZoomIndices = [MIMainViewController getNbHZoomIndices];
	SInt32					nbVZoomIndices = [MIMainViewController getNbVZoomIndices];

	newRect = vRect;
	
	newRect.size.width = max((float)1.0, fLastPoint.x - NSMinX(vRect));
	newRect.size.height = max((float)1.0, NSMaxY(vRect)-fLastPoint.y);
	
	newRect.origin.y = NSMaxY(vRect)-NSHeight(newRect);
	
	// compute the new desired visible rectangle in TLC coords
	newRect.origin.x /= hRatio;
	newRect.origin.y /= vRatio;
	newRect.size.width /= hRatio;
	newRect.size.height /= vRatio;
	
	// convert that to model coordinates
	desiredRect = [ctrl NSRectToMIRect:newRect];
	
	oldVZoomLevel = [ctrl vZoomIndex];
	oldHZoomLevel = [ctrl hZoomIndex];
	vZoomLevel = oldVZoomLevel;
	hZoomLevel = oldHZoomLevel;
	
	visibleSize = [[ctrl topLevelContainerView] visibleRect].size;
	// try finding the closest zoom level at which the visible rectangle size matches the  
	for ( int i = 0; i < max(nbHZoomIndices, nbVZoomIndices); i++ )
	{
		SInt32 convertFactor;
		
		if ( i < nbVZoomIndices )
		{
			convertFactor = [MIMainViewController getVZoomPNValueFromIndex:i];
			// convert the horiontal visbible size of the TLC (in pixels) in ArmeTime and compare that to the best delta stored
			if ( convertFactor > 0 )
				testValue = (SInt64)(visibleSize.height*convertFactor);
			else
				testValue = (SInt64)(visibleSize.height/(-convertFactor));
				
			if ( fabs(testValue - GET_VALUE(desiredRect.size.height)) < bestVDelta )
			{
				bestVDelta = fabs(testValue - GET_VALUE(desiredRect.size.height));
				vZoomLevel = i;
			}
		}
		if ( i < nbHZoomIndices )
		{
			convertFactor = [MIMainViewController getHZoomPNValueFromIndex:i];
			// convert the horiontal visbible size of the TLC (in pixels) in ArmeTime and compare that to the best delta stored
			if ( convertFactor > 0 )
				testValue = (SInt64)(visibleSize.width*convertFactor);
			else
				testValue = (SInt64)(visibleSize.width/(-convertFactor));

			if ( testValue - GET_VALUE(desiredRect.size.duration) < 0 )
			{
				hZoomLevel = (i == nbHZoomIndices ? nbHZoomIndices : i-1);
				break;
			}
/*
			if ( fabs(testValue - GET_VALUE(desiredRect.size.duration)) < bestHDelta )
			{
				bestHDelta = fabs(testValue - GET_VALUE(desiredRect.size.duration));
				hZoomLevel = i;
			}
*/
		}
	}
	
	*finalhZoom = hZoomLevel;
	*finalvZoom = vZoomLevel;
	
	double ol, ne;
	newRect = vRect;

	ol = (double)[MIMainViewController getHZoomPNValueFromIndex:oldHZoomLevel];
	ne = (double)[MIMainViewController getHZoomPNValueFromIndex:hZoomLevel];
	if ( ol < 0 )
		ol = -1/ol;
	if ( ne < 0 )
		ne = -1/ne;
		
	newRect.size.width /= ol/ne;

	ol = (double)[MIMainViewController getVZoomPNValueFromIndex:oldVZoomLevel];
	ne = (double)[MIMainViewController getVZoomPNValueFromIndex:vZoomLevel];
	if ( ol < 0 )
		ol = -1/ol;
	if ( ne < 0 )
		ne = -1/ne;
	newRect.size.height /= ol/ne;

	newRect.origin.y = NSMaxY(vRect) - NSHeight(newRect);
	
	return newRect;
}

- (void)moved:(NSPoint)where withModifiers:(unsigned int)modifiers
{
	fLastPoint = where;
	[fInView setNeedsDisplay:YES];
}

- (void)setEdge:(MIRectEdge)theEdge
{
	fEdge = theEdge;
}

- (void)draw:(NSRect)rect
{
	int tmp1, tmp2;
	
	[[NSColor redColor] set];
//	[NSBezierPath strokeRect:RectWithPoints(fTopLeft, fLastPoint)];
	[NSBezierPath strokeRect:[self findBestSnappedRectAndVZoom:&tmp1 HZoom:&tmp2]];
}
@end

@implementation MIDragThingsFromOutsideInOverviewTracker: MIMouseTracker
- (void)start:(NSPoint)where withModifiers:(unsigned int)modifiers
{
	fTmpDest = nil;
	fLastPoint = where;
	fTmpDest = [[[NSApp mainViewController] topLevelContainerView] hitTestForMIObject:fLastPoint];
}

- (void)moved:(NSPoint)where withModifiers:(unsigned int)modifiers
{
	fLastPoint = where;
	
	if ( fTmpDest )
	{
		[fTmpDest setNeedsDisplay];
	}
	
	fTmpDest = [[[NSApp mainViewController] topLevelContainerView] hitTestForMIObject:fLastPoint];
	
	if ( ![[fTmpDest representedObject] isContainer] )
	{
		// the destination is not a container
		fTmpDest = [[[fTmpDest representedObject] parentObject] findRepresentationInTLC:fInView];
		if ( fTmpDest == fInView )
			fTmpDest = nil;
	}
	
	if ( fTmpDest )
	{
		[fTmpDest setNeedsDisplay];
	}
	
}

- (void)stop:(NSPoint)where withModifiers:(unsigned int)modifiers
{
	if ( fTmpDest )
	{
		[fTmpDest setNeedsDisplay];
	}
	[[MIApplicationController sharedApplicationController] modelChangedForDocument:[[NSDocumentController sharedDocumentController] currentDocument]];
}
/*
- (void)draw:(NSRect)rect
{
	if ( fTmpDest )
	{
		NSRect	tlcFrame;
		[[NSColor colorWithCalibratedRed:1.0f green:0.0f blue:0.0f alpha:1.0f] set];
		
		tlcFrame = [fTmpDest getTLCFrame];
		[NSBezierPath strokeRect:tlcFrame];
	}
}
*/
- (MIObjectRep*)destination
{
	return fTmpDest;
}

@end


@implementation MIOverviewSelectionTracker
- (void)start:(NSPoint)where withModifiers:(unsigned int)modifiers
{
	MIMontageDocument		*mDoc;

	mDoc = [[NSDocumentController sharedDocumentController] currentDocument];
	fCtrl = [NSApp mainViewController];
	
	[fCtrl clearSelection];
	[fCtrl selectionSetObjectsType];

	[[MITimePaletteController sharedTimePaletteController] setFeedbackType:MITimePaletteSelectionFeedback];
	[[MITimePaletteController sharedTimePaletteController] setFeedbackObject:[fCtrl selection]];
	[[MITimePaletteController sharedTimePaletteController] getValuesFromMainView];
	
	
}

- (void)moved:(NSPoint)where withModifiers:(unsigned int)modifiers
{
	// do like the finder does : do not care about the 'modifiers'
	// param when moving. we only care about the initial state
	// of 'modifiers' (the one that was given to the start() method)
	// NOTE : this isn't true for a move/copy/alias tracker where
	// the user can change the policy as he wants until the releases
	// the mousebutton

	// look for views in the rectangle defined by the old and new location...
	// origin is lower left corner.
	
	NSRect  				newRectOV;
	NSRect					newRectTLC;
	NSEnumerator			*viewsEnumerator;
	MIObject				*anObject;
	MIObjectRep				*anObjectRep;
	NSMutableArray			*newSel = [[NSMutableArray alloc] initWithObjects:nil];
	NSEnumerator			*objEnum;
	NSArray					*oldSel;
	
	
	// the whole new selection
	newRectOV = RectWithPoints(fStartPoint, where);

	[fInView setNeedsDisplayInRect:NSUnionRect(fCurrentSelectionRect, newRectOV)];
	fCurrentSelectionRect = newRectOV;

	newRectTLC = ScaleNSRect(newRectOV, 1/[fInView getHRatio], 1/[fInView getVRatio]);
	

	viewsEnumerator = [[[fCtrl getTLC] getChildren] objectEnumerator];
		
	while ( anObject = [viewsEnumerator nextObject] )
	{
		anObjectRep = [anObject findRepresentationInTLC:[fCtrl topLevelContainerView]];
	
		NSRect objFrame = [anObjectRep getTLCFrame];
			
		if ((NSContainsRect(newRectTLC,objFrame) || NSIntersectsRect(newRectTLC, objFrame)) && !NSContainsRect(objFrame, newRectTLC) )
			[newSel addObject:anObjectRep];
	}
	
	// first remove all the unnecessary objects from the old selection
	objEnum = [[fCtrl selection] objectEnumerator];
	while ( anObjectRep = [objEnum nextObject] )
	{
		if ( ![newSel containsObject:anObjectRep] )
			[[fCtrl selection] removeObject:anObjectRep];
	}
	// then add all the other objects
	
	objEnum = [newSel objectEnumerator];
	oldSel = [[fCtrl selection] objArray];
	while ( anObjectRep = [objEnum nextObject] )
	{
		if ( ![oldSel containsObject:anObjectRep] )
			[fCtrl addObjectToSelection:anObjectRep];
	}
	
	[[[MIApplicationController sharedApplicationController] timePaletteController] getValuesFromMainView];
}
	
-(void)stop:(NSPoint)where withModifiers:(unsigned int)modifiers
{
	[fInView setNeedsDisplay:YES];
}
	
-(void)draw:(NSRect)rect
{
	
	[[[NSColor selectedTextBackgroundColor] colorWithAlphaComponent:0.7] set];
	[NSBezierPath fillRect:fCurrentSelectionRect];
	[[[NSColor whiteColor] colorWithAlphaComponent:0.7] set];
	[NSBezierPath strokeRect:NSInsetRect(fCurrentSelectionRect,1,1)];
	
//	[[NSColor redColor] set];
//	[aPath stroke];
}

@end


@implementation MIOverviewMoveSelectionTracker

- (void)start:(NSPoint)where withModifiers:(unsigned int)modifiers
{
	fCtrl = [NSApp mainViewController];
	
	fDragImage = [self createImage];
}

- (void)moved:(NSPoint)where withModifiers:(unsigned int)modifiers
{
	NSRect imgRect;
	
	if ( fHorizontalLock )
		where.x = fStartPoint.x;

	imgRect.origin.x = where.x + fImageOriginOffset.x;
	imgRect.origin.y = where.y + fImageOriginOffset.y;
	
	imgRect.size = [fDragImage size];
	
	[fInView setNeedsDisplayInRect:NSUnionRect(fLastRect, imgRect)];
	
	fLastRect = imgRect;
}

- (void)stop:(NSPoint)where withModifiers:(unsigned int)modifiers
{
	MIMoveCommand	*cmd;
	NSPoint			moveOffset;
	
	if ( fHorizontalLock ) 
		where.x = fStartPoint.x;
		
	moveOffset.x = (where.x - fStartPoint.x) / [fInView getHRatio];
	moveOffset.y = (where.y - fStartPoint.y) / [fInView getVRatio];
	
	cmd = [[[MIMoveCommand alloc] initWithSelection:[fCtrl selection]] autorelease];
	VUSERCMD1("Move (through overview), offset = %d", (int)([fCtrl NSPointToMIPoint:moveOffset].time.data));
	VUSERCMD_PRINT_SELECTION([fCtrl selection]);
	VUSERCMD("-New Parent = tlc");
	
	[cmd setOffset:[fCtrl NSPointToMIPoint:moveOffset]];
	[cmd setNewParent:[fCtrl getTLC]];
	[[fCtrl document] doCmd:cmd];
}

- (void)dealloc
{
	[fDragImage dealloc];
	[super dealloc];
}


- (NSImage*)createImage
{
	NSEnumerator	*selObjectsEnum;
	NSRect			allSelRect;
	NSRect			anObjectFrame;
	NSRect			drawFrame;
	MIObjectRep		*anObjectRep;
	NSPoint			offset;
	NSImage			*theImage;
	
	// first go all over the selection, check all the objects coordinates and determine maxRect from that
	selObjectsEnum = [fCtrl selectionObjectEnumerator];

	allSelRect = NSZeroRect;

	while ( anObjectRep = [selObjectsEnum nextObject] )
	{
		anObjectFrame = ScaleNSRect([anObjectRep getTLCFrame], [fInView getHRatio], [fInView getVRatio]);

		if ( NSEqualRects( allSelRect, NSZeroRect) )
		{
			allSelRect = anObjectFrame;
		}
		else
		{
			allSelRect = NSUnionRect( allSelRect, anObjectFrame);
		}
	}
	
	selObjectsEnum = [fCtrl selectionObjectEnumerator];

	offset.x = -allSelRect.origin.x;
	offset.y = -allSelRect.origin.y;

	theImage = [[NSImage alloc] initWithSize:allSelRect.size];
	[theImage lockFocus];
		gCurrentlyDrawingToImage = YES;
		while ( anObjectRep = [selObjectsEnum nextObject] )
		{
			drawFrame = ScaleNSRect([anObjectRep getTLCFrame], [fInView getHRatio], [fInView getVRatio]);
			drawFrame.origin.x += offset.x;
			drawFrame.origin.y += offset.y;
			
			[[anObjectRep backgroundColor] set];
			[NSBezierPath fillRect:drawFrame];
			[[NSColor blackColor] set];
			[NSBezierPath strokeRect:drawFrame];
		}
		gCurrentlyDrawingToImage = NO;
	[theImage unlockFocus];

	fImageOriginOffset = TranslateNSPoint(offset,fStartPoint);
	fImageOriginOffset.x = - fImageOriginOffset.x;
	fImageOriginOffset.y = - fImageOriginOffset.y;
	
	
	return theImage;
}
- (void)draw:(NSRect)rect
{
	NSPoint		blitPoint;
	
	blitPoint = fLastRect.origin;

	// display the image
	[fDragImage dissolveToPoint:blitPoint fraction:0.5f];    
}

@end


@implementation MIOverviewMoveLocatorTracker
- (void)start:(NSPoint)where withModifiers:(unsigned int)modifiers
{
	fLastPos = where;

	fCtrl = [NSApp mainViewController];
}

- (void)setLocator:(MILocatorType)locator
{
	fType = locator;
	
	if(fType == MIPlayBarLocator)
	{
		// if it's a playbar locator move, then we disable the posInDoc incrementation to
		// get that nice time stretcher effect
		TArmeDoc* armeDoc = [[[NSDocumentController sharedDocumentController] currentDocument] armeDocument];
		armeDoc->GetTLCPlayer()->SetPosInDocIncrementation(false);
	}
}

- (void)moved:(NSPoint)where withModifiers:(unsigned int)modifiers
{
	float		newPos;
	ArmeTime	theTime;
	TArmeDoc	*theDoc;
	fLastPos = where;

	fLastPos = where;

	if ( fLastPos.x > NSMaxX([fInView bounds]) )
		fLastPos.x = NSMaxX([fInView bounds]);

	theDoc = [[fCtrl document] armeDocument]; 
	
	newPos = where.x / [fInView getHRatio];
	
	theTime = [fCtrl pixelToArmeTime:newPos];
	
	GET_VALUE(theTime) = (SInt64)(GET_VALUE(theTime));

	switch(fType)
	{
		case MIPlayBarLocator: 
			// play marker
//			theDoc->LockWrite(); $$$BENJA that's ok. ask benja
			{
				theDoc->Preroll(theTime);
				theDoc->Seek(theTime);
			}
//			theDoc->UnlockWrite();
			break;
		case MILeftLocator:
//			theDoc->LockWrite();
			{
				theDoc->SetLLocator(theTime);
			}
//			theDoc->UnlockWrite();
			break;
		case MIRightLocator:
//			theDoc->LockWrite();
			{
				theDoc->SetRLocator(theTime);
			}
//			theDoc->UnlockWrite();
			break;
		case MICustomMarker:
		case MIUnknownLocator:
			break;
	}

	switch(fType)
	{
		case MIPlayBarLocator:
			// play marker
			[fCtrl updatePlayBar];
			break;
		case MILeftLocator:
		case MIRightLocator:
			[fCtrl updateLocators];
			break;
		case MICustomMarker:
		case MIUnknownLocator:
			break;
	}
	
	[fInView setNeedsDisplay:YES];
	
/*	if ( fLastPos.x > NSMaxX([fInView frame]) )
		fLastPos.x = NSMaxX([fInView frame]);
	[fInView setNeedsDisplay:YES];*/
	[[[MIApplicationController sharedApplicationController] timePaletteController] getValuesFromMainView];
}

- (void)stop:(NSPoint)where withModifiers:(unsigned int)modifiers
{
	if(fType == MIPlayBarLocator)
	{
		// if it's a playbar locator move, then we disable the posInDoc incrementation to
		// get that nice time stretcher effect
		TArmeDoc* armeDoc = [[fCtrl document] armeDocument];
		armeDoc->GetTLCPlayer()->SetPosInDocIncrementation(true);
	}

	if ( ( fType == MILeftLocator ) || ( fType == MIRightLocator ) )
		[[fCtrl document] updateDocumentChangeCount:MIChangeDone];

}

- (void)draw:(NSRect)rect
{
}

@end





@implementation MICreateNewVisibleRectTracker
- (void)start:(NSPoint)where withModifiers:(unsigned int)modifiers
{
//	NSRect 	vRect = [fInView getVisibleRect];

	[fInView refreshRatios];
	[self moved:where withModifiers:modifiers];
}

- (void)stop:(NSPoint)where withModifiers:(unsigned int)modifiers
{
	NSRect					newRectTLC;
	NSRect					newRectOV;
	int						vZoomLevel, hZoomLevel;
	int						oldVZoomLevel, oldHZoomLevel;
	MIMainViewController	*ctrl = [[NSApp mainWindow] windowController];
	MITopLevelContainerRep	*tlc = [ctrl topLevelContainerView];
	
	fLastPoint = where;

	oldVZoomLevel = [ctrl vZoomIndex];
	oldHZoomLevel = [ctrl hZoomIndex];

	newRectOV = [self findBestSnappedRectAndVZoom:&vZoomLevel HZoom:&hZoomLevel];

	if ( oldVZoomLevel != vZoomLevel )
		[ctrl setVZoom:vZoomLevel];
	
	if ( oldHZoomLevel != hZoomLevel )
		[ctrl setHZoom:hZoomLevel];
	
	// transform the origin in TLC coordinates
	[fInView refreshRatios];
	newRectTLC = ScaleNSRect(newRectOV, 1/[fInView getHRatio], 1/[fInView getVRatio]);

	[tlc scrollPoint:newRectTLC.origin];

	[fInView setNeedsDisplay:YES];
}

- (NSRect)findBestSnappedRectAndVZoom:(int*)finalvZoom HZoom:(int*)finalhZoom 
{
	NSRect					vRect = [fInView getVisibleRect]; // the current visible portion of the TLC, in OV coords
	NSRect					cRect;
	MIMainViewController	*ctrl = [[NSApp mainWindow] windowController];
	NSRect					newRect;
	float					hRatio = [fInView getHRatio];
	float					vRatio = [fInView getVRatio];
	MIRect					desiredRect;
	NSSize					visibleSize;
	double					bestVDelta = 1LL << 62;
	double					bestHDelta = 1LL << 62;
	SInt32					vZoomLevel, hZoomLevel, oldVZoomLevel, oldHZoomLevel;
	SInt64					testValue;
	SInt32					nbHZoomIndices = [MIMainViewController getNbHZoomIndices];
	SInt32					nbVZoomIndices = [MIMainViewController getNbVZoomIndices];


	/*
		vRect: current visible rectangle, in OV coords
		cRect : the current rectangle being drawn
		desiredRect: the current rectangle, in model coords
		newRect: the new visible Rectangle, in OV coords
	*/
	cRect = RectWithPoints(fStartPoint, fLastPoint);
	
	newRect = cRect;
//	newRect.size.width = max((float)1.0, fLastPoint.x - NSMinX(vRect));
//	newRect.size.height = max((float)1.0, NSMaxY(vRect)-fLastPoint.y);
	
//	newRect.origin.y = NSMaxY(vRect)-NSHeight(newRect);
	
	// compute the new desired visible rectangle in TLC coords
	newRect.origin.x /= hRatio;
	newRect.origin.y /= vRatio;
	newRect.size.width /= hRatio;
	newRect.size.height /= vRatio;
	
	// convert that to model coordinates
	desiredRect = [ctrl NSRectToMIRect:newRect];
	
	oldVZoomLevel = [ctrl vZoomIndex];
	oldHZoomLevel = [ctrl hZoomIndex];
	vZoomLevel = oldVZoomLevel;
	hZoomLevel = oldHZoomLevel;
	
	visibleSize = [[ctrl topLevelContainerView] visibleRect].size;
	// try finding the closest zoom level at which the visible rectangle size matches the  
	for ( int i = 0; i < max(nbHZoomIndices, nbVZoomIndices); i++ )
	{
		SInt32 convertFactor;
		
		if ( i < nbVZoomIndices )
		{
			convertFactor = [MIMainViewController getVZoomPNValueFromIndex:i];
			// convert the horiontal visbible size of the TLC (in pixels) in ArmeTime and compare that to the best delta stored
			if ( convertFactor > 0 )
				testValue = (SInt64)(visibleSize.height*convertFactor);
			else
				testValue = (SInt64)(visibleSize.height/(-convertFactor));
				
			if ( fabs(testValue - GET_VALUE(desiredRect.size.height)) < bestVDelta )
			{
				bestVDelta = fabs(testValue - GET_VALUE(desiredRect.size.height));
				vZoomLevel = i;
			}
		}
		if ( i < nbHZoomIndices )
		{
			convertFactor = [MIMainViewController getHZoomPNValueFromIndex:i];
			// convert the horiontal visbible size of the TLC (in pixels) in ArmeTime and compare that to the best delta stored
			if ( convertFactor > 0 )
				testValue = (SInt64)(visibleSize.width*convertFactor);
			else
				testValue = (SInt64)(visibleSize.width/(-convertFactor));
				
			if ( fabs(testValue - GET_VALUE(desiredRect.size.duration)) < bestHDelta )
			{
				bestHDelta = fabs(testValue - GET_VALUE(desiredRect.size.duration));
				hZoomLevel = i;
			}
		}
	}
	
	
	*finalhZoom = hZoomLevel;
	*finalvZoom = vZoomLevel;
	
	double ol, ne;
	newRect = vRect;

	ol = (double)[MIMainViewController getHZoomPNValueFromIndex:oldHZoomLevel];
	ne = (double)[MIMainViewController getHZoomPNValueFromIndex:hZoomLevel];
	if ( ol < 0 )
		ol = -1/ol;
	if ( ne < 0 )
		ne = -1/ne;
		
	newRect.size.width /= ol/ne;

	ol = (double)[MIMainViewController getVZoomPNValueFromIndex:oldVZoomLevel];
	ne = (double)[MIMainViewController getVZoomPNValueFromIndex:vZoomLevel];
	if ( ol < 0 )
		ol = -1/ol;
	if ( ne < 0 )
		ne = -1/ne;
	newRect.size.height /= ol/ne;


	// depending on where the drag started compared to the current position, determine where rectangle is placed
	
	// keep top
	if ( fStartPoint.y > fLastPoint.y )
		newRect.origin.y = NSMaxY(cRect) - NSHeight(newRect);
	else
		newRect.origin.y = NSMinY(cRect);
		
	// keep right
	if ( fStartPoint.x > fLastPoint.x )
		newRect.origin.x = NSMaxX(cRect) - NSWidth(newRect);
	else
		newRect.origin.x = NSMinX(cRect);
	// bottom (nothing)
	
	// left (nothing)
	
	return newRect;
}

- (void)moved:(NSPoint)where withModifiers:(unsigned int)modifiers
{
	fLastPoint = where;
	[fInView setNeedsDisplay:YES];
}

- (void)draw:(NSRect)rect
{
	int tmp1, tmp2;
	
	[[NSColor redColor] set];
//	[NSBezierPath strokeRect:RectWithPoints(fTopLeft, fLastPoint)];
	[NSBezierPath strokeRect:[self findBestSnappedRectAndVZoom:&tmp1 HZoom:&tmp2]];
}
@end
