/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
#import "MIRulerTrackers.h"
#import "MIRuler.h"
#import "MITopLevelContainerRep.h"
#import "MIMainViewController.h"
#import "MIMontageDocument.h"
#import "MITime.h"
#import "MIApplicationController.h"
#import "MITimePaletteController.h"
#import "MIPreferences.h"
#import "MIOverviewController.h"
#import "MICreateDeleteMarkerCommand.h"
#import "MIMoveMarkerCommand.h"
#import "MIImageController.h"

#ifndef TArmeDoc_h
#include "TArmeDoc.h"
#endif //TArmeDoc_h



@implementation MIMoveLocatorTracker
- (void)start:(NSPoint)where withModifiers:(unsigned int)modifiers
{
	fLastPos = where;
	
	[[MITimePaletteController sharedTimePaletteController] setFeedbackType:MITimePaletteTrackerFeedback];
	[[MITimePaletteController sharedTimePaletteController] setFeedbackObject:self];

	if ( fType == MICustomMarker && fMarkerIndex == (UInt32)-1 )
		fShouldCreateMarker	= YES;
	else
		fShouldCreateMarker = NO;

	[self moved:where withModifiers:modifiers]; // TT 729
}

- (void)setLocatorType:(MILocatorType)locator index:(UInt32)index
{
	MIMainViewController*	controller = [[fInView window] windowController];
	MIMontageDocument*		document = [controller document];
	fType = locator;
	
	if(fType == MIPlayBarLocator)
	{
		// if it's a playbar locator move, then we disable the posInDoc incrementation to
		// get that nice time stretcher effect
		//MIMainViewController* mainViewController = [[fInView getTopLevelContainerView] mainViewController];
		//TArmeDoc* armeDoc = [[mainViewController document] armeDocument];
	}
	
	fMarkerIndex = index; // store this in case we have to remove the marker.
	if ( index != (UInt32)-1 && fType == MICustomMarker )
		fInitialMarkerTime = [document getMarker:fMarkerIndex];
}

- (void)moved:(NSPoint)where withModifiers:(unsigned int)modifiers
{
	float		newPos;
	ArmeTime	theTime;
	TArmeDoc	*theDoc;
	float		tmp;
	NSRect		visibleRect = [[fInView getTopLevelContainerView] visibleRect];
	
	fLastPos = where;

	// cap the position (prevent negative times)
	if ( fLastPos.x > NSMaxX([fInView bounds]) )
		fLastPos.x = NSMaxX([fInView bounds]);

	if ( fLastPos.x < NSMinX([fInView bounds]) )
		fLastPos.x = NSMinX([fInView bounds]);

	//newPos = fLastPos.x; // + [[fInView getTopLevelContainerView] visibleRect].origin.x;
	// snap the time to the grid
	
	MIMainViewController* mainViewController = [[fInView getTopLevelContainerView] mainViewController];
	
	theDoc = [[mainViewController document] armeDocument]; 
	
	// WE DONT WANT A LOC TO SNAP ON ITSELF
	SInt32 avoidLocatorIndex = 0;

	switch(fType)
	{
	case MILeftLocator:
		avoidLocatorIndex = -3;
		break;
	case MIPlayBarLocator: 
		avoidLocatorIndex = -2;
		break;
	case MIRightLocator:
		avoidLocatorIndex = -1;
		break;
	case MICustomMarker:
		avoidLocatorIndex = 0;
		break;
	case MIUnknownLocator:
		avoidLocatorIndex = 0;
		break;
	}
	
	tmp = fLastSnappedX;
	float pixDelta; // we don't use this return value
	newPos = [mainViewController armeTimeToPixel:[[mainViewController snapper] 
			snapTime:[mainViewController pixelToArmeTime:fLastPos.x]
			pixDelta:&pixDelta
			modifiers:modifiers 
			avoidLocatorIndex:avoidLocatorIndex]];
	
	fLastSnappedX = newPos;
	
	theTime = [[fInView getTopLevelContainerView] pixelToArmeTime:newPos];
	
	GET_VALUE(theTime) = (SInt64)(GET_VALUE(theTime));
	
	fMainTime = theTime;

	switch(fType)
	{
		case MIPlayBarLocator: 
			// play marker
			{
				if(modifiers & NSAlternateKeyMask)
					theDoc->GetTLCPlayer()->SetPosInDocIncrementation(false);
				else
					theDoc->GetTLCPlayer()->SetPosInDocIncrementation(true);
				theDoc->Seek(theTime);
			}
			break;
		case MILeftLocator:
			theDoc->SetLLocator(theTime);
			break;
		case MIRightLocator:
			theDoc->SetRLocator(theTime);
			break;
		case MICustomMarker:
			[[fInView getTopLevelContainerView]
				setNeedsDisplayInRect:
					NSMakeRect(min(fLastSnappedX, tmp) - 0.5, NSMinY(visibleRect), fabs(fLastSnappedX - tmp) +1 , NSHeight(visibleRect) )];
			if ( fMarkerIndex == (UInt32)-1 && NSPointInRect( where, [fInView rulerBottomRect] ) )
			{
				fShouldDrawGhost = NO;
				[self createMarkerAtPoint:where withModifiers:modifiers];
			}
			else if ( fMarkerIndex != (UInt32)-1 && NSPointInRect( where, [fInView rulerTopRect] ) )
			{
				fShouldDrawGhost = YES;
				[self removeMarker];
			}

			if ( fMarkerIndex != (UInt32)-1 )
				[[mainViewController document] moveMarker:fMarkerIndex toTime:theTime];
			break;
		case MIUnknownLocator:
			break;
	}

	switch(fType)
	{
		case MIPlayBarLocator:
			// play marker
			[[[fInView getTopLevelContainerView] mainViewController] updatePlayBar];
			break;
		case MILeftLocator:
		case MIRightLocator:
			[[[fInView getTopLevelContainerView] mainViewController] updateLocators];
			break;
		case MICustomMarker:
		case MIUnknownLocator:
			break;
	}
	
	[fInView setNeedsDisplay:YES];

	[[MIOverviewController sharedOverviewController] setViewNeedsDisplay];
/*	if ( fLastPos.x > NSMaxX([fInView frame]) )
		fLastPos.x = NSMaxX([fInView frame]);
	[fInView setNeedsDisplay:YES];*/
	[[[MIApplicationController sharedApplicationController] timePaletteController] getValuesFromMainView];
}

- (void)stop:(NSPoint)where withModifiers:(unsigned int)modifiers
{
	ArmeTime 				theTime = [[fInView getTopLevelContainerView] pixelToArmeTime:fLastSnappedX];
	MIMainViewController*	controller = [[fInView window] windowController];
	MIMontageDocument*		document = [controller document];
	id						cmd;

	if ( fType ==  MIPlayBarLocator )
	{ 
		// play marker
		// if it's a playbar locator move, then we enable the posInDoc incrementation again
		MIMainViewController* mainViewController = [[fInView getTopLevelContainerView] mainViewController];
		TArmeDoc* armeDoc = [[mainViewController document] armeDocument];
		armeDoc->GetTLCPlayer()->SetPosInDocIncrementation(true);
		armeDoc->Seek(theTime);
		armeDoc->Preroll(theTime);
		[[[fInView getTopLevelContainerView] mainViewController] updatePlayBar];
	}
	else if ( fType == MICustomMarker )
	{
		if ( !NSPointInRect( where, [fInView rulerTopRect] ) )
		{
			// is in bottom part. If no marker existed, create one, only if no other marker existed at that position.
			if ( fShouldCreateMarker )
			{
				// marker creation
				[self removeMarker]; // remove the marker that we temporarily created first
				cmd = [[MICreateDeleteMarkerCommand alloc] initWithDocument:document andArmeTime:theTime];
				VUSERCMD1("Create Marker at time = %d", (int)(theTime.data));
				[document doCmd:cmd];
				[cmd release];
			}
			else
			{
				// marker move
				cmd = [[MIMoveMarkerCommand alloc] initWithDocument:document currentArmeTime:[document getMarker:fMarkerIndex] andDestinationArmeTime:theTime];
				VUSERCMD2("Move Marker from %d to %d", (int)([document getMarker:fMarkerIndex].data), (int)(theTime.data));
				[document doCmd:cmd];
				[cmd release];
			}
		}
		else
		{
			// is it top part
			if ( !fShouldCreateMarker )
			{
				[document addMarkerAtTime:fInitialMarkerTime]; // put a marker back at the initial position
				cmd = [[MICreateDeleteMarkerCommand alloc] initWithDocument:document andArmeTime:fInitialMarkerTime];
				VUSERCMD1("Delete Marker at time = %d", (int)(fInitialMarkerTime.data));
				[document doCmd:cmd];
				[cmd release];
			}
		}
	}
	
	if ( ( fType == MILeftLocator ) || ( fType == MIRightLocator ) )
		[[[[fInView getTopLevelContainerView] mainViewController] document] updateDocumentChangeCount:MIChangeDone];

	[[MIOverviewController sharedOverviewController] setViewNeedsDisplay];
	[fInView setNeedsDisplay:YES];
}

- (void)draw:(NSRect)rect
{
	NSPoint p;
	p = fLastPos;
	
	if ( fType == MICustomMarker && fLastPos.y > 20)
	{
		p.x -= [mainImageController->fRulerGenericLocator size].width /2;
		p.y -= [mainImageController->fRulerGenericLocator size].height /2;
		[mainImageController->fRulerGenericLocator compositeToPoint:p operation:NSCompositeSourceAtop];
	}
	
	if ( [[MITimePaletteController sharedTimePaletteController] isVisible] )
		return;
	// the following code draws a yellow box with the time written inside
	// disabled as per georges' wish
	
	NSRect		markerRect = NSMakeRect(0,26,10,10);
	float		markerPos = 0;
	NSPoint		visibleRectOrigin = [[fInView getTopLevelContainerView] visibleRect].origin;
	MITime		*theTime = [MITime timeWithDocument:[[[fInView getTopLevelContainerView] mainViewController] document]];
	NSString	*theString;
	NSSize		stringSize;
	NSRect		tmpRect;

	markerPos = fLastSnappedX;
	switch(fType)
	{
	case MIPlayBarLocator: 
		// play marker
		[[NSColor blueColor] set];
		break;
	case MILeftLocator:
		// left locator
		[[NSColor greenColor] set];
		break;
	case MIRightLocator:
		// right locator
		[[NSColor redColor] set];
		break;
	case MICustomMarker:
		[[NSColor grayColor] set];
		break;
	case MIUnknownLocator:
		// nothing to be done here, it's actually a bad case, should never happen.
		break;
	}
	
	markerRect.origin.x = markerPos + 1;
	
	[theTime setArmeTime:[[fInView getTopLevelContainerView] pixelToArmeTime:markerPos]];
	if ([[MIPreferences sharedPrefs] timeDisplay] & kTimeDisplaySMPTE)
		theString = [theTime getSMPTEString];	
	else
		theString = [theTime getBPMString];
		
	stringSize = [theString sizeWithAttributes:nil];
	
	tmpRect = NSMakeRect(markerPos+10,(22-(stringSize.height/2)), 10+stringSize.width, stringSize.height);
	
//	[[NSColor yellowColor] set];
	[[NSColor cyanColor] set];
	[NSBezierPath fillRect:tmpRect];
	
	[[NSColor blackColor] set];
	[NSBezierPath strokeRect:tmpRect];
	tmpRect.origin.x += 5;
	[theString drawAtPoint:tmpRect.origin withAttributes:nil];
	
	if ( fType == MICustomMarker && fMarkerIndex == (UInt32)-1 )
	{
		NSImage *img = mainImageController->fRulerGenericLocator;
		NSPoint point = NSMakePoint(fLastSnappedX-([img size].width/2),fLastPos.y-([img size].height/2));
		[img compositeToPoint:point operation:NSCompositeSourceOver fraction:0.5];
	}
	
}

- (void)createMarkerAtPoint:(NSPoint)point withModifiers:(unsigned int)modifiers
{
    MIMainViewController *controller = [[fInView window] windowController];
    MIMontageDocument *document = [controller document];

	float pixDelta; // we don't use this return value
    fMarkerIndex = [document addMarkerAtTime:[[controller snapper] 
			snapTime:[controller pixelToArmeTime:point.x]
			pixDelta:&pixDelta
			modifiers:modifiers 
			avoidLocatorIndex:0]];
}

- (void)removeMarker
{
    MIMainViewController *controller = [[fInView window] windowController];
    MIMontageDocument *document = [controller document];

    [document removeMarker:fMarkerIndex];
	fMarkerIndex = (UInt32) -1;
}



@end
