/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#import "MISliceSelectionTracker.h"
#import "MIObjectRep.h"
#import "MIObject.h"
#import "MITopLevelContainerRep.h"
#import "MIMainViewController.h"
#import "MITimePaletteController.h"
#import "MIImageController.h"
#include "TArmeDoc.h"
#include "MovieHelper.h"

@implementation MISliceSelectionTracker

- (id)initWithObjectRep:(MIObjectRep*)rep inView:(id)inView atPos:(NSPoint)where withModifiers:(unsigned int)modifiers;
{
	[super initWithView:inView atPos:where withModifiers:modifiers];

	fObjRep = rep; // WARNING: fObjRep CAN BE TOPLEVELCONTAINER
	fOriginAlreadySet = NO;

	return self;
}

- (void)setStartTime:(ArmeTime)time
{
	fOriginAlreadySet = YES;

	// destroy global selection
	[[[fInView mainViewController] selection] clear];

	// create a null length selection
	[[[fObjRep getTopLevelContainerRep] mainViewController] selectionSetTimeSliceType];	
	[[[fObjRep getTopLevelContainerRep] mainViewController] addObjectToSelection:fObjRep]; // WARNING: fObjRep CAN BE TOPLEVELCONTAINER
	[[[fObjRep getTopLevelContainerRep] mainViewController] setSelectionStartTime:time];
	[[[fObjRep getTopLevelContainerRep] mainViewController] setSelectionEndxTime:time];
}

- (void)start:(NSPoint)where withModifiers:(unsigned int)modifiers
{
	NSPoint localWhere = [fObjRep convertPointFromTLCCoords:where];
	ArmeTime globalTime = [[[fObjRep getTopLevelContainerRep] mainViewController] pixelToArmeTime:fStartPoint.x];
	globalTime = [[fObjRep representedObject] WSObject]->GranulateTime(globalTime); // WARNING: fObjRep CAN BE TOPLEVELCONTAINER
	
	if ( !fOriginAlreadySet )
	{
		// if the object is already the only one of the slice selection and the shift key is pressed, make a union instead of deleting
		if ( [[[fInView mainViewController] selection] type] == kMIWSSelectionTypeTimeSlice &&
			 [[[fInView mainViewController] selection] containsObject:fObjRep] &&
			 modifiers & NSShiftKeyMask )
		{
			[[fInView mainViewController] setSelectionEndxTime:[[[fObjRep getTopLevelContainerRep] mainViewController] pixelToArmeTime:where.x]];
		}
		else
		{
			// destroy global selection
			[[[fInView mainViewController] selection] clear];
		
			// create a null length selection
			[[[fObjRep getTopLevelContainerRep] mainViewController] selectionSetTimeSliceType];	
			[[[fObjRep getTopLevelContainerRep] mainViewController] addObjectToSelection:fObjRep];
			[[[fObjRep getTopLevelContainerRep] mainViewController] setSelectionStartTime:globalTime];
			
			// "HACK"
			// if the tracker is initialized with a different position than it is started with. set the end
			// time to the time it is started with and the start time with the time it has been initialized with.
			[[[fObjRep getTopLevelContainerRep] mainViewController] setSelectionEndxTime:[[[fObjRep getTopLevelContainerRep] mainViewController] pixelToArmeTime:where.x]];
		}
		fOriginAlreadySet = YES;
	}
// WARNING: fObjRep CAN BE TOPLEVELCONTAINER
	[fObjRep setNeedsDisplay];
	[[[fObjRep representedObject] parentObject] moveChildToTopOfStack:[fObjRep representedObject]];
	[[[fObjRep representedObject] parentObject] setRepresentationsNeedDisplay];

	//update the view in the Viewer
	TArmeDoc* doc = [fObjRep getDoc];
	doc->SetDoesUpdateMovieArmeTime(false);
	SetMovieArmeTime(doc->fMasterMovie->GetQTMovie(), doc, globalTime);

	[mainImageController->fLeftRightCarretCursor set];
	
	[[MITimePaletteController sharedTimePaletteController] setFeedbackType:MITimePaletteSelectionFeedback];
	[[MITimePaletteController sharedTimePaletteController] setFeedbackObject:[[fInView mainViewController] selection]];
	[[MITimePaletteController sharedTimePaletteController] getValuesFromMainView];
}

- (void)stop:(NSPoint)where withModifiers:(unsigned int)modifiers
{
//	NSPoint localWhere = [fObjRep convertPointFromTLCCoords:where];

	//update the view in the Viewer
	
	[[[fObjRep representedObject] parentObject] moveChildToTopOfStack:[fObjRep representedObject]];
	[[[fObjRep representedObject] parentObject] setRepresentationsNeedDisplay];
	
	TArmeDoc* doc = [fObjRep getDoc];
	doc->SetDoesUpdateMovieArmeTime(true);
	SetMovieArmeTime(doc->fMasterMovie->GetQTMovie(), doc, doc->GetPlayPos());
	
	[[NSCursor arrowCursor] set];
}

- (void)moved:(NSPoint)where withModifiers:(unsigned int)modifiers
{
	if ( !fHasMoved )
		return;
		
	NSPoint localWhere = [fObjRep convertPointFromTLCCoords:where];
	ArmeTime globalEndTime = [[fInView mainViewController] pixelToArmeTime:where.x];
	ArmeTime objectEndxTime = [[fObjRep representedObject] WSObject]->GetAbsoluteEndxTime();
	ArmeTime objectStartTime = [[fObjRep representedObject] WSObject]->GetAbsoluteStartTime();
	
	float pixDelta;
	globalEndTime = [[[fInView mainViewController] snapper] snapTime:globalEndTime pixDelta:&pixDelta modifiers:[[NSApp currentEvent] modifierFlags]];
	globalEndTime = [[fObjRep representedObject] WSObject]->GranulateTime(globalEndTime);
	
	if(globalEndTime > objectEndxTime)
		globalEndTime = objectEndxTime;
	if(globalEndTime < objectStartTime)
		globalEndTime = objectStartTime;
	
	[[fInView mainViewController] setSelectionEndxTime:globalEndTime];
	[fObjRep setNeedsDisplay];
	
	//update the view in the Viewer
	TArmeDoc* doc = [fObjRep getDoc];
	SetMovieArmeTime(doc->fMasterMovie->GetQTMovie(), doc, globalEndTime);
	
	[[MITimePaletteController sharedTimePaletteController] getValuesFromMainView];

}

- (void)draw:(NSRect)rect
{
}

@end
