/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
 
#import "MITimeStretchTracker.h"
#import "MITimeStretchCommand.h"
#import "MITopLevelContainerRep.h"
#import "MIAudioClipRep.h"
#import "MIObject.h"
#import "MIMontageDocument.h"
#import "MIApplicationController.h"
#import "MIMainViewController.h"
#import "MIVideoClipRep.h"
#import "MISingleMoveCommand.h"
#import "MITimePaletteController.h"
#import "MIImageController.h"
#import "MIFreezeCommand.h"
#import "MIPreferences.h"
#import "MontageVerbose.h"


#ifndef TArmeWSObj_h
#include "TArmeWSObj.h"
#endif //TArmeWSObj_h

#ifndef TArmeDoc_h
#include "TArmeDoc.h"
#endif //TArmeDoc_h


@implementation MITimeStretchTracker

- (void)start:(NSPoint)where withModifiers:(unsigned int)modifiers
{
    NSString *str;
    
    [super start:where withModifiers:modifiers];

    /* assert that fResizedObject is a MIAudioClipRep*/
    str = NSStringFromClass( [fResizedObject class] ) ;
    fAudioClip = [str isEqualToString:@"MIAudioClipRep"];
    fVideoClip = [str isEqualToString:@"MIVideoClipRep"];
	fTextClip = [str isEqualToString:@"MITextClipRep"];
    NSAssert1(fAudioClip or fVideoClip or fTextClip, @"Cannot time strech object of class %@", str ) ;

	NSRect frame = [fResizedObject frame];

    //force an update of the cached image

    if (fAudioClip)
    {
        // now, we are going to ask a new image that at least contains these
        // two start times
        [((MIAudioClipRep*)fResizedObject) 
            updateCachedImageFrom:0 
            to:GET_VALUE([[fResizedObject representedObject] getLength])
            wPixels:NSWidth(frame) > 800 ? 800 : NSWidth(frame) 
            hPixels:NSHeight(frame)];
    
        // now disable other cached image updates
        [((MIAudioClipRep*)fResizedObject) setEnableCachedImageUpdates:NO];
    }
	
/*  
	
	$$$SAIBOT WAS HERE: old code from marc follows
	
    float clipStart = NSMinX(frame);
    float clipEndx = NSMaxX(frame);

    // wsObj is the Arme object of the MIObject we represent
    TArmeWSObj* wsObj = [[fResizedObject representedObject] WSObject];

    // length is our length in frames
    SInt64 armeTotalLength = GET_VAL(wsObj->GetDuration());

    // we are going to use a proportional rule to find
    // the start time and time between which we need to
    // draw. In our coords,
    // pixel ourFrameInDrawCoords.origin.x is arme time 0
    // pixel ourFrameInDrawCoords.origin.x+ourFrameInDrawCoords.size.width is arme time armeTotalLength
    // thus, this means
    double framesPerPixelInDrawCoords = (double)armeTotalLength/NSWidth(frame);

    // thus, the start and end time of the region to draw are :
    SInt64 drawStartTime = (SInt64)((clipStart - frame.origin.x)*framesPerPixelInDrawCoords);
    SInt64 drawEndxTime = (SInt64)((clipEndx - frame.origin.x)*framesPerPixelInDrawCoords);

    if (fAudioClip)
    {
        // now, we are going to ask a new image that at least contains these
        // two start times
        [((MIAudioClipRep*)fResizedObject) 
            updateCachedImageFrom:drawStartTime 
            to:drawEndxTime 
            wPixels:(clipEndx - clipStart) > 800 ? 800 : (clipEndx - clipStart) 
            hPixels:frame.size.height];
    
        // now disable other cached image updates
        [((MIAudioClipRep*)fResizedObject) setEnableCachedImageUpdates:NO];
    }
*/
	if ( fEdge1 == NSMaxXEdge )
		[mainImageController->fTimeStretchRightCursor set];
	else if ( fEdge1 == NSMinXEdge )
		[mainImageController->fTimeStretchLeftCursor set];
		

	[[[MIApplicationController sharedApplicationController] timePaletteController] setFeedbackType:MITimePaletteTrackerFeedback];
	[[[MIApplicationController sharedApplicationController] timePaletteController] setFeedbackObject:self];
	[[[MIApplicationController sharedApplicationController] timePaletteController] getValuesFromMainView];
}

- (void)moved:(NSPoint)where withModifiers:(unsigned int)modifiers 
{
/*
    NSRect		redrawRect;
    NSRect		oldRect;
	NSRect		newRect;
	MIRect		newFrame;

    oldRect = [fInView MIRectToNSRect:[self calculateNewFrame:YES modifiers:modifiers]];

    fLastPos = where;

	newFrame = [self calculateNewFrame:YES modifiers:modifiers];

	fStartTime = MIMinX(newFrame);
	fEndTime = MIMaxX(newFrame);
	
    newRect = [fInView MIRectToNSRect:newFrame];

    redrawRect = NSUnionRect(newRect,oldRect);
    redrawRect = NSInsetRect(redrawRect,-6,-8);
    [fInView setNeedsDisplayInRect:redrawRect];

    [fResizedObject setFrame:newRect];
*/

    fLastPos = where;
	MIRect	newModelFrame = [self calculateNewFrame:YES modifiers:modifiers];
	newModelFrame = [[[fResizedObject representedObject] parentObject] convertRectFromTLCCoords:newModelFrame];
	NSRect	newFrame = [fInView MIRectToNSRect:newModelFrame];

	fStartTime = MIMinX(newModelFrame);
	fEndTime = MIMaxX(newModelFrame);

	if(NSWidth(newFrame) < 0)
		newFrame.size.width = 0;

//	if ( NSWidth(newFrame) > 0 )
//	{
		[fResizedObject setNeedsDisplay];
//		[fResizedObject setFrame:newFrame];
		[fResizedObject setTempFrameWithoutRegeneratingImages:newFrame];
		[fResizedObject setNeedsDisplay];
//	}
//	else
//	{
//		[fResizedObject setNeedsDisplay];
//		[fResizedObject recreateFrame]; ////////////////
//		[fResizedObject setNeedsDisplay];
//	}
	
	[[[MIApplicationController sharedApplicationController] timePaletteController] getValuesFromMainView];
}

- (void)stop:(NSPoint)where withModifiers:(unsigned int)modifiers
{
    NSRect newFrame;
    MIRect	newModelFrame;
    fLastPos = where;
    
	newModelFrame = [self calculateNewFrame:NO modifiers:modifiers];
    newFrame = [fInView MIRectToNSRect:newModelFrame];

	if (fAudioClip)
	{
		if(newFrame.size.width <= 0)
		{
			NSRunAlertPanel(@"Error",@"You cannot stretch an object to an empty size",@"OK",nil,nil);
			// restore audio clip drawing
			[((MIAudioClipRep*)fResizedObject) setEnableCachedImageUpdates:YES];
			[fResizedObject recreateFrame]; ////////////////
			[fResizedObject setNeedsDisplay];
			[[NSCursor arrowCursor] set];
			return;
		}
	}
	if (fVideoClip)
	{
		ArmeTime newDuration = newModelFrame.size.duration;
		[[fResizedObject representedObject] WSObject]->GetDoc()->RoundArmeTimeToNearestVideoFrame(newDuration);
		if (newDuration <= ARME_TIME_ZERO)
		{
			NSRunAlertPanel(@"Error",@"You cannot stretch a video clip to less than one frame",@"OK",nil,nil);

			[fResizedObject recreateFrame]; ////////////////
			[fResizedObject setNeedsDisplay];
			[[NSCursor arrowCursor] set];
			return;
		}
	}

	/* $$$STEPHAN:
		Alain, please make your stuff here for it to work with Video, your code is commented below, please make use
		of the commands! don't use direct coord setters in here, rather, do it in the command.
	*/
	if ( fAudioClip || fVideoClip || fTextClip )
	{
//		MIMainViewController *mainViewController = [[fResizedObject getTopLevelContainerRep]  mainViewController];
		
		ArmeTime oldDuration = [[fResizedObject representedObject] WSObject]->GetDuration();
		ArmeTime newDuration = MIWidth(newModelFrame);
		
		if(newDuration != oldDuration)
		{	
			bool bRender = false;
		
			if (fVideoClip && (GET_VALUE(newDuration) < GET_VALUE(oldDuration)/2))
			{
				int result = NSRunAlertPanel(nil, @"Stretching a video clip to less than half its original size can make the playback irregular.  Clicking the Render button below will give you a much better result but could take a long time to process depending on the size of the clip.\n ", @"Render", @"Don't Render", @"Cancel");
				if (result == NSAlertDefaultReturn)
				{
					bRender = true;
				}
				else if (result == NSAlertOtherReturn )
				{
					[fResizedObject recreateFrame]; ////////////////
					[fResizedObject setNeedsDisplay];
					[[NSCursor arrowCursor] set];
					return;
				}
			}
			MITimeStretchCommand *timeStretchCommand = [[MITimeStretchCommand alloc] initWithSingleClipRep:fResizedObject newDuration:newDuration];
			VUSERCMD1("TimeStretch, new duration = %d", (int)(newDuration.data));
			VUSERCMD_PRINT_OBJECT([fResizedObject representedObject]);
		
			MISingleMoveCommand *moveCommand = [[MISingleMoveCommand alloc] initWithSelection:nil];
			[moveCommand setObject:[fResizedObject representedObject]];
			[moveCommand setNewOrigin:newModelFrame.origin];
		
			[timeStretchCommand addCommand:moveCommand];
			[moveCommand release];
			if (bRender)
			{
				MIFreezeLowLevelCommand* renderCommand = [[[MIFreezeLowLevelCommand alloc] initWithObjectToFreeze:[fResizedObject representedObject]] autorelease];
				[timeStretchCommand addCommand:renderCommand];
			}
			
			if ([timeStretchCommand displaySetupDialog])
				[[[fResizedObject representedObject] montageDoc] doCmd:timeStretchCommand];
			
			[timeStretchCommand release];
		}
		
		if (fAudioClip)
		{
			[((MIAudioClipRep*)fResizedObject) setEnableCachedImageUpdates:YES];
			[[((MIAudioClipRep*)fResizedObject) representedObject] invalidateRepresentationsCachedImages];
			[[((MIAudioClipRep*)fResizedObject) representedObject] invalidateRep];
			[[((MIAudioClipRep*)fResizedObject) representedObject] setRepresentationsNeedDisplay];
		}
	}
	/*
		else if (fVideoClip)
		{
			[((MIVideoClipRep*)fResizedObject) setEffectiveDuration:MIWidth(newModelFrame)];
			
			// Why do we set newModelFrame, the command is supposed to be doing it already.
			// Answer : the translation is now done with a subcommand for the audio clip
			[[fResizedObject representedObject] setModelFrame:newModelFrame];
			[[fResizedObject representedObject] recreateFrame];
	
			NSLog(@" ***** CREATE A COMMAND DAMNIT **** ");
			
			[[[fResizedObject representedObject] parentObject] resizeFrameToFullyIncludeChildren];
			[[[fResizedObject representedObject] parentObject] resizeFrameToFullyIncludeChild:[fResizedObject representedObject]];
			[[fResizedObject representedObject] setRepresentationsNeedDisplay];
			[[NSCursor arrowCursor] set];
	
			[[[fResizedObject getTopLevelContainerRep] window] invalidateCursorRectsForView:[fResizedObject getTopLevelContainerRep]];
	
			[[MIApplicationController sharedApplicationController] modelChangedForDocument:[[fInView mainViewController] document]];
			
		}
	*/
	[[NSCursor arrowCursor] set];
	if ( [[MIPreferences sharedPrefs] gridShowMask] & kPrefsGridShowVObjLines ||
		 [[MIPreferences sharedPrefs] gridShowMask] & kPrefsGridShowHObjLines )
		[[MIApplicationController sharedApplicationController] redrawAllMainWindows];
}

- (void)draw:(NSRect)rect 
{
}

- (void)drawCachedImage:(NSImage*)cachedImage inFrame:(NSRect)frameInDrawCoords clippedTo:(NSRect)clippingRect
{
	NSSize size = [cachedImage size] ;
	NSRect frame = [fResizedObject frame] ;
	double ratio = frame.size.width / clippingRect.size.width ;
	NSRect sourceRect = NSMakeRect( clippingRect.origin.x != frameInDrawCoords.origin.x ? size.width - (size.width / ratio) : 0, 0, size.width / ratio, size.height );

	[cachedImage drawInRect:clippingRect fromRect:sourceRect operation:NSCompositeCopy fraction:1.0];
}


@end
