/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
#import "MITrimTracker.h"
#import "MITimeStretchCommand.h"
#import "MITopLevelContainerRep.h"
#import "MIAudioClipRep.h"
#import "MIVideoClipRep.h"
#import "MIObject.h"
#import "MIMontageDocument.h"
#import "MIApplicationController.h"
#import "MIMainViewController.h"
#import "MITimePaletteController.h"
#import "MITrimCommand.h"
#import "MIRuler.h"
#import "MIImageController.h"
#import "MIPreferences.h"
#import "MontageVerbose.h"

#ifndef TArmeWSObj_h
#include "TArmeWSObj.h"
#endif //TArmeWSObj_h

#undef VER_DEF_TOPIC
#define VER_DEF_TOPIC MITrimTracker

@implementation MITrimTracker

- (void)start:(NSPoint)where withModifiers:(unsigned int)modifiers
{
    NSString *str;
    
    [super start:where withModifiers:modifiers];

    /* assert that fResizedObject is a MIAudioClipRep*/
    str = NSStringFromClass( [fResizedObject class] ) ;
    fAudioClip = [str isEqualToString:@"MIAudioClipRep"];
    fVideoClip = [str isEqualToString:@"MIVideoClipRep"];
	fTextClip = [str isEqualToString:@"MITextClipRep"];
    NSAssert1(fAudioClip or fVideoClip or fTextClip, @"Cannot trim object of class %@", str ) ;

	/* force an update of the cached image*/
//	fOriginalFrame = [fResizedObject frame] ;

	// wsObj is the Arme object of the MIObject we represent
	TArmeWSObj* wsObj = [[fResizedObject representedObject] WSObject];

	// length is our length in frames
	//fOriginalLength = wsObj->GetDuration();

// DO NOT DO THIS!
//	MIWSSelection *sel = [[MIWSSelection alloc] initEmpty];
	
//	MIWSSelection *sel = [[fInView mainViewController] selection];
	
//	[sel addObject:fResizedObject];
//	fTrimCommand = [[MITrimCommand alloc] initWithSelection:sel];
	fTrimCommand = [[MITrimCommand alloc] initWithObject:[fResizedObject representedObject]];
	
	// compute the min and max values the object limits can reach
	GET_VAL(fMinimalStartTime) = GET_VAL(wsObj->GetAbsoluteStartTime()) - GET_VAL(wsObj->GetStartTrim());
	GET_VAL(fMaximalStartTime) = GET_VAL(wsObj->GetAbsoluteEndxTime())-100; // can't be smaller than 100 frames
	
	GET_VAL(fMinimalEndxTime) = GET_VAL(wsObj->GetAbsoluteStartTime())+100;
	GET_VAL(fMaximalEndxTime) = GET_VAL(wsObj->GetAbsoluteEndxTime())+ GET_VAL(wsObj->GetEndTrim()); // can't be smaller than 100 frames
		

	if ( fEdge1 == NSMaxXEdge )
		[mainImageController->fTrimRightCursor set];
	else if ( fEdge1 == NSMinXEdge )
		[mainImageController->fTrimLeftCursor set];

	[[[MIApplicationController sharedApplicationController] timePaletteController] setFeedbackType:MITimePaletteTrackerFeedback];
	[[[MIApplicationController sharedApplicationController] timePaletteController] setFeedbackObject:self];
	[[[MIApplicationController sharedApplicationController] timePaletteController] getValuesFromMainView];
}

- (void)moved:(NSPoint)where withModifiers:(unsigned int)modifiers
{
    NSRect		oldRect;
    
    oldRect = [fInView MIRectToNSRect:[self calculateNewFrame:YES modifiers:modifiers]];
    [fInView setNeedsDisplayInRect:NSInsetRect(oldRect,-2, -2)];
	fLastPos = where;

	[[[fInView mainViewController] ruler] setNeedsDisplayInRect:NSMakeRect( NSMinX(fRedrawRect) - 1, 0, 3, 20)];
	[[[fInView mainViewController] ruler] setNeedsDisplayInRect:NSMakeRect( NSMaxX(fRedrawRect) - 1, 0, 3, 20)];

	fCurrentModelFrame = [self calculateNewFrame:YES modifiers:modifiers];
	
	// now we need to clip the fCurrentModelFrame 
	// according to the trimming values
	if(fCurrentModelFrame.origin.time < fMinimalStartTime)
	{
		ArmeTime maxTime = MIMaxX(fCurrentModelFrame);
		
		vernl(6,"fCurrentModelFrame.origin.time (" << fCurrentModelFrame.origin.time << ") < fMinimalStartTime (" << fMinimalStartTime);
		fCurrentModelFrame.origin.time = fMinimalStartTime;
		GET_VALUE(fCurrentModelFrame.size.duration) = GET_VALUE(maxTime) - GET_VALUE(MIMinX(fCurrentModelFrame));
		
	}
	else if(fCurrentModelFrame.origin.time > fMaximalStartTime)
	{
		ArmeTime maxTime = MIMaxX(fCurrentModelFrame);

		vernl(6,"fCurrentModelFrame.origin.time (" << fCurrentModelFrame.origin.time << ") > fMaximalStartTime (" << fMaximalStartTime);
		fCurrentModelFrame.origin.time = fMaximalStartTime;

		GET_VALUE(fCurrentModelFrame.size.duration) = GET_VALUE(maxTime) - GET_VALUE(MIMinX(fCurrentModelFrame));
	}
	else if(fCurrentModelFrame.origin.time + fCurrentModelFrame.size.duration < fMinimalEndxTime)
	{
		vernl(6,"fCurrentModelFrame.origin.time + fCurrentModelFrame.size.duration (" << fCurrentModelFrame.origin.time + fCurrentModelFrame.size.duration << ") < fMinimalEndxTime (" << fMinimalEndxTime);
		fCurrentModelFrame.size.duration = fMinimalEndxTime - fCurrentModelFrame.origin.time;
	}
	else if(fCurrentModelFrame.origin.time + fCurrentModelFrame.size.duration > fMaximalEndxTime)
	{
		vernl(6,"fCurrentModelFrame.origin.time + fCurrentModelFrame.size.duration (" << fCurrentModelFrame.origin.time + fCurrentModelFrame.size.duration << ") > fMaximalEndxTime (" << fMaximalEndxTime);
		fCurrentModelFrame.size.duration = fMaximalEndxTime - fCurrentModelFrame.origin.time;
	}
			
    fRedrawRect = [fInView MIRectToNSRect:fCurrentModelFrame];

	fStartTime = MIMinX(fCurrentModelFrame);
	fEndTime = MIMaxX(fCurrentModelFrame);

	[fInView setNeedsDisplayInRect:NSInsetRect(fRedrawRect,-2,-2)];

	[[[fInView mainViewController] ruler] setNeedsDisplayInRect:NSMakeRect( NSMinX(fRedrawRect) - 1, 0, 3, 20)];
	[[[fInView mainViewController] ruler] setNeedsDisplayInRect:NSMakeRect( NSMaxX(fRedrawRect) - 1, 0, 3, 20)];
	[[[MIApplicationController sharedApplicationController] timePaletteController] getValuesFromMainView];
/*

	// if the trimming reduces the size, we must first invalidate the previous pos of the object
	[[fResizedObject representedObject] setRepresentationsNeedDisplay];

	TArmeWSObj* wsObj = [[fResizedObject representedObject] WSObject];

	SInt64 oldStartTrim = GET_VAL(wsObj->GetStartTrim());
	vvernl(6,oldStartTrim);
	SInt64 newAbsStart = GET_VAL(fStartTime);
	vvernl(6,newAbsStart);
	SInt64 oldAbsStart = GET_VAL(wsObj->GetAbsoluteStartTime());
	vvernl(6,oldAbsStart);
	// if the new abs x is bigger than before, it means the start trim INCREASES.
	SInt64 newStartTrim = oldStartTrim + (newAbsStart - oldAbsStart);
	vvernl(6,newStartTrim);
	ZzAssert(newStartTrim >= 0);
	[fTrimCommand setStartTrim:_ArmeTime(newStartTrim)];
	
	// the trim command does what it has to so the the end is preserved when the
	// start trim changes. so the we can check for end trim now
	SInt64 oldEndTrim = GET_VAL(wsObj->GetEndTrim());
	vvernl(6,oldEndTrim);
	SInt64 newAbsEndx = GET_VAL(fEndTime);
	vvernl(6,newAbsEndx);
	SInt64 oldAbsEndx = GET_VAL(wsObj->GetAbsoluteEndxTime());
	vvernl(6,oldAbsEndx);
	// if the new abs end is bigger than before, it means the end trim DECREASES.
	SInt64 newEndTrim = oldEndTrim - (newAbsEndx - oldAbsEndx);
	vvernl(6,newEndTrim);
	ZzAssert(newEndTrim >= 0);
	[fTrimCommand setEndTrim:_ArmeTime(newEndTrim)];
*/
}

- (void)stop:(NSPoint)where withModifiers:(unsigned int)modifiers
{
	ArmeTime originalStartTime = [[fResizedObject representedObject] WSObject]->GetAbsoluteStartTime();
	ArmeTime newStartTime = fCurrentModelFrame.origin.time;
	ArmeTime originalStartTrim = [[fResizedObject representedObject] WSObject]->GetStartTrim();
	// if the new start is bigger, it means the trimming gets bigger
	ArmeTime newStartTrim = originalStartTrim + newStartTime - originalStartTime;

	ArmeTime originalEndxTime = [[fResizedObject representedObject] WSObject]->GetAbsoluteEndxTime();
	ArmeTime newEndxTime = fCurrentModelFrame.origin.time + fCurrentModelFrame.size.duration;
	ArmeTime originalEndTrim = [[fResizedObject representedObject] WSObject]->GetEndTrim();
	// if the new end is smaller, it means the trimming gets bigger
	ArmeTime newEndTrim = originalEndTrim + originalEndxTime - newEndxTime;
	
	[fTrimCommand setStartTrim:newStartTrim];
	[fTrimCommand setEndTrim:newEndTrim];

	VUSERCMD2("Trim, startTrim = %d, endTrim = %d", (int)(newStartTrim.data), (int)(newEndTrim.data));
	VUSERCMD_PRINT_OBJECT([fResizedObject representedObject]);
	
	// since moved is always called before stop, we don't do nothing here but check that something actually moved.
	//	

	if ( fHasMoved && ((originalStartTrim != newStartTrim) || (originalEndTrim != newEndTrim)))
	{
		if ([fTrimCommand displaySetupDialog])
			[[[fResizedObject representedObject] montageDoc] doCmd:fTrimCommand];
	}

	// release the command anyway, because it has either been added to the undo stack or should disappear
	[fTrimCommand release];
	fTrimCommand = nil;
	[[NSCursor arrowCursor] set];
	if ( [[MIPreferences sharedPrefs] gridShowMask] & kPrefsGridShowVObjLines ||
		 [[MIPreferences sharedPrefs] gridShowMask] & kPrefsGridShowHObjLines )
		[[MIApplicationController sharedApplicationController] redrawAllMainWindows];
}

- (void)draw:(NSRect)rect 
{
	[[NSColor colorWithCalibratedRed:0.0 green:0.0 blue:0.0 alpha:1.0] set];
	[NSBezierPath strokeRect:[fInView MIRectToNSRect:fCurrentModelFrame]];
}

/*- (void)drawCachedImage:(NSImage*)cachedImage inFrame:(NSRect)frameInDrawCoords clippedTo:(NSRect)clippingRect
{
	NSSize size = [cachedImage size] ;
	NSRect frame = [fResizedObject frame] ;
	double ratio = fOriginalFrame.size.width / clippingRect.size.width ;
	NSRect sourceRect;

	if ( fOriginalFrame.origin.x == frame.origin.x )
	{
		// user is pulling on the left edge
	double ratio2 = ( clippingRect.origin.x - fOriginalFrame.origin.x ) / fOriginalFrame.size.width ;
	sourceRect = NSMakeRect(size.width * ratio2, 0, size.width / ratio, size.height );
	}
	else
	{
		// user is pulling on the right edge
	double ratio2 = fOriginalFrame.size.width / frame.size.width ;
	sourceRect = NSMakeRect(size.width - (size.width / ratio2), 0, size.width / ratio, size.height );
	}

	[[fResizedObject backgroundColor] set];
	[NSBezierPath fillRect:clippingRect];

	SInt32 waveformtotal = (SInt32)clippingRect.size.height - 8;
	SInt32 waveformleft = waveformtotal >> 1; // divide by 2
	SInt32 waveformright = waveformtotal - waveformleft;

	// setup the destination of separator lines
	float blankbot = (SInt32)NSMinY(clippingRect) + 0.5;
	float rsepbot  = blankbot + 1;
	float rwavebot = rsepbot + 1;
	float rseptop  = rwavebot + waveformright;
	//SInt32 rwavetop = rseptop;
	float blankmid1 = rseptop + 1;
	float blankmid2 = blankmid1 + 1;
	float lsepbot  = blankmid2 + 1;
	float lwavebot = lsepbot + 1;
	float lseptop  = lwavebot + waveformleft;
	//SInt32 lwavetop = lseptop;
	//SInt32 blanktop = lseptop + 1;

	[[NSColor colorWithCalibratedRed:0.5 green:0.5 blue:0.5 alpha:1.0] set];

	[NSBezierPath strokeLineFromPoint:NSMakePoint( NSMinX(clippingRect), rsepbot ) toPoint:NSMakePoint( NSMaxX(clippingRect), rsepbot )];
	[NSBezierPath strokeLineFromPoint:NSMakePoint( NSMinX(clippingRect), rseptop ) toPoint:NSMakePoint( NSMaxX(clippingRect), rseptop )];
	[NSBezierPath strokeLineFromPoint:NSMakePoint( NSMinX(clippingRect), lsepbot ) toPoint:NSMakePoint( NSMaxX(clippingRect), lsepbot )];
	[NSBezierPath strokeLineFromPoint:NSMakePoint( NSMinX(clippingRect), lseptop ) toPoint:NSMakePoint( NSMaxX(clippingRect), lseptop )];

	[cachedImage drawInRect:clippingRect fromRect:sourceRect operation:NSCompositeCopy fraction:1.0];
}*/

@end
