/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#include "cSpline.h"

#define GET_VALUE(a) (a.data)

cSplineKnot::cSplineKnot( ArmeTime t, float value) : fTime(t), fValue(value)
{
	fNext = NULL;
	fPrevious = NULL;
	for ( int i = 0; i < 4; i++ )
		fSplineCoeff[i] = .0;
	fInterpolationType = cSplineNoInterpolation;
}

cSplineKnot::cSplineKnot()
{
	fNext = NULL;
	fPrevious = NULL;
	fValue = 0.0f;
	GET_VALUE(fTime) = 0;
	for ( int i = 0; i < 4; i++ )
		fSplineCoeff[i] = .0;
	fInterpolationType = cSplineNoInterpolation;
}

cSplineInterpolationType cSplineKnot::interpolationType()
{
	return fInterpolationType;
}

void cSplineKnot::setInterpolationType( cSplineInterpolationType type )
{
	fInterpolationType = type;
}


cSplineKnot* cSplineKnot::nextKnot()
{
	return fNext;
}

cSplineKnot* cSplineKnot::previousKnot()
{
	return fPrevious;
}

void cSplineKnot::setNextKnot( cSplineKnot *next )
{
	fNext = next;
}

void cSplineKnot::setPreviousKnot( cSplineKnot *prev )
{
	fPrevious = prev;
}

float cSplineKnot::value()
{
	return fValue;
}

ArmeTime cSplineKnot::time()
{
	return fTime;
}

void cSplineKnot::setValue( float value )
{
	fValue = value;
}

void cSplineKnot::setTime( ArmeTime time )
{
	fTime = time;
}

void cSplineKnot::setCoefficients( float a, float b, float c, float d )
{
	fSplineCoeff[0] = a;
	fSplineCoeff[1] = b;
	fSplineCoeff[2] = c;
	fSplineCoeff[3] = d;
	
//	printf("XX-- a:%f b:%f c:%f d:%f\n", a,b,c,d);
}

float* cSplineKnot::getCoefficients()
{
	return fSplineCoeff;
}









cSpline::cSpline()
{
	fCoeffsCalculated = FALSE;
	fFirstKnot = NULL;
	fEndCondition = cSplineNaturalEnd;
	fCount = 0;
}

cSpline::~cSpline()
{
	cSplineKnot *knot = fFirstKnot;
	cSplineKnot	*nextKnot;

	do
	{
		nextKnot = knot->nextKnot();
		delete knot;
		knot = nextKnot;
	} while ( knot );
}

cSplineKnot* cSpline::firstKnot()
{
	return fFirstKnot;
}


void cSpline::addKnot( cSplineKnot *newKnot, cSplineKnot *previousKnot )
{
	fCoeffsCalculated = FALSE;
	fCount++;
	
	if ( previousKnot )
	{
		newKnot->setNextKnot( previousKnot->nextKnot() );
		newKnot->setPreviousKnot( previousKnot );
		previousKnot->setNextKnot( newKnot );
		if ( newKnot->nextKnot() )
			newKnot->nextKnot()->setPreviousKnot( newKnot);
			
	}
	else
	{
		if ( fFirstKnot )
		{
			newKnot->setNextKnot( fFirstKnot );
			fFirstKnot->setPreviousKnot( newKnot );
			fFirstKnot = newKnot;
		}
		else
			fFirstKnot = newKnot;
	}
}

void cSpline::addKnot( cSplineKnot* newKnot)
{
	// find the previous knot and give it to the other addKnot function
	cSplineKnot	*aKnot = fFirstKnot;
	cSplineKnot *lastKnot = NULL;
	
	while ( aKnot && aKnot->time() < newKnot->time() )
	{
		lastKnot = aKnot;
		aKnot = aKnot->nextKnot();
	}
	
	addKnot( newKnot, lastKnot );
}


void cSpline::removeKnot( cSplineKnot *delKnot )
{
	if ( !delKnot )
		return;
		
	fCoeffsCalculated = FALSE;
	fCount--;
	
	if ( delKnot == fFirstKnot )
	{
		fFirstKnot = fFirstKnot->nextKnot();
		fFirstKnot->setPreviousKnot( NULL );
	}
	else
	{
		if ( delKnot->nextKnot() )
			delKnot->nextKnot()->setPreviousKnot( delKnot->previousKnot() );
		if ( delKnot->previousKnot() )
			delKnot->previousKnot()->setNextKnot( delKnot->nextKnot() );
	}
	
	delete delKnot;
}

void cSpline::moveKnot( cSplineKnot *movedKnot, cSplineKnot *newPreviousKnot )
{
	fCount--;
	if ( movedKnot->nextKnot() )
		movedKnot->nextKnot()->setPreviousKnot( movedKnot->previousKnot() );
	if ( movedKnot->previousKnot() )
		movedKnot->previousKnot()->setNextKnot( movedKnot->nextKnot() );
	
	addKnot( movedKnot, newPreviousKnot );
}

void cSpline::changeKnotValues( cSplineKnot *knot, ArmeTime newTime, float newValue )
{
	cSplineKnot	*aKnot;
	cSplineKnot	*lastKnot;
	
	if ( !knot )
		return;

	if ( newTime == knot->time() && newValue == knot->value() )
		return;
	
	knot->setTime(newTime);
	knot->setValue(newValue);
	
	// check if the knot is still at the right position
	if ( knot->previousKnot() && knot->previousKnot()->time() > newTime )
	{
		aKnot = knot->previousKnot();
		while ( aKnot && aKnot->time() > newTime )
		{
			aKnot = aKnot->previousKnot();
		}
		
		// move the knot to the previous knot
		moveKnot( knot, aKnot);
	}
	else if ( knot->nextKnot() && knot->nextKnot()->time() < newTime )
	{
		aKnot = knot->nextKnot();
		lastKnot = aKnot;
		while ( aKnot && aKnot->time() < newTime )
		{
			lastKnot = aKnot;
			aKnot = aKnot->nextKnot();
		}
		
		// move the knot to the previous knot
		moveKnot( knot, lastKnot);
	}

	fCoeffsCalculated = FALSE;
}


cSplineKnot* cSpline::knotAround( ArmeTime t, float val, ArmeTime hTresh, float vTresh )
{
	ArmeTime	minT = t - hTresh;
	ArmeTime	maxT = t + hTresh;
	float		minV = val - vTresh;
	float		maxV = val + vTresh;
	cSplineKnot *knot = fFirstKnot;
	
	while ( knot && knot->time() < minT )
		knot = knot->nextKnot();
	
	if ( knot )
	{
		while ( knot && knot->time() < maxT )
		{
			if ( knot->time() <= maxT && knot->value() >= minV && knot->value() <= maxV )
				return knot;
			knot = knot->nextKnot();
		}
	}
	
	return NULL;
}

void cSpline::computeCoeffs()
{
	double 			prevDx, curDx; // delta x
	double 			prevDy, curDy; // one half of g
	double** 		solutionMatrix = new double*[fCount+1]; // intervals
	double*			solutionVector = new double[fCount+1];
	double*			x = new double[fCount+1];
	double*			y = new double[fCount+1];
	cSplineKnot		*knot;
	
	if ( fCount < 2 )
		return;

	knot = fFirstKnot;
	for ( int i = 0; i < fCount; i++ )
	{
		x[i] = GET_VALUE(knot->time());
		y[i] = knot->value();
		knot = knot->nextKnot();
	}
	
	for ( int k=0 ; k < fCount ; k++)
		solutionMatrix[k] = new double[4];
	--x;
	--y;
	
	prevDx = x[2] - x[1];
	prevDy = ((y[2]-y[1])/prevDx) * 6.0;
	for ( int i = 1; i <= fCount -2; ++i )
	{
		curDx = x[i+2] - x[i+1];
		curDy = (y[i+2] - y[i+1])/curDx * 6.0;
		
		//
		// the matrix looks like this
		// |  k1     h2      0     ...     0      0     0   |		|  c1  |		|  g1  |
		// |  h2     k2     h3     ...     0      0     0   |		|  c2  |		|  g2  |
		// |  0      h3     k3     ...     0      0     0   |		|  c3  |		|  g3  |
		//
		// where k[i] = 2 * (h[i] + h[i+1])
		// and h[i] = x[i] - x[i-1]
		// and g[i] = 3 * ( (y[i+1] - y[i])/h[i+1] - (y[i] - y[i-1])/h[i])
		
		
		solutionMatrix[i][0] = prevDx;
		solutionMatrix[i][1] = 2*(prevDx+curDx);
		solutionMatrix[i][2] = curDx;
		solutionMatrix[i][3] = curDy - prevDy;
		prevDx = curDx;
		prevDy = curDy;
	}
	
	int first = 2;
	int last = fCount - 2;
	
	switch( fEndCondition )
	{
		case cSplineNaturalEnd:
		case cSplineLinearEnd:
		case cSplineCubicEnd:
		case cSplineDerivativeEnd:
			// to do later
			// for now consider everything natural end
			break;
	}
	
	
	// solution of the tridiagonal system:
	for ( int i = first; i <= last; ++i )
	{
		solutionMatrix[i][0] = solutionMatrix[i][0]/solutionMatrix[i-1][1];
		solutionMatrix[i][1] = solutionMatrix[i][1] - solutionMatrix[i][0]*solutionMatrix[i-1][2];
		solutionMatrix[i][3] = solutionMatrix[i][3] - solutionMatrix[i][0]*solutionMatrix[i-1][3];
	}
	
	
	// back substituion
	solutionMatrix[last][3] = solutionMatrix[last][3]/solutionMatrix[last][1];
	for ( int i = last-1; i >= first-1; --i )
		solutionMatrix[i][3] = (solutionMatrix[i][3] - solutionMatrix[i][2]*solutionMatrix[i+1][3])/solutionMatrix[i][1];
	
	for ( int i = first-1; i <= last; ++i )
		solutionVector[i+1] = solutionMatrix[i][3];
		
	// special end condition
	switch( fEndCondition )
	{
		case cSplineNaturalEnd:
		case cSplineLinearEnd:
		case cSplineCubicEnd:
		case cSplineDerivativeEnd:
				// to do later
				// for now consider everything natural end
			solutionVector[1] = 0;
			solutionVector[fCount] = 0;
			break;
	}
	
	knot = fFirstKnot;
	double	dx;
	double	a, b, c, d;
	for ( int i=1; i <= fCount -1; ++i )
	{
		dx = x[i+1] - x[i];
		a = knot->value();
		b = ((knot->nextKnot()->value()-knot->value())/dx) - ((2.0*dx*solutionVector[i] + dx*solutionVector[i+1])/6.0);
		c = solutionVector[i]/2;
		d = (solutionVector[i+1] - solutionVector[i])/(6.0*dx);
		knot->setCoefficients(a,b,c,d);
		knot = knot->nextKnot();
	}
	if ( knot && !knot->nextKnot())
		knot->setCoefficients(knot->value(), 0, 0, 0);
	
	if ( solutionVector )
		delete [] solutionVector;
	for ( int k = 0; k < fCount; k++ )
		if ( solutionMatrix[k] )
			delete [] solutionMatrix[k];
	
	if ( solutionMatrix )
		delete [] solutionMatrix;
	
	fCoeffsCalculated = TRUE;
}

float cSpline::splineValue( const ArmeTime t, bool &isRecorded )
{
	cSplineKnot	*knot = fFirstKnot;
	cSplineKnot	*lastKnot = NULL;
	float		*coeffs;
	double		dx;
	double		u = GET_VALUE(t);
	double		result;
	
	if ( !fCoeffsCalculated )
		computeCoeffs();


	// find the right interval:
	while ( knot && GET_VALUE(knot->time()) <= u )
	{
		lastKnot = knot;
		knot = knot->nextKnot();
	}
	if ( !lastKnot )
		return 0;
	
	dx = u - GET_VALUE(lastKnot->time());
	
	coeffs = lastKnot->getCoefficients();
	
	switch( lastKnot->interpolationType() )
	{
		case cSplineNoInterpolation:
			result = coeffs[0];
			break;
		case cSplineLinearInterpolation:
			result = coeffs[0] + dx*coeffs[1];
			break;
		default:
		case cSplineCubicInterpolation:
			result = coeffs[0] + dx*(coeffs[1]+dx*(coeffs[2]+dx*coeffs[3]));
			break;
	}
	
	// cap the result in the {0,1} range
	result = (result < 0 ? 0 : ( result > 1 ? 1 : result ) );
	
	return result;
}


