/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/


#ifndef   AutoPan_h
#include "AutoPan.h"
#endif    AutoPan_h

#ifndef   DSPFunctions_h
#include "DSPFunctions.h"
#endif    DSPFunctions_h


void AutoPanPlugin::ProcessFramesAdd(SInt32 nbFrames)
{

    float* ibuf   = GetInputBuffer(0);
    SInt32 ichans = GetInputChannels(0);

    float* obuf   = GetOutputBuffer(0);
    SInt32 ochans = GetOutputChannels(0);
    
	// buffer pointers
    float *inL, *inR, *outL, *outR;
    float outputscale;
	
	// prepare for MONO or STEREO operation
    switch (ichans)
    {
        case 1:		inL = inR = ibuf; 
                    break;
        default:    inL = ibuf; 
                    inR = ibuf + 1; 
                    break;
    }
    switch (ochans)
    {
        case 1:		outL = outR = obuf; 
                    outputscale = 0.5f;
                    break;
        default:    outL = obuf; 
                    outR = obuf + 1; 
                    outputscale = 1.0f;
                    break;
    }


    float sr =GetFrameRate();
    float inv_sr = 1.0f / sr;

    
    // effects params
   float position =		arboSat(fParams.guiParams[kAutoPan_Position] / 100.0f);
   float depth =		satmin(fParams.guiParams[kAutoPan_Depth] / 100.0f, 0.0f);
   float rate =	 		2.0f * ArboPi * (fParams.guiParams[kAutoPan_Rate] * inv_sr);
   
   float c_position =	fParams.c_position;
   float c_depth = 		fParams.c_depth;
   float c_rate = 		fParams.c_rate;

   float p =			fParams.p;
   float q =			fParams.q;


   // intermediates
   int i;
   float x,y, xx,yy, pan;
   float pnext, qnext;
   const float half = .5;

   for (i=0; i<nbFrames; i++)
   {
        // READ INPUT 
		x  = *inL;
		xx = *inR;
        inL += ichans;
        inR += ichans;
                 
         pan = depth * p;
         pan = arboSat(pan + position);
         pan = half + half * pan;
         
         // APPLY PAN
         y = x * (1.0-pan);
         yy = xx * pan;
         
         // UPDATE OSC;
         pnext = arboSat(p + rate * q);
         qnext = arboSat(-p * rate + q);
         p = pnext;
         q = qnext;
         
		// WRITE OUTPUT
 		*outL += outputscale * y;
		*outR += outputscale * yy;
        outL += ochans;
        outR += ochans;
        
   }
   
    

   fParams.c_rate = c_rate;
   fParams.c_depth = c_depth;
   fParams.c_position = c_position;
   fParams.p = p;
   fParams.q = q;

}

AutoPanPluginClass::AutoPanPluginClass()
{



    // define parameter info on the stack
    // params: <name>, <unit>, <absMin>, <absMax>, <Min>, <Max>, <Default>
    ARPIParamInfo p[] = {
        {"Position (%)",		ARPIUnit_percent,	-100.0f, 100.0f,		-100.0f, 100.0f, 	   0.0f},
        {"Rate (Hz)",			ARPIUnit_Hz,		0.0f, 20.0f,			0.0f, 20.0f, 	   	   50.0f},
        {"Depth (%)",			ARPIUnit_percent,	0.0f, 100.0f,			-0.0f, 100.0f, 	   	   50.0f}
    };
    
    // copy info to data member
    assert(sizeof(p) == sizeof(fParamInfoTemplate));
   	memcpy(fParamInfoTemplate, p, sizeof(p));

	// pointer used by PRESET_xxx macros
	ARPIBlueWindowPreset* presetPtr = &fFactoryPreset[0];


	// AutoPan preset 1
	PRESET_NAME("Left Heavy");
	PRESET_PARAM(-50.000000, -100.000000, 100.000000, 0); //Position (%)
	PRESET_PARAM(2.000000, 0.000000, 20.000000, 1); //Rate (Hz)
	PRESET_PARAM(50.000000, 0.000000, 100.000000, 2); //Depth (%)
	PRESET_SOURCE(0.750000); //BW X-axis
	PRESET_SOURCE(0.500000); //BW Y-axis
	PRESET_SOURCE(1.000000); //BW Z-axis
	PRESET_NEXT;

	// AutoPan preset 2
	PRESET_NAME("Fast and Centered");
	PRESET_PARAM(0.000000, -100.000000, 100.000000, 0); //Position (%)
	PRESET_PARAM(10.000000, 2.000000, 20.000000, 1); //Rate (Hz)
	PRESET_PARAM(100.000000, 0.000000, 100.000000, 2); //Depth (%)
	PRESET_SOURCE(0.700000); //BW X-axis
	PRESET_SOURCE(1.000000); //BW Y-axis
	PRESET_SOURCE(1.000000); //BW Z-axis
	PRESET_NEXT;

	// AutoPan preset 3
	PRESET_NAME("Slow and Deep");
	PRESET_PARAM(0.000000, -100.000000, 100.000000, 0); //Position (%)
	PRESET_PARAM(0.100000, 0.100000, 1.000000, 1); //Rate (Hz)
	PRESET_PARAM(100.000000, 0.000000, 100.000000, 2); //Depth (%)
	PRESET_SOURCE(0.000000); //BW X-axis
	PRESET_SOURCE(1.000000); //BW Y-axis
	PRESET_SOURCE(1.000000); //BW Z-axis
	PRESET_NEXT;
	
	// AutoPan preset
	PRESET_NAME("Slow Right");
	PRESET_PARAM(50.000000, -100.000000, 100.000000, 0); //Position (%)
	PRESET_PARAM(0.100000, 0.100000, 2.000000, 1); //Rate (Hz)
	PRESET_PARAM(100.000000, 0.000000, 100.000000, 2); //Depth (%)
	PRESET_SOURCE(0.000000); //BW X-axis
	PRESET_SOURCE(1.000000); //BW Y-axis
	PRESET_SOURCE(1.000000); //BW Z-axis
	PRESET_NEXT;

	// AutoPan preset 5
	PRESET_NAME("Center Buzz");
	PRESET_PARAM(0.000000, -100.000000, 100.000000, 0); //Position (%)
	PRESET_PARAM(20.000000, 1.000000, 20.000000, 1); //Rate (Hz)
	PRESET_PARAM(50.000000, 0.000000, 100.000000, 2); //Depth (%)
	PRESET_SOURCE(1.000000); //BW X-axis
	PRESET_SOURCE(0.500000); //BW Y-axis
	PRESET_SOURCE(1.000000); //BW Z-axis
	PRESET_NEXT;
	
	// AutoPan preset 6
	PRESET_NAME("Position + Rate");
	PRESET_PARAM(2.105263, -100.000000, 100.000000, 1); //Position (%)
	PRESET_PARAM(0.924240, 0.200000, 5.000000, 2); //Rate (Hz)
	PRESET_PARAM(50.000000, 0.000000, 100.000000, 0); //Depth (%)
	PRESET_SOURCE(0.510526); //BW X-axis
	PRESET_SOURCE(0.475524); //BW Y-axis
	PRESET_SOURCE(1.000000); //BW Z-axis
	PRESET_NEXT;
}

SInt32 AutoPanPluginClass::GetNbFactoryPresets() const
{
	return kAutoPan_NbFactoryPresets;
}

const char* AutoPanPluginClass::GetFactoryPresetName(SInt32 preset) const 
{
	return fFactoryPreset[preset].GetName();
}

ARPIBlueWindowPreset* AutoPanPluginClass::CreatePresetFromFactoryPreset(SInt32 factoryPreset) const
{
	ARPIBlueWindowPreset* p = new ARPIBlueWindowPreset();
	*p = fFactoryPreset[factoryPreset];
	return p;
}


                AutoPanPlugin::AutoPanPlugin(const AutoPanPluginClass* theClass) : VARPIPlugin(theClass)
{
	// here we need to set the default values and initialize the plugin
    fParams.p = 1.0f;
    fParams.q = 0.0f;
    
	fParams.guiParams[kAutoPan_Rate] = 2.0f;
    fParams.guiParams[kAutoPan_Depth] = 50.0f;
    fParams.guiParams[kAutoPan_Position] = 0.0f;

}


                AutoPanPlugin::~AutoPanPlugin(){}

bool			AutoPanPlugin::SyncParam(SInt32 param)// OBSOLETE
{
	return true;
}	

bool			AutoPanPlugin::SetParamValue(SInt32 param, float val)
{
	assert(param >= 0 && param < kAutoPan_Params);
	fParams.guiParams[param] = val;
	SyncParam(param);
	return true;
}

float			AutoPanPlugin::GetParamValue(SInt32 param) const
{
	assert(param >= 0 && param < kAutoPan_Params);
	return fParams.guiParams[param];
}


bool			AutoPanPlugin::FormatChange()
{	
	// only for debugging purposes
	vertl(hyperPrismTom,5,"AutoPanPlugin::FormatChange()\n");
	return true;
}


const char*		AutoPanPluginClass::GetManufacturerName() const	{return "Arboretum Systems, Inc.";}
const char*		AutoPanPluginClass::GetPluginName()	const		{return "AutoPan";}
VARPIPlugin*	AutoPanPluginClass::CreatePlugin() const 				{return new AutoPanPlugin(this);}
SInt32			AutoPanPluginClass::GetNbParams() const 		{return kAutoPan_Params;}
void*			AutoPanPluginClass::CreateView(void* miobject) const 
                                                                {return NULL;}
void			AutoPanPluginClass::GetParamInfo(ARPIParamInfo* paramInfo, SInt32 param) const
{
	assert(param >= 0 && param < kAutoPan_Params);
	memcpy(paramInfo,&(fParamInfoTemplate[param]),sizeof(ARPIParamInfo));
}

AutoPanPluginClass::~AutoPanPluginClass()
{
}

//IMPLEMENT_PLUGIN_ARCHIVE_METHODS(AutoPanPlugin)
