/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/


#define AutoPan_h

#ifndef AutoPan_h
#include "AutoPan.h"
#endif AutoPan_h

#ifndef ARPI_h
#include "ARPI.h"
#endif //ARPI_h

class AutoPanPluginClass;

enum{
    kAutoPan_Position=0,
    kAutoPan_Rate,
    kAutoPan_Depth,

    kAutoPan_Params	
};

#define kAutoPan_Smooth 0.01
#define kAutoPan_NbFactoryPresets 6

struct AutoPanData
{
   // gui values (need to store them because the view needs to
   // be able to read them)
   float guiParams[kAutoPan_Params]; // pos, rate, depth

    // oscillator state variables
    float p;
    float q;
   
   // smoothing variables
   float c_position;
   float c_rate;
   float c_depth;
   
};



class AutoPanPlugin : public VARPIPlugin
{
public :
                    AutoPanPlugin(const AutoPanPluginClass* theClass);
					//DECLARE_PLUGIN_ARCHIVE_METHODS(AutoPanPlugin)
	virtual			~AutoPanPlugin();

public :
	virtual bool	FormatChange();
	virtual bool	SetParamValue(SInt32 param, float value);
	virtual float	GetParamValue(SInt32 param) const;
    virtual void 	ProcessFramesAdd(SInt32 nbFrames);
private :
	bool 			SyncParam(SInt32 param);
private :
	AutoPanData		fParams;
};



class AutoPanPluginClass : public VARPIPluginClassBase<AutoPanPluginClass>
{
public :
		AutoPanPluginClass();
	virtual ~AutoPanPluginClass();
public :
	virtual const char*		GetManufacturerName() const;
	virtual const char*		GetPluginName() const;
	virtual VARPIPlugin*	CreatePlugin() const;
	virtual void*			CreateView(void* miobject) const;
	virtual SInt32			GetNbParams() const;
	virtual void			GetParamInfo(ARPIParamInfo* paramInfo, SInt32 param) const;
	virtual SInt32						GetNbFactoryPresets() const;
	virtual const char*					GetFactoryPresetName(SInt32 preset) const;
	virtual ARPIBlueWindowPreset*		CreatePresetFromFactoryPreset(SInt32 factoryPreset) const;
private :
	ARPIParamInfo			fParamInfoTemplate[kAutoPan_Params];
	SInt32					fNbFactoryPresets;
	ARPIBlueWindowPreset	fFactoryPreset[kAutoPan_NbFactoryPresets];

};
