/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/



#ifndef   BandPass_h
#include "BandPass.h"
#endif    BandPass_h

#ifndef ZzVerbose_h
//#include "ZzVerbose.h"
#endif //ZzVerbose_h

void BandPassPlugin::ProcessFramesAdd(SInt32 nbFrames)
{

    float* ibuf   = GetInputBuffer(0);
    SInt32 ichans = GetInputChannels(0);

    float* obuf   = GetOutputBuffer(0);
    SInt32 ochans = GetOutputChannels(0);
    
	// buffer pointers
    float *inL, *inR, *outL, *outR;
    float outputscale;
	
	// prepare for MONO or STEREO operation
    switch (ichans)
    {
        case 1:		inL = inR = ibuf; 
                    break;
        default:    inL = ibuf; 
                    inR = ibuf + 1; 
                    break;
    }
    switch (ochans)
    {
        case 1:		outL = outR = obuf; 
                    outputscale = 0.5f;
                    break;
        default:    outL = obuf; 
                    outR = obuf + 1; 
                    outputscale = 1.0f;
                    break;
    }
    
    float sr = GetFrameRate();

    float x,y;

    // set the parameters of the filter
    filter.setBP(guiParams[kBandPass_Frequency] / sr, 
                pow(10.0f, guiParams[kBandPass_Quality]/20.0f));



    for (int i=0; i<nbFrames*2; i+=2)
    {	
        // READ INPUT 
        x  = *inL;
        y  = *inR;
        inL += ichans;
        inR += ichans;
                
        // filter
        filter.Bang2Smooth(x,y,x,y,kBandPass_Smooth);
                
        // WRITE OUTPUT
        *outL += outputscale * x;
        *outR += outputscale * y;
        outL += ochans;
        outR += ochans;
    }
}


BandPassPluginClass::BandPassPluginClass()
{

    // define parameter info on the stack
    ARPIParamInfo p[] = {
        {"Frequency (Hz)",	ARPIUnit_Hz,		0.0f, 20000.0f,	300.0f, 3000.0f, 	1000.0f},
        {"Q (dB)",			ARPIUnit_dB, 		0.0f, 50.0f, 		0.0f, 30.0f,		3.0f}
    };
    
    // copy info to data member
    assert(sizeof(p) == sizeof(fParamInfoTemplate));
   	memcpy(fParamInfoTemplate, p, sizeof(p));
	
		
	// pointer used by PRESET_xxx macros
	ARPIBlueWindowPreset* presetPtr = &fFactoryPreset[0];
	
	// BandPass preset 1
	PRESET_NAME("Wah Wah");
	PRESET_PARAM(300.000000, 300.000000, 5000.000000, 1); //Frequency
	PRESET_PARAM(30.000000, 0.000000, 30.000000, 2); //Q
	PRESET_SOURCE(0.000000); //BW X-axis
	PRESET_SOURCE(1.000000); //BW Y-axis
	PRESET_SOURCE(1.000000); //BW Z-axis
	PRESET_NEXT;
	
	// BandPass preset 2
	PRESET_NAME("Low End Kick");
	PRESET_PARAM(120.000000, 20.000000, 120.000000, 1); //Frequency
	PRESET_PARAM(3.000000, 0.000000, 20.000000, 2); //Q
	PRESET_SOURCE(0.000000); //BW X-axis
	PRESET_SOURCE(0.000000); //BW Y-axis
	PRESET_SOURCE(1.000000); //BW Z-axis
	PRESET_NEXT;
	
	// BandPass preset 3
	PRESET_NAME("Low End Ring");
	PRESET_PARAM(500.000000, 20.000000, 500.000000, 1); //Frequency
	PRESET_PARAM(0.000000, 0.000000, 50.000000, 2); //Q
	PRESET_SOURCE(1.000000); //BW X-axis
	PRESET_SOURCE(0.000000); //BW Y-axis
	PRESET_SOURCE(1.000000); //BW Z-axis
	PRESET_NEXT;
	
	// BandPass preset 4
	PRESET_NAME("High End Ring");
	PRESET_PARAM(5000.000000, 500.000000, 5000.000000, 1); //Frequency
	PRESET_PARAM(0.000000, 0.000000, 50.000000, 2); //Q
	PRESET_SOURCE(1.000000); //BW X-axis
	PRESET_SOURCE(0.000000); //BW Y-axis
	PRESET_SOURCE(1.000000); //BW Z-axis
	PRESET_NEXT;
	
	
	// BandPass preset 5
	PRESET_NAME("Dog Catcher");
	PRESET_PARAM(20000.000000, 5000.000000, 20000.000000, 1); //Frequency
	PRESET_PARAM(0.000000, 0.000000, 50.000000, 2); //Q
	PRESET_SOURCE(1.000000); //BW X-axis
	PRESET_SOURCE(0.000000); //BW Y-axis
	PRESET_SOURCE(1.000000); //BW Z-axis
	PRESET_NEXT;
}

SInt32 BandPassPluginClass::GetNbFactoryPresets() const
{
	return kBandPass_NbFactoryPresets;
}

const char* BandPassPluginClass::GetFactoryPresetName(SInt32 preset) const 
{
	return fFactoryPreset[preset].GetName();
}

ARPIBlueWindowPreset* BandPassPluginClass::CreatePresetFromFactoryPreset(SInt32 factoryPreset) const
{
	ARPIBlueWindowPreset* p = new ARPIBlueWindowPreset();
	*p = fFactoryPreset[factoryPreset];
	return p;
}







                BandPassPlugin::BandPassPlugin(const BandPassPluginClass* theClass) : VARPIPlugin(theClass)
{


	guiParams[kBandPass_Frequency] = 1000.0;
    guiParams[kBandPass_Quality] = 3.0;
    
}


                BandPassPlugin::~BandPassPlugin(){}

bool			BandPassPlugin::SetParamValue(SInt32 param, float val)
{
	assert(param >= 0 && param < kBandPass_Params);
	guiParams[param] = val;
	return true;
}

float			BandPassPlugin::GetParamValue(SInt32 param) const
{
	assert(param >= 0 && param < kBandPass_Params);
	return guiParams[param];
}


bool			BandPassPlugin::FormatChange()
{	
	// only for debugging purposes
	vertl(hyperPrismTom,5,"BandPassPlugin::FormatChange()\n");
	return true;
}


const char*		BandPassPluginClass::GetManufacturerName() const	{return "Arboretum Systems, Inc.";}
const char*		BandPassPluginClass::GetPluginName()	const		{return "BandPass";}
VARPIPlugin*	BandPassPluginClass::CreatePlugin() const 				{return new BandPassPlugin(this);}
SInt32			BandPassPluginClass::GetNbParams() const 		{return kBandPass_Params;}
void*			BandPassPluginClass::CreateView(void* miobject) const 
                                                                {return NULL;}
void			BandPassPluginClass::GetParamInfo(ARPIParamInfo* paramInfo, SInt32 param) const
{
	assert(param >= 0 && param < kBandPass_Params);
	memcpy(paramInfo,&(fParamInfoTemplate[param]),sizeof(ARPIParamInfo));
}

BandPassPluginClass::~BandPassPluginClass()
{
}

//IMPLEMENT_PLUGIN_ARCHIVE_METHODS(BandPassPlugin)
