/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/


#include "ABOTfilters.h"

#ifndef BandPass_h
#define BandPass_h

#ifndef ARPI_h
#include "ARPI.h"
#endif //ARPI_h

class BandPassPluginClass;

enum {
    kBandPass_Frequency=0,
    kBandPass_Quality,

    kBandPass_Params
};

#define kBandPass_NbFactoryPresets 5
#define kBandPass_Smooth 0.001f

class BandPassPlugin : public VARPIPlugin
{
public :
                    BandPassPlugin(const BandPassPluginClass* theClass);
					//DECLARE_PLUGIN_ARCHIVE_METHODS(BandPassPlugin)
	virtual			~BandPassPlugin();

public :
	virtual bool	FormatChange();
	virtual bool	SetParamValue(SInt32 param, float value);
	virtual float	GetParamValue(SInt32 param) const;
    virtual void 	ProcessFramesAdd(SInt32 nbFrames);
private :
	bool 			SyncParam(SInt32 param);
private :
   float guiParams[kBandPass_Params];
   ABOTfilterOrtho	filter;

};



class BandPassPluginClass : public VARPIPluginClassBase<BandPassPluginClass>
{
public :
		BandPassPluginClass();
	virtual ~BandPassPluginClass();
public :
	virtual const char*		GetManufacturerName() const;
	virtual const char*		GetPluginName() const;
	virtual VARPIPlugin*	CreatePlugin() const;
	virtual void*			CreateView(void* miobject) const;
	virtual SInt32			GetNbParams() const;
	virtual void			GetParamInfo(ARPIParamInfo* paramInfo, SInt32 param) const;
	virtual SInt32						GetNbFactoryPresets() const;
	virtual const char*					GetFactoryPresetName(SInt32 preset) const;
	virtual ARPIBlueWindowPreset*		CreatePresetFromFactoryPreset(SInt32 factoryPreset) const;
private :
	ARPIParamInfo			fParamInfoTemplate[kBandPass_Params];
	SInt32					fNbFactoryPresets;
	ARPIBlueWindowPreset	fFactoryPreset[kBandPass_NbFactoryPresets];
};

#endif