/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#define ZzCoreAudioStreamOperators_h

#ifndef ZzCoreAudioStreamOperators_h
#include "ZzCoreAudioStreamOperators.h"
#endif //ZzCoreAudioStreamOperators_h

#ifndef ZzString_h
#include "ZzString.h"
#endif //ZzString_h

#include <CoreAudio/CoreAudio.h>

ZzString getEnumString_AudioStreamBasicDescription_mFormatID(UInt32 formatID)
{
	switch(formatID)
	{
	case kAudioFormatLinearPCM : //			= 'lpcm',	//	linear PCM data, uses standard flags
		return "kAudioFormatLinearPCM";
		break;
	case kAudioFormatAC3 : //					= 'ac-3',	//	AC-3, has no flags
		return "kAudioFormatAC3";
		break;
	case kAudioFormat60958AC3 : //			= 'cac3',	//	AC-3 packaged for transport over an IEC 60958 compliant digital audio interface, uses standard flags
		return "kAudioFormat60958AC3";
		break;
/*	case kAudioFormatMPEG : //				= 'mpeg',	//	MPEG-1 or MPEG-2, has no flags
/*		return "kAudioFormatMPEG";
		break;*/
	case kAudioFormatAppleIMA4 : //			= 'ima4',	//	Apple's implementation of IMA 4:1 ADPCM encoding, has no flags
		return "kAudioFormatAppleIMA4";
		break;
	case kAudioFormatMPEG4AAC : //			= 'aac ',	//	MPEG-4 AAC, the flags field contains the MPEG-4 audio object type constant indicating the specific kind of data
		return "kAudioFormatMPEG4AAC";
		break;
	case kAudioFormatMPEG4CELP : //			= 'celp',	//	MPEG-4 CELP, the flags field contains the MPEG-4 audio object type constant indicating the specific kind of data
		return "kAudioFormatMPEG4CELP";
		break;
	case kAudioFormatMPEG4HVXC : //			= 'hvxc',	//	MPEG-4 HVXC, the flags field contains the MPEG-4 audio object type constant indicating the specific kind of data
		return "kAudioFormatMPEG4HVXC";
		break;
	case kAudioFormatMPEG4TwinVQ : //			= 'twvq'	//	MPEG-4 TwinVQ, the flags field contains the MPEG-4 audio object type constant indicating the specific kind of data
		return "kAudioFormatMPEG4TwinVQ";
		break;
	default :
		{
			char s[256];
			sprintf(s,"%p",(void*)formatID);
			return ZzString(s);
		}
		break;
	}
}
	
inline std::ostream& operator<<(std::ostream& s, const AudioStreamBasicDescription& streamDesc)
{
	s << "(mSampleRate = " << streamDesc.mSampleRate << 
		" mFormatID = " << getEnumString_AudioStreamBasicDescription_mFormatID(streamDesc.mFormatID) << 
		" mFormatFlags = " << streamDesc.mFormatFlags << 
		" mBytesPerPacket = " << streamDesc.mBytesPerPacket << 
		" mFramesPerPacket = " << streamDesc.mFramesPerPacket << 
		" mBytesPerFrame = " << streamDesc.mBytesPerFrame << 
		" mChannelsPerFrame = " << streamDesc.mChannelsPerFrame << 
		" mBitsPerChannel = " << streamDesc.mBitsPerChannel << ")";
	return s;
}



	
	