/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#include <stdio.h>

#ifndef ZzDebugPrintFunctions_h
#include "ZzDebugPrintFunctions.h"
#endif //ZzDebugPrintFunctions_h

#ifdef __APPLE__

#include <CoreAudio/CoreAudioTypes.h>

#define PRINT_ENUM_NAME_IF_MATCHING(testedVal,enumVal) { if(testedVal == enumVal) printf("%s\n",#enumVal); }

/*
	AudioStreamBasicDescription
*/

void ZzDebugPrintStringFrom_AudioStreamBasicDescription_FormatID(UInt32 fmtid)
{
	PRINT_ENUM_NAME_IF_MATCHING(fmtid,kAudioFormatLinearPCM);
	PRINT_ENUM_NAME_IF_MATCHING(fmtid,kAudioFormatAC3);
	PRINT_ENUM_NAME_IF_MATCHING(fmtid,kAudioFormat60958AC3);
//	PRINT_ENUM_NAME_IF_MATCHING(fmtid,kAudioFormatMPEG);
	PRINT_ENUM_NAME_IF_MATCHING(fmtid,kAudioFormatAppleIMA4);
	PRINT_ENUM_NAME_IF_MATCHING(fmtid,kAudioFormatMPEG4AAC);
	PRINT_ENUM_NAME_IF_MATCHING(fmtid,kAudioFormatMPEG4CELP);
	PRINT_ENUM_NAME_IF_MATCHING(fmtid,kAudioFormatMPEG4HVXC);
	PRINT_ENUM_NAME_IF_MATCHING(fmtid,kAudioFormatMPEG4TwinVQ);
	printf("!!!unknown format constant!!!\n");
}

void ZzDebugPrintStringFrom_AudioStreamBasicDescription_FormatFlags(UInt32 fmtFlags)
{
	bool aFlagAlreadyFound = false;

	if(fmtFlags & kAudioFormatFlagIsFloat)
	{
		if(aFlagAlreadyFound)
			printf(" | ");
		printf("kAudioFormatFlagIsFloat");
		aFlagAlreadyFound = true;
	}
	if(fmtFlags & kAudioFormatFlagIsBigEndian)
	{
		if(aFlagAlreadyFound)
			printf(" | ");
		printf("kAudioFormatFlagIsBigEndian");
		aFlagAlreadyFound = true;
	}
	if(fmtFlags & kAudioFormatFlagIsSignedInteger)
	{
		if(aFlagAlreadyFound)
			printf(" | ");
		printf("kAudioFormatFlagIsSignedInteger");
		aFlagAlreadyFound = true;
	}
	if(fmtFlags & kAudioFormatFlagIsPacked)
	{
		if(aFlagAlreadyFound)
			printf(" | ");
		printf("kAudioFormatFlagIsPacked");
		aFlagAlreadyFound = true;
	}
	if(fmtFlags & kAudioFormatFlagIsAlignedHigh)
	{
		if(aFlagAlreadyFound)
			printf(" | ");
		printf("kAudioFormatFlagIsAlignedHigh");
		aFlagAlreadyFound = true;
	}
	if(fmtFlags & kAudioFormatFlagIsNonInterleaved)
	{
		if(aFlagAlreadyFound)
			printf(" | ");
		printf("kAudioFormatFlagIsNonInterleaved");
		aFlagAlreadyFound = true;
	}
	if(!aFlagAlreadyFound)
	{
		if(fmtFlags == 0)
			printf("0");
		else
			printf("!!!no matching system defined flags!!!");
	}
	printf("\n");
}

void ZzDebugPrint_AudioStreamBasicDescription(AudioStreamBasicDescription* sbd)
{
	printf("mSampleRate = %f\n",sbd->mSampleRate);
	printf("mFormatID = "); 
	ZzDebugPrintStringFrom_AudioStreamBasicDescription_FormatID(sbd->mFormatID);
	printf("mFormatFlags = "); 
	ZzDebugPrintStringFrom_AudioStreamBasicDescription_FormatFlags(sbd->mFormatFlags);
	printf("mBytesPerPacket = %lu\n",sbd->mBytesPerPacket);
	printf("mFramesPerPacket = %lu\n",sbd->mFramesPerPacket);
	printf("mBytesPerFrame = %lu\n",sbd->mBytesPerFrame);
	printf("mChannelsPerFrame = %lu\n",sbd->mChannelsPerFrame);
	printf("mBitsPerChannel = %lu\n",sbd->mBitsPerChannel);
}

#endif //__APPLE__
