/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#ifndef ZzException_h
#include "ZzException.h"
#endif //ZzException_h

#ifndef ZzHelpers_h
#include "ZzHelpers.h"
#endif //ZzHelpers_h

#ifdef SHOW_DEBUG_INFO_IN_ERROR_MSG
ZzException::ZzException(const char* sourceFile, int lineNumber)
{
	strcpy(fSourceFile, sourceFile);
	fLineNumber = lineNumber;
	fErrorMessage[0] = 0;

	fHasBeenSignaledToUser = false;
}

ZzException::ZzException(const char* message, const char* sourceFile, int lineNumber)
{
	strcpy(fErrorMessage, message);
	strcpy(fSourceFile, sourceFile);
	fLineNumber = lineNumber;

	fHasBeenSignaledToUser = false;
}

#else
ZzException::ZzException()
{
	fErrorMessage[0] = 0;
	fHasBeenSignaledToUser = false;
}
ZzException::ZzException(const char* message)
{
	strcpy(fErrorMessage, message);
	fHasBeenSignaledToUser = false;
}
#endif SHOW_DEBUG_INFO_IN_ERROR_MSG

ZzException::~ZzException()
{
}

void ZzException::DefaultCatcher()
{
	if (!fHasBeenSignaledToUser)
	{
		if (ErrorMessage())
		{
			//display a message box with the error
#ifdef SHOW_DEBUG_INFO_IN_ERROR_MSG
			char string[1024];
			sprintf(string, "Exception raised in %s at line %d : %s", fSourceFile, fLineNumber, ErrorMessage());
			ZzWarningMessageBox(string);
#else //SHOW_DEBUG_INFO_IN_ERROR_MSG			
			ZzWarningMessageBox(ErrorMessage());
#endif //SHOW_DEBUG_INFO_IN_ERROR_MSG
			fHasBeenSignaledToUser = true;
		}
	}
}

const char* ZzException::ErrorMessage() const
{
	if (fErrorMessage[0] == 0)
		return "Unknown exception";
	else
		return fErrorMessage;
}

const char* ZzExceptionOutOfMemory::ErrorMessage() const
{
	return "Out of memory";
}

const char* ZzExceptionTest::ErrorMessage() const
{
	return "Randomly generated error to check that errors are handled correctly";
}

const char* ZzExceptionFile::ErrorMessage() const
{
	return "Unkown File Exception";
}

const char* ZzExceptionDiskIsFull::ErrorMessage() const
{
	return "Not enough disk space";
}

const char* ZzExceptionFileNotFound::ErrorMessage() const
{
	return "File Not Found";
}

const char* ZzExceptionCancel::ErrorMessage() const
{
//	return "Operation cancelled, you should catch this exception separately such that this message is not displayed";
	return NULL;
}

const char* ZzAudioClipImportException::ErrorMessage() const
{
	return fUserFriendlyMessage;
}

const char* ZzGenericException::ErrorMessage() const
{
	return fUserFriendlyMessage;
}

