/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#define ZzException_h

#ifndef ZzException_h
#include "ZzException.h"
#endif //ZzException_h

//#ifdef DEBUG
#define SHOW_DEBUG_INFO_IN_ERROR_MSG
//#endif

//define macros for throwing exceptions
#ifdef SHOW_DEBUG_INFO_IN_ERROR_MSG
#define ZZ_THROW(ExceptionClass) throw new ExceptionClass(__FILE__, __LINE__)
#define ZZ_THROW_1ARG(ExceptionClass, arg1) throw new ExceptionClass(arg1, __FILE__, __LINE__)
#else //SHOW_DEBUG_INFO_IN_ERROR_MSG
#define ZZ_THROW(ExceptionClass) throw new ExceptionClass()
#define ZZ_THROW_1ARG(ExceptionClass, arg1) throw new ExceptionClass(arg1)
#endif

class ZzException
{
	bool	fHasBeenSignaledToUser;
	char	fErrorMessage[1024];

#ifdef SHOW_DEBUG_INFO_IN_ERROR_MSG
	char	fSourceFile[1024];
	int		fLineNumber;
#endif//SHOW_DEBUG_INFO_IN_ERROR_MSG

public :
#ifdef SHOW_DEBUG_INFO_IN_ERROR_MSG
	ZzException(const char* sourceFile, int lineNumber);
	ZzException(const char* message, const char* sourceFile, int lineNumber);
#else
	ZzException();
	ZzException(const char* message);
#endif SHOW_DEBUG_INFO_IN_ERROR_MSG

	virtual ~ZzException();
	
	virtual const char* ErrorMessage() const;
	virtual void DefaultCatcher();
};

class ZzExceptionTest : public ZzException
{

public :
#ifdef SHOW_DEBUG_INFO_IN_ERROR_MSG
	ZzExceptionTest(const char* sourceFile, int lineNumber) : ZzException(sourceFile, lineNumber) {};
#else
	ZzExceptionTest() : ZzException() {};
#endif SHOW_DEBUG_INFO_IN_ERROR_MSG

	virtual const char* ErrorMessage() const;
};

class ZzExceptionCancel : public ZzException
{
public :
#ifdef SHOW_DEBUG_INFO_IN_ERROR_MSG
	ZzExceptionCancel(const char* sourceFile, int lineNumber) : ZzException(sourceFile, lineNumber) {};
#else
	ZzExceptionCancel() : ZzException() {};
#endif SHOW_DEBUG_INFO_IN_ERROR_MSG
	virtual const char* ErrorMessage() const;
};

class ZzExceptionOutOfMemory : public ZzException
{

public :
#ifdef SHOW_DEBUG_INFO_IN_ERROR_MSG
	ZzExceptionOutOfMemory(const char* sourceFile, int lineNumber) : ZzException(sourceFile, lineNumber) {};
#else
	ZzExceptionOutOfMemory() : ZzException() {};
#endif SHOW_DEBUG_INFO_IN_ERROR_MSG
	virtual const char* ErrorMessage() const;
};


class ZzExceptionFile : public ZzException
{

public :
#ifdef SHOW_DEBUG_INFO_IN_ERROR_MSG
	ZzExceptionFile(const char* sourceFile, int lineNumber) : ZzException(sourceFile, lineNumber) {};
#else
	ZzExceptionFile() : ZzException() {};
#endif SHOW_DEBUG_INFO_IN_ERROR_MSG

	virtual const char* ErrorMessage() const;
};

class ZzExceptionDiskIsFull : public ZzExceptionFile
{

public :
#ifdef SHOW_DEBUG_INFO_IN_ERROR_MSG
	ZzExceptionDiskIsFull(const char* sourceFile, int lineNumber) : ZzExceptionFile(sourceFile, lineNumber) {};
#else
	ZzExceptionDiskIsFull() : ZzExceptionFile() {};
#endif SHOW_DEBUG_INFO_IN_ERROR_MSG
	virtual const char* ErrorMessage() const;
};

class ZzExceptionFileNotFound : public ZzExceptionFile
{

public :
#ifdef SHOW_DEBUG_INFO_IN_ERROR_MSG
	ZzExceptionFileNotFound(const char* sourceFile, int lineNumber) : ZzExceptionFile(sourceFile, lineNumber) {};
#else
	ZzExceptionFileNotFound() : ZzExceptionFile() {};
#endif SHOW_DEBUG_INFO_IN_ERROR_MSG
	virtual const char* ErrorMessage() const;
};

class ZzAudioClipImportException : public ZzException
{

public :
#ifdef SHOW_DEBUG_INFO_IN_ERROR_MSG
	ZzAudioClipImportException(const char* userFriendlyMessage, const char* sourceFile, int lineNumber) : ZzException(sourceFile, lineNumber) { fUserFriendlyMessage = userFriendlyMessage; };
#else
	ZzAudioClipImportException(const char* userFriendlyMessage) : ZzException() { fUserFriendlyMessage = userFriendlyMessage; };
#endif SHOW_DEBUG_INFO_IN_ERROR_MSG
	virtual const char* ErrorMessage() const;
private :
	const char* fUserFriendlyMessage;
};

class ZzGenericException : public ZzException
{

public :
#ifdef SHOW_DEBUG_INFO_IN_ERROR_MSG
	ZzGenericException(const char* userFriendlyMessage, const char* sourceFile, int lineNumber) : ZzException(sourceFile, lineNumber) { fUserFriendlyMessage = userFriendlyMessage; };
#else
	ZzGenericException(const char* userFriendlyMessage) : ZzException() { fUserFriendlyMessage = userFriendlyMessage; };
#endif SHOW_DEBUG_INFO_IN_ERROR_MSG
	virtual const char* ErrorMessage() const;
private :
	const char* fUserFriendlyMessage;
};

