/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
 /*
	misc helper functions
*/

#ifndef ZzHelpers_h
#include "ZzHelpers.h"
#endif //ZzHelpers_h

#include <fcntl.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <sys/stat.h>

#ifndef ZzString_h
#include "ZzString.h"
#endif //ZzString_h

#ifndef ZzVerbose_h
#include "ZzVerbose.h"
#endif //ZzVerbose_h

#ifndef ZzCoreAudioStreamOperators_h
#include "ZzCoreAudioStreamOperators.h"
#endif //ZzCoreAudioStreamOperators_h

#include <AudioToolbox/AudioFile.h>
#include <CoreAudio/HostTime.h>

#ifndef MIProgressBarDlgManager_h
#include "MIProgressBarDlgManager.h"
#endif //MIProgressBarDlgManager_h

#include "MoreFilesX.h" //$$$TODO check updates from time to time

#ifndef ZzTimer_h
#include "ZzTimer.h"
#endif ZzTimer_h

extern "C" 
{
	#include <mach/thread_policy.h>
}
//#include <Kernel/mach/policy.h>

extern "C" 
{
	#include <mach/thread_act.h>
}

#define THREAD_SET_PRIORITY				0
#define THREAD_SCHEDULED_PRIORITY		1

//#include <Kernel/mach/thread_policy.h>
//#include <Kernel/mach/mach_time.h>

std::strstream gStreamToString;

#ifdef WIN32
void ZzAlert_(const char* message)
{
	MessageBox(NULL,message,"Alert",MB_OK | MB_ICONEXCLAMATION );
}
#elif defined __BEOS__
#include <Alert.h>
void ZzAlert_(const char* message)
{
	BAlert* alert = new BAlert("Alert",message,"OK");
	alert->Go();
}
#endif //platform	

/************************************************************************************************************

	string functions
	
*************************************************************************************************************/

/************************************************************************************************************

	string functions
	
*************************************************************************************************************/
void ZzCopyStr64( ZzStrPtr dest, ZzStrCPtr src )
{
	strncpy(dest,src,63);
	dest[63] = 0;
}

/************************************************************************************************************

	ring buffer functions
	
*************************************************************************************************************/

// this function can copy from a ring buffer to a normal buffer
// and split the operation in two copies if the source buffer 
// crosses the ring buffer boundary.

// please note that you can't ask for a source buffer size so
// large that it would need more than two operations ! (i.e. the
// same source data can't be read twice)

void ZzCopyFromRingBufferToNormalBuffer(
	uint8*	dest,
	uint8*	ringBufStartAddr,
	uint8*	copiedZoneStartAddr,
	UInt32	ringBufSize,
	UInt32	copiedZoneSize )
{
	// selfexpl!
	uint8* ringBufEndAddr = ringBufStartAddr + ringBufSize;
	
	// where the source zone would end if it was a linear buffer
	uint8* srcEndAddrExcl = copiedZoneStartAddr + copiedZoneSize;
	
	// maybe its a normal copy !
	if( srcEndAddrExcl < ringBufEndAddr )
	{
		memcpy(dest,copiedZoneStartAddr,copiedZoneSize);
	}
	else
	{
		// this first copy from the source zone start addr to the					  
		// end of the ring buffer
		UInt32 firstCopySize = ringBufEndAddr - copiedZoneStartAddr;
		// the rest
		UInt32 remainingSize = srcEndAddrExcl - ringBufEndAddr;
		ZzAssert(remainingSize < ringBufSize);
		// copy part 1
		memcpy(dest,copiedZoneStartAddr,firstCopySize);
		// copy part 2
		memcpy(dest+firstCopySize,ringBufStartAddr,remainingSize);
	}
}

void ZzIncrementPointerInRingBuffer(
	uint8*	ringbufStart, 
	UInt32	ringbufSize, 
	uint8*&	pointer,
	UInt32	increment )
{
//	sdmpn("ZzIncrementPointerInRingBuffer() - before : ");
//	dmp((void*)ringbufStart);
//	dmp((int)ringbufSize);
//	dmp((void*)pointer);
//	dmpn((int)increment);

	uint8* ringbufEnd = ringbufStart + ringbufSize;

	/*
		m_virtualPlayPosition is the incremented play pos if the
		queue were linear. Then, if we overrun the end of the queue,
		the rest is added from the start of the queue
	*/
	uint8* virtualPointer = pointer + increment;
	
	/* if we have or not crossed the queue end when incrementing pos */
	if( virtualPointer < ringbufEnd )
		pointer = virtualPointer;
	else
	{
		UInt32 rest = virtualPointer - ringbufEnd;
		pointer = ringbufStart + rest;
	}
//	sdmp("after : ");
//	dmpn(pointer);
};


void ZzFill37StringWithGUID(char* string, int maxLen)
{
	ZzAssert(maxLen >= 37);
	CFUUIDRef uuidRef = CFUUIDCreate(kCFAllocatorDefault);
	CFStringRef uuidStringRef = CFUUIDCreateString(kCFAllocatorDefault,uuidRef);
	CFStringGetCString(uuidStringRef,string,37,kCFStringEncodingASCII);
	CFRelease(uuidStringRef);
	CFRelease(uuidRef);
}


void ZzSetDRTThreadPriority(pthread_t thrid)
{
	ZzSetThreadPriority(thrid,63,true,0);
}

void ZzSetThreadPriority(pthread_t inThread,UInt32 inPriority,bool inIsFixed, UInt64 inHALIOProcCycleDurationInNanoseconds)
{
	if (inPriority == 96)
	{
		ZzAssert(inHALIOProcCycleDurationInNanoseconds != 0);
        // REAL-TIME / TIME-CONSTRAINT THREAD
        thread_time_constraint_policy_data_t		theTCPolicy;
		UInt64										theComputeQuanta;
		UInt64										thePeriod;
		UInt64										thePeriodNanos;
		
        thePeriodNanos = inHALIOProcCycleDurationInNanoseconds;
        theComputeQuanta = AudioConvertNanosToHostTime ( (UInt64)((double)thePeriodNanos * 0.15) );
		thePeriod = AudioConvertNanosToHostTime (thePeriodNanos);
        
		theTCPolicy.period = thePeriod;
		theTCPolicy.computation = theComputeQuanta;
		theTCPolicy.constraint = thePeriod;
		theTCPolicy.preemptible = true;
		thread_policy_set (pthread_mach_thread_np(inThread), THREAD_TIME_CONSTRAINT_POLICY, (thread_policy_t)&theTCPolicy, THREAD_TIME_CONSTRAINT_POLICY_COUNT);
	} else {
        // OTHER THREADS
		thread_extended_policy_data_t		theFixedPolicy;
        thread_precedence_policy_data_t		thePrecedencePolicy;
        SInt32								relativePriority;
        
		// [1] SET FIXED / NOT FIXED
        theFixedPolicy.timeshare = !inIsFixed;
        thread_policy_set (pthread_mach_thread_np(inThread), THREAD_EXTENDED_POLICY, (thread_policy_t)&theFixedPolicy, THREAD_EXTENDED_POLICY_COUNT);
        
		// [2] SET PRECEDENCE
        // N.B.: We expect that if thread A created thread B, and the program wishes to change
        // the priority of thread B, then the call to change the priority of thread B must be
        // made by thread A.
        // This assumption allows us to use pthread_self() to correctly calculate the priority
        // of the feeder thread (since precedency policy's importance is relative to the
        // spawning thread's priority.)
        relativePriority = inPriority - ZzGetThreadSetPriority (pthread_self());
        
        thePrecedencePolicy.importance = relativePriority;
        thread_policy_set (pthread_mach_thread_np(inThread), THREAD_PRECEDENCE_POLICY, (thread_policy_t)&thePrecedencePolicy, THREAD_PRECEDENCE_POLICY_COUNT);
	}
}

// returns the thread's priority as it was last set by the API
UInt32 ZzGetThreadSetPriority (pthread_t inThread)
{
    return ZzGetThreadPriority (inThread, THREAD_SET_PRIORITY);
}

// returns the thread's priority as it was last scheduled by the Kernel
UInt32 ZzGetThreadScheduledPriority (pthread_t inThread)
{
    return ZzGetThreadPriority (inThread, THREAD_SCHEDULED_PRIORITY);
}

UInt32 ZzGetThreadPriority (pthread_t inThread, int inWhichPriority)
{
    thread_basic_info_data_t			threadInfo;
	policy_info_data_t					thePolicyInfo;
	unsigned int						count;
    
    // get basic info
    count = THREAD_BASIC_INFO_COUNT;
    thread_info (pthread_mach_thread_np (inThread), THREAD_BASIC_INFO, (thread_info_t)&threadInfo, &count);
    
	switch (threadInfo.policy) {
		case POLICY_TIMESHARE:
			count = POLICY_TIMESHARE_INFO_COUNT;
			thread_info(pthread_mach_thread_np (inThread), THREAD_SCHED_TIMESHARE_INFO, (thread_info_t)&(thePolicyInfo.ts), &count);
            if (inWhichPriority == THREAD_SCHEDULED_PRIORITY) {
                return thePolicyInfo.ts.cur_priority;
            } else {
                return thePolicyInfo.ts.base_priority;
            }
            break;
            
        case POLICY_FIFO:
			count = POLICY_FIFO_INFO_COUNT;
			thread_info(pthread_mach_thread_np (inThread), THREAD_SCHED_FIFO_INFO, (thread_info_t)&(thePolicyInfo.fifo), &count);
            if ( (thePolicyInfo.fifo.depressed) && (inWhichPriority == THREAD_SCHEDULED_PRIORITY) ) {
                return thePolicyInfo.fifo.depress_priority;
            }
            return thePolicyInfo.fifo.base_priority;
            break;
            
		case POLICY_RR:
			count = POLICY_RR_INFO_COUNT;
			thread_info(pthread_mach_thread_np (inThread), THREAD_SCHED_RR_INFO, (thread_info_t)&(thePolicyInfo.rr), &count);
			if ( (thePolicyInfo.rr.depressed) && (inWhichPriority == THREAD_SCHEDULED_PRIORITY) ) {
                return thePolicyInfo.rr.depress_priority;
            }
            return thePolicyInfo.rr.base_priority;
            break;
	}
    
    return 0;
}







