/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
 
#define ZzHelpers_h
#ifndef ZzHelpers_h
#include "ZzHelpers.h"
#endif //ZzHelpers_h
/*
	misc helper functions
*/

#ifndef ZzTypes_h
#include "ZzTypes.h"
#endif //ZzTypes_h

#ifndef ZzString_h
#include "ZzString.h"
#endif //ZzString_h

#ifndef ArmeErrors_h
#include "ArmeErrors.h"
#endif //ArmeErrors_h

/*
	ZzAlert

	this function displays an information messagebox.
	This is not debug since it can happen when a file format
	can't be parsed, etc etc etc
*/

#include <strstream>
using namespace std;

extern std::strstream gStreamToString;

#define ZzAlert(a)															\
{																			\
	gStreamToString << a; ZzAlert_(gStreamToString.str());					\
	gStreamToString.rdbuf()->freeze(0); gStreamToString.seekp(0);			\
}

void ZzAlert_(const char* message);

void ZzFill37StringWithGUID(char* string, int maxLen);

template <class T> inline const T& ZzMax(const T& a, const T& b)
{
	if( a > b )
		return a;
	else
		return b;
}

template <class T> inline const T& ZzMin(const T& a, const T& b)
{
	if( a < b )
		return a;
	else
		return b;
}

/*
inline std::ostream& operator<<( std::ostream& s, int64 val )
{
	UInt32* dwords;
	dwords = (UInt32*)(&val);
	char txt[32];
	sprintf(txt,"(%lx,%lx)", dwords[0], dwords[1] );
	s << txt;
	return s;
}
*/

void ZzCopyFromRingBufferToNormalBuffer(
	uint8*	dest,
	uint8*	ringBufStartAddr,
	uint8*	copiedZoneStartAddr,
	UInt32	ringBufSize,
	UInt32	copiedZoneSize );

void ZzIncrementPointerInRingBuffer(
	uint8*	ringbufStart, 
	UInt32	ringbufSize, 
	uint8*&	pointer,
	UInt32	increment );

inline float ZzGetYFromXAnd2Points(float x, float x1, float y1, float x2, float y2)
{
	return ((y2-y1)/(x2-x1))*(x-x1) + y1;
}


/*! @function ZzSetDRTThreadPriority This function adjust the pthread_t priority so that it's suitable for a disk feeder thread (that currently can potentially block the ioproc !!!) */
void ZzSetDRTThreadPriority(pthread_t thrid);

/*! @function ZzSetThreadPriority Adjust a thread priority. fixedPriority means the thread priority will never gets reduced as it 
is the case for normal threads. Setting a priority of 96 means a real time thread with time constraints. In this case, the 
parameter inHALIOProcCycleDurationInNanoseconds is used, and the function assumes a 15% duty cycle */
void ZzSetThreadPriority(pthread_t inThread,UInt32 inPriority,bool isFixed, UInt64 inHALIOProcCycleDurationInNanoseconds = 0);

UInt32 ZzGetThreadSetPriority (pthread_t inThread);
UInt32 ZzGetThreadPriority (pthread_t inThread, int inWhichPriority);
UInt32 ZzGetThreadScheduledPriority (pthread_t inThread);

SInt64 ZzAudioFileGetDuration(const char* path);

ZzString ZzGUIChooseFile(bool bOpen = true);
void ZzWarningMessageBox(const char* msg);

enum ZzMessageBoxReply
{
	ZZ_MESSAGEBOXREPLY_DEFAULT_BUTTON = 1515,
	ZZ_MESSAGEBOXREPLY_ALT_BUTTON,
	ZZ_MESSAGEBOXREPLY_OTHER_BUTTON,
};

ZzMessageBoxReply ZzMessageBox(
	const ZzString& zztitle, const ZzString& zzmessage, 
	const ZzString& zzdefaultButton, const ZzString& zzaltButton, const ZzString& zzotherButton);

bool ZzConvertNumberStringToDecimalNumber(SInt64* outNumber, const ZzString& inForeignNumber, SInt32 inBase);
ZzString ZzConvertDecimalNumberToNumberString(SInt64 inNumber, SInt32 inBase);
