/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#ifndef ZzHelpers_h
#include "ZzHelpers.h"
#endif //ZzHelpers_h

ZzString ZzGUIChooseFile(bool bOpen)
{
	if(bOpen)
	{
		NSOpenPanel* openPanel = [NSOpenPanel openPanel];
		[openPanel setAllowsMultipleSelection:NO];
		[openPanel runModalForTypes:nil];
		NSString* fileName = [[openPanel filenames] objectAtIndex:0];
		return ZzString((CFStringRef)fileName);
	}
	else
	{
		ZzAssert(0); // save panel not impl'ed yet
		return ZzString("BAD_FILENAME"); // beeek
	}
}

void ZzWarningMessageBox(const char* msg)
{
	NSRunAlertPanel(@"Warning",[NSString stringWithCString:msg], @"OK", nil, nil);
}

ZzMessageBoxReply ZzMessageBox(
	const ZzString& zztitle, const ZzString& zzmessage, 
	const ZzString& zzdefaultButton, const ZzString& zzaltButton, const ZzString& zzotherButton)
{
	NSString* message = nil;
	NSString* title = nil;
	NSString* defaultButton = nil;
	NSString* altButton = nil;
	NSString* otherButton = nil;

	// convert the dlg message 
	message = zzmessage.getNSString();

	// convert the dlg title if non empty 
	if(zztitle != ZzString(""))
		title = zztitle.getNSString();

	// convert the dlg buttons if non empty...
	if(zzdefaultButton != ZzString(""))
		defaultButton = zzdefaultButton.getNSString();
	if(zzaltButton != ZzString(""))
		altButton = zzaltButton.getNSString();
	if(zzotherButton != ZzString(""))
		otherButton = zzotherButton.getNSString();
		
	int result = NSRunAlertPanel(title,message,defaultButton,altButton,otherButton);
	switch(result)
	{
	case NSAlertDefaultReturn :
		return ZZ_MESSAGEBOXREPLY_DEFAULT_BUTTON;
		break;
	case NSAlertAlternateReturn :
		return ZZ_MESSAGEBOXREPLY_ALT_BUTTON;
		break;
	case NSAlertOtherReturn :
		return ZZ_MESSAGEBOXREPLY_OTHER_BUTTON;
		break;
	case NSAlertErrorReturn :
		ZZ_THROW(ZzException);
		break;
	default :
		ZZ_THROW(ZzException);
		break;
	};
}

SInt32 ZzBaseConversionDigitToValue(char digit)
{
	SInt32 _limit0 = '0';
	SInt32 _limit9 = '9';
	SInt32 _limita = 'a';
	SInt32 _limitz = 'z';
	SInt32 _limitA = 'A';
	SInt32 _limitZ = 'Z';
	
	// replacement
	if(digit == 'w')
		digit = 'o';
	if(digit == 'W')
		digit = 'O';
	
	if(digit >= _limit0 && digit <= _limit9)
	{
		return digit - _limit0;
	}
	if(digit >= _limita && digit <= _limitz)
	{
		return digit - _limita + 10;
	}
	if(digit >= _limitA && digit <= _limitZ)
	{
		return digit - _limitA + 10;
	}
	return -1;
}

char ZzBaseConversionValueToDigit(SInt32 digit)
{
	SInt32 _limit0 = '0';
//	SInt32 _limit9 = '9';
//	SInt32 _limita = 'a';
//	SInt32 _limitz = 'z';
	SInt32 _limitA = 'A';
	SInt32 _limitZ = 'Z';
	
	if(digit >= 0 && digit <= 9)
		return digit + _limit0;
		
	if(digit >= 10 && digit <= (10 + _limitZ -_limitA))
	{
		char returnValue = digit + _limitA - 10;
		// REPLACEMENT
		if(returnValue == 'O')
			returnValue = 'W';
		return returnValue;
	}
	return 0;
}
			
bool ZzConvertNumberStringToDecimalNumber(SInt64* outNumber, const ZzString& inForeignNumber, SInt32 inBase)
{
	int stringLength = inForeignNumber.getLength();
	// copy the string in a C buffer
	char* dstString = new char[stringLength];
	const char* srcString = inForeignNumber.getUTF8String();
	strcpy(dstString,srcString);
	
	int digitIndex; // we start parsing from right to left
	
	SInt64 digitFactor = 1LL;
	SInt64 number = 0LL;
				
	for(digitIndex = stringLength-1; digitIndex >= 0; --digitIndex)
	{
		char digit = dstString[digitIndex];
		SInt32 digitValue = ZzBaseConversionDigitToValue(digit);
		if(digitValue == -1)
		{
			delete [] dstString;
			*outNumber = 0LL;
			return false;
		}
		if(digitValue >= inBase)
		{
			delete [] dstString;
			*outNumber = 0LL;
			return false;
		}
		number += digitFactor*digitValue;
		digitFactor *= inBase;
	}
	*outNumber = number;	
	delete [] dstString;
	return true;
}

ZzString ZzConvertDecimalNumberToNumberString(SInt64 inNumber, SInt32 inBase)
{
	//SInt32 currentDigitValue;
	
	// very rough calculation to compute the number of needed digits.
	SInt32 nbDigits = (SInt32)(floor(log((double)inNumber)/log((double)inBase) + 1)); // +1 because "1000" is 4 chars and log(1000) = 3
	
	char* returnString = new char[nbDigits+1]; // +1 for the trailing BYLL
	
	for(SInt32 i = 0; i < nbDigits; i++)
	{
		returnString[i] = ' ';
	}
	returnString[nbDigits] = 0;
				
	SInt32 digitIndex = 0;
	
	bool shouldContinue = true;
	while(shouldContinue) // we use a "while", not a "do" loop "so that" 0 is handled correctly...
	{
		SInt32 digitValue = inNumber % inBase;
		char digit = ZzBaseConversionValueToDigit(digitValue);
		ZzAssert(nbDigits-1-digitIndex >= 0);
		returnString[nbDigits-1-digitIndex] = digit;
		digitIndex++;
		inNumber /= inBase;
		if(inNumber == 0)
			shouldContinue = false;
	}
	ZzString retVal = ZzString(returnString);
	delete [] returnString;
	return retVal;
}


