/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
 
#ifndef ZzLocker_h 
#include "ZzLocker.h" 
#endif //ZzLocker_h  

#ifndef _WIN32 	
#include <pthread.h> 
#endif //_WIN32

#include "ZzVerbose.h"

//#include <stdio.h>  

#ifdef _WIN32
  #ifndef ZzAssert
    #include <cassert>
    #define ZzAssert(a) assert(a)
  #endif //ZzAssert
#endif //_WIN32

ZzLocker::ZzLocker()
{ 
#ifdef _WIN32 	
  m_hMutex = CreateMutex(NULL,FALSE,NULL); // nameless Mutex
  ZzAssert(m_hMutex != NULL ); 
#elif defined __USE_CARBON_MP_FOR_ZZLOCKER__ 	
  OSStatus status = MPCreateSemaphore(1,0,&semaID);
  ZzAssert(status == noErr);
#elif defined __APPLE__ 	
  pthread_mutex_init( &ptMutex, NULL );
#else 	
  #error("please provide ZzLocker::ZzLocker() for your platform") 
#endif //plafform 
}

bool ZzLocker::Lock()
{
#ifdef _WIN32
  DWORD hr = WaitForSingleObject(m_hMutex,INFINITE);
  if( hr == WAIT_OBJECT_0 )
    return true; 	
  return false; 
#elif defined __USE_CARBON_MP_FOR_ZZLOCKER__ 	
  OSStatus status = MPWaitOnSemaphore(semaID,kDurationForever);
  if(status == noErr)
    return true;
  return false; 
#elif defined __APPLE__
  pthread_mutex_lock(&ptMutex);
  return true; 
#else
  #error("please provide ZzLocker::Lock() for your platform") 
#endif //plafform 
} 

bool ZzLocker::Unlock()
{
#ifdef _WIN32 	
  if( ReleaseMutex(m_hMutex) ) 		
    return true; 	
  else 		
    return false; 
#elif defined __USE_CARBON_MP_FOR_ZZLOCKER__ 	
  OSStatus status = MPSignalSemaphore(semaID);
  if(status == noErr) 		
    return true; 	
  return false; 
#elif defined __APPLE__ 	
  pthread_mutex_unlock(&ptMutex); 	
  return true; 
#else 	
  #error("please provide ZzLocker::Unlock() for your platform") 
#endif //plafform 
}

ZzLocker::~ZzLocker()
{
#ifdef _WIN32 
#elif defined __USE_CARBON_MP_FOR_ZZLOCKER__ 	
  OSStatus status = MPDeleteSemaphore(semaID); 	
  if(status != noErr) 		
    ZzAssert(0); 
#elif defined __APPLE__ 	// ptMutex was allocated in the object memory so no need to delete anything 
#else 	
  #error("please provide ZzLocker::~ZzLocker() for your platform") 
#endif //plafform 
}   

#if 0 
ZzRwLocker::ZzRwLocker()
{
}

void ZzRwLocker::LockRead() 
{ 	
  m_lock.Lock(); 
}  

void ZzRwLocker::UnlockRead() 
{ 	
  m_lock.Unlock(); 
}  
void ZzRwLocker::LockWrite( ) 
{ 	
  m_lock.Lock(); 
}  
void ZzRwLocker::UnlockWrite() 
{ 	
  m_lock.Unlock(); 
}  
#else  
#ifdef __APPLE__ //NYI under _WIN32
ZzRwLocker::ZzRwLocker() 
{ 
#ifdef USE_PTHREAD_RWLOCK 	// default attributes 	
  pthread_rwlock_init(&fRWLock,NULL); 
#elif defined USE_BUTENHOF_RWLOCK 	
  rwl_init(&fButenhofRWLock); 
#else //USE_PTHREAD_RWLOCK 
#endif //USE_PTHREAD_RWLOCK 
}  
  
void ZzRwLocker::LockRead() 
{ 
#ifdef USE_PTHREAD_RWLOCK 	
  pthread_rwlock_rdlock(&fRWLock); 
#elif defined USE_BUTENHOF_RWLOCK 	
  rwl_readlock(&fButenhofRWLock); 
#else //USE_PTHREAD_RWLOCK 	
  m_accessLock.Lock(); 	
  // if we are the first one to request read only access, 	
  // we must actually take the lock 	
  if( m_readLockcounter == 0 ) 	
  { 		
    m_readLockcounter = 1; 		
    m_lock.Lock(); 	
  } 	
  // otherwise, we simply increment the counter 	
  else 	
  { 		
    m_readLockcounter++; 	
  } 	
  m_lockMode = 1; 	
  m_accessLock.Unlock(); 
#endif //USE_PTHREAD_RWLOCK 
}  

void ZzRwLocker::UnlockRead() 
{ 
#ifdef USE_PTHREAD_RWLOCK 	
  pthread_rwlock_unlock(&fRWLock); 
#elif defined USE_BUTENHOF_RWLOCK 	
  rwl_readunlock(&fButenhofRWLock); 
#else //USE_PTHREAD_RWLOCK 	
  m_accessLock.Lock(); 	
  // if we are the last one to own the lock 	
  if( m_readLockcounter == 1 ) 	
  { 		
    m_readLockcounter = 0; 		
    m_lock.Unlock(); 	
  } 	
  // otherwise, we simply decrement the counter 	
  else 	
  { 		
    m_readLockcounter--; 	
  } 	
  m_accessLock.Unlock(); 
#endif //USE_PTHREAD_RWLOCK 
}  

void ZzRwLocker::LockWrite() 
{ 
#ifdef USE_PTHREAD_RWLOCK 	
  pthread_rwlock_wrlock(&fRWLock); 
#elif defined USE_BUTENHOF_RWLOCK 	
  rwl_writelock(&fButenhofRWLock); 
#else //USE_PTHREAD_RWLOCK 	
  m_lock.Lock(); 
#endif //USE_PTHREAD_RWLOCK 
}  

void ZzRwLocker::UnlockWrite() 
{ 
#ifdef USE_PTHREAD_RWLOCK 	
  pthread_rwlock_unlock(&fRWLock); 
#elif defined USE_BUTENHOF_RWLOCK 	
  rwl_writeunlock(&fButenhofRWLock); 
#else //USE_PTHREAD_RWLOCK 	
  m_lock.Unlock(); 
#endif //USE_PTHREAD_RWLOCK 
}  

#ifdef USE_BUTENHOF_RWLOCK 
ZzRwLocker::~ZzRwLocker() 
{ 	
  rwl_destroy(&fButenhofRWLock); 
} 
#endif //USE_BUTENHOF_RWLOCK   
#endif //#ifdef __APPLE__
#endif //0 

