/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
#define ZzLocker_h
#ifndef ZzLocker_h
#include "ZzLocker.h"
#endif //ZzLocker_h

/*!
	provides a simple locker and the read/write locker
*/

#ifdef __APPLE__
#include <pthread.h>
#endif //__APPLE__

#ifdef WIN32
#include <windows.h>
#endif //WIN32

/*! 
	@class ZzLocker
	This class is a simple mutex object.
*/

class ZzLocker
{
public :
			ZzLocker();
			~ZzLocker();
	/*! @function Lock Returns true if successfully locked and false if the locker died with owning thread */
	bool	Lock();
	/*! @function Unlock() Returns false if the calling thread does not own the locker */
	bool 	Unlock();

private :
#ifdef WIN32
	HANDLE	m_hMutex;
#elif defined __BEOS__
	BLocker	m_locker;
#elif defined __USE_CARBON_MP_FOR_ZZLOCKER__
	MPSemaphoreID semaID;
#elif defined __APPLE__
	/*! @var ptMutex Implements the mutex using the pthread API */
	pthread_mutex_t ptMutex;
#endif //platform
};

#ifdef __APPLE__
/*!
	@class ZzRwLocker

	Defines a read-write lock mechanism. Such a sync object can be locked read-only by as many
	threads as needed, while it can only be locked read-write by one thread.
*/

// under POXIS OSes, we'll use the posix threads api (pthread.h) NOT YET IMPLEMENTED
// #define USE_PTHREAD_RWLOCK
#define USE_BUTENHOF_RWLOCK

#ifdef USE_BUTENHOF_RWLOCK
#include "rwlock.h"
#endif //USE_BUTENHOF_RWLOCK

class ZzRwLocker
{
public :
				ZzRwLocker();
#ifdef USE_BUTENHOF_RWLOCK
				~ZzRwLocker();
#endif //USE_BUTENHOF_RWLOCK
	/*! @function LockRead Locks the object in a read-only fashion */ 
	void		LockRead();

	/*! @function UnlockRead Unlocks the object locked in a read-only 
	fashion $$$Behaviour undefined if thread doesn't own a read lock. */ 
	void		UnlockRead();

	/*! @function LockWrite Locks the object in a read-write fashion $$$Object 
	must be totally free of locks for this to succeed.*/ 
	void		LockWrite();

	/*! @function UnlockWrite Unlocks an object locked read-write $$$Behaviour
	undefined if thread doesn't own the read-write lock. */ 
	void		UnlockWrite();
	
private :
#ifdef USE_PTHREAD_RWLOCK
$$$$$	pthread_rwlock_t	fRWLock;
#elif defined USE_BUTENHOF_RWLOCK
	rwlock_t			fButenhofRWLock;
#else //USE_PTHREAD_RWLOCK
$$$$$$	ZzLocker	m_lock;   
	ZzLocker	m_accessLock;
	int			m_lockMode; // 0 is no lock 1 is lock read 2 is lock write
	int			m_readLockcounter;
#endif //USE_PTHREAD_RWLOCK
};

#endif //__APPLE__
