/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#ifndef ZzRtAssert_h
#include "ZzRtAssert.h"
#endif //ZzRtAssert_h

#include "ZzProgressMngrLogic2.h"

#undef ZzRtAssert
#define ZzRtAssert(a)
//#define ZzRtAssert(a) {if(!(a)){printf("progress bar error in %s LINE : %d\n",__FILE__,__LINE__);}}

ZzProgressMngrLogic2::ZzProgressMngrLogic2()
{
	Reset();
}

void ZzProgressMngrLogic2::Reset()
{
	fCurLevel = 0;
	fProgress = 0.0;
	fCurAbsOffset = 0.0;

	fNodeExtentInCurrentBranch[0] = 1.0;

	fRemainingSiblingsToTraverseForCurrentBranch[0] = 1;

	fCurNodeType = kTypeUndefined;
}

void ZzProgressMngrLogic2::SetNbSubTasks(SInt32 n)
{
//	printf("ZzProgressMngrLogic2::SetNbSubTasks(%d)\n",n);
	if(fCurNodeType == kTypeUndefined)
	{
		fCurNodeType = kTypeParent;
	}
	else
	{
		ZzRtAssert(0);
		return;
	}
	
	fCurLevel++;
	if(fCurLevel >= kMaxProgressMngr2HierarchDepth)
		fCurLevel = kMaxProgressMngr2HierarchDepth-1;
	fRemainingSiblingsToTraverseForCurrentBranch[fCurLevel] = n;
	if(fCurLevel == 0)
		fNodeExtentInCurrentBranch[fCurLevel] = 1.0/(double)n; // DEBUG SHOULD NOT HAPPEN
	else
		fNodeExtentInCurrentBranch[fCurLevel] = fNodeExtentInCurrentBranch[fCurLevel-1]/(double)n;
	fCurNodeType = kTypeOutsideOfSiblings;
}

void ZzProgressMngrLogic2::SetProgress(double p)
{
//	printf("ZzProgressMngrLogic2::SetProgress(%f) ",p);
	if(fCurNodeType == kTypeUndefined)
	{
		fCurNodeType = kTypeLeaf;
	}
	if(fCurNodeType != kTypeLeaf)
	{
		return; // we don't want to change this... this will only be 1.0 in case of a leave sub task
 	}
	else
	{
		ZzRtAssert(fCurNodeType == kTypeLeaf);
	}	
//	ZzRtAssert(p >= 0.0);
//	ZzRtAssert(p <= 1.0);
	fProgress = fCurAbsOffset + p*fNodeExtentInCurrentBranch[fCurLevel];
//	printf("--> total progress = %f\n", fProgress);
//	ZzRtAssert(fProgress >= 0.0);
//	ZzRtAssert(fProgress <= 1.0);
}

double ZzProgressMngrLogic2::GetWholeProgress() const
{
	return fProgress;
}

void ZzProgressMngrLogic2::EnterSubTask()
{
//	printf("ZzProgressMngrLogic2::EnterSubTask()\n");
	ZzRtAssert(fCurNodeType == kTypeOutsideOfSiblings);
	fCurNodeType = kTypeUndefined;
	ZzRtAssert(fRemainingSiblingsToTraverseForCurrentBranch[fCurLevel] > 0);
}

void ZzProgressMngrLogic2::LeaveSubTask()
{
//	printf("ZzProgressMngrLogic2::LeaveSubTask()\n");
	fRemainingSiblingsToTraverseForCurrentBranch[fCurLevel]--;

	// make sure we're not going back to upper level while we have siblings not traversed
	if(fCurNodeType == kTypeOutsideOfSiblings)
	{
		ZzRtAssert(fRemainingSiblingsToTraverseForCurrentBranch[fCurLevel] == 0);
	}

	if(fCurNodeType == kTypeLeaf)
		fCurAbsOffset += fNodeExtentInCurrentBranch[fCurLevel];
	if(fRemainingSiblingsToTraverseForCurrentBranch[fCurLevel] == 0)
	{
		if(fCurLevel == 0)
		{
			Reset();
			return;
		}
		fCurLevel--;
		if(fCurLevel < 0)
			fCurLevel = 0;
		fCurNodeType = kTypeParent;
	}
	else
	{
		fCurNodeType = kTypeOutsideOfSiblings;
	}
}	

