/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
 
#define ZzProgressMngrLogic2_h

#ifndef ZzProgressMngrLogic2_h
#include "ZzProgressMngrLogic2.h"
#endif //ZzProgressMngrLogic2_h

#ifndef ZzTypes_h
#include "ZzTypes.h"
#endif //ZzTypes_h

#define kMaxProgressMngr2HierarchDepth 128


/*
	the structure of operations is a tree
	
	the three starts with a big top node.

	a node can be a leaf or a parent. 
	
	when you enter a node, if you start calling SetProgress, then
	this node is a leaf and CANNOT be a parent
	
	when you enter a node, if you start calling SetNbSubTasks, then
	this node is a parent and CANNOT be a leaf
	
	a node has a weight. that is, it represent a small step in the overall
	progress.
*/	

class ZzProgressMngrLogic2
{
	enum
	{
		kTypeUndefined,
		kTypeParent,
		kTypeLeaf,
		kTypeOutsideOfSiblings
	};
		
public :
	ZzProgressMngrLogic2();
	void	SetNbSubTasks(SInt32 n);
	void	SetProgress(double p);
	double	GetWholeProgress() const;
	void	EnterSubTask();
	void	LeaveSubTask();

private :
	void 	Reset();
	
	SInt32	fCurLevel; // 0 by default
	
	double	fNodeExtentInCurrentBranch[kMaxProgressMngr2HierarchDepth];

	SInt32	fRemainingSiblingsToTraverseForCurrentBranch[kMaxProgressMngr2HierarchDepth];

	SInt32	fCurNodeType;
	
	double  fProgress;
	double  fCurAbsOffset;
};

