/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#define ZzQueue_h

#ifndef ZzQueue_h
#include "ZzQueue.h"
#endif //ZzQueue_h

#import <mach/mach.h>

struct ZzQueueItem
{
	void* aa;
	void* bb;
	void* cc;
	void* dd;
};

class ZzQueue
{
public :
	ZzQueue(int maxItems);
	~ZzQueue();

public :
	void Notify(void* aa = NULL, void* bb = NULL, void* cc = NULL, void* dd = NULL);
	void Wait(void** aa = NULL, void** bb = NULL, void** cc = NULL, void** dd = NULL);

private :
	ZzQueue(const ZzQueue&);

private :
	UInt32	WriteIndex();
	UInt32	ReadIndex();
	void	SignalSemaphore();	
	void	WaitForSemaphore();

private :
	ZzQueueItem*	fItems;
	UInt32			fLog2MaxItems;
	UInt32			fWritePos;
	UInt32			fReadPos;
	semaphore_t		fSemaphore;
};

inline UInt32 ZzQueue::WriteIndex()
{
	return fWritePos & ((1 << fLog2MaxItems)-1);
}

inline UInt32 ZzQueue::ReadIndex()
{
	return fReadPos & ((1 << fLog2MaxItems)-1);
}


