/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
#define ZzRABI_h

#ifndef ZzRABI_h
#include "ZzRABI.h"
#endif //ZzRABI_h

#include "ZzRABI_Generic.h"


/*! @class ZzRABI_FSpDelete Deletes a file given by its FSSpec */
class ZzRABI_FSpDelete : public ZzRABI
{
public :
			ZzRABI_FSpDelete(FSSpec* fsSpec) { fFSSpec = fsSpec; }
			~ZzRABI_FSpDelete() { if(fDeleteResource) { FSpDelete(fFSSpec); } }
private :
	FSSpec* fFSSpec;
};

/*! @class ZzRABI_DisposeMovie Calls DisposeMovie( ) on a given movie */
class ZzRABI_DisposeMovie : public ZzRABI
{
public :
	ZzRABI_DisposeMovie(Movie movie) { fMovie = movie; }
	~ZzRABI_DisposeMovie() { if(fDeleteResource) { ::DisposeMovie(fMovie); } }
private :
	Movie	fMovie;
};

/*! 
	@class ZzRABI_DeleteWSObjectsInArchPtrArray Deletes all the objects that are inside a 
	given ZzArchPtrArray 
*/

#ifndef TArmeDoc_h
#include "TArmeDoc.h"
#endif //TArmeDoc_h

class ZzRABI_DeleteWSObjectsInArchPtrArray : public ZzRABI
{
public : 
			ZzRABI_DeleteWSObjectsInArchPtrArray(ZzArchPtrArray* array) { fArray = array; }
			~ZzRABI_DeleteWSObjectsInArchPtrArray();
private :
	ZzArchPtrArray* fArray;
};

inline ZzRABI_DeleteWSObjectsInArchPtrArray::~ZzRABI_DeleteWSObjectsInArchPtrArray()
{
	if(fDeleteResource) 
	{
		TArmeWSObj* obj = NULL;
		ArmeArchivable* archObj = NULL;
		for(SInt32 i = 0; i < fArray->CountItems(); i++)
		{
			archObj = fArray->ItemAt(i);
			obj = dynamic_cast<TArmeWSObj*>(archObj);
			ZzRtAssert(obj);
			TArmeDoc* doc = obj->GetDoc();
			doc->LockWrite();
			delete obj;
			doc->UnlockWrite();
		}
		fArray->RemoveAll();
	}
}

/*! @class ZzRABI_SubTaskScope calls progressMngr->EnterSubTask() in its ctor 
	and progressMngr->LeaveSubTask() in its dtor.
	Please note this is NOT a ZzRABI class 
	
	ALSO, it's not very clean right now because it doesn't properly track entersubtask/leavesubtask 
	but it works as long as there are NO exceptions. and, anyway, when there's an exception, we will
	hide the progress bar so it's ok */

#ifndef MIProgressBarDlgManager_h
#include "MIProgressBarDlgManager.h"
#endif //MIProgressBarDlgManager_h

class ZzRABI_SubTaskScope
{
public :
	ZzRABI_SubTaskScope(MIProgressBarDlgManager* progressMngr) { fProgressMngr = progressMngr; fProgressMngr->EnterSubTask(); }
	~ZzRABI_SubTaskScope() { fProgressMngr->LeaveSubTask(); }
private :	
	MIProgressBarDlgManager* fProgressMngr;
};


#ifndef ArmeDataFileRef_h
#include "ArmeDataFileRef.h"
#endif //ArmeDataFileRef_h

/*!
	@class ZzRABI_RemoveFile
	
	This class deletes the given file in its dtor, unless SaveResource() has been called

*/

class ZzRABI_RemoveFile : public ZzRABI
{
public :
	ZzRABI_RemoveFile(const char* path) { fPath = path; }
	~ZzRABI_RemoveFile() { if(fDeleteResource) { remove(fPath); } };
public :
	const char* fPath;
};


