/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#include "ZzRingBuffer.h"

ZzRingBuffer::ZzRingBuffer(SInt32 itemSize, SInt32 rbNbBits, SInt32 delta)
{
	fRingBufferNbItems = 1 << rbNbBits;
	fItemSize = itemSize;
	fRingBuffer = (UInt8*)malloc(fItemSize*fRingBufferNbItems);
	// zero the no mans land part
	memset(fRingBuffer,0,delta*itemSize);
	fWritePos = 0;
	fReadPos = 0;
}

void ZzRingBuffer::GetWriteFromFlatBufferInfo(SInt32 nbItems, UInt8** part1, SInt32* part1Size, UInt8** part2, SInt32* part2Size )
{
	ZzRtAssert(nbItems < fRingBufferNbItems);
	ZzRtAssert(nbItems < fRingBufferNbItems);
	UInt32 itemIndexInRingBuffer = fWritePos & (fRingBufferNbItems - 1);
	UInt8* dstPtr = fRingBuffer + itemIndexInRingBuffer*fItemSize;
//	vernl(4,"writing " << nbItems << " items at dstPtr " << (void*)dstPtr);
	// part 2 length in items
	SInt32 part2NbItems = itemIndexInRingBuffer + nbItems - fRingBufferNbItems;
	if(part2NbItems <= 0)
	{
		*part1 = dstPtr;
		*part2 = NULL;
		*part1Size = nbItems;
		*part2Size = 0;
	}
	else
	{
		SInt32 part1NbItems = nbItems - part2NbItems;
		*part1 = dstPtr;
		*part2 = fRingBuffer;
		*part1Size = part1NbItems;
		*part2Size = part2NbItems;
	}
}

