/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#define ZzSplitTask_h

#ifndef ZzSplitTask_h
#include "ZzSplitTask.h"
#endif //ZzSplitTask_h

#ifndef ZzTypes_h
#include "ZzTypes.h"
#endif //ZzTypes_h

#ifndef ArmeErrors_h
#include "ArmeErrors.h"
#endif //ArmeErrors_h

#ifndef ZzVerbose_h
//#include "ZzVerbose.h"
#endif //ZzVerbose_h

#ifndef ZzArray_h
#include "ZzArray.h"
#endif //ZzArray_h

class MIProgressBarDlgManager;
/*!
	@class ZzSplitTask
*/

class ZzSplitTask
{
public :
	enum
	{
		kSplitTaskNotStartedYet,
		kSplitTaskBeingPerformed,
		kSplitTaskFinished,
		kSplitTaskCancelled,
		kSplitTaskInError,
	};
			
						ZzSplitTask();
	virtual				~ZzSplitTask();
	ArmeErr				_PerformNextBit();
	void				SetProgress(float progress);
  	SInt32				GetState() const;
public :
	/*! @function _PerformNextBit Override this to actually perform your action */
	virtual ArmeErr		PerformNextBit() = 0;

	/*! @function GetDesc Provide a description for the operation */
	virtual const char*	GetDesc() const = 0;

	/*! @function Cancel Cancels the current operation and revert to initial state. must be done 
	    in agreement with the calling command. default impl crashes. */
	virtual void		Cancel();

	/*! @function CanBeCancelled Must return false if the operation cannot be cancelled, such as
	              preview computing. Default returns false. */
	virtual bool		CanBeCancelled();
	
	/*! @function IsLengthy Override this to return false is your task is fast. fast means it's not worth displaying
		a progress bar. Of course this can depend upon parameters of the split task. */
	virtual bool 		IsLengthy() const;

	
	/*! @function CanBeCancelled Must return true if the operation can be executed while the user
	              is interacting with the app. Warning : the task needs to be deleted in some conditions
			      and the protocol remains to be defined. Default impl returns false. */
	virtual bool		CanBeBackgrounded();
	
	/*! @function PerformOnce Override this to add just-before-processing initialisation. For example, to
	    open a file created by a previous split task, etc.. */
	virtual ArmeErr		PerformOnce();

	/*
		paste this for overriding : 
		
	virtual ArmeErr		PerformNextBit();
	virtual const char*	GetDesc() const;
	// at least one of Cancel or CanBeCancelled needs to return true
	virtual bool		CanBeCancelled(); //optional
	virtual void		Cancel(); //optional
	
	
	virtual bool		CanBeBackgrounded(); //optional
		
	*/

	/*! @function GetProgress Return the fProgress variable. As soon as it returns >= 1, the
	              task CAN be deleted so be careful */
	virtual float		GetProgress() const;

	/*! @function DescChanged Call this from within your ProcessNextBit function to update the display */
	virtual void		DescChanged();
	
	virtual bool		IsDescChanged();
	
	static ArmeErr		ExecuteInProgressManagerContext(ZzSplitTask* splitTask, MIProgressBarDlgManager* progressMngr);
	
protected :
	SInt32	fState;
	float	fProgress;
	bool	fDescChanged;
};

inline SInt32 ZzSplitTask::GetState() const
{
	return fState;
}

class ZzMacroSplitTask : public ZzSplitTask
{
public :
					ZzMacroSplitTask();
	virtual			~ZzMacroSplitTask();
	void			AdoptSubTask(ZzSplitTask* task);
	SInt32			GetNbSubTasks() const;
			
public :	
	virtual ArmeErr		PerformNextBit();
	virtual void		Cancel();
	virtual const char*	GetDesc() const;
	virtual float		GetProgress() const;
	virtual bool		IsDescChanged();
	// we return "false" only if there are less than 10 tasks and they ALL return false to IsLengthy()
	virtual bool 		IsLengthy() const;

	
private :
	ZzArray<ZzSplitTask*>	fSubTasks;
	SInt32					fCurTask;
};






