/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
 
#define ZzString_h
#ifndef ZzString_h
#include "ZzString.h"
#endif //ZzString_h
/*
	ZzString.h

	configuration macros : AVOID_COCOA_CODE to suppress uses of NSString. This is useful in AudioUnits
	to avoid loading the Cocoa dynamic lib which cannot be unloaded without crashes

*/

#ifndef ZzVerbose_h
#include "ZzVerbose.h"
#endif //ZzVerbose_h

/* PLATFORM */
#ifdef __APPLE__
#include <CoreFoundation/CoreFoundation.h>
#elif defined __WIN32__
#error("ZzString.h");
#endif
/* END PLATFORM */

class ZzString
{
/*
	friend global functions
*/
friend ZzString operator+(const ZzString& arg1, const ZzString& arg2);

public :
	ZzString();
/* PLATFORM */
#ifdef __APPLE__
	
	#ifdef __OBJC__
		#ifndef AVOID_COCOA_CODE
	ZzString(NSString* cfstr);
		#endif //#infdef AVOID_COCOA_CODE
	#else //__OBJC__
//	ZzString(void* cfstr);
	#endif //__OBJC__

	ZzString(CFStringRef cfstr);
	CFStringRef	getCFString() const;
#elif defined __WIN32__
	ZzString(const CString& cstr);
#endif
/* END PLATFORM */

	/*!	
		@ctor ZzString the C string encoding is assumed to always be UTF8
	*/
		ZzString(const char* utf8str);
		~ZzString();


		ZzString(const ZzString& other);
	
	/*!	
		@method getUTF8String the C ptr returned should NOT be stored. any subsequent method 
		call on the string object can invalidate it. the encoding is always kCFStringEncodingUTF8
	*/
	const char*	getUTF8String() const; 

	bool 		operator==(const ZzString& other) const;
	bool 		operator!=(const ZzString& other) const;

#ifdef __APPLE__
#ifdef __OBJC__
	#ifndef AVOID_COCOA_CODE
	/*! @method getNSString Returns an autoreleased NSString */
	NSString*	getNSString() const;
	#endif //#ifndef AVOID_COCOA_CODE
#endif //#ifdef __OBJC__
#endif //#ifdef __APPLE__

	// this is the length in number of chars. DO NOT RELY ON THIS FOR THE UTF8 STRING NUMBER OF BYTES
	SInt32		getLength() const;
	
	/*! @method fillUTF8String fills a user supplied buffer with an UTF8 encoded  C string. 
		Returns false if buffer is too small. The length of the buffer, including the
		null char that will be written, needs to be specified in the bufSize parameter */
	bool		fillUTF8String(char* buffer, UInt32 bufSize);
	
	/*! 
		@function isEqual Tells whether the UTF8 encoding representation of the string is exactly the 
		same as the parameter supplied 
	*/
	bool		isEqual(const char* szString);

public : /* operators */
	ZzString& 	operator+=(const ZzString& other);
	ZzString& 	operator=(const ZzString& other);
	
private : 
	void		invalidateCachedUTF8String();
		
private :

/* PLATFORM */
#ifdef __APPLE__
	/*! @var fCFSTR 								*/
	CFMutableStringRef	fCFSTR;
#elif defined __WIN32__
#error("ZzString.h");
#endif
/* END PLATFORM */

	/*! @var fCachedCString							 */
	mutable char*		fCachedUTF8String;
	
public :
	static ZzString		CreateFromFSRepresentation(const char* filePath);
};

inline std::ostream& operator<<(std::ostream& s, const ZzString& zzstring)
{
	s << zzstring.getUTF8String();
	return s;
}

ZzString operator+(const ZzString& arg1, const ZzString& arg2);


