/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
 
#ifndef ZzString_h
#include "ZzString.h"
#endif //ZzString_h

ZzString::ZzString(CFStringRef cfstr)
{
/* PLATFORM */
#ifdef __APPLE__
	fCFSTR = CFStringCreateMutableCopy(NULL,0,cfstr);
#elif defined __WIN32__
	ZzAssert(0);
#endif
/* END PLATFORM */
	ZzAssert(fCFSTR != NULL);
	fCachedUTF8String = NULL;
}

CFStringRef	ZzString::getCFString() const
{
	return fCFSTR;
}

ZzString::ZzString(const char* utf8str)
{
	CFStringRef immutCFSTR = CFStringCreateWithCString(NULL,utf8str,kCFStringEncodingUTF8);
	fCFSTR = CFStringCreateMutableCopy(NULL,0,immutCFSTR);
	fCachedUTF8String = NULL;
	ZzAssert(fCFSTR != NULL);
	CFRelease(immutCFSTR);
}

ZzString::~ZzString()
{
#ifdef __APPLE__
	CFRelease(fCFSTR);
#endif //#ifdef __APPLE__
	invalidateCachedUTF8String();
}

SInt32 ZzString::getLength() const
{
#ifdef __APPLE__
	CFIndex length = CFStringGetLength(fCFSTR);
	return length;
#else //__APPLE__
	#error("ZzString::getLength()")
#endif //__APPLE__
}
	
const char*	ZzString::getUTF8String() const
{
	if(fCachedUTF8String)
		return fCachedUTF8String;
	else
	{
#ifdef __APPLE__
#ifdef __OBJC__
		const char* utf8str = CFStringGetCStringPtr(fCFSTR,kCFStringEncodingUTF8);
		if(utf8str)
			return utf8str;
		else
		{
			CFIndex length = CFStringGetMaximumSizeForEncoding(CFStringGetLength(fCFSTR), kCFStringEncodingUTF8);
			if(length == 0)
				return "";
			fCachedUTF8String = (char*)malloc(sizeof(char)*(length + 1));
			Boolean ok = CFStringGetCString(fCFSTR,fCachedUTF8String,length+1,kCFStringEncodingUTF8);
			ZzAssert(ok);
			return fCachedUTF8String;
		}
#endif //#ifdef __OBJC__
#endif //#ifdef __APPLE__
	}
}

void ZzString::invalidateCachedUTF8String()
{
	if(fCachedUTF8String)
	{
		free(fCachedUTF8String);
		fCachedUTF8String = NULL;
	}
}

ZzString& ZzString::operator=(const ZzString& other)
{
	if(&other == this)
		return *this;
	invalidateCachedUTF8String();
#ifdef __APPLE__
	CFStringRef ourString = fCFSTR;  // (**)
	fCFSTR = CFStringCreateMutableCopy(NULL,0,other.fCFSTR);
	CFRelease(ourString); // (**) and this..... i know these are not needed because of the "if(&other == this) return *this;" 
	// but it's nice to remain trained to good-style code.
#endif //#ifdef __APPLE__
	return *this;
}

bool ZzString::operator==(const ZzString& other) const
{
	CFComparisonResult compRes = CFStringCompare(other.getCFString(),fCFSTR,0);
	return(compRes == kCFCompareEqualTo);
}

bool ZzString::operator!=(const ZzString& other) const
{
	return (!(*this == other));
}

bool ZzString::isEqual(const char* utf8String)
{
#ifdef __APPLE__
	CFStringRef immutOther = CFStringCreateWithCString(NULL,utf8String,kCFStringEncodingUTF8);
	CFComparisonResult compRes = CFStringCompare(immutOther,fCFSTR,0);
	CFRelease(immutOther);
	if(compRes == kCFCompareEqualTo)
		return true;
	else
		return false;
#else //plaftorm
	#error("ZzString::isEqual");
	return false;
#endif //plaftorm
}

ZzString::ZzString()
{
#ifdef __APPLE__
	fCFSTR = CFStringCreateMutable(NULL,32);
	ZzAssert(fCFSTR != NULL);
	fCachedUTF8String = NULL;
#else //plaftorm
	#error("ZzString::ZzString()");
#endif //plaftorm
}


ZzString operator+(const ZzString& arg1, const ZzString& arg2)
{
	//fCFSTR
	ZzString retVal = arg1;
	retVal += arg2;
	return retVal;
}

ZzString& ZzString::operator+=(const ZzString& other)
{
#ifdef __APPLE__
	CFStringAppend(this->fCFSTR,other.fCFSTR);
#else
	#error("ZzString& ZzString::operator+=(const ZzString& other)");
#endif //PLAFTORM
	return *this;
}
	
ZzString::ZzString(const ZzString& other)
{
	fCFSTR = CFStringCreateMutableCopy(
		NULL, // default allocator
		0, // no maximum size specified for the new object
		other.fCFSTR // the other string
		);
	ZzAssert(fCFSTR != NULL);
	// retain count is now 1 which is ok.
	fCachedUTF8String = NULL;
}

#ifndef AVOID_COCOA_CODE
ZzString::ZzString(NSString* cfstr)
{
/* PLATFORM */
#ifdef __APPLE__
	fCFSTR = CFStringCreateMutableCopy(NULL,0,(CFStringRef)cfstr);
#elif defined __WIN32__
	ZzAssert(0);
#endif
/* END PLATFORM */
	ZzAssert(fCFSTR != NULL);
	fCachedUTF8String = NULL;
}
#endif //#ifndef AVOID_COCOA_CODE


#ifdef __APPLE__
#ifndef AVOID_COCOA_CODE
NSString* ZzString::getNSString() const
{
	NSString* _string = (NSString*)CFStringCreateMutableCopy(
		NULL, // default allocator
		0, // no maximum size specified for the new object
		fCFSTR // the other string
		);
	return [_string autorelease];
}
#endif //#ifndef AVOID_COCOA_CODE
#endif //#ifdef __APPLE__

#ifdef __APPLE__
ZzString ZzString::CreateFromFSRepresentation(const char* filePath)
{
#ifndef AVOID_COCOA_CODE
	NSString* nsString = [[NSFileManager defaultManager] stringWithFileSystemRepresentation:filePath length:strlen(filePath)];
	return ZzString(nsString);
#else //#ifndef AVOID_COCOA_CODE
	ZzAssert(0);
	return ZzString("");
#endif //#ifndef AVOID_COCOA_CODE
}
	
#endif //#ifdef __APPLE__
