/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#define ZzTimer_h

#ifndef ZzTimer_h
#include "ZzTimer.h"
#endif ZzTimer_h

#ifndef ZzTypes_h
#include "ZzTypes.h"
#endif //ZzTypes_h

#ifdef __APPLE__
#include <CoreServices/CoreServices.h>
#elif defined _MSC_VER
#include <mmsystem.h>
#else // unknown platform
#error Please implement ZzTimer for your platform
#endif // platform

// WARNING : this is not a high-performance counter in Windows. to fix this, one 
// needs to use QueryPerformanceCounter and QueryPerformanceFrequency (but this
// needs to be done at init time)
class ZzTimer
{
public :
	                ZzTimer();
	SInt32 	        GetMS( ) const;
	float 	        GetMSFloat( ) const;
	void	          Reset( );
	UInt64	        GetUS( ) const;
  static  UInt64  CurrentUS( );
public :
	UInt64	        fStartVal;
};

///System/Library/Frameworks/CoreServices.framework/Versions/A/Frameworks/CarbonCore.framework/Versions/A/Headers/MacTypes.h

inline UInt64 ZzTimer::CurrentUS( )
{
	UInt64 retVal;
#ifdef __APPLE__
	Microseconds((UnsignedWide*)(&retVal));
#elif defined _WIN32
  retVal = timeGetTime( )*1000LL;
#else // unknown platform
#error Please implement ZzTimer for your platform
#endif // platform
  return retVal;
}

inline ZzTimer::ZzTimer()
{
  fStartVal = CurrentUS( );
}

inline SInt32 ZzTimer::GetMS() const
{
  UInt64 tmp = CurrentUS( );
	return (SInt32)((tmp - fStartVal)/1000LL);
}

inline UInt64 ZzTimer::GetUS() const
{
  UInt64 tmp = CurrentUS( );
	return (SInt32)(tmp - fStartVal);
}

inline float ZzTimer::GetMSFloat() const
{
  UInt64 tmp = CurrentUS( );
	return (float)((double)(tmp - fStartVal)/1000.0);
}

inline void ZzTimer::Reset()
{
  fStartVal = CurrentUS( );
}

/*
inline void ZzTimerStart(int threadIndex, int indentLevel)
{
	ZzTimer::ms_globalTimers[10*threadIndex+indentLevel].Reset();
}

//char* ZzTimerEnd(int threadIndex, char* dstBuff, int indentLevel, const char* topicName);

inline char* ZzTimerEnd(int threadIndex, char* dstBuff, int indentLevel, const char* topicName)
{
	UInt64 us = ZzTimer::ms_globalTimers[10*threadIndex+indentLevel].GetUS();
	int charsWritten = sprintf(dstBuff,"%s%s TOOK %d MICROSECONDS\n",ZzTimer::ms_indentStrings[indentLevel],topicName,(UInt32)us);
	return (dstBuff + charsWritten);
}

#define ZzTimerPrint(topicName,threadIndex,indentLevel){char text[1024]; ZzTimerEnd(threadIndex,text,indentLevel,topicName); vernl(4,text); }
*/

#define ZZTIMERS_USED_HERE char* ZZTIMERS_topicName; ZzTimer ZZTIMERS_timer; UInt64 ZZTIMERS_us;

#define ZZSTART_TIMER(topicName) ZZTIMERS_topicName = topicName
#define ZZEND_TIMER() { ZZTIMERS_us = ZZTIMERS_timer.GetUS(); vernl(4,ZZTIMERS_topicName << " TIMING : " << ZZTIMERS_us << " MICROSECONDS"); }


