/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

/*
	A version of ZzArchPtrArray that adds dynamic_cast on getters to
	allow for cleaner code. The template parameter is the type of the
	pointer you want to retrieve from the collection. The stored objects
	must be dynamic_cast-able to this type, of course. The type must be
	the pointer type (such as TVideoEffect*), not the non-pointer type
	(TVideoEffect)
*/

#define ZzTypedArchPtrArray_h

#ifndef ZzTypedArchPtrArray_h
#include "ZzTypedArchPtrArray.h"
#endif //ZzTypedArchPtrArray_h

#ifndef ZzArchPtrArray_h
#include "ZzArchPtrArray.h"
#endif //ZzArchPtrArray_h

template <class T> class ZzTypedArchPtrArray : public ZzArchPtrArray
{
public:
	// returns a a copy of the item at position i and returns 
	T			ItemAt( SInt32 i ) const;
	T			operator[]( SInt32 i ) const { return dynamic_cast<T>(ItemAt(i)); };
	
	// adds a new item at the end of the array
	void		AddItem( T derived );
		
	/*! @function FindIndexForItem Returns the FIRST index of an item inside the array or -1 if item can't be found. */
	SInt32		FindIndexForItem(T derived) const;

	/*! @function RemoveItemByVal Remove the FIRST item with the given value */
	void		RemoveItemByValue(T derived);
};

template<class T> inline T ZzTypedArchPtrArray<T>::ItemAt( SInt32 i ) const
{
	ArmeArchivable* base = ZzArchPtrArray::ItemAt(i);
	T derived = dynamic_cast<T>(base);
	ZzAssert(derived != NULL);
	return derived;
}
	
template<class T> inline void ZzTypedArchPtrArray<T>::AddItem( T derived )
{
	ArmeArchivable* base = dynamic_cast<ArmeArchivable*>(derived);
	ZzAssert(base != NULL);
	ZzArchPtrArray::AddItem(base);
	ZzAssert(0);
}

template<class T> inline SInt32 ZzTypedArchPtrArray<T>::FindIndexForItem(T derived) const
{
	ArmeArchivable* base = dynamic_cast<ArmeArchivable*>(derived);
	ZzAssert(base != NULL);
	return ZzArchPtrArray::FindIndexForItem(base);
}

template<class T> inline void ZzTypedArchPtrArray<T>::RemoveItemByValue(T derived)
{
	ArmeArchivable* base = dynamic_cast<ArmeArchivable*>(derived);
	ZzAssert(base != NULL);
	ZzArchPtrArray::RemoveItemByValue(base);
}
