/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#ifdef __cplusplus

#define ZzTypes_h

#ifndef ZzTypes_h
#include "ZzTypes.h"
#endif //ZzTypes_h

#ifdef __APPLE__

  #include <Quicktime/Quicktime.h>

  #ifdef __OBJC__
    #include <Cocoa/Cocoa.h>
  #endif

  #ifndef NULL
    #define NULL 0
  #endif //NULL

#endif //__APPLE__

#ifdef WIN32

  #ifndef NULL
    #define NULL 0
  #endif //NULL

  typedef unsigned __int8 UInt8;
  typedef unsigned __int16 UInt16;
  typedef unsigned __int32 UInt32;
  typedef unsigned __int64 UInt64;

  typedef __int8 SInt8;
  typedef __int16 SInt16;
  typedef __int32 SInt32;
  typedef __int64 SInt64;

#elif defined __APPLE__

/*
	some constants.
*/

  #define kMaxPathLength 1024
  #define kMaxFileLength 256

  typedef UInt8 uint8;
  typedef UInt16 uint16;
  typedef UInt32 UInt32;
  typedef UInt64 uint64;
  typedef SInt8 int8;
  typedef SInt16 int16;
  typedef SInt32 int32;
  typedef SInt64 int64;

#else

  #error("check platform symbols in ZzTypes.h")

#endif 

/* 
	$$$international 
*/
typedef char		ZzChar;
typedef UInt32		ZzBool;
typedef char		ZzStr64[64];
typedef const char*	ZzStrCPtr;
typedef char* 		ZzStrPtr;

//ZzString64 : length --> 01234567890123456789012345678901234567890123456789012345678901234

typedef struct ZzGUID 
{
	UInt8	_data[8];
} ZzGUID;

/*
	misc helperz
*/
void ZzCopyStr64( ZzStrPtr dest, ZzStrCPtr src );

#define ArboPi 3.1415926535897932384626433832795

#ifndef min
template<typename T> inline T min(const T& a, const T& b)
{
	if(a < b)
		return a;
	return b;
}
#endif //#ifndef min

#ifndef max
template<typename T> inline T max(const T& a, const T& b)
{
	if(a > b)
		return a;
	return b;
}
#endif //#ifndef max

#if (!defined(M_PI))
  #define M_PI 3.14159265358979323846264338327
  #define M_PI_2 (M_PI/2.0)
#endif

/*
	file ref functions (USING PATH RIGHT NOW$$$)
*/
// converts a path to a file ref. The file may not exist yet
//int				ZzCreateFileRefFromPath( ZzFileRef fileRef, char* path );
//int				ZzFreeFileRef( ZzFileRef fileRef );

#endif //__cplusplus
