/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/
 
#define ZzVerbose_h
#ifndef ZzVerbose_h
#include "ZzVerbose.h"
#endif //ZzVerbose_h


// uncommenting next line disable verbose and debug calls
#define NO_VERBOSE_AT_ALL
#define LOG_USER_COMMAND_TO_STANDARD_OUTPUT
#define LOG_USER_COMMAND_TO_LOG_FILE
#define ZZVERBOSE_USE_STRINGSTREAM

/*
	
	THE MAIN MACROS CALLS
	
	ver(stream)		: sends the stream to file or console, depending on runtime setting (see below)
					  it uses the VER_DEF_TOPIC macro for the topic of the message, the VER_DEF_LEVEL
					  macro for the level of the message. No endl is added to the stream...
					  Please note that these macros MUST be defined, otherwise it won't compile.
	vern(stream)	: same as ver() except that an endl is added
	vver(var)		: executes ver(#var < " = " < var)
	vvern(var)		: executes vern(#var < " = " < var)
	
	---------------------------------------------------------------------------------------------------
	THE OTHER MACROS
		
	vert(topic,stream)	: 
	vernt(topic,stream)	: 
	vvert(topic,stream) : 
	vvernt(topic,stream) : 
	--> these four macros are similar to the main ones, except that the topic is specified, instead of using
	    the VER_DEF_TOPIC macro
	
	verl(level,stream)	: 
	vernl(level,stream)	: 
	vverl(level,stream) : 
	vvernl(level,stream) : 
	--> these four macros are similar the main ones, except that the level is specified, instead of using
	    the VER_DEF_LEVEL macro
				
	
	vertl(topic,level,stream)	: 
	verntl(topic,level,stream)	: 
	vvertl(topic,level,stream) 	: 
	vverntl(topic,level,stream)	: 
	--> these four macros are similar to the main ones, except that the topic is specified, instead of using
	    the VER_DEF_TOPIC macro, and the level is specified, instead of using the VER_DEF_LEVEL macro. Level
		is between 0 and 6 with 
		0 being critically important
		6 not important at all
		
		A message is allowed to be sent to cout or to the log file if its level is less or equal than the
		configured level for that topic. Since the level can't be less than zero, critically important messages
		can't be blocked.
			
	MACRO SYMBOLS
	
	NO_VERBOSE_AT_ALL : If it's defined, then all the verbose macros expand to nothing.
		
	VER_DEF_TOPIC the current default topic (don't forget to undefine it at the end of your 
	              header file if you're using it into inlined functions)
	
	VER_DEF_LEVEL the current default level (don't forget to undefine it at the end of your
	                  header file if you're using it into inlined functions)
	
	CONFIG MACRO CALLS (run-time configuration)
	
	SET_VER_OUTPUT(flags)
	ADD_VER_OUTPUT(flags)
	REMOVE_VER_OUTPUT(flags)
	SET_MIN_LEVEL_FOR_ALL_TOPICS(level) remark : set level to something big (1000) to disable 
												 verbose at run-time
	DISCARD_ALL_TOPICS() : puts the level to 1000 for all topics
	SET_MIN_LEVEL_FOR_TOPIC(level,topic)
	
	output flags can be a combination of kVERScreen and kVERFile
	the file is always verbose.txt in the application folder
	
	you need to include a file that defines the set of kVERTopic_XXXX constants !
*/

#if (defined(_WIN32) && !defined(NO_VERBOSE_AT_ALL)) || (!defined(_WIN32))

#ifndef ZzTypes_h
#include "ZzTypes.h"
#endif //ZzTypes_h

#ifndef ZzLocker_h
#include "ZzLocker.h"
#endif //ZzLocker_h

#include <iostream>
#include <fstream>
#include <sstream>
//#include <strstream>
//#include "/usr/include//gcc/darwin/3.1/g++-v3/backward/strstream"

#if defined(LOG_USER_COMMAND_TO_STANDARD_OUTPUT) || defined(LOG_USER_COMMAND_TO_LOG_FILE)
extern char tmpStringForCommandLog[1024];
extern void LogUserCommand(const char* msg);

#ifdef LOG_USER_COMMAND_TO_LOG_FILE
extern char* UserCommandsLogFileName;
#endif //LOG_USER_COMMAND_TO_LOG_FILE


#define VUSERCMD(msg) {LogUserCommand(msg);}
#define VUSERCMD1(msg, arg1) {sprintf(tmpStringForCommandLog, msg, arg1); LogUserCommand(tmpStringForCommandLog);}
#define VUSERCMD2(msg, arg1, arg2) {sprintf(tmpStringForCommandLog, msg, arg1, arg2); LogUserCommand(tmpStringForCommandLog);}
#define VUSERCMD3(msg, arg1, arg2, arg3) {sprintf(tmpStringForCommandLog, msg, arg1, arg2, arg3); LogUserCommand(tmpStringForCommandLog);}
#define VUSERCMD4(msg, arg1, arg2, arg3, arg4) {sprintf(tmpStringForCommandLog, msg, arg1, arg2, arg3, arg4); LogUserCommand(tmpStringForCommandLog);}
#define VUSERCMD5(msg, arg1, arg2, arg3, arg4, arg5) {sprintf(tmpStringForCommandLog, msg, arg1, arg2, arg3, arg4, arg5); LogUserCommand(tmpStringForCommandLog);}
#else //(LOG_USER_COMMAND_TO_STANDARD_OUTPUT || LOG_USER_COMMAND_TO_LOG_FILE)

#define VUSERCMD(msg) {}
#define VUSERCMD1(msg, arg1) {}
#define VUSERCMD2(msg, arg1, arg2) {}
#define VUSERCMD3(msg, arg1, arg2, arg3) {}
#define VUSERCMD4(msg, arg1, arg2, arg3, arg4) {}
#define VUSERCMD5(msg, arg1, arg2, arg3, arg4, arg5) {}

#endif //(LOG_USER_COMMAND_TO_STANDARD_OUTPUT || LOG_USER_COMMAND_TO_LOG_FILE)


#ifndef NO_VERBOSE_AT_ALL

#define VER_DEF_LEVEL 3
#define VER_DEF_TOPIC NULLTopic


enum
{
	kVERFile = 1,
	kVERScreen = 2
};

/*
	stream operator for objc classes
*/
#ifdef __OBJC__

inline std::ostream& operator<<(std::ostream& s, id obj)
{
	if(obj == nil)
		return s << "<objc_object* obj == nil>";
	else
		return s << [[obj description] cString];
}

inline std::ostream& operator<<(std::ostream& s, const NSRect& rect)
{
	return s << "(x =" << rect.origin.x << " y =" << rect.origin.y << " w = " << rect.size.width << " h = " << rect.size.height << ")";
}

inline std::ostream& operator<<(std::ostream& s, const NSPoint& pt)
{
	return s << "(x =" << pt.x << " y =" << pt.y << ")";
}

#endif

inline std::ostream& operator<<(std::ostream& s, const FSSpec& fss)
{
	char fileName[kMaxPathLength];
	CopyPascalStringToC(fss.name,fileName);
	return s << "<FSSpec = " << fileName << ">";
}

class ZzVerbose
{
	static ZzVerbose		fsInstance;  // create just to be able to close the file when exiting
public :
	static ZzLocker			fsLock; // serializes access
	static const char*		fsTopicStrings[kVERTopic_LastTopic + 1];
	static const char*		fsStringOfSpaces;
	static UInt32			fsTopicLevels[kVERTopic_LastTopic + 1];

	static std::ofstream*		fsFileOutputStream;

#ifdef ZZVERBOSE_USE_STRINGSTREAM
	static std::stringstream	fsStringStream;
#else //#ifdef ZZVERBOSE_USE_STRINGSTREAM
	static std::ostream*		fsScreenOutputStream;
#endif //#ifdef ZZVERBOSE_USE_STRINGSTREAM

	static UInt32			fsActiveOutputs;

	static void SetLevel(UInt32 topic, UInt32 level) {fsTopicLevels[topic] = level;};
	static void SetAllLevels(UInt32 level);
	static void SetScreenOutputStream(std::ostream& aStream); 
	static void SetOutputFilePath(const char* path = "verbose.txt");
	static void EnableOutput(UInt32 flags) {fsActiveOutputs |= flags; };
	static void DisableOutput(UInt32 flags) {fsActiveOutputs &= ~flags; };

	ZzVerbose();
	~ZzVerbose();
};

#define ver(stream) vertl(VER_DEF_TOPIC,VER_DEF_LEVEL,stream)
#define vver(stream) vvertl(VER_DEF_TOPIC,VER_DEF_LEVEL,stream)
#define vern(stream) verntl(VER_DEF_TOPIC,VER_DEF_LEVEL,stream)
#define vvern(stream) vverntl(VER_DEF_TOPIC,VER_DEF_LEVEL,stream)
//#define vert(topic,stream) vertl(topic,VER_DEF_LEVEL,stream)
#define vvert(topic,stream) vvertl(topic,VER_DEF_LEVEL,stream)
#define vernt(topic,stream) verntl(topic,VER_DEF_LEVEL,stream)
#define vvernt(topic,stream) vverntl(topic,VER_DEF_LEVEL,stream)
#define verl(level,stream) vertl(VER_DEF_TOPIC,level,stream)
#define vverl(level,stream) vvertl(VER_DEF_TOPIC,level,stream)
#define vernl(level,stream) verntl(VER_DEF_TOPIC,level,stream)
#define vvernl(level,stream) vverntl(VER_DEF_TOPIC,level,stream)

#define ZZVERBOSE_TOPIC(a) kVERTopic_##a
#define ZZVERBOSE_STRINGIZE(a) #a

#ifdef ZZVERBOSE_USE_STRINGSTREAM

#define vertl(topic , level, stream) { \
	ZzVerbose::fsLock.Lock(); \
	if ( ZzVerbose::fsActiveOutputs & kVERScreen && level <= ZzVerbose::fsTopicLevels[ZZVERBOSE_TOPIC(topic)] ) \
	{\
		ZzVerbose::fsStringStream << ZZVERBOSE_STRINGIZE(topic) << "(" << level << ") : " << stream; \
		printf(ZzVerbose::fsStringStream.str().data()); \
		ZzVerbose::fsStringStream.str(""); \
	}\
	if ( ZzVerbose::fsActiveOutputs & kVERFile && level <= ZzVerbose::fsTopicLevels[ZZVERBOSE_TOPIC(topic)] ) \
	{ \
		if ( ZzVerbose::fsFileOutputStream->is_open() ) \
		{\
			*ZzVerbose::fsFileOutputStream << ZZVERBOSE_STRINGIZE(topic) << "(" << level << ") : " << stream << std::flush; \
		}\
		else \
		{\
			*((int*)0) = 0;\
		}\
	} \
	ZzVerbose::fsLock.Unlock(); \
}

#define verntl(topic , level, stream) { \
	ZzVerbose::fsLock.Lock(); \
	if ( ZzVerbose::fsActiveOutputs & kVERScreen && level <= ZzVerbose::fsTopicLevels[ZZVERBOSE_TOPIC(topic)] ) \
	{\
		ZzVerbose::fsStringStream << ZZVERBOSE_STRINGIZE(topic) << "(" << level << ") : " << stream << std::endl; \
		printf(ZzVerbose::fsStringStream.str().data()); \
		ZzVerbose::fsStringStream.str(""); \
	}\
	if ( ZzVerbose::fsActiveOutputs & kVERFile && level <= ZzVerbose::fsTopicLevels[ZZVERBOSE_TOPIC(topic)] ) \
	{ \
		if ( ZzVerbose::fsFileOutputStream->is_open() ) \
		{\
			*ZzVerbose::fsFileOutputStream << ZZVERBOSE_STRINGIZE(topic) << "(" << level << ") : " << stream << std::endl << std::flush; \
		}\
		else \
		{\
			*((int*)0) = 0;\
		}\
	} \
	ZzVerbose::fsLock.Unlock(); \
}


#define vvertl(topic , level, var) { \
	ZzVerbose::fsLock.Lock(); \
	if ( ZzVerbose::fsActiveOutputs & kVERScreen && level <= ZzVerbose::fsTopicLevels[ZZVERBOSE_TOPIC(topic)] ) \
	{\
		ZzVerbose::fsStringStream << ZZVERBOSE_STRINGIZE(topic) << "(" << level << ") : " << #var << " = " << var; \
		printf(ZzVerbose::fsStringStream.str().data()); \
		ZzVerbose::fsStringStream.str(""); \
	}\
	if ( ZzVerbose::fsActiveOutputs & kVERFile && level <= ZzVerbose::fsTopicLevels[ZZVERBOSE_TOPIC(topic)] ) \
	{ \
		if ( ZzVerbose::fsFileOutputStream->is_open() ) \
		{\
			*ZzVerbose::fsFileOutputStream << ZZVERBOSE_STRINGIZE(topic) << "(" << level << ") : " << #var << " = " << var << std::flush; \
		}\
		else \
		{\
			*((int*)0) = 0;\
		}\
	} \
	ZzVerbose::fsLock.Unlock(); \
}

#define vverntl(topic , level, var) { \
	ZzVerbose::fsLock.Lock(); \
	if ( ZzVerbose::fsActiveOutputs & kVERScreen && level <= ZzVerbose::fsTopicLevels[ZZVERBOSE_TOPIC(topic)] ) \
	{\
		ZzVerbose::fsStringStream << ZZVERBOSE_STRINGIZE(topic) << "(" << level << ") : " << #var << " = " << var << std::endl; \
		printf(ZzVerbose::fsStringStream.str().data()); \
		ZzVerbose::fsStringStream.str(""); \
	}\
	if ( ZzVerbose::fsActiveOutputs & kVERFile && level <= ZzVerbose::fsTopicLevels[ZZVERBOSE_TOPIC(topic)] ) \
	{ \
		if ( ZzVerbose::fsFileOutputStream->is_open() ) \
		{\
			*ZzVerbose::fsFileOutputStream << ZZVERBOSE_STRINGIZE(topic) << "(" << level << ") : " << #var << " = " << var << std::endl << std::flush; \
		}\
		else \
		{\
			*((int*)0) = 0;\
		}\
	} \
	ZzVerbose::fsLock.Unlock(); \
}


#else //#ifdef ZZVERBOSE_USE_STRINGSTREAM

#define vertl(topic , level, stream) { \
	ZzVerbose::fsLock.Lock(); \
	if ( ZzVerbose::fsActiveOutputs & kVERScreen && level <= ZzVerbose::fsTopicLevels[ZZVERBOSE_TOPIC(topic)] ) \
	{\
		*ZzVerbose::fsScreenOutputStream << ZZVERBOSE_STRINGIZE(topic) << "(" << level << ") : " << stream; \
	}\
	if ( ZzVerbose::fsActiveOutputs & kVERFile && level <= ZzVerbose::fsTopicLevels[ZZVERBOSE_TOPIC(topic)] ) \
	{ \
		if ( ZzVerbose::fsFileOutputStream->is_open() ) \
		{\
			*ZzVerbose::fsFileOutputStream << ZZVERBOSE_STRINGIZE(topic) << "(" << level << ") : " << stream << std::flush; \
		}\
		else \
		{\
			*((int*)0) = 0;\
		}\
	} \
	ZzVerbose::fsLock.Unlock(); \
}

#define verntl(topic , level, stream) { \
	ZzVerbose::fsLock.Lock(); \
	if ( ZzVerbose::fsActiveOutputs & kVERScreen && level <= ZzVerbose::fsTopicLevels[ZZVERBOSE_TOPIC(topic)] ) \
	{\
		*ZzVerbose::fsScreenOutputStream << ZZVERBOSE_STRINGIZE(topic) << "(" << level << ") : " << stream << std::endl; \
	}\
	if ( ZzVerbose::fsActiveOutputs & kVERFile && level <= ZzVerbose::fsTopicLevels[ZZVERBOSE_TOPIC(topic)] ) \
	{ \
		if ( ZzVerbose::fsFileOutputStream->is_open() ) \
		{\
			*ZzVerbose::fsFileOutputStream << ZZVERBOSE_STRINGIZE(topic) << "(" << level << ") : " << stream << std::endl << std::flush; \
		}\
		else \
		{\
			*((int*)0) = 0;\
		}\
	} \
	ZzVerbose::fsLock.Unlock(); \
}


#define vvertl(topic , level, var) { \
	ZzVerbose::fsLock.Lock(); \
	if ( ZzVerbose::fsActiveOutputs & kVERScreen && level <= ZzVerbose::fsTopicLevels[ZZVERBOSE_TOPIC(topic)] ) \
	{\
		*ZzVerbose::fsScreenOutputStream << ZZVERBOSE_STRINGIZE(topic) << "(" << level << ") : " << #var << " = " << var; \
	}\
	if ( ZzVerbose::fsActiveOutputs & kVERFile && level <= ZzVerbose::fsTopicLevels[ZZVERBOSE_TOPIC(topic)] ) \
	{ \
		if ( ZzVerbose::fsFileOutputStream->is_open() ) \
		{\
			*ZzVerbose::fsFileOutputStream << ZZVERBOSE_STRINGIZE(topic) << "(" << level << ") : " << #var << " = " << var << std::flush; \
		}\
		else \
		{\
			*((int*)0) = 0;\
		}\
	} \
	ZzVerbose::fsLock.Unlock(); \
}

#define vverntl(topic , level, var) { \
	ZzVerbose::fsLock.Lock(); \
	if ( ZzVerbose::fsActiveOutputs & kVERScreen && level <= ZzVerbose::fsTopicLevels[ZZVERBOSE_TOPIC(topic)] ) \
	{\
		*ZzVerbose::fsScreenOutputStream << ZZVERBOSE_STRINGIZE(topic) << "(" << level << ") : " << #var << " = " << var << std::endl; \
	}\
	if ( ZzVerbose::fsActiveOutputs & kVERFile && level <= ZzVerbose::fsTopicLevels[ZZVERBOSE_TOPIC(topic)] ) \
	{ \
		if ( ZzVerbose::fsFileOutputStream->is_open() ) \
		{\
			*ZzVerbose::fsFileOutputStream << ZZVERBOSE_STRINGIZE(topic) << "(" << level << ") : " << #var << " = " << var << std::endl << std::flush; \
		}\
		else \
		{\
			*((int*)0) = 0;\
		}\
	} \
	ZzVerbose::fsLock.Unlock(); \
}

#endif //#ifdef ZZVERBOSE_USE_STRINGSTREAM

#define verUserAction(msg) verntl(UserAction, 1, msg)

void _ZzDisplayAssertErrorMsg(const char* cond, const char* msg, const char* file, int line);

#define ZzAssertMsg(a,msg) { if(!(a)){ _ZzDisplayAssertErrorMsg(#a,msg,__FILE__,__LINE__);}}
//#define ZzAssert(a){if(!(a)){verntl(VER_DEF_TOPIC,0,#a<<" condition failed in "<<__FILE__<<" at line "<<__LINE__);*((int*)0) = 1990;}}
#define ZzAssert(a) { if(!(a)){ _ZzDisplayAssertErrorMsg(#a,"(fatal error)",__FILE__,__LINE__);}}
#define ZzVerbose_SetOutputFilePath(a) ZzVerbose::SetOutputFilePath(a)

#else // if NO_VERBOSE_AT_ALL is defined

#define ZzVerbose_SetOutputFilePath(a) while(0){}

/*
#define ver(stream) while(0){}
#define vern(stream) while(0){}
#define vver(var) while(0){}
#define vvern(var) while(0){}

#define verl(level,stream) while(0){}
#define vernl(level,stream) while(0){}
#define vverl(level,stream) while(0){}
#define vvernl(level,stream) while(0){}

#define vertl(topic , level, stream) while(0){}
#define verntl(topic , level, stream) while(0){}
#define vvertl(topic , level, stream) while(0){}
#define vverntl(topic , level, stream) while(0){}

#define ZzAssert(a) while(0){}
#define ZzAssertMsg(a,msg) while(0){}
*/

#define ver(stream) {}
#define vern(stream) {}
#define vver(var) {}
#define vvern(var) {}

#define verl(level,stream) {}
#define vernl(level,stream) {}
#define vverl(level,stream) {}
#define vvernl(level,stream) {}

#define vertl(topic , level, stream) {}
#define verntl(topic , level, stream) {}
#define vvertl(topic , level, stream) {}
#define vverntl(topic , level, stream) {}


#define ZzAssert(a) {}
#define ZzAssertMsg(a,msg) {}


#endif // whether NO_VERBOSE_AT_ALL is defined or not


#endif // whether NO_VERBOSE_AT_ALL is defined or not
