/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#ifndef ZzVerbose_h
#include "ZzVerbose.h"
#endif //ZzVerbose_h

#ifdef LOG_USER_COMMAND_TO_STANDARD_OUTPUT || LOG_USER_COMMAND_TO_LOG_FILE
char tmpStringForCommandLog[1024];

#ifdef LOG_USER_COMMAND_TO_LOG_FILE
char* UserCommandsLogFileName = NULL;
#endif //LOG_USER_COMMAND_TO_LOG_FILE

void LogUserCommand(const char* msg)
{
	struct timeval tp;
	char temp[256];
	gettimeofday(&tp, NULL);
	ctime_r((const time_t*)&(tp.tv_sec), temp);
	//remove trailing '\n'
	temp[strlen(temp)-1] = 0;

#ifdef LOG_USER_COMMAND_TO_STANDARD_OUTPUT
	printf("USRCMD %s : %s\n", temp, msg);
#endif

#ifdef LOG_USER_COMMAND_TO_LOG_FILE
	if (UserCommandsLogFileName != NULL)
	{
		SInt32 fileSize = 0;
		FILE* fp = fopen(UserCommandsLogFileName, "a+t");
		if (fp)
		{
			fprintf(fp, "USRCMD %s : %s\n", temp, msg);
			fileSize = ftell(fp);
			fclose(fp);
		}
		//if log file is getting too big :
		//bazooka solution :
		if (fileSize > 128000)
		{
			char* temp = new char[4096*256];
			int i = 0;
			
			//read the content of file
			fp = fopen(UserCommandsLogFileName, "rb");
			while (!feof(fp) && (i < 4096))
			{
				fgets(&(temp[i++*256]), 256, fp);
			}
			fclose(fp);

			//delete log file
			remove(UserCommandsLogFileName);

			//rewrite the content without the X first lines
			FILE* fp = fopen(UserCommandsLogFileName, "a+t");
			if (fp)
			{
				for (int j=100; j < i; ++j)
					fprintf(fp, "%s", &(temp[j*256]));
				fclose(fp);
			}
			delete [] temp;
		}
	}
#endif

}
#endif //(LOG_USER_COMMAND_TO_STANDARD_OUTPUT || LOG_USER_COMMAND_TO_LOG_FILE)


#ifndef NO_VERBOSE_AT_ALL

ZzVerbose	ZzVerbose::fsInstance;

#ifdef ZZVERBOSE_USE_STRINGSTREAM
std::stringstream	ZzVerbose::fsStringStream;
#else //#ifdef ZZVERBOSE_USE_STRINGSTREAM
std::ostream*		ZzVerbose::fsScreenOutputStream = NULL;
#endif //#ifdef ZZVERBOSE_USE_STRINGSTREAM

std::ofstream*	ZzVerbose::fsFileOutputStream = NULL;
UInt32			ZzVerbose::fsActiveOutputs;
const char* 	ZzVerbose::fsStringOfSpaces = "                  ";

#ifndef THIS_IS_NOT_MONTAGE
const char* ZzVerbose::fsTopicStrings[kVERTopic_LastTopic + 1] =
{
#ifdef VER_TOPIC
#error ("VER_TOPIC already defined")
#endif //VER_TOPIC
#define VER_TOPIC(a) #a,
	"NULLTopic",
#include "ArmeVerboseTopicsDefs.h"
#undef VER_TOPIC
	"LastTopic"
};
#endif //#ifndef THIS_IS_NOT_MONTAGE

// the lock that serializes access to the ver macros
ZzLocker ZzVerbose::fsLock;

UInt32 ZzVerbose::fsTopicLevels[kVERTopic_LastTopic + 1];

/*
Change the verbose level of all categories
*/
void ZzVerbose::SetAllLevels(UInt32 level)
{
	for (UInt32 i=0; i < kVERTopic_LastTopic; ++i)
		SetLevel(i, level);
}

/*
Change the stream on which verbose is output
  */
void ZzVerbose::SetScreenOutputStream(std::ostream& aStream)
{
	fsActiveOutputs = kVERScreen;
#ifndef ZZVERBOSE_USE_STRINGSTREAM
	fsScreenOutputStream = &aStream;
#endif //#ifndef ZZVERBOSE_USE_STRINGSTREAM
}

/*
Change the file on which verbose is output
*/
void ZzVerbose::SetOutputFilePath(const char* path)
{
	if(fsFileOutputStream)
		delete fsFileOutputStream;
	
	if(path)
		fsFileOutputStream = new std::ofstream(path);
}


// constructor simply initializes output stream
ZzVerbose::ZzVerbose()
{
	SetScreenOutputStream(std::cerr);
	EnableOutput(kVERScreen);
	SetAllLevels(1);
}


// destructor simply closes the file if it is open
ZzVerbose::~ZzVerbose()
{
	if (fsFileOutputStream)
		delete fsFileOutputStream;
}

void _ZzDisplayAssertErrorMsg(const char* cond, const char* msg, const char* file, int line)
{
#ifndef AVOID_COCOA_CODE	
	NSString* message = [NSString stringWithFormat:@"Condition %s failed - %s - file : %s - line : %d \n What shall we do ?",cond,msg,file,line];
	int ret = NSRunAlertPanel(@"FATAL ERROR",message,@"Debug",@"Exit",@"Continue (dangerous)");
	if(ret == NSAlertDefaultReturn)
	{
		Debugger();
//		*((int*)0x0000000) = 0;
	}
	else if(ret == NSAlertAlternateReturn)
	{
		exit(0);
	}
	else if(ret == NSAlertOtherReturn)
	{
	}
	else
	{
		exit(0);
	}
#else
	exit(0);
#endif
}

#endif // NO_VERBOSE_AT_ALL

