/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#define Zz_PathHelpers_h
#ifndef Zz_PathHelpers_h
#include "Zz_PathHelpers.h"
#endif //Zz_PathHelpers_h

#ifndef ZzString_h
#include "ZzString.h"
#endif //ZzString_h


/*
	path manipulation
*/
#ifndef WIN32
#define DIRECTORY_SEPARATOR_CHAR '/' 
#else
#define DIRECTORY_SEPARATOR_CHAR '\\' 
#endif

// input : /usr/foo/bar/foobar.html
// output : foobar.html
// returns the position of the filename in path
int ZzGetFileNameFromPath( char* dest, int maxChars, const char* path );

ZzString ZzGetFileNameFromPath( const ZzString& path );

// input : FSRef
// output : absolute posix path
// false is returned there is an error (bogus FSRef, ...). in this case outPath is
// not modified.
//bool ZzPathFromFSRef(ZzString& outPath, FSRef *inRef);
// USE ZzFSRefToPathCopy INSTEAD AT THE MOMENT

// input : /usr/foo/bar/foobar.html
// output : /usr/foo/bar/
void ZzGetDirectory( char* dest, int maxChars, const char* path );

// see the -(NSString*)[NSString stringByDeletingLastPathComponent] docs
ZzString ZzDeleteLastPathComponent(ZzString path);

// see the -(NSString*)[NSString lastPathComponent] docs
ZzString ZzLastPathComponent(ZzString path);

// input : /usr/foo/bar/foobar.html
// output : foobar
// returns the position of the filename in path
int ZzGetFileTitleFromPath( char* dest, int maxChars, const char* path );

// input : /usr/foo/bar/foobar.html
// output : html
// returns the position of the '.' of the extension
int ZzGetFileExtensionFromPath( char* dest, int maxChars, const char* path );

int ZzGetUpperCaseFileExtensionFromPath( char* dest, int maxChars, const char* path);

// input : /usr/foo/bar/foobar.html
// output : /usr/foo/bar/foobar
void ZzRemoveFileExtensionFromPath( char* path, int maxChars);

// input : /usr/foo/bar and foobar.html
//output : /usr/foo/bar/foobar.html
void ZzAppendFileNameToPath( char* dest, int maxChars, const char* path, const char* filename);

// input : (/usr/foo/bar/foobar.html and .html) or (/usr/foo/bar/foobar and .html)
//output : /usr/foo/bar/foobar.html
void ZzAppendExtension(char* path, int maxChars, const char* extension);

// input : (/usr/foo/bar/foobar.html) => output : (/usr/foo/bar/foobar_2.html)
// input : (/usr/foo/bar/foobar_2.html) => output : (/usr/foo/bar/foobar_3.html)
void ZzGenerateAlternativePath(char* path, int maxChars);

// input : (/usr/foo/bar/foobar.html & 2) => output : (/usr/foo/bar/foobar_2.html)
// input : (/usr/foo/bar/foobar_2.html & 5) => output : (/usr/foo/bar/foobar_2_5.html)
void ZzAddIndexToFileName(char* path, int maxChars, int index);
void ZzAddIndexToFileNameUsingSeparator(char* path, int maxChars, int index, char separator);

/*! 
	@function ZzRemoveBadFileNameCharacters Turns * and / in the filename to x and -
*/
void ZzRemoveBadFileNameCharacters(char* fileName); 

ZzString ZzGUILocateFile(ZzString dialogTitle);
ZzString ZzGUILocateFile();

OSStatus ZzGetCreatorTypeFromPath( const ZzString& path, OSType* outFileCreator, OSType* outFileType);

/*! @function ZzCopyFile Copies a file (simple posix copy) */
class MIProgressBarDlgManager;
bool ZzCopyFile(const char *from, const char *topath, MIProgressBarDlgManager* progressMngr = NULL);

#ifdef __APPLE__
/*! 
	@function ZzPathToFSRef converts a path (in a CFStringRef) to an FSRef 
	please note you can get a CFStringRef with ZzString::getCFString()
*/
OSStatus ZzPathToFSRef(CFStringRef inPath, FSRef *outRef);

/*! 
	@function ZzPathToFSSpec converts a path (in a CFStringRef) to an FSSpec 
	please note you can get a CFStringRef with ZzString::getCFString()
*/
OSStatus ZzPathToFSSpec(CFStringRef inPath, FSSpec *outRef);

/*! @function ZzCopyMacFile Copies a file (data fork + resource fork + type & creator codes) */
bool ZzCopyMacFile(const char *from, const char *to, MIProgressBarDlgManager* progressMngr = NULL);
#endif //#ifdef __APPLE__

/*!	
	@function ZzGetMainBundlePath 
	Returns the app main bundle path. allow to acces files copied there in the build phase 
*/
ZzString ZzGetMainBundlePath();

/*!
	ZzGetLogFileFolder
	
	Returns the folder where the log file should be. 
	
	you can do this, for example : 
	
	ZzString logPath = ZzGetLogFileFolder() + ZzString("/log1.txt");
	
*/
ZzString ZzGetLogFileFolder();




