/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#ifndef Zz_PathHelpers_h
#include "Zz_PathHelpers.h"
#endif //Zz_PathHelpers_h

#include <fcntl.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <sys/stat.h>

#ifndef ZzTimer_h
#include "ZzTimer.h"
#endif ZzTimer_h

#ifndef MIProgressBarDlgManager_h
#include "MIProgressBarDlgManager.h"
#endif //MIProgressBarDlgManager_h

#include "MoreFilesX.h" //$$$TODO check updates from time to time

// input : /usr/foo/bar/foobar.html
// output : foobar.html
// returns the position of the filename in path
int ZzGetFileNameFromPath( char* dest, int maxChars, const char* path )
{
	int	pathLen;
	int lenToCopy;
	int startCopyRange;
	int	c;
	pathLen = strlen(path);
	for( c = pathLen-1; c >= 0; c-- )
	{
		if( path[c] == DIRECTORY_SEPARATOR_CHAR )
			break;
	}
	// the length of the string to copy :
	// if c == pathLen-1 we need to copy 0 chars
	// if c == 0 that's pathLen-1
	// if c == -1 we need to copy the whole pathLen chars
	lenToCopy = pathLen-1-c;
	startCopyRange = pathLen-lenToCopy;
	if (lenToCopy > maxChars)
		lenToCopy = maxChars;
	memcpy(dest,path+startCopyRange,lenToCopy);
	dest[lenToCopy] = 0;
	
	return c+1;
}

// input : /usr/foo/bar/foobar.html
// output : /usr/foo/bar/
void ZzGetDirectory( char* dest, int maxChars, const char* path )
{
	ZzAssert(maxChars > int(strlen(path)));
	
	strcpy(dest, path);
	
	char* temp = new char[maxChars];
	
	int file_pos = ZzGetFileNameFromPath(temp, maxChars, path);
	dest[file_pos] = 0;
	delete [] temp;
}



// input : /usr/foo/bar/foobar.html
// output : foobar
// returns the position of the filename in path
int ZzGetFileTitleFromPath( char* dest, int maxChars, const char* path )
{
	int	fileNameLen;
	int lenToCopy;
	int	c;
	char fileName[FILENAME_MAX];
	int fileNamePos = ZzGetFileNameFromPath(fileName,FILENAME_MAX,path);
		
	fileNameLen = strlen(fileName);
	for( c = fileNameLen-1; c >= 0; c-- )
	{
		if( fileName[c] == '.' )
			break;
	}
	// the length of the string to copy :
	// if c == pathLen-1 we need to copy fileNameLen-1 chars
	// if c == 0 that's 0
	// if c == -1 we need to copy the whole fileNameLen chars
	if( c == -1 )
		lenToCopy = fileNameLen;
	else
		lenToCopy = c;
	
	ZzAssert(maxChars > lenToCopy);
	memcpy(dest,fileName,lenToCopy);
	dest[lenToCopy] = 0;
	
	return fileNamePos;
}

// input : /usr/foo/bar/foobar.html
// output : html
int ZzGetFileExtensionFromPath( char* dest, int maxChars, const char* path )
{
	int	fileNameLen;
	int lenToCopy;
	int startCopyRange;
	int	c;
	char fileName[FILENAME_MAX];
	int fileNamePos = ZzGetFileNameFromPath(fileName,FILENAME_MAX,path);
		
	fileNameLen = strlen(fileName);
	for( c = fileNameLen-1; c >= 0; c-- )
	{
		if( fileName[c] == '.' )
			break;
	}
	// the length of the string to copy :
	// if c == fileNameLen-1 we need to copy 0 chars (empty extension)
	// if c == 0 that's fileNameLen-1 (the name is the whole extension)
	// if c == -1 there is no extension (there is no dot '.' in the file name)
	if( c == -1 )
		dest[0] = 0;
	else
	{
		lenToCopy = fileNameLen-1-c;
		startCopyRange = fileNameLen-lenToCopy;
		ZzAssert(maxChars > lenToCopy);
		memcpy(dest,fileName+startCopyRange,lenToCopy);
		dest[lenToCopy] = 0;
	}

	return fileNamePos + c;
}

int ZzGetUpperCaseFileExtensionFromPath( char* dest, int maxChars, const char* path)
{
	char* temp = new char[maxChars];
	
	int ret = ZzGetFileExtensionFromPath(temp, maxChars, path);
	
	for (unsigned int i=0; i < strlen(temp); ++i)
	{
		if (temp[i] >= 'a' && temp[i] <= 'z')
			temp[i] = temp[i] - 'a' + 'A';
	}
		
	strcpy(dest, temp);
	delete [] temp;
	
	return ret;
}

SInt32 ZzFindCharacterIndexFromEnd(const char* theString, char theChar)
{
	SInt32 len = strlen(theString);
	for(SInt32 index = len-1; index >=0; index--)
	{
		if(theString[index] == theChar)
			return index;
	}
	return -1;
}

// input : /usr/foo/bar/foobar.html
// output : /usr/foo/bar/foobar
void ZzRemoveFileExtensionFromPath( char* path, int maxChars)
{
	char extension[256];
	int extensionPos = ZzGetFileExtensionFromPath(extension, 256, path);
		
	SInt32 dirsepcharPos = ZzFindCharacterIndexFromEnd(path,DIRECTORY_SEPARATOR_CHAR);
	
	// if there is a dirsepchar in the pathname, it means it's a relative path...
	if(dirsepcharPos > -1)
	{
		// we can only nullify the dot if it's AFTER the dirsepchar
		if (extensionPos > 0 && extensionPos > dirsepcharPos)
			path[extensionPos] = 0;
	}
	else
	{
		// if there is no dirsepchar, then we can safely nullify the dot
		if (extensionPos > 0)
			path[extensionPos] = 0;
	}
}
    
// input : /usr/foo/bar and foobar.html
//output : /usr/foo/bar/foobar.html
void ZzAppendFileNameToPath( char* dest, int maxChars, const char* path, const char* filename)
{
	ZzAssert(int(strlen(path) + strlen(filename) + 2) < maxChars);
	
	strcpy(dest, path);
	int pathLen = strlen(path);

	// add a separator if there is none
	if (dest[pathLen-1] != DIRECTORY_SEPARATOR_CHAR)
		dest[pathLen++] = DIRECTORY_SEPARATOR_CHAR;

	strcpy(&(dest[pathLen]), filename);
	
	dest[pathLen + strlen(filename)] = 0;
}

// input : (/usr/foo/bar/foobar.html and .html) or (/usr/foo/bar/foobar and .html)
//output : /usr/foo/bar/foobar.html
void ZzAppendExtension(char* path, int maxChars, const char* extension)
{
	char ext[50];
	ZzGetFileExtensionFromPath(ext, 50, path);
	if (strcmp(ext, extension) == 0)
		return;
	
	int pathLen = (int)strlen(path);
	
	ZzAssert(int(pathLen + strlen(extension) + 1) < maxChars);
	
	// if a dot is missing, add it
	if ((path[pathLen-1] != '.') && (extension[0] != '.'))
	{
		path[pathLen++] = '.';
		path[pathLen] = 0;
	}
	
	strcat(path, extension);
}

// input : (/usr/foo/bar/foobar.html) => output : (/usr/foo/bar/foobar_2.html)
// input : (/usr/foo/bar/foobar_2.html) => output : (/usr/foo/bar/foobar_3.html)
void ZzGenerateAlternativePath(char* path, int maxChars)
{
	char extension[50];
	ZzGetFileExtensionFromPath(extension, 50, path);
	ZzRemoveFileExtensionFromPath(path, maxChars);
	
	//then, check if the filename is already ending by _n
	int i = strlen(path)-1;
	while ((path[i] >= '0') && (path[i] <= '9') && (i > 0))
		i--;

	if (path[i] == '_')
	{
		int counter;
		sscanf(&(path[i+1]), "%d", &counter);
		counter++;
		sprintf(&(path[i+1]),"%d", counter);
		ZzAppendExtension(path, maxChars, extension);
	}
	else
	{
		ZzAssert(int(strlen(path) + 2) < maxChars);
		strcat(path, "_2");
		ZzAppendExtension(path, maxChars, extension);
	}	
}

// input : (/usr/foo/bar/foobar.html & 2) => output : (/usr/foo/bar/foobar_2.html)
// input : (/usr/foo/bar/foobar_2.html & 5) => output : (/usr/foo/bar/foobar_2_5.html)
void ZzAddIndexToFileName(char* path, int maxChars, int index)
{
	char ext[50];
	path[ZzGetFileExtensionFromPath(ext, 50, path)] = 0;
	sprintf(path, "%s_%d.%s", path, index, ext);
}

void ZzAddIndexToFileNameUsingSeparator(char* path, int maxChars, int index, char separator)
{
	char ext[50];
	path[ZzGetFileExtensionFromPath(ext, 50, path)] = 0;
	sprintf(path, "%s%c%d.%s", path, separator, index, ext);
}

OSStatus ZzGetCreatorTypeFromFSRef(const FSRef *ref, OSType* outFileCreator, OSType* outFileType);

OSStatus ZzGetCreatorTypeFromPath( const ZzString& path, OSType* outFileCreator, OSType* outFileType)
{
	FSRef outRef;
	OSStatus st;
	*outFileCreator = 0; 
	*outFileType = 0; 
	CFStringRef inPath = path.getCFString();
	st = ZzPathToFSRef( inPath, &outRef );
	if( st != noErr ) 
		return st;

	st = ZzGetCreatorTypeFromFSRef( &outRef, outFileCreator, outFileType );
	if( st != noErr ) 
		return st;
	return noErr; // less BadCode prone when adding code 
}

bool _ZzCopyFile(int fdfrom, int fdto, MIProgressBarDlgManager* progressMngr);

/*
	the 64 bit functions :
	
	int		open(const char *path, int flags, mode_t mode); // failure is -1, not zero
	int		close(int d); // error is -1
	
	ssize_t	read(int d, void *buf, size_t nbytes);
	ssize_t	write(int d, const void *buf, size_t nbytes);
	
	off_t	lseek(int fildes, off_t offset, int whence);
*/

bool ZzCopyFile(const char *from, const char *topath, MIProgressBarDlgManager* progressMngr)
{
	int fdfrom,fdto;
	int retval;

	fdfrom = open(from,O_RDONLY,0);
	if (fdfrom < 0)
		return false;

	/* Open R/W by owner, R by everyone else        */

	fdto=open(topath,O_CREAT|O_TRUNC|O_RDWR,S_IREAD|S_IWRITE);
	if (fdto >= 0)
	{
		if (retval = _ZzCopyFile(fdfrom, fdto, progressMngr))
		{
			close(fdto);
			close(fdfrom);
			return retval;
		}
		else
		{
			close(fdto);
			remove(topath);             /* delete any partial file  */
		}
	}
	close(fdfrom);
	return false;
}

bool _ZzCopyFile(int fdfrom, int fdto, MIProgressBarDlgManager* progressMngr)
{
	int bufsiz;
	bool retval = false;
	
	// compute file sizes here
	ZzAssert(sizeof(off_t) >= 8); // make sure we DO support 64 bit files
	off_t fileSize =lseek(fdfrom,0LL,SEEK_END);
	// DEBUG
	double megaSize = (double)fileSize/(1024.0*1024.0);
	vernl(6,"_ZzCopyFile() : file size is " << megaSize << " MB.");
	// GUBED
	
	off_t posZero =lseek(fdfrom,0LL,SEEK_SET);
	ZzAssert(posZero == 0);
	off_t curPos = 0LL;
	
	/* Use the largest buffer we can get    */

	ZzTimer copyTimer;
	copyTimer.Reset();
	for (bufsiz = 0x800000; bufsiz >= 128; bufsiz >>= 1) // since buffer is max 0x800000, it's ok to restrict copy granularity to this size. snotokay if copying down an idisk or something but sokay for now
	{
		register char *buffer;

		buffer = (char *) malloc(bufsiz);
		if (buffer)
		{
			while (1)
			{
				int n; // cool therez nothing to do to enable 64 bit support

				n = read(fdfrom,buffer,bufsiz);
				if (n == -1)            /* if error             */
					break;
				if (n == 0)             /* if end of file       */
				{
					retval = true;
					break;
				}
				if (n != write(fdto,buffer,(unsigned) n))
					break;
				
				curPos += n;
				double progressValue = (double)curPos/(double)fileSize;
				if(progressMngr)
					progressMngr->Update((float)progressValue);
			}
			free(buffer);
			break;
		}
	}
	float duration = copyTimer.GetMSFloat();
	vernl(6,"_ZzCopyFile() : copy duration is " << duration << " ms.");
	vernl(6,"_ZzCopyFile() : copy speed = " << megaSize*1000.0/duration << " Mb/sec.");
	return retval;
}

#ifdef __APPLE__
OSStatus ZzGetCreatorTypeFromFSRef(const FSRef *ref, OSType* outFileCreator, OSType* outFileType)
{
	OSErr			result;
	FSCatalogInfo	catalogInfo;
	FSRef			parentRef;
	
	/* get nodeFlags, finder info, and parent FSRef */
	result = FSGetCatalogInfo(ref, kFSCatInfoNodeFlags + kFSCatInfoFinderInfo, &catalogInfo , NULL, NULL, &parentRef);
	if(result != noErr)
		return result;
			
	/* make sure FSRef was to a file */
	if((catalogInfo.nodeFlags & kFSNodeIsDirectoryMask) != 0 )
		return notAFileErr;
	
	*outFileType = ((FileInfo *)&catalogInfo.finderInfo)->fileType;
	*outFileCreator = ((FileInfo *)&catalogInfo.finderInfo)->fileCreator;
	
	return result;
}

bool ZzCopyMacFile(const char *from, const char *to, MIProgressBarDlgManager* progressMngr)
{
	bool ok;
	ok = ZzCopyFile(from,to,progressMngr);
	if(!ok)
		return ok;
	ZzString zsFrom(from);
	ZzString zsTo(to);
	zsFrom += "/..namedfork/rsrc";
	zsTo += "/..namedfork/rsrc";
	ok = ZzCopyFile(zsFrom.getUTF8String(),zsTo.getUTF8String(),NULL); // no progress bar on resource fork copy
	if(!ok)
		return ok; // if we can't copy the resource forks, that's an error !!!
	
	// now copy the type and creator codes
	FSRef srcRef;
	FSRef dstRef;
	OSStatus osst;
	
	osst = ZzPathToFSRef(zsFrom.getCFString(), &srcRef);
	if(osst != noErr)
		return false;

	osst = ZzPathToFSRef(zsTo.getCFString(), &dstRef);
	if(osst != noErr)
		return false;
		
	OSType _creator,_type;
	
	osst = ZzGetCreatorTypeFromFSRef(&srcRef,&_creator,&_type);
	if(osst != noErr)
		return false;
	
	osst = FSChangeCreatorType(&dstRef,_creator,_type);
	if(osst != noErr)
		return false;
	return true;
}
#endif //#ifdef __APPLE__


OSStatus ZzPathToFSRef(CFStringRef inPath, FSRef *outRef)
{
    CFURLRef tempURL = NULL;
    Boolean gotRef = false;

    tempURL = CFURLCreateWithFileSystemPath(kCFAllocatorDefault, inPath, kCFURLPOSIXPathStyle, false);

    if (tempURL == NULL)
        return fnfErr;

    gotRef = CFURLGetFSRef(tempURL, outRef);

    CFRelease(tempURL);

    if (!gotRef)
        return fnfErr;

    return noErr;
}

OSStatus ZzPathToFSSpec(CFStringRef inPath, FSSpec *fsSpec)
{
	FSRef _fsRef;
	OSStatus st = ZzPathToFSRef(inPath, &_fsRef);
	if(st != noErr)
		return st;

	return FSGetCatalogInfo(&_fsRef, kFSCatInfoNone, NULL, NULL, fsSpec, NULL);
}

/*
bool ZzPathFromFSRef(ZzString& outPath, FSRef *inRef)
{
	CFURLRef tempURL = CFURLCreateFromFSRef(kCFAllocatorDefault,inRef);
	if(tempURL == NULL)
		return false;
	CFStringRef thePath = CFURLCopyFileSystemPath(tempURL,kCFURLPOSIXPathStyle);
	if(thePath == NULL)
	{
		CFRelease(tempURL);
		return false;
	}
	outPath = thePath;
	CFRelease(thePath);
	CFRelease(tempURL);
	return true;
}
*/

CFStringRef ZzFSRefToPathCopy(const FSRef *inRef)
{
	CFURLRef tempURL = NULL;
	CFStringRef result = NULL;
	if (inRef != NULL) 
	{
		tempURL = CFURLCreateFromFSRef(kCFAllocatorDefault, inRef);
		if (tempURL)
		{
			result = CFURLCopyFileSystemPath(tempURL, kCFURLPOSIXPathStyle);
			CFRelease(tempURL);
		}
	}
	return result;
}


ZzString ZzGetFileNameFromPath( const ZzString& path )
{
	NSString* tmpPath = path.getNSString();
	NSString* result = [tmpPath lastPathComponent];
	return ZzString(result);
}

ZzString ZzGetLogFileFolder()
{
	NSString* logFileFolder = @"~/Library/Logs/";
	NSString* expandedLogFileFolder = [logFileFolder stringByExpandingTildeInPath];
	return ZzString(expandedLogFileFolder);
}


ZzString ZzGetMainBundlePath()
{
#ifdef WIN32 
	ZzAssert(0);
	return ZzString(_T(""));
#elif defined __APPLE__
	return ZzString((CFStringRef)([[NSBundle mainBundle] bundlePath]));
#else 
	ZzAssert(0);
	return ZzString("");
#endif
}


ZzString ZzDeleteLastPathComponent(ZzString path)
{
#ifdef WIN32 
	ZzAssert(0);
	return ZzString(_T(""));
#elif defined __APPLE__
	NSString* inPath = path.getNSString(); //autoreleased
	NSString* outPath = [inPath stringByDeletingLastPathComponent];
	return ZzString(outPath);
#else 
	ZzAssert(0);
	return ZzString("");
#endif
}

ZzString ZzLastPathComponent(ZzString path)
{
#ifdef WIN32 
	ZzAssert(0);
	return ZzString(_T(""));
#elif defined __APPLE__
	NSString* inPath = path.getNSString(); //autoreleased
	NSString* outPath = [inPath lastPathComponent];
	return ZzString(outPath);
#else 
	ZzAssert(0);
	return ZzString("");
#endif
}

ZzString ZzGUILocateFile()
{
	return ZzGUILocateFile(ZzString("Open"));
}

ZzString ZzGUILocateFile(ZzString title)
{
	NSOpenPanel* openPanel = [NSOpenPanel openPanel];
	[openPanel setTitle:title.getNSString()];
	[openPanel setAllowsMultipleSelection:NO];
	int ret = [openPanel runModalForDirectory:NSHomeDirectory() file:nil types:nil];
	if(ret == NSOKButton)
	{
		ZzString retVal([[openPanel filenames] objectAtIndex:0]);
		return retVal;
	}
	else
	{
		ZZ_THROW(ZzExceptionCancel);
	}
}

void ZzRemoveBadFileNameCharacters(char* fileName)
{
	SInt32 len = strlen(fileName);
	for(SInt32 i = 0; i < len; i++)
	{
		if(fileName[i] == '*')
			fileName[i] = 'x';
		if(fileName[i] == '/')
			fileName[i] = '-';
	}
}

